/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.clientCostCenter
{
   /// <sample-summary>
   /// Summary description for ClientCostCenterRtf.
   /// 
   /// This samples loads a client cost center from RPM repository,
   /// it creates a new rtf assignement with a cost center rtf
   /// loaded from repository and value and assigns it to the
   /// client cost center.
   /// </sample-summary>
   [TestFixture]
   public class ClientCostCenterRtf
   {
      public ClientCostCenterRtf()
      {

      }
      //All these variables are used in this sample
      public static String cccName = "IBM_COST_CENTER_ABC";
      // client rtf value must exist in RPM repository
      public static String rtfType = "Client Description";
      public static String rtfValue = "{\\rtf1\\ansi\\deff0{\\"
         + "fonttbl{\\f0\\fnil\\fcharset0 Arial;}{\\f1\\fnil\\"
         + "fcharset0 Tahoma;}}\n{\\colortbl ;\\red0\\green255\\"
         + "blue0;}\n\\viewkind4\\uc1\\pard\\cf1\\lang1033\\ul\\"
         + "fs16 This is a test\\cf0\\ulnone\\f1\\fs20 \n\\par }";

      // Session ID is use for all transaction and is get from the 
      // Authenticate.login function. 
      // The session ID will replace the
      // User password while transacting
      public static String sessionid = "";

      [Test]
      public void sampleTest()
      {
         
         // start the communication with the api
         sessionid = APISetup.SetUp();

         SaveResult save = null;
         // create a client cost center
         ClientCostCenter ccc = new ClientCostCenter();

         ccc.name = cccName;
         
         save = APISetup.application.save(sessionid,ccc,
            null,ReloadType.ReloadResult);

         // create a client cost center scope 
         // and adjust it so it saves the rtf assignement
         ClientCostCenterScope cccScope = 
            new ClientCostCenterScope();
         cccScope.rtfAssignments = true;

         ccc = (ClientCostCenter)save.rpmObject;

         RtfAssignment rtfAssign = new RtfAssignment();

         ClientRTF cccRTF = (ClientRTF)APISetup.application.
            loadFromXpath(sessionid, 
            "/ClientRTF[value='" + rtfType + "']", 
            null).rpmObjectList[0];

         // initialize value for rtf assignement
         // exemple of a rtf value :
         // {\\rtf1\\ansi\\deff0{\\fonttbl{\\f0\\fnil\\fcharset0 
         // Arial;}{\\f1\\fnil\\fcharset0 Tahoma;}}\n{\\colortbl 
         // ;\\red0\\green255\\blue0;}\n\\viewkind4\\uc1\\pard\\
         // cf1\\lang1033\\ul\\fs16 This is a test.\\cf0\\ulnone
         // \\f1\\fs20 \n\\par }

         // assign rtf to the assignement
         rtfAssign.rtf = cccRTF;

         rtfAssign.value = rtfValue;

         int power;
         int bitFlag;

         // create a bitFlag for the rtf assignement
         power = int.Parse(cccRTF.name.Trim());
         bitFlag = ((int)Math.Pow(2, power-1));

         // assign the bitFlag
         //rtfAssign.bitFlag = bitFlag;

         // link the assignement with the wbs object
         rtfAssign.parent = ccc;
         ccc.rtfAssignments = new RtfAssignment[] {rtfAssign};

         // save the folder and client cost center in 
         // the RPM repository use a reload type that 
         // enables you to use the saved data
         save = APISetup.application.
            save(sessionid, ccc, cccScope, ReloadType.ReloadResult );
         APISetup.checkForErrors( save );

         ccc = null;
         ccc = (ClientCostCenter)save.rpmObject;

         Assert.IsNotNull(ccc.rtfAssignments);

         // end the communication with the api.
         APISetup.CleanUp(sessionid);

      }


   }
}
