/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.pool
{
   /// <sample-summary>
   /// Summary description for PoolRef.
   /// 
   /// This sample creates a reference and assigns it
   /// to an existing pool.
   /// </sample-summary>
   [TestFixture]
   public class PoolRef
   {
      public PoolRef()
      {

      }

      //All these variables are used in this sample
      public static String poolName = "IBMPOOL_123";
      public static String orgName = "IBMORGANIZATION";
      public static String orgAdr= "ADR_123";
      public static String conName = "IBMCONTACT";

      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace the
      //User password while transacting
      public static String sessionid = "";

      public static void Main()
      {
      }

      [Test]
      public void javaEnumType()
      {
         sessionid = APISetup.SetUp();

         RPMCalendar cal = new RPMCalendar();
         cal.name = "WS-C#-CETC1-Calendar";
         CalendarException ce = new CalendarException();
         ce.day = DateTime.Today;

         WorkInterval wi1 = new WorkInterval();
         wi1.from = "13:00";
         wi1.to = "17:00";
         ce.intervals = new WorkInterval[] {wi1};

         ce.type = CalendarExceptionType.NonWorking;
         cal.exceptions = new CalendarException[] {ce};

         CalendarScope scope = new CalendarScope();
         scope.exceptions = true;
         SaveResult saveResult = APISetup.application.save(sessionid, cal, scope, ReloadType.ReloadResult);
            cal = saveResult.rpmObject;
         Assert.IsNotNull(cal);
         ce = cal.exceptions[0];
         Assert.IsNotNull(ce);
         Assert.IsNotNull(ce.intervals);         
         Assert.IsNotNull(ce.type);


         String xpath = "/RPMCalendar[@name='WS-C#-CETC1-Calendar']";
         LoadResult lr = APISetup.application.loadFromXpath(sessionid, xpath,scope);
         cal = (RPMCalendar)lr.rpmObjectList[0];
         ce = cal.exceptions[0];
         Assert.IsNotNull(ce.type);
         Console.Out.WriteLine(ce.type.ToString());
      }

      [Test]
      public void sampleTest()
      {


         // start the communication with the api
         sessionid = APISetup.SetUp();


         // Declare the pool wich we are going to use
         Pool pool = null;

         // create a poolscope to keep 
         // track of the relation between 
         // the pool and its attribute
         PoolScope pScope = new PoolScope();

         // create a pool reference scope
         PoolReferenceScope prScope = new PoolReferenceScope();

         // adjust the pool reference scope 
         // to imply the company size
         prScope.companySize = true;

         // assign the scope to link with its attribute
         pScope.poolReferences = prScope;

         // declare a save result object
         SaveResult pSave = null;

         // retrieve the pool in RPM using XPath. 
         pool = (Pool)APISetup.application.loadFromXpath(sessionid, 
            "/Pool[name='" + poolName + "']",
            pScope).rpmObjectList[0];

         // create a pool reference
         PoolReference pRef = new PoolReference();

         // assign the pool as the parent of the reference
         pRef.parent = pool;

         // initialize the reference
         pRef.organizationName = orgName;
         pRef.organizationAddress = orgAdr;
         pRef.companySize = (CompanySize)APISetup.application.
            loadFromXpath(sessionid, "/Companysize", 
            null).rpmObjectList[0];
         pRef.contactName = conName;

         // assign the reference to the pool
         pool.poolReferences = new PoolReference[] {pRef};

         // save the pool in the RPM repository
         // use the sessionid acquired in the setUp
         // a scope is necessary to keep the reference
         pSave = new SaveResult();
         pSave = APISetup.application.save(sessionid, 
            pool, pScope, ReloadType.ReloadResult);

         // method showing the detail of any error in the save
         APISetup.checkForErrors( pSave );

         pool = null;
         pool = (Pool)pSave.rpmObject;

         Assert.IsNotNull(pool.poolReferences);

         // end the communication with the api.
         APISetup.CleanUp(sessionid);

      }

   }
}
