/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
   /// <sample-summary>
   /// Summary description for WBSCommitRes.
   /// 
   /// this sample runs the method : commit affected resources
   /// on a project loaded from RPM repository 
   /// </sample-summary>
   [TestFixture]
   public class WBSCommitRes
   {
      public WBSCommitRes()
      {

      }

      public static String projectName = "IBM_GENERIC_PROJECT_ABC";
      
      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid;

      [Test]
      public void sampleTest()
      {

         // initialize a session with the API
         sessionid = APISetup.SetUp();

         // create a task scope
         // this scope enables us to save 
         // information around the task
         WorkElementScope pScope = new WorkElementScope();

         // create a project object
         Project project = new Project();

         // query the project
         project = (Project)APISetup.application.loadFromXpath(
            sessionid, "/Project[name='" + projectName + "']", 
            pScope ).rpmObjectList[0];

         // run method commit affected resources on the project
         // with the use of a scope, this method can affect
         // many objects in a logical tree
         if (APISetup.application.commitAffectedResources(
            sessionid, project, pScope).successful)
         {
            //commit affected resources worked
         }
         else
         {
            Assert.Fail();
         }


      // close connection with API
      APISetup.CleanUp(sessionid);
      }

   }
}
