/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
    /// <sample-summary>
    /// Summary description for WBSFillOpportunity.
    /// 
    /// this sample loads a Project and fills the opportunity
    /// then saves the project back to the RPM repository
    /// </sample-summary>
    [TestFixture]
      public class WBSFillOpportunity
      {
        public WBSFillOpportunity()
        {

        }

        public static String projectName = "IBM_GENERIC_PROJECT_ABC";
        public static String conNumber = "IBM_CONTACT_NUMBER";
        public static String conType = "IBM_CONTACT_TYPE";
        public static String cusNumber = "IBM_CUSTOMER_NUMBER";
        public static String exClient = "IBM_EXTERNAL_CLIENT";
        public static String oppOwner = "IBM_OPPORTUNITY_OWNER";
        public static String oppTrNumber = 
            "IBM_OPPORTUNITY_TRACKING_NUMBER";
        public static int benBegin = 14;
        public static int benDuration = 10;
        public static double magnitude = 100000.00;

        //Session ID is use for all transaction and is get from the 
        //Authenticate.login function. The session ID will replace 
        // the User password while transacting
        public static String sessionid;

        [Test]
            public void sampleTest() 
            {

              // initialize a session with the API
              sessionid = APISetup.SetUp();

              // create a save result object 
              SaveResult save = null;

              // create a project scope
              // this scope enables us to work with a project and 
              // any child object we wish to use
              WorkElementScope weScope = new WorkElementScope();

              // create a project object that we will use to either 
              // create one or load one project
              Project project = null;

              // adjust project scope so it includes opportunity
              weScope.opportunity = true;

              // query the project
              project = (Project)APISetup.application.
                loadFromXpath(sessionid, 
                      "/Project[name='" + projectName + "']", 
                      weScope ).rpmObjectList[0];

              Opportunity opportunity = new Opportunity();

              // initialize opportunity object
              opportunity.forecastClosed = 
                new DateTime(2008, 10,10,10,10,10,10);
              opportunity.pursuitFinish = 
                new DateTime(2008, 10,10,10,10,10,10);
              opportunity.pursuitStart = 
                new DateTime(2006, 10,10,10,10,10,10);


              opportunity.benefitBeginsMonths = benBegin;
              opportunity.benefitDurationMonths = benDuration;
              opportunity.orderOfMagnitudeCost = magnitude;

              opportunity.contactNumber = conNumber;
              opportunity.contactType = conType;
              opportunity.customerNumber = cusNumber;
              opportunity.externalClient = exClient;
              opportunity.opportunityOwner = oppOwner;
              opportunity.opportunityTrackingNumber = 
                oppTrNumber;

              opportunity.orderOfMagnitudeGPSavingsPercent = 0;
              opportunity.orderOfMagnitudeRevenueBenefits = 0;
              //opportunity.probabilityPercent = 100;

              // assign the project as tha parent of the opportunity
              opportunity.parent = project;

              // save the project into the RPM repository
              // a scope is necessary to keep the opportunity 
              // linked with the portfolio
              save = new SaveResult();
              save = APISetup.application.save(sessionid, 
                    opportunity, null, ReloadType.ReloadResult);
              APISetup.checkForErrors( save );

              opportunity = null;
              opportunity = (Opportunity)save.rpmObject;

              Assert.IsNotNull(opportunity);

              // close connection with API
              APISetup.CleanUp(sessionid);
            }

      }
}
