/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
   /// <sample-summary>
   /// Summary description for WBSTaskResource.
   /// 
   /// this sample loads a task from RPM repository
   /// it then assigns resources of various roles to it.
   /// </sample-summary>
   [TestFixture]
   public class WBSTaskResource
   {
      public WBSTaskResource()
      {

      }

      // these are all variables that are used in this sample
      public static String taskName = "IBM_CHILD_TASK";
      public static double eetcHours = 0.00;
      public static String roleName = "Task Responsible";
      public static String resName = "IBM_TASK_RESOURCE";
      public static String poolName = "IBMPOOL_123";
      
      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid;

      [Test]
      public void sampleTest()
      {

         // initialize a session with the API
         sessionid = APISetup.SetUp();

         // create a save result object 
         SaveResult save = null;

         // create a task scope
         // this scope enables us to save 
         // information around the task
         WorkElementScope tScope = new WorkElementScope();

         // create a resource task assignement scope
         // because we have to save a resource task object
         // in this sample
         ResourceTaskAssignmentScope rtaScope = 
            new ResourceTaskAssignmentScope();

         // initialize task scope
         tScope.resourceTaskAssignments = 
            new ResourceTaskAssignmentScope();
         tScope.resourceTaskAssignments.resource = 
            new ResourceScope();

         // initialize resource task assignement
         rtaScope.taskAssignment = 
            new TaskAssignmentScope();
         rtaScope.resource = new ResourceScope();
      
         // create a task object
         Task task = new Task();

         // create a new Resource
         Resource res = new Resource();

         // query the task
         task = (Task)APISetup.application.loadFromXpath(
            sessionid, "/Task[name='" + taskName + "']", 
            tScope ).rpmObjectList[0];

         // load pool from RPM repository
         Pool pool = (Pool)APISetup.application.
            loadFromXpath(sessionid, 
            "/Pool[name='" + poolName + "']",
            null).rpmObjectList[0];

         // initialize resource
         res.fullName = resName;
         res.parent = pool;

         //  save the new resource
         // use a reload type that enables you 
         // to use your saved data
         save = new SaveResult();
         save = APISetup.application.save(sessionid, 
            res, null, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         // load the saved resource
         res = (Resource)save.rpmObject;

         // create a resource task assignement object
         ResourceTaskAssignment rtAssign = 
            new ResourceTaskAssignment();

         // create a new task assignement object
         TaskAssignment tAssign = new TaskAssignment();

         tAssign.actualStartDate = new DateTime(2006, 03, 13, 0 , 0, 0);
         tAssign.EETCHours = eetcHours;

         // initialize resource task assignement
         // with objects created or loaded before
         rtAssign.parent = task;
         rtAssign.resource = res;

         // create a new security role
         SecurityRole sRole = new SecurityRole();

         sRole.name = roleName;

         save = APISetup.application.save(sessionid, sRole
            , null, ReloadType.ReloadResult);

         // assign a security role from RPM repository
         sRole = (SecurityRole)save.rpmObject;

         rtAssign.securityRole = sRole;

         // assign task assignement to resource task assignement
         rtAssign.taskAssignment = tAssign;

         //  save the new resource task assignement
         // use a reload type that enables you 
         // to use your saved data
         // use a resource task assignment scope well initialized
         save = new SaveResult();
         save = APISetup.application.save(sessionid, rtAssign, 
            rtaScope, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         // load resource task assignement object saved
         rtAssign = (ResourceTaskAssignment)save.rpmObject;

         // assign to the task
         task.resourceTaskAssignments = 
            new ResourceTaskAssignment[] {rtAssign};

         // save task into RPM repository
         // use a scope that includes any information added
         // a reload type is not necessary at this point since 
         // we don't use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, task, 
            tScope, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         task = null;
         task = (Task)save.rpmObject;

         Assert.IsNotNull(task.resourceTaskAssignments[0].resource);

         // close connection with API
         APISetup.CleanUp(sessionid);
      }

   }
}
