/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
   /// <sample-summary>
   /// Summary description for WBSTemplateAtt.
   /// 
   /// This sample loads a project from RPM repository. It then
   /// assigns existing attributes to the project as 
   /// actionAttributeAssignement and saves the 
   /// project. It creates an action and links it to the project
   /// then it checks if
   /// attributes are available in the action object
   /// </sample-summary>
   [TestFixture]
   public class WBSTemplateAtt
   {

      // these are all variables that are used in this sample
      public static String projectName = "IBM_GENERIC_PROJECT_ABC";
      public static String actionName = "IBM_GENERIC_ACTION_ABC";
      public static String attContexte = "Scope";
      public static int MaxAttributes = 2;
      
      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid;

      public WBSTemplateAtt()
      {}

      [Test]
      public void sampleTest()
      {

         // initialize a session with the API
         sessionid = APISetup.SetUp();

         // create a save result object 
         SaveResult save = null;

         // create a project scope
         // this scope enables us to save 
         // information around the [roject
         WorkElementScope pScope = new WorkElementScope();

         // create a scope for acton object
         ScopeElementScope aScope = new ScopeElementScope();

         // initialize scopes
         pScope.relatedAttributeAssignments = true;
         aScope.attributeAssignments = true;

         // load an existing project from RPM repository
         Project project = (Project)APISetup.application.
            loadFromXpath(sessionid, 
            "/Project[name='" + projectName + "']", 
            pScope).rpmObjectList[0];

         // load existing attributes 
         LoadResult loader = APISetup.
            application.loadFromXpath(sessionid, 
            "/Attribute", 
            null);

         // create an array that will contain 
         // interesting attributes
         TestFramework.ApplicationAPI.Attribute [] attributes = 
            new TestFramework.ApplicationAPI.
            Attribute[loader.rpmObjectList.Length]; 

         // this logic enables us to reach Scope attributes
         int j = 0;
         for(int i = 0 ; loader.rpmObjectList.Length > i ; i ++)
         {
            if (loader.rpmObjectList[i].contextName == "Scope")
            {
               attributes[j] = (TestFramework.ApplicationAPI.
                  Attribute)loader.rpmObjectList[i];
               j ++;
            }
         }

         // create AttributeAssignement object
         // you can create as many as attributes to be assigned
         // to the prject
         AttributeAssignment [] attAssign = 
            new AttributeAssignment[2];

         for (int i = 0 ; i < MaxAttributes ; i ++)
         {
            attAssign[i] = new AttributeAssignment();

            // assign attributes to attribute assignments
            attAssign[i].attribute = attributes[i];

            // assign the project as a parent 
            // to attribute assignement
            attAssign[i].parent = project;
         }

            // set the publishedType of your project
            //project.published = PublishedType.None;

         // assign attribute assignement to the project
         project.actionAttributeAssignments = attAssign;

            //  from tcp/ip monitor :
            //<assetCode xsi:nil="true"/>
            //<assignmentType xsi:nil="true"/>
            //<published xsi:nil="true"/>
            // published with a value :
            //<published xmlns:q3="http://types.framework.rpm.ibm.com" xsi:type="q3:PublishedType">None</published>

         // save project into RPM repository
         // use a scope that includes any information added
         // a reload type is not necessary at this point since
         // we don't use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, project,
            pScope, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         project = null;
         project = (Project)save.rpmObject;

         Assert.IsNotNull(project.actionAttributeAssignments);

         // load the scope folder associated to the project used
         // juste before in wich we assigned attribute assignements
         ScopeFolder folder = (ScopeFolder)APISetup.application.
            loadFromXpath(sessionid, 
            "/ScopeFolder[name='" + projectName + "']", 
            aScope).rpmObjectList[0];

         // create an action object
         Action action = new Action();

         // initialize action object
         action.name = actionName;

         // assigne scope folder as parent of action object
         action.parent = folder;

         // save action into RPM repository
         // use a scope that includes any information added
         // a reload type is necessary at this point since
         // we want to use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, action,
            aScope, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         action = null;
         action = (Action)save.rpmObject;

         Assert.IsNotNull(action.attributeAssignments);

         // close connection with API
         APISetup.CleanUp(sessionid);
      }
   }
}
