/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
   /// <sample-summary>
   /// Summary description for WBSTemplateSC.
   /// 
   /// 
   /// </sample-summary>
   [TestFixture]
   public class WBSTemplateSC
   {

      // these are all variables that are used in this sample
      public static String projectName = "IBM_GENERIC_PROJECT_ABC";
      public static String requirementName = "IBM_GENERIC_REQUIREMENT_ABC";
      public static String attContexte = "Scope";
      
      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid;

      public WBSTemplateSC()
      {}

      [Test]
      public void sampleTest()
      {

         // initialize a session with the API
         sessionid = APISetup.SetUp();

         // create a save result object 
         SaveResult save = null;

         // create a project scope
         // this scope enables us to save 
         // information around the [roject
         WorkElementScope pScope = new WorkElementScope();

         // create a scope for acton object
         ScopeElementScope rScope = new ScopeElementScope();

         // initialize scopes
         pScope.scopeElement.assignedScorecard = new ScorecardScope();
         pScope.assignedScorecards = new ScorecardScope();
         rScope.assignedScorecard = new ScorecardScope();

         // load an existing project from RPM repository
         Project project = (Project)APISetup.application.
            loadFromXpath(sessionid, 
            "/Project[name='" + projectName + "']", 
            pScope).rpmObjectList[0];
         
         // create a scope that enables us to load scorecards
         // that can be assigned to a requirement
         ScopeManagementModuleScope smmScope = new ScopeManagementModuleScope();
         smmScope.scorecardFolders = new ScorecardFolderScope();
         smmScope.scorecardFolders.scorecards = new ScorecardScope();

         // load existing attributes 
         LoadResult loader = APISetup.application.loadFromXpath(sessionid, "/ScopeManagementModule", smmScope);

         // create an array that will contain 
         // interesting scorecards
         ScorecardFolder [] scorecardFolders = ((ScopeManagementModule)loader.rpmObjectList[0]).scorecardFolders;

         Scorecard scorecard = new Scorecard();

         for (int i = 0 ; i < scorecardFolders.Length ; i ++)
         {
            if (scorecardFolders[i].scorecards[0].active == true)
            {
               scorecard = scorecardFolders[i].scorecards[0];
            }
         }

         // assign attribute assignement to the project
         //project.requirementAssignedScorecards = new AssignedScorecard[] {scorecard};

         WorkElementScope pScope2 = new WorkElementScope();
         pScope2.scopeElement.assignedScorecard = new ScorecardScope();

         // save project into RPM repository
         // use a scope that includes any information added
         // a reload type is not necessary at this point since
         // we don't use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, project,
            pScope2, ReloadType.None);
         APISetup.checkForErrors( save );

         // load the scope folder associated to the project used
         // juste before in wich we assigned attribute assignements
         ScopeFolder folder = (ScopeFolder)APISetup.application.
            loadFromXpath(sessionid, 
            "/ScopeFolder[name='" + projectName + "']", 
            rScope).rpmObjectList[0];

         // create an action object
         Requirement requirement = new Requirement();

         // initialize action object
         requirement.name = requirementName;

         // assigne scope folder as parent of action object
         requirement.parent = folder;

         // save requirement into RPM repository
         // use a scope that includes any information added
         // a reload type is necessary at this point since
         // we want to use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, requirement,
            rScope, ReloadType.ReloadResult);
         APISetup.checkForErrors( save );

         requirement = null;
         requirement = (Requirement)save.rpmObject;

         // TODO : manager not implemented yet, sample cannot work
         Assert.IsNotNull(requirement.assignedScorecard);

         // close connection with API
         APISetup.CleanUp(sessionid);
      }
   }
}
