/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;
using NUnit.Framework;
using System.Collections;
using TestFramework.ApplicationAPI;
namespace Tests.calendar 
{
	/// <summary>
	/// Shows use of [ExpectedException].
	/// </summary>
	[TestFixture]
	public class RPMCalendarFixture : TestFramework.TestHelper
	{

		public RPMCalendar createFixture()
		{

			TestFramework.ApplicationAPI.RPMCalendar calendar = new TestFramework.ApplicationAPI.RPMCalendar();
			calendar.name = "DotNetFixture";
			TestFramework.ApplicationAPI.CalendarScope scope = new TestFramework.ApplicationAPI.CalendarScope();

			TestFramework.TestHelper.setSkipper();
			TestFramework.TestUtilities.setByIntrospection(calendar, 3, setSkipper());

			return (RPMCalendar) save(calendar, scope);
		}

		public RPMCalendar saveRPMCalendar(TestFramework.ApplicationAPI.RPMCalendar calendar ,TestFramework.ApplicationAPI.CalendarScope scope)
		{
			return (RPMCalendar) save(calendar, scope);
		}

		public TestFramework.ApplicationAPI.RPMCalendar RPMCalendarloadFromXPath(String type, String name, TestFramework.ApplicationAPI.CalendarScope scope)
		{
			TestFramework.ApplicationAPI.RPMCalendar cal = (TestFramework.ApplicationAPI.RPMCalendar)loadFromXPath(type, name, scope)[0];
			return cal;
		}

		public static TestFramework.ApplicationAPI.RPMCalendar createFixture(TestFramework.ApplicationAPI.CalendarScope scope,
			int nbChildrenByLevel, int nbExceptionByLevel)
		{
			TestFramework.ApplicationAPI.RPMCalendar mainCalendar = createCalendar((new Random(100)).Next(), scope, nbChildrenByLevel,
				nbExceptionByLevel);

			return mainCalendar;
		}

		private static TestFramework.ApplicationAPI.RPMCalendar createCalendar(int index, TestFramework.ApplicationAPI.CalendarScope scope,
			int nbChildrenByLevel, int nbExceptionByLevel) 
		{
			TestFramework.ApplicationAPI.RPMCalendar calendar = new TestFramework.ApplicationAPI.RPMCalendar();

			calendar = (TestFramework.ApplicationAPI.RPMCalendar)TestFramework.TestUtilities.setByIntrospection(calendar, index, setSkipper());
			
			
			
			calendar.hoursPerDay = 16;
			calendar.hoursPerWeek = 90;

			// Creates standardMonday
			TestFramework.ApplicationAPI.CalendarWeekDay standardMonday = createCalendarWeekDay(true);
			calendar.standardMonday = standardMonday;

			// Creates standardTuesday
			TestFramework.ApplicationAPI.CalendarWeekDay standardTuesday = createCalendarWeekDay(true);
			calendar.standardTuesday = standardTuesday;

			// Creates standardWednesday
			TestFramework.ApplicationAPI.CalendarWeekDay standardWednesday = createCalendarWeekDay(true);
			calendar.standardWednesday = standardWednesday;

			// Creates StandardThursday
			TestFramework.ApplicationAPI.CalendarWeekDay standardThursday = createCalendarWeekDay(true);
			calendar.standardThursday = standardThursday;

			// Creates standardFriday
			TestFramework.ApplicationAPI.CalendarWeekDay standardFriday = createCalendarWeekDay(true);
			calendar.standardFriday = standardFriday;

			// Creates standardSaturday
			TestFramework.ApplicationAPI.CalendarWeekDay standardSaturday = createCalendarWeekDay(true);
			calendar.standardSaturday = standardSaturday;

			// Creates standardSunday
			TestFramework.ApplicationAPI.CalendarWeekDay standardSunday = createCalendarWeekDay(true);
			calendar.standardSunday = standardSunday;

			if (scope == null)
				return calendar;

			if (scope.parent != null) 
			{
				TestFramework.ApplicationAPI.RPMCalendar parent = createCalendar(index + 5, null, 0, 0);
				parent.calendars = new TestFramework.ApplicationAPI.RPMCalendar[] { calendar };
				calendar.parent = parent;
			}

			if (scope.calendars != null) 
			{
				TestFramework.ApplicationAPI.RPMCalendar[] childCals = new TestFramework.ApplicationAPI.RPMCalendar[nbChildrenByLevel];
				for (int i = 0; i < nbChildrenByLevel; i++) 
				{
					childCals[i] = createCalendar((new Random(100)).Next(), scope.calendars, nbChildrenByLevel,
						nbExceptionByLevel);
				}
				calendar.calendars = childCals;
			}

			return calendar;
		}


		private static TestFramework.ApplicationAPI.CalendarWeekDay createCalendarWeekDay(Boolean canBeDefault)
		{

			TestFramework.ApplicationAPI.CalendarWeekDayType type = getRandomDayType(canBeDefault);

			TestFramework.ApplicationAPI.CalendarWeekDay calWD = new TestFramework.ApplicationAPI.CalendarWeekDay();

			if (type == TestFramework.ApplicationAPI.CalendarWeekDayType.UseDefault) 
			{

				calWD.type = TestFramework.ApplicationAPI.CalendarWeekDayType.UseDefault;
				//Default system intervals...

			} 
			else if (type == TestFramework.ApplicationAPI.CalendarWeekDayType.NonWorking) 
			{

				calWD.minutes = 0;
				calWD.type = TestFramework.ApplicationAPI.CalendarWeekDayType.NonWorking;

			} 
			else 
			{

				calWD.type = TestFramework.ApplicationAPI.CalendarWeekDayType.NonDefaultWorking;

				TestFramework.ApplicationAPI.WorkInterval[] intervals = new TestFramework.ApplicationAPI.WorkInterval[5];
				intervals[0] = createWorkInterval(800, 1000);
				intervals[1] = createWorkInterval(10015, 12015);
				intervals[2] = createWorkInterval(1300, 14030);
				intervals[3] = createWorkInterval(1500, 1600);
				intervals[4] = createWorkInterval(16030, 1700);

				calWD.intervals = intervals;
				calWD.minutes =  420;
			}

			return calWD;
		}

		private static TestFramework.ApplicationAPI.WorkInterval createWorkInterval(long from, int to)
		{

			TestFramework.ApplicationAPI.WorkInterval interval = null;

			interval = new TestFramework.ApplicationAPI.WorkInterval();

			interval.from = "08:00";
			interval.to = "20:00";


			return interval;
		}

		private static TestFramework.ApplicationAPI.CalendarWeekDayType getRandomDayType(Boolean canBeDefault) 
		{

			TestFramework.ApplicationAPI.CalendarWeekDayType calendarWeekDayType = new TestFramework.ApplicationAPI.CalendarWeekDayType();
			double nb = (new Random(5)).NextDouble();

			if (nb <= 1 && canBeDefault) 
			{
				calendarWeekDayType = TestFramework.ApplicationAPI.CalendarWeekDayType.UseDefault;
			} 
			else if (nb <= 2) 
			{
				calendarWeekDayType = TestFramework.ApplicationAPI.CalendarWeekDayType.NonWorking;
			} 
			else 
			{
				calendarWeekDayType = TestFramework.ApplicationAPI.CalendarWeekDayType.NonDefaultWorking;
			}

			return calendarWeekDayType;

		}

	}

}
