/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;
using NUnit.Framework;
using TestFramework.ApplicationAPI;

namespace Tests.customfield
{
	public class TestCustomField : CustomFieldFixture
	{
		[TearDown]
		public new void CleanUp() 
		{
			CustomFieldFixture.deleteAllTestCustomFields();
			base.CleanUp();
		}

		public void testCreateCustomFieldWithText()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.CLIENT_CONTEXT_NAME, FieldInputType.Text);
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNullText()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.ASSET_CONTEXT_NAME, FieldInputType.Text);
			original.defaultValue = null;
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithCurrency()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.CLIENT_CONTEXT_NAME, FieldInputType.Currency);
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNullCurrency()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.CLIENT_CONTEXT_NAME, FieldInputType.Currency);
			original.defaultValue = null;
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithCheckbox()
		{
			CustomField original = CustomFieldFixture.createCustomField(
					CustomFieldFixture.DOCUMENT_CONTEXT_NAME, FieldInputType.Checkbox);
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNullCheckbox()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.DOCUMENT_CONTEXT_NAME, FieldInputType.Checkbox);
			original.defaultValue = null;
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithDate()
		{
			CustomField original = CustomFieldFixture.createCustomField(
					CustomFieldFixture.RESOURCE_CONTEXT_NAME, FieldInputType.Date);
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNullDate()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.RESOURCE_CONTEXT_NAME, FieldInputType.Date);
			original.defaultValue = null;
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNumeric()
		{
			CustomField original = CustomFieldFixture.createCustomField(
					CustomFieldFixture.WBS_AND_SCOPE_CONTEXT_NAME, FieldInputType.Numeric);
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithNullNumeric()
		{
			CustomField original = CustomFieldFixture.createCustomField(
				CustomFieldFixture.WBS_AND_SCOPE_CONTEXT_NAME, FieldInputType.Numeric);
			original.defaultValue = null;
			doTestCreate(original, null);
		}

		public void testCreateCustomFieldWithDatafield()
		{
			CustomField original = CustomFieldFixture.createCustomField(
					CustomFieldFixture.RESOURCE_CONTEXT_NAME, FieldInputType.Datafield);
			doTestCreate(original, null);
		}

		private GenericCustomField doTestCreate(GenericCustomField original,
				CustomFieldScope scope)
		{
			return CustomFieldFixture.Save(original, scope);
		}
	}
}
