/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.framework;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

public class ManualCheckoutSample extends SampleTestController
{
    /**
     * This method is called by the superclass
     */
    protected void runSample() throws Exception
    {
        System.out.println("Disabling automatic checkout");
        application.disableAutomaticCheckoutMode(sessionID);

        System.out.println("Create and Save Pool");
        Pool pool = new Pool();
        pool.setName("TCFoo");
        SaveResult saveResult = save(pool, null, ReloadType.SavedResult);
        if (!saveResult.isSuccessful())
        {
            throw new Exception(saveResult.getErrors()[0]);
        }

        pool = (Pool) saveResult.getRpmObject();
        Resource resource = new Resource();
        resource.setFullName("TCBar");
        Calendar calendar = Calendar.getInstance();
        calendar.set(2008, 3, 11);
        resource.setEffectiveStartDate(calendar);
        resource.setActive(Boolean.TRUE);

        LoadResult loadResult = loadFromXpath(
            "/RPMCalendar[ID = 'DEFAULT_CALENDAR']", null);
        printRPMExceptions(loadResult);
        if (!loadResult.isSuccessful())
        {
            throw new Exception(loadResult.getErrors()[0]);
        }

        RPMObject[] objects = loadResult.getRpmObjectList();
        resource.setCalendar((RPMCalendar) objects[0]);

        pool.setResources(new Resource[]
        {
            resource
        });

        PoolScope poolScope = new PoolScope();
        ResourceScope rScope = new ResourceScope();
        rScope.setCalendar(new CalendarScope());
        poolScope.setResources(rScope);

        System.out.println("Checking out the Pool");
        Result result = application.checkOut(sessionID, pool, false);
        printRPMExceptions(result);
        if (!result.isSuccessful())
        {
            throw new Exception(result.getErrors()[0]);
        }

        System.out.println("Updating the Pool with a Resource");
        saveResult = application.save(sessionID, pool, poolScope,
            ReloadType.SavedResult);
        printRPMExceptions(saveResult);

        System.out.println("Checking in the Pool");
        result = application.checkIn(sessionID, pool);
        printRPMExceptions(result);
        if (!result.isSuccessful())
        {
            throw new Exception(result.getErrors()[0]);
        }
    }

    public static void main(String[] args) throws Exception
    {
        new ManualCheckoutSample();
    }
}
