/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.scope.PoolReferenceScope;
import com.ibm.rpm.resource.scope.PoolScope;

/**
 * This sample adds a reference to an existing pool
 * 
 * @preconditions : poolName must exist in the repository
 * @desc Add a reference to an existing pool
 * 
 */
public class PoolRefSample extends SampleTestController
{
    public static String poolName = "POOL_123";

    public static String orgName = "ORGANIZATION";

    public static String orgAdr = "ADR_123";

    public static String conName = "CONTACT";

    protected void runSample() throws Exception
    {
        /**
         * Indicate that we wish to work on the references category as well.
         */
        PoolReferenceScope prScope = new PoolReferenceScope();
        prScope.setCompanySize(true);
        PoolScope pScope = new PoolScope();
        pScope.setPoolReferences(prScope);

        Pool pool = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
            pScope).getRpmObjectList()[0];

        /**
         * Initialize the pool reference
         */
        PoolReference pRef = new PoolReference();
        pRef.setParent(pool);
        pRef.setOrganizationName(orgName);
        pRef.setOrganizationAddress(orgAdr);
        pRef.setCompanySize((CompanySize) loadFromXpath("/Companysize", null)
            .getRpmObjectList()[0]);
        pRef.setContactName(conName);

        /**
         * Associate the reference object we've created to the pool and save it
         * in the repository
         */
        pool.setPoolReferences(new PoolReference[]
        {
            pRef
        });
        save(pool, pScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new PoolRefSample();
    }
}
