/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.util.Calendar;

import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample creates a number of custom fields of different types and makes
 * them available for use. You may confirm their creation in the Administration
 * view or in the Custom Fields portlet.
 * 
 * @desc Create new custom fields.
 * 
 */
public class ResourceCField extends SampleTestController
{
    public static String resourceName = "RESOURCE";

    public static String cFieldName = "FIELD";

    public static String cfcName = "CATEGORY";

    public static FieldInputType[] fieldType = new FieldInputType[]
    {
            FieldInputType.Checkbox, FieldInputType.Currency,
            FieldInputType.Datafield, FieldInputType.Date,
            FieldInputType.Numeric, FieldInputType.Text
    };

    protected void runSample() throws Exception
    {
        ResourceScope rScope = new ResourceScope();
        rScope.setCustomFieldAssignments(true);

        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        CustomFieldCategory cfc = new CustomFieldCategory();
        cfc.setName(cfcName);
        cfc.setContextName("Resource");

        SaveResult result = save(cfc, null, ReloadType.ReloadResult);
        cfc = (CustomFieldCategory) result.getRpmObject();

        /**
         * Associate each field to its parent Resource
         */
        CustomFieldAssignment[] fieldAssignments = new CustomFieldAssignment[fieldType.length];
        CustomFieldAssignment assign = null;
        for (int i = 0; i < fieldType.length; i++)
        {
            CustomField cField = new CustomField();
            cField.setParent(cfc);
            cField.setName(cFieldName + i);
            cField = saveCustomField(cField, fieldType[i]);

            assign = new CustomFieldAssignment();
            assign.setCustomField(cField);
            assign.setParent(res);

            fieldAssignments[i] = assign;
        }

        /**
         * Associate all of the fields with the resource we want to work on and
         * save everyting into the RPM repository
         */
        res.setCustomFieldAssignments(fieldAssignments);
        result = save(res, rScope, ReloadType.ReloadResult);
        res = (Resource) result.getRpmObject();

        CustomFieldScope cfScope = new CustomFieldScope();
        cfScope.setChildren(new CustomFieldScope());

        CustomFieldCategory cfCategory = (CustomFieldCategory) loadFromXpath(
            "/ResourceModule/customfieldcategory" + "[name='" + cfcName + "']",
            cfScope).getRpmObjectList()[0];

        /**
         * Write to the fields we have just created into the repository
         */
        for (int i = 0; i < fieldType.length; i++)
        {
            /*
             * Since the results are not ordered as returned from the
             * repository, we have to verify what we're modifying first
             */
            int pos = 0;
            for (; pos < fieldType.length; ++pos)
            {
                String key = cFieldName + pos;

                if (key.equals(cfCategory.getChildren()[pos].getName()))
                {
                    break;
                }
            }

            /**
             * Populate each of the custom fields we have saved above with
             * default values.
             */
            CustomField field = cfCategory.getChildren()[pos];
            FieldInputType fieldType = field.getInputType();
            if (fieldType == FieldInputType.Text)
            {
                field.setDefaultValue("DEFAULT_CUSTOM_VALUE");
            }
            else if (fieldType == FieldInputType.Numeric)
            {
                field.setDefaultValue(new Integer(1));
            }
            else if (fieldType == FieldInputType.Date)
            {
                field.setDefaultValue(Calendar.getInstance());
            }
            else if (fieldType == FieldInputType.Datafield)
            {
                field
                    .setDefaultValue(loadFromXpath(
                        "/Experience[@value='1 year +']", null)
                        .getRpmObjectList()[0]);
            }
            else if (fieldType == FieldInputType.Currency)
            {
                field.setDefaultValue(loadFromXpath(
                    "/Currency[@name='Canadian dollars']", null)
                    .getRpmObjectList()[0]);
            }
            else if (fieldType == FieldInputType.Checkbox)
            {
                field.setDefaultValue(Boolean.FALSE);
            }
        }

        save(cfCategory, cfScope, ReloadType.None);
    }

    /**
     * This method saves a custom field into the RPM repository using the
     * supplied FieldInputType
     * 
     * @param field
     * @param type
     * @return The custom field as with the ID field populated by the repository
     */
    protected CustomField saveCustomField(CustomField field, FieldInputType type)
    {
        CustomFieldScope cfScope = new CustomFieldScope();

        field.setInputType(type);

        SaveResult result = save(field, cfScope, ReloadType.ReloadResult);

        return (CustomField) result.getRpmObject();
    }

    public static void main(String[] args)
    {
        new ResourceCField();
    }
}
