/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.rmi.RemoteException;

import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This Sample assigns a pay rate to a resource in the RPM repository then
 * stores the result back
 * 
 * @preconditions : - resourceName must exist as a resource in the database
 * 
 * @desc Update a resource's pay rate.
 */
public class ResourceRate extends SampleTestController
{
    /*
     * The name of the resource as it will appear in RPM
     */
    public static String resourceName = "testABCDEF";

    /*
     * The resource must exist under the following pool for the sameple to run
     * successfully.
     */
    static String poolName = "POOL_123";

    /*
     * All the different pay rates which will be assigned to this resource.
     */
    public static double rate = 50.00;

    public static double costph = 50.00;

    public static double sellingRateph = 60.00;

    public static double specialCostph = 75.00;

    public static double specialSellingRateph = 90.00;

    protected void runSample() throws RemoteException
    {
        Resource resource = null;
        ResourceRates rates = new ResourceRates();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setRates(true);

        /*
         * Load the resource from the RPM repository
         */
        resource = (Resource) application.loadFromXpath(sessionID,
            "/Resource[fullname='" + resourceName + "']", resourceScope)
            .getRpmObjectList()[0];

        /*
         * Initialize the ResourceRate object
         */
        rates.setRegularCostPerHour(new Double(costph));
        rates.setSellingRatePerHour(new Double(sellingRateph));
        rates.setSpecialCostPerHour(new Double(specialCostph));
        rates.setSpecialSellingRatePerHour(new Double(specialSellingRateph));

        /*
         * assign rates to resource.rates
         */
        rates.setParent(resource);
        resource.setRates(rates);

        Pool pool = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
            null).getRpmObjectList()[0];
        resource.setParent(pool);
        /*
         * Save the resource in the RPM repository
         */
        SaveResult result = application.save(sessionID, resource,
            resourceScope, ReloadType.ReloadResult);
        printRPMExceptions(result);

        /*
         * Verify that the save completed successfully.
         */
        resource = (Resource) result.getRpmObject();
        if (resource.getRates() == null)
        {
            System.err
                .println("*** Fatal : Did not save successfully the rate of the resource");
        }
    }

    public static void main(String[] args)
    {
        new ResourceRate();
    }
}
