/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * This sample associates a skill level to a resource
 * 
 * @desc Associate a skill level to a resource.
 * 
 */
public class SkillCompRessSample extends SampleTestController
{
    static String category = "CATEGORY";

    static String classification = "CLASSIFICATION";

    static String proficiency = "PROFICIENCY";

    static String skillName = "SKILL";

    static String compName = "COMPETENCY";

    protected void runSample() throws Exception
    {
        ProficiencyLevel level = (ProficiencyLevel) loadFromXpath(
            "/ProficiencyLevel[@value='Applied']", null).getRpmObjectList()[0];

        ResourceAttributeCompetencyCategory compCat = new ResourceAttributeCompetencyCategory();
        ResourceAttributeCompetencyClassification compClassi = new ResourceAttributeCompetencyClassification();
        ResourceAttributeCompetency comp = new ResourceAttributeCompetency();
        ResourceAttributeCompetencyProficiencyLevel compProf = new ResourceAttributeCompetencyProficiencyLevel();

        /**
         * We define a scope, since we will be saving an object graph
         */
        ResourceAttributeScope cScope = new ResourceAttributeScope();
        compCat.setName(category);
        cScope.setCategories(new ResourceAttributeScope());

        compClassi.setParent(compCat);
        compClassi.setName(classification);
        compCat
            .setClassifications(new ResourceAttributeCompetencyClassification[]
            {
                compClassi
            });
        cScope.setClassifications(new ResourceAttributeScope());

        comp.setParent(compClassi);
        comp.setName(compName);
        compClassi.setAttributes(new ResourceAttributeCompetency[]
        {
            comp
        });
        cScope.getClassifications().setAttributes(new ResourceAttributeScope());

        compProf.setParent(comp);
        compProf.setName(level.getValue());
        compProf.setProficiencyLevel(level);
        comp
            .setProficiencyLevels(new ResourceAttributeCompetencyProficiencyLevel[]
            {
                compProf
            });
        cScope.getClassifications().getAttributes().setProficiencyLevels(true);

        /**
         * Now that all objects are created and associated, save the Resource
         * Attribute Competency object with a scope to include all elements
         * created.
         */
        compCat = (ResourceAttributeCompetencyCategory) save(compCat, cScope,
            ReloadType.ReloadResult).getRpmObject();

        /**
         * Skill
         * 
         * Declare all objects needed to create a resource skill. We need a
         * proficiency level with a skill as parent. The skill needs a parent
         * classification, and the classification will beed a category parent
         */
        ResourceAttributeSkillCategory skillCat = new ResourceAttributeSkillCategory();
        ResourceAttributeSkillClassification skillClassi = new ResourceAttributeSkillClassification();
        ResourceAttributeSkill skill = new ResourceAttributeSkill();
        ResourceAttributeSkillProficiencyLevel skillProf = new ResourceAttributeSkillProficiencyLevel();

        ResourceAttributeScope sScope = new ResourceAttributeScope();
        skillCat.setName(category);
        sScope.setCategories(new ResourceAttributeScope());

        skillClassi.setParent(skillCat);
        skillClassi.setName(classification);
        skillCat.setClassifications(new ResourceAttributeSkillClassification[]
        {
            skillClassi
        });
        sScope.setClassifications(new ResourceAttributeScope());

        skill.setParent(skillClassi);
        skill.setName(skillName);
        skillClassi.setAttributes(new ResourceAttributeSkill[]
        {
            skill
        });
        sScope.getClassifications().setAttributes(new ResourceAttributeScope());

        skillProf.setParent(skill);
        skillProf.setName(level.getValue());
        skillProf.setProficiencyLevel(level);
        skill.setProficiencyLevels(new ResourceAttributeSkillProficiencyLevel[]
        {
            skillProf
        });
        sScope.getClassifications().getAttributes().setProficiencyLevels(true);

        /**
         * After the creation of the objects, save the Resource attribute. Use a
         * scope to include all elements
         */
        save(skillCat, sScope, ReloadType.ReloadResult);
    }

    public static void main(String[] args)
    {
        new SkillCompRessSample();
    }
}