/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.scopemanagement.sample;

import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

public class ScopeElementConversion extends SampleTestController
{
    public static void main(String[] args) throws Exception
    {
        new ScopeElementConversion();
    }

    private static LoadResult createScopeFolder(String folderName)
            throws Exception
    {
        LoadResult loadResult = null;

        loadResult = application.loadFromXpath(sessionID, "/ScopeFolder[name='"
                + folderName + "']", null);

        if (loadResult == null || loadResult.getRpmObjectList() == null)
        {
            // create a Proposal and save it
            System.out.println("Creating a Proposal");

            Proposal proposal = new Proposal();
            proposal.setName(folderName);
            SingleResult singleResult = application.save(sessionID, proposal,
                null, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                // load the ScopeFolder associated with the proposal
                loadResult = application.loadFromXpath(sessionID,
                    "/ScopeFolder[name='" + folderName + "']", null);

                printRPMExceptions(loadResult);
            }
        }
        else
        {
            System.out.println("ERROR: ScopeFolder with name " + folderName
                    + " already exists.");
            loadResult.setSuccessful(false);
        }

        return loadResult;
    }

    private static void convertIssueToRisk() throws Exception
    {
        System.out.println("\nconvertIssueToRisk - Begin");

        ScopeElementScope scopeElementScope = new ScopeElementScope();

        // create the parent ScopeFolder
        LoadResult result = createScopeFolder("TC-convertIssueToRisk");

        if (result != null && result.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) result.getRpmObjectList()[0];

            // create an Issue and save it
            System.out.println("Creating an Issue");

            Issue issue = new Issue();
            issue.setParent(scopeFolder);
            issue.setName("TC issue");
            SingleResult singleResult = application.save(sessionID, issue,
                scopeElementScope, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                issue = (Issue) singleResult.getRpmObject();

                // convert the Issue to a Risk
                System.out.println("Converting the Issue to a Risk");

                singleResult = application.convertContainer(sessionID, issue,
                    new Risk(), scopeElementScope);
                printRPMExceptions(singleResult);

                if (singleResult.isSuccessful())
                {
                    LoadResult loadResult = application.loadFromXpath(
                        sessionID, "/Risk[id='" + issue.getID() + "']",
                        scopeElementScope);

                    printRPMExceptions(loadResult);

                    if (loadResult.getRpmObjectList()[0] instanceof Risk)
                    {
                        System.out
                            .println("Successful conversion of Issue to Risk");
                    }
                }
            }
        }
        System.out.println("convertIssueToRisk - End");

    }

    private static void convertDefectToTask() throws Exception
    {
        System.out.println("\nconvertDefectToTask - Begin");

        ScopeElementScope scopeElementScope = new ScopeElementScope();

        // create the parent ScopeFolder
        LoadResult result = createScopeFolder("TC-convertDefectToTask");

        if (result != null && result.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) result.getRpmObjectList()[0];

            // create a Defect and save it
            System.out.println("Creating a Defect");

            Defect defect = new Defect();
            defect.setParent(scopeFolder);
            defect.setName("TC defect");
            SingleResult singleResult = application.save(sessionID, defect,
                scopeElementScope, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                defect = (Defect) singleResult.getRpmObject();

                // convert the Defect to a DefectTask
                System.out.println("Converting the Defect to a task");
                defect.setTask(Boolean.TRUE);
                singleResult = application.save(sessionID, defect,
                    scopeElementScope, ReloadType.ReloadResult);

                if (singleResult.isSuccessful())
                {
                    printRPMExceptions(singleResult);
                    defect = (Defect) singleResult.getRpmObject();

                    // reload the DefectTask
                    LoadResult loadResult = application.loadFromXpath(
                        sessionID, "/Defect[id='" + defect.getID() + "']",
                        scopeElementScope);

                    printRPMExceptions(loadResult);

                    // ensure that the conversion was successful
                    if (loadResult.isSuccessful()
                            && loadResult.getRpmObjectList()[0] instanceof Defect)
                    {
                        Defect reloaded = (Defect) loadResult
                            .getRpmObjectList()[0];

                        if (reloaded.getTask() != null
                                && reloaded.getTask().booleanValue())
                        {
                            System.out
                                .println("Successful conversion of a Defect to a DefectTask");
                        }
                    }
                }
            }
        }

        System.out.println("convertDefectToTask - End");
    }

    private static void convertDefectTaskToDefect() throws Exception
    {
        System.out.println("\nconvertTaskToDefect - Begin");

        ScopeElementScope scopeElementScope = new ScopeElementScope();

        // load an existing Defect task
        System.out.println("Loading a DefectTask");

        LoadResult loadResult = application.loadFromXpath(sessionID,
            "/Defect[name='TC defect']", scopeElementScope);

        printRPMExceptions(loadResult);

        if (loadResult.isSuccessful())
        {
            Defect defect = null;
            if (loadResult.getRpmObjectList() != null)
            {
                defect = (Defect) loadResult.getRpmObjectList()[0];
            }

            if (defect != null && defect.getTask() != null
                    && defect.getTask().booleanValue())
            {
                // convert the DefectTask to a Defect
                defect.setTask(Boolean.FALSE);
                SingleResult singleResult = application.save(sessionID, defect,
                    scopeElementScope, ReloadType.ReloadResult);

                printRPMExceptions(singleResult);

                if (singleResult.isSuccessful())
                {
                    // check that the conversion was successful
                    defect = (Defect) singleResult.getRpmObject();
                    if (defect.getTask() != null
                            && !defect.getTask().booleanValue())
                    {
                        System.out
                            .println("Successful conversion of a DefectTask to a Defect");
                    }
                }
            }
            else
            {
                System.out.println("ERROR: Couldn't load a DefectTask");
            }
        }

        System.out.println("convertTaskToDefect - End");
    }

    private static void convertActionToActionTaskToRequirementTask()
            throws Exception
    {
        System.out
            .println("\nconvertActionToActionTaskToRequirementTask - Begin");

        ScopeElementScope scopeElementScope = new ScopeElementScope();

        // create the parent ScopeFolder
        LoadResult result = createScopeFolder("TC-convertActionToActionTaskToRequirementTask");

        if (result != null && result.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) result.getRpmObjectList()[0];

            // create a Defect and save it
            System.out.println("Creating an Action");

            Action action = new Action();
            action.setParent(scopeFolder);
            action.setName("TC action");
            SingleResult singleResult = application.save(sessionID, action,
                scopeElementScope, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                action = (Action) singleResult.getRpmObject();

                // convert the Action to an ActionTask
                System.out.println("Converting the Action to a task");
                action.setTask(Boolean.TRUE);
                singleResult = application.save(sessionID, action,
                    scopeElementScope, ReloadType.ReloadResult);

                if (singleResult.isSuccessful())
                {
                    printRPMExceptions(singleResult);
                    action = (Action) singleResult.getRpmObject();

                    // reload the ActionTask
                    LoadResult loadResult = application.loadFromXpath(
                        sessionID, "/Action[id='" + action.getID() + "']",
                        scopeElementScope);

                    printRPMExceptions(loadResult);

                    // ensure that the conversion was successful
                    if (loadResult.isSuccessful()
                            && loadResult.getRpmObjectList()[0] instanceof Action)
                    {
                        Action actionTask = (Action) loadResult
                            .getRpmObjectList()[0];

                        if (actionTask.getTask() != null
                                && actionTask.getTask().booleanValue())
                        {
                            System.out
                                .println("Successful conversion of an Action to an ActionTask");

                            // convert the Action Task container to a
                            // Requirement Task
                            Requirement requirementTask = new Requirement();
                            requirementTask.setTask(Boolean.TRUE);

                            SingleResult convertResult = application
                                .convertContainer(sessionID, actionTask,
                                    requirementTask, null);

                            if (convertResult.isSuccessful())
                            {
                                Requirement outputRequirementTask = (Requirement) convertResult
                                    .getRpmObject();

                                if (outputRequirementTask.getTask() != null
                                        && outputRequirementTask.getTask()
                                            .booleanValue())
                                {
                                    System.out
                                        .println("Successful conversion of an ActionTask to a RequirementTask");
                                }
                            }
                        }
                    }
                }
            }
        }

        System.out.println("convertActionToActionTaskToRequirementTask - End");
    }

    protected void runSample() throws Exception
    {
        convertIssueToRisk();

        convertDefectToTask();

        convertDefectTaskToDefect();

        convertActionToActionTaskToRequirementTask();

        System.out.println("\nDone");
    }
}