/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.scopemanagement.sample;

import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScopeTransferAndMoveUnderWbs extends SampleTestController
{

    public static void main(String args[]) throws Exception
    {
        new ScopeTransferAndMoveUnderWbs();
    }

    private static LoadResult createScopeFolder(String folderName)
            throws Exception
    {
        LoadResult loadResult = null;

        loadResult = application.loadFromXpath(sessionID, "/ScopeFolder[name='"
                + folderName + "']", null);

        if (loadResult == null || loadResult.getRpmObjectList() == null)
        {
            // create a Proposal and save it
            System.out.println("Creating a Proposal");

            Proposal proposal = new Proposal();
            proposal.setName(folderName);
            SingleResult singleResult = application.save(sessionID, proposal,
                null, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                // load the ScopeFolder associated with the proposal
                loadResult = application.loadFromXpath(sessionID,
                    "/ScopeFolder[name='" + folderName + "']", null);

                printRPMExceptions(loadResult);
            }
        }
        else
        {
            System.out.println("ERROR: ScopeFolder with name " + folderName
                    + " already exists.");
            loadResult.setSuccessful(false);
        }

        return loadResult;
    }

    private static void transferUnderWBSRoot(WorkElementScope workElementScope)
            throws Exception
    {
        System.out.println("\ntransferUnderWBSRoot - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-transferUnderWBSRoot");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save a Requirement");
            Requirement requirement = new Requirement();
            requirement.setParent(scopeFolder);
            requirement.setName("TC Requirement Name");
            SingleResult singleResult = application.save(sessionID,
                requirement, null, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                requirement = (Requirement) singleResult.getRpmObject();

                System.out.println("Transfer the Requirement under WBS");
                Result result = application.transferUnderWBS(sessionID, null,
                    requirement);
                printRPMExceptions(result);

                if (result.isSuccessful())
                {
                    // retrieve the Proposal that contains the ScopeFolder
                    LoadResult loadResult2 = application.loadFromXpath(
                        sessionID, "/Proposal[name='" + scopeFolder.getName()
                                + "']", workElementScope);

                    printRPMExceptions(loadResult);

                    if (loadResult2 != null && loadResult2.isSuccessful())
                    {
                        Proposal proposal = (Proposal) loadResult2
                            .getRpmObjectList()[0];

                        // check that the requirement was transferred to the
                        // Proposal
                        // under WBS; it will be a SummaryTask that contains a
                        // Requirement
                        // scope element
                        if (proposal.getChildren() != null
                                && proposal.getChildren()[0] != null
                                && proposal.getChildren()[0] instanceof SummaryTask
                                && proposal.getChildren()[0].getScopeElement() instanceof Requirement)
                        {
                            System.out
                                .println("Transfer Under WBS Root successful");

                        }
                        else
                        {
                            System.out
                                .println("Transfer Under WBS Root not successful");
                        }

                    }
                }
            }
        }
        System.out.println("transferUnderWBSRoot - End");
    }

    private static void crossMove(ScopeElementScope scopeElementScope)
            throws Exception
    {
        System.out.println("\ncrossMove - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-crossMove");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save a ServiceRequest");

            // create an AbstractScope and reload it to get a populated project
            // field
            ServiceRequest serviceRequest = new ServiceRequest();
            serviceRequest.setParent(scopeFolder);
            serviceRequest.setName("TC Service Request Name");
            SingleResult singleResult = application.save(sessionID,
                serviceRequest, scopeElementScope, ReloadType.ReloadResult);
            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                serviceRequest = (ServiceRequest) singleResult.getRpmObject();

                System.out.println("Create and save a WorkOrganization parent");

                // create a WorkElement to move the AbstractScope to, and its
                // parent
                WorkOrganization workOrganization = new WorkOrganization();
                workOrganization.setName("TC Work Organization Name");
                SingleResult singleResult2 = application.save(sessionID,
                    workOrganization, null, ReloadType.ReloadResult);
                printRPMExceptions(singleResult2);

                if (singleResult2.isSuccessful())
                {
                    workOrganization = (WorkOrganization) singleResult2
                        .getRpmObject();

                    System.out.println("Create and save a WorkProduct");
                    WorkProduct workProduct = new WorkProduct();
                    workProduct.setName("TC Work Product Name");
                    workProduct.setParent(workOrganization);
                    SingleResult singleResult3 = application.save(sessionID,
                        workProduct, null, ReloadType.ReloadResult);
                    printRPMExceptions(singleResult3);

                    if (singleResult3.isSuccessful())
                    {
                        workProduct = (WorkProduct) singleResult3
                            .getRpmObject();
                        System.out
                            .println("Move the ServiceRequest to the WorkProduct");

                        // cross move the AbstractScope to the WorkElement
                        Result result = application.moveUnderWBS(sessionID,
                            workProduct, serviceRequest);
                        printRPMExceptions(result);

                        if (result.isSuccessful())
                        {
                            // check that the project's ScopeFolder
                            // now contains the AbstractScope
                            LoadResult loadResult2 = application.loadFromXpath(
                                sessionID, "/ScopeFolder[name='"
                                        + workOrganization.getName() + "']",
                                scopeElementScope);
                            printRPMExceptions(loadResult2);

                            if (loadResult2 != null
                                    && loadResult2.isSuccessful())
                            {
                                // check that the serviceRequest was moved
                                ScopeFolder folder = (ScopeFolder) loadResult2
                                    .getRpmObjectList()[0];

                                if (folder.getChildren() != null
                                        && folder.getChildren()[0] instanceof ServiceRequest
                                        && serviceRequest.getID().equals(
                                            folder.getChildren()[0].getID()))
                                {
                                    System.out
                                        .println("ServiceRequest successfully moved to WorkProduct");
                                }
                                else
                                {
                                    System.out.println("Move not successful");
                                }
                            }
                        }
                    }
                }
            }
        }
        System.out.println("crossMove - End");
    }

    private static void transferWithin(WorkElementScope workElementScope,
            ScopeElementScope scopeElementScope) throws Exception
    {
        System.out.println("\ntransferWithin - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-transferWithin");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save a Risk");

            // create an AbstractScope and reload it to get a populated project
            // field
            Risk risk = new Risk();
            risk.setParent(scopeFolder);
            risk.setName("TC Risk Name");
            SingleResult singleResult = application.save(sessionID, risk,
                scopeElementScope, ReloadType.ReloadResult);
            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                risk = (Risk) singleResult.getRpmObject();

                // retrieve the Proposal that contains the ScopeFolder
                LoadResult loadResult2 = application.loadFromXpath(sessionID,
                    "/Proposal[name='" + scopeFolder.getName() + "']",
                    workElementScope);
                printRPMExceptions(loadResult2);

                if (loadResult2 != null && loadResult2.isSuccessful())
                {
                    Proposal proposal = (Proposal) loadResult2
                        .getRpmObjectList()[0];

                    System.out.println("Transfer the risk under WBS");

                    // transfer the AbstractScope to the Proposal
                    Result result = application.transferUnderWBS(sessionID,
                        proposal, risk);
                    printRPMExceptions(result);

                    if (result.isSuccessful())
                    {
                        // check that the proposal now contains the
                        // AbstractScope
                        LoadResult loadResult3 = application.loadFromXpath(
                            sessionID, "/Proposal[id='" + proposal.getID()
                                    + "']", workElementScope);
                        printRPMExceptions(loadResult3);

                        if (loadResult3 != null && loadResult3.isSuccessful())
                        {
                            proposal = (Proposal) loadResult3
                                .getRpmObjectList()[0];

                            // check that the risk was transferred to the
                            // Proposal
                            // under WBS; it will be a SummaryTask that contains
                            // a Risk
                            // scope element
                            if (proposal.getChildren() != null
                                    && proposal.getChildren()[0] != null
                                    && proposal.getChildren()[0] instanceof SummaryTask
                                    && proposal.getChildren()[0]
                                        .getScopeElement() instanceof Risk)
                            {
                                System.out
                                    .println("Transfer within successful");

                            }
                            else
                            {
                                System.out
                                    .println("Transfer within not successful");
                            }

                            System.out.println("transferWithin - End");
                        }
                    }
                }
            }
        }
    }

    private static void moveWithin(WorkElementScope workElementScope,
            ScopeElementScope scopeElementScope) throws Exception
    {
        System.out.println("\nmoveWithin - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-moveWithin");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save a Defect");

            // create an AbstractScope and reload it to get a populated project
            // field
            Defect defect = new Defect();
            defect.setParent(scopeFolder);
            defect.setName("TC Defect Name");
            SingleResult singleResult = application.save(sessionID, defect,
                scopeElementScope, ReloadType.ReloadResult);
            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                defect = (Defect) singleResult.getRpmObject();

                // retrieve the Proposal that contains the ScopeFolder
                LoadResult loadResult2 = application.loadFromXpath(sessionID,
                    "/Proposal[name='" + scopeFolder.getName() + "']",
                    workElementScope);
                printRPMExceptions(loadResult2);

                if (loadResult2 != null && loadResult2.isSuccessful())
                {
                    Proposal proposal = (Proposal) loadResult2
                        .getRpmObjectList()[0];

                    System.out.println("Create and save a Deliverable");

                    Deliverable deliverable = new Deliverable();
                    deliverable.setName("TC Deliverable Name");
                    deliverable.setParent(proposal);
                    SingleResult singleResult2 = application.save(sessionID,
                        deliverable, workElementScope, ReloadType.ReloadResult);

                    if (singleResult2.isSuccessful())
                    {
                        deliverable = (Deliverable) singleResult2
                            .getRpmObject();

                        System.out
                            .println("Move the Defect to the Deliverable");

                        // transfer the AbstractScope to the Deliverable
                        Result result = application.moveUnderWBS(sessionID,
                            deliverable, defect);
                        printRPMExceptions(result);

                        if (result.isSuccessful())
                        {
                            // check that the project's ScopeFolder
                            // now contains the AbstractScope
                            LoadResult loadResult3 = application.loadFromXpath(
                                sessionID, "/ScopeFolder[name='"
                                        + proposal.getName() + "']",
                                scopeElementScope);

                            ScopeFolder folder = (ScopeFolder) loadResult3
                                .getRpmObjectList()[0];

                            if (folder.getChildren() != null
                                    && folder.getChildren()[0] instanceof Defect
                                    && defect.getID().equals(
                                        folder.getChildren()[0].getID()))
                            {
                                System.out
                                    .println("Defect successfully moved to Deliverable");
                            }
                            else
                            {
                                System.out.println("Move not successful");
                            }
                        }
                    }
                }
            }
        }
        System.out.println("moveWithin - End");
    }

    private static void moveAndTransferWithin(
            WorkElementScope workElementScope,
            ScopeElementScope scopeElementScope) throws Exception
    {
        System.out.println("\nmoveAndTransferWithin - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-moveAndTransferWithin");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save a ChangeRequest");

            // create an AbstractScope and reload it to get a populated project
            // field
            ChangeRequest changeRequest = new ChangeRequest();
            changeRequest.setParent(scopeFolder);
            changeRequest.setName("TC Change Request Name");
            SingleResult singleResult = application.save(sessionID,
                changeRequest, scopeElementScope, ReloadType.ReloadResult);
            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                changeRequest = (ChangeRequest) singleResult.getRpmObject();

                // retrieve the Proposal that contains the ScopeFolder
                LoadResult loadResult2 = application.loadFromXpath(sessionID,
                    "/Proposal[name='" + scopeFolder.getName() + "']",
                    workElementScope);

                printRPMExceptions(loadResult2);

                if (loadResult2 != null && loadResult2.isSuccessful())
                {
                    Proposal proposal = (Proposal) loadResult2
                        .getRpmObjectList()[0];

                    System.out.println("Create and save a SummaryTask");

                    SummaryTask summaryTask = new SummaryTask();
                    summaryTask.setName("TC Summary Task Name");
                    summaryTask.setParent(proposal);
                    SingleResult singleResult2 = application.save(sessionID,
                        summaryTask, workElementScope, ReloadType.ReloadResult);

                    if (singleResult2.isSuccessful())
                    {
                        summaryTask = (SummaryTask) singleResult2
                            .getRpmObject();

                        System.out
                            .println("Move and transfer the ChangeRequest to the SummaryTask");

                        // transfer the AbstractScope to the SummaryTask
                        Result result = application.moveAndTransferUnderWBS(
                            sessionID, summaryTask, changeRequest);
                        printRPMExceptions(result);

                        if (result.isSuccessful())
                        {
                            // check that the proposal's ScopeFolder
                            // now contains the AbstractScope (move)
                            LoadResult loadResult3 = application.loadFromXpath(
                                sessionID, "/ScopeFolder[name='"
                                        + proposal.getName() + "']",
                                scopeElementScope);
                            printRPMExceptions(loadResult3);

                            if (loadResult3 != null
                                    && loadResult3.isSuccessful())
                            {
                                ScopeFolder folder = (ScopeFolder) loadResult3
                                    .getRpmObjectList()[0];

                                if (folder.getChildren() != null
                                        && folder.getChildren()[0] instanceof ChangeRequest
                                        && changeRequest.getID().equals(
                                            folder.getChildren()[0].getID()))
                                {
                                    System.out
                                        .println("ChangeRequest successfully moved to SummaryTask");
                                }
                                else
                                {
                                    System.out.println("Move not successful");
                                }
                            }

                            // check that the SummaryTask now contains the
                            // AbstractScope (transfer);
                            // it will be a SummaryTask that contains a
                            // ChangeRequest scopeElement
                            LoadResult loadResult4 = application.loadFromXpath(
                                sessionID, "/SummaryTask[id='"
                                        + summaryTask.getID() + "']",
                                workElementScope);
                            printRPMExceptions(loadResult4);

                            if (loadResult4 != null
                                    && loadResult4.isSuccessful())
                            {
                                summaryTask = (SummaryTask) loadResult4
                                    .getRpmObjectList()[0];

                                if (summaryTask.getChildren() != null
                                        && summaryTask.getChildren()[0] != null
                                        && summaryTask.getChildren()[0] instanceof SummaryTask
                                        && summaryTask.getChildren()[0]
                                            .getScopeElement() instanceof ChangeRequest)
                                {
                                    System.out.println("Transfer successful");

                                }
                                else
                                {
                                    System.out
                                        .println("Transfer not successful");
                                }
                            }
                        }
                    }
                }
            }
        }
        System.out.println("moveAndTransferWithin - End");
    }

    private static void crossMoveAndTransfer(WorkElementScope workElementScope,
            ScopeElementScope scopeElementScope) throws Exception
    {
        System.out.println("\ncrossMoveAndTransfer - Begin");

        // create the parent ScopeFolder
        LoadResult loadResult = createScopeFolder("TC-crossMoveAndTransfer");
        printRPMExceptions(loadResult);

        if (loadResult != null && loadResult.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) loadResult
                .getRpmObjectList()[0];

            System.out.println("Create and save an Issue");

            // create an AbstractScope and reload it to get a populated project
            // field
            Issue issue = new Issue();
            issue.setParent(scopeFolder);
            issue.setName("TC Issue Name");
            SingleResult singleResult = application.save(sessionID, issue,
                scopeElementScope, ReloadType.ReloadResult);
            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                issue = (Issue) singleResult.getRpmObject();

                System.out.println("Create and save a Project parent");

                // create a WorkElement to move and transfer the AbstractScope
                // to
                Project project = new Project();
                project.setName("TC Project Name");
                SingleResult singleResult2 = application.save(sessionID,
                    project, workElementScope, ReloadType.ReloadResult);
                printRPMExceptions(singleResult2);

                if (singleResult2.isSuccessful())
                {
                    project = (Project) singleResult2.getRpmObject();

                    System.out.println("Create and save a Deliverable");

                    Deliverable deliverable = new Deliverable();
                    deliverable.setName("TC Deliverable Name");
                    deliverable.setParent(project);
                    SingleResult singleResult3 = application.save(sessionID,
                        deliverable, workElementScope, ReloadType.ReloadResult);
                    printRPMExceptions(singleResult3);

                    if (singleResult3.isSuccessful())
                    {
                        deliverable = (Deliverable) singleResult3
                            .getRpmObject();

                        System.out
                            .println("Move and transfer the Issue to the Deliverable");

                        // cross move and transfer the AbstractScope to the
                        // WorkElement
                        Result result = application.moveAndTransferUnderWBS(
                            sessionID, deliverable, issue);
                        printRPMExceptions(result);

                        if (result.isSuccessful())
                        {
                            // the ScopeFolder associated with the project
                            // should also contain the AbstractScope (move)
                            LoadResult loadResult2 = application.loadFromXpath(
                                sessionID, "/ScopeFolder[name='"
                                        + project.getName() + "']",
                                scopeElementScope);
                            printRPMExceptions(loadResult2);

                            if (loadResult2 != null
                                    && loadResult2.isSuccessful())
                            {
                                ScopeFolder folder = (ScopeFolder) loadResult2
                                    .getRpmObjectList()[0];

                                if (folder.getChildren() != null
                                        && folder.getChildren()[0] instanceof Issue)
                                {
                                    System.out
                                        .println("Issue successfully moved to Deliverable");
                                }
                                else
                                {
                                    System.out.println("Move not successful");
                                }
                            }

                            // check that the deliverable now contains the
                            // AbstractScope (transfer);
                            // it will be a SummaryTask that contains an Issue
                            // ScopeElement
                            LoadResult loadResult3 = application.loadFromXpath(
                                sessionID, "/Deliverable[id='"
                                        + deliverable.getID() + "']",
                                workElementScope);
                            printRPMExceptions(loadResult3);

                            if (loadResult3 != null
                                    && loadResult3.isSuccessful())
                            {
                                deliverable = (Deliverable) loadResult3
                                    .getRpmObjectList()[0];

                                if (deliverable.getChildren() != null
                                        && deliverable.getChildren()[0] != null
                                        && deliverable.getChildren()[0] instanceof SummaryTask
                                        && deliverable.getChildren()[0]
                                            .getScopeElement() instanceof Issue)
                                {
                                    System.out.println("Transfer successful");

                                }
                                else
                                {
                                    System.out
                                        .println("Transfer not successful");
                                }

                            }

                        }
                    }
                }
            }
        }
        System.out.println("crossMoveAndTransfer - End");
    }

    protected void runSample() throws Exception
    {
        // setup scope
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setProject(new WorkElementScope());
        scopeElementScope.setChildren(new ScopeElementScope());
        scopeElementScope.setParent(new ScopeElementScope());

        WorkElementScope childrenScope = new WorkElementScope();
        childrenScope.setScopeElement(new ScopeElementScope());

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setChildren(childrenScope);
        workElementScope.setParent(new WorkElementScope());
        workElementScope.setScopeElement(new ScopeElementScope());

        // call sample methods
        transferUnderWBSRoot(workElementScope);
        crossMove(scopeElementScope);
        transferWithin(workElementScope, scopeElementScope);
        moveWithin(workElementScope, scopeElementScope);
        moveAndTransferWithin(workElementScope, scopeElementScope);
        crossMoveAndTransfer(workElementScope, scopeElementScope);
    }
}