/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.timesheet.sample;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class CreateAndUpdateCreditAndCreditTimesheet extends
        SampleTestController
{
    private static TaskAssignment taskAssignment = null;

    private static SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

    private static Calendar weekOf = null;

    private static Calendar creditWeekOf = null;

    private static void createAndSubmitSummaryTimesheetForCreditWeek()
            throws Exception
    {

        System.out.println("\nCreate a SummaryTimesheet for credit week");

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(creditWeekOf);
        summaryTimesheet.setRegularMonday(new Double(8));

        SingleResult result1 = application.save(sessionID, summaryTimesheet,
            null, ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result1.getRpmObject();

            System.out.println("SummaryTimesheet created for week of "
                    + format.format(summaryTimesheet.getWeekOf().getTime()));

            System.out.println("\nSubmit the SummaryTimesheet for credit week");

            TimesheetApprovalStatus approvalStatus = new TimesheetApprovalStatus();
            approvalStatus.setApprovalStatus(TimesheetStatus.Submitted);
            summaryTimesheet.setApprovalStatus(approvalStatus);

            // Add approvalStatus to scope

            SummaryTimesheetScope stScope = new SummaryTimesheetScope();
            stScope.setApprovalStatus(new TimesheetApprovalStatusScope());

            SingleResult result2 = application.save(sessionID,
                summaryTimesheet, stScope, ReloadType.ReloadResult);

            printRPMExceptions(result2);

        }

    }

    private static void createSummaryTimesheet() throws Exception
    {

        System.out.println("\nCreate a SummaryTimesheet");

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        SingleResult result1 = application.save(sessionID, summaryTimesheet,
            null, ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result1.getRpmObject();

            System.out.println("SummaryTimesheet created for week of "
                    + format.format(summaryTimesheet.getWeekOf().getTime()));
        }

    }

    private static void createStepWithStepTimesheet() throws Exception
    {

        System.out.println("\nCreate a Step with a StepTimesheet");

        // Create step
        Step step = new Step();
        step.setName("Sample step");

        // Create stepTimesheet (step must have at least one stepTimesheet)
        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setWeekOf(weekOf);

        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(new StepTimesheet[]
        {
            timesheet
        });

        // Add stepTimesheet (mandatory to save the stepTimesheet)
        StepScope stScope = new StepScope();
        stScope.setTimesheets(true);

        SingleResult result1 = application.save(sessionID, step, stScope,
            ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            step = (Step) result1.getRpmObject();
            timesheet = step.getStepTimesheets()[0];

            System.out
                .println("Step was created with a StepTimesheet for week of "
                        + format.format(timesheet.getWeekOf().getTime()));
        }

    }

    private static void createCreditWithCreditTimesheet() throws Exception
    {

        System.out.println("\nCreate a Credit with a CreditTimesheet");

        // Create credit
        Credit credit = new Credit();
        credit.setName("Sample credit");

        // Create creditTimesheet (credit must have a CreditTimesheet)
        CreditTimesheet timesheet = new CreditTimesheet();
        timesheet.setWeekOf(weekOf);
        timesheet.setCreditWeekOf(creditWeekOf);
        timesheet.setRegularTuesday(new Double(0));

        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(timesheet);

        // Add creditTimesheet (mandatory to save the creditTimesheet)
        // Credit uses the same scope object as the step
        StepScope creditScope = new StepScope();
        creditScope.setTimesheets(true);

        SingleResult result1 = application.save(sessionID, credit, creditScope,
            ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            credit = (Credit) result1.getRpmObject();
            timesheet = credit.getCreditTimesheet();

            System.out
                .println("Credit was created with a CreditTimesheet for week of "
                        + format.format(timesheet.getWeekOf().getTime())
                        + " and credit week of "
                        + format.format(timesheet.getCreditWeekOf().getTime()));
        }

    }

    private static void loadFromXPathAndUpdate() throws Exception
    {

        System.out
            .println("\nLoad a CreditTimesheet from xpath and update the CreditTimesheet");

        CreditTimesheet timesheet = null;

        String xpath = "/Credit[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName
                + "'] and @workElement[name = '"
                + TimesheetSampleSetUp.taskName + "']]/CreditTimesheet";

        System.out.println("xpath: " + xpath);

        ArrayResult result1 = application.loadFromXpath(sessionID, xpath, null);

        printRPMExceptions(result1);

        if (result1.getRpmObjectList() != null)
        {
            timesheet = (CreditTimesheet) result1.getRpmObjectList()[0];
            System.out.println("CreditTimesheet was loaded for week of "
                    + format.format(timesheet.getWeekOf().getTime())
                    + ", Regular tuesday hours = "
                    + timesheet.getRegularTuesday());

            timesheet.setRegularTuesday(new Double(3));

            SingleResult result2 = application.save(sessionID, timesheet, null,
                ReloadType.ReloadResult);

            printRPMExceptions(result2);

            if (result2.isSuccessful())
            {
                timesheet = (CreditTimesheet) result2.getRpmObject();

                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

                System.out.println("CreditTimesheet was updated for week of "
                        + format.format(timesheet.getWeekOf().getTime())
                        + ", Regular tuesday hours = "
                        + timesheet.getRegularTuesday());

            }
        }
        else
        {
            throw new Exception("Unable to load CreditTimesheet from xpath");
        }

    }

    protected void runSample() throws Exception
    {
        System.out.println("\n***CreateAndUpdateCreditAndCreditTimesheet***");

        // Create objects needed to run this code sample
        TimesheetSampleSetUp.setUp();

        System.out.println("\nLoading an existing TaskAssignment from xpath");

        String xpath = "Task[name = '" + TimesheetSampleSetUp.taskName
                + "']/ResourceTaskAssignment[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName + "']]/TaskAssignment";

        System.out.println("xpath: " + xpath);

        LoadResult result1 = application.loadFromXpath(sessionID, xpath, null);

        if (result1.getRpmObjectList() != null)
        {
            taskAssignment = (TaskAssignment) result1.getRpmObjectList()[0];
        }
        else
        {
            throw new Exception(
                "Task assignment not found. Use TimesheetSampleSetup to create the TaskAssignment");
        }

        creditWeekOf = (Calendar) taskAssignment.getAssignmentStartDate()
            .clone();

        weekOf = (Calendar) creditWeekOf.clone();
        weekOf.add(Calendar.WEEK_OF_YEAR, 1);

        // A submitted SummaryTimesheet must exist for the credit week.
        createAndSubmitSummaryTimesheetForCreditWeek();

        // A summaryTimesheet must exist for the week of the CreditTimesheet
        createSummaryTimesheet();

        // A StepTimesheet must exist for the week of the CreditTimesheet
        createStepWithStepTimesheet();

        createCreditWithCreditTimesheet();

        loadFromXPathAndUpdate();

        System.out.println("Done");
    }

    public static void main(String[] args) throws Exception
    {
        new CreateAndUpdateCreditAndCreditTimesheet();
    }
}
