/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.timesheet.sample;

import java.text.SimpleDateFormat;

import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class CreateAndUpdateSummaryTimesheet extends SampleTestController
{

    private static TaskAssignment taskAssignment = null;

    private static SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

    public static void main(String[] args) throws Exception
    {
        new CreateAndUpdateSummaryTimesheet();
    }

    private static void createSummaryTimesheet() throws Exception
    {

        System.out.println("\nCreate a SummaryTimesheet");

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(taskAssignment.getAssignmentStartDate());
        summaryTimesheet.setRegularMonday(new Double(8));

        // Add resource and task to the scope (needed to print these
        // information)
        SummaryTimesheetScope stScope = new SummaryTimesheetScope();
        stScope.setResource(true);
        stScope.setWorkElement(new WorkElementScope());

        SingleResult result1 = application.save(sessionID, summaryTimesheet,
            stScope, ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result1.getRpmObject();

            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

            System.out.println("SummaryTimesheet created for week of "
                    + format.format(summaryTimesheet.getWeekOf().getTime())
                    + ", Resource "
                    + summaryTimesheet.getResource().getFullName() + ", Task "
                    + summaryTimesheet.getWorkElement().getName());
        }

    }

    private static void loadFromXPathAndUpdate() throws Exception
    {

        System.out
            .println("\nLoad the SummaryTimesheet from xpath and update the SummaryTimesheet.");

        SummaryTimesheet summaryTimesheet = null;

        String xpath = "/SummaryTimesheet[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName
                + "'] and @workElement[name = '"
                + TimesheetSampleSetUp.taskName + "']]";

        System.out.println("xpath: " + xpath);

        ArrayResult result1 = application.loadFromXpath(sessionID, xpath, null);

        printRPMExceptions(result1);

        if (result1.getRpmObjectList() != null)
        {

            summaryTimesheet = (SummaryTimesheet) result1.getRpmObjectList()[0];

            System.out.println("SummaryTimesheet was loaded for week of "
                    + format.format(summaryTimesheet.getWeekOf().getTime())
                    + ", Total hours " + summaryTimesheet.getTotalWeek());

            summaryTimesheet.setSpecialThursday(new Double(5));

            SingleResult result2 = application.save(sessionID,
                summaryTimesheet, null, ReloadType.ReloadResult);

            printRPMExceptions(result2);

            if (result2.isSuccessful())
            {
                summaryTimesheet = (SummaryTimesheet) result2.getRpmObject();

                System.out.println("SummaryTimesheet was updated for week of "
                        + format.format(summaryTimesheet.getWeekOf().getTime())
                        + ", Total hours " + summaryTimesheet.getTotalWeek());

            }
        }
        else
        {
            throw new Exception("Unable to load SummaryTimesheet from xpath");
        }

    }

    protected void runSample() throws Exception
    {
        System.out.println("\n***CreateAndUpdateSummaryTimesheet***");

        // Create objects needed to run this code sample
        TimesheetSampleSetUp.setUp();

        System.out.println("\nLoad an existing TaskAssignment from xpath");

        String xpath = "Task[name = '" + TimesheetSampleSetUp.taskName
                + "']/ResourceTaskAssignment[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName + "']]/TaskAssignment";

        System.out.println("xpath: " + xpath);

        LoadResult result1 = application.loadFromXpath(sessionID, xpath, null);

        if (result1.getRpmObjectList() != null)
        {
            taskAssignment = (TaskAssignment) result1.getRpmObjectList()[0];
        }
        else
        {
            throw new Exception(
                "Task assignment not found. Use TimesheetSampleSetup to create the TaskAssignment");
        }

        createSummaryTimesheet();

        loadFromXPathAndUpdate();

        System.out.println("Done");
    }

}
