/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * This sample uploads a new document at the project level and the downloads it
 * back
 * 
 * @desc Upload and download a document from a project.
 * 
 */
public class WBSDocSample extends SampleTestController
{
    static String projectName = "GENERIC_PROJECT_ABC";

    static String contextName = "WBS";

    static String docName = "Hypothesis";

    static String uploadAction = "UPLOADUNZIPPED";

    static String dwnldAction = "DOWNLOADZIPPED";

    /*
     * What will be uploaded to the servlet engine
     */
    public static String filePath = "C:\\test.doc";

    /*
     * The name of the file when it will be read back
     */
    public static String filePathOut = "C:\\test.doc.zip";

    private static int BUFFER_SIZE = 2048;

    protected void runSample() throws Exception
    {
        WorkElementScope pScope = new WorkElementScope();
        pScope.setDocumentFolder(new DocumentScope());
        pScope.getDocumentFolder().setChildren(new DocumentScope());

        Project project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", pScope).getRpmObjectList()[0];

        Document wNewDoc = new Document();
        wNewDoc.setName(docName);
        wNewDoc.setParent(project.getDocumentFolder());

        SaveResult pSave = save(wNewDoc, null, ReloadType.ReloadResult);
        wNewDoc = (Document) pSave.getRpmObject();

        uploadFile(wNewDoc);

        project.getDocumentFolder().setChildren(new DocumentElement[]
        {
            wNewDoc
        });

        pSave = save(project, pScope, ReloadType.ReloadResult);
        project = (Project) pSave.getRpmObject();

        downloadFile();
    }

    /**
     * Uploads the file in question using the upload servlet located on the same
     * container as the web services API.
     * 
     * @throws IOException
     * 
     */
    private void uploadFile(Document doc) throws IOException
    {
        File inputFile = new File(filePath);

        /*
         * Set up an http POST request by setting up the header properly and
         * establish the connection to the servlet. The address of the servlet
         * we use to upload the test file
         * 
         * http://127.0.0.1:8080/rpm/servlet/DocumentTransfer"
         */
        URL uploadUrl = new URL("http", "localhost", 8080,
            "/rpm/servlet/DocumentTransfer");
        HttpURLConnection uploadConnection = (HttpURLConnection) uploadUrl
            .openConnection();
        uploadConnection.setDoOutput(true);
        uploadConnection.setRequestMethod("POST");

        uploadConnection.addRequestProperty("ACTION", uploadAction);
        uploadConnection.addRequestProperty("DOCUMENT_ID", doc.getID());
        uploadConnection.addRequestProperty("CONTEXT_NAME", contextName);
        uploadConnection.addRequestProperty("SESSION_ID", sessionID);
        uploadConnection.addRequestProperty("FILE", filePath);
        uploadConnection.addRequestProperty("CONTENT_LENGTH", ""
                + inputFile.length());

        uploadConnection.connect();

        /*
         * Transfer the file
         */
        BufferedInputStream fromFile = new BufferedInputStream(
            new FileInputStream(inputFile));

        BufferedOutputStream toServlet = new BufferedOutputStream(
            uploadConnection.getOutputStream());

        byte[] buffer = new byte[BUFFER_SIZE];
        while (fromFile.read(buffer) != -1)
        {
            toServlet.write(buffer);
        }

        /*
         * Verify if the transfer completed successfully
         */
        System.out.println("HTTP Upload result : "
                + uploadConnection.getResponseMessage());

        uploadConnection.disconnect();
    }

    /**
     * Retrieve the document via HTTP using the document's database-generated ID
     * 
     * @throws IOException
     * 
     */
    private void downloadFile() throws IOException
    {
        Document toRetrieve = (Document) loadFromXpath(
            "/Document[name='" + docName + "']", null).getRpmObjectList()[0];

        /*
         * Craft the HTTP request URL
         */
        URL downloadURL = new URL("http", "localhost", 8080,
            "/rpm/servlet/DocumentTransfer?ACTION=" + dwnldAction
                    + "&DOCUMENT_ID=" + toRetrieve.getID() + "&CONTEXT_NAME="
                    + contextName + "&SESSION_ID=" + sessionID);

        HttpURLConnection downloadConnection = (HttpURLConnection) downloadURL
            .openConnection();
        downloadConnection.setRequestMethod("GET");
        downloadConnection.connect();

        /*
         * Restore the filename of the object we're retrieving
         */
        String filename = "filename=\"";
        String contentDisposition = downloadConnection
            .getHeaderField("Content-Disposition");
        int start = filename.length() + contentDisposition.indexOf(filename);
        int end = contentDisposition.lastIndexOf('\"');
        filename = contentDisposition.substring(start, end);
        System.out.println("File name as restored form the repository: "
                + filename + " fileSize: "
                + downloadConnection.getContentLength());

        BufferedInputStream fromServlet = new BufferedInputStream(
            downloadConnection.getInputStream());

        FileOutputStream toFile = new FileOutputStream(new File(filePathOut));

        byte[] buffer = new byte[BUFFER_SIZE];
        while (fromServlet.read(buffer) != -1)
        {
            toFile.write(buffer);
        }

        toFile.close();
        fromServlet.close();

        System.out.println("Response code from download : "
                + downloadConnection.getResponseMessage());
    }

    public static void main(String[] args)
    {
        new WBSDocSample();
    }
}
