/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * This sample loads a project, assigns it an opportunity and stores it into the
 * repository
 * 
 * @desc Assign an opportunity to a project.
 * 
 */
public class WBSFillOpportunity extends SampleTestController
{
    static String projectName = "GENERIC_PROJECT_ABC";

    static String conNumber = "CONTACT_NUMBER";

    static String conType = "CONTACT_TYPE";

    static String cusNumber = "CUSTOMER_NUMBER";

    static String exClient = "EXTERNAL_CLIENT";

    static String oppOwner = "OPPORTUNITY_OWNER";

    static String oppTrNumber = "OPPORTUNITY_TRACKING_NUMBER";

    static int benBegin = 14;

    static int benDuration = 10;

    static double magnitude = 100000.00;

    protected void runSample() throws Exception
    {
        WorkElementScope weScope = new WorkElementScope();
        weScope.setOpportunity(true);

        Project project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", weScope)
            .getRpmObjectList()[0];

        /**
         * Create the opportunity object and initialize it
         */
        Opportunity opportunity = new Opportunity();
        Calendar future = Calendar.getInstance();
        future.roll(Calendar.YEAR, true);

        opportunity.setForecastClosed(future);
        opportunity.setPursuitFinish(future);
        opportunity.setPursuitStart(Calendar.getInstance());

        opportunity.setBenefitBeginsMonths(new Integer(benBegin));
        opportunity.setBenefitDurationMonths(new Integer(benDuration));
        opportunity.setOrderOfMagnitudeCost(new Double(magnitude));

        opportunity.setContactNumber(conNumber);
        opportunity.setContactType(conType);
        opportunity.setCustomerNumber(cusNumber);
        opportunity.setExternalClient(exClient);
        opportunity.setOpportunityOwner(oppOwner);
        opportunity.setOpportunityTrackingNumber(oppTrNumber);
        opportunity.setOrderOfMagnitudeGPSavingsPercent(new Double(0));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(0));

        /**
         * Associate the project and the opportunity
         */
        opportunity.setParent(project);

        save(opportunity, null, ReloadType.None);

    }

    public static void main(String[] args)
    {
        new WBSFillOpportunity();
    }

}
