/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * Create a number of projects under a parent portfolio
 * 
 * @desc Create projects under a parent portfolio
 * 
 */
public class WBSProjectSample extends SampleTestController
{
    static String projectExternalID = "external";

    static String projectName = "GENERIC_PROJECT_ABC";

    static String parentProjectExternalID = "parentExternalID";

    static String chargeCodeName = "charge code name";

    static String portName = "PORTFOLIO_ABC";

    protected void runSample() throws Exception
    {
        PortfolioScope pScope = new PortfolioScope();
        pScope.setProjects(new WorkElementScope());

        LoadResult result = loadFromXpath("/Portfolio[@name='" + portName
                + "']", pScope);

        /**
         * Try to load the portfolio from the repository. If it doesn't exist,
         * create it
         */
        Portfolio parentProject = null;
        if (result.getRpmObjectList() == null
                || result.getRpmObjectList().length == 0)
        {
            parentProject = new Portfolio();
            parentProject.setName(portName);

            SaveResult saveResult = save(parentProject, pScope,
                ReloadType.ReloadResult);
            parentProject = (Portfolio) saveResult.getRpmObject();
        }
        else
        {
            parentProject = (Portfolio) result.getRpmObjectList()[0];
        }

        Proposal proposal = createProposal(parentProject);

        Project project = createProject(parentProject);

        WorkOrganization organization = createOrganization(parentProject);

        parentProject.setProjects(new GenericProject[]
        {
                proposal, project, organization
        });

        save(parentProject, pScope, ReloadType.None);
    }

    /**
     * Save an organization in the repository and return it
     * 
     * @param parent
     * @return
     */
    private WorkOrganization createOrganization(Portfolio parent)
    {
        WorkOrganization org = (WorkOrganization) createGenericProject(
            new WorkOrganization(), parent);

        SaveResult save = save(org, null, ReloadType.ReloadResult);

        return (WorkOrganization) save.getRpmObject();
    }

    /**
     * Create a project in the repository and return it
     * 
     * @param parent
     * @return
     */
    private Project createProject(Portfolio parent)
    {
        Project project = (Project) createGenericProject(new Project(), parent);

        SaveResult save = save(project, null, ReloadType.ReloadResult);

        return (Project) save.getRpmObject();
    }

    /**
     * Create a proposal and store it in the repository
     * 
     * @param parentProject
     * @return
     */
    private Proposal createProposal(Portfolio parentProject)
    {
        Proposal proposal = (Proposal) createGenericProject(new Proposal(),
            parentProject);

        SaveResult save = save(proposal, null, ReloadType.ReloadResult);
        proposal = (Proposal) save.getRpmObject();

        return proposal;
    }

    /**
     * Create a new project in the repository
     * 
     * @param projectToUpdate
     * @param parent
     * @return
     */
    private GenericProject createGenericProject(GenericProject projectToUpdate,
            Portfolio parent)
    {
        projectToUpdate.setExternalID(projectExternalID);
        projectToUpdate.setName(projectName);
        projectToUpdate.setPortfolio(parent);

        RPMCalendar projectCalendar = (RPMCalendar) loadFromXpath(
            "/RPMCalendar[@name='Standard (Default)']", null)
            .getRpmObjectList()[0];

        projectToUpdate.setCalendar(projectCalendar);

        Currency projectCurrency = (Currency) loadFromXpath(
            "/Currency[@name='Canadian dollars" + "']", null)
            .getRpmObjectList()[0];

        projectToUpdate.setCurrency(projectCurrency);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        endDate.roll(Calendar.YEAR, true);

        projectToUpdate.setEstimatedStartDate(startDate);
        projectToUpdate.setEstimatedFinishDate(endDate);
        projectToUpdate.setAssignmentType(AssignmentType.Work);
        projectToUpdate.setPublished(ProjectPublishedType.None);

        return projectToUpdate;
    }

    public static void main(String[] args)
    {
        new WBSProjectSample();
    }
}
