/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * This sample creates a custom scorecard, made of questions and answers, then
 * assigns it to a project
 * 
 * @desc Assign a custom scorecard to a project.
 * 
 */
public class WBSProjectScoreCardExample extends SampleTestController
{
    static String projectName = "GENERIC_PROJECT_ABC";

    static String scfName = "SCORECARD_FOLDER";

    static String scName = "SCORECARD";

    static String sccName = "SCORECARD_CATEGORY";

    static String scqName = "SCORECARD_QUESTION";

    static String scrName = "SCORECARD_RESPONSE";

    /**
     * number of questions
     */
    final static int nbrQ = 5;

    /**
     * number of responses per question
     */
    static int nbrR = 3;

    /**
     * The weight of each question
     */
    static int iWeight = 1;

    protected void runSample() throws Exception
    {
        /**
         * Indicate that we wish to operate on the object graph downwards, down
         * to responses
         */
        WorkElementScope pScope = new WorkElementScope();

        pScope.setAssignedScorecards(new ScorecardScope());
        pScope.getAssignedScorecards().setParent(new RPMObjectScope());
        pScope.getAssignedScorecards().setScorecardCategories(
            new ScorecardCategoryScope());
        pScope.getAssignedScorecards().getScorecardCategories()
            .setScorecardQuestions(new ScorecardQuestionScope());

        pScope.getAssignedScorecards().getScorecardCategories()
            .getScorecardQuestions().setScorecardResponses(
                new ScorecardResponseScope());

        Project project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", pScope).getRpmObjectList()[0];

        ScorecardFolder scFolder = new ScorecardFolder();
        Scorecard sCard = new Scorecard();
        ScorecardCategory scCat = new ScorecardCategory();

        WbsModule module = new WbsModule();
        scFolder.setName(scfName);

        /**
         * initialize the scorecard
         */
        sCard.setName(scName);
        sCard.setWeight(new Integer(iWeight));
        sCard.setProbabilityType(ProbabilityType.None);
        sCard.setParent(scFolder);

        scCat.setName(sccName);
        scCat.setParent(sCard);
        scCat.setWeight(new Integer(iWeight));

        ScorecardQuestion[] questions = new ScorecardQuestion[nbrQ];

        /**
         * For each question, create all of its possible responses
         */
        ScorecardQuestion scQuestion = null;
        ScorecardResponse[] responses = null;
        ScorecardResponse scResponse = null;
        for (int i = 0; i < nbrQ; i++)
        {
            scQuestion = new ScorecardQuestion();
            scQuestion.setName(scqName + i);
            scQuestion.setParent(scCat);
            scQuestion.setWeight(new Integer(iWeight));

            responses = new ScorecardResponse[nbrR];

            /**
             * Add each possible repsonse, one by one
             */
            for (int j = 0; j < nbrR; j++)
            {
                scResponse = new ScorecardResponse();
                scResponse.setName(scrName + j);
                scResponse.setParent(scQuestion);
                scResponse.setWeight(new Integer(iWeight));

                /**
                 * Each question can have only one default response
                 */
                if (j == 0)
                {
                    scResponse.setDefaultResponse(Boolean.TRUE);
                }
                else
                {
                    scResponse.setDefaultResponse(Boolean.FALSE);
                }

                responses[j] = scResponse;
            }

            scQuestion.setScorecardResponses(responses);

            questions[i] = scQuestion;
        }

        scCat.setScorecardQuestions(questions);

        sCard.setScorecardCategories(new ScorecardCategory[]
        {
            scCat
        });
        sCard.setProbabilityType(ProbabilityType.None);

        scFolder.setScorecards(new Scorecard[]
        {
            sCard
        });

        module.setScorecardFolders(new ScorecardFolder[]
        {
            scFolder
        });

        /**
         * Create the scope object deep enough to save the object graph we have
         * created, down to each response, then save it.
         */
        WbsModuleScope wbsmScope = new WbsModuleScope();
        wbsmScope.setScorecardFolders(new ScorecardFolderScope());
        wbsmScope.getScorecardFolders().setScorecards(new ScorecardScope());
        wbsmScope.getScorecardFolders().getScorecards().setScorecardCategories(
            new ScorecardCategoryScope());

        wbsmScope.getScorecardFolders().getScorecards()
            .getScorecardCategories().setScorecardQuestions(
                new ScorecardQuestionScope());
        wbsmScope.getScorecardFolders().getScorecards()
            .getScorecardCategories().getScorecardQuestions()
            .setScorecardResponses(new ScorecardResponseScope());

        SaveResult result = save(module, wbsmScope, ReloadType.ReloadResult);

        WbsModule mod = (WbsModule) result.getRpmObject();
        scFolder = mod.getScorecardFolders()[0];
        sCard = scFolder.getScorecards()[0];

        sCard.setActive(Boolean.TRUE);

        SingleResult res = application.assignScoreCard(sessionID, project,
            pScope, sCard, ReloadType.None);
        if (!res.isSuccessful())
        {
            System.err.println("Failed assigning the scorecard");
        }

        /**
         * Reload the project that contains the newly-created assignedCard
         */
        project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", pScope).getRpmObjectList()[0];

        ScorecardScope publishScope = new ScorecardScope();

        publishScope.setScorecardCategories(new ScorecardCategoryScope());

        publishScope.getScorecardCategories().setScorecardQuestions(
            new ScorecardQuestionScope());

        publishScope.getScorecardCategories().getScorecardQuestions()
            .setScorecardResponses(new ScorecardResponseScope());

        AssignedScorecard assign = project.getAssignedScorecards()[0];
        assign.setParent(project);

        /**
         * Publish the scorecard so it can be viewed as a report.
         */
        res = application.publishScorecard(sessionID, assign, publishScope);
        if (!res.isSuccessful())
        {
            System.err.println("Failed to publish the scoprecard");
        }

        result = save(project, null, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new WBSProjectScoreCardExample();
    }
}
