/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;

/**
 * Create three scorecards on different objects : Assets, WBS and
 * ScopeManagement
 * 
 * @desc Create three types of scorecards.
 * 
 */
public class WBSScorecardSample extends SampleTestController
{
    public static String projectName = "GENERIC_PROJECT_ABC";

    public static String scfName = "SCORECARD_FOLDER";

    public static String scName = "SCORECARD";

    public static String sccName = "SCORECARD_CATEGORY";

    public static String scqName = "SCORECARD_QUESTION";

    public static String scrName = "SCORECARD_RESPONSE";

    // number of questions
    public static int nbrQ = 5;

    // number of responses per question
    public static int nbrR = 3;

    public static int iWeight = 1;

    protected void runSample() throws Exception
    {
        // create every object needed to
        // create a complete scorecard
        ScorecardFolder scFolder = new ScorecardFolder();
        Scorecard sCard = new Scorecard();
        ScorecardCategory scCat = new ScorecardCategory();
        ScorecardQuestion scQuestion = null;
        ScorecardResponse scResponse = null;

        scFolder.setName(scfName);
        sCard.setName(scName);
        sCard.setWeight(new Integer(iWeight));
        sCard.setProbabilityType(ProbabilityType.None);
        sCard.setParent(scFolder);

        scCat.setName(sccName);
        scCat.setParent(sCard);
        scCat.setWeight(new Integer(iWeight));

        ScorecardQuestion[] questions = new ScorecardQuestion[nbrQ];

        /**
         * Add question by question
         */
        for (int i = 0; i < nbrQ; i++)
        {
            scQuestion = new ScorecardQuestion();
            scQuestion.setName(scqName + i);
            scQuestion.setParent(scCat);
            scQuestion.setWeight(new Integer(iWeight));

            /**
             * For each possible response
             */
            ScorecardResponse[] responses = new ScorecardResponse[nbrR];
            for (int j = 0; j < nbrR; j++)
            {
                scResponse = new ScorecardResponse();
                scResponse.setName(scrName + j);
                scResponse.setParent(scQuestion);
                scResponse.setWeight(new Integer(iWeight));

                /**
                 * Scorecard questions must have one scorecard response set to
                 * the default
                 */
                if (j == 0)
                {
                    scResponse.setDefaultResponse(Boolean.TRUE);
                }
                else
                {
                    scResponse.setDefaultResponse(Boolean.FALSE);
                }

                responses[j] = scResponse;
            }

            /**
             * Assign responses to the question
             */
            scQuestion.setScorecardResponses(responses);

            questions[i] = scQuestion;
        }

        scCat.setScorecardQuestions(questions);

        /**
         * Assign a scorecardCategory to the scorecard
         */
        sCard.setScorecardCategories(new ScorecardCategory[]
        {
            scCat
        });

        scFolder.setScorecards(new Scorecard[]
        {
            sCard
        });

        saveWbsScorecardFolder(scFolder);
        saveAssetScorecardFolder(scFolder);
        saveScopeManagementScorecardFoler(scFolder);
    }

    private void saveScopeManagementScorecardFoler(ScorecardFolder scFolder)
    {
        ScopeManagementModule module = new ScopeManagementModule();

        scFolder.setParent(module);
        module.setScorecardFolders(new ScorecardFolder[]
        {
            scFolder
        });

        /**
         * Initialize the ModuleScope so that it includes the full structure of
         * the scorecard we wish to create
         */
        ScopeManagementModuleScope mScope = new ScopeManagementModuleScope();
        mScope.setScorecardFolders(new ScorecardFolderScope());
        mScope.getScorecardFolders().setScorecards(new ScorecardScope());
        mScope.getScorecardFolders().getScorecards().setScorecardCategories(
            new ScorecardCategoryScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .setScorecardQuestions(new ScorecardQuestionScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .getScorecardQuestions().setScorecardResponses(
                new ScorecardResponseScope());
        /**
         * Save the scorecard folder by saving the module first. A new scorecard
         * folder cannot be saved on its own. Use the ModuleScope to save every
         * element of the scorecard desired
         */
        save(module, mScope, ReloadType.ReloadResult);
    }

    private void saveAssetScorecardFolder(ScorecardFolder scFolder)
    {
        // create a module to save the score card folder
        AssetModule module = new AssetModule();

        scFolder.setParent(module);
        module.setScorecardFolders(new ScorecardFolder[]
        {
            scFolder
        });

        /**
         * Initialize the ModuleScope so that it includes the full structure of
         * the scorecard we wish to create
         */
        AssetModuleScope mScope = new AssetModuleScope();
        mScope.setScorecardFolders(new ScorecardFolderScope());
        mScope.getScorecardFolders().setScorecards(new ScorecardScope());
        mScope.getScorecardFolders().getScorecards().setScorecardCategories(
            new ScorecardCategoryScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .setScorecardQuestions(new ScorecardQuestionScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .getScorecardQuestions().setScorecardResponses(
                new ScorecardResponseScope());

        save(module, mScope, ReloadType.ReloadResult);
    }

    private void saveWbsScorecardFolder(ScorecardFolder scFolder)
    {
        WbsModule module = new WbsModule();
        scFolder.setParent(module);
        module.setScorecardFolders(new ScorecardFolder[]
        {
            scFolder
        });

        /**
         * Initialize the ModuleScope so that it includes the full structure of
         * the scorecard we wish to create
         */
        WbsModuleScope mScope = new WbsModuleScope();
        mScope.setScorecardFolders(new ScorecardFolderScope());
        mScope.getScorecardFolders().setScorecards(new ScorecardScope());
        mScope.getScorecardFolders().getScorecards().setScorecardCategories(
            new ScorecardCategoryScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .setScorecardQuestions(new ScorecardQuestionScope());
        mScope.getScorecardFolders().getScorecards().getScorecardCategories()
            .getScorecardQuestions().setScorecardResponses(
                new ScorecardResponseScope());

        save(module, mScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new WBSScorecardSample();
    }

}
