/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.util.AttributeValidator;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;

public class AttributeCheckpoint extends GenericAttributeCheckpoint
{
    private static final List ATTRIBUTE_PARENTS = new ArrayList();

    protected static AttributeCheckpoint instance = new AttributeCheckpoint();

    public static AttributeCheckpoint getInstance()
    {
        return instance;
    }

    static
    {
        ATTRIBUTE_PARENTS.add(AttributeClassification.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (!AttributeValidator.validateInstance(rpmObject,
            Attribute.class, msgCtx))
        {
            return;
        }
        super.validateSave(rpmObject, msgCtx, true, ATTRIBUTE_PARENTS);

        Attribute attribute = (Attribute) rpmObject;

        // if the object has been already created then inputType
        // cannot be null
        if (attribute.getID() != null)
        {
            AttributeValidator.validateNotNull(attribute,
                ValidationConstants.INPUT_TYPE_FIELD, attribute.getInputType(),
                msgCtx);
        }
    }

    public void validateDelete(RPMObject rpmObject,
        MessageContext messageContext)
    {
        String id = rpmObject.getID();

        if( id == null )
        {
            AttributeValidator.validateCannotDelete(rpmObject, messageContext);
        }
        else if ( id.indexOf("_") != -1 )
        {
            AttributeValidator.validateCannotDelete(rpmObject, messageContext);
        }
        else
        {
            super.validateDelete(rpmObject, messageContext);
        }
    }

}