/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.applicationadministration.util.AttributeValidator;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;

public class GenericAttributeCheckpoint extends AbstractCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx,
            boolean parentRequired, List validParentTypes)
    {
        super.isValidParent(rpmObject, msgCtx, parentRequired,
            validParentTypes);

        // Initialize
        GenericAttribute attribute = (GenericAttribute) rpmObject;

        boolean valid = AttributeValidator.validateMandatory(attribute,
            ValidationConstants.NAME_FIELD, attribute.getName(), msgCtx);
        
        if( rpmObject.getID() == null )
        {
            AttributeValidator
                .validateMustBeNull(
                    rpmObject,
                    com.ibm.rpm.framework.constants.ValidationConstants.CONTEXT_NAME_FIELD,
                    rpmObject.getContextName(),
                    "Context Name must be null during creation.", msgCtx);
        }
        
        if (valid && attribute.testNameModified())
        {
            AttributeValidator.validateMaxLength(attribute,
                ValidationConstants.NAME_FIELD, attribute.getName(),
                ValidationConstants.ATTRIBUTE_NAME_MAX, msgCtx);
            AttributeValidator.validateMinLength(attribute,
                ValidationConstants.NAME_FIELD, attribute.getName(),
                ValidationConstants.ATTRIBUTE_NAME_MIN, msgCtx);
        }

        if (attribute.testDescriptionModified())
        {
            AttributeValidator.validateMaxLength(attribute,
                ValidationConstants.DESCRIPTION_FIELD, attribute
                    .getDescription(),
                ValidationConstants.ATTRIBUTE_DESCRIPTION_MAX, msgCtx);
        }

        // Field validation: parent
        if (valid && attribute.testParentModified())
        {
            // ensure that the attribute has a context (i.e. the root is
            // a Module object)
            if (rpmObject.getParent() != null)
            {
                AttributeValidator.validateAttributeRoot(attribute, msgCtx);
            }
        }
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractCheckpoint#validateDelete(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.MessageContext)
     */
    public void validateDelete(RPMObject rpmObject,
        MessageContext messageContext)
    {
        String id = rpmObject.getID();
        
        if( id == null )
        {
            AttributeValidator.validateCannotDelete(rpmObject, messageContext);
        }
        else if ( id.indexOf("_") != -1 )
        {
            AttributeValidator.validateCannotDelete(rpmObject, messageContext);
        }
        else
        {
            super.validateDelete(rpmObject, messageContext);
        }
    }
}