/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.City;
import com.ibm.rpm.applicationadministration.containers.Country;
import com.ibm.rpm.applicationadministration.containers.CountryState;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Location;
import com.ibm.rpm.applicationadministration.containers.Region;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class GeographicalCheckpoint extends AbstractCheckpoint
{
    private static final List LOCATION_PARENTS = new ArrayList();

    private static final List CITY_PARENTS = new ArrayList();

    private static final List COUNTRYSTATE_PARENTS = new ArrayList();

    private static final List REGION_PARENTS = new ArrayList();

    private static final List COUNTRY_PARENTS = new ArrayList();

    static
    {
        COUNTRY_PARENTS.add(AdministrationModule.class);
        COUNTRY_PARENTS.add(Region.class);
        
        LOCATION_PARENTS.add(Country.class);
        LOCATION_PARENTS.add(Region.class);
        LOCATION_PARENTS.add(City.class);
        LOCATION_PARENTS.add(CountryState.class);
        LOCATION_PARENTS.add(Location.class);

        CITY_PARENTS.add(Country.class);
        CITY_PARENTS.add(Region.class);
        CITY_PARENTS.add(CountryState.class);

        COUNTRYSTATE_PARENTS.add(Country.class);
        COUNTRYSTATE_PARENTS.add(Region.class);

        REGION_PARENTS.add(AdministrationModule.class);
        REGION_PARENTS.add(Country.class);
        REGION_PARENTS.add(Region.class);
    }

    protected static GeographicalCheckpoint instance = new GeographicalCheckpoint();

    public static GeographicalCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (!GenericValidator.validateInstance(rpmObject, Geographical.class, msgCtx))
            return;

        if (rpmObject instanceof Location)
        {
            super.isValidParent(rpmObject, msgCtx, true, LOCATION_PARENTS);
        }
        else if (rpmObject instanceof City)
        {
            super.isValidParent(rpmObject, msgCtx, true, CITY_PARENTS);
        }
        else if (rpmObject instanceof CountryState)
        {
            super.isValidParent(rpmObject, msgCtx, true, COUNTRYSTATE_PARENTS);
        }
        else if (rpmObject instanceof Region)
        {
            super.isValidParent(rpmObject, msgCtx, false, REGION_PARENTS);
        }
        else if (rpmObject instanceof Country)
        {
            super.isValidParent(rpmObject, msgCtx, false, COUNTRY_PARENTS);
        }
        else
        {
            List expectedTypes = new ArrayList();
            
            expectedTypes.add( Location.class );
            expectedTypes.add( City.class );
            expectedTypes.add( CountryState.class );
            expectedTypes.add( Region.class );
            expectedTypes.add( Country.class );

            GenericValidator.validateCannotCreate(rpmObject, expectedTypes,
                msgCtx);
        }

        // Initialize
        Geographical mainGeographical = (Geographical) rpmObject;

        // Field validation: name
        if (mainGeographical.testNameModified())
        {
            GenericValidator.validateMaxLength(mainGeographical, ValidationConstants.NAME_FIELD,
                mainGeographical.getName(), ValidationConstants.GEOGRAPHICAL_NAME_MAX,
                msgCtx);
        }

        // Field validation: description
        if (mainGeographical.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainGeographical,
                ValidationConstants.DESCRIPTION_FIELD, mainGeographical.getDescription(),
                ValidationConstants.GEOGRAPHICAL_DESCRIPTION_MAX, msgCtx);
        }       
        
    }    
    
}
