/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.NoteMinutesRTF;
import com.ibm.rpm.applicationadministration.containers.ProjectRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.applicationadministration.containers.WorkProductRTF;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author gosselin
 */
public class RtfAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final List RTFASSIGNMENT_PARENTS = new ArrayList();

    static
    {
        RTFASSIGNMENT_PARENTS.add(ClientCostCenter.class);
        RTFASSIGNMENT_PARENTS.add(NoteMinutes.class);
        RTFASSIGNMENT_PARENTS.add(Asset.class);
        RTFASSIGNMENT_PARENTS.add(Action.class);
        RTFASSIGNMENT_PARENTS.add(ChangeRequest.class);
        RTFASSIGNMENT_PARENTS.add(Defect.class);
        RTFASSIGNMENT_PARENTS.add(Issue.class);
        RTFASSIGNMENT_PARENTS.add(Requirement.class);
        RTFASSIGNMENT_PARENTS.add(Risk.class);
        RTFASSIGNMENT_PARENTS.add(ServiceRequest.class);
        RTFASSIGNMENT_PARENTS.add(Deliverable.class);
        RTFASSIGNMENT_PARENTS.add(Milestone.class);
        RTFASSIGNMENT_PARENTS.add(GenericProject.class);
        RTFASSIGNMENT_PARENTS.add(SummaryTask.class);
        RTFASSIGNMENT_PARENTS.add(Task.class);
        RTFASSIGNMENT_PARENTS.add(WorkProduct.class);
        RTFASSIGNMENT_PARENTS.add(ScopeTask.class);
        RTFASSIGNMENT_PARENTS.add(ScopeMilestone.class);
        RTFASSIGNMENT_PARENTS.add(ScopeDeliverable.class);
    }

    protected static RtfAssignmentCheckpoint instance = new RtfAssignmentCheckpoint();

    public static RtfAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, RTFASSIGNMENT_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, RtfAssignment.class,
            msgCtx))
            return;

        // Initialize
        RtfAssignment mainRtfA = (RtfAssignment) rpmObject;

        // Field validation: rtf
        GenericValidator.validateMandatoryID(mainRtfA, "rtf",
            mainRtfA.getRtf(), msgCtx);

        // parent and rtf field validation
        RPMObject parent = mainRtfA.getParent();

        // Make the parent the ScopeElement parent, if its a SummaryTask that
        // has been transferred under WBS
        if (parent instanceof SummaryTask || parent instanceof Task)
        {
            try
            {
                parent = ((WorkElement) ScopeManagementUtil.loadScopeElement(
                    rpmObject.getParent(), msgCtx)).getScopeElement();
            }
            catch (Exception e)
            {
                msgCtx.addExceptionNoThrow(e);
            }
        }

        if (parent instanceof Action)
        {
            GenericValidator.validateType(mainRtfA, ActionRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Asset)
        {
            GenericValidator.validateType(mainRtfA, AssetRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof ChangeRequest)
        {
            GenericValidator.validateType(mainRtfA, ChangeRequestRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof ClientCostCenter)
        {
            GenericValidator.validateType(mainRtfA, ClientRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Deliverable
                || parent instanceof ScopeDeliverable)
        {
            GenericValidator.validateType(mainRtfA, DeliverableRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Defect)
        {
            GenericValidator.validateType(mainRtfA, DefectRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Issue)
        {
            GenericValidator.validateType(mainRtfA, IssueRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Milestone
                || parent instanceof ScopeMilestone)
        {
            GenericValidator.validateType(mainRtfA, MilestoneRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof NoteMinutes)
        {
            GenericValidator.validateType(mainRtfA, NoteMinutesRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof GenericProject)
        {
            GenericValidator.validateType(mainRtfA, ProjectRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Requirement)
        {
            GenericValidator.validateType(mainRtfA, RequirementRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Risk)
        {
            GenericValidator.validateType(mainRtfA, RiskRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof ServiceRequest)
        {
            GenericValidator.validateType(mainRtfA, ServiceRequestRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof Task || parent instanceof ScopeTask)
        {
            GenericValidator.validateType(mainRtfA, TaskRTF.class, "rtf",
                mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof WorkProduct)
        {
            GenericValidator.validateType(mainRtfA, WorkProductRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }
        else if (parent instanceof SummaryTask
                && parent.retrieveTypeID().intValue() == SummaryTask.TYPE_ID)
        {
            // Not tranferred
            GenericValidator.validateType(mainRtfA, SummaryTaskRTF.class,
                "rtf", mainRtfA.getRtf(), msgCtx);
        }

        // Field validation: value
        GenericValidator.validateMaxLength(mainRtfA, "value",
            mainRtfA.getValue(), ValidationConstants.RTFASSIGNMENT_VALUE_MAX,
            msgCtx);
    }
}
