/*
 * Created on Aug 11, 2005
 *
 */
package com.ibm.rpm.applicationadministration.managers;

import java.lang.reflect.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.applicationadministration.util.AttributeUtil;
import com.ibm.rpm.applicationadministration.util.InputTypeUtil;
import com.ibm.rpm.applicationadministration.util.SelectTypeUtil;
import com.ibm.rpm.asset.managers.AssetModuleManager;
import com.ibm.rpm.clientcostcenters.managers.ClientModuleManager;
import com.ibm.rpm.document.managers.DocumentModuleManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.managers.PoolModuleManager;
import com.ibm.rpm.scopemanagement.managers.ScopeManagementModuleManager;
import com.ibm.rpm.wbs.managers.WbsModuleManager;

/**
 * @author karim
 *
 */
public class AttributeManager extends AbstractRPMObjectManager {

    private static final HashMap FIELDPROPERTYMAP = new HashMap();   
    
	private static String CATEGORY_CONTAINER_NAME = "ATTRIBUTECATEGORY";
	private static String CLASSIFICATION_CONTAINER_NAME = "ATTRIBUTECLASSIFICATION";
	private static String ATTRIBUTE_CONTAINER_NAME = "ATTRIBUTE";
	
    // =============
    // CONTAINER MAP
    // =============

    private static Class[] containerClasses =
    {
            GenericAttribute.class, AttributeCategory.class,
            AttributeClassification.class,Attribute.class
    };

    private static ContainerMap containerMap = 
        new ContainerMap(AttributeManager.class, containerClasses);
    static
    {
        containerMap.addExtraTypeIds(AttributeClassification.class, 
                new int[] {AttributeClassification.TYPE_ID2});
        containerMap.addExtraTypeIds(Attribute.class, 
                new int[] {Attribute.TYPE_ID2});
    }
    
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "LEVEL_ID";
    public static final String PROPERTY_LEVEL_ID = "LEVEL_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID, NAME_LEVEL_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TYPE_ID";
    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "MAX_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * 
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "CHILD_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     * 
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "DELETED_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;
    public static final String NAME_ELEMENT_NAME = "ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "PARENT_ID";
    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * 
     */
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * 
     */
    public static final int ID_REC_STATUS = 10;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * 
     */
    public static final int ID_REC_DATETIME = 11;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column MULTI_SELECT of type Types.CHAR mapped to String.
     * 
     */
    public static final int ID_MULTI_SELECT = 12;
    public static final int TYPE_MULTI_SELECT = Types.CHAR;
    public static final String NAME_MULTI_SELECT = "MULTI_SELECT";
    public static final String PROPERTY_MULTI_SELECT = "SELECTTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MULTI_SELECT, NAME_MULTI_SELECT);
    }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     * 
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     * 
     */
    public static final int ID_DESCRIPTION = 14;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "DESCRIPTION";
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column WEIGHT of type Types.SMALLINT mapped to Integer.
     * 
     */
    public static final int ID_WEIGHT = 15;
    public static final int TYPE_WEIGHT = Types.SMALLINT;
    public static final String NAME_WEIGHT = "WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WEIGHT,NAME_WEIGHT);}

    /**
     * Column LOCAL of type Types.CHAR mapped to String.
     * 
     */
    public static final int ID_LOCAL = 16;
    public static final int TYPE_LOCAL = Types.CHAR;
    public static final String NAME_LOCAL = "LOCAL";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LOCAL = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LOCAL,NAME_LOCAL);}

    /**
     * Column PARENT_NAME of type Types.VARCHAR mapped to String.
     * 
     */
    public static final int ID_PARENT_NAME = 17;
    public static final int TYPE_PARENT_NAME = Types.VARCHAR;
    public static final String NAME_PARENT_NAME = "PARENT_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PARENT_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_NAME,NAME_PARENT_NAME);}

    /**
     * Column INPUT_TYPE of type Types.SMALLINT mapped to Integer.
     * 
     */
    public static final int ID_INPUT_TYPE = 18;
    public static final int TYPE_INPUT_TYPE = Types.SMALLINT;
    public static final String NAME_INPUT_TYPE = "INPUT_TYPE";
    public static final String PROPERTY_INPUT_TYPE = "INPUTTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_INPUT_TYPE, SqlUtil.castAsChar(NAME_INPUT_TYPE,1));
    }


    /**
     * Create an array of type string containing all the fields of the TMT_CLIENT_ATTRIBUTES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_MULTI_SELECT
        ,NAME_RANK
        ,NAME_DESCRIPTION
        ,NAME_WEIGHT
        ,NAME_LOCAL
        ,NAME_PARENT_NAME
        ,NAME_INPUT_TYPE
    };
    
    /**
     * Field that contains the comma separated fields of the TMT_CLIENT_ATTRIBUTES table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };    
	
    public AttributeManager(){
    	super();
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames(){
        
        return FIELD_NAMES;
    }
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields() {
        return ALL_FIELDS;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey() {
        return NAME_ELEMENT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }
       
	public String getTableName(){
	    ExceptionUtil.handleGetTableName(this);
		return null;
	}
	
	public String getTableName(RPMObject rpmObject)
    {
	    if (rpmObject.getContextName() == null)
        {
            // find out the context name by examining the module type
            rpmObject.setContextName(AttributeUtil.getInstance()
                .getContextNameFromModule((GenericAttribute) rpmObject));
        }
        return AttributeUtil.getInstance().getTableName(
            rpmObject.getContextName());
    }
	
	public String getTableName(String contextName)
    {
        return AttributeUtil.getInstance().getTableName(contextName);
    }
	
	protected Integer getTableType(RPMObject rpmObject)
    {
	    if (rpmObject.getContextName() == null)
        {
            // find out the context name by examining the module type
            rpmObject.setContextName(AttributeUtil.getInstance()
                .getContextNameFromModule((GenericAttribute) rpmObject));
        }
        return AttributeUtil.getInstance().getTableType(
            rpmObject.getContextName());
    }
	
    protected boolean isCategory(int typeId){
    	return typeId == AttributeCategory.TYPE_ID;
    }
    protected boolean isClassification(int typeId){
    	return typeId == AttributeClassification.TYPE_ID||typeId == AttributeClassification.TYPE_ID2;
    }
    protected boolean isAttribute(int typeId){
    	return typeId == Attribute.TYPE_ID||typeId == Attribute.TYPE_ID2;
    }
    
    protected RPMObject createParentInstance(int childTypeId){
    	if(isCategory(childTypeId)){
    		return new AttributeCategory();
    	} else if (isClassification(childTypeId)){
    		return new AttributeCategory();
    	} else if (isAttribute(childTypeId)){
    		return new AttributeClassification();
    	}
    	return null;
    }
    
    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");

        return filter.toString();
    }
    
  	// TODO SLegault: implement "/AttributeCategory" XPath. 
    // We should use "LEVEL_ID=0" filtering.
    // TODO SLegault: it seems that we do not delete the "local" attribute category
    // when we delete a project. They accumulate VERY FAST. We had 20000 records of
    // garbage in each table when I looked at it.
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException {    	
        JoinCondition joinCondition = null;

        String typeId = "";
        
         if (containerName != null){
             if(containerName.equalsIgnoreCase(CATEGORY_CONTAINER_NAME)){
                 typeId = String.valueOf(AttributeCategory.TYPE_ID);
             }else if(containerName.equalsIgnoreCase(CLASSIFICATION_CONTAINER_NAME)){
                 typeId = String.valueOf(AttributeClassification.TYPE_ID)+","+String.valueOf(AttributeClassification.TYPE_ID2);
             }else if(containerName.equalsIgnoreCase(ATTRIBUTE_CONTAINER_NAME)){
                 typeId = String.valueOf(Attribute.TYPE_ID)+","+String.valueOf(Attribute.TYPE_ID2);
             }else{
                 ExceptionUtil.handleNoTypeId(this, containerName);
             }
         }
         if(manager == null){
            
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName(contextName));
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" in ( ");
    	    where.append(typeId);
    	    where.append(" ) ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    where.append(" AND ");
    	    where.append(getFilter());
    	    
    	    joinCondition.setCondition(where.toString());
            
        }else if(manager instanceof AttributeManager){
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            if (containerName != null || condition != null)
            {
                where.append(" WHERE ");

                //ContainerName added to control the type of the parent
                // objects.
                //Not used when we do a loadByForeignKey
                if (containerName != null)
                {
                    where.append(NAME_TYPE_ID);
                    where.append(" in ( ");
                    where.append(typeId);
                    where.append(" ) ");
                }
                if (condition != null)
                {
                    if (containerName != null)
                    {
                        where.append(" AND ");
                    }
                    where.append(condition);
                }
            }

            joinCondition.setCondition(where.toString());          
            
        } else if (isSupportedModuleManager(manager)){
        	joinCondition = new JoinCondition();
        	
        }else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }               	
        
        return joinCondition;
        
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        if (manager instanceof AttributeManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(getFilter());

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof AttributeAssignmentManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(getFilter());

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }   
   
    
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException{
        
        JoinCondition joinCondition = null;
        
        if(manager instanceof AttributeManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(getPrimaryKey());
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, contextName);

            StringBuffer sb = new StringBuffer();
            sb.append(joinCondition.getCondition());
            sb.append(" )");
                        
            if(extraWhere != null && extraWhere.length() > 0){

                sb.append(" AND ");
                sb.append(extraWhere);
            }

            joinCondition.setCondition(sb.toString());           
            joinCondition.setTableName(getTableName(contextName));
        }
        else if (isSupportedModuleManager(manager))
        {
            joinCondition = new JoinCondition(getTableName(contextName));
            joinCondition.appendEqual(NAME_LEVEL_ID, 0);
            joinCondition.appendCondition(extraWhere);
        }
        else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    } 
    
    private boolean isSupportedModuleManager(RPMObjectManager manager)
    {
        return manager instanceof AssetModuleManager
                || manager instanceof ClientModuleManager
                || manager instanceof DocumentModuleManager
                || manager instanceof PoolModuleManager
                || manager instanceof ScopeManagementModuleManager
                || manager instanceof WbsModuleManager;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap() {       
        return FIELDPROPERTYMAP;
    }
        
    protected RPMObject createContainer(int typeId) throws RPMException 
    {
        return containerMap.createContainer(typeId);
    }

      
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {
            
            if (rpmScope.getParent() != null) 
            {
                loadParent(rpmObject, rpmScope, messageContext, relatedFields, index,
                    isDelta);
            }
                        
            if(rpmScope instanceof AttributeScope){
	            
                AttributeScope scope = (AttributeScope) rpmScope;
	            
	            if (rpmObject instanceof AttributeCategory)
	            {
	                if (scope.getCategories() != null)
	                {
	                    //Categories
	                    loadCategories(rpmObject, scope, messageContext, isDelta);
	                }
	
	                if (scope.getClassifications() != null)
	                {
	                    //Classifications
	                    loadClassifications(rpmObject, scope, messageContext,
	                        isDelta);
	                }
	            }
	            else if (rpmObject instanceof AttributeClassification)
	            {
	                if (scope.getAttributes() != null)
	                {
	                    //Attributes
	                    loadAttributes(rpmObject, scope, messageContext, isDelta);
	                }
	            }
            }
        }
        return rpmObject;
    }
    
    private void loadParent(RPMObject rpmObject, RPMObjectScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);
        RPMObject newParent = null;
        if (isDelta)
        {
            newParent = rpmObject.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = createParentInstance(rpmObject.retrieveTypeID()
                    .intValue());
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    newParent.setContextName(rpmObject.getContextName());
                    oldParent = loadByPrimaryKey(newParent, scope.getParent(),
                        messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the scope to
                    // avoid problem with the cache
                    oldParent.setContextName(rpmObject.getContextName());
                    oldParent = loadByPrimaryKey(oldParent, null,
                        messageContext, false);
                }
            }
            rpmObject.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {//newParent not in cache
                    newParent = createParentInstance(rpmObject.retrieveTypeID()
                        .intValue());
                    newParent.setID(oldParentId);
                    newParent.setContextName(rpmObject.getContextName());
                    newParent = loadByPrimaryKey(newParent, scope.getParent(),
                        messageContext, false);
                }
                rpmObject.setParent(newParent);
            }
            else
            {
                rpmObject.setParent(AttributeUtil.getInstance()
                    .getModuleFromContextName(rpmObject.getContextName()));
            }
        }
    }

    private void loadCategories(RPMObject rpmObject, AttributeScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
    	AttributeCategory category = (AttributeCategory) rpmObject;
    	AttributeCategory[] subCategories = category.getCategories();
		ArrayList temp = loadChildren(rpmObject, subCategories, scope.getCategories(), messageContext, new int[]{AttributeCategory.TYPE_ID}, isDelta);
		if (temp != null) {
		    setParent(temp,rpmObject);
			subCategories = new AttributeCategory[temp.size()];
			temp.toArray(subCategories);
			category.setCategories(subCategories);
		}
    }
    
    private void loadClassifications(RPMObject rpmObject, AttributeScope scope, MessageContext messageContext, boolean isDelta)throws RPMException, SQLException, ParseException
	{
    	AttributeCategory category = (AttributeCategory) rpmObject;
    	AttributeClassification[] children = category.getClassifications();
		ArrayList temp = loadChildren(rpmObject, children, scope.getClassifications(), messageContext, new int[]{AttributeClassification.TYPE_ID, AttributeClassification.TYPE_ID2}, isDelta);
		if (temp != null) {
		    setParent(temp,rpmObject);
			children = new AttributeClassification[temp.size()];
			temp.toArray(children);
			category.setClassifications(children);
		}
    }
    
    private void loadAttributes(RPMObject rpmObject, AttributeScope scope, MessageContext messageContext, boolean isDelta)throws RPMException, SQLException, ParseException
	{
    	AttributeClassification classification = (AttributeClassification) rpmObject;
    	Attribute[] children = classification.getAttributes();
		ArrayList temp = loadChildren(rpmObject, children, scope.getAttributes(), messageContext, new int[]{Attribute.TYPE_ID, Attribute.TYPE_ID2}, isDelta);
		if (temp != null) {
		    setParent(temp,rpmObject);
			children = new Attribute[temp.size()];
			temp.toArray(children);
			classification.setAttributes(children);
		}
    }

    private ArrayList loadChildren(RPMObject parent, RPMObject[] oldChildren, AttributeScope childrenScope, MessageContext messageContext, int[] typeIds, boolean isDelta) throws RPMException, ParseException, SQLException
	{   
    	String extraWhere = constructExtraWhere(typeIds);
    	return loadByForeignKey(oldChildren,childrenScope, messageContext,  this, new Object[]{parent.getID()}, extraWhere, parent.getContextName(), isDelta);
	}
    
    private String constructExtraWhere(int[] typeIds){
    	StringBuffer extraWhere = new StringBuffer();
    	
    	if(typeIds!=null && typeIds.length!=0){
			extraWhere.append("TYPE_ID in (");
    		for(int i=0;i<typeIds.length;i++){
    			extraWhere.append(typeIds[i]);
    			if(i+1<typeIds.length){
    				extraWhere.append(" , ");
    			}
    		}
			extraWhere.append(")");
    	}
    	
    	return extraWhere.toString();
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject, int)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
    throws SQLException {
    	
    	GenericAttribute genericAttribute = (GenericAttribute)rpmObject;
        
        int typeId = genericAttribute.retrieveTypeID().intValue();
    	
        genericAttribute.setID(rs.getString(ID_ELEMENT_ID));
        if (!isDelta) {

            genericAttribute.setName(rs.getString(ID_ELEMENT_NAME));
            genericAttribute.setDescription(rs.getString(ID_DESCRIPTION));
        	
            if (isClassification(typeId)){
				AttributeClassification classification = (AttributeClassification) genericAttribute;
				//set selectType
				String selectTypeIndex = rs.getString(ID_MULTI_SELECT);
				SelectType selectType = SelectTypeUtil.getSelectTypeByIndex(selectTypeIndex);
				classification.setSelectType(selectType);
			} else if (isAttribute(typeId)){
				Attribute attribute = (Attribute) genericAttribute;
				//set inputType
				String inputTypeIndex = rs.getString(ID_INPUT_TYPE);
				InputType inputType = InputTypeUtil.getInputTypeByIndex(inputTypeIndex);
				attribute.setInputType(inputType);
			}
		}
        else
        {	
            genericAttribute.deltaName(rs.getString(ID_ELEMENT_NAME));
            genericAttribute.deltaDescription(rs.getString(ID_DESCRIPTION));
            
            if(isClassification(typeId)){
				AttributeClassification classification = (AttributeClassification) genericAttribute;
				//delta selectType
				String selectTypeIndex = rs.getString(ID_MULTI_SELECT);
				SelectType selectType = SelectTypeUtil.getSelectTypeByIndex(selectTypeIndex);
				classification.deltaSelectType(selectType);
			} else if (isAttribute(typeId)){
				Attribute attribute = (Attribute) genericAttribute;
				//delta inputType
				String inputTypeIndex = rs.getString(ID_INPUT_TYPE);
				InputType inputType = InputTypeUtil.getInputTypeByIndex(inputTypeIndex);
				attribute.deltaInputType(inputType);
			}
        }

        return rpmObject;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        
        return relatedFields;
    }
          
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        rpmObject.setID(SP_I_ATR((GenericAttribute)rpmObject, messageContext));
    }
    
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
    	int typeId = rpmObject.retrieveTypeID().intValue();
		AttributeScope scope = (AttributeScope)rpmScope;

		if (rpmObject != null)
        {
            if (isCategory(typeId))
            {
                rpmObject = saveCategory((AttributeCategory) rpmObject, scope,
                    messageContext);
            }
            else if (isClassification(typeId))
            {
                rpmObject = saveClassification(
                    (AttributeClassification) rpmObject, scope, messageContext);
            }
            else if (isAttribute(typeId))
            {
                rpmObject = saveAttribute((Attribute) rpmObject, messageContext);
            }
        }
		return rpmObject;
	}
    
    private RPMObject saveCategory(AttributeCategory category, AttributeScope scope, MessageContext messageContext)throws RPMException, SQLException, ParseException{

    	if (category.testNameModified() || category.testDescriptionModified()){
    		SP_U_ATR(category, messageContext);
    	}
    	
    	if (scope != null)
    	{
	    	propagateContextName(category, category.getCategories());
	    	category.setCategories((AttributeCategory[]) 
	                updateChildren(category, 
	                        category.getCategories(), 
	                        scope.getCategories(), 
	                        messageContext, null));
	        
	    	propagateContextName(category, category.getClassifications());
	    	category.setClassifications((AttributeClassification[]) 
	                updateChildren(category, 
	                        category.getClassifications(), 
	                        scope.getClassifications(), 
	                        messageContext, null));
    	}
    	return category;
    }
    
    private RPMObject saveClassification(AttributeClassification classification, AttributeScope scope, MessageContext messageContext)throws RPMException, SQLException, ParseException{

        boolean defaultSelectTypeSet = false;
        
       	//if select type is null set default value
       	if(classification.getSelectType()==null){
           	classification.setSelectType(SelectType.Single);
           	defaultSelectTypeSet = true;
       	}
        
    	if (classification.testNameModified() || classification.testDescriptionModified()){
    		SP_U_ATR(classification, messageContext);
    	}    	
		if (classification.testSelectTypeModified()||defaultSelectTypeSet){
			SP_U_ATR_MS(classification, messageContext);
		}
		
		if (classification.getAttributes() != null)
		{
			Attribute[] attributes = classification.getAttributes();
			for (int i = 0; i < attributes.length; i++)
			{
			    if (attributes[i] != null)
                {
                    if (classification.getSelectType() != null)
                    {
                        if (classification.getSelectType().getValue().equals(
                            SelectType._Multi))
                        { /// multiselect
                            attributes[i].assignTypeID(new Integer(
                                Attribute.TYPE_ID));
                        }
                        else if (classification.getSelectType().getValue()
                            .equals(SelectType._Single))
                        { /// singleselect
                            attributes[i].assignTypeID(new Integer(
                                Attribute.TYPE_ID2));
                        }
                    }
                }
			}
		}
    	
    	if (scope != null)
    	{
	    	propagateContextName(classification, classification.getAttributes());
	    	classification.setAttributes((Attribute[]) 
	                updateChildren(classification, 
	                        classification.getAttributes(), 
	                        scope.getAttributes(), 
	                        messageContext, null));
    	}
    	
    	return classification;
    }
    
    private RPMObject saveAttribute(Attribute attribute,
            MessageContext messageContext) throws RPMException, SQLException
    {
        boolean defaultInputTypeSet = false;
        
        //if input type is null set default value
        if(attribute.getInputType()==null){
            attribute.setInputType(InputType.Numeric);
            defaultInputTypeSet = true;
        }
        
    	if (attribute.testNameModified() || attribute.testDescriptionModified() || attribute.testInputTypeModified()||defaultInputTypeSet){
    		SP_U_ATR(attribute, messageContext);
    	}
    	
    	return attribute;
    }
        
    protected void doDelete(RPMObject object, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (object != null)
        {
            SP_D_ATR(object, messageContext);
        }
    }
    
    private String SP_I_ATR(GenericAttribute rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        // must send only the TYPE_IDs supported by the SP during the insert
        int typeId = rpmObject.retrieveTypeID().intValue();
        if (isCategory(typeId))
        {
            typeId = AttributeCategory.TYPE_ID;
        }
        else if (isClassification(typeId))
        {
            typeId = AttributeClassification.TYPE_ID2;
        }
        else if (isAttribute(typeId))
        {
            typeId = Attribute.TYPE_ID2;
        }
        
        String name = "SP_I_ATR";
        Object args[] = new Object[5];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getName();
        args[2] = (rpmObject.getParent() != null) ? rpmObject.getParent()
            .getID() : null;
        args[3] = new Integer(typeId);
        args[4] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_ATR(GenericAttribute rpmObject, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ATR";

        int typeId = rpmObject.retrieveTypeID().intValue();
        
        Object args[] = new Object[8];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
       	args[2] = (rpmObject.getParent()!=null)?rpmObject.getParent().getID():null;
        args[3] = rpmObject.getName();
    	args[4] = rpmObject.getDescription();
        args[5] = new Integer(0);
    	
        if(isCategory(typeId) || isClassification(typeId)){
            args[6] = new Integer(3);
        } else if (isAttribute(typeId)) {
            args[6] = InputTypeUtil.getIndexByInputType(((Attribute)rpmObject).getInputType());
        }
        args[7] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_U_ATR_MS(AttributeClassification classification, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ATR_MS";
        Object args[] = new Object[5];
        args[0] = getTableType(classification);
        args[1] = classification.getID();
       	args[2] = (classification.getParent()!=null)?classification.getParent().getID():null;
       	args[3] = SelectTypeUtil.getIndexBySelectType(classification.getSelectType());
        args[4] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_D_ATR(RPMObject rpmObject, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_D_ATR";
        Object args[] = new Object[5];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();  
        args[2] = ManagerUtil.getParentId( this, rpmObject, messageContext, getTableName(rpmObject));       
        args[3] = "X";
        args[4] = getUser(messageContext).getID();
        
        executeProcedure(messageContext,name, args);
    }        
    
	
    // ==========
    // MODULES
    // ==========
	
    public AttributeCategory[] loadAttributeCategories(AbstractModule parentModule,
            AttributeCategory[] attributeCategories, AttributeScope scope,
            RPMObjectManager manager, String contextName,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String extraWhere = "TYPE_ID = " + AttributeCategory.TYPE_ID;
        Object[] args = null;
        
        ArrayList temp = loadByForeignKeyWithSpecifiedParent(parentModule, attributeCategories, scope,
            messageContext, manager, args, extraWhere, contextName, isDelta);
        AttributeCategory[] objs = null;
        if (temp != null)
        {
            objs = (AttributeCategory[]) Array.newInstance(
                AttributeCategory.class, temp.size());
            temp.toArray(objs);
        }
        return objs;
    }
}
