/*
 * Created on Apr 21, 2005
 *
 */
package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.StringTokenizer;

import com.ibm.rpm.applicationadministration.constants.ErrorCodes;
import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.types.CalendarExceptionType;
import com.ibm.rpm.applicationadministration.util.WorkIntervalUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * @author Genevieve
 *
 */
public final class CalendarExceptionManager extends AbstractRPMObjectManager {

    private static final int NON_DEFAULT_WORKING = 4;
    
	//This constant is used when creating a new Calendar_Exception   
    private static final Integer DEFAULT_CALENDAR_EXCEPTION = new Integer(28);
    
    private static final HashSet CONTAINERS = new HashSet();
    
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    
    static{       
        CONTAINERS.add(CalendarException.class.getName());
        
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++) {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    CalendarExceptionManager.class);
        }        
    }     
    /**
     * Column ELEMENT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ELEMENT_TYPE = 1;
    public static final int TYPE_ELEMENT_TYPE = Types.INTEGER;
    public static final String NAME_ELEMENT_TYPE = "CALENDAR_EXCEPTIONS.ELEMENT_TYPE";
    public static final String PROPERTY_ELEMENT_TYPE = "TYPE";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_TYPE,NAME_ELEMENT_TYPE);}

    /**
     * Column DAY_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_DAY_STATUS = 2;
    public static final int TYPE_DAY_STATUS = Types.CHAR;
    public static final String NAME_DAY_STATUS = "CALENDAR_EXCEPTIONS.DAY_STATUS";
    public static final String PROPERTY_DAY_STATUS = "DAY";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DAY_STATUS,NAME_DAY_STATUS);}
   
    /**
     * Column DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     */
    public static final int ID_DEFAULT_MIN = 3;
    public static final int TYPE_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_DEFAULT_MIN = "CALENDAR_EXCEPTIONS.DEFAULT_MIN";
    public static final String PROPERTY_DEFAULT_MIN = "MINUTES";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_MIN,NAME_DEFAULT_MIN);}
    
    /**
     * Column EXCEPTION_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_MIN,NAME_EXCEPTION_MIN);}
     */
    public static final int ID_EXCEPTION_MIN = 4;
    public static final int TYPE_EXCEPTION_MIN = Types.SMALLINT;
    public static final String NAME_EXCEPTION_MIN = "CALENDAR_EXCEPTIONS.EXCEPTION_MIN";

    /**
     * Column EXCEPTION_DATE of type Types.DATE mapped to java.sql.Date.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_DATE = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_DATE,NAME_EXCEPTION_DATE);}
     */
    public static final int ID_EXCEPTION_DATE = 5;
    public static final int TYPE_EXCEPTION_DATE = Types.DATE;
    public static final String NAME_EXCEPTION_DATE = "CALENDAR_EXCEPTIONS.EXCEPTION_DATE";

    /**
     * Column EXCEPTION_ID of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_ID = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_ID,NAME_EXCEPTION_ID);}
     */
    public static final int ID_EXCEPTION_ID = 6;
    public static final int TYPE_EXCEPTION_ID = Types.CHAR;
    public static final String NAME_EXCEPTION_ID = "CALENDAR_EXCEPTIONS.EXCEPTION_ID";

    /**
     * Column CALENDAR_ID of type Types.CHAR mapped to String.
     *
     */
    public static final int ID_CALENDAR_ID = 7;
    public static final int TYPE_CALENDAR_ID = Types.CHAR;
    public static final String NAME_CALENDAR_ID = "CALENDAR_EXCEPTIONS.CALENDAR_ID";
    public static final String PROPERTY_CALENDAR_ID = "CALENDAR";
    static{ FIELDPROPERTYMAP.put(PROPERTY_CALENDAR_ID,NAME_CALENDAR_ID);}
    
    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}
     */
    public static final int ID_ELEMENT_ID = 8;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "CALENDAR_EXCEPTIONS.EXCEPTION_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_REC_USER = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);} 
     */    
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "CALENDAR_EXCEPTIONS.REC_USER";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.sql.Timestamp.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);} 
     */
    public static final int ID_REC_DATETIME = 10;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "CALENDAR_EXCEPTIONS.REC_DATETIME";

    /**
     * Column DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_BIT,NAME_DEFAULT_BIT);} 
     */
    public static final int ID_DEFAULT_BIT = 11;
    public static final int TYPE_DEFAULT_BIT = Types.BINARY;
    public static final String NAME_DEFAULT_BIT = "PRINT_TIMES(CALENDAR_EXCEPTIONS.DEFAULT_BIT)";

    /**
     * Column EXCEPTION_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_BIT,NAME_EXCEPTION_BIT);} 
     */
    public static final int ID_EXCEPTION_BIT = 12;
    public static final int TYPE_EXCEPTION_BIT = Types.BINARY;
    public static final String NAME_EXCEPTION_BIT = "PRINT_TIMES(CALENDAR_EXCEPTIONS.EXCEPTION_BIT)";

    /**
     * Column EXCEPTION_NAME of type Types.VARCHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_NAME = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_NAME,NAME_EXCEPTION_NAME);} 
     */
    public static final int ID_EXCEPTION_NAME = 13;
    public static final int TYPE_EXCEPTION_NAME = Types.VARCHAR;
    public static final String NAME_EXCEPTION_NAME = "CALENDAR_EXCEPTIONS.EXCEPTION_NAME";

    /**
     * Column CALENDAR_RANK of type Types.VARCHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_CALENDAR_RANK = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_CALENDAR_RANK,NAME_CALENDAR_RANK);} 
     */
    public static final int ID_CALENDAR_RANK = 14;
    public static final int TYPE_CALENDAR_RANK = Types.VARCHAR;
    public static final String NAME_CALENDAR_RANK = "CALENDAR_EXCEPTIONS.CALENDAR_RANK";

    /**
     * Column EXCEPTION_DESCRIPTION of type Types.VARCHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case.
     * public static final String PROPERTY_EXCEPTION_DESCRIPTION = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_EXCEPTION_DESCRIPTION,NAME_EXCEPTION_DESCRIPTION);} 
     */
    public static final int ID_EXCEPTION_DESCRIPTION = 15;
    public static final int TYPE_EXCEPTION_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_EXCEPTION_DESCRIPTION = "CALENDAR_EXCEPTIONS.EXCEPTION_DESCRIPTION";


    private static final String TABLE_NAME = "CALENDAR_EXCEPTIONS";

    /**
     * Create an array of type string containing all the fields of the CALENDAR_EXCEPTIONS table.
     */
    private static final String[] FIELD_NAMES = 
    {
        NAME_ELEMENT_TYPE
        ,NAME_DAY_STATUS
        ,NAME_DEFAULT_MIN
        ,NAME_EXCEPTION_MIN
        ,NAME_EXCEPTION_DATE
        ,NAME_EXCEPTION_ID
        ,NAME_CALENDAR_ID
        ,NAME_ELEMENT_ID
        ,NAME_REC_USER
        ,NAME_REC_DATETIME
        ,NAME_DEFAULT_BIT
        ,NAME_EXCEPTION_BIT
        ,NAME_EXCEPTION_NAME
        ,NAME_CALENDAR_RANK
        ,NAME_EXCEPTION_DESCRIPTION
    };

    /**
     * Field that contains the comma separated fields of the CALENDAR_EXCEPTIONS table.
     */
    private static final String ALL_FIELDS;
    static
	{
    	StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
    	for(int i = 1 ; i < FIELD_NAMES.length; i++)
    	{
    		fieldcat.append(",");
    		fieldcat.append(FIELD_NAMES[i]);
    	}
    	ALL_FIELDS = fieldcat.toString();
    }
	
    /**
     */
    public CalendarExceptionManager() {
        super();
    }

   
    /**
     * Creates a new RPMCalendar instance.
     *
     * @return the new RPMCalendar 
     */
    protected RPMObject createContainer( int typeId) 
    {
        RPMObject object = null;

        object = new CalendarException();
        
        return object;
    }
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName() {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames() {
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields() {
        return ALL_FIELDS;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey() {

        return NAME_EXCEPTION_ID;
    }


    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws
            SQLException {
        
        int typeId = resultSet.getInt(ID_ELEMENT_TYPE);        
        return new Integer(typeId);
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException {

   
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }

        }        
        else if(manager instanceof RPMCalendarManager){
            
            
            joinCondition = new JoinCondition();
            
            joinCondition.setTableName(getTableName());
            
            StringBuffer where = new StringBuffer();
            where.append(manager.getPrimaryKey());
            where.append(" = ");
            where.append(NAME_CALENDAR_ID);
            
            joinCondition.setCondition(where.toString()); 
            
        }else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }               	
        
        return joinCondition;
        
    }
    
    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;
        
        if (manager instanceof RPMCalendarManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            
            where.append(RPMCalendarManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_CALENDAR_ID);
            where.append(" FROM ");
            where.append(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject, int)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta) throws RPMException,
            SQLException, ParseException {

        CalendarException pObject = (CalendarException) rpmObject;

        pObject.assignTypeID(Manager.getInteger(rs, ID_ELEMENT_TYPE));
        //pObject.setElementType(Manager.getInteger(rs, ID_ELEMENT_TYPE));

        if (rs.getInt(ID_DAY_STATUS) == NON_DEFAULT_WORKING){
            pObject.setType(CalendarExceptionType.NonDefaultWorking);            
        }else{
            pObject.setType(CalendarExceptionType.NonWorking);
        }
        pObject.setMinutes(Manager.getInteger(rs, ID_EXCEPTION_MIN));

        pObject.setDay(Manager.getCalendar(rs, ID_EXCEPTION_DATE));

        pObject.setID(rs.getString(ID_EXCEPTION_ID));

        if (pObject.getType() == CalendarExceptionType.NonDefaultWorking) {
            pObject.setIntervals(getIntervals(rs.getString(ID_EXCEPTION_BIT)));
        }

        pObject.setDay(Manager.getCalendar(rs, ID_EXCEPTION_DATE));
        return pObject;

    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException,
            RPMException
    {
        CalendarException pObject = (CalendarException) rpmObject;
        pObject.setID(SP_I_CAL_EXC(pObject, messageContext));
    }
    
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        CalendarException pObject = (CalendarException) rpmObject;
        SP_U_CAL_EXC_HOURS(pObject, messageContext);
        return pObject;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject object, MessageContext messageContext) throws RPMException, SQLException {

        CalendarException weekDay = null;
        
        if(object != null){
            
            weekDay = (CalendarException)object;
            
            SP_D_CAL_EXC(weekDay, messageContext);
            
        }	         
    }
    private WorkInterval[] getIntervals(String intervals) throws RPMException
	{
    	ArrayList workintervals = new ArrayList();
		if((intervals == null) || (intervals.length() == 0)){
		    ExceptionUtil.handleRPMException(this, ErrorCodes.NO_INTERVALS);
		}
		StringTokenizer intervalTokens = new StringTokenizer(intervals, ",");
		if(intervalTokens.countTokens()%2 != 0){
		    ExceptionUtil.handleRPMException(this, ErrorCodes.UNBALANCED_INTERVALS);
		}
		while(intervalTokens.hasMoreTokens())
			workintervals.add(WorkIntervalUtil.createWorkInterval(intervalTokens.nextToken(),intervalTokens.nextToken()));
		WorkInterval workintarray[] = new WorkInterval[workintervals.size()];
		workintervals.toArray(workintarray);
		return workintarray;
	}
    
    private String SP_I_CAL_EXC(CalendarException CalendarException, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_CAL_EXC";
        Object args[] = new Object[4];
        args[0] = "Exception";
        args[1] = (CalendarException.getParent()!=null)?CalendarException.getParent().getID():null;
        args[2] = DEFAULT_CALENDAR_EXCEPTION;
        args[3] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext,name, args);
    } 
    
   
    private void SP_U_CAL_EXC_HOURS(CalendarException weekday, MessageContext messageContext)
            throws RPMException, SQLException {
     
        String name = "SP_U_CAL_EXC_HOURS";
        Object args[] = new Object[15];
        args[0] = weekday.getID();
        args[1] = (weekday.getParent()!=null)?weekday.getParent().getID():null;
        args[2] = DateUtil.convertToString(weekday.getDay());


        //This parameter represents the day of the week but it not used by the stored procedure
        args[3] = new Integer(1);
        
        WorkInterval[] workintervals = weekday.getIntervals();
        
        WorkIntervalUtil.validate(workintervals);
        
        //Added to return the type of exception to the user        
        if(workintervals == null){
            weekday.setType(CalendarExceptionType.NonWorking);
        }else{
            weekday.setType(CalendarExceptionType.NonDefaultWorking);
        }
        
        for (int i = 0; i < 5; i++) {
            if (workintervals != null && i < workintervals.length) {
                if(workintervals[i] != null){
	                args[2 * i + 4] = WorkIntervalUtil.formatTime(workintervals[i].getFrom());
	                args[2 * i + 5] = WorkIntervalUtil.formatTime(workintervals[i].getTo());
                }
            } else {
                args[2 * i + 4] = null;
                args[2 * i + 5] = null;
            }
        }

        args[14] = getUser(messageContext).getID();

        executeProcedure(messageContext,name, args);
    }
    
   
    private void SP_D_CAL_EXC(CalendarException CalendarException, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_D_CAL_EXC";
        Object args[] = new Object[4];
        args[0] = CalendarException.getID();  	
        args[1] = CalendarException.getParent().getID();           
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext,name, args);
    }       
    
  
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException{
        
        JoinCondition joinCondition = null;
        
        if(manager instanceof RPMCalendarManager){

    	    StringBuffer condition = new StringBuffer();
    	    condition.append(NAME_CALENDAR_ID);
    	    condition.append(" = ?");
    	    joinCondition = new JoinCondition();
    	    joinCondition.setCondition(condition.toString());

            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    }
}
