package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.City;
import com.ibm.rpm.applicationadministration.containers.Country;
import com.ibm.rpm.applicationadministration.containers.CountryState;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Location;
import com.ibm.rpm.applicationadministration.containers.Region;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.xpathparser.XPathContainer;

public class GeographicalManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(Region.class.getName());
        CONTAINERS.add(Country.class.getName());
        CONTAINERS.add(CountryState.class.getName());
        CONTAINERS.add(City.class.getName());
        CONTAINERS.add(Location.class.getName());
        CONTAINERS.add(Geographical.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                GeographicalManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_GEOGRAPHIES.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_GEOGRAPHIES.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_GEOGRAPHIES.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_GEOGRAPHIES.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_GEOGRAPHIES.DELETED_COUNT";

    public static final String PROPERTY_DELETED_COUNT = "DELETED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT, NAME_DELETED_COUNT);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_GEOGRAPHIES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_GEOGRAPHIES.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_GEOGRAPHIES.PARENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_GEOGRAPHIES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_GEOGRAPHIES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_GEOGRAPHIES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DEFAULT_FLAG = 12;

    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;

    public static final String NAME_DEFAULT_FLAG = "TMT_GEOGRAPHIES.DEFAULT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DEFAULT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_GEOGRAPHIES.RANK";

    public static final String PROPERTY_RANK = "RANKID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RANK, NAME_RANK);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 14;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_GEOGRAPHIES.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    private static final String TABLE_NAME = "TMT_GEOGRAPHIES";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_GEOGRAPHIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_DEFAULT_FLAG, NAME_RANK, NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     */
    public GeographicalManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }
    
    public String getFilter()
    {
        
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D'");
        sql.appendAnd();
        sql.append(NAME_ELEMENT_ID);
        sql.append(" IS NOT NULL ");
        
        return sql.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {

        case Region.TYPE_ID:
            object = new Region();
            break;
        case Country.TYPE_ID:
            object = new Country();
            break;

        case CountryState.TYPE_ID:
            object = new CountryState();
            break;

        case City.TYPE_ID:
            object = new City();
            break;

        case Location.TYPE_ID:
            object = new Location();
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

        Geographical temp = (Geographical) rpmObject;

        temp.setID(rs.getString(ID_ELEMENT_ID));

        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }

        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaDescription(rs.getString(ID_DESCRIPTION));
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            temp.setName(name);
            temp.setDescription(rs.getString(ID_DESCRIPTION));
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {

        Geographical geo = null;

        if (rpmObject != null)
        {

            geo = (Geographical) rpmObject;

            SP_D_GEO(geo, messageContext);

        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof GeographicalManager)
        {            
            joinCondition = new JoinCondition(getTableName());
            
            //condition on the children must not be in the sub select.
            joinCondition.appendCondition(extraWhere);
            
            //condition of parent.
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID, getTableName(), getPrimaryKey() + " = ?");
        }
        else if (manager instanceof AdministrationModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName());             
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    protected String getInitialCondition(Integer typeId){
        
        SqlBuffer initialCondition = new SqlBuffer();
        if(typeId != null){
            initialCondition.appendEqual(NAME_TYPE_ID, typeId.intValue());
        }else{
            //A generic object is loaded
            int[] allTypes = new int[]{Region.TYPE_ID, Country.TYPE_ID, CountryState.TYPE_ID, City.TYPE_ID, Location.TYPE_ID};
            initialCondition.appendIn(NAME_TYPE_ID, allTypes);            
        }
        initialCondition.appendCondition(getFilter());
        
        return initialCondition.toString();
    }
    /**
     *
     */

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        Integer typeId = container.getTypeId();
        
        JoinCondition joinCondition = null;

        if (manager == null)
        {                       
            joinCondition = new JoinCondition(getTableName(), getInitialCondition(typeId)); 
        }
        else if (manager instanceof GeographicalManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID, getTableName(), getInitialCondition(typeId));
        }
        else
        {
            ExceptionUtil.handleNotImplementedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
     
        JoinCondition joinCondition = null;

        if (manager instanceof GeographicalManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(NAME_ELEMENT_ID, NAME_PARENT_ID, getTableName(), getInitialCondition(container.getTypeId()));
        }
        else
        {
            ExceptionUtil.handleNotImplementedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {

        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {

        return FIELDPROPERTYMAP;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {

            Geographical object = (Geographical) rpmObject;


            //Parent
            if (rpmScope.getParent() != null)
            {

                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                RPMObject newParent = null;

                if (isDelta)
                {
                    newParent = object.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new Geographical();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = loadByPrimaryKey(
                                newParent, rpmScope.getParent(), messageContext,
                                true);
                        }
                        else
                        {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                    }
                    else
                    {
                        if (newParent != null && !(newParent instanceof AdministrationModule))
                        {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                        else
                        {
                            oldParent = new AdministrationModule();
                        }
                    }
                    object.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        newParent = new Geographical();
                        newParent.setID(oldParentId);
                        newParent = (Geographical) loadByPrimaryKey(newParent,
                            rpmScope.getParent(), messageContext, false);
                        object.setParent(newParent);
                    }
                    else
                    {
                        AdministrationModule moduleParent = new AdministrationModule();
                        object.setParent(moduleParent);
                    }
                }
            }

            if(rpmScope instanceof GeographicalScope){
            
	            GeographicalScope scope = (GeographicalScope) rpmScope;
	            //children
	            if (scope.getChildren() != null)
	            {
	                SqlBuffer where = new SqlBuffer();
	                where.appendIn(NAME_TYPE_ID, new int[]
	                {
	                        Region.TYPE_ID, Country.TYPE_ID, CountryState.TYPE_ID,
	                        City.TYPE_ID, Location.TYPE_ID
	                });
	
	                ArrayList temp = loadByForeignKey(object.getChildren(), scope
	                    .getChildren(), messageContext, this, new Object[]
	                {
	                    object.getID()
	                }, where.toString(), null, isDelta);
	
	                if (temp != null)
	                {
	                    //Set the parent of all children
	                    setParent(temp, object);
	                    Geographical[] children = new Geographical[temp.size()];
	                    temp.toArray(children);
	                    object.setChildren(children);
	                }
	            }
            }

        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Geographical pObject = (Geographical) rpmObject;
        pObject.setID(SP_I_GEO(pObject, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        Geographical pObject = (Geographical) rpmObject;
        GeographicalScope scope = (GeographicalScope) rpmScope;
        if (pObject.testNameModified() || pObject.testDescriptionModified())
            SP_U_GEO(pObject, messageContext);

        if (scope != null)
        {
            pObject.setChildren((Geographical[]) updateChildren(rpmObject,
                pObject.getChildren(), scope.getChildren(), messageContext,
                Geographical.class));
        }
        return pObject;
    }

    private String SP_I_GEO(Geographical geographical,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_I_GEO";
        Object args[] = new Object[4];
        args[0] = geographical.getName();

        RPMObject parent = geographical.getParent();
        if (parent != null && !(parent instanceof AbstractModule))
        {
            args[1] = parent.getID();
        }

        args[2] = geographical.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        elementId = executeProcedure(messageContext, name, args);

        return elementId;

    }

    private void SP_U_GEO(Geographical geographical,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_GEO";
        Object args[] = new Object[4];
        args[0] = geographical.getID();
        args[1] = geographical.getName();
        args[2] = geographical.getDescription();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_GEO(Geographical geographical,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_GEO";
        Object args[] = new Object[4];
        args[0] = geographical.getID();
        args[1] = ManagerUtil.getParentId(this, geographical, messageContext, TABLE_NAME);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
}