/*
 * Created on Apr 4, 2005
 *
 */

package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Department;
import com.ibm.rpm.applicationadministration.containers.Group;
import com.ibm.rpm.applicationadministration.containers.Office;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.asset.managers.AssetOrganizationsManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.managers.ResourceManager;

/**
 * @author Genevieve
 *  
 */
public final class OrganizationalManager extends AbstractRPMObjectManager
{
    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(Organization.class.getName());
        CONTAINERS.add(Department.class.getName());
        CONTAINERS.add(Group.class.getName());
        CONTAINERS.add(Office.class.getName());
        CONTAINERS.add(Organizational.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                OrganizationalManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     *  
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_ORGANIZATIONS.LEVEL_ID";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_ORGANIZATIONS.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_ORGANIZATIONS.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_ORGANIZATIONS.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_ORGANIZATIONS.DELETED_COUNT";

    public static final String PROPERTY_DELETED_COUNT = "DELETED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT, NAME_DELETED_COUNT);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_ORGANIZATIONS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_ORGANIZATIONS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_ORGANIZATIONS.PARENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_ORGANIZATIONS.REC_USER";

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_ORGANIZATIONS.REC_STATUS";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_ORGANIZATIONS.REC_DATETIME";

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DEFAULT_FLAG = 12;

    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;

    public static final String NAME_DEFAULT_FLAG = "TMT_ORGANIZATIONS.DEFAULT_FLAG";

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_ORGANIZATIONS.RANK";

    public static final String PROPERTY_RANK = "RANKID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RANK, NAME_RANK);
    }

    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_FS_CODE = 14;

    public static final int TYPE_FS_CODE = Types.VARCHAR;

    public static final String NAME_FS_CODE = "TMT_ORGANIZATIONS.FS_CODE";

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 15;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_ORGANIZATIONS.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    private static final String TABLE_NAME = "TMT_ORGANIZATIONS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_ORGANIZATIONS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_DEFAULT_FLAG, NAME_RANK, NAME_FS_CODE, NAME_DESCRIPTION
    };

    /**
     * Field that contains the comma separated fields of the TMT_ORGANIZATIONS
     * table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public OrganizationalManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {

        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {

        return ALL_FIELDS;

    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    private String determineTypeId(String containerName) throws RPMException
    {
        StringBuffer typeIdWhere = new StringBuffer();
        int typeId = 0;

        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Organization.class)))
                typeId = Organization.TYPE_ID;
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Department.class)))
                typeId = Department.TYPE_ID;
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Group.class)))
                typeId = Group.TYPE_ID;
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Office.class)))
                typeId = Office.TYPE_ID;
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Organizational.class)))
            {
                typeIdWhere.append(NAME_TYPE_ID);
                typeIdWhere.append(" in ( ").append(Organization.TYPE_ID);
                typeIdWhere.append(" , ").append(Department.TYPE_ID);
                typeIdWhere.append(" , ").append(Group.TYPE_ID);
                typeIdWhere.append(" , ").append(Office.TYPE_ID);
                typeIdWhere.append(" )");
                
                return typeIdWhere.toString();
            }
            else
            {
                ExceptionUtil.handleNoTypeId(this, containerName);
            }
        }

        if (typeId != 0)
            typeIdWhere.append(NAME_TYPE_ID).append(" = ").append(typeId);
        return typeIdWhere.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getJoinCondition()
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        String typeIdWhere = determineTypeId(containerName);

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(typeIdWhere);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof OrganizationalManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());

            if (!typeIdWhere.equals(""))
            {
                where.append(" WHERE ");
                where.append(typeIdWhere);
                if (condition != null)
                {
                    where.append(" AND ");
                    where.append(condition);
                }
            }
            else
            {
                if (condition != null)
                {
                    where.append(" WHERE ");
                    where.append(condition);
                }
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();

            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();

            where.append(ResourceManager.NAME_RESOURCE_ID);
            where.append(" = ");
            where.append(NAME_ELEMENT_ID);
            where.append(" AND ");
            where.append(NAME_TYPE_ID);
            where.append(" = 13 ");
            where.append(" AND ");
            where.append(NAME_PARENT_ID);
            where.append(" in (");
            where.append("SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(typeIdWhere);

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof AssetOrganizationsManager)
        {
            // for XPath queries like
            // Organizaion[<condition>]/AssetOrganizationalAssignment
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(AssetOrganizationsManager.NAME_ORGANIZATION_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_ELEMENT_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");
            where.append(typeIdWhere);

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        String typeIdWhere = determineTypeId(containerName);

        if (manager instanceof OrganizationalManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();

            where.append(NAME_ELEMENT_ID).append(" IN (SELECT ");
            where.append(NAME_PARENT_ID).append(" FROM ")
                .append(getTableName());
            where.append(" WHERE ").append(typeIdWhere);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof AssetOrganizationsManager)
        {
            // for XPath queries like
            // /AssetOrganizationalAssignment[Organizaion[<condition>]
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(AssetOrganizationsManager.NAME_ORGANIZATION_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_ELEMENT_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");
            where.append(typeIdWhere);

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof OrganizationalManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(getPrimaryKey());
            condition.append(" = ?");

            //In this case, we don't need a condition on the type of the parent
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                // Fix for type-id 13 not found error, added ')' before AND
                sb.append(" ) AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof AdministrationModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName()); 
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {

        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {

        case Organization.TYPE_ID:
            object = new Organization();
            break;
        case Department.TYPE_ID:
            object = new Department();
            break;

        case Group.TYPE_ID:
            object = new Group();
            break;

        case Office.TYPE_ID:
            object = new Office();
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {

            Organizational object = (Organizational) rpmObject;

            //Parent
            if (rpmScope.getParent() != null)
            {

                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);

                RPMObject newParent = null;

                if (isDelta)
                {
                    newParent = object.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new Organizational();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = loadByPrimaryKey(
                                newParent, rpmScope.getParent(), messageContext,
                                true);
                        }
                        else
                        {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                    }
                    else
                    {
                        if (newParent != null && !(newParent instanceof AdministrationModule))
                        {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                        else
                        {
                            oldParent = new AdministrationModule();
                        }
                    }
                    object.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {
                            newParent = new Organizational();
                            newParent.setID(oldParentId);
                            newParent = loadByPrimaryKey(
                                newParent, rpmScope.getParent(), messageContext,
                                false);
                        }
                        object.setParent(newParent);
                    }
                    else
                    {
                        AdministrationModule moduleParent = new AdministrationModule();
                        object.setParent(moduleParent);
                    }
                }
            }

            
            if(rpmScope instanceof OrganizationalScope){
                            
	            OrganizationalScope scope = (OrganizationalScope) rpmScope;
	            
	            //children
	            if (scope.getChildren() != null)
	            {
	
	                SqlBuffer where = new SqlBuffer();
	                where.appendIn(NAME_TYPE_ID, new int[]
	                {
	                        Organization.TYPE_ID, Department.TYPE_ID,
	                        Group.TYPE_ID, Office.TYPE_ID
	                });
	
	                ArrayList temp = loadByForeignKey(object.getChildren(), scope
	                    .getChildren(), messageContext, this, new Object[]
	                {
	                    object.getID()
	                }, where.toString(), null, isDelta);
	
	                if (temp != null)
	                {
	                    //Set the parent of all children
	                    setParent(temp, object);
	                    Organizational[] children = new Organizational[temp.size()];
	                    temp.toArray(children);
	                    object.setChildren(children);
	                }
	            }   
            
            }
        }

        return rpmObject;

    }

    public RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {
        Organizational temp = (Organizational) rpmObject;

        temp.setID(rs.getString(ID_ELEMENT_ID));

        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }

        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaDescription(rs.getString(ID_DESCRIPTION));
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            temp.setName(name);
            temp.setDescription(rs.getString(ID_DESCRIPTION));
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    private String SP_I_ORG(Organizational organizational,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_I_ORG";
        Object args[] = new Object[4];
        args[0] = organizational.getName();

        RPMObject parent = organizational.getParent();
        if (parent != null && !(parent instanceof AbstractModule))
        {
            args[1] = parent.getID();
        }

        args[2] = organizational.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        elementId = executeProcedure(messageContext, name, args);

        return elementId;

    }

    private void SP_U_ORG(Organizational organizational,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ORG";
        Object args[] = new Object[4];
        args[0] = organizational.getID();
        args[1] = organizational.getName();
        args[2] = organizational.getDescription();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_ORG(Organizational organizational,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_ORG";
        Object args[] = new Object[4];
        args[0] = organizational.getID();
        args[1] = ManagerUtil.getParentId(this, organizational, messageContext, TABLE_NAME);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Organizational pObject = (Organizational) rpmObject;
        pObject.setID(SP_I_ORG(pObject, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        Organizational pObject = (Organizational) rpmObject;
        if (pObject.testNameModified() || pObject.testDescriptionModified())
            SP_U_ORG(pObject, messageContext);

        if (rpmObject != null && rpmScope != null)
        {
            OrganizationalScope scope = (OrganizationalScope) rpmScope;

            pObject.setChildren((Organizational[]) updateChildren(rpmObject,
                pObject.getChildren(), scope.getChildren(), messageContext,
                Organizational.class));
        }
        return pObject;
    }

     /**
     * @param organization
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {

        Organizational org = null;

        if (rpmObject != null)
        {

            org = (Organizational) rpmObject;

            SP_D_ORG(org, messageContext);

        }

    }
}