package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.PoolModuleManager;

public class PoolDomainsManager extends AbstractRPMObjectManager
{

    private static Log logger = LogFactory.getLog(PoolDomainsManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(PoolDomain.class.getName());
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                PoolDomainsManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_DOMAINS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_DOMAINS.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_DOMAINS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_DOMAINS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_DOMAINS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_DOMAINS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_DOMAINS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_DOMAINS.PARENT_ID";

    public static final String PROPERTY_MAX_RANK = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK, NAME_MAX_RANK);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_DOMAINS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_DOMAINS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_DOMAINS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_RANK = 12;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_DOMAINS.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 13;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_DOMAINS.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    private static final String TABLE_NAME = "TMT_DOMAINS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_DOMAINS table.
     */
    private static final String[] FIELD_NAMES =
    {
        NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
        NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_REC_USER,
        NAME_REC_STATUS, NAME_REC_DATETIME, NAME_RANK, NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     */
    public PoolDomainsManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     *  
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {
        case PoolDomain.TYPE_ID:
            object = new PoolDomain();
            break;
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
            throws SQLException
    {
        PoolDomain temp = (PoolDomain) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));

        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }

        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaDescription(rs.getString(ID_DESCRIPTION));
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            temp.setName(name);
            temp.setDescription(rs.getString(ID_DESCRIPTION));
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        PoolDomain pd = null;
        if (rpmObject != null)
        {
            pd = (PoolDomain) rpmObject;
            SP_D_DOMAIN(pd, messageContext);
        }
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");
        filter.append(" AND ");
        filter.append(" NOT ");
        filter.append(NAME_ELEMENT_ID);
        filter.append(" IS ");
        filter.append(" NULL ");

        return filter.toString();
    }
    
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            
            StringBuffer where = new StringBuffer();
            
            if (condition != null)
            {
                where.append(condition);
                where.append(" AND ");
            }
            
            where.append(getFilter());
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof PoolDomainsManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            int rankLenght = 4 * level;
            String rank = getTableName() + ".RANK";

            where.append("(LEVEL_ID >= ");
            where.append(level);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }      
        else if (manager instanceof PoolModuleManager){

        	joinCondition = new JoinCondition();      	

        } 
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName)
            throws RPMException
    {        
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {
	        joinCondition = new JoinCondition();
	
	        StringBuffer where = new StringBuffer();
	        where.append(PoolManager.NAME_DOMAIN_ID);
	        where.append(" IN (SELECT ");
	        where.append(NAME_ELEMENT_ID);
	        where.append(" FROM ");
	        where.append(getTableName());
	        where.append(" WHERE ");
	        where.append(getFilter());
	
	        if (condition != null)
	        {
	            where.append(" AND ");
	            where.append(condition);
	        }
	
	        joinCondition.setCondition(where.toString());
        }
        //TODO Implement method getSubTableJoinCondition if needed
        else
        {
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	            ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager instanceof PoolDomainsManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(getPrimaryKey());
            condition.append(" = ?");
            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof PoolModuleManager){

        	joinCondition = getJoinCondition(manager,null,2, null, contextName);

            if(extraWhere != null && extraWhere.length() > 0){

                joinCondition.setCondition(extraWhere);

            }

        	joinCondition.setTableName(getTableName(contextName));

        }
        
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields, int index,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {

            PoolDomain object = (PoolDomain) rpmObject;


            //Parent
            if (rpmScope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);
                PoolDomain newParent = null;
                if (isDelta)
                {
                    newParent = (PoolDomain) object.getParent();
                    PoolDomain oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new PoolDomain();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                            && newParent.getID().equals(oldParentId))
                        {
                            oldParent = (PoolDomain) loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, true);
                        }
                        else
                        {
                            //In this case, old parent is loaded
                            // without the scope to avoid problem with
                            // the cache
                            oldParent = (PoolDomain) loadByPrimaryKey(oldParent, null,
                                messageContext, false);
                        }
                    }
                    deltaParent(object, oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {//newParent not in cache
                            newParent = new PoolDomain();
                            newParent.setID(oldParentId);
                            newParent = (PoolDomain) loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, false);
                        }
                        setParent(object, newParent);
                    }
                }
            }

            
            if(rpmScope instanceof PoolDomainScope){
            
	            PoolDomainScope scope = (PoolDomainScope) rpmScope;
	            //children
	            object = loadPoolDomain(object, scope, messageContext, isDelta);
            }
        }

        return rpmObject;
    }

    private void deltaParent(RPMObject rpmChild, RPMObject rpmParent) throws RPMException
    {
        rpmChild.deltaParent(rpmParent);
    }

    private PoolDomain loadPoolDomain(PoolDomain pd, PoolDomainScope scope,
            MessageContext messageContext, boolean isDelta) throws RPMException,
            ParseException, SQLException
    {
        if (scope != null)
        {
            //If the poolDomains from pd are not loaded, get it from the db
            if (scope.getPoolDomains() != null)
            {
                ArrayList temp = loadByForeignKey(pd.getPoolDomains(), scope
                    .getPoolDomains(), messageContext, this, new Object[]
                {
                    pd.getID()
                }, "TYPE_ID = " + PoolDomain.TYPE_ID, null, isDelta);
                if (temp != null)
                {
                    PoolDomain[] pDomains = new PoolDomain[temp.size()];
                    temp.toArray(pDomains);
                    pd.setPoolDomains(pDomains);
                }
            }
        }
        return pd;
    }

    /**
     * @param pObject
     * @throws SQLException
     * @throws RPMException
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        PoolDomain pObject = (PoolDomain) rpmObject;
        pObject.setID(SP_I_DOMAIN(pObject, messageContext));
    }

    /**
     * @param rpmObject
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        PoolDomain pObject = (PoolDomain) rpmObject;
        PoolDomainScope scope = (PoolDomainScope) rpmScope;
        if (pObject.testNameModified())
        {
            SP_U_DOMAIN(pObject, messageContext);
        }

        if (scope != null)
        {
            if (rpmObject != null)
            {
                pObject = savePoolDomain(pObject, scope, messageContext);
            }
        }
        return pObject;
    }

    protected PoolDomain savePoolDomain(PoolDomain aPool, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException,
            SQLException, ParseException
    {
        PoolDomainScope scope = (PoolDomainScope) rpmScope;
        aPool.setPoolDomains((PoolDomain[]) 
                updateChildren(aPool, 
                        aPool.getPoolDomains(), 
                        scope.getPoolDomains(), 
                        messageContext, null));
        return aPool;
    }

    private PoolDomain setParent(RPMObject rpmChild, RPMObject rpmParent)
    {
        PoolDomain pdChild = (PoolDomain) rpmChild;
        PoolDomain pdParent = (PoolDomain) rpmParent;
        pdChild.setParent(pdParent);
        return pdChild;
    }

    /*
     * Build an array containing name, parentID, typeID and userID
     */
    private String SP_I_DOMAIN(PoolDomain domain, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_I_DOMAIN";
        Object args[] = new Object[4];
        args[0] = domain.getName();
        if (domain.getParent() != null)
        {
            args[1] = domain.getParent().getID();
        }
        args[2] = domain.retrieveTypeID();
        args[3] = getUser(messageContext).getID();
        //The needed parameters for the SP are:
        //ELEMENT_NAME, PARENT_ID, TYPE_ID, REC_USER
        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_DOMAIN(PoolDomain domain, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_DOMAIN";
        Object args[] = new Object[4];
        args[0] = domain.getID();
        args[1] = domain.getName();
        args[2] = domain.getDescription();
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        //The needed parameters for the SP are:
        //ELEMENT_ID, ELEMENT_NAME, DESCRIPTION, REC_USER
        executeProcedure(messageContext, name, args);
    }

    private void SP_D_DOMAIN(PoolDomain domain, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_DOMAIN";
        Object args[] = new Object[4];
        args[0] = domain.getID();
        if (domain.getParent() != null)
        {
            args[1] = domain.getParent().getID();
        }
        args[2] = "X";
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        //The needed parameters for the SP are:
        //ELEMENT_ID, PARENT_ID, REC_STATUS, REC_USER
        executeProcedure(messageContext, name, args);
    }
}
