/*
 * Created on Apr 20, 2005
 *
 */

package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.StringTokenizer;

import com.ibm.rpm.applicationadministration.constants.ErrorCodes;
import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.applicationadministration.util.WorkIntervalUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.managers.WbsManager;


/**
 * @author Genevieve
 *
 */
public final class RPMCalendarManager extends AbstractRPMObjectManager {

    public final static int NON_WORKING_DEFAULT = 0;   /// for old version value 0
    public final static int USE_DEFAULT = 1;
	public final static int DERIVED_NON_WORKING = 3;
	public final static int DERIVED_NON_DEFAULT = 4;
	public final static int NON_WORKING = 5;
	public final static int NON_DEFAULT = 6;
	
	//This constant is used when creating a new RPMCalendar
	public final static Integer DEFAULT_CALENDAR = new Integer(27);
    
    
    private static final HashSet CONTAINERS = new HashSet();
    
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    
    static{       
        CONTAINERS.add(RPMCalendar.class.getName());
                
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++) {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    RPMCalendarManager.class);
        }        
    } 
    
    /**
     * Column MONDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MONDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MONDAY_STATUS,NAME_MONDAY_STATUS);}
     */
    public static final int ID_MONDAY_STATUS = 1;
    public static final int TYPE_MONDAY_STATUS = Types.CHAR;
    public static final String NAME_MONDAY_STATUS = "TMT_CALENDARS.MONDAY_STATUS";
    
    /**
     * Column TUESDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TUESDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_STATUS,NAME_TUESDAY_STATUS);}
     */
    public static final int ID_TUESDAY_STATUS = 2;
    public static final int TYPE_TUESDAY_STATUS = Types.CHAR;
    public static final String NAME_TUESDAY_STATUS = "TMT_CALENDARS.TUESDAY_STATUS";
    
    /**
     * Column WEDNESDAY_STATUS of type Types.CHAR mapped to String.    
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEDNESDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_STATUS,NAME_WEDNESDAY_STATUS);}
     */
    public static final int ID_WEDNESDAY_STATUS = 3;
    public static final int TYPE_WEDNESDAY_STATUS = Types.CHAR;
    public static final String NAME_WEDNESDAY_STATUS = "TMT_CALENDARS.WEDNESDAY_STATUS";

    /**
     * Column THURSDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_THURSDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_STATUS,NAME_THURSDAY_STATUS);}
     */
    public static final int ID_THURSDAY_STATUS = 4;
    public static final int TYPE_THURSDAY_STATUS = Types.CHAR;
    public static final String NAME_THURSDAY_STATUS = "TMT_CALENDARS.THURSDAY_STATUS";
    
    /**
     * Column FRIDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_FRIDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_STATUS,NAME_FRIDAY_STATUS);}
     */
    public static final int ID_FRIDAY_STATUS = 5;
    public static final int TYPE_FRIDAY_STATUS = Types.CHAR;
    public static final String NAME_FRIDAY_STATUS = "TMT_CALENDARS.FRIDAY_STATUS";
  
    /**
     * Column SATURDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SATURDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_STATUS,NAME_SATURDAY_STATUS);}
     */
    public static final int ID_SATURDAY_STATUS = 6;
    public static final int TYPE_SATURDAY_STATUS = Types.CHAR;
    public static final String NAME_SATURDAY_STATUS = "TMT_CALENDARS.SATURDAY_STATUS";
 
    /**
     * Column SUNDAY_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SUNDAY_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_STATUS,NAME_SUNDAY_STATUS);}
     */
    public static final int ID_SUNDAY_STATUS = 7;
    public static final int TYPE_SUNDAY_STATUS = Types.CHAR;
    public static final String NAME_SUNDAY_STATUS = "TMT_CALENDARS.SUNDAY_STATUS";

    /**
     * Column MONDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MONDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MONDAY_DEFAULT_MIN,NAME_MONDAY_DEFAULT_MIN);}
     */
    public static final int ID_MONDAY_DEFAULT_MIN = 8;
    public static final int TYPE_MONDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_MONDAY_DEFAULT_MIN = "TMT_CALENDARS.MONDAY_DEFAULT_MIN";
    
    /**
     * Column TUESDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TUESDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_DEFAULT_MIN,NAME_TUESDAY_DEFAULT_MIN);}
     */
    public static final int ID_TUESDAY_DEFAULT_MIN = 9;
    public static final int TYPE_TUESDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_TUESDAY_DEFAULT_MIN = "TMT_CALENDARS.TUESDAY_DEFAULT_MIN";
    
    /**
     * Column WEDNESDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEDNESDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_DEFAULT_MIN,NAME_WEDNESDAY_DEFAULT_MIN);}
     */
    public static final int ID_WEDNESDAY_DEFAULT_MIN = 10;
    public static final int TYPE_WEDNESDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_WEDNESDAY_DEFAULT_MIN = "TMT_CALENDARS.WEDNESDAY_DEFAULT_MIN";
    
    /**
     * Column THURSDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_THURSDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_DEFAULT_MIN,NAME_THURSDAY_DEFAULT_MIN);}
     */
    public static final int ID_THURSDAY_DEFAULT_MIN = 11;
    public static final int TYPE_THURSDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_THURSDAY_DEFAULT_MIN = "TMT_CALENDARS.THURSDAY_DEFAULT_MIN";
    
    /**
     * Column FRIDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_FRIDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_DEFAULT_MIN,NAME_FRIDAY_DEFAULT_MIN);}
     */
    public static final int ID_FRIDAY_DEFAULT_MIN = 12;
    public static final int TYPE_FRIDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_FRIDAY_DEFAULT_MIN = "TMT_CALENDARS.FRIDAY_DEFAULT_MIN";
    
    /**
     * Column SATURDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SATURDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_DEFAULT_MIN,NAME_SATURDAY_DEFAULT_MIN);}
     */
    public static final int ID_SATURDAY_DEFAULT_MIN = 13;
    public static final int TYPE_SATURDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_SATURDAY_DEFAULT_MIN = "TMT_CALENDARS.SATURDAY_DEFAULT_MIN";
    
    /**
     * Column SUNDAY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SUNDAY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_DEFAULT_MIN,NAME_SUNDAY_DEFAULT_MIN);}
     */
    public static final int ID_SUNDAY_DEFAULT_MIN = 14;
    public static final int TYPE_SUNDAY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_SUNDAY_DEFAULT_MIN = "TMT_CALENDARS.SUNDAY_DEFAULT_MIN";
    
    /**
     * Column WEEKLY_DEFAULT_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEEKLY_DEFAULT_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_DEFAULT_MIN,NAME_WEEKLY_DEFAULT_MIN);}
     */
    public static final int ID_WEEKLY_DEFAULT_MIN = 15;
    public static final int TYPE_WEEKLY_DEFAULT_MIN = Types.SMALLINT;
    public static final String NAME_WEEKLY_DEFAULT_MIN = "TMT_CALENDARS.WEEKLY_DEFAULT_MIN";

    /**
     * Column MONDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MONDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MONDAY_MIN,NAME_MONDAY_MIN);}
     */
    public static final int ID_MONDAY_MIN = 16;
    public static final int TYPE_MONDAY_MIN = Types.SMALLINT;
    public static final String NAME_MONDAY_MIN = "TMT_CALENDARS.MONDAY_MIN";
    public static final String PROPERTY_MONDAY_MIN = "STANDARDMONDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONDAY_MIN, NAME_MONDAY_MIN);
    }

    /**
     * Column TUESDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TUESDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_MIN,NAME_TUESDAY_MIN);}
     */
    public static final int ID_TUESDAY_MIN = 17;
    public static final int TYPE_TUESDAY_MIN = Types.SMALLINT;
    public static final String NAME_TUESDAY_MIN = "TMT_CALENDARS.TUESDAY_MIN";
    public static final String PROPERTY_TUESDAY_MIN = "STANDARDTUESDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_MIN, NAME_TUESDAY_MIN);
    }

    /**
     * Column WEDNESDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEDNESDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_MIN,NAME_WEDNESDAY_MIN);}
     */
    public static final int ID_WEDNESDAY_MIN = 18;
    public static final int TYPE_WEDNESDAY_MIN = Types.SMALLINT;
    public static final String NAME_WEDNESDAY_MIN = "TMT_CALENDARS.WEDNESDAY_MIN";
    public static final String PROPERTY_WEDNESDAY_MIN = "STANDARDWEDNESDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_MIN, NAME_WEDNESDAY_MIN);
    }
 
    /**
     * Column THURSDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_THURSDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_MIN,NAME_THURSDAY_MIN);}
     */
    public static final int ID_THURSDAY_MIN = 19;
    public static final int TYPE_THURSDAY_MIN = Types.SMALLINT;
    public static final String NAME_THURSDAY_MIN = "TMT_CALENDARS.THURSDAY_MIN";
    public static final String PROPERTY_THURSDAY_MIN = "STANDARDTHURSDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_MIN, NAME_THURSDAY_MIN);
    }
 
    /**
     * Column FRIDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_FRIDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_MIN,NAME_FRIDAY_MIN);}
     */
    public static final int ID_FRIDAY_MIN = 20;
    public static final int TYPE_FRIDAY_MIN = Types.SMALLINT;
    public static final String NAME_FRIDAY_MIN = "TMT_CALENDARS.FRIDAY_MIN";
    public static final String PROPERTY_FRIDAY_MIN = "STANDARDFRIDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_MIN, NAME_FRIDAY_MIN);
    }
 
    /**
     * Column SATURDAY_MIN of type Types.SMALLINT mapped to Integer.
     *
     * REPLACE PROPERTYNAME with container property in upper case.  
     * public static final String PROPERTY_SATURDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_MIN,NAME_SATURDAY_MIN);}
     */
    public static final int ID_SATURDAY_MIN = 21;
    public static final int TYPE_SATURDAY_MIN = Types.SMALLINT;
    public static final String NAME_SATURDAY_MIN = "TMT_CALENDARS.SATURDAY_MIN";
    public static final String PROPERTY_SATURDAY_MIN = "STANDARDSATURDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_MIN, NAME_SATURDAY_MIN);
    }
 
    /**
     * Column SUNDAY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SUNDAY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_MIN,NAME_SUNDAY_MIN);}
     */
    public static final int ID_SUNDAY_MIN = 22;
    public static final int TYPE_SUNDAY_MIN = Types.SMALLINT;
    public static final String NAME_SUNDAY_MIN = "TMT_CALENDARS.SUNDAY_MIN";
    public static final String PROPERTY_SUNDAY_MIN = "STANDARDSUNDAYMINUTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_MIN, NAME_SUNDAY_MIN);
    }
 
    /**
     * Column WEEKLY_MIN of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEEKLY_MIN = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_MIN,NAME_WEEKLY_MIN);}
     */
    public static final int ID_WEEKLY_MIN = 23;
    public static final int TYPE_WEEKLY_MIN = Types.SMALLINT;
    public static final String NAME_WEEKLY_MIN = "TMT_CALENDARS.WEEKLY_MIN";
    
    /**
     * Column MINUTES_PER_DAY of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MINUTES_PER_DAY = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MINUTES_PER_DAY,NAME_MINUTES_PER_DAY);}
     */
    public static final int ID_MINUTES_PER_DAY = 24;
    public static final int TYPE_MINUTES_PER_DAY = Types.SMALLINT;
    public static final String NAME_MINUTES_PER_DAY = "TMT_CALENDARS.MINUTES_PER_DAY";
    public static final String PROPERTY_MINUTES_PER_DAY = "HOURSPERDAY";
    static{ FIELDPROPERTYMAP.put(PROPERTY_MINUTES_PER_DAY,NAME_MINUTES_PER_DAY+"/60.0");}

    /**
     * Column MINUTES_PER_WEEK of type Types.SMALLINT mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MINUTES_PER_WEEK = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MINUTES_PER_WEEK,NAME_MINUTES_PER_WEEK);}
     */
    public static final int ID_MINUTES_PER_WEEK = 25;
    public static final int TYPE_MINUTES_PER_WEEK = Types.SMALLINT;
    public static final String NAME_MINUTES_PER_WEEK = "TMT_CALENDARS.MINUTES_PER_WEEK";
    public static final String PROPERTY_MINUTES_PER_WEEK = "HOURSPERWEEK";
    static{ FIELDPROPERTYMAP.put(PROPERTY_MINUTES_PER_WEEK,NAME_MINUTES_PER_WEEK+"/60.0");}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}
     */
    public static final int ID_LEVEL_ID = 26;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_CALENDARS.LEVEL_ID";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}
     */
    public static final int ID_TYPE_ID = 27;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_CALENDARS.TYPE_ID";

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}
     */
    public static final int ID_MAX_RANK = 28;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_CALENDARS.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}
     */
    public static final int ID_CHILD_COUNT = 29;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_CALENDARS.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}
     */
    public static final int ID_DELETED_COUNT = 30;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_CALENDARS.DELETED_COUNT";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}
     */
    public static final int ID_REC_DATETIME = 31;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_CALENDARS.REC_DATETIME";

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}
     */
    public static final int ID_REC_STATUS = 32;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_CALENDARS.REC_STATUS";

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 33;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_CALENDARS.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}     

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 34;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_CALENDARS.PARENT_ID";
    public static final String PROPERTY_PARENT_ID = "PARENT";
    static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}
    
    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_REC_USER = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}
     */
    public static final int ID_REC_USER = 35;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_CALENDARS.REC_USER";
   
    /**
     * Column MONDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MONDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MONDAY_DEFAULT_BIT,NAME_MONDAY_DEFAULT_BIT);}
     */
    public static final int ID_MONDAY_DEFAULT_BIT = 36;
    public static final int TYPE_MONDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_MONDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.MONDAY_DEFAULT_BIT)";

    /**
     * Column TUESDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TUESDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_MIN,NAME_MONDAY_MIN);}
     */
    public static final int ID_TUESDAY_DEFAULT_BIT = 37;
    public static final int TYPE_TUESDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_TUESDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.TUESDAY_DEFAULT_BIT)";
  
    /**
     * Column WEDNESDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEDNESDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_DEFAULT_BIT,NAME_WEDNESDAY_DEFAULT_BIT);}
     */
    public static final int ID_WEDNESDAY_DEFAULT_BIT = 38;
    public static final int TYPE_WEDNESDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_WEDNESDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.WEDNESDAY_DEFAULT_BIT)";
  
    /**
     * Column THURSDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_THURSDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_DEFAULT_BIT,NAME_THURSDAY_DEFAULT_BIT);}
     * 
     */
    public static final int ID_THURSDAY_DEFAULT_BIT = 39;
    public static final int TYPE_THURSDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_THURSDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.THURSDAY_DEFAULT_BIT)";
   
    /**
     * Column FRIDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_FRIDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_DEFAULT_BIT,NAME_FRIDAY_DEFAULT_BIT);}
     */
    public static final int ID_FRIDAY_DEFAULT_BIT = 40;
    public static final int TYPE_FRIDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_FRIDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.FRIDAY_DEFAULT_BIT)";
  
    /**
     * Column SATURDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SATURDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_DEFAULT_BIT,NAME_SATURDAY_DEFAULT_BIT);}
     */
    public static final int ID_SATURDAY_DEFAULT_BIT = 41;
    public static final int TYPE_SATURDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_SATURDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.SATURDAY_DEFAULT_BIT)";
  
    /**
     * Column SUNDAY_DEFAULT_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SUNDAY_DEFAULT_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_DEFAULT_BIT,NAME_SUNDAY_DEFAULT_BIT);}
     */
    public static final int ID_SUNDAY_DEFAULT_BIT = 42;
    public static final int TYPE_SUNDAY_DEFAULT_BIT = Types.CHAR;
    public static final String NAME_SUNDAY_DEFAULT_BIT = "PRINT_TIMES(TMT_CALENDARS.SUNDAY_DEFAULT_BIT)";
   
    /**
     * Column MONDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_MONDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_MONDAY_BIT,NAME_MONDAY_BIT);}
     */
    public static final int ID_MONDAY_BIT = 43;
    public static final int TYPE_MONDAY_BIT = Types.CHAR;
    public static final String NAME_MONDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.MONDAY_BIT)";
  
    /**
     * Column TUESDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_TUESDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_BIT,NAME_TUESDAY_BIT);}
     */
    public static final int ID_TUESDAY_BIT = 44;
    public static final int TYPE_TUESDAY_BIT = Types.CHAR;
    public static final String NAME_TUESDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.TUESDAY_BIT)";
 
    /**
     * Column WEDNESDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_WEDNESDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_BIT,NAME_WEDNESDAY_BIT);}
     */
    public static final int ID_WEDNESDAY_BIT = 45;
    public static final int TYPE_WEDNESDAY_BIT = Types.CHAR;
    public static final String NAME_WEDNESDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.WEDNESDAY_BIT)";

    /**
     * Column THURSDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_THURSDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_BIT,NAME_THURSDAY_BIT);}
     */
    public static final int ID_THURSDAY_BIT = 46;
    public static final int TYPE_THURSDAY_BIT = Types.CHAR;
    public static final String NAME_THURSDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.THURSDAY_BIT)";

    /**
     * Column FRIDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_FRIDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_BIT,NAME_FRIDAY_BIT);}
     */
    public static final int ID_FRIDAY_BIT = 47;
    public static final int TYPE_FRIDAY_BIT = Types.CHAR;
    public static final String NAME_FRIDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.FRIDAY_BIT)";
    
    /**
     * Column SATURDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SATURDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_BIT,NAME_SATURDAY_BIT);}
     */
    public static final int ID_SATURDAY_BIT = 48;
    public static final int TYPE_SATURDAY_BIT = Types.CHAR;
    public static final String NAME_SATURDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.SATURDAY_BIT)";
    
    /**
     * Column SUNDAY_BIT of type Types.BINARY mapped to byte[].
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_SUNDAY_BIT = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_BIT,NAME_SUNDAY_BIT);}
     */
    public static final int ID_SUNDAY_BIT = 49;
    public static final int TYPE_SUNDAY_BIT = Types.CHAR;
    public static final String NAME_SUNDAY_BIT = "PRINT_TIMES(TMT_CALENDARS.SUNDAY_BIT)";
    
    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 50;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_CALENDARS.ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}
    
    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. 
     * public static final String PROPERTY_RANK = "PROPERTYNAME";
     * static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}
     */
    public static final int ID_RANK = 51;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_CALENDARS.RANK";
    
    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 52;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_CALENDARS.DESCRIPTION";
    public static final String PROPERTY_ELEMENT_DESCRIPTION = "DESCRIPTION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}     
    
    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 53;
    public static final int TYPE_START_DATE = Types.DATE;
    public static final String NAME_START_DATE = "TMT_CALENDARS.START_DATE";
    public static final String PROPERTY_START_DATE = "STARTDATE";
    static{ FIELDPROPERTYMAP.put(PROPERTY_START_DATE,NAME_START_DATE);}
    
    
    public static final String TABLE_NAME = "TMT_CALENDARS";
    
    /**
     * Create an array of type string containing all the fields of the TMT_CALENDARS table.
     */
    private static final String[] FIELD_NAMES = 
    {
            NAME_MONDAY_STATUS
            ,NAME_TUESDAY_STATUS
            ,NAME_WEDNESDAY_STATUS
            ,NAME_THURSDAY_STATUS
            ,NAME_FRIDAY_STATUS
            ,NAME_SATURDAY_STATUS
            ,NAME_SUNDAY_STATUS
            ,NAME_MONDAY_DEFAULT_MIN
            ,NAME_TUESDAY_DEFAULT_MIN
            ,NAME_WEDNESDAY_DEFAULT_MIN
            ,NAME_THURSDAY_DEFAULT_MIN
            ,NAME_FRIDAY_DEFAULT_MIN
            ,NAME_SATURDAY_DEFAULT_MIN
            ,NAME_SUNDAY_DEFAULT_MIN
            ,NAME_WEEKLY_DEFAULT_MIN
            ,NAME_MONDAY_MIN
            ,NAME_TUESDAY_MIN
            ,NAME_WEDNESDAY_MIN
            ,NAME_THURSDAY_MIN
            ,NAME_FRIDAY_MIN
            ,NAME_SATURDAY_MIN
            ,NAME_SUNDAY_MIN
            ,NAME_WEEKLY_MIN
            ,NAME_MINUTES_PER_DAY
            ,NAME_MINUTES_PER_WEEK
            ,NAME_LEVEL_ID
            ,NAME_TYPE_ID
            ,NAME_MAX_RANK
            ,NAME_CHILD_COUNT
            ,NAME_DELETED_COUNT
            ,NAME_REC_DATETIME
            ,NAME_REC_STATUS
            ,NAME_ELEMENT_ID
            ,NAME_PARENT_ID
            ,NAME_REC_USER
            ,NAME_MONDAY_DEFAULT_BIT
            ,NAME_TUESDAY_DEFAULT_BIT
            ,NAME_WEDNESDAY_DEFAULT_BIT
            ,NAME_THURSDAY_DEFAULT_BIT
            ,NAME_FRIDAY_DEFAULT_BIT
            ,NAME_SATURDAY_DEFAULT_BIT
            ,NAME_SUNDAY_DEFAULT_BIT
            ,NAME_MONDAY_BIT
            ,NAME_TUESDAY_BIT
            ,NAME_WEDNESDAY_BIT
            ,NAME_THURSDAY_BIT
            ,NAME_FRIDAY_BIT
            ,NAME_SATURDAY_BIT
            ,NAME_SUNDAY_BIT
            ,NAME_ELEMENT_NAME
            ,NAME_RANK
            ,NAME_DESCRIPTION
            ,NAME_START_DATE
    };
    
    /**
     * Field that contains the comma separated fields of the TMT_CALENDARS table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    }      


    /**
     * Creates a new CalendarManager instance.
     * @return CalendarManager 
     */
    public RPMCalendarManager() {
        super();        
    }
      
    /**
     * Creates a new RPMCalendar instance.
     *
     * @return the new RPMCalendar 
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        //typeId is not used since all the objects loaded from this table are RPMCalendar

        object = new RPMCalendar();

        
        return object;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName() {       
        return TABLE_NAME;
    }
    /**
     *
     */

    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D'");
        sql.appendAnd();
        sql.append(NAME_ELEMENT_ID);
        sql.append(" IS NOT NULL ");
        
        return sql.toString();
    }
    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames(){
        
        return FIELD_NAMES;
    }
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields() {
        return ALL_FIELDS;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey() {
        return NAME_ELEMENT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws 
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }

    private static final String TYPE_FILTER = NAME_TYPE_ID + "=" 
		+ RPMCalendar.TYPE_ID;

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException 
    {
        JoinCondition joinCondition = null;
        if(manager instanceof RPMCalendarManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME, TYPE_FILTER);
        	joinCondition.appendSubSelect(
        	        NAME_PARENT_ID, NAME_ELEMENT_ID, 
        	        TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if(manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME, TYPE_FILTER);
        	joinCondition.appendSubSelect(
        	        NAME_ELEMENT_ID, NAME_PARENT_ID, 
        	        TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if(manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME, TYPE_FILTER);
        	joinCondition.appendSubSelect(
        	        NAME_ELEMENT_ID, NAME_PARENT_ID, 
        	        TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof AdministrationModuleManager)
        {            
            joinCondition = new JoinCondition(TABLE_NAME, extraWhere);
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    }
    
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException 
    {
        JoinCondition joinCondition = null;
        
        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, condition);
            joinCondition.appendCondition(TYPE_FILTER);
        }
        else if (manager instanceof RPMCalendarManager)
        {
        	joinCondition = JoinCondition.createSubSelect(
        	        NAME_PARENT_ID, NAME_ELEMENT_ID, 
        	        TABLE_NAME, condition);
            joinCondition.appendCondition(TYPE_FILTER);
        }
        else if (manager instanceof ResourceManager) 
        {
            joinCondition = new JoinCondition(TABLE_NAME);
        	joinCondition.appendEqual(ResourceManager.NAME_RESOURCE_ID, 
        	        NAME_ELEMENT_ID);
        	joinCondition.appendEqual(NAME_TYPE_ID, Resource.TYPE_ID);
        	joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID, 
        	        TABLE_NAME, condition);
        } 
        else if (manager instanceof WbsManager) 
        {
        	joinCondition = new JoinCondition(TABLE_NAME);
        	joinCondition.appendEqual(WbsManager.NAME_ELEMENT_ID, 
        	        NAME_ELEMENT_ID);
        	joinCondition.appendEqual(NAME_TYPE_ID, Project.TYPE_ID);
        	joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID, 
        	        TABLE_NAME, condition);
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.append(ResourceManager.NAME_RESOURCE_ID);
            joinCondition.append(" IN (SELECT ");
            joinCondition.append(NAME_ELEMENT_ID);
            joinCondition.appendFrom();
            joinCondition.append(getTableName());
            joinCondition.appendWhere();
            joinCondition.append(getFilter());
            joinCondition.appendAnd();
            joinCondition.append(NAME_PARENT_ID);
            joinCondition.append(" IN (SELECT ");
            joinCondition.append(NAME_ELEMENT_ID);
            joinCondition.appendFrom();
            joinCondition.append(getTableName());
            joinCondition.appendWhere();
            joinCondition.append(getFilter());
            joinCondition.appendCondition(condition);
            joinCondition.appendCondition(TYPE_FILTER);
        }
        else if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.append(WbsManager.NAME_ELEMENT_ID);
            joinCondition.append(" IN (SELECT ");
            joinCondition.append(NAME_ELEMENT_ID);
            joinCondition.appendFrom();
            joinCondition.append(getTableName());
            joinCondition.appendWhere();
            joinCondition.append(getFilter());
            joinCondition.appendAnd();
            joinCondition.append(NAME_PARENT_ID);
            joinCondition.append(" IN (SELECT ");
            joinCondition.append(NAME_ELEMENT_ID);
            joinCondition.appendFrom();
            joinCondition.append(getTableName());
            joinCondition.appendWhere();
            joinCondition.append(getFilter());
            joinCondition.appendCondition(condition);
            joinCondition.appendCondition(TYPE_FILTER);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }
        return joinCondition;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap() {       
        return FIELDPROPERTYMAP;
    }
       
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet, com.ibm.rpm.scope.RPMObjectScope, int)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta) throws RPMException,
            SQLException, ParseException {

        if (rpmScope != null){
            

            RPMCalendar object = (RPMCalendar)rpmObject;
            
            //Parent
            if (rpmScope.getParent() != null) {
                
                String oldParentId = (String)relatedFields.get(index, ID_PARENT_ID);
                
                RPMCalendar newParent = null;
                
                if(isDelta){
                    newParent = (RPMCalendar)object.getParent();
                    RPMCalendar oldParent = null;
                    if(oldParentId != null){
                        oldParent = new RPMCalendar();
                        oldParent.setID(oldParentId);
                        if(newParent != null && newParent.getID() != null && newParent.getID().equals(oldParentId)){
                            oldParent = (RPMCalendar)loadByPrimaryKey(newParent,rpmScope.getParent(), messageContext, true);
                        }else{
                            ExceptionUtil.handleParentReference(this, object);
                        }
                    }else{
                        if(newParent != null){
                            ExceptionUtil.handleParentReference(this, object);	                                
                        }
                    }
                    object.deltaParent(oldParent);
                }else{
                    if (oldParentId != null) {
                        if(newParent == null){
                            newParent = new RPMCalendar();
                            newParent.setID(oldParentId);
                            newParent = (RPMCalendar)loadByPrimaryKey(newParent,rpmScope.getParent(), messageContext, false);	                                
                        }	                                
                        object.setParent(newParent);
                    } 	                        
                }
            }
            
            if(rpmScope instanceof CalendarScope){

                CalendarScope scope = (CalendarScope)rpmScope;
	            
	            if (scope.isExceptions()) {
	                CalendarExceptionManager manager = (CalendarExceptionManager) RPMManagerFactory
	                .getInstance().getRPMObjectManager(CalendarException.class.getName());
	                ArrayList temp = manager.loadByForeignKey(object
	                        .getExceptions(), null, messageContext,
	                        this, new Object[] { object.getID() }, null, null, isDelta);
	                if (temp != null && !temp.isEmpty()) {
	                    //Set parent
	                    for (int i = 0; i < temp.size(); i++) {
	                        ((CalendarException) temp.get(i))
	                        .setParent(object);
	                    }
	                    CalendarException[] children = new CalendarException[temp
	                                                                     .size()];
	                    temp.toArray(children);
	                    object.setExceptions(children);
	                }
	            }
	            
	            if (scope.getCalendars() != null) {
	                ArrayList temp = loadByForeignKey(
	                        object.getCalendars(), scope, messageContext,
	                        this, new Object[] { object.getID() }, null, null, isDelta);
	                if (temp != null && !temp.isEmpty()) {
	                    //set parent
	                    for (int i = 0; i < temp.size(); i++) {
	                        ((RPMCalendar) temp.get(i)).setParent(object);
	                    }
	                    RPMCalendar[] calendars = new RPMCalendar[temp
	                                                              .size()];
	                    temp.toArray(calendars);
	                    object.setCalendars(calendars);
	                }
	            }
            }
        }
        return rpmObject;
    }
    
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject, int)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject object, boolean isDelta) throws RPMException,
            SQLException, ParseException {

        RPMCalendar pObject = (RPMCalendar) object;

        pObject.setID(rs.getString(ID_ELEMENT_ID));
        
        String name = rs.getString(ID_ELEMENT_NAME);
        if(name != null){
            name = name.trim();
        }
        
        if(isDelta){
	        pObject.deltaStandardMonday(getCalendarWeekDay(rs
                    .getInt(ID_MONDAY_STATUS), rs
                    .getInt(ID_MONDAY_DEFAULT_MIN), rs
                    .getString(ID_MONDAY_DEFAULT_BIT), rs
                    .getInt(ID_MONDAY_MIN), rs.getString(ID_MONDAY_BIT)));

            pObject.deltaStandardTuesday(getCalendarWeekDay(rs
                    .getInt(ID_TUESDAY_STATUS), rs
                    .getInt(ID_TUESDAY_DEFAULT_MIN), rs
                    .getString(ID_TUESDAY_DEFAULT_BIT), rs
                    .getInt(ID_TUESDAY_MIN), rs.getString(ID_TUESDAY_BIT)));

            pObject.deltaStandardWednesday(getCalendarWeekDay(rs
                    .getInt(ID_WEDNESDAY_STATUS), rs
                    .getInt(ID_WEDNESDAY_DEFAULT_MIN), rs
                    .getString(ID_WEDNESDAY_DEFAULT_BIT), rs
                    .getInt(ID_WEDNESDAY_MIN), rs.getString(ID_WEDNESDAY_BIT)));

            pObject.deltaStandardThursday(getCalendarWeekDay(rs
                    .getInt(ID_THURSDAY_STATUS), rs
                    .getInt(ID_THURSDAY_DEFAULT_MIN), rs
                    .getString(ID_THURSDAY_DEFAULT_BIT), rs
                    .getInt(ID_THURSDAY_MIN), rs.getString(ID_THURSDAY_BIT)));

            pObject.deltaStandardFriday(getCalendarWeekDay(rs
                    .getInt(ID_FRIDAY_STATUS), rs
                    .getInt(ID_FRIDAY_DEFAULT_MIN), rs
                    .getString(ID_FRIDAY_DEFAULT_BIT), rs
                    .getInt(ID_FRIDAY_MIN), rs.getString(ID_FRIDAY_BIT)));

            pObject.deltaStandardSaturday(getCalendarWeekDay(rs
                    .getInt(ID_SATURDAY_STATUS), rs
                    .getInt(ID_SATURDAY_DEFAULT_MIN), rs
                    .getString(ID_SATURDAY_DEFAULT_BIT), rs
                    .getInt(ID_SATURDAY_MIN), rs.getString(ID_SATURDAY_BIT)));

            pObject.deltaStandardSunday(getCalendarWeekDay(rs
                    .getInt(ID_SATURDAY_STATUS), rs
                    .getInt(ID_SUNDAY_DEFAULT_MIN), rs
                    .getString(ID_SUNDAY_DEFAULT_BIT),
                    rs.getInt(ID_SUNDAY_MIN), rs.getString(ID_SUNDAY_BIT)));
		
	        pObject.deltaHoursPerDay(new Double(Manager.getInteger(rs,
                ID_MINUTES_PER_DAY).intValue() / 60.0));

	        pObject.deltaHoursPerWeek(new Double(Manager.getInteger(rs,
                ID_MINUTES_PER_WEEK).intValue() / 60.0));

	        pObject.deltaStartDate(Manager.getCalendar(rs, ID_START_DATE));
	
	        pObject.deltaName(name);
        
        }else{

	        pObject.setStandardMonday(getCalendarWeekDay(rs
                    .getInt(ID_MONDAY_STATUS), rs
                    .getInt(ID_MONDAY_DEFAULT_MIN), rs
                    .getString(ID_MONDAY_DEFAULT_BIT), rs
                    .getInt(ID_MONDAY_MIN), rs.getString(ID_MONDAY_BIT)));

            pObject.setStandardTuesday(getCalendarWeekDay(rs
                    .getInt(ID_TUESDAY_STATUS), rs
                    .getInt(ID_TUESDAY_DEFAULT_MIN), rs
                    .getString(ID_TUESDAY_DEFAULT_BIT), rs
                    .getInt(ID_TUESDAY_MIN), rs.getString(ID_TUESDAY_BIT)));

            pObject.setStandardWednesday(getCalendarWeekDay(rs
                    .getInt(ID_WEDNESDAY_STATUS), rs
                    .getInt(ID_WEDNESDAY_DEFAULT_MIN), rs
                    .getString(ID_WEDNESDAY_DEFAULT_BIT), rs
                    .getInt(ID_WEDNESDAY_MIN), rs.getString(ID_WEDNESDAY_BIT)));

            pObject.setStandardThursday(getCalendarWeekDay(rs
                    .getInt(ID_THURSDAY_STATUS), rs
                    .getInt(ID_THURSDAY_DEFAULT_MIN), rs
                    .getString(ID_THURSDAY_DEFAULT_BIT), rs
                    .getInt(ID_THURSDAY_MIN), rs.getString(ID_THURSDAY_BIT)));

            pObject.setStandardFriday(getCalendarWeekDay(rs
                    .getInt(ID_FRIDAY_STATUS), rs
                    .getInt(ID_FRIDAY_DEFAULT_MIN), rs
                    .getString(ID_FRIDAY_DEFAULT_BIT), rs
                    .getInt(ID_FRIDAY_MIN), rs.getString(ID_FRIDAY_BIT)));

            pObject.setStandardSaturday(getCalendarWeekDay(rs
                    .getInt(ID_SATURDAY_STATUS), rs
                    .getInt(ID_SATURDAY_DEFAULT_MIN), rs
                    .getString(ID_SATURDAY_DEFAULT_BIT), rs
                    .getInt(ID_SATURDAY_MIN), rs.getString(ID_SATURDAY_BIT)));

            pObject.setStandardSunday(getCalendarWeekDay(rs
                    .getInt(ID_SUNDAY_STATUS), rs
                    .getInt(ID_SUNDAY_DEFAULT_MIN), rs
                    .getString(ID_SUNDAY_DEFAULT_BIT),
                    rs.getInt(ID_SUNDAY_MIN), rs.getString(ID_SUNDAY_BIT)));
            	
	        pObject.setHoursPerDay(new Double(Manager.getInteger(rs,
	                ID_MINUTES_PER_DAY).intValue() / 60.0));
	
	        pObject.setHoursPerWeek(new Double(Manager.getInteger(rs,
	                ID_MINUTES_PER_WEEK).intValue() / 60.0));
	
	        pObject.setStartDate(Manager.getCalendar(rs, ID_START_DATE));
	
	        pObject.setName(name);            
            
        }
        return pObject;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException{
    
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        
        return relatedFields;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject object, MessageContext messageContext) throws RPMException, SQLException {
        
        RPMCalendar cal = null;
        
        if(object != null){
            
            cal = (RPMCalendar)object;
            
            SP_D_CAL(cal, messageContext);
            
        }	         
    }
    
    private CalendarWeekDay getCalendarWeekDay(int status, int defaultMinutes, String defaultDay, int dayMinutes, String day) throws RPMException 
    {
        CalendarWeekDay intervals = null;
        switch(status)
        {
        case USE_DEFAULT:
            if(defaultMinutes > 0){
                intervals = getIntervals(defaultDay, true);
            }else{
                intervals = new CalendarWeekDay();
                intervals.setType(CalendarWeekDayType.UseDefault);                
            }
        break;
        case NON_DEFAULT:
        case DERIVED_NON_DEFAULT:
            if(dayMinutes > 0){
                intervals = getIntervals(day, false);
            }else{
                intervals = new CalendarWeekDay();
                intervals.setType(CalendarWeekDayType.NonDefaultWorking);                
            }
        break;
        case NON_WORKING_DEFAULT:     ////for old version value 0
        case NON_WORKING:
        case DERIVED_NON_WORKING:
            intervals = new CalendarWeekDay();
            intervals.setType(CalendarWeekDayType.NonWorking);
        break;
        default:
            ExceptionUtil.handleInvalidEnumeratedType(CalendarWeekDayType.class, new Integer(status));
        }
        intervals.setMinutes(new Integer(dayMinutes));
        return intervals;
    }    
    
    private CalendarWeekDay getIntervals(String intervals, boolean defaultflag) throws RPMException
    {
        CalendarWeekDay weekdays = new CalendarWeekDay();
        ArrayList workintervals = new ArrayList();
        if((intervals == null) || (intervals.length() == 0)){
            ExceptionUtil.handleRPMException(this, ErrorCodes.NO_INTERVALS);
        }
        StringTokenizer intervalTokens = new StringTokenizer(intervals, ",");
        if(intervalTokens.countTokens()%2 != 0){
            ExceptionUtil.handleRPMException(this, ErrorCodes.UNBALANCED_INTERVALS);
        }
        while(intervalTokens.hasMoreTokens())
            workintervals.add(WorkIntervalUtil.createWorkInterval(intervalTokens.nextToken(),intervalTokens.nextToken()));
        WorkInterval workintarray[] = new WorkInterval[workintervals.size()];
        workintervals.toArray(workintarray);
        weekdays.setIntervals(workintarray);
        if(defaultflag)
            weekdays.setType(CalendarWeekDayType.UseDefault);
        else
            weekdays.setType(CalendarWeekDayType.NonDefaultWorking);
        return weekdays;
    }
    
    private String SP_I_CAL(RPMCalendar calendar, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_CAL";
        Object args[] = new Object[4];
        args[0] = calendar.getName();
        if(calendar.getParent() != null)
            args[1] = calendar.getParent().getID();
        else
            args[1] = null;
        args[2] = DEFAULT_CALENDAR;
        args[3] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_CAL(RPMCalendar calendar, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CAL";
        Object args[] = new Object[4];
        args[0] = calendar.getID();
        args[1] = calendar.getName();
        args[2] = null;
        args[3] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_U_CAL_HOURS(RPMCalendar calendar, CalendarWeekDay weekday,
            int weekint, MessageContext messageContext) throws RPMException,
            SQLException {
               
        int weekdayStatus = 0;
        if((weekday.getType() == null) || (weekday.getType() == CalendarWeekDayType.UseDefault))
        {
            weekdayStatus = 1;
        }
        if(weekday.getType() == CalendarWeekDayType.NonWorking)
        {
            weekdayStatus = 2;
        }
        if(weekday.getType() == CalendarWeekDayType.NonDefaultWorking)
        {
            weekdayStatus = 2;
        }
        String name = "SP_U_CAL_HOURS";
        Object args[] = new Object[15];
        args[0] = calendar.getID();
        args[1] = new Integer(weekint);
        
        WorkInterval[] wintervals = weekday.getIntervals();
        WorkIntervalUtil.validate(wintervals);
        
        for(int i = 0 ; i < 5 ; i++)
        {
            if(wintervals != null && i < wintervals.length)
            {
                args[2*i+2] = WorkIntervalUtil.formatTime(wintervals[i].getFrom());
                args[2*i+3] = WorkIntervalUtil.formatTime(wintervals[i].getTo());
            }
            else
            {
                args[2*i+2] = null;
                args[2*i+3] = null;
            }
        }
        args[12] = new Integer(weekdayStatus);
        java.util.Calendar startdate = calendar.getStartDate();
        if(startdate == null){
            startdate = DateUtil.getCalendarInstance();
        }
        args[13] = DateUtil.convertToString(startdate);
        
        args[14] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_U_CAL_MINUTES(RPMCalendar calendar, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CAL_MINUTES";
        Object args[] = new Object[4];
        args[0] = calendar.getID();
        
        if (calendar.getHoursPerDay() != null)
        {
            //hours per days truncated to the closest multiple of 0.25 hours
            args[1] = new Integer((int)(Math.floor(calendar.getHoursPerDay().doubleValue()*4.0)/4.0 * 60.0));
        }

        if (calendar.getHoursPerWeek() != null)
        {
            //hours per week truncated to the closest multiple of 0.25 hours
            args[2] = new Integer((int)(Math.floor(calendar.getHoursPerWeek().doubleValue()*4.0)/4.0 * 60.0));
        }
        
        args[3] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_D_CAL(RPMCalendar calendar, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_D_CAL";
        Object args[] = new Object[4];
        args[0] = calendar.getID();  	
        args[1] = ManagerUtil.getParentId(this,calendar, messageContext, TABLE_NAME);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext,name, args);
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject, boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
        RPMCalendar pObject = (RPMCalendar) rpmObject;
        CalendarScope scope = (CalendarScope)rpmScope;
        
        if (pObject.testStandardMondayModified()) 
            SP_U_CAL_HOURS(pObject, pObject.getStandardMonday(), 1, messageContext);
        
        if (pObject.testStandardTuesdayModified()) 
            SP_U_CAL_HOURS(pObject, pObject.getStandardTuesday(), 2, messageContext);
        
        if (pObject.testStandardWednesdayModified())
            SP_U_CAL_HOURS(pObject, pObject.getStandardWednesday(), 3, messageContext);
        
        if (pObject.testStandardThursdayModified())
            SP_U_CAL_HOURS(pObject, pObject.getStandardThursday(), 4, messageContext);
        
        if (pObject.testStandardFridayModified())
            SP_U_CAL_HOURS(pObject, pObject.getStandardFriday(), 5, messageContext);
        
        if (pObject.testStandardSaturdayModified())
            SP_U_CAL_HOURS(pObject, pObject.getStandardSaturday(), 6, messageContext);
        
        if (pObject.testStandardSundayModified())
            SP_U_CAL_HOURS(pObject, pObject.getStandardSunday(), 0, messageContext);
        
        if (pObject.testHoursPerDayModified() || pObject.testHoursPerWeekModified())
            SP_U_CAL_MINUTES(pObject, messageContext);
        
        if (pObject.testNameModified()) 
            SP_U_CAL(pObject, messageContext);
        //     afterUpdate(pObject); 
        // listener callback                           
        
        if (scope != null) 
        {
            //Calendars
            pObject.setCalendars((RPMCalendar[]) 
                    updateChildren(pObject, 
                            pObject.getCalendars(), 
                            scope.getCalendars(), 
                            messageContext, null));
            //Exceptions
            pObject.setExceptions((CalendarException[]) 
                    updateChildren(pObject, 
                            pObject.getExceptions(), 
                            scope.isExceptions(), 
                            messageContext, null));
        }
        return pObject;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException
    {
        RPMCalendar pObject = (RPMCalendar) rpmObject;
        pObject.setID(SP_I_CAL(pObject, messageContext));
    }    
}
