/*
 * Created on Aug 9, 2005
 */
package com.ibm.rpm.applicationadministration.managers;

import java.sql.Clob;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.NoteMinutesRTF;
import com.ibm.rpm.applicationadministration.containers.ProjectRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.applicationadministration.containers.WorkProductRTF;
import com.ibm.rpm.applicationadministration.util.RtfUtil;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * @author karim
 */
public class RtfAssignmentManager extends AbstractRPMObjectManager
{

    {
        //this is set to true for this manager so that we create the statements
        // correctly when
        //loading CLOBS
        useDBStatementsForLOBS = true;
    }

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();
    static
    {
        CONTAINERS.add(RtfAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                RtfAssignmentManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column BIT_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BIT_FLAG = 2;

    public static final int TYPE_BIT_FLAG = Types.INTEGER;

    public static final String NAME_BIT_FLAG = "BIT_FLAG";
    
    public static final String PROPERTY_BIT_FLAG = "BITFLAG";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BIT_FLAG, NAME_BIT_FLAG);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 3;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column CLOB_DATA of type Types.CLOB mapped to Integer.
     */
    public static final int ID_CLOB_DATA = 4;

    public static final int TYPE_CLOB_DATA = Types.CLOB;

    public static final String NAME_CLOB_DATA = "CLOB_DATA";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CLOB_DATA = "VALUE";
    static{ FIELDPROPERTYMAP.put(PROPERTY_CLOB_DATA, NAME_CLOB_DATA);}

    /**
     * Create an array of type string containing all the fields of the
     * TMT_CLIENTS table.
     */
    private static final String[] FIELD_NAMES =
    {
        NAME_ELEMENT_ID, NAME_BIT_FLAG, NAME_PARENT_ID, NAME_CLOB_DATA
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public String getTableName()
    {
        ExceptionUtil.handleGetTableName(this);
        return null;
    }

    public String getTableName(RPMObject rpmObject)
    {
        String contextName = getContextName(rpmObject);
        return getTableName(contextName);
    }

    public String getTableName(String contextName)
    {
        return RtfUtil.getInstance().getTableName(contextName);
    }

    protected Integer getTableType(RPMObject rpmObject, MessageContext messageContext) 
        throws RPMException, SQLException
    {
        String contextName = getContextName(rpmObject);
        Integer tableID = getTableType(contextName);
        
        /*
         * In the delphi client, if a scope Element was transferred under WBS, 
         * and one of its rtf assignments is modified, it is CRI_RTF which 
         * gets updated, and not WBS_RTF
         * 
         * if rpmObject is a scope element and it was transferred, its
         * correct table ID is that of CRI_RTF, and not WBS_RTF.
         * 
         * GenericProjects cannot be the result of a transfer so there is
         * no need to check in the db 
         * 
         */
        RPMObject parent = rpmObject.getParent();
        if (parent != null && parent.getID() != null
                && parent instanceof WorkElement
                && !(parent instanceof GenericProject))
        {
            if (ScopeManagementUtil.scopeElementExists(parent.getID(),
                messageContext))
            {
                // this was originally a scopeElement which got transferred under WBS
                tableID = getTableType(ContextUtil.SCOPE_CONTEXT);                
            }
        }
        
        return tableID;
    }

    protected Integer getTableType(String contextName)
    {
        return RtfUtil.getInstance().getTableType(contextName);
    }

    private String getContextName(RPMObject rpmObject){
        String contextName = rpmObject.getContextName();
        if(contextName==null){
            contextName = getContextNameFromParent(rpmObject);
        }
        return contextName;
    }
    
    private String getContextNameFromParent(RPMObject rpmObject){
        RPMObject parent = rpmObject.getParent();
        if(parent != null){
             if(parent instanceof ClientCostCenter){
                 return ContextUtil.CLIENT_CONTEXT;
             } else if (parent instanceof Asset){
                 return ContextUtil.ASSET_CONTEXT;
             } else if (parent instanceof WorkElement){
                 return ContextUtil.WBS_CONTEXT;
             } else if (parent instanceof DocumentElement){
                 return ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
             } else if(parent instanceof ScopeElement){
                 return ContextUtil.SCOPE_CONTEXT;
             }
        }
        return null;
    }
    
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return null; ////no primary key
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        /*
         * ELEMENT_ID is mapped to the parent and PARENT_ID is mapped to the
         * rtf
         */
        return new String[]
        {
            NAME_ELEMENT_ID, NAME_BIT_FLAG 
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        RtfAssignment object = (RtfAssignment) rpmObject;

        // The rtfAssignment's ID should be equal to the parent's ID
        String parentId = object.getID();

        Integer bitFlag = null;
        
        if (object.getRtf() != null && object.getRtf().getName() != null)
        {
            bitFlag = transformNameToBitFlag(object.getRtf().getName());
        }
              
        return new Object[]
        {
        	parentId, bitFlag
        };
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject)
    {
        RtfAssignment rtfAssignment = (RtfAssignment) rpmObject;
        rtfAssignment.setID((String) primaryKey[0]);
        
        if (rtfAssignment.getRtf() == null)
        {
            try
            {
                rtfAssignment.setRtf(initRtf(rtfAssignment));
            }
            catch(RPMException e)
            {
                rtfAssignment.setRtf(new DatafieldRTF());
            }
        }
        
        rtfAssignment.getRtf().setName(transformBitFlagToName(Integer.valueOf((String) primaryKey[1])));
        
        return rtfAssignment;
    }

    private DatafieldRTF initRtf(RtfAssignment rtfAssignment)
            throws RPMException
    {
        String contextName = getContextName(rtfAssignment);

        DatafieldRTF rtf = null;
        if (ContextUtil.CLIENT_CONTEXT.equals(contextName))
        {
            rtf = new ClientRTF();
        }
        else if (ContextUtil.ASSET_CONTEXT.equals(contextName))
        {
            rtf = new AssetRTF();
        }
        else if (ContextUtil.WBS_CONTEXT.equals(contextName))
        {
            rtf = initWBSRtf(rtfAssignment);
        }
        else if (ContextUtil.UMBRELLA_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.WBS_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.ASSET_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.CLIENT_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.RESOURCE_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.SCOPE_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.POOL_DOCUMENT_CONTEXT.equals(contextName))
        {
            rtf = new NoteMinutesRTF();
        }
        else if (ContextUtil.SCOPE_CONTEXT.equals(contextName))
        {
            rtf = initScopeRtf(rtfAssignment);
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, rtfAssignment,
                    ExceptionUtil.INIT_RTF);
        }
        //set contextName
        rtf.setContextName(contextName);
        return rtf;
    }
    
    private DatafieldRTF initScopeRtf(RtfAssignment rtfAss) throws RPMException
    {
        RPMObject parent = rtfAss.getParent();
        
        if ((parent instanceof SummaryTask || parent instanceof Task) 
                && ((WorkElement) parent).getScopeElement() != null)
        {
            parent = ((WorkElement) parent).getScopeElement();      
        }
        
        if(parent instanceof ScopeTask)
        {
            return new TaskRTF();
        }
        else if(parent instanceof ScopeMilestone)
        {
            return new MilestoneRTF();
        }
        else if(parent instanceof ScopeSummaryTask)
        {
            return new SummaryTaskRTF();
        }
        else if(parent instanceof ScopeDeliverable)
        {
            return new DeliverableRTF();
        }
        else if(parent instanceof Defect)
        {
            return new DefectRTF();
        }
        else if(parent instanceof ChangeRequest)
        {
            return new ChangeRequestRTF();
        }
        else if(parent instanceof ServiceRequest)
        {
            return new ServiceRequestRTF();
        }
        else if(parent instanceof Issue)
        {
            return new IssueRTF();
        }
        else if(parent instanceof Action)
        {
            return new ActionRTF();
        }
        else if(parent instanceof Requirement)
        {
            return new RequirementRTF();
        }
        else if(parent instanceof Risk)
        {
            return new RiskRTF();
        }
        else
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rtfAss.getClass()),
                    ((parent != null)? StringUtil.getShortClassName(parent.getClass()): "null"),
                    "ScopeMilestone, ScopeSummaryTask, ScopeDeliverable, Defect, ChangeRequest, ServiceRequest, Issue, Action, Requirement, Risk, SummaryTask, Task"
            };
            throw new RPMException(ErrorCodes.WRONG_PARENT_TYPE, errorParams);
        }
    }
    
    private DatafieldRTF initWBSRtf(RtfAssignment rtfAssignment) throws RPMException
    {
        RPMObject parent = rtfAssignment.getParent();
        if (parent instanceof GenericProject)
        {
            return new ProjectRTF();
        }
        else if (parent instanceof Deliverable || parent instanceof ScopeDeliverable)
        {
            return new DeliverableRTF();
        }
        else if (parent instanceof WorkProduct)
        {
            return new WorkProductRTF();
        }
        else if (parent instanceof SummaryTask)
        {
            return new SummaryTaskRTF();
        }
        else if (parent instanceof Milestone || parent instanceof ScopeMilestone)
        {
            return new MilestoneRTF();
        }
        else if (parent instanceof Task || parent instanceof ScopeTask)
        {
            return new TaskRTF();
        }
        else
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(rtfAssignment.getClass()),
                ((parent != null)? StringUtil.getShortClassName(parent.getClass()): "null"),
                "GenericProject, Deliverable, ScopeDeliverable, WorkProduct, SummaryTask, Milestone, ScopeMilestone, Task and ScopeTask"
            };
            throw new RPMException(ErrorCodes.WRONG_PARENT_TYPE, errorParams);
        }
    }

    private RPMObject initParent(RtfAssignment rtfAssignment)
            throws RPMException
    {
        String contextName = getContextName(rtfAssignment);

        RPMObject parent = null;
        if (ContextUtil.CLIENT_CONTEXT.equals(contextName))
        {
            parent = new GenericClient();
        }
        else if (ContextUtil.ASSET_CONTEXT.equals(contextName))
        {
            parent = new GenericAsset();
        }
        else if (ContextUtil.WBS_CONTEXT.equals(contextName))
        {
            parent = new WorkElement();
        }
        else if (ContextUtil.UMBRELLA_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.CLIENT_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.SCOPE_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.WBS_DOCUMENT_CONTEXT.equals(contextName)
                || ContextUtil.ASSET_DOCUMENT_CONTEXT.equals(contextName))
        {
            parent = new GenericDocument();
        }
        else if (ContextUtil.SCOPE_CONTEXT.equals(contextName))
        {
            parent = new ScopeElement();
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, rtfAssignment,
                    ExceptionUtil.INIT_RTF);
        }

        parent.setID(rtfAssignment.getID());
        parent.setContextName(contextName);

        return parent;
    }
    
    protected Integer getBitFlag(RtfAssignment rtfAssignment, MessageContext messageContext)
    	throws RPMException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(DatafieldManager.NAME_ELEMENT_ID);

        Object[] arguments = new Object[]
        {
            Integer.valueOf(rtfAssignment.getRtf().getID())
        };
        
        String datafieldRtfNameStr = (String) ManagerUtil.selectColumnValue(
            String.class, rtfAssignment.getRtf(), "name", DatafieldManager.NAME_ELEMENT_NAME, 
            DatafieldManager.TABLE_NAME, condition, arguments, messageContext);
        
        Integer datafieldRtfNameInt = null;
        
        if (datafieldRtfNameStr != null)
            datafieldRtfNameInt = Integer.valueOf(datafieldRtfNameStr.trim());
        
        // Convert the datafield name into the corresponding bitFlag
        Integer bitFlag = new Integer(new Double(Math.pow(2, datafieldRtfNameInt.intValue() - 1)).intValue());
        
        return bitFlag;
    }
    
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException
    {
        return new Integer(-1);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId)
    {

        RPMObject object = null;

        object = new RtfAssignment();
        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        RtfAssignment rtfValue = (RtfAssignment) rpmObject;

        rtfValue.setID(rs.getString(ID_ELEMENT_ID));

        Clob clobData = rs.getClob(ID_CLOB_DATA);
        String value = (clobData != null) ? clobData.getSubString(1, (int) clobData
            .length()) : null;

        if (isDelta)
        {
            rtfValue.deltaValue(value);
        }
        else
        {
            rtfValue.setValue(value);
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException
    {

        String parentId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, parentId);

        Integer bitFlagId = new Integer(resultSet.getInt(ID_BIT_FLAG));
        relatedFields.put(index, ID_BIT_FLAG, bitFlagId);
        
        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else if (manager instanceof RtfAssignmentManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            int rankLenght = 4 * level;

            String rank = getTableName(contextName) + ".RANK";

            where.append("(LEVEL_ID >= ");
            where.append(level);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName)
            throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;
        if (isValidParentManager(manager))
        {
            joinCondition = new JoinCondition();
            StringBuffer sb = new StringBuffer();
            sb.append(manager.getPrimaryKey() + " IN ");
            sb
                .append("( SELECT " + NAME_ELEMENT_ID + " FROM "
                    + getTableName(contextName));
            if (condition != null)
            {
                sb.append(" WHERE " + condition);
            }
            else
            {
            }
            joinCondition.setCondition(sb.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }
        return joinCondition;
    }
    
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
			String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (isValidParentManager(manager))
        {
            String where = "";

            if (manager instanceof DocumentManager
                    || manager instanceof WbsManager
                    || manager instanceof ScopeManager)
            {
                where = " ELEMENT_ID = ? ";
            }
            else
            {
                // TODO: please make sure if PARENT_ID is necessary at all and
                // can be replaced by ELEMENT_ID
                where = " PARENT_ID = ? ";
            }

            // ScopeTask, ScopeDeliverable and ScopeMilestone have a context of WBS even though 
            // their rtfAssignments are in the scope tables
            if (manager instanceof ScopeManager && contextName == ContextUtil.WBS_CONTEXT)
            {
                contextName = ContextUtil.SCOPE_CONTEXT;
            }
            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(where);
            joinCondition.setTableName(getTableName(contextName));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

	//TODO activate managers 
    private boolean isValidParentManager(RPMObjectManager manager)
    {
        return manager instanceof ClientManager
                || manager instanceof DocumentManager
                || manager instanceof AssetManager 
                || manager instanceof WbsManager 
                || manager instanceof ScopeManager;
    }
	
    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta
    ) throws RPMException, SQLException, ParseException {
        //Everything is loaded without scope
        RtfAssignment rtfAssignment = (RtfAssignment) rpmObject;

        // Parent
        if (rpmObject.getParent() == null)
        {
            loadParent(rtfAssignment, messageContext, isDelta);
        }
        
        //Related object rtf
        DatafieldRTF rtf = null;
        if (rtfAssignment.getRtf() != null && rtfAssignment.getRtf().retrieveTypeID() != null)
        {
            rtf = rtfAssignment.getRtf();
        }
        else
        {
            rtfAssignment.setParent(ScopeManagementUtil
                .loadScopeElement(rtfAssignment.getParent(), messageContext));
            
            rtf = initRtf(rtfAssignment);
        }

        Object[] args = new Object[]
        {
            transformBitFlagToName((Integer) relatedFields.get(index, ID_BIT_FLAG)), rtf.retrieveTypeID()
        };

        DatafieldManager datafieldManager = (DatafieldManager) getManagerInstance(rtf
            .getClass().getName());
        ArrayList rtfs = datafieldManager.loadByForeignKey(new RPMObject[]
        {
            rtfAssignment.getRtf()
        }, null, messageContext, this, args, null, rtfAssignment.getContextName(),
            isDelta);

        if (rtfs != null && rtfs.size() != 0)
        {
            if (isDelta)
            {
                rtfAssignment.deltaRtf((DatafieldRTF) rtfs.get(0));
            }
            else
            {
                rtfAssignment.setRtf((DatafieldRTF) rtfs.get(0));
            }
        }
        return rpmObject;
    }

    private void loadParent(RtfAssignment rtfAssignment,
            MessageContext messageContext, boolean isDelta) throws RPMException,
            ParseException, SQLException
    {
        RPMObject parentObject = initParent(rtfAssignment);
        
        RPMObjectManager manager = getManagerInstance(parentObject.getClass().getName());
        parentObject = manager.loadByPrimaryKey(parentObject, null, messageContext, isDelta);
        
        if (isDelta)
        {
            rtfAssignment.deltaParent(parentObject);
        }
        else
        {
            rtfAssignment.setParent(parentObject);
        }        
    }
        
    private String transformBitFlagToName(Integer bitFlag)
    {
        int name = 1;
        
        int bitflag = bitFlag.intValue();
        
        while(bitflag != 1){
            
            bitflag = bitflag >> 1;
            ++name;
        }
        
        return String.valueOf(name);
    }
    
    private Integer transformNameToBitFlag(String name)
    {
        return new Integer((int) (Math.pow(2, Integer.valueOf(name).intValue() - 1)));
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        if(rpmObject.getContextName()==null){
            rpmObject.setContextName(getContextName(rpmObject));            
        }
        rpmObject.setID(SP_U_RTF((RtfAssignment) rpmObject, messageContext));

    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        if(rpmObject.getContextName()==null){
            rpmObject.setContextName(getContextName(rpmObject));            
        }
        SP_U_RTF((RtfAssignment) rpmObject, messageContext);

        return rpmObject;
    }

    private String SP_U_RTF(RtfAssignment rtfAssignment, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RTF";
        Object args[] = new Object[5];
        args[0] = getTableType(rtfAssignment, messageContext);
        if (rtfAssignment.getParent() != null)
        {
            args[1] = rtfAssignment.getParent().getID();
        }
        args[2] = getBitFlag(rtfAssignment, messageContext);
        args[3] = rtfAssignment.getValue();
        args[4] = getUser(messageContext).getID();

        return ManagerUtil.executeProcedure(this.getClass(), messageContext,
            name, args);
    }

    public void isValidPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();
        
        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }
        
        if (((RtfAssignment) rpmObject).getRtf() == null
                || ((RtfAssignment) rpmObject).getRtf().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(", ");
            invalidPrimaryKeys.append("Rtf.ID");
            valid = false;
        }
        
        if (valid)
        {
            try
            {
                //TODO use a direct sql instead
                
                // Although Rtf.ID is mandatory, we actually need Rtf.Name
                // This is the only place we can really load it before getPrimaryKeyValues is called
	            DatafieldManager datafieldManager = (DatafieldManager) getManagerInstance(
	                ((RtfAssignment) rpmObject).getRtf().getClass().getName());
	            DatafieldRTF datafieldRTF = (DatafieldRTF) datafieldManager.loadByPrimaryKey(((RtfAssignment) rpmObject).getRtf(),
	                null, messageContext, false);
	            ((RtfAssignment) rpmObject).getRtf().setName(datafieldRTF.getName());     
	            
	            // Load parent scopeElement, if applicable
	            rpmObject.setParent(ScopeManagementUtil
	                .loadScopeElement(rpmObject.getParent(), messageContext));
            }
            catch(ParseException e)
            {
                ExceptionUtil.handleException(this, e);
            }
            catch(SQLException e)
            {
                ExceptionUtil.handleException(this, e);
            }
        }
        else
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, invalidPrimaryKeys.toString());
        }
    }
}
