package com.ibm.rpm.applicationadministration.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.managers.DocumentUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.resource.containers.Resource;

public class StatusUpdateManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{

    	CONTAINERS.add(StatusUpdate.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                		StatusUpdateManager.class);
        }
    }
    
    public static final int HIST_ID_LEVEL_ID = 1;

    public static final int HIST_TYPE_LEVEL_ID = Types.INTEGER;

    public static final String HIST_NAME_LEVEL_ID = "LEVEL_ID";

    public static final int HIST_ID_TYPE_ID = 2;

    public static final int HIST_TYPE_TYPE_ID = Types.INTEGER;

    public static final String HIST_NAME_TYPE_ID = "TYPE_ID";

    public static final int HIST_ID_RANK = 3;

    public static final int HIST_TYPE_RANK = Types.INTEGER;

    public static final String HIST_NAME_RANK = "RANK";

    public static final int HIST_ID_PARENT_ID = 4;

    public static final int HIST_TYPE_PARENT_ID = Types.CHAR;

    public static final String HIST_NAME_PARENT_ID = "PARENT_ID";

    public static final int HIST_ID_ELEMENT_ID = 5;

    public static final int HIST_TYPE_ELEMENT_ID = Types.CHAR;

    public static final int HIST_ID_ELEMENT_NAME = 6;

    public static final int HIST_TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String HIST_NAME_ELEMENT_NAME = "ELEMENT_NAME";

    public static final int HIST_ID_CHILD_COUNT = 7;

    public static final int HIST_TYPE_CHILD_COUNT = Types.INTEGER;

    public static final int HIST_ID_DELETED_COUNT = 8;

    public static final int HIST_TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String HIST_NAME_DELETED_COUNT = "DELETED_COUNT";

    public static final int HIST_ID_CYCLE_NAME = 9;

    public static final int HIST_TYPE_CYCLE_NAME = Types.VARCHAR;

    public static final int HIST_ID_DESCRIPTION = 10;

    public static final int HIST_TYPE_DESCRIPTION = Types.VARCHAR;

    public static final int HIST_ID_REVISION = 11;

    public static final int HIST_TYPE_REVISION = Types.VARCHAR;

    public static final int HIST_ID_REVISION_IMPACT = 12;

    public static final int HIST_TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String HIST_NAME_REVISION_IMPACT = "REVISION_IMPACT";

    public static final int HIST_ID_REVISION_HISTORY = 13;

    public static final int HIST_TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String HIST_NAME_REVISION_HISTORY = "REVISION_HISTORY";

    public static final int HIST_ID_REVISION_TYPE = 14;

    public static final int HIST_TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String HIST_NAME_REVISION_TYPE = "REVISION_TYPE";

    public static final int HIST_ID_DOCUMENT_SIZE = 15;

    public static final int HIST_TYPE_DOCUMENT_SIZE = Types.INTEGER;

    public static final int HIST_ID_REC_DATETIME = 16;

    public static final int HIST_TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String HIST_NAME_REC_DATETIME = "REC_DATETIME";

    public static final int HIST_ID_AUTHOR = 17;

    public static final int HIST_TYPE_AUTHOR = Types.VARCHAR;

    /**
     * Creates a new StatusUpdateManager instance.
     * 
     * @return StatusUpdateManager
     */
    public StatusUpdateManager()
    {
        super();
    }

    protected String[] getFieldsNames()
    {
        return null;
    }

    protected String getAllFields()
    {
        return null;
    }

    public String getPrimaryKey()
    {
        return null;
    }

    protected Integer getContainerType(ResultSet resultSet)
    throws RPMException, SQLException
	{
    	return null;
	}

	public String getTableName(){
	    ExceptionUtil.handleGetTableName(this);
		return null;
	}
	
	public String getDocumentTableName(RPMObject rpmObject){
		return getDocumentTableName(rpmObject.getContextName());
	}
	public String getDocumentTableName(String contextName){
		return DocumentUtil.getInstance().getHistoryTableName(contextName);
	}
	protected Integer getDocumentTableType(RPMObject rpmObject){
		return getDocumentTableType(rpmObject.getContextName());
	}
	protected Integer getDocumentTableType(String contextName){
		return DocumentUtil.getInstance().getTableType(contextName);
	}

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected StatusUpdate decodeHistoryRow(ResultSet rs)
            throws RPMException, SQLException, ParseException
    {
    	
    	// LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, 
    	// CHILD_COUNT, DELETED_COUNT, CYCLE_NAME, DESCRIPTION, REVISION, 
    	// REVISION_IMPACT, REVISION_HISTORY, REVISION_TYPE, DOCUMENT_SIZE, 
    	// REC_DATETIME, AUTHOR, SUBJECT, ELEMENT_NUMBER, ATTACHEMENT, CHECKLIST, 
    	// STAGE_NAME ,OUTCOME_NAME

        StatusUpdate statusUpdate = new StatusUpdate();
        statusUpdate.setID(rs.getString("ELEMENT_ID"));
        statusUpdate.assignTypeID(new Integer(rs.getInt("TYPE_ID")));
        statusUpdate.setNameStep(rs.getString("ELEMENT_NAME"));
        
        statusUpdate.setRevision(rs.getString("REVISION"));
        statusUpdate.setChangeBrief(rs.getString("REVISION_HISTORY"));

        Timestamp tempTimeStamp = rs.getTimestamp("REC_DATETIME");
        Calendar tempCalendar = DateUtil.getCalendarInstance();

        if (tempTimeStamp != null)
        {
            tempCalendar.setTimeInMillis(tempTimeStamp.getTime());
        }
        statusUpdate.setDateTime(tempCalendar);

        Resource tempResource = new Resource();
        tempResource.setFullName(rs.getString("AUTHOR"));
        statusUpdate.setParent(tempResource);

        return statusUpdate;
    }

//    protected ArrayList convertResultSetToArrayList(ResultSet rs)
//            throws RPMException, SQLException, ParseException
//    {
//
//        StatusUpdate statusUpdate = null;
//        ArrayList statusUpdates = new ArrayList();
//        while (rs.next())
//        {
//            statusUpdate = decodeHistoryRow(rs);
//            statusUpdates.add(statusUpdate);
//        }
//
//        return statusUpdates;
//
//    }

    private ArrayList SP_S_DOC_REVISION(GenericDocument documentNonFolder,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        String name = "SP_S_DOC_REVISION";

        Object args[] = new Object[3];

        args[0] = getDocumentTableType(documentNonFolder);
        args[1] = documentNonFolder.getID();
        args[2] = new Integer(0);

        ArrayList arrayList = loadFromProcedure(messageContext, name,
                                    args);

        return arrayList;
    }

    /**
     * @param procedureName
     * @param args
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    public static ArrayList loadFromProcedure(MessageContext context,
            String procedureName, Object[] args) throws SQLException,
            RPMException, ParseException
    {
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(context);
            rs = Manager.executeStoredProcedure(c, procedureName, args, true, context);

            //Check if we really want to use an SP_S.. stored procedure
            //ArrayList arrayList = convertResultSetToArrayList(rs);
            return null;
        }
        finally
        {
            if (rs != null)
            {
                Manager.close(context,rs);
            }
            if (c != null)
            {
                Manager.releaseConnection(context, c);
            }
        }
    }    
    
    
    public ArrayList loadStatusUpdates(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            RPMObjectManager manager, Object args[], String extraWhere,
            String contextName, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
    	if (manager instanceof DocumentManager)
        {
    		return SP_S_DOC_REVISION((GenericDocument) rpmObject, messageContext);
    		
        } else {
        	ExceptionUtil.handleRPMException(this, ErrorCodes.NO_PARENT_MANAGER);
        }
    	return null;
    }


    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
    	// nothing

        return null;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName) throws RPMException
    {
    	// nothing

        return null;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
    	// nothing

        return null;
    }
        
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return null;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        return null;
    }

    public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
    	// nothing

    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
    	// nothing

        return null;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
    	// nothing

    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
    	// nothing

    }
}
