/**
 * Created on Mar 3, 2006
 * @author shahla
 */
package com.ibm.rpm.applicationadministration.util;

import java.util.List;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.applicationadministration.constants.ErrorCodes;

public class AttributeValidator extends GenericValidator
{
    public static boolean validateAttributeAssignmentType(RPMObject object,
            AttributeAssignment assignment, Class expectedType,
            String fieldName, MessageContext messageContext)
    {
        boolean valid = true;
        RPMObject root = null;
        String contextName = null;
        if (assignment.getAttribute() != null)
            contextName = assignment.getAttribute().getContextName();
        
        if (contextName != null)
        {
            root = AttributeUtil.getInstance().getModuleFromContextName(
                contextName);
        }
        else
        {
            root = AttributeUtil.getInstance().getAttributeRoot(
                assignment.getAttribute());
        }
        
        valid = expectedType.isInstance(root);
        
        if (!valid)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()),
                    StringUtil.getShortClassName(expectedType), fieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.ATTRIBUTE_ASSIGNMENT_TYPE_MISMATCH, errorParams,
                assignment.getClass().getName(), fieldName, assignment.getID());
            addException(e, messageContext);
        }

        return valid;
    }
    
    public static boolean validateAttributeRoot(GenericAttribute attribute,
            MessageContext messageContext)
    {
        RPMObject object = AttributeUtil.getInstance().getAttributeRoot(
            attribute);

        // the root should either be a module object or have a context name
        boolean valid = object instanceof AbstractModule
                || object.getContextName() != null;

        if (!valid)
        {
            List validRootsList = AttributeUtil.getInstance()
                .getValidAttributeRoot();
            StringBuffer validRoots = new StringBuffer();
            for (int i = 0; i < validRootsList.size(); i++)
            {
                validRoots.append(StringUtil
                    .getShortClassName((Class) validRootsList.get(i)));
                if (i < validRootsList.size() - 1)
                {
                    validRoots.append(", ");
                    if (i == validRootsList.size() - 2)
                        validRoots.append("or ");
                }
            }

            String[] errorParams =
            {
                    StringUtil.getShortClassName(attribute.getClass()),
                    validRoots.toString()
            };
            RPMException e = new RPMException(
                ErrorCodes.INVALID_ATTRIBUTE_ROOT, errorParams, attribute
                    .getClass().getName(), null, attribute.getID());
            addException(e, messageContext);
        }

        return valid;
    }
}
