/*
 * Created on Jan 9, 2006
 * @author mmatwiy
 * Checkpoint validation of container CrossChargeFinancials
 */
package com.ibm.rpm.asset.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class CrossChargeFinancialsCheckpoint extends AbstractCheckpoint
{

    private static final List CROSS_CHARGE_PARENTS = new ArrayList();

    static
    {
        CROSS_CHARGE_PARENTS.add(Asset.class);
    }

    protected static CrossChargeFinancialsCheckpoint instance = new CrossChargeFinancialsCheckpoint();

    public static CrossChargeFinancialsCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, CROSS_CHARGE_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            CrossChargeFinancials.class, msgCtx))
        {
            return;
        }

        CrossChargeFinancials financials = (CrossChargeFinancials) rpmObject;

        GenericValidator.validateMandatory(financials,
            ValidationConstants.COST_CENTER_FIELD, financials.getCostCenter(),
            msgCtx);

        if (financials.testCostModified())
        {
            GenericValidator.validateRange(financials,
                ValidationConstants.COST_FIELD, financials.getCost(),
                ValidationConstants.COST_MIN, ValidationConstants.COST_MAX,
                msgCtx);
        }

        if (financials.testEstimatedBudgetModified())
        {
            GenericValidator.validateRange(financials,
                ValidationConstants.ESTIMATED_BUDGET_FIELD, financials
                    .getEstimatedBudget(),
                ValidationConstants.ESTIMATED_BUDGET_MIN,
                ValidationConstants.ESTIMATED_BUDGET_MAX, msgCtx);
        }

        if (financials.testEstimatedActualModified())
        {
            GenericValidator.validateRange(financials,
                ValidationConstants.ESTIMATED_ACTUAL_FIELD, financials
                    .getEstimatedActual(),
                ValidationConstants.ESTIMATED_ACTUAL_MIN,
                ValidationConstants.ESTIMATED_ACTUAL_MAX, msgCtx);
        }

        if (financials.testEstimatedPaidModified())
        {
            GenericValidator.validateRange(financials,
                ValidationConstants.ESTIMATED_PAID_FIELD, financials
                    .getEstimatedPaid(),
                ValidationConstants.ESTIMATED_PAID_MIN,
                ValidationConstants.ESTIMATED_PAID_MAX, msgCtx);
        }

        if (financials.testQuantityModified())
        {
            GenericValidator.validateRange(financials,
                ValidationConstants.FINANCIALS_QUANTITY_FIELD, financials
                    .getQuantity(),
                ValidationConstants.FINANCIALS_QUANTITY_MIN,
                ValidationConstants.FINANCIALS_QUANTITY_MAX, msgCtx);
        }
    }

}
