/*
 * Created on 28-Nov-2005
 */
package com.ibm.rpm.asset.checkpoints;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author dominiqu
 */
public class MaintenanceEstimatesCheckpoint extends AbstractCheckpoint
{
    private static final Class MAINTENANCEESTIMATES_PARENTS = Asset.class;

    protected static MaintenanceEstimatesCheckpoint instance = new MaintenanceEstimatesCheckpoint();

    public static MaintenanceEstimatesCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        if (!GenericValidator.validateInstance(rpmObject,
            MaintenanceEstimates.class, msgCtx))
            return;

        super.isValidParent(rpmObject, msgCtx, true,
            MAINTENANCEESTIMATES_PARENTS);

        MaintenanceEstimates estimates = (MaintenanceEstimates) rpmObject;

        if (estimates.testActualCostModified())
        {
            GenericValidator.validateRange(estimates,
                ValidationConstants.ACTUAL_COST_FIELD, estimates
                    .getActualCost(), ValidationConstants.ACTUAL_COST_MIN,
                ValidationConstants.ACTUAL_COST_MAX, msgCtx);
        }

        if (estimates.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(estimates,
                ValidationConstants.DESCRIPTION_FIELD, estimates
                    .getDescription(), ValidationConstants.DESCRIPTION_MAX,
                msgCtx);
        }

        if (estimates.testEstimatedCostModified())
        {
            GenericValidator.validateRange(estimates,
                ValidationConstants.ESTIMATED_COST_FIELD, estimates
                    .getEstimatedCost(),
                ValidationConstants.ESTIMATED_COST_MIN,
                ValidationConstants.ESTIMATED_COST_MAX, msgCtx);
        }

        if (estimates.testPerformedByModified())
        {
            GenericValidator.validateMaxLength(estimates,
                ValidationConstants.PERFORMED_BY_FIELD, estimates
                    .getPerformedBy(), ValidationConstants.PERFORMED_BY_MAX,
                msgCtx);
        }
    }
}
