package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.ChargeType;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.scope.CrossChargeFinancialsScope;
import com.ibm.rpm.clientcostcenters.containers.AbstractCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.wbs.managers.WbsManager;


public class AssetCrossChargeManager extends AbstractRPMObjectManager{
    
    public static final String DEFAULT_CHARGE_TYPE = "6601";

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        //Add the complete class name of all the containers that could be loaded by this managers.
        CONTAINERS.add(CrossChargeFinancials.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                AssetCrossChargeManager.class);
        }
    }
    /**
     * Column CROSS_CHARGE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CROSS_CHARGE_ID = 1;
    public static final int TYPE_CROSS_CHARGE_ID = Types.CHAR;
    public static final String NAME_CROSS_CHARGE_ID = "ASSET_CROSS_CHARGE.CROSS_CHARGE_ID";
    public static final String PROPERTY_CROSS_CHARGE_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_CROSS_CHARGE_ID,NAME_CROSS_CHARGE_ID);}

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 2;
    public static final int TYPE_ASSET_ID = Types.CHAR;
    public static final String NAME_ASSET_ID = "ASSET_CROSS_CHARGE.ASSET_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID);}

    /**
     * Column PROJECT_CLIENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_CLIENT_ID = 3;
    public static final int TYPE_PROJECT_CLIENT_ID = Types.CHAR;
    public static final String NAME_PROJECT_CLIENT_ID = "ASSET_CROSS_CHARGE.PROJECT_CLIENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PROJECT_CLIENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_CLIENT_ID,NAME_PROJECT_CLIENT_ID);}

    /**
     * Column QUANTITY of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_QUANTITY = 4;
    public static final int TYPE_QUANTITY = Types.DECIMAL;
    public static final String NAME_QUANTITY = "ASSET_CROSS_CHARGE.QUANTITY";
    public static final String PROPERTY_QUANTITY = "QUANTITY";
    static{ FIELDPROPERTYMAP.put(PROPERTY_QUANTITY,NAME_QUANTITY);}

    /**
     * Column ASSET_CHARGE_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ASSET_CHARGE_TYPE = 5;
    public static final int TYPE_ASSET_CHARGE_TYPE = Types.INTEGER;
    public static final String NAME_ASSET_CHARGE_TYPE = "ASSET_CROSS_CHARGE.ASSET_CHARGE_TYPE";
    public static final String PROPERTY_ASSET_CHARGE_TYPE = "ChargeType";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_CHARGE_TYPE,NAME_ASSET_CHARGE_TYPE);}

    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 6;
    public static final int TYPE_START_DATE = Types.DATE;
    public static final String NAME_START_DATE = "ASSET_CROSS_CHARGE.START_DATE";
    public static final String PROPERTY_START_DATE = "startDate";
    static{ FIELDPROPERTYMAP.put(PROPERTY_START_DATE,NAME_START_DATE);}

    /**
     * Column FINISH_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FINISH_DATE = 7;
    public static final int TYPE_FINISH_DATE = Types.DATE;
    public static final String NAME_FINISH_DATE = "ASSET_CROSS_CHARGE.FINISH_DATE";
    public static final String PROPERTY_FINISH_DATE = "finishDate";
    static{ FIELDPROPERTYMAP.put(PROPERTY_FINISH_DATE,NAME_FINISH_DATE);}

    /**
     * Column UNIT_PRICE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_UNIT_PRICE = 8;
    public static final int TYPE_UNIT_PRICE = Types.DECIMAL;
    public static final String NAME_UNIT_PRICE = "ASSET_CROSS_CHARGE.UNIT_PRICE";
    public static final String PROPERTY_UNIT_PRICE = "Cost";
    static{ FIELDPROPERTYMAP.put(PROPERTY_UNIT_PRICE,NAME_UNIT_PRICE);}

    /**
     * Column ESTIMATED_BUDGET of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BUDGET = 9;
    public static final int TYPE_ESTIMATED_BUDGET = Types.DECIMAL;
    public static final String NAME_ESTIMATED_BUDGET = "ASSET_CROSS_CHARGE.ESTIMATED_BUDGET";
    public static final String PROPERTY_ESTIMATED_BUDGET = "estimatedBudget";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BUDGET,NAME_ESTIMATED_BUDGET);}

    /**
     * Column ESTIMATED_ACTUAL of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL = 10;
    public static final int TYPE_ESTIMATED_ACTUAL = Types.DECIMAL;
    public static final String NAME_ESTIMATED_ACTUAL = "ASSET_CROSS_CHARGE.ESTIMATED_ACTUAL";
    public static final String PROPERTY_ESTIMATED_ACTUAL = "estimatedActual";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL,NAME_ESTIMATED_ACTUAL);}

    /**
     * Column ESTIMATED_PAID of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_PAID = 11;
    public static final int TYPE_ESTIMATED_PAID = Types.DECIMAL;
    public static final String NAME_ESTIMATED_PAID = "ASSET_CROSS_CHARGE.ESTIMATED_PAID";
    public static final String PROPERTY_ESTIMATED_PAID = "estimatedActual";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_PAID,NAME_ESTIMATED_PAID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 12;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "ASSET_CROSS_CHARGE.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 13;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "ASSET_CROSS_CHARGE.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 14;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "ASSET_CROSS_CHARGE.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    private static final String TABLE_NAME = "ASSET_CROSS_CHARGE";

    /**
     * Create an array of type string containing all the fields of the ASSET_CROSS_CHARGE table.
     */
    private static final String[] FIELD_NAMES =
    {
         NAME_CROSS_CHARGE_ID
        ,NAME_ASSET_ID
        ,NAME_PROJECT_CLIENT_ID
        ,NAME_QUANTITY
        ,NAME_ASSET_CHARGE_TYPE
        ,NAME_START_DATE
        ,NAME_FINISH_DATE
        ,NAME_UNIT_PRICE
        ,NAME_ESTIMATED_BUDGET
        ,NAME_ESTIMATED_ACTUAL
        ,NAME_ESTIMATED_PAID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    }

    /**
     * Creates a new AssetCrossChargeManager instance.
     * @return AssetCrossChargeManager
     */
    public AssetCrossChargeManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }

    public String getPrimaryKey() {
        return NAME_CROSS_CHARGE_ID;
    }

    protected Integer getContainerType(ResultSet resultSet) throws RPMException, SQLException {

    	//If container type doesn't apply to this manager
    	return new Integer(0);
    }

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }

    protected HashSet getValidContainerNames() {
        return CONTAINERS;
    }

    protected RPMObject createContainer(int typeId) throws RPMException {

        RPMObject object = new CrossChargeFinancials();
        return object;
     }
    /**
     * Create a record for Cross Charge Financials in DB and set its ID.
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException {

    	CrossChargeFinancials pObject = (CrossChargeFinancials)rpmObject;
                
        // Set default values
    	if (pObject.getCost() == null)
             pObject.setCost(new Double(0.00));
    	
    	if (pObject.getEstimatedBudget() == null)
             pObject.setEstimatedBudget(new Double(0.00));
    	
    	if (pObject.getEstimatedActual() == null)
             pObject.setEstimatedActual(new Double(0.00));
    	
    	if (pObject.getEstimatedPaid() == null)
             pObject.setEstimatedPaid(new Double(0.00));
    	
    	if (pObject.getStartDate() == null)
             pObject.setStartDate(DateUtil.getCalendarInstance());
    	
    	if (pObject.getFinishDate() == null)
             pObject.setFinishDate(DateUtil.getCalendarInstance());
    	
    	if (pObject.getQuantity() == null)
             pObject.setQuantity(new Double(0.00));
    	
    	if (pObject.getChargeType() == null)
    	{
            ChargeType chargeType = new ChargeType();
            chargeType.setID(DEFAULT_CHARGE_TYPE);
            chargeType.setValue(DEFAULT_CHARGE_TYPE);
            pObject.setChargeType(chargeType);
    	}

        // Insert the record
        pObject.setID(SP_A_ASSET_PROJECT(pObject, messageContext));
    } // createObject

    /**
     * decode a row of Cross Charge Financials.
     */


    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException {

    	CrossChargeFinancials pObject = (CrossChargeFinancials) rpmObject;

        pObject.setID(rs.getString(ID_CROSS_CHARGE_ID));
        if (isDelta)
        {
        	pObject.deltaQuantity(new Double(rs.getDouble(ID_QUANTITY)));
            pObject.deltaStartDate(Manager.getCalendar(rs, ID_START_DATE));
            pObject.deltaFinishDate(Manager.getCalendar(rs, ID_FINISH_DATE));
        	pObject.deltaCost(new Double(rs.getDouble(ID_UNIT_PRICE)));
        	pObject.deltaEstimatedBudget(new Double(rs.getDouble(ID_ESTIMATED_BUDGET)));
        	pObject.deltaEstimatedActual(new Double(rs.getDouble(ID_ESTIMATED_ACTUAL)));
        	pObject.deltaEstimatedPaid(new Double(rs.getDouble(ID_ESTIMATED_PAID)));
        }
        else
        {
        	pObject.setQuantity(new Double(rs.getDouble(ID_QUANTITY)));
            pObject.setStartDate(Manager.getCalendar(rs, ID_START_DATE));
            pObject.setFinishDate(Manager.getCalendar(rs, ID_FINISH_DATE));
        	pObject.setCost(new Double(rs.getDouble(ID_UNIT_PRICE)));
        	pObject.setEstimatedBudget(new Double(rs.getDouble(ID_ESTIMATED_BUDGET)));
        	pObject.setEstimatedActual(new Double(rs.getDouble(ID_ESTIMATED_ACTUAL)));
        	pObject.setEstimatedPaid(new Double(rs.getDouble(ID_ESTIMATED_PAID)));
        }

        return pObject;
    } // decodeRow

    /**
     * delete a Cross Charge Financials record
     */


    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException {

    	if (rpmObject != null)
        {
    		SP_UA_ASSET_PROJECT((CrossChargeFinancials) rpmObject,
                                messageContext);
        }

    }// delete

    /**
     * Returns the foreign key sql conditions of the asset that exists in the
     * Cross Charge Financials table
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {

        if (!(manager instanceof AssetManager))
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);

        JoinCondition joinCondition = new JoinCondition();
        joinCondition.setTableName(getTableName());

        //where clause of the join condition
        StringBuffer condition = new StringBuffer();
        condition.append(NAME_ASSET_ID);
        condition.append(" = ?");
        if (extraWhere != null && extraWhere.length() > 0)
        {
            condition.append(" AND ");
            condition.append(extraWhere);
        }
        joinCondition.setCondition(condition.toString());

        return joinCondition;
    } // getForeignKeyCondition

    /**
     * Return JoinCondition object with condition input parameter
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName) throws RPMException
            {

                JoinCondition joinCondition = new JoinCondition();
                if (manager instanceof AssetManager)
                {
                    // for XPath queries like CrossChargeFinancials[<condition>]/Asset
                    StringBuffer where = new StringBuffer();
                    where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_ASSET_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {
                        where.append(" WHERE ").append(condition);
                    }
                    joinCondition.setCondition(where.toString());
                }
                else if (manager == null)

                {
                    joinCondition.setTableName(getTableName());

                    if (condition != null)
                    {
                        joinCondition.setCondition(condition);
                    }

                }

                else if (manager instanceof ClientManager)
                {
                    // for XPath queries like /CrossChargeFinancials[<condition>]/ClientCostCenter
                    StringBuffer where = new StringBuffer();
                    where.append(ClientManager.NAME_ELEMENT_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_PROJECT_CLIENT_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {
                        where.append(" WHERE ").append(condition);
                    }
                    joinCondition.setCondition(where.toString());
                }

                else if (manager instanceof WbsManager)
                {
                    // for XPath queries like /CrossChargeFinancials[<condition>]/Project
                    StringBuffer where = new StringBuffer();
                    where.append(WbsManager.NAME_ELEMENT_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_PROJECT_CLIENT_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {
                        where.append(" WHERE ").append(condition);
                    }
                    joinCondition.setCondition(where.toString());
                }

                else
        	    {
                    ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                            ExceptionUtil.JOIN);
                }

                return joinCondition;
            } // getJoinCondition

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
			{

	    		JoinCondition joinCondition = new JoinCondition();
	    		if (manager instanceof AssetManager)
	    		{
	    			// for XPath queries like /Asset[CrossChargeFinancials[<condition>]]
	    			StringBuffer where = new StringBuffer();
	    			where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
	    			where.append(" SELECT ").append(NAME_ASSET_ID);
	    			where.append(" FROM ").append(getTableName());
	    			if (condition != null)
	    			{
	    				where.append(" WHERE ").append(condition);
	    			}
	    			joinCondition.setCondition(where.toString());
	    		}
	    		else if (manager instanceof ClientManager)
	    		{
	    			// for XPath queries like /ClientCostCenter[CrossChargeFinancials[<condition>]]
	    			StringBuffer where = new StringBuffer();
	    			where.append(ClientManager.NAME_ELEMENT_ID).append(" IN (");
	    			where.append(" SELECT ").append(NAME_PROJECT_CLIENT_ID);
	    			where.append(" FROM ").append(getTableName());
	    			if (condition != null)
	    			{
	    				where.append(" WHERE ").append(condition);
	    			}
	    			joinCondition.setCondition(where.toString());
	    		}
	    		else if (manager instanceof WbsManager)
	    		{
	    			// for XPath queries like /Project[CrossChargeFinancials[<condition>]]
	    			StringBuffer where = new StringBuffer();
	    			where.append(WbsManager.NAME_ELEMENT_ID).append(" IN (");
	    			where.append(" SELECT ").append(NAME_PROJECT_CLIENT_ID);
	    			where.append(" FROM ").append(getTableName());
	    			if (condition != null)
	    			{
	    				where.append(" WHERE ").append(condition);
	    			}
	    			joinCondition.setCondition(where.toString());
	    		}
	    		else
	    		{
	    			ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                        ExceptionUtil.SUB_TABLE_JOIN);
	    		}

        return joinCondition;
    }// getSubTableJoinCondition


	/**
	 * The decoded related objects is the parent asset and Project/Client
	 *
	 */
	protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
        FieldValueMap relatedFields, int index) throws RPMException,
        SQLException
	{
		String parentId = resultSet.getString(ID_ASSET_ID);
		relatedFields.put(index, ID_ASSET_ID, parentId);
		String projectOrClientId = resultSet.getString(ID_PROJECT_CLIENT_ID);
		relatedFields.put(index, ID_PROJECT_CLIENT_ID, projectOrClientId);
        String chargeTypeId = new String(resultSet.getString(ID_ASSET_CHARGE_TYPE));
        relatedFields.put(index, ID_ASSET_CHARGE_TYPE, chargeTypeId);
		return relatedFields;
	} // decodeRelatedFields

	/**
	 * There are three related objects: parent asset, Charged Project/Client and chargeType
	 * which is Datafield
	 */
	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
        RPMObjectScope rpmScope, MessageContext messageContext,
        FieldValueMap relatedFields, int index, boolean isDelta)
        throws RPMException, SQLException, ParseException
		{
			CrossChargeFinancials object = (CrossChargeFinancials) rpmObject;

			//ID of Related Charge Type
			String chargeTypeId = (String) relatedFields.get(index, ID_ASSET_CHARGE_TYPE);
			
			ChargeType chargeType = null;
			
			if(chargeTypeId != null && !chargeTypeId.equals("0")){

				// Load the Datafield Manager
	            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
	            .getInstance().getRPMObjectManager(
	            		ChargeType.class.getName());
	
	            chargeType = new ChargeType();
				// Set the ChargeType ID
	            chargeType.setID(chargeTypeId);
				// Load the ChargeType record
	            chargeType = (ChargeType) manager.loadByPrimaryKey(chargeType, null,
	            		messageContext, isDelta);
            
			}
			
            if (isDelta)
            {
            	object.deltaChargeType(chargeType);
            }
            else
            {
            	object.setChargeType(chargeType);
            }
		


			if (rpmScope == null)
				return rpmObject;

			//ID of Related asset
			String assetId = (String) relatedFields.get(index, ID_ASSET_ID);

			if (assetId != null && rpmScope.getParent()!=null)
			{
				// Load the Asset Manager
				AssetManager assetManager = (AssetManager) RPMManagerFactory
                	.getInstance().getRPMObjectManager(Asset.class.getName());
				Asset asset = new Asset();
				// Set the Asset ID
				asset.setID(assetId);

				// Load the Asset record
				asset = (Asset) assetManager.loadByPrimaryKey(asset, rpmScope
                .getParent(), messageContext, isDelta);
				if (isDelta)
				{
					object.deltaParent(asset);
				}
				else
				{
					object.setParent(asset);
				}
			}// if (assetId != null)

			
			if(rpmScope instanceof CrossChargeFinancialsScope){
			    
				CrossChargeFinancialsScope sScope = (CrossChargeFinancialsScope) rpmScope;
				
				//ID of Charged Object
				String projectOrClientId = (String) relatedFields.get(index, ID_PROJECT_CLIENT_ID);
	
				if (projectOrClientId != null && sScope.getCostCenter() != null)
				{
					// Try to load the record with the ClientsManager Manager
		            ClientManager clientsManager = (ClientManager) RPMManagerFactory
		                .getInstance().getRPMObjectManager(ClientCostCenter.class.getName());
		            GenericClient costCenter  = new GenericClient();
					// Set the abstractCostCenter ID
		            costCenter.setID(projectOrClientId);
					// Load the abstractCostCenter record
		            costCenter = (AbstractCostCenter) clientsManager.loadByPrimaryKey(costCenter, sScope
		                .getCostCenter(), messageContext, isDelta);
		            if (costCenter != null)
		            {
		            	if (isDelta)
						{
		            		object.deltaCostCenter((AbstractCostCenter)costCenter);
						}
						else
						{
							object.setCostCenter((AbstractCostCenter)costCenter);
						}
	
		            }
				}
	
	
	
				if ( (assetId == null) && (projectOrClientId == null) )
				{
		            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.ASSET_ID);
		            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.PROJECT_OR_CLIENT_ID);
				} // if ( (assetId == null) && (projectOrClientId == null) )

			}
			return rpmObject;
		} // loadRelatedObjects

    /**
     * saveObjectParents - Dummy implementation
     */

    public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException {
        //    	 nothing
    }

    /**
     * Update the CrossChargeFinancials fields in DB
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
    	CrossChargeFinancials crossChargeFinancials = (CrossChargeFinancials) rpmObject;
        if (crossChargeFinancials.testQuantityModified()
                || crossChargeFinancials.testChargeTypeModified()
                || crossChargeFinancials.testStartDateModified()
                || crossChargeFinancials.testFinishDateModified()
                || crossChargeFinancials.testCostModified())
        {
        	SP_U_ASSET_CROSS_CHARGE(crossChargeFinancials, messageContext);

            if (crossChargeFinancials.testEstimatedBudgetModified())
            	SP_U_ASSET_TIMEPHASE(crossChargeFinancials, new Integer(3)
            			, crossChargeFinancials.getEstimatedBudget(), messageContext);
            if (crossChargeFinancials.testEstimatedActualModified())
            	SP_U_ASSET_TIMEPHASE(crossChargeFinancials, new Integer(4)
            			, crossChargeFinancials.getEstimatedActual(), messageContext);
            if (crossChargeFinancials.testEstimatedPaidModified())
            	SP_U_ASSET_TIMEPHASE(crossChargeFinancials, new Integer(12)
            			, crossChargeFinancials.getEstimatedPaid(), messageContext);
        }
        return crossChargeFinancials;
    }

    /**
     * Inserts a Cross Charge Financials record
     *
     * @param crossChargeFinancials -
     *            crossChargeFinancials record to save
     * @param messageContext -
     *            context
     * @return a String that represents the ID of the new created CrossChargeFinancials
     * @throws SQLException
     * @throws RPMException
     */

    private String SP_A_ASSET_PROJECT(CrossChargeFinancials crossChargeFinancials, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_ASSET_PROJECT";
        Object args[] = new Object[4];

        args[1] = crossChargeFinancials.getParent().getID();

        args[2] = crossChargeFinancials.getCostCenter().getID();
        if (crossChargeFinancials.getCostCenter().retrieveTypeID().intValue() == ClientCostCenter.TYPE_ID) {
        	// client
            args[0]= new Integer(11);
        }
        else {
        	// Proposal
            args[0]= new Integer(1);
        }

        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    } // SP_A_ASSET_PROJECT

    /**
     * Delete a a Cross Charge Financials record
     *
     * @param assignment -
     *            CrossChargeFinancials record to delete
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_UA_ASSET_PROJECT(CrossChargeFinancials crossChargeFinancials, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_ASSET_PROJECT";
        Object args[] = new Object[4];

        args[1] = crossChargeFinancials.getParent().getID();
        args[2] = crossChargeFinancials.getCostCenter().getID();
        if (crossChargeFinancials.getCostCenter().retrieveTypeID().intValue() == ClientCostCenter.TYPE_ID) {
         	// client
           	args[0]= new Integer(11);
        }
        else {
        	// Proposal
            args[0]= new Integer(1);
        }

        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    } // SP_UA_ASSET_PROJECT

    /**
     * Update a CrossChargeFinancials record in DB
     *
     * @param CrossChargeFinancials -
     *            CrossChargeFinancials information
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_U_ASSET_CROSS_CHARGE(CrossChargeFinancials crossChargeFinancials,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ASSET_CROSS_CHARGE";
        java.sql.Date dt;
        Object args[] = new Object[9];

        // Project or Client ID
        args[2] = crossChargeFinancials.getCostCenter().getID();
        // Table type ID
        args[0]= crossChargeFinancials.getCostCenter().retrieveTypeID();

        // Asset ID
        args[1] = crossChargeFinancials.getParent().getID();


        // Start date
        if (crossChargeFinancials.getStartDate() != null)
        {
            dt = new java.sql.Date(crossChargeFinancials.getStartDate()
                    .getTimeInMillis());
            args[3] = dt.toString();
        }
        // Finish date
        if (crossChargeFinancials.getFinishDate() != null)
        {
            dt = new java.sql.Date(crossChargeFinancials.getFinishDate()
                    .getTimeInMillis());
            args[4] = dt.toString();
        }
        // Quantity
        args[5] = crossChargeFinancials.getQuantity();
        // Charge type
        args[6] = new Integer (crossChargeFinancials.getChargeType().getID());
        // Cost is Unit Price
        args[7] = crossChargeFinancials.getCost();
        args[8] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }  // SP_U_ASSET_CROSS_CHARGE

    /**
     * Update ASSET TIMEPHASE information  in DB
     *
     * @param CrossChargeFinancials -
     *            CrossChargeFinancials information
     * @param budgetType  -
     *            Budget type category
     * @param budgetAmount  -
     *            Budget Amount
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_U_ASSET_TIMEPHASE(CrossChargeFinancials crossChargeFinancials, Integer BudgetType,
    		Double budgetAmount, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ASSET_TIMEPHASE";
        Object args[] = new Object[18];

        // Project or Client ID
        args[3] = crossChargeFinancials.getCostCenter().getID();

        // Table type ID
        args[0]= crossChargeFinancials.getCostCenter().retrieveTypeID();

        // Asset ID
        args[1] = crossChargeFinancials.getParent().getID();


        // Year
        Calendar startCalendar = crossChargeFinancials.getStartDate();
        Calendar endCalendar = crossChargeFinancials.getFinishDate();
        args[2] = new Integer(startCalendar.get(Calendar.YEAR));
        // Budget type
        args[4] = BudgetType;
        // Month amount
        int startMonth = startCalendar.get(Calendar.MONTH);
        int endMonth = endCalendar.get(Calendar.MONTH);
        startMonth--;
        endMonth--;
        Double averageBudget = new Double (budgetAmount.doubleValue()/ (endMonth - startMonth + 1) );
        // Budget equaly devided between months
        for (int i=0; i < 12; i++)
        {
        	if ( (i < startMonth) || (i > endMonth) )
        		args[5 + i] = new Double (0);
        	else
        		args[5 + i] = averageBudget;

        }

        args[17] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }  // SP_U_ASSET_TIMEPHASE
}

