package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;

/**
 * @author ragoler
 *
 * Manages the Depreciation Schedules for assets. Manages table
 * ASSET_DEPRECIATION. The depreciation loading should always be done with the
 * parent asset because a delete requires the asset Id
 */
public class AssetDepreciationManager extends AbstractRPMObjectManager
{

    private static Log logger = LogFactory
            .getLog(AssetDepreciationManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        // The complete class name of all the containers that could be
        // loaded by this managers.

        CONTAINERS.add(DepreciationSchedule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    AssetDepreciationManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ASSET_DEPRECIATION.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 2;

    public static final int TYPE_ASSET_ID = Types.CHAR;

    public static final String NAME_ASSET_ID = "ASSET_DEPRECIATION.ASSET_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ASSET_ID = "ASSETID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID, NAME_ASSET_ID);
    }

    /**
     * Column DEPRECIATION_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DEPRECIATION_DATE = 3;

    public static final int TYPE_DEPRECIATION_DATE = Types.DATE;

    public static final String NAME_DEPRECIATION_DATE = "ASSET_DEPRECIATION.DEPRECIATION_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIATION_DATE = "DEPRECIATIONDATE";
    static
    {
        FIELDPROPERTYMAP
                .put(PROPERTY_DEPRECIATION_DATE, NAME_DEPRECIATION_DATE);
    }

    /**
     * Column DEPRECIATION_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_DEPRECIATION_AMOUNT = 4;

    public static final int TYPE_DEPRECIATION_AMOUNT = Types.DECIMAL;

    public static final String NAME_DEPRECIATION_AMOUNT = "ASSET_DEPRECIATION.DEPRECIATION_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIATION_AMOUNT = "AMOUNT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPRECIATION_AMOUNT,
                NAME_DEPRECIATION_AMOUNT);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 5;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "ASSET_DEPRECIATION.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 6;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "ASSET_DEPRECIATION.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 7;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "ASSET_DEPRECIATION.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    private static final String TABLE_NAME = "ASSET_DEPRECIATION";

    /**
     * Create an array of type string containing all the fields of the
     * ASSET_DEPRECIATION table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ASSET_ID, NAME_DEPRECIATION_DATE,
            NAME_DEPRECIATION_AMOUNT, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_REC_USER
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new AssetDepreciationManager instance.
     *
     * @return AssetDepreciationManager
     */
    public AssetDepreciationManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws SQLException
    {

        //If container type doesn't apply to this manager
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        return new DepreciationSchedule();
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        DepreciationSchedule pObject = (DepreciationSchedule) rpmObject;

        // Set defaults
        if (pObject.getDepreciationDate() == null)
            pObject.setDepreciationDate(DateUtil.getCalendarInstance());
       
        if (pObject.getAmount() == null)
            pObject.setAmount(new Double(0));
        
        pObject.setID(SP_I_DEPRECIATION(pObject, messageContext));
    }

    /**
     * decode a row of depreciation schedule. this includes: 1. date 2. ammount
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DepreciationSchedule depreciationObj = (DepreciationSchedule) rpmObject;

        depreciationObj.setID(rs.getString(ID_ELEMENT_ID));
        if (isDelta)
        {
            depreciationObj.deltaDepreciationDate(DateUtil.convertToCalendar(rs
                    .getDate(ID_DEPRECIATION_DATE)));
            depreciationObj.deltaAmount(new Double(rs
                    .getDouble(ID_DEPRECIATION_AMOUNT)));
        }
        else
        {
            depreciationObj.setDepreciationDate(DateUtil.convertToCalendar(rs
                    .getDate(ID_DEPRECIATION_DATE)));
            depreciationObj.setAmount(new Double(rs
                    .getDouble(ID_DEPRECIATION_AMOUNT)));
        }

        return depreciationObj;
    }

    /**
     * delete a dpreciation schedule
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        if (rpmObject != null)
        {
            SP_D_DEPRECIATION((DepreciationSchedule) rpmObject, messageContext);
        }
    }

    /**
     * This is the foregn key of the asset that exists in the depreciation table
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if ((manager instanceof AssetManager))
        {

	        joinCondition = new JoinCondition();
	        joinCondition.setTableName(getTableName());
	
	        //where clause of the join condition
	        StringBuffer condition = new StringBuffer();
	        condition.append(NAME_ASSET_ID);
	        condition.append(" = ?");
	
	
	        if (extraWhere != null && extraWhere.length() > 0)
	        {
	            condition.append(" AND ");
	            condition.append(extraWhere);
	
	        }
	        joinCondition.setTableName(getTableName());
	        joinCondition.setCondition(condition.toString());

    	}
		else
		{
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
		}
        
        return joinCondition;
    }

    /**
     *
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = new JoinCondition();


        if(manager==null)
        {
	        joinCondition.setTableName(getTableName());
	        
	        if (condition != null)
	        {
	            joinCondition.setCondition(condition);
	        }
        } else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
		JoinCondition joinCondition = new JoinCondition();
        if(manager instanceof AssetManager)
        {
            // for XPath queries like /Asset[DepreciationSchedule[<condition>]]
            StringBuffer where = new StringBuffer();
            where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ASSET_ID);
            where.append(" FROM ").append(getTableName());
            if(condition!=null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }


        return joinCondition;
    }
    
    /**
     * The only related object is the parent asset
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_ASSET_ID);
        relatedFields.put(index, ID_ASSET_ID, parentId);

        return relatedFields;
    }

    /**
     * The only related object is the parent asset
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if(rpmScope ==null)
            return rpmObject;


        if(rpmScope.getParent()==null)
        	return rpmObject;

        DepreciationSchedule object = (DepreciationSchedule) rpmObject;

        //Related asset
        String assetId = (String) relatedFields.get(index, ID_ASSET_ID);
        if (assetId != null)
        {
            AssetManager assetManager = (AssetManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(Asset.class.getName());
            Asset asset = new Asset();
            asset.setID(assetId);

            asset = (Asset) assetManager.loadByPrimaryKey(asset, rpmScope.getParent(),
                    messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(asset);
            }
            else
            {
                object.setParent(asset);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.ASSET_ID);
        }

        return rpmObject;

    }

    /**
     * update the depreciation schedule fields
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        DepreciationSchedule depreciation = (DepreciationSchedule) rpmObject;

        if (depreciation.testAmountModified()
                || depreciation.testDepreciationDateModified()
                || depreciation.testParentModified())
            SP_U_DEPRECIATION((DepreciationSchedule) rpmObject, messageContext);

        return depreciation;
    }

    /**
     * Inserts an empty depreciation schedule for the asset
     *
     * @param depreciation -
     *            depreciation schedule to save
     * @param messageContext -
     *            context
     * @return a String that represents the ID of the new created depreciation
     *         schedule
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_I_DEPRECIATION(DepreciationSchedule depreciation,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String procedureName = "SP_I_ASSET_DEPRECIATION";
        Object args[] = new Object[4];
        args[0] = null;
        args[1] = depreciation.getParent().getID();
        args[2] = new Integer(93);
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    /**
     * Update a depreciation schedule
     *
     * @param depreciation -
     *            depreciation information
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_U_DEPRECIATION(DepreciationSchedule depreciation,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ASSET_DEPRECIATION";
        Object args[] = new Object[4];
        args[0] = depreciation.getID();
        args[1] = new java.sql.Date(depreciation.getDepreciationDate()
                .getTimeInMillis());
        args[2] = depreciation.getAmount();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    /**
     * Deletes a depreciation schedule
     *
     * @param depreciation -
     *            depreciation information
     * @param messageContext -
     *            context
     * @throws RPMException
     * @throws SQLException
     */
    private void SP_D_DEPRECIATION(DepreciationSchedule depreciation,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_ASSET_DEPRECIATION";
        Object args[] = new Object[4];
        args[0] = depreciation.getID();
        args[1] = depreciation.getParent().getID();

        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
}