
package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AssetCategory;
import com.ibm.rpm.applicationadministration.containers.AssetRevisionImpact;
import com.ibm.rpm.applicationadministration.containers.AssetRevisionType;
import com.ibm.rpm.applicationadministration.containers.AssetState;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.GeographicalAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.managers.DocumentUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DatafieldUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;

public class AssetManager extends AbstractRPMObjectManager
{
    public static final String DEFAULT_ASSET_CATEGORY = "5152";
    
    protected static final HashMap FIELDPROPERTYMAP = new HashMap();
    
    // =============
    // CONTAINER MAP
    // =============

    private static ContainerMap containerMap = new ContainerMap(
        AssetManager.class);
    static
    {
        containerMap.add(GenericAsset.class);
        containerMap.add(Asset.class);
        containerMap.add(AssetFolder.class);
    }
    
    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_ASSETS.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //    public static final String PROPERTY_RANK = "RANK";
    //    static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 2;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_ASSETS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //    public static final String PROPERTY_LEVEL_ID = "LEVEL_ID";
    //    static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 3;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_ASSETS.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    //    static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 4;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_ASSETS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 5;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_ASSETS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //    public static final String PROPERTY_CHILD_COUNT = "CHILD_COUNT";
    //    static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 6;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_ASSETS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_ASSETS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_ASSETS.ELEMENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_ASSETS.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PARENT_ID = "PARENT_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 10;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_ASSETS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 11;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_ASSETS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 12;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_ASSETS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column DOC_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DOC_COUNT = 13;

    public static final int TYPE_DOC_COUNT = Types.INTEGER;

    public static final String NAME_DOC_COUNT = "TMT_ASSETS.DOC_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DOC_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DOC_COUNT,NAME_DOC_COUNT);}

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAJOR_REVISION = 14;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "TMT_ASSETS.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINOR_REVISION = 15;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "TMT_ASSETS.MINOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION);}

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_IMPACT = 16;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "TMT_ASSETS.REVISION_IMPACT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_TYPE = 17;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "TMT_ASSETS.REVISION_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION_HISTORY = 18;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "TMT_ASSETS.REVISION_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

    /**
     * Column CATEGORY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CATEGORY_ID = 19;

    public static final int TYPE_CATEGORY_ID = Types.INTEGER;

    public static final String NAME_CATEGORY_ID = "TMT_ASSETS.CATEGORY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CATEGORY_ID = "CATEGORY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CATEGORY_ID, NAME_CATEGORY_ID);
    }

    /**
     * Column STATUS_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STATUS_ID = 20;

    public static final int TYPE_STATUS_ID = Types.INTEGER;

    public static final String NAME_STATUS_ID = "TMT_ASSETS.STATUS_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_STATUS_ID = "STATUS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_STATUS_ID, NAME_STATUS_ID);
    }

    /**
     * Column MAKE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MAKE = 21;

    public static final int TYPE_MAKE = Types.VARCHAR;

    public static final String NAME_MAKE = "TMT_ASSETS.MAKE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAKE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAKE,NAME_MAKE);}

    /**
     * Column MODEL of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MODEL = 22;

    public static final int TYPE_MODEL = Types.VARCHAR;

    public static final String NAME_MODEL = "TMT_ASSETS.MODEL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MODEL = "MODEL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MODEL, NAME_MODEL);
    }

    /**
     * Column MODEL_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MODEL_NUMBER = 23;

    public static final int TYPE_MODEL_NUMBER = Types.VARCHAR;

    public static final String NAME_MODEL_NUMBER = "TMT_ASSETS.MODEL_NUMBER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MODEL_NUMBER = "MODEL_NUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MODEL_NUMBER, NAME_MODEL_NUMBER);
    }

    /**
     * Column SERIAL_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SERIAL_NUMBER = 24;

    public static final int TYPE_SERIAL_NUMBER = Types.VARCHAR;

    public static final String NAME_SERIAL_NUMBER = "TMT_ASSETS.SERIAL_NUMBER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SERIAL_NUMBER = "SERIALNUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SERIAL_NUMBER, NAME_SERIAL_NUMBER);
    }

    /**
     * Column BARCODE_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_BARCODE_NUMBER = 25;

    public static final int TYPE_BARCODE_NUMBER = Types.VARCHAR;

    public static final String NAME_BARCODE_NUMBER = "TMT_ASSETS.BARCODE_NUMBER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_BARCODE_NUMBER = "barcodeNumber";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BARCODE_NUMBER, NAME_BARCODE_NUMBER);
    }

    /**
     * Column DATE_ACQUIRED of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DATE_ACQUIRED = 26;

    public static final int TYPE_DATE_ACQUIRED = Types.DATE;

    public static final String NAME_DATE_ACQUIRED = "TMT_ASSETS.DATE_ACQUIRED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DATE_ACQUIRED = "dateAcquired";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DATE_ACQUIRED, NAME_DATE_ACQUIRED);
    }

    /**
     * Column DATE_SOLD of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DATE_SOLD = 27;

    public static final int TYPE_DATE_SOLD = Types.DATE;

    public static final String NAME_DATE_SOLD = "TMT_ASSETS.DATE_SOLD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DATE_SOLD = "dispositionDate";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DATE_SOLD, NAME_DATE_SOLD);
    }

    /**
     * Column PURCHASE_PRICE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PURCHASE_PRICE = 28;

    public static final int TYPE_PURCHASE_PRICE = Types.DECIMAL;

    public static final String NAME_PURCHASE_PRICE = "TMT_ASSETS.PURCHASE_PRICE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PURCHASE_PRICE = "initialCost";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PURCHASE_PRICE, NAME_PURCHASE_PRICE);
    }

    /**
     * Column CAPITAL_VALUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CAPITAL_VALUE = 29;

    public static final int TYPE_CAPITAL_VALUE = Types.DECIMAL;

    public static final String NAME_CAPITAL_VALUE = "TMT_ASSETS.CAPITAL_VALUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CAPITAL_VALUE = "capitalValue";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_VALUE, NAME_CAPITAL_VALUE);
    }

    /**
     * Column TOTAL_DEPRECIATION of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_TOTAL_DEPRECIATION = 30;

    public static final int TYPE_TOTAL_DEPRECIATION = Types.DECIMAL;

    public static final String NAME_TOTAL_DEPRECIATION = "TMT_ASSETS.TOTAL_DEPRECIATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TOTAL_DEPRECIATION = "depreciationToDate";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TOTAL_DEPRECIATION,
                NAME_TOTAL_DEPRECIATION);
    }

    /**
     * Column TOTAL_MAINTENANCE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_TOTAL_MAINTENANCE = 31;

    public static final int TYPE_TOTAL_MAINTENANCE = Types.DECIMAL;

    public static final String NAME_TOTAL_MAINTENANCE = "TMT_ASSETS.TOTAL_MAINTENANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TOTAL_MAINTENANCE = "maintenanceToDate";
    static
    {
        FIELDPROPERTYMAP
                .put(PROPERTY_TOTAL_MAINTENANCE, NAME_TOTAL_MAINTENANCE);
    }

    /**
     * Column DEPRECIATION_METHOD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_DEPRECIATION_METHOD = 32;

    public static final int TYPE_DEPRECIATION_METHOD = Types.SMALLINT;

    public static final String NAME_DEPRECIATION_METHOD = "TMT_ASSETS.DEPRECIATION_METHOD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIATION_METHOD = "DEPRECIATION_METHOD";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPRECIATION_METHOD,
                NAME_DEPRECIATION_METHOD);
    }

    /**
     * Column QUANTITY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_QUANTITY = 33;

    public static final int TYPE_QUANTITY = Types.INTEGER;

    public static final String NAME_QUANTITY = "TMT_ASSETS.QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_QUANTITY = "quantity";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_QUANTITY, NAME_QUANTITY);
    }

    /**
     * Column QUANTITY_USED of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_QUANTITY_USED = 34;

    public static final int TYPE_QUANTITY_USED = Types.INTEGER;

    public static final String NAME_QUANTITY_USED = "TMT_ASSETS.QUANTITY_USED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_QUANTITY_USED = "QUANTITY_USED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_QUANTITY_USED, NAME_QUANTITY_USED);
    }

    /**
     * Column DEPRECIABLE_LIFE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DEPRECIABLE_LIFE = 35;

    public static final int TYPE_DEPRECIABLE_LIFE = Types.INTEGER;

    public static final String NAME_DEPRECIABLE_LIFE = "TMT_ASSETS.DEPRECIABLE_LIFE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIABLE_LIFE = "DEPRECIABLE_LIFE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPRECIABLE_LIFE, NAME_DEPRECIABLE_LIFE);
    }

    /**
     * Column SALVAGE_VALUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SALVAGE_VALUE = 36;

    public static final int TYPE_SALVAGE_VALUE = Types.DECIMAL;

    public static final String NAME_SALVAGE_VALUE = "TMT_ASSETS.SALVAGE_VALUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SALVAGE_VALUE = "SALVAGE_VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SALVAGE_VALUE, NAME_SALVAGE_VALUE);
    }

    /**
     * Column CURRENT_VALUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CURRENT_VALUE = 37;

    public static final int TYPE_CURRENT_VALUE = Types.DECIMAL;

    public static final String NAME_CURRENT_VALUE = "TMT_ASSETS.CURRENT_VALUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CURRENT_VALUE = "CURRENT_VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CURRENT_VALUE, NAME_CURRENT_VALUE);
    }

    /**
     * Column NEXT_SCHED_MAINT of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_NEXT_SCHED_MAINT = 38;

    public static final int TYPE_NEXT_SCHED_MAINT = Types.DATE;

    public static final String NAME_NEXT_SCHED_MAINT = "TMT_ASSETS.NEXT_SCHED_MAINT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_NEXT_SCHED_MAINT = "NEXT_SCHED_MAINT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_NEXT_SCHED_MAINT, NAME_NEXT_SCHED_MAINT);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 39;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_ASSETS.DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DESCRIPTION = "description";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column RTF_BITS of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_RTF_BITS = 40;

    public static final int TYPE_RTF_BITS = Types.SMALLINT;

    public static final String NAME_RTF_BITS = "TMT_ASSETS.RTF_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RTF_BITS,NAME_RTF_BITS);}

    /**
     * Column LICENCE_EXPIRY_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LICENCE_EXPIRY_DATE = 41;

    public static final int TYPE_LICENCE_EXPIRY_DATE = Types.DATE;

    public static final String NAME_LICENCE_EXPIRY_DATE = "TMT_ASSETS.LICENCE_EXPIRY_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LICENCE_EXPIRY_DATE = "LICENCE_EXPIRY_DATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LICENCE_EXPIRY_DATE,
                NAME_LICENCE_EXPIRY_DATE);
    }

    /**
     * Column WARRANTY_EXPIRY_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_WARRANTY_EXPIRY_DATE = 42;

    public static final int TYPE_WARRANTY_EXPIRY_DATE = Types.DATE;

    public static final String NAME_WARRANTY_EXPIRY_DATE = "TMT_ASSETS.WARRANTY_EXPIRY_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_WARRANTY_EXPIRY_DATE = "WARRANTY_EXPIRY_DATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WARRANTY_EXPIRY_DATE,
                NAME_WARRANTY_EXPIRY_DATE);
    }

    /**
     * Column CONTACTOR of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACTOR = 43;

    public static final int TYPE_CONTACTOR = Types.VARCHAR;

    public static final String NAME_CONTACTOR = "TMT_ASSETS.CONTACTOR";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CONTACTOR = "CONTACTOR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACTOR, NAME_CONTACTOR);
    }

    /**
     * Column CONTACTOR_PHONE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACTOR_PHONE = 44;

    public static final int TYPE_CONTACTOR_PHONE = Types.VARCHAR;

    public static final String NAME_CONTACTOR_PHONE = "TMT_ASSETS.CONTACTOR_PHONE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CONTACTOR_PHONE = "CONTACTOR_PHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACTOR_PHONE, NAME_CONTACTOR_PHONE);
    }

    /**
     * Column CONTACTOR_EMAIL of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACTOR_EMAIL = 45;

    public static final int TYPE_CONTACTOR_EMAIL = Types.VARCHAR;

    public static final String NAME_CONTACTOR_EMAIL = "TMT_ASSETS.CONTACTOR_EMAIL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CONTACTOR_EMAIL = "CONTACTOR_EMAIL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACTOR_EMAIL, NAME_CONTACTOR_EMAIL);
    }

    /**
     * Column DEPRECIATION_START of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DEPRECIATION_START = 46;

    public static final int TYPE_DEPRECIATION_START = Types.DATE;

    public static final String NAME_DEPRECIATION_START = "TMT_ASSETS.DEPRECIATION_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIATION_START = "DEPRECIATION_START";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPRECIATION_START,
                NAME_DEPRECIATION_START);
    }

    /**
     * Column DEPRECIATION_FINISH of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DEPRECIATION_FINISH = 47;

    public static final int TYPE_DEPRECIATION_FINISH = Types.DATE;

    public static final String NAME_DEPRECIATION_FINISH = "TMT_ASSETS.DEPRECIATION_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEPRECIATION_FINISH = "DEPRECIATION_FINISH";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPRECIATION_FINISH,
                NAME_DEPRECIATION_FINISH);
    }

    /**
     * Column SCORECARD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCORECARD_COUNT = 48;

    public static final int TYPE_SCORECARD_COUNT = Types.INTEGER;

    public static final String NAME_SCORECARD_COUNT = "TMT_ASSETS.SCORECARD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_COUNT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_COUNT,NAME_SCORECARD_COUNT);}

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_SCORE = 49;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "TMT_ASSETS.CONVERTED_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE);}

    /**
     * Column EXPENSE_ACTUAL_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPENSE_ACTUAL_COST = 50;

    public static final int TYPE_EXPENSE_ACTUAL_COST = Types.DECIMAL;

    public static final String NAME_EXPENSE_ACTUAL_COST = "TMT_ASSETS.EXPENSE_ACTUAL_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPENSE_ACTUAL_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_ACTUAL_COST,NAME_EXPENSE_ACTUAL_COST);}

    /**
     * Column EXPENSE_SCHED_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPENSE_SCHED_COST = 51;

    public static final int TYPE_EXPENSE_SCHED_COST = Types.DECIMAL;

    public static final String NAME_EXPENSE_SCHED_COST = "TMT_ASSETS.EXPENSE_SCHED_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_EXPENSE_SCHED_COST = "PROPERTYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_SCHED_COST,
                NAME_EXPENSE_SCHED_COST);
    }

    /**
     * Column EXPENSE_COST_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPENSE_COST_ETC = 52;

    public static final int TYPE_EXPENSE_COST_ETC = Types.DECIMAL;

    public static final String NAME_EXPENSE_COST_ETC = "TMT_ASSETS.EXPENSE_COST_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPENSE_COST_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_COST_ETC,NAME_EXPENSE_COST_ETC);}

    /**
     * Column CAPITAL_ACTUAL_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CAPITAL_ACTUAL_COST = 53;

    public static final int TYPE_CAPITAL_ACTUAL_COST = Types.DECIMAL;

    public static final String NAME_CAPITAL_ACTUAL_COST = "TMT_ASSETS.CAPITAL_ACTUAL_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_ACTUAL_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_ACTUAL_COST,NAME_CAPITAL_ACTUAL_COST);}

    /**
     * Column CAPITAL_SCHED_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CAPITAL_SCHED_COST = 54;

    public static final int TYPE_CAPITAL_SCHED_COST = Types.DECIMAL;

    public static final String NAME_CAPITAL_SCHED_COST = "TMT_ASSETS.CAPITAL_SCHED_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_SCHED_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_SCHED_COST,NAME_CAPITAL_SCHED_COST);}

    /**
     * Column CAPITAL_COST_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CAPITAL_COST_ETC = 55;

    public static final int TYPE_CAPITAL_COST_ETC = Types.DECIMAL;

    public static final String NAME_CAPITAL_COST_ETC = "TMT_ASSETS.CAPITAL_COST_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_COST_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_COST_ETC,NAME_CAPITAL_COST_ETC);}

    /**
     * Column EXPENSE_ACTUAL_HOURS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPENSE_ACTUAL_HOURS = 56;

    public static final int TYPE_EXPENSE_ACTUAL_HOURS = Types.INTEGER;

    public static final String NAME_EXPENSE_ACTUAL_HOURS = "TMT_ASSETS.EXPENSE_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPENSE_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_ACTUAL_HOURS,NAME_EXPENSE_ACTUAL_HOURS);}

    /**
     * Column EXPENSE_SCHED_HOURS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPENSE_SCHED_HOURS = 57;

    public static final int TYPE_EXPENSE_SCHED_HOURS = Types.INTEGER;

    public static final String NAME_EXPENSE_SCHED_HOURS = "TMT_ASSETS.EXPENSE_SCHED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPENSE_SCHED_HOURS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_SCHED_HOURS,NAME_EXPENSE_SCHED_HOURS);}

    /**
     * Column EXPENSE_HOURS_ETC of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPENSE_HOURS_ETC = 58;

    public static final int TYPE_EXPENSE_HOURS_ETC = Types.INTEGER;

    public static final String NAME_EXPENSE_HOURS_ETC = "TMT_ASSETS.EXPENSE_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPENSE_HOURS_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPENSE_HOURS_ETC,NAME_EXPENSE_HOURS_ETC);}

    /**
     * Column CAPITAL_ACTUAL_HOURS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CAPITAL_ACTUAL_HOURS = 59;

    public static final int TYPE_CAPITAL_ACTUAL_HOURS = Types.INTEGER;

    public static final String NAME_CAPITAL_ACTUAL_HOURS = "TMT_ASSETS.CAPITAL_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_ACTUAL_HOURS,NAME_CAPITAL_ACTUAL_HOURS);}

    /**
     * Column CAPITAL_SCHED_HOURS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CAPITAL_SCHED_HOURS = 60;

    public static final int TYPE_CAPITAL_SCHED_HOURS = Types.INTEGER;

    public static final String NAME_CAPITAL_SCHED_HOURS = "TMT_ASSETS.CAPITAL_SCHED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_SCHED_HOURS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_SCHED_HOURS,NAME_CAPITAL_SCHED_HOURS);}

    /**
     * Column CAPITAL_HOURS_ETC of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CAPITAL_HOURS_ETC = 61;

    public static final int TYPE_CAPITAL_HOURS_ETC = Types.INTEGER;

    public static final String NAME_CAPITAL_HOURS_ETC = "TMT_ASSETS.CAPITAL_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_HOURS_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_HOURS_ETC,NAME_CAPITAL_HOURS_ETC);}

    /**
     * Column FINANCIAL_CATEGORY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FINANCIAL_CATEGORY = 62;

    public static final int TYPE_FINANCIAL_CATEGORY = Types.INTEGER;

    public static final String NAME_FINANCIAL_CATEGORY = "TMT_ASSETS.FINANCIAL_CATEGORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_FINANCIAL_CATEGORY = "FINANCIAL_CATEGORY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FINANCIAL_CATEGORY,
                NAME_FINANCIAL_CATEGORY);
    }

    /**
     * Column ACTIVITY_START of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_ACTIVITY_START = 63;

    public static final int TYPE_ACTIVITY_START = Types.DATE;

    public static final String NAME_ACTIVITY_START = "TMT_ASSETS.ACTIVITY_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTIVITY_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTIVITY_START,NAME_ACTIVITY_START);}

    /**
     * Column ACTIVITY_FINISH of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_ACTIVITY_FINISH = 64;

    public static final int TYPE_ACTIVITY_FINISH = Types.DATE;

    public static final String NAME_ACTIVITY_FINISH = "TMT_ASSETS.ACTIVITY_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTIVITY_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTIVITY_FINISH,NAME_ACTIVITY_FINISH);}

    /**
     * Column SD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SD = 65;

    public static final int TYPE_SD = Types.INTEGER;

    public static final String NAME_SD = "TMT_ASSETS.SD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SD,NAME_SD);}

    /**
     * Column FD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FD = 66;

    public static final int TYPE_FD = Types.INTEGER;

    public static final String NAME_FD = "TMT_ASSETS.FD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FD,NAME_FD);}

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 67;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "TMT_ASSETS.CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column TRACKED_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_TRACKED_FLAG = 68;

    public static final int TYPE_TRACKED_FLAG = Types.CHAR;

    public static final String NAME_TRACKED_FLAG = "TMT_ASSETS.TRACKED_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TRACKED_FLAG = "TRACKED_FLAG";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TRACKED_FLAG, NAME_TRACKED_FLAG);
    }

    /**
     * Column TOTAL_EST_MAINTENANCE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_TOTAL_EST_MAINTENANCE = 69;

    public static final int TYPE_TOTAL_EST_MAINTENANCE = Types.DECIMAL;

    public static final String NAME_TOTAL_EST_MAINTENANCE = "TMT_ASSETS.TOTAL_EST_MAINTENANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TOTAL_EST_MAINTENANCE = "TOTAL_EST_MAINTENANCE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TOTAL_EST_MAINTENANCE,
                NAME_TOTAL_EST_MAINTENANCE);
    }

    /**
     * Column ESTIMATED_CROSS_CHARGE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_CROSS_CHARGE = 70;

    public static final int TYPE_ESTIMATED_CROSS_CHARGE = Types.DECIMAL;

    public static final String NAME_ESTIMATED_CROSS_CHARGE = "TMT_ASSETS.ESTIMATED_CROSS_CHARGE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ESTIMATED_CROSS_CHARGE = "ESTIMATED_CROSS_CHARGE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_CROSS_CHARGE,
                NAME_ESTIMATED_CROSS_CHARGE);
    }

    /**
     * Column ACTUAL_CROSS_CHARGE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_CROSS_CHARGE = 71;

    public static final int TYPE_ACTUAL_CROSS_CHARGE = Types.DECIMAL;

    public static final String NAME_ACTUAL_CROSS_CHARGE = "TMT_ASSETS.ACTUAL_CROSS_CHARGE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ACTUAL_CROSS_CHARGE = "ACTUAL_CROSS_CHARGE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_CROSS_CHARGE,
                NAME_ACTUAL_CROSS_CHARGE);
    }

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 72;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_ASSETS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CREATED_BY = "CREATED_BY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY, NAME_CREATED_BY);
    }

    /**
     * Column CREATED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATETIME = 73;

    public static final int TYPE_CREATED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATETIME = "TMT_ASSETS.CREATED_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_CREATED_DATETIME = "CREATED_DATETIME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATETIME, NAME_CREATED_DATETIME);
    }

    protected static final String TABLE_NAME = "TMT_ASSETS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_ASSETS table.
     */
    protected static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK,
            NAME_CHILD_COUNT, NAME_DELETED_COUNT, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_DOC_COUNT, NAME_MAJOR_REVISION,
            NAME_MINOR_REVISION, NAME_REVISION_IMPACT, NAME_REVISION_TYPE,
            NAME_REVISION_HISTORY, NAME_CATEGORY_ID, NAME_STATUS_ID, NAME_MAKE,
            NAME_MODEL, NAME_MODEL_NUMBER, NAME_SERIAL_NUMBER,
            NAME_BARCODE_NUMBER, NAME_DATE_ACQUIRED, NAME_DATE_SOLD,
            NAME_PURCHASE_PRICE, NAME_CAPITAL_VALUE, NAME_TOTAL_DEPRECIATION,
            NAME_TOTAL_MAINTENANCE, NAME_DEPRECIATION_METHOD, NAME_QUANTITY,
            NAME_QUANTITY_USED, NAME_DEPRECIABLE_LIFE, NAME_SALVAGE_VALUE,
            NAME_CURRENT_VALUE, NAME_NEXT_SCHED_MAINT, NAME_DESCRIPTION,
            NAME_RTF_BITS, NAME_LICENCE_EXPIRY_DATE, NAME_WARRANTY_EXPIRY_DATE,
            NAME_CONTACTOR, NAME_CONTACTOR_PHONE, NAME_CONTACTOR_EMAIL,
            NAME_DEPRECIATION_START, NAME_DEPRECIATION_FINISH,
            NAME_SCORECARD_COUNT, NAME_CONVERTED_SCORE,
            NAME_EXPENSE_ACTUAL_COST, NAME_EXPENSE_SCHED_COST,
            NAME_EXPENSE_COST_ETC, NAME_CAPITAL_ACTUAL_COST,
            NAME_CAPITAL_SCHED_COST, NAME_CAPITAL_COST_ETC,
            NAME_EXPENSE_ACTUAL_HOURS, NAME_EXPENSE_SCHED_HOURS,
            NAME_EXPENSE_HOURS_ETC, NAME_CAPITAL_ACTUAL_HOURS,
            NAME_CAPITAL_SCHED_HOURS, NAME_CAPITAL_HOURS_ETC,
            NAME_FINANCIAL_CATEGORY, NAME_ACTIVITY_START, NAME_ACTIVITY_FINISH,
            NAME_SD, NAME_FD, NAME_CONVERTED_WEIGHT, NAME_TRACKED_FLAG,
            NAME_TOTAL_EST_MAINTENANCE, NAME_ESTIMATED_CROSS_CHARGE,
            NAME_ACTUAL_CROSS_CHARGE, NAME_CREATED_BY, NAME_CREATED_DATETIME
    };

    protected static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        if (typeId == AssetFolder.TYPE_ID)
        {
            return new Integer(typeId);
        }
        else if (typeId == Asset.TYPE_ID)
        {
            return new Integer(typeId);
        }
        else
        {
            return new Integer(0);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {

        //TODO create container of the requested type
        case 0:
            //case Container.TYPE_ID:
            //    object = new Container();
            break;
        case AssetFolder.TYPE_ID:

            object = new AssetFolder();
            break;
        case Asset.TYPE_ID:

            object = new Asset();
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }

    /**
     * Creates an asset as described by rpmObject. If rpmObject is null, does
     * nothing.
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (rpmObject == null)
        {
            return;
        }
        
        if (rpmObject instanceof Asset)
        {
            Asset asset = (Asset) rpmObject;
            
            // Set Asset defaults
            if (asset.getAssetLife() == null) 
                 asset.setAssetLife(new Integer(36));
            
            if (asset.getCapitalValue() == null)
                 asset.setCapitalValue(new Double(0.00));
            
            if (asset.getCrossChargeToDate() == null)
                 asset.setCrossChargeToDate(new Double(0.00));
            
            if (asset.getCurrentValue() == null)
                 asset.setCurrentValue(new Double(0.00));
            
            if (asset.getDateAcquired() == null) 
                 asset.setDateAcquired(DateUtil.getCalendarInstance());
            
            if (asset.getDepreciationMethod() == null)
                 asset.setDepreciationMethod(new Integer(1));
            
            if (asset.getDepreciationToDate() == null)
                 asset.setDepreciationToDate(new Double(0.00));
            
            if (asset.getEstimateCrossCharge() == null)
                 asset.setEstimateCrossCharge(new Double(0.00));
            
            if (asset.getEstimateMaintenance()== null)
                 asset.setEstimateMaintenance(new Double(0.00));
            
            if (asset.getInitialCost() == null)
                 asset.setInitialCost(new Double(0.00));
            
            if (asset.getMaintenanceToDate() == null)
                 asset.setMaintenanceToDate(new Double(0.00));
            
            if (asset.getQuantity() == null)
                 asset.setQuantity(new Integer(0));
           
            if (asset.getSalvageValue() == null)
                 asset.setSalvageValue(new Double(0.00));
            
            if (asset.getCategory() == null)
            {
                 AssetCategory category = new AssetCategory();
                 category.setID(DEFAULT_ASSET_CATEGORY);
                 category.setValue(DEFAULT_ASSET_CATEGORY);
                 asset.setCategory(category);
            }
            
            asset.setID(SP_I_ASSET(asset, messageContext));
        }
        else {
            GenericAsset savedObj = (GenericAsset) rpmObject;
            
            savedObj.setID(SP_I_ASSET(savedObj, messageContext));
        }
    }

    /**
     * Read a row from rs into rpmObject
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String id = rs.getString(ID_ELEMENT_ID);
        String name = rs.getString(ID_ELEMENT_NAME);
        if(name!=null)
        {
            name = name.trim();
        }
        if (rpmObject instanceof Asset)
        {
            Asset resObj = (Asset) rpmObject;
            resObj.setID(id);

            if (isDelta)
            {
                resObj.deltaName(name);
                resObj.deltaBarcodeNumber(rs.getString(ID_BARCODE_NUMBER));
                resObj.deltaCapitalValue(new Double(rs
                        .getDouble(ID_CAPITAL_VALUE)));
                resObj.deltaContact(rs.getString(ID_CONTACTOR));
                resObj.deltaContactEmail(rs.getString(ID_CONTACTOR_EMAIL));
                resObj.deltaContactPhone(rs.getString(ID_CONTACTOR_PHONE));
                resObj.deltaCurrentValue(new Double(rs
                        .getDouble(ID_CURRENT_VALUE)));
                resObj.deltaModel(rs.getString(ID_MODEL));
                resObj.deltaSalvageValue(new Double(rs
                        .getDouble(ID_SALVAGE_VALUE)));
                resObj.deltaAssetLife(new Integer(rs
                        .getInt(ID_DEPRECIABLE_LIFE)));

                resObj.deltaTracked(Manager.getBoolean(rs, ID_TRACKED_FLAG));
                resObj.deltaDateAcquired(DateUtil.convertToCalendar(rs
                        .getDate(ID_DATE_ACQUIRED)));
                resObj.deltaModelNumber(rs.getString(ID_MODEL_NUMBER));
                resObj.deltaSerialNumber(rs.getString(ID_SERIAL_NUMBER));
                resObj.deltaQuantity(new Integer(rs.getString(ID_QUANTITY)));
                resObj.deltaDispositionDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_DATE_SOLD)));
                resObj.deltaNextScheduleMaintenance(DateUtil
                        .convertToCalendar(rs.getDate(ID_NEXT_SCHED_MAINT)));
                resObj.deltaWarrantySLAExpiryDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_WARRANTY_EXPIRY_DATE)));
                resObj.deltaLicenseExpiryDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_LICENCE_EXPIRY_DATE)));
                resObj.deltaInitialCost(new Double(rs
                        .getDouble(ID_PURCHASE_PRICE)));
                resObj.deltaDepreciationMethod(new Integer(rs
                        .getInt(ID_DEPRECIATION_METHOD)));
            }
            else
            {
                resObj.setName(name);
                resObj.setBarcodeNumber(rs.getString(ID_BARCODE_NUMBER));
                resObj.setCapitalValue(new Double(rs
                        .getDouble(ID_CAPITAL_VALUE)));
                resObj.setContact(rs.getString(ID_CONTACTOR));
                resObj.setContactEmail(rs.getString(ID_CONTACTOR_EMAIL));
                resObj.setContactPhone(rs.getString(ID_CONTACTOR_PHONE));
                resObj.setCurrentValue(new Double(rs
                        .getDouble(ID_CURRENT_VALUE)));
                resObj.setModel(rs.getString(ID_MODEL));
                resObj.setSalvageValue(new Double(rs
                        .getDouble(ID_SALVAGE_VALUE)));
                resObj
                        .setAssetLife(new Integer(rs
                                .getInt(ID_DEPRECIABLE_LIFE)));
                resObj.setTracked(Manager.getBoolean(rs, ID_TRACKED_FLAG));
                resObj.setDateAcquired(DateUtil.convertToCalendar(rs
                        .getDate(ID_DATE_ACQUIRED)));
                resObj.setModelNumber(rs.getString(ID_MODEL_NUMBER));
                resObj.setSerialNumber(rs.getString(ID_SERIAL_NUMBER));
                resObj.setQuantity(new Integer(rs.getString(ID_QUANTITY)));
                resObj.setDispositionDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_DATE_SOLD)));
                resObj.setNextScheduleMaintenance(DateUtil.convertToCalendar(rs
                        .getDate(ID_NEXT_SCHED_MAINT)));
                resObj.setWarrantySLAExpiryDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_WARRANTY_EXPIRY_DATE)));
                resObj.setLicenseExpiryDate(DateUtil.convertToCalendar(rs
                        .getDate(ID_LICENCE_EXPIRY_DATE)));
                resObj.setInitialCost(new Double(rs
                        .getDouble(ID_PURCHASE_PRICE)));
                resObj.setDepreciationMethod(new Integer(rs
                        .getInt(ID_DEPRECIATION_METHOD)));
            }
            return resObj;
        }
        else if (rpmObject instanceof AssetFolder)
        {
            AssetFolder resObj = (AssetFolder) rpmObject;
            resObj.setID(id);
            if (isDelta)
            {
                resObj.deltaName(name);
            }
            else
            {
                resObj.setName(name);
            }
            return resObj;

        }
        else
        {
            logger.error(this.getClass().getName()
                    + " does not handle object of class "
                    + rpmObject.getClass().getName());
            return null;
        }
    }

    /**
     *
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        GenericAsset asset = null;
        if (rpmObject instanceof GenericAsset)
        {
            asset = (GenericAsset) rpmObject;
            SP_D_ASSET(asset, messageContext);
        }
    }

    /**
     *
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        // TODO Auto-generated method stub
        JoinCondition joinCondition = null;
        joinCondition = new JoinCondition();//getJoinCondition(manager,
        if ((manager instanceof AssetManager))
        {
            StringBuffer sb = new StringBuffer();
            sb.append(NAME_PARENT_ID + " =? ");
            if ((extraWhere != null) && (!(extraWhere.trim().length() == 0)))
            {
                sb.append(" AND " + extraWhere);
            }
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof AssetModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName()); 
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /**
     *
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        int typeId = -1;
        if (containerName.equals("AssetFolder"))
        {
            typeId = AssetFolder.TYPE_ID;
        }
        else if (containerName.equals("Asset"))
        {
            typeId = Asset.TYPE_ID;
        }
        JoinCondition joinCondition = null;
        if (manager == null)
        { //TODO: change 'D' into 'X'
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            StringBuffer where = new StringBuffer();
            if (condition != null)
                condition += " AND " + NAME_REC_STATUS + " != 'D'";
            else
                condition = NAME_REC_STATUS + "!= 'D'";
            where.append(condition);
            if (typeId >= 0)
            {
                where.append(" AND " + NAME_TYPE_ID + " = " + typeId);
            }
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof AssetManager)
        {
            joinCondition = new JoinCondition();
            AssetManager assetManager = (AssetManager) manager;
            StringBuffer where = new StringBuffer();

            where.append(AssetManager.NAME_PARENT_ID);
            where.append(" IN ");
            where.append("( SELECT " + AssetManager.NAME_ELEMENT_ID + " FROM "
                    + assetManager.getTableName());
            if (condition != null)
            {
                where.append(" WHERE " + condition + " ");
                if (typeId >= 0)
                {
                    where.append(" AND " + NAME_TYPE_ID + " = " + typeId);
                }
            }
            else
            {
                where.append(" WHERE " + NAME_TYPE_ID + " = " + typeId);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if ((manager instanceof AssetDepreciationManager)
                || (manager instanceof AssetOrganizationsManager)
                || (manager instanceof AssetResourcesManager)
                || (manager instanceof AssetCrossChargeManager)
                || (manager instanceof AssetMaintenanceManager))
        {
            StringBuffer where = new StringBuffer();
            if (manager instanceof AssetDepreciationManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetDepreciationManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetDepreciationManager.NAME_ASSET_ID);
            }
            else if (manager instanceof AssetOrganizationsManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetOrganizationsManager.NAME_ASSET_ID)
                        .append(" AND ");
                where.append(AssetOrganizationsManager.NAME_ASSET_ID);
            }
            else if (manager instanceof AssetResourcesManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetResourcesManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetResourcesManager.NAME_ASSET_ID);
            }
            else if (manager instanceof AssetCrossChargeManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                		AssetCrossChargeManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetCrossChargeManager.NAME_ASSET_ID);
            }
            else
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetMaintenanceManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetMaintenanceManager.NAME_ASSET_ID);
            }
            where.append(" IN (SELECT ");
            where.append(AssetManager.NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(AssetManager.NAME_TYPE_ID).append("=").append(
                    Asset.TYPE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof CustomFieldAssignmentManager)
        {
            if (ContextUtil.ASSET_CONTEXT.equalsIgnoreCase(contextName))
            {
                // for XPath queries like
                // Asset[<condition>]/AssignedCustomField
                joinCondition = new JoinCondition();
                StringBuffer where = new StringBuffer();
                where.append(CustomFieldAssignmentUtil
                        .getTableName(ContextUtil.ASSET_CONTEXT));
                where.append(".");
                where.append(CustomFieldAssignmentManager.NAME_FOREIGN_ID);
                where.append(" IN (");
                where.append(" SELECT ").append(NAME_ELEMENT_ID);
                where.append(" FROM ").append(getTableName());
                if (condition != null)
                {
                    where.append(" WHERE ");
                    where.append(condition);
                }
                joinCondition.setCondition(where.toString());
            }
        }
        else if (manager instanceof PublishedScorecardManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(PublishedScorecardManager.NAME_TYPE_ID);
            where.append(" = ").append(Scorecard.TYPE_ID);
            where.append(" AND ");
            where.append(PublishedScorecardManager.NAME_PROJECT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE " + condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof DocumentManager)
        {
            String docParentID = DocumentUtil.getInstance().getTableName(
                contextName)
                    + "." + DocumentManager.NAME_PARENT_ID;
            joinCondition = JoinCondition.createSubSelect(docParentID,
                NAME_ELEMENT_ID, TABLE_NAME, condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof AssetManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(AssetManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(AssetManager.NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(AssetManager.TABLE_NAME);
            if (condition != null)
            {
                where.append(" WHERE " + condition);

                if ((containerName != null)
                        && (containerName.equalsIgnoreCase("asset")))
                {
                    where.append(" AND ");
                    where.append(AssetManager.NAME_TYPE_ID);
                    where.append(" = ");
                    where.append(Asset.TYPE_ID);
                }
                else if ((containerName != null)
                        && (containerName.equalsIgnoreCase("assetfolder")))
                {
                    where.append(" AND ");
                    where.append(AssetManager.NAME_TYPE_ID);
                    where.append(" = ");
                    where.append(AssetFolder.TYPE_ID);

                }
                else
                    ExceptionUtil.handleNoTypeId(this, null);

            }
            else
            {
                if (containerName != null)
                {
                    where.append(" WHERE ");
                    where.append(AssetManager.NAME_TYPE_ID);
                    where.append(" = ");
                    where.append(Asset.TYPE_ID);
                }
                else
                    ExceptionUtil.handleNoTypeId(this, null);

            }

            joinCondition.setCondition(where.toString());
        }
        else if ((manager instanceof AssetDepreciationManager)
                || (manager instanceof AssetOrganizationsManager)
                || (manager instanceof AssetResourcesManager)
                || (manager instanceof AssetCrossChargeManager)
                || (manager instanceof AssetMaintenanceManager))
        {

            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            if (manager instanceof AssetDepreciationManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetDepreciationManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetDepreciationManager.NAME_ASSET_ID);
                where.append(" IN (SELECT ");
                where.append(AssetManager.NAME_ELEMENT_ID);
            }
            else if (manager instanceof AssetOrganizationsManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetOrganizationsManager.NAME_ASSET_ID)
                        .append(" AND ");
                where.append(AssetOrganizationsManager.NAME_ASSET_ID);
                where.append(" IN (SELECT ");
                where.append(AssetManager.NAME_ELEMENT_ID);
            }
            else if (manager instanceof AssetResourcesManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetResourcesManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetResourcesManager.NAME_ASSET_ID);
                where.append(" IN (SELECT ");
                where.append(AssetManager.NAME_ELEMENT_ID);
            }
            else if (manager instanceof AssetCrossChargeManager)
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                		AssetCrossChargeManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetCrossChargeManager.NAME_ASSET_ID);
                where.append(" IN (SELECT ");
                where.append(AssetManager.NAME_ELEMENT_ID);
            }
            else
            {
                where.append(AssetManager.NAME_ELEMENT_ID).append("=").append(
                        AssetMaintenanceManager.NAME_ASSET_ID).append(" AND ");
                where.append(AssetManager.NAME_ELEMENT_ID);
                where.append(" IN (SELECT ");
                where.append(AssetMaintenanceManager.NAME_ASSET_ID);
            }

            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(AssetManager.NAME_TYPE_ID).append("=").append(
                    Asset.TYPE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ScopeManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ScopeManager.NAME_ASSET_ID, NAME_ELEMENT_ID, TABLE_NAME,
                condition);
        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#saveObjectParents(boolean,
     *      com.ibm.rpm.containers.RPMObject)
     */
    public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericAsset savedObj = (GenericAsset) rpmObject;
        AssetScope scope = (AssetScope) rpmScope;
        if (scope != null && scope.getParent() != null)
        {
            RPMObject parent = rpmObject.getParent();
            if (parent != null)
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                        .getRPMObjectManager(parent.getClass().getName());
                if (parent instanceof Asset)
                {
                    AssetManager astMgr = (AssetManager) manager;
                    parent = astMgr.internalSave(parent, scope.getParent(),
                            messageContext);
                }
                else if (parent instanceof AssetFolder)
                {
                    AssetManager fldMgr = (AssetManager) manager;
                    parent = fldMgr.internalSave(parent, scope.getParent(),
                            messageContext);

                }
                //Move resource if pool Id has been changed and resource is not
                // new

            }

            // TODO from Dominique : Please move this call into the updateObject method and delete saveObjectParents
            if (savedObj.getID() != null && savedObj.testParentModified())
            {
                SP_M_ASSET(savedObj, messageContext);
            }

        }

    }

    /**
     *
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        if (rpmObject instanceof Asset)
        {
            Asset asset = (Asset) rpmObject;

            if ((asset.testNameModified()))
                SP_U_ASSET_NAME(asset, messageContext);
            if (asset.testBarcodeNumberModified()
                    || asset.testQuantityModified()
                    || asset.testStateModified()
//                    || asset.testCapitalValueModified()
                    || asset.testSerialNumberModified()
                    || asset.testModelNumberModified()
                    || asset.testModelModified()
//                    || asset.testNextScheduleMaintenanceModified()
//                    || asset.testSalvageValueModified()
//                    || asset.testCurrentValueModified()
                    || asset.testCategoryModified()
                    || asset.testBarcodeNumberModified()
                    || asset.testDispositionDateModified())
                SP_U_ASSET_DESC(asset, messageContext);
            if (asset.testTrackedModified())
                SP_U_ASSET_TRACKED(asset, messageContext);
            if (asset.testInitialCostModified()
                    || asset.testDateAcquiredModified()
                    || asset.testAssetLifeModified()
                    || asset.testDepreciationMethodModified())
                SP_U_ASSET_PRICE(asset, messageContext);
            if ((asset.testContactModified())
                    || (asset.testContactEmailModified())
                    || (asset.testContactPhoneModified())
                    || (asset.testLicenseExpiryDateModified())
                    || (asset.testWarrantySLAExpiryDateModified()))
                SP_U_ASSET_CONTACTOR(asset, messageContext);

            if (rpmScope != null)
            {
                // need to add for each scope children
                AssetScope scope = (AssetScope) rpmScope;
                saveAttributes(asset, scope, messageContext);
                saveAssetChildren(asset, scope, messageContext);
                saveDepreciationSchedule(asset, scope, messageContext);
                saveEstimates(asset, scope, messageContext);
                saveRtfAssignments(asset, scope, messageContext);
                saveOrganizations(asset, scope, messageContext);
                saveScorecard(asset, scope, messageContext);
                saveGeographicalAssignment(asset, scope, messageContext);
                saveResources(asset, scope, messageContext);
                saveCrossChargeFinancials(asset, scope, messageContext);
                
   				saveDocumentFolder(asset, scope, messageContext);

                propagateContextName(asset, asset.getCustomFieldAssignments());
                asset
                        .setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
                                asset, asset.getCustomFieldAssignments(), scope
                                        .isCustomFieldAssignments(),
                                messageContext, null));
            }
            SP_U_ASSET_REVISION(asset, messageContext, null);
            
            return asset;
        }
        else if (rpmObject instanceof AssetFolder)
        {
            AssetFolder savedObj = (AssetFolder) rpmObject;
            AssetScope scope = (AssetScope) rpmScope;
            if (savedObj.testNameModified())
                SP_U_ASSET_NAME(savedObj, messageContext);

            if (scope != null)
            {
                saveFolders(savedObj, scope, messageContext);
                saveAssetChildren(savedObj, scope, messageContext);

            }
            return savedObj;

        }
        else
        {
            logger.error(this.getClass().getName()
                    + " does not handle object of class "
                    + rpmObject.getClass().getName());
            return null;
        }

    }

    /**
     * Saves the AssetFolder children of the object
     *
     * @param savedObj
     * @param scope
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    private void saveFolders(AssetFolder savedObj, AssetScope scope,
            MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        AssetFolder[] children = savedObj.getFolders();
        children = (AssetFolder[]) updateChildren(savedObj, children,
                scope.getFolders(), messageContext, null);
        savedObj.setFolders(children);
    }

    private void saveAssetChildren(GenericAsset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericAsset[] children = asset.getAssets();
        children = (GenericAsset[]) updateChildren(asset, children,
                scope.getAssets(), messageContext, GenericAsset.class);
        asset.setAssets(children);
    }

    private void saveAttributes(GenericAsset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (asset instanceof Asset)
        {
            Asset ast = (Asset) asset;
            AttributeAssignment[] children = ast.getAttributeAssignments();
            children = (AttributeAssignment[]) updateChildren(ast,
                    children, scope.isAttributeAssignments(),
                    messageContext, null);
            ast.setAttributeAssignments(children);
        }
    }

    private void saveDepreciationSchedule(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        DepreciationSchedule[] children = asset.getDepreciationSchedules();
        children = (DepreciationSchedule[]) updateChildren(asset,
                children, scope.getDepreciationSchedules(),
                messageContext, null);
        asset.setDepreciationSchedules(children);
        updateSalvageValue(asset);
    }

    private void saveScorecard(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setAssignedScorecard((AssignedScorecard)
                updateChild(asset, asset.getAssignedScorecard(),
                        scope.getAssignedScorecard(), messageContext));
    }

    /**
     * Saves the Estimate if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void saveEstimates(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setMaintenanceEstimates((MaintenanceEstimates[])
                updateChildren(asset, asset.getMaintenanceEstimates(),
                scope.getMaintenanceEstimates(),
                messageContext, null));
    }

    /**
     * Saves the AssetOrganizationAssignment record if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void saveOrganizations(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setAssetOrganizationalAssignment((AssetOrganizationalAssignment)
                updateChild(asset, asset.getAssetOrganizationalAssignment(),
                scope.isAssetOrganizationalAssignment(), messageContext));
    }

    private Asset saveGeographicalAssignment(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setGeographicalAssignment((GeographicalAssignment)
                updateChild(asset, asset.getGeographicalAssignment(),
                scope.isGeographicalAssignment(), messageContext));
        return asset;
    }

    /**
     * Saves the resources if they are in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void saveResources(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setAssetResourceAssignments((AssetResourceAssignment[])
                updateChildren(asset, asset.getAssetResourceAssignments(),
                scope.isAssetResourceAssignments(),
                messageContext, null));
    }
    /**
     * Saves the Cross Charge Financials if they are in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void saveCrossChargeFinancials(Asset asset, AssetScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        asset.setCrossChargeFinancials((CrossChargeFinancials[])
                updateChildren(asset, asset.getCrossChargeFinancials(),
                scope.getCrossChargeFinancials(),
                messageContext, null));

    } // saveCrossChargeFinancials
    
    private static Log logger = LogFactory.getLog(AssetManager.class);


    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        String status = new String(resultSet.getString(ID_STATUS_ID));
        relatedFields.put(index, ID_STATUS_ID, status);
        String category = new String(resultSet.getString(ID_CATEGORY_ID));
        relatedFields.put(index, ID_CATEGORY_ID, category);

        return relatedFields;
    }

    private void loadChildren(RPMObject object, AssetScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (object instanceof Asset)
        {
            if (scope != null && scope.getAssets() != null)
            {
                Asset asset = (Asset) object;
                AssetManager manager = (AssetManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                AssetFolder.class.getName());

                //Actual parent
                ArrayList temp = manager.loadByForeignKey(asset.getAssets(),
                        scope.getAssets(), messageContext, this, new Object[]
                        {
                            asset.getID()
                        }, "(" + NAME_TYPE_ID + " = " + Asset.TYPE_ID + ")",
                        null, isDelta);

                Asset[] assets = new Asset[0];
                if (temp != null)
                {
                    setParent(temp, asset);// set the children assets' parents
                    assets = new Asset[temp.size()];
                    temp.toArray(assets);
                    asset.setAssets(assets);
                }

            }
        }
        else if (object instanceof AssetFolder)
        {
            AssetFolder folder = (AssetFolder) object;
            if (scope != null && scope.getFolders() != null)
            {
                AssetManager manager = (AssetManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                AssetFolder.class.getName());

                //Actual parent
                ArrayList temp = manager.loadByForeignKey(folder.getFolders(),
                        scope.getFolders(), messageContext, this, new Object[]
                        {
                            folder.getID()
                        }, "(" + NAME_TYPE_ID + " = " + AssetFolder.TYPE_ID
                                + ")", null, isDelta);

                AssetFolder[] folders = new AssetFolder[0];
                if (temp != null)
                {
                    setParent(temp, folder);// set the children assets' parents
                    folders = new AssetFolder[temp.size()];
                    temp.toArray(folders);
                    folder.setFolders(folders);
                }

            }
            if (scope != null && scope.getAssets() != null)
            {
                AssetManager manager = (AssetManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                Asset.class.getName());

                //Actual parent
                ArrayList temp = manager.loadByForeignKey(folder.getAssets(),
                        scope.getAssets(), messageContext, this, new Object[]
                        {
                            folder.getID()
                        }, "(" + NAME_TYPE_ID + " = " + Asset.TYPE_ID + ")",
                        null, isDelta);

                GenericAsset[] assets = new Asset[0];
                if (temp != null)
                {
                    setParent(temp, folder);// set the children assets' parents
                    assets = new GenericAsset[temp.size()];
                    temp.toArray(assets);
                    folder.setAssets(assets);
                }

            }

        }
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        
        loadParent(rpmObject, rpmScope, messageContext, relatedFields, index,
            isDelta);

        if (rpmObject instanceof Asset)
        {
            Asset object = (Asset) rpmObject;

            if (rpmScope != null  && rpmScope instanceof AssetScope)
            {
                AssetScope scope = (AssetScope) rpmScope;
                loadChildren(object, scope, messageContext, relatedFields,
                        isDelta);
                loadDepreciationSchedule(object, scope, messageContext, isDelta);
                loadAttributes(object, scope, messageContext, isDelta);
                loadEstimates(object, scope, messageContext, isDelta);
                loadState(object, scope, messageContext, relatedFields, index,
                        isDelta);
                loadGeographical(object, scope, messageContext, relatedFields,
                        index, isDelta);
                loadAssetRtfAssignments(object, messageContext, isDelta);
                loadOrganization(object, scope, messageContext, isDelta);
                loadScorecard(object, scope, messageContext, isDelta);
                loadResources(object, scope, messageContext, relatedFields,
                        index, isDelta);
                loadCrossChargeFinancials(object, scope, messageContext, relatedFields,
                        index, isDelta);
                loadDocumentFolder(object, scope, messageContext, isDelta);

                //todo: add for other children
	            if (scope.isCustomFieldAssignments())
                {
                    object.setCustomFieldAssignments(CustomFieldAssignmentUtil
                        .loadAssignedCustomFields(this, object
                            .getCustomFieldAssignments(), object,
                            messageContext, isDelta));
                    propagateContextName(object, object
                            .getCustomFieldAssignments());
                }
            }
            loadCategory(object,messageContext, relatedFields,
                    index, isDelta);

        }
        else if (rpmObject instanceof AssetFolder)
        {
            AssetFolder object = (AssetFolder) rpmObject;

            if (rpmScope != null && rpmScope instanceof AssetScope)
            {
                AssetScope scope = (AssetScope) rpmScope;
                loadChildren(object, scope, messageContext, relatedFields,
                        isDelta);

            }

        }
        else
        {
            logger.error(this.getClass().getName()
                    + " does not handle object of class "
                    + rpmObject.getClass().getName());

        }
        return rpmObject;
    }

    private void loadParent(RPMObject object, RPMObjectScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        
        if(scope != null){
            
        
	        if (object instanceof Asset)
	        {
	            Asset asset = (Asset) object;
	            if (scope != null && scope.getParent() != null)
	            {
	                AbstractRPMObjectManager tmpManager = (AssetManager) RPMManagerFactory
	                        .getInstance().getRPMObjectManager(
	                                AssetFolder.class.getName());
	                GenericAsset tmpAsset = new GenericAsset();
	                tmpAsset.setID((String) relatedFields.get(index, ID_PARENT_ID));
	                RPMObject newParent = tmpManager.loadByPrimaryKey(tmpAsset,
	                        scope.getParent(), messageContext, false);// isDelta=false
	                GenericAsset actualParent = null;
	
	                actualParent = (GenericAsset) newParent;
	
	                if (isDelta)
	                {
	                    asset.deltaParent(actualParent);
	                }
	                else
	                {
	                    asset.setParent(actualParent);
	                }
	
	            }
	        }
	        else if (object instanceof AssetFolder)
	        {
	            AssetFolder folder = (AssetFolder) object;
	            if (scope != null && scope.getParent() != null)
	            {
	                AssetManager manager = (AssetManager) RPMManagerFactory
	                        .getInstance().getRPMObjectManager(
	                                AssetFolder.class.getName());
	                AssetFolder parent = new AssetFolder();
	                if (relatedFields.get(index, ID_PARENT_ID) != null)
	                {
	                    parent.setID((String) relatedFields
	                            .get(index, ID_PARENT_ID));
	                    AssetFolder temp = (AssetFolder) manager.loadByPrimaryKey(
	                            parent, scope.getParent(), messageContext, false);
	
	                    if (isDelta)
	                    {
	                        folder.deltaParent(temp);
	                    }
	                    else
	                    {
	                        folder.setParent(temp);
	                    }
	                }
	                else
	                {
	                    folder.setParent(null);
	                }
	            }
	
	        }
	        else
	        {
	            logger.error(this.getClass().getName()
	                    + " does not handle object of class "
	                    + object.getClass().getName());
	
	        }
        }
    }

    /**
     * Loads the depreciationSchedule if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadDepreciationSchedule(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.getDepreciationSchedules() != null)
        {

            AssetDepreciationManager manager = (AssetDepreciationManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            DepreciationSchedule.class.getName());

            ArrayList temp = manager.loadByForeignKey(asset
                    .getDepreciationSchedules(), null, messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
                DepreciationSchedule[] depreciations = new DepreciationSchedule[temp
                        .size()];
                temp.toArray(depreciations);
                asset.setDepreciationSchedules(depreciations);

                //set Asset as a parent
                DepreciationSchedule dep = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    dep = (DepreciationSchedule) iter.next();
                    dep.setParent(asset);
                }//while
                updateSalvageValue(asset);
            }//if
        }//if
    }//loadDepreciationSchedule

    /**
     * Loads the scorecards if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadScorecard(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.getAssignedScorecard() != null)
        {

            PublishedScorecardManager manager = (PublishedScorecardManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AssignedScorecard.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
               asset, new RPMObject[]
            {
               asset.getAssignedScorecard()
            }, scope.getAssignedScorecard(), messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, asset.getContextName(), isDelta);

            // there can be only 1 assignedScorecard
            if (temp != null && temp.size() > 0)
            {
                AssignedScorecard scorecard = (AssignedScorecard) temp.get(0);
                if (isDelta)
                    asset.deltaAssignedScorecard(scorecard);
                else
                    asset.setAssignedScorecard(scorecard);
                scorecard.setParent(asset);
            }//if
        }//if
    }

    /**
     * Loads the loadAttributes if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadAttributes(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.isAttributeAssignments())
        {

            AttributeAssignmentManager manager = (AttributeAssignmentManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AttributeAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(asset
                    .getAttributeAssignments(), null, messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, getContextName(asset), isDelta);

            if (temp != null)
            {
                AttributeAssignment[] asgn = new AttributeAssignment[temp
                        .size()];
                temp.toArray(asgn);
                asset.setAttributeAssignments(asgn);

                //set Asset as a parent
                AttributeAssignment asg = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    asg = (AttributeAssignment) iter.next();
                    asg.setParent(asset);
                }//while
            }//if
        }//if
    }//loadAttributes

    private void loadAssetRtfAssignments(GenericAsset absAsset,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (absAsset instanceof Asset)
        {
            Asset ast = (Asset) absAsset;
            RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
                    .getName());

            Object[] args = new Object[]
            {
                ast.getID()
            };
            ArrayList temp = rtfAssignmentManager
                    .loadByForeignKeyWithSpecifiedParent(ast, ast
                            .getRtfAssignments(), null, messageContext, this,
                            args, null, getContextName(ast), isDelta);

            RtfAssignment[] rtfAssignments = null;
            if (temp != null && !temp.isEmpty())
            {
                setParent(temp, ast);
                rtfAssignments = new RtfAssignment[temp.size()];
                temp.toArray(rtfAssignments);
                ast.setRtfAssignments(rtfAssignments);
            }
        }
    }

    private GenericAsset saveRtfAssignments(GenericAsset absAst,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (absAst instanceof Asset)
        {
            Asset asset = (Asset) absAst;
            AssetScope scope = (AssetScope) rpmScope;
            asset.setRtfAssignments((RtfAssignment[])
                    updateChildren(asset, asset.getRtfAssignments(),
                    scope.isRtfAssignments(),
                    messageContext, null));
        }
        return absAst;
    }

    private void loadDocumentFolder(Asset asset, AssetScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if(scope.getDocumentFolder()!=null){
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                    asset.getDocumentFolder()
            };
            Object[] args = new Object[]
            {
                    asset.getID()
            };
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(asset, oldDocumentFolder, scope.getDocumentFolder(),
                    messageContext, this, args, null, ContextUtil.ASSET_DOCUMENT_CONTEXT, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder.setContextName(ContextUtil.ASSET_DOCUMENT_CONTEXT);
                asset.setDocumentFolder(documentFolder);
            }            
        }
    }
    
    private Asset saveDocumentFolder(Asset asset,
            AssetScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getDocumentFolder()!=null && asset.getDocumentFolder() != null)
        {
            RPMObjectManager manager = getManagerInstance(DocumentFolder.class
                    .getName());
            DocumentFolder documentFolder = asset.getDocumentFolder();
            documentFolder.setParent(asset);
            manager.internalSave(documentFolder, scope.getDocumentFolder(), messageContext);
            documentFolder.setContextName(ContextUtil.ASSET_DOCUMENT_CONTEXT);
            asset.setDocumentFolder(documentFolder);
        }
        return asset;
    }
    
    /**
     * Loads the status updates
     * Currently not used
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadStatusUpdates(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null)
        {

            AssetDepreciationManager manager = (AssetDepreciationManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            StatusUpdate.class.getName());

            ArrayList temp = manager.loadByForeignKey(asset.getStatusUpdates(),
                    null, messageContext, this, new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
                StatusUpdate[] updates = new StatusUpdate[temp.size()];
                temp.toArray(updates);
                asset.setStatusUpdates(updates);

                StatusUpdate upd = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    upd = (StatusUpdate) iter.next();
                    upd.setParent(asset);
                }//while
            }//if
        }//if
    }//loadDepreciationSchedule

    /**
     * Loads the status if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadState(Asset asset, AssetScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        if (scope != null)
        {//&& scope.isState()
            AssetState state = new AssetState();
            state.setID((String) relatedFields.get(index, ID_STATUS_ID));
            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AssetState.class.getName());
            state = (AssetState) manager.loadByPrimaryKey(state, null,
                    messageContext, isDelta);
            if (isDelta)
            {
                asset.deltaState(state);
            }
            else
            {
                asset.setState(state);
            }
        }//if
    }

    /**
     * Loads the Estimates if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadEstimates(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.getMaintenanceEstimates() != null)
        {

            // Get the AssetMaintenanceManager manager
            AssetMaintenanceManager manager = (AssetMaintenanceManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            MaintenanceEstimates.class.getName());

            // Load all Estimates
            ArrayList temp = manager.loadByForeignKey(asset
                    .getMaintenanceEstimates(), null, messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
                MaintenanceEstimates[] estimates = new MaintenanceEstimates[temp
                        .size()];
                temp.toArray(estimates);
                // Assign to asset
                asset.setMaintenanceEstimates(estimates);

                //Set asset as parent in estimates
                MaintenanceEstimates estimate = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    estimate = (MaintenanceEstimates) iter.next();
                    estimate.setParent(asset);
                }//while
            }//if
        }//if
    }//loadDEstimates

    /**
     * Loads the Asset Organization if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadOrganization(Asset asset, AssetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.isAssetOrganizationalAssignment())
        {

            // Get the AssetOrganizationsManager manager
            AssetOrganizationsManager manager = (AssetOrganizationsManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AssetOrganizationalAssignment.class.getName());

            // Load Organizational Assignment
            AssetOrganizationalAssignment[] orgs = new AssetOrganizationalAssignment[1];
            orgs[0] = asset.getAssetOrganizationalAssignment();
            ArrayList temp = manager.loadByForeignKey(orgs, null,
                messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
                temp.toArray(orgs);
                // Assign to asset
                asset.setAssetOrganizationalAssignment(orgs[0]);

                //Set asset as parent in Organizations
                AssetOrganizationalAssignment org = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    org = (AssetOrganizationalAssignment) iter.next();
                    org.setParent(asset);
                }//while
            }//if
        }//if
    }//loadOrganization

    /*
     * Loads the Resources Assignments if it is in the scope
     *
     * @throws RPMException @throws ParseException @throws SQLException
     */
    private void loadResources(Asset asset, AssetScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        if (scope != null && scope.isAssetResourceAssignments())
        {

            // Get the AssetResourcesManager manager
            AssetResourcesManager manager = (AssetResourcesManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AssetResourceAssignment.class.getName());

            // Load all Resource assignments
            ArrayList temp = manager.loadByForeignKey(asset
                    .getAssetResourceAssignments(), null, messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
                AssetResourceAssignment[] resourceAssignments = new AssetResourceAssignment[temp
                        .size()];
                temp.toArray(resourceAssignments);
                // Assign to asset
                asset.setAssetResourceAssignments(resourceAssignments);

                //Set asset as parent in resourceAssignment
                AssetResourceAssignment resourceAssignment = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    resourceAssignment = (AssetResourceAssignment) iter.next();
                    resourceAssignment.setParent(asset);
                }//while
            }//if
        }//if
    }//loadResources

    /*
     * Loads the Cross Charge Financials if it is in the scope
     *
     * @throws RPMException @throws ParseException @throws SQLException
     */
    private void loadCrossChargeFinancials(Asset asset, AssetScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        if (scope != null && scope.getCrossChargeFinancials() != null)
        {

            // Get the AssetCrossChargeManager manager
        	AssetCrossChargeManager manager = (AssetCrossChargeManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                    		CrossChargeFinancials.class.getName());

            // Load all CrossChargeFinancials
            ArrayList temp = manager.loadByForeignKey(asset
                    .getCrossChargeFinancials(), scope.getCrossChargeFinancials(), messageContext, this,
                    new Object[]
                    {
                        asset.getID()
                    }, null, null, isDelta);

            if (temp != null)
            {
            	CrossChargeFinancials[] crossChargeFinancials = new CrossChargeFinancials[temp
                        .size()];
                temp.toArray(crossChargeFinancials);
                // Assign to asset
                asset.setCrossChargeFinancials(crossChargeFinancials);

                //Set asset as parent in crossChargeFinancial
                CrossChargeFinancials crossChargeFinancial = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                	crossChargeFinancial = (CrossChargeFinancials) iter.next();
                	crossChargeFinancial.setParent(asset);
                }//while
            }//if
        }//if
    }//loadCrossChargeFinancials

    /**
     * Loads the status if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadCategory(Asset asset, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

            AssetCategory category = new AssetCategory();
            category.setID((String) relatedFields.get(index, ID_CATEGORY_ID));
            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            AssetCategory.class.getName());
            category = (AssetCategory) manager.loadByPrimaryKey(category, null,
                    messageContext, isDelta);
            if (isDelta)
            {
                asset.deltaCategory(category);
            }
            else
            {
                asset.setCategory(category);
            }
    }

    /**
     * Loads the status if it is in the scope
     *
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private void loadGeographical(Asset asset, AssetScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        if (scope != null && scope.isGeographicalAssignment())
        {

            GeographicalAssignmentManager manager = (GeographicalAssignmentManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            GeographicalAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                asset.getGeographicalAssignment()
            }, null, messageContext, this, new Object[]
            {
                asset.getID()
            }, null, null, isDelta);

            GeographicalAssignment geoAssignment = null;
            if (temp != null)
            {
                GeographicalAssignment[] geos = new GeographicalAssignment[temp
                        .size()];
                temp.toArray(geos);
                geoAssignment = geos[0];
            }
            if (isDelta)
            {
                asset.deltaGeographicalAssignment(geoAssignment);
            }
            else
            {
                asset.setGeographicalAssignment(geoAssignment);
            }
        }
    }

    private String SP_M_ASSET(GenericAsset ast, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_M_ASSET";
        Object args[] = new Object[4];

        if (ast.getParent() != null)
        {
            args[0] = ast.getParent().getID();
        }

        args[1] = ast.getID();
        args[2] = new Integer(4);
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset name in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_U_ASSET_NAME(GenericAsset asset,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_NAME";
        Object args[] = new Object[3];
        args[0] = asset.getID();
        args[1] = asset.getName();
        args[2] = getUser(messageContext).getID();

        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates various fields for the Asset in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_U_ASSET_DESC(Asset asset, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_DESC";
        Object args[] = new Object[18];
        Integer categoryId = null;
        if (asset.getCategory() != null)
        {
            categoryId = new Integer(asset.getCategory().getID());
        }
        else
        {
            categoryId = new Integer(5152);
        }
        args[0] = asset.getID();
        args[1] = categoryId;
        if (asset.getState() != null)
        {
            args[2] = new Integer(asset.getState().getID());
        }
        else
        {
            args[2] = new Integer(4971);//Status ID
        }
        args[3] = asset.getQuantity()==null?new Integer(0):asset.getQuantity();
        args[4] = null; //p_make
        args[5] = asset.getModel();
        args[6] = asset.getModelNumber();
        args[7] = asset.getSerialNumber();
        args[8] = asset.getBarcodeNumber();
        args[9] = DateUtil.convertToString(asset.getDispositionDate());
        args[10] = null; //DateUtil.convertToString(asset.getNextScheduleMaintenance());
        args[11] = null;
        args[12] = null; //asset.getSalvageValue();
        args[13] = null; //asset.getCurrentValue();
        args[14] = null; //asset.getCapitalValue();
        args[15] = null; //asset.getMaintenanceToDate();
        args[16] = null; //asset.getDepreciationToDate();
        args[17] = getUser(messageContext).getID();
        //args[3] = ;
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset price and related fiedls in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_U_ASSET_PRICE(Asset asset, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_PRICE";
        Object args[] = new Object[6];
        args[0] = asset.getID();
        args[1] = asset.getAssetLife() != null ? asset.getAssetLife()
                : new Integer(0);//asset.getCategory().getValue();
        args[2] = asset.getDepreciationMethod() != null ? asset
                .getDepreciationMethod() : new Integer(1);
        args[3] = asset.getDateAcquired() != null ? DateUtil
                .convertToString(asset.getDateAcquired()) : DateUtil
                .convertToString(DateUtil.getCalendarInstance());
        args[4] = asset.getInitialCost() != null ? asset.getInitialCost()
                : new Double(0); // "make"?
        args[5] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset contactor in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_U_ASSET_CONTACTOR(Asset asset,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_CONTACTOR";
        Object args[] = new Object[7];
        args[0] = asset.getID();
        args[1] = DateUtil.convertToString(asset.getLicenseExpiryDate());
        args[2] = DateUtil.convertToString(asset.getWarrantySLAExpiryDate());
        args[3] = asset.getContact();
        args[4] = asset.getContactPhone();
        args[5] = asset.getContactEmail();
        args[6] = getUser(messageContext).getID();
        //args[3] = ;
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset tracked flag in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     * @throws ParseException
     * @throws TransformerException
     */
    private String SP_U_ASSET_REVISION(Asset asset,
            MessageContext messageContext, RPMObjectScope scope)
            throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_REVISION";
        List revImpacts = DatafieldUtil.getDatafieldValue(StringUtil
            .getShortClassName(AssetRevisionImpact.class.getName()), "1",
            scope, messageContext);
        AssetRevisionImpact ri = (AssetRevisionImpact) revImpacts.get(0);//new
        // AssetsRevisionImpact();
        String riValue = ri.getID();
        List revTypes = DatafieldUtil.getDatafieldValue(StringUtil
            .getShortClassName(AssetRevisionType.class.getName()), "1", scope,
            messageContext);
        AssetRevisionType rt = (AssetRevisionType) revTypes.get(0);//new
        // AssetsRevisionImpact();
        String rtValue = rt.getID();
        Object args[] = new Object[6];
        args[0] = asset.getID();
        args[1] = new Integer(0);
        args[2] = new Integer(Integer.parseInt(riValue));
        args[3] = new Integer(Integer.parseInt(rtValue)); // should be changed to rev. type
        args[4] = null;
        args[5] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset tracked flag in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     * @throws ParseException
     * @throws TransformerException
     */
    private String SP_S_ASSET_REVISION(Asset asset,
            MessageContext messageContext, RPMObjectScope scope)
            throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_S_ASSET_REVISION";
        Object args[] = new Object[2];
        args[0] = asset.getID();
        args[1] = new Integer(0);
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Asset tracked flag in the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_U_ASSET_TRACKED(Asset asset, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_ASSET_TRACKED";
        Object args[] = new Object[3];
        args[0] = asset.getID();
        args[1] = dbStringValue(asset.getTracked().booleanValue());
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Deletes the asset in the DB
     *
     * @param asset
     * @param messageContext
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_D_ASSET(GenericAsset asset, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_D_ASSET";
        Object args[] = new Object[4];
        args[0] = asset.getID();
        args[1] = ManagerUtil.getParentId(this, asset, messageContext,
            getTableName(asset));
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    /**
     * Inserts an asset to the DB
     *
     * @param asset
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_I_ASSET(GenericAsset asset, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_I_ASSET";
        Object args[] = new Object[4];
        args[0] = asset.getName();

        if ((args[1] == null) && (asset.getParent() != null))
        {
            args[1] = asset.getParent().getID();
        }

        args[2] = asset.retrieveTypeID();
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Checks out an Asset
     *
     * @param res
     * @param messageContext
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_CKOUT_ASSET(GenericAsset res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKOUT_ASSET";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    /**
     * Undo a check out in the DB
     *
     * @param res
     * @param messageContext
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_UCKOUT_ASSET(GenericAsset res,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_UCKOUT_ASSET";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    /**
     *
     */
    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_UCKOUT_ASSET((GenericAsset) rpmObject, messageContext);
        }
        catch (SQLException e)
        {
            logger.error(e);
        }
    }

    /**
     * Checks in an asset to the DB
     *
     * @param res
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     * @throws SQLException
     * @throws ParseException
     */
    private void SP_CKIN_ASSET(GenericAsset res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKIN_ASSET";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    
    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        if (rpmObject instanceof Asset || rpmObject instanceof AssetFolder)
        {
            try
            {
                SP_CKOUT_ASSET((GenericAsset) rpmObject, messageContext);
            }
            catch (SQLException e)
            {
                logger.error(e);
            }
        }
    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        if (rpmObject instanceof Asset || rpmObject instanceof AssetFolder)
        {
            try
            {
                SP_CKIN_ASSET((GenericAsset) rpmObject, messageContext);
            }
            catch (SQLException e)
            {
                logger.error(e);
            }
        }
    }

    protected static String dbStringValue(boolean val)
    {
        if (val)
        {
            return "Y";
        }
        return "N";
    }

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }

    private String getContextName(RPMObject rpmObject) throws RPMException
    {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }

    public void updateSalvageValue(Asset asset){
    	double totalDepreciation=0.0;
    	double depreciationToDate=0.0;
    	Calendar now=DateUtil.getCalendarInstance();
    	if (asset.getDepreciationSchedules()!=null){
    		DepreciationSchedule[] scheds=asset.getDepreciationSchedules();
    		for(int i =0;i<scheds.length;++i){
    			totalDepreciation+=scheds[i].getAmount().doubleValue();
    			if (!scheds[i].getDepreciationDate().after(now)){
    			depreciationToDate+=scheds[i].getAmount().doubleValue();
    			}
    		}
    	}
    	asset.setDepreciationToDate(new Double(depreciationToDate));
    	if (asset.getCapitalValue()!=null){
    	asset.setSalvageValue(new Double(asset.getCapitalValue().doubleValue()-totalDepreciation));
    	}
    }

    public void updateMaintenaceToDate(Asset asset){
    	double mainenaceToDate=0.0;
    	Calendar now=DateUtil.getCalendarInstance();
    	if (asset.getMaintenanceEstimates()!=null){
    		MaintenanceEstimates[] estimates=asset.getMaintenanceEstimates();
    		for(int i =0;i<estimates.length;++i){
				if (estimates[i].getActualCost()!=null){
					mainenaceToDate+=estimates[i].getActualCost().doubleValue();
				}else if (!estimates[i].getScheduleDate().after(now)){
					mainenaceToDate+=estimates[i].getEstimatedCost().doubleValue();
    			}
    		}
    	}
    	asset.setMaintenanceToDate(new Double(mainenaceToDate));
    }

}