package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.managers.OrganizationalManager;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;

public class AssetOrganizationsManager extends AbstractRPMObjectManager{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        //the complete class name of all the containers that could be loaded by this managers.
        CONTAINERS.add(AssetOrganizationalAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                AssetOrganizationsManager.class);
        }
    }
    /**
     * Column ASSET_ORG_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ORG_ID = 1;
    public static final int TYPE_ASSET_ORG_ID = Types.CHAR;
    public static final String NAME_ASSET_ORG_ID = "ASSET_ORGANIZATIONS.ASSET_ORG_ID";
    public static final String PROPERTY_ASSET_ORG_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ORG_ID,NAME_ASSET_ORG_ID);}


    /**
     * Column ORGANIZATION_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ORGANIZATION_ID = 2;
    public static final int TYPE_ORGANIZATION_ID = Types.CHAR;
    public static final String NAME_ORGANIZATION_ID = "ASSET_ORGANIZATIONS.ORGANIZATION_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ORGANIZATION_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ORGANIZATION_ID,NAME_ORGANIZATION_ID);}

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 3;
    public static final int TYPE_ASSET_ID = Types.CHAR;
    public static final String NAME_ASSET_ID = "ASSET_ORGANIZATIONS.ASSET_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 4;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "ASSET_ORGANIZATIONS.DESCRIPTION";
    //public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}


    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 5;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "ASSET_ORGANIZATIONS.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 6;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "ASSET_ORGANIZATIONS.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 7;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "ASSET_ORGANIZATIONS.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    private static final String TABLE_NAME = "ASSET_ORGANIZATIONS";

    /**
     * Create an array of type string containing all the fields of the ASSET_ORGANIZATIONS table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_ASSET_ORG_ID
        ,NAME_ORGANIZATION_ID
        ,NAME_ASSET_ID
        ,NAME_DESCRIPTION
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_REC_USER
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    }

    /**
     * Creates a new AssetOrganizationsManager instance.
     * @return AssetOrganizationsManager
     */   
    public AssetOrganizationsManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() {
        return NAME_ASSET_ORG_ID;
    }  

    protected Integer getContainerType(ResultSet resultSet) throws SQLException {

    	//If container type doesn't apply to this manager
    	return new Integer(0);
    }  

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }  

    protected HashSet getValidContainerNames() {
        return CONTAINERS;
    }  

    protected RPMObject createContainer(int typeId) throws RPMException {

        RPMObject object = new AssetOrganizationalAssignment();

         return object;
     }

    /**
     * Create a record for Asset Organization in DB and set its ID.
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException {

        AssetOrganizationalAssignment pObject = (AssetOrganizationalAssignment)rpmObject;

        // Insert the record
        pObject.setID(SP_A_ASSET_ORG(pObject, messageContext));
    } // createObject
    

    /**
     * decode a row of Organizational Assignment. this includes only the id
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException {
        AssetOrganizationalAssignment pObject = (AssetOrganizationalAssignment)rpmObject;
		
        pObject.setID(rs.getString(ID_ASSET_ORG_ID));
        
        return pObject;
    }

    /**
     * delete an Organization record
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException 
    {

        if(rpmObject != null)
        {
            	SP_UA_ASSET_ORG((AssetOrganizationalAssignment) rpmObject,
                        messageContext);
        }
    } // delete

    /**
     * Returns the foreign key sql conditions of the asset that exists in the
     * Asset Organization table
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {
        if (!(manager instanceof AssetManager))
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);


        JoinCondition joinCondition = new JoinCondition();
        joinCondition.setTableName(getTableName());

        //where clause of the join condition
        StringBuffer condition = new StringBuffer();
        condition.append(NAME_ASSET_ID);
        condition.append(" = ?");
        if (extraWhere != null && extraWhere.length() > 0)
        {
            condition.append(" AND ");
            condition.append(extraWhere);
        }
        joinCondition.setCondition(condition.toString());

        return joinCondition;
    } // getForeignKeyCondition    

    /**
     * Return JoinCondition object with condition input parameter
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = new JoinCondition();
        if (manager instanceof AssetManager)
        {
            // for XPath queries like AssetOrganizationsManager[<condition>]/Asset
            StringBuffer where = new StringBuffer();
            where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ASSET_ID);
            where.append(" FROM ").append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else if (manager == null)

        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }
    
        }

        else if (manager instanceof OrganizationalManager)
        {
            // for XPath queries like /AssetOrganizationsManager[<condition>]/Organization
            StringBuffer where = new StringBuffer();
            where.append(OrganizationalManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ORGANIZATION_ID);
            where.append(" FROM ").append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }

        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
    
        return joinCondition;
    }
    
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

    	JoinCondition joinCondition = new JoinCondition();
        if (manager instanceof AssetManager)
        {
            // for XPath queries like /Asset[AssetOrganizationsManager[<condition>]]
            StringBuffer where = new StringBuffer();
            where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ASSET_ID);
            where.append(" FROM ").append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof OrganizationalManager)
        {
            // for XPath queries like /Organization[AssetOrganizationsManager[<condition>]]
            StringBuffer where = new StringBuffer();
            where.append(OrganizationalManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ORGANIZATION_ID);
            where.append(" FROM ").append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
	    {
        	ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
	    }

        return joinCondition;
    }
        
    /**
     * The decoded related objects is the parent asset and organitional
     * 
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_ASSET_ID);
        relatedFields.put(index, ID_ASSET_ID, parentId);
        String organizationalId = resultSet.getString(ID_ORGANIZATION_ID);
        relatedFields.put(index, ID_ORGANIZATION_ID, organizationalId);

        return relatedFields;
    }
    
    /**
     * There are two related objects: parent asset & Assigned Organizational
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // Load Assigned Organizational. The parent will be setted by the assetManager.        
        AssetOrganizationalAssignment object = (AssetOrganizationalAssignment) rpmObject;

        //ID of Related organizaion
        String organizationalId = (String) relatedFields.get(index, ID_ORGANIZATION_ID);

        if ( (organizationalId != null) )
        {
            // Load the Organizational Manager
            OrganizationalManager organizationalManager = (OrganizationalManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(Organization.class.getName());
            Organizational organizational = new Organizational();
            // Set the Organizational ID
            organizational.setID(organizationalId);

            // Load the Organizational record
            organizational = (Organizational) organizationalManager.loadByPrimaryKey(organizational, null, messageContext, isDelta);
            if (isDelta)
            {
            	object.deltaOrganizational(organizational);
            }
            else
            {
                object.setOrganizational(organizational);
            }
        } // if (organizationalId != null)
        if ( (organizationalId == null))
        {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.ORGANIZATION_ID);
        } //if ( (organizationalId == null) && (assetId == null) )

        return rpmObject;
    } // loadRelatedObjects

    /**
     * updateObject - Nothing to update
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException {
        // nothing to update      
        return rpmObject;
    }

    /**
     * Inserts an Asset Organization record
     * 
     * @param assignment -
     *            AssetOrganizationalAssignment record to save
     * @param messageContext -
     *            context
     * @return a String that represents the ID of the new created organization
     *         schedule
     * @throws SQLException
     * @throws RPMException
     */
    private String SP_A_ASSET_ORG(AssetOrganizationalAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_ASSET_ORG";
        Object args[] = new Object[3];
        args[0] = assignment.getParent().getID();
        args[1] = assignment.getOrganizational().getID();
        args[2] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    } // SP_A_ASSET_ORG      

    /**
     * Delete a Asset Organization record from the DB
     * 
     * @param AssetOrganizationalAssignment -
     *            Organizational Assignment object
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */    
    private void SP_UA_ASSET_ORG(AssetOrganizationalAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_ASSET_ORG";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        args[1] = assignment.getID();
        args[2] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }    
}
