package com.ibm.rpm.client.junit;

import java.lang.reflect.Method;
import java.net.URL;
import java.rmi.RemoteException;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.interfaces.Authenticate;
import com.ibm.rpm.interfaces.AuthenticateLiteral;

/**
 * @author eturcot
 */
public class AuthenticateController implements Authenticate
{
    private Authenticate authInterface = null;
    private AuthenticateLiteral authLiteralInterface = null;
    
    private boolean useLiteral;
    
    public AuthenticateController(String url, boolean useLiteral) throws Exception {
        super();
        this.useLiteral = useLiteral;
        if (useLiteral) {
            URL service = new URL(url + "AuthenticateLiteral");
            Class locatorClass = Class.forName("com.ibm.rpm.interfaces.AuthenticateLiteralServiceLocator");
            Object locator = locatorClass.newInstance();
            Method getAuthenticate = locatorClass.getMethod("getAuthenticateLiteral", new Class[]{service.getClass()});
            authLiteralInterface = (AuthenticateLiteral)getAuthenticate.invoke(locator, new Object[]{service});
        } else {
            URL service = new URL(url + "Authenticate");
            Class locatorClass = Class.forName("com.ibm.rpm.interfaces.AuthenticateServiceLocator");
            Object locator = locatorClass.newInstance();
            Method getAuthenticate = locatorClass.getMethod("getAuthenticate", new Class[]{service.getClass()});
            authInterface = (Authenticate)getAuthenticate.invoke(locator, new Object[]{service});
        }
    }

    public String login(String username, String password, String dsn)
            throws RemoteException, RPMException
    {
        if (useLiteral)
        {
            return authLiteralInterface.login(username, password, dsn);
        }
        return authInterface.login(username, password, dsn);
    }

    public void logout(String sessionID) throws RemoteException, RPMException
    {
        if (useLiteral)
        {
            authLiteralInterface.logout(sessionID);
        }
        else
        {
        	authInterface.logout(sessionID);
        }
    }

    public void changePassword(String username, String oldPw, String newPw,
            String dsn) throws RemoteException, RPMException
    {
        if (useLiteral)
        {
            authLiteralInterface.changePassword(username, oldPw, newPw, dsn);
        }
        else
        {
            authInterface.changePassword(username, oldPw, newPw, dsn);
        }
    }

    public void assignWorkflowToSession(String sessionId, String workflowId)
            throws RemoteException, RPMException
    {
        if (useLiteral)
        {
            authLiteralInterface.assignWorkflowToSession(sessionId, workflowId);
        }
        else
        {
        	authInterface.assignWorkflowToSession(sessionId, workflowId);
        }
    }
}
