package com.ibm.rpm.client.junit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.interfaces.Authenticate;
import com.ibm.rpm.interfaces.InternalServices;
import com.ibm.rpm.interfaces.impl.ManagerCaller;
import com.ibm.rpm.interfaces.impl.ServerFactory;

public class SpecificTestController
{
    public Authenticate authenticateInterface = null;

    public Application appInterface = null;
    
    public InternalServices internalServices = null;
    
    private static Map overrideOptions = new HashMap();
    
    /**
     * Note this method needs to be executed before the setup phase is done
     * @param option
     * @param value
     */
    public static void overrideOption(String option, Object value)
    {
    	//only support single override
    	if(!overrideOptions.containsKey(option))
    	{
            overrideOptions.put(option, value);    		
    	}
    }
    
    public static void clearOverrideOptions()
    {
        overrideOptions.clear();
        getOptions().clearAllOptionOverride();
    }
    
    private static RpmOptions getOptions()
    {
        ServerFactory factory = ServerFactory.getInstance();
        RpmOptions currentOptions = factory.getCurrentOptions();
        return currentOptions;
    }

    public void setUp() throws Exception
    {
        RpmOptions options = RpmOptions.getDefaultOptions();
        Iterator it = overrideOptions.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry element = (Map.Entry) it.next();
            options.overrideOption((String) element.getKey(),
                element.getValue());
        }
        options.logInitialOptions();
        ServerFactory.restartServerFactory();

        ManagerCaller.PRINT_EXCEPTION_STACKTRACE = false;
        ServerFactory.getInstance().register(UnitTestSessionUtility.INSTANCE);
        authenticateInterface = new Authenticate();
        appInterface = new Application();
        internalServices = new InternalServices();
        
        
    }

    public void tearDown() throws Exception
    {
        // Nothing specific to do.
        clearOverrideOptions();
    }

    /**
     * Return true when the tests are executed remotely, through the Web Service
     * interface. Otherwise, returns false.
     * 
     * @return True when the tests are executed remotely.
     */
    public boolean isRemote()
    {
        return false;
    }
    
    /**
     * returns ClientTestController.INTERFACE_PROVIDER.url if any
     * @return
     */
    public String getRemoteURL()
    {
        return null;
    }
    
    public ArrayResult loadFromXpath(String sessionID, String xpath,
			RPMObjectScope scope, Integer idsType, String idsList,
			String attributesIdsList, Integer viewType, Integer inclusions,
			Integer startIndex, Integer endIndex, String extraWhere, String orderBy,
			boolean buildSecurity) {
		return internalServices.loadFromXpath(sessionID, xpath, scope, idsType,
				idsList, attributesIdsList, viewType, inclusions, startIndex,
				endIndex, extraWhere, orderBy, buildSecurity);
	}
    
    public Result executeProcedure(String sessionID, Class callerClass, String procedureName,
            Object[] args)
    {
        return internalServices.executeProcedure(sessionID, callerClass, procedureName,
            args);
    }
    
    public Result executeSqlUpdate(String sessionID, String updateString,
            Object[] args)
    {
        return internalServices.executeSqlUpdate(sessionID, updateString, args);
    }
    
    public ArrayList executeSqlQuery(String sessionID, String query,
            Object[] args) throws RPMException
    {
        return internalServices.executeSqlQuery(sessionID, query, args);
    }
}
