package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AdministrationModuleScope;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;

public class AdministrationModuleFixture extends AbstractModuleFixture
{

    public static AdministrationModule createFixture(
            AdministrationModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        return createModule(scope, ctc);
    }

    public static AdministrationModule updateFixture(
            AdministrationModule module, AdministrationModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static AdministrationModule createModule(
            AdministrationModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        AdministrationModule module = new AdministrationModule();

        if (scope != null)
        {
            if(scope.getRpmCalendars()!=null)
            {
                RPMCalendar[] rpmCalendar = (RPMCalendar[]) createElements(scope
                        .getRpmCalendars(), RPMCalendar.class, ctc);
                    module.setRpmCalendars(rpmCalendar);
            }
            if(scope.getOrganizationals()!=null)
            {
                Organizational[] orgnizational = (Organizational[]) createElements(
                        scope.getOrganizationals(), Organizational.class, ctc);
                    module.setOrganizationals(orgnizational);
            }
            if(scope.getGeographicals()!=null)
            {
                Geographical[] geographical = (Geographical[]) createElements(scope
                        .getGeographicals(), Geographical.class, ctc);
                    module.setGeographicals(geographical);
            }
        }

        return module;

    }

    private static AdministrationModule updateModule(
            AdministrationModule module, AdministrationModuleScope scope,
            ClientTestController ctc) throws Exception
    {

        if (scope != null)
        {
            if(scope.getRpmCalendars()!=null)
            {
                RPMCalendar[] rpmCalendar = (RPMCalendar[]) updateElements(module
                        .getRpmCalendars(), scope.getRpmCalendars(), ctc);
                    module.setRpmCalendars(rpmCalendar);
            }
            if(scope.getOrganizationals()!=null)
            {
                Organizational[] organizational = (Organizational[]) updateElements(
                        module.getOrganizationals(), scope.getOrganizationals(), ctc);
                    module.setOrganizationals(organizational);
            }
            if(scope.getGeographicals()!=null)
            {
                Geographical[] geographical = (Geographical[]) updateElements(
                        module.getGeographicals(), scope.getGeographicals(), ctc);
                    module.setGeographicals(geographical);
            }
        }

        return module;

    }

    public static AdministrationModuleScope createFullScope()

    {
        return createScope(true, true, true, true, true);
    }
    
    private static AdministrationModuleScope createScope(boolean isCurrency,
            boolean isRpmCalendar, boolean isOrganizational,
            boolean isGeographical, boolean isDatafieldCategory)

    {
        AdministrationModuleScope scope = new AdministrationModuleScope();

        if (isCurrency)
        {
            scope.setCurrencies(true);
        }
        if (isRpmCalendar)
        {
            scope.setRpmCalendars(new CalendarScope());
        }
        if (isOrganizational)
        {
            scope.setOrganizationals(new OrganizationalScope());
        }
        if (isGeographical)
        {
            scope.setGeographicals(new GeographicalScope());
        }
        if(isDatafieldCategory)
        {
            scope.setDatafieldCategories(true);
        }
        
        return scope;
    }

    public static void compare(AdministrationModule obj1,
            AdministrationModule obj2, AdministrationModuleScope scope)
            throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if(scope.isCurrencies())
            {
                CurrencyFixture.compare(obj1.getCurrencies(), obj2.getCurrencies());
            }
            if(scope.getRpmCalendars()!=null)
            {
                RPMCalendarFixture.compare(obj1.getRpmCalendars(), obj2
                        .getRpmCalendars(), scope.getRpmCalendars());
            }
            if(scope.getOrganizationals()!=null)
            {
                OrganizationalFixture.compare(obj1.getOrganizationals(), obj2
                        .getOrganizationals(), scope.getOrganizationals());
            }
            if(scope.getGeographicals()!=null)
            {
                GeographicalFixture.compare(obj1.getGeographicals(), obj2
                        .getGeographicals(), scope.getGeographicals());
            }
            if (scope.isDatafieldCategories())
            {
                DatafieldFixture.compare(obj1.getDatafieldCategories(), obj2
                        .getDatafieldCategories());
            }
        }
    }

}
