/*
 * Created on Jul 7, 2005
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author gosselin
 */
public class ContactGroupFixture extends RPMObjectFixture
{
	private static final int DEFAULT_NB_LEVELS = 4;
	
    public static ContactGroup createSaveFixture(int contactGroupsByLevel, ClientTestController ctc) throws Exception {

    	ContactGroupScope scope = createScope(1, DEFAULT_NB_LEVELS);
        ContactGroup contactGroups = createFixture(contactGroupsByLevel, scope);

        //Save ContactGroup
        contactGroups = (ContactGroup) save(ctc, contactGroups, scope);

        return contactGroups;

    }
	
    public static ContactGroup createFixture(int contactGroupsByLevel, ContactGroupScope scope) throws Exception
    {
    	ContactGroup contactGroups =  createContactGroup(random(), contactGroupsByLevel, null, scope);
    	return contactGroups;
    }
	
    
    private static ContactGroup createContactGroup(int index, int contactGroupsByLevel, ContactGroup parentContactGroup, ContactGroupScope scope)throws Exception
	{
    	ContactGroup contactGroup = new ContactGroup();
    	HelperTestUtilities.setByIntrospection(contactGroup, index, skipSetters);
    	contactGroup.setParent(parentContactGroup);
    	if(scope!=null && scope.getContactGroups()!=null){
    		ContactGroup childContactGroup = null;
    		ContactGroup[] childContactGroups = new ContactGroup[contactGroupsByLevel]; 
    		for(int i=1;i<=contactGroupsByLevel;i++){
    			childContactGroup = createContactGroup(random(), contactGroupsByLevel, contactGroup, scope.getContactGroups());
    			childContactGroups[i-1] = childContactGroup;
    		}
    		contactGroup.setContactGroups(childContactGroups);
    	}
    	return contactGroup;
    }
    
    //update fields
    public static void updateFixtureFields(ContactGroup contactGroup, ContactGroupScope scope) throws Exception
    {
    	updateFields(contactGroup, random(), scope);
    }
    
    private static void updateFields(ContactGroup contactGroup, int index, ContactGroupScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(contactGroup, index, skipSetters);
		if (scope != null && scope.getContactGroups() != null) {
			for(int i=0;i<contactGroup.getContactGroups().length;i++){
				updateFixtureFields(contactGroup.getContactGroups()[i], scope.getContactGroups());
			}
		}
    }
    
    //update structure
    public static void updateFixtureStructure(ContactGroup contactGroup, int deltaNbContactGroupsByLevel, int nbContactGroupsByLevel, ContactGroupScope scope)throws Exception{
    	updateStructure(contactGroup, deltaNbContactGroupsByLevel, nbContactGroupsByLevel, scope);
    }
    
    private static void updateStructure(ContactGroup contactGroup, int deltaNbContactGroupsByLevel, int nbContactGroupsByLevel, ContactGroupScope scope)throws Exception{
		if (scope != null && scope.getContactGroups() != null) {
			if(deltaNbContactGroupsByLevel>0){//add new elements
				ContactGroup newContactGroup = null;
				ContactGroup[] newContactGroups = new ContactGroup[deltaNbContactGroupsByLevel];
		    	for(int i=0;i<deltaNbContactGroupsByLevel;i++){
		    		newContactGroup = createFixture(nbContactGroupsByLevel, scope.getContactGroups());
		    		newContactGroup.setParent(contactGroup);
		    		newContactGroups[i] = newContactGroup;
		    	}
		    	
		    	ArrayList temp = mergeArrays(contactGroup.getContactGroups(), newContactGroups);
		    	ContactGroup[] totalContactGroups = new ContactGroup[temp.size()];
		    	temp.toArray(totalContactGroups);
		    	
		    	contactGroup.setContactGroups(totalContactGroups);		    	
			}
			for(int i=0;i<contactGroup.getContactGroups().length;i++){
				updateFixtureStructure(contactGroup.getContactGroups()[i], deltaNbContactGroupsByLevel, nbContactGroupsByLevel, scope.getContactGroups());
			}
		}
    }
    
    public static void compare(ContactGroup p1, ContactGroup p2,
            ContactGroupScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(p1, p2);

        if (scope == null) return;
        
        if(scope.getContactGroups() != null)
        {
            ContactGroupScope s = scope.getContactGroups();
            if(s != null)
            {
                s.setParent(null);
            }
            
            ContactGroupFixture.compare(p1.getContactGroups(), p2.getContactGroups(), s);
        }
        
        if(scope.getParent() != null)
        {
            ContactGroupScope s = (ContactGroupScope)scope.getParent();
            if (s != null)
            {
                s.setContactGroups(null);
            }

            ContactGroupFixture.compare((ContactGroup) p1.getParent(),
                    (ContactGroup) p2.getParent(), s);
        }
    }

    public static void compare(ContactGroup[] pArray1,
            ContactGroup[] pArray2, ContactGroupScope scope)
            throws AssertionFailedError, Exception
    {
    	sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(pArray1, pArray2);
        if(pArray1 == null) return;
        
        for(int i = 0; i < pArray1.length; i++)
        {
            ContactGroupFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }
    
    public static ContactGroupScope createScope(int level, int maxLevels){
    	ContactGroupScope scope = new ContactGroupScope();
    	if(level<maxLevels){
    		level++;
    		ContactGroupScope childScope = createScope(level,maxLevels);
    		childScope.setParent(scope);
    		scope.setContactGroups(childScope);
    	}
    	return scope;
    }
    
    //TODO move elsewhere
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }
    

}
