package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.CurrencyExchangeRange;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author karim
 */
public class CurrencyExchangeRangeFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;
    static
    {
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setStartDate");
        localSkipSetters.add("setFinishDate");

        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getStartDate");
        localSkipGetters.add("getFinishDate");
        localSkipGetters.add("getName");

    }

    public static CurrencyExchangeRange createFixture(ClientTestController ctc)
            throws Exception
    {
        Currency parentCurrency = CurrencyFixture.createFixture(true, 0, ctc);
        return createFixture(parentCurrency);
    }

    public static CurrencyExchangeRange createFixture(Currency parentCurrency)
            throws Exception
    {
        return createCurrencyExchangeRange(random(), parentCurrency);
    }

    private static CurrencyExchangeRange createCurrencyExchangeRange(int index,
            Currency parentCurrency) throws Exception
    {
        CurrencyExchangeRange currencyExchangeRange = new CurrencyExchangeRange();
        HelperTestUtilities.setByIntrospection(currencyExchangeRange, index,
                localSkipSetters);
        currencyExchangeRange.setParent(parentCurrency);

        //update the currency rate with the new generated rate
        parentCurrency.setExchangeRate(currencyExchangeRange.getExchangeRate());

        return currencyExchangeRange;
    }

    public static void updateFixture(CurrencyExchangeRange currencyExchangeRange)
            throws Exception
    {
        updateCurrencyExchangeRange(currencyExchangeRange, random());
    }

    private static void updateCurrencyExchangeRange(
            CurrencyExchangeRange currencyExchangeRange, int index)
            throws Exception
    {
        HelperTestUtilities.setByIntrospection(currencyExchangeRange, index,
                localSkipSetters);

        //update the currency rate with the new generated rate
        Currency parentCurrency = (Currency) currencyExchangeRange.getParent();
        if (parentCurrency != null)
        {
            parentCurrency.setExchangeRate(currencyExchangeRange
                    .getExchangeRate());
        }
    }

    public static void compare(CurrencyExchangeRange range1,
            CurrencyExchangeRange range2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compareWithSkip(range1, range2, localSkipGetters);
        Assert
                .assertTrue((range1.getStartDate() != null && range2
                        .getStartDate() != null)
                        || (range1.getStartDate() == null && range2
                                .getStartDate() == null));
        if (range1.getStartDate() != null)
        {
            Assert.assertEquals(range1.getStartDate().get(Calendar.YEAR),
                    range2.getStartDate().get(Calendar.YEAR));
            Assert.assertEquals(range1.getStartDate().get(Calendar.MONTH),
                    range2.getStartDate().get(Calendar.MONTH));
        }
    }

    public static void compare(CurrencyExchangeRange[] pArray1,
            CurrencyExchangeRange[] pArray2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            if (i == 0)
            {
                RPMObjectFixture.compareWithSkip(pArray1[i], pArray2[i],
                        localSkipGetters);
            }
            else
            {
                compare(pArray1[i], pArray2[i]);
            }
        }
    }

    public static int random()
    {
        return RPMObjectFixture.random()%20;
    }
}