/*
 * Created on Dec 13, 2005
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AdministrationModuleScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AdministrationModuleFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;

/**
 * @author Priti
 *
 */
public class TestAdministrationModule extends ModuleTestController
{
	public TestAdministrationModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = AdministrationModuleFixture.createFullScope();
        moduleContaineName = "ADMINISTRATIONMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {
    	AdministrationModule module = (AdministrationModule) aModule;
    	AdministrationModuleScope scope = (AdministrationModuleScope) aScope;

        AdministrationModule loadedModule = new AdministrationModule();

        if(scope!=null)
        {
            if(scope.isCurrencies())
            {
                Currency[] currency = (Currency[]) loadElements(
                        module.getCurrencies(),
                        null,
        				Currency.class);
                loadedModule.setCurrencies(currency);                
            }
            if(scope.getRpmCalendars()!=null)
            {
                RPMCalendar[] rpmCalendar= (RPMCalendar[]) loadElements(
                        module.getRpmCalendars(),
                        scope.getRpmCalendars(),
        				RPMCalendar.class);
                loadedModule.setRpmCalendars(rpmCalendar);                
            }
            if(scope.getOrganizationals()!=null)
            {
                Organizational[] organizational = (Organizational[])loadElements(
                        module.getOrganizationals(),
                        scope.getOrganizationals(),
        				Organizational.class);
                loadedModule.setOrganizationals(organizational);                
            }
            if(scope.getGeographicals()!=null)
            {
                Geographical[] geographical =  (Geographical[])loadElements(
                        module.getGeographicals(),
                        scope.getGeographicals(),
        				Geographical.class);
                loadedModule.setGeographicals(geographical);                
            }
            if(scope.isDatafieldCategories())
            {
                DatafieldCategory[] datafieldCategories = loadDatafieldCategories(
                        module.getDatafieldCategories(), scope.isDatafieldCategories(),
                        DatafieldCategory.class);
                module.setDatafieldCategories(datafieldCategories);
            }
        }

        return loadedModule;
    }

    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {
    	AdministrationModule module = (AdministrationModule) aModule;
    	AdministrationModuleScope scope = (AdministrationModuleScope) aScope;
    	
        //delete the element
        if (module != null && scope != null)
        {
        	if(module.getRpmCalendars()!=null && scope.getRpmCalendars()!=null)
        	{
        			delete(module.getRpmCalendars());
        	}
        	if(module.getOrganizationals()!=null && scope.getOrganizationals()!=null)
        	{
        			delete(module.getOrganizationals());
        	}
        	if(module.getGeographicals()!=null && scope.getGeographicals()!=null)
        	{
        			delete(module.getGeographicals());
        	}
        	
            //reload the deleted element
            AdministrationModule deletedModule = (AdministrationModule) loadDeletedModule(module, scope);

            //Validate the deleted element
        	if(scope.getRpmCalendars()!=null)
        	{
                assertNull(deletedModule.getRpmCalendars());
        	}
        	if(scope.getOrganizationals()!=null)
        	{
        	    assertNull(deletedModule.getOrganizationals());
        	}
        	if(scope.getGeographicals()!=null)
        	{
        	    assertNull(deletedModule.getGeographicals());
        	}
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return AdministrationModuleFixture.createFixture((AdministrationModuleScope) scope, this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return AdministrationModuleFixture.updateFixture((AdministrationModule) module,
            (AdministrationModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        AdministrationModuleFixture.compare((AdministrationModule) module,
            (AdministrationModule) returnedModule, (AdministrationModuleScope) scope);
    }
}
