/*
 * Created on Sep 12, 2006
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestRtfAssignment extends ClientTestController
{
    RPMObject parent = null;

    RPMObjectScope parentScope = null;

    public TestRtfAssignment()
    {
        //Default values
        parent = new Proposal();
    }

    public TestRtfAssignment(String testName, RPMObject parent)
    {
        setName(testName);
        this.parent = parent;
    }

    public void setUp() throws Exception
    {
        super.setUp();

        parentScope = RtfAssignmentFixture.createParentScope(parent);
        parent = RtfAssignmentFixture.createParent(parent, parentScope, this);
        
        // save the parent but not the rtf assignments
        SingleResult result = save(parent, null);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject().getClass().equals(parent.getClass()));
        RPMObject savedParent = (RPMObject) result.getRpmObject();
        parent.setID(savedParent.getID());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(true);
    }

    private void insert(boolean loadFromId) throws AssertionError, Exception
    {
        //insert an element
        parent = insertRtfAssignment(loadFromId);
    }

    private RPMObject insertRtfAssignment(boolean loadFromId) throws Exception
    {
        //Insert the rtfAssignments into the DB individually, with parent set to null
        RtfAssignment[] assignments = RtfAssignmentFixture.getRtfAssignmentsFromParent(parent);
        assertNotNull(assignments);
        for (int i = 0; assignments != null && i < assignments.length; ++i)
        {
            SingleResult result = save(assignments[i], null);
            
            // Compare the rtfAssignment with the one that was returned
            RtfAssignment returnedAssignment = (RtfAssignment) result.getRpmObject();
            RtfAssignmentFixture.compare(assignments[i], returnedAssignment);
            
            // Reload the rtfAssignment with only primary key and compare
            returnedAssignment = (RtfAssignment) load(returnedAssignment, null, loadFromId);
            RtfAssignmentFixture.compare(assignments[i], returnedAssignment);
        }
        
        //reload the parent and compare it
        RPMObject returnedParent = load(parent, parentScope, loadFromId);
        RtfAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //return the parent
        return returnedParent;
    }

    private void update(boolean loadFromId) throws AssertionError, Exception
    {
        //insert an element
        parent = insertRtfAssignment(loadFromId);

        //update the element
        parent = updateRtfAssignmentFields(loadFromId);
    }

    private RPMObject updateRtfAssignmentFields(boolean loadFromId)
            throws Exception
    {
        return updateRtfAssignment(loadFromId, false);
    }

    private RPMObject updateRtfAssignment(boolean loadFromId,
            boolean updateStructure) throws Exception
    {
        RtfAssignmentFixture.updateRtfAssignmentsFieldsFromParent(parent);

        //Insert the rtfAssignments into the DB individually, with parent set to null
        RtfAssignment[] assignments = RtfAssignmentFixture.getRtfAssignmentsFromParent(parent);
        for (int i = 0; assignments != null && i < assignments.length; ++i)
        {
            SingleResult result = save(assignments[i], null);
            
            // Compare the rtfAssignment with the one that was returned
            RtfAssignment returnedAssignment = (RtfAssignment) result.getRpmObject();
            RtfAssignmentFixture.compare(assignments[i], returnedAssignment);
            
            // Reload the rtfAssignment with only primary key and compare
            returnedAssignment = (RtfAssignment) load(returnedAssignment, null, loadFromId);
            RtfAssignmentFixture.compare(assignments[i], returnedAssignment);
        }

        //reload the element and compare it
        RPMObject returnedParent = load(parent, parentScope, loadFromId);
        RtfAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //return the parent
        return returnedParent;
    }

    private RPMObject load(RPMObject parent, RPMObjectScope parentScope,
            boolean loadFromId) throws Exception
    {
        RPMObject obj = null;

        if (loadFromId || parent instanceof RtfAssignment) // can't do a load by xpath on rtfAssignment
        {
            obj = (RPMObject) parent.getClass().newInstance();
            obj.setID(parent.getID());
            
            if (parent instanceof RtfAssignment)
            {
                // Set additional primary key field 
                ((RtfAssignment) obj).setRtf((DatafieldRTF)((RtfAssignment) parent).getRtf().getClass().newInstance());
                ((RtfAssignment) obj).getRtf().setID(((RtfAssignment) parent).getRtf().getID());
            }
            
            return loadFromID(obj, parentScope);
        }
        else
        {
            //load from an xpath
            String containerName = StringUtil.getShortClassName(parent.getClass());
            RPMObject[] objs = loadFromXpath("/" + containerName + "[id='"
                    + parent.getID() + "']", parentScope);
            if (objs == null || objs.length == 0)
            {
                return null;
            }
            else
            {
                return objs[0];
            }
        }
    }
}
