/*
 * Created on Sep 26, 2006
 */
package com.ibm.rpm.client.junit.applicationadministration.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author dominiqu
 */
public class Test55990 extends ClientTestController
{
    WorkElement wbsParent = null;

    WorkElementScope wbsParentScope = null;
    
    ScopeElement scopeParent = null;

    ScopeElementScope scopeParentScope = null;

    ScopeElement scopeParentParent = null;

    int NB_CHILDREN = 2;

    // Before transfer - After transfer
    // All except below - SummaryTask - Same as before transfer
    // ScopeDeliverable - Deliverable - Same DeliverableRTF
    // ScopeTask - Task - Same TaskRTF
    // ScopeMilestone - Milestone - Same MilestoneRTF

    public void setUp() throws Exception
    {
        super.setUp();

        scopeParentScope = new ScopeElementScope();
        scopeParentScope.setRtfAssignments(true);

        wbsParentScope = new WorkElementScope();
        wbsParentScope.setRtfAssignments(true);

        scopeParentParent = ScopeElementFixture.createSaveChangeRequest(this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testTransferredActionRtfAssignmentValidation() throws Exception
    {
        scopeParent = ScopeElementFixture.createActionFixture(scopeParentScope,
            NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof ActionRTF);
    }

    public void testTransferredChangeRequestRtfAssignmentValidation()
            throws Exception
    {
        scopeParent = ScopeElementFixture.createChangeRequestFixture(
            scopeParentScope, NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof ChangeRequestRTF);
    }

    public void testTransferredDefectRtfAssignmentValidation() throws Exception
    {
        scopeParent = ScopeElementFixture.createDefectFixture(scopeParentScope,
            NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof DefectRTF);
    }

    public void testTransferredIssueRtfAssignmentValidation() throws Exception
    {
        scopeParent = ScopeElementFixture.createIssueFixture(scopeParentScope,
            NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof IssueRTF);
    }

    public void testTransferredRequirementRtfAssignmentValidation()
            throws Exception
    {
        scopeParent = ScopeElementFixture.createRequirementFixture(
            scopeParentScope, NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof RequirementRTF);
    }

    public void testTransferredRiskRtfAssignmentValidation() throws Exception
    {
        scopeParent = ScopeElementFixture.createRiskFixture(
            scopeParentScope, NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof RiskRTF);
    }

    public void testTransferredServiceRequestRtfAssignmentValidation()
            throws Exception
    {
        scopeParent = ScopeElementFixture.createServiceRequestFixture(
            scopeParentScope, NB_CHILDREN, scopeParentParent, this);

        insertAndUpdate();

        assertTrue(wbsParent.getRtfAssignments()[0].getRtf() instanceof ServiceRequestRTF);
    }

    public void insertAndUpdate() throws Exception
    {
        SingleResult result = save(scopeParent, scopeParentScope);

        scopeParent = (ScopeElement) result.getRpmObject();
        scopeParent.setParent(scopeParentParent);

        Result result2 = getApplicationInterface().transferUnderWBS(
            getSessionID(), null,
            (AbstractScope) scopeParent.getParent().getParent());

        assertNull(result2.getErrors());

        wbsParent = new SummaryTask();
        wbsParent.setID(scopeParent.getID());
        wbsParent = (WorkElement) loadFromID(wbsParent, wbsParentScope);

        assertNotNull(scopeParent.getClass().getName()
                + " RtfAssignments were not loaded after transfer under WBS",
            wbsParent.getRtfAssignments());
        assertNotNull(scopeParent.getClass().getName()
                + " RtfAssignments were not loaded after transfer under WBS",
            wbsParent.getRtfAssignments()[0]);

        // Update the fixture, including new rtf assignments, and save
        RtfAssignment[] rtfAssignments = RtfAssignmentFixture.createAllAssignmentFixtures(
            wbsParent, this);
        wbsParent.setRtfAssignments(rtfAssignments);

        SingleResult result3 = save(wbsParent, wbsParentScope);

        assertNull(result3.getErrors());

        wbsParent = (WorkElement) result3.getRpmObject();

        assertNotNull(wbsParent);
        assertNotNull(wbsParent.getRtfAssignments());
        assertNotNull(wbsParent.getRtfAssignments()[0]);
    }
}
