package com.ibm.rpm.client.junit.applicationadministration.testcases.fixes;

import org.omg.PortableInterceptor.SUCCESSFUL;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00257017 extends ClientTestController {
	/* This Case Tests if the SoftDeleted Attribute is loaded with the AttributeAssignment
	 * for the Task
	 */
	public void testLoadAttributeAssignmentWithSoftDeletedAttribute() throws Exception
	{
		WorkElementScope wScope = new WorkElementScope();
		wScope.setAttributeAssignments(true);
			
		WorkElement wElement = WorkElementFixture.createSaveFixture(new Task(),wScope , this);
		assertNotNull(wElement);
		assertNotNull(wElement.getAttributeAssignments());
		assertTrue(wElement.getAttributeAssignments().length > 0);
		
		String weID = wElement.getID();
		assertNotNull(weID);
		assertTrue(weID.length() > 0);
		
		GenericAttribute attribute = wElement.getAttributeAssignments()[0].getAttribute();
		String attributeID = attribute.getID();
		assertNotNull(attribute);
		
		Result result = delete(attribute);
		assertTrue(result.isSuccessful());
		
		Task reloadedTask = new Task();
		reloadedTask.setID(weID);
		reloadedTask = (Task)loadFromID(reloadedTask, wScope);
		assertNotNull(reloadedTask);
		assertNotNull(reloadedTask.getAttributeAssignments());
		assertTrue(reloadedTask.getAttributeAssignments().length > 0);
		assertNotNull(reloadedTask.getAttributeAssignments()[0].getAttribute());
		assertTrue(attributeID.equals(reloadedTask.getAttributeAssignments()[0].getAttribute().getID()));
	}
	/* If the AttributeAssignment is deleted it should not be loaded while loading the Task.
	 */
	public void testLoadAttributeAssignmentWithSoftDeletedAttributeAssignment() throws Exception
	{
		WorkElementScope wScope = new WorkElementScope();
		wScope.setAttributeAssignments(true);
		
		WorkElement wElement = WorkElementFixture.createSaveFixture(new Task(),wScope , this);
		assertNotNull(wElement);
		assertNotNull(wElement.getAttributeAssignments());
		assertTrue(wElement.getAttributeAssignments().length > 0);
		
		String weID = wElement.getID();
		assertNotNull(weID);
		assertTrue(weID.length() > 0);
		
		AttributeAssignment assignment = wElement.getAttributeAssignments()[0];
					
		Result result = delete(assignment);
		assertTrue(result.isSuccessful());
		
		Task reloadedTask = new Task();
		reloadedTask.setID(weID);
		reloadedTask = (Task)loadFromID(reloadedTask, wScope);
		assertNotNull(reloadedTask);
		assertNull(reloadedTask.getAttributeAssignments());
	}
	/* This Case Tests if the SoftDeleted Attribute is loaded with the AttributeAssignment
	 * for the Issue
	 */
	public void testLoadAttributeAssignmentWithSoftDeletedAttributeWithIssue() throws Exception
	{
		WorkElementScope wScope = new WorkElementScope();
		wScope.setScopeElement(new ScopeElementScope());
		
		WorkElement wElement = WorkElementFixture.createSaveFixture(new Proposal(),wScope , this);
		assertNotNull(wElement);
		assertNotNull(wElement.getScopeElement());
					
		ScopeElementScope sScope = new ScopeElementScope();
		sScope.setAttributeAssignments(true);
		
		AbstractScope sElement = ScopeElementFixture.createIssueFixture(sScope, 1, wElement.getScopeElement(), this);
		
		SingleResult sr = save(sElement, sScope);
		sElement = (AbstractScope)sr.getRpmObject();
		assertNotNull(sElement);
		assertNotNull(sElement.getAttributeAssignments());
		assertTrue(sElement.getAttributeAssignments().length > 0);
		
		String seID = sElement.getID();
		assertNotNull(seID);
		assertTrue(seID.length() > 0);
		
		GenericAttribute attribute = sElement.getAttributeAssignments()[0].getAttribute();
		String attributeID = attribute.getID();
		assertNotNull(attribute);
		
		Result result = delete(attribute);
		assertTrue(result.isSuccessful());
		
		Issue reloadedIssue = new Issue();
		reloadedIssue.setID(seID);
		reloadedIssue = (Issue)loadFromID(reloadedIssue, sScope);
		assertNotNull(reloadedIssue);
		assertNotNull(reloadedIssue.getAttributeAssignments());
		assertTrue(reloadedIssue.getAttributeAssignments().length > 0);
		assertNotNull(reloadedIssue.getAttributeAssignments()[0].getAttribute());
		assertTrue(attributeID.equals(reloadedIssue.getAttributeAssignments()[0].getAttribute().getID()));
	}
	public void testLoadDeletedAttributeByID() throws Exception
	{
		AttributeCategory aCategory = AttributeFixture.createSaveFixture("WBS", this);
		assertNotNull(aCategory);
		assertNotNull(aCategory.getClassifications());
		assertTrue(aCategory.getClassifications().length > 0);
		assertNotNull(aCategory.getClassifications()[0]);
		assertNotNull(aCategory.getClassifications()[0].getAttributes());
		assertTrue(aCategory.getClassifications()[0].getAttributes().length > 0);
		assertNotNull(aCategory.getClassifications()[0].getAttributes()[0]);
		
		Attribute attribute = aCategory.getClassifications()[0].getAttributes()[0];
		
		String attributeID = attribute.getID();
		assertNotNull(attributeID);
		assertTrue(attributeID.length() > 0);
		
		Result result = delete(attribute);
		assertTrue(result.isSuccessful());
		
		RPMObject reloadedAttribute = new Attribute();
		reloadedAttribute.setID(attributeID);
		boolean exceptionOccurred = false;
		try{
			reloadedAttribute = loadFromID(reloadedAttribute, new AttributeScope());
		}
		catch(RPMException ex)
		{
			exceptionOccurred = true;
			assertTrue("Attribute".equals(ex.getClassName()));
			assertTrue(attributeID.equals(ex.getContainerID()));
		}
		assertTrue(exceptionOccurred);
	}
	public void testLoadDeletedAttributeByXPATH() throws Exception
	{
		AttributeCategory aCategory = AttributeFixture.createSaveFixture("WBS", this);
		assertNotNull(aCategory);
		assertNotNull(aCategory.getClassifications());
		assertTrue(aCategory.getClassifications().length > 0);
		assertNotNull(aCategory.getClassifications()[0]);
		assertNotNull(aCategory.getClassifications()[0].getAttributes());
		assertTrue(aCategory.getClassifications()[0].getAttributes().length > 0);
		assertNotNull(aCategory.getClassifications()[0].getAttributes()[0]);
		
		Attribute attribute = aCategory.getClassifications()[0].getAttributes()[0];
		
		String attributeID = attribute.getID();
		String attributeName = attribute.getName();
		assertNotNull(attributeID);
		assertTrue(attributeID.length() > 0);
		
		RPMObject[] reloadedAttributes = loadFromXpath("/Attribute[@name='"+attributeName+"']", new AttributeScope());
		assertNotNull(reloadedAttributes);
		assertNotNull(reloadedAttributes[0]);
		assertTrue(attributeID.equals(reloadedAttributes[0].getID()));
				
		Result result = delete(attribute);
		assertTrue(result.isSuccessful());
		
		reloadedAttributes = loadFromXpath("/Attribute[@name='"+attributeName+"']", new AttributeScope());
		assertNull(reloadedAttributes);
	}
}
