/*
 * Created on 12-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestContactGroupValidation extends GenericValidation
{
    private ContactGroup mainContactGroup;

    protected void setUp() throws Exception
    {
        super.setUp();
        mainContactGroup = new ContactGroup();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainContactGroup.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainContactGroup.getClass().getName(), "parent",
                mainContactGroup.getID())
        };

        verifySaveMustFail(mainContactGroup, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainContactGroup
            .setName(getTestString(ValidationConstants.CONTACTGROUP_NAME_MAX));
        mainContactGroup
            .setDescription(getTestString(ValidationConstants.CONTACTGROUP_DESCRIPTION_MAX));

        // Attempt to save the Resource object with the RtfAssignment
        mainContactGroup = (ContactGroup) saveMustSucceed(mainContactGroup,
            ContactGroup.class, null);

        // Load the inserted education from id
        RPMObject obj = new ContactGroup();
        obj.setID(mainContactGroup.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ContactGroup);

        // Check that the loaded contact group assignments are identical to the
        // ones we requested to store
        ContactGroupFixture.compare(mainContactGroup, ((ContactGroup) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainContactGroup
            .setName(getTestString(ValidationConstants.CONTACTGROUP_NAME_MAX + 1));
        mainContactGroup
            .setDescription(getTestString(ValidationConstants.CONTACTGROUP_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainContactGroup.getClass().getName(), "name",
                mainContactGroup.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainContactGroup.getClass().getName(),
                "description", mainContactGroup.getID())
        };

        verifySaveMustFail(mainContactGroup, null, expectedExceptions);
    }
}
