package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.CurrencyExchangeRange;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.CurrencyExchangeRangeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestCurrencyExchangeValidation extends GenericValidation
{
    private CurrencyExchangeRange mainCurrencyExchange;

    protected void setUp() throws Exception
    {
        super.setUp();
        mainCurrencyExchange = CurrencyExchangeRangeFixture.createFixture(this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainCurrencyExchange.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainCurrencyExchange.getClass()
                .getName(), "parent", mainCurrencyExchange.getID())
        };

        verifySaveMustFail(mainCurrencyExchange, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainCurrencyExchange.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainCurrencyExchange
                .getClass().getName(), "parent", mainCurrencyExchange.getID())
        };

        verifySaveMustFail(mainCurrencyExchange, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainCurrencyExchange.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MAX));

        // Attempt to save the CurrencyExchangeRange object
        mainCurrencyExchange = (CurrencyExchangeRange) saveMustSucceed(
            mainCurrencyExchange, CurrencyExchangeRange.class, null);

        // Load the inserted currency exchange range from xpath
        RPMObject[] rpmObject1 = loadFromXpath("/CurrencyExchangeRange[id='"
            + mainCurrencyExchange.getID() + "']", null);

        // Check that the loaded currency exchange range is identical to the one
        // we requested to store
        CurrencyExchangeRangeFixture.compare(mainCurrencyExchange,
            (CurrencyExchangeRange) rpmObject1[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainCurrencyExchange.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MIN));

        // Attempt to save the CurrencyExchangeRange object
        mainCurrencyExchange = (CurrencyExchangeRange) saveMustSucceed(
            mainCurrencyExchange, CurrencyExchangeRange.class, null);

        // Load the inserted currency exchange range from xpath
        RPMObject[] rpmObject1 = loadFromXpath("/CurrencyExchangeRange[id='"
            + mainCurrencyExchange.getID() + "']", null);

        // Check that the loaded currency exchange range is identical to the one
        // we requested to store
        CurrencyExchangeRangeFixture.compare(mainCurrencyExchange,
            (CurrencyExchangeRange) rpmObject1[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        mainCurrencyExchange.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MAX + 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainCurrencyExchange.getClass()
                .getName(), "exchangeRate", mainCurrencyExchange.getID())
        };

        verifySaveMustFail(mainCurrencyExchange, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min-1 and verify that they are not accepted
        mainCurrencyExchange.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MIN - 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainCurrencyExchange.getClass()
                .getName(), "exchangeRate", mainCurrencyExchange.getID())
        };

        verifySaveMustFail(mainCurrencyExchange, null, expectedExceptions);
    }
}
