/*
 * Created on 12-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.CurrencyFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestCurrencyValidation extends GenericValidation
{
    Currency mainCurrency;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainCurrency = CurrencyFixture.createFixture(false, 0, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainCurrency
            .setName(getTestString(ValidationConstants.CURRENCY_NAME_MAX));
        mainCurrency.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MAX));

        // Attempt to save the Currency object
        mainCurrency = (Currency) saveMustSucceed(mainCurrency, Currency.class,
            null);

        // Load the inserted currency from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Currency[id='"
                + mainCurrency.getID() + "']", null);

        // Check that the loaded currency is identical to the one we requested
        // to store
        CurrencyFixture.compare(mainCurrency, (Currency) rpmObjects[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainCurrency.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MIN));

        // Attempt to save the Currency object
        mainCurrency = (Currency) saveMustSucceed(mainCurrency, Currency.class,
            null);

        // Load the inserted currency from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Currency[id='"
                + mainCurrency.getID() + "']", null);

        // Check that the loaded currency is identical to the one we requested
        // to store
        CurrencyFixture.compare(mainCurrency, (Currency) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainCurrency
            .setName(getTestString(ValidationConstants.CURRENCY_NAME_MAX + 1));
        mainCurrency.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MAX + 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainCurrency
                    .getClass().getName(), "name", mainCurrency.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCurrency
                    .getClass().getName(), "exchangeRate", mainCurrency.getID())
        };

        verifySaveMustFail(mainCurrency, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min-1 and verify that they are not accepted
        mainCurrency.setExchangeRate(new Double(
            ValidationConstants.CURRENCY_EXCHANGERATE_MIN - 0.01));
        mainCurrency.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCurrency
                    .getClass().getName(), "exchangeRate", mainCurrency.getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, mainCurrency
                    .getClass().getName(), "name", mainCurrency.getID())

        };

        verifySaveMustFail(mainCurrency, null, expectedExceptions);
    }

    public void testFailureMandatoryFields() throws AssertionError, Exception
    {
        // name is mandatory when exchangeRate exists
        mainCurrency.setExchangeRate(new Double(2.0));
        mainCurrency.setName(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, mainCurrency.getClass()
                .getName(), "name", mainCurrency.getID())
        };

        verifySaveMustFail(mainCurrency, null, expectedExceptions);
    }
}
