/*
 * Created on 12-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestDatafieldValidation extends GenericValidation
{
    public static String RISKSTATE_1 = "4830";

    public static String RISKRTF_1 = "5690";

    private CompanySize testDatafield;

    private RiskState testDatafieldState;

    private RiskRTF testDatafieldRTF;

    protected void setUp() throws Exception
    {
        super.setUp();

        testDatafield = new CompanySize();
        testDatafield.setID(DatafieldFixture.COMPANY_SIZE_FORTUNE_100);
        testDatafieldState = new RiskState();
        testDatafieldState.setID(RISKSTATE_1);
        testDatafieldState.setClosedState(Boolean.FALSE);
        testDatafieldRTF = new RiskRTF();
        testDatafieldRTF.setID(RISKRTF_1);
        testDatafieldRTF.setAppendOnly(Boolean.FALSE);
    }

    protected void tearDown() throws Exception
    {
        // Update the datafields back to their original values
        testDatafield.setValue("Fortune 100");
        testDatafieldState.setValue("Open");
        testDatafieldRTF.setValue("Risk Description");

        save(testDatafield, null);
        save(testDatafieldState, null);
        save(testDatafieldRTF, null);

        super.tearDown();
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        testDatafield.setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX));
        testDatafieldState
            .setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX));
        testDatafieldRTF.setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX));

        // Attempt to save the datafields
        testDatafield = (CompanySize) saveMustSucceed(testDatafield, CompanySize.class,
            null);
        testDatafieldState = (RiskState) saveMustSucceed(testDatafieldState,
            RiskState.class, null);
        testDatafieldRTF = (RiskRTF) saveMustSucceed(testDatafieldRTF, RiskRTF.class,
            null);

        // Load the inserted datafields from id
        RPMObject rpmDatafieldObject = loadFromID(testDatafield, null);
        RPMObject rpmDatafieldStateObject = loadFromID(testDatafieldState, null);
        RPMObject rpmDatafieldRTFObject = loadFromID(testDatafieldRTF, null);

        // Check that the loaded datafields are identical to the ones we
        // requested to store
        DatafieldFixture.compare(testDatafield, (CompanySize) rpmDatafieldObject);
        DatafieldFixture.compare(testDatafieldState, (RiskState) rpmDatafieldStateObject);
        DatafieldFixture.compare(testDatafieldRTF, (RiskRTF) rpmDatafieldRTFObject);
    }

    public void testFailureDatafieldValidation() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        testDatafield
            .setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testDatafield.getClass().getName(),
                "value", testDatafield.getID())
        };

        verifySaveMustFail(testDatafield, null, expectedExceptions);
    }

    public void testFailureDatafieldStateValidation() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        testDatafieldState
            .setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testDatafieldState.getClass()
                .getName(), "value", testDatafieldState.getID())
        };

        verifySaveMustFail(testDatafieldState, null, expectedExceptions);
    }

    public void testFailureDatafieldRTFValidation() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        testDatafieldRTF
            .setValue(getTestString(ValidationConstants.DATAFIELD_VALUE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testDatafieldRTF.getClass()
                .getName(), "value", testDatafieldRTF.getID())
        };

        verifySaveMustFail(testDatafieldRTF, null, expectedExceptions);
    }
}
