/*
 * Created on Nov 29, 2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ChargeType;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.asset.scope.CrossChargeFinancialsScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.AbstractCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;

/**
 * @author adler
 *
 * A Fixture that simulates a datagraph of Cross Charge Financials objects
 */
public class AssetCrossChargeFinancialsFixture extends RPMObjectFixture{

    protected static ArrayList crossChargeFinancialsSkipSetters, crossChargeFinancialsSkipGetters;

    static
    {
    	crossChargeFinancialsSkipSetters = new ArrayList();
    	crossChargeFinancialsSkipSetters.addAll(skipSetters);
    	crossChargeFinancialsSkipSetters.add("setStartDate");
    	crossChargeFinancialsSkipSetters.add("setFinishDate");
    	crossChargeFinancialsSkipSetters.add("setChargeType");

    	crossChargeFinancialsSkipGetters = new ArrayList();
    	crossChargeFinancialsSkipGetters.addAll(skipGetters);
    	crossChargeFinancialsSkipGetters.add("getStartDate");
    	crossChargeFinancialsSkipGetters.add("getFinishDate");
    	crossChargeFinancialsSkipSetters.add("getChargeType");
    }

    /**
     * createFixture - Fixture for creating an CrossChargeFinancials objects
     *
     * @param Asset
     *            the parent of created object
     * @param projectOrClient
     *            Array of objects to be charged
     * @return Created CrossChargeFinancials objects
     * @throws Exception
     *
     */
    public static CrossChargeFinancials[] createFixture(Asset asset, AbstractCostCenter[] projectOrClient,int size)
            throws Exception
    {
    	// One CrossChargeFinancials for each charged objects (ProjectCostCenter, ClientCostCenter)
    	CrossChargeFinancials[] crossChargeFinancials = new CrossChargeFinancials[size];
        int index;
        for (int i = 0; i < size; i++)
        {
            index = getNextInt();
        	if(asset != null && projectOrClient !=null)
        	{
        		crossChargeFinancials[i] = createCrossChargeFinancial(index);
        		// Set the asset as parent
        		crossChargeFinancials[i].setParent(asset);
        		// Set the Charged
        		crossChargeFinancials[i].setCostCenter(projectOrClient[i]);
        	}
        }

        return crossChargeFinancials;
    }  // createFixture

    /**
     * updateFixture - Fixture for updating an CrossChargeFinancials objects
     *
     * @param Asset
     *            the parent of created object
     * @param projectOrClient
     *            Array of objects to be charged
     * @return Created CrossChargeFinancials objects
     * @throws Exception
     *
     */
    public static CrossChargeFinancials[] updateFixture(Asset asset, AbstractCostCenter[] projectOrClient,int size)
            throws Exception
    {
    	// One CrossChargeFinancials for each charged objects (ProjectCostCenter, ClientCostCenter)

        CrossChargeFinancials[] existingCrossChargeFinancials = asset.getCrossChargeFinancials();


        if(existingCrossChargeFinancials!=null)
            size += existingCrossChargeFinancials.length;

        CrossChargeFinancials[] crossChargeFinancials = new CrossChargeFinancials[size];;

        int i=0;
        for (;existingCrossChargeFinancials!=null&&i < existingCrossChargeFinancials.length; i++)
        {
            crossChargeFinancials[i]=existingCrossChargeFinancials[i];
            crossChargeFinancials[i].setDeleted(Boolean.TRUE);
        }

        int index;
        for (; i < crossChargeFinancials.length; i++)
        {
            index = getNextInt();
        	if(asset != null && projectOrClient !=null)
        	{
        		crossChargeFinancials[i] = createCrossChargeFinancial(index);
        		// Set the asset as parent
        		crossChargeFinancials[i].setParent(asset);
        		// Set the Charged
        		crossChargeFinancials[i].setCostCenter(projectOrClient[existingCrossChargeFinancials==null?i:i-existingCrossChargeFinancials.length]);
        	}
        }

        return crossChargeFinancials;
    }  // createFixture


    /**
     * createCrossChargeFinancial - Create a CrossChargeFinancials object with random id
     * number, predeined Start and Finish dates, and charge types
     *
     * @param index
     *            the random id number
     * @return Created CrossChargeFinancials object
     * @throws Exception
     *
     */
    private static CrossChargeFinancials createCrossChargeFinancial(int index)
            throws Exception
    {
    	CrossChargeFinancials crossChargeFinancial = new CrossChargeFinancials();
        HelperTestUtilities.setByIntrospection(crossChargeFinancial, index,
        		crossChargeFinancialsSkipSetters);
        // Start Date date
        crossChargeFinancial.setStartDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2005-09-06")));
        // Finish Date date
        crossChargeFinancial.setFinishDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2005-12-06")));
        //ChargeType
        ChargeType chargeType = DatafieldFixture.createChargeType(DatafieldFixture.ASSET_CHARGE_TYPE_NONE);
        crossChargeFinancial.setChargeType(chargeType);


        return crossChargeFinancial;
    } // createCrossChargeFinancial

    /**
     * compare - Compare two CrossChargeFinancials objects
     *
     * @param o1
     *            the first CrossChargeFinancials object to be compared
     * @param o2
     *            the second CrossChargeFinancials object to be compared
     * @param CrossChargeFinancialsScope
     *            the compared scope
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *
     */
    public static void compare(CrossChargeFinancials o1,
    		CrossChargeFinancials o2, CrossChargeFinancialsScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(o1, o2, crossChargeFinancialsSkipGetters);

        validateCalendarTypes(o1.getStartDate(), o2.getStartDate());
        validateCalendarTypes(o1.getFinishDate(), o2.getFinishDate());
        // Compare the assets
        if ((scope != null) && (o2.getParent() != null))
            AssetFixture.compare((Asset) o1.getParent(),
                    (Asset) o2.getParent(), (AssetScope)scope.getParent());
        // Compare the charged objects
        if ((scope != null) && (o2.getCostCenter() != null))
        {
            if (o2.getCostCenter() instanceof ClientCostCenter)
            	ClientCostCenterFixture.compare((ClientCostCenter)o1.getCostCenter(),
            			(ClientCostCenter)o2.getCostCenter(), (ClientCostCenterScope)scope.getCostCenter());
            else
            	// TODO implement ProjectCostCenter
            	Assert.fail("Need to implement ProjectCostCenter");
        }
    }

    /**
     * compare - Compare two CrossChargeFinancials array objects
     *
     * @param oArray1
     *            the first CrossChargeFinancials array objects to be compared
     * @param oArray2
     *            thednoce CrossChargeFinancials array objects to be compared
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *
     */
    public static void compare(CrossChargeFinancials[] oArray1,
    		CrossChargeFinancials[] oArray2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(oArray1, oArray2);

        if (oArray1 == null)
            return;

        Arrays.sort(oArray1, RPMObjectIDComparator.INSTANCE);
        Arrays.sort(oArray2, RPMObjectIDComparator.INSTANCE);

        for (int i = 0; i < oArray1.length; i++)
        {
            compare(oArray1[i], oArray2[i]);
        }
    }

    /**
     * validateCalendarTypes - Validate two Calendar objects
     *
     * @param c1
     *            the first Calendar to validate
     * @param c2
     *            the second Calendar to validate
     * @return None
     * @throws AssertionError &
     *             Exception
     *
     */
    private static void validateCalendarTypes(Calendar c1, Calendar c2)
            throws AssertionError, Exception
    {
        if (c1 != null && c2 != null)
        {
            Assert.assertEquals(c1.get(Calendar.YEAR), c2.get(Calendar.YEAR));
            Assert.assertEquals(c1.get(Calendar.DAY_OF_MONTH), c2
                    .get(Calendar.DAY_OF_MONTH));
            Assert.assertEquals(c1.get(Calendar.MONTH), c2.get(Calendar.MONTH));
        }
        if ((c1 == null && c2 != null) || (c1 != null && c2 == null))
        {
            Assert.fail();
        }
    }    private static int INDEX = 1;

    private static int getNextInt()
    {
        return INDEX++;
    }
}
