/*
 * Created on 22/08/2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author rosman
 *  
 */
public class AssetFolderFixture extends RPMObjectFixture
{
    protected static ArrayList folderSkipSetters, folderSkipGetters;

    static
    {
        //Setters
        folderSkipSetters = new ArrayList();
        folderSkipSetters.addAll(skipSetters);
        folderSkipSetters.add("setDescription");

        //Getters
        folderSkipGetters = new ArrayList();
        folderSkipGetters.addAll(skipGetters);
        folderSkipGetters.add("getDescription");

    }

    public static AssetFolder createSaveFixture(ClientTestController ctc,
            AssetScope scope) throws Exception
    {

        AssetFolder assetFolder = createFixture(false, 0, scope, ctc);
        if (scope == null)
        {
            scope = new AssetScope();
        }
        assetFolder = (AssetFolder) save(ctc, assetFolder, scope);

        return assetFolder;

    }

    /**
     * @param createParent
     * @param nbChildren
     * @param nbLayers
     * @param scope
     * @throws Exception
     */
    public static AssetFolder createFixture(boolean createParent,
            int nbChildren, AssetScope scope, ClientTestController ctc)
            throws Exception
    {
        AssetFolder mainFolder = new AssetFolder();
        HelperTestUtilities.setByIntrospection(mainFolder, random(), folderSkipSetters);

        if (scope != null)
        {
            // Check if we need to create a parent
            if (scope.getParent() != null && createParent)
            {
                AssetFolder parentFolder = createFixture(false, 0, (AssetScope)scope
                        .getParent(), ctc);
                parentFolder.setFolders(new AssetFolder[]
                {
                    mainFolder
                });
                mainFolder.setParent(parentFolder);
            }

            if (scope.getFolders() != null) // TODO: check this
            {

                AssetFolder[] childrenFolders = new AssetFolder[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    AssetFolder f = createFixture(false, nbChildren, scope
                            .getFolders(), ctc);
                    f.setParent(mainFolder);
                    childrenFolders[i] = f;
                }

                //Insert a null object
                childrenFolders[nbChildren] = null;

                mainFolder.setFolders(childrenFolders);
            }

            if (scope.getAssets() != null)
            {
                AssetScope aScope = scope.getAssets();
                aScope.setParent(null);

                Asset[] aArray = new Asset[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    Asset a = AssetFixture.createFixture(mainFolder, aScope,
                            ctc);
                    aArray[i] = a;
                }

                //Insert a null object
                aArray[nbChildren] = null;

                mainFolder.setAssets(aArray);
            }

        }

        return mainFolder;
    }

    public static AssetFolder updateFixture(AssetFolder folder, int nbChildren,
            ClientTestController ctc) throws Exception
    {
        AssetFolder updFolder = createFixture(false, 0, null, ctc); // TODO:
                                                                    // check it
        updFolder.setID(folder.getID());
        if (folder.getParent() != null)
        {
            updFolder.setParent(folder.getParent());
        }
        if (folder.getAssets() != null)
        {

            GenericAsset[] assets = folder.getAssets();
            GenericAsset[] rArray = new Asset[nbChildren * 2];
            for (int i = 0; i < nbChildren; i++)
            {
                //delete existing resource
                assets[i].setDeleted(Boolean.TRUE);
                rArray[i] = assets[i];

                //Add new resources
                Asset r = AssetFixture.createFixture(folder, null, ctc);
                rArray[nbChildren + i] = r;
            }

            updFolder.setAssets(rArray);
        }
        if (folder.getFolders() != null)
        {

            AssetFolder[] assets = folder.getFolders();
            AssetFolder[] rArray = new AssetFolder[nbChildren * 2];
            for (int i = 0; i < nbChildren; i++)
            {
                //delete existing resource
                assets[i].setDeleted(Boolean.TRUE);
                rArray[i] = assets[i];

                //Add new resources
                AssetFolder r = AssetFolderFixture.createFixture(false, 0,
                        null, ctc);
                rArray[nbChildren + i] = r;
                r.setParent(folder);
            }

            updFolder.setFolders(rArray);
        }
        return updFolder;
    }

    public static void compare(AssetFolder p1, AssetFolder p2, AssetScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(p1, p2, folderSkipGetters);

        if (scope == null)
            return;

        if (scope.getParent() != null)
        {
            if (p2.getParent() == null)
                Assert.fail("RPMObject in scope is null");

            //Be careful with the scope to avoid infinite loop
            AssetScope parentScope = (AssetScope)scope.getParent();
            if (parentScope != null)
            {
                parentScope.setFolders(null);
            }

            AssetFolderFixture.compare((AssetFolder) p1.getParent(),
                    (AssetFolder) p2.getParent(), parentScope);
        }

        if (scope.getFolders() != null)
        {
            if ((p2.getFolders() == null) && (p1.getFolders() != null))
            {
                Assert.fail("RPMObject in scope is null");
            }
            else if ((p1.getFolders() == null) && (p2.getFolders() != null))
            {
                Assert.fail("RPMObject in scope is null");
            }
            else if ((p1.getFolders() != null) && (p2.getFolders() != null))
            {

                //Be careful with the scope to avoid infinite loop
                AssetScope childrenScope = scope.getFolders();
                if (childrenScope != null)
                {
                    childrenScope.setParent(null);
                }

                AssetFolderFixture.compare(p1.getFolders(), p2.getFolders(),
                        childrenScope);
            }
        }
        if (scope.getAssets()!=null){
            AssetScope chldScope=scope.getAssets();
            GenericAsset[] a1Assets=p1.getAssets();
            GenericAsset[] a2Assets=p2.getAssets();
            if (a1Assets!=null && a2Assets!=null){
                Assert.assertTrue(a1Assets.length!=a2Assets.length);
                for(int i=0;i<a1Assets.length;++i){
                    Asset a1a=(Asset)a1Assets[i];
                    Asset a2a=(Asset)a2Assets[i];
                    AssetFixture.compare(a1a,a2a,chldScope);
                }
            }
        }

        if (scope.getAssets() != null)
        {
            if ((p2.getAssets() == null))
            {
                Assert.fail("RPMObject in scope is null");
            }

            AssetScope aScope = scope.getAssets();
            aScope.setParent(null);
            AssetFolderFixture
                    .compare(p1.getFolders(), p2.getFolders(), aScope);
            //AssetFixture.compareWithoutCanProperties(p1.getAsset(),
            // p2.getAsset(), aScope);
        }
    }

    public static AssetScope createScope(int currentLevel, int maxLevels,
            int nbFoldersByLevel, int nbAssetsByLevel, AssetScope parentScope)
    {
        AssetScope scope = new AssetScope();
        if (currentLevel <= maxLevels)
        {
            if (nbFoldersByLevel > 0 && currentLevel < maxLevels)
            {
                //create subfolders scope
                AssetScope subFolderScope = AssetFolderFixture.createScope(
                        currentLevel + 1, maxLevels, nbFoldersByLevel,
                        nbAssetsByLevel, scope);
                //                subFolderScope.setParent(scope);
                scope.setFolders(subFolderScope);
            }
            if (nbAssetsByLevel > 0 && currentLevel < maxLevels)
            {
                AssetScope childScope = AssetFolderFixture.createScope(
                        currentLevel + 1, maxLevels, 0, nbAssetsByLevel, scope);
                //                childScope.setParent(scope);
                scope.setAssets(childScope);
            }

        }
        if (parentScope != null)
        {
            scope.setParent(parentScope);
        }
        return scope;
    }

    public static void compare(AssetFolder[] pArray1, AssetFolder[] pArray2,
            AssetScope scope) throws AssertionFailedError, Exception
    {
        pArray1 = removeDeletedFolder(pArray1);

        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            AssetFolderFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }

    public static AssetFolder[] removeDeletedFolder(AssetFolder[] pArray)
    {

        AssetFolder[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new AssetFolder[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }
}