/*
 * Created on Oct 29, 2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;

/**
 * @author adler
 * 
 * A Fixture that simulates a datagraph of OrganizationalAssignment
 *  
 */
public class AssetOrganizationalAssignmentFixture extends RPMObjectFixture
{
    protected static ArrayList assignmentSkipSetters, assignmentSkipGetters;
    
    static
    {
        assignmentSkipSetters = new ArrayList();
        assignmentSkipSetters.addAll(skipSetters);
        
        assignmentSkipGetters = new ArrayList();
        assignmentSkipGetters.addAll(skipGetters);
    }

    /**
     * createFixture - Fixture for creating an OrganizationalAssignment object
     * 
     * @param Asset
     *            the parent of created object
     * @param organizational
     *            organizational to be assigned
     * @return Created OrganizationalAssignment object
     * @throws Exception
     *  
     */
    public static AssetOrganizationalAssignment createFixture(Asset asset, Organizational organizational)
            throws Exception
    {
        AssetOrganizationalAssignment orgAss = null;
        
    	if(asset != null && organizational !=null)
    	{
            //OrganizationalAssignment has no children
    	    orgAss = createAssetOrganizationalAssignment();
            // Set the asset as parent
    	    orgAss.setParent(asset);
    	    orgAss.setOrganizational(organizational);
        }

        return orgAss;
    }

    /**
     * createAssetOrganizationalAssignment - Create a AssetOrganizationalAssignment object 
     * @return Created createOrganizationalAssignment object
     * @throws Exception
     *  
     */
    private static AssetOrganizationalAssignment createAssetOrganizationalAssignment()
            throws Exception
    {
        AssetOrganizationalAssignment orgAss = new AssetOrganizationalAssignment();
        return orgAss;
    }    

    /**
     * compare - Compare two AssetOrganizationalAssignment objects
     * 
     * @param o1
     *            the first AssetOrganizationalAssignment object to be compared
     * @param o2
     *            the second AssetOrganizationalAssignment object to be compared
     * @param AssetOrganizationalAssignmentScope
     *            the compared scope
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(AssetOrganizationalAssignment o1,
            AssetOrganizationalAssignment o2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(o1, o2, assignmentSkipGetters);

        // Compare the assets
            AssetFixture.compare((Asset) o1.getParent(),
                    (Asset) o2.getParent(), null);
        // Compare the organizations
            OrganizationalFixture.compare(o1.getOrganizational(), o2.getOrganizational(), null);    	
    }  
    
    /**
     * compare - Compare two OrganizationalAssignment array objects
     * 
     * @param oArray1
     *            the first AssetOrganizationalAssignment array objects to be compared
     * @param oArray2
     *            thednoce AssetOrganizationalAssignment array objects to be compared
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(AssetOrganizationalAssignment[] oArray1,
            AssetOrganizationalAssignment[] oArray2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(oArray1, oArray2);

        if (oArray1 == null)
            return;

        Arrays.sort(oArray1, new AssetOrganizationalAssignmentComparator());
        Arrays.sort(oArray2, new AssetOrganizationalAssignmentComparator());

        for (int i = 0; i < oArray1.length; i++)
        {
            compare(oArray1[i], oArray2[i]);
        }

    } 

    
    private static class AssetOrganizationalAssignmentComparator implements Comparator
    {
        public int compare(Object o1, Object o2)
        {
            return ((AssetOrganizationalAssignment) o1).getID().compareTo(
                    ((AssetOrganizationalAssignment) o2).getID());
        }
    }

}
