/*
 * Created on Nov 8, 2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author adler
 *
 * A Fixture that simulates a datagraph of AssetResourceAssignment
 */
public class AssetResourcelAssignmentFixture extends RPMObjectFixture{
    protected static ArrayList assignmentSkipSetters, assignmentSkipGetters;
    
    static
    {
        assignmentSkipSetters = new ArrayList();
        assignmentSkipSetters.addAll(skipSetters);
        
        assignmentSkipGetters = new ArrayList();
        assignmentSkipGetters.addAll(skipGetters);
    }

    /**
     * createFixture - Fixture for creating an AssetResourceAssignment object
     * 
     * @param Asset
     *            the parent of created object
     * @param Resource
     *            Array of Resources to be assigned
     * @param size
     *            Array size
     * @return Created AssetResourceAssignment objects
     * @throws Exception
     *  
     */
    public static AssetResourceAssignment[] createFixture(Asset asset, Resource[] resource, int size)
            throws Exception
    {
    	AssetResourceAssignment[] resourceAssignment = new AssetResourceAssignment[size];
        
        for (int i = 0; i < size; i++)
        {
        	if(asset != null && resource !=null)
        	{
        		//AssetResourceAssignment has no children
        		resourceAssignment[i] = createAssetResourceAssignment();
        		// Set the asset as parent
        		resourceAssignment[i].setParent(asset);
        		resourceAssignment[i].setResource(resource[i]);
        	}
        }

        return resourceAssignment;
    }

    /**
     * createAssetResourceAssignment - Create an AssetResourceAssignment object with random id
     * number 
     * 
     * @return Created AssetResourceAssignment object
     * @throws Exception
     *  
     */
    private static AssetResourceAssignment createAssetResourceAssignment()
            throws Exception
    {
    	AssetResourceAssignment resourceAssignment = new AssetResourceAssignment();
        return resourceAssignment;
    } 
    
    /**
     * compare - Compare two AssetResourceAssignment objects
     * 
     * @param o1
     *            the first AssetResourceAssignment object to be compared
     * @param o2
     *            the second AssetResourceAssignment object to be compared
     * @param AssetResourceAssignmentScope
     *            the compared scope
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(AssetResourceAssignment o1,
    		AssetResourceAssignment o2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(o1, o2, assignmentSkipGetters);

        // Compare the assets
            AssetFixture.compare((Asset) o1.getParent(),
                    (Asset) o2.getParent(), null);
        // Compare the resources
            ResourceFixture.compare(o1.getResource(), o2.getResource(), null);    	
    }
    /**
     * compare - Compare two AssetResourceAssignment array objects
     * 
     * @param oArray1
     *            the first AssetResourceAssignment array objects to be compared
     * @param oArray2
     *            thednoce AssetResourceAssignment array objects to be compared
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(AssetResourceAssignment[] oArray1,
    		AssetResourceAssignment[] oArray2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(oArray1, oArray2);

        if (oArray1 == null)
            return;

        Arrays.sort(oArray1, new AssetResourceAssignmentComparator());
        Arrays.sort(oArray2, new AssetResourceAssignmentComparator());

        for (int i = 0; i < oArray1.length; i++)
        {
            compare(oArray1[i], oArray2[i]);
        }
    }    

    private static class AssetResourceAssignmentComparator implements Comparator
	{
    	public int compare(Object o1, Object o2)
	    {
	            return ((AssetResourceAssignment) o1).getID().compareTo(
	                    ((AssetResourceAssignment) o2).getID());
	    }
	}

}    

