package com.ibm.rpm.client.junit.asset.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture.RPMObjectIDComparator;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObjectScope;

/**
 * @author ragoler
 *
 * A Fixture that emulates a datagraph of DepreciationSchedule
 *
 */
public class DepreciationScheduleFixture extends RPMObjectFixture
{

    protected static ArrayList depreciationSkipSetters, depreciationSkipGetters;

    static
    {
        depreciationSkipSetters = new ArrayList();
        depreciationSkipSetters.addAll(skipSetters);

        depreciationSkipGetters = new ArrayList();
        depreciationSkipGetters.addAll(skipGetters);

    }

    public static DepreciationSchedule[] createFixtures(Asset asset, int size) throws Exception
	{
        DepreciationSchedule[] deps = new DepreciationSchedule[size];

        int index;
        for (int i = 0; i < size; i++)
        {
            index = getNextInt();
            DepreciationSchedule depreciation = new DepreciationSchedule();
    		HelperTestUtilities.setByIntrospection(depreciation, index, depreciationSkipSetters);
    		depreciation.setParent(asset);
    		deps[i] = depreciation;
        }
		return deps;
	}

	public static void compare(DepreciationSchedule d1, DepreciationSchedule d2, RPMObjectScope scope)
	    throws AssertionFailedError, Exception
	{
		RPMObjectFixture.compareWithSkip(d1, d2, depreciationSkipGetters);

		if(scope!=null && d2.getParent()!=null)
		    AssetFixture.compare((Asset) d1.getParent(), (Asset) d2.getParent(), (AssetScope)scope.getParent());
	}

	public static void compare(DepreciationSchedule[] dArray1,
	    DepreciationSchedule[] dArray2) throws AssertionFailedError, Exception
	{
		RPMObjectFixture.compare(dArray1, dArray2);
		if (dArray1 == null)
		    return;


		Arrays.sort(dArray1, RPMObjectIDComparator.INSTANCE);
		Arrays.sort(dArray2, RPMObjectIDComparator.INSTANCE);

		for (int i = 0; i < dArray1.length; i++) {
		    compare(dArray1[i], dArray2[i]);
		}
	}

    private static int INDEX = 1;

    private static int getNextInt()
    {
        return INDEX++;
    }
    

}
