/*
 * Created on Sep 6, 2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture.RPMObjectIDComparator;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;

/**
 * @author adler
 * 
 * A Fixture that simulates a datagraph of MaintenanceEstimates
 *  
 */
public class MaintenanceEstimatesFixture extends RPMObjectFixture
{

    protected static ArrayList estimatesSkipSetters, estimatesSkipGetters;

    static
    {
        estimatesSkipSetters = new ArrayList();
        estimatesSkipSetters.addAll(skipSetters);
        estimatesSkipSetters.add("setScheduleDate");

        estimatesSkipGetters = new ArrayList();
        estimatesSkipGetters.addAll(skipGetters);
        estimatesSkipGetters.add("getScheduleDate");
    }

    /**
     * createFixture - Fixture for creating an MaintenanceEstimates object
     * 
     * @param Asset
     *            the parent of created object
     * @return Created MaintenanceEstimates objects
     * @throws Exception
     *  
     */
    public static MaintenanceEstimates[] createFixture(Asset asset, int size)
            throws Exception
    {
        MaintenanceEstimates[] ests = new MaintenanceEstimates[size];
        int index;
        for (int i = 0; i < size; i++)
        {
            //MaintenanceEstimates has no children
            index = getNextInt();
            ests[i] = createEstimate(index);
            // Set the asset as parent
            ests[i].setParent(asset);
        }

        return ests;
    }

    /**
     * createEstimate - Create an MaintenanceEstimates object with random id
     * number and with predeined schedule date
     * 
     * @param index
     *            the random id number
     * @return Created MaintenanceEstimates object
     * @throws Exception
     *  
     */
    private static MaintenanceEstimates createEstimate(int index)
            throws Exception
    {
        MaintenanceEstimates estimates = new MaintenanceEstimates();
        HelperTestUtilities.setByIntrospection(estimates, index,
                estimatesSkipSetters);
        // Scheduled date
        estimates.setScheduleDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2005-09-06")));
        return estimates;
    }

    /**
     * compare - Compare two MaintenanceEstimates objects
     * 
     * @param e1
     *            the first MaintenanceEstimates object to be compared
     * @param e2
     *            the second MaintenanceEstimates object to be compared
     * @param MaintenanceEstimatesScope
     *            the compared scope
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(MaintenanceEstimates e1,
            MaintenanceEstimates e2, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(e1, e2, estimatesSkipGetters);

        if ((scope != null) && (e2.getParent() != null))
            AssetFixture.compare((Asset) e1.getParent(),
                    (Asset) e2.getParent(), (AssetScope)scope.getParent());
        validateCalendarTypes(e1.getScheduleDate(), e2.getScheduleDate());
    }

    /**
     * compare - Compare two MaintenanceEstimates array objects
     * 
     * @param eArray1
     *            the first MaintenanceEstimates array objects to be compared
     * @param eArray2
     *            the first MaintenanceEstimates array objects to be compared
     * @return None
     * @throws AssertionFailedError &
     *             Exception
     *  
     */
    public static void compare(MaintenanceEstimates[] eArray1,
            MaintenanceEstimates[] eArray2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(eArray1, eArray2);

        if (eArray1 == null)
            return;

        Arrays.sort(eArray1, RPMObjectIDComparator.INSTANCE);
        Arrays.sort(eArray2, RPMObjectIDComparator.INSTANCE);

        for (int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }

    }

    /**
     * validateCalendarTypes - Validate two Calendar objects
     * 
     * @param c1
     *            the first Calendar to validate
     * @param c2
     *            the second Calendar to validate
     * @return None
     * @throws AssertionError &
     *             Exception
     *  
     */
    private static void validateCalendarTypes(Calendar c1, Calendar c2)
            throws AssertionError, Exception
    {
        if (c1 != null && c2 != null)
        {
            Assert.assertEquals(c1.get(Calendar.YEAR), c2.get(Calendar.YEAR));
            Assert.assertEquals(c1.get(Calendar.DAY_OF_MONTH), c2
                    .get(Calendar.DAY_OF_MONTH));
            Assert.assertEquals(c1.get(Calendar.MONTH), c2.get(Calendar.MONTH));
        }
        if ((c1 == null && c2 != null) || (c1 != null && c2 == null))
        {
            Assert.fail();
        }
    }

    private static int INDEX = 1;

    private static int getNextInt()
    {
        return INDEX++;
    }
    
}
