package com.ibm.rpm.client.junit.asset.testcases;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.asset.scope.CrossChargeFinancialsScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetCrossChargeFinancialsFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetOrganizationalAssignmentFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetResourcelAssignmentFixture;
import com.ibm.rpm.client.junit.asset.fixtures.DepreciationScheduleFixture;
import com.ibm.rpm.client.junit.asset.fixtures.MaintenanceEstimatesFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.clientcostcenters.containers.AbstractCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardScope;

/**
 * @author rosman
 */
public class TestAsset extends ClientTestController
{
    protected Asset mainAsset;

    protected AssetFolder parentAssetFolder = null;

    protected AssetScope mainScope;

    private static final int NB_MAX_LEVELS = 2;

    private static final int NB_FOLDERS_BY_LEVEL = 1;

    private static final int NB_ASSETS_BY_LEVEL = 1;

    // todo: this should be removed once we have it in an organized place
    public static final int DEPRECIATION_MANUAL = 0;

    public static final int DEPRECIATION_LINEAR = 1;

    private AssetScope parentScope = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        parentScope = AssetFolderFixture.createScope(1, NB_MAX_LEVELS, 0, 0,
                null);
        mainScope = AssetFixture.createScope(1, NB_MAX_LEVELS,
                NB_ASSETS_BY_LEVEL, parentScope);
    }

    private AssetFolder getParentAssetFolder() throws Exception
    {
        if (parentAssetFolder == null)
        {
        parentAssetFolder = AssetFolderFixture.createSaveFixture(this, null);
        }
        return parentAssetFolder;
    }

    public void testInsert() throws AssertionError, Exception
    {
        String AssetName = "TCAssetFolderTC3";

        mainScope = new AssetScope();

        AssetFolder folder = new AssetFolder();
        folder.setName(AssetName);
        mainScope.setFolders(new AssetScope());

        Asset asset = new Asset();

        //Asset Identification
        asset.setName("TCAsset");

        folder.setAssets(new GenericAsset[]
        {
            asset
        });
        mainScope.setAssets(new AssetScope());

        SingleResult result = save(folder, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof AssetFolder);
        assertNull(result.getErrors());

        //load from XPath
        RPMObject[] assetFolders = loadFromXpath("/AssetFolder[name='"
                + AssetName + "']", mainScope);
        assertNotNull(assetFolders);

        AssetFolder mainFolder = (AssetFolder) assetFolders[0];
        assertNotNull(mainFolder);
        mainAsset = (Asset) mainFolder.getAssets()[0];
        assertNotNull(mainAsset);

        //Asset RtfAssignment
        RtfAssignment rtfAss = new RtfAssignment();

        AssetRTF assetRTF = new AssetRTF();
        assetRTF.setID("6041");
        //		DatafieldRTF assetDatafield = getAssetRTF("6041");
        rtfAss.setRtf(assetRTF);
        rtfAss.setValue("TC-Asset-Description");
        rtfAss.setParent(mainAsset);

        SingleResult result3 = save(mainFolder, mainScope);
        assertNotNull(result3);
        assertNotNull(result3.getRpmObject());
        assertTrue(result3.getRpmObject() instanceof AssetFolder);
        assertNull(result3.getErrors());

    }

    public void testInsertWithAssetAttributeAssignment() throws AssertionError,
            Exception
    {
        mainScope.setAttributeAssignments(true);
        mainScope = new AssetScope();
        mainScope.setAttributeAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testInsertWithDocumentFolder() throws AssertionError, Exception
    {
        mainScope = new AssetScope();
        mainScope.setDocumentFolder(new DocumentScope());
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testInsertWithFullScope() throws AssertionError, Exception
    {
        mainScope = new AssetScope();
        mainScope.setAttributeAssignments(true);
        mainScope.setAssetResourceAssignments(true);
        mainScope.setCrossChargeFinancials(new CrossChargeFinancialsScope());
        mainScope.setDepreciationSchedules(new RPMObjectScope());
        mainScope.setGeographicalAssignment(true);
        mainScope.setMaintenanceEstimates(new RPMObjectScope());
        mainScope.setAssetOrganizationalAssignment(true);
        mainScope.setRtfAssignments(true);
        mainScope.setAssignedScorecard(new ScorecardScope());
        mainScope.setAssetOrganizationalAssignment(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testInsertWithScorecardAssignment() throws AssertionError,
            Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);
        // assignedAsset instead?
        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);

        AssetFixture.compare(assignedAsset, loaded, mainScope);

        PublishedScorecardFixture.deleteScorecard(loaded, this);

    }

    public void testInsertWithScorecardAssignmentReloadTypeSaved()
            throws AssertionError, Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.SavedResult);

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);

        PublishedScorecardFixture.compare(assignedAsset.getAssignedScorecard(),
                loaded.getAssignedScorecard(), PublishedScorecardFixture
                        .createScorecardScope(mainScope));
    }

    public void testInsertWithScorecardAssignmentReloadTypeNone()
            throws AssertionError, Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.None);

        Assert.assertNull(assignedAsset.getAssignedScorecard());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);

        Assert.assertNotNull(loaded.getAssignedScorecard());
    }

    public void testUpdateWithScorecardAssignment() throws AssertionError,
            Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        PublishedScorecardFixture.updateAssignment(10, assignedAsset
                .getAssignedScorecard());

        save(assignedAsset, mainScope, ReloadType.ReloadResult);

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);

        AssetFixture.compare(assignedAsset, loaded, mainScope);

        PublishedScorecardFixture.deleteScorecard(loaded, this);

    }

    public void testInsertWithScorecardXpathCondition1() throws AssertionError,
            Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        RPMObject[] rpmObjects = loadFromXpath("/Asset[/AssignedScorecard[Id='"
                + assignedAsset.getAssignedScorecard().getID() + "']]",
                mainScope);
        Assert.assertEquals(rpmObjects.length, 1);

        AssetFixture.compare(assignedAsset, (Asset) rpmObjects[0], mainScope);

        PublishedScorecardFixture.deleteScorecard(assignedAsset, this);

    }

    public void testInsertWithScorecardXpathCondition2() throws AssertionError,
            Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        RPMObject[] rpmObjects = loadFromXpath(
                "/Asset[/AssignedScorecard[/AssignedScorecardCategory[Id='"
                        + assignedAsset.getAssignedScorecard()
                                .getAssignedScorecardCategories()[0].getID()
                        + "']]]", mainScope);
        Assert.assertEquals(rpmObjects.length, 1);

        AssetFixture.compare(assignedAsset, (Asset) rpmObjects[0], mainScope);

        PublishedScorecardFixture.deleteScorecard(assignedAsset, this);

    }

    public void testXPathAssignedScorecardLoad() throws AssertionError,
            Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        RPMObject[] rpmObjects = loadFromXpath("/Asset[@id='"
                + assignedAsset.getID() + "']/AssignedScorecard", mainScope
            .getAssignedScorecard());
        Assert.assertEquals(rpmObjects.length, 1);

        PublishedScorecardFixture
            .compare(assignedAsset.getAssignedScorecard(),
                (AssignedScorecard) rpmObjects[0], mainScope
                    .getAssignedScorecard());

        PublishedScorecardFixture.deleteScorecard(assignedAsset, this);

    }

    public void testPulishScorecard() throws AssertionError, Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        AssignedScorecard publishedSC = publishScorecard(
                assignedAsset.getAssignedScorecard(),
                PublishedScorecardFixture.createScorecardScope(mainScope));

        PublishedScorecardFixture.comparePublishedScorecard(publishedSC);

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);

        PublishedScorecardFixture.comparePublishedScorecard(loaded
                .getAssignedScorecard());

    }

    private Asset prepareScorecardAssignment(ReloadType reloadType)
            throws Exception, AssertionError, RPMException,
            AssertionFailedError
    {
        mainScope.setAssignedScorecard(PublishedScorecardFixture
                .createScorecardScope(mainScope));
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        AssetModuleScope assetModuleScope = AssetModuleFixture.createScope(false,false,false,false,false,false,false,true);
        
        AssetModule module = AssetModuleFixture.createFixture(assetModuleScope, this);
        module = (AssetModule) save(module, assetModuleScope).getRpmObject();
       
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];
        
        Asset assignedAsset = (Asset) PublishedScorecardFixture
                .createAssignmentFixture(mainAsset, mainScope, this,
                        reloadType, scorecardFolder, 1);

        return assignedAsset;
    }


    // /Asset[Id=XXX], load attributes from the container - should succeed
    public void testInsertWithAssetAttributeAssignmentXPath3()
            throws AssertionError, Exception
    {
        mainScope.setAttributeAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AttributeAssignment[] asgns = saved.getAttributeAssignments();
        assertNotNull(asgns);
        RPMObject[] loadedAttrAsgn = loadFromXpath("/Asset[ID='"
                + saved.getID() + "']", mainScope);
        assertNotNull(loadedAttrAsgn);
        assertTrue(loadedAttrAsgn.length > 0);
        Asset loadedXpath = (Asset) loadedAttrAsgn[0];
        AssetFixture.compare(loaded, saved, mainScope);
        AssetFixture.compare(loadedXpath, saved, mainScope);
    }

    // Try "/Asset[AttributeAssignment[ID='XXX']]"
    public void testInsertWithAssetAttributeAssignmentXPath4()
            throws AssertionError, Exception
    {
        mainScope.setAttributeAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AttributeAssignment[] asgns = saved.getAttributeAssignments();
        assertNotNull(asgns);
        RPMObject[] loadedAttrAsgn = loadFromXpath("/Asset"
                + "[AttributeAssignment[ID='" + (asgns[0]).getID() + "']]",
                mainScope);
        assertNotNull(loadedAttrAsgn);
        assertTrue(loadedAttrAsgn.length > 0);
        Asset loadedXpath = (Asset) loadedAttrAsgn[0];
        AssetFixture.compare(loaded, loadedXpath, mainScope);
    }

    public void testInsertWithGeographicalAssignment() throws AssertionError,
            Exception
    {
        mainScope.setGeographicalAssignment(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());
        AssetFixture.compare(mainAsset, saved, mainScope);

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testUpdateWithGeographicalAssignment() throws AssertionError,
            Exception
    {
        mainScope.setGeographicalAssignment(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());
        AssetFixture.compare(mainAsset, saved, mainScope);
        AssetFixture.updateFixture(mainAsset, 1, this);
        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testInsertWithRtfAssignment() throws AssertionError, Exception
    {
        mainScope.setRtfAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
        AssetFixture.compare(mainAsset, saved, mainScope);
    }

    public void testInsertWithAttributeAssignment() throws AssertionError,
            Exception
    {
        mainScope.setAttributeAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        Asset loaded = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(loaded, saved, mainScope);
    }

    // /Asset[name='XXX'], load rtf by the container
    public void testInsertWithRtfAssignmentLoadAssetByXPath()
            throws AssertionError, Exception
    {
        mainScope.setRtfAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='"
                + mainAsset.getID() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects.length > 0);
        Asset loaded = (Asset) rpmObjects[0];
        AssetFixture.compare(loaded, saved, mainScope);
    }

    // /Asset[RtfAssignment[ID=""]] - getSubTableJoinCondition() fails
    public void testInsertWithRtfAssignmentLoadAssetByXPath2()
            throws AssertionError, Exception
    {
        mainScope.setRtfAssignments(true);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());
        RtfAssignment[] asgns = saved.getRtfAssignments();
        RPMObject[] rpmObjects = loadFromXpath("/Asset[RtfAssignment[ID='"
                + asgns[0].getID() + "']]", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects.length > 0);
        Asset loaded = (Asset) rpmObjects[0];
        AssetFixture.compare(loaded, saved, mainScope);
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, null, this);
        Asset saved = saveAsset(mainAsset, null);
        mainAsset.setID(saved.getID());
        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='"
                + mainAsset.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        AssetFixture.compare(mainAsset, (Asset) rpmObjects[0], null);

    }

    public void testInsertWithScopeLoadFromXPathLevel2()
            throws AssertionError, Exception
    {
        initAssetWithFullScope();
        AssetFolder savedParent = saveAssetFolder(getParentAssetFolder(), parentScope);
        addAssetsToDeletion(savedParent);
        //addAssetsToDeletion((AbstractAsset) saved.getParent());
        GenericAsset[] savedAssets = savedParent.getAssets();
        Asset saved = (Asset) savedAssets[0];
        mainAsset.setID(saved.getID());
        GenericAsset[] children = saved.getAssets();
        assertNotNull(children);
        assertNotNull(children[0]);

        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='" + saved.getID()
                + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        AssetFixture.compare(mainAsset, (Asset) rpmObjects[0], null);
        AssetFixture.compare(saved, (Asset) rpmObjects[0], mainScope);

        RPMObject[] children2 = loadFromXpath("/Asset[name='" + saved.getName()
                + "']/Asset[name='" + children[0].getName() + "']", mainScope);
        assertNotNull(children2);
        assertTrue(!children2[0].getID().equals(saved.getID()));

    }

    public void testInsertWithScopeLoadFromXPathLevelSubTable()
            throws AssertionError, Exception
    {
        initAssetWithFullScope();
        AssetFolder savedParent = saveAssetFolder((AssetFolder) mainAsset
                .getParent(), parentScope);
        addAssetsToDeletion(savedParent);
        //addAssetsToDeletion((AbstractAsset) saved.getParent());
        GenericAsset[] savedAssets = savedParent.getAssets();
        Asset saved = (Asset) savedAssets[0];
        mainAsset.setID(saved.getID());
        GenericAsset[] children = saved.getAssets();
        assertNotNull(children);
        assertNotNull(children[0]);

        RPMObject[] rpmObjects = loadFromXpath("/Asset[name='"
                + mainAsset.getName() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        AssetFixture.compare(mainAsset, (Asset) rpmObjects[0], null);
        AssetFixture.compare(saved, (Asset) rpmObjects[0], mainScope);
        RPMObject[] reloadedAsset = loadFromXpath("/Asset[Asset[name='"
                + children[0].getName() + "']]", mainScope);
        assertTrue(!children[0].getID().equals(mainAsset.getID()));
        Asset[] children2 = (Asset[])((Asset)reloadedAsset[0]).getAssets();
        AssetFixture.compare((Asset)mainAsset.getAssets()[0], children2[0], null);
        AssetFixture.compare((Asset)saved.getAssets()[0], children2[0], mainScope);

    }

    public void testLoadFromXPathIncludeFolder() throws AssertionError,
            Exception
    {
        initAssetWithFullScope();
        AssetFolder savedParent = saveAssetFolder((AssetFolder) mainAsset
                .getParent(), parentScope);
        addAssetsToDeletion(savedParent);
        //addAssetsToDeletion((AbstractAsset) saved.getParent());
        GenericAsset[] savedAssets = savedParent.getAssets();
        Asset saved = (Asset) savedAssets[0];
        mainAsset.setID(saved.getID());
        GenericAsset[] children = mainAsset.getAssets();
        assertNotNull(children);
        assertNotNull(children[0]);

        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[name='"
                + getParentAssetFolder().getName() + "']/Asset[name='"
                + mainAsset.getName() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        AssetFixture.compare(mainAsset, (Asset) rpmObjects[0], null);

    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        Asset saved = saveAsset(mainAsset, null);
        addAssetsToDeletion(saved);
        loadAssetFromId(saved, null); // includes the comparison
    }

    public void testFailInsertionWithNoParent() throws AssertionError,
            Exception
    {
        mainScope = new AssetScope();
        mainAsset = new Asset();
        mainScope.setParent(new AssetScope());
        try
        {
            saveAsset(mainAsset, mainScope);
        }
        catch (Exception e)
        {
            return;
        }
        fail();
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        Asset saved = saveAsset(mainAsset, mainScope);
        addAssetsToDeletion(saved);
        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope); // includes the comparison
        Asset loaded = (Asset) loadFromID(saved, mainScope);
        AssetFixture.compare(loaded, mainAsset, null);
    }

    public void testAssetWithScopeForManualDepreciationSchedule()
            throws AssertionError, Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        mainAsset.setDepreciationMethod(new Integer(DEPRECIATION_MANUAL));
        mainAsset.setDepreciationSchedules(DepreciationScheduleFixture
                .createFixtures(mainAsset, 5));

        mainScope.setDepreciationSchedules(new RPMObjectScope());

        Asset saved = saveAsset(mainAsset, mainScope);
        addAssetsToDeletion(saved);

        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope);
    }

    public void testAssetWithScopeForAutomaticDepreciationSchedule()
            throws AssertionError, Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        mainAsset.setDepreciationMethod(new Integer(DEPRECIATION_LINEAR));
        mainAsset.setAssetLife(new Integer(10));

        mainScope.setDepreciationSchedules(new RPMObjectScope());

        Asset saved = saveAsset(mainAsset, mainScope);
        addAssetsToDeletion(saved);

        mainAsset.setID(saved.getID());
        Asset newAsset = (Asset) loadFromID(mainAsset, mainScope);
        AssetFixture.compare(saved, newAsset, mainScope);

    }

    public void testAssetWithScopeForMaintenanceEstimates()
            throws AssertionError, Exception
    {
        MaintenanceEstimates[] est = new MaintenanceEstimates[5];
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        mainScope.setMaintenanceEstimates(new RPMObjectScope());
        est = MaintenanceEstimatesFixture.createFixture(mainAsset, 5);
        mainAsset.setMaintenanceEstimates(est);

        Asset saved = saveAsset(mainAsset, mainScope);

        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope);
    }

    public void testAssetWithScopeForOrganizationAssignment()
            throws AssertionError, Exception
    {
        AssetOrganizationalAssignment assetOrganizationalAssignment = new AssetOrganizationalAssignment();

        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);

        mainScope.setAssetOrganizationalAssignment(true);

        //create and save new organization
        Organizational organizational = OrganizationalFixture
                .createSaveFixture(1, this);

        //create new Organizational Assignment
        assetOrganizationalAssignment = AssetOrganizationalAssignmentFixture
                .createFixture(mainAsset, organizational);

        mainAsset
                .setAssetOrganizationalAssignment(assetOrganizationalAssignment);

        // Organizational Assignment scope
        mainScope.setAssetOrganizationalAssignment(true);
        // Save the asset with Organizational Assignment scope
        Asset saved = saveAsset(mainAsset, mainScope);

        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope);
    }

    public void testAssetWithScopeForResourceAssignment()
            throws AssertionError, Exception
    {

        AssetResourceAssignment[] assetResourceAssignmentArray = new AssetResourceAssignment[5];
        Resource[] resources = new Resource[5];

        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);

        mainScope.setAssetResourceAssignments(true);

        for (int i = 0; i < 5; i++)
        {
            //create and save new 5 resources
            resources[i] = ResourceFixture.createSaveFixture(this);
            //create new 5 Asset Resource Assignment
            assetResourceAssignmentArray = AssetResourcelAssignmentFixture
                    .createFixture(mainAsset, resources, 5);
        }

        mainAsset.setAssetResourceAssignments(assetResourceAssignmentArray);

        // Resource Assignment scope
        mainScope.setAssetResourceAssignments(true);

        // Save the asset with Resource Assignment scope
        Asset saved = saveAsset(mainAsset, mainScope);

        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope);

    }

    public void testAssetWithScopeForCrossChargeFinancials()
            throws AssertionError, Exception
    {
        // TODO Raise the number for more AbstractCostCenter objects
        int numberCrossChargedObjects = 1;

        CrossChargeFinancials[] crossChargeFinancialsArray = new CrossChargeFinancials[numberCrossChargedObjects];
        AbstractCostCenter[] projectOrClient = new AbstractCostCenter[numberCrossChargedObjects];

        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);

        //create and save a new ClientCostCenter
        projectOrClient[0] = ClientCostCenterFixture.createSaveFixture(this);

        //TODO add ProjectCostCenter

        //create new CrossChargeFinancials
        crossChargeFinancialsArray = AssetCrossChargeFinancialsFixture
                .createFixture(mainAsset, projectOrClient,
                        numberCrossChargedObjects);

        // Assign the Cross Charged Objects to Asset
        mainAsset.setCrossChargeFinancials(crossChargeFinancialsArray);

        // Cross Charge Financials Scope
        CrossChargeFinancialsScope crossChargeFinancialsScope = new CrossChargeFinancialsScope();

        // Assign Cross Charge Financials Scope parent as asset
        crossChargeFinancialsScope.setParent(mainScope);
        // TODO work with two scopes
        crossChargeFinancialsScope.setCostCenter(new ClientCostCenterScope());

        mainScope.setCrossChargeFinancials(crossChargeFinancialsScope);

        // Save the asset with Cross Charged Financials scope
        Asset saved = saveAsset(mainAsset, mainScope);

        mainAsset.setID(saved.getID());
        loadAssetFromId(saved, mainScope);

    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(), null, this);
        Asset saved = saveAsset(mainAsset, null);
        mainAsset.setID(saved.getID());

        Asset updatedAsset = AssetFixture.updateFixture(mainAsset,
                NB_ASSETS_BY_LEVEL, this);
        updatedAsset.setID(mainAsset.getID());
        Asset saved2 = saveAsset(updatedAsset, null);
        Asset loaded = new Asset();
        loaded.setID(updatedAsset.getID());
        loaded = (Asset) loadFromID(loaded, null);
        AssetFixture.compare(updatedAsset, saved2, null);
        AssetFixture.compare(saved2, loaded, null);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        initAssetWithFullScope();
        AssetFolder savedParent = saveAssetFolder((AssetFolder) mainAsset
                .getParent(), parentScope);
        addAssetsToDeletion(savedParent);
        GenericAsset[] savedAssets = savedParent.getAssets();
        Asset saved = (Asset) savedAssets[0];
        mainAsset.setID(saved.getID());
        GenericAsset[] children = mainAsset.getAssets();
        assertNotNull(children);
        assertNotNull(children[0]);

        Asset updatedAsset = AssetFixture.updateFixture(saved,
                NB_ASSETS_BY_LEVEL, this);
        Asset saved2 = saveAsset(updatedAsset, null);

        Asset loaded = new Asset();
        loaded.setID(updatedAsset.getID());
        loaded = (Asset) loadFromID(loaded, mainScope);

        updatedAsset = (Asset) loadFromID(updatedAsset, mainScope);
        AssetFixture.compare(updatedAsset, saved2, mainScope);
        AssetFixture.compare(saved2, loaded, mainScope);
        assertFalse(saved2.getContact().equals(saved.getContact()));
        // Sanity check - the values in the saved asset should be different now
        assertFalse(saved2.getSerialNumber().equals(saved.getSerialNumber())); 
    }

    protected Asset saveAsset(Asset ast, AssetScope scope)
            throws AssertionError, Exception
    {
        //Parent needs to be in scope to be saved...
        if (scope == null)
        {
            scope = new AssetScope();
        }

        if (scope.getParent() == null)
        {
            scope.setParent(new AssetScope());
        }
        SingleResult result = save(ast, scope);

        ast = (Asset) result.getRpmObject();
        return ast;
    }

    protected AssetFolder saveAssetFolder(AssetFolder ast, AssetScope scope)
            throws AssertionError, Exception
    {

        //Parent needs to be in scope to be saved...
        if (scope == null)
        {
            scope = new AssetScope();
        }

        if (scope.getParent() == null)
        {
            scope.setParent(new AssetScope());
        }

        SingleResult result = save(ast, scope);

        ast = (AssetFolder) result.getRpmObject();
        return ast;
    }

    private void loadAssetFromId(Asset objectToCompare, AssetScope scope)
            throws AssertionError, Exception
    {
        RPMObject obj = new Asset();
        obj.setID(objectToCompare.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Asset);
        AssetFixture.compare((Asset) rpmObject, objectToCompare, scope);
    }

    public void testDeleteAsset() throws AssertionError, Exception
    {
        mainAsset = AssetFixture.createFixture(getParentAssetFolder(),
                NB_ASSETS_BY_LEVEL, mainScope, this);
        mainAsset = saveAsset(mainAsset, mainScope);

        RPMObject rpmObject = loadFromID(mainAsset, mainScope);

        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Asset);
        AssetFixture.compare(mainAsset, (Asset) rpmObject, mainScope);

        delete(mainAsset);

        RPMObject deleted = null;
        try
        {
            deleted = loadFromID(mainAsset, null);
        }
        catch (Exception e)
        {
            //deleted...
        }

        assertNull(deleted);
    }

    private void initAssetWithFullScope() throws Exception
    {

        mainScope = new AssetScope();
        parentScope = new AssetScope();
        parentScope.setAssets(mainScope);
        mainScope.setParent(parentScope);
        AssetScope childrenScope = new AssetScope();
        mainScope.setAssets(childrenScope);
        AssetScope grandChildrenScope = new AssetScope();
        childrenScope.setAssets(grandChildrenScope);
        parentAssetFolder = AssetFolderFixture.createFixture(false, 0,
                parentScope, this);
        mainAsset = AssetFixture.createFixture(parentAssetFolder,
                NB_FOLDERS_BY_LEVEL, mainScope, this);
        parentAssetFolder.setAssets(new GenericAsset[]
        {
            mainAsset
        });
    }

    private void addAssetsToDeletion(GenericAsset saved)
    {
        // TODO SLegault: check that this does not increase garbage in DB.
        // ... and does decrease errors at tearDown.
		//objectsToDelete.add(saved);
		//GenericAsset[] children = saved.getAssets();
		//if (children != null)
		//{
		//    for (int i = 0; i < children.length; ++i)
		//    {
		//        GenericAsset child = children[i];
		//        if (child != null)
		//        {
		//            addAssetsToDeletion(child);
		//        }
		//    }
		//}
    }
}