/*
 * Created on Oct 4, 2006
 */
package com.ibm.rpm.client.junit.asset.testcases.fixes;

import junit.framework.Assert;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

public class TestDefect50263 extends ClientTestController
{
    protected Asset mainAsset;

    protected AssetFolder parentAssetFolder = null;

    protected AssetScope mainScope;

    private static final int NB_MAX_LEVELS = 2;

    private static final int NB_ASSETS_BY_LEVEL = 1;

    // todo: this should be removed once we have it in an organized place
    public static final int DEPRECIATION_MANUAL = 0;

    public static final int DEPRECIATION_LINEAR = 1;

    private AssetScope parentScope = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        parentScope = AssetFolderFixture.createScope(1, NB_MAX_LEVELS, 0, 0,
            null);
        mainScope = AssetFixture.createScope(1, NB_MAX_LEVELS,
            NB_ASSETS_BY_LEVEL, parentScope);
    }

    public void testDocumentFolderLoadFromXPath() throws Exception
    {
        mainScope = new AssetScope();
        DocumentScope docScope = new DocumentScope();
        docScope.setChildren(new DocumentScope());
        mainScope.setDocumentFolder(docScope);
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        
        loadFromXpath("/Asset[@id='"
            + mainAsset.getID() + "']/documentFolder",null);
        
//        AssetFixture.compare(loaded, saved, mainScope);
        return;
    }

    public void testAssignedScorecardLoadFromXPath() throws Exception
    {
        Asset assignedAsset = prepareScorecardAssignment(ReloadType.ReloadResult);

        RPMObject[] rpmObjects = loadFromXpath("/Asset[@id='"
                + assignedAsset.getID() + "']/AssignedScorecard", mainScope
            .getAssignedScorecard());
        Assert.assertEquals(rpmObjects.length, 1);

        PublishedScorecardFixture
            .compare(assignedAsset.getAssignedScorecard(),
                (AssignedScorecard) rpmObjects[0], mainScope
                    .getAssignedScorecard());

        PublishedScorecardFixture.deleteScorecard(assignedAsset, this);

    }

    private Asset prepareScorecardAssignment(ReloadType reloadType)
            throws Exception
    {
        mainScope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(mainScope));
        mainAsset = AssetFixture.createFixture(mainScope, this);

        Asset saved = saveAsset(mainAsset, mainScope);
        mainAsset.setID(saved.getID());

        AssetModuleScope assetModuleScope = AssetModuleFixture.createScope(
            false, false, false, false, false, false, false, true);

        AssetModule module = AssetModuleFixture.createFixture(assetModuleScope,
            this);
        module = (AssetModule) save(module, assetModuleScope).getRpmObject();

        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        Asset assignedAsset = (Asset) PublishedScorecardFixture
            .createAssignmentFixture(mainAsset, mainScope, this, reloadType,
                scorecardFolder, 1);

        return assignedAsset;
    }

    protected Asset saveAsset(Asset ast, AssetScope scope) throws Exception
    {
        // Parent needs to be in scope to be saved...
        if (scope == null)
        {
            scope = new AssetScope();
        }

        if (scope.getParent() == null)
        {
            scope.setParent(new AssetScope());
        }
        SingleResult result = save(ast, scope);

        ast = (Asset) result.getRpmObject();
        return ast;
    }
}
