/*
 * Created on January 10, 2006
 * @author mmatwiy 
 * Test cases for creation of a CrossChargeFinancials container.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.client.junit.asset.fixtures.AssetCrossChargeFinancialsFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestCrossChargeFinancialsValidation extends GenericValidation
{

    private CrossChargeFinancials crossChargeFinancials;

    public void setUp() throws Exception
    {
        super.setUp();

        crossChargeFinancials = new CrossChargeFinancials();
        crossChargeFinancials.setCostCenter(ClientCostCenterFixture
            .createSaveFixture(this));
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAsset() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        // Attempt to save the object
        crossChargeFinancials = (CrossChargeFinancials) saveMustSucceed(
            crossChargeFinancials, CrossChargeFinancials.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/CrossChargeFinancials[id='"
                + crossChargeFinancials.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof CrossChargeFinancials);

        // Check that the loaded CrossChargeFinancials is identical to the one we
        // requested to store
        AssetCrossChargeFinancialsFixture.compare(crossChargeFinancials,
            (CrossChargeFinancials) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        crossChargeFinancials.setParent(parentFolder);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                crossChargeFinancials.getClass().getName(),
                ValidationConstants.PARENT, crossChargeFinancials.getID())
        };

        verifySaveMustFail(crossChargeFinancials, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, crossChargeFinancials
                .getClass().getName(), ValidationConstants.PARENT,
                crossChargeFinancials.getID())
        };

        verifySaveMustFail(crossChargeFinancials, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        crossChargeFinancials.setCostCenter(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, crossChargeFinancials
                .getClass().getName(), ValidationConstants.COST_CENTER_FIELD,
                crossChargeFinancials.getID())
        };

        verifySaveMustFail(crossChargeFinancials, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        crossChargeFinancials.setCost(new Double(ValidationConstants.COST_MAX));
        crossChargeFinancials.setEstimatedBudget(new Double(
            ValidationConstants.ESTIMATED_BUDGET_MAX));
        crossChargeFinancials.setEstimatedActual(new Double(
            ValidationConstants.ESTIMATED_ACTUAL_MAX));
        crossChargeFinancials.setEstimatedPaid(new Double(
            ValidationConstants.ESTIMATED_PAID_MAX));
        crossChargeFinancials.setQuantity(new Double(
            ValidationConstants.FINANCIALS_QUANTITY_MAX));

        // Attempt to save the CrossChargeFinancials object
        crossChargeFinancials = (CrossChargeFinancials) saveMustSucceed(
            crossChargeFinancials, CrossChargeFinancials.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/CrossChargeFinancials[id='"
                + crossChargeFinancials.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof CrossChargeFinancials);

        // Check that the loaded CrossChargeFinancials is identical to the one we
        // requested to store
        AssetCrossChargeFinancialsFixture.compare(crossChargeFinancials,
            (CrossChargeFinancials) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        crossChargeFinancials.setCost(new Double(
            ValidationConstants.COST_MAX + 1));
        crossChargeFinancials.setEstimatedBudget(new Double(
            ValidationConstants.ESTIMATED_BUDGET_MAX + 1));
        crossChargeFinancials.setEstimatedActual(new Double(
            ValidationConstants.ESTIMATED_ACTUAL_MAX + 1));
        crossChargeFinancials.setEstimatedPaid(new Double(
            ValidationConstants.ESTIMATED_PAID_MAX + 1));
        crossChargeFinancials.setQuantity(new Double(
            ValidationConstants.FINANCIALS_QUANTITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.COST_FIELD, crossChargeFinancials
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_BUDGET_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_ACTUAL_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_PAID_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.FINANCIALS_QUANTITY_FIELD,
                    crossChargeFinancials.getID()),
        };

        verifySaveMustFail(crossChargeFinancials, null, expectedExceptions);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        // Set minimum values and verify that they are accepted
        crossChargeFinancials.setCost(new Double(ValidationConstants.COST_MIN));
        crossChargeFinancials.setEstimatedBudget(new Double(
            ValidationConstants.ESTIMATED_BUDGET_MIN));
        crossChargeFinancials.setEstimatedActual(new Double(
            ValidationConstants.ESTIMATED_ACTUAL_MIN));
        crossChargeFinancials.setEstimatedPaid(new Double(
            ValidationConstants.ESTIMATED_PAID_MIN));
        crossChargeFinancials.setQuantity(new Double(
            ValidationConstants.FINANCIALS_QUANTITY_MIN));

        // Attempt to save the CrossChargeFinancials object
        crossChargeFinancials = (CrossChargeFinancials) saveMustSucceed(
            crossChargeFinancials, CrossChargeFinancials.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/CrossChargeFinancials[id='"
                + crossChargeFinancials.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof CrossChargeFinancials);

        // Check that the loaded CrossChargeFinancials is identical to the one we
        // requested to store
        AssetCrossChargeFinancialsFixture.compare(crossChargeFinancials,
            (CrossChargeFinancials) rpmObjects[0], null);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        crossChargeFinancials.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        crossChargeFinancials.setCost(new Double(
            ValidationConstants.COST_MIN - 1));
        crossChargeFinancials.setEstimatedBudget(new Double(
            ValidationConstants.ESTIMATED_BUDGET_MIN - 1));
        crossChargeFinancials.setEstimatedActual(new Double(
            ValidationConstants.ESTIMATED_ACTUAL_MIN - 1));
        crossChargeFinancials.setEstimatedPaid(new Double(
            ValidationConstants.ESTIMATED_PAID_MIN - 1));
        crossChargeFinancials.setQuantity(new Double(
            ValidationConstants.FINANCIALS_QUANTITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.COST_FIELD, crossChargeFinancials
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_BUDGET_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_ACTUAL_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.ESTIMATED_PAID_FIELD,
                    crossChargeFinancials.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    crossChargeFinancials.getClass().getName(),
                    ValidationConstants.FINANCIALS_QUANTITY_FIELD,
                    crossChargeFinancials.getID()),
        };

        verifySaveMustFail(crossChargeFinancials, null, expectedExceptions);
    }

}
