/*
 * Created on January 10, 2006
 * @author mmatwiy 
 * Test cases for creation of a MaintenanceEstimates container.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.asset.fixtures.MaintenanceEstimatesFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestMaintenanceEstimatesValidation extends GenericValidation
{

    private MaintenanceEstimates maintenanceEstimates;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAsset() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();

        Asset parentAsset = AssetFixture.createSaveFixture(this);
        maintenanceEstimates.setParent(parentAsset);

        // Attempt to save the object
        maintenanceEstimates = (MaintenanceEstimates) saveMustSucceed(
            maintenanceEstimates, MaintenanceEstimates.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/MaintenanceEstimates[id='"
                + maintenanceEstimates.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof MaintenanceEstimates);

        // Check that the loaded MaintenanceEstimates is identical to the one we
        // requested to store
        MaintenanceEstimatesFixture.compare(maintenanceEstimates,
            (MaintenanceEstimates) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();

        // create and set invalid parent
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        maintenanceEstimates.setParent(parentFolder);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                maintenanceEstimates.getClass().getName(),
                ValidationConstants.PARENT, maintenanceEstimates.getID())
        };

        verifySaveMustFail(maintenanceEstimates, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, maintenanceEstimates
                .getClass().getName(), ValidationConstants.PARENT,
                maintenanceEstimates.getID())
        };

        verifySaveMustFail(maintenanceEstimates, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();
        
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        maintenanceEstimates.setParent(parentAsset);

        maintenanceEstimates.setActualCost(new Double(
            ValidationConstants.ACTUAL_COST_MAX));
        maintenanceEstimates
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX));
        maintenanceEstimates.setEstimatedCost(new Double(
            ValidationConstants.ESTIMATED_COST_MAX));
        maintenanceEstimates
            .setPerformedBy(getTestString(ValidationConstants.PERFORMED_BY_MAX));

        // Attempt to save the MaintenanceEstimates object
        maintenanceEstimates = (MaintenanceEstimates) saveMustSucceed(
            maintenanceEstimates, MaintenanceEstimates.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/MaintenanceEstimates[id='"
                + maintenanceEstimates.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof MaintenanceEstimates);

        // Check that the loaded MaintenanceEstimates is identical to the one we
        // requested to store
        MaintenanceEstimatesFixture.compare(maintenanceEstimates,
            (MaintenanceEstimates) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();
        
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        maintenanceEstimates.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        maintenanceEstimates.setActualCost(new Double(
            ValidationConstants.ACTUAL_COST_MAX + 1));
        maintenanceEstimates
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX + 1));
        maintenanceEstimates.setEstimatedCost(new Double(
            ValidationConstants.ESTIMATED_COST_MAX + 1));
        maintenanceEstimates
            .setPerformedBy(getTestString(ValidationConstants.PERFORMED_BY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    maintenanceEstimates.getClass().getName(),
                    ValidationConstants.ACTUAL_COST_FIELD, maintenanceEstimates
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, maintenanceEstimates
                    .getClass().getName(),
                    ValidationConstants.DESCRIPTION_FIELD, maintenanceEstimates
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    maintenanceEstimates.getClass().getName(),
                    ValidationConstants.ESTIMATED_COST_FIELD,
                    maintenanceEstimates.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, maintenanceEstimates
                    .getClass().getName(),
                    ValidationConstants.PERFORMED_BY_FIELD,
                    maintenanceEstimates.getID())
        };

        verifySaveMustFail(maintenanceEstimates, null, expectedExceptions);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();
        
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        maintenanceEstimates.setParent(parentAsset);

        // Set minimum values and verify that they are accepted
        maintenanceEstimates.setActualCost(new Double(
            ValidationConstants.ACTUAL_COST_MIN));
        maintenanceEstimates.setEstimatedCost(new Double(
            ValidationConstants.ESTIMATED_COST_MIN));

        // Attempt to save the MaintenanceEstimates object
        maintenanceEstimates = (MaintenanceEstimates) saveMustSucceed(
            maintenanceEstimates, MaintenanceEstimates.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/MaintenanceEstimates[id='"
                + maintenanceEstimates.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof MaintenanceEstimates);

        // Check that the loaded MaintenanceEstimates is identical to the one we
        // requested to store
        MaintenanceEstimatesFixture.compare(maintenanceEstimates,
            (MaintenanceEstimates) rpmObjects[0], null);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        maintenanceEstimates = new MaintenanceEstimates();
        
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        maintenanceEstimates.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        maintenanceEstimates.setActualCost(new Double(
            ValidationConstants.ACTUAL_COST_MIN - 1));
        maintenanceEstimates.setEstimatedCost(new Double(
            ValidationConstants.ESTIMATED_COST_MIN - 1));
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    maintenanceEstimates.getClass().getName(),
                    ValidationConstants.ACTUAL_COST_FIELD, maintenanceEstimates
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                    maintenanceEstimates.getClass().getName(),
                    ValidationConstants.ESTIMATED_COST_FIELD,
                    maintenanceEstimates.getID())
        };

        verifySaveMustFail(maintenanceEstimates, null, expectedExceptions);
    }

}
