/*
 * Created on Jul 12, 2005
 */
package com.ibm.rpm.client.junit.clientcostcenters.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;

/**
 * @author Ritu, Karim
 */
public class DivisionFixture extends RPMObjectFixture
{
	private static final int DEFAULT_NB_CLIENT_COST_CENTERS = 1;
	
    public static Division createSaveFixture(int nbClientCostCenters, ClientTestController ctc) throws Exception {

    	ClientCostCenterScope scope = createScope(DEFAULT_NB_CLIENT_COST_CENTERS);
    	Division division = createFixture(DEFAULT_NB_CLIENT_COST_CENTERS, scope, ctc);

        //Save Division
    	division = (Division) save(ctc, division, scope);

        return division;

    }
	
	public static Division createFixture(int nbClientCostCenters, ClientCostCenterScope scope, ClientTestController ctc) throws Exception
    {
    	return createDivision(random(), nbClientCostCenters, scope, ctc);
    }
	
    private static Division createDivision(int index, int nbClientCostCenters, ClientCostCenterScope scope, ClientTestController ctc)throws Exception
	{
    	Division division = new Division();

    	HelperTestUtilities.setByIntrospection(division, index, skipSetters);

    	if(scope!=null && scope.getClientCostCenters()!=null){
    		ClientCostCenter childClientCostCenter = null;
    		ClientCostCenter[] childClientCostCenters = new ClientCostCenter[nbClientCostCenters]; 
    		for(int i=1;i<=nbClientCostCenters;i++){
    			childClientCostCenter = ClientCostCenterFixture.createFixture(scope.getClientCostCenters(), ctc);
    			childClientCostCenters[i-1] = childClientCostCenter;
    		}
    		division.setClientCostCenters(childClientCostCenters);
    	}
    	return division;
    }
    
    //update fields
    public static void updateFixtureFields(Division division, ClientCostCenterScope scope) throws Exception
    {
    	updateFields(division, random(), scope);
    }
    
    private static void updateFields(Division division, int index, ClientCostCenterScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(division, index, skipSetters);
		if (scope != null && scope.getClientCostCenters() != null) {
			for(int i=0;i<division.getClientCostCenters().length;i++){
				ClientCostCenterFixture.updateFixture(division.getClientCostCenters()[i], scope.getClientCostCenters());
			}
		}
    }
    
    //update structure
    public static void updateFixtureStructure(Division division, int deltaNbClientCostCenters, ClientCostCenterScope scope, ClientTestController ctc)throws Exception{
    	updateStructure(division, deltaNbClientCostCenters, scope, ctc);
    }
    
    private static void updateStructure(Division division, int deltaNbClientCostCenters, ClientCostCenterScope scope, ClientTestController ctc)throws Exception{
		if (scope != null && scope.getClientCostCenters() != null) {
			if(deltaNbClientCostCenters>0){//add new elements
	    		ClientCostCenter newClientCostCenter = null;
	    		ClientCostCenter[] newClientCostCenters = new ClientCostCenter[deltaNbClientCostCenters]; 
		    	for(int i=0;i<deltaNbClientCostCenters;i++){
		    		newClientCostCenter = ClientCostCenterFixture.createFixture(scope.getClientCostCenters(), ctc);
		    		newClientCostCenter.setParent(division);
		    		newClientCostCenters[i] = newClientCostCenter;
		    	}
		    	
		    	ArrayList temp = mergeArrays(division.getClientCostCenters(), newClientCostCenters);
		    	ClientCostCenter[] totalClientCostCenters = new ClientCostCenter[temp.size()];
		    	temp.toArray(totalClientCostCenters);
		    	
		    	division.setClientCostCenters(totalClientCostCenters);		    	
			}
		}
    }
	
	
    public static void compare(Division d1, Division d2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(d1, d2);

        if (scope == null)
            return;

        //validate children
        if (scope.getClientCostCenters() != null)
        {
        	ClientCostCenterScope s = scope.getClientCostCenters();
            if (s != null)
            {
                s.setParent(null);
            }

            ClientCostCenterFixture.compare(d1.getClientCostCenters(), d2.getClientCostCenters(), s);
        }

        //validate parents
        if (scope.getParent() != null)
        {
        	ClientCostCenterScope s = (ClientCostCenterScope)scope.getParent();
            if (s != null)
            {
                s.setDivisions(null);
            }
            FolderFixture.compare((ClientFolder) d1.getParent(),
                    (ClientFolder) d2.getParent(), s);
        }
    }

    public static void compare(Division[] dArray1, Division[] dArray2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
    	sortWithComparator(dArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(dArray2, RPMObjectNameAndIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(dArray1, dArray2);
        if(dArray1 == null) return;

        for (int i = 0; i < dArray1.length; i++)
        {
            DivisionFixture.compare(dArray1[i], dArray2[i], scope);
        }
    }
    
    public static ClientCostCenterScope createScope(int nbClientCostCenters){
    	ClientCostCenterScope scope = new ClientCostCenterScope();
    	if(nbClientCostCenters>0){
    		ClientCostCenterScope childScope = ClientCostCenterFixture.createScope(true, true);
    		childScope.setParent(scope);
    		scope.setClientCostCenters(childScope);
    	}
    	return scope;
    }
    
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }
    

}
