package com.ibm.rpm.client.junit.clientcostcenters.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.CostCenterState;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientModuleFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.scope.ClientModuleScope;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;

/**
 * @author karim
 *  
 */
public class TestClientModule extends ModuleTestController
{

    public TestClientModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = ClientModuleFixture.createFullScope();
        moduleContaineName = "CLIENTMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testXPathAllCustomFields() throws Exception
    {
        ClientModule module = (ClientModule) createFixture(mainScope);
        save(module, mainScope);

        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(1);
        RPMObject[] objs = loadFromXpath(
            "/ClientModule/CustomFieldCategory/CustomField", scope);
        assertNotNull(objs);
    }

    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {

        ClientModule module = (ClientModule) aModule;
        ClientModuleScope scope = (ClientModuleScope) aScope;

        ClientModule loadedModule = new ClientModule();

        if (scope != null)
        {

            if (scope.getClients() != null)
            {
                //Clients
                GenericClient[] clients = (GenericClient[]) loadElements(module
                    .getClients(), scope.getClients(), GenericClient.class);
                loadedModule.setClients(clients);
            }

            if (scope.getAttributeCategories() != null)
            {
                //AttributeCategories
                AttributeCategory[] attributeCategories = loadAttributeCategories(
                    module.getAttributeCategories(), scope
                        .getAttributeCategories());
                loadedModule.setAttributeCategories(attributeCategories);
            }

            if (scope.isClientRtfs())
            {
                //ClientRtfs
                ClientRTF[] clientRtfs = (ClientRTF[]) loadRtfs(module
                    .getClientRtfs(), scope.isClientRtfs(), ClientRTF.class);
                loadedModule.setClientRtfs(clientRtfs);
            }

            if (scope.isCostCenterStages())
            {
                //costCenterStages
                CostCenterState[] costCenterStages = (CostCenterState[]) loadDatafields(
                    module.getCostCenterStages(), scope.isCostCenterStages(),
                    CostCenterState.class);
                loadedModule.setCostCenterStages(costCenterStages);
            }

            if (scope.getCustomFieldCategories() != null)
            {
                //CustomFields
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) loadElements(
                    module.getCustomFieldCategories(), scope
                        .getCustomFieldCategories(), CustomFieldCategory.class);
                loadedModule.setCustomFieldCategories(customFieldCategories);
            }
        }

        return loadedModule;
    }

    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {

        ClientModule module = (ClientModule) aModule;
        ClientModuleScope scope = (ClientModuleScope) aScope;

        //delete the element
        if (module != null && scope != null)
        {
            if (module.getClients() != null && scope.getClients() != null)
            {
                delete(module.getClients());
            }
            if (module.getAttributeCategories() != null
                    && scope.getAttributeCategories() != null)
            {
                delete(module.getAttributeCategories());
            }
            if (module.getCustomFieldCategories() != null
                    && scope.getCustomFieldCategories() != null)
            {
                delete(module.getCustomFieldCategories());
            }

            //reload the deleted element
            ClientModule deletedModule = (ClientModule) loadDeletedModule(
                module, scope);

            //Validate the deleted element
            if (scope.getClients() != null)
            {
                assertNull(deletedModule.getClients());
            }
            if (scope.getAttributeCategories() != null)
            {
                assertNull(deletedModule.getAttributeCategories());
            }
            if (scope.getCustomFieldCategories() != null)
            {
                assertNull(deletedModule.getCustomFieldCategories());
            }
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return ClientModuleFixture.createFixture((ClientModuleScope) scope,
            this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return ClientModuleFixture.updateFixture((ClientModule) module,
            (ClientModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        ClientModuleFixture.compare((ClientModule) module,
            (ClientModule) returnedModule, (ClientModuleScope) scope);
    }
}