package com.ibm.rpm.client.junit.clientcostcenters.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.DivisionFixture;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;


/**
 * @author karim
 *
 */
public class TestDivision extends ClientTestController
{

    private ClientCostCenterScope mainScope;
    private static final int DELTA_NB_CLIENT_COST_CENTERS = 2;

    public TestDivision(String testName){
        setName(testName);
    }
	
    private static int nbClientCostCenters = 2;
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		mainScope = DivisionFixture.createScope(nbClientCostCenters);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null,true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope,true);
    }
  
    private void insert(ClientCostCenterScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a Division
        Division  division = insertDivision(scope, loadFromId);
    	
    	//delete the Division
    	deleteDivision(division, loadFromId);    	
    }
    
    private Division insertDivision(ClientCostCenterScope scope, boolean loadFromId) throws Exception
    {
    	Division division = DivisionFixture.createFixture(nbClientCostCenters, scope, this);
    	
        //Insert the Division into the DB
        SingleResult result =  save(division,scope);
        
        //compare the returned Division
        Division returnedDivision = (Division)result.getRpmObject(); 
        DivisionFixture.compare(division, returnedDivision, scope);
        
        //reload the Division and compare it
        returnedDivision = (Division)load(returnedDivision, scope, loadFromId);
        DivisionFixture.compare(division, returnedDivision, scope);
        
        //return the Division
        return returnedDivision;
    }
    
    
    private void deleteDivision(Division division, boolean loadFromId)throws Exception
	{    	   	
        //delete the Division
       	delete(division);
    	
        //reload the deleted Division
       	Division deletedDivision = loadDeletedDivision(division, loadFromId);
        
        //Validate the deleted Division
        assertNull(deletedDivision);
    }
    
    private RPMObject load(Division division, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new Division();
            obj.setID(division.getID());
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/Division[id='"+ division.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
    private Division loadDeletedDivision(Division division, boolean loadFromId)throws Exception{
        try{
        	division = (Division)load(division, null, loadFromId);
           	return division;
        }catch (Exception e){
            //Division does not exist
        }
        return null;
    }
    
    private void update(ClientCostCenterScope costCenterScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a division
    	Division division = insertDivision(costCenterScope, loadFromId);
    	
        //update division
    	division = updateDivisionFields(division, costCenterScope, loadFromId);
    	division = updateDivisionStructure(division, costCenterScope, loadFromId);
    	
    	//delete the folder
    	deleteDivision(division, loadFromId);
    }
	
    
    
    private Division updateDivisionFields(Division division, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	return updateDivision(division, scope, loadFromId, false);
	}
    
    private Division updateDivisionStructure(Division division, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	return updateDivision(division, scope, loadFromId, true);
	}

    private Division updateDivision(Division division, ClientCostCenterScope scope, boolean loadFromId, boolean updateStructure) throws Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements    	
    		DivisionFixture.updateFixtureStructure(division,DELTA_NB_CLIENT_COST_CENTERS,scope, this);
    		
    	} else {
    		//only updates the fields
    		DivisionFixture.updateFixtureFields(division, scope);
    	}

        //Insert the updated folder into the DB
        SingleResult result =  save(division,scope);
        
        //compare the returned folder
        Division returnedDivision = (Division)result.getRpmObject();
        DivisionFixture.compare(division, returnedDivision, scope);
        
        //reload the folder and compare it
        returnedDivision = (Division)load(returnedDivision, scope, loadFromId);
        DivisionFixture.compare(division, returnedDivision, scope);
        
        //return the folder
        return returnedDivision;
	}
}
