package com.ibm.rpm.client.junit.clientcostcenters.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.FolderFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 *
 */
public class TestFolder extends ClientTestController 
{

    private ClientCostCenterScope mainScope;
    //TODO: Confirm these values by couple of runs
    private static final int NB_MAX_LEVELS = 2;
    private static final int NB_FOLDERS_BY_LEVEL = 1;
    private static final int NB_DIVISIONS_BY_LEVEL = 1;
    private static final int NB_CLIENT_COSTCENTERS_BY_LEVEL = 1;
    
    
    private static final int DELTA_FOLDERS_BY_LEVEL = 1;
    private static final int DELTA_DIVISIONS_BY_LEVEL = 1;
    private static final int DELTA_CLIENTCOSTCENTERS_BY_LEVEL = 1;
    
       public TestFolder(String testName){
        setName(testName);
    }
	
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();
		mainScope = FolderFixture.createScope(1,NB_MAX_LEVELS,NB_FOLDERS_BY_LEVEL, NB_DIVISIONS_BY_LEVEL, NB_CLIENT_COSTCENTERS_BY_LEVEL);//.createScope(nbClientCostCenters);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }
    
    private void insert(ClientCostCenterScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a AssetFolder
        ClientFolder folder = insertFolder(scope, loadFromId);
    	
    	//delete the Division
    	deleteFolder(folder, loadFromId);    	
    }
    
    private ClientFolder insertFolder(ClientCostCenterScope scope, boolean loadFromId) throws Exception
    {
    	ClientFolder folder = FolderFixture.createFixture(NB_FOLDERS_BY_LEVEL, NB_DIVISIONS_BY_LEVEL, NB_CLIENT_COSTCENTERS_BY_LEVEL,scope, this);
    	
        //Insert the AssetFolder into the DB
        SingleResult result =  save(folder,scope);
        
        //compare the returned folder
        ClientFolder returnedFolder = (ClientFolder)result.getRpmObject(); 
        FolderFixture.compare(folder, returnedFolder, scope);
        
        //reload the AssetFolder and compare it
        returnedFolder = (ClientFolder)load(returnedFolder, scope, loadFromId);
        FolderFixture.compare(folder, returnedFolder, scope);
        
        //return the AssetFolder
        return returnedFolder;
    }
    
    
    private void update(ClientCostCenterScope costCenterScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a AssetFolder
    	ClientFolder folder = insertFolder(costCenterScope, loadFromId);
    	
        //update the folder
    	folder = updateFolderFields(folder, costCenterScope, loadFromId);
    	folder = updateFolderStructure(folder, costCenterScope, loadFromId);
    	
    	//delete the folder
    	deleteFolder(folder, loadFromId);
    }
    private ClientFolder updateFolderFields(ClientFolder folder, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	return updateFolder(folder, scope, loadFromId, false);
	}
    
    private ClientFolder updateFolderStructure(ClientFolder folder, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	return updateFolder(folder, scope, loadFromId, true);
	}

    private ClientFolder updateFolder(ClientFolder folder, ClientCostCenterScope scope, boolean loadFromId, boolean updateStructure) throws Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements    	
    		FolderFixture.updateFixtureStructure(folder,DELTA_FOLDERS_BY_LEVEL,DELTA_DIVISIONS_BY_LEVEL,DELTA_CLIENTCOSTCENTERS_BY_LEVEL,scope, this);
    	} else {
    		//only updates the fields
    		FolderFixture.updateFixtureFields(folder, scope);
    	}

        //Insert the updated folder into the DB
        SingleResult result =  save(folder,scope);
        
        //compare the returned folder
        ClientFolder returnedFolder = (ClientFolder)result.getRpmObject();
        FolderFixture.compare(folder, returnedFolder, scope);
        
        //reload the folder and compare it
        returnedFolder = (ClientFolder)load(returnedFolder, scope, loadFromId);
        FolderFixture.compare(folder, returnedFolder, scope);
        
        //return the folder
        return returnedFolder;
	}
    
    private void deleteFolder(ClientFolder folder, boolean loadFromId)throws Exception
	{    	   	
        //delete the AssetFolder
       	delete(folder);
    	
        //reload the deleted Division
       	ClientFolder deletedFolder = loadDeletedFolder(folder, loadFromId);
        
        //Validate the deleted Division
        assertNull(deletedFolder);
    }
    private ClientFolder loadDeletedFolder(ClientFolder folder, boolean loadFromId)throws Exception{
        try{
        	folder = (ClientFolder)load(folder, null, loadFromId);
           	return folder;
        }catch (Exception e){
            //AssetFolder does not exist
        }
        return null;
    }
    private RPMObject load(ClientFolder folder, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new ClientFolder();
            obj.setID(folder.getID());
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/CLIENTFOLDER[id='"+ folder.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
}
