/*
 * Created on Oct 26, 2005
 */
package com.ibm.rpm.client.junit.clientcostcenters.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.DivisionFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.FolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.clientcostcenters.constants.ValidationConstants;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;

/**
 * @author mmatwiy Test cases for creation of a client / cost center and
 *         parents.
 */
public class TestClientCostCenterValidation extends GenericValidation
{

    private ClientCostCenter clientCostCenter;

    private ClientFolder parentFolder;

    private Division parentDivision;

    public void setUp() throws Exception
    {
        super.setUp();

        parentFolder = FolderFixture.createSaveFixture(1, this);
        parentDivision = DivisionFixture.createSaveFixture(1, this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        clientCostCenter = ClientCostCenterFixture
            .createSaveFixture(null, this);
        clientCostCenter.setParent(null);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testInsertWithParentFolder() throws AssertionError, Exception
    {
        clientCostCenter = ClientCostCenterFixture
            .createSaveFixture(null, this);
        clientCostCenter.setParent(parentFolder);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        clientCostCenter = new ClientCostCenter();

        // set mandatory field
        clientCostCenter.setName("test");

        // create and set invalid parent
        Pool pool = PoolFixture.createSaveFixture(this);
        clientCostCenter.setParent(pool);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, clientCostCenter
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                clientCostCenter.getID())
        };

        verifySaveMustFail(clientCostCenter, null, expectedExceptions);
    }

    public void testInsertWithNestedParentFolder() throws AssertionError,
            Exception
    {
        clientCostCenter = ClientCostCenterFixture
            .createSaveFixture(null, this);
        ClientFolder nestedParentFolder = FolderFixture.createSaveFixture(1,
            this);

        ClientFolder[] clientFolders =
        {
            nestedParentFolder
        };
        parentFolder.setFolders(clientFolders);

        clientCostCenter.setParent(parentFolder);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testInsertWithParentDivision() throws AssertionError, Exception
    {
        clientCostCenter = ClientCostCenterFixture
            .createSaveFixture(null, this);
        clientCostCenter.setParent(parentDivision);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testInsertWithParentDivisionInFolder() throws AssertionError,
            Exception
    {
        clientCostCenter = ClientCostCenterFixture
            .createSaveFixture(null, this);

        parentDivision.setParent(parentFolder);
        clientCostCenter.setParent(parentDivision);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        clientCostCenter = new ClientCostCenter();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, clientCostCenter
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                clientCostCenter.getID())
        };

        verifySaveMustFail(clientCostCenter, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        clientCostCenter = new ClientCostCenter();
        clientCostCenter.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, clientCostCenter
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                clientCostCenter.getID())
        };

        verifySaveMustFail(clientCostCenter, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        clientCostCenter = new ClientCostCenter();
        clientCostCenter.setName(getTestString(ValidationConstants.NAME_MAX));
        clientCostCenter
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX));
        clientCostCenter.setDefaultFlag(Boolean.TRUE);

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        clientCostCenter = new ClientCostCenter();

        // Set field values with maxLength and verify that they are accepted
        clientCostCenter.setName(getTestString(ValidationConstants.NAME_MAX));
        clientCostCenter
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX));

        // Attempt to save the ClientCostCenter object
        clientCostCenter = (ClientCostCenter) saveMustSucceed(clientCostCenter,
            ClientCostCenter.class, null);

        // Load the inserted clientCostCenter from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientCostCenter[id='"
                + clientCostCenter.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientCostCenter);

        // Check that the loaded ClientCostCenter is identical to the one we
        // requested to store
        ClientCostCenterFixture.compare(clientCostCenter,
            (ClientCostCenter) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        clientCostCenter = new ClientCostCenter();

        // Set field values with maxLength+1 and verify that they are not
        // accepted
        clientCostCenter
            .setName(getTestString(ValidationConstants.NAME_MAX + 1));
        clientCostCenter
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, clientCostCenter
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    clientCostCenter.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, clientCostCenter
                    .getClass().getName(),
                    ValidationConstants.DESCRIPTION_FIELD, clientCostCenter
                        .getID())
        };

        verifySaveMustFail(clientCostCenter, null, expectedExceptions);
    }
}
