package com.ibm.rpm.client.junit.clientcostcenters.testcases.validation;

/**
 * @author mmatwiy Test cases for creation of a client folder.
 */
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.FolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.clientcostcenters.constants.ValidationConstants;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;

public class TestClientFolderValidation extends GenericValidation
{
    private ClientFolder folder;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertFolderWithInvalidParent() throws AssertionError,
            Exception
    {
        // create and set invalid parent
        Pool pool = PoolFixture.createSaveFixture(this);

        ClientFolder folder = new ClientFolder();
        folder.setName("test");
        folder.setParent(pool);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, folder.getClass()
                .getName(), ValidationConstants.PARENT_FIELD, folder.getID())
        };

        verifySaveMustFail(folder, null, expectedExceptions);

    }

    public void testInsertFolderSuccess() throws AssertionError, Exception
    {
        folder = new ClientFolder();
        folder.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the ClientFolder object
        folder = (ClientFolder) saveMustSucceed(folder, ClientFolder.class,
            null);

        // Load the inserted ClientFolder from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ClientFolder[id='"
                + folder.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ClientFolder);

        // Check that the loaded ClientFolder is identical to the one we
        // requested to store
        FolderFixture.compare(folder, (ClientFolder) rpmObjects[0], null);
    }

    public void testFailureValidationMaxFolder() throws AssertionError,
            Exception
    {
        ClientFolder folder = new ClientFolder();

        // Set field values with maxLength+1 and verify that they are not
        // accepted
        folder.setName(getTestString(ValidationConstants.NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, folder.getClass()
                .getName(), ValidationConstants.NAME_FIELD, folder.getID())
        };

        verifySaveMustFail(folder, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        ClientFolder folder = new ClientFolder();
        folder.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, folder.getClass()
                .getName(), ValidationConstants.NAME_FIELD, folder.getID())
        };

        verifySaveMustFail(folder, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        ClientFolder folder = new ClientFolder();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, folder
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                folder.getID())
        };

        verifySaveMustFail(folder, null, expectedExceptions);
    }

}