package com.ibm.rpm.client.junit.clientcostcenters.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.DivisionFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.clientcostcenters.constants.ValidationConstants;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;

/**
 * @author mmatwiy 
 * Test cases for creation of a division.
 */
public class TestDivisionValidation extends GenericValidation
{
    private Division division;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertDivisionWithInvalidParent() throws AssertionError,
            Exception
    {
        // create and set invalid parent
        Pool pool = PoolFixture.createSaveFixture(this);

        Division division = new Division();
        division.setName("test");
        division.setParent(pool);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, division
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                division.getID())
        };

        verifySaveMustFail(division, null, expectedExceptions);
    }

    public void testInsertDivisionSuccess() throws AssertionError, Exception
    {
        division = new Division();
        division.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the Division object
        division = (Division) saveMustSucceed(division, Division.class, null);

        // Load the inserted Division from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Division[id='"
                + division.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Division);

        // Check that the loaded Division is identical to the one we
        // requested to store
        DivisionFixture.compare(division, (Division) rpmObjects[0], null);
    }

    public void testFailureValidationMaxDivision() throws AssertionError,
            Exception
    {
        Division division = new Division();

        // Set field values with maxLength+1 and verify that they are not
        // accepted
        division.setName(getTestString(ValidationConstants.NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, division
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                division.getID())
        };

        verifySaveMustFail(division, null, expectedExceptions);
    }
    

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Division division = new Division();
        division.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, division.getClass()
                .getName(), ValidationConstants.NAME_FIELD, division.getID())
        };

        verifySaveMustFail(division, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        Division division = new Division();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, division
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                division.getID())
        };

        verifySaveMustFail(division, null, expectedExceptions);
    }

}