/*
 * Created on Nov 14, 2006
 */
package com.ibm.rpm.client.junit.communications.fixtures;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;

public class AbstractTestGenericCommunication extends ClientTestController
{
    protected GenericCommunicationScope mainScope;

    protected RPMObject load(GenericCommunication genericCommunication,
            GenericCommunicationScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            // Load from id
            RPMObject obj = new Notification();
            obj.setID(genericCommunication.getID());
            return loadFromID(obj, scope);
        }
        // Load from xpath
        RPMObject[] objs = loadFromXpath("/"
                + StringUtil.getShortClassName(genericCommunication.getClass())
                + "[id='" + genericCommunication.getID() + "']", scope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }

    protected RPMObject load(WorkflowResponse workflowResponse,
            WorkflowResponseScope scope, boolean loadFromId) throws Exception
    {
        if (loadFromId)
        {
            // Load from id
            RPMObject obj = new WorkflowResponse();
            obj.setID(workflowResponse.getID());
            return loadFromID(obj, scope);
        }
        // Load from xpath
        RPMObject[] objs = loadFromXpath("/WorkflowResponse[id='"
                + workflowResponse.getID() + "']", scope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }

    protected GenericCommunication updateReadAndFolder(
            GenericCommunication object, boolean loadFromId) throws Exception
    {
        GenericCommunication genericCommunication = GenericCommunicationFixture.createSaveFixture(
            null, object, mainScope, this);

        if (genericCommunication instanceof WorkflowStepNotification)
        {
            getApplicationInterface().suspendWorkflow(
                sessionid,
                ((WorkflowStepNotification) genericCommunication).getRunningWorkflowProcess(),
                null);
        }

        genericCommunication.getResponses()[0].setFolder(CommunicationsFolder.Archived);
        genericCommunication.getResponses()[0].setRead(Boolean.TRUE);

        SingleResult result = getApplicationInterface().updateReadAndFolder(
            sessionid, genericCommunication.getResponses()[0],
            mainScope.getResponses());

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(result.getRpmObject());

        assertTrue(((WorkflowResponse) result.getRpmObject()).getFolder() == CommunicationsFolder.Archived);
        assertTrue(((WorkflowResponse) result.getRpmObject()).getRead() == Boolean.TRUE);

        // Reload the container and compare it
        WorkflowResponse returnedWorkflowResponse = (WorkflowResponse) load(
            (WorkflowResponse) result.getRpmObject(), mainScope.getResponses(),
            loadFromId);
        assertTrue(returnedWorkflowResponse.getFolder() == CommunicationsFolder.Archived);
        assertTrue(returnedWorkflowResponse.getRead() == Boolean.TRUE);

        return (GenericCommunication) result.getRpmObject().getParent();
    }

    protected SingleResult saveWorkflowResponse(WorkflowResponse response,
            WorkflowResponseScope scope) throws Exception
    {
        return getApplicationInterface().saveWorkflowResponse(sessionid, response,
            scope);
    }
    
    protected SingleResult sendWorkflowResponse(WorkflowResponse response,
            WorkflowResponseScope scope) throws Exception
    {
        return getApplicationInterface().sendWorkflowResponse(sessionid, response,
            scope);
    }
}
