/*
 * Created on Aug 22, 2006
 */
package com.ibm.rpm.client.junit.communications.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

/**
 * @author dominiqu
 */
public class GenericCommunicationFixture extends RPMObjectFixture
{
    protected static ArrayList gcSkipSetters, gcSkipGetters;

    static
    {
        // Setters
        gcSkipSetters = new ArrayList();
        gcSkipSetters.addAll(skipSetters);
        // Read only fields
        gcSkipSetters.add("setAttachmentName");
        gcSkipSetters.add("setAuthor");
        gcSkipSetters.add("setProjectName");
        gcSkipSetters.add("setReceivedDate");
        gcSkipSetters.add("setName");

        // Getters
        gcSkipGetters = new ArrayList();
        gcSkipGetters.addAll(skipGetters);
        gcSkipGetters.add("getAttachmentName");
        gcSkipGetters.add("getAuthor");
        gcSkipGetters.add("getProjectName");
        gcSkipGetters.add("getReceivedDate");
        gcSkipGetters.add("getName");
    }

    private static final int DEFAULT_NB_PARTICIPANTS = 2;

    public static final String PROCESS_NAME = "ProjectProcess1";

    public static final String WORKFLOW1GUY_USERNAME = "Workflow1 Guy";
    
    public static final String WORKFLOW1GUY_PASSWORD = "rpm";

    public static final int WORKFLOW1GUY_EETCHours = 8;

    public static GenericCommunicationScope createFullScope(
            boolean fullWorkflowResponseScope)
    {
        GenericCommunicationScope scope = new GenericCommunicationScope();
        scope.setWorkflowAttachment(true);
        scope.setAuthor(true);
        scope.setRunningWorkflowProcess(true);

        if (fullWorkflowResponseScope)
        {
            scope.setResponses(WorkflowResponseFixture.createFullScope());
        }
        else
        {
            scope.setResponses(new WorkflowResponseScope());
        }

        scope.setParent(new RPMObjectScope());

        return scope;
    }

    public static GenericCommunication createSaveFixture(
            WorkflowAttachment attachment, GenericCommunication object,
            GenericCommunicationScope scope, ClientTestController ctc)
            throws Exception
    {
        if (object instanceof WorkflowStepNotification)
        {
            return createSaveWorkflowStepNotificationFixture(scope, ctc);
        }
        else
        {
            object = createFixture(attachment, object, scope, ctc);

            // Save means actually sending the communication

            Resource[] resource = new Resource[object.getResponses().length];
            for (int i = 0; i < resource.length; ++i)
            {
                if (object.getResponses()[i] != null)
                {
                    resource[i] = object.getResponses()[i].getResource();
                }
            }

            SingleResult result = null;

            if (object instanceof StaffingRequest)
            {
                ResourceArray resourceArray = new ResourceArray();
                resourceArray.setResourceArray(resource);
                result = ctc.getApplicationInterface().sendStaffingRequest(
                    ctc.getSessionID(), scope,
                    object.getWorkflowAttachment().getAttachment(),
                    resourceArray, object.getSubject(), object.getComments(),
                    ((StaffingRequest) object).getDueDate());
            }
            else if (object instanceof Notification)
            {
                ResourceArray resourceArray = new ResourceArray();
                resourceArray.setResourceArray(resource);
                result = ctc.getApplicationInterface().sendNotification(
                    ctc.getSessionID(), scope,
                    object.getWorkflowAttachment().getAttachment(),
                    resourceArray, object.getSubject(), object.getComments());
            }
            return (GenericCommunication) result.getRpmObject();
        }
    }

    public static GenericCommunication createFixture(
            WorkflowAttachment attachment, GenericCommunication object,
            GenericCommunicationScope scope, ClientTestController ctc)
            throws Exception
    {
        Assert.assertFalse(
            "The method createFixture cannot be used to create a WorkflowStepNotification",
            object instanceof WorkflowStepNotification);

        if (object == null)
        {
            // Create a notification by default
            object = new Notification();
        }

        HelperTestUtilities.setByIntrospection(object, random(), gcSkipSetters);

        if (scope == null)
        {
            return object;
        }

        // Note: attachment is mandatory on creation
        if (scope.isWorkflowAttachment())
        {
            if (attachment == null)
            {
                // Always set an attachment of type project by default
                Project attachmentProject = (Project) WorkElementFixture.createSaveFixture(
                    new Project(), null, ctc);
                attachment = new WorkflowAttachment();
                attachment.setAttachment(attachmentProject);
            }
            attachment.setParent(object);
            object.setWorkflowAttachment(attachment);
        }

        // Note: Author is mandatory on creation
        if (scope.isAuthor())
        {
            Resource admin = new Resource();
            admin.setID("PMO_SUPERVISOR");
            admin = (Resource) ctc.loadFromID(admin, null);
            object.setAuthor(admin);
        }

        if (scope.getResponses() != null)
        {
            WorkflowResponse[] responses = new WorkflowResponse[DEFAULT_NB_PARTICIPANTS];
            for (int i = 0; i < DEFAULT_NB_PARTICIPANTS; ++i)
            {
                responses[i] = WorkflowResponseFixture.createFixture(object,
                    scope.getResponses(), ctc);
            }
            object.setResponses(responses);
        }

        return object;
    }

    public static WorkflowStepNotification createSaveWorkflowStepNotificationFixture(
            GenericCommunicationScope scope, ClientTestController ctc)
            throws Exception
    {
        WorkflowScope workflowScope = new WorkflowScope();
        workflowScope.setChildren(new WorkflowScope());

        RunningWorkflowProcessScope runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);

        WorkElementScope workOrganizationScope = new WorkElementScope();
        workOrganizationScope.setContainingProject(new WorkElementScope());
        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture.createSaveFixture(
            new WorkOrganization(), workOrganizationScope, ctc);

        // Get Workflow1 Guy
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        RPMObject[] wObjects = ctc.loadFromXpath("/Resource[username='"
                + WORKFLOW1GUY_USERNAME + "']", resourceScope);
        
        Resource workflow1Guy = null;
        if (wObjects == null || !(wObjects[0] instanceof Resource))
        {
            // create the workflow resource
            workflow1Guy = ResourceFixture
                .createSaveFixture(resourceScope, ctc);
            workflow1Guy.getResourceSecurityRights().setCanUseWebServicesAPI(
                Boolean.TRUE);
            workflow1Guy.setUserName(WORKFLOW1GUY_USERNAME);
            workflow1Guy = (Resource) ctc.save(workflow1Guy, resourceScope)
                .getRpmObject();
            ctc.changePassword(workflow1Guy.getUserName(), "ibmrpm",
                WORKFLOW1GUY_PASSWORD);
        }
        else
        {
            workflow1Guy = (Resource) wObjects[0];
        }

        // Create a task with Reference #12345, and assign Workflow1 Guy
        // This way, Workflow1 Guy can log effort when responding

        WorkElementScope wes = new WorkElementScope();
        wes.setContainingProject(new WorkElementScope());

        Task task = new Task();
        task.setParent(workOrganization);
        task.setName(HelperTestUtilities.createName());
        task.setReferenceNumber("12345");
        task = (Task) save(ctc, task, wes);

        SecurityRole securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");

        Calendar startDate = new GregorianCalendar(2006, 1, 1);

        TaskAssignment ta = new TaskAssignment();
        ta.setAssignmentStartDate(startDate);
        ta.setEETCHours(new Double(WORKFLOW1GUY_EETCHours));

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());

        ResourceTaskAssignment rta = new ResourceTaskAssignment();
        rta.setSecurityRole(securityRole);
        rta.setResource(workflow1Guy);
        rta.setParent(task);
        rta.setTaskAssignment(ta);
        rta = (ResourceTaskAssignment) save(ctc, rta, rtaScope);

        // Assign the resource to its parent work organization too
        // The notification only gets sent to the direct participants of it
        securityRole.setID("SCPROJECTMANAGER");

        ResourceTaskAssignment rta2 = new ResourceTaskAssignment();
        rta2.setSecurityRole(securityRole);
        rta2.setResource(workflow1Guy);
        rta2.setParent(workOrganization);
        rta2 = (ResourceTaskAssignment) save(ctc, rta2, rtaScope);
        
        // TODO: replace this load once the API is able to create a
        // WorkflowProcess
        RPMObject[] objects = ctc.loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        Assert.assertNotNull(objects);
        Assert.assertTrue(objects[0] instanceof WorkflowProcess);

        // associate the WorkflowProcess to the WorkOrganization
        ctc.getApplicationInterface().associateWorkflow(ctc.getSessionID(),
            (WorkflowProcess) objects[0], workOrganization, true, true);

        // start the WorkflowProcess
        RunningWorkflowProcess runningProcess = (RunningWorkflowProcess) ctc.getApplicationInterface().startWorkflow(
            ctc.getSessionID(), (WorkflowProcess) objects[0], workOrganization,
            runningScope).getRpmObject();

        ClientTestController.assertNotNull(runningProcess);

        RPMObject[] objects2 = ctc.loadFromXpath(
            "/WorkflowStepNotification[RunningWorkflowProcess[ID='"
                    + runningProcess.getID() + "']]", scope);
        Assert.assertNotNull(objects2);
        Assert.assertTrue(objects2[0] instanceof WorkflowStepNotification);

        return (WorkflowStepNotification) objects2[0];
    }

    public static GenericCommunication updateFixture(GenericCommunication gc,
            GenericCommunicationScope scope) throws Exception
    {
        // Note: No fields in GenericCommunication can be updated.

        if (scope != null)
        {
            if (scope.getResponses() != null)
            {
                WorkflowResponse[] replies = gc.getResponses();
                for (int i = 0; i < DEFAULT_NB_PARTICIPANTS; ++i)
                {
                    replies[i] = WorkflowResponseFixture.updateFixture(replies[i]);
                }
                gc.setResponses(replies);
            }
        }

        return gc;
    }

    public static void validateReadOnlyValues(GenericCommunication gc1)
            throws AssertionFailedError, Exception
    {
        Assert.assertNotNull(gc1.getAttachmentName());
        if (gc1.getWorkflowAttachment() != null
                && gc1.getWorkflowAttachment().getAttachment() != null)
        {
            if (gc1.getWorkflowAttachment().getAttachment() instanceof WorkElement)
            {
                Assert.assertTrue(gc1.getAttachmentName().equals(
                    ((WorkElement) gc1.getWorkflowAttachment().getAttachment()).getName()));
            }
        }
        Assert.assertNotNull(gc1.getProjectName());
        Assert.assertNotNull(gc1.getReceivedDate());
        Calendar currentDate = Calendar.getInstance();
        Assert.assertTrue(gc1.getReceivedDate().get(Calendar.YEAR) == currentDate.get(Calendar.YEAR));
        Assert.assertTrue(gc1.getReceivedDate().get(Calendar.MONTH) == currentDate.get(Calendar.MONTH));
        Assert.assertTrue(gc1.getReceivedDate().get(Calendar.DAY_OF_YEAR) == currentDate.get(Calendar.DAY_OF_YEAR));
    }

    public static void validateFolderAndRead(GenericCommunication gc1,
            Resource[] resources, CommunicationsFolder folder, Boolean read)
            throws AssertionFailedError, Exception
    {
        if (gc1.getResponses() != null && resources != null)
        {
            for (int i = 0; i < gc1.getResponses().length; ++i)
            {
                for (int j = 0; j < resources.length; ++j)
                {
                    if (gc1.getResponses()[i].getResource() != null
                            && gc1.getResponses()[i].getResource().getID()
                                .equalsIgnoreCase(resources[j].getID()))
                    {
                        Assert.assertEquals("Response for resource "
                                + gc1.getResponses()[i].getResource()
                                    .getUserName()
                                + " is not in expected folder", folder, gc1
                            .getResponses()[i].getFolder());
                        Assert
                            .assertEquals(
                                "Response for resource "
                                        + gc1.getResponses()[i].getResource()
                                            .getUserName()
                                        + " does not have its read flag set to expected value",
                                read, gc1.getResponses()[i].getRead());
                        break;
                    }
                }
            }
        }
    }
    
    public static void compare(GenericCommunication gc1,
            GenericCommunication gc2, GenericCommunicationScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(gc1, gc2, gcSkipGetters);

        if (scope == null)
            return;

        // Handling attachment
        if (scope.isWorkflowAttachment())
        {
            Assert.assertTrue((gc1.getWorkflowAttachment() != null && gc2.getWorkflowAttachment() != null)
                    || (gc1.getWorkflowAttachment() == null && gc2.getWorkflowAttachment() == null));
            if (gc1.getWorkflowAttachment() != null
                    && gc1.getWorkflowAttachment().getAttachment() != null)
            {
                Assert.assertEquals(
                    gc1.getWorkflowAttachment().getAttachment().getID(),
                    gc2.getWorkflowAttachment().getAttachment().getID());
            }
        }

        // Handling author
        if (scope.isAuthor())
        {
            Assert.assertTrue((gc1.getAuthor() != null && gc2.getAuthor() != null)
                    || (gc1.getAuthor() == null && gc2.getAuthor() == null));
            if (gc1.getAuthor() != null)
            {
                Assert.assertEquals(gc1.getAuthor().getID(),
                    gc2.getAuthor().getID());
            }
        }

        // Handling replies
        if (scope.getResponses() != null)
        {
            String expectedAuthorUsername = RpmTestProperties.getProperty("rpm.test.user.name");
            WorkflowResponseFixture.compare(gc1.getResponses(),
                gc2.getResponses(), expectedAuthorUsername);
        }
    }

    public static void compare(GenericCommunication[] eArray1,
            GenericCommunication[] eArray2) throws AssertionFailedError,
            Exception
    {
        sortWithComparator(eArray1, new RPMObjectIDComparator());
        sortWithComparator(eArray2, new RPMObjectIDComparator());

        RPMObjectFixture.compare(eArray1, eArray2);
        if (eArray1 == null)
            return;

        for (int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
}
