/*
 * Created on Sep 19, 2006
 */
package com.ibm.rpm.client.junit.communications.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceRoleAssignmentFixture;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;

/**
 * @author dominiqu
 */
public class WorkflowResponseFixture extends RPMObjectFixture
{
    protected static ArrayList wrSkipSetters, wrSkipGetters;

    static
    {
        // Setters
        wrSkipSetters = new ArrayList();
        wrSkipSetters.addAll(skipSetters);
        // Read only fields
        wrSkipSetters.add("setResponseDate");
        wrSkipSetters.add("setVisible");
        wrSkipSetters.add("setRead");
        wrSkipSetters.add("setWeight");
        // This field is read only for all parents except for
        // WorkflowStepNotifications that have a Task
        // with Reference #12345 and the resource assigned to the Task
        wrSkipSetters.add("setEffort");

        // Getters
        wrSkipGetters = new ArrayList();
        wrSkipGetters.addAll(skipGetters);
        wrSkipGetters.add("getResponseDate");
        wrSkipGetters.add("getVisible");
        wrSkipGetters.add("getRead");
        wrSkipGetters.add("getWeight");
        // This field is read only for all parents except for
        // WorkflowStepNotifications that have a Task
        // with Reference #12345 and the resource assigned to the Task
        wrSkipGetters.add("getEffort");
    }

    public static WorkflowResponseScope createFullScope()
    {
        WorkflowResponseScope scope = new WorkflowResponseScope();
        scope.setResource(true);
        scope.setResponse(true);
        scope.setParent(new GenericCommunicationScope());
        return scope;
    }

    public static WorkflowResponse createSaveFixture(
            GenericCommunication parent, WorkflowResponseScope scope,
            ClientTestController ctc) throws Exception
    {
        if (parent instanceof WorkflowStepNotification)
        {
            return null;
        }

        if (scope == null)
        {
            scope = new WorkflowResponseScope();
        }

        GenericCommunicationScope parentScope = new GenericCommunicationScope();
        parentScope.setResponses(scope);

        if (parent == null)
        {
            parent = GenericCommunicationFixture.createFixture(null, null,
                GenericCommunicationFixture.createFullScope(false), ctc);
        }

        WorkflowResponse object = createFixture(parent, scope, ctc);
        parent.setResponses(new WorkflowResponse[]
        {
            object
        });

        Resource[] recipients =
        {
            object.getResource()
        };

        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        SingleResult result = null;

        if (parent instanceof StaffingRequest)
        {
            result = ctc.getApplicationInterface().sendStaffingRequest(
                ctc.getSessionID(), parentScope,
                parent.getWorkflowAttachment().getAttachment(), resourceArray,
                parent.getSubject(), parent.getComments(),
                ((StaffingRequest) parent).getDueDate());
        }
        else if (parent instanceof Notification)
        {
            result = ctc.getApplicationInterface().sendNotification(
                ctc.getSessionID(), parentScope,
                parent.getWorkflowAttachment().getAttachment(), resourceArray,
                parent.getSubject(), parent.getComments());
        }

        if (result != null)
        {
            parent = (GenericCommunication) result.getRpmObject();
        }

        return parent.getResponses()[0];
    }

    public static WorkflowResponse createFixture(GenericCommunication parent,
            WorkflowResponseScope scope, ClientTestController ctc)
            throws Exception
    {
        WorkflowResponse object = new WorkflowResponse();

        // Note: Responses are mandatory on creation, but only the resource is
        // saved
        // Also setting default values here, although they aren't used by the
        // save function
        object.setRead(Boolean.FALSE);
        object.setFolder(CommunicationsFolder.Inbox);
        object.setState(WorkflowResponseState.NotComplete);
        if (parent instanceof WorkflowStepNotification)
        {
            object.setEffort(new Integer(
                GenericCommunicationFixture.WORKFLOW1GUY_EETCHours - 1));
        }
        else
        {
            object.setEffort(new Integer(0));
        }

        object.setParent(parent);

        if (scope != null && scope.isResource())
        {
            ResourceScope resourceScope = new ResourceScope();
            resourceScope.setCalendar(new CalendarScope());
            Resource resourceManager = ResourceFixture.createSaveFixture(
                resourceScope, ctc);
            if (parent instanceof StaffingRequest)
            {
                // Resource must be a Resource Manager
                SecurityRole poolSecurityRole = new SecurityRole();
                poolSecurityRole.setID("SCRESOURCEMANAGER");
                ResourceRoleAssignment assignment = ResourceRoleAssignmentFixture.createFixture(
                    resourceManager, resourceManager.getParent(),
                    poolSecurityRole);
                ctc.save(assignment, null);
            }
            object.setResource(resourceManager);
        }

        return object;
    }

    public static WorkflowResponse updateFixture(
            WorkflowResponse workflowResponse) throws Exception
    {
        // Cannot update the Resource, but every other field that is not read
        // only can be updated

        int index = random();

        HelperTestUtilities.setByIntrospection(workflowResponse, index,
            wrSkipSetters);

        if (index % 2 == 0)
        {
            workflowResponse.setState(WorkflowResponseState.Accepted);
        }
        else
        {
            workflowResponse.setState(WorkflowResponseState.Rejected);
        }

        // Update the effort only for Workflow1 Guy
        if (workflowResponse.getParent() instanceof WorkflowStepNotification
                && workflowResponse.getResource() != null
                && workflowResponse.getResource().getUserName().compareTo(
                    GenericCommunicationFixture.WORKFLOW1GUY_USERNAME) == 0)
        {
            workflowResponse.setEffort(new Integer(index
                    % GenericCommunicationFixture.WORKFLOW1GUY_EETCHours));
        }
        else
        {
            workflowResponse.setEffort(new Integer(0));
        }

        GenericResponseTypeFixture.updateFixture(workflowResponse.getResponse());

        return workflowResponse;
    }

    public static void validateReadOnlyValues(WorkflowResponse wr1)
            throws AssertionFailedError, Exception
    {
        Assert.assertNotNull(wr1.getWeight());
        Assert.assertNotNull(wr1.getRead());
        Assert.assertNotNull(wr1.getVisible());
        Calendar currentDate = Calendar.getInstance();
        Assert.assertTrue(wr1.getResponseDate().get(Calendar.YEAR) == currentDate.get(Calendar.YEAR));
        Assert.assertTrue(wr1.getResponseDate().get(Calendar.MONTH) == currentDate.get(Calendar.MONTH));
        Assert.assertTrue(wr1.getResponseDate().get(Calendar.DAY_OF_YEAR) == currentDate.get(Calendar.DAY_OF_YEAR));
    }

    public static void compare(WorkflowResponse wr1, WorkflowResponse wr2,
            WorkflowResponseScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(wr1, wr2, wrSkipGetters);

        Assert.assertEquals("Response for resource "
                + wr1.getResource().getUserName()
                + " is not in expected folder", wr1.getFolder(), wr2
            .getFolder());
        
        // Compare fields that are only writeable
        if (wr1.getParent() instanceof WorkflowStepNotification)
        {
            Assert.assertEquals(wr1.getEffort(), wr2.getEffort());
        }

        if (scope != null && scope.isResponse())
        {
            GenericResponseTypeFixture.compare(wr1.getResponse(),
                wr2.getResponse());
        }
    }

    public static void compare(WorkflowResponse[] eArray1,
            WorkflowResponse[] eArray2) throws AssertionFailedError, Exception
    {
        String expectedAuthorUsername = RpmTestProperties.getProperty("rpm.test.user.name");
        compare(eArray1, eArray2, expectedAuthorUsername);
    }

    public static void compare(RPMObject[] eArray1, RPMObject[] eArray2,
            String expectedAuthorUsername) throws AssertionFailedError,
            Exception
    {
        sortWithComparator(eArray1, new RPMObjectIDComparator());
        sortWithComparator(eArray2, new RPMObjectIDComparator());

        if (eArray1 != null && eArray1.length == 0)
            eArray1 = null;
        if (eArray2 != null && eArray2.length == 0)
            eArray2 = null;

        RPMObject[] eArray3 = null;

        if (expectedAuthorUsername == null)
        {
            RPMObjectFixture.compare(eArray1, eArray2);
        }
        else
        {
            // The arrays will not match because of the author WorkflowResponse
            // that will have been added
            Assert.assertTrue((eArray1 == null && eArray1 == null)
                    || (eArray1 != null && eArray2 != null));

            if (eArray1 != null)
            {
                // The length of one array can be longer than the other by
                // exactly one, if one of the arrays contains a WorkflowResponse
                // for the author and one of them does not
                Assert.assertTrue((eArray1.length == eArray2.length)
                        || (eArray1.length + 1 == eArray2.length)
                        || eArray1.length == eArray2.length + 1);

                if (eArray1.length > eArray2.length)
                {
                    // Remove the author WorkflowResponse from eArray1
                    eArray3 = new RPMObject[eArray1.length];

                    // Perform the removal
                    for (int i = 0; i < eArray1.length; i++)
                    {
                        if (!(eArray1[i] != null && ((WorkflowResponse) eArray1[i]).getResource().getUserName().compareTo(
                            expectedAuthorUsername) == 0))
                        {
                            eArray3[i] = eArray1[i];
                        }
                    }
                    eArray3 = removeNullAndDeletedRPMObjects(eArray3);

                    for (int i = 0; i < eArray3.length; i++)
                    {
                        compareWithSkip(eArray3[i], eArray2[i], wrSkipGetters);
                    }
                }
                else if (eArray1.length < eArray2.length)
                {
                    // Remove the author WorkflowResponse from eArray2
                    eArray3 = new RPMObject[eArray2.length];

                    // Perform the removal
                    for (int i = 0; i < eArray2.length; i++)
                    {
                        if (!(eArray2[i] != null && ((WorkflowResponse) eArray2[i]).getResource().getUserName().compareTo(
                            expectedAuthorUsername) == 0))
                        {
                            eArray3[i] = eArray2[i];
                        }
                    }
                    eArray3 = removeNullAndDeletedRPMObjects(eArray3);

                    for (int i = 0; i < eArray3.length; i++)
                    {
                        compareWithSkip(eArray1[i], eArray3[i], wrSkipGetters);
                    }
                }
                else
                {
                    for (int i = 0; i < eArray1.length; i++)
                    {
                        compareWithSkip(eArray1[i], eArray2[i], wrSkipGetters);
                    }
                }
            }
        }
    }
}
