/*
 * Created on Oct 17, 2006
 */
package com.ibm.rpm.client.junit.communications.testcases;

import com.ibm.rpm.client.junit.communications.fixtures.AbstractTestGenericCommunication;
import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.communications.fixtures.WorkflowResponseFixture;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;

/**
 * @author dominiqu
 */
public class TestWorkflowStepNotification extends
        AbstractTestGenericCommunication
{
    WorkflowStepNotification mainWorkflowStepNotification;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = GenericCommunicationFixture.createFullScope(true);
    }

    public void tearDown() throws Exception
    {
        if (mainWorkflowStepNotification != null)
        {
            if (mainWorkflowStepNotification.getRunningWorkflowProcess() == null)
            {
                mainWorkflowStepNotification = (WorkflowStepNotification) load(
                    mainWorkflowStepNotification, mainScope, true);
            }

            RunningWorkflowProcess process = mainWorkflowStepNotification.getRunningWorkflowProcess();

            if (process == null || process.getState() == WorkflowState.Closed)
            {
                // Check if a step was created, so we can cancel that one
                // instead.
                RPMObject[] objects = loadFromXpath(
                    "/RunningWorkflowProcess[workflowprocessname='"
                            + GenericCommunicationFixture.PROCESS_NAME
                            + "' and state=\"Open\"]", null);

                if (objects != null)
                {
                    process = (RunningWorkflowProcess) objects[0];
                }
            }

            if (process != null && process.getState() != WorkflowState.Closed)
            {
                getApplicationInterface().cancelWorkflow(sessionid, process);
            }
        }

        super.tearDown();
    }

    public void testLoadWorkflowStepNotificationWithNoScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = createWorkflowStepNotification(null,
            true);
    }

    public void testLoadWorkflowStepNotificationWithNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = createWorkflowStepNotification(null,
            false);
    }

    public void testLoadWorkflowStepNotificationWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = createWorkflowStepNotification(
            mainScope, true);
    }

    public void testLoadWorkflowStepNotificationWithScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = createWorkflowStepNotification(
            mainScope, false);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = (WorkflowStepNotification) updateReadAndFolder(
            new WorkflowStepNotification(), true);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = (WorkflowStepNotification) updateReadAndFolder(
            new WorkflowStepNotification(), false);
    }

    public void testDelete() throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);

        SingleResult result = getApplicationInterface().suspendWorkflow(
            sessionid,
            mainWorkflowStepNotification.getRunningWorkflowProcess(), null);
        assertTrue(result == null || result.getErrors() == null);

        WorkflowStepNotification wsnToDelete = new WorkflowStepNotification();
        wsnToDelete.setID(mainWorkflowStepNotification.getID());

        Result result2 = delete(wsnToDelete);
        assertTrue(result2 == null || result2.getErrors() == null);

        RPMObject obj = null;
        try
        {
            obj = load(mainWorkflowStepNotification.getResponses()[0], null,
                true);
        }
        catch (RPMException e)
        {
            assertTrue(e.getErrorID() == ErrorCodes.AM_INVALID_RPMOBJECTID);
        }
        assertNull(obj);

        // Give one of the other resources the right to log into web services
        Resource resource;
        if (mainWorkflowStepNotification.getResponses()[0].getResource().getID().indexOf(
            "PMO_SUPERVISOR") == 0)
        {
            resource = mainWorkflowStepNotification.getResponses()[1].getResource();
        }
        else
        {
            resource = mainWorkflowStepNotification.getResponses()[0].getResource();
        }

        resource.getResourceSecurityRights().setCanUseWebServicesAPI(
            Boolean.TRUE);
        resource = (Resource) save(resource, null).getRpmObject();

        // Now, test that the WorkflowStepNotification can still be loaded by
        // other resources
        getAuthenticateInterface().logout(sessionid);

        performLogin(resource.getUserName(),
            GenericCommunicationFixture.WORKFLOW1GUY_PASSWORD);

        WorkflowStepNotification obj2 = (WorkflowStepNotification) load(
            mainWorkflowStepNotification, mainScope, true);
        assertNotNull(obj2);
        for (int i = 0; i < obj2.getResponses().length; ++i)
        {
            if (obj2.getResponses()[i].getVisible() == Boolean.FALSE)
            {
                // This is the administrator's WorkflowResponse, because the
                // administrator sent the WorkflowStepNotification
                assertTrue(obj2.getResponses()[i].getFolder() == CommunicationsFolder.NotVisible);
                break;
            }
        }

        // Log back in as administrator, for teardown
        performLogin();
    }

    public void testSaveResponseChecklistWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        mainWorkflowStepNotification = saveResponse(
            mainWorkflowStepNotification, mainScope, true);
    }

    public void testSaveResponseChecklistWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        mainWorkflowStepNotification = saveResponse(
            mainWorkflowStepNotification, mainScope, false);
    }

    public void testSendResponseChecklistWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, true);
    }

    public void testSendResponseChecklistWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, false);
    }

    public void testSendResponseVoteWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        // Must respond to the first checklist step first
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, true);
        RPMObject[] objects = loadFromXpath(
            "/WorkflowStepNotification[RunningWorkflowProcess[workflowprocessname='"
                    + GenericCommunicationFixture.PROCESS_NAME
                    + "' and state=\"Open\"]]", mainScope);
        assertNotNull(objects);
        mainWorkflowStepNotification = (WorkflowStepNotification) objects[0];
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, true);
    }

    public void testSendResponseVoteWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            mainScope, this);
        // Must respond to the first checklist step first
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, true);
        RPMObject[] objects = loadFromXpath(
            "/WorkflowStepNotification[RunningWorkflowProcess[workflowprocessname='"
                    + GenericCommunicationFixture.PROCESS_NAME
                    + "' and state=\"Open\"]]", mainScope);
        assertNotNull(objects);
        mainWorkflowStepNotification = (WorkflowStepNotification) objects[0];
        mainWorkflowStepNotification = sendResponse(
            mainWorkflowStepNotification, mainScope, true);
    }

    private WorkflowStepNotification createWorkflowStepNotification(
            GenericCommunicationScope scope, boolean loadFromId)
            throws Exception
    {
        mainWorkflowStepNotification = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            scope, this);

        // Reload the container and compare it
        WorkflowStepNotification returnedWorkflowStepNotification = (WorkflowStepNotification) load(
            mainWorkflowStepNotification, scope, loadFromId);
        GenericCommunicationFixture.compare(mainWorkflowStepNotification,
            returnedWorkflowStepNotification, scope);
        GenericCommunicationFixture.validateReadOnlyValues(returnedWorkflowStepNotification);

        return returnedWorkflowStepNotification;
    }

    private WorkflowStepNotification saveResponse(WorkflowStepNotification wsn,
            GenericCommunicationScope scope, boolean loadFromId)
            throws Exception
    {
        if (scope.getResponses() == null)
            return null;

        for (int i = 0; i < wsn.getResponses().length; ++i)
        {
            if (wsn.getResponses()[i].getVisible() == Boolean.TRUE)
            {
                wsn.getResponses()[i] = WorkflowResponseFixture.updateFixture(
                    wsn.getResponses()[i]);

                // Send the response to the workflow step notification
                SingleResult result = saveWorkflowResponse(wsn.getResponses()[i],
                    scope.getResponses());

                // Compare the returned container
                WorkflowResponse returnedWorkflowResponse = (WorkflowResponse) result.getRpmObject();
                WorkflowResponseFixture.compare(wsn.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());

                // Reload the container and compare it
                returnedWorkflowResponse = (WorkflowResponse) load(
                    returnedWorkflowResponse, scope.getResponses(), loadFromId);
                WorkflowResponseFixture.compare(wsn.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());

                wsn.getResponses()[i] = returnedWorkflowResponse;
            }
        }

        WorkflowStepNotification returnedWorkflowStepNotification = (WorkflowStepNotification) load(
            wsn, scope, loadFromId);

        return returnedWorkflowStepNotification;
    }

    private WorkflowStepNotification sendResponse(WorkflowStepNotification wsn,
            GenericCommunicationScope scope, boolean loadFromId)
            throws Exception
    {
        if (scope.getResponses() == null)
            return null;

        for (int i = 0; i < wsn.getResponses().length; ++i)
        {
            if (wsn.getResponses()[i].getVisible() == Boolean.TRUE)
            {
                wsn.getResponses()[i] = WorkflowResponseFixture.updateFixture(
                    wsn.getResponses()[i]);

                // Send the response to the workflow step notification
                SingleResult result = sendWorkflowResponse(wsn.getResponses()[i],
                    scope.getResponses());

                // Compare the returned container
                WorkflowResponse returnedWorkflowResponse = (WorkflowResponse) result.getRpmObject();
                WorkflowResponseFixture.compare(wsn.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());
                WorkflowResponseFixture.validateReadOnlyValues(returnedWorkflowResponse);

                // Reload the container and compare it
                returnedWorkflowResponse = (WorkflowResponse) load(
                    returnedWorkflowResponse, scope.getResponses(), loadFromId);
                WorkflowResponseFixture.compare(wsn.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());
                WorkflowResponseFixture.validateReadOnlyValues(returnedWorkflowResponse);

                wsn.getResponses()[i] = returnedWorkflowResponse;
            }
        }

        WorkflowStepNotification returnedWorkflowStepNotification = (WorkflowStepNotification) load(
            wsn, scope, loadFromId);

        return returnedWorkflowStepNotification;
    }
}
