/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.client.junit.communications.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.communications.fixtures.WorkflowResponseFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.communications.constants.ValidationConstants;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;

/**
 * @author dominiqu
 */
public class TestSendResponseValidation extends GenericValidation
{
    private WorkflowStepNotification wsnParent = null;

    private WorkflowResponse mainWorkflowResponse;

    private WorkflowResponseScope fullScope = null;

    private WorkflowAttachment workflowAttachment = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        fullScope = WorkflowResponseFixture.createFullScope();

        Project attachment = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(attachment);

        StaffingRequest parent = (StaffingRequest) GenericCommunicationFixture.createSaveFixture(
            workflowAttachment, new StaffingRequest(),
            GenericCommunicationFixture.createFullScope(true), this);

        if (parent.getResponses()[0].getVisible() == Boolean.TRUE)
        {
            mainWorkflowResponse = parent.getResponses()[0];
        }
        else
        {
            mainWorkflowResponse = parent.getResponses()[1];
        }

        mainWorkflowResponse.setState(WorkflowResponseState.Accepted);
    }

    protected void tearDown() throws Exception
    {
        if (wsnParent != null)
        {
            RunningWorkflowProcess process = wsnParent.getRunningWorkflowProcess();
            if (process != null && process.getState() != WorkflowState.Closed)
            {
                getApplicationInterface().cancelWorkflow(sessionid, process);
            }
        }

        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        Notification wrongParent = new Notification();
        wrongParent.setID("DUMMY_ID");
        mainWorkflowResponse.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.communications.constants.ErrorCodes.NO_SEND_RESPONSE,
                mainWorkflowResponse.getClass().getName(), null,
                mainWorkflowResponse.getID())
        };

        verifySendWorkflowResponseMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);
    }

    public void testFailureWithoutMandatoryID() throws AssertionError,
            Exception
    {
        mainWorkflowResponse.setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID,
                mainWorkflowResponse.getClass().getName())
        };

        verifySendWorkflowResponseMustFail(mainWorkflowResponse, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithNullParameters() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mainWorkflowResponse
                .getClass().getName())
        };

        mainWorkflowResponse = null;
        
        verifySendWorkflowResponseMustFail(mainWorkflowResponse, fullScope,
            expectedExceptions, null);
    }
    
    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        mainWorkflowResponse.setState(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY,
                mainWorkflowResponse.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_STATE_FIELD,
                mainWorkflowResponse.getID())
        };

        verifySendWorkflowResponseMustFail(mainWorkflowResponse, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithInvalidValue() throws AssertionError, Exception
    {
        mainWorkflowResponse.setState(WorkflowResponseState.NotComplete);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_VALUE,
                mainWorkflowResponse.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_STATE_FIELD,
                mainWorkflowResponse.getID())
        };

        verifySendWorkflowResponseMustFail(mainWorkflowResponse, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithNotAllItemsChecked() throws AssertionError,
            Exception
    {
        // This test requires a WorkflowStepNotification and not a
        // StaffingRequest
        wsnParent = (WorkflowStepNotification) GenericCommunicationFixture.createSaveFixture(
            workflowAttachment, new WorkflowStepNotification(),
            GenericCommunicationFixture.createFullScope(true), this);

        WorkflowResponse workflowResponse = null;

        if (wsnParent.getResponses()[0].getVisible() == Boolean.TRUE)
        {
            workflowResponse = wsnParent.getResponses()[0];
        }
        else
        {
            workflowResponse = wsnParent.getResponses()[1];
        }

        ((ChecklistItem) workflowResponse.getResponse()[0]).setComplete(Boolean.FALSE);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.communications.constants.ErrorCodes.ALL_CHECKLIST_ITEMS_MUST_BE_COMPLETE,
                workflowResponse.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
                workflowResponse.getID())
        };

        verifySendWorkflowResponseMustFail(workflowResponse, fullScope,
            expectedExceptions, null);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainWorkflowResponse.setComments(getTestString(ValidationConstants.WORKFLOWRESPONSE_COMMENTS_MAX));

        // Attempt to save the WorkflowResponse object
        WorkflowResponse saved = (WorkflowResponse) sendWorkflowResponseMustSucceed(
            mainWorkflowResponse, fullScope);
        mainWorkflowResponse.setID(saved.getID());

        // Load the inserted StaffingRequest from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkflowResponse[Id='"
                + mainWorkflowResponse.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkflowResponse);

        // Check that the loaded WorkflowResponse is identical to the one we
        // requested to store
        WorkflowResponseFixture.compare(mainWorkflowResponse,
            (WorkflowResponse) rpmObjects[0], null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainWorkflowResponse.setComments(getTestString(ValidationConstants.WORKFLOWRESPONSE_COMMENTS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH,
                mainWorkflowResponse.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_COMMENTS_FIELD,
                mainWorkflowResponse.getID())
        };

        verifySendWorkflowResponseMustFail(mainWorkflowResponse, fullScope,
            expectedExceptions, null);
    }

    /**
     * Generic sendResponse function for sendResponse calls that must succeed
     */
    protected RPMObject sendWorkflowResponseMustSucceed(WorkflowResponse obj,
            WorkflowResponseScope scope) throws AssertionError, Exception
    {
        assertNotNull(obj);
        assertTrue(WorkflowResponse.class.isInstance(obj));

        addObjectToDelete(obj);

        // Attempt to save the object
        SingleResult result = getApplicationInterface().sendWorkflowResponse(
            sessionid, obj, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(WorkflowResponse.class.isInstance(result.getRpmObject()));
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        obj = (WorkflowResponse) result.getRpmObject();
        return obj;
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed sendResponse
     */
    protected void verifySendWorkflowResponseMustFail(WorkflowResponse obj,
            WorkflowResponseScope scope, RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        addObjectToDelete(obj);

        // Attempt to save the object
        SingleResult saveResult = getApplicationInterface().sendWorkflowResponse(
            sessionid, obj, scope);

        if (saveResult == null)
        {
            Assert.fail("Result is null");
        }

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}
