/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.client.junit.communications.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.communications.constants.ErrorCodes;
import com.ibm.rpm.communications.constants.ValidationConstants;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author dominiqu
 */
public class TestUpdateReadAndFolderValidation extends GenericValidation
{
    protected void setUp() throws Exception
    {
        super.setUp();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureMoveToInbox() throws AssertionError, Exception
    {
        WorkflowResponse mainWorkflowResponse = new WorkflowResponse();
        mainWorkflowResponse.setID("MOCK_ID");
        mainWorkflowResponse.setParent(new Notification());
        mainWorkflowResponse.setFolder(CommunicationsFolder.Inbox);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                mainWorkflowResponse.getID())
        };

        verifyUpdateReadAndFolderMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);
    }

    public void testFailureMoveToNotVisible() throws AssertionError, Exception
    {
        WorkflowResponse mainWorkflowResponse = new WorkflowResponse();
        mainWorkflowResponse.setID("MOCK_ID");
        mainWorkflowResponse.setParent(new Notification());
        mainWorkflowResponse.setFolder(CommunicationsFolder.NotVisible);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                mainWorkflowResponse.getID())
        };

        verifyUpdateReadAndFolderMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);
    }

    public void testWorkflowStepNotificationFailureMoveToArchived()
            throws AssertionError, Exception
    {
        WorkflowStepNotification wsn = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            GenericCommunicationFixture.createFullScope(false), this);

        WorkflowResponse mainWorkflowResponse = wsn.getResponses()[0];
        mainWorkflowResponse.setFolder(CommunicationsFolder.Archived);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                mainWorkflowResponse.getID())
        };

        verifyUpdateReadAndFolderMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);

        getApplicationInterface().cancelWorkflow(sessionid,
            wsn.getRunningWorkflowProcess());
    }

    public void testWorkflowStepNotificationFailureMoveToDeleted()
            throws AssertionError, Exception
    {
        WorkflowStepNotification wsn = GenericCommunicationFixture.createSaveWorkflowStepNotificationFixture(
            GenericCommunicationFixture.createFullScope(false), this);

        WorkflowResponse mainWorkflowResponse = wsn.getResponses()[0];
        mainWorkflowResponse.setFolder(CommunicationsFolder.Deleted);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                mainWorkflowResponse.getID())
        };

        verifyUpdateReadAndFolderMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);

        getApplicationInterface().cancelWorkflow(sessionid,
            wsn.getRunningWorkflowProcess());
    }

    public void testFailureMoveFromNotVisible() throws AssertionError,
            Exception
    {
        StaffingRequest sr = (StaffingRequest) GenericCommunicationFixture.createSaveFixture(
            null, new StaffingRequest(),
            GenericCommunicationFixture.createFullScope(true), this);

        String userResponseID = null;
        // The resource that is currently logged in
        String username = RpmTestProperties.getProperty("rpm.test.user.name");
        for (int i = 0; sr.getResponses() != null
                && i < sr.getResponses().length; ++i)
        {
            if (username.equals(sr.getResponses()[i].getResource().getUserName()))
            {
                userResponseID = sr.getResponses()[i].getID();
            }
        }

        StaffingRequest srToDelete = new StaffingRequest();
        srToDelete.setID(sr.getID());

        delete(srToDelete);

        WorkflowResponse mainWorkflowResponse = new WorkflowResponse();
        mainWorkflowResponse.setID(userResponseID);
        mainWorkflowResponse.setParent(sr);
        mainWorkflowResponse.setFolder(CommunicationsFolder.Archived);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                mainWorkflowResponse.getID())
        };

        verifyUpdateReadAndFolderMustFail(mainWorkflowResponse, null,
            expectedExceptions, null);
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed updateReadAndFolder
     */
    protected void verifyUpdateReadAndFolderMustFail(WorkflowResponse obj,
            WorkflowResponseScope scope, RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        addObjectToDelete(obj);

        // Attempt to save the object
        SingleResult saveResult = getApplicationInterface().updateReadAndFolder(
            sessionid, obj, scope);

        if (saveResult == null)
        {
            Assert.fail("Result is null");
        }

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}
