/*
 * Created on Oct 14, 2005
 *
 */

package com.ibm.rpm.client.junit.contract.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ContractType;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.scope.ContractScope;

/**
 * @author shiv
 *
 */

public class ContractFixture extends RPMObjectFixture

{
	private static final int NB_CHARGE_SCHEDULES = 1;

	protected static ArrayList localSkipGetters = new ArrayList();

	protected static ArrayList localSkipSetters = new ArrayList();

	static {
		localSkipGetters.addAll(skipGetters);
		localSkipGetters.add("getMaximumAmount");
		//set skipSetters
		localSkipSetters.addAll(skipSetters);
		localSkipSetters.add("setID");
	}

	public static Contract createSaveFixture(ClientTestController ctc)
			throws Exception {
		ContractScope scope = createScope();
		return createSaveFixture(scope, ctc);
	}

	public static Contract createSaveFixture(ContractScope scope,
			ClientTestController ctc) throws Exception {
		Contract contract = createFixture(scope, ctc);

		ContractType ct = new ContractType();
		ct.setID("6704");
		contract.setContractType(ct);

		contract = (Contract) save(ctc, contract, scope);
		return contract;
	}

	public static Contract createFixture(ContractScope scope,
			ClientTestController ctc) throws Exception {
		return createContract(random(), scope, ctc);
	}

	private static Contract createContract(int index, ContractScope scope,
			ClientTestController ctc) throws Exception {
		Contract contract = new Contract();
		HelperTestUtilities.setByIntrospection(contract, index,
				localSkipSetters);
		
		// hardcoded to test ChargeSchedules as 
		// ChargeSchedules are applicable only for 6703 and 6704
		ContractType ct = new ContractType();
		ct.setID("6703");
		contract.setContractType(ct);
		
        Currency currency = DatafieldFixture
        .createCurrency(DatafieldFixture.CURRENCY_USD);
        
        contract.setCurrency(currency);

    	if(scope!=null && scope.isChargeSchedules()){
            ContractChargeSchedule[] ccsArr = new ContractChargeSchedule[NB_CHARGE_SCHEDULES];
            for (int i = 0; i < NB_CHARGE_SCHEDULES; i++)
            {
            	ContractChargeSchedule ccs = ContractChargeScheduleFixture.createFixture(scope, ctc);
            	ccsArr[i] = ccs;
            }        
            contract.setChargeSchedules(ccsArr);
    	}
    	
		return contract;
	}

	public static void updateFixture(Contract contract, ContractScope scope)
			throws Exception {
		updateContract(contract, random(), scope);
	}

	private static void updateContract(Contract contract, int index,
			ContractScope scope) throws Exception {
		//update the fields
		HelperTestUtilities.setByIntrospection(contract, index,
				localSkipSetters);
		
		// update type to 6704
		// hardcoded to test ChargeSchedules as 
		// ChargeSchedules are applicable only for 6703 and 6704
		
		ContractType ct = new ContractType();
		ct.setID("6704");
		contract.setContractType(ct);
		
        Currency currency = DatafieldFixture
        .createCurrency(DatafieldFixture.CURRENCY_CAD);
        
        contract.setCurrency(currency);

    	if(scope!=null && scope.isChargeSchedules()){
            ContractChargeSchedule[] ccsArr = contract.getChargeSchedules();
            for (int i = 0; i < NB_CHARGE_SCHEDULES; i++)
            {
            	ContractChargeScheduleFixture.updateFixture(ccsArr[i], scope);
            }        
    	}
	}

	public static void compare(Contract contract1, Contract contract2,
			ContractScope scope) throws AssertionFailedError, Exception {
		RPMObjectFixture
				.compareWithSkip(contract1, contract2, localSkipGetters);
		if (scope == null)
			return;

		if(scope.isChargeSchedules())
		{
			scope.setParent(null);
			ContractChargeScheduleFixture.compare(contract1.getChargeSchedules(),contract2.getChargeSchedules(),scope);
		}

		//validate the parents
		if (scope.getParent() != null) {
			ContractScope parentScope = (ContractScope)scope.getParent();
			if (parentScope != null) {
				parentScope.setChildren(null);
			}
			ContractFolderFixture.compare((ContractFolder) contract1
						.getParent(), (ContractFolder) contract2.getParent(),
						parentScope);

		}
	}

	public static void compare(ContractType c1, ContractType c2,
			ContractScope scope) throws AssertionFailedError, Exception {
		RPMObjectFixture
				.compareWithSkip(c1, c2, localSkipGetters);
	}

	public static void compare(Contract[] cArray1, Contract[] cArray2,
			ContractScope scope) throws AssertionFailedError, Exception {
		sortWithComparator(cArray1, RPMObjectNameAndIDComparator.INSTANCE);
		sortWithComparator(cArray2, RPMObjectNameAndIDComparator.INSTANCE);
		RPMObjectFixture.compare(cArray1, cArray2);
		if (cArray1 == null)
			return;
		for (int i = 0; i < cArray1.length; i++) {
			ContractFixture.compare(cArray1[i], cArray2[i], scope);
		}
	}

	public static ContractScope createScope() {
		ContractScope scope = new ContractScope();
		scope.setChargeSchedules(true);
		scope.setContractType(true);
		return scope;
	}


}