/*
 * Created on January 3, 2006
 * @author mmatwiy 
 * Test cases for creation of a ContractFolder.
 */
package com.ibm.rpm.client.junit.contract.testcases.validation;

import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.contract.constants.ValidationConstants;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestContractFolderValidation extends GenericValidation
{

    private ContractFolder contractFolder;
    private ContractScope scope;

    public void setUp() throws Exception
    {
        super.setUp();
       
        scope = ContractFixture.createScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentContractFolder() throws AssertionError,
            Exception
    {
        contractFolder = new ContractFolder();
        contractFolder.setName(getTestString(ValidationConstants.NAME_MAX));

        ContractFolder parentFolder = ContractFolderFixture.createSaveFixture(
            1, this);
        contractFolder.setParent(parentFolder);

        // Attempt to save the object
        contractFolder = (ContractFolder) saveMustSucceed(contractFolder, ContractFolder.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractFolder[id='"
                + contractFolder.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractFolder);

        // Check that the loaded object is identical to the one we
        // requested to store
        ContractFolderFixture.compare(contractFolder, (ContractFolder) rpmObjects[0], scope);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        contractFolder = new ContractFolder();

        // set mandatory field
        contractFolder.setName("test");

        // create and set invalid parent
        Contract parentContract = ContractFixture.createSaveFixture(this);
        contractFolder.setParent(parentContract);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, contractFolder
                .getClass().getName(), ValidationConstants.PARENT, contractFolder
                .getID())
        };

        verifySaveMustFail(contractFolder, scope, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        contractFolder = new ContractFolder();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, contractFolder.getClass()
                .getName(), ValidationConstants.NAME_FIELD, contractFolder.getID())
        };

        verifySaveMustFail(contractFolder, scope, expectedExceptions);
    }
    
    public void testFailureValidationMin() throws AssertionError, Exception
    {
        contractFolder = new ContractFolder();
        contractFolder.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MIN_LENGTH, contractFolder.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, contractFolder
                    .getID())
        };

        verifySaveMustFail(contractFolder, scope, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        contractFolder = new ContractFolder();
        contractFolder.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the ContractFolder object
        contractFolder = (ContractFolder) saveMustSucceed(contractFolder, ContractFolder.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractFolder[id='"
                + contractFolder.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractFolder);

        // Check that the loaded ContractFolder is identical to the one we
        // requested to store
        ContractFolderFixture.compare(contractFolder, (ContractFolder) rpmObjects[0], scope);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        contractFolder = new ContractFolder();

        // Set field values with maxLength and verify that they are accepted
        contractFolder.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the ContractFolder object
        contractFolder = (ContractFolder) saveMustSucceed(contractFolder, ContractFolder.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractFolder[id='"
                + contractFolder.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractFolder);

        // Check that the loaded ContractFolder is identical to the one we
        // requested to store
        ContractFolderFixture.compare(contractFolder, (ContractFolder) rpmObjects[0], scope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        contractFolder = new ContractFolder();

        // Set field values with maxLength+1 and verify that they are not
        // accepted
        contractFolder.setName(getTestString(ValidationConstants.NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, contractFolder.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, contractFolder
                    .getID())
        };

        verifySaveMustFail(contractFolder, scope, expectedExceptions);
    }

 }
