package com.ibm.rpm.client.junit.customfield.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;

public class TestCustomField extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        // Delete all saved generic custom fields.
        CustomFieldFixture.deleteAllCustomFields(this);
        super.tearDown();
    }

    // ==========
    // LOAD TESTS
    // ==========

    //This test is not valid anymore, as the db is now preventing from deleting 
    //datafields when there are customfields pointing to them
    //TODO delete this test later
//    /**
//     * When the refered Datafield or DatafieldCategory has been deleted, the
//     * load should not fail but the corresponding reference should be set to
//     * null.
//     */
//    public void testReloadCustomFieldWithDeletedDatafield() throws Exception
//    {
//        // Null input type means CustomDatafield.
//        CustomField original = CustomFieldFixture.createCustomField((FieldInputType) null);
//        
//        //contexName is mandatory since we don't want to test the ModuleManager
//        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);
//        
//        doTestCreate(original, null);
//        delete((CustomDatafield) original.getDefaultValue());
//        delete(original.getDatafieldCategory());
//        CustomField updated = (CustomField) loadFromID(original, null);
//        assertNull("Dangling reference should be set to null.", updated
//                .getDatafieldCategory());
//        assertNull("Dangling reference should be set to null.", updated
//                .getDefaultValue());
//    }

    // ==============
    // CREATION TESTS
    // ==============

    public void testCreateCustomFieldWithCurrency() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Currency);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_WBS_SCOPE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithCheckbox() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Checkbox);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_WBS_SCOPE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithDate() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Date);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_WBS_SCOPE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithNumeric() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Numeric);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithDatafield() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Datafield);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithCustomDatafield() throws Exception
    {
        // When inputType is null, a CustomDatafield is created.
        CustomField original = CustomFieldFixture.createCustomField((FieldInputType) null);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldWithDatafield2() throws Exception
    {
        CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_RESOURCE_CONTEXT, FieldInputType.Datafield,
                this);
    }

    public void testCreateCustomFieldCategoryWithoutScope() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory();

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_WBS_SCOPE_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateCustomFieldCategoryWithScope() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory();

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_ASSET_CONTEXT);

        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(1);
        doTestCreate(original, scope);
    }

    public void testCreateCustomFieldCategoryWithTooLargeScope()
            throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory();

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_ASSET_CONTEXT);

        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(3)
            .getChildren();
        doTestCreate(original, scope);
    }

    public void testCreateCustomFieldWithoutScope() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Checkbox);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreateWithDatafieldCategoryId() throws Exception
    {
        // Datafield category's type id will be loaded from the database.
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Datafield);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        original.getDatafieldCategory().setID(
            DatafieldFixture.PROJECT_RTF_CATEGORY_ID);
        
        original.setDefaultValue(null);
        doTestCreate(original, null);
    }

    public void testCreateWithDatafieldCategoryTypeId() throws Exception
    {
        // TODO: Always comment the following code before check-in
        // because TypeID is not available from WSDL.

        // Datafield category's type id will be used directly.
//        CustomField original = CustomFieldFixture.createCustomField(FieldInputType.Datafield);
//        
//        //contexName is mandatory since we don't want to test the ModuleManager
//        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);
//        
//        original.getDatafieldCategory().setID(null);
//        original.getDatafieldCategory().assignTypeID(
//                new Integer(Language.TYPE_ID));
//        original.setDefaultValue(null);
//        doTestCreate(original, null);
    }

    public void testCreateWithDatafieldDefaultValueTypeId() throws Exception
    {
        // Default value's type id will be used to create a category of with the
        // same type id.
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Datafield);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        doTestCreate(original, null);
    }

    public void testCreate2LevelTree() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory(2, FieldInputType.Currency);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_CLIENT_CONTEXT);

        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(2);
        doTestCreate(original, scope);
    }

    private GenericCustomField doTestCreate(GenericCustomField original,
            CustomFieldScope scope) throws Exception
    {
        return CustomFieldFixture.save(this, original, scope);
    }

    // ============
    // UPDATE TESTS
    // ============

    public void testUpdateCustomFieldCategory() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory();

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(1);
        doTestUpdate(original, scope);
    }

    public void testUpdateCustomField() throws Exception
    {
        CustomField original = CustomFieldFixture.createCustomField();
        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(1);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        doTestUpdate(original, scope);
    }

    public void testUpdate2LevelTree() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
                .createCustomFieldCategory(3);
        
        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_CLIENT_CONTEXT);
        
        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(2);
        doTestUpdate(original, scope);
    }

    private void doTestUpdate(GenericCustomField original,
            CustomFieldScope scope) throws Exception
    {
        // Create the object in the database.
        GenericCustomField modified = doTestCreate(original, scope);
        // Modify the object.
        CustomFieldFixture.modify(modified, scope);
        // Save the modified object.
        save(modified, scope, ReloadType.None);
        // Reload the modified object.
        RPMObject reloaded = loadFromID(modified, scope);
        // Ensures all original objects have been deleted.
        CustomFieldFixture.assertEquals(modified, reloaded, scope);
    }

    // ===========
    // XPATH TESTS
    // ===========

    public void testXPathChildrenFromParentX() throws Exception
    {
        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(3);
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory(3);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);

        original = (CustomFieldCategory) doTestCreate(original, scope);

        // Build query.
        final String query = "/CustomFieldCategory[Name='" + original.getName()
                + "']/CustomField";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject[] reloadedObjects = loadFromXpath(query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        CustomField[] originalChildren = original.getChildren();
        assertEquals(originalChildren.length, reloadedObjects.length);
        for (int i = 0; i < originalChildren.length; i++)
        {
            GenericCustomField reloaded = CustomFieldFixture.findInArray(
                    originalChildren[i], reloadedObjects);
            assertTrue("Original object not found in reloaded objects.",
                    reloaded != null);
            CustomFieldFixture.assertEquals(originalChildren[i], reloaded,
                    scope);
        }
    }

    public void testXPathParentWithChildX() throws Exception
    {
        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(3);
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory(3);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_RESOURCE_CONTEXT);

        original = (CustomFieldCategory) doTestCreate(original, scope);

        // Build query.
        final String query = "/CustomFieldCategory[CustomField[starts-with(Name, '"+ CustomFieldFixture.NAME_PREFIX +"')]]";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject[] reloadedObjects = loadFromXpath(query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        GenericCustomField reloaded = CustomFieldFixture.findInArray(original,
                reloadedObjects);
        assertTrue("Original object not fount in reloaded objects.",
                reloaded != null);

        // Compare the original objects with the reloaded.
        CustomFieldFixture.assertEquals(original, reloaded, scope);
    }

    // ==============
    // DELETION TESTS
    // ==============

    public void testDeleteCustomFieldCategory() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory();

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_DOCUMENT_CONTEXT);

        doTestDelete(original);
    }

    public void testDeleteCustomField() throws Exception
    {
        CustomField original = CustomFieldFixture
            .createCustomField(FieldInputType.Numeric);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_ASSET_CONTEXT);

        doTestDelete(original);
    }

    public void testDelete2LevelTree() throws Exception
    {
        CustomFieldCategory original = CustomFieldFixture
            .createCustomFieldCategory(2, FieldInputType.Datafield);

        //contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomFieldFixture.CF_CLIENT_CONTEXT);

        doTestDelete(original);
    }

    public void testDeleteAll() throws Exception
    {
        // Delete all saved generic custom fields.
        CustomFieldFixture.deleteAllCustomFields(this);
        // Check that all generic custom fields have been deleted.
        CustomFieldFixture.assertAllDeleted(this);
    }

    private void doTestDelete(GenericCustomField original) throws Exception
    {
        // Create the object in the database.
        CustomFieldScope scope = CustomFieldFixture.createCustomFieldScope(3)
                .getChildren();
        GenericCustomField reloaded = doTestCreate(original, scope);
        // Delete the object from the database.
        // Ensures the delete works even when the parent is not set.
        reloaded.setParent(null);
        delete(reloaded);
        // Ensures all original objects have been deleted.
        CustomFieldFixture.assertDeleted(this, original);
    }
}
