/*
 * Created on Sep 14, 2006
 */
package com.ibm.rpm.client.junit.customfield.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomPortletFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.scope.CustomPortletScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

public class TestCustomPortlet extends ClientTestController
{

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testCreateCustomPortletCategoryWithoutScope() throws Exception
    {
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(0);
        insert(original, null);
    }
    
    public void testCreateCustomPortletCategoryWithScope() throws Exception
    {
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(1);
        // contexName is mandatory since we don't want to test the ModuleManager
        original.setContextName(CustomPortletFixture.CF_WBS_SCOPE_CONTEXT);
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        insert(original, scope);
    }
    
    public void testUpdateNameCustomPortlet() throws Exception
    {
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(1);
        original.setContextName(CustomPortletFixture.CF_WBS_SCOPE_CONTEXT);
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        original = (CustomPortletCategory)insert(original, scope);
        CustomPortlet portlet = original.getChildren()[0];
        portlet = (CustomPortlet) CustomPortletFixture
            .updateGenericCustomPortletName(portlet);
        insert(original, scope);
    }
    
    public void testAddChildrenCustomPortletCategory() throws Exception
    {
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(1);
        original.setContextName(CustomPortletFixture.CF_WBS_SCOPE_CONTEXT);
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        original = (CustomPortletCategory) insert(original, scope);

        original = CustomPortletFixture.addChildren(original, 3);
        insert(original, scope);
    }
    
    public void testLoadCustomPortletWithScopeFromXPath() throws Exception
    {
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        scope.setParent(scope);
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(3);

        original = (CustomPortletCategory) insert(original, scope);

        // Build query.
        final String query = "/CustomPortletCategory[Name='" + original.getName()
                + "']/CustomPortlet";
        logger.debug("XPath query: " + query);

        RPMObject[] reloadedObjects = loadFromXpath(
            query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        CustomPortlet[] originalChildren = original.getChildren();
        assertEquals(originalChildren.length, reloadedObjects.length);        
        CustomPortletFixture.compare(originalChildren, reloadedObjects);
    }
    
    public void testLoadCustomPortletFromXPath() throws Exception
    {
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        scope.setParent(scope);
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(1);

        original = (CustomPortletCategory) insert(original, scope);

        // Build query.
        String childName = original.getChildren()[0].getName();
        final String query = "/CustomPortlet[Name='" + childName
                + "']";
        logger.debug("XPath query: " + query);

        RPMObject[] reloadedObjects = loadFromXpath(
            query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        CustomPortlet originalChild = original.getChildren()[0];
        CustomPortletFixture.compare(originalChild, reloadedObjects[0]);
    }
    
    public void testLoadCustomPortletFromGroupType() throws Exception
    {
        CustomPortletScope scope = new CustomPortletScope();
        scope.setChildren(true);
        scope.setParent(scope);
        CustomPortletCategory original = CustomPortletFixture
            .createCustomPortletCategory(1);

        original = (CustomPortletCategory) insert(original, scope);

        // Build query.
        String groupType = original.getGroupType().getValue();
        final String query = "/CustomPortletCategory[GROUPTYPE='" + groupType 
                + "']";
        logger.debug("XPath query: " + query);

        RPMObject[] reloadedObjects = loadFromXpath(
            query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);        
    }

    public GenericCustomPortlet insert(CustomPortletCategory original,
            CustomPortletScope scope) throws Exception
    {
        // Ensure the custom datafield is created first.
        SingleResult result = null;
        result = save(original, scope);
        
        CustomPortletCategory reloaded = (CustomPortletCategory) result
            .getRpmObject();
        CustomPortletFixture.compare(reloaded, original, scope);
        return reloaded;
    }
}
