/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.document.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.NoteMinutesRTF;
import com.ibm.rpm.applicationadministration.containers.NoteState;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentModuleFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.scope.DocumentModuleScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;

/**
 * @author karim
 *  
 */
public class TestDocumentModule extends ModuleTestController
{
    public TestDocumentModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = DocumentModuleFixture.createFullScope();
        moduleContaineName = "DOCUMENTMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testXPathAllCustomFields() throws Exception
    {
        // A custom field must exist in order to be able to load something
        CustomFieldFixture.createSaveCustomField(CustomFieldFixture.CF_DOCUMENT_CONTEXT, FieldInputType.Text, this);
        RPMObjectScope scope = CustomFieldFixture.createCustomFieldScope(1);
        RPMObject[] objs = loadFromXpath(
                "/DocumentModule/CustomFieldCategory/CustomField", scope);
        assertNotNull(objs);
    }

    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {

        DocumentModule module = (DocumentModule) aModule;
        DocumentModuleScope scope = (DocumentModuleScope) aScope;

        DocumentModule loadedModule = new DocumentModule();

        if (scope != null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = loadAttributeCategories(
                        module.getAttributeCategories(), scope.getAttributeCategories());
                loadedModule.setAttributeCategories(attributeCategories);
            }
            if(scope.isDocumentStates())
            {
                //loadDocumentStates
        	    DocumentState[] documentStates= (DocumentState[]) loadDatafields(
        	    		module.getDocumentStates(),
        				scope.isDocumentStates(),
        				DocumentState.class);
        	    loadedModule.setDocumentStates(documentStates);
            }
            if(scope.isNoteStates())
            {
        	    //loadNoteStates
        	    NoteState[] noteStates = (NoteState[]) loadDatafields(
        	    		module.getNoteStates(),
        				scope.isNoteStates(),
        				NoteState.class);
        	    loadedModule.setNoteStates(noteStates);
            }
            if(scope.isNoteMinutesRtfs())
            {
        	    //loadNoteMinutesRtfs
        	    NoteMinutesRTF[] noteMinutesRTFs = (NoteMinutesRTF[]) loadDatafields(
        	    		module.getNoteMinutesRtfs(),
        				scope.isNoteMinutesRtfs(),
        				NoteMinutesRTF.class);
        	    loadedModule.setNoteMinutesRtfs(noteMinutesRTFs);
            }
            if(scope.getCustomFieldCategories()!=null)
            {
        	    //CustomFieldCategories
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) loadElements(
                        module.getCustomFieldCategories(), scope
                                .getCustomFieldCategories(), CustomFieldCategory.class);
                loadedModule.setCustomFieldCategories(customFieldCategories);
            }
        }

        return loadedModule;
    }

    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {

        DocumentModule module = (DocumentModule) aModule;
        DocumentModuleScope scope = (DocumentModuleScope) aScope;
        
        //delete the element
        if (module != null && scope != null)
        {
            if (module.getAttributeCategories() != null && scope.getAttributeCategories()!=null)
            {
                delete(module.getAttributeCategories());
            }
        	if (module.getCustomFieldCategories() != null && scope.getCustomFieldCategories()!=null)
        	{
        	    delete(module.getCustomFieldCategories());
        	}
        	
            //reload the deleted element
            DocumentModule deletedModule = (DocumentModule) loadDeletedModule(
                    module, scope);

            //Validate the deleted element
            if(scope.getAttributeCategories()!=null)
            {
                assertNull(deletedModule.getAttributeCategories());                
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                assertNull(deletedModule.getCustomFieldCategories());                
            }
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return DocumentModuleFixture.createFixture((DocumentModuleScope) scope,
                this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return DocumentModuleFixture.updateFixture((DocumentModule) module,
                (DocumentModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        DocumentModuleFixture.compare((DocumentModule) module,
                (DocumentModule) returnedModule, (DocumentModuleScope) scope);
    }
}
