package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.constants.ErrorCodes;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Proposal;

public class TestLockDocumentValidation extends GenericValidation {

	Document documentToLock; 
	
	public void setUp() throws Exception
	{
		super.setUp();
		
		//	create a Proposal and save it; this will create a DocumentFolder
		Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
		
		RPMObject[] rpmObjects = loadFromXpath("/DocumentFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DocumentFolder);
        DocumentFolder docFolder = (DocumentFolder) rpmObjects[0];
        
        DocumentScope documentScope = new DocumentScope();
        documentScope.setParent(new DocumentScope());
        
		documentToLock = new Document();
        documentToLock.setParent(docFolder);
        documentToLock.setName(HelperTestUtilities.createName());
        documentToLock = (Document)save(documentToLock, documentScope).getRpmObject();
        
        assertNotNull(documentToLock);
        
	}
	
	public void tearDown() throws Exception
    {
        super.tearDown();
    }

	public void testErrorAlreadyLocked() throws Exception
	{
		//This will lock the document
        documentToLock = (Document)lockDocument(documentToLock).getRpmObject();
        
        assertNotNull(documentToLock);
        assertNotNull(documentToLock.getLocked());
        assertEquals(true, documentToLock.getLocked().booleanValue());
        
        //This should result an exception
        SingleResult result = doLockDocument(documentToLock);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.EXCEPTION_DOC_ALREADY_LOCKED, documentToLock.getClass().getName()),
        };

        verifyFailure(documentToLock, expectedExceptions, null, result);
        verifyExceptions(expectedExceptions, result.getErrors());
	}
	
	public void testErrorCheckedOut() throws Exception
	{
		// This will Checkout the document
        disableAutomaticCheckout();
        checkOut(documentToLock);
        
        //load the CheckedOut Document
        documentToLock = (Document)loadFromID(documentToLock, null);
        
        assertNotNull(documentToLock);
        // This should result an exception
        SingleResult result = doLockDocument(documentToLock);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.EXCEPTION_LOCKING_CHECKEDOUT_DOC, documentToLock.getClass().getName()),
        };

        verifyFailure(documentToLock, expectedExceptions, null, result);
        verifyExceptions(expectedExceptions, result.getErrors());
	}
	
}
