/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestSoftwareValidation extends GenericValidation
{
    private ClientCostCenter mainGrandParent;

    private DocumentFolder mainParent;

    private Software mainSoftware;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ClientCostCenterFixture.createSaveFixture(null, this);

        // Load the ClientCostCenter's DocumentFolder
        ClientCostCenterScope scope = new ClientCostCenterScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ClientCostCenter) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();

        mainSoftware = new Software();
        mainSoftware.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainSoftware.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainSoftware.getClass().getName(),
                "parent", mainSoftware.getID())
        };

        verifySaveMustFail(mainSoftware, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainSoftware.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainSoftware.getClass()
                .getName(), "parent", mainSoftware.getID())
        };

        verifySaveMustFail(mainSoftware, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create Document with the max values
        mainSoftware.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainSoftware
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX));
        mainSoftware
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX));
        mainSoftware
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX));

        // Attempt to save the Document
        mainSoftware = (Software) saveMustSucceed(mainSoftware, Software.class, null);

        // Load the inserted Document from id
        RPMObject rpmObject = loadFromID(mainSoftware, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Software);

        // Check that the loaded Document is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainSoftware, ((Software) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create Document with maximum+1 values
        mainSoftware
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainSoftware
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX + 1));
        mainSoftware
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX + 1));
        mainSoftware
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainSoftware.getClass().getName(),
                "name", mainSoftware.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainSoftware.getClass().getName(),
                "keywords", mainSoftware.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainSoftware.getClass().getName(),
                "referenceNumber", mainSoftware.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainSoftware.getClass().getName(),
                "summary", mainSoftware.getID())
        };

        verifySaveMustFail(mainSoftware, null, expectedExceptions);
    }
}
