/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestURLValidation extends GenericValidation
{
    private ClientCostCenter mainGrandParent;

    private DocumentFolder mainParent;

    private UrlDocument mainURL;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ClientCostCenterFixture.createSaveFixture(null, this);

        // Load the ClientCostCenter's DocumentFolder
        ClientCostCenterScope scope = new ClientCostCenterScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ClientCostCenter) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();

        mainURL = new UrlDocument();
        mainURL.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainURL.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainURL.getClass().getName(),
                "parent", mainURL.getID())
        };

        verifySaveMustFail(mainURL, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainURL.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                mainURL.getClass().getName(), "parent", mainURL.getID())
        };

        verifySaveMustFail(mainURL, null, expectedExceptions);
    }

    public void testFailureWithInvalidBooleans() throws AssertionError, Exception
    {
        mainURL.setProcessLink(Boolean.TRUE);
        mainURL.setRUPLink(Boolean.TRUE);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_CONDITIONAL_VALUE, mainURL.getClass().getName(),
                "processLink", mainURL.getID()),
            createRPMException(ErrorCodes.INVALID_CONDITIONAL_VALUE, mainURL.getClass().getName(),
                "RUPLink", mainURL.getID())
        };

        verifySaveMustFail(mainURL, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create URL with the max values
        mainURL.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainURL.setUrl(getTestString(ValidationConstants.URL_MAX));
        mainURL
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX));
        mainURL
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX));
        mainURL
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX));

        // Attempt to save the URL
        mainURL = (UrlDocument) saveMustSucceed(mainURL, UrlDocument.class, null);

        // Load the inserted URL from id
        RPMObject rpmObject = loadFromID(mainURL, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof UrlDocument);

        // Check that the loaded URL is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainURL, ((UrlDocument) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create URL with maximum+1 values
        mainURL.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainURL.setUrl(getTestString(ValidationConstants.URL_MAX + 1));
        mainURL
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX + 1));
        mainURL
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX + 1));
        mainURL
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainURL.getClass().getName(),
                "name", mainURL.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainURL.getClass().getName(),
                "url", mainURL.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainURL.getClass().getName(),
                "keywords", mainURL.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainURL.getClass().getName(),
                "referenceNumber", mainURL.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainURL.getClass().getName(),
                "summary", mainURL.getID())
        };

        verifySaveMustFail(mainURL, null, expectedExceptions);
    }
}
