package com.ibm.rpm.client.junit.financial.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.financial.types.RevenueRecognitionMethodType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author legaults
 * @author ikusakov
 */
public class FinancialFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters = new ArrayList();
    protected static ArrayList localSkipGetters = new ArrayList();
    
    static
    {
        //set skipGetters
        localSkipGetters.addAll(skipGetters);
        
        // calculated by the proc
        localSkipGetters.add("getProbabilityAmount");
        localSkipGetters.add("getETC");
        
        // calculated in FinancialBusinessLogicUtil
        // these are all manually compared
        localSkipGetters.add("getEAC");
        localSkipGetters.add("getEACVariance");
        localSkipGetters.add("getBaselineVariance");
        localSkipGetters.add("getRemaining");
        localSkipGetters.add("getActual");
        localSkipGetters.add("getEstimate");
        localSkipGetters.add("getProbabilityPercent");
        
        localSkipGetters.add("getSubElementRemaining");
        localSkipGetters.add("getSubElementActual");
        localSkipGetters.add("getSubElementEstimate");
        localSkipGetters.add("getSubElementETC");
        localSkipGetters.add("getSubElementEAC");

        

        
    }
    
    public static FinancialScope createAllScope()
    {
        FinancialScope fs = new FinancialScope();

        fs.setParent(new WorkElementScope());
        fs.setChargeCode(true);
        fs.setChargeCode(true);
        fs.setActualTimePhased(true);
        fs.setEstimateTimePhased(true);

        return fs;
    }
    
    public static FinancialScope createAllScopeNoTimephased()
    {
        FinancialScope fs = new FinancialScope();

        fs.setParent(new WorkElementScope());
        fs.setChargeCode(true);
        fs.setChargeCode(true);
        fs.setActualTimePhased(false);
        fs.setEstimateTimePhased(false);

        return fs;
    }
    

    public static WorksheetFinancial createWorksheetFixture(
            FinancialScope scope, ClientTestController ctc) throws Exception
    {
        return (WorksheetFinancial) createFixture(true,
                WorksheetFinancial.class, scope, ctc);
    }

    public static WorksheetFinancial createWorksheetFixture(
            FinancialScope scope, RPMObject parent, ClientTestController ctc)
            throws Exception
    {
        AbstractFinancial absFin = createFixture(false,
                WorksheetFinancial.class, scope, ctc);
        absFin.setParent(parent);
        return (WorksheetFinancial) absFin;
    }

    public static WbsFinancial createWbsFixture(FinancialScope scope,
            RPMObject parent, ClientTestController ctc) throws Exception
    {
        AbstractFinancial absFin = createFixture(false, WbsFinancial.class,
                scope, ctc);
        absFin.setParent(parent);
        return (WbsFinancial) absFin;
    }

    public static WbsFinancial createWbsFixture(FinancialScope scope,
            ClientTestController ctc) throws Exception
    {
        return (WbsFinancial) createFixture(true, WbsFinancial.class, scope,
                ctc);
    }

    public static AbstractFinancial createFixture(boolean createParent,
            Class container, FinancialScope scope, ClientTestController ctc)
            throws Exception
    {

        AbstractFinancial financial = (AbstractFinancial) container
                .newInstance();

        if (scope == null || !(scope.isActualTimePhased()
                || scope.isEstimateTimePhased()))
        {
            // can't set the worksheet/wbs totals and the timephased
            // values and save them together
            if (WorksheetFinancial.class.equals(container))
            {
                financial.setEstimate(new Double(1001.0));
                financial.setProbabilityAmount(financial.getEstimate());
                financial.setActual(new Double(1002.0));
                financial.setETC(new Double(1004.0));
                financial.setEACVariance(new Double(1005.0));
                financial.setBaselineVariance(new Double(1001.0));
            }
            else if (WbsFinancial.class.equals(container))
            {
                // WBS gets calculated from a Timesheet
                // So we put all 0s for now
                financial.setEstimate(new Double(0));
                financial.setProbabilityAmount(financial.getEstimate());
                financial.setActual(new Double(0));
                financial.setETC(new Double(0));
                financial.setEACVariance(new Double(0));
                financial.setBaselineVariance(new Double(0));
            }
            else
            {
                throw new Exception("Unsupported container: " + container);
            }
        }

        //financial.setProbabilityPercent(new Double(100));
        financial.setBaseline(new Double(0));
        financial.setBaselineInitial(new Double(0));
        financial.setBaselinePrevious(new Double(0));
        financial.setBilled(new Double(0));
        financial.setGL(new Double(0));
        financial.setRevRecType(RevenueRecognitionMethodType.Ratio);

        if (createParent)
        {
            Proposal stubProposal = new Proposal();
            WorkElement parentWorkElement = WorkElementFixture
                    .retrieveStaticFixture(stubProposal, ctc);
            financial.setParent(parentWorkElement);
        }
        else
        {
            // it has to be there - to identify wbs vs scope
            financial.setParent(new WorkElement());
        }

        if (scope != null && scope.isChargeCode())
        {
            // ChargeCode cc = new ChargeCode();
            // cc.setID("EXPENSE_HOURS ");
            ChargeCode cc = ChargeCodeFixture
                    .createAndSaveSimpleChargeCode(ctc);
            financial.setChargeCode((ChargeCode) ctc.loadFromID(cc, null));
        }

        if (scope != null && scope.isEstimateTimePhased())
        {
            TimePhasedFinancial tpfEstimate = TimePhasedFinancialFixture
                    .createFixture(null, null, ctc);
            
            tpfEstimate.setBudgetType(BudgetType.Estimate);
            tpfEstimate.setParent(financial);

            financial.setEstimateTimePhased(new TimePhasedFinancial[]
            {
                tpfEstimate
            });
        }
        
        if (scope != null && scope.isActualTimePhased())
        {
            TimePhasedFinancial tpfActual = TimePhasedFinancialFixture
                    .createFixture(null, null, ctc);
            
            tpfActual.setBudgetType(BudgetType.Actual);
            tpfActual.setParent(financial);

            financial.setActualTimePhased(new TimePhasedFinancial[]
            {
                tpfActual
            });
        }

        return financial;
    }

    public static void compare(AbstractFinancial f1, AbstractFinancial f2,
            FinancialScope scope) throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compareWithSkip(f1, f2, localSkipGetters);
        
        Assert.assertEquals(FinancialUtil.nullToZero(f1.getETC()), FinancialUtil
            .nullToZero(f2.getETC()));
                
        // these values are calculated in FinancialBusinessLogicUtil
        double actual = f1.getActual() == null ? 0.0 : f1.getActual().doubleValue();
        double etc = f1.getETC() == null ? 0.0 : f1.getETC().doubleValue();
        double estimate = f1.getEstimate() == null ? 0.0 : f1.getEstimate().doubleValue();
        double baseline = f1.getBaseline() == null ? 0.0 : f1.getBaseline().doubleValue();
        
        double f2Actual = f2.getActual() == null ? 0.0 : f2.getActual().doubleValue();
        double f2ETC = f2.getETC() == null ? 0.0 : f2.getETC().doubleValue();
        double f2Estimate = f2.getEstimate() == null ? 0.0 : f2.getEstimate().doubleValue();
        double f2Baseline = f2.getBaseline() == null ? 0.0 : f2.getBaseline().doubleValue();
        
        // EAC = actual + ETC
        Assert.assertEquals(new Double(f2Actual + f2ETC), new Double(actual
                + etc));

        // EACVariance = ETC - remaining
        Assert.assertEquals(new Double(f2ETC - (f2Estimate - f2Actual)),
            new Double(etc - (estimate - actual)));

        // remaining = estimate - actual
        Assert.assertEquals(new Double(f2Estimate - f2Actual), new Double(
            estimate - actual));

        // baselineVariance = estimate - baseline
        Assert.assertEquals(new Double(f2Estimate - f2Baseline), new Double(
            estimate - baseline));
        
        if (scope != null && scope.isChargeCode())
        {
            if (f2.getChargeCode() == null)
                Assert.fail("RPMObject in scope is null");

            ChargeCodeFixture.compare(f1.getChargeCode(), f2.getChargeCode());
        }

        if (scope != null && scope.getParent() != null)
        {
            if (f2.getParent() == null)
                Assert.fail("RPMObject in scope is null");

            if (f2.getParent() instanceof WorkElement)
            {
                //Be careful with the scope to avoid infinite loop
                WorkElementScope parentScope = (WorkElementScope) scope
                        .getParent();
                if (parentScope != null)
                {
                    parentScope.setParent(null);
                }

                WorkElementFixture.compare((WorkElement) f1.getParent(),
                        (WorkElement) f2.getParent(), parentScope);
            }
        }

        if (scope == null
                || (!(scope.isActualTimePhased() || scope.isEstimateTimePhased())))
        {
            // not calculated from timephased values; 
            // if these values are calculated using timephased values, 
            // they need to be compared manually (outside of this method)
            Assert.assertEquals(FinancialUtil.nullToZero(f1.getActual()), f2
                .getActual());
            Assert.assertEquals(FinancialUtil.nullToZero(f1.getEstimate()), f2
                .getEstimate());
        }

        if (scope != null)
        {
            if (scope.isActualTimePhased())
            {
                compareTimePhasedArray(f1.getActualTimePhased(), f2
                    .getActualTimePhased());
            }
            if (scope.isEstimateTimePhased())
            {
                compareTimePhasedArray(f1.getEstimateTimePhased(), f2
                    .getEstimateTimePhased());
            }
            if (scope.isBaselineInitialTimePhased())
            {
                compareTimePhasedArray(f1.getBaselineInitialTimePhased(), f2
                    .getBaselineInitialTimePhased());
            }
            if (scope.isBaselinePreviousTimePhased())
            {
                compareTimePhasedArray(f1.getBaselinePreviousTimePhased(), f2
                    .getBaselinePreviousTimePhased());
            }
            if (scope.isBaselineTimePhased())
            {
                compareTimePhasedArray(f1.getBaselineTimePhased(), f2
                    .getBaselineTimePhased());
            }
            if (scope.isGLTimePhased())
            {
                compareTimePhasedArray(f1.getGLTimePhased(), f2
                    .getGLTimePhased());
            }
            if (scope.isBilledTimePhased())
            {
                compareTimePhasedArray(f1.getBilledTimePhased(), f2
                    .getBilledTimePhased());
            }
        }

                
    }

    public static void compare(AbstractFinancial[] fArray1,
            AbstractFinancial[] fArray2, FinancialScope scope)
            throws AssertionFailedError, Exception
    {

        sortWithComparator(fArray1, new RPMObjectIDComparator());
        sortWithComparator(fArray2, new RPMObjectIDComparator());

        if (fArray1 == null)
            return;

        RPMObjectFixture.compare(fArray1, fArray2);

        for (int i = 0; i < fArray1.length; i++)
        {
            FinancialFixture.compare(fArray1[i], fArray2[i], scope);
        }
    }

    public static void compareTimePhasedArray(TimePhasedFinancial[] array1,
            TimePhasedFinancial[] array2) throws AssertionFailedError,
            Exception
    {
        if (array1 == null && array2 == null)
            return;
        if (array1 == null && array2 != null)
            Assert.fail("TimePhased array in e1 is null, but corresponding e2 array is not null");
        if (array1 != null && array2 == null)
            Assert.fail("TimePhased array in e1 is not null, but corresponding e2 array is null");

        Assert.assertEquals("TimePhased arrays length", array1.length,
                array2.length);

        for (int i = 0; i < array1.length; i++)
        {
            TimePhasedFinancialFixture.compare(array1[i], array2[i], null);
        }
    }
    
    public static WorksheetFinancial[] createEmptyWorksheetFinancialSystemChargeCodesArray(
            AggregateNode parent, ClientTestController controller) throws Exception
    {
        WorksheetFinancial[] wWSheetFinancial = null;
        String SystemChargeCodeName = "System Charge Codes";
        RPMObject[] resultArray = null;
        ChargeCodeCategory wCCCategory = null;
        ChargeCodeCategory wExpenseCategory = null;
        
        ChargeCode wExpenseHour = null;
        ChargeCode wExpenseAvg = null;
        ChargeCode wExpenseLabor = null;
        ChargeCode wExpenseFTE = null;


        final int HOUR = 0;
        final int AVG_COST = 1;
        final int EXP_LABOR = 2;
        final int FTE = 3;

        // ChargeCodeScope needed to load the ChargeCodeCategory
        ChargeCodeScope wCCMainScope = new ChargeCodeScope();
        wCCMainScope.setChildren(new ChargeCodeScope());
        wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

        resultArray = controller.loadFromXpath(
                "/ChargeCodeCategory[name='" + SystemChargeCodeName + "']",
                wCCMainScope);
        

        if (resultArray != null
                && resultArray[0] != null
                && (resultArray[0] instanceof ChargeCodeCategory)
                && parent != null)
        {
            wCCCategory = (ChargeCodeCategory) resultArray[0];
            wExpenseCategory = (ChargeCodeCategory) wCCCategory.getChildren()[0];
            
            //Creating Worksheet Financial
            wWSheetFinancial = new WorksheetFinancial[4];

            wExpenseHour = (ChargeCode) wExpenseCategory.getChildren()[0];
            wWSheetFinancial[HOUR] = new WorksheetFinancial();
            wWSheetFinancial[HOUR].setParent(parent);
            wWSheetFinancial[HOUR].setChargeCode(wExpenseHour);

            wExpenseAvg = (ChargeCode) wExpenseCategory.getChildren()[1];
            wWSheetFinancial[AVG_COST] = new WorksheetFinancial();
            wWSheetFinancial[AVG_COST].setParent(parent);
            wWSheetFinancial[AVG_COST].setChargeCode(wExpenseAvg);

            wExpenseLabor = (ChargeCode) wExpenseCategory.getChildren()[2];
            wWSheetFinancial[EXP_LABOR] = new WorksheetFinancial();
            wWSheetFinancial[EXP_LABOR].setParent(parent);
            wWSheetFinancial[EXP_LABOR].setChargeCode(wExpenseLabor);

            wExpenseFTE = (ChargeCode) wExpenseCategory.getChildren()[3];
            wWSheetFinancial[FTE] = new WorksheetFinancial();
            wWSheetFinancial[FTE].setParent(parent);
            wWSheetFinancial[FTE].setChargeCode(wExpenseFTE);

        }
        
        return wWSheetFinancial;        
    }
    
    
    public static WbsFinancial[] createEmptyWbsFinancialSystemChargeCodesArray(
            WorkElement parent, ClientTestController controller) throws Exception
    {
        WbsFinancial[] wbsFinancial = null;
        String SystemChargeCodeName = "System Charge Codes";
        RPMObject[] resultArray = null;
        ChargeCodeCategory wCCCategory = null;
        ChargeCodeCategory wExpenseCategory = null;
        
        ChargeCode wExpenseHour = null;
        ChargeCode wExpenseAvg = null;
        ChargeCode wExpenseLabor = null;
        ChargeCode wExpenseFTE = null;


        final int HOUR = 0;
        final int AVG_COST = 1;
        final int EXP_LABOR = 2;
        final int FTE = 3;

        // ChargeCodeScope needed to load the ChargeCodeCategory
        ChargeCodeScope wCCMainScope = new ChargeCodeScope();
        wCCMainScope.setChildren(new ChargeCodeScope());
        wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

        resultArray = controller.loadFromXpath(
                "/ChargeCodeCategory[name='" + SystemChargeCodeName + "']",
                wCCMainScope);
        

        if (resultArray != null
                && resultArray[0] != null
                && (resultArray[0] instanceof ChargeCodeCategory)
                && parent != null)
        {
            wCCCategory = (ChargeCodeCategory) resultArray[0];
            wExpenseCategory = (ChargeCodeCategory) wCCCategory.getChildren()[0];
            
            //Creating Worksheet Financial
            wbsFinancial = new WbsFinancial[4];

            wExpenseHour = (ChargeCode) wExpenseCategory.getChildren()[0];
            wbsFinancial[HOUR] = new WbsFinancial();
            wbsFinancial[HOUR].setParent(parent);
            wbsFinancial[HOUR].setChargeCode(wExpenseHour);

            wExpenseAvg = (ChargeCode) wExpenseCategory.getChildren()[1];
            wbsFinancial[AVG_COST] = new WbsFinancial();
            wbsFinancial[AVG_COST].setParent(parent);
            wbsFinancial[AVG_COST].setChargeCode(wExpenseAvg);

            wExpenseLabor = (ChargeCode) wExpenseCategory.getChildren()[2];
            wbsFinancial[EXP_LABOR] = new WbsFinancial();
            wbsFinancial[EXP_LABOR].setParent(parent);
            wbsFinancial[EXP_LABOR].setChargeCode(wExpenseLabor);

            wExpenseFTE = (ChargeCode) wExpenseCategory.getChildren()[3];
            wbsFinancial[FTE] = new WbsFinancial();
            wbsFinancial[FTE].setParent(parent);
            wbsFinancial[FTE].setChargeCode(wExpenseFTE);

        }
        
        return wbsFinancial;        
    }
    
    public static WbsFinancial[] createEmptyWbsFinancialSystemChargeCodesArrayForScopElement(
            ScopeElement parent, ClientTestController controller)
            throws Exception
    {
        if (parent != null
                && (parent instanceof AbstractScopeTask || parent instanceof AbstractAggregateScope))
        {
            WbsFinancial[] wbsFinancial = null;
            String SystemChargeCodeName = "System Charge Codes";
            RPMObject[] resultArray = null;
            ChargeCodeCategory wCCCategory = null;
            ChargeCodeCategory wExpenseCategory = null;

            ChargeCode wExpenseHour = null;
            ChargeCode wExpenseAvg = null;
            ChargeCode wExpenseLabor = null;
            ChargeCode wExpenseFTE = null;

            final int HOUR = 0;
            final int AVG_COST = 1;
            final int EXP_LABOR = 2;
            final int FTE = 3;

            // ChargeCodeScope needed to load the ChargeCodeCategory
            ChargeCodeScope wCCMainScope = new ChargeCodeScope();
            wCCMainScope.setChildren(new ChargeCodeScope());
            wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

            resultArray = controller.loadFromXpath("/ChargeCodeCategory[name='"
                    + SystemChargeCodeName + "']", wCCMainScope);

            if (resultArray != null && resultArray[0] != null
                    && (resultArray[0] instanceof ChargeCodeCategory)
                    && parent != null)
            {
                wCCCategory = (ChargeCodeCategory) resultArray[0];
                wExpenseCategory = (ChargeCodeCategory) wCCCategory
                    .getChildren()[0];

                // Creating Worksheet Financial
                wbsFinancial = new WbsFinancial[4];

                wExpenseHour = (ChargeCode) wExpenseCategory.getChildren()[0];
                wbsFinancial[HOUR] = new WbsFinancial();
                wbsFinancial[HOUR].setParent(parent);
                wbsFinancial[HOUR].setChargeCode(wExpenseHour);

                wExpenseAvg = (ChargeCode) wExpenseCategory.getChildren()[1];
                wbsFinancial[AVG_COST] = new WbsFinancial();
                wbsFinancial[AVG_COST].setParent(parent);
                wbsFinancial[AVG_COST].setChargeCode(wExpenseAvg);

                wExpenseLabor = (ChargeCode) wExpenseCategory.getChildren()[2];
                wbsFinancial[EXP_LABOR] = new WbsFinancial();
                wbsFinancial[EXP_LABOR].setParent(parent);
                wbsFinancial[EXP_LABOR].setChargeCode(wExpenseLabor);

                wExpenseFTE = (ChargeCode) wExpenseCategory.getChildren()[3];
                wbsFinancial[FTE] = new WbsFinancial();
                wbsFinancial[FTE].setParent(parent);
                wbsFinancial[FTE].setChargeCode(wExpenseFTE);

            }

            return wbsFinancial;
        }
        else
        {
            return null;
        }
    }

    
}
