/*
 * Created on 27-Oct-06
 * @author mmatwiy
 * 
 * Test cases for defect 57046. 
 * Saving a timephased container directly doesn't update its parent WorksheetFinancial's values.
 */
package com.ibm.rpm.client.junit.financial.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test57046 extends ClientTestController
{
    private WorksheetFinancial worksheetFinancial = null;

    private FinancialScope finScope = null;

    public void setUp() throws Exception
    {
        super.setUp();

        finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setParent(new WorkElementScope());

        // create a parent Project and WorksheetFinancial
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());
        save(project, null);

        worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(project);

        project.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        worksheetFinancial.setChargeCode(ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this));
    }

    public void testSaveTimephasedOnlyIgnoreParentModification()
            throws Exception
    {
        // save the parent first
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        estimateTimePhased05.setParent(worksheetFinancial);

        // we're going to ignore this change because we're not saving the parent
        worksheetFinancial.setEstimate(new Double(1.0));

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(15.0));
    }

    public void testSaveTimephasedAddWorksheetValues() throws Exception
    {
        // set a worksheet estimate then save the parent first
        worksheetFinancial.setEstimate(new Double(11.0));
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        estimateTimePhased05.setParent(worksheetFinancial);

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(26.0));
    }

    public void testDeductTimephasedFromWorksheetValues() throws Exception
    {
        // set a worksheet estimate then save the parent first
        worksheetFinancial.setEstimate(new Double(20.0));
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        estimateTimePhased05.setParent(worksheetFinancial);

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(35.0));

        // deduct the month values by 9 and make sure the worksheet
        // estimate value is reduced by 9
        estimateTimePhased05.setMonth01Value(new Double(4.0));
        estimateTimePhased05.setMonth02Value(new Double(2.0));
        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded2);
        assertNotNull(reloaded2.getEstimate());
        assertEquals(reloaded2.getEstimate(), new Double(26.0));
    }
    
    public void testClearTimephasedDeductFromWorksheetValues() throws Exception
    {
        // set a worksheet estimate then save the parent first
        worksheetFinancial.setEstimate(new Double(20.0));
        worksheetFinancial = (WorksheetFinancial) save(worksheetFinancial,
            finScope).getRpmObject();

        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        estimateTimePhased05.setParent(worksheetFinancial);

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(35.0));

        // deduct the month values by 9 and make sure the worksheet
        // estimate value is reduced by 9
        estimateTimePhased05.setMonth01Value(new Double(0));
        estimateTimePhased05.setMonth02Value(new Double(0));
        save(estimateTimePhased05, tpFinScope);

        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded2);
        assertNotNull(reloaded2.getEstimate());
        assertEquals(reloaded2.getEstimate(), new Double(20.0));
    }

    public void testSaveWorksheet() throws Exception
    {
        // create estimates for 2005
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            estimateTimePhased05
        });

        save(worksheetFinancial, finScope);

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheetFinancial, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(15.0));
    }
}
