package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author rbadette
 * 
 * This class will test all failures financials basic business logic rules
 */
public class TestFinancialBusinessLogicValidation extends GenericValidation
{
    private static final String INVALID_PARAMETER = "INVALID_CHARGECODE_DEFECT_48651";

    private static final String EXPENSE_HOURS = "EXPENSE_HOURS";

    public void setUp() throws Exception
    {
        super.setUp();

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testDefect_47402() throws AssertionError, Exception
    {
        // We should not be able to save an invalid system charge code with
        // 1 or 2 or 3 ChargeCode so a checkpoint is added to verify that.

        // Portfolio
        Portfolio portfolio = new Portfolio();
        portfolio.setName(HelperTestUtilities.createName());

        // Proposal
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);
        proposal.setAssignmentType(AssignmentType.Work);

        // Scope used to load Charge Codes from RPM
        ChargeCodeScope ccScope = new ChargeCodeScope();
        ccScope.setChildren(new ChargeCodeScope());
        ccScope.getChildren().setChildren(new ChargeCodeScope());

        RPMObject[] objs = null;

        objs = loadFromXpath("/ChargeCodeCategory[name='System Charge Codes']",
            ccScope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof ChargeCodeCategory);

        // Get All Charge Codes from the Charge Code Category
        ChargeCodeCategory ccCategory = (ChargeCodeCategory) objs[0];
        assertNotNull(ccCategory.getChildren());
        assertTrue(ccCategory.getChildren()[0] instanceof ChargeCodeCategory);
        assertEquals("Expense", ccCategory.getChildren()[0].getName());

        ccCategory = (ChargeCodeCategory) ccCategory.getChildren()[0];
        assertNotNull(ccCategory.getChildren());
        assertEquals("1. Hours", ccCategory.getChildren()[0].getName());
        assertEquals("2. Average Cost / Hr", ccCategory.getChildren()[1]
            .getName());
        assertEquals("3. Expense Labor", ccCategory.getChildren()[2].getName());
        assertEquals("4. FTE", ccCategory.getChildren()[3].getName());

        // Financial Work Sheets
        WorksheetFinancial[] wsFinancials = new WorksheetFinancial[1];
        wsFinancials[0] = new WorksheetFinancial();
        wsFinancials[0].setChargeCode((ChargeCode) ccCategory.getChildren()[0]);

        // Scope used to save the portfolio
        PortfolioScope pfScope = new PortfolioScope();
        pfScope.setProjects(new WorkElementScope());
        pfScope.getProjects().setWorksheetFinancials(new FinancialScope());
        pfScope.getProjects().getWorksheetFinancials().setChargeCode(true);

        // Linking all objects together
        portfolio.setProjects(new Proposal[]
        {
            proposal
        });
        proposal.setWorksheetFinancials(wsFinancials);

        // List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_SYSTEM_WORKSHEET_FINANCIALS,
                proposal.getClass().getName(),
                ValidationConstants.WORKSHEET_FINANCIALS_FIELD, proposal
                    .getID())
        };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(portfolio, pfScope, expectedErrors, null);
    }

    public void testDefect_48073() throws AssertionError, Exception
    {
        // We should not be able to delete a WorkSheetFinancial SystemChargeCode
        // that has been check-in

        // Proposal
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        // Saving to RPM
        SaveResult saveResult = save(proposal, wes);
        assertNotNull(saveResult);
        assertNotNull(saveResult.getRpmObject());

        proposal = (Proposal) saveResult.getRpmObject();

        // Testing that delete cannot be performed and that the related error
        // message is thrown
        for (int i = 0; i < proposal.getWorksheetFinancials().length; i++)
        {
            // List of expected Errors...
            RPMExceptionMock[] expectedErrors =
            {
                createRPMException(ErrorCodes.NO_DELETE, proposal
                    .getWorksheetFinancials()[i].getClass().getName())
            };

            verifyDeleteMustFail(proposal.getWorksheetFinancials()[i],
                expectedErrors, null);
        }

    }

    public void testDefect_48651() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        // Saving to RPM
        SaveResult saveResult = null;
        saveResult = save(proposal, wes);

        assertNotNull(saveResult);
        assertTrue(saveResult.getRpmObject() instanceof Proposal);
        proposal = (Proposal) saveResult.getRpmObject();

        // --------------------------Test---------------------------------|

        // Modifying saved proposal
        int hours = 50;
        int percent = 25;

        ChargeCode matchingCC = new ChargeCode();
        matchingCC.setName("1. Hours");

        WorksheetFinancial wsfHours = FinancialUtil
            .getWorksheetFinancialWithChargeCode(proposal, matchingCC);
        assertNotNull(wsfHours);

        wsfHours.setEstimate(new Double(hours));
        wsfHours.setProbabilityPercent(new Double(percent));

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        // List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_BENEFITS_CATEGORY,
                getMatchingChargeCode(EXPENSE_HOURS,
                    proposal.getWorksheetFinancials()).getClass().getName(),
                com.ibm.rpm.financial.constants.ValidationConstants.PROBABILITY_PERCENT_FIELD,
                getMatchingChargeCode(EXPENSE_HOURS,
                    proposal.getWorksheetFinancials()).getID())
        };

        // //List of expected Warnings...
        // RPMExceptionMock[] expectedWarnings =
        // {
        // createRPMException(
        // com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_FTE_READONLY,
        // getMatchingWorksheetinancial(FTE,
        // proposal.getWorksheetFinancials()).getClass().getName(),
        // null, getMatchingWorksheetinancial(FTE,
        // proposal.getWorksheetFinancials()).getID())
        // };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(proposal, wess, expectedErrors, null);
    }

    private ChargeCode getMatchingChargeCode(String chargecode,
            WorksheetFinancial[] financial)
    {
        int i = 0;
        ChargeCode wrongChargeCode = new ChargeCode();
        wrongChargeCode.setName(INVALID_PARAMETER);
        wrongChargeCode.setID(INVALID_PARAMETER);

        if (chargecode == null || financial == null)
        {
            return wrongChargeCode;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode().getID().startsWith(chargecode))
                {
                    break;
                }
            }
            return financial[i].getChargeCode();
        }
    }

    private WorksheetFinancial getMatchingWorksheetinancial(String chargecode,
            WorksheetFinancial[] financial)
    {
        int i = 0;
        WorksheetFinancial wrongWorksheet = new WorksheetFinancial();
        wrongWorksheet.setID(INVALID_PARAMETER);

        if (chargecode == null || financial == null)
        {
            return wrongWorksheet;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode().getID().startsWith(chargecode))
                {
                    break;
                }
            }
            return financial[i];
        }
    }

    public void testDefect_56424_externalID() throws Exception
    {
        // Should not be able to delete a chargecode that has external_id set

        // Creating ChargeCode
        ChargeCode customChargeCode = null;
        customChargeCode = ChargeCodeFixture.createSimpleChargeCode();
        customChargeCode.setName(HelperTestUtilities.createName());
        customChargeCode.setExternalID("TEST_REFERENCE_NUMBER");

        // Scope used to load Charge Codes from RPM
        ChargeCodeScope ccScope = new ChargeCodeScope();
        ccScope.setChildren(new ChargeCodeScope());
        ccScope.getChildren().setChildren(new ChargeCodeScope());

        // Saving ChargeCode
        SaveResult saveResult = save(customChargeCode.getParent(), ccScope);
        assertNotNull(saveResult.getRpmObject());

        // Loading ChargeCode
        RPMObject[] objs = null;
        objs = loadFromXpath("/ChargeCode[name='" + customChargeCode.getName()
                + "']", ccScope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof ChargeCode);

        ChargeCode testChargeCode = (ChargeCode) objs[0];

        // List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(ErrorCodes.NO_DELETE, testChargeCode.getClass()
                .getName())
        };

        // Trying to Delete ChargeCode
        verifyDeleteMustFail(testChargeCode, expectedErrors, null);

    }

    public void testDefect_56424_systemCC() throws Exception
    {
        // Should not be able to delete a system chargecode

        // Scope used to load Charge Codes from RPM
        ChargeCodeScope ccScope = new ChargeCodeScope();
        ccScope.setChildren(new ChargeCodeScope());
        ccScope.getChildren().setChildren(new ChargeCodeScope());

        RPMObject[] objs = null;
        objs = loadFromXpath("/ChargeCodeCategory[name='System Charge Codes']",
            ccScope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof ChargeCodeCategory);

        // Get All Charge Codes from the Charge Code Category
        ChargeCodeCategory ccCategory = (ChargeCodeCategory) objs[0];
        assertNotNull(ccCategory.getChildren());
        assertTrue(ccCategory.getChildren()[0] instanceof ChargeCodeCategory);

        assertEquals("Expense", ccCategory.getChildren()[0].getName());

        ccCategory = (ChargeCodeCategory) ccCategory.getChildren()[0];

        assertNotNull(ccCategory.getChildren());
        assertEquals("1. Hours", ccCategory.getChildren()[0].getName());
        assertEquals("2. Average Cost / Hr", ccCategory.getChildren()[1]
            .getName());
        assertEquals("3. Expense Labor", ccCategory.getChildren()[2].getName());
        assertEquals("4. FTE", ccCategory.getChildren()[3].getName());

        // Testing that delete cannot be performed and that the related error
        // message is thrown
        for (int i = 0; i < ccCategory.getChildren().length; i++)
        {
            // List of expected Errors...
            RPMExceptionMock[] expectedErrors =
            {
                createRPMException(ErrorCodes.NO_DELETE, ccCategory
                    .getChildren()[i].getClass().getName())
            };

            verifyDeleteMustFail(ccCategory.getChildren()[i], expectedErrors,
                null);
        }
    }

    public void testDefect_56709_InvalidRootChargeCode() throws Exception
    {
        // Should not be able to load an object
        // ID:PMO_PROJECT__TIMEPHASE_CALENDAR__
        // because it is an invalid ChargeCode but that exist in the DB for
        // obscure purposes.
        // So we filtered it out in ChargeCodeManager.getFilter()

        ChargeCode cCode = new ChargeCode();
        cCode.setID("PMO_PROJECT_TIMEPHASE_CALENDAR__");

        // List of expected Errors...
        RPMExceptionMock expectedErrors = createRPMException(
            ErrorCodes.AM_INVALID_RPMOBJECTID, cCode.getClass().getName(),
            null, cCode.getID());

        verifyLoadFromIDMustFail(cCode, null, expectedErrors);
    }

    // AVERAGE cannot be set alone on update if HOUR & EXPENSE_LABOR are null
    // or empty...
    public void testDefect_61272_AVG_UpdateOnEmptySave() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        SaveResult result = null;
        result = save(proposal, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        // --------------------------Test---------------------------------|

        Proposal loaded = (Proposal) result.getRpmObject();
        WorksheetFinancial expenseAvgWorksheetReloaded = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", loaded.getWorksheetFinancials());

        expenseAvgWorksheetReloaded.setEstimate(new Double(20));

        // List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                expenseAvgWorksheetReloaded.getClass().getName(), "average",
                expenseAvgWorksheetReloaded.getID())
        };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(loaded, wess, expectedErrors, null);
    }

    // AVERAGE cannot be set alone on create
    public void testDefect_61272_AVG_CREATE() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

        expenseAvgWorksheet.setEstimate(new Double(20));

        // --------------------------Test---------------------------------|

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        // List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                expenseAvgWorksheet.getClass().getName(), "average",
                expenseAvgWorksheet.getID())
        };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(proposal, wess, expectedErrors, null);
    }

    // REMAINING & EAC are READ-ONLY so a warning should be throwned
    public void testDefect_61272_REMAINING_EAC() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        WorksheetFinancial expenseHoursWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", proposal.getWorksheetFinancials());

        WorksheetFinancial expenseLaborWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", proposal.getWorksheetFinancials());

        // --------------------------Test---------------------------------|
        expenseHoursWorksheet.setRemaining(new Double(20));

        expenseLaborWorksheet.setEAC(new Double(20));

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        // List of expected Errors...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    expenseHoursWorksheet.getClass().getName(), "EAC",
                    expenseHoursWorksheet.getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    expenseLaborWorksheet.getClass().getName(), "REMAINING",
                    expenseLaborWorksheet.getID())
        };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(proposal, wess, null, expectedWarnings);
    }
    
    
    //  FTE IS READ-ONLY AND SHOULD NOT BE WRITABLE (ETC)
        public void testSetFTE_ETCChargeCodes() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        WorksheetFinancial fteWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_FULL_TIME_EQUIVALENT", proposal.getWorksheetFinancials());

        //------Test        
        fteWorksheet.setETC(new Double(20));

        //---Test End

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        // List of expected Errors...
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_FTE_READONLY,
                fteWorksheet.getClass().getName())
        };

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(proposal, wess, null, expectedWarnings);

    }
        
        
        //  FTE IS READ-ONLY AND SHOULD NOT BE WRITABLE (ACTUAL)
            public void testSetFTEActualChargeCodes() throws Exception
        {
            // --------------------INIT---------------//

            Proposal proposal = null;

            // Proposal
            proposal = new Proposal();
            proposal.setName(HelperTestUtilities.createName());
            proposal.setPublished(ProjectPublishedType.None);

            WorksheetFinancial[] wsFinancials = FinancialFixture
                .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

            WorkElementScope wes = new WorkElementScope();
            wes.setWorksheetFinancials(new FinancialScope());
            wes.getWorksheetFinancials().setChargeCode(true);

            // Linking objects together
            proposal.setWorksheetFinancials(wsFinancials);

            WorksheetFinancial fteWorksheet = getMatchingWorksheetinancial(
                "EXPENSE_FULL_TIME_EQUIVALENT", proposal.getWorksheetFinancials());

            //------Test        
            fteWorksheet.setActual(new Double(20));

            //---Test End

            // creating proper scoping
            WorkElementScope wess = new WorkElementScope();
            wess.setWorksheetFinancials(new FinancialScope());
            wess.getWorksheetFinancials().setChargeCode(true);

            // List of expected Errors...
            RPMExceptionMock[] expectedWarnings =
            {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_FTE_READONLY,
                    fteWorksheet.getClass().getName())
            };

            // Testing that saving to RPM does not work and the required message is
            // thrown
            verifySaveMustFail(proposal, wess, null, expectedWarnings);

        }



            //  FTE IS READ-ONLY AND SHOULD NOT BE WRITABLE (ESTIMATE)
                public void testSetFTEEstimateChargeCodes() throws Exception
            {
                // --------------------INIT---------------//

                Proposal proposal = null;

                // Proposal
                proposal = new Proposal();
                proposal.setName(HelperTestUtilities.createName());
                proposal.setPublished(ProjectPublishedType.None);

                WorksheetFinancial[] wsFinancials = FinancialFixture
                    .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

                WorkElementScope wes = new WorkElementScope();
                wes.setWorksheetFinancials(new FinancialScope());
                wes.getWorksheetFinancials().setChargeCode(true);

                // Linking objects together
                proposal.setWorksheetFinancials(wsFinancials);

                WorksheetFinancial fteWorksheet = getMatchingWorksheetinancial(
                    "EXPENSE_FULL_TIME_EQUIVALENT", proposal.getWorksheetFinancials());

                //------Test        
                fteWorksheet.setEstimate(new Double(20));
                //---Test End

                // creating proper scoping
                WorkElementScope wess = new WorkElementScope();
                wess.setWorksheetFinancials(new FinancialScope());
                wess.getWorksheetFinancials().setChargeCode(true);

                // List of expected Errors...
                RPMExceptionMock[] expectedWarnings =
                {
                    createRPMException(
                        com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_FTE_READONLY,
                        fteWorksheet.getClass().getName())
                };

                // Testing that saving to RPM does not work and the required message is
                // thrown
                verifySaveMustFail(proposal, wess, null, expectedWarnings);

            }
                
                //AVG-COST is not writable by itself and so this test
                //should failed. (ETC)
                public void testSetAverageCostETCChargeCodes() throws Exception
            {
                // --------------------INIT---------------//

                Proposal proposal = null;

                // Proposal
                proposal = new Proposal();
                proposal.setName(HelperTestUtilities.createName());
                proposal.setPublished(ProjectPublishedType.None);

                WorksheetFinancial[] wsFinancials = FinancialFixture
                    .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

                WorkElementScope wes = new WorkElementScope();
                wes.setWorksheetFinancials(new FinancialScope());
                wes.getWorksheetFinancials().setChargeCode(true);

                // Linking objects together
                proposal.setWorksheetFinancials(wsFinancials);

                WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
                    "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

                //------Test        
                expenseAvgWorksheet.setETC(new Double(20));
                //---Test End

                // creating proper scoping
                WorkElementScope wess = new WorkElementScope();
                wess.setWorksheetFinancials(new FinancialScope());
                wess.getWorksheetFinancials().setChargeCode(true);

                // List of expected Errors...
                RPMExceptionMock[] expectedErrors =
                {
                        createRPMException(
                            com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                            expenseAvgWorksheet.getClass().getName(), "average",
                            expenseAvgWorksheet.getID())
                };

                // Testing that saving to RPM does not work and the required message is
                // thrown
                verifySaveMustFail(proposal, wess, expectedErrors, null);

            }
                
                //AVG-COST is not writable by itself and so this test
                //should failed. (ACTUAL)
                public void testSetAverageCostActualChargeCodes() throws Exception
            {
                // --------------------INIT---------------//

                Proposal proposal = null;

                // Proposal
                proposal = new Proposal();
                proposal.setName(HelperTestUtilities.createName());
                proposal.setPublished(ProjectPublishedType.None);

                WorksheetFinancial[] wsFinancials = FinancialFixture
                    .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

                WorkElementScope wes = new WorkElementScope();
                wes.setWorksheetFinancials(new FinancialScope());
                wes.getWorksheetFinancials().setChargeCode(true);

                // Linking objects together
                proposal.setWorksheetFinancials(wsFinancials);

                WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
                    "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

                //------Test        
                expenseAvgWorksheet.setActual(new Double(20));
                //---Test End

                // creating proper scoping
                WorkElementScope wess = new WorkElementScope();
                wess.setWorksheetFinancials(new FinancialScope());
                wess.getWorksheetFinancials().setChargeCode(true);

                // List of expected Errors...
                RPMExceptionMock[] expectedErrors =
                {
                        createRPMException(
                            com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                            expenseAvgWorksheet.getClass().getName(), "average",
                            expenseAvgWorksheet.getID())
                };

                // Testing that saving to RPM does not work and the required message is
                // thrown
                verifySaveMustFail(proposal, wess, expectedErrors, null);

            }


                //AVG-COST is not writable by itself and so this test
                //should failed. (ESTIMATE)
                public void testSetAverageCostEstimateChargeCodes() throws Exception
            {
                // --------------------INIT---------------//

                Proposal proposal = null;

                // Proposal
                proposal = new Proposal();
                proposal.setName(HelperTestUtilities.createName());
                proposal.setPublished(ProjectPublishedType.None);

                WorksheetFinancial[] wsFinancials = FinancialFixture
                    .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

                WorkElementScope wes = new WorkElementScope();
                wes.setWorksheetFinancials(new FinancialScope());
                wes.getWorksheetFinancials().setChargeCode(true);

                // Linking objects together
                proposal.setWorksheetFinancials(wsFinancials);

                WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
                    "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

                //------Test        
                expenseAvgWorksheet.setEstimate(new Double(20));
                //---Test End

                // creating proper scoping
                WorkElementScope wess = new WorkElementScope();
                wess.setWorksheetFinancials(new FinancialScope());
                wess.getWorksheetFinancials().setChargeCode(true);

                // List of expected Errors...
                RPMExceptionMock[] expectedErrors =
                {
                        createRPMException(
                            com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_AVERAGECOST_USAGE,
                            expenseAvgWorksheet.getClass().getName(), "average",
                            expenseAvgWorksheet.getID())
                };

                // Testing that saving to RPM does not work and the required message is
                // thrown
                verifySaveMustFail(proposal, wess, expectedErrors, null);

            }


}
