package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.TimePhasedFinancialFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author ikusakov
 */
public class TestTimePhasedFinancialValidation extends GenericValidation
{
    private static final Class TIMEPHASED_FINANCIAL_CLASS = TimePhasedFinancial.class;

    private static final String TIMEPHASED_FINANCIAL_CLASS_NAME = TimePhasedFinancial.class
            .getName();

    private TimePhasedFinancial mainTimePhasedFinancial;
    
    private TimePhasedFinancialScope scope = null;

    protected void setUp() throws Exception
    {
        super.setUp();
        scope = new TimePhasedFinancialScope();
        scope.setParent(new FinancialScope());

        mainTimePhasedFinancial = TimePhasedFinancialFixture
                .createWorksheetFixture(scope, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        ChargeCode wrongParent = new ChargeCode();
        mainTimePhasedFinancial.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                    mainTimePhasedFinancial.getClass().getName(), "parent",
                    mainTimePhasedFinancial.getID())
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws Exception
    {
        // Set field with maximum values.
        mainTimePhasedFinancial.setMonth01Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth02Value(new Double(
                ValidationConstants.FINANCIAL_MIN));
        mainTimePhasedFinancial.setMonth03Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth04Value(new Double(
                ValidationConstants.FINANCIAL_MIN));
        mainTimePhasedFinancial.setMonth05Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth06Value(new Double(
                ValidationConstants.FINANCIAL_MIN));
        mainTimePhasedFinancial.setMonth07Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth08Value(new Double(
                ValidationConstants.FINANCIAL_MIN));
        mainTimePhasedFinancial.setMonth09Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth10Value(new Double(
                ValidationConstants.FINANCIAL_MIN));
        mainTimePhasedFinancial.setMonth11Value(new Double(
                ValidationConstants.FINANCIAL_MAX));
        mainTimePhasedFinancial.setMonth12Value(new Double(
                ValidationConstants.FINANCIAL_MIN));

        saveMustSucceed(mainTimePhasedFinancial, TIMEPHASED_FINANCIAL_CLASS,
                null);
    }

    public void testSuccessValidationNull() throws Exception
    {
        mainTimePhasedFinancial.setMonth01Value(null);
        mainTimePhasedFinancial.setMonth02Value(null);
        mainTimePhasedFinancial.setMonth03Value(null);
        mainTimePhasedFinancial.setMonth04Value(null);
        mainTimePhasedFinancial.setMonth05Value(null);
        mainTimePhasedFinancial.setMonth06Value(null);
        mainTimePhasedFinancial.setMonth07Value(null);
        mainTimePhasedFinancial.setMonth08Value(null);
        mainTimePhasedFinancial.setMonth09Value(null);
        mainTimePhasedFinancial.setMonth10Value(null);
        mainTimePhasedFinancial.setMonth11Value(null);
        
        // we have to put at least something not-null, otherwise SP_U_PBB will delete the row
        mainTimePhasedFinancial.setMonth12Value(new Double(1));

        saveMustSucceed(mainTimePhasedFinancial, TIMEPHASED_FINANCIAL_CLASS,
                null);
    }

    public void testFailureValidation() throws Exception
    {
        // Set field with invalid values.
        mainTimePhasedFinancial.setMonth01Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth02Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainTimePhasedFinancial.setMonth03Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth04Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainTimePhasedFinancial.setMonth05Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth06Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainTimePhasedFinancial.setMonth07Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth08Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainTimePhasedFinancial.setMonth09Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth10Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainTimePhasedFinancial.setMonth11Value(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainTimePhasedFinancial.setMonth12Value(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));

        String mainId = mainTimePhasedFinancial.getID();
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month01", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month02", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month03", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month04", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month05", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month06", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month07", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month08", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month09", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month10", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month11", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "month12", mainId),
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }

    public void testFailureWithoutParentID() throws Exception
    {
        mainTimePhasedFinancial.setParent(new WorksheetFinancial());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID,
                    TIMEPHASED_FINANCIAL_CLASS_NAME, "parent",
                    mainTimePhasedFinancial.getID()),
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }
    
    public void testReadOnlyFields() throws Exception
    {
        // Set read-only field value
        mainTimePhasedFinancial.setTotalForYear(new Double(19.0));

        String mainId = mainTimePhasedFinancial.getID();
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "totalForYear", mainId),
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, null, expectedWarnings);
    }
    
    public void testFailureMandatoryFields() throws Exception
    {
        // Send null value for mandatory field
        mainTimePhasedFinancial.setBudgetType(null);

        String mainId = mainTimePhasedFinancial.getID();
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY,
                        TIMEPHASED_FINANCIAL_CLASS_NAME, "budgetType", mainId),
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }
    
    public void testFailureMandatoryFieldsInsert() throws Exception
    {
        // Send null value for mandatory field
        mainTimePhasedFinancial.setBudgetType(null);
        
        // parent's parent id is mandatory for inserts
        mainTimePhasedFinancial.getParent().getParent().setID(null);
        
        // parent's charge code id is mandatory for inserts and updates
        ((AbstractFinancial) mainTimePhasedFinancial.getParent()).getChargeCode().setID(null);

        String mainId = mainTimePhasedFinancial.getID();
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY,
                    TIMEPHASED_FINANCIAL_CLASS_NAME, "budgetType", mainId),
                createRPMException(ErrorCodes.NO_ID,
                    TIMEPHASED_FINANCIAL_CLASS_NAME,
                    ValidationConstants.PARENT_PARENT_FIELD, mainId),
                createRPMException(ErrorCodes.NO_ID,
                    TIMEPHASED_FINANCIAL_CLASS_NAME,
                    ValidationConstants.PARENT_CHARGE_CODE_FIELD, mainId),
        };

        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }
    
    public void testFailureUpdateWithoutParent() throws Exception
    {
        mainTimePhasedFinancial.setBudgetType(BudgetType.Estimate);
        save(mainTimePhasedFinancial, null);
        
        // reload the timephased container without the parent in scope
        TimePhasedFinancial reloaded = (TimePhasedFinancial) loadFromID(
            mainTimePhasedFinancial, null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MISSING_PRIMARY_KEY,
                TIMEPHASED_FINANCIAL_CLASS_NAME, null, mainTimePhasedFinancial
                    .getID())
        };

        // this is validated in isValidPrimaryKey
        verifySaveMustFail(reloaded, null, expectedExceptions);
    }
    
    
    public void testFailureUpdateWithoutBudgetType() throws Exception
    {
        mainTimePhasedFinancial.setBudgetType(BudgetType.Estimate);
        save(mainTimePhasedFinancial, null);

        // send null value for mandatory budgetType
        mainTimePhasedFinancial.setBudgetType(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MISSING_PRIMARY_KEY,
                TIMEPHASED_FINANCIAL_CLASS_NAME, null, mainTimePhasedFinancial
                    .getID())
        };

        // this is validated in isValidPrimaryKey
        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }
    
    public void testFailureMandatoryFieldsUpdate() throws Exception
    {
        mainTimePhasedFinancial.setBudgetType(BudgetType.Estimate);
        
        SingleResult result = save(mainTimePhasedFinancial, scope);
        
        mainTimePhasedFinancial = (TimePhasedFinancial) result.getRpmObject();

        // the parent's charge code id is mandatory
        ((AbstractFinancial)mainTimePhasedFinancial.getParent()).getChargeCode().setID(null);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MISSING_PRIMARY_KEY,
                TIMEPHASED_FINANCIAL_CLASS_NAME, null, mainTimePhasedFinancial
                    .getID())
        };

        
        verifySaveMustFail(mainTimePhasedFinancial, null, expectedExceptions);
    }
}
