package com.ibm.rpm.client.junit.fixtures;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.framework.testcases.util.ScheduleDateUtil;
import com.ibm.rpm.framework.containers.ScheduleDate;

public abstract class ScheduleDateFixture extends RPMObjectFixture 
{
	protected static ArrayList localSkipGetters = new ArrayList();
	protected static ArrayList localSkipSetters = new ArrayList();
	public static int nbChildren = 1;
	static  {
    	
	    //set skipGetters
    	localSkipGetters.addAll(skipGetters);
    	//read-only fields
    	localSkipGetters.add("getDays");
    	localSkipGetters.add("getHours");
    	localSkipGetters.add("getFinishDate");
    	//This field can only be overriden by a field from the parent
    	localSkipGetters.add("getWorkHours");
        
        // this field needs to be compared manually b/c it gets rounded up
        localSkipGetters.add("getDurationInMinutes");
        
        
    	
    	//set skipSetters
    	localSkipSetters.addAll(skipSetters);
      	//read-only fields
    	localSkipSetters.add("setDays");
    	localSkipSetters.add("setHours");
    	localSkipSetters.add("setFinishDate");
    	//This field can only be overriden by a field from the parent
    	localSkipSetters.add("setWorkHours");
    }
	
    public static void compare(ScheduleDate scheduleDate1, ScheduleDate scheduleDate2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(scheduleDate1, scheduleDate2, localSkipGetters);
        
        compareDurationInMinutse(scheduleDate1, scheduleDate2);
                
    }
    
    
    protected static ScheduleDate initSheduleDate(ScheduleDate scheduleDate){
        
        scheduleDate.setDurationInMinutes(new Integer(608));
        
        Calendar startDate = new GregorianCalendar();
        scheduleDate.setStartDate(startDate);
        
        return scheduleDate;
    }
    
    protected static ScheduleDate updateScheduleDate(ScheduleDate scheduleDate){
        
        scheduleDate.setDurationInMinutes(new Integer(1208));
        Calendar updatedStartDate= scheduleDate.getStartDate();
        int year = updatedStartDate.get(Calendar.YEAR);
        int month = updatedStartDate.get(Calendar.MONDAY);
        int day = updatedStartDate.get(Calendar.DAY_OF_MONTH);
        
        updatedStartDate = new GregorianCalendar(year + 1, month, day);
        scheduleDate.setStartDate(updatedStartDate);
        
        return scheduleDate;
    }
    
    private static void compareDurationInMinutse(ScheduleDate scheduleDate1,
            ScheduleDate scheduleDate2) throws AssertionFailedError, Exception
    {
        if (scheduleDate1 != null && scheduleDate2 != null)
        {
            int roundedDuration1 = ScheduleDateUtil
                .roundDurationUpToQuarterHour(scheduleDate1
                    .getDurationInMinutes());
            int roundedDuration2 = ScheduleDateUtil
                .roundDurationUpToQuarterHour(scheduleDate2
                    .getDurationInMinutes());

            Assert.assertEquals(roundedDuration1, roundedDuration2);
        }
    }
}





