/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.client.junit.framework.testcases;

import java.lang.reflect.Array;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *  
 */
public abstract class ModuleTestController extends ClientTestController
{
    protected String moduleContaineName;

    protected RPMObjectScope mainScope;
    
    public abstract RPMObject load(AbstractModule module, RPMObjectScope scope)
            throws Exception;

    protected abstract void deleteModule(AbstractModule module,
            RPMObjectScope scope) throws AssertionError, Exception;

    protected abstract AbstractModule createFixture(RPMObjectScope scope)
            throws Exception;

    protected abstract AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception;

    protected abstract void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception;

    //dummy test to verify foreignkey conditions between the managers
    public void testLoadFromXPath() throws AssertionError, Exception
    {
        RPMObject[] objs = loadFromXpath("/"+moduleContaineName, mainScope);
        assertNotNull(objs);
    }    
    
    public void testInsertWithScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        //insert an module content
        AbstractModule module = null;

        module = insertModule(mainScope);

        //delete the module content
        deleteModule(module, mainScope);

    }

    public void testUpdateWithScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        //insert an module content
        AbstractModule module = null;

        module = insertModule(mainScope);
        module = updateModule(module, mainScope);

        //delete the module content
        deleteModule(module, mainScope);
    }

    private AbstractModule insertModule(RPMObjectScope scope)
            throws AssertionError, Exception
    {
        AbstractModule module = createFixture(scope);

        //Insert the element into the DB
        SingleResult result = save(module, scope);
        
        //compare the returned element
        AbstractModule returnedModule = (AbstractModule) result.getRpmObject();
        compare(module, returnedModule, scope);
        
        //reload the element and compare it
        returnedModule = (AbstractModule) load(returnedModule, scope);
        compare(module, returnedModule, scope);

        //return the element
        return returnedModule;
    }

    private AbstractModule updateModule(AbstractModule module,
            RPMObjectScope scope) throws AssertionError, Exception
    {
        updateFixture(module, scope);

        //update the element
        save(module, scope);

        //reload the element and compare it
        AbstractModule returnedModule = (AbstractModule) load(module, scope);
        compare(module, returnedModule, scope);

        //return the element
        return returnedModule;
    }

    protected AbstractModule loadDeletedModule(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        try
        {
            AbstractModule deletedModule = (AbstractModule) load(module, scope);
            return deletedModule;
        }
        catch (Exception e)
        {
            //module does not exist
        }
        return null;
    }

    protected AttributeCategory[] loadAttributeCategories(
            AttributeCategory[] attributeCategories, AttributeScope scope)
            throws Exception
    {
        return (AttributeCategory[])loadElements(attributeCategories, scope, AttributeCategory.class);
    }
    
    protected GenericProject[] loadGenericProjects(
            GenericProject[] genericProjects, WorkElementScope scope)
            throws Exception
    {
        return (GenericProject[])loadElements(genericProjects, scope, GenericProject.class);
    }

    protected DatafieldRTF[] loadRtfs(
            DatafieldRTF[] rtfs, boolean isInScopeFlag, Class baseClass)
            throws Exception
    {
        return (DatafieldRTF[])loadElements(rtfs, null, isInScopeFlag, baseClass);            
    }

    protected Datafield[] loadDatafields(
            Datafield[] datafields, boolean isInScopeFlag, Class baseClass)
            throws Exception
    {
        return (Datafield[])loadElements(datafields, null, isInScopeFlag, baseClass);            
    }
    
    protected DatafieldCategory[] loadDatafieldCategories(
            DatafieldCategory[] datafieldCategories, boolean isInScopeFlag, Class baseClass)
            throws Exception
    {
        return (DatafieldCategory[])loadElements(datafieldCategories, null, isInScopeFlag, baseClass);            
    }
    
    protected RPMObject[] loadElements(
            RPMObject[] oldObjs, RPMObjectScope scope, Class baseClass)
            throws Exception
    {
        boolean dummy = false;
        return loadElements(oldObjs, scope, dummy, baseClass);
    }
    
    private RPMObject[] loadElements(
            RPMObject[] oldObjs, RPMObjectScope scope, boolean isInScopeFlag, Class baseClass)
            throws Exception
    {
        RPMObject[] loadedObjs = null;

        if(scope!=null || isInScopeFlag)
        {
            if(oldObjs!=null && oldObjs.length!=0)
            {
                String containerName = StringUtil.getShortClassName(baseClass);
                String XPathCondition = constructXPathCondition(
                        oldObjs, containerName);
                RPMObject[] objs = loadFromXpath("/" + moduleContaineName
                        + XPathCondition, scope);

                if (objs != null && objs.length != 0)
                {
                    loadedObjs = constructArray(objs, baseClass);
                }                            
            }
        }

        return loadedObjs;
    }
	
    private String constructXPathCondition(
            RPMObject[] objs, String containerName)
    {
        StringBuffer XPathCondition = new StringBuffer();

        if (objs != null && objs.length != 0)
        {
            XPathCondition.append("/"+containerName+"[");
            for (int i = 0; i < objs.length; i++)
            {
                if (i != 0)
                {
                    XPathCondition.append(" or ");
                }               
                if(objs[i] instanceof Currency)                	
                	XPathCondition.append("ID=" + objs[i].getID());                
                else                	
                	XPathCondition.append("ID='" + objs[i].getID() + "'");
            }
            XPathCondition.append("]");
        }

        return XPathCondition.toString();
    }

    private RPMObject[] constructArray(
            RPMObject[] rpmObjects, Class baseClass)
    {
        RPMObject[] array = (RPMObject[])Array.newInstance(baseClass, rpmObjects.length);
        for (int i = 0; i < rpmObjects.length; i++)
        {
            array[i] = rpmObjects[i];
        }
        return array;
    }
     
}
