package com.ibm.rpm.client.junit.framework.testcases;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.constants.AdditionalInformation;
import com.ibm.rpm.metadata.model.CalendarPrecisionMap;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.CreationMode;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.metadata.model.MetadataUtility;
import com.ibm.rpm.metadata.model.PointerLink;
import com.ibm.rpm.metadata.model.Rule;
import com.ibm.rpm.metadata.model.Scope;

/**
 * <p>
 * TODO test load and save with all the scope flags set one by one
 * <p>
 * TODO test XPath
 * <p>
 * TODO support test where object is created by the parent
 * <p>
 * TODO for validation, make tests on both create and update step
 * <p>
 * TODO test string with unicode and large UTF-8 characters max
 * <p>
 * TODO test double assignment of the same object in arrays of attributes
 * <p>
 * TODO test assignments of objects that don't exist in the DB
 * <p>
 * TODO test negative xpath results
 * <p>
 * 
 * @author mcduffs
 */
public abstract class RpmObjectAbstractTester
{
    public static Log logger = LogFactory.getLog(ClientTestController.class);

    private Container classInformation;

    private ArrayList updateFixtures = new ArrayList();

    private ArrayList compareFixture = new ArrayList();

    private ArrayList errorList = new ArrayList();

    private ArrayList errorMessage = new ArrayList();

    private ArrayList pendingTestList = new ArrayList();

    private ArrayList fixtureErrors = new ArrayList();

    private ClientTestController testController;

    private boolean isPlanned = false;

    private boolean failOnFirstError = false;

    private int startTestIndex = 0;

    private int stopTestIndex = Integer.MAX_VALUE;

    private RPMObject xpathFixture;

    private RPMObject fullLoadFixture;

    private RPMObject badUpdateFixture = null;

    public RpmObjectAbstractTester(Class setObjectClass)
    {
        this(setObjectClass, ClientTestController.initTestController());
    }

    public RpmObjectAbstractTester(Class setObjectClass,
            ClientTestController setTestController)
    {
        Assert.assertNotNull(setObjectClass);
        Assert.assertNotNull(setTestController);
        classInformation = MetadataStore.getInstance().getContainerByJavaClass(
            setObjectClass);
        testController = setTestController;
    }

    public void addFixtureError(String errorDescription)
    {
        fixtureErrors.add(errorDescription);
    }

    public int getStartTestIndex()
    {
        return startTestIndex;
    }

    public void setStartTestIndex(int startTestIndex)
    {
        this.startTestIndex = startTestIndex;
    }

    public int getStopTestIndex()
    {
        return stopTestIndex;
    }

    public void setStopTestIndex(int stopTestIndex)
    {
        this.stopTestIndex = stopTestIndex;
    }

    public boolean isFailOnFirstError()
    {
        return failOnFirstError;
    }

    public void setFailOnFirstError(boolean failOnFirstError)
    {
        this.failOnFirstError = failOnFirstError;
    }

    public Container getClassInformation()
    {
        return classInformation;
    }

    protected String getNameFieldName()
    {
        return classInformation.getNameField().getName();
    }

    public Field getFieldInformation(String fieldName)
    {
        return classInformation.getField(fieldName);
    }

    public Class getObjectClass()
    {
        return classInformation.getJavaClass();
    }

    public RPMObject createObject() throws Exception
    {
        return createObject(0);
    }

    public RPMObject createObject(int seed) throws Exception
    {
        // logger.debug("creating object: " + classInformation.getJavaClass());
        RPMObject retVal = null;
        // Container currentClassInformation = getClassInformation();
        // TODO : support the create by parent and abstract create mode as well
        // if (currentClassInformation.creationModeValue() ==
        // CreationMode.NORMAL)
        // {
        retVal = (RPMObject) getObjectClass().getConstructor(null).newInstance(
            null);
        // }
        // else
        // {
        // Assert.fail("Automatic test fixtures support only normal object
        // creation mode");
        // // retVal = loadStaticObject(seed, new RPMObjectScope());
        // }
        return retVal;
    }

    /**
     * @param seed
     * @return
     * @throws Exception
     */
    private RPMObject loadStaticObject(int seed, RPMObjectScope scope)
            throws Exception
    {
        RPMObject retVal;
        // object can't be created, just load one

        ArrayResult loadResult = testController.getApplicationInterface()
            .loadFromXpath(testController.getSessionID(),
                getLoadUncreatableObjectXPath(seed), scope);

        if (!loadResult.isSuccessful())
        {
            Assert.fail("failed to load uncreatable object : "
                    + loadResult.getErrors()[0].getMessage());
        }

        if (loadResult.getRpmObjectList().length == 0)
        {
            Assert.fail("failed to load uncreatable object : empty result set");
        }

        int returnIndex = loadResult.getRpmObjectList().length
                - (seed % loadResult.getRpmObjectList().length) - 1;

        retVal = loadResult.getRpmObjectList()[returnIndex];
        return retVal;
    }

    public RPMObject[] createObjectArray(int size) throws Exception
    {
        RPMObject[] retVal = new RPMObject[size];

        for (int i = 0; i < size; i++)
        {
            retVal[i] = createObject(i);
        }

        return retVal;
    }

    public void fillObject(RPMObject objectToFill, RPMObjectScope scope,
            boolean forCreation) throws Exception
    {
        fillObject(objectToFill, scope, 0, forCreation);
    }

    public void fillObject(RPMObject objectToFill, RPMObjectScope scope,
            int seed, boolean forCreation) throws Exception
    {
        int i = 0;
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();

            if (forCreation)
            {
                if (!fieldInfo.isReadOnlyOnCreate())
                {
                    if (isFieldInScope(fieldInfo, scope)
                            || !fieldInfo.createCanBeNullValue())
                    {
                        fillField(objectToFill, fieldInfo, seed + i,
                            forCreation);
                    }
                }
            }
            else
            {
                if (!fieldInfo.isReadOnlyOnUpdate())
                {
                    fillField(objectToFill, fieldInfo, seed + i, forCreation);
                }
            }
        }
    }

    public void fillField(RPMObject objectToFill, Field fieldInfo, int seed,
            boolean forCreation) throws Exception, ClassNotFoundException
    {
        // logger.debug("filling field: "
        // + StringUtil.getShortClassName(objectToFill.getClass())
        // + " " + fieldInfo.getName());

        Object fillValue = null;
        if (fieldInfo.getName().equals("parent")
                && classInformation.creationModeValue() == CreationMode.CREATE_BY_PARENT)
        {
            PointerLink pointer = (PointerLink) fieldInfo
                .getValidTypePointerList().get(0);
            RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
                .findFixture(pointer.getLastClass());
            fillValue = findFixture.createSavedTestFixture();
        }
        else
        {
            fillValue = HelperTestUtilities.findSetValue(objectToFill,
                classInformation, fieldInfo, seed, forCreation);
        }

        fillField(objectToFill, fieldInfo, fillValue);
    }

    public boolean isFieldInScope(Field fieldInfo, RPMObjectScope scope)
            throws Exception
    {
        Field scopeField = fieldInfo.getScopeField();
        if (scopeField != null)
        {
            Object invoke = internalGetScopeObject(fieldInfo, scope);
            if (scopeField.isBoolean() && invoke != null)
            {
                Boolean retVal = (Boolean) invoke;
                return retVal.booleanValue();
            }
            return invoke != null;
        }
        return true;
    }

    public RPMObjectScope getFieldLinkedScope(Field fieldInfo,
            RPMObjectScope scope) throws Exception
    {
        Object invoke = null;
        Field scopeField = fieldInfo.getScopeField();
        if (scopeField != null)
        {
            if (scope == null)
            {
                return null;
            }

            if (!scopeField.isBoolean())
            {
                invoke = internalGetScopeObject(fieldInfo, scope);
            }
        }
        return (RPMObjectScope) invoke;
    }

    private Object internalGetScopeObject(Field fieldInfo, RPMObjectScope scope)
            throws Exception
    {
        Object invoke = null;
        Field scopeField = fieldInfo.getScopeField();
        if (scopeField != null)
        {
            if (scope == null)
            {
                return invoke;
            }

            // we might be using an invalid scope. Check if the getter method
            // exists for scope to be used before calling it
            Method getterMethod = scopeField.getGetterMethod();
            Method[] allScopeMethods = scope.getClass().getDeclaredMethods();
            boolean found = false;
            for (int i = 0; i < allScopeMethods.length && !found; i++)
            {
                if (allScopeMethods[i].equals(getterMethod))
                {
                    found = true;
                }
            }

            if (!found)
            {
                return invoke;
            }

            invoke = getterMethod.invoke(scope, null);

        }
        return invoke;
    }

    public void fillMandatoryFields(RPMObject objectToFill, boolean forCreation)
            throws Exception
    {
        fillMandatoryFields(objectToFill, 0, forCreation);
    }

    public void fillMandatoryFields(RPMObject objectToFill, int seed,
            boolean forCreation) throws Exception
    {
        int i = 0;
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            boolean needToFill = false;
            if (forCreation)
            {
                if (!fieldInfo.createCanBeNullValue())
                {
                    needToFill = true;
                }
            }
            else
            {
                if (!fieldInfo.canBeNullValue())
                {
                    needToFill = true;
                }
            }

            if (needToFill)
            {
                fillField(objectToFill, fieldInfo, seed + i++, forCreation);
            }
        }
    }

    public void fillFieldsWithDefaultValues(RPMObject objectToFill)
            throws Exception
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();

            if (fieldInfo.getDefaultValue() != null)
            {
                // get the current value
                Method methodToInvoke = fieldInfo.getGetterMethod();
                Object currValue = methodToInvoke.invoke(objectToFill,
                    new Object[0]);

                if (currValue == null)
                {
                    fillField(objectToFill, fieldInfo, fieldInfo
                        .getTypedDefaultValue());
                }
            }
        }
    }

    public void fillField(RPMObject objectToFill, Field fieldInfo, Object value)
            throws Exception
    {
        fillField(objectToFill, fieldInfo, value, fieldInfo.getSetterMethod());
    }

    public void fillField(RPMObject objectToFill, Field fieldInfo,
            Object value, Method methodToCall) throws Exception
    {
        Object[] args = new Object[1];
        args[0] = value;

        if (methodToCall == null)
        {
            Assert.fail("no method to set field " + fieldInfo.getName());
        }
        try
        {
            methodToCall.invoke(objectToFill, args);
        }
        catch (Exception e)
        {
            throw new Exception("Failed to assign field: "
                    + fieldInfo.getName() + ", with value: " + value, e);
        }
    }

    public Object getFieldValue(RPMObject object, Field fieldInfo,
            Method methodToCall) throws Exception
    {
        Object[] args = new Object[0];

        if (methodToCall == null)
        {
            Assert.fail("no method to set field " + fieldInfo.getName());
        }

        return methodToCall.invoke(object, args);
    }

    public void fillObjectArray(RPMObject[] objectToFill, RPMObjectScope scope,
            boolean forCreation) throws Exception
    {
        for (int i = 0; i < objectToFill.length; i++)
        {
            RPMObject object = objectToFill[i];
            fillObject(object, scope, i, forCreation);
        }
    }

    public SingleResult saveObject(RPMObject objectToSave, RPMObjectScope scope)
            throws Exception
    {
        SingleResult save = testController.save(objectToSave, scope);
        return save;
    }

    public SingleResult saveMustFail(RPMObject objectToSave,
            RPMObjectScope scope) throws Exception
    {
        return testController.saveMustFail(objectToSave, scope);
    }

    public RPMObject loadObject(RPMObject objectToLoad, RPMObjectScope scope)
            throws Exception
    {
        return testController.loadFromID(objectToLoad, scope);
    }

    public RPMObject[] loadArray(RPMObject[] objectsToLoad, RPMObjectScope scope)
            throws Exception
    {
        return testController.loadArrayFromID(objectsToLoad, scope);
    }

    public void updateObject(RPMObject objectToUpdate, int seed)
            throws Exception
    {

        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            if (!fieldInfo.isReadOnlyOnUpdate())
            {
                updateField(objectToUpdate, fieldInfo, seed);
            }
        }
    }

    public void updateField(RPMObject objectToUpdate, Field fieldInfo, int seed)
            throws Exception
    {
        Method getterMethod = fieldInfo.getGetterMethod();
        Object result = getterMethod.invoke(objectToUpdate, null);
        Container container = MetadataStore.getInstance()
            .getContainerByJavaClass(objectToUpdate.getClass());
        Object newValue = HelperTestUtilities.findUpdateValue(container,
            fieldInfo, result, seed);

        fillField(objectToUpdate, fieldInfo, newValue);
    }

    public void updateOneField(RPMObject objectToUpdate, int seed)
            throws Exception
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field info = (Field) it.next();
            if (!info.isArrayType()
                    && !info.isReadOnlyOnUpdate()
                    && info.comparableValue()
                    && !RPMObject.class.isAssignableFrom(info
                        .getJavaTypeNoArray()))
            {
                updateField(objectToUpdate, info, seed);
            }
        }
    }

    public Method findGetterMethod(String fieldName)
    {
        Method retVal = null;

        return retVal;
    }

    public void assertCompare(RPMObject source, RPMObject destination,
            RPMObjectScope scope, boolean afterCreation) throws Exception
    {
        List compareErrors = new ArrayList();
        boolean compareObject = compareObject(source, destination, scope,
            compareErrors, afterCreation);
        Iterator it = compareErrors.iterator();
        StringBuffer errorBuffer = new StringBuffer();
        errorBuffer.append("Compare errors:");
        while (it.hasNext())
        {
            AssertionFailedError element = (AssertionFailedError) it.next();
            element.printStackTrace();
            errorBuffer.append("\n   ");
            errorBuffer.append(element.getMessage());
        }
        Assert.assertTrue(errorBuffer.toString(), compareObject);
    }

    public boolean compareObject(Object source, Object destination,
            RPMObjectScope scope, List compareErrors, boolean afterCreation)
            throws Exception
    {
        Assert.assertNotSame(source, destination);

        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            compareField(fieldInfo, source, destination, scope, compareErrors,
                afterCreation);
        }

        return compareErrors.isEmpty();
    }

    public boolean compareField(Field fieldInfo, Object source,
            Object destination, RPMObjectScope scope, List compareErrors,
            boolean afterCreation) throws Exception
    {
        boolean retVal = true;

        if (afterCreation)
        {
            if (!fieldInfo.createComparableValue())
            {
                return retVal;
            }
        }
        else if (!fieldInfo.comparableValue())
        {
            return retVal;
        }

        if (!isFieldInScope(fieldInfo, scope))
        {
            return retVal;
        }

        Method getterMethod = fieldInfo.getGetterMethod();

        Object sourceValue = getterMethod.invoke(source, null);
        Object destinationValue = getterMethod.invoke(destination, null);

        String currentCompare = "compare "
                + classInformation.getShortTypeName() + " field "
                + fieldInfo.getName();

        try
        {
            // if the source is parent and is a type of Module, then don't
            // compare because when we load an object which has Module as its
            // parent back, the parent field is null
            if (!((sourceValue instanceof AbstractModule) && fieldInfo
                .getName().equals("parent")))
            {
                RPMObjectScope fieldScope = getFieldLinkedScope(fieldInfo,
                    scope);

                compareFieldAndThrow(fieldInfo, compareErrors, sourceValue,
                    destinationValue, currentCompare, fieldScope, afterCreation);
            }
        }
        catch (AssertionFailedError e)
        {
            retVal = false;
            compareErrors.add(e);
        }
        return retVal;
    }

    public void compareFieldAndThrow(Field fieldInfo, List compareErrors,
            Object sourceValue, Object destinationValue, String currentCompare,
            RPMObjectScope scope, boolean afterCreation) throws Exception
    {
        // use default value for comparison when sourceValue is null
        if (sourceValue == null && afterCreation)
        {
            sourceValue = fieldInfo.getTypedDefaultValue();
        }

        if (fieldInfo.isArrayType())
        {
            Object[] sourceArray = (Object[]) sourceValue;
            Object[] destinationArray = (Object[]) destinationValue;
            compareArrays(fieldInfo, sourceArray, destinationArray,
                currentCompare, scope, afterCreation);
        }
        else if (sourceValue == null)
        {
            Assert.assertNull(currentCompare
                    + " source is null, destination is not null",
                destinationValue);
        }
        else
        {
            Assert.assertNotNull(currentCompare
                    + " source is not null, destination is null",
                destinationValue);

            if (sourceValue instanceof RPMObject)
            {
                RPMObject sourceRPMObject = (RPMObject) sourceValue;
                RPMObject destinationRPMObject = (RPMObject) destinationValue;

                compareSubObject(fieldInfo, compareErrors, currentCompare,
                    sourceRPMObject, destinationRPMObject, scope, afterCreation);
            }
            else if (sourceValue instanceof String
                    && destinationValue instanceof String
                    && fieldInfo.fixedLengthValue())
            {
                Number typedMaxValue = (Number) fieldInfo.getTypedMaxValue();
                int maxSize = 0;
                if (typedMaxValue != null)
                {
                    maxSize = typedMaxValue.intValue();
                }
                StringBuffer blank = new StringBuffer();
                for (int i = 0; i < maxSize - sourceValue.toString().length(); i++)
                {
                    blank.append(' ');
                }
                sourceValue = sourceValue + blank.toString();
                Assert.assertEquals(currentCompare, sourceValue,
                    destinationValue);
            }
            else if (sourceValue instanceof Calendar
                    && destinationValue instanceof Calendar)
            {
                try
                {
                    DateFormat df = null;

                    Rule calendarRule = fieldInfo
                        .getRule(AdditionalInformation.RULE_TYPE_VALIDATION);
                    String predefined = calendarRule.getPredefined();
                    df = CalendarPrecisionMap
                        .getFormatFromPrecision(predefined);

                    Assert.assertEquals(currentCompare, df
                        .format(((Calendar) sourceValue).getTime()), df
                        .format(((Calendar) destinationValue).getTime()));
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                    Assert.fail("failed to compare calendar : "
                            + e.getMessage());
                }
            }
            else
            {
                Assert.assertEquals(currentCompare, sourceValue,
                    destinationValue);
            }
        }
    }

    public void compareArrays(Field fieldInfo, Object[] sourceArray,
            Object[] destinationArray, String currentCompare,
            RPMObjectScope scope, boolean afterCreation) throws Exception
    {
        if (isArrayEmpty(sourceArray))
        {
            assertEmptyArray(currentCompare, destinationArray);
        }
        else if (isArrayEmpty(destinationArray))
        {
            assertEmptyArray(currentCompare, sourceArray);
        }
        else
        {
            // it is not empty
            if (RPMObject.class.isAssignableFrom(sourceArray.getClass()
                .getComponentType()))
            {
                Object[] modifiedDestination = (Object[]) destinationArray
                    .clone();
                for (int i = 0; i < sourceArray.length; i++)
                {
                    Object object = sourceArray[i];
                    findInDestinationArray(fieldInfo, currentCompare, object,
                        modifiedDestination, scope, afterCreation);
                }

                assertEmptyArray(currentCompare
                        + " remaining destination array should be empty",
                    modifiedDestination);
            }
            else
            {
                Assert.assertTrue("arrays of non-RPM objects are not equal",
                    Arrays.equals(sourceArray, destinationArray));
            }
        }
    }

    public void findInDestinationArray(Field fieldInfo, String currentCompare,
            Object sourceObject, Object modifiedDestination[],
            RPMObjectScope scope, boolean afterCreation) throws Exception
    {
        if (sourceObject == null)
        {
            return;
        }

        List tempErrors = new ArrayList();
        boolean found = false;

        for (int i = 0; i < modifiedDestination.length; i++)
        {
            Object destinationObject = modifiedDestination[i];
            if (destinationObject != null)
            {
                boolean isEqual = compareSubObject(fieldInfo, tempErrors,
                    currentCompare, sourceObject, destinationObject, scope,
                    afterCreation);

                if (isEqual)
                {
                    modifiedDestination[i] = null;
                    found = true;
                    break;
                }
            }
        }

        if (!found)
        {
            Assert.fail(currentCompare + " failed to find "
                    + sourceObject.getClass().getName());
        }

    }

    public void assertEmptyArray(String currentCompare,
            Object[] destinationArray)
    {
        if (!isArrayEmpty(destinationArray))
        {
            Assert.fail(currentCompare + " expected empty array");
        }
    }

    public static boolean isArrayEmpty(Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                Object object = array[i];
                if (object != null)
                {
                    return false;
                }
            }
        }
        return true;
    }

    public boolean compareSubObject(Field fieldInfo, List compareErrors,
            String currentCompare, Object sourceRPMObject,
            Object destinationRPMObject, RPMObjectScope scope,
            boolean afterCreation) throws Exception
    {
        boolean retVal = true;
        boolean isRPMObject = sourceRPMObject instanceof RPMObject;
        if (!fieldInfo.compositeValue())
        {
            if (!isRPMObject)
            {
                Assert
                    .fail("trying to compare object references to a non-rpm object");
            }

            String sourceID = null;

            if (sourceRPMObject != null)
            {
                sourceID = ((RPMObject) sourceRPMObject).getID();
            }

            String destinationID = null;

            if (destinationRPMObject != null)
            {
                destinationID = ((RPMObject) destinationRPMObject).getID();
            }

            try
            {
                Assert.assertEquals(currentCompare, sourceID, destinationID);
            }
            catch (AssertionFailedError e)
            {
                retVal = false;
                compareErrors.add(e);
            }
        }
        else
        {
            retVal = RpmTestFramework.INSTANCE.findFixture(
                sourceRPMObject.getClass()).compareObject(sourceRPMObject,
                destinationRPMObject, scope, compareErrors, afterCreation);
        }
        return retVal;
    }

    public RPMObject getBadUpdateFixture() throws Exception
    {
        if (badUpdateFixture == null)
        {
            int index = 0;
            RPMObjectScope fullScope = createFullScope();
            if (classInformation.creationModeValue() == CreationMode.NORMAL)
            {
                badUpdateFixture = createObject(index);
                fillObject(badUpdateFixture, fullScope, index, true);
                badUpdateFixture = saveObject(badUpdateFixture, fullScope)
                    .getRpmObject();
            }
            else
            {
                badUpdateFixture = loadStaticObject(index, fullScope);
            }
        }
        return badUpdateFixture;
    }

    public RPMObject getUpdateFixture(int index) throws Exception
    {
        RPMObject retVal = null;
        ensureListSize(updateFixtures, index);

        retVal = (RPMObject) updateFixtures.get(index);

        if (retVal == null)
        {
            createUpdateFixture(index);
            retVal = (RPMObject) updateFixtures.get(index);
        }
        return retVal;
    }

    public RPMObject getCompareFixture(int index) throws Exception
    {
        RPMObject retVal = null;
        ensureListSize(compareFixture, index);

        retVal = (RPMObject) compareFixture.get(index);

        if (retVal == null)
        {
            createUpdateFixture(index);
            retVal = (RPMObject) compareFixture.get(index);
        }
        return retVal;
    }

    public RPMObject loadStaticFixture(int index) throws Exception
    {
        String loadXPath = getLoadStaticFixtureXPath(index);

        RPMObject[] loadFromXpath = getTestController().loadFromXpath(
            loadXPath, createFullScope());

        RPMObject retVal = null;

        if (loadFromXpath == null || loadFromXpath.length == 0)
        {
            retVal = createStaticFixture(index);
        }
        else
        {
            retVal = loadFromXpath[0];
        }

        return retVal;
    }

    public RPMObject loadStaticFixture(RPMObject parent, int index)
            throws Exception
    {
        String loadXPath = getLoadStaticFixtureXPath(parent, index);

        RPMObject[] loadFromXpath = getTestController().loadFromXpath(
            loadXPath, null);

        RPMObject retVal = null;

        if (loadFromXpath == null || loadFromXpath.length == 0)
        {
            retVal = createStaticFixture(parent, index);
        }
        else
        {
            retVal = loadFromXpath[0];
        }

        return retVal;
    }

    public String getLoadStaticFixtureXPath(RPMObject parent, int index)
    {
        String parentCondition = "";
        if (parent != null)
        {
            parentCondition = "/"
                    + StringUtil.getShortClassName(parent.getClass())
                    + parentCondition;

            if (parent.getID() != null)
            {
                parentCondition += "[id='" + parent.getID() + "']";
            }

        }

        Field fieldInfo = classInformation.getNameField();

        String loadXPath = null;

        if (fieldInfo == null)
        {
            loadXPath = parentCondition + "/"
                    + StringUtil.getShortClassName(getObjectClass());
        }
        else
        {
            loadXPath = parentCondition + "/"
                    + StringUtil.getShortClassName(getObjectClass()) + "["
                    + getNameFieldName() + "='" + getStaticFixtureName(index)
                    + "']";
        }

        return loadXPath;
    }

    public String getLoadStaticFixtureXPath(int index) throws Exception
    {
        String loadXPath = "/" + StringUtil.getShortClassName(getObjectClass())
                + "[" + getNameFieldName() + "='" + getStaticFixtureName(index)
                + "']";
        return loadXPath;
    }

    public String getLoadUncreatableObjectXPath(int index) throws Exception
    {
        String loadXPath = "/" + StringUtil.getShortClassName(getObjectClass());
        return loadXPath;
    }

    public RPMObject createStaticFixture(int index) throws Exception
    {
        RPMObject createObject = null;
        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            createObject = createObject(index);

            fillMandatoryFields(createObject, true);

            Field nameField = classInformation.getNameField();
            if (nameField != null)
            {
                fillField(createObject, nameField, getStaticFixtureName(index));
            }

            createObject = getTestController().saveWithoutDelete(createObject,
                createFullScope()).getRpmObject();
        }
        else
        {
            createObject = loadStaticObject(index + 2, null);

            Field nameField = classInformation.getNameField();
            if (nameField != null)
            {
                fillField(createObject, nameField, getStaticFixtureName(index));
                createObject = getTestController().saveWithoutDelete(
                    createObject, null).getRpmObject();
            }
        }
        return createObject;
    }

    public RPMObject createStaticFixture(RPMObject parent, int index)
            throws Exception
    {
        RPMObject createObject = null;
        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            createObject = createObject(index);

            fillMandatoryFields(createObject, true);

            Field nameField = classInformation.getNameField();
            if (nameField != null)
            {
                fillField(createObject, nameField, getStaticFixtureName(index));
            }

            if (parent != null)
            {
                Field parentField = classInformation.getParentField();
                if (parentField != null)
                {
                    fillField(createObject, parentField, parent);
                }
            }

            createObject = getTestController().saveWithoutDelete(createObject,
                null).getRpmObject();

        }
        else
        {
            createObject = loadStaticObject(index + 2, null);
        }

        return createObject;
    }

    public void createUpdateFixture(int index) throws Exception
    {
        ensureListSize(compareFixture, index);
        ensureListSize(updateFixtures, index);

        RPMObject createObject = null;
        RPMObject compareObject = null;
        RPMObjectScope fullScope = createFullScope();
        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            createObject = createObject(index);
            fillObject(createObject, fullScope, index, true);
            createObject = saveObject(createObject, fullScope).getRpmObject();
            compareObject = createObject(index);
            copyMinimumIDFields(createObject, compareObject);
            compareObject = loadObject(compareObject, fullScope);
        }
        else
        {
            createObject = loadStaticObject(index, fullScope);
            compareObject = loadStaticObject(index, fullScope);
        }

        updateFixtures.set(index, createObject);
        compareFixture.set(index, compareObject);
    }

    public static void ensureListSize(List listToGrow, int index)
    {
        while (listToGrow.size() <= index)
        {
            listToGrow.add(null);
        }
    }

    public RpmTestFramework getTestFramework()
    {
        return RpmTestFramework.INSTANCE;
    }

    public ClientTestController getTestController()
    {
        return testController;
    }

    public String getStaticFixtureName(int index)
    {
        String retVal = "STATIC_"
                + StringUtil.getShortClassName(getObjectClass()) + "_" + index;
        return retVal;
    }

    public void testObject() throws Throwable
    {
        setUp();

        planTestList();

        runPlannedTest();

        failAllErrors();

        tearDown();
    }

    public void failAllErrors()
    {
        if (errorList.isEmpty())
        {
            return;
        }

        StringBuffer errorMessageBuffer = new StringBuffer();

        errorMessageBuffer.append("Test Failed with ");
        errorMessageBuffer.append(errorList.size());
        errorMessageBuffer.append(" errors on ");
        errorMessageBuffer.append(pendingTestList.size());
        errorMessageBuffer.append(" tests (");
        errorMessageBuffer.append(getClass().getName());
        errorMessageBuffer.append("):");

        for (Iterator it = errorList.iterator(); it.hasNext();)
        {
            Throwable error = (Throwable) it.next();
            error.printStackTrace();
        }

        int i = 0;
        for (Iterator it = errorMessage.iterator(); it.hasNext();)
        {
            String currentErrorMessage = (String) it.next();

            errorMessageBuffer.append("\n  ");
            errorMessageBuffer.append(i);
            errorMessageBuffer.append(": ");
            errorMessageBuffer.append(currentErrorMessage);

            i++;
        }

        Assert.fail(errorMessageBuffer.toString());
    }

    public void setUp()
    {
    }

    public void tearDown()
    {
    }

    public void planXPathTests() throws Exception
    {
        if (classInformation.creationModeValue() != CreationMode.STATIC)
        {
            // create one of those objects with each valid parent, load
            // object, verify that all the instances come back.
            addPlannedTest("testXPathOnAllParents", null);
        }

        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field element = (Field) it.next();
            planFieldXPath(element);
        }
    }

    public void planFieldXPath(Field element)
    {
        String fieldName = element.getName();
        if (element.xpathFunctionalValue())
        {
            if (element.isArrayType()
                    || RPMObject.class.isAssignableFrom(element
                        .getJavaTypeNoArray()))
            {
                // RPMObject relationship XPath

                // test /object[@ID='x']/child
                addPlannedTest("testXPathObjectSlashChildType", fieldName,
                    element);

                // test /object[@ID='x']/@fieldName
                addPlannedTest("testXPathObjectSlashFieldName", fieldName,
                    element);

                // test /object[child[@ID='x']]
                addPlannedTest("testXPathObjectConditionChildType", fieldName,
                    element);

                // test /object[@fieldName[@ID='x']]
                addPlannedTest("testXPathObjectConditionFieldName", fieldName,
                    element);
            }
            else
            {
                // base field XPath

                // test /Object[field='value']
                addPlannedTest("testXPathObjectBaseFieldSuccess", fieldName, element);
                addPlannedTest("testXPathObjectBaseFieldFailure", fieldName, element);
            }
        }
        else
        {
            // test failing field XPath /Object[field='value']
            addPlannedTest("testXPathFailedField", fieldName, element);
        }
    }

    public void testXPathOnAllParents() throws Exception
    {
        List objectWithParentList = createObjectOnAllParents();

        // The query generated here is of this type :
        // "/Object[@id='xxx']"
        for (Iterator iter = objectWithParentList.iterator(); iter.hasNext();)
        {
            RPMObject element = (RPMObject) iter.next();
            
            StringBuffer xpath = new StringBuffer();
            xpath.append(XpathTesterUtils.FORWARD_SLASH);
            xpath.append(classInformation.getShortTypeName());
            xpath.append("[@id='");
            xpath.append(element.getID());
            xpath.append("']");

            loadXpathAndFindResult(element, xpath.toString(),
                classInformation.getJavaClass());
        }
    }

    public List createObjectOnAllParents() throws Exception
    {
        List objectWithParentList = new ArrayList();

        Field parentField = classInformation.getParentField();

        CreationMode creationModeValue = classInformation.creationModeValue();

        if (parentField.createCanBeNullValue()
                && creationModeValue == CreationMode.NORMAL)
        {
            PointerLink pointerLink = MetadataUtility
                .createPointer(classInformation.getJavaClass());
            RPMObject pointerOnNullParent = RpmTestFramework.INSTANCE
                .getStaticFixtureFromPointer(pointerLink, 0);
            objectWithParentList.add(pointerOnNullParent);
        }

        Iterator it = parentField.getValidTypePointerList().iterator();
        while (it.hasNext())
        {
            PointerLink parentLink = (PointerLink) it.next();

            RPMObject parent = null;

            if (parentLink.getPointerClass() == getObjectClass())
            {
                parent = loadStaticFixture(0);
            }
            else
            {
                parent = RpmTestFramework.INSTANCE.getStaticFixtureFromPointer(
                    parentLink, 0);
            }

            objectWithParentList.add(createObjectWithParent(parent));
        }

        return objectWithParentList;
    }

    public RPMObject createObjectWithParent(RPMObject parent) throws Exception
    {
        return createObjectWithParent(parent, 0);
    }

    public RPMObject createObjectWithParent(RPMObject parent, int seed)
            throws Exception
    {
        RPMObject createObject = createObject(seed);

        fillMandatoryFields(createObject, true);

        if (classInformation.creationModeValue() == CreationMode.CREATE_BY_PARENT)
        {
            PointerLink pointer = (PointerLink) classInformation
                .getParentField().getValidTypePointerList().get(0);
            RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
                .findFixture(pointer.getLastClass());
            parent = findFixture.createSavedTestFixture();
        }

        createObject.setParent(parent);

        RPMObject savedObject = saveObject(createObject, null).getRpmObject();

        return savedObject;
    }

    public void testXPathFailedField(Field element) throws Exception
    {
        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@");
        xpath.append(element.getName());
        xpath.append("='x'");
        xpath.append("]");

        try
        {
            testController.loadFromXpath(xpath.toString(), null);
            Assert.fail("XPath with field " + element.getName()
                    + " shouldn't work.");
        }
        catch (RPMException e)
        {
            int errorCode = e.getErrorID();
            int errorCode_1 = ErrorCodes.XPATH_UNSUPPORTED_FIELD;
            int errorCode_2 = ErrorCodes.XPATH_UNSUPPORTED_OPERATION;

            Assert.assertTrue("Failed XPath error code mismatch",
                (errorCode == errorCode_1) || (errorCode == errorCode_2));
        }
    }

    public void testXPathObjectBaseFieldSuccess(Field fieldInfo) throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        Object currentValue = getXpathValue(fieldInfo, currentXpathFixture);

        // The query generated here is of this type :
        // "/Object[@id='xx' and @Field='field value']"
        // ex: "/Reference[@id='xx' and @OrganizationName='OrganizationName 39']"
        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@ID='");
        xpath.append(currentXpathFixture.getID());
        xpath.append("' and @");
        xpath.append(fieldInfo.getName());
        xpath.append("=");
 
        appendXpathValue(fieldInfo.getJavaTypeNoArray(), currentValue, xpath);

        xpath.append("]");

        loadXpathAndFindResult(currentXpathFixture, xpath.toString(),
            classInformation.getJavaClass());
    }


    public void testXPathObjectBaseFieldFailure(Field fieldInfo) throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        Object currentValue = getXpathValue(fieldInfo, currentXpathFixture);

        // The query generated here is of this type :
        // "/Object[@id='xx' and @Field='field value']"
        // ex: "/Reference[@id='xx' and @OrganizationName='OrganizationName 39']"
        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@ID='");
        xpath.append(currentXpathFixture.getID());
        xpath.append("' and @");
        xpath.append(fieldInfo.getName());
        xpath.append("=");
 
        // find a different value
        Object badValue = HelperTestUtilities.findUpdateValue(getClassInformation(), fieldInfo, currentValue ,0);
        appendXpathValue(fieldInfo.getJavaTypeNoArray(), badValue, xpath);

        xpath.append("]");

        String xpathString = xpath.toString();
        RPMObject[] loadResult = testController.loadFromXpath(xpathString, null);

        if( loadResult != null && loadResult.length > 0 )
        {
            Assert.fail( "Unexpected result found in XPath query : " + xpathString );
        }
    }    
    
    public void loadXpathAndFindResult(List objectsToFind, String xpath,
            Class fieldType) throws Exception
    {
        RPMObject[] loadResult = testController.loadFromXpath(xpath, null);

        Iterator it = objectsToFind.iterator();
        while (it.hasNext())
        {
            RPMObject findObject = (RPMObject) it.next();
            findLoadedObject(findObject, loadResult, xpath, fieldType);
        }
    }

    public void loadXpathAndFindResult(RPMObject currentXpathFixture,
            String xpath, Class fieldType) throws Exception
    {
        loadXpathAndFindResult(currentXpathFixture, xpath, null, fieldType);
    }

    public void loadXpathAndFindResult(RPMObject currentXpathFixture,
            String xpath, RPMObjectScope scope, Class fieldType)
            throws Exception
    {
        RPMObject[] loadResult = testController.loadFromXpath(xpath, scope);

        findLoadedObject(currentXpathFixture, loadResult, xpath, fieldType);
    }

    public void findLoadedObject(RPMObject currentXpathFixture,
            RPMObject[] loadResult, String xpath, Class fieldType)
    {
        if (loadResult != null)
        {
            for (int i = 0; i < loadResult.length; i++)
            {
                RPMObject object = loadResult[i];
                if (object.getID().equals(currentXpathFixture.getID()))
                {
                    return;
                }

                // make sure all the returned results match the expected
                // container type
                if (!fieldType.isInstance(object))
                {
                    Assert
                        .fail("XPath returned result incompatible with tested class. Tested class: "
                                + classInformation.getJavaClass().getName()
                                + ", found in the result: "
                                + object.getClass().getName()
                                + ", XPath: "
                                + xpath);
                }
            }
        }
        Assert.fail("failed to find object in load result. XPath: " + xpath);
    }

    public void appendXpathValue(Class fieldType, Object currentValue,
            StringBuffer buffer)
    {
        String currentStringValue = null;
        boolean wrapComma = false;

        if (fieldType == Calendar.class)
        {
            // Test to convert Calendars to a date value in format :
            // 'yyyy-MM-dd'
            currentStringValue = XpathTesterUtils
                .convertToString((Calendar) currentValue);
            wrapComma = true;
        }
        else if (fieldType == String.class)
        {
            currentStringValue = (String) currentValue;
            wrapComma = true;
        }
        else if (XpathTesterUtils.isTypeBoolean(fieldType))
        {
            Boolean value = (Boolean) currentValue;
            if (value.booleanValue())
            {
                currentStringValue = "true";
            }
            else
            {
                currentStringValue = "false";
            }
            wrapComma = true;
        }
        else if (XpathTesterUtils.isTypeNumeric(fieldType))
        {
            currentStringValue = currentValue.toString();
        }
        else if (XpathTesterUtils.isEnumeratedType(fieldType))
        {
            currentStringValue = currentValue.toString();
            wrapComma = true;
        }

        if (wrapComma)
        {
            buffer.append("'");
        }

        buffer.append(currentStringValue);

        if (wrapComma)
        {
            buffer.append("'");
        }
    }

    public RPMObject getXpathFixture(Field fieldInfo) throws Exception
    {
        if (xpathFixture == null)
        {
            RPMObjectScope fullScope = createFullScope();
            if (classInformation.creationModeValue() == CreationMode.NORMAL)
            {
                RPMObject createObject = createObject();
                fillObject(createObject, fullScope, 0, true);
                createObject = saveObject(createObject, fullScope)
                    .getRpmObject();

                xpathFixture = createObject();
                copyMinimumIDFields(createObject, xpathFixture);
                xpathFixture = loadObject(xpathFixture, fullScope);
            }
            else
            {
                xpathFixture = loadStaticObject(0, fullScope);
            }
        }
        return xpathFixture;
    }

    public void testXPathObjectSlashChildType(Field fieldInfo) throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        RPMObject currentValue = getRPMXpathValue(fieldInfo,
            currentXpathFixture);

        // The query generated here is of this type :
        // /object[@ID='x']/child

        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@ID='");
        xpath.append(currentXpathFixture.getID());
        xpath.append("']/");
        Class fieldType = fieldInfo.getJavaTypeNoArray();
        xpath.append(StringUtil.getShortClassName(fieldType));

        loadXpathAndFindResult(currentValue, xpath.toString(), fieldType);
    }

    public RPMObject getRPMXpathValue(Field fieldInfo,
            RPMObject currentXpathFixture) throws Exception
    {
        RPMObject retVal = null;
        Object objectValue = getXpathValue(fieldInfo, currentXpathFixture);
        if (fieldInfo.isArrayType())
        {
            Object[] array = (Object[]) objectValue;
            retVal = (RPMObject) array[0];
        }
        else
        {
            retVal = (RPMObject) objectValue;
        }
        return retVal;
    }

    public Object getXpathValue(Field fieldInfo, RPMObject currentXpathFixture)
            throws Exception
    {
        Object currentValue = fieldInfo.getGetterMethod().invoke(
            currentXpathFixture, null);

        Assert
            .assertNotNull(
                "unable to do XPath test, fixture returned a null value in a field",
                currentValue);

        return currentValue;
    }

    public void testXPathObjectSlashFieldName(Field fieldInfo) throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        RPMObject currentValue = getRPMXpathValue(fieldInfo,
            currentXpathFixture);

        // The query generated here is of this type :
        // /object[@ID='x']/@fieldName

        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@ID='");
        xpath.append(currentXpathFixture.getID());
        xpath.append("']/@");
        xpath.append(fieldInfo.getName());

        loadXpathAndFindResult(currentValue, xpath.toString(), fieldInfo
            .getJavaTypeNoArray());
    }

    public void testXPathObjectConditionChildType(Field fieldInfo)
            throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        RPMObject currentValue = getRPMXpathValue(fieldInfo,
            currentXpathFixture);

        // The query generated here is of this type :
        // /object[child[@ID='x']]

        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[");
        xpath.append(StringUtil.getShortClassName(fieldInfo
            .getJavaTypeNoArray()));
        xpath.append("[@ID=");
        xpath.append("'");
        xpath.append(currentValue.getID());
        xpath.append("'");
        xpath.append("]]");

        loadXpathAndFindResult(currentXpathFixture, xpath.toString(),
            classInformation.getJavaClass());
    }

    public void testXPathObjectConditionFieldName(Field fieldInfo)
            throws Exception
    {
        RPMObject currentXpathFixture = getXpathFixture(fieldInfo);

        RPMObject currentValue = getRPMXpathValue(fieldInfo,
            currentXpathFixture);

        // The query generated here is of this type :
        // /object[@fieldName[@ID='x']]

        StringBuffer xpath = new StringBuffer();
        xpath.append("/");
        xpath.append(classInformation.getShortTypeName());
        xpath.append("[@");
        xpath.append(fieldInfo.getName());
        xpath.append("[@ID='");
        xpath.append(currentValue.getID());
        xpath.append("']]");

        loadXpathAndFindResult(currentXpathFixture, xpath.toString(),
            classInformation.getJavaClass());
    }

    public void testDelete() throws Exception
    {
        RPMObject objectToDelete = createSavedTestFixture();

        validateDelete(objectToDelete);
    }

    public void testDeleteFlag() throws Exception
    {
        RPMObject objectToDelete = createSavedTestFixture();

        objectToDelete.setDeleted(Boolean.TRUE);

        try
        {
            getTestController().save(objectToDelete, null);
        }
        catch (Throwable ex)
        {
            // TODO Auto-generated catch block
            ex.printStackTrace();
        }

        assertDeleted(objectToDelete);
    }

    public void testDeleteFailure() throws Exception
    {
        RPMObject objectToDelete = createSavedTestFixture();

        Result deleteResult = testController.getApplicationInterface().delete(
            testController.getSessionID(), objectToDelete);

        validateDeleteFailureResult(deleteResult);
    }

    public void testDeleteFlagFailure() throws Exception
    {
        RPMObject objectToDelete = createSavedTestFixture();

        objectToDelete.setDeleted(Boolean.TRUE);

        Result deleteResult = getTestController().getApplicationInterface()
            .save(testController.getSessionID(), objectToDelete, null,
                ReloadType.SavedResult);

        validateDeleteFailureResult(deleteResult);
    }

    public void validateDeleteFailureResult(Result deleteResult)
    {
        Assert.assertFalse(deleteResult.isSuccessful());
        RPMException[] errors = deleteResult.getErrors();

        Assert.assertEquals(1, errors.length);

        RPMException currentError = errors[0];
        Assert.assertEquals(classInformation.getShortTypeName(), currentError
            .getClassName());
        Assert.assertEquals(ErrorCodes.NO_DELETE, currentError.getErrorID());
    }

    public void testLoadMissingPrimaryKey(Field fieldInfo) throws Exception
    {
        RPMObject createObject = createObject();
        fillMandatoryFields(createObject, true);

        // set primary key to null
        Method methodToInvoke = fieldInfo.getSetterMethod();
        methodToInvoke.invoke(createObject, new Object[]
        {
            null
        });

        try
        {
            loadObject(createObject, null);
        }
        catch (RPMException e)
        {
            // verify we got the right error code
            Assert.assertEquals(ErrorCodes.MISSING_PRIMARY_KEY, e.getErrorID());
        }
    }

    public void assertDeleted(RPMObject objectToDelete) throws Exception
    {
        // test loading deleted object
        SingleResult result = testController.getApplicationInterface()
            .loadFromID(testController.getSessionID(), objectToDelete, null);

        Assert.assertEquals(false, result.isSuccessful());
        Assert.assertEquals(1, result.getErrors().length);

        RPMException e = result.getErrors()[0];
        // verify we got the right error code
        Assert.assertEquals(ErrorCodes.AM_INVALID_RPMOBJECTID, e.getErrorID());

        // test deleting deleted object
        Result deleteResult = testController.getApplicationInterface().delete(
            testController.getSessionID(), objectToDelete);

        Assert.assertEquals(false, deleteResult.isSuccessful());
        Assert.assertEquals(1, deleteResult.getErrors().length);

        e = deleteResult.getErrors()[0];

        // verify we got the right error code
        Assert.assertEquals(ErrorCodes.AM_INVALID_RPMOBJECTID, e.getErrorID());

        testController.removeObjectToDelete(objectToDelete);
    }

    public void validateDelete(RPMObject objectToDelete) throws Exception
    {
        // force the parent to null
        objectToDelete.setParent(null);

        getTestController().delete(objectToDelete);

        assertDeleted(objectToDelete);
    }

    public RPMObject createSavedTestFixture() throws Exception
    {
        return createSavedTestFixture(0);
    }

    public RPMObject createSavedTestFixture(int index) throws Exception
    {
        RPMObject updateFixture = null;
        RPMObjectScope createMinimumScope = createMinimumScope(updateFixture);

        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            updateFixture = createObject(index);
            fillObject(updateFixture, createMinimumScope, index, true);
            updateFixture = saveObject(updateFixture, createMinimumScope)
                .getRpmObject();
        }
        else
        {
            updateFixture = loadStaticObject(index, createMinimumScope);
        }

        return updateFixture;
    }

    public void planValidation() throws Throwable
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field element = (Field) it.next();
            planFieldValidation(element);
        }
    }

    public void planFieldValidation(Field element) throws Exception
    {
        Class fieldType = element.getJavaTypeNoArray();
        String fieldName = element.getName();

        // creation specific validation tests
        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            if (element.isReadOnlyOnCreate())
            {
                // TODO plan a test to update the field and get a warning
            }
            else if (element.createCanBeNullValue())
            {
                addPlannedTest("testNullCreateSuccess", fieldName, element);
            }
            else
            {
                // null create failures are done in mandatory field testing
            }
        }

        if (element.isReadOnlyOnUpdate())
        {
            // TODO plan a test to update the field and get a warning
            return;
        }

        // test all the possible validations
        if (fieldType == null)
        {
            addPlannedTest("testFailure", "field type is null on field: "
                    + fieldName);
            return;
        }

        addPlannedTest("testUpdateField", fieldName, element);

        if (element.getScopeField() == null || !element.isArrayType()
                || !element.compositeValue())
        {
            // test null update on all fields that match one of the following
            // conditions:
            // basic fields (string, ints, double, etc...)
            // object relationships that have a cardinality of 0..1
            // object array relationships that are references

            // if null value is not allowed on creation then we should plan
            // a test for this
            if (element.canBeNullValue())
            {
                // test that when we create an object with a null value, the
                // value stays null (no default applied)
                addPlannedTest("testNullUpdateSuccess", fieldName, element);
            }
            else
            {
                // no need to test null creation, its already done in mandatory
                // field creation testing and default values testing
                addPlannedTest("testNullUpdateFailure", fieldName, element);
            }
        }

        if (fieldType == String.class)
        {
            // test string length
            addPlannedTest("testStringMaximumLengthSuccess", fieldName, element);
            addPlannedTest("testStringMaximumLengthFailure", fieldName, element);

            if (element.getMin() != null)
            {
                addPlannedTest("testStringMinimumLengthFailure", fieldName,
                    element);
            }
            addPlannedTest("testStringMinimumLengthSuccess", fieldName, element);

            // test invalid chars
            String invalidChars = element.getInvalidChars();
            if (invalidChars != null)
            {
                int length = invalidChars.length();
                for (int i = 0; i < length; ++i)
                {
                    Character invalidChar = new Character(invalidChars
                        .charAt(i));

                    addPlannedTest("testStringInvalidChars", fieldName,
                        element, invalidChar);
                }
            }
        }
        else if (fieldType == Calendar.class)
        {
            // test date max value
            addPlannedTest("testDateMaximumValueFailure", fieldName, element);
            addPlannedTest("testDateMaximumValueSuccess", fieldName, element);

            // test date min value
            addPlannedTest("testDateMinimumValueFailure", fieldName, element);
            addPlannedTest("testDateMinimumValueSuccess", fieldName, element);
        }
        else if (Number.class.isAssignableFrom(fieldType))
        {
            // test number max value
            addPlannedTest("testNumberMaximumValueFailure", fieldName, element);
            addPlannedTest("testNumberMaximumValueSuccess", fieldName, element);

            // test number min value
            addPlannedTest("testNumberMinimumValueFailure", fieldName, element);
            addPlannedTest("testNumberMinimumValueSuccess", fieldName, element);
        }
        else if (element.isEnumeration())
        {
            List fields = null;
            if (element.getEnumerationInfo() != null)
            {
                fields = element.getEnumerationInfo().getStates();
            }
            else
            {
                throw new Exception(
                    "The enumeration list has not been filled. Please call the method fillEnumerationField() in your test");
            }

            for (int i = 0; i < fields.size(); i++)
            {
                Object field = fields.get(i);
                // add a test for each enumerated field
                addPlannedTest("testUpdateFieldWithSeed", fieldName
                        + " enum value: " + field.toString(), element,
                    new Integer(i));
            }
        }

        List invalidChildTypeList = element.getInvalidTypePointerList();
        if (invalidChildTypeList != null)
        {
            // test invalid child types
            for (Iterator iter = invalidChildTypeList.iterator(); iter
                .hasNext();)
            {
                PointerLink invalidChildType = (PointerLink) iter.next();
                addPlannedTest("testInvalidChildType", fieldName + " "
                        + invalidChildType.getLastClassName(), element,
                    invalidChildType);
            }
        }
    }

    public void testInvalidChildType(Field element, PointerLink invalidChildType)
            throws Exception
    {
        int errorCode = 0; // set the error code

        // test invalid child type
        if (!element.compositeValue())
        {
            // invalid child reference
            if (element.isArrayType())
            {
                Assert.fail("unsupported child type : array of reference");
            }
            else
            {
                // set invalid reference
                RPMObject invalidNewValue = RpmTestFramework.INSTANCE
                    .getStaticFixtureFromPointer(invalidChildType, 0);

                testInvalidUpdate(element, invalidNewValue, errorCode);
            }
        }
        else
        {
            Class invalidChildClass = invalidChildType.getLastClass();
            RpmObjectAbstractTester childFixture = RpmTestFramework.INSTANCE
                .findFixture(invalidChildClass);

            RPMObject invalidNewValue = childFixture.createObject();
            childFixture.fillMandatoryFields(invalidNewValue, true);

            // invalid child ownership
            if (element.isArrayType())
            {
                Object[] invalidUpdateArray = (Object[]) Array.newInstance(
                    invalidChildClass, 1);

                invalidUpdateArray[0] = invalidNewValue;
                testInvalidUpdate(element, invalidUpdateArray, errorCode);
            }
            else
            {
                testInvalidUpdate(element, invalidNewValue, errorCode);
            }
        }
    }

    public void testNumberMinimumValueSuccess(Field element) throws Exception
    {
        // test number min value success
        Number newValidValue = (Number) element.getTypedMinValue();

        testValidUpdate(element, newValidValue);
    }

    public void testNumberMaximumValueSuccess(Field element) throws Exception
    {
        // test number max value success
        Number newValidValue = (Number) element.getTypedMaxValue();

        testValidUpdate(element, newValidValue);
    }

    public void testNumberMinimumValueFailure(Field element) throws Exception
    {
        // test number min value failure
        Number newInvalidValue = getInvalidMinValue(element);

        int errorCode = ErrorCodes.OUT_OF_RANGE; // set error code

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public void testNumberMaximumValueFailure(Field element) throws Exception
    {
        // test number max value failure
        Number newInvalidValue = getInvalidMaxValue(element);

        int errorCode = ErrorCodes.OUT_OF_RANGE; // set error code

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public Number getInvalidMaxValue(Field element)
    {
        Number retVal = null;
        Number maxValue = (Number) element.getTypedMaxValue();
        Number delta = (Number) element.getTypedDelta();
        Class fieldType = element.getJavaTypeNoArray();
        if (fieldType == Integer.TYPE || fieldType == Integer.class)
        {
            retVal = new Integer(maxValue.intValue() + delta.intValue());
        }
        else if (fieldType == Float.TYPE || fieldType == Float.class)
        {
            retVal = new Float(maxValue.floatValue() + delta.floatValue());
        }
        else if (fieldType == Double.TYPE || fieldType == Double.class)
        {
            retVal = new Double(maxValue.doubleValue() + delta.doubleValue());
        }
        else
        {
            Assert.fail("unhandled number type : " + fieldType.getName());
        }
        return retVal;
    }

    public Number getInvalidMinValue(Field element)
    {
        Number retVal = null;
        Number minValue = (Number) element.getTypedMinValue();
        Number delta = (Number) element.getTypedDelta();
        Class fieldType = element.getJavaTypeNoArray();

        if (fieldType == Integer.TYPE || fieldType == Integer.class)
        {
            retVal = new Integer(minValue.intValue() - delta.intValue());
        }
        else if (fieldType == Float.TYPE || fieldType == Float.class)
        {
            retVal = new Float(minValue.floatValue() - delta.floatValue());
        }
        else if (fieldType == Double.TYPE || fieldType == Double.class)
        {
            retVal = new Double(minValue.doubleValue() - delta.doubleValue());
        }
        else
        {
            Assert.fail("unhandled number type : " + fieldType.getName());
        }
        return retVal;
    }

    public void testDateMinimumValueSuccess(Field element) throws Exception
    {
        // test date min value success
        Calendar newValidValue = (Calendar) element.getTypedMinValue();

        testValidUpdate(element, newValidValue);
    }

    public void testDateMaximumValueSuccess(Field element) throws Exception
    {
        // test date max value success
        Calendar newValidValue = (Calendar) element.getTypedMaxValue();

        testValidUpdate(element, newValidValue);
    }

    public void testDateMinimumValueFailure(Field element) throws Exception
    {
        // test date min value failure
        Calendar newInvalidValue = (Calendar) ((Calendar) element
            .getTypedMinValue()).clone();

        Number typedDelta = (Number) element.getTypedDelta();
        long delta = typedDelta.longValue() * 1000;
        newInvalidValue.setTimeInMillis(newInvalidValue.getTimeInMillis()
                - delta);

        int errorCode = ErrorCodes.OUT_OF_RANGE; // set error code

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public void testDateMaximumValueFailure(Field element) throws Exception
    {
        // test date max value failure
        Calendar newInvalidValue = (Calendar) ((Calendar) element
            .getTypedMaxValue()).clone();

        Number typedDelta = (Number) element.getTypedDelta();
        long delta = typedDelta.longValue() * 1000;
        newInvalidValue.setTimeInMillis(newInvalidValue.getTimeInMillis()
                + delta);

        int errorCode = ErrorCodes.OUT_OF_RANGE; // set error code

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public void testStringMinimumLengthSuccess(Field element) throws Exception
    {
        Number typedMin = (Number) element.getTypedMinValue();

        int minLength = 0;

        if (typedMin != null)
        {
            minLength = typedMin.intValue();
        }

        if (minLength == 0)
        {
            // never use 0 as a minimum length since oracle will
            // not return the same value as DB2
            minLength = 1;
        }

        // test string min length success
        String newValidValue = GenericValidation.getTestString(minLength);

        testValidUpdate(element, newValidValue);
    }

    public void testStringMinimumLengthFailure(Field element) throws Exception
    {
        Number typedMin = (Number) element.getTypedMinValue();
        int minSize = typedMin.intValue();
        // test string min length failure
        Assert.assertTrue(minSize > 0);
        String newInvalidValue = GenericValidation.getTestString(minSize - 1);

        int errorCode = ErrorCodes.MIN_LENGTH;

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public void testStringMaximumLengthSuccess(Field element) throws Exception
    {
        Number typedMax = (Number) element.getTypedMaxValue();
        int maxSize = typedMax.intValue();
        // test string max length success
        Assert.assertTrue("can't test max string length with a 0 max length.",
            maxSize > 0);
        String newValidValue = GenericValidation.getTestString(maxSize);

        testValidUpdate(element, newValidValue);
    }

    public void testValidUpdate(Field element, Object newValidValue)
            throws Exception
    {
        RPMObject updateFixture = getUpdateFixture(0);
        RPMObject compareFixture = getCompareFixture(0);

        fillField(updateFixture, element, newValidValue);
        fillField(compareFixture, element, newValidValue);

        saveAndCompare(updateFixture, compareFixture, null, false);
    }

    public void testValidCreate(Field element, Object newValidValue)
            throws Exception
    {
        RPMObject createObject = createObject();
        RPMObject compareFixture = createObject();

        fillMandatoryFields(createObject, true);
        fillMandatoryFields(compareFixture, true);

        fillField(createObject, element, newValidValue);
        fillField(compareFixture, element, newValidValue);

        saveAndCompare(createObject, compareFixture, null, true);
    }

    public void testStringMaximumLengthFailure(Field element) throws Exception
    {
        Number typedMax = (Number) element.getTypedMaxValue();
        int maxSize = typedMax.intValue();

        // test string max length failure
        Assert.assertTrue("can't test max string length with a 0 max length.",
            maxSize > 0);
        String newInvalidValue = GenericValidation.getTestString(maxSize + 1);

        int errorCode = ErrorCodes.MAX_LENGTH;

        testInvalidUpdate(element, newInvalidValue, errorCode);
    }

    public void testInvalidCreate(Field element, Object newInvalidValue,
            int errorCode) throws Exception
    {
        RPMObject createObject = createObject();
        fillMandatoryFields(createObject, true);

        fillField(createObject, element, newInvalidValue);

        RPMObjectScope scope = createScope();
        fillScope(scope, element);

        SingleResult objectSaved = saveMustFail(createObject, scope);

        validateFieldError(element, objectSaved, errorCode, classInformation
            .getShortTypeName());
    }

    public void testInvalidUpdate(Field element, Object newInvalidValue,
            int errorCode) throws Exception
    {
        RPMObject object = getBadUpdateFixture();
        Object oldValue = getFieldValue(object, element, element
            .getGetterMethod());

        RPMObjectScope scope = createScope();
        fillScope(scope, element);

        try
        {
            fillField(object, element, newInvalidValue);

            SingleResult objectSaved = saveMustFail(object, scope);

            if (element.compositeValue()
                    && (newInvalidValue instanceof RPMObject || newInvalidValue instanceof RPMObject[]))
            {
                errorCode = ErrorCodes.WRONG_PARENT_TYPE;

                RPMObject objectWithInvalidParent = null;
                if (newInvalidValue instanceof RPMObject[])
                {
                    RPMObject[] invalidRpmObjectValue = (RPMObject[]) newInvalidValue;
                    objectWithInvalidParent = invalidRpmObjectValue[0];
                }
                else
                {
                    objectWithInvalidParent = (RPMObject) newInvalidValue;
                }

                RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
                    .findFixture(objectWithInvalidParent.getClass());

                // trying to create invalid child type
                validateFieldError(findFixture.classInformation
                    .getParentField(), objectSaved, errorCode,
                    findFixture.classInformation.getShortTypeName());
            }
            else
            {
                validateFieldError(element, objectSaved, errorCode,
                    classInformation.getShortTypeName());
            }
        }
        finally
        {
            fillField(object, element, oldValue);
        }
    }

    public void testStringInvalidChars(Field element, char invalidChar)
            throws Exception
    {
        RPMObject createObject = createObject();
        fillMandatoryFields(createObject, true);

        String setValue = "" + invalidChar;
        fillField(createObject, element, setValue);

        SingleResult objectSaved = saveMustFail(createObject, null);

        int errorCode = ErrorCodes.INVALID_CHARS;

        validateFieldError(element, objectSaved, errorCode, classInformation
            .getShortTypeName());
    }

    public void validateFieldError(Field fieldWithError,
            SingleResult objectSaved, int errorCode, String className)
    {
        RPMException[] exceptions = objectSaved.getErrors();
        Assert.assertNotNull("Did not recieve exceptions", exceptions);
        Assert.assertEquals("Received too many exceptions", 1,
            exceptions.length);

        RPMException error = exceptions[0];

        Assert.assertEquals(fieldWithError.getName().toLowerCase(), error
            .getFieldName().toLowerCase());

        Assert.assertEquals(errorCode, error.getErrorID());

        Assert.assertEquals(className, error.getClassName());
    }

    public void testUpdateField(Field element) throws Exception
    {
        testUpdateFieldWithSeed(element, new Integer(0));
    }

    public void testUpdateFieldWithSeed(Field element, Integer seed)
            throws Exception
    {
        // Design note: we should not use the seed to get a fixture. Doing so
        // will result the creation of a new fixture.
        RPMObject updateFixture = getUpdateFixture(0);
        RPMObject compareFixture = getCompareFixture(0);

        RPMObjectScope fullScope = createFullScope();

        updateField(updateFixture, element, seed.intValue());
        updateField(compareFixture, element, seed.intValue());

        saveAndCompare(updateFixture, compareFixture, fullScope, false);
    }

    public void testUpdateWithInvalidScope(Field element) throws Exception
    {
        RPMObject updateFixture = getUpdateFixture(0);

        RPMObjectScope invalidScope = createEmptyInvalidScope();

        updateField(updateFixture, element, 0);

        try
        {
            testController.saveMustFail(updateFixture, invalidScope);
        }
        catch (RPMException exception)
        {
            Assert.assertEquals(exception.getErrorID(),
                ErrorCodes.INVALID_SCOPE);
        }
    }

    public void testCreationMinimumMandatoryFields() throws Exception
    {
        RPMObject createObject = createObject();
        RPMObject compareObject = createObject();

        fillMandatoryFields(createObject, true);
        fillMandatoryFields(compareObject, true);

        RPMObjectScope scope = createMinimumScope(createObject);

        RPMObject reloadedObject = saveAndCompare(createObject, compareObject,
            scope, true);

        validateDefaultValues(reloadedObject);
    }

    public void testNullUpdateSuccess(Field fieldInfo) throws Exception
    {
        // test reset to null
        testValidUpdate(fieldInfo, null);
    }

    public void testNullUpdateFailure(Field fieldInfo) throws Exception
    {
        // test failed reset to null
        int errorCode = ErrorCodes.CANNOT_CLEAR_FIELD; // set error value

        if (!fieldInfo.createCanBeNullValue())
        {
            // field may never be null, during create or update
            errorCode = ErrorCodes.MANDATORY;
        }

        testInvalidUpdate(fieldInfo, null, errorCode);
    }

    public void testNullCreateSuccess(Field fieldInfo) throws Exception
    {
        // test reset to null
        testValidCreate(fieldInfo, null);
    }

    public void testCreationWithOneScope(Field fieldInfo) throws Exception
    {
        RPMObject createObject = createObject();
        RPMObject compareObject = createObject();

        RPMObjectScope fullScope = createFullScope();

        fillObject(createObject, fullScope, true);

        // create scope with only 1 object in scope
        RPMObjectScope minimumScope = createMinimumScope(createObject);
        createFieldScope(minimumScope, fieldInfo.getScopeField());
        fillObject(compareObject, minimumScope, true);

        // save with only 1 object in scope, but give a full structure
        RPMObject objectSaved = saveObject(createObject, minimumScope)
            .getRpmObject();

        // load the object with a full scope
        RPMObject loadObject = createMinimumLoadFixture(objectSaved);
        RPMObject loadedObject = loadObject(loadObject, fullScope);

        // compare with full scope, only the object in scope should
        // be added
        if (classInformation.creationModeValue() == CreationMode.CREATE_BY_PARENT)
        {
            compareObject.setParent(createObject.getParent());
        }

        assertCompare(compareObject, loadedObject, minimumScope, true);
    }

    public void testCreateWithInvalidScope() throws Exception
    {
        RPMObject createObject = createObject();
        RPMObjectScope invalidScope = createEmptyInvalidScope();

        fillObject(createObject, invalidScope, true);

        try
        {
            testController.saveMustFail(createObject, invalidScope);
        }
        catch (RPMException exception)
        {
            Assert.assertEquals(exception.getErrorID(),
                ErrorCodes.INVALID_SCOPE);
        }
    }

    public RPMObject getFullScopeLoadFixture() throws Exception
    {
        if (fullLoadFixture == null)
        {
            RPMObjectScope fullScope = createFullScope();
            if (classInformation.creationModeValue() == CreationMode.NORMAL)
            {
                RPMObject createObject = createObject();
                fillObject(createObject, fullScope, true);

                // save with only 1 object in scope, but give a full structure
                fullLoadFixture = saveObject(createObject, fullScope)
                    .getRpmObject();
            }
            else
            {
                fullLoadFixture = loadStaticObject(0, fullScope);
            }
        }
        return fullLoadFixture;
    }

    public void testLoadWithOneScope(Field fieldInfo) throws Exception
    {
        RPMObject objectSaved = getFullScopeLoadFixture();

        RPMObjectScope fullScope = createFullScope();
        RPMObject compareObject = null;

        // create scope with only 1 object in scope
        RPMObjectScope minimumScope = createMinimumScope(objectSaved);
        createFieldScope(minimumScope, fieldInfo.getScopeField());

        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            compareObject = createObject();
            fillObject(compareObject, minimumScope, true);
        }
        else
        {
            compareObject = loadStaticObject(0, minimumScope);
        }

        // load the object with a full scope
        RPMObject loadObject = createMinimumLoadFixture(objectSaved);
        RPMObject loadedObject = loadObject(loadObject, minimumScope);

        // compare with full scope, only the object in scope should
        // be added
        if (classInformation.creationModeValue() == CreationMode.CREATE_BY_PARENT)
        {
            compareObject.setParent(loadedObject.getParent());
        }

        // we don't have children for security roles

        // Object currentValueInScope = fieldInfo.getGetterMethod().invoke(
        // loadedObject, null);

        // Assert.assertNotNull(
        // "Loaded value in scope is null. Failed to make a valid scope test on
        // field: "
        // + fieldInfo.getName(), currentValueInScope);

        assertCompare(compareObject, loadedObject, fullScope, true);
    }

    public void testLoadFromIDWithInvalidScope() throws Exception
    {
        RPMObject objectSaved = getFullScopeLoadFixture();
        RPMObjectScope invalidScope = createEmptyInvalidScope();

        try
        {
            // load the object with an invalid scope
            loadObject(objectSaved, invalidScope);
        }
        catch (RPMException e)
        {
            int errorCode = ErrorCodes.INVALID_SCOPE;
            Assert.assertEquals(errorCode, e.getErrorID());
        }
    }

    public void testLoadArrayFromIDWithInvalidScope() throws Exception
    {
        RPMObject[] objects =
        {
                getUpdateFixture(0), getUpdateFixture(1)
        };
        RPMObjectScope invalidScope = createEmptyInvalidScope();

        try
        {
            // load the object with an invalid scope
            loadArray(objects, invalidScope);
        }
        catch (RPMException e)
        {
            int errorCode = ErrorCodes.INVALID_SCOPE;
            Assert.assertEquals(errorCode, e.getErrorID());
        }
    }

    public void testLoadFromXPathWithInvalidScope() throws Exception
    {
        RPMObject objectSaved = getFullScopeLoadFixture();
        RPMObjectScope invalidScope = createEmptyInvalidScope();

        try
        {
            // load using the id
            StringBuffer xpath = new StringBuffer();
            xpath.append(XpathTesterUtils.FORWARD_SLASH).append(
                classInformation.getShortTypeName()).append(
                XpathTesterUtils.OPEN_BRACKET).append("ID").append(
                XpathTesterUtils.EQUALS).append("'")
                .append(objectSaved.getID()).append("'");
            xpath.append(XpathTesterUtils.CLOSE_BRACKET);

            // load the object with an invalid scope
            loadXpathAndFindResult(objectSaved, xpath.toString(), invalidScope,
                classInformation.getJavaClass());
        }
        catch (RPMException e)
        {
            int errorCode = ErrorCodes.INVALID_SCOPE;
            Assert.assertEquals(errorCode, e.getErrorID());
        }
    }

    public void validateDefaultValues(RPMObject reloadedObject)
            throws Exception
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            validateFieldDefaultValue(reloadedObject, fieldInfo);
        }
    }

    public void validateFieldDefaultValue(RPMObject reloadedObject,
            Field fieldInfo) throws IllegalAccessException,
            InvocationTargetException
    {
        Object defaultValue = fieldInfo.getTypedDefaultValue();
        if (fieldInfo.createCanBeNullValue() && defaultValue != null)
        {
            Object currentValue = fieldInfo.getGetterMethod().invoke(
                reloadedObject, null);
            Assert.assertEquals(defaultValue, currentValue);
        }
    }

    public RPMObject createMinimumLoadFixture(RPMObject objectSaved)
            throws Exception
    {
        RPMObject loadObject = createObject();

        copyMinimumIDFields(objectSaved, loadObject);

        return loadObject;
    }

    public void copyMinimumIDFields(RPMObject sourceObject,
            RPMObject destinationObject)
    {
        destinationObject.setContextName(sourceObject.getContextName());
        destinationObject.setID(sourceObject.getID());
    }

    public void planCreationMissingMandatoryFields() throws Throwable
    {
        if (!isMandatoryFieldPresent())
        {
            return;
        }

        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            if (!fieldInfo.createCanBeNullValue())
            {
                addPlannedTest("testCreationMissingMandatoryField", fieldInfo
                    .getName(), fieldInfo);
            }
        }
    }

    public void planLoadingMissingPrimaryKeys() throws Throwable
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            if (fieldInfo.primaryKeyValue())
            {
                addPlannedTest("testLoadMissingPrimaryKey",
                    fieldInfo.getName(), fieldInfo);
            }
        }
    }

    public void testCreationMissingMandatoryField(Field missingFieldInfo)
            throws Exception
    {
        RPMObject createObject = createObject();

        fillMandatoryFieldsAndSkipOne(missingFieldInfo, createObject, true);

        RPMObjectScope scope = createScope();
        fillScope(scope, missingFieldInfo);

        SingleResult saveMustFail = saveMustFail(createObject, scope);

        RPMException[] exceptions = saveMustFail.getErrors();

        String errors = "Too many errors found: ";
        for (int i = 0; i < exceptions.length; i++)
        {
            RPMException exception = exceptions[i];
            errors += "\n  " + exception.getRpmMessage();
        }
        errors += "\n";

        Assert.assertEquals(errors, 1, exceptions.length);
        RPMException missingFieldError = exceptions[0];

        Assert.assertEquals(missingFieldError.getFieldName().toLowerCase(),
            missingFieldInfo.getName());

        int errorCode = ErrorCodes.MANDATORY;

        if (missingFieldInfo.getName().equals("parent"))
        {
            errorCode = ErrorCodes.NO_PARENT;
        }

        Assert.assertEquals(errorCode, missingFieldError.getErrorID());
    }

    public void fillMandatoryFieldsAndSkipOne(Field missingFieldInfo,
            RPMObject createObject, boolean forCreation) throws Exception
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();
            if (!fieldInfo.createCanBeNullValue()
                    && missingFieldInfo != fieldInfo)
            {
                fillField(createObject, fieldInfo, 0, forCreation);
            }
        }
    }

    public boolean isMandatoryFieldPresent()
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field element = (Field) it.next();
            if (!element.createCanBeNullValue())
            {
                return true;
            }
        }
        return false;
    }

    public void planCreationFailsOnInvalidParents() throws Throwable
    {
        Field parentField = classInformation.getParentField();

        if (parentField == null)
        {
            return;
        }

        List parentClassList = parentField.getInvalidTypePointerList();
        Iterator it = parentClassList.iterator();
        while (it.hasNext())
        {
            PointerLink parentClass = (PointerLink) it.next();
            addPlannedTest("testCreateWithInvalidParent", StringUtil
                .getShortClassName(parentClass.getLastClassName()), parentClass);
        }

        if (!parentField.createCanBeNullValue())
        {
            addPlannedTest("testCreateWithInvalidParent", "null parent", null);
        }
    }

    public void planCallsWithOneScope() throws Throwable
    {
        Iterator it = classInformation.getFields().iterator();

        while (it.hasNext())
        {
            Field info = (Field) it.next();
            if (info.getScopeField() != null)
            {
                if (classInformation.creationModeValue() == CreationMode.NORMAL)
                {
                    addPlannedTest("testCreationWithOneScope", info.getName(),
                        info);
                }
                addPlannedTest("testLoadWithOneScope", info.getName(), info);
            }
        }
    }

    public void planCallsWithInvalidScope() throws Throwable
    {
        Scope invalidScope = getInvalidScopeClass();

        if (invalidScope == null)
        {
            // addPlannedTest(
            // "testFailure",
            // "The invalid scope has not been set. Please call
            // classInfo.setInvalidScopeClass() in the method fillClassInfo() in
            // your fixture");
            return;
        }

        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            addPlannedTest("testCreateWithInvalidScope", StringUtil
                .getShortClassName(invalidScope.getClass()));
        }

        addPlannedTest("testUpdateWithInvalidScope", StringUtil
            .getShortClassName(invalidScope.getClass()), classInformation
            .getFields().toArray()[0]);
        addPlannedTest("testLoadFromIDWithInvalidScope", StringUtil
            .getShortClassName(invalidScope.getClass()));
        addPlannedTest("testLoadFromXPathWithInvalidScope", StringUtil
            .getShortClassName(invalidScope.getClass()));
        addPlannedTest("testLoadArrayFromIDWithInvalidScope", StringUtil
            .getShortClassName(invalidScope.getClass()));
    }

    public void planCreationOnAllValidParents() throws Throwable
    {
        Field parentField = classInformation.getParentField();

        if (parentField == null)
        {
            return;
        }

        List parentClassList = parentField.getValidTypePointerList();

        if (parentField.createCanBeNullValue())
        {
            addPlannedTest("testCreateWithValidParent", "null parent", null);
        }

        Iterator it = parentClassList.iterator();
        while (it.hasNext())
        {
            PointerLink parentClass = (PointerLink) it.next();
            addPlannedTest("testCreateWithValidParent", StringUtil
                .getShortClassName(parentClass.getLastClassName()), parentClass);
        }
    }

    public void logTestName(String testName)
    {
        String buff = " ============== ";
        logger.info(buff + testName + buff);
    }

    public void testCreateWithValidParent(PointerLink parentPointer)
            throws Exception
    {
        RPMObject parent = null;

        if (parentPointer != null)
        {
            parent = RpmTestFramework.INSTANCE.getStaticFixtureFromPointer(
                parentPointer, 0);
        }

        RPMObject createObject = createObject();
        RPMObject compareObject = createObject();
        RPMObjectScope scope = null;

        Class parentClass = null;
        if (parent != null)
        {
            parentClass = parent.getClass();
        }
        scope = createScopeWithParent(parentClass);

        fillObject(createObject, scope, true);
        fillObject(compareObject, scope, true);

        createObject.setParent(parent);
        compareObject.setParent(parent);

        RPMObject savedObject = saveAndCompare(createObject, compareObject,
            scope, true);

        if (classInformation.isDeletable())
        {
            validateDelete(savedObject);
        }
    }

    public RPMObjectScope createScopeWithParent(Class parentClass)
            throws InstantiationException, IllegalAccessException,
            InvocationTargetException, NoSuchMethodException, Exception
    {
        RPMObjectScope scope = null;
        RpmObjectAbstractTester parentFixture = null;

        if (parentClass != null)
        {
            parentFixture = RpmTestFramework.INSTANCE.findFixture(parentClass);
        }

        Scope scopeClass = classInformation.getScopeMetadata();
        if (scopeClass != null)
        {
            scope = createScope();

            Field parentField = (Field) scopeClass.getField("parent");

            if (parentField != null && parentFixture != null)
            {
                RPMObjectScope setValue = parentFixture.createEmptyScope();
                Object[] args = new Object[]
                {
                    setValue
                };
                parentField.getSetterMethod().invoke(scope, args);
            }
        }
        return scope;
    }

    public RPMObjectScope createScope() throws Exception
    {
        RPMObjectScope scope = null;
        
        Scope scopeClass = classInformation.getScopeMetadata();
        if (scopeClass != null)
        {
            scope = (RPMObjectScope) scopeClass.getJavaClass().getConstructor(
                null).newInstance(null);
        }
        return scope;
    }

    public void fillScope(RPMObjectScope scopeToFill,
            Field fieldToIncludeInScope) throws Exception
    {
        Field scopeField = fieldToIncludeInScope.getScopeField();
        if (scopeToFill == null || scopeField == null)
        {
            return;
        }

        Object setValue = null;
        Class fieldType = scopeField.getJavaTypeNoArray();
        if (fieldType == Boolean.class || fieldType == Boolean.TYPE)
        {
            setValue = Boolean.TRUE;
        }
        else
        {
            setValue = fieldType.getConstructor(null).newInstance(null);
        }

        scopeField.getSetterMethod().invoke(scopeToFill, new Object[]
        {
            setValue
        });
    }

    public RPMObject saveAndCompare(RPMObject createObject,
            RPMObject compareObject, RPMObjectScope scope, boolean afterCreation)
            throws Exception
    {

        // fillFieldsWithDefaultValues(createObject);

        RPMObject objectSaved = saveObject(createObject, scope).getRpmObject();

        RPMObject loadObject = createMinimumLoadFixture(objectSaved);

        RPMObject loadedObject = loadObject(loadObject, scope);

        fillFieldsWithDefaultValues(compareObject);

        // assertCompare(objectSaved, loadedObject, scope, false);
        // assertCompare(objectSaved, compareObject, scope, afterCreation);
        assertCompare(compareObject, loadedObject, scope, afterCreation);

        return loadedObject;
    }

    public RPMObjectScope createEmptyScope() throws Exception
    {
        Scope scopeClass = classInformation.getScopeMetadata();

        RPMObjectScope newScope = (RPMObjectScope) scopeClass.getJavaClass()
            .getConstructor(null).newInstance(null);

        return newScope;
    }

    public RPMObjectScope createEmptyInvalidScope() throws Exception
    {
        Scope scopeClass = getInvalidScopeClass();

        RPMObjectScope newScope = (RPMObjectScope) scopeClass.getJavaClass()
            .getConstructor(null).newInstance(null);

        return newScope;
    }

    public Scope getInvalidScopeClass()
    {
        // TODO find a invalid scope automatically
        return null;
    }

    public RPMObjectScope createFullScope() throws Exception
    {
        Scope scopeClass = classInformation.getScopeMetadata();
        if (scopeClass == null)
        {
            return null;
        }

        RPMObjectScope newScope = (RPMObjectScope) scopeClass.getJavaClass()
            .getConstructor(null).newInstance(null);

        Iterator it = scopeClass.getFields().iterator();
        while (it.hasNext())
        {
            Field scopeField = (Field) it.next();

            createFieldScope(newScope, scopeField);
        }

        return newScope;
    }

    public void createFieldScope(RPMObjectScope newScope, Field scopeField)
            throws Exception
    {
        Class scopeClass = scopeField.getJavaTypeNoArray();

        // if (scopeField.isTypeAbstract())
        // here when the scopeField is a type of RPMObjectScope, although it
        // is not Abstract, but we do need to reset the scope by going through
        // the same steps like dealing with the abstract scope.
        if (false)
        {
            Field field = classInformation.getField(scopeField.getName());
            Assert.assertNotNull("failed to find field named "
                    + scopeField.getName(), field);
            if (!field.getValidTypePointerList().isEmpty())
            {
                PointerLink child = (PointerLink) field
                    .getValidTypePointerList().get(0);
                Class fieldType = child.getPointerClass();
                RpmObjectAbstractTester fixture = RpmTestFramework.INSTANCE
                    .findFixture(fieldType);
                scopeClass = fixture.getClassInformation().getScopeMetadata()
                    .getJavaClass();
            }
        }

        Method setterMethod = scopeField.getSetterMethod();
        if (scopeField.isBoolean())
        {
            Object[] trueScopeArgument = new Object[]
            {
                Boolean.TRUE
            };
            setterMethod.invoke(newScope, trueScopeArgument);
        }
        else
        {
            Object setValue = scopeClass.getConstructor(null).newInstance(null);
            Object[] setScopeArgument = new Object[]
            {
                setValue
            };
            setterMethod.invoke(newScope, setScopeArgument);
        }
    }

    public RPMObjectScope createMinimumScope(RPMObject object) throws Exception
    {
        RPMObjectScope retVal = null;
        Scope scopeClass = classInformation.getScopeMetadata();
        if (scopeClass != null)
        {
            retVal = (RPMObjectScope) scopeClass.getJavaClass().getConstructor(
                null).newInstance(null);

            if (false) // object.getParent() != null)
            {
                // TODO flush this
                Field scopeParent = scopeClass.getField("parent");

                if (scopeParent != null)
                {
                    RpmObjectAbstractTester parentClassFixture = RpmTestFramework.INSTANCE
                        .findFixture(object.getParent().getClass());

                    Object parentScope = null;
                    if (false) // scopeParent.isTypeAbstract())
                    // the comments are the same as createFieldScope() function
                    {
                        Scope parentScopeClass = parentClassFixture
                            .getClassInformation().getScopeMetadata();

                        if (parentScopeClass != null)
                        {
                            parentScope = parentScopeClass.getJavaClass()
                                .getConstructor(null).newInstance(null);
                        }
                    }
                    else
                    {
                        parentScope = scopeParent.getJavaTypeNoArray()
                            .getConstructor(null).newInstance(null);
                    }

                    if (parentScope != null)
                    {
                        Object[] params = new Object[]
                        {
                            parentScope
                        };

                        scopeParent.getSetterMethod().invoke(retVal, params);
                    }
                }
            }
        }
        return retVal;
    }

    public void testFailure(String failureDescription) throws Exception
    {
        Assert.fail(failureDescription);
    }

    public void testCreateWithInvalidParent(PointerLink parentPointer)
            throws Exception
    {
        RPMObject parent = null;

        if (parentPointer != null)
        {
            parent = RpmTestFramework.INSTANCE.getStaticFixtureFromPointer(
                parentPointer, 0);
        }

        Class parentClass = null;
        if (parent != null)
        {
            parentClass = parent.getClass();
        }

        RPMObject createObject = createObject();
        RPMObjectScope objectScope = createScopeWithParent(parentClass);
        fillObject(createObject, objectScope, true);

        createObject.setParent(parent);

        SingleResult objectSaved = saveMustFail(createObject, objectScope);
        RPMException[] exceptions = objectSaved.getErrors();
        Assert.assertNotNull("Save should generate errors", exceptions);
        Assert.assertEquals("Save should generate only one error",
            exceptions.length, 1);

        RPMException wrongParentError = exceptions[0];

        Assert.assertEquals("parent", wrongParentError.getFieldName());

        int errorCode = ErrorCodes.WRONG_PARENT_TYPE;

        if (parent == null)
        {
            errorCode = ErrorCodes.NO_PARENT;
        }

        Assert.assertEquals(wrongParentError.getErrorID(), errorCode);

        Assert.assertEquals(wrongParentError.getClassName(), classInformation
            .getShortTypeName());
    }

    public void runTestFailOnError(String testName) throws Throwable
    {
        runTest(testName, null, null);
        failAllErrors();
    }

    public void runPlannedTest(int index) throws Throwable
    {
        planTestList();

        PlannedTest plannedTest = (PlannedTest) pendingTestList.get(index);

        runPlannedTest(plannedTest);

        failAllErrors();
    }

    public void runPlannedTest() throws Throwable
    {
        planTestList();

        int index = 0;
        Iterator it = pendingTestList.iterator();
        while (it.hasNext())
        {
            PlannedTest plannedTest = (PlannedTest) it.next();
            if (startTestIndex <= index && stopTestIndex >= index)
            {
                runPlannedTest(plannedTest);
            }
            ++index;
        }
    }

    public void printTestPlan() throws Throwable
    {
        planTestList();

        Iterator it = pendingTestList.iterator();
        while (it.hasNext())
        {
            PlannedTest plannedTest = (PlannedTest) it.next();
            logTestName(plannedTest.getLogString());
        }
    }

    public void runPlannedTest(PlannedTest plannedTest) throws Throwable
    {
        runTest(plannedTest.testName, plannedTest.getLogString(),
            plannedTest.args);
    }

    public void runTest(String testName, String logString, Object[] args)
            throws Throwable
    {
        logTestName(logString);

        try
        {
            Method method = findMethod(testName);
            method.invoke(this, args);
        }
        catch (InvocationTargetException e)
        {
            Throwable source = e.getCause();
            if (source == null)
            {
                source = e;
            }
            stackError(logString, source);
        }
        catch (Throwable x)
        {
            stackError(logString, x);
        }
    }

    public Method findMethod(String testName)
    {
        Method[] methods = getClass().getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            Method method = methods[i];
            if (method.getName().equals(testName))
            {
                return method;
            }
        }
        Assert.fail("unable to find test named " + testName);
        return null;
    }

    public void stackError(String logString, Throwable source) throws Throwable
    {
        errorList.add(source);
        errorMessage.add("fail test " + logString + " : "
                + source.getClass().getName() + ", " + source.getMessage());

        if (failOnFirstError)
        {
            throw source;
        }
    }

    public final void planTestList() throws Throwable
    {
        if (isPlanned)
        {
            return;
        }

        internalPlanTestList();

        isPlanned = true;
    }

    protected void internalPlanTestList() throws Throwable, Exception
    {
        if (classInformation.creationModeValue() == CreationMode.NORMAL)
        {
            planCreationOnAllValidParents();
            planCreationFailsOnInvalidParents();
            planCreationMissingMandatoryFields();
        }

        if (classInformation.isDeletable())
        {
            addPlannedTest("testDeleteFlag", null);
        }
        else
        {
            addPlannedTest("testDeleteFlagFailure", null);
            addPlannedTest("testDeleteFailure", null);
        }

        planValidation();

        if (isMandatoryFieldPresent())
        {
            addPlannedTest("testCreationMinimumMandatoryFields", null);
        }

        planCallsWithOneScope();
        planCallsWithInvalidScope();
        planLoadingMissingPrimaryKeys();
        planXPathTests();
        planFixtureErrorsFailedTests();
    }

    public void planFixtureErrorsFailedTests()
    {
        Iterator it = fixtureErrors.iterator();
        while (it.hasNext())
        {
            String error = (String) it.next();
            addPlannedTest("testFailure", error);
        }
    }

    public PlannedTest addPlannedTest(String testName, String comment,
            Object arg1)
    {
        PlannedTest add = addPlannedTest(testName, comment);
        add.args = new Object[]
        {
            arg1
        };
        return add;
    }

    public PlannedTest addPlannedTest(String testName, String comment)
    {
        PlannedTest add = new PlannedTest();
        add.testName = testName;
        add.testComment = comment;
        add.testIndex = pendingTestList.size();
        add.testClass = getObjectClass();
        pendingTestList.add(add);
        return add;
    }

    public PlannedTest addPlannedTest(String testName, String comment,
            Object arg1, Object arg2)
    {
        PlannedTest add = addPlannedTest(testName, comment);
        add.args = new Object[]
        {
                arg1, arg2
        };
        return add;
    }

    public static PointerLink getPointer(Class pointClass)
    {
        return RpmTestFramework.INSTANCE.getStaticPointer(pointClass);
    }

    public void clearErrorList()
    {
        errorList.clear();
        errorMessage.clear();
    }

    public ArrayList getPendingTestList()
    {
        return pendingTestList;
    }
}