/*
 * Created on Jul 17, 2006
 */
package com.ibm.rpm.client.junit.framework.testcases;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;

/**
 * @author gosselin
 */
public class TestManualCheckout extends ClientTestController
{
    public void setUp() throws Exception 
    {
        super.setUp();
    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testPositiveAlreadyCheckedOutResult() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkOut(pool, false);
        AlreadyCheckedOutResult acr = getApplicationInterface().isCheckedOut(
            getSessionID(), pool);
        assertNotNull(acr.getCheckedOutBy());
        assertNotNull(acr.getCheckedOutSince());
        assertTrue(acr.isCheckedOut());
        assertTrue(acr.isSuccessful());
    }
    
    public void testNegativeAlreadyCheckedOutResult() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        AlreadyCheckedOutResult acr = getApplicationInterface().isCheckedOut(
            getSessionID(), pool);
        assertNull(acr.getCheckedOutBy());
        assertNull(acr.getCheckedOutSince());
        assertFalse(acr.isCheckedOut());
        assertTrue(acr.isSuccessful());
    }    
    
    public void testTimelessAndSessionCheckOut() throws Exception
    {
        disableAutomaticCheckout();
        Resource resource = createResource();
        Pool pool = (Pool)resource.getParent();
        checkOut(pool, true);
        assertTrue(isCheckedOut(pool));
        checkOut(resource, false);
        assertTrue(isCheckedOut(resource));
        getAuthenticateInterface().logout(getSessionID());
        
        performLogin();        
        disableAutomaticCheckout();
        assertTrue(isCheckedOut(pool));
        assertFalse(isCheckedOut(resource));
        checkIn(pool);
    }
    
    public void testTimelessReLoginWithAutomatic_defect56699() throws Exception
    {
        disableAutomaticCheckout();
        Resource resource = createResource();
        Pool pool = (Pool)resource.getParent();
        checkOut(pool, true);
        assertTrue(isCheckedOut(pool));
        checkOut(resource, true);
        assertTrue(isCheckedOut(resource));
        getAuthenticateInterface().logout(getSessionID());
        
        performLogin();        
        assertTrue(isCheckedOut(pool));
        assertTrue(isCheckedOut(resource));
        pool.setAddress("new address");
        save(pool, null);
    }
    
    public void testForceCheckinOnRPMObjectCheckOut() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        String tmpSessionID = sessionid;        
        checkOut(pool);
        
        performLogin();        
        disableAutomaticCheckout();
        forceCheckinOnRPMObjectCheckOut(pool);
        
        checkOut(pool, false);
        pool.setAddress("1948 Sesame Street");
        save(pool, new PoolScope());
        checkIn(pool);
        getAuthenticateInterface().logout(tmpSessionID);
    }
    
    public void testForceCheckinOnUsersCheckOut() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool1 = createPool();
        Pool pool2 = createPool();
        String tmpSessionID = sessionid;

        performLogin(); //get a new session ID
        disableAutomaticCheckout();
        Resource user = (Resource) loadFromXpath(
            "/Resource[id = 'PMO_SUPERVISOR']", null)[0];
        forceCheckinOnUsersCheckOut(user);

        checkOut(pool1, false);
        pool1.setAddress("1948 Sesame Street");
        save(pool1, new PoolScope());
        checkIn(pool1);
        
        checkOut(pool2, false);
        pool2.setAddress("1948 Sesame Street");
        save(pool2, new PoolScope());
        checkIn(pool2);
        
        getAuthenticateInterface().logout(tmpSessionID);
    }
    
    public void testForceCheckinOnSessionsCheckOut() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool1 = createPool();
        Pool pool2 = createPool();
        String tmpSessionID = sessionid;

        performLogin(); // get a new session ID
        disableAutomaticCheckout();
        forceCheckinOnSessionsCheckOut(tmpSessionID);
        

        checkOut(pool1, false);
        pool1.setAddress("1948 Sesame Street");
        save(pool1, new PoolScope());
        checkIn(pool1);
        
        checkOut(pool2, false);
        pool2.setAddress("1948 Sesame Street");
        save(pool2, new PoolScope());
        checkIn(pool2);
        
        getAuthenticateInterface().logout(tmpSessionID);
    }
    
    public void testHierarchicalAlreadyCheckedOut() throws Exception
    {
        // create Pool
        Pool pool = createPool();
        Resource resource = ResourceFixture.createFixture(pool, null, this);
        pool.setResources(new Resource[]
        {
            resource
        });
        PoolScope scope = new PoolScope();
        scope.setResources(new ResourceScope());
        // Assign Resource
        pool = (Pool) save(pool, scope, ReloadType.SavedResult).getRpmObject();
        disableAutomaticCheckout();
        // Checkout the Resource
        checkOut(pool.getResources()[0]);

        String tmpSessionID = sessionid;

        performLogin(); // get a new session ID
        pool.setAddress("blabla");
        boolean failed = false;
        try
        {
            // Save with the already checked out resource in scope
            save(pool, scope);
        }
        catch (Exception e)
        {
            failed = true;
        }

        if (!failed)
        {
            Assert
                .fail("Save should have fail since the object is checked out with the previous session");
        }
        
        //Pool is unlocked by logging out the previous session
        getAuthenticateInterface().logout(tmpSessionID);
    }
    
    public void testUpdateWithNewAutomaticSession() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkOut(pool);
        
        String tmpSessionID = sessionid;        
        
        performLogin();        
        pool.setAddress("1948 Sesame Street");
        boolean failed = false;
        try
        {
            //Pool is locked by the other session
            save(pool, new PoolScope());
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            failed = true;
        }
        
        if (!failed)
        {
            Assert
                .fail("Save should have fail since the object is not checked out");
        }
        
        //Pool is unlocked by logging out the previous session
        getAuthenticateInterface().logout(tmpSessionID);
        
        save(pool, new PoolScope());
    }
    
    public void testUpdateWithNewManualSession() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        
        String tmpSessionID = sessionid;        
        
        performLogin(); 
        disableAutomaticCheckout();
        pool.setAddress("1948 Sesame Street");
        boolean failed = false;
        try
        {
            //Pool is locked by the other session
            save(pool, new PoolScope());
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            failed = true;
        }
        
        if (!failed)
        {
            Assert
                .fail("Save should have fail since the object is not checked out");
        }
        
        //Pool is unlocked by logging out the previous session
        getAuthenticateInterface().logout(tmpSessionID);
        
        try
        {
            //Oups ! in manual checkout mode the Pool needs to be checked out
            save(pool, new PoolScope());
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            failed = true;
        }
        
        if (!failed)
        {
            Assert
                .fail("Save should have fail since the object is not checked out");
        }
        
        checkOut(pool);
        save(pool, new PoolScope());
    }
    
    public void testUpdateWithNewSession() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkOut(pool);
        String tmpSessionID = sessionid;        
        
        performLogin();        
        pool.setAddress("1948 Sesame Street");
        boolean failed = false;
        try
        {
            //Pool is locked by the other session
            save(pool, new PoolScope());
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            failed = true;
        }
        
        if (!failed)
        {
            Assert
                .fail("Save should have fail since the object is not checked out");
        }
        getAuthenticateInterface().logout(tmpSessionID);
    }
   
    public void testDeleteWithAlreadyCheckedOut() throws AssertionError, Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkOut(pool);
        try
        {
            delete(pool);
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            return;
        }
        Assert
            .fail("Delete should have fail since the object is already checked out by some other user");
        checkIn(pool);//to allow deletion
    }
    
    public void testUpdateWithoutCheckOut() throws AssertionError, Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkIn(pool);
        pool.setAddress("1948 Sesame Street");        
        
        try
        {
            save(pool, new PoolScope());
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            return;
        }
        Assert
            .fail("Save should have fail since the object is not checked out");
    }
    
    public void testUpdateAutomaticCheckout() throws AssertionError, Exception
    {
        Pool pool = createPool();
        try
        {
            checkIn(pool);
        }
        catch (Exception e)
        {
            //success: this scenario must fail
            return;
        }
        Assert
            .fail("Checkin should have fail since the automatic checkout mode is enable");
    }
    
    public void testSuccessfulManualCheckout() throws AssertionError, Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkIn(pool);
        checkOut(pool);
        pool.setAddress("1948 Sesame Street");
        save(pool, new PoolScope());
        checkIn(pool);
    }
    
    public void testTimelessCheckout() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkIn(pool);
        checkOut(pool,true);
        pool.setAddress("1948 Sesame Street");
        save(pool, new PoolScope());
        getAuthenticateInterface().logout(getSessionID());
        
        performLogin();
        disableAutomaticCheckout();
        AlreadyCheckedOutResult result = getApplicationInterface()
            .isCheckedOut(getSessionID(), pool);
        
        assertTrue(result.isCheckedOut());
        checkIn(pool);
    }
    
    public void testNotTimelessCheckout() throws Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkIn(pool);
        checkOut(pool,false);
        pool.setAddress("1948 Sesame Street");
        save(pool, new PoolScope());
        getAuthenticateInterface().logout(getSessionID());
        
        performLogin();
        disableAutomaticCheckout();
        AlreadyCheckedOutResult result = getApplicationInterface()
            .isCheckedOut(getSessionID(), pool);
        
        assertFalse(result.isCheckedOut());
    }
    
    public void testSuccessfulAutomaticCheckout() throws Exception
    {
        Pool pool = createPool();
        pool.setAddress("1948 Sesame Street");
        save(pool, new PoolScope());
    }       
    
    public void testAlreadyCheckedOutByThisSession() throws AssertionError,
            Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkOut(pool, false);
        pool.setAddress("1948 Sesame Street");
        pool = (Pool)save(pool, new PoolScope()).getRpmObject();
        
        //already checked out in session db
        pool.setContactName("Big bird");
        save(pool, new PoolScope());
        
        checkIn(pool);
    }
    
    public void testManualMultipleCheckedOut() throws AssertionError, Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
        checkIn(pool);
        checkOut(pool, false);
        checkOut(pool, false);
        checkIn(pool);//to allow deletion
    }
    
    public void testManualCheckoutAfterSave() throws AssertionError, Exception
    {
        disableAutomaticCheckout();
        Pool pool = createPool();
//      ok, since the checkout occurs within the same session as the createPool
        checkOut(pool, false); 
        checkIn(pool);//to allow deletion
    }
    
    public void testTaskStaticFixtureCheckout() throws Exception
    {
        disableAutomaticCheckout();        
        Task element = (Task)WorkElementFixture.retrieveStaticFixture(new Task(), this);
        checkOut(element);
        Assert.assertTrue(isCheckedOut(element));
        checkIn(element);
        Assert.assertFalse(isCheckedOut(element));               
    }
    
    public void testProposalStaticFixtureCheckout() throws Exception
    {
        disableAutomaticCheckout();
        Proposal element = (Proposal) WorkElementFixture.retrieveStaticFixture(
            new Proposal(), this);
        checkOut(element);
        Assert.assertTrue(isCheckedOut(element));
        checkIn(element);
        Assert.assertFalse(isCheckedOut(element));
    }
    
    public void testProjectStaticFixtureCheckout() throws Exception
    {
        disableAutomaticCheckout();
        Project element = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);
        checkOut(element);
        Assert.assertTrue(isCheckedOut(element));
        checkIn(element);
        Assert.assertFalse(isCheckedOut(element));
    }
    
    public Pool createPool() throws Exception
    {
        Pool pool = PoolFixture.createFixture(false, 0, null, this);
        SingleResult result;
        result = save(pool, new PoolScope());

        return (Pool) result.getRpmObject();
    }
    
    public Resource createResource() throws Exception
    {        
        Resource mainResource = ResourceFixture.createSaveFixture(this);
        return mainResource;
    }
}
