/*
 * Created on 23-Nov-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.util;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;

import junit.framework.Assert;

import com.ibm.rpm.framework.RPMContainer;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.AbstractModule;

/**
 * @author dominiqu
 */
public class PackageUtil
{
    public static ArrayList getClasses(Package thePackage)
    {
        return getClasses(thePackage, Object.class);
    }

    public static ArrayList getClasses(Package thePackage, Class baseClass)
    {
        ArrayList classesInPackage = new ArrayList();

        String thePackageName = thePackage.getName();

        File directory = null;
        try
        {
            ClassLoader cld = Thread.currentThread().getContextClassLoader();
            if (cld == null)
            {
                Assert.fail("Can't get class loader.");
            }

            String path = thePackageName.replace('.', '/');
            URL resource = cld.getResource(path);
            if (resource == null)
            {
                Assert.fail("No resource for " + path);
            }
                 
            String filename = resource.getFile().replaceAll("%20", " ");
            directory = new File(filename);

            if (directory.exists())
            {
                // Get the list of the files contained in the package
                String[] files = directory.list();
                for (int i = 0; i < files.length; i++)
                {
                    // Get class files from the directory
                    String file = files[i];
                    if (file.endsWith(".class"))
                    {
                        // Removes the .class extension
                        String shortClassName = file.substring(0, files[i]
                                .length() - 6);
                        String fullClassName = thePackageName + '.'
                        + shortClassName;
                        Class theClass = Class.forName(fullClassName);
                        if (baseClass.isAssignableFrom(theClass))
                        {
                            classesInPackage.add(theClass);
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            if (e.getMessage() != null)
                Assert.fail(e.getMessage());
        }

        return classesInPackage;
    }

    public static ArrayList getAllRPMObjectClasses()
    {
        ArrayList rpmObjectClasses = new ArrayList();

        Package[] allPackages = Package.getPackages();

        for (int i = 0; i < allPackages.length; ++i)
        {
            if (allPackages[i].getName().indexOf("containers") != -1)
                rpmObjectClasses.addAll(getClasses(allPackages[i],
                        RPMObject.class));
        }

        // Remove exceptions.
        Iterator j = rpmObjectClasses.iterator();
        while (j.hasNext())
        {
            Class c = (Class) j.next();
            if (c.isAssignableFrom(RPMContainer.class))
            {
                j.remove();
            }
            else if (c.isAssignableFrom(AbstractModule.class))
            {
                j.remove();
            }
        }

        return rpmObjectClasses;
    }
}
