/*
 * Created on 24-Nov-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.validation;

import java.util.ArrayList;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.framework.testcases.util.PackageUtil;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestPrimaryKeyValidation extends GenericValidation
{
    protected void setUp() throws Exception
    {
        super.setUp();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testAllRPMObjects() throws Exception
    {
        ArrayList rpmObjects = PackageUtil.getAllRPMObjectClasses();

        StringBuffer failedContainers = new StringBuffer();

        for (int i = 0; i < rpmObjects.size(); ++i)
        {
            try
            {
                RPMObject testMissingAllKeys = (RPMObject) ((Class) rpmObjects.get(i))
                    .newInstance();

                RPMExceptionMock expectedExceptions = createRPMException(
                    ErrorCodes.MISSING_PRIMARY_KEY, testMissingAllKeys.getClass()
                        .getName());

                failedContainers.append(verifyLoadFromIDMustFail(testMissingAllKeys,
                    null, expectedExceptions, false));
            }
            catch (InstantiationException e)
            {
                // The class is abstract, cannot instantiate
            }
        }

        if (failedContainers.length() != 0)
        {
            Assert.fail("The following container(s) are not validated correctly:\n"
                + failedContainers.toString());
        }
    }

    public void testAttributeAssignment() throws Exception
    {
        ClientCostCenterScope mainScope = new ClientCostCenterScope();
        mainScope.setAttributeAssignments(true);
        ClientCostCenter mainCCC = ClientCostCenterFixture.createFixture(mainScope, this);
        assertNotNull(mainCCC.getAttributeAssignments());
        assertNotNull(mainCCC.getAttributeAssignments()[0]);

        AttributeAssignment mainMissing1Key = new AttributeAssignment();
        mainMissing1Key.setID(null);

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.MISSING_PRIMARY_KEY, mainMissing1Key.getClass().getName());

        verifyLoadFromIDMustFail(mainMissing1Key, null, expectedExceptions);
    }

    public void testContactGroupAssignment() throws Exception
    {
        ResourceScope mainScope = new ResourceScope();
        mainScope.setContactGroupAssignments(true);
        Pool mainPool = PoolFixture.createSaveFixture(this);
        Resource mainResource = ResourceFixture.createFixture(mainPool, mainScope, this);
        assertNotNull(mainResource.getContactGroupAssignments());
        assertNotNull(mainResource.getContactGroupAssignments()[0]);

        ContactGroupAssignment mainMissing1Key = mainResource
            .getContactGroupAssignments()[0];
        mainMissing1Key.setID(null);

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.MISSING_PRIMARY_KEY, mainMissing1Key.getClass().getName());

        verifyLoadFromIDMustFail(mainMissing1Key, null, expectedExceptions);
    }

    public void testGeographicalAssignment() throws Exception
    {
        ResourceScope mainScope = new ResourceScope();
        mainScope.setGeographicalAssignment(true);
        Pool mainPool = PoolFixture.createSaveFixture(this);
        Resource mainResource = ResourceFixture.createFixture(mainPool, mainScope, this);
        assertNotNull(mainResource.getGeographicalAssignment());

        GeographicalAssignment mainMissing1Key = mainResource.getGeographicalAssignment();
        mainMissing1Key.setID(null);

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.MISSING_PRIMARY_KEY, mainMissing1Key.getClass().getName());

        verifyLoadFromIDMustFail(mainMissing1Key, null, expectedExceptions);
    }

    public void testOrganizationalAssignment() throws Exception
    {
        ResourceScope mainScope = new ResourceScope();
        mainScope.setOrganizationalAssignment(true);
        Pool mainPool = PoolFixture.createSaveFixture(this);
        Resource mainResource = ResourceFixture.createFixture(mainPool, mainScope, this);
        assertNotNull(mainResource.getOrganizationalAssignment());

        OrganizationalAssignment mainMissing1Key = mainResource
            .getOrganizationalAssignment();
        mainMissing1Key.setID(null);

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.MISSING_PRIMARY_KEY, mainMissing1Key.getClass().getName());

        verifyLoadFromIDMustFail(mainMissing1Key, null, expectedExceptions);
    }

    public void testRtfAssignment() throws Exception
    {
        ClientCostCenterScope mainScope = new ClientCostCenterScope();
        mainScope.setRtfAssignments(true);
        ClientCostCenter mainCCC = ClientCostCenterFixture.createFixture(mainScope, this);
        assertNotNull(mainCCC.getRtfAssignments());
        assertNotNull(mainCCC.getRtfAssignments()[0]);

        RtfAssignment mainMissing1Key = mainCCC.getRtfAssignments()[0];
        mainMissing1Key.setID(null);

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.MISSING_PRIMARY_KEY, mainMissing1Key.getClass().getName());

        verifyLoadFromIDMustFail(mainMissing1Key, null, expectedExceptions);
    }
}
