/*
 * Created on 21-Nov-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolModuleFixture;
import com.ibm.rpm.clientcostcenters.scope.ClientModuleScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolModuleScope;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This class only tests that an exception indeed gets thrown when attempting to
 * save an RPMObject with an invalid scope. It does not test all that all scopes
 * are correctly included in the ContainerScopeMap.
 * 
 * @author dominiqu
 */
public class TestScopeValidation extends GenericValidation
{
    protected void setUp() throws Exception
    {
        super.setUp();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessSaveWithNoScope() throws AssertionError, Exception
    {
        Pool mainPool = PoolFixture.createFixture(false, 0, null, this);

        // Attempt to save the Pool
        mainPool = (Pool) saveMustSucceed(mainPool, Pool.class, null);

        // Load the inserted Pool from id
        RPMObject obj = new Pool();
        obj.setID(mainPool.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);

        // Check that the loaded Pool is identical to the one we
        // requested to save
        PoolFixture.compare(mainPool, ((Pool) rpmObject), null);
    }
    
    public void testSuccessSaveWithCorrectScope() throws AssertionError, Exception
    {
        Pool mainPool = PoolFixture.createFixture(false, 0, null, this);
        PoolScope correctScope = new PoolScope();

        // Attempt to save the Pool
        mainPool = (Pool) saveMustSucceed(mainPool, Pool.class, correctScope);

        // Load the inserted Pool from id
        RPMObject obj = new Pool();
        obj.setID(mainPool.getID());
        RPMObject rpmObject = loadFromID(obj, correctScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);

        // Check that the loaded Pool is identical to the one we
        // requested to save
        PoolFixture.compare(mainPool, ((Pool) rpmObject), correctScope);
    }

    public void testFailureSaveWithWrongScope() throws AssertionError, Exception
    {
        Pool mainPool = new Pool();
        ResourceScope wrongScope = new ResourceScope();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_SCOPE, mainPool.getClass().getName())
        };

        verifySaveMustFail(mainPool, wrongScope, expectedExceptions);
    }
    
    
    public void testFailureSaveWithScope() throws AssertionError, Exception
    {
        // The scope of an Education object should be null
        Education mainEducation = new Education();
        ResourceScope wrongScope = new ResourceScope();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_SCOPE, mainEducation.getClass().getName())
        };

        verifySaveMustFail(mainEducation, wrongScope, expectedExceptions);
    }

    public void testSuccessLoadFromIDModuleWithCorrectScope() throws AssertionError,
            Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);
        PoolScope correctScope = new PoolScope();

        // Load the inserted Pool from id
        RPMObject obj = new Pool();
        obj.setID(mainPool.getID());
        RPMObject rpmObject = loadFromID(obj, correctScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);

        // Check that the loaded PoolModule is identical to the one we
        // requested to save
        PoolFixture.compare(mainPool, ((Pool) rpmObject), correctScope);
    }

    public void testFailureLoadFromIDWithWrongScope() throws AssertionError, Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);

        ResourceScope wrongScope = new ResourceScope();

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.INVALID_SCOPE, mainPool.getClass().getName());

        verifyLoadFromIDMustFail(mainPool, wrongScope, expectedExceptions);
    }

    public void testSuccessModuleLoadFromIDModuleWithCorrectScope()
            throws AssertionError, Exception
    {
        PoolModuleScope correctScope = new PoolModuleScope();
        PoolModule mainPoolModule = PoolModuleFixture.createFixture(correctScope, this);

        // Attempt to save the PoolModule
        mainPoolModule = (PoolModule) saveMustSucceed(mainPoolModule, PoolModule.class,
            correctScope);

        // Load the inserted PoolModule from id
        RPMObject obj = new PoolModule();
        obj.setID(mainPoolModule.getID());
        RPMObject rpmObject = loadFromID(obj, correctScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof PoolModule);

        // Check that the loaded PoolModule is identical to the one we
        // requested to save
        PoolModuleFixture.compare(mainPoolModule, ((PoolModule) rpmObject), correctScope);
    }

    public void testFailureModuleLoadFromIDWithWrongScope() throws AssertionError,
            Exception
    {
        PoolModuleScope correctScope = PoolModuleFixture.createFullScope();
        PoolModule mainPoolModule = PoolModuleFixture.createFixture(correctScope, this);

        // Attempt to save the PoolModule
        mainPoolModule = (PoolModule) saveMustSucceed(mainPoolModule, PoolModule.class,
            correctScope);

        ClientModuleScope wrongScope = new ClientModuleScope();

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.INVALID_SCOPE, mainPoolModule.getClass().getName());

        verifyLoadFromIDMustFail(mainPoolModule, wrongScope, expectedExceptions);
    }

    public void testSuccessLoadArrayFromIDModuleWithCorrectScope() throws AssertionError,
            Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);
        PoolScope correctScope = new PoolScope();

        // Load the inserted Pool from id
        RPMObject[] obj = new Pool[1];
        obj[0] = new Pool();
        obj[0].setID(mainPool.getID());
        RPMObject[] rpmObject = loadArrayFromID(obj, correctScope);
        assertNotNull(rpmObject);
        assertNotNull(rpmObject[0]);
        assertTrue(rpmObject[0] instanceof Pool);

        // Check that the loaded PoolModule is identical to the one we
        // requested to save
        PoolFixture.compare(mainPool, ((Pool) rpmObject[0]), correctScope);
    }

    public void testFailureLoadArrayFromIDWithWrongScope() throws AssertionError,
            Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);

        ResourceScope wrongScope = new ResourceScope();

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.INVALID_SCOPE, mainPool.getClass().getName());

        verifyLoadArrayFromIDMustFail(new RPMObject[]
        {
            mainPool
        }, wrongScope, expectedExceptions);
    }

    public void testSuccessLoadFromXpathModuleWithCorrectScope() throws AssertionError,
            Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);
        PoolScope correctScope = new PoolScope();

        // Load the inserted pool from xpath
        RPMObject[] rpmObject = loadFromXpath("/Pool[Id='" + mainPool.getID() + "']",
            null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject[0] instanceof Pool);

        // Check that the loaded PoolModule is identical to the one we
        // requested to save
        PoolFixture.compare(mainPool, ((Pool) rpmObject[0]), correctScope);
    }

    public void testFailureLoadFromXpathWithWrongScope() throws AssertionError, Exception
    {
        Pool mainPool = PoolFixture.createSaveFixture(this);

        ResourceScope wrongScope = new ResourceScope();

        RPMExceptionMock expectedExceptions = createRPMException(
            ErrorCodes.INVALID_SCOPE, StringUtil.getShortClassName(mainPool.getClass()));

        verifyLoadFromXpathMustFail("/Pool[Id='" + mainPool.getID() + "']", wrongScope,
            expectedExceptions);
    }
}
