/*
 * Created on 25-Oct-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.Vector;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.ValueDatafield;
import com.ibm.rpm.client.junit.framework.testcases.xpath.stats.StatsManager;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;

/**
 * @author njarazi
 */
public abstract class DatafieldXPathTester extends XpathTester
{
    public DatafieldXPathTester()
    {
        skipList = generateSkipList();
        skipList.add("getAppendOnly");
        skipList.add("getClosedState");
        skipList.add("getProposal");
    }

    public void setUp() throws Exception
    {
        super.setUp();
        initWithFullScope();
    }
    
    protected RPMObject createObject(RPMObject object)
    {
        RPMObject temp = null;
        try
        {
            String xpath = generateDatafieldTypeQuery(object);

            RPMObject loadedObjects[] = loadFromXpath(xpath, null);
            
            if (loadedObjects != null)
            {
                for (int i = 0; i < loadedObjects.length; i++)
                {
                    if (((ValueDatafield) loadedObjects[i]).getValue() != null)
                    {
                        //assign a value to the reference object
                        temp = loadedObjects[i];
                        break;
                    }
                }
                if(temp == null){
                    temp = loadedObjects[0];
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        
        if (temp == null)
        {
            errorHandler.logOtherError("The requested object was not found in the database", "");
        }
        
        return temp;
    }
    
    public void testVerifyDatafieldWithValueField()
    {
        String xpath;

        try
        {
            xpath = generateDatafieldTypeQuery(myObject);
            RPMObject loadedObjects[] = loadFromXpath(xpath, null);

            if (loadedObjects != null)
            {
                for (int i = 0; i < loadedObjects.length; i++)
                {
                    if (((ValueDatafield) loadedObjects[i]).getValue() != null)
                    {
                        //assign a value to the reference object
                        myObject = loadedObjects[i];
                        doDatafieldComparison(myObject, loadedObjects[i]);
                        break;
                    }
                }
            }
        }
        catch (NullPointerException n)
        {
            errorHandler.logOtherError("The object returned by the query is null", n.getMessage());
            n.printStackTrace();
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured while testing for the value field.", e.getMessage());
            e.printStackTrace();
        }
        
        errorHandler.handleError();
    }
    
    public void verifyContainerWithXPathQueries(RPMObject datafield)
            throws Exception
    {
        //Query which asks for all object of this type and makes sure the return is all of the same type
        String xpath = generateDatafieldTypeQuery(datafield);
        if (xpath.length() > 0)
        {
            loadDatafieldFromXPath(datafield.getClass(), xpath, datafield
                    .getID());
        }

        try
        {
            String[] queries = generateSingleDatafieldQuery(myObject);
            for (int j = 0; j < queries.length; j++)
            {
                StatsManager.incrementSimplePropertiesCount();
                internalLoadDatafieldFromXPathAndCompare(myObject, queries[j],
                        null, myObject.getID());
            }
        }
        catch (ClassCastException e)
        {
            //This is here for the moment as "types" are not supported by the xpath framework
            StatsManager.appendFieldError(
                    "An error occured during the generation of the XPath query. " +
                    	"The object of type " + datafield.getClass().getName() + 
                    	" was detected and could not be cast into an RPMObject" +
                    	" For the moment, there is not mechanism to test 'enumerated types' with" +
                    	" the XPath framework.  Eventually, means of testing will be implemented"
                    );
            StatsManager.appendFieldErrorMessage(e.getMessage());
        }
        catch (Exception e)
        {
            errorHandler.logXPathError(0, xpath, e, false, simpleType);
        }

        errorHandler.handleError();
    }

    /**
     * @param dataField
     */
    private String generateDatafieldTypeQuery(RPMObject datafield)
            throws Exception
    {
        StringBuffer xpath = new StringBuffer();
        
        //The query generated here is of this type :
        //  "/Datafield"
        //  ex: "/CommonScopeRTF"        
        xpath.append(XpathTesterUtils.FORWARD_SLASH).append(XpathTesterUtils.getObjectName(datafield));

        return xpath.toString();
    }

    private String[] generateSingleDatafieldQuery(RPMObject datafield)
            throws Exception
    {
        StringBuffer xpath = new StringBuffer();

        Method[] methods = ((Datafield) datafield).getClass().getMethods();

        Vector v = new Vector();

        for (int i = 0; i < methods.length; i++)
        {
            Method tempMethod = methods[i];
            if (tempMethod.getName().startsWith(XpathTesterUtils.GET_PREFIX)
                    && !tempMethod.getName().endsWith(XpathTesterUtils.MODIFIED_TRAILER)
                    && !isSkipped(tempMethod, skipList))
            {
                //The query generated here is of this type :
                //  "/Object[Field='field value']"
                //  ex: "/CommonScopeRTF[Name='1']"
                xpath.append(XpathTesterUtils.FORWARD_SLASH).append(
                        XpathTesterUtils.getObjectName(datafield)).append(
                        XpathTesterUtils.OPEN_BRACKET).append(
                        tempMethod.getName().substring(3));
                
                

                Object o = tempMethod.invoke(datafield, null);
                if (o != null)
                {
                    xpath.append(
                            XpathTesterUtils.EQUALS);
                    
                    
                    String returnedId = o.toString();
                    
                    xpath.append(XpathTesterUtils.SINGLE_QUOTE);
                    
                    xpath.append(returnedId);
                        
                    xpath.append(XpathTesterUtils.SINGLE_QUOTE);
                }
                

                xpath.append(XpathTesterUtils.CLOSE_BRACKET);

                if  (tempMethod.getName().equalsIgnoreCase("getValue") && o != null)
                {
                    v.add(xpath.toString());
                }
                else if (o != null)
                {
                    v.add(xpath.toString());
                }
                xpath.delete(0, xpath.length());
            }
        }

        String[] retVal = new String[v.size()];
        int i = 0;
        for (Iterator iter = v.iterator(); iter.hasNext();)
        {
            retVal[i++] = (String) iter.next();
        }
        return retVal;
    }

    protected void compareSingleObjects(RPMObject localObject,
            RPMObject[] dbObjects)
    {
        for (int i = 0; i < dbObjects.length; i++)
        {
            localObject = dbObjects[i];

            doDatafieldComparison(localObject, dbObjects[i]);
        }
    }

    protected void doDatafieldComparison(RPMObject localObject,
            RPMObject dbObject)
    {
        //Comparison between dbOject (the object retrieved from
        //the xpath query) and the control object
        try
        {
            if (dbObject != null)
            {
                objectCompare(localObject, dbObject, null);
            }
        }
        catch (AssertionError assertionError)
        {
            errorHandler.logOtherError(
                    "An assertion error occured while doing the comparison",
                    assertionError.getMessage());
        }
        catch (AssertionFailedError assertionFailedError)
        {
            errorHandler.logOtherError(
                    "An assertion failed error occured while doing the comparison",
                    assertionFailedError.getMessage());
        }
        catch (Exception e)
        {
            errorHandler.logOtherError(
                    "An exception occured while doing the comparison", e.getMessage());
        }
    }

    public RPMObject loadDatafieldFromXPath(Class objectClass, String xPath,
            String idOfObjectToFind)
    {
        RPMObject[] o = null;
        RPMObject c = null;
        int i = 0;

        try
        {
            o = loadFromXpath(xPath, null);

            if (o != null)
            {
                for (i = 0; i < o.length; i++)
                {
                    boolean isInstanceOf = objectClass.isInstance(o[i]);

                    if (!isInstanceOf)
                    {
                        StringBuffer errorMsg = new StringBuffer();

                        errorMsg.append("Received an object of type ").append(
                                o[i].getClass()).append(
                                " when expecting an object of type ").append(
                                objectClass);
                        errorHandler.logOtherError(errorMsg.toString(), null);
                    }
                }

                if (idOfObjectToFind != null)
                {
                    c = o[findObjectInArray(o, idOfObjectToFind)];
                }
            }
            else
            {
                errorHandler.logXPathError(i, xPath, null, true, simpleType);
            }
        }
        catch (Exception e)
        {
            errorHandler.logXPathError(i, xPath, e, false, simpleType);

            e.printStackTrace();
        }

        return c;
    }
    
    abstract protected void internalLoadDatafieldFromXPathAndCompare(
            RPMObject localObject, String xPath, RPMObjectScope scope,
            String idOfObjectToFind);

}