/*
 * Created on 16-Sep-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.framework.testcases.xpath.stats.StatsCounter;
import com.ibm.rpm.client.junit.framework.testcases.xpath.stats.StatsManager;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author njarazi
 */
public abstract class ObjectXPathTester extends XpathTester
{
    private static final int INDEX_XPATH_QUERY = 0;

    private static final int INDEX_OBJECT_ID = 1;

    protected ArrayList validSimpleTypes = null;

    public ObjectXPathTester()
    {
        validSimpleTypes = generateValidSimpleTypesList();
        skipList = generateSkipList();
    }

    public void setUp() throws Exception
    {
        super.setUp();

        initWithFullScope();
        if (myObject.getID() == null)
        {
            myObject = internalSave(myObject, mainScope);
        }
    }

    private ArrayList generateValidSimpleTypesList()
    {
        ArrayList l = new ArrayList();

        l.add(String.class);
        l.add(String[].class);
        l.add(Calendar.class);
        l.add(Calendar[].class);
        l.add(Boolean.class);
        l.add(Boolean[].class);
        l.add(Boolean.TYPE);
        l.add(Integer.class);
        l.add(Integer[].class);
        l.add(Integer.TYPE);
        l.add(Byte.class);
        l.add(Byte[].class);
        l.add(Byte.TYPE);
        l.add(Short.class);
        l.add(Short[].class);
        l.add(Short.TYPE);
        l.add(Long.class);
        l.add(Long[].class);
        l.add(Long.TYPE);
        l.add(Float.class);
        l.add(Float[].class);
        l.add(Float.TYPE);
        l.add(Double.class);
        l.add(Double[].class);
        l.add(Double.TYPE);
        l.add(Character.class);
        l.add(Character[].class);
        l.add(Character.TYPE);

        l.add(Object.class);
        l.add(Object[].class);

        l.trimToSize();
        return l;
    }

    /**
     * This method creates an RPMObject (type of the object is decided by
     * inheritence), saves it to the DB and queries it (xPath) with every simple
     * (String, Boolean, Integer, etc.) field.
     * 
     * @throws Exception
     */
    public void verifyContainerWithXPathQueries(RPMObject o) throws Exception
    {
        Method[] methods = o.getClass().getMethods();

        for (int i = 0; i < methods.length; i++)
        {
            Method tempMethod = methods[i];

            if (tempMethod.getName().startsWith(XpathTesterUtils.GET_PREFIX)
                    && !tempMethod.getName().endsWith(
                        XpathTesterUtils.MODIFIED_TRAILER)
                    && !isSkipped(tempMethod, skipList))
            {
                simpleType = isSimpleType(tempMethod, validSimpleTypes);
                if (simpleType)
                {
                    String xpath;

                    StatsManager.incrementSimplePropertiesCount();
                    xpath = generateSimpleXPathQuery(o, tempMethod);
                    if (xpath.length() > 0)
                    {

                        logger.info("XPath Simple #"
                                + StatsCounter.getInstance()
                                    .getSimplePropertiesCount() + ": " + xpath);

                        doObjectComparison(o, loadRPMObjectFromXPath(o
                            .getClass(), xpath, mainScope, o.getID(), 0));
                    }
                }
                else
                {
                    StatsManager.incrementComplexPropertiesCount();

                    ArrayList xpathQuery = generateLevelTwoXPathQuery(o,
                        tempMethod);

                    String objectName = o.getClass().getName();
                    String returnedObject = new String();
                    if (xpathQuery != null)
                    {
                        int queryNumber = 0;
                        for (Iterator iter = xpathQuery.iterator(); iter
                            .hasNext();)
                        {
                            String[] xpath = (String[]) iter.next();

                            returnedObject = getDbReturnedObjectName(xpath[INDEX_XPATH_QUERY]);
                            if (returnedObject.equalsIgnoreCase(objectName
                                .substring(objectName
                                    .lastIndexOf(XpathTesterUtils.PERIOD) + 1)))
                            {
                                logger.info("XPath Complex #"
                                        + StatsCounter.getInstance()
                                            .getComplexPropertiesCount() + "-"
                                        + queryNumber + ": "
                                        + xpath[INDEX_XPATH_QUERY]);

                                doObjectComparison(o, loadRPMObjectFromXPath(o
                                    .getClass(), xpath[INDEX_XPATH_QUERY],
                                    mainScope, xpath[INDEX_OBJECT_ID],
                                    ++queryNumber));
                            }
                            else
                            {
                                //Make sure that the object being returned from
                                // the database is the child of the myObject
                                // instance.
                                internalLoadRPMObjectFromXPathAndRelationship(
                                    tempMethod, xpath[INDEX_XPATH_QUERY],
                                    ++queryNumber);

                            }

                        }
                    }
                }

            }
            else if (isSkipped(tempMethod, skipList))
            {
                //Test to make sure that skipped elements have the correct
                // behavior.
                //These tests should make sure that the skipped elements are
                // actually not supported by the xpath framework
                verifyUnsupportedMethod(tempMethod);
            }
        }

        errorHandler.handleError();
    }

    protected String getDbReturnedObjectName(String xpath)
    {
        String ret = new String();
        char[] c = xpath.toCharArray();
        int forwardSlashCounter = 0;
        for (int i = 0; i < c.length; i++)
        {
            if (String.valueOf(c[i]).equalsIgnoreCase(
                XpathTesterUtils.FORWARD_SLASH))
            {
                forwardSlashCounter++;
            }
        }
        if (forwardSlashCounter == 1)
        {
            ret = xpath.substring(
                xpath.indexOf(XpathTesterUtils.FORWARD_SLASH) + 1, xpath
                    .indexOf(XpathTesterUtils.OPEN_BRACKET));
        }
        else
        {
            ret = xpath.substring(xpath
                .lastIndexOf(XpathTesterUtils.FORWARD_SLASH) + 1, xpath
                .lastIndexOf(XpathTesterUtils.OPEN_BRACKET));
        }

        return ret;
    }

    protected void doObjectComparison(RPMObject localObject, RPMObject dbObject)
    {
        if (localObject == null)
        {
            errorHandler
                .logOtherError(
                    "The returned object cannot be compared to the control object because the control object is null",
                    "");
        }
        
        //Comparison between dbOject (the object retrieved from
        //the xpath query) and the control object
        try
        {
            if (dbObject != null)
            {
                if (simpleType)
                {
                    objectCompare(localObject, dbObject, null);
                }
                else
                {
                    objectCompare(localObject, dbObject, mainScope);
                }
            }
        }
        catch (AssertionError assertionError)
        {
            assertionError.printStackTrace();
            errorHandler.logOtherError(
                "An assertion error occured while doing the comparison",
                assertionError.getMessage());
        }
        catch (AssertionFailedError assertionFailedError)
        {
            assertionFailedError.printStackTrace();
            errorHandler.logOtherError(
                "An assertion failed error occured while doing the comparison",
                assertionFailedError.getMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
            errorHandler.logOtherError(
                "An exception occured while doing the comparison", e
                    .getMessage());
        }
    }

    /**
     * Enables the generation of simple XPath queries (or queries on simple
     * objects such as int, boolean, etc. (refer to simpleTypeList to see all of
     * them)
     * 
     * @param object
     *            the RPMObject for which the query is being made
     * @param method
     *            the method of the RPMObject for which the query is being made
     * @return a String containing the resulting query
     * @throws Exception
     */
    public String generateSimpleXPathQuery(RPMObject object, Method method)
            throws Exception
    {
        List methodReturn = getMethodReturn(object, method);
        StringBuffer xpath = new StringBuffer();

        String objectName = object.getClass().getName();

        if (!methodReturn.isEmpty())
        {

            if (methodReturn.get(0) != null)
            {
                String returnTypeValue = getReturnValue(object, method);

                //The query generated here is of this type :
                //  "/Object[Field='field value']"
                //  ex: "/Reference[OrganizationName='OrganizationName 39']"
                xpath.append(XpathTesterUtils.FORWARD_SLASH).append(
                    objectName.substring(objectName
                        .lastIndexOf(XpathTesterUtils.PERIOD) + 1, objectName
                        .length())).append(XpathTesterUtils.OPEN_BRACKET).append("@")
                    .append(method.getName().substring(3)).append(
                        XpathTesterUtils.EQUALS);

                xpath.append(getMethodReturnValue(method, returnTypeValue));

                xpath.append(XpathTesterUtils.CLOSE_BRACKET);
            }
        }

        return xpath.toString();
    }

    public ArrayList generateLevelTwoXPathQuery(RPMObject object, Method method)
            throws Exception
    {
        StringBuffer xpath = new StringBuffer();
        ArrayList returnList = new ArrayList();
        String objectName = object.getClass().getName();

        String returnType = method.getReturnType().getName();
        returnType = returnType.substring(returnType
            .lastIndexOf(XpathTesterUtils.PERIOD) + 1, returnType.length());

        if (returnType.endsWith(XpathTesterUtils.SEMI_COLON))
        {
            returnType = returnType.substring(0, returnType.length() - 1);
        }

        if (method.getName().equalsIgnoreCase(XpathTesterUtils.GET_PARENT))
        {
            returnType = objectName.substring(objectName
                .lastIndexOf(XpathTesterUtils.PERIOD) + 1, objectName.length());
        }

        List methodReturn = getMethodReturn(object, method);

        if (!methodReturn.isEmpty() && methodReturn.get(0) != null)
        {

            String fieldName = null;
            try
            {
                String id = object.getID();

                StringBuffer sb = new StringBuffer(method.getName());

                if (method.getName().startsWith("get"))
                {
                    String firstChar = sb.substring(3, 4);
                    firstChar = firstChar.toLowerCase();
                    sb.replace(0, 4, firstChar);
                }
                fieldName = sb.toString();

                //The query generated here is of this type :
                //  "/Object1[[Object2FieldName[Field='field value']]"
                //  ex: "/Reference[companySize[ID='150']]"
                xpath.append(XpathTesterUtils.FORWARD_SLASH).append(
                    objectName.substring(objectName
                        .lastIndexOf(XpathTesterUtils.PERIOD) + 1, objectName
                        .length())).append(XpathTesterUtils.OPEN_BRACKET).append("@")
                    .append(fieldName).append(XpathTesterUtils.OPEN_BRACKET).append("@")
                    .append(XpathTesterUtils.ID_MARKER).append(
                        XpathTesterUtils.EQUALS);

                String returnedId = getRPMObjectID((RPMObject) methodReturn
                    .get(0));

                xpath.append(XpathTesterUtils.SINGLE_QUOTE);
                xpath.append(returnedId);
                xpath.append(XpathTesterUtils.SINGLE_QUOTE);

                xpath.append(XpathTesterUtils.CLOSE_BRACKET).append(
                    XpathTesterUtils.CLOSE_BRACKET);

                if (returnedId != null && returnedId.length() > 0)
                {
                    returnList.add(new String[]
                    {
                            xpath.toString(), id
                    });
                }
                else
                {
                    errorHandler
                        .logOtherError(
                            xpath.toString()
                                    + "\n\tXPath cannot be generated because the object "
                                    + returnType
                                    + " doesn't have an Id in this context", "");
                }

            }
            catch (ClassCastException e)
            {
                //This is here for the moment as "types" are not supported by
                // the xpath framework
                StatsManager.appendFieldError(
	                "\n\tXPath was not generated for field "
	                + fieldName
	                + " because "
	                + StringUtil.getShortClassName(methodReturn.get(0)
	                    .getClass()) + " is not an RPMObject.");
            }


            returnList.trimToSize();

            if (returnList.size() > 0)
            {
                return returnList;
            }
        }

        return null;
    }

    private String getRPMObjectID(RPMObject o)
    {
        String id = o.getID();

        if (id != null)
        {
            return id;
        }

        return new String();
    }

    protected void internalLoadRPMObjectFromXPathAndRelationship(Method method,
            String xpath, int queryNumber)
    {
        RPMObject[] childObjects = null;
        try
        {
            childObjects = loadFromXpath(xpath, null);

            List methodReturn = getMethodReturn(myObject, method);

            if (childObjects != null)
            {
                for (int i = 0; i < childObjects.length; i++)
                {
                    if (!methodReturn.isEmpty())
                    {
                        for (Iterator iter = methodReturn.iterator(); iter
                            .hasNext();)
                        {
                            RPMObject element = (RPMObject) iter.next();

                            if (!element.getID().equalsIgnoreCase(
                                childObjects[i].getID()))
                            {
                                errorHandler.logXPathError(queryNumber, xpath,
                                    null, true, simpleType);
                            }
                        }
                    }
                }
            }
            else
            {
                errorHandler.logXPathError(queryNumber, xpath, null, true,
                    simpleType);
            }
        }
        catch (Exception e)
        {
            errorHandler
                .logXPathError(queryNumber, xpath, e, false, simpleType);
        }
    }

    public boolean isSimpleType(Method method, ArrayList validTypes)
    {
        if (validTypes.contains(method.getReturnType()))
        {
            return true;
        }

        return false;
    }

    public RPMObject loadRPMObjectFromXPath(Class objectClass, String xPath,
            RPMObjectScope scope, String idOfObjectToFind, int queryNumber)
    {
        RPMObject[] o = null;
        RPMObject c = null;

        try
        {
            o = loadFromXpath(xPath, scope);

            String expectedClassName = xPath.substring(xPath
                .lastIndexOf(XpathTesterUtils.FORWARD_SLASH) + 1, xPath
                .indexOf(XpathTesterUtils.OPEN_BRACKET, xPath
                    .lastIndexOf(XpathTesterUtils.FORWARD_SLASH)));

            String packageName = objectClass.getPackage().toString();

            Class expectedClass = Class.forName(packageName
                .substring(packageName.lastIndexOf(XpathTesterUtils.SPACE) + 1)
                    + XpathTesterUtils.PERIOD + expectedClassName);

            Class testedClass;

            if (objectClass.equals(expectedClass))
            {
                testedClass = objectClass;
            }
            else
            {
                testedClass = expectedClass;
            }

            if (o != null)
            {
                for (int i = 0; i < o.length; i++)
                {
                    boolean isInstanceOf = testedClass.isInstance(o[i]);

                    if (!isInstanceOf)
                    {
                        StringBuffer errorMsg = new StringBuffer();

                        errorMsg.append("Received an object of type ").append(
                            o[i].getClass()).append(
                            " when expecting an object of type ").append(
                            objectClass);

                        errorHandler.logOtherError(errorMsg.toString(), null);
                    }
                }

                c = o[findObjectInArray(o, idOfObjectToFind)];
            }
            else
            {
                errorHandler.logXPathError(queryNumber, xPath, null, true,
                    simpleType);
            }
        }
        catch (ArrayIndexOutOfBoundsException oob)
        {
            errorHandler.logXPathError(queryNumber, xPath, oob, false,
                simpleType);
        }
        catch (Exception e)
        {
            errorHandler
                .logXPathError(queryNumber, xPath, e, false, simpleType);
            e.printStackTrace();
        }

        return c;
    }

    /**
     * Saves the object in the database and gets the ID generated by the save.
     * Takes the ID and assigns it to the object for future comparisons.
     * 
     * @param object
     *            the object to save in the DB
     * @param scope
     *            the scope of the object to be saved
     */
    protected RPMObject internalSave(RPMObject object, RPMObjectScope scope)
    {
        SingleResult result = null;
        try
        {
            result = save(object, scope);
            if (!result.isSuccessful())
            {
                for (int i = 0; i < result.getErrors().length; i++)
                {
                    errorHandler.logOtherError("", result.getErrors()[i]
                        .getMessage());
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        if (result != null)
            return result.getRpmObject();
        return null;
    }
}