/*
 * Created on 25-Oct-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath;

import java.lang.reflect.Method;
import java.util.*;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.framework.testcases.xpath.errors.ErrorHandler;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;

/**
 * @author njarazi
 */
public abstract class XpathTester extends ClientTestController
{
    protected RPMObject myObject = null;
    protected RPMObjectScope mainScope = null;
    protected RPMObject dbObject = null;

    protected ErrorHandler errorHandler = new ErrorHandler();
    
    protected ArrayList skipList = null;

    protected boolean simpleType;
    
    protected ArrayList localNumberWithQuotes = null;
    protected ArrayList localStringWithoutQuotes = null;
    
    public void testContainer() throws Exception
    {
        verifyContainerWithXPathQueries(myObject);
    }

    public void testContainerInstances()
    {
        verifyContainerInstanceWithXPathQueries(myObject);
    }
        
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
        
    /**
     * @param myObject2
     */
    private void verifyContainerInstanceWithXPathQueries(RPMObject object)
    {
        StringBuffer xpath = new StringBuffer();

        String objectName = object.getClass().getName();
        //The query generated here is of this type :
        //  "/Object1"
        //  ex: "/Reference"
        xpath.append(XpathTesterUtils.FORWARD_SLASH).append(
                objectName.substring(objectName.lastIndexOf(XpathTesterUtils.PERIOD) + 1,
                        objectName.length()));

        try
        {
            RPMObject loadedObjects[] = loadFromXpath(xpath.toString(), null);
            if (loadedObjects != null)
            {
	            for (int i = 0; i < loadedObjects.length; i++)
	            {
	                verifyContainerInstance(loadedObjects[i]);	                
	            }
            }
            else
            {
                errorHandler.logXPathError(-1, xpath.toString(), null, true, simpleType);
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("Container instance test : An error happened during the load of " + xpath.toString(), e.getMessage());
        }

        errorHandler.handleError();
    }

    private void verifyContainerInstance(RPMObject object)
    {
        Class objectClass = object.getClass();
        
        try
        {
	        boolean isInstanceOf = objectClass.isInstance(object);
	        
	        if (!isInstanceOf)
	        {
	            StringBuffer errorMsg = new StringBuffer();
	
	            errorMsg
	                    .append(
	                            "Container instance test : Received an object of type ")
	                    .append(object.getClass()).append(
	                            " when expecting an object of type ")
	                    .append(objectClass);
	            errorHandler.logOtherError(errorMsg.toString(), null);
	        }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("Container instance test : An error has occured", e.getMessage());
        }
    }

    public ArrayList generateSkipList()
    {
        ArrayList l = new ArrayList();

        l.add("getClass");
        l.add("getDeleted");
        l.add("getContextName");
        l.add("getParent");

        //Types are skipped for the moment...
        //ApplicationAdministration
        l.add("getInputType");
        l.add("getSelectType");

        
        //Fields which are arrays and are not mapped 1:1 in DB
        l.add("getMyPortalURLs");
        
        //Deal with local skip list
        ArrayList local = generateLocalSkipList();
        if (local != null)
        {
            for (int i = 0; i < local.size(); i++)
            {
                l.add(local.get(i));
            }
        }
        
        l.trimToSize();
        return l;
    }
    
    protected ArrayList generateLocalSkipList()
    {
        return null;
    }

    public boolean isSkipped(Method method, ArrayList skipList)
    {
        if (skipList.contains(method.getName()))
        {
            return true;
        }
        
        return false;
    }

    public int findObjectInArray(RPMObject[] array, String id)
    {
        for (int i = 0; i < array.length; i++)
        {
            //find the object in the array of returned objects
            if (array[i].getID().equalsIgnoreCase(id))
            {
                return i;
            }
        }

        return XpathTesterUtils.NOT_FOUND;
    }

    /**
     * @param tempMethod
     */
    protected void verifyUnsupportedMethod(Method method)
    {
        /*
         ******************
         * Not yet completed so will be delivered as comments...
         ******************
         */
        
//        try
//        {
//            StringBuffer xpath = new StringBuffer();
//            
//            //Generate XPath query
//            xpath.append(XpathTesterUtils.FORWARD_SLASH)
//            	.append(XpathTesterUtils.getObjectName(myObject))
//            	.append(XpathTesterUtils.OPEN_BRACKET)
//            	.append(method.getName().substring(3))
//            	.append(XpathTesterUtils.EQUALS)
//            	.append(XpathTesterUtils.getMethodReturnValue(method, getReturnValue(myObject, method)))
//            	.append(XpathTesterUtils.CLOSE_BRACKET);
//            
//            //execute query
//            RPMObject[] dbResult = loadFromXpath(xpath.toString(), mainScope);
//            
//            assertTrue(dbResult == null);            
//        }
//        catch (Exception e)
//        {
//            e.printStackTrace();
//        }
    }
    
    protected String getReturnValue(RPMObject object, Method method)
    {
        List methodReturn = null;
        String returnTypeValue = null;
        try
        {
            methodReturn = getMethodReturn(object, method);
            
             returnTypeValue = methodReturn.get(0).toString();

            //Test to convert Calendars to a date value in format : 'yyyy-MM-dd'
            if (returnTypeValue.split("\\" + XpathTesterUtils.PERIOD).length > 1 
                    && returnTypeValue.matches(".*" + XpathTesterUtils.GREGORIAN_CALENDAR_STRING + ".*"))
            {
                returnTypeValue = returnTypeValue.substring(0, methodReturn
                        .get(0).toString().indexOf(XpathTesterUtils.OPEN_BRACKET));
                returnTypeValue = returnTypeValue.substring(returnTypeValue
                        .lastIndexOf(XpathTesterUtils.PERIOD) + 1, returnTypeValue.length());
                returnTypeValue = XpathTesterUtils.convertToString((Calendar)methodReturn.get(0));
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        
        return returnTypeValue;
    }
    
    protected ArrayList getMethodReturn(RPMObject object, Method method)
            throws Exception
    {
        ArrayList list = new ArrayList();

        if (method.getReturnType().isArray())
        {
            Object[] methodReturn = null;
            try
            {
                methodReturn = (Object[]) method.invoke(object, null);
            }
            catch (Exception e)
            {
                StringBuffer errorMsg = new StringBuffer();

                errorMsg.append("Error getting method return for ").append(
                        method.getDeclaringClass().getName()).append(".")
                        .append(method.getName());

                errorHandler.logOtherError(errorMsg.toString(), e.getMessage());
            }

            if (methodReturn != null)
            {
                for (int i = 0; i < methodReturn.length; i++)
                {
                    list.add(methodReturn[i]);
                }
            }
        }
        else
        {
            list.add(method.invoke(object, null));
        }

        list.trimToSize();

        return list;
    }

    public abstract void verifyContainerWithXPathQueries(RPMObject o)
            throws Exception;

    protected abstract RPMObject createObject() throws Exception;
    protected abstract RPMObjectScope createScope() throws Exception;

    protected void initWithFullScope() throws Exception
    {
        mainScope = createScope();
        myObject = createObject();
    }
    
    abstract protected void objectCompare(RPMObject myObject,
            RPMObject dbObject, RPMObjectScope scope)
            throws AssertionFailedError, Exception;

    public String getMethodReturnValue(Method method, String returnTypeValue)
    {
        StringBuffer ret = new StringBuffer();
        
        boolean isNumeric = XpathTesterUtils.isMethodReturnNumeric(method);
        boolean isBoolean = XpathTesterUtils.isMethodReturnBoolean(method);
        
        boolean requiresQuotes = !XpathTesterUtils.booleanFieldsWithoutQuotes.contains(method.getName()) 
        	|| XpathTesterUtils.numericFieldsWithQuotes.contains(method.getName());
        
        // Check local needs.
        setLocalQuoteList();
        requiresQuotes = verifyLocalQuotes(requiresQuotes, method);
    
        if (isBoolean)
        {
            //always append single quotes
            ret.append(XpathTesterUtils.SINGLE_QUOTE);
            
            if (returnTypeValue.equalsIgnoreCase(XpathTesterUtils.TRUE))
            {
                ret.append("true");
            }
            else
            {
                ret.append("false");
            }
            
            ret.append(XpathTesterUtils.SINGLE_QUOTE);
        }
        else
        {
            if (!isNumeric && requiresQuotes)
            {
                ret.append(XpathTesterUtils.SINGLE_QUOTE);
            }
            
            // This covers the situation that a number field needs quotes
            if (localNumberWithQuotes != null || localStringWithoutQuotes != null)
            {
    	        if (isNumeric && requiresQuotes)
    	        {
    	            ret.append(XpathTesterUtils.SINGLE_QUOTE);
    	        }
            }   
            
            ret.append(returnTypeValue);
            
            if (!isNumeric && requiresQuotes)
            {
                ret.append(XpathTesterUtils.SINGLE_QUOTE);
            }
            
            if (localNumberWithQuotes != null || localStringWithoutQuotes != null)
            {
    	        if (isNumeric && requiresQuotes)
    	        {
    	            ret.append(XpathTesterUtils.SINGLE_QUOTE);
    	        }
            }  
        }
    
        return ret.toString();
    }
    
    protected void setLocalQuoteList() 
    {
        // Do Nothing now.
        // Child class needs to override this whenever needed.
    }
    
    private boolean verifyLocalQuotes(boolean preRequirement, Method method)
    {
        boolean requiresQuotes = preRequirement;
        
        if (localNumberWithQuotes != null)
        {
            if (localNumberWithQuotes.contains(method.getName()))
            {
                requiresQuotes = true;
            }
        }
        
        if (localStringWithoutQuotes != null)
        {
            if (localStringWithoutQuotes.contains(method.getName()))
            {
                requiresQuotes = false;
            }
        }
        
        return requiresQuotes;
    }
}
