/*
 * Created on Jun 28, 2005
 */
package com.ibm.rpm.client.junit.helpers;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.types.CalendarExceptionType;
import com.ibm.rpm.client.junit.framework.testcases.RpmObjectAbstractTester;
import com.ibm.rpm.client.junit.framework.testcases.RpmTestFramework;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.types.TimeUnitType;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.PointerLink;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class HelperTestUtilities
{

    private static boolean TOGGLE = true;

    private static String userName = null;

    //Prefix length must be <= 2.
    public static final String unitTestPrefix = "TC";

    static
    {
        userName = System.getProperty("user.name");
        if (userName == null)
        {
            userName = "test";
        }
    }

    /**
     * Creates a Calendar for the Date type
     * 
     * @param date
     * @return
     */
    public static Calendar convertToCalendar(Date date)
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        return calendar;
    }

    /**
     * This method will call all the setters on the objectToSet with different
     * values. Only basic types are used
     * 
     * @param objectToSet
     * @param index
     * @throws Exception
     */
    public static void setByIntrospection(Object objectToSet, int index)
            throws Exception
    {
        setByIntrospection(objectToSet, index, new ArrayList());
    }

    public static Object findSetValue(RPMObject objectToFill, Container classInfo,
            Field fieldInfo, int seed,boolean forCreation) throws Exception
    {
        Class parameterType = fieldInfo.getJavaType();
        String fieldName = fieldInfo.getName();

        Object retVal = null;
        if (parameterType == String.class)
        {
            String value = null;
            if (fieldName.equalsIgnoreCase("Name"))
            {
                value = HelperTestUtilities.createName() + seed;
            }
            else
            {
                value = fieldName + " " + seed;
            }

            retVal = value;
        }
        else if (parameterType == Boolean.TYPE
                || parameterType == Boolean.class)
        {
            boolean toggle = seed % 2 == 0;
            Boolean valueOf = Boolean.valueOf(toggle);
            retVal = valueOf;
        }
        else if (parameterType == Integer.TYPE
                || parameterType == Integer.class)
        {
            retVal = new Integer(seed);
        }
        else if (parameterType == Long.TYPE
                || parameterType == Long.class)
        {
            retVal = new Long(seed);
        }
        else if (parameterType == Float.TYPE || parameterType == Float.class)
        {
            retVal = new Float(seed * 1000 + seed / 200);
        }
        else if (parameterType == Double.TYPE || parameterType == Double.class)
        {
            retVal = new Double(seed * 1000 + seed / 200);
        }
        else if (parameterType == Calendar.class)
        {
            Calendar calendar = createCalendar(seed);
            retVal = calendar;
        }
        else if (parameterType == Date.class)
        {
            Calendar calendar = createCalendar(seed);
            retVal = calendar.getTime();
        }
        else if (parameterType.isArray())
        {
            if (!fieldInfo.compositeValue())
            {
                Assert
                    .fail("there should be no array of reference in the model : "
                            + fieldInfo.getName());
            }

            retVal = createArrayOfChilds(objectToFill, fieldInfo);
        }
        else if (RPMObject.class.isAssignableFrom(parameterType))
        {
            List childTypeList = fieldInfo.getValidTypePointerList();

            // The parent can be null for this situation
            if (fieldName.equals("parent") && fieldInfo.createCanBeNullValue()
                    && childTypeList == null || childTypeList.size() < 1)
            {
                retVal = null;
            }
            else
            {
                Assert.assertNotNull(childTypeList);
                Assert.assertTrue(childTypeList.size() > 0);
                PointerLink childPointer = (PointerLink) childTypeList.get(0);

                if (!fieldInfo.compositeValue())
                {
                    childPointer = getValidChildType(classInfo, fieldInfo);
                    retVal = RpmTestFramework.INSTANCE
                        .getStaticFixtureFromPointer(childPointer, seed);
                }
                else
                {
                    RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
                        .findFixture(childPointer.getPointerClass());

                    RPMObject createObject = findFixture.createObject();
                    findFixture.fillObject(createObject, null, seed, forCreation);
                    retVal = createObject;
                }
            }
        }
        else if (fieldInfo.isEnumeration())
        {
            Collection enumValues = fieldInfo.getEnumerationInfo()
            	.getJavaStateMap().values();
	        int index = seed % enumValues.size();
	        
	        retVal = enumValues.toArray()[index];
        }
        // TODO make it more general, right now just for scorecard package
        else if (parameterType == ProbabilityType.class)
        {
            retVal = ProbabilityType.None;
        }
        else if (parameterType == ProjectPublishedType.class)
        {
            retVal = ProjectPublishedType.None;
        }
        else if (parameterType == CalendarExceptionType.class)
        {
            retVal = CalendarExceptionType.NonDefaultWorking;
        }
        else if (parameterType == LeadLagType.class)
        {
            retVal = LeadLagType.Lead;
        }
        else if (parameterType == TimeUnitType.class)
        {
            retVal = TimeUnitType.Days;
        }
        else if (parameterType == DependencyType.class)
        {
            retVal = DependencyType.FinishStart;
        }
        else if (parameterType == ElementSecurityType.class)
        {
            retVal = ElementSecurityType.ByInvitation;
        }
        else if (parameterType == PublishedType.class)
        {
            retVal = PublishedType.None;
        }
        //TODO remove comments to implement ProjectReconciliation
        /*else if (parameterType == ReconciliationState.class)
        {
            retVal = ReconciliationState.Accepted;
        }*/
        else if (parameterType == AssignmentType.class)
        {
            retVal = AssignmentType.Work;
        }
        else if (parameterType == Object.class)
        {
            retVal = null;
        }
        else
        {
            Assert.fail("unable to set object type " + parameterType.getName());
        }

        return retVal;
    }

    public static PointerLink getValidChildType(Container classInfo,
            Field fieldInfo)
    {
        List children = fieldInfo.getValidTypePointerList();
        if(children.size()==1){
            return (PointerLink)children.get(0);
        }
        
        PointerLink child = null;
        for (int i = 0; i < children.size() && child == null; i++)
        {
            PointerLink temp = (PointerLink) children.get(i);           
            if (temp.getPointerClass() != classInfo.getJavaClass())
            {
                child = temp;
            }
        }
        
        //if we couldn't find one that is not equal to us, then allow equal to us
        if(child==null){
            for (int i = 0; i < children.size() && child == null; i++)
            {
                PointerLink temp = (PointerLink) children.get(i);           
                if (temp!=null)
                {
                    child = temp;
                }
            }
            
        }
        return child;
    }

    public static Object createArrayOfChilds(RPMObject parent, Field fieldInfo)
            throws Exception
    {
        Class parameterType = fieldInfo.getJavaType();
        Object retVal;
        Class componentType = parameterType.getComponentType();

        List childTypeList = fieldInfo.getValidTypePointerList();

        Assert.assertTrue(
            "no defined child for element " + fieldInfo.getName() +
            " in container " + StringUtil.getShortClassName(fieldInfo.getGetterMethod().getDeclaringClass()),
            childTypeList.size() > 0);

        List createdChilds = new ArrayList();

        // always add a null to make sure we handle them well
        createdChilds.add(null);

        Iterator it = childTypeList.iterator();
        while (it.hasNext())
        {
            PointerLink childType = (PointerLink) it.next();
            RpmObjectAbstractTester childFixture = RpmTestFramework.INSTANCE
                .findFixture(childType.getPointerClass());

            // skip abstract classes
            if (!childFixture.getClassInformation().isAbstractClass())
            {
                RPMObject createObject = childFixture.createObject();                
                childFixture.fillMandatoryFields(createObject, true);
                //// no need: createObject.setParent(parent);
                createdChilds.add(createObject);
            }
            else
            {
                Assert
                    .fail("detected abstract class in valid child list of field:"
                            + fieldInfo.getName()
                            + ", abstract class: "
                            + childFixture.getClassInformation()
                                .getShortTypeName());
            }
        }

        Object[] retValArray = (Object[]) Array.newInstance(componentType,
            createdChilds.size());
        retValArray = createdChilds.toArray(retValArray);
        retVal = retValArray;
        return retVal;
    }

    /**
     * @param seed
     * @return
     */
    private static Calendar createCalendar(int seed)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.set(2000 + (seed % 50), 1 + (seed % 12), 1 + (seed % 28));
        return calendar;
    }
    
    public static final String UPDATE_STRING1 = "TCuA";
    public static final String UPDATE_STRING2 = "TCuB";
    
    public static Object findUpdateValue(Container container, Field fieldInfo,
            Object oldValue, int seed) throws Exception
    {
        Class parameterType = fieldInfo.getJavaType();

        if (oldValue != null)
        {
            oldValue.getClass();
        }
        Object retVal = null;
        if (parameterType == String.class)
        {
            if ( oldValue == null )
            {
                retVal = UPDATE_STRING1;
            }
            else
            {
                if( oldValue.equals(UPDATE_STRING1))
                {
                    retVal = UPDATE_STRING2;
                }
                else
                {
                    retVal = UPDATE_STRING1;
                }
            }
        }
        else if (parameterType == Boolean.TYPE
                || parameterType == Boolean.class)
        {
            Boolean oldValueBoolean = (Boolean) oldValue;
            Boolean valueOf = Boolean.valueOf(!oldValueBoolean.booleanValue());
            retVal = valueOf;
        }
        else if (parameterType == Integer.TYPE
                || parameterType == Integer.class)
        {
            Number typedMaxValue = (Number) fieldInfo.getTypedMaxValue();
            Integer oldNumber = (Integer) oldValue;
            retVal = new Integer(oldNumber.intValue() + 10);
            if (typedMaxValue != null
                    && ((Integer) retVal).intValue() > (typedMaxValue.intValue()))
            {
                retVal = new Integer(typedMaxValue.intValue());
            }
        }
        else if (parameterType == Float.TYPE || parameterType == Float.class)
        {
            Number typedMaxValue = (Number) fieldInfo.getTypedMaxValue();
            Float oldNumber = (Float) oldValue;
            retVal = new Float(oldNumber.floatValue() + 10.1);
            if (typedMaxValue != null
                    && ((Float) retVal).floatValue() > ((Float) typedMaxValue).floatValue())
            {
                retVal = new Float(typedMaxValue
                    .floatValue());
            }
        }
        else if (parameterType == Double.TYPE || parameterType == Double.class)
        {
            Number typedMaxValue = (Number) fieldInfo.getTypedMaxValue();
            Double oldNumber = (Double) oldValue;
            retVal = new Double(oldNumber.doubleValue() + 10.1);
            if (typedMaxValue != null
                    && ((Double) retVal).doubleValue() > (typedMaxValue.doubleValue()))
            {
                retVal = new Double(typedMaxValue
                    .doubleValue());
            }
        }
        else if (parameterType == Calendar.class)
        {
            Calendar oldCalendar = (Calendar) oldValue;
            Calendar calendar = null;
            if (fieldInfo.isStartDate())
            {
                calendar = updateStartDate(oldCalendar);
            }
            else
            {
                calendar = updateCalendar(oldCalendar);
            }

            retVal = calendar;
        }
        else if (parameterType == Date.class)
        {
            Date oldDate = (Date) oldValue;
            Calendar oldCalendar = Calendar.getInstance();
            oldCalendar.setTime(oldDate);
            Calendar calendar = updateCalendar(oldCalendar);

            retVal = calendar.getTime();
        }
        else if (parameterType.isArray())
        {
            Object[] oldArray = (Object[]) oldValue;
            if (!fieldInfo.compositeValue())
            {
                Assert
                    .fail("there should be no array of reference in the model : "
                            + fieldInfo.getName());
            }
            else
            {
                if ( oldArray != null )
                {
	                for (int i = 0; i < oldArray.length; i++)
	                {
	                    Object object = oldArray[i];
	                    updateRPMObject(object, seed);
	                }
                }
                else
                {
                    Assert.fail("failed to update empty array : "
                            + fieldInfo.getName());
                }
            }
            retVal = oldValue;
        }
        else if (RPMObject.class.isAssignableFrom(parameterType))
        {
            List childTypeList = fieldInfo.getValidTypePointerList();
            Assert.assertNotNull(childTypeList);
            Assert.assertTrue(childTypeList.size() > 0);

            if (!fieldInfo.compositeValue())
            {
                PointerLink childPointer = (PointerLink) childTypeList.get(0);
                retVal = RpmTestFramework.INSTANCE
                    .getStaticFixtureFromPointer(childPointer, seed + 1);
            }
            else
            {
                retVal = updateRPMObject(oldValue, seed);
            }
        }
        else if (fieldInfo.isEnumeration())
        {
            Collection enumValues = fieldInfo.getEnumerationInfo()
                .getJavaStateMap().values();
            int index = seed % enumValues.size();
            
            Object[] toArray = enumValues.toArray();
            retVal = toArray[index];
            
            // make sure we dont reuse the same value
            if( retVal == oldValue)
            {
                index = (seed +1)% enumValues.size();
                
                retVal = toArray[index];
            }
        }
        // TODO same thing with findSetValue
        // right now just for scorecard
        else if (parameterType == ProbabilityType.class)
        {
            retVal = ProbabilityType.None;
        }
        else
        {
            Assert.fail("unable to update object type "
                    + parameterType.getName());
            ;
        }

        return retVal;
    }

    private static Calendar updateStartDate(Calendar oldCalendar)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.set(oldCalendar.get(Calendar.YEAR), oldCalendar
            .get(Calendar.MONTH), oldCalendar.get(Calendar.DAY_OF_MONTH));
        calendar.add(Calendar.DATE, -1);
        return calendar;
    }

    /**
     * @param oldValue
     * @param seed
     * @return
     * @throws Exception
     */
    private static Object updateRPMObject(Object oldValue, int seed)
            throws Exception
    {
        Object retVal;
        RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
            .findFixture(oldValue.getClass());

        RPMObject updateObject = (RPMObject) oldValue;
        findFixture.updateOneField(updateObject, seed);
        retVal = updateObject;
        return retVal;
    }

    /**
     * @param oldCalendar
     * @return
     */
    private static Calendar updateCalendar(Calendar oldCalendar)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.set(oldCalendar.get(Calendar.YEAR) + 10, 1 + ((oldCalendar
            .get(Calendar.MONTH) + 3) % 12), 1 + ((oldCalendar
            .get(Calendar.DAY_OF_MONTH) + 3) % 28));
        return calendar;
    }

    public static List getSetterMethodList(Class objectClass)
    {
        List retVal = new ArrayList();
        Method[] methods = objectClass.getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            Method temp = methods[i];
            Class[] parameterTypes = temp.getParameterTypes();
            if (temp.getName().startsWith("set") && parameterTypes.length == 1
                    && !temp.getName().endsWith("Modified"))
            {
                retVal.add(temp);
            }
        }

        return retVal;
    }

    public static List getFieldList(Class objectClass)
    {
        List retVal = new ArrayList();
        Method[] methods = objectClass.getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            Method temp = methods[i];
            Class[] parameterTypes = temp.getParameterTypes();
            if (temp.getName().startsWith("set") && parameterTypes.length == 1
                    && !temp.getName().endsWith("Modified"))
            {
                String field = temp.getName().substring(3);
                retVal.add(field);
            }
        }

        return retVal;
    }

    /**
     * 
     * @param objectToSet
     * @param index
     * @param skipMethodList:
     *            list of set methods to skip
     * @throws Exception
     */
    public static void setByIntrospection(Object objectToSet, int index,
            final List skipMethodList) throws Exception
    {
        if (objectToSet == null)
        {
            return;
        }
        Method[] methods = objectToSet.getClass().getMethods();
        TOGGLE = index % 2 == 0;
        for (int i = 0; i < methods.length; i++)
        {
            Method temp = methods[i];
            Class[] parameterTypes = temp.getParameterTypes();
            if (temp.getName().startsWith("set") && parameterTypes.length == 1
                    && !skipMethodList.contains(temp.getName())
                    && !temp.getName().endsWith("Modified"))
            {
                Object[] args = new Object[1];
                Class param = parameterTypes[0];

                if (param == String.class)
                {
                    String field = temp.getName().substring(3);
                    String value = null;
                    if (field.equals("Name"))
                    {
                        value = createName() + index;
                    }
                    else
                    {
                        value = field + " " + index;
                    }

                    args[0] = value;
                }
                else if (param == Boolean.TYPE || param == Boolean.class)
                {
                    Boolean valueOf = Boolean.valueOf(TOGGLE);
                    TOGGLE = !TOGGLE;
                    args[0] = valueOf;
                }
                else if (param == Integer.TYPE || param == Integer.class)
                {
                    args[0] = new Integer(index + i);
                }
                else if (param == Float.TYPE || param == Float.class)
                {
                    args[0] = new Float(index + i * 1000 + index / 200);
                }
                else if (param == Double.TYPE || param == Double.class)
                {
                    args[0] = new Double(index + i * 1000 + index / 200);
                }
                else if (param == Calendar.class)
                {
                    // Largest year in the table FISCAL_CALENDAR is 222
                    Calendar calendar = Calendar.getInstance();
                    calendar.set(2000 + (i % 50), 1 + (i % 12),
                        1 + (index % 28));
                    args[0] = calendar;
                }
                else if (param == Date.class)
                {
                    // Largest year in the table FISCAL_CALENDAR is 2226
                    Calendar calendar = Calendar.getInstance();
                    calendar.set(2000 + (i % 50), 1 + (i % 12),
                        1 + (index % 28));
                    args[0] = calendar.getTime();
                }
                else
                {
                    continue;
                }
                temp.invoke(objectToSet, args);
            }
        }
    }

    public static final List VALID_CLASS_LIST = new ArrayList();

    static
    {
        VALID_CLASS_LIST.add(String.class);

        VALID_CLASS_LIST.add(Boolean.class);
        VALID_CLASS_LIST.add(Boolean.TYPE);

        VALID_CLASS_LIST.add(Integer.class);
        VALID_CLASS_LIST.add(Integer.TYPE);

        VALID_CLASS_LIST.add(Float.class);
        VALID_CLASS_LIST.add(Float.TYPE);

        VALID_CLASS_LIST.add(Double.class);
        VALID_CLASS_LIST.add(Double.TYPE);

        VALID_CLASS_LIST.add(Calendar.class);
    }

    /**
     * this method will set all the object setters that use RPMObjects
     * 
     * @param objectToSet
     *            object on which to set
     * @param index
     * @param recursive
     * @throws Exception
     */

    public static void setChildsByIntrospection(Object objectToSet, int index,
            boolean recursive, final List skipMethodList) throws Exception
    {
        internalSetChild(objectToSet, index, recursive, new LinkedList(),
            skipMethodList);
    }

    public static void setChildsByIntrospection(Object objectToSet, int index,
            boolean recursive) throws Exception
    {
        internalSetChild(objectToSet, index, recursive, new LinkedList(),
            new ArrayList());
    }

    private static void internalSetChild(Object objectToSet, int index,
            boolean recursive, List parsedObjects, List skipMethodList)
            throws Exception
    {
        if (parsedObjects.contains(objectToSet.getClass()))
        {
            throw new IllegalArgumentException(
                "Object reference loop detected, don't fill recursively");
        }

        parsedObjects.add(0, objectToSet.getClass());

        Method[] methods = objectToSet.getClass().getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            Method temp = methods[i];
            Class[] parameterTypes = temp.getParameterTypes();
            if (temp.getName().startsWith("set") && parameterTypes.length == 1)
            {
                if (temp.getName().equals("setCurrency"))
                {
                    int junk = 4;
                    junk++;
                }
                Object[] args = new Object[1];
                Class param = parameterTypes[0];
                if (RPMObject.class.isAssignableFrom(param))
                {
                    Object newInstance = param.getConstructor(null)
                        .newInstance(null);
                    setByIntrospection(newInstance, index, skipMethodList);

                    if (recursive)
                    {
                        internalSetChild(newInstance, index, recursive,
                            parsedObjects, skipMethodList);
                    }
                    args[0] = newInstance;
                    temp.invoke(objectToSet, args);
                }
            }
        }

        parsedObjects.remove(0);
    }

    /**
     * this method will verify that all the parameters that have getter methods
     * have an equal value. Only basic types are checked
     * 
     * @param expected
     * @param value
     * @throws Exception
     */
    public static void assertEqualsByIntrospection(Object expected, Object value)
            throws Exception
    {
        assertEqualsByIntrospection(expected, value, new ArrayList());
    }

    public static void assertEqualsByIntrospection(Object expected,
            Object value, List skipMethodList) throws Exception
    {
        assertEqualsByIntrospection(expected, value, skipMethodList, null);

    }

    public static void assertEqualsByIntrospection(Object expected,
            Object value, List skipMethodList, String[] skipRegexPatterns)
            throws Exception
    {

        if (expected == null)
        {
            Assert.assertNull(value);
            return;
        }

        Assert.assertSame(expected.getClass(), value.getClass());

        Method[] methods = expected.getClass().getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            Method method = methods[i];

            boolean compare = true;
            if (skipRegexPatterns != null)
            {
                for (int j = 0; j < skipRegexPatterns.length; j++)
                {
                    if (method.getName().matches(skipRegexPatterns[j]))
                    {
                        compare = false;
                    }
                }
            }

            Class[] parameterTypes = method.getParameterTypes();
            if (compare && method.getName().startsWith("get")
                    && parameterTypes.length == 0
                    && !skipMethodList.contains(method.getName()))
            {
                Object[] args = new Object[0];
                Class param = method.getReturnType();
                if (VALID_CLASS_LIST.contains(param))
                {
                    Object expectValue = method.invoke(expected, args);
                    Object resultValue = method.invoke(value, args);

                    if (expectValue instanceof String
                            && resultValue instanceof String)
                    {
                        expectValue = ((String) expectValue).trim();
                        resultValue = ((String) resultValue).trim();

                        Assert.assertEquals("expected equal property : "
                                + method.getName(), expectValue, resultValue);

                    }
                    else if (expectValue instanceof Calendar
                            && resultValue instanceof Calendar)
                    {

                        String tempExpectValue = convertCalendarToString((Calendar) expectValue);
                        String tempResultValue = convertCalendarToString((Calendar) resultValue);

                        Assert.assertEquals("expected equal property : "
                                + method.getName(), tempExpectValue,
                            tempResultValue);
                    }
                    else
                    {
                        Assert.assertEquals("expected equal property : "
                                + method.getName(), expectValue, resultValue);
                    }
                }
            }
        }
    }

    public static String convertCalendarToString(Calendar calendar)
    {

        String date = null;

        if (calendar == null)
        {
            date = null;
        }
        else
        {

            date = calendar.get(GregorianCalendar.YEAR) + "-"
                    + (calendar.get(GregorianCalendar.MONTH) + 1) + "-"
                    + calendar.get(GregorianCalendar.DAY_OF_MONTH);
        }

        return date;

    }

    public static RPMObject findRPMObjectFromList(String id,
            RPMObject[] objArray)
    {
        for (int i = 0; i < objArray.length; i++)
        {
            if (id.startsWith(objArray[i].getID()))
                return objArray[i];
        }
        return null;
    }

    public static String createName()
    {

        StringBuffer testName = new StringBuffer(unitTestPrefix + userName);

        Throwable t = new Throwable();
        StackTraceElement[] stackTraceElement = t.getStackTrace();
        for (int i = 0; i < stackTraceElement.length; i++)
        {
            String className = StringUtil
                .getShortClassName(stackTraceElement[i].getClassName());
            if (className.startsWith("Test"))
            {
                className = className.substring(4, Math.min(17, className
                    .length()));
                testName.append(className);
                testName.append("_");
                String methodName = stackTraceElement[i].getMethodName();
                methodName = methodName.substring(0, Math.min(12, methodName
                    .length()));
                testName.append(methodName);
                break;
            }

        }

        return testName.toString();

    }

    /**
     * @return Returns the userName.
     */
    public static String getUserName()
    {
        return userName;
    }

    /**
     * @return Returns the unitTestPrefix.
     */
    public static String getUnitTestPrefix()
    {
        return unitTestPrefix;
    }
}