package com.ibm.rpm.client.junit.resource.fixtures;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author lygagnon
 *  
 */
public class EducationFixture extends RPMObjectFixture 
{
    protected static ArrayList educationSkipSetters, educationSkipGetters;
    
    static
    {
        educationSkipSetters = new ArrayList();
        educationSkipSetters.addAll(skipSetters);
        educationSkipSetters.add("setName");
        educationSkipSetters.add("setAppointedDate");
        educationSkipSetters.add("setCertificationGraduateDate");
        
        educationSkipGetters = new ArrayList();
        educationSkipGetters.addAll(skipGetters);
        educationSkipGetters.add("getName");
        educationSkipGetters.add("getAppointedDate");
        educationSkipGetters.add("getCertificationGraduateDate");
    }
    public static Education createFixture(Resource resource) throws Exception 
    {
        //      Education has no scope
        Education educ = createEducation(new 
                Double(Math.random() * 100).intValue());
        educ.setParent(resource);
        
        return educ;
    }
    
    private static Education createEducation(int index) throws Exception
    {
        Education educ = new Education();
        HelperTestUtilities.setByIntrospection(educ, index, educationSkipSetters);
        educ.setAppointedDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2005-07-06")));
        educ.setCertificationGraduateDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2007-04-08")));
        return educ;
    }
    
    public static void compare(Education e1, Education e2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(e1,e2,educationSkipGetters);
        
        ResourceFixture.compareWithoutCanProperties((Resource) e1.getParent(), (Resource) e2.getParent(), null);
        validateCalendarTypes(e1.getAppointedDate(), e2.getAppointedDate());
        validateCalendarTypes(e1.getCertificationGraduateDate(), e2.getCertificationGraduateDate());
    }
    
    public static void compare(Education[] eArray1, Education[] eArray2) throws AssertionFailedError, Exception
    {
    	sortWithComparator(eArray1, new RPMObjectComparator());
    	sortWithComparator(eArray2, new RPMObjectComparator()); 
        
        RPMObjectFixture.compare(eArray1, eArray2);
        if(eArray1 == null) return;
        
        for(int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
    
    private static void validateCalendarTypes(Calendar c1, Calendar c2)throws AssertionError,
    Exception
    {
        if (c1 != null && c2 != null)
        { 
            Assert.assertEquals(c1.get(Calendar.YEAR), c2.get(Calendar.YEAR));
        	Assert.assertEquals(c1.get(Calendar.DAY_OF_MONTH), c2.get(Calendar.DAY_OF_MONTH));
        	Assert.assertEquals(c1.get(Calendar.MONTH), c2.get(Calendar.MONTH));
        }
        if ( (c1 == null && c2 != null) || (c1 != null && c2 == null))
        {
            Assert.fail();
        }
    }

    protected static class RPMObjectComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            Education obj1 = (Education)o1;
            Education obj2 = (Education)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getOrganization();
            }
            if(obj2 != null){
                key2 = obj2.getOrganization();
            }                
            
            int compare = compareStrings(key1, key2);
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getDegreeProgram();
                }
                if(obj2 != null){
                    key2 = obj2.getDegreeProgram();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();
                }
                if(obj2 != null){
                    key2 = obj2.getID();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    }
}
