package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.Resource;

public class IndustryExperienceFixture extends RPMObjectFixture 
{

    protected static ArrayList industryExperienceSkipSetters, industryExperienceSkipGetters;
    
    static
    {
        industryExperienceSkipSetters = new ArrayList();
        industryExperienceSkipSetters.addAll(skipSetters);
        industryExperienceSkipSetters.add("setName");
        
        industryExperienceSkipGetters = new ArrayList();
        industryExperienceSkipGetters.addAll(skipGetters);
        industryExperienceSkipGetters.add("getName");
    }     
    
    public static IndustryExperience createFixture(Resource resource) throws Exception 
    {
        IndustryExperience educ = createIndustryExperience(new 
                Double(Math.random() * 100).intValue());
        educ.setParent(resource);
        
        return educ;
    }
    
    private static IndustryExperience createIndustryExperience(int index) throws Exception
    {
        IndustryExperience educ = new IndustryExperience();
        HelperTestUtilities.setByIntrospection(educ, index, industryExperienceSkipSetters);
        

        Classification classification = DatafieldFixture.createRandomClassification();
        educ.setClassification(classification);
        
        IndustryType industryType = DatafieldFixture.createRandomIndustryType();
        educ.setIndustry(industryType);
        
        ProficiencyLevel proficiencyLevel = DatafieldFixture.createRandomProficiencyLevel();
        educ.setProficiency(proficiencyLevel);
        
        return educ;
    }
    
    public static void compare(IndustryExperience e1, IndustryExperience e2)
            throws AssertionFailedError, Exception {
        
        RPMObjectFixture.compareWithSkip(e1, e2, industryExperienceSkipGetters);

        DatafieldFixture
                .compare(e1.getClassification(), e2.getClassification());

        DatafieldFixture.compare(e1.getIndustry(), e2.getIndustry());

        DatafieldFixture.compare(e1.getProficiency(), e2.getProficiency());

    }
    
    public static void compare(IndustryExperience[] eArray1, IndustryExperience[] eArray2) throws AssertionFailedError, Exception
    {
    	sortWithComparator(eArray1, new RPMObjectComparator());
    	sortWithComparator(eArray2, new RPMObjectComparator()); 
        
        RPMObjectFixture.compare(eArray1, eArray2);
        if(eArray1 == null) return;
        
        for(int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
    
    protected static class RPMObjectComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            IndustryExperience obj1 = (IndustryExperience)o1;
            IndustryExperience obj2 = (IndustryExperience)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                if(obj1.getIndustry()!=null){
                    key1 = obj1.getClassification().getID();                    
                }
            }
            if(obj2 != null){
                if(obj2.getIndustry()!=null){
                    key2 = obj2.getClassification().getID();                    
                }
            }                
            int compare = compareStrings(key1, key2);
            
            if(compare == 0){
                if(obj1 != null){
                    if(obj1.getIndustry()!=null){
                        key1 = obj1.getIndustry().getID();                    
                    }
                }
                if(obj2 != null){
                    if(obj2.getIndustry()!=null){
                        key2 = obj2.getIndustry().getID();                    
                    }
                }                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    if(obj1.getProficiency()!=null){
                        key1 = obj1.getProficiency().getID();                    
                    }
                }
                if(obj2 != null){
                    if(obj2.getProficiency()!=null){
                        key2 = obj2.getProficiency().getID();                    
                    }
                }
                compare = compareStrings(key1, key2);
            }

            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();                    
                }
                if(obj2 != null){
                    key2 = obj2.getID();     
                }
                compare = compareStrings(key1, key2);
            }
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    }
}
