package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.GregorianCalendar;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.resource.containers.*;

/**
 * @author simon
 */
public class PassportFixture extends RPMObjectFixture 
{
    public static Passport createFixture(Resource resource, Integer nbVisas) throws Exception 
    {
        Passport p;
        if (nbVisas == null)
        {
	        //Passport created with 2 visas by default... if nbVisas == null
	        p = createPassport(resource, new Integer(2));
        }
        else
        {
            p = createPassport(resource, nbVisas);
        }
    	return p;
    }
    
    public static Passport updateFixture(Resource updatedResource, int nbVisas) throws Exception
    {
        //Passport created with nbVisas visas...
        return createPassport(updatedResource, new Integer(nbVisas));
    }
    
    private static Passport createPassport(Resource resource, Integer nbVisas) throws Exception
    {
    	Passport passport = new Passport();
        passport.setHasValidPassport(Boolean.TRUE);         
        passport.setExpiryDate(new GregorianCalendar());
        passport.setParent(resource);
        
        int testNbVisas = nbVisas.intValue();
        
        Visa[] visas = new Visa[testNbVisas];
        for(int i = 0;i < testNbVisas; i++)
        {
            visas[i] = getVisa(passport, 0);
        }    
        passport.setVisas(visas);
        
        return passport;
    }
    
    public static void compare(Passport e1, Passport e2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(e1,e2);
           	
        compare(e1.getVisas(),e2.getVisas());
    }

    
    public static void compare(Visa e1, Visa e2) throws AssertionFailedError, Exception
    {
    	
        RPMObjectFixture.compare(e1,e2);
        
        if(e1 != null && e1.getExpiryDate() != null){
            DatafieldFixture.compare(e1.getCountry(),e2.getCountry());
        }
    }    
    
    public static void compare(Passport[] eArray1, Passport[] eArray2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(eArray1, eArray2);
        if(eArray1 == null) return;
        
        for(int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
    
    public static void compare(Visa[] eArray1, Visa[] eArray2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(eArray1, eArray2);
        if(eArray1 == null) return;
        
        for(int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }    
    
    private static Visa getVisa(Passport parent, int citizenshipCountryCodeNumber)
    {
        Visa visa = new Visa();
        visa.setExpiryDate(new GregorianCalendar());
        visa.setCountry(DatafieldFixture.createCitizenshipCountryCode(String.valueOf(citizenshipCountryCodeNumber)));
        visa.setParent(parent);
        return visa;
    }
}
