package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;


/**
 * @author lygagnon
 *  
 */
public class ProjectHistoryFixture extends RPMObjectFixture 
{
    
    protected static ArrayList projectHistorySkipSetters, projectHistorySkipGetters;
    
    static
    {
        projectHistorySkipSetters = new ArrayList();
        projectHistorySkipSetters.addAll(skipSetters);
        projectHistorySkipSetters.add("setName");
        projectHistorySkipSetters.add("setEndDate");
        
        projectHistorySkipGetters = new ArrayList();
        projectHistorySkipGetters.addAll(skipGetters);
        projectHistorySkipGetters.add("getName");
    }     
    
    
    public static ProjectHistory createFixture(Resource resource) throws Exception 
    {
        //      ProjectHistory has no scope
        ProjectHistory educ = createProjectHistory(new 
                Double(Math.random() * 100).intValue());
        educ.setParent(resource);
        
        return educ;
    }
    
    private static ProjectHistory createProjectHistory(int index)
            throws Exception {
        
        ProjectHistory educ = new ProjectHistory();
        HelperTestUtilities.setByIntrospection(educ, index, projectHistorySkipSetters);

        educ.setEndDate(educ.getStartDate());
        
        Classification classification = DatafieldFixture
                .createRandomClassification();
        educ.setClassification(classification);

        IndustryType industryType = DatafieldFixture
                .createRandomIndustryType();
        educ.setIndustry(industryType);

        return educ;
    }
    
    public static void compare(ProjectHistory e1, ProjectHistory e2)
            throws AssertionFailedError, Exception {
        RPMObjectFixture.compareWithSkip(e1, e2, projectHistorySkipGetters);

        if (e1.getClassification() != null && e2.getClassification() != null)
        {       
             DatafieldFixture.compare(e1.getClassification(), e2.getClassification());
        }

        if (e1.getIndustry() != null && e2.getIndustry() != null)
        {
             DatafieldFixture.compare(e1.getIndustry(), e2.getIndustry());
        }

    }
    
    public static void compare(ProjectHistory[] eArray1, ProjectHistory[] eArray2) throws AssertionFailedError, Exception
    {
    	sortWithComparator(eArray1, new RPMObjectComparator());
    	sortWithComparator(eArray2, new RPMObjectComparator()); 
        
        RPMObjectFixture.compare(eArray1, eArray2);
        if(eArray1 == null) return;
        
        for(int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
    
    protected static class RPMObjectComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            ProjectHistory obj1 = (ProjectHistory)o1;
            ProjectHistory obj2 = (ProjectHistory)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getClient();
            }
            if(obj2 != null){
                key2 = obj2.getClient();
            }                
            
            int compare = compareStrings(key1, key2);
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getProjectName();
                }
                if(obj2 != null){
                    key2 = obj2.getProjectName();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();
                }
                if(obj2 != null){
                    key2 = obj2.getID();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    }
}
