/*
 * Created on Jul 19, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.PreferenceRank;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;
import com.ibm.rpm.wbs.containers.ProfileResourceAttributeAssignment;

/**
 * @author karim
 *  
 */
public class ResourceAttributeAssignmentFixture extends RPMObjectFixture
{
    public static final RPMObject COMPETENCY = new ResourceAttributeCompetencyCategory();
    
    public static ResourceAttributeProficiencyLevel STATIC_COMPETENCY_FIXTURE = null;
    public static ResourceAttributeProficiencyLevel STATIC_SKILL_FIXTURE = null;
    
    private static String XPATH_COMPETENCY = "STATIC_COMPETENCY";
    private static String XPATH_SKILL = "STATIC_SKILL";

    public static final RPMObject SKILL = new ResourceAttributeSkillCategory();

    private static final Integer DEFAULT_FLAG_ON = new Integer(0);
    
    public static ResourceAttributeAssignment createFixture(RPMObject type,
            Resource resource, ClientTestController ctc) throws Exception
    {
        Experience experience = DatafieldFixture.createRandomExperience();
        PreferenceRank preferenceRank = DatafieldFixture
            .createRandomPreferenceRank();
        ResourceAttributeProficiencyLevel proficiencyLevel = createProficiencyLevelStructure(
            type, ctc);
        ResourceRates rates = proficiencyLevel.getRates();

        return createFixture(resource, proficiencyLevel, experience,
            preferenceRank, rates);
    }
    
    public static ResourceAttributeAssignment retrieveResourceAttributeAssignmentFixture(
            RPMObject type, Resource resource, ClientTestController ctc)
            throws Exception
    {
        if (type instanceof ResourceAttributeCompetencyCategory)
        {
            if (STATIC_COMPETENCY_FIXTURE == null)
            {
                STATIC_COMPETENCY_FIXTURE = loadProficiencyStructFixture(type,
                    XPATH_COMPETENCY, ctc);
            }

        }
        Experience experience = DatafieldFixture.createRandomExperience();
        PreferenceRank preferenceRank = DatafieldFixture
            .createRandomPreferenceRank();
        ResourceRates rates = STATIC_COMPETENCY_FIXTURE.getRates();
        return createFixture(resource, STATIC_COMPETENCY_FIXTURE, experience,
            preferenceRank, rates);
    }

    public static ResourceAttributeAssignment createFixture(Resource resource,
            ResourceAttributeProficiencyLevel proficiencyLevel,
            Experience experience, PreferenceRank preferenceRank,
            ResourceRates rates) throws Exception
    {
        if (resource != null && proficiencyLevel != null && experience != null
                && preferenceRank != null)
        {
            ResourceAttributeAssignment assignment = createAssignmentFixture(
                resource, proficiencyLevel, experience, preferenceRank, rates);
            return assignment;
        }
        return null;
    }

    public static ResourceAttributeAssignment[] createFixtureFromProfile(
            ProfileResourceAttributeAssignment[] profAssignments)
    {
        if (profAssignments == null)
            return null;
        ResourceAttributeAssignment[] assignments = new ResourceAttributeAssignment[profAssignments.length];
        for (int i = 0; i < profAssignments.length; i++)
        {
            ResourceAttributeAssignment attAss = new ResourceAttributeAssignment();
            ProfileResourceAttributeAssignment profAss = profAssignments[i];
            attAss.setProficiencyLevel(profAss.getProficiencyLevel());
            assignments[i] = attAss;
        }
        return assignments;
    }

    public static ResourceAttributeProficiencyLevel retrieveProficiencyStructStaticFixture(
            RPMObject type, ClientTestController ctc) throws Exception
    {
        if (type instanceof ResourceAttributeCompetencyCategory)
        {
            if (STATIC_COMPETENCY_FIXTURE == null)
            {
                STATIC_COMPETENCY_FIXTURE = loadProficiencyStructFixture(type,
                    XPATH_COMPETENCY, ctc);
            }
            return STATIC_COMPETENCY_FIXTURE;
        }
        else if (type instanceof ResourceAttributeSkillCategory)
        {
            if (STATIC_SKILL_FIXTURE == null)
            {
                STATIC_SKILL_FIXTURE = loadProficiencyStructFixture(type,
                    XPATH_SKILL, ctc);
            }
            return STATIC_SKILL_FIXTURE;
        }
        else
        {
            return null;
        }
    }
    
    private static ResourceAttributeProficiencyLevel loadProficiencyStructFixture(
            RPMObject type, String xpathName, ClientTestController ctc)
            throws Exception
    {
        RPMObject[] rpmObjects = null;
        ResourceAttributeProficiencyLevel profLevel = null;
        
        ResourceAttributeScope scope = new ResourceAttributeScope();
        scope.setClassifications(scope);
        scope.setCategories(scope);
        scope.setProficiencyLevels(true);
        scope.setAttributes(scope);
        
        if (type instanceof ResourceAttributeCompetencyCategory)
        {
            rpmObjects = ctc
                .loadFromXpath("/" + StringUtil
                                .getShortClassName(ResourceAttributeCompetencyCategory.class)
                            + "[name='" + xpathName + "']", scope);

            ResourceAttributeCompetencyCategory category = null;

            if (rpmObjects != null && rpmObjects.length > 0
                    && rpmObjects[0] != null)
            {
                category = (ResourceAttributeCompetencyCategory) rpmObjects[0];
                
                if (category.getClassifications() != null
                        && category.getClassifications()[0] != null
                        && category.getClassifications()[0].getAttributes() != null
                        && category.getClassifications()[0].getAttributes()[0] != null
                        && category.getClassifications()[0].getAttributes()[0]
                            .getProficiencyLevels() != null)
                {
                    profLevel = category.getClassifications()[0]
                        .getAttributes()[0].getProficiencyLevels()[0];
                }
                else
                {
                    // if the static fixture has no proficiency level, delete it
                    // and create a new one
                    ctc.delete(category);
                }
            }
            
            if (profLevel == null)
            {
                ResourceAttributeScope rAttrScope = CompetencyFixture
                    .createScope(1, 1, 3);
                category = CompetencyFixture.createFixture(1, 1, 3, rAttrScope, ctc);
                category.setName(xpathName);
                category = (ResourceAttributeCompetencyCategory) saveWithoutDelete(
                    ctc, category, rAttrScope);

                profLevel = category.getClassifications()[0].getAttributes()[0]
                    .getProficiencyLevels()[0];
            }
        }
        else if (type instanceof ResourceAttributeSkillCategory)
        {
            rpmObjects = ctc
                .loadFromXpath(
                    "/"
                            + StringUtil
                                .getShortClassName(ResourceAttributeSkillCategory.class)
                            + "[name='" + xpathName + "']", scope);

            ResourceAttributeSkillCategory category = null;

            if (rpmObjects != null && rpmObjects.length > 0
                    && rpmObjects[0] != null)
            {
                category = (ResourceAttributeSkillCategory) rpmObjects[0];

                if (category.getClassifications() != null
                        && category.getClassifications()[0] != null
                        && category.getClassifications()[0].getAttributes() != null
                        && category.getClassifications()[0].getAttributes()[0] != null
                        && category.getClassifications()[0].getAttributes()[0]
                            .getProficiencyLevels() != null)
                {
                    profLevel = category.getClassifications()[0]
                        .getAttributes()[0].getProficiencyLevels()[0];
                }
                else
                {
                    // if the static fixture has no proficiency level, delete it
                    // and create a new one
                    ctc.delete(category);
                }
            }

            if (profLevel == null)
            {
                ResourceAttributeScope rAttrScope = SkillFixture.createScope(1,
                    1, 3);
                category = SkillFixture.createFixture(1, 1, 3, rAttrScope, ctc);
                category.setName(xpathName);
                category = (ResourceAttributeSkillCategory) saveWithoutDelete(
                    ctc, category, rAttrScope);

                profLevel = category.getClassifications()[0].getAttributes()[0]
                    .getProficiencyLevels()[0];
            }
        }
        return profLevel;
    }

    public static ResourceAttributeProficiencyLevel createProficiencyLevelStructure(
            RPMObject type, ClientTestController ctc) throws Exception
    {
        ResourceAttributeProficiencyLevel proficiencyLevel = null;
        if (type instanceof ResourceAttributeCompetencyCategory)
        {
            ResourceAttributeCompetencyCategory category = CompetencyFixture
                .createSaveFixture(ctc);
            proficiencyLevel = category.getClassifications()[0]
                .getAttributes()[0].getProficiencyLevels()[0];
        }
        else if (type instanceof ResourceAttributeSkillCategory)
        {
            ResourceAttributeSkillCategory category = SkillFixture
                .createSaveFixture(ctc);
            proficiencyLevel = category.getClassifications()[0]
                .getAttributes()[0].getProficiencyLevels()[0];
        }
        return proficiencyLevel;
    }

    private static ResourceAttributeAssignment createAssignmentFixture(
            Resource resource,
            ResourceAttributeProficiencyLevel proficiencyLevel,
            Experience experience, PreferenceRank preferenceRank,
            ResourceRates rates) throws Exception
    {
        return createAssignment(resource, proficiencyLevel, experience,
            preferenceRank, rates, random());
    }

    private static ResourceAttributeAssignment createAssignment(
            Resource resource,
            ResourceAttributeProficiencyLevel proficiencyLevel,
            Experience experience, PreferenceRank preferenceRank,
            ResourceRates rates, int index) throws Exception
    {
        ResourceAttributeAssignment assignment = new ResourceAttributeAssignment();

        assignment.setParent(resource);

        assignment.setExperience(experience);

        assignment.setPreferenceRank(preferenceRank);

        assignment.setProficiencyLevel(proficiencyLevel);

        setRates(assignment, rates);

        assignment.setStrategicFlag("0");

        assignment.setDefaultFlag(DEFAULT_FLAG_ON);

        return assignment;
    }

    public static void updateRates(ResourceAttributeAssignment assignment)
    {
        ResourceRates rates = assignment.getRates();
        rates.setRegularCostPerHour(updateRate(rates.getRegularCostPerHour()));
        rates.setSellingRatePerHour(updateRate(rates.getSellingRatePerHour()));
        rates.setSpecialCostPerHour(updateRate(rates.getSpecialCostPerHour()));
        rates.setSpecialSellingRatePerHour(updateRate(rates
            .getSpecialSellingRatePerHour()));
    }

    private static Double updateRate(Double oldRate)
    {
        return new Double(oldRate.doubleValue() + 0.99);
    }

    //Compare section
    public static void compare(ResourceAttributeAssignment[] array1,
            ResourceAttributeAssignment[] array2) throws AssertionFailedError,
            Exception
    {
    	sortWithComparator(array1, RPMObjectIDComparator.INSTANCE);
    	sortWithComparator(array2, RPMObjectIDComparator.INSTANCE);

        RPMObjectFixture.compare(array1, array2);

        for (int i = 0; i < array1.length; i++)
        {
            compare(array1[i], array2[i]);
        }
    }

    public static void compare(ResourceAttributeAssignment obj1,
            ResourceAttributeAssignment obj2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(obj1, obj2);
        DatafieldFixture.compare(obj1.getExperience(), obj2.getExperience());
        DatafieldFixture.compare(obj1.getPreferenceRank(), obj2
            .getPreferenceRank());
        RPMObjectFixture.compare(obj1.getProficiencyLevel(), obj2
            .getProficiencyLevel());
        compareRates(obj1.getRates(), obj2.getRates());
    }

    public static void compareRates(ResourceRates obj1, ResourceRates obj2)
            throws AssertionFailedError, Exception
    {

        //here we don't call the RPMObjectFixture.compare because it compares
        // the IDs
        //and in this case, the SP generates a new ID for the updated rates.
        Assert.assertNotNull(obj1);
        Assert.assertNotNull(obj2);
        HelperTestUtilities
            .assertEqualsByIntrospection(obj1, obj2, skipGetters);
    }

    private static void setRates(ResourceAttributeAssignment assignment,
            ResourceRates rates)
    {
        if (rates != null)
        {
            ResourceRates assignmentRates = new ResourceRates();
            setRates(assignmentRates, rates);
            assignment.setRates(assignmentRates);
        }
    }

    private static void setRates(ResourceRates assignmentRates,
            ResourceRates proficiencyLevelRates)
    {
        assignmentRates.setID(proficiencyLevelRates.getID());
        assignmentRates.setParent(proficiencyLevelRates.getParent());
        assignmentRates.setRegularCostPerHour(proficiencyLevelRates
            .getRegularCostPerHour());
        assignmentRates.setSellingRatePerHour(proficiencyLevelRates
            .getSellingRatePerHour());
        assignmentRates.setSpecialCostPerHour(proficiencyLevelRates
            .getSpecialCostPerHour());
        assignmentRates.setSpecialSellingRatePerHour(proficiencyLevelRates
            .getSpecialSellingRatePerHour());
    }

}
