/*
 * Created on Jun 29, 2005
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.EmployeeStatus;
import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkingStatus;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;


/**
 * @author gosselin
 */
public class ResourceFixture extends RPMObjectFixture
{
    protected static Resource STATIC_RESOURCE_FIXTURE = null;

    protected static String XPATH_RESOURCE_NAME = "STATIC_RESOURCE_FIXTURE";

    public static String STATIC_RESOURCE_PASSWORD = "rpm";
    
    protected static ArrayList resourceSkipSetters, resourceSkipGetters;

    static
    {
        //Setters
        resourceSkipSetters = new ArrayList();
        resourceSkipSetters.addAll(skipSetters);
        resourceSkipSetters.add("setName");
        resourceSkipSetters.add("setUserName");
        resourceSkipSetters.add("setPassword");
        resourceSkipSetters.add("setDiscontinueDate");
        resourceSkipSetters.add("setReVerified");
        resourceSkipSetters.add("setReVerifiedBy");
        resourceSkipSetters.add("setVerified");
        resourceSkipSetters.add("setVerifiedBy");

        //Getters
        resourceSkipGetters = new ArrayList();
        resourceSkipGetters.addAll(skipGetters);
        resourceSkipGetters.add("getName");
        resourceSkipGetters.add("getUserName");
        resourceSkipGetters.add("getPassword");
    }

    /*
     * Use this method to create a Resource that will not be deleted.
     */
    public static Resource retrieveStaticFixture(ClientTestController ctc)
            throws Exception
    {
        if (STATIC_RESOURCE_FIXTURE == null)
        {
            STATIC_RESOURCE_FIXTURE = loadOrCreateStaticFixture(ctc,
                XPATH_RESOURCE_NAME);
        }
        return STATIC_RESOURCE_FIXTURE;
    }
    
    public static Resource loadStaticFixture(ClientTestController ctc,
            ResourceScope scope) throws Exception
    {
        RPMObject[] rpmObjects = null;
        Resource resource = null;
        String xpathQuery = "/Resource[username='" + XPATH_RESOURCE_NAME + "']";
        rpmObjects = ctc.loadFromXpath(xpathQuery, scope);

        if (rpmObjects != null && rpmObjects.length > 0
                && rpmObjects[0] != null)
        {
            resource = (Resource) rpmObjects[0];
            if (ctc.isCheckedOut(resource))
            {
                forceCheckinStaticFixture(resource, ctc);
            }
        }
        return resource;
    }
    
    public static Resource loadOrCreateStaticFixture(ClientTestController ctc,
            String xpathName) throws Exception
    {
        RPMObject[] rpmObjects = null;
        Resource resource = null;
        String xpathQuery = "/Resource[username='" + xpathName + "']";
        rpmObjects = ctc.loadFromXpath(xpathQuery, new ResourceScope());
        
        if(rpmObjects != null && rpmObjects.length > 0 && rpmObjects[0] != null)
        {
            resource = (Resource)rpmObjects[0];
            if(ctc.isCheckedOut(resource))
            {
                forceCheckinStaticFixture(resource, ctc);
            }
        }
        else
        {
            //Create the Pool and Resource
            Pool pool = PoolFixture.retrieveStaticFixture(ctc);
            resource = createFixture(pool, null, ctc);
            resource.setFullName(XPATH_RESOURCE_NAME);
            resource.setUserName(XPATH_RESOURCE_NAME);
            resource.setParent(pool);
            resource.setActive(new Boolean(true));
            resource.setCalendar(RPMCalendarFixture
                .retrieveStaticFixtureForInserts(ctc));  
            resource.setPassword("ibm");
            resource.getResourceSecurityRights().setCanUseWebServicesAPI(
                Boolean.TRUE);
            ResourceScope scope = new ResourceScope();
            scope.setCalendar(new CalendarScope());
            scope.setResourceSecurityRights(true);
            resource = (Resource)saveWithoutDelete(ctc, resource, scope);
            
            // Changing resource passwords to avoid the password expired
            //error message on the first logon.
            //WARNING: don't change this password since the
            //rpm-webui-functional-test projects depends on it.
            ctc.changePassword(resource.getUserName(), "ibm",
                STATIC_RESOURCE_PASSWORD);
            resource.setPassword(STATIC_RESOURCE_PASSWORD);
        }
        return resource;
    }
    
    protected static void forceCheckinStaticFixture(Resource resource,
            ClientTestController ctc) throws Exception
    {
        // best effort to login with a user a force checkin the resource
        AlreadyCheckedOutResult checkedOut = ctc.getApplicationInterface()
            .isCheckedOut(ctc.getSessionID(), resource);
        Resource checkedOutUser = checkedOut.getCheckedOutBy();
        if (checkedOutUser != null
                && checkedOutUser.getUserName().equals("administrator"))
        {
            ctc.forceCheckinOnRPMObjectCheckOut(resource);
        }
        else
        {
            String sessionid = null;
            try
            {
                // hard coded password
                sessionid = ctc.getAuthenticateInterface().login(
                    checkedOutUser.getUserName(), "rpm", ClientTestController.DSN);
                ctc.getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
                    sessionid, resource);
            }
            finally
            {
                if (sessionid != null)
                {
                    ctc.getAuthenticateInterface().logout(sessionid);
                }
            }
        }
    }
        
    
    /**
     * @param scope
     * @param clientTestController
     * @return
     * @throws Exception
     */
    public static Resource createFixture(Pool pool, ResourceScope scope,
            ClientTestController ctc) throws Exception
    {
        return ResourceFixture.createFixture(pool, 1, scope, ctc);
    }

    public static Resource createFixture(Pool pool, int nbChildren,
            ResourceScope scope, ClientTestController ctc) throws Exception
    {
        Resource res = new Resource();
        
        HelperTestUtilities.setByIntrospection(res, new Double(
            Math.random() * 100).intValue(), resourceSkipSetters);

        res.setActive(new Boolean(true));
        
        //      TestProfile a besoin d'une date avec une annee sous 2011 sinon
        // l'entree
        //dans la table CALENDAR_DAY_OF_YEAR n'est pas creee !!
        // discontinueDate >= effectiveStartDate
        Calendar calendar = Calendar.getInstance();
        calendar.set(2006, 2, 8);
        res.setEffectiveStartDate(calendar);

        //ResourceSecurityRights
        ResourceSecurityRights resourceSecurityRights = new ResourceSecurityRights();
        res.setResourceSecurityRights(resourceSecurityRights);

        res.setUserName("UserName" + Math.random());
        res.setCountryCode("code1");
        res.setCompanyCode("code2");
        res.setDivisionCode("code3");
        res.setEmployeeCode("code4");
        res.setDepartmentCode("code5");

        //My Settings
        res.setOverrideOriginalDocumentOnCheckout(Boolean.FALSE);
        
        res.setMyPortalURL1("aaa");
        res.setMyPortalURL2("bbb");
        res.setMyPortalURL3("ccc");
        res.setMyPortalURL4("ddd");
        res.setMyPortalURL5("eee");
        
        res.setDashboard("aaa");

        //Assigned parent
        res.setParent(pool);

        if (scope != null)
        {

            // ClientCostCenter
            if (scope.isResourceCostCenterAssignments())
            {
                ResourceCostCenterAssignment[] rccA = new ResourceCostCenterAssignment[nbChildren + 1];

                for (int i = 0; i < nbChildren; i++)
                {
                    ResourceCostCenterAssignment rcc = ResourceCostCentersFixture
                        .createFixture(res, ctc);
                    rcc.setParent(res);
                    rccA[i] = rcc;
                }

                //Insert a null object
                rccA[nbChildren] = null;

                res.setResourceCostCenterAssignments(rccA);
            }

            //        TODO Create DocumentFolder
            //        TODO Create ExecutiveRole
            //        TODO Create Portfolio
            //        TODO Create ProjectSecurityRole

            //Resource rates
            if (scope.isRates())
            {
                res.setRates(createRates());
                res.getRates().setParent(res);
            }

            //Currency
            if (scope.isCurrency())
            {
                Currency currency = DatafieldFixture.createRandomCurrency();
                res.setCurrency(currency);
            }

            //        Education
            if (scope.isEducations())
            {

                Education[] rArray = new Education[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    Education e = EducationFixture.createFixture(res);
                    e.setParent(res);
                    rArray[i] = e;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setEducations(rArray);
            }

            //        EmployeeStatus
            if (scope.isEmployeeStatus())
            {
                EmployeeStatus es = DatafieldFixture
                    .createRandomEmployeeStatus();
                res.setEmployeeStatus(es);
            }

            //        EmploymentHistory
            if (scope.isEmploymentHistories())
            {

                EmploymentHistory[] rArray = new EmploymentHistory[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    EmploymentHistory e = EmploymentHistoryFixture
                        .createFixture(res);
                    e.setParent(res);
                    rArray[i] = e;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setEmploymentHistories(rArray);
            }

            //        Experience
            if (scope.isExperience())
            {
                Experience ex = DatafieldFixture.createRandomExperience();
                res.setExperience(ex);
            }

            //        IndustryExperience
            if (scope.isIndustryExperiences())
            {
                IndustryExperience[] rArray = new IndustryExperience[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    IndustryExperience ie = IndustryExperienceFixture
                        .createFixture(res);
                    rArray[i] = ie;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setIndustryExperiences(rArray);
            }

            //        LanguageProficiency
            if (scope.isLanguageProficiencies())
            {

                LanguageProficiency[] rArray = new LanguageProficiency[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    LanguageProficiency e = LanguageProficiencyFixture
                        .createFixture(res);
                    e.setParent(res);
                    
                    // Only one language can be set to default, let's always make it the first one
                    if (i == 0)
                        e.setDefaultLanguage(Boolean.TRUE);
                    else
                        e.setDefaultLanguage(Boolean.FALSE);
                    
                    rArray[i] = e;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setLanguageProficiencies(rArray);
            }

            //        LocationPreferences
            if (scope.isLocationPreferences())
            {
                LocationPreferences location = LocationPreferencesFixture
                    .createFixture(res);
                res.setLocationPreferences(location);
                res.getLocationPreferences().setParent(res);
            }

            //        Passport
            if (scope.isPassport())
            {
                Passport passport = PassportFixture.createFixture(res, null);
                res.setPassport(passport);
            }

            //        ProjectHistory
            if (scope.isProjectHistories())
            {
                ProjectHistory[] rArray = new ProjectHistory[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    ProjectHistory ie = ProjectHistoryFixture
                        .createFixture(res);
                    rArray[i] = ie;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setProjectHistories(rArray);
            }

            //        WBSAttributes
            if (scope.isExecutiveAttributes())
            {
                AttributeAssignment[] newArray = new AttributeAssignment[nbChildren + 1];

                for (int i = 0; i < nbChildren; i++)
                {

                    AttributeAssignment ea = AttributeAssignmentFixture
                        .createAssignmentFixture(res, ctc);
                    newArray[i] = ea;
                }

                //Insert a null object
                newArray[nbChildren] = null;

                res.setExecutiveAttributes(newArray);
            }

            //        WBSAttributesClassification
            if (scope.isExecutiveAttributeClassifications())
            {
                AttributeAssignment[] newArray = new AttributeAssignment[nbChildren + 1];

                for (int i = 0; i < nbChildren; i++)
                {

                    AttributeAssignment ea = AttributeAssignmentFixture
                        .createWBSClassificationAssignmentFixture(res, ctc);
                    newArray[i] = ea;
                }

                //Insert a null object
                newArray[nbChildren] = null;

                res.setExecutiveAttributeClassifications(newArray);
            }

            //        WorkingStatus
            if (scope.isWorkingStatus())
            {
                WorkingStatus ws = DatafieldFixture.createRandomWorkingStatus();
                res.setWorkingStatus(ws);
            }

            //        WorkLocation
            if (scope.isWorkLocation())
            {
                WorkLocation wl = DatafieldFixture.createRandomWorkLocation();
                res.setWorkLocation(wl);
            }

            //        SkillAssignment
            if (scope.isSkillAttributeAssignments())
            {
                ResourceAttributeAssignment[] rArray = new ResourceAttributeAssignment[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    ResourceAttributeAssignment r = ResourceAttributeAssignmentFixture
                        .createFixture(
                            ResourceAttributeAssignmentFixture.SKILL, res, ctc);
                    r.setParent(res);
                    rArray[i] = r;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setSkillAttributeAssignments(rArray);
            }

            //        CompetenciesAssignment
            if (scope.isCompetencyAttributeAssignments())
            {
                //ResourceAttributeAssignment[] rArray = new ResourceAttributeAssignment[nbChildren + 1];
                // The static fixture only has one proficiency level and we cannot assign it twice.
                ResourceAttributeAssignment[] rArray = new ResourceAttributeAssignment[2];
                for (int i = 0; i < nbChildren; i++)
                {
                    ResourceAttributeAssignment r = ResourceAttributeAssignmentFixture
                    .retrieveResourceAttributeAssignmentFixture(
                            ResourceAttributeAssignmentFixture.COMPETENCY, res,
                            ctc);
                    r.setParent(res);
                    rArray[i] = r;
                }

                //Insert a null object
                //rArray[nbChildren] = null;
                rArray[1] = null;

                res.setCompetencyAttributeAssignments(rArray);
            }

            //        ContactGroup
            if (scope.isContactGroupAssignments())
            {

                ContactGroupAssignment[] rArray = new ContactGroupAssignment[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    ContactGroupAssignment r = ResourceContactGroupAssignmentFixture
                        .createAssignmentFixture(res, ctc);
                    rArray[i] = r;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                res.setContactGroupAssignments(rArray);
            }

            //        Geographical
            if (scope.isGeographicalAssignment())
            {
                GeographicalAssignment r = ResourceGeographicalAssignmentFixture
                    .createFixture(res, ctc);
                res.setGeographicalAssignment(r);
            }

            //        Organizational
            if (scope.isOrganizationalAssignment())
            {
                OrganizationalAssignment r = ResourceOrganizationalAssignmentFixture
                    .createFixture(res, ctc);
                res.setOrganizationalAssignment(r);
            }

            //        RPMCalendar
            if (scope.getCalendar() != null)
            {
                RPMCalendar cal = RPMCalendarFixture
                    .retrieveStaticFixtureForInserts(ctc);
                res.setCalendar(cal);
            }

            //Document folder
            if (scope.getDocumentFolder() != null)
            {
                DocumentFolder documentFolder = (DocumentFolder) DocumentElementFixture
                    .createFixture(res, new DocumentFolder(), 1, scope
                        .getDocumentFolder(), ctc);
                res.setDocumentFolder(documentFolder);
            }
                        
            if (scope.getPersonalProject() != null)
            {
                //do nothing, this object is created automatically with the resource
            }
            
            if (scope.isVerifiedBy())
            {
                ResourceScope nestedScope = new ResourceScope();
                nestedScope.setCalendar(new CalendarScope());
                res.setVerifiedBy(createSaveFixture(nestedScope, ctc));
            }
            
            if (scope.isReVerifiedBy())
            {

                ResourceScope nestedScope = new ResourceScope();
                nestedScope.setCalendar(new CalendarScope());
                res.setReVerifiedBy(createSaveFixture(nestedScope, ctc));
            }
        }
        return res;
    }

    public static Resource createSaveFixture(ClientTestController ctc)
            throws Exception
    {
        return createSaveFixture(null, ctc);
    }

    public static Resource createSaveFixture(ResourceScope scope,
            ClientTestController ctc) throws Exception
    {

        Pool pool = PoolFixture.retrieveStaticFixture(ctc);
        Resource resource = ResourceFixture.createFixture(pool, scope, ctc);

        if (scope == null)
        {
            scope = new ResourceScope();
        }
        scope.setParent(new PoolScope());

        resource = (Resource) save(ctc, resource, scope);

        return resource;
    }

    /**
     * @param scope
     * @param clientTestController
     * @return
     * @throws Exception
     */
    public static Resource updateFixture(Resource resource, int nbChildren,
            ClientTestController ctc) throws Exception
    {

        Resource updatedResource = new Resource();
        HelperTestUtilities.setByIntrospection(updatedResource, new Double(Math
            .random() * 100).intValue(), resourceSkipSetters);

        //Keep original ID
        updatedResource.setID(resource.getID());

        //TODO Remove after SP_UA_RES_CONTACTS is modified to use resource_Id
        //instead of nickname
        //Keep original nickname because contact groups can't be delete
        //if the nickname has changed
        updatedResource.setNickName(resource.getNickName());

        // Cannot clear password
        updatedResource.setPassword(resource.getPassword());
        
        updatedResource.setVerifiedBy(null);
        updatedResource.setReVerifiedBy(null);
        //        

        updatedResource.setUserName("UserName" + Math.random());
        updatedResource.setCountryCode("code2");
        updatedResource.setCompanyCode("code3");
        updatedResource.setDivisionCode("code4");
        updatedResource.setEmployeeCode("code5");
        updatedResource.setDepartmentCode("code6");

        //My Settings
        updatedResource.setOverrideOriginalDocumentOnCheckout(Boolean.FALSE);
        
        updatedResource.setMyPortalURL1("eee");
        updatedResource.setMyPortalURL2("aaa");
        updatedResource.setMyPortalURL3("bbb");
        updatedResource.setMyPortalURL4("ccc");
        updatedResource.setMyPortalURL5("ddd");
        
        updatedResource.setDashboard("bbb");

        //Update Parent
        Pool parent = PoolFixture.createSaveFixture(ctc);
        updatedResource.setParent(parent);

        // ClientCostCenter
        if (resource.getResourceCostCenterAssignments() != null)
        {

            ResourceCostCenterAssignment[] actuals = resource
                .getResourceCostCenterAssignments();
            ResourceCostCenterAssignment[] newArray = new ResourceCostCenterAssignment[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                ResourceCostCenterAssignment newRcc = ResourceCostCentersFixture
                    .createFixture(updatedResource, ctc);
                newRcc.setParent(updatedResource);
                newArray[nbChildren + i] = newRcc;
            }
            updatedResource.setResourceCostCenterAssignments(newArray);
        }

        //        TODO Update DocumentFolder
        //        TODO Update Portfolio
        //        TODO Update ProjectSecurityRole
        //        TODO Update ExecutiveRole

        //		Rates and Currency
        if (resource.getRates() != null || resource.getCurrency() != null)
        {

            updatedResource.setRates(createRates());
            updatedResource.getRates().setParent(updatedResource);

            Currency currency = DatafieldFixture.createRandomCurrency();
            updatedResource.setCurrency(currency);

        }

        //        Update Education
        if (resource.getEducations() != null)
        {

            Education[] actuals = resource.getEducations();
            Education[] newArray = new Education[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                Education e = EducationFixture.createFixture(updatedResource);
                newArray[nbChildren + i] = e;
            }
            updatedResource.setEducations(newArray);
        }

        //        Update EmployeeStatus
        if (resource.getEmployeeStatus() != null)
        {
            EmployeeStatus es = DatafieldFixture.createRandomEmployeeStatus();
            updatedResource.setEmployeeStatus(es);
        }

        //        Update EmploymentHistory
        if (resource.getEmploymentHistories() != null)
        {

            EmploymentHistory[] actuals = resource.getEmploymentHistories();
            EmploymentHistory[] newArray = new EmploymentHistory[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                EmploymentHistory eh = EmploymentHistoryFixture
                    .createFixture(updatedResource);
                newArray[nbChildren + i] = eh;
            }
            updatedResource.setEmploymentHistories(newArray);
        }

        //        Update Experience
        if (resource.getExperience() != null)
        {
            Experience ex = DatafieldFixture.createRandomExperience();
            updatedResource.setExperience(ex);
        }

        //        Update IndustryExperience
        if (resource.getIndustryExperiences() != null)
        {

            IndustryExperience[] actuals = resource.getIndustryExperiences();
            IndustryExperience[] newArray = new IndustryExperience[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                IndustryExperience ie = IndustryExperienceFixture
                    .createFixture(updatedResource);
                newArray[nbChildren + i] = ie;
            }

            updatedResource.setIndustryExperiences(newArray);
        }

        //        Update LanguageProficiency
        if (resource.getLanguageProficiencies() != null)
        {

            LanguageProficiency[] actuals = resource.getLanguageProficiencies();
            LanguageProficiency[] newArray = new LanguageProficiency[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                LanguageProficiency lp = LanguageProficiencyFixture
                    .createFixture(updatedResource);
                // Only one language can be set to default, let's always make it the first one
                if (i == 0)
                    lp.setDefaultLanguage(Boolean.TRUE);
                else
                    lp.setDefaultLanguage(Boolean.FALSE);
                newArray[nbChildren + i] = lp;
            }

            updatedResource.setLanguageProficiencies(newArray);
        }

        //        Update LocationPreferences
        if (resource.getLocationPreferences() != null)
        {
            LocationPreferences location = LocationPreferencesFixture
                .createFixture(updatedResource);
            updatedResource.setLocationPreferences(location);
        }

        //        Update Passport
        if (resource.getPassport() != null)
        {
            Passport passport = PassportFixture.updateFixture(updatedResource,
                3);
            updatedResource.setPassport(passport);
        }

        //        Update ProjectHistory
        if (resource.getProjectHistories() != null)
        {
            ProjectHistory[] rArray = new ProjectHistory[nbChildren];
            for (int i = 0; i < nbChildren; i++)
            {
                ProjectHistory ie = ProjectHistoryFixture
                    .createFixture(updatedResource);
                rArray[i] = ie;
            }
            updatedResource.setProjectHistories(rArray);
        }

        //        Update WBSAttributes
        if (resource.getExecutiveAttributes() != null)
        {

            AttributeAssignment[] actuals = resource.getExecutiveAttributes();
            AttributeAssignment[] newArray = new AttributeAssignment[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                AttributeAssignment ea = AttributeAssignmentFixture
                    .createAssignmentFixture(updatedResource, ctc);
                newArray[nbChildren + i] = ea;
            }

            updatedResource.setExecutiveAttributes(newArray);
        }

        //        Update WBSAttributesClassification
        if (resource.getExecutiveAttributeClassifications() != null)
        {

            AttributeAssignment[] actuals = resource
                .getExecutiveAttributeClassifications();
            AttributeAssignment[] newArray = new AttributeAssignment[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                AttributeAssignment ea = AttributeAssignmentFixture
                    .createWBSClassificationAssignmentFixture(updatedResource,
                        ctc);

                newArray[nbChildren + i] = ea;
            }

            updatedResource.setExecutiveAttributeClassifications(newArray);
        }

        //        Update WorkingStatus
        if (resource.getWorkingStatus() != null)
        {
            WorkingStatus ws = DatafieldFixture.createRandomWorkingStatus();
            updatedResource.setWorkingStatus(ws);
        }

        //        Update WorkLocation
        if (resource.getWorkLocation() != null)
        {
            WorkLocation wl = DatafieldFixture.createRandomWorkLocation();
            updatedResource.setWorkLocation(wl);
        }

        //        Update SkillAssignment
        if (resource.getSkillAttributeAssignments() != null)
        {

            ResourceAttributeAssignment[] actuals = resource
                .getSkillAttributeAssignments();
            ResourceAttributeAssignment[] newArray = new ResourceAttributeAssignment[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                ResourceAttributeAssignment s = ResourceAttributeAssignmentFixture
                    .createFixture(ResourceAttributeAssignmentFixture.SKILL,
                        updatedResource, ctc);
                newArray[nbChildren + i] = s;
            }

            updatedResource.setSkillAttributeAssignments(newArray);
        }

        //        Update CompetenciesAssignment
        if (resource.getCompetencyAttributeAssignments() != null)
        {

            ResourceAttributeAssignment[] actuals = resource
                .getCompetencyAttributeAssignments();
            //ResourceAttributeAssignment[] newArray = new ResourceAttributeAssignment[nbChildren * 2];
            ResourceAttributeAssignment[] newArray = new ResourceAttributeAssignment[1 * 2];

            for (int i = 0; i < 1; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                ResourceAttributeAssignment s = ResourceAttributeAssignmentFixture
                    .createFixture(
                        ResourceAttributeAssignmentFixture.COMPETENCY,
                        updatedResource, ctc);
                newArray[1 + i] = s;
            }
            updatedResource.setCompetencyAttributeAssignments(newArray);
        }

        //        Update ContactGroup
        if (resource.getContactGroupAssignments() != null)
        {

            ContactGroupAssignment[] actuals = resource
                .getContactGroupAssignments();
            ContactGroupAssignment[] newArray = new ContactGroupAssignment[nbChildren * 2];

            for (int i = 0; i < nbChildren; i++)
            {

                //Remove old containers
                actuals[i].setDeleted(Boolean.TRUE);
                newArray[i] = actuals[i];

                //Create new containers
                ContactGroupAssignment cg = ResourceContactGroupAssignmentFixture
                    .createAssignmentFixture(updatedResource, ctc);
                newArray[nbChildren + i] = cg;
            }

            updatedResource.setContactGroupAssignments(newArray);
        }

        //        Update Geographical
        if (resource.getGeographicalAssignment() != null)
        {
            GeographicalAssignment r = ResourceGeographicalAssignmentFixture
                .createFixture(updatedResource, ctc);
            updatedResource.setGeographicalAssignment(r);
        }

        //        Update Organizational
        if (resource.getOrganizationalAssignment() != null)
        {
            OrganizationalAssignment r = ResourceOrganizationalAssignmentFixture
                .createFixture(updatedResource, ctc);
            updatedResource.setOrganizationalAssignment(r);
        }

        //        Update RPMCalendar
        if (resource.getCalendar() != null)
        {
            RPMCalendar cal = RPMCalendarFixture
                .retrieveStaticFixtureForUpdates(ctc);
            updatedResource.setCalendar(cal);
        }
        
        if (resource.getPersonalProject() != null)
        {

            //do nothing. fields cannot be updated.
        }

        return updatedResource;
    }

    public static void compare(Resource r1, Resource r2, ResourceScope scope)
            throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compareWithSkip(r1, r2, resourceSkipGetters);

        compareWithScope(r1, r2, scope);
    }

    public static void compareWithoutCanProperties(Resource r1, Resource r2,
            ResourceScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(r1, r2, resourceSkipGetters,
            new String[]
            {
                "getCan.*"
            });

        compareWithScope(r1, r2, scope);
    }

    private static void compareWithScope(Resource r1, Resource r2,
            ResourceScope scope) throws AssertionFailedError, Exception
    {

        if (scope == null)
            return;

        // ClientCostCenter
        if (scope.isResourceCostCenterAssignments())
        {
            if (r2.getResourceCostCenterAssignments() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceCostCentersFixture.compare(r1
                .getResourceCostCenterAssignments(), r2
                .getResourceCostCenterAssignments());
        }

        //Resource rates
        if (scope.isRates())
        {
            if (r2.getRates() == null)
            {
                Assert.fail("RPMObject in scope is null");
            }
            RPMObjectFixture.compare(r1.getRates(), r2.getRates());
        }

        //Currency
        if (scope.isCurrency())
        {
            if (r2.getCurrency() == null)
            {
                Assert.fail("RPMObject in scope is null");
            }
            Assert.assertEquals(r1.getCurrency().getID(), r2.getCurrency()
                .getID());
        }

        //        CompetenciesAssignment
        if (scope.isCompetencyAttributeAssignments())
        {
            if (r2.getCompetencyAttributeAssignments() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceAttributeAssignmentFixture.compare(r1
                .getCompetencyAttributeAssignments(), r2
                .getCompetencyAttributeAssignments());
        }

        //        ContactGroup
        if (scope.isContactGroupAssignments())
        {
            if (r2.getContactGroupAssignments() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceContactGroupAssignmentFixture.compare(r1
                .getContactGroupAssignments(), r2.getContactGroupAssignments());
        }

        // DocumentFolder
        if (scope.getDocumentFolder() != null)
        {
            DocumentFolder documentFolder1 = r1.getDocumentFolder();
            DocumentFolder documentFolder2 = r2.getDocumentFolder();
            DocumentElementFixture.compare(documentFolder1, documentFolder2,
                scope.getDocumentFolder());
        }

        //        Education
        if (scope.isEducations())
        {
            if (r2.getEducations() == null)
                Assert.fail("RPMObject in scope is null");
            EducationFixture.compare(r1.getEducations(), r2.getEducations());
        }

        //        EmployeeStatus
        if (scope.isEmployeeStatus())
        {
            if (r2.getEmployeeStatus() == null)
                Assert.fail("RPMObject in scope is null");
            DatafieldFixture.compare(r1.getEmployeeStatus(), r2
                .getEmployeeStatus());
        }

        //        EmploymentHistory
        if (scope.isEmploymentHistories())
        {
            if (r2.getEmploymentHistories() == null)
                Assert.fail("RPMObject in scope is null");
            EmploymentHistoryFixture.compare(r1.getEmploymentHistories(), r2
                .getEmploymentHistories());
        }

        //        Experience
        if (scope.isExperience())
        {
            if (r2.getExperience() == null)
                Assert.fail("RPMObject in scope is null");
            DatafieldFixture.compare(r1.getExperience(), r2.getExperience());
        }

        //        Geographical
        if (scope.isGeographicalAssignment())
        {
            if (r2.getGeographicalAssignment() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceGeographicalAssignmentFixture.compare(r1
                .getGeographicalAssignment(), r2.getGeographicalAssignment());
        }

        //        IndustryExperience
        if (scope.isIndustryExperiences())
        {
            if (r2.getIndustryExperiences() == null)
                Assert.fail("RPMObject in scope is null");
            IndustryExperienceFixture.compare(r1.getIndustryExperiences(), r2
                .getIndustryExperiences());
        }

        //        LanguageProficiency
        if (scope.isLanguageProficiencies())
        {
            if (r2.getLanguageProficiencies() == null)
                Assert.fail("RPMObject in scope is null");
            LanguageProficiencyFixture.compare(r1.getLanguageProficiencies(),
                r2.getLanguageProficiencies());
        }

        //        Organizational
        if (scope.isOrganizationalAssignment())
        {
            if (r2.getOrganizationalAssignment() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceOrganizationalAssignmentFixture.compare(r1
                .getOrganizationalAssignment(), r2
                .getOrganizationalAssignment());
        }

        //        LocationPreferences
        if (scope.isLocationPreferences())
        {
            if (r2.getLocationPreferences() == null)
                Assert.fail("RPMObject in scope is null");
            LocationPreferencesFixture.compare(r1.getLocationPreferences(), r2
                .getLocationPreferences());
        }

        //		  Passport
        if (scope.isPassport())
        {
            if (r2.getPassport() == null)
                Assert.fail("RPMObject in scope is null");
            PassportFixture.compare(r1.getPassport(), r2.getPassport());
        }

        //   TODO Compare Portfolio

        //        ProjectHistory
        if (scope.isProjectHistories())
        {
            if (r2.getProjectHistories() == null)
                Assert.fail("RPMObject in scope is null");
            ProjectHistoryFixture.compare(r1.getProjectHistories(), r2
                .getProjectHistories());
        }

        /*
         * TODO Compare ProjectSecurityRole if (scope.getExecutiveRoleScope() !=
         * null) { }
         */

        //        RPMCalendar
        if (scope.getCalendar() != null)
        {
            if (r2.getCalendar() == null)
                Assert.fail("RPMObject in scope is null");
            RPMCalendarFixture.compare(r1.getCalendar(), r2.getCalendar());
        }

        //        SkillAssignment
        if (scope.isSkillAttributeAssignments())
        {
            if (r2.getSkillAttributeAssignments() == null)
                Assert.fail("RPMObject in scope is null");
            ResourceAttributeAssignmentFixture.compare(r1
                .getSkillAttributeAssignments(), r2
                .getSkillAttributeAssignments());
        }

        //        WBSAttributes
        if (scope.isExecutiveAttributes())
        {
            if (r2.getExecutiveAttributes() == null)
                Assert.fail("RPMObject in scope is null");
        }

        //      WBSAttributeClasification
        if (scope.isExecutiveAttributeClassifications())
        {
            if (r2.getExecutiveAttributeClassifications() == null)
                Assert.fail("RPMObject in scope is null");
        }

        //        WorkingStatus
        if (scope.isWorkingStatus())
        {
            if (r2.getWorkingStatus() == null)
                Assert.fail("RPMObject in scope is null");
            DatafieldFixture.compare(r1.getWorkingStatus(), r2
                .getWorkingStatus());
        }

        //        WorkLocation
        if (scope.isWorkLocation())
        {
            if (r2.getWorkLocation() == null)
                Assert.fail("RPMObject in scope is null");
            DatafieldFixture
                .compare(r1.getWorkLocation(), r2.getWorkLocation());
        }

        //        Pool
        if (scope.getParent() != null)
        {
            if (r2.getParent() == null)
                Assert.fail("RPMObject in scope is null");
            PoolFixture.compare((Pool) r1.getParent(), (Pool) r2.getParent(),
                (PoolScope)scope.getParent());
        }

        if (scope.getPersonalProject() != null)
        {
            if (r2.getPersonalProject() == null)
                Assert.fail("RPMObject in scope is null");
        }
        
        //Cannot compare because they are created automatically
        if (scope.isPersonalTaskAssignments())
        {
            if (r2.getPersonalTaskAssignments() == null)
                Assert.fail("RPMObject in scope is null");
        }
        
        //Cannot compare because they are created automatically
        if (scope.isAdministrativeTaskAssignments())
        {
            if (r2.getAdministrativeTaskAssignments() == null)
                Assert.fail("RPMObject in scope is null");
        }
        
        if (scope.isVerifiedBy())
        {
            ResourceFixture.compare(r1.getVerifiedBy(), r2.getVerifiedBy());
        }
        
        if (scope.isReVerifiedBy())
        {
            ResourceFixture.compare(r1.getReVerifiedBy(), r2.getReVerifiedBy());
        }
    }

    //TODO Compare executiveRoleScope
    //if(scope.getExecutiveRoleScope() != null)
    //{

    //}

    public static void compare(Resource[] rArray1, Resource[] rArray2,
            ResourceScope scope) throws AssertionFailedError, Exception
    {
        rArray1 = removeDeletedResource(rArray1);

        sortWithComparator(rArray1, new ResourceFullNameComparator());
        sortWithComparator(rArray2, new ResourceFullNameComparator());

        RPMObjectFixture.compare(rArray1, rArray2);
        if (rArray1 == null)
            return;

        for (int i = 0; i < rArray1.length; i++)
        {
            ResourceFixture.compare(rArray1[i], rArray2[i], scope);
        }
    }

    public static void compareWithoutCanProperties(Resource[] rArray1,
            Resource[] rArray2, ResourceScope scope)
            throws AssertionFailedError, Exception
    {

        rArray1 = removeDeletedResource(rArray1);

        sortWithComparator(rArray1, new ResourceFullNameComparator());
        sortWithComparator(rArray2, new ResourceFullNameComparator());

        RPMObjectFixture.compare(rArray1, rArray2);
        if (rArray1 == null)
            return;

        for (int i = 0; i < rArray1.length; i++)
        {
            ResourceFixture.compareWithoutCanProperties(rArray1[i], rArray2[i],
                scope);
        }
    }

    public static ResourceRates createRates()
    {
        int base = random();
        ResourceRates rates = new ResourceRates();
        rates.setRegularCostPerHour(new Double(base + 1));
        rates.setSellingRatePerHour(new Double(base + 2));
        rates.setSpecialCostPerHour(new Double(base + 3));
        rates.setSpecialSellingRatePerHour(new Double(base + 4));
        return rates;
    }

    public static Resource[] removeDeletedResource(Resource[] pArray)
    {

        Resource[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new Resource[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

    protected static class ResourceFullNameComparator implements Comparator
    {
        /**
         *  
         */
        public ResourceFullNameComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            Resource obj1 = (Resource) o1;
            Resource obj2 = (Resource) o2;

            String key1 = null;
            String key2 = null;

            if (obj1 != null)
            {
                key1 = obj1.getFullName();
            }
            if (obj2 != null)
            {
                key2 = obj2.getFullName();
            }

            return compareStrings(key1, key2);
        }

    }
}
