/*
 * Created on Jul 19, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * @author karim
 *
 */
public class SkillFixture extends RPMObjectFixture
{

	private static final int DEFAULT_NB_CLASSIFICATIONS = 1;
	private static final int DEFAULT_NB_SKILLS = 1;
	private static final int DEFAULT_NB_PROFICIENCY_LEVELS = 1;
	
	protected static ProficiencyLevel[] availableProficiencyLevels = null;
	protected static ArrayList skipSettersForProficiencyLevel = new ArrayList();
	protected static ArrayList skipGettersForProficiencyLevel = new ArrayList();
    static
    {
    	//set skipSetters
    	skipSettersForProficiencyLevel.addAll(skipSetters);
    	skipSettersForProficiencyLevel.add("setName");
    	
    	skipGettersForProficiencyLevel.addAll(skipGetters);
    	skipGettersForProficiencyLevel.add("getName");
    }
	
    //creates a new structure
    public static ResourceAttributeSkillCategory createSaveFixture(ClientTestController ctc)throws Exception
	{
    	ResourceAttributeScope scope = createScope(DEFAULT_NB_CLASSIFICATIONS, DEFAULT_NB_SKILLS, DEFAULT_NB_PROFICIENCY_LEVELS);
    	ResourceAttributeSkillCategory category = createFixture(DEFAULT_NB_CLASSIFICATIONS, DEFAULT_NB_SKILLS, DEFAULT_NB_PROFICIENCY_LEVELS, scope, ctc);
       	category = (ResourceAttributeSkillCategory)save(ctc, category, scope);    		

    	return category;
    }
    
    public static ResourceAttributeSkillCategory createFixture(int nbClassifications, int nbSkillsByClass, int nbProficiencyLevelsBySkill, ResourceAttributeScope scope, ClientTestController ctc)throws Exception
	{
    	if(availableProficiencyLevels==null){
    		initAvailableProficiencyLevels(ctc);
    	}
    	return createCategory(nbClassifications, nbSkillsByClass, nbProficiencyLevelsBySkill, scope);
    }
    
    private static ResourceAttributeSkillCategory createCategory(int nbClassifications, int nbSkillsByClass, int nbProficiencyLevelsBySkill, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeSkillCategory category = new ResourceAttributeSkillCategory();
    	HelperTestUtilities.setByIntrospection(category, index, skipSetters);
       
		if (scope != null && scope.getClassifications() != null) {
			if(nbClassifications>0){
		    	ResourceAttributeSkillClassification classification = null;
		    	ResourceAttributeSkillClassification[] classifications = new ResourceAttributeSkillClassification[nbClassifications];
		    	for(int i=0;i<nbClassifications;i++){
		    		classification = createClassification(nbSkillsByClass, nbProficiencyLevelsBySkill, scope.getClassifications());
		    		classification.setParent(category);
		    		classifications[i] = classification;
		    	}
		    	category.setClassifications(classifications);	
			}
		}
		return category;
    }    
    
    private static ResourceAttributeSkillClassification createClassification(int nbSkillsByClass, int nbProficiencyLevelsBySkill, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeSkillClassification classification = new ResourceAttributeSkillClassification();
    	HelperTestUtilities.setByIntrospection(classification, index, skipSetters);
       
		if (scope != null && scope.getAttributes() != null) {
			if(nbSkillsByClass>0){
		    	ResourceAttributeSkill Skill = null;
		    	ResourceAttributeSkill[] skills = new ResourceAttributeSkill[nbSkillsByClass];
		    	for(int i=0;i<nbSkillsByClass;i++){
		    		Skill = createSkill(nbProficiencyLevelsBySkill, scope.getAttributes());
		    		Skill.setParent(classification);
		    		skills[i] = Skill;
		    	}
		    	classification.setAttributes(skills);				
			}
		}
		return classification;
    }
    
    private static ResourceAttributeSkill createSkill(int nbProficiencyLevelsBySkill, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeSkill Skill = new ResourceAttributeSkill();
    	HelperTestUtilities.setByIntrospection(Skill, index, skipSetters);
       
		if (scope != null && scope.isProficiencyLevels()) {
			if(nbProficiencyLevelsBySkill>0){
				ResourceAttributeSkillProficiencyLevel proficiencyLevel = null;
		    	ResourceAttributeSkillProficiencyLevel[] proficiencyLevels = new ResourceAttributeSkillProficiencyLevel[nbProficiencyLevelsBySkill];
		    	ProficiencyLevel proficiencyLevelDatafield;
		    	for(int i=0;i<nbProficiencyLevelsBySkill;i++){
		    	    proficiencyLevelDatafield = availableProficiencyLevels[i%availableProficiencyLevels.length];
		    		proficiencyLevel = createProficiencyLevel(proficiencyLevelDatafield);
		    		proficiencyLevel.setParent(Skill);
		    		proficiencyLevels[i] = proficiencyLevel;
		    	}
		    	Skill.setProficiencyLevels(proficiencyLevels);	
			}
		}
		return Skill;
    }    
    
    private static ResourceAttributeSkillProficiencyLevel createProficiencyLevel(ProficiencyLevel proficiencyLevelDatafield) throws Exception
    {
    	int index = random();
    	//create rates
    	ResourceRates rates = new ResourceRates();
    	HelperTestUtilities.setByIntrospection(rates, index, skipSettersForProficiencyLevel);
    	
    	ResourceAttributeSkillProficiencyLevel proficiencyLevel = new ResourceAttributeSkillProficiencyLevel();
    	HelperTestUtilities.setByIntrospection(proficiencyLevel, index, skipSettersForProficiencyLevel);
    	proficiencyLevel.setProficiencyLevel(proficiencyLevelDatafield);
    	proficiencyLevel.setRates(rates);
    	rates.setParent(proficiencyLevel);
    	
		return proficiencyLevel;
    }    
    
    
    
    
    //updates the names and rates
    public static void updateCategoryFixture(ResourceAttributeSkillCategory category, ResourceAttributeScope scope)throws Exception{
    	updateCategory(category, scope, random());
    }
    public static void updateClassificationFixture(ResourceAttributeSkillClassification classification, ResourceAttributeScope scope)throws Exception{
    	updateClassification(classification, scope, random());
    }
    public static void updateSkillFixture(ResourceAttributeSkill Skill, ResourceAttributeScope scope)throws Exception{
    	updateSkill(Skill, scope, random());
    }
    public static void updateProficiencyLevelFixture(ResourceAttributeSkillProficiencyLevel proficiencyLevel)throws Exception{
    	updateProficiencyLevel(proficiencyLevel, random());
    }
    
    private static void updateCategory(ResourceAttributeSkillCategory category, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(category, index, skipSetters);
		if (scope != null && scope.getClassifications() != null) {
			for(int i=0;i<category.getClassifications().length;i++){
				updateClassificationFixture(category.getClassifications()[i], scope.getClassifications());
			}
		}
    }    
    
    private static void updateClassification(ResourceAttributeSkillClassification classification, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(classification, index, skipSetters);
       
		if (scope != null && scope.getAttributes() != null) {
			for(int i=0;i<classification.getAttributes().length;i++){
				updateSkillFixture(classification.getAttributes()[i], scope.getAttributes());
			}
		}
    }
    
    private static void updateSkill(ResourceAttributeSkill Skill, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(Skill, index, skipSetters);
       
		if (scope != null && scope.isProficiencyLevels()) {
			for(int i=0;i<Skill.getProficiencyLevels().length;i++){
				updateProficiencyLevelFixture(Skill.getProficiencyLevels()[i]);
			}
		}
    }    
    
    private static void updateProficiencyLevel(ResourceAttributeSkillProficiencyLevel proficiencyLevel, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(proficiencyLevel, index, skipSettersForProficiencyLevel);
    	HelperTestUtilities.setByIntrospection(proficiencyLevel.getRates(), index, skipSettersForProficiencyLevel);
    }    
    
    
    
    
    //updates the structure add/remove/element
    public static void updateCategoryFixture(ResourceAttributeSkillCategory category, int deltaNbClassifications, int deltaNbSkillsByClass, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
    	updateCategory(category, deltaNbClassifications, deltaNbSkillsByClass, deltaNbProficiencyLevelsBySkill, scope);
    }
    public static void updateClassificationFixture(ResourceAttributeSkillClassification classification, int deltaNbSkillsByClass, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
    	updateClassification(classification, deltaNbSkillsByClass, deltaNbProficiencyLevelsBySkill, scope);
    }
    public static void updateSkillFixture(ResourceAttributeSkill Skill, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
    	updateSkill(Skill, deltaNbProficiencyLevelsBySkill, scope);
    }

    private static void updateCategory(ResourceAttributeSkillCategory category, int deltaNbClassifications, int deltaNbSkillsByClass, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.getClassifications() != null) {
			if(deltaNbClassifications>0){//add new elements
		    	ResourceAttributeSkillClassification newClassification = null;
		    	ResourceAttributeSkillClassification[] newClassifications = new ResourceAttributeSkillClassification[deltaNbClassifications];
		    	for(int i=0;i<deltaNbClassifications;i++){
		    		newClassification = createClassification(deltaNbSkillsByClass, deltaNbProficiencyLevelsBySkill, scope.getClassifications());
		    		newClassification.setParent(category);
		    		newClassifications[i] = newClassification;
		    	}
		    	
		    	ArrayList temp = mergeArrays(category.getClassifications(), newClassifications);
		    	ResourceAttributeSkillClassification[] totalClassifications = new ResourceAttributeSkillClassification[temp.size()];
		    	temp.toArray(totalClassifications);
		    	
		    	category.setClassifications(totalClassifications);		    	
			}
			for(int i=0;i<category.getClassifications().length;i++){
				updateClassificationFixture(category.getClassifications()[i], deltaNbSkillsByClass, deltaNbProficiencyLevelsBySkill, scope.getClassifications());
			}
		}
    }
    private static void updateClassification(ResourceAttributeSkillClassification classification, int deltaNbSkillsByClass, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.getAttributes() != null) {
			if(deltaNbSkillsByClass>0){//add new elements
		    	ResourceAttributeSkill newSkill = null;
		    	ResourceAttributeSkill[] newSkills = new ResourceAttributeSkill[deltaNbSkillsByClass];
		    	for(int i=0;i<deltaNbSkillsByClass;i++){
		    		newSkill = createSkill(deltaNbProficiencyLevelsBySkill, scope.getAttributes());
		    		newSkill.setParent(classification);
		    		newSkills[i] = newSkill;
		    	}
		    	
		    	ArrayList temp = mergeArrays(classification.getAttributes(), newSkills);
		    	ResourceAttributeSkill[] totalSkills = new ResourceAttributeSkill[temp.size()];
		    	temp.toArray(totalSkills);
		    	
		    	classification.setAttributes(totalSkills);		    	
			}
			for(int i=0;i<classification.getAttributes().length;i++){
				updateSkillFixture(classification.getAttributes()[i], deltaNbProficiencyLevelsBySkill, scope.getAttributes());
			}
		}
    }
    private static void updateSkill(ResourceAttributeSkill skill, int deltaNbProficiencyLevelsBySkill, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.isProficiencyLevels()) {
			if(deltaNbProficiencyLevelsBySkill>0){//add new elements
			    int currentNbProficiencyLevels = (skill.getProficiencyLevels()!=null)?skill.getProficiencyLevels().length:0;
			    int nbPossibleAdditionalProficiencyLevels = getPossibleAdditionalProficiencyLevels(currentNbProficiencyLevels, deltaNbProficiencyLevelsBySkill);
		    	ResourceAttributeSkillProficiencyLevel newProficiencyLevel = null;
		    	ResourceAttributeSkillProficiencyLevel[] newProficiencyLevels = new ResourceAttributeSkillProficiencyLevel[nbPossibleAdditionalProficiencyLevels];
		    	ProficiencyLevel proficiencyLevelDatafield;
		    	for(int i=0;i<nbPossibleAdditionalProficiencyLevels;i++){
		    	    proficiencyLevelDatafield = availableProficiencyLevels[(currentNbProficiencyLevels+i)%availableProficiencyLevels.length];
		    		newProficiencyLevel = createProficiencyLevel(proficiencyLevelDatafield);
		    		newProficiencyLevel.setParent(skill);
		    		newProficiencyLevels[i] = newProficiencyLevel;
		    	}
		    	
		    	ArrayList temp = mergeArrays(skill.getProficiencyLevels(), newProficiencyLevels);
		    	ResourceAttributeSkillProficiencyLevel[] totalProficiencyLevels = new ResourceAttributeSkillProficiencyLevel[temp.size()];
		    	temp.toArray(totalProficiencyLevels);
		    	
		    	skill.setProficiencyLevels(totalProficiencyLevels);		    	
			}
		}
    }
    
    private static int getPossibleAdditionalProficiencyLevels(int currentNbProficiencyLevels, int deltaNbProficiencyLevelsByCompetency){
        if(currentNbProficiencyLevels + deltaNbProficiencyLevelsByCompetency > DatafieldFixture.PROFICIENCYLEVELS.length){
            return DatafieldFixture.PROFICIENCYLEVELS.length - currentNbProficiencyLevels;
        } else {
            return deltaNbProficiencyLevelsByCompetency;
        }
    }
    
    //Compare section
	public static void compare(ResourceAttribute[] array1, ResourceAttribute[] array2,
			ResourceAttributeScope scope) throws AssertionFailedError, Exception {

    	sortWithComparator(array1, RPMObjectIDComparator.INSTANCE);
    	sortWithComparator(array2, RPMObjectIDComparator.INSTANCE);
		
		RPMObjectFixture.compare(array1, array2);

		for (int i = 0; i < array1.length; i++) {
			SkillFixture.compare(array1[i], array2[i], scope);
		}
	}
	
    public static void compare(ResourceAttribute obj1, ResourceAttribute obj2, ResourceAttributeScope scope)
	throws AssertionFailedError, Exception {
    	
        if (obj1 instanceof ResourceAttributeSkillProficiencyLevel)
        {
            RPMObjectFixture.compareWithSkip(obj1, obj2, skipGettersForProficiencyLevel);
            compareSkillProficiencyLevel(obj1, obj2, scope);
        }
        else
        {
            RPMObjectFixture.compare(obj1, obj2);
            if(obj1 instanceof ResourceAttributeSkillCategory){
        		compareSkillCategory(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeSkillClassification){
        		compareSkillClassification(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeSkill){
        		compareSkill(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeSkillProficiencyLevel){
        		
        	} else {
        		Assert.fail("ResourceAttributeFixture.compare : class "+obj1.getClass().getName()+" not supported.");
        	}
        }
    }
	
    public static void compareSkillCategory(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeSkillCategory category1 = (ResourceAttributeSkillCategory)obj1;
    	ResourceAttributeSkillCategory category2 = (ResourceAttributeSkillCategory)obj2;

    	//compare children
		if (scope != null && scope.getClassifications() != null) {
			SkillFixture.compare(category1.getClassifications(), category2.getClassifications(), scope.getClassifications());
		}

	}
    
    public static void compareSkillClassification(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeSkillClassification classification1 = (ResourceAttributeSkillClassification)obj1;
    	ResourceAttributeSkillClassification classification2 = (ResourceAttributeSkillClassification)obj2;

    	//compare children
		if (scope != null && scope.getAttributes() != null) {
			SkillFixture.compare(classification1.getAttributes(), classification2.getAttributes(), scope.getAttributes());
		}

	}

    public static void compareSkill(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeSkill Skill1 = (ResourceAttributeSkill)obj1;
    	ResourceAttributeSkill Skill2 = (ResourceAttributeSkill)obj2;

    	//compare children
		if (scope != null && scope.isProficiencyLevels()) {
			SkillFixture.compare(Skill1.getProficiencyLevels(), Skill2.getProficiencyLevels(), null);
		}

	}
    
    public static void compareSkillProficiencyLevel(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeSkillProficiencyLevel proficiencyLevel1 = (ResourceAttributeSkillProficiencyLevel)obj1;
    	ResourceAttributeSkillProficiencyLevel proficiencyLevel2 = (ResourceAttributeSkillProficiencyLevel)obj2;

    	//create and use ResourceRatesFixture class
		SkillFixture.compareRates(proficiencyLevel1.getRates(), proficiencyLevel2.getRates(), null);
		
		// The name is set automatically by the proficiency level if a proficiency level datafield is supplied
        // This ensures that the name of the proficiency levels match on both objects
        // The IDs of the two proficiency levels take precedence over everything else, but if both aren't available,
        // then the names are used
        Assert.assertTrue("Proficiency levels do not match", obj1.getName() == obj2.getName() 
            || (((ResourceAttributeSkillProficiencyLevel) obj1).getProficiencyLevel() != null 
                    && ((ResourceAttributeSkillProficiencyLevel) obj2).getProficiencyLevel() != null
                    && ((ResourceAttributeSkillProficiencyLevel) obj1).getProficiencyLevel().getID().compareTo(((ResourceAttributeSkillProficiencyLevel) obj2).getProficiencyLevel().getID()) == 0)
            || (((ResourceAttributeSkillProficiencyLevel) obj2).getProficiencyLevel() != null 
                    && ((ResourceAttributeSkillProficiencyLevel) obj2).getProficiencyLevel().getValue().compareTo(obj1.getName()) == 0)
            || (((ResourceAttributeSkillProficiencyLevel) obj1).getProficiencyLevel() != null 
                    && ((ResourceAttributeSkillProficiencyLevel) obj1).getProficiencyLevel().getValue().compareTo(obj2.getName()) == 0));
	}
    
    public static void compareRates(ResourceRates obj1, ResourceRates obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	//here we don't call the RPMObjectFixture.compare because it compares the IDs
    	//and in this case, the SP generates a new ID for the updated rates.
        Assert.assertNotNull(obj1);
        Assert.assertNotNull(obj2);
        HelperTestUtilities.assertEqualsByIntrospection(obj1, obj2, skipGetters);
	}
    
    
    private static void initAvailableProficiencyLevels(ClientTestController ctc)throws Exception{
		//set possible proficiency level values
		int nbProficiencyLevels = DatafieldFixture.PROFICIENCYLEVELS.length;
		availableProficiencyLevels = new ProficiencyLevel[nbProficiencyLevels];
		for(int i=0;i<nbProficiencyLevels;i++){
		    availableProficiencyLevels[i] = createAvailableProficiencyLevel(DatafieldFixture.PROFICIENCYLEVELS[i],ctc);
		}
    }
    
    private static ProficiencyLevel createAvailableProficiencyLevel(String name, ClientTestController ctc)throws Exception{
    	ProficiencyLevel proficiencyLevel = DatafieldFixture.createProficiencyLevel(name);
    	proficiencyLevel = (ProficiencyLevel)ctc.loadFromID(proficiencyLevel, null);
    	return proficiencyLevel;
    }
    
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }    

    public static ResourceAttributeScope createScope(int nbClassifications, int nbSkillsByClass, int nbProficiencyLevelsBySkill){
    	ResourceAttributeScope scope = new ResourceAttributeScope();
    	if(nbClassifications>0){
    		ResourceAttributeScope classificationScope = new ResourceAttributeScope();
    		if(nbSkillsByClass>0){
        		ResourceAttributeScope skillScope = new ResourceAttributeScope();
        		if(nbProficiencyLevelsBySkill>0){
        			skillScope.setProficiencyLevels(true);
        		}
        		skillScope.setParent(classificationScope);
        		classificationScope.setAttributes(skillScope);
    		}
    		classificationScope.setParent(scope);
    		scope.setClassifications(classificationScope);
    	}
    	return scope;
    }

	
}
