package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.CompetencyFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * @author karim
 *
 */
public class TestCompetency extends ClientTestController 
{

    private ResourceAttributeScope mainScope;

    //Original structure
	private int nbClassifications = 1; 
	private int nbCompetenciesByClass = 2; 
	private int nbProficiencyLevelsByCompetency = 2;
    
	//Additional elements in the structure
	private int deltaNbClassifications = 1; //must be >0
	private int deltaNbCompetenciesByClass = 1; //must be >0 
	private int deltaNbProficiencyLevelsByCompetency = 1; //must be >0
	
    public TestCompetency(String testName){
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();
		mainScope = CompetencyFixture.createScope(nbClassifications, nbCompetenciesByClass, nbProficiencyLevelsByCompetency);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }
    
    public void testloadFromXPath() throws AssertionError, Exception
    {
        
        //This test validates that only requested type of object is returned

        RPMObject[] objs1 = loadFromXpath(
                "/ResourceAttributeCompetencyCategory", null);

        if (objs1 != null)
        {
            for (int i = 0; i < objs1.length; i++)
            {
                assertTrue(objs1[i] instanceof ResourceAttributeCompetencyCategory);
            }
        }

        RPMObject[] objs2 = loadFromXpath(
                "/ResourceAttributeCompetencyClassification", null);

        if (objs2 != null)
        {
            for (int i = 0; i < objs2.length; i++)
            {
                assertTrue(objs2[i] instanceof ResourceAttributeCompetencyClassification);
            }
        }

        RPMObject[] objs3 = loadFromXpath("/ResourceAttributeCompetency", null);

        if (objs3 != null)
        {
            for (int i = 0; i < objs3.length; i++)
            {
                assertTrue(objs3[i] instanceof ResourceAttributeCompetency);
            }
        }

        RPMObject[] objs4 = loadFromXpath(
                "/ResourceAttributeCompetencyProficiencyLevel", null);

        if (objs4 != null)
        {
            for (int i = 0; i < objs4.length; i++)
            {
                assertTrue(objs4[i] instanceof ResourceAttributeCompetencyProficiencyLevel);
            }
        }

        RPMObject[] objs5 = loadFromXpath(
                "/ResourceAttributeCompetency/ResourceAttributeCompetencyProficiencyLevel",
                null);

        if (objs5 != null)
        {
            for (int i = 0; i < objs5.length; i++)
            {
                assertTrue(objs5[i] instanceof ResourceAttributeCompetencyProficiencyLevel);
            }
        }

    }    

    private void insert(ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
    	ResourceAttributeCompetencyCategory category = insertCategory(scope, loadFromId);
    	
    	//delete the element
    	deleteCategory(category, loadFromId);  
    }
    
    private void update(ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
    	ResourceAttributeCompetencyCategory category = insertCategory(scope, loadFromId);
    	
    	category = updateNamesAndRates(category, scope, loadFromId);
    	
    	category = updateStructure(category, scope, loadFromId);
    	
    	//delete the element
    	deleteCategory(category, loadFromId);  
    }
    
    
    
    private ResourceAttributeCompetencyCategory insertCategory(ResourceAttributeScope scope, boolean loadFromId) throws Exception
    {
    	ResourceAttributeCompetencyCategory category = CompetencyFixture.createFixture(nbClassifications, nbCompetenciesByClass, nbProficiencyLevelsByCompetency, scope, this);
    	       
        //Insert the element into the DB
        SingleResult result =  save(category,scope);
        
        //compare the returned element
        ResourceAttributeCompetencyCategory returnedCategory = (ResourceAttributeCompetencyCategory)result.getRpmObject();
        CompetencyFixture.compare(category, returnedCategory, scope);
        
        //reload the element and compare it
        returnedCategory = (ResourceAttributeCompetencyCategory)load(returnedCategory, scope, loadFromId);
        CompetencyFixture.compare(category, returnedCategory, scope);
        
        //return the category
        return returnedCategory;
    }
    
    private ResourceAttributeCompetencyCategory updateNamesAndRates(ResourceAttributeCompetencyCategory originalCategory, ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
    	return update(originalCategory, scope, loadFromId, false);
    }
    
    private ResourceAttributeCompetencyCategory updateStructure(ResourceAttributeCompetencyCategory originalCategory, ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
    	return update(originalCategory, scope, loadFromId, true);
    }
    
    private ResourceAttributeCompetencyCategory update(ResourceAttributeCompetencyCategory category, ResourceAttributeScope scope, boolean loadFromId, boolean updateStructure) throws AssertionError, Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements 
        	CompetencyFixture.updateCategoryFixture(category, deltaNbClassifications, deltaNbCompetenciesByClass, deltaNbProficiencyLevelsByCompetency, scope);    		
    	} else {
    		//only updates the names and the rates
        	CompetencyFixture.updateCategoryFixture(category, scope);
    	}

        //Insert the element into the DB
        SingleResult result =  save(category,scope);
    	
        //compare the returned element
        ResourceAttributeCompetencyCategory returnedCategory = (ResourceAttributeCompetencyCategory)result.getRpmObject();
        CompetencyFixture.compare(category, returnedCategory, scope);
        
        //reload the element and compare it
        returnedCategory = (ResourceAttributeCompetencyCategory)load(returnedCategory, scope, loadFromId);
        CompetencyFixture.compare(category, returnedCategory, scope);
        
        //return the category
        return returnedCategory;
    }
    
    private void deleteCategory(ResourceAttributeCompetencyCategory category, boolean loadFromId)throws Exception
	{    	   	
        //delete the element
       	delete(category);
    	
        //reload the deleted element
       	ResourceAttributeCompetencyCategory deletedCategory = loadDeletedCategory(category, loadFromId);
        
        //validate deleted element
        assertNull(deletedCategory);
    }
    
    private ResourceAttributeCompetencyCategory loadDeletedCategory(ResourceAttributeCompetencyCategory category, boolean loadFromId)throws Exception{
        try{
        	category = (ResourceAttributeCompetencyCategory)load(category, null, loadFromId);
           	return category;
        }catch (Exception e){
            //element does not exist
        }
        return null;
    }
    
    private RPMObject load(ResourceAttributeCompetencyCategory category, ResourceAttributeScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new ResourceAttributeCompetencyCategory();
            obj.setID(category.getID());
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/RESOURCEATTRIBUTECOMPETENCYCATEGORY[id='"+ category.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
    private void deleteCategory(ResourceAttributeCompetencyCategory category)
    {
    	try{
    		if(category.getID()!=null){
    			delete(category);
    		}
		} catch(Exception e){
			//do nothing
		}
    }


	
}
