package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolReferenceScope;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author gosselin
 */
public class TestPool extends ClientTestController
{
    private Pool mainPool;
    private Pool parent = null;
    private PoolScope mainScope = new PoolScope();
    private int nbChildren = 2;

    public void setUp() throws Exception 
    {
        super.setUp();
        mainScope = new PoolScope();
    }

    public void tearDown() throws Exception 
    {
        if (parent != null){
            try{
                delete(parent);
            }catch(Exception e){
                //do nothing
            }
        }
        
        super.tearDown();

    }
    
    public void testInsertWithPoolAttributeAssignment() throws AssertionError,
            Exception
    {
        mainScope.setAttributeAssignments(true);
        mainPool = PoolFixture.createFixture(false, nbChildren, mainScope, this);
        

        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());
        
        loadPoolFromId(mainPool, mainScope);
    }

    public void testInsertWithPoolDomain() throws AssertionError, Exception {
               
        mainScope.setDomain(new PoolDomainScope());
        mainPool = PoolFixture.createFixture(false, 0, mainScope, this);

        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());

        loadPoolFromId(mainPool, mainScope);

    }

    public void testInsertWithResourceAssignment() throws AssertionError,
            Exception {
        
        mainScope.setResourceAssignments(true);
        mainPool = PoolFixture.createFixture(false, nbChildren, mainScope, this);
        

        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());
        
        loadPoolFromId(mainPool, mainScope);
    }
    
    public void testInsertWithAdministrativeTasks() throws AssertionError,
            Exception
    {

        mainScope.setAdministrativeTasks(true);
        mainPool = PoolFixture
            .createFixture(false, nbChildren, mainScope, this);

        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());

        loadPoolFromId(mainPool, mainScope);

    }

    public void testInsertWithoutResource() throws AssertionError, Exception {
        
        initWithFullScope();
        
        mainScope.setResources(null);
        mainPool.setResources(null);
        
        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());

        loadPoolFromId(mainPool, mainScope);

    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception {
              
        mainPool = PoolFixture.createFixture(false, 0, null, this);
        
        Pool saved = savePool(mainPool, null);
        mainPool.setID(saved.getID());

        //load from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Pool[Id='" + mainPool.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Pool);
        PoolFixture.compare(mainPool, (Pool) rpmObjects[0], null);
    }
    
    public void testLoadFromName() throws AssertionError,
            Exception
    {
        mainPool = PoolFixture.createFixture(false, 0, null, this);

        Pool saved = savePool(mainPool, null);
        mainPool.setID(saved.getID());

        //load from xpath
        RPMObject[] rpmObjects = loadFromXpath(
                "/Pool[name='" + mainPool.getName() + "']", null);
        assertNotNull(rpmObjects);
        
        //Check if all objects are instance of Pool and if the condition is respected
        for (int i = 0; i < rpmObjects.length; i++)
        {
            assertTrue(rpmObjects[i] instanceof Pool);
            assertTrue(((Pool)rpmObjects[i]).getName().trim().equals(mainPool.getName().trim()));
        }

    }    

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception {
            
        mainPool = PoolFixture.createFixture(false, 0, null, this);
        
        Pool saved = savePool(mainPool, null);
        mainPool.setID(saved.getID());

        loadPoolFromId(mainPool, null);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception {
        
        initWithFullScope();
        
        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());
        
        loadPoolFromId(mainPool, mainScope);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception {
             
        mainPool = PoolFixture.createFixture(false, 0, null, this);

        Pool saved = savePool(mainPool, null);
        mainPool.setID(saved.getID());
        
        //Create a new Pool
        Pool updatedPool = PoolFixture.createFixture(false, 0, null, this);
        //By setting the ID to the mainPool ID, the updatedPool represent
        //the mainPool with all its attribute updated.
        updatedPool.setID(mainPool.getID());

        //Save the modifications
        savePool(updatedPool, null);
 
        loadPoolFromId(updatedPool, null);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception {
        
        initWithFullScope();
        
        Pool saved = savePool(mainPool, mainScope);

        Pool updatedPool = PoolFixture.updateFixture(saved, this);

        //Save the modifications
        savePool(updatedPool, mainScope);

        //load from an id
        loadPoolFromId(updatedPool, mainScope);
    }
    
    public void testDeleteAll() throws Exception
    {
        deleteAllTestObjects(Pool.class);
    }
    
    private Pool savePool(Pool p, PoolScope scope) throws AssertionError,
            Exception {
        SingleResult result;
        result = save(p, scope);

        p = (Pool) result.getRpmObject();
        return p;
    }

    private void loadPoolFromId(Pool objectToCompare, PoolScope scope) throws AssertionError,
            Exception {
        logger.info("loadPoolFromId");

        RPMObject obj = new Pool();
        obj.setID(objectToCompare.getID());
              
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);

        if (scope != null) {
            logger.info("Scoping" + scope.toString());
        } else {
            logger.info("Scoping is null");
        }
        logger.info("======================");
        logger.info("ObjectToCompare: " + objectToCompare);
        logger.info("======================");
        logger.info("LoadedObject: " + rpmObject);

        
        //Needed by tearDown method
        if(rpmObject != null){
            parent = (Pool)rpmObject.getParent();
        }
        
        PoolFixture.compare(objectToCompare, (Pool) rpmObject, scope);
    }
    
    private void initWithFullScope() throws Exception{

        PoolScope parentScope = new PoolScope();       
        mainScope.setParent(parentScope);        
        mainScope.setAttributeAssignments(true);
        mainScope.setPools(new PoolScope());
        mainScope.setPoolReferences(new PoolReferenceScope());
        mainScope.setAttributeAssignments(true);
        mainScope.setResources(new ResourceScope());
        mainScope.setDomain(new PoolDomainScope());
        mainScope.setResourceAssignments(true);
        mainScope.setAdministrativeTasks(true);
               
        mainPool = PoolFixture.createFixture(true, nbChildren, mainScope, this);
        
    }   

    public void testReloadTypeNone() throws Exception
    {
        
        mainPool = PoolFixture.createFixture(false, 0, mainScope, this);

        SingleResult save = getApplicationInterface().save(sessionid, mainPool, mainScope, ReloadType.None);
        
        assertNull( save.getRpmObject() );
    }
}
