package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ProjectHistoryFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;


public class TestResourceProjectHistory extends ClientTestController
{
    private Resource mainResource;

    private ProjectHistory mainProjectHistory;

    public void setUp() throws Exception
    {
        super.setUp();

        mainResource = ResourceFixture.createSaveFixture(this);

        mainProjectHistory = ProjectHistoryFixture.createFixture(mainResource);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ProjectHistory ph = saveProjectHistory(mainProjectHistory);
        
        //load from an xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
                + ph.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);
        ProjectHistoryFixture.compare(mainProjectHistory, (ProjectHistory)rpmObjects[0]);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        ProjectHistory ph = saveProjectHistory(mainProjectHistory);

        //load from an id
        RPMObject obj = new ProjectHistory();
        obj.setID(ph.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ProjectHistory);
        ProjectHistoryFixture.compare(mainProjectHistory, (ProjectHistory) rpmObject);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainProjectHistory = saveProjectHistory(mainProjectHistory);
        //Create a new ProjectHistory
        ProjectHistory updatedProjectHistory = ProjectHistoryFixture.createFixture(mainResource);
        //By setting the ID to the mainProjectHistory ID, the updatedProjectHistory represent
        //the mainProjectHistory with all its attribute updated.
        updatedProjectHistory.setID(mainProjectHistory.getID());

        //Save the modifications
        ProjectHistory ph = saveProjectHistory(updatedProjectHistory);

        //load from an id
        RPMObject obj = new ProjectHistory();
        obj.setID(ph.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ProjectHistory);
        ProjectHistoryFixture.compare(updatedProjectHistory, (ProjectHistory) rpmObject);
    }

    private ProjectHistory saveProjectHistory(ProjectHistory p) throws AssertionError, Exception
    {
        SingleResult result = save(p, null);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof ProjectHistory);

        p = (ProjectHistory) result.getRpmObject();
        return p;
    }
}
