package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

public class TestCompetencyDelete extends GenericValidation
{
    private static ResourceAttributeCompetencyCategory mainCategory = null;
    private static ResourceAttributeCompetencyProficiencyLevel myProficiency = null;
    private static ResourceAttributeScope mainScope = null;
    private static String comptName = null;
    
    public void setUp() throws Exception
    {
        super.setUp();       
        comptName = HelperTestUtilities.createName();
    }
        
    public void testDeleteAndVerifyCompetency() throws Exception
    {
    	mainScope = new ResourceAttributeScope();
		//Add competency categorie
		ResourceAttributeCompetencyCategory cat = new ResourceAttributeCompetencyCategory();
		cat.setName(comptName);
		mainScope.setCategories(new ResourceAttributeScope());
		
		//Add competency classification to category
		ResourceAttributeCompetencyClassification classification = new ResourceAttributeCompetencyClassification();
		classification.setName("WSQACompetencyClassification");
		cat.setClassifications(new ResourceAttributeCompetencyClassification[]{classification});
		mainScope.setClassifications(new ResourceAttributeScope());
		
		//Add competency  to classification	
		ResourceAttributeCompetency competency = new ResourceAttributeCompetency();
		competency.setName("WSQACompetency");
		classification.setAttributes(new ResourceAttributeCompetency[]{competency});
		mainScope.getClassifications().setAttributes(new ResourceAttributeScope());
		
		// Create a proficiency level
    	ProficiencyLevel proficiencyLevel = DatafieldFixture.createProficiencyLevel(DatafieldFixture.PROFICIENCYLEVEL_1);
		SingleResult result = save(proficiencyLevel, null);
		assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof ProficiencyLevel);    	
    	proficiencyLevel = (ProficiencyLevel) result.getRpmObject();
		//Add profeciency level  to competency	
		ResourceAttributeCompetencyProficiencyLevel proficiency = new ResourceAttributeCompetencyProficiencyLevel();
		proficiency.setName(proficiencyLevel.getValue());
		
		//set rates
		ResourceRates rates = new ResourceRates();
		rates.setRegularCostPerHour(new Double(25));
		rates.setSellingRatePerHour(new Double(40));
		rates.setSpecialCostPerHour(new Double(35));
		rates.setSpecialSellingRatePerHour(new Double(55));
		proficiency.setRates(rates);
		
		competency.setProficiencyLevels(new ResourceAttributeCompetencyProficiencyLevel[]{proficiency});		
		mainScope.getClassifications().getAttributes().setProficiencyLevels(true);
		
		SingleResult result1 = save(cat, mainScope);
		assertNotNull(result1);
        assertNotNull(result1.getRpmObject());
        assertTrue(result1.getRpmObject() instanceof ResourceAttributeCompetencyCategory);
        
    	//load from XPath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceAttributeCompetencyCategory[name='" + comptName + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceAttributeCompetencyCategory);	
		
        mainCategory = (ResourceAttributeCompetencyCategory) rpmObjects[0];
		assertNotNull(mainCategory);
		
		myProficiency = mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0];
		assertNotNull(myProficiency);

		rates.setRegularCostPerHour(new Double(250));
		rates.setSellingRatePerHour(new Double(400));
		rates.setSpecialCostPerHour(new Double(350));
		rates.setSpecialSellingRatePerHour(new Double(550));
    	myProficiency.setRates(rates);
		SingleResult result2 = save(mainCategory, mainScope);
		assertNotNull(result2);
        assertNotNull(result2.getRpmObject());
        assertTrue(result2.getRpmObject() instanceof ResourceAttributeCompetencyCategory);
        
    	delete(myProficiency);
    	RPMObject[] rpmObjects1 = loadFromXpath("/ResourceAttributeCompetencyCategory[name='" + comptName + "']", mainScope);
        assertNotNull(rpmObjects1);
    }
    
    public void testDeleteAndVerifyCategory() throws Exception
    {
    	mainScope = new ResourceAttributeScope();
		//Add competency categorie
		ResourceAttributeCompetencyCategory cat = new ResourceAttributeCompetencyCategory();
		cat.setName(comptName);
		mainScope.setCategories(new ResourceAttributeScope());
		
		//Add competency classification to category
		ResourceAttributeCompetencyClassification classification = new ResourceAttributeCompetencyClassification();
		classification.setName("WSQACompetencyClassification");
		cat.setClassifications(new ResourceAttributeCompetencyClassification[]{classification});
		mainScope.setClassifications(new ResourceAttributeScope());
		
		//Add competency  to classification	
		ResourceAttributeCompetency competency = new ResourceAttributeCompetency();
		competency.setName("WSQACompetency");
		classification.setAttributes(new ResourceAttributeCompetency[]{competency});
		mainScope.getClassifications().setAttributes(new ResourceAttributeScope());
		
		// Create a proficiency level
    	ProficiencyLevel proficiencyLevel = DatafieldFixture.createProficiencyLevel(DatafieldFixture.PROFICIENCYLEVEL_1);
		SingleResult result = save(proficiencyLevel, null);
		assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof ProficiencyLevel);    	
    	proficiencyLevel = (ProficiencyLevel) result.getRpmObject();
    	
		// Add the created proficiency level to competency	
		ResourceAttributeCompetencyProficiencyLevel proficiency = new ResourceAttributeCompetencyProficiencyLevel();
		proficiency.setName(proficiencyLevel.getValue());
		
		//set rates
		ResourceRates rates = new ResourceRates();
		rates.setRegularCostPerHour(new Double(25));
		rates.setSellingRatePerHour(new Double(40));
		rates.setSpecialCostPerHour(new Double(35));
		rates.setSpecialSellingRatePerHour(new Double(55));
		proficiency.setRates(rates);
		
		competency.setProficiencyLevels(new ResourceAttributeCompetencyProficiencyLevel[]{proficiency});		
		mainScope.getClassifications().getAttributes().setProficiencyLevels(true);
		
		SingleResult result1 = save(cat, mainScope);
		assertNotNull(result1);
        assertNotNull(result1.getRpmObject());
        assertTrue(result1.getRpmObject() instanceof ResourceAttributeCompetencyCategory);
        
    	//load from XPath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceAttributeCompetencyCategory[name='" + comptName + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceAttributeCompetencyCategory);	
		
        mainCategory = (ResourceAttributeCompetencyCategory) rpmObjects[0];
		assertNotNull(mainCategory);
		
		myProficiency = mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0];
		assertNotNull(myProficiency);

		rates.setRegularCostPerHour(new Double(250));
		rates.setSellingRatePerHour(new Double(400));
		rates.setSpecialCostPerHour(new Double(350));
		rates.setSpecialSellingRatePerHour(new Double(550));
    	myProficiency.setRates(rates);
		SingleResult result2 = save(mainCategory, mainScope);
		assertNotNull(result2);
        assertNotNull(result2.getRpmObject());
        assertTrue(result2.getRpmObject() instanceof ResourceAttributeCompetencyCategory);
        
    	delete(mainCategory);
    	RPMObject[] rpmObjects2 = loadFromXpath("/ResourceAttributeCompetencyCategory[name='" + comptName + "']", mainScope);
        assertNull(rpmObjects2);
    }
       
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
}
