package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.EducationFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class TestEducationValidation extends GenericValidation
{
    private Resource mainResource;

    private Education mainEducation;

    public void setUp() throws Exception
    {
        super.setUp();

        mainResource = ResourceFixture.createSaveFixture(this);
        mainEducation = EducationFixture.createFixture(mainResource);
        mainResource.setEducations(new Education[]
        {
            mainEducation
        });
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainEducation.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainEducation.getClass().getName(),
                "parent", mainEducation.getID())
        };

        verifySaveMustFail(mainEducation, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainEducation.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainEducation.getClass()
                .getName(), "parent", mainEducation.getID())
        };

        verifySaveMustFail(mainEducation, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainEducation
            .setOrganization(getTestString(ValidationConstants.EDUCATION_ORGANIZATION_MAX));
        mainEducation
            .setDegreeProgram(getTestString(ValidationConstants.EDUCATION_DEGREEPROGRAM_MAX));
        mainEducation
            .setSpecialization(getTestString(ValidationConstants.EDUCATION_SPECIALIZATION_MAX));
        mainEducation
            .setHonoursAwards(getTestString(ValidationConstants.EDUCATION_HONOURSAWARDS_MAX));

        // Attempt to save the Education object
        mainEducation = (Education) saveMustSucceed(mainEducation, Education.class, null);

        // Load the inserted education from id
        RPMObject obj = new Education();
        obj.setID(mainEducation.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Education);

        // Check that the loaded education is identical to the one we requested
        // to store
        EducationFixture.compare(mainEducation, (Education) rpmObject);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainEducation
            .setOrganization(getTestString(ValidationConstants.EDUCATION_ORGANIZATION_MAX + 1));
        mainEducation
            .setDegreeProgram(getTestString(ValidationConstants.EDUCATION_DEGREEPROGRAM_MAX + 1));
        mainEducation
            .setSpecialization(getTestString(ValidationConstants.EDUCATION_SPECIALIZATION_MAX + 1));
        mainEducation
            .setHonoursAwards(getTestString(ValidationConstants.EDUCATION_HONOURSAWARDS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainEducation.getClass().getName(),
                "organization", mainEducation.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEducation.getClass().getName(),
                "degreeProgram", mainEducation.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEducation.getClass().getName(),
                "specialization", mainEducation.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEducation.getClass().getName(),
                "honoursAwards", mainEducation.getID())
        };

        verifySaveMustFail(mainEducation, null, expectedExceptions);
    }
}
