package com.ibm.rpm.client.junit.resource.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ProjectHistoryFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.ProjectHistory;

/**
 * @author dominiqu
 */
public class TestProjectHistoryValidation extends GenericValidation
{

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, projectHistory.getClass()
                .getName(), "parent", projectHistory.getID())
        };

        verifySaveMustFail(projectHistory, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        ClientCostCenter wrongParent = new ClientCostCenter();
        
        projectHistory.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, projectHistory
                .getClass().getName(), "parent", projectHistory.getID())
        };

        verifySaveMustFail(projectHistory, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        // Set field values with maxLength and verify that they are accepted
        projectHistory
            .setClient(getTestString(ValidationConstants.PROJECTHISTORY_CLIENT_MAX));
        projectHistory
            .setDescription(getTestString(ValidationConstants.PROJECTHISTORY_DESCRIPTION_MAX));
        projectHistory
            .setProjectLength(getTestString(ValidationConstants.PROJECTHISTORY_PROJECTLENGTH_MAX));
        projectHistory
            .setTimeSpent(getTestString(ValidationConstants.PROJECTHISTORY_TIMESPENT_MAX));
        projectHistory
            .setRole(getTestString(ValidationConstants.PROJECTHISTORY_ROLE_MAX));
        projectHistory
            .setSkillsResponsibilities(getTestString(ValidationConstants.PROJECTHISTORY_SKILLSRESPONSIBILITIES_MAX));
        projectHistory
            .setProjectName(getTestString(ValidationConstants.PROJECTHISTORY_PROJECTNAME_MAX));

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        projectHistory
            .setClient(getTestString(ValidationConstants.PROJECTHISTORY_CLIENT_MAX + 1));
        projectHistory
            .setDescription(getTestString(ValidationConstants.PROJECTHISTORY_DESCRIPTION_MAX + 1));
        projectHistory
            .setProjectLength(getTestString(ValidationConstants.PROJECTHISTORY_PROJECTLENGTH_MAX + 1));
        projectHistory
            .setTimeSpent(getTestString(ValidationConstants.PROJECTHISTORY_TIMESPENT_MAX + 1));
        projectHistory
            .setRole(getTestString(ValidationConstants.PROJECTHISTORY_ROLE_MAX + 1));
        projectHistory
            .setSkillsResponsibilities(getTestString(ValidationConstants.PROJECTHISTORY_SKILLSRESPONSIBILITIES_MAX + 1));
        projectHistory
            .setProjectName(getTestString(ValidationConstants.PROJECTHISTORY_PROJECTNAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "client", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "description", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "projectLength", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "timeSpent", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "role", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "skillsResponsibilities", projectHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, projectHistory.getClass()
                .getName(), "projectName", projectHistory.getID())
        };

        verifySaveMustFail(projectHistory, null, expectedExceptions);
    }
    
    public void testInsertSuccess() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
    
    public void testInsertClassificationIndustrySuccess() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
                
        projectHistory.setClassification(new Classification());
        projectHistory.setIndustry(DatafieldFixture
            .createRandomIndustryType());
        
        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
    
    public void testInsertStartDateLargerThanEndDate() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        projectHistory.setStartDate(new GregorianCalendar(2007, 12, 12));
        projectHistory.setEndDate(new GregorianCalendar(2007, 1, 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, projectHistory
                .getClass().getName(), "startDate", projectHistory.getID())
        };

        verifySaveMustFail(projectHistory, null, expectedExceptions);
    }
    
    public void testInsertStartDateNullEndDate() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        projectHistory.setStartDate(new GregorianCalendar());

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
    
    public void testInsertEndDateNullStartDate() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        projectHistory.setEndDate(new GregorianCalendar());

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
    
    public void testInsertDatesSameValues() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        projectHistory.setStartDate(new GregorianCalendar(2007, 1, 1));
        projectHistory.setEndDate(new GregorianCalendar(2007, 1, 1));

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
    
    public void testInsertDatesSuccess() throws AssertionError, Exception
    {
        ProjectHistory projectHistory = new ProjectHistory();
        projectHistory.setParent(ResourceFixture.createSaveFixture(this));
        
        projectHistory.setStartDate(new GregorianCalendar(2007, 1, 1));
        projectHistory.setEndDate(new GregorianCalendar(2007, 12, 12));

        // Attempt to save the ProjectHistory object
        projectHistory = (ProjectHistory) saveMustSucceed(projectHistory,
            ProjectHistory.class, null);

        // Load the inserted project history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ProjectHistory[id='"
            + projectHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ProjectHistory);

        // Check that the loaded project history is identical to the one we
        // requested to store
        ProjectHistoryFixture.compare(projectHistory, (ProjectHistory) rpmObjects[0]);
    }
}
