package com.ibm.rpm.client.junit.resource.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestResourceCostCenterAssignmentValidation extends GenericValidation
{
    private ResourceScope mainScope;

    private Resource mainResource;

    private ResourceCostCenterAssignment[] costCenters;

    private Pool parentPool;

    private Calendar calendar1;

    private Calendar calendar2;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ResourceScope();
        mainScope.setResourceCostCenterAssignments(true);
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, 1, mainScope, this);

        assertNotNull(mainResource.getResourceCostCenterAssignments());
        assertNotNull(mainResource.getResourceCostCenterAssignments()[0]);
        assertNotNull(mainResource.getResourceCostCenterAssignments()[0]
            .getClientCostCenter());

        costCenters = mainResource.getResourceCostCenterAssignments();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        costCenters[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, costCenters[0].getClass().getName(),
                "parent", costCenters[0].getID())
        };

        verifySaveMustFail(costCenters[0], null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        costCenters[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, costCenters[0].getClass()
                .getName(), "parent", costCenters[0].getID())
        };

        verifySaveMustFail(costCenters[0], null, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError, Exception
    {
        costCenters[0].setClientCostCenter(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, costCenters[0].getClass().getName(),
                "clientCostCenter", costCenters[0].getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }
}