package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestResourceRatesValidation extends GenericValidation
{
    private ResourceScope mainScope;

    private Pool parentPool;

    private Resource mainResource;

    private ResourceRates mainRates;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ResourceScope();
        mainScope.setRates(true);
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, mainScope, this);
        mainResource.setParent(parentPool);
        mainRates = mainResource.getRates();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Create ResourceRates with the max values
        mainRates
            .setRegularCostPerHour(new Double(ValidationConstants.RESOURCERATES_MAX));
        mainRates
            .setSellingRatePerHour(new Double(ValidationConstants.RESOURCERATES_MAX));
        mainRates
            .setSpecialCostPerHour(new Double(ValidationConstants.RESOURCERATES_MAX));
        mainRates.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MAX));

        // Attempt to save the resource with ResourceRates
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
            + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getRates());

        // Check that the loaded rates are identical to the ones we requested to
        // store
        ResourceFixture.compare(mainResource, ((Resource) rpmObjects[0]), mainScope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Create ResourceRates with the min values
        mainRates
            .setRegularCostPerHour(new Double(ValidationConstants.RESOURCERATES_MIN));
        mainRates
            .setSellingRatePerHour(new Double(ValidationConstants.RESOURCERATES_MIN));
        mainRates
            .setSpecialCostPerHour(new Double(ValidationConstants.RESOURCERATES_MIN));
        mainRates.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MIN));

        // Attempt to save the resource with ResourceRates
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
            + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getRates());

        // Check that the loaded rates are identical to the ones we requested to
        // store
        ResourceFixture.compare(mainResource, ((Resource) rpmObjects[0]), mainScope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Create ResourceRates with maximum+0.01 values
        mainRates.setRegularCostPerHour(new Double(
            ValidationConstants.RESOURCERATES_MAX + 0.01));
        mainRates.setSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MAX + 0.01));
        mainRates.setSpecialCostPerHour(new Double(
            ValidationConstants.RESOURCERATES_MAX + 0.01));
        mainRates.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MAX + 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "regularCostPerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "sellingRatePerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "specialCostPerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "specialSellingRatePerHour", mainRates.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Create ResourceRates with minimum-0.01 values
        mainRates.setRegularCostPerHour(new Double(
            ValidationConstants.RESOURCERATES_MIN - 0.01));
        mainRates.setSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MIN - 0.01));
        mainRates.setSpecialCostPerHour(new Double(
            ValidationConstants.RESOURCERATES_MIN - 0.01));
        mainRates.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.RESOURCERATES_MIN - 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "regularCostPerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "sellingRatePerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "specialCostPerHour", mainRates.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainRates.getClass().getName(),
                "specialSellingRatePerHour", mainRates.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }
}
