/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.scope.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

/**
 * @author karim
 *  
 */
public class ScopeManagementModuleFixture extends AbstractModuleFixture
{

    public static ScopeManagementModule createFixture(
            ScopeManagementModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        return createModule(scope, ctc);
    }

    public static ScopeManagementModule updateFixture(
            ScopeManagementModule module, ScopeManagementModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static ScopeManagementModule createModule(
            ScopeManagementModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        ScopeManagementModule module = new ScopeManagementModule();

        if(scope!=null)
        {
            if(scope.getScopeFolders()!=null)
            {
                ScopeFolder[] scopeFolders = (ScopeFolder[]) createElements(scope
                        .getScopeFolders(), ScopeFolder.class, ctc);
                    module.setScopeFolders(scopeFolders);                
            }
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                        scope.getAttributeCategories(), AttributeCategory.class, ctc);
                    module.setAttributeCategories(attributeCategories);                
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                module.setCustomFieldCategories(new CustomFieldCategory[]
                {
                    CustomFieldFixture.createCustomFieldCategory(0)
                });                
            }
            if(scope.getScorecardFolders()!=null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) createElements(
                        scope.getScorecardFolders(), ScorecardFolder.class,
                        ctc);
                    module.setScorecardFolders(scorecardFolders);                
            }
        }

        return module;
    }

    private static ScopeManagementModule updateModule(
            ScopeManagementModule module, ScopeManagementModuleScope scope, ClientTestController ctc)
            throws Exception
    {
        if(scope!=null)
        {
            if(scope.getScopeFolders()!=null)
            {
                ScopeFolder[] scopeFolders = (ScopeFolder[]) updateElements(module
                        .getScopeFolders(), scope.getScopeFolders(), ctc);
                    module.setScopeFolders(scopeFolders);
            }
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) updateElements(
                        module.getAttributeCategories(), scope.getAttributeCategories(), ctc);
                module.setAttributeCategories(attributeCategories);                
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                module.setCustomFieldCategories(CustomFieldFixture
                        .modifyCustomFieldCategories(module.getCustomFieldCategories(),
                            scope.getCustomFieldCategories()));
            }
            if(scope.getScorecardFolders()!=null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) updateElements(
                        module.getScorecardFolders(), scope.getScorecardFolders(), ctc);
                    module.setScorecardFolders(scorecardFolders);
            }
        }
        return module;
    }

    public static ScopeManagementModuleScope createFullScope()
    {
        return createScope(true,true,true,true,true,true,true,
	    		true,true,true,true,true,true,true,true,true,true,true,true,true,true,true);
    }
    
    private static ScopeManagementModuleScope createScope(
            boolean isScopeFolders, boolean isAttributeCategories,
            boolean isChangeRequestStates, boolean isIssueStates,
            boolean isRiskStates, boolean isRequirementsStates,
            boolean isDefectStates, boolean isServiceRequestStates,
            boolean isActionStates, boolean isDeliverableRtfs,
            boolean isSummaryTaskRtfs, boolean isMilestoneRfts,
            boolean isTaskRtfs, boolean isRequirementRtfs,
            boolean isChangeRequestRtfs, boolean isServiceRequestRtfs,
            boolean isDefectRtfs, boolean isIssueRfts, boolean isRiskRtfs,
            boolean isActionRtfs, boolean isCustomFields,
            boolean isScorecardFolders)
    {
        ScopeManagementModuleScope scope = new ScopeManagementModuleScope();
 
        if (isScopeFolders)
        {
            scope.setScopeFolders(new ScopeElementScope());
        }
        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(CustomFieldFixture
                .createCustomFieldScope(2));
        }
        if (isChangeRequestStates)
        {
            scope.setChangeRequestStates(true);
        }
        if (isIssueStates)
        {
            scope.setIssueStates(true);
        }
        if (isRiskStates)
        {
            scope.setRiskStates(true);
        }
        if (isRequirementsStates)
        {
            scope.setRequirementStates(true);
        }
        if (isDefectStates)
        {
            scope.setDefectStates(true);
        }
        if (isServiceRequestStates)
        {
            scope.setServiceRequestStates(true);
        }
        if (isActionStates)
        {
            scope.setActionStates(true);
        }
        if (isDeliverableRtfs)
        {
            scope.setDeliverableRtfs(true);
        }
        if (isSummaryTaskRtfs)
        {
            scope.setSummaryTaskRtfs(true);
        }
        if (isMilestoneRfts)
        {
            scope.setMilestoneRtfs(true);
        }
        if (isTaskRtfs)
        {
            scope.setTaskRtfs(true);
        }
        if (isRequirementRtfs)
        {
            scope.setRequirementRtfs(true);
        }
        if (isChangeRequestRtfs)
        {
            scope.setChangeRequestRtfs(true);
        }
        if (isServiceRequestRtfs)
        {
            scope.setServiceRequestRtfs(true);
        }
        if (isDefectRtfs)
        {
            scope.setDefectRtfs(true);
        }
        if (isIssueRfts)
        {
            scope.setIssueRtfs(true);
        }
        if (isRiskRtfs)
        {
            scope.setRiskRtfs(true);
        }
        if (isActionRtfs)
        {
            scope.setActionRtfs(true);
        }
        if (isScorecardFolders)
        {        
            scope.setScorecardFolders(ScorecardFixture.createScope());
        }

        return scope;
    }

    public static void compare(ScopeManagementModule obj1,
            ScopeManagementModule obj2, ScopeManagementModuleScope scope)
            throws AssertionFailedError, Exception
    {
        if(scope!=null)
        {
            if(scope.getScopeFolders()!=null)
            {
                ScopeElementFixture.compare(obj1.getScopeFolders(), obj2
                        .getScopeFolders(), scope.getScopeFolders());
            }
            if(scope.getAttributeCategories()!=null)
            {
                AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                        .getAttributeCategories(), scope.getAttributeCategories());
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                CustomFieldFixture.assertIdEqual(obj1.getCustomFieldCategories(), obj2
                        .getCustomFieldCategories(), scope.getCustomFieldCategories());
            }
            if(scope.getScorecardFolders()!=null)
            {
                ScorecardFixture.compare(obj1.getScorecardFolders(), obj2
                        .getScorecardFolders(), scope.getScorecardFolders());
            }
            if(scope.isChangeRequestStates())
            {
                DatafieldFixture.compare(obj1.getChangeRequestStates(), obj2
                        .getChangeRequestStates());
            }
            if(scope.isIssueStates())
            {
                DatafieldFixture.compare(obj1.getIssueStates(), obj2.getIssueStates());
            }
            if(scope.isRiskStates())
            {
                DatafieldFixture.compare(obj1.getRiskStates(), obj2.getRiskStates());
            }
            if(scope.isRequirementStates())
            {
                DatafieldFixture.compare(obj1.getRequirementStates(), obj2
                        .getRequirementStates());
            }
            if(scope.isDefectStates())
            {
                DatafieldFixture
                .compare(obj1.getDefectStates(), obj2.getDefectStates());
            }
            if(scope.isServiceRequestStates())
            {
                DatafieldFixture.compare(obj1.getServiceRequestStates(), obj2
                        .getServiceRequestStates());
            }
            if(scope.isActionStates())
            {
                DatafieldFixture
                .compare(obj1.getActionStates(), obj2.getActionStates());
            }
            if(scope.isDeliverableRtfs())
            {
                DatafieldFixture.compare(obj1.getDeliverableRtfs(), obj2
                        .getDeliverableRtfs());
            }
            if(scope.isSummaryTaskRtfs())
            {
                DatafieldFixture.compare(obj1.getSummaryTaskRtfs(), obj2
                        .getSummaryTaskRtfs());
            }
            if(scope.isMilestoneRtfs())
            {
                DatafieldFixture.compare(obj1.getMilestoneRtfs(), obj2
                        .getMilestoneRtfs());
            }
            if(scope.isTaskRtfs())
            {
                DatafieldFixture.compare(obj1.getTaskRtfs(), obj2.getTaskRtfs());
            }
            if(scope.isRequirementRtfs())
            {
                DatafieldFixture.compare(obj1.getRequirementRtfs(), obj2
                        .getRequirementRtfs());
            }
            if(scope.isChangeRequestRtfs())
            {
                DatafieldFixture.compare(obj1.getChangeRequestRtfs(), obj2
                        .getChangeRequestRtfs());
            }
            if(scope.isServiceRequestRtfs())
            {
                DatafieldFixture.compare(obj1.getServiceRequestRtfs(), obj2
                        .getServiceRequestRtfs());
            }
            if(scope.isDefectRtfs())
            {
                DatafieldFixture.compare(obj1.getDefectRtfs(), obj2.getDefectRtfs());
            }
            if(scope.isIssueRtfs())
            {
                DatafieldFixture.compare(obj1.getIssueRtfs(), obj2.getIssueRtfs());
            }
            if(scope.isRiskRtfs())
            {
                DatafieldFixture.compare(obj1.getRiskRtfs(), obj2.getRiskRtfs());
            }
            if(scope.isActionRtfs())
            {
                DatafieldFixture.compare(obj1.getActionRtfs(), obj2.getActionRtfs());
            }
        }  
    }
}
