/*
 * Created on Sep 28, 2006
 *
 */
package com.ibm.rpm.client.junit.scopemanagement.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author karim
 *
 */
public class ReqProIntegrationFolderFixture extends RPMObjectFixture
{
	private static int DEFAULT_NB_REQPROSCOPEFOLDERS_BY_LEVEL = 1;
	
    protected static ArrayList localSkipGetters = new ArrayList();
    protected static ArrayList localSkipSetters = new ArrayList();
    static
    {
        //set skipGetters
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getTargetDate");
        localSkipGetters.add("getCreateNewWhenUpdated");
        localSkipGetters.add("getDeleteOnServerRemoval");  
        localSkipGetters.add("getWbsSortingRank");
        localSkipGetters.add("getParentSortingRank");
        localSkipGetters.add("getTemplateSortingRank");
        
        //set skipSetters
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setTargetDate");
        localSkipSetters.add("setCreateNewWhenUpdated");
        localSkipSetters.add("setDeleteOnServerRemoval");
        localSkipSetters.add("setWbsSortingRank");
        localSkipSetters.add("setParentSortingRank");
        localSkipSetters.add("setTemplateSortingRank");
        
    }
	
    public static ReqProIntegrationFolder createFixture(ScopeFolder parent,
    		ScopeElementScope scope) throws Exception
    {
        return createFixture(parent, DEFAULT_NB_REQPROSCOPEFOLDERS_BY_LEVEL, scope);
    }
	
    public static ReqProIntegrationFolder createFixture(ScopeFolder parent, int nbReqProScopeFoldersByLevel ,
    		ScopeElementScope scope) throws Exception
    {
        return createReqProIntegrationFolder(parent, random(), nbReqProScopeFoldersByLevel, scope);
    }
	
    public static ReqProIntegrationFolder updateFixture(ReqProIntegrationFolder reqProIntegrationFolder,
    		ScopeElementScope scope) throws Exception
    {
        return updateReqProIntegrationFolder(reqProIntegrationFolder, random(), scope);
    }
    
    private static ReqProIntegrationFolder createReqProIntegrationFolder(ScopeFolder parent, int index, int nbReqProScopeFoldersByLevel,
			ScopeElementScope scope) throws Exception
    {
    	ReqProIntegrationFolder reqProIntegrationFolder = new ReqProIntegrationFolder();

    	HelperTestUtilities.setByIntrospection(reqProIntegrationFolder, index, localSkipSetters);
    	
    	reqProIntegrationFolder.setParent(parent);

    	if(scope!=null)
    	{
    		if(scope.getReqProData()!=null)
    		{
    			ReqProConnection reqProConnection = ReqProConnectionFixture.createFixture(reqProIntegrationFolder);
    			reqProIntegrationFolder.setReqProConnection(reqProConnection);
    			
    		}
            if (scope.getChildren() != null)
            {
            	ReqProScopeFolder[] children = new ReqProScopeFolder[nbReqProScopeFoldersByLevel];
                for(int i=0;i<nbReqProScopeFoldersByLevel;i++)
                {
                	children[i] = ReqProScopeFolderFixture.createFixture(reqProIntegrationFolder, scope.getChildren());
                }
                reqProIntegrationFolder.setChildren(children);
            }            
    	}
        return reqProIntegrationFolder;
    }

    private static ReqProIntegrationFolder updateReqProIntegrationFolder(ReqProIntegrationFolder reqProIntegrationFolder, int index,
			ScopeElementScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(reqProIntegrationFolder, index, localSkipSetters);
    	
    	if(scope!=null)
    	{
    		if(scope.getReqProData()!=null)
    		{
    			ReqProConnectionFixture.updateFixture(reqProIntegrationFolder.getReqProConnection());
    			
    		}
            if (scope.getChildren() != null)
            {
            	ScopeElement[] children = reqProIntegrationFolder.getChildren();
            	if(children!=null)
            	{
                    for(int i=0;i<children.length;i++)
                    {
                    	if(children[i] instanceof ReqProScopeFolder)
                    	{
                    		ReqProScopeFolderFixture.updateFixture((ReqProScopeFolder)children[i], scope.getChildren());                    		
                    	}
                    }
            	}
            }            
    	}
        return reqProIntegrationFolder;
    }
    
    
//    public static ReqProIntegrationFolder updateReqProIntegrationFolderStructure(ReqProIntegrationFolder reqProIntegrationFolder, int index, int deltaReqProScopeFoldersByLevel,
//			ScopeElementScope scope) throws Exception
//    {
//    	String id = reqProIntegrationFolder.getID();
//    	int nbReqProScopeFoldersByLevel = (reqProIntegrationFolder.getChildren()!=null)?reqProIntegrationFolder.getChildren().length:0;
//    	int newNbReqProScopeFoldersByLevel = nbReqProScopeFoldersByLevel + deltaReqProScopeFoldersByLevel;
//    	
//    	ReqProIntegrationFolder updatedReqProIntegrationFolder = createReqProIntegrationFolder((ScopeFolder)reqProIntegrationFolder.getParent(),index,newNbReqProScopeFoldersByLevel,scope);
//    	u
//    	
//    	
//    	return updatedReqProIntegrationFolder;
//    }
    
    public static void compare(ReqProIntegrationFolder f1, ReqProIntegrationFolder f2,
    		ScopeElementScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(f1, f2, localSkipGetters);
        if (scope == null)
            return;
        
        if(scope.getReqProData()!=null)
        {
            ReqProConnectionFixture.compare(f1.getReqProConnection(), f2.getReqProConnection());
        }
        
        if(scope.getChildren()!=null)
        {
        	ReqProScopeFolder[] subFolders1 = getReqProScopeFolders(f1.getChildren());
        	ReqProScopeFolder[] subFolders2 = getReqProScopeFolders(f2.getChildren());
        	ReqProScopeFolderFixture.compare(subFolders1, subFolders2, scope.getChildren());
        }
    }
    
    private static ReqProScopeFolder[] getReqProScopeFolders(ScopeElement[] scopeElements)
    {
    	ReqProScopeFolder[] folders = null;
    	if(scopeElements!=null || scopeElements.length!=0)
    	{
    		ArrayList foldersList = new ArrayList();
    		for(int i=0;i<scopeElements.length;i++)
    		{
    			if(scopeElements[i] instanceof ReqProScopeFolder)
    			{
    				foldersList.add(scopeElements[i]);
    			}
    		}
    		if(foldersList.size()!=0)
    		{
    			folders = new ReqProScopeFolder[foldersList.size()];
    			foldersList.toArray(folders);
    		}
    	}
    	return folders;
    }
    
    public static ScopeElementScope getFullScope()
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setReqProData(new RPMObjectScope());
        scope.setChildren(scope);
        return scope;
    }
}
