/*
 * Created on 5-Jun-2006
 * @author mmatwiy
 * Test conversion of ScopeElement containers.
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Proposal;

public class TestScopeElementConversion extends ClientTestController
{
    ScopeFolder scopeFolder;

    public void setUp() throws Exception
    {
        super.setUp();

        // create a Proposal and save it; this will create a ScopeFolder
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        scopeFolder = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testConvertIssueToRisk() throws Exception
    {
        Issue issue = new Issue();
        issue.setParent(scopeFolder);
        issue.setName(HelperTestUtilities.createName());
        save(issue, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), issue, new Risk(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Risk);
        assertEquals(result.getRpmObject().getID(), issue.getID());
    }

    public void testConvertRequirementToDefect() throws Exception
    {
        Requirement requirement = new Requirement();
        requirement.setParent(scopeFolder);
        requirement.setName(HelperTestUtilities.createName());
        save(requirement, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), requirement, new Defect(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertEquals(result.getRpmObject().getID(), requirement.getID());
    }

    public void testConvertChangeRequestToIssue() throws Exception
    {
        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setParent(scopeFolder);
        changeRequest.setName(HelperTestUtilities.createName());
        save(changeRequest, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), changeRequest, new Issue(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertEquals(result.getRpmObject().getID(), changeRequest.getID());
    }

    public void testConvertServiceRequestToChangeRequest() throws Exception
    {
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setParent(scopeFolder);
        serviceRequest.setName(HelperTestUtilities.createName());
        save(serviceRequest, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), serviceRequest, new ChangeRequest(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof ChangeRequest);
        assertEquals(result.getRpmObject().getID(), serviceRequest.getID());
    }

    public void testConvertActionToServiceRequest() throws Exception
    {
        Action action = new Action();
        action.setParent(scopeFolder);
        action.setName(HelperTestUtilities.createName());
        save(action, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), action, new ServiceRequest(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof ServiceRequest);
        assertEquals(result.getRpmObject().getID(), action.getID());
    }

    public void testConvertRiskToRequirement() throws Exception
    {
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), risk, new Requirement(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Requirement);
        assertEquals(result.getRpmObject().getID(), risk.getID());
    }

    public void testConvertDefectToAction() throws Exception
    {
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setName(HelperTestUtilities.createName());
        save(defect, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), defect, new Action(), null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Action);
        assertEquals(result.getRpmObject().getID(), defect.getID());
    }
    
    public void testConvertDefectToSavedRisk() throws Exception
    {
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setName(HelperTestUtilities.createName());
        save(defect, null);
        
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), defect, risk, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Risk);
        assertEquals(result.getRpmObject().getID(), defect.getID());
    }

    public void testConvertDefectToTaskBackToDefect() throws Exception
    {
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setName(HelperTestUtilities.createName());
        defect = (Defect) save(defect, null).getRpmObject();

        // convert the defect to a DefectTask
        defect.setTask(new Boolean(true));
        SingleResult result = save(defect, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertEquals(result.getRpmObject().getID(), defect.getID());

        // make sure the task flag is set
        Defect defectTask = (Defect) result.getRpmObject();
        assertTrue(defectTask.getTask().booleanValue());

        // reload the defectTask container
        RPMObject[] rpmObjects = loadFromXpath("/Defect[id='"
                + defectTask.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Defect);
        assertTrue(defectTask.getTask().booleanValue());

        // convert it back to a Defect
        defectTask.setTask(new Boolean(false));
        result = save(defectTask, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertEquals(result.getRpmObject().getID(), defectTask.getID());

        // make sure the task flag is false
        Defect convertedDefect = (Defect) result.getRpmObject();
        assertFalse(convertedDefect.getTask().booleanValue());
    }

    public void testConvertChangeRequestToTaskBackToChangeRequest()
            throws Exception
    {
        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setParent(scopeFolder);
        changeRequest.setName(HelperTestUtilities.createName());
        changeRequest = (ChangeRequest) save(changeRequest, null)
            .getRpmObject();

        // convert the ChangeRequest to a ChangeRequestTask
        changeRequest.setTask(new Boolean(true));
        SingleResult result = save(changeRequest, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof ChangeRequest);
        assertEquals(result.getRpmObject().getID(), changeRequest.getID());

        // make sure the task flag is set
        ChangeRequest changeRequestTask = (ChangeRequest) result.getRpmObject();
        assertTrue(changeRequestTask.getTask().booleanValue());

        // reload the changeRequestTask container
        RPMObject[] rpmObjects = loadFromXpath("/ChangeRequest[id='"
                + changeRequestTask.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ChangeRequest);
        assertTrue(changeRequestTask.getTask().booleanValue());

        // convert it back to a ChangeRequest
        changeRequestTask.setTask(new Boolean(false));
        result = save(changeRequestTask, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof ChangeRequest);
        assertEquals(result.getRpmObject().getID(), changeRequestTask.getID());

        // make sure the task flag is false
        ChangeRequest convertedChangeRequest = (ChangeRequest) result
            .getRpmObject();
        assertFalse(convertedChangeRequest.getTask().booleanValue());
    }

    public void testConvertIssueToIssueTaskToRiskTask() throws Exception
    {
        Issue issue = new Issue();
        issue.setParent(scopeFolder);
        issue.setName(HelperTestUtilities.createName());
        issue = (Issue) save(issue, null).getRpmObject();

        // convert the Issue to a IssueTask
        issue.setTask(new Boolean(true));
        SingleResult result = save(issue, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertEquals(result.getRpmObject().getID(), issue.getID());

        // make sure the task flag is set
        Issue issueTask = (Issue) result.getRpmObject();
        assertTrue(issueTask.getTask().booleanValue());

        // reload the issueTask container
        RPMObject[] rpmObjects = loadFromXpath("/Issue[id='"
                + issueTask.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Issue);
        assertTrue(issueTask.getTask().booleanValue());

        // convert the Issue Task container to a Risk Task
        Risk riskTask = new Risk();
        riskTask.setTask(new Boolean(true));

        SingleResult convertResult = getApplicationInterface()
            .convertContainer(getSessionID(), issueTask, riskTask, null);
        
        // check that the conversion worked and that the task flag is set
        assertTrue(convertResult.isSuccessful());
        assertTrue(convertResult.isWarningFree());
        assertTrue(convertResult.getRpmObject() instanceof Risk);
        assertEquals(convertResult.getRpmObject().getID(), issue.getID());

        Risk outputRiskTask = (Risk) convertResult.getRpmObject();
        assertTrue(outputRiskTask.getTask().booleanValue());
    }
    
    public void testConvertActionToActionTaskToRequirementTask() throws Exception
    {
        Action action = new Action();
        action.setParent(scopeFolder);
        action.setName(HelperTestUtilities.createName());
        action = (Action) save(action, null).getRpmObject();

        // convert the Action to a ActionTask
        action.setTask(new Boolean(true));
        SingleResult result = save(action, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof Action);
        assertEquals(result.getRpmObject().getID(), action.getID());

        // make sure the task flag is set
        Action actionTask = (Action) result.getRpmObject();
        assertTrue(actionTask.getTask().booleanValue());

        // reload the actionTask container
        RPMObject[] rpmObjects = loadFromXpath("/Action[id='"
                + actionTask.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Action);
        assertTrue(actionTask.getTask().booleanValue());

        // convert the Action Task container to a Requirement Task
        Requirement requirementTask = new Requirement();
        requirementTask.setTask(new Boolean(true));

        SingleResult convertResult = getApplicationInterface()
            .convertContainer(getSessionID(), actionTask, requirementTask, null);
        
        // check that the conversion worked and that the task flag is set
        assertTrue(convertResult.isSuccessful());
        assertTrue(convertResult.isWarningFree());
        assertTrue(convertResult.getRpmObject() instanceof Requirement);
        assertEquals(convertResult.getRpmObject().getID(), action.getID());

        Requirement outputRequirementTask = (Requirement) convertResult.getRpmObject();
        assertTrue(outputRequirementTask.getTask().booleanValue());
    }
    
    public void testCreateServiceRequestTask() throws Exception
    {
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setParent(scopeFolder);
        serviceRequest.setName(HelperTestUtilities.createName());
        serviceRequest.setTask(new Boolean(true));
        SingleResult result = save(serviceRequest, null);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertTrue(result.getRpmObject() instanceof ServiceRequest);

        // make sure the task flag is set
        ServiceRequest serviceRequestTask = (ServiceRequest) result.getRpmObject();
        assertTrue(serviceRequestTask.getTask().booleanValue());

        // reload the serviceRequestTask container
        RPMObject[] rpmObjects = loadFromXpath("/ServiceRequest[id='"
                + serviceRequestTask.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ServiceRequest);
        assertTrue(serviceRequestTask.getTask().booleanValue());

     }

}