package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestScopeFinancials extends ClientTestController
{
    protected ScopeElementScope mainScope;

    protected ScopeFolder mainFolder = null;

    protected WorkElementScope proposalScope;

    protected Proposal proposal = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        proposalScope = new WorkElementScope();

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        
        SaveResult result = null;
        result = save(proposal, new WorkElementScope());
        assertNotNull(result);
        assertTrue(result.isSuccessful());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        mainFolder = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        delete(proposal);
        super.tearDown();
    }
    
    private WorksheetFinancial getMatchingWorksheetinancial(String chargecode,
            WorksheetFinancial[] financial)
    {
        int i = -1;
        int index = -1;

        WorksheetFinancial wrongWorksheet = new WorksheetFinancial();
        wrongWorksheet.setID("");

        if (chargecode == null || financial == null)
        {
            return wrongWorksheet;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode().getID().startsWith(chargecode))
                {
                    index = i;
                    break;
                }
            }
            return financial[index];
        }
    }

    
    public void testSaveScopeFinancials() throws AssertionError,
            Exception
    {
        //1. Saving a Defect Task without Financials : should be OK
        
        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(mainFolder);

        SaveResult result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        //2. Adding Financials : should be OK
        
        mainScope.setWorksheetFinancials(new FinancialScope());
        mainScope.getWorksheetFinancials().setChargeCode(true);
        
        defect = (Defect) result.getRpmObject();
        
        WorksheetFinancial[] wfFinancials = ScopeElementFixture
        .createEmptyWorksheetFinancialSystemChargeCodesArray(defect, this);

        defect.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNotNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());
        
        //3. Set Defect to Task : should be OK
        
        defect = (Defect) result.getRpmObject();

        defect.setTask(new Boolean(true));
        
        result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNotNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());

    }

    
    public void testSaveScopeFinancialsDefectTask_CreateNoScope()
            throws AssertionError, Exception
    {

        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(mainFolder);

        defect.setTask(new Boolean(true));

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(defect, this);

        defect.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertTrue(result.isSuccessful());

    }

    public void testSaveScopeFinancialsDefectTask_UpdateNoScope()
            throws AssertionError, Exception
    {
        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(mainFolder);

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(defect, this);

        defect.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertTrue(result.isSuccessful());

        // ---Updating Financials on Task
        defect = ((Defect) result.getRpmObject());

        expenseHourWorksheet = getMatchingWorksheetinancial("EXPENSE_HOURS",
            defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(400));

        defect.setTask(new Boolean(true));

        result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertTrue(result.isSuccessful());
    }
    
    public void testSaveScopeFinancialsDefect_Create() throws AssertionError,
            Exception
    {
        mainScope.setWorksheetFinancials(new FinancialScope());
        mainScope.getWorksheetFinancials().setChargeCode(true);

        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(mainFolder);

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(defect, this);

        defect.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNotNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());
    }
    
    public void testSaveScopeFinancialsDefect_Update() throws AssertionError,
            Exception
    {
        mainScope.setWorksheetFinancials(new FinancialScope());
        mainScope.getWorksheetFinancials().setChargeCode(true);

        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(mainFolder);
        

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(defect, this);

        defect.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNotNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());
        
        //---Updating Financials on Task
        defect = (Defect) result.getRpmObject();
        
        expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", defect.getWorksheetFinancials());
        
        expenseHourWorksheet.setActual(new Double(400));

        result = null;
        result = save(defect, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Defect);
        assertNotNull(((Defect) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());
        
        
    }

    
    
    
    
  
    
    
    
    
    public void testSaveScopeFinancialsIssueTask_CreateNoScope()
            throws AssertionError, Exception
    {

        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(mainFolder);

        issue.setTask(new Boolean(true));

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(issue, this);

        issue.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertTrue(result.isSuccessful());

    }

    public void testSaveScopeFinancialsIssueTask_UpdateNoScope()
            throws AssertionError, Exception
    {
        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(mainFolder);

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(issue, this);

        issue.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertTrue(result.isSuccessful());

        // ---Updating Financials on Task
        issue = ((Issue) result.getRpmObject());

        expenseHourWorksheet = getMatchingWorksheetinancial("EXPENSE_HOURS",
            issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(400));

        issue.setTask(new Boolean(true));

        result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertTrue(result.isSuccessful());
    }

    public void testSaveScopeFinancialsIssue_Create() throws AssertionError,
            Exception
    {
        mainScope.setWorksheetFinancials(new FinancialScope());
        mainScope.getWorksheetFinancials().setChargeCode(true);

        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(mainFolder);

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(issue, this);

        issue.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertNotNull(((Issue) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());
    }

    public void testSaveScopeFinancialsIssue_Update() throws AssertionError,
            Exception
    {
        mainScope.setWorksheetFinancials(new FinancialScope());
        mainScope.getWorksheetFinancials().setChargeCode(true);

        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(mainFolder);

        WorksheetFinancial[] wfFinancials = ScopeElementFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(issue, this);

        issue.setWorksheetFinancials(wfFinancials);

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(200));

        SaveResult result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertNotNull(((Issue) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        //---Updating Financials on Task
        issue = (Issue) result.getRpmObject();

        expenseHourWorksheet = getMatchingWorksheetinancial("EXPENSE_HOURS",
            issue.getWorksheetFinancials());

        expenseHourWorksheet.setActual(new Double(400));

        result = null;
        result = save(issue, mainScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Issue);
        assertNotNull(((Issue) result.getRpmObject()).getWorksheetFinancials());
        assertTrue(result.isSuccessful());

    }
    
}
