/*
 * Created on Oct 24, 2006
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestScopeParentSwitch extends ClientTestController
{
    protected ScopeElementScope mainScope;

    protected ScopeFolder mainFolder = null;

    protected WorkElementScope proposalScope;

    protected Proposal proposal = null;

    protected int nbChildren = 1;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        proposalScope = new WorkElementScope();

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        save(proposal, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        mainFolder = (ScopeFolder) rpmObjects[0];
        ScopeElementFixture.createParentSwitchFixture(mainScope, mainFolder,
            this);
        SingleResult result = save(mainFolder, mainScope,
            ReloadType.SavedResult);
        mainFolder = (ScopeFolder) result.getRpmObject();
    }
    
    public void testMoveTaskAwayFromProposal() throws Exception
    {
        proposalScope = new WorkElementScope();

        Proposal newProposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        save(newProposal, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + newProposal.getName() + "']", new ScopeElementScope());
        ScopeFolder newFolder = (ScopeFolder) rpmObjects[0];
        
        ScopeTask task = (ScopeTask) findClass(ScopeTask.class, mainFolder
            .getChildren());
        
        moveBeforeSiblingMustFail(task, newFolder);
    }
    
    public void testMoveRequirementUnderAnotherProposal() throws Exception
    {
        proposalScope = new WorkElementScope();

        Proposal newProposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        save(newProposal, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + newProposal.getName() + "']", new ScopeElementScope());
        ScopeFolder newFolder = (ScopeFolder) rpmObjects[0];
        
        Requirement req = (Requirement) findClass(Requirement.class, mainFolder
            .getChildren());
        
        validateNotParentChild(newFolder.getID(), req);
        req.setParent(newFolder);        
        saveMustFail(req, mainScope);
        return;
    }

    public void testMoveFolderUnderReq() throws AssertionError, Exception
    {
        ScopeFolder folder = (ScopeFolder) findClass(ScopeFolder.class,
            mainFolder.getChildren());
        Requirement req = (Requirement) findClass(Requirement.class, mainFolder
            .getChildren());
        req.setChildren(new ScopeElement[]
        {
            folder
        });
        folder.setParent(req);

        validateNotParentChild(req.getID(), folder);
        save(folder, mainScope, ReloadType.SavedResult);
        validateParentChild(req.getID(), folder);
    }

    public void testMoveFolderUnderDeliverable() throws AssertionError,
            Exception
    {
        ScopeFolder folder = (ScopeFolder) findClass(ScopeFolder.class,
            mainFolder.getChildren());
        ScopeDeliverable del = (ScopeDeliverable) findClass(
            ScopeDeliverable.class, mainFolder.getChildren());
        del.setChildren(new ScopeElement[]
        {
            folder
        });
        folder.setParent(del);

        validateNotParentChild(del.getID(), folder);
        saveMustFail(folder, mainScope);
        validateNotParentChild(del.getID(), folder);
    }

    public void testMoveReqUnderDeliverable() throws AssertionError, Exception
    {
        Requirement req = (Requirement) findClass(Requirement.class, mainFolder
            .getChildren());
        ScopeDeliverable del = (ScopeDeliverable) findClass(
            ScopeDeliverable.class, mainFolder.getChildren());
        del.setChildren(new ScopeElement[]
        {
            req
        });
        req.setParent(del);

        validateNotParentChild(del.getID(), req);
        saveMustFail(req, mainScope);
        validateNotParentChild(del.getID(), req);
    }

    public void testMoveTaskUnderFolder() throws AssertionError, Exception
    {
        ScopeFolder folder = (ScopeFolder) findClass(ScopeFolder.class,
            mainFolder.getChildren());
        ScopeTask task = (ScopeTask) findClass(ScopeTask.class, mainFolder
            .getChildren());
        folder.setChildren(new ScopeElement[]
        {
            task
        });
        task.setParent(folder);

        validateNotParentChild(folder.getID(), task);
        saveMustFail(task, mainScope);
        validateNotParentChild(folder.getID(), task);
    }
    
    public void testMoveTaskBeforeFolder() throws Exception
    {
        ScopeFolder folder = (ScopeFolder) findClass(ScopeFolder.class,
            mainFolder.getChildren());
        ScopeTask task = (ScopeTask) findClass(
            ScopeTask.class, mainFolder.getChildren());
        validateNotParentChild(mainFolder.getID(), task);
        moveBeforeSibling(task, folder);
        validateParentChild(mainFolder.getID(), task);
    }

    public void testMoveDeliverableUnderFolder() throws AssertionError,
            Exception
    {
        ScopeFolder folder = (ScopeFolder) findClass(ScopeFolder.class,
            mainFolder.getChildren());
        ScopeDeliverable del = (ScopeDeliverable) findClass(
            ScopeDeliverable.class, mainFolder.getChildren());
        folder.setChildren(new ScopeElement[]
        {
            del
        });
        del.setParent(folder);

        validateNotParentChild(folder.getID(), del);
        saveMustFail(del, mainScope);
        validateNotParentChild(folder.getID(), del);
    }

    private ScopeElement findClass(Class clazz, ScopeElement[] scopeElements)
    {
        for (int i = 0; i < scopeElements.length; i++)
        {
            if (scopeElements[i].getClass().equals(clazz))
            {
                return scopeElements[i];
            }

            if (scopeElements[i] instanceof AggregateScope)
            {
                ScopeElement found = findClass(clazz,
                    ((AggregateScope) scopeElements[i]).getChildren());
                if (found != null)
                {
                    return found;
                }
            }
        }
        return null;
    }

    private void validateParentChild(String parentID, RPMObject child)
            throws Exception
    {
        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());
        RPMObject loadedRPMObject = loadFromID(child, scope);
        assertTrue("Not the expected parent", loadedRPMObject.getParent()
            .getID().equals(parentID));
    }

    private void validateNotParentChild(String parentID, RPMObject child)
            throws Exception
    {
        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());
        RPMObject loadedRPMObject = loadFromID(child, scope);
        assertFalse("Not the expected parent", loadedRPMObject.getParent()
            .getID().equals(parentID));
    }
}
