package com.ibm.rpm.client.junit.scopemanagement.testcases.fixes;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

public class Test62103 extends ClientTestController
{
    Defect defect;   
    
    protected void setUp() throws Exception
    {
        super.setUp();
        
        // Create a requirement
        ScopeElementScope scopeScope = new ScopeElementScope();
        scopeScope.setChildren(new ScopeElementScope());
        ScopeFolder parentFolder = (ScopeFolder) 
            ScopeElementFixture.createFixture(scopeScope, this, 1);
        assertNotNull(parentFolder.getChildren());
        assertEquals(1, parentFolder.getChildren().length);
        
        parentFolder = (ScopeFolder) save(parentFolder, scopeScope).getRpmObject();

        ScopeElementScope insertScope = new ScopeElementScope();
        insertScope.setAsset(true);
        insertScope.setClientCostCenter(true);
        insertScope.setParent(new RPMObjectScope());
        
        defect = ScopeElementFixture.createDefectFixture(
            insertScope, 1, parentFolder, this);
        
        defect = (Defect) save(defect, insertScope).getRpmObject();
    }

    public void testUpdateClientCostCenter() throws Exception
    {
        ClientCostCenter ccc1 = ClientCostCenterFixture.createSaveFixture(this);
        ClientCostCenter ccc = ccc1;
        
        defect.setClientCostCenter(ccc);
        
        ScopeElementScope insertScope = new ScopeElementScope();
        insertScope.setClientCostCenter(true);
        defect = (Defect) save(defect, insertScope).getRpmObject();
        
        ClientCostCenter afterSave = defect.getClientCostCenter();
        
        ClientCostCenterFixture.compare(ccc, afterSave, new ClientCostCenterScope());
    }

    public void testUpdateAsset() throws Exception, AssertionFailedError
    {
        ScopeElementScope updateScope = new ScopeElementScope();
        updateScope.setAsset(true);
        Asset newAsset1 = AssetFixture.createFixture(new AssetScope(), this);
        newAsset1 = (Asset) save(newAsset1, new AssetScope()).getRpmObject();
        
        Asset newAsset = newAsset1;
        defect.setAsset(newAsset);
        
        Defect afterSave = (Defect) save(defect, updateScope).getRpmObject();
        Defect reloadedDefect = (Defect) loadFromID(afterSave, updateScope);
        assertNotNull(reloadedDefect);
        Asset assetAftersave = reloadedDefect.getAsset();
        
        AssetFixture.compare(newAsset, assetAftersave, new AssetScope());
    }
}
