package com.ibm.rpm.client.junit.scopemanagement.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.wbs.containers.Proposal;

public class Test63997 extends ClientTestController {
	
	private Proposal proposal = null;
	
	private ScopeFolder parent = null;
	
	protected void setUp() throws Exception
    {
        super.setUp();
        proposal = createProposal();
        parent = loadRootScopeFolder();
	}
	
	public void testSaveScopeElementWithProposedByNameAsNull() throws Exception
	{
		Calendar calendar = Calendar.getInstance();
		
		AbstractScope scopeElement = saveScopeElementWithCalendars(calendar,null,null);
		Calendar byDate = scopeElement.getProposedByDate();
		
		String currentUserFullName = getCurrentUser().getFullName();
		
		assertNotNull(scopeElement);
		assertEquals(currentUserFullName, scopeElement.getProposedByName());
		validateByDate(calendar,byDate);
	}
	
	public void testSaveScopeElementWithApprovedByNameAsNull() throws Exception
	{
		Calendar calendar = Calendar.getInstance();
		
		AbstractScope scopeElement = saveScopeElementWithCalendars(null,calendar,null);
		Calendar byDate = scopeElement.getApprovedByDate();

		String currentUserFullName = getCurrentUser().getFullName();
		
		assertNotNull(scopeElement);
		assertEquals(currentUserFullName, scopeElement.getApprovedByName());
		validateByDate(calendar,byDate);
	}
	
	public void testSaveScopeElementWithClosedByNameAsNull() throws Exception
	{
		Calendar calendar = Calendar.getInstance();
		
		AbstractScope scopeElement = saveScopeElementWithCalendars(null,null,calendar);
		Calendar byDate = scopeElement.getClosedByDate();
		
		String currentUserFullName = getCurrentUser().getFullName();
		
		assertNotNull(scopeElement);
		assertEquals(currentUserFullName, scopeElement.getClosedByName());
		validateByDate(calendar,byDate);
	}
	
	public void testSaveAbstractScopeElementWithResourceNames() throws Exception
	{
		saveAndValidateAbstractScopeElementWithResourceNames(true);
	}
	
	public void testSaveAbstractScopeElementWithInvalidResourceNames() throws Exception
	{
		saveAndValidateAbstractScopeElementWithResourceNames(false);
	}
	
	public void testUpdateAbstractScopeElementWithNullResourceNames() throws Exception
	{
		AbstractScope scopeElement = saveAndValidateAbstractScopeElementWithResourceNames(true);
		
		scopeElement.setProposedByName(null);
		scopeElement.setApprovedByName(null);
		scopeElement.setClosedByName(null);
		
		scopeElement = (AbstractScope)save(scopeElement,null).getRpmObject();
		
		assertNotNull(scopeElement);
		assertNotNull(scopeElement.getProposedByDate());
		assertNotNull(scopeElement.getApprovedByDate());
		assertNotNull(scopeElement.getClosedByDate());
		assertNull(scopeElement.getProposedByName());
		assertNull(scopeElement.getApprovedByName());
		assertNull(scopeElement.getClosedByName());
	}
	
	public void testUpdateAbstractScopeElementWithNullCalendars() throws Exception
	{
		AbstractScope scopeElement = saveAndValidateAbstractScopeElementWithResourceNames(true);
		
		scopeElement.setProposedByDate(null);
		scopeElement.setApprovedByDate(null);
		scopeElement.setClosedByDate(null);
		
		scopeElement = (AbstractScope)save(scopeElement,null).getRpmObject();
		
		assertNotNull(scopeElement);
		assertNull(scopeElement.getProposedByDate());
		assertNull(scopeElement.getApprovedByDate());
		assertNull(scopeElement.getClosedByDate());
		assertNotNull(scopeElement.getProposedByName());
		assertNotNull(scopeElement.getApprovedByName());
		assertNotNull(scopeElement.getClosedByName());
	}
	
	public void testLoadAbstractScopeElementResourceNames() throws Exception
	{
		AbstractScope scopeElement = saveAndValidateAbstractScopeElementWithResourceNames(false);
		
		String xPath = "/AbstractScope [ProposedByName = '" + scopeElement.getProposedByName() +
						"' and ApprovedByName = '" + scopeElement.getApprovedByName() + 
						"' and ClosedbyName = '" + scopeElement.getClosedByName() +"']";
		RPMObject rpmObject[]  = loadFromXpath(xPath, null);
		
		assertNotNull(rpmObject);
		assertTrue(rpmObject.length > 0);
		
		boolean found = false;
		for(int i=0; i<rpmObject.length && !found; i++)
		{
			if(scopeElement.getID().equals(rpmObject[i].getID()))
			{
				found = true;
			}
		}
		
		assertEquals(true, found);
	}
	
	private AbstractScope saveAndValidateAbstractScopeElementWithResourceNames(boolean isValidResourceName) throws Exception
	{
		String proposedByResourceName;
		String approvedByResourceName;
		String closedByResourceName;
		
		if(isValidResourceName)
		{
			Resource proposedByResource = createAndSaveResource();
			Resource approvedByResource = createAndSaveResource();
			Resource closedByResource = createAndSaveResource();
			
			proposedByResourceName = proposedByResource.getFullName();
			approvedByResourceName = approvedByResource.getFullName();
			closedByResourceName = closedByResource.getFullName();
		}
		else
		{
			proposedByResourceName = HelperTestUtilities.createName();
			approvedByResourceName = HelperTestUtilities.createName();
			closedByResourceName = HelperTestUtilities.createName();
		}
		
		AbstractScope scopeElement = saveAbstractScopeElementWithResourceNames(proposedByResourceName, approvedByResourceName, closedByResourceName);
		
		assertNotNull(scopeElement);
		assertEquals(proposedByResourceName, scopeElement.getProposedByName());
		assertEquals(approvedByResourceName, scopeElement.getApprovedByName());
		assertEquals(closedByResourceName, scopeElement.getClosedByName());
		assertNotNull(scopeElement.getProposedByDate());
		assertNotNull(scopeElement.getApprovedByDate());
		assertNotNull(scopeElement.getClosedByDate());
	
		return scopeElement;
		
	}
	
	private AbstractScope saveScopeElementWithCalendars(Calendar proposedByDate, Calendar approvedByDate, Calendar closedByDate) throws Exception
	{
		Defect defect = new Defect();
		defect.setName(HelperTestUtilities.createName());
		defect.setParent(parent);
		defect.setProposedByDate(proposedByDate);
		defect.setApprovedByDate(approvedByDate);
		defect.setClosedByDate(closedByDate);
		
		defect = (Defect)save(defect,null).getRpmObject();
		
		return defect;
	}
	
	private Proposal createProposal() throws Exception
	{
		Proposal proposal = new Proposal();
		proposal.setName(HelperTestUtilities.createName());
		
		proposal = (Proposal)save(proposal, null).getRpmObject();
		
		assertNotNull(proposal);
		
		return proposal;
	}
	
	private Resource createAndSaveResource() throws Exception
	{
		Pool parentPool = PoolFixture.retrieveStaticFixture(this);
		Resource resource = ResourceFixture.createFixture(parentPool,null, this);

		resource = (Resource)save(resource, null ).getRpmObject();

		assertNotNull(resource);
		
		return resource;
	}
	
	private AbstractScope saveAbstractScopeElementWithResourceNames(String proposedByResourceName, String approvedByResourceName, 
			String closedByResourceName) throws Exception
	{
		Defect defect = new Defect();
		defect.setName(HelperTestUtilities.createName());
		defect.setParent(parent);
		defect.setProposedByName(proposedByResourceName);
		defect.setApprovedByName(approvedByResourceName);
		defect.setClosedByName(closedByResourceName);
		
		defect = (Defect)save(defect,null).getRpmObject();
		
		return defect;
	}
	
	private ScopeFolder loadRootScopeFolder() throws Exception
	{
		String xPath = "/ScopeFolder [name = '" + proposal.getName() + "']";
		ScopeFolder rootScopeFolder = (ScopeFolder)loadOneFromXpath(xPath, null);
		assertNotNull(rootScopeFolder);
		return rootScopeFolder;
	}
	
	private Resource getCurrentUser() throws Exception
	{
		String userName = RpmTestProperties.getProperty("rpm.test.user.name");
		Resource resource = (Resource)loadFromXpath("/Resource[username = '" + userName + "']", null)[0];
		assertNotNull(resource);
		return resource;
	}

	private void validateByDate(Calendar expectedByDate, Calendar byDate)
	{
		assertNotNull(byDate);
		assertEquals(expectedByDate.get(Calendar.YEAR),byDate.get(Calendar.YEAR));
		assertEquals(expectedByDate.get(Calendar.MONTH),byDate.get(Calendar.MONTH));
		assertEquals(expectedByDate.get(Calendar.DAY_OF_MONTH),byDate.get(Calendar.DAY_OF_MONTH));
	}
}