package com.ibm.rpm.client.junit.scopemanagement.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00307395 extends ClientTestController {

	private Attribute scope_attribute = null;
	private Attribute wbs_attribute = null;
	
	public void testRequirement() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new Requirement(), scope_attribute);
	}
	
	public void testChangeRequest() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new ChangeRequest(), scope_attribute);
	}
	
	public void testServiceRequest() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new ServiceRequest(), scope_attribute);
	}
	
	public void testAction() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new Action(), scope_attribute);
	}
	
	public void testDefect() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new Defect(), scope_attribute);
	}
	
	public void testIssue() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new Issue(), scope_attribute);
	}
	
	public void testRisk() throws Exception
	{
		createSaveAttributesForScope();
		runTestForScopeElement(new Risk(), scope_attribute);
	}
	
	public void testScopeMilestone() throws Exception
	{
		createSaveAttributesForWbs();
		runTestForScopeElement(new ScopeMilestone(), wbs_attribute);
	}
	
	public void testScopeTask() throws Exception
	{
		createSaveAttributesForWbs();
		runTestForScopeElement(new ScopeTask(), wbs_attribute);
	}
	
	public void testScopeDeliverable() throws Exception
	{
		createSaveAttributesForWbs();
		runTestForScopeElement(new ScopeDeliverable(), wbs_attribute);
	}
	
	public void testScopeSummaryTask() throws Exception
	{
		createSaveAttributesForWbs();
		runTestForScopeElement(new ScopeSummaryTask(), wbs_attribute);
	}
	
	private void runTestForScopeElement(ScopeElement scopeElement, Attribute attribute) throws Exception
	{
		ScopeFolder rootParent = createSaveRootScopeFolder(); 
		
		scopeElement = ScopeElementFixture.createSaveScopeElement(rootParent, scopeElement, 0, null, this);
		
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		scopeElementScope.setAttributeAssignments(true);
		
		AttributeAssignment attributeAssignment = new AttributeAssignment();
		attributeAssignment.setAttribute(attribute);
		attributeAssignment.setValue("10");
		
		if(scopeElement instanceof AbstractScopeTask)
		{
			((AbstractScopeTask)scopeElement).setAttributeAssignments(new AttributeAssignment[] {attributeAssignment});
		} 
		else if(scopeElement instanceof AbstractAggregateScope)
		{
			((AbstractAggregateScope)scopeElement).setAttributeAssignments(new AttributeAssignment[] {attributeAssignment});
		}
		
		scopeElement = (ScopeElement)save(scopeElement,scopeElementScope).getRpmObject();
		assertNotNull(scopeElement);
		
		AttributeAssignment returnedAssignment[] = null;
		if(scopeElement instanceof AbstractScopeTask)
		{
			returnedAssignment = ((AbstractScopeTask)scopeElement).getAttributeAssignments();
		} 
		else if(scopeElement instanceof AbstractAggregateScope)
		{
			returnedAssignment = ((AbstractAggregateScope)scopeElement).getAttributeAssignments();
		}
		
		assertNotNull(returnedAssignment);
		assertTrue(returnedAssignment.length == 1);
		assertNotNull(returnedAssignment[0]);
		assertTrue(returnedAssignment[0].getValue().equals("10"));
	
//		 TODO: Uncomment the following line when running the test
		if(scopeElement instanceof AbstractScopeTask)
		{
			((AbstractScopeTask)scopeElement).getAttributeAssignments()[0].setValue("20");
//			((AbstractScopeTask)scopeElement).getAttributeAssignments()[0].assignTypeID(null);
		} 
		else if(scopeElement instanceof AbstractAggregateScope)
		{
			((AbstractAggregateScope)scopeElement).getAttributeAssignments()[0].setValue("20");
//			((AbstractAggregateScope)scopeElement).getAttributeAssignments()[0].assignTypeID(null);
		}
		
		scopeElement = (ScopeElement)save(scopeElement,scopeElementScope).getRpmObject();
		
		assertNotNull(scopeElement);
		
		if(scopeElement instanceof AbstractScopeTask)
		{
			returnedAssignment = ((AbstractScopeTask)scopeElement).getAttributeAssignments();
		} 
		else if(scopeElement instanceof AbstractAggregateScope)
		{
			returnedAssignment = ((AbstractAggregateScope)scopeElement).getAttributeAssignments();
		}

		assertNotNull(returnedAssignment);
		assertTrue(returnedAssignment.length == 1);
		assertNotNull(returnedAssignment[0]);
		assertTrue(returnedAssignment[0].getValue().equals("20"));
	}
	
	private ScopeFolder createSaveRootScopeFolder() throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setScopeElement(new ScopeElementScope());
		WorkElement workElement = WorkElementFixture.createSaveFixture(new Proposal(), workElementScope, this);
		assertNotNull(workElement);
		assertNotNull(workElement.getScopeElement());
		return (ScopeFolder)workElement.getScopeElement();
	}
	
	private void createSaveAttributesForScope() throws Exception
	{
		ScopeManagementModule module = new ScopeManagementModule();

		AttributeCategory category = createAttributeHierarchy();
		module.setAttributeCategories(new AttributeCategory[]{category});
		AttributeScope attrScope = new AttributeScope();   	    	          
		attrScope.setCategories(attrScope);
		attrScope.setClassifications(attrScope);
		attrScope.setAttributes(attrScope);

		ScopeManagementModuleScope moduleScope = new ScopeManagementModuleScope();
		moduleScope.setAttributeCategories(attrScope);    	

		module = (ScopeManagementModule)save(module, moduleScope).getRpmObject();
		
		assertNotNull(module);
		assertNotNull(module.getAttributeCategories());
		assertTrue(module.getAttributeCategories().length == 1);
		assertNotNull(module.getAttributeCategories()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications());
		assertTrue(module.getAttributeCategories()[0].getClassifications().length == 1);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes());
		assertTrue(module.getAttributeCategories()[0].getClassifications()[0].getAttributes().length == 2);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1]);
		
		scope_attribute = module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1];
	}
	
	private void createSaveAttributesForWbs() throws Exception
	{
		WbsModule module = new WbsModule();

		AttributeCategory category = createAttributeHierarchy();
		
		module.setAttributeCategories(new AttributeCategory[]{category});
		AttributeScope attrScope = new AttributeScope();   	    	          
		attrScope.setCategories(attrScope);
		attrScope.setClassifications(attrScope);
		attrScope.setAttributes(attrScope);

		WbsModuleScope moduleScope = new WbsModuleScope();
		moduleScope.setAttributeCategories(attrScope);    	

		module = (WbsModule)save(module, moduleScope).getRpmObject();
		
		assertNotNull(module);
		assertNotNull(module.getAttributeCategories());
		assertTrue(module.getAttributeCategories().length == 1);
		assertNotNull(module.getAttributeCategories()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications());
		assertTrue(module.getAttributeCategories()[0].getClassifications().length == 1);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes());
		assertTrue(module.getAttributeCategories()[0].getClassifications()[0].getAttributes().length == 2);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1]);
		
		wbs_attribute = module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1];
	}
	
	private AttributeCategory createAttributeHierarchy() throws Exception
	{
		AttributeCategory category = new AttributeCategory();
		category.setName("My Category");

		AttributeClassification  classification = new AttributeClassification();
		classification.setName("My AssetUsage");
		classification.setSelectType(SelectType.Multi);

		Attribute attribute1 = new Attribute();
		attribute1.setName("Production");
		attribute1.setInputType(InputType.Text);

		Attribute attribute2 = new Attribute();
		attribute2.setName("Consumption");
		attribute2.setInputType(InputType.Numeric);

		classification.setAttributes(new Attribute[] {attribute1, attribute2});

		category.setClassifications(new AttributeClassification[]{ classification });
		
		return category;
	}
}
