/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.framework.testcases.RpmObjectAbstractTester;
import com.ibm.rpm.client.junit.framework.testcases.RpmTestFramework;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestActionValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Action mainAction;

    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        RpmObjectAbstractTester findFixture = RpmTestFramework.INSTANCE
            .findFixture(Action.class);

        mainAction = new Action();
        mainAction.setName("Name");
        mainAction.setParent(mainParent);
        
        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainAction.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainAction.getClass()
                .getName(), "parent", mainAction.getID())
        };

        verifySaveMustFail(mainAction, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainAction.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainAction
                .getClass().getName(), "parent", mainAction.getID())
        };

        verifySaveMustFail(mainAction, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainAction.setName(getTestString(ValidationConstants.NAME_MAX));
        mainAction
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainAction.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainAction.setRank(new Integer(ValidationConstants.RANK_MAX));

        // Dates
        mainAction.setEstimatedStartDate(calendar1);
        mainAction.setEstimatedFinishDate(calendar1);

        // Attempt to save the Action
        mainAction = (Action) saveMustSucceed(mainAction, Action.class, null);

        // Load the inserted Action from id
        RPMObject obj = new Action();
        obj.setID(mainAction.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Action);

        // Check that the loaded Action is identical
        ScopeElementFixture.compare(mainAction, (Action) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainAction.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainAction.setRank(new Integer(ValidationConstants.RANK_MIN));

        // Attempt to save the Action
        mainAction = (Action) saveMustSucceed(mainAction, Action.class, null);

        // Load the inserted Action from id
        RPMObject obj = new Action();
        obj.setID(mainAction.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Action);

        // Check that the loaded Action is identical
        ScopeElementFixture.compare(mainAction, (Action) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainAction.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainAction.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainAction.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainAction.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainAction
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainAction
            .setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainAction.setRank(new Integer(ValidationConstants.RANK_MAX + 1));

        // Dates
        mainAction.setEstimatedStartDate(calendar1);
        mainAction.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainAction.getClass()
                    .getName(), "name", mainAction.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainAction.getClass()
                    .getName(), "proposedByName", mainAction.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainAction.getClass()
                    .getName(), "approvedByName", mainAction.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainAction.getClass()
                    .getName(), "closedByName", mainAction.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainAction.getClass()
                    .getName(), "externalReferenceNumber", mainAction.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainAction
                    .getClass().getName(), "priority", mainAction.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainAction
                    .getClass().getName(), "rank", mainAction.getID()),
                createRPMException(ErrorCodes.INVALID_DATES, mainAction
                    .getClass().getName(), "estimatedStartDate", mainAction
                    .getID())
        };

        verifySaveMustFail(mainAction, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainAction
            .setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainAction.setRank(new Integer(ValidationConstants.RANK_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainAction
                    .getClass().getName(), "priority", mainAction.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainAction
                    .getClass().getName(), "rank", mainAction.getID())
        };

        verifySaveMustFail(mainAction, null, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setState(true);

        // Action must have state of type ActionState
        mainAction.setState(new DefectState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainAction.getClass()
                .getName(), "state", mainAction.getID())
        };

        verifySaveMustFail(mainAction, scope, expectedExceptions);
    }
}
