/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestDefectValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Defect mainDefect;

    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainDefect = new Defect();
        mainDefect.setName("Name");
        mainDefect.setParent(mainParent);

        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainDefect.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainDefect.getClass().getName(),
                "parent", mainDefect.getID())
        };

        verifySaveMustFail(mainDefect, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainDefect.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainDefect.getClass()
                .getName(), "parent", mainDefect.getID())
        };

        verifySaveMustFail(mainDefect, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainDefect.setName(getTestString(ValidationConstants.NAME_MAX));
        mainDefect
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainDefect.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainDefect.setRank(new Integer(ValidationConstants.RANK_MAX));
        mainDefect.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX));

        // Dates
        mainDefect.setEstimatedStartDate(calendar1);
        mainDefect.setEstimatedFinishDate(calendar1);

        // Attempt to save the Defect
        mainDefect = (Defect) saveMustSucceed(mainDefect, Defect.class, null);

        // Load the inserted Defect from id
        RPMObject obj = new Defect();
        obj.setID(mainDefect.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Defect);

        // Check that the loaded Defect is identical
        ScopeElementFixture.compare(mainDefect, (Defect) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainDefect.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainDefect.setRank(new Integer(ValidationConstants.RANK_MIN));
        mainDefect.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN));

        // Attempt to save the Defect
        mainDefect = (Defect) saveMustSucceed(mainDefect, Defect.class, null);

        // Load the inserted Defect from id
        RPMObject obj = new Defect();
        obj.setID(mainDefect.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Defect);

        // Check that the loaded Defect is identical
        ScopeElementFixture.compare(mainDefect, (Defect) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainDefect.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainDefect.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainDefect.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainDefect.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainDefect
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainDefect.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainDefect.setRank(new Integer(ValidationConstants.RANK_MAX + 1));
        mainDefect.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX + 1));

        // Dates
        mainDefect.setEstimatedStartDate(calendar1);
        mainDefect.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainDefect.getClass().getName(),
                "name", mainDefect.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDefect.getClass()
                .getName(), "proposedByName", mainDefect.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDefect.getClass()
                .getName(), "approvedByName", mainDefect.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDefect.getClass()
                .getName(), "closedByName", mainDefect.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDefect.getClass().getName(),
                "externalReferenceNumber", mainDefect.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "priority", mainDefect.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "rank", mainDefect.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "numberTimesReassigned", mainDefect.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainDefect.getClass().getName(),
                "estimatedStartDate", mainDefect.getID())
        };

        verifySaveMustFail(mainDefect, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainDefect.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainDefect.setRank(new Integer(ValidationConstants.RANK_MIN - 1));
        mainDefect.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "priority", mainDefect.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "rank", mainDefect.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainDefect.getClass().getName(),
                "numberTimesReassigned", mainDefect.getID())
        };

        verifySaveMustFail(mainDefect, null, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setState(true);
        
        // Defect must have state of type DefectState
        mainDefect.setState(new ActionState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainDefect.getClass().getName(),
                "state", mainDefect.getID())
        };

        verifySaveMustFail(mainDefect, scope, expectedExceptions);
    }
    
    public void testFailureWithUnsavedCCC() throws AssertionError, Exception
    {
    	ScopeElementScope scope = new ScopeElementScope();
        scope.setClientCostCenter(true);
        
        ClientCostCenter ccc = ClientCostCenterFixture.createFixture(null, this);
        mainDefect.setClientCostCenter(ccc);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scopemanagement.constants.ErrorCodes.CANNOT_ASSIGN_UNSAVED_CLIENT_COST_CENTER, mainDefect.getClass().getName(),
                "clientCostCenter", mainDefect.getID())
        };
        verifySaveMustFail(mainDefect, scope, expectedExceptions);
    }
    
    public void testFailureWithUnsavedAsset() throws AssertionError, Exception
    {
    	ScopeElementScope scope = new ScopeElementScope();
    	scope.setAsset(true);
    	
    	Asset asset = AssetFixture.createFixture(null, this);
    	mainDefect.setAsset(asset);
    	
    	RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scopemanagement.constants.ErrorCodes.CANNOT_ASSIGN_UNSAVED_ASSET, mainDefect.getClass().getName(),
                "asset", mainDefect.getID())
        };
        verifySaveMustFail(mainDefect, scope, expectedExceptions);
    	
    }
}
