/*
 * Created on 20-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author dominiqu
 */
public class TestDuplicateValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private ScopeElementScope mainScope;

    private Portfolio rootTestPortfolio;
    
    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Action mainAction;

    private Action duplicateAction;
    
    private Duplicate[] mainDuplicates;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setDuplicates(true);

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            new WorkElementScope());
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainAction = new Action();
        mainAction.setName("mainActionName");
        mainAction.setParent(mainParent);
        
        duplicateAction = new Action();
        duplicateAction.setName("duplicateActionName");
        duplicateAction.setParent(mainParent);
        duplicateAction = (Action) saveMustSucceed(duplicateAction, Action.class, null);

        mainDuplicates = new Duplicate[1];

        mainDuplicates[0] = new Duplicate();
        mainDuplicates[0].setParent(mainAction);
        mainDuplicates[0].setScopeElement(duplicateAction);

        mainAction.setDuplicates(mainDuplicates);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError, Exception
    {
        mainDuplicates[0].setScopeElement(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mainDuplicates[0].getClass()
                .getName(), "scopeElement", mainDuplicates[0].getID())
        };

        verifySaveMustFail(mainAction, mainScope, expectedExceptions);
    }

    public void testFailureWithoutMandatoryID() throws AssertionError, Exception
    {
        mainDuplicates[0].getScopeElement().setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mainDuplicates[0].getClass().getName(),
                "scopeElement", mainDuplicates[0].getID())
        };

        verifySaveMustFail(mainAction, mainScope, expectedExceptions);
    }

    public void testFailureWithEqualDuplicate() throws AssertionError, Exception
    {
        // Save the action so it has an ID
        mainAction = (Action) saveMustSucceed(mainAction, Action.class, null);
        
        mainDuplicates[0].setScopeElement(mainAction);
        mainAction.setDuplicates(mainDuplicates);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scopemanagement.constants.ErrorCodes.
                DUPLICATE_CANNOT_HAVE_SAME_ID,
                mainDuplicates[0].getClass().getName())
        };

        verifySaveMustFail(mainAction, mainScope, expectedExceptions);
    }

    public void testFailureWithInvalidType() throws AssertionError, Exception
    {
        ChangeRequest wrongType = new ChangeRequest();
        // Bypass the ID validation without saving the change request
        wrongType.setID(duplicateAction.getID());

        mainDuplicates[0].setScopeElement(wrongType);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainDuplicates[0].getClass()
                .getName(), "scopeElement", mainDuplicates[0].getID())
        };

        verifySaveMustFail(mainAction, mainScope, expectedExceptions);
    }

    public void testFailureWithDuplicateFromOtherProject() throws AssertionError,
            Exception
    {
        // Create an action in another project

        Proposal mainGrandParentDuplicate = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        mainGrandParentDuplicate = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParentDuplicate.setParent(rootTestPortfolio);
        mainGrandParentDuplicate = (Proposal) saveMustSucceed(mainGrandParentDuplicate,
            Proposal.class, null);
        mainGrandParentDuplicate = (Proposal) loadFromID(mainGrandParentDuplicate, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParentDuplicate.getName() + "']", new ScopeElementScope());

        ScopeFolder mainParentDuplicate = (ScopeFolder) rpmObjects[0];
        mainParentDuplicate.setContextName(contextName);

        Action duplicateAction2 = new Action();
        duplicateAction2.setName("duplicateAction2Name");
        duplicateAction2.setParent(mainParentDuplicate);
        saveMustSucceed(duplicateAction2, Action.class, null);

        mainDuplicates[0].setScopeElement(duplicateAction2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scopemanagement.constants.ErrorCodes.
                DUPLICATE_MUST_BE_IN_SAME_PROJECT,
                mainDuplicates[0].getClass().getName())
        };

        verifySaveMustFail(mainAction, mainScope, expectedExceptions);
    }
    
    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Attempt to save the Action
        mainAction = (Action) saveMustSucceed(mainAction, Action.class, mainScope);

        // Load the inserted Action from id
        RPMObject obj = new Action();
        obj.setID(mainAction.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Action);

        // Check that the loaded Issue is identical
        ScopeElementFixture.compare(mainAction, (Action) rpmObject, null);

    }
}
