/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestIssueValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Issue mainIssue;
    
    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainIssue = new Issue();
        mainIssue.setName("Name");
        mainIssue.setParent(mainParent);
        
        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainIssue.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainIssue.getClass().getName(),
                "parent", mainIssue.getID())
        };

        verifySaveMustFail(mainIssue, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainIssue.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainIssue.getClass()
                .getName(), "parent", mainIssue.getID())
        };

        verifySaveMustFail(mainIssue, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainIssue.setName(getTestString(ValidationConstants.NAME_MAX));
        mainIssue
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainIssue.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainIssue.setRank(new Integer(ValidationConstants.RANK_MAX));
        mainIssue.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX));

        // Dates
        mainIssue.setEstimatedStartDate(calendar1);
        mainIssue.setEstimatedFinishDate(calendar1);

        // Attempt to save the Issue
        mainIssue = (Issue) saveMustSucceed(mainIssue, Issue.class, null);

        // Load the inserted Issue from id
        RPMObject obj = new Issue();
        obj.setID(mainIssue.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Issue);

        // Check that the loaded Issue is identical
        ScopeElementFixture.compare(mainIssue, (Issue) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainIssue.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainIssue.setRank(new Integer(ValidationConstants.RANK_MIN));
        mainIssue.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN));

        // Attempt to save the Issue
        mainIssue = (Issue) saveMustSucceed(mainIssue, Issue.class, null);

        // Load the inserted Issue from id
        RPMObject obj = new Issue();
        obj.setID(mainIssue.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Issue);

        // Check that the loaded Issue is identical
        ScopeElementFixture.compare(mainIssue, (Issue) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainIssue.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainIssue.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainIssue.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainIssue.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainIssue
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainIssue.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainIssue.setRank(new Integer(ValidationConstants.RANK_MAX + 1));
        mainIssue.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX + 1));

        // Dates
        mainIssue.setEstimatedStartDate(calendar1);
        mainIssue.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainIssue.getClass().getName(),
                "name", mainIssue.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainIssue.getClass()
                .getName(), "proposedByName", mainIssue.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainIssue.getClass()
                .getName(), "approvedByName", mainIssue.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainIssue.getClass()
                .getName(), "closedByName", mainIssue.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainIssue.getClass().getName(),
                "externalReferenceNumber", mainIssue.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "priority", mainIssue.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "rank", mainIssue.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "numberTimesReassigned", mainIssue.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainIssue.getClass().getName(),
                "estimatedStartDate", mainIssue.getID())
        };

        verifySaveMustFail(mainIssue, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainIssue.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainIssue.setRank(new Integer(ValidationConstants.RANK_MIN - 1));
        mainIssue.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "priority", mainIssue.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "rank", mainIssue.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainIssue.getClass().getName(),
                "numberTimesReassigned", mainIssue.getID())
        };

        verifySaveMustFail(mainIssue, null, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        // Issue must have state of type IssueState
        mainIssue.setState(new ActionState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainIssue.getClass().getName(),
                "state", mainIssue.getID())
        };

        verifySaveMustFail(mainIssue, null, expectedExceptions);
    }
}
