/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.MitigationTrigger;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestMitigationFactorValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private ScopeElementScope mainScope;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Risk mainRisk;

    private MitigationFactor[] mainMitigationFactor;

    private MitigationTrigger[] mainMitigationTrigger;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ScopeElementScope();
        MitigationFactorScope mitigationFactorScope = new MitigationFactorScope();
        mitigationFactorScope.setMitigationTriggers(true);
        mainScope.setMitigationFactors(mitigationFactorScope);

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainRisk = new Risk();
        mainRisk.setName("Name");
        mainRisk.setParent(mainParent);

        mainMitigationFactor = new MitigationFactor[1];
        mainMitigationFactor[0] = new MitigationFactor();
        mainMitigationFactor[0].setParent(mainRisk);
        mainRisk.setMitigationFactors(mainMitigationFactor);

        mainMitigationTrigger = new MitigationTrigger[1];
        mainMitigationTrigger[0] = new MitigationTrigger();
        mainMitigationTrigger[0].setParent(mainMitigationFactor[0]);
        mainMitigationFactor[0].setMitigationTriggers(mainMitigationTrigger);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainMitigationFactor[0]
            .setDescription(getTestString(ValidationConstants.MITIGATIONFACTOR_DESCRIPTION_MAX));
        mainMitigationFactor[0].setMitigationCost(new Double(
            ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MAX));
        mainMitigationFactor[0].setPriority(new Integer(
            ValidationConstants.MITIGATIONFACTOR_PRIORITY_MAX));

        mainMitigationTrigger[0]
            .setDescription(getTestString(ValidationConstants.MITIGATIONTRIGGER_DESCRIPTION_MAX));

        // Attempt to save the Risk
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Load the inserted Risk from id
        RPMObject obj = new Risk();
        obj.setID(mainRisk.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Risk);

        // Check that the loaded Risk is identical
        ScopeElementFixture.compare(mainRisk, (Risk) rpmObject, mainScope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainMitigationFactor[0].setMitigationCost(new Double(
            ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MIN));
        mainMitigationFactor[0].setPriority(new Integer(
            ValidationConstants.MITIGATIONFACTOR_PRIORITY_MIN));

        // Attempt to save the Risk
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Load the inserted Risk from id
        RPMObject obj = new Risk();
        obj.setID(mainRisk.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Risk);

        // Check that the loaded Risk is identical
        ScopeElementFixture.compare(mainRisk, (Risk) rpmObject, mainScope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainMitigationFactor[0]
            .setDescription(getTestString(ValidationConstants.MITIGATIONFACTOR_DESCRIPTION_MAX + 1));
        mainMitigationFactor[0].setMitigationCost(new Double(
            ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MAX + 0.01));
        mainMitigationFactor[0].setPriority(new Integer(
            ValidationConstants.MITIGATIONFACTOR_PRIORITY_MAX + 1));

        mainMitigationTrigger[0]
            .setDescription(getTestString(ValidationConstants.MITIGATIONTRIGGER_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainMitigationFactor[0].getClass()
                .getName(), "description", mainMitigationFactor[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainMitigationFactor[0]
                .getClass().getName(), "mitigationCost", mainMitigationFactor[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainMitigationFactor[0]
                .getClass().getName(), "priority", mainMitigationFactor[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainMitigationTrigger[0].getClass()
                .getName(), "description", mainMitigationTrigger[0].getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainMitigationFactor[0].setMitigationCost(new Double(
            ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MIN - 0.01));
        mainMitigationFactor[0].setPriority(new Integer(
            ValidationConstants.MITIGATIONFACTOR_PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainMitigationFactor[0]
                .getClass().getName(), "mitigationCost", mainMitigationFactor[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainMitigationFactor[0]
                .getClass().getName(), "priority", mainMitigationFactor[0].getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }
}
