/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestRiskValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private ScopeElementScope mainScope;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private Risk mainRisk;

    private RiskMatrix mainRiskMatrix;

    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setRiskMatrix(true);
        mainScope.setImpact(true);
        mainScope.setState(true);

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent,
            Proposal.class, null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainRisk = new Risk();
        mainRisk.setName("Name");
        mainRisk.setParent(mainParent);

        mainRiskMatrix = new RiskMatrix();
        mainRiskMatrix
            .setImpact((Integer) ValidationConstants.RISKMATRIX_VALUES.get(0));
        mainRiskMatrix
            .setProbability((Integer) ValidationConstants.RISKMATRIX_VALUES
                .get(0));
        mainRisk.setRiskMatrix(mainRiskMatrix);

        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainRisk.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainRisk.getClass()
                .getName(), "parent", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainRisk.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainRisk
                .getClass().getName(), "parent", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainRisk.setName(getTestString(ValidationConstants.NAME_MAX));
        mainRisk
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainRisk.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainRisk.setRank(new Integer(ValidationConstants.RANK_MAX));
        mainRisk.getRiskMatrix().setConsequenceCost(
            new Double(ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MAX));

        // Dates
        mainRisk.setEstimatedStartDate(calendar1);
        mainRisk.setEstimatedFinishDate(calendar1);

        // Attempt to save the Risk
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Load the inserted Risk from id
        RPMObject obj = new Risk();
        obj.setID(mainRisk.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Risk);

        // Check that the loaded Risk is identical
        ScopeElementFixture.compare(mainRisk, (Risk) rpmObject, mainScope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainRisk.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainRisk.setRank(new Integer(ValidationConstants.RANK_MIN));
        mainRisk.getRiskMatrix().setConsequenceCost(
            new Double(ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MIN));

        // Attempt to save the Risk
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Load the inserted Risk from id
        RPMObject obj = new Risk();
        obj.setID(mainRisk.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Risk);

        // Check that the loaded Risk is identical
        ScopeElementFixture.compare(mainRisk, (Risk) rpmObject, mainScope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainRisk.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainRisk.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainRisk.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainRisk.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainRisk
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainRisk.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainRisk.setRank(new Integer(ValidationConstants.RANK_MAX + 1));
        mainRisk.getRiskMatrix().setConsequenceCost(
            new Double(
                ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MAX + 0.01));

        // Dates
        mainRisk.setEstimatedStartDate(calendar1);
        mainRisk.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainRisk.getClass()
                    .getName(), "name", mainRisk.getID()),
                    createRPMException(ErrorCodes.MAX_LENGTH, mainRisk.getClass()
                            .getName(), "proposedByName", mainRisk.getID()),
                        createRPMException(ErrorCodes.MAX_LENGTH, mainRisk.getClass()
                            .getName(), "approvedByName", mainRisk.getID()),
                        createRPMException(ErrorCodes.MAX_LENGTH, mainRisk.getClass()
                            .getName(), "closedByName", mainRisk.getID()),
                        
                createRPMException(ErrorCodes.MAX_LENGTH, mainRisk.getClass()
                    .getName(), "externalReferenceNumber", mainRisk.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRisk.getClass()
                    .getName(), "priority", mainRisk.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRisk.getClass()
                    .getName(), "rank", mainRisk.getID()),
                createRPMException(ErrorCodes.INVALID_DATES, mainRisk
                    .getClass().getName(), "estimatedStartDate", mainRisk
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRiskMatrix
                    .getClass().getName(), "consequenceCost", mainRiskMatrix
                    .getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainRisk.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainRisk.setRank(new Integer(ValidationConstants.RANK_MIN - 1));
        mainRisk.getRiskMatrix().setConsequenceCost(
            new Double(
                ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MIN - 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRisk.getClass()
                    .getName(), "priority", mainRisk.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRisk.getClass()
                    .getName(), "rank", mainRisk.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainRiskMatrix
                    .getClass().getName(), "consequenceCost", mainRiskMatrix
                    .getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureInvalidValue() throws AssertionError, Exception
    {
        mainRiskMatrix.setImpact(new Integer(1));
        mainRiskMatrix.setProbability(new Integer(101));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_VALUE, mainRiskMatrix
                    .getClass().getName(), "impact", mainRisk.getID()),
                createRPMException(ErrorCodes.INVALID_VALUE, mainRiskMatrix
                    .getClass().getName(), "probability", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        // Risk must have state of type RiskState
        mainRisk.setState(new ActionState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainRisk.getClass()
                .getName(), "state", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureCannotSetTaskToDefault() throws AssertionError,
            Exception
    {
        mainRisk.setDefaultScope(Boolean.FALSE);
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Convert to task
        mainRisk.setTask(Boolean.TRUE);
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Set Task to default must fail
        mainRisk.setDefaultScope(Boolean.TRUE);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_CONDITIONAL_VALUE, mainRisk
                .getClass().getName(), "defaultScope", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }

    public void testFailureCannotConvertDefaultToTask() throws AssertionError,
            Exception
    {
        mainRisk.setDefaultScope(Boolean.TRUE);
        mainRisk.setTask(Boolean.FALSE);
        mainRisk = (Risk) saveMustSucceed(mainRisk, Risk.class, mainScope);

        // Convert to task must fail
        mainRisk.setTask(Boolean.TRUE);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_CONDITIONAL_VALUE, mainRisk
                .getClass().getName(), "task", mainRisk.getID())
        };

        verifySaveMustFail(mainRisk, mainScope, expectedExceptions);
    }
}
