/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestScopeDeliverableValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGreatGrandParent;

    private ScopeFolder mainGrandParent;

    private Requirement mainParent;
    
    private ScopeDeliverable mainScopeDeliverable;
    
    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGreatGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(), 
            null, this);
        mainGreatGrandParent = (Proposal) saveMustSucceed(mainGreatGrandParent, Proposal.class,
            null);
        mainGreatGrandParent = (Proposal) loadFromID(mainGreatGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGreatGrandParent.getName() + "']", new ScopeElementScope());

        mainGrandParent = (ScopeFolder) rpmObjects[0];
        mainGrandParent.setContextName(contextName);

        mainParent = new Requirement();
        mainParent.setName("Name");
        mainParent.setParent(mainGrandParent);
        
        mainParent = (Requirement) saveMustSucceed(mainParent, Requirement.class, null);
        
        mainScopeDeliverable = new ScopeDeliverable();
        mainScopeDeliverable.setName("Name");
        mainScopeDeliverable.setParent(mainParent);
        
        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainScopeDeliverable.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainScopeDeliverable.getClass().getName(),
                "parent", mainScopeDeliverable.getID())
        };

        verifySaveMustFail(mainScopeDeliverable, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainScopeDeliverable.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainScopeDeliverable.getClass()
                .getName(), "parent", mainScopeDeliverable.getID())
        };

        verifySaveMustFail(mainScopeDeliverable, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainScopeDeliverable.setName(getTestString(ValidationConstants.NAME_MAX));
        mainScopeDeliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Dates
        mainScopeDeliverable.setEstimatedStartDate(calendar1);
        mainScopeDeliverable.setEstimatedFinishDate(calendar1);

        // Attempt to save the ScopeDeliverable
        mainScopeDeliverable = (ScopeDeliverable) saveMustSucceed(mainScopeDeliverable, ScopeDeliverable.class, null);

        // Load the inserted ScopeDeliverable from id
        RPMObject obj = new ScopeDeliverable();
        obj.setID(mainScopeDeliverable.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeDeliverable);

        // Check that the loaded ScopeDeliverable is identical
        ScopeElementFixture.compare(mainScopeDeliverable, (ScopeDeliverable) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainScopeDeliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the ScopeDeliverable
        mainScopeDeliverable = (ScopeDeliverable) saveMustSucceed(mainScopeDeliverable, ScopeDeliverable.class, null);

        // Load the inserted ScopeDeliverable from id
        RPMObject obj = new ScopeDeliverable();
        obj.setID(mainScopeDeliverable.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeDeliverable);

        // Check that the loaded ScopeDeliverable is identical
        ScopeElementFixture.compare(mainScopeDeliverable, (ScopeDeliverable) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainScopeDeliverable.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainScopeDeliverable.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainScopeDeliverable.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainScopeDeliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        // Dates
        mainScopeDeliverable.setEstimatedStartDate(calendar1);
        mainScopeDeliverable.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeDeliverable.getClass().getName(),
                "name", mainScopeDeliverable.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeDeliverable.getClass()
                .getName(), "proposedByName", mainScopeDeliverable.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeDeliverable.getClass()
                .getName(), "closedByName", mainScopeDeliverable.getID()),
                    
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeDeliverable.getClass().getName(),
                "priority", mainScopeDeliverable.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainScopeDeliverable.getClass().getName(), 
                "estimatedStartDate", mainScopeDeliverable.getID())
        };

        verifySaveMustFail(mainScopeDeliverable, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainScopeDeliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeDeliverable.getClass().getName(),
                "priority", mainScopeDeliverable.getID())
        };

        verifySaveMustFail(mainScopeDeliverable, null, expectedExceptions);
    }

}
