package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.wbs.containers.Proposal;

public class TestScopeElementValidation extends GenericValidation {
	
	public void testUpdateWithoutMandatoryField() throws AssertionError, Exception
	{
		Proposal proposal = createAndSaveProposal();
		
		ScopeFolder rootScopeFolder = loadRootScopeFolder(proposal);
		
		ScopeElement scopeElement = createAndSaveScopeElement(rootScopeFolder);

		//Set elementSecurityType to null
		scopeElement.setElementSecurityType(null);
		
		RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, scopeElement.getClass().getName(),
                		ValidationConstants.ELEMENT_SECURITY_TYPE, scopeElement.getID()),
        };

        verifySaveMustFail(scopeElement, null, expectedExceptions);
	}
	
	private Proposal createAndSaveProposal() throws Exception
	{
		Proposal proposal = new Proposal();
		proposal.setName(HelperTestUtilities.createName());
		
		proposal = (Proposal)save(proposal,null).getRpmObject();
		assertNotNull(proposal);
		return proposal;
	}
	
	private ScopeFolder loadRootScopeFolder(Proposal proposal) throws Exception
	{
		String xpath = "/ScopeFolder[name = '" +  proposal.getName() + "']";
		ScopeFolder rootScopeFolder = (ScopeFolder)loadFromXpath(xpath, null)[0];
		assertNotNull(rootScopeFolder);
		return rootScopeFolder;
	}
	
	private ScopeElement createAndSaveScopeElement(ScopeFolder parent) throws Exception
	{
		Requirement requirement = new Requirement();
		requirement.setName(HelperTestUtilities.createName());
		requirement.setParent(parent);
		
		requirement = (Requirement)save(requirement, null).getRpmObject();
		assertNotNull(requirement);
		
		return requirement;
	}

}
