/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestServiceRequestValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private ServiceRequest mainServiceRequest;

    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainServiceRequest = new ServiceRequest();
        mainServiceRequest.setName("Name");
        mainServiceRequest.setParent(mainParent);

        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainServiceRequest.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainServiceRequest.getClass()
                .getName(), "parent", mainServiceRequest.getID())
        };

        verifySaveMustFail(mainServiceRequest, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainServiceRequest.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainServiceRequest
                .getClass().getName(), "parent", mainServiceRequest.getID())
        };

        verifySaveMustFail(mainServiceRequest, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainServiceRequest.setName(getTestString(ValidationConstants.NAME_MAX));
        mainServiceRequest
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainServiceRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainServiceRequest.setRank(new Integer(ValidationConstants.RANK_MAX));
        mainServiceRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX));

        // Dates
        mainServiceRequest.setEstimatedStartDate(calendar1);
        mainServiceRequest.setEstimatedFinishDate(calendar1);

        // Attempt to save the ServiceRequest
        mainServiceRequest = (ServiceRequest) saveMustSucceed(mainServiceRequest,
            ServiceRequest.class, null);

        // Load the inserted ServiceRequest from id
        RPMObject obj = new ServiceRequest();
        obj.setID(mainServiceRequest.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ServiceRequest);

        // Check that the loaded ServiceRequest is identical
        ScopeElementFixture.compare(mainServiceRequest, (ServiceRequest) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainServiceRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainServiceRequest.setRank(new Integer(ValidationConstants.RANK_MIN));
        mainServiceRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN));

        // Attempt to save the ServiceRequest
        mainServiceRequest = (ServiceRequest) saveMustSucceed(mainServiceRequest,
            ServiceRequest.class, null);

        // Load the inserted ServiceRequest from id
        RPMObject obj = new ServiceRequest();
        obj.setID(mainServiceRequest.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ServiceRequest);

        // Check that the loaded ServiceRequest is identical
        ScopeElementFixture.compare(mainServiceRequest, (ServiceRequest) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainServiceRequest.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainServiceRequest.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainServiceRequest.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainServiceRequest.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainServiceRequest
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainServiceRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainServiceRequest.setRank(new Integer(ValidationConstants.RANK_MAX + 1));
        mainServiceRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX + 1));

        // Dates
        mainServiceRequest.setEstimatedStartDate(calendar1);
        mainServiceRequest.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainServiceRequest.getClass()
                .getName(), "name", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainServiceRequest.getClass()
                .getName(), "proposedByName", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainServiceRequest.getClass()
                .getName(), "approvedByName", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainServiceRequest.getClass()
                .getName(), "closedByName", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainServiceRequest.getClass()
                .getName(), "externalReferenceNumber", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "priority", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "rank", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "numberTimesReassigned", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainServiceRequest.getClass()
                .getName(), "estimatedStartDate", mainServiceRequest.getID())
        };

        verifySaveMustFail(mainServiceRequest, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainServiceRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainServiceRequest.setRank(new Integer(ValidationConstants.RANK_MIN - 1));
        mainServiceRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "priority", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "rank", mainServiceRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainServiceRequest.getClass()
                .getName(), "numberTimesReassigned", mainServiceRequest.getID())
        };

        verifySaveMustFail(mainServiceRequest, null, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setState(true);
        
        // ServiceRequest must have state of type ServiceRequestState
        mainServiceRequest.setState(new ActionState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainServiceRequest.getClass()
                .getName(), "state", mainServiceRequest.getID())
        };

        verifySaveMustFail(mainServiceRequest, scope, expectedExceptions);
    }
}
