package com.ibm.rpm.client.junit.scorecard.fixtures;

import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.AssignedScorecardCategory;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Iftach Ragoler
 */
public class PublishedScorecardFixture extends RPMObjectFixture
{

    static
    {
        skipGetters.add("getDescription");
        skipGetters.add("getScore");
        skipGetters.add("getResponseDate");
        // Active can only be compared when the scorecard's parent is a project.
        skipGetters.add("getActive"); 

        skipSetters.add("setDescription");
        skipSetters.add("setScore");
        skipSetters.add("setResponseDate");
    }

    public static RPMObject createAssignmentFixture(RPMObject parent,
            RPMObjectScope scope, ClientTestController ctc,
            ReloadType reloadType, ScorecardFolder folder, int quantity)
            throws RPMException, AssertionFailedError, Exception
    {
        Scorecard[] scorecards = folder.getScorecards();
        List tempScorecards = new LinkedList();
        for (int i = 0; i < quantity; i++)
        {
            Scorecard scorecard = scorecards[i % scorecards.length];

            if (!scorecard.getActive().booleanValue())
            {
                scorecard.setActive(new Boolean(true));
                save(ctc, scorecard, null);
            }

            SingleResult result = ctc.getApplicationInterface().assignScoreCard(ClientTestController.sessionid, parent,
                scope, scorecard, reloadType);
            ClientTestController.logAndThrow(result, "createAssignmentFixture");
            //we only set the parent if the reloadType is not NONE
            if(reloadType != ReloadType.None)
            {
                parent = result.getRpmObject();                
            }
            tempScorecards.add(scorecard);
        }

        //we only compare assigned scorecards if reloadType is not NONE
        if(reloadType != ReloadType.None)
        {
            AssignedScorecard[] tempAssignedScorecards = getAssignedScorecardsFromParent(parent);
            for (int i = 0; tempAssignedScorecards != null
                    && i < tempAssignedScorecards.length; i++)
            {
                PublishedScorecardFixture.compare(
                    (Scorecard) tempScorecards.get(i), tempAssignedScorecards[i],
                    false);
            }            
        }

        return parent;
    }

    public static void updateAssignment(int index, AssignedScorecard scorecard)
            throws Exception
    {
        scorecard.setName("Scorecard " + index++);
        AssignedScorecardState state = scorecard.getScorecardState();
        state.setActive(new Boolean(!state.getActive().booleanValue()));

        AssignedScorecardCategory[] categories = scorecard
            .getAssignedScorecardCategories();
        for (int i = 0; i < categories.length; i++)
        {
            // there is a bug in the database so this can not be set
            // categories[i].setDescription("Description " + index++);
            AssignedScorecardQuestion[] questions = categories[i]
                .getAssignedScorecardQuestions();
            for (int j = 0; j < questions.length; j++)
            {
                questions[j].setToReport(new Boolean(!(questions[j]
                    .getToReport().booleanValue())));
                AssignedScorecardResponse[] responses = questions[j]
                    .getAssignedScorecardResponses();
                for (int k = 0; k < responses.length; k++)
                {
                    // set all to false and later we set one to true
                    responses[k].setSelectedAnswer(new Boolean(false));
                }
                responses[responses.length - 1].setSelectedAnswer(new Boolean(
                    true));
                responses[responses.length - 1].setComments("Comment "
                        + index++);

            }
        }
    }

    public static void comparePublishedScorecard(AssignedScorecard aScorecard)
    {
        Assert.assertEquals(aScorecard.getScorecardState().getPublished()
            .booleanValue(), true);

    }

    public static void compareScorecardStateActive(AssignedScorecard aScorecard1, AssignedScorecard aScorecard2)
    {
        Assert.assertEquals(aScorecard1.getScorecardState().getActive()
            .booleanValue(), aScorecard2.getScorecardState().getActive()
            .booleanValue());
    }
    
    //TODO activate parentFixtures when implemented
    private static void compareParents(RPMObject parent1, RPMObject parent2)
            throws AssertionFailedError, Exception
    {

        if (!parent1.getID().equals(parent2.getID()))
        {
            Assert.fail("Parents have different ID");
        }

        if (!parent1.getClass().equals(parent2.getClass()))
        {
            Assert.fail("Parents have different Type");
        }

    }

public static void compare(AbstractScorecardElement obj1,
            AbstractScorecardElement obj2, AbstractScorecardScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(obj1, obj2);

        if (obj1 instanceof AssignedScorecard)
        {
            ScorecardScope scorecardScope = (ScorecardScope) scope;
            if (obj1.getParent() != null && obj2.getParent() != null) 
            {
                compareParents(obj1.getParent(), obj2.getParent());
            }
            
            if (scope != null)
            {
                if (scorecardScope.getScorecardCategories() != null)
                {
                    ScorecardCategoryScope categoryScope = scorecardScope
                        .getScorecardCategories();
                    compare(((AssignedScorecard) obj1)
                        .getAssignedScorecardCategories(),
                        ((AssignedScorecard) obj2)
                            .getAssignedScorecardCategories(), categoryScope);
                }

            RPMObjectFixture.compare(((AssignedScorecard) obj1)
                    .getScorecardState(), ((AssignedScorecard) obj2)
                    .getScorecardState());
            }
        }
        else if (obj1 instanceof AssignedScorecardCategory)
        {
            ScorecardQuestionScope questionScope = scope == null ? null 
                    : ((ScorecardCategoryScope) scope).getScorecardQuestions();
            
            compare(((AssignedScorecardCategory) obj1)
                    .getAssignedScorecardQuestions(),
                    ((AssignedScorecardCategory) obj2)
                            .getAssignedScorecardQuestions(),
                    questionScope);
        }
        else if (obj1 instanceof AssignedScorecardQuestion)
        {
            ScorecardResponseScope responseScope = scope == null ? null 
                    : ((ScorecardQuestionScope) scope).getScorecardResponses();
            
            compare(((AssignedScorecardQuestion) obj1)
                    .getAssignedScorecardResponses(),
                    ((AssignedScorecardQuestion) obj2)
                            .getAssignedScorecardResponses(),
                    responseScope);
        }

    }    public static void compare(AbstractScorecardElement[] pArray1,
            AbstractScorecardElement[] pArray2, AbstractScorecardScope scope)
            throws AssertionFailedError, Exception
    {

        sortWithComparator(pArray1, RPMObjectIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);

        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            compare(pArray1[i], pArray2[i], scope);
        }
    }

    /**
     * Verify whether the scoecard is the same as the assignment
     *
     * @param scorecard -
     *            scorecard that we assigned
     * @param assignedScorecard -
     *            the resulted assigned
     * @param updated -
     *            whether we updated the assigned after the assignment
     * @throws AssertionFailedError
     * @throws Exception
     */
    public static void compare(Scorecard scorecard,
            AssignedScorecard assignedScorecard, boolean updated)
            throws AssertionFailedError, Exception
    {
        if (!updated) // name of scorecard can be updated
        {
            Assert.assertEquals(scorecard.getName(), assignedScorecard
                .getName());
        }

        compare(scorecard.getScorecardCategories(), assignedScorecard
            .getAssignedScorecardCategories(), updated);
    }

    public static void compare(ScorecardCategory scorecard,
            AssignedScorecardCategory assignedScorecard, boolean updated)
            throws AssertionFailedError, Exception
    {
        Assert.assertEquals(scorecard.getName(), assignedScorecard.getName());

        compare(scorecard.getScorecardQuestions(), assignedScorecard
            .getAssignedScorecardQuestions(), updated);
    }

    public static void compare(ScorecardQuestion scorecard,
            AssignedScorecardQuestion assignedScorecard, boolean updated)
            throws AssertionFailedError, Exception
    {
        if (!updated)
        {
            Assert.assertEquals(scorecard.getToReport().booleanValue(),
                assignedScorecard.getToReport().booleanValue());
        }

        Assert.assertEquals(scorecard.getName(), assignedScorecard.getName());

        compare(scorecard.getScorecardResponses(), assignedScorecard
            .getAssignedScorecardResponses(), updated);
    }

    public static void compare(ScorecardResponse scorecard,
            AssignedScorecardResponse assignedScorecard, boolean updated)
            throws AssertionFailedError, Exception
    {
        if (!updated)
        {
            Assert.assertEquals(scorecard.getDefaultResponse().booleanValue(),
                assignedScorecard.getSelectedAnswer().booleanValue());
        }

        Assert.assertEquals(scorecard.getName(), assignedScorecard.getName());
    }

    public static void compare(AbstractScorecardElement[] pArray1,
            AbstractScorecardElement[] pArray2, boolean updated)
            throws AssertionFailedError, Exception
    {

        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);

        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            if (pArray1[i] instanceof Scorecard)
                compare((Scorecard) pArray1[i], (AssignedScorecard) pArray2[i],
                    updated);
            else if (pArray1[i] instanceof ScorecardCategory)
                compare((ScorecardCategory) pArray1[i],
                    (AssignedScorecardCategory) pArray2[i], updated);
            else if (pArray1[i] instanceof ScorecardQuestion)
                compare((ScorecardQuestion) pArray1[i],
                    (AssignedScorecardQuestion) pArray2[i], updated);
            else if (pArray1[i] instanceof ScorecardResponse)
                compare((ScorecardResponse) pArray1[i],
                    (AssignedScorecardResponse) pArray2[i], updated);
            else
                Assert.fail("Not proper type: "
                        + pArray1[i].getClass().getName());

        }
    }

    public static ScorecardScope createScorecardScope(RPMObjectScope parentScope)
    {
        ScorecardScope scope = ScorecardFixture.createScope().getScorecards();
        scope.setParent(parentScope);

        return scope;
    }

    public static RPMObject createParent(RPMObject parent, RPMObjectScope parentScope,
    		ClientTestController ctc) throws Exception
    {
    	RPMObject createdParent = null;

        if (parent instanceof Asset)
        {
            createdParent = AssetFixture.createFixture((AssetScope)parentScope, ctc);
        }
        else if (parent instanceof Resource)
        {
            Pool pool = PoolFixture.createSaveFixture(ctc);
            createdParent = ResourceFixture.createFixture(pool, (ResourceScope)parentScope, ctc);
        }
        else if (parent instanceof GenericProject)
        {
            createdParent = WorkElementFixture.createFixture(
                    (WorkElement)parent, (WorkElementScope)parentScope, ctc);
        }
        else if (parent instanceof AbstractScope)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            createdParent = ScopeElementFixture.createScopeElementFixture((ScopeElementScope)parentScope, 1, (ScopeElement)parent, mainScopeFolder, ctc);
        } 
        else
        {
            throw new Exception("PublishedScorecardFixture.createParent : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return createdParent;
    }
    
    public static AssignedScorecard[] getAssignedScorecardsFromParent(
            RPMObject parent) throws Exception
    {

        if (parent instanceof Asset)
        {
            AssignedScorecard aScorecard = ((Asset) parent)
                .getAssignedScorecard();
            if (aScorecard != null)
                return new AssignedScorecard[]
                {
                    aScorecard
                };

        }
        else if (parent instanceof GenericProject)
            return ((GenericProject) parent).getAssignedScorecards();
        else if (parent instanceof AbstractScope)
        {
            AssignedScorecard aScorecard = ((AbstractScope) parent)
                .getAssignedScorecard();

            if (aScorecard != null)
                return new AssignedScorecard[]
                {
                    aScorecard
                };
        }
        else
            Assert.fail("parent is not Scorecard Assignable");

        return null;
    }

    /**
     * @param loaded
     * @throws Exception
     */
    public static void deleteScorecard(RPMObject parent,
            ClientTestController ctc)
    {
        try
        {
            ctc.delete(getAssignedScorecardsFromParent(parent)[0]);
        }
        catch (Exception e)
        {
        }
    }

}