/* Created on 4-May-2006
 * @author mmatwiy
 * Test deletion of assigned scorecards for all three appplicable contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardAssetsValidationUtility;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardScopeValidationUtility;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardWBSValidationUtility;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestDeleteAssignedScorecard extends ClientTestController
{

    public void testDeleteAssignedScorecardScope() throws Exception
    {
        ScorecardScopeValidationUtility utility = new ScorecardScopeValidationUtility();

        ScopeManagementModule module = utility.createModule();
        
        Action assignedAction = utility.prepareActionAssignment(module);
        
        assertNotNull(assignedAction);
        
        ScopeElementScope scope = new ScopeElementScope();
        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Action[id='"
                + assignedAction.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Action);
        
        Action reloadedAction = (Action)rpmObjects[0];
        assertNotNull(reloadedAction.getAssignedScorecard());
        
        delete(reloadedAction.getAssignedScorecard());
        
        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Action[id='"
                + assignedAction.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Action);
        
        Action reloadedAction2 = (Action)rpmObjects[0];
        
        assertNull(reloadedAction2.getAssignedScorecard());
    }
    
    public void testDeleteAssignedScorecardAsset() throws Exception
    {
        ScorecardAssetsValidationUtility utility = new ScorecardAssetsValidationUtility();

        AssetModule module = utility.createAssetModule();
        
        Asset assignedAsset = utility.prepareScorecardAssignment(module);
        
        assertNotNull(assignedAsset);
        
        AssetScope scope = new AssetScope();
        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='"
                + assignedAsset.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        
        Asset reloadedAsset = (Asset)rpmObjects[0];
        assertNotNull(reloadedAsset.getAssignedScorecard());
        
        delete(reloadedAsset.getAssignedScorecard());
        
        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Asset[id='"
                + assignedAsset.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);
        
        Asset reloadedAsset2 = (Asset)rpmObjects[0];
        
        assertNull(reloadedAsset2.getAssignedScorecard());
    }
    
    public void testDeleteAssignedScorecardWBS() throws Exception
    {
        ScorecardWBSValidationUtility utility = new ScorecardWBSValidationUtility();

        WbsModule module = utility.createModule();
        
        Project assignedProject = utility.prepareProjectAssignment(module);
        
        assertNotNull(assignedProject);
        
        WorkElementScope scope = new WorkElementScope();
        scope.setAssignedScorecards(PublishedScorecardFixture
            .createScorecardScope(scope));
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + assignedProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);
        
        Project reloadedProject = (Project)rpmObjects[0];
        assertNotNull(reloadedProject.getAssignedScorecards()[0]);
        
        delete(reloadedProject.getAssignedScorecards()[0]);
        
        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Project[id='"
                + assignedProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);
        
        Project reloadedProject2 = (Project)rpmObjects[0];
        
        assertNull(reloadedProject2.getAssignedScorecards());
    }

}