/*
 * Created on 8-May-2006
 * @author mmatwiy
 * Description
 */
package com.ibm.rpm.client.junit.scorecard.testcases;

import java.util.GregorianCalendar;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardAssetsValidationUtility;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardScopeValidationUtility;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardWBSValidationUtility;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

public class TestPublishScorecard extends ClientTestController
{

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void testWBSPublish() throws Exception
    {
        String proposalName = HelperTestUtilities.unitTestPrefix 
        	+ "proposalM2-2";
        String portfolioName = HelperTestUtilities.unitTestPrefix 
        	+ "Publish Portfolio2-2";

        WorkElementScope scope = new WorkElementScope();

        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(scope);
        scope.setAssignedScorecards(scorecardScope);

        PortfolioScope portfolioScope = new PortfolioScope();
        portfolioScope.setProjects(scope);
        portfolioScope.setPortfolios(new PortfolioScope());

        Proposal proposal = new Proposal();
        proposal.setName(proposalName);
        proposal.setAssignmentType(AssignmentType.Duration);
        proposal.setEstimatedStartDate(new GregorianCalendar(2005, 10, 15));
        proposal.setEstimatedFinishDate(new GregorianCalendar(2005, 10, 25));
        proposal.setDefaultProject(new Boolean(false));

        Portfolio portfolio = new Portfolio();
        portfolio.setName(portfolioName);
        portfolio.setProjects(new Proposal[]
        {
            proposal
        });

        SaveResult result = saveWithoutDelete(portfolio, portfolioScope);
        assertTrue(result.isSuccessful());

        ScorecardWBSValidationUtility util = new ScorecardWBSValidationUtility();
        WbsModule module = util.createModule();

        Proposal assignedProposal = (Proposal) getApplicationInterface()
            .assignScoreCard(getSessionID(), proposal, scope,
                module.getScorecardFolders()[0].getScorecards()[0],
                ReloadType.ReloadResult).getRpmObject();

        assertNotNull(assignedProposal.getAssignedScorecards());

        SingleResult result2 = getApplicationInterface().publishScorecard(
            getSessionID(), assignedProposal.getAssignedScorecards()[0],
            scorecardScope);

        assertTrue(result2.isSuccessful());

        AssignedScorecard publishedScorecard = (AssignedScorecard) result2
            .getRpmObject();

        assertNotNull(publishedScorecard);

        logger.debug("published scorecard: " + publishedScorecard.getName());
    }
    
    public void testAssetsPublish() throws Exception
    {
        AssetScope assetScope = new AssetScope();

        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(assetScope);
        assetScope.setAssignedScorecard(scorecardScope);

        Asset asset = new Asset();
        asset.setParent(AssetFolderFixture.createSaveFixture(this, null));
        asset.setName("asset test publish");

        SaveResult result = saveWithoutDelete(asset, assetScope);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        ScorecardAssetsValidationUtility util = new ScorecardAssetsValidationUtility();
        AssetModule module = util.createAssetModule();

        Asset assignedAsset = (Asset) getApplicationInterface()
            .assignScoreCard(getSessionID(), asset, assetScope,
                module.getScorecardFolders()[0].getScorecards()[0],
                ReloadType.ReloadResult).getRpmObject();

        assertNotNull(assignedAsset.getAssignedScorecard());

        SingleResult result2 = getApplicationInterface().publishScorecard(
            getSessionID(), assignedAsset.getAssignedScorecard(),
            scorecardScope);

        assertTrue(result2.isSuccessful());

        AssignedScorecard publishedScorecard = (AssignedScorecard) result2
            .getRpmObject();

        assertNotNull(publishedScorecard);

        logger.debug("published scorecard: " + publishedScorecard.getName());
    }
    
    /**
     * The API needs to know what scorecard to assign to which resource under which project.
     * 
     * We need to verify that the resource does indeed have work assigned to the project before
     * assigning a scorecard to it. In other words, we can't evaluate work which hasn't been at least
     * assigned to the resource.
     * 
     * @throws Exception
     */
    public void testResourcePublish() throws Exception
    {
        ResourceScope resourceScope = new ResourceScope();

        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(resourceScope);
        resourceScope.setAssignedScorecard(scorecardScope);
        
        Resource resourceToReceiveScorecard = ResourceFixture.createSaveFixture(this);
        
        GenericProject projectToRate = createSaveWorkForResourceUnderNewProject(resourceToReceiveScorecard);
        
        ScorecardAssetsValidationUtility util = new ScorecardAssetsValidationUtility();
        ResourceModule module = util.createResourceModule();
        
        assertNotNull(module.getScorecardFolders());
        assertTrue(module.getScorecardFolders().length > 0);
        assertNotNull(module.getScorecardFolders()[0].getScorecards());
        assertTrue(module.getScorecardFolders()[0].getScorecards().length > 0);
        
        Scorecard scorecardToBeAssigned  = module.getScorecardFolders()[0].getScorecards()[0];
        
        Resource hasReceivedScorecard = 
            (Resource)getApplicationInterface().assignScoreCardToResource(
            getSessionID(), projectToRate, resourceToReceiveScorecard, resourceScope, scorecardToBeAssigned, 
            ReloadType.ReloadResult).getRpmObject();
        assertNotNull("Returned object is null", hasReceivedScorecard);
        assertNotNull("Assigned scorecard was not loaded ", hasReceivedScorecard.getAssignedScorecard());

        RPMObject[] results = loadFromXpath("/Resource[@id='" + resourceToReceiveScorecard.getID() + 
            "']/ResourceAssignedScorecard[@project[@id='" + projectToRate.getID() + "']]", null);
        
        assertTrue("Did not receive exactly one result : " + results, results != null && results.length == 1);
        assertTrue("Return type is incorrect ", results[0] instanceof AssignedScorecard);
    }
     
    
    private GenericProject createSaveWorkForResourceUnderNewProject(Resource receiveScorecard) throws Exception
    { 
        ResourceTaskAssignment taskAssignment = ResourceTaskAssignmentFixture.createFixture(new 
            ResourceTaskAssignmentScope(), false, this);
        taskAssignment.setResource(receiveScorecard);
        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setParent(new RPMObjectScope());
        taskAssignment = (ResourceTaskAssignment) save(taskAssignment, rtaScope).getRpmObject();
        
        WorkElement attachedTo = (WorkElement) taskAssignment.getParent();
        WorkElementScope weScope = new WorkElementScope();
        weScope.setContainingProject(new WorkElementScope());
        attachedTo = (WorkElement) loadFromID(attachedTo, weScope);
        
        assertNotNull("Could not assign work to resource", attachedTo);
        assertNotNull("Containing project is null", attachedTo.getContainingProject());
        
        return (GenericProject) attachedTo.getContainingProject();
    }

    public void testScopePublish() throws Exception
    {
        ScopeElementScope scopeElementScope = new ScopeElementScope();

        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(scopeElementScope);
        scopeElementScope.setAssignedScorecard(scorecardScope);

        ScorecardScopeValidationUtility util = new ScorecardScopeValidationUtility();
        ScopeManagementModule module = util.createModule();
        
        Action action = new Action();
        action.setParent(util.createScopeFolder(scopeElementScope));
        action.setName("action test publish");

        SaveResult result = saveWithoutDelete(action, scopeElementScope);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        Action assignedAction = (Action) getApplicationInterface()
            .assignScoreCard(getSessionID(), action, scopeElementScope,
                module.getScorecardFolders()[0].getScorecards()[0],
                ReloadType.ReloadResult).getRpmObject();

        assertNotNull(assignedAction.getAssignedScorecard());

        SingleResult result2 = getApplicationInterface().publishScorecard(
            getSessionID(), assignedAction.getAssignedScorecard(),
            scorecardScope);

        assertTrue(result2.isSuccessful());

        AssignedScorecard publishedScorecard = (AssignedScorecard) result2
            .getRpmObject();

        assertNotNull(publishedScorecard);

        logger.debug("published scorecard: " + publishedScorecard.getName());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
}