package com.ibm.rpm.client.junit.scorecard.testcases;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scorecard.containers.AbstractScorecardEntity;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.wbs.containers.WbsModule;

/**
 * @author ragoler
 *
 * Various unit tests to verify the Scorecards implmenetation
 */
public class TestScorecard extends ClientTestController
{

    protected ScorecardFolderScope mainScope;

    protected AbstractModule module;
    

    public TestScorecard()
    {
        //default context
        module = new AssetModule();
    }

    public TestScorecard(String testName, String contextName)
    {
        setName(testName);

        if (contextName.equals(AttributeFixture.WBS_CONTEXT))
        {
            module = new WbsModule();
        }
        else if (contextName.equals(AttributeFixture.SCOPE_CONTEXT))
        {
            module = new ScopeManagementModule();
        }
        else if (contextName.equals(AttributeFixture.ASSET_CONTEXT))
        {
            module = new AssetModule();
        }
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");

        super.setUp();

        //set mainScope
        mainScope = ScorecardFixture.createScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    /**
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromXPath()
     *
     * Test xpath for /SCORECARDFOLDER[id=' <id>'] w/o scope
     */
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ScorecardFolder folder = createScorecardFolder(null);

        verifyXPath("/SCORECARDFOLDER[id='" + folder.getID() + "']", folder,
            null);
        
        deleteFolder(folder);
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for /SCORECARDFOLDER[Name=' <id>'] w scope
     */
    public void testInsertWithScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        verifyXPath("/SCORECARDFOLDER[Name='" + folder.getName() + "']",
            folder, mainScope);
        
        deleteFolder(folder);
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for /SCORECARDFOLDER[/SCORECARD[id=' <id>']]
     */
    public void testInsertWithScopeLoadFromXPathUsingScorecard()
            throws AssertionError, Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        verifyXPath(
            "/SCORECARDFOLDER[/SCORECARD[Id='"
                    + folder.getScorecards()[ScorecardFixture.INITIAL_NB_SCORECARDS - 1]
                        .getID() + "']]", folder, mainScope);
        
        deleteFolder(folder);
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for /SCORECARDFOLDER[name='
     *             <name>']/SCORECARD[Id=' <id>]
     */
    public void testInsertWithScopeLoadScorecardFromXPath()
            throws AssertionError, Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        Scorecard scorecard = folder.getScorecards()[ScorecardFixture.INITIAL_NB_SCORECARDS - 1];

        verifyXPath("/SCORECARDFOLDER[Name='" + folder.getName()
                + "']/SCORECARD[Id='" + scorecard.getID() + "']", scorecard,
            mainScope.getScorecards());
        
        deleteFolder(folder);
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for /SCORECARDFOLDER[name='
     *             <name>']/SCORECARD[Name=' <name>]/SCORECARDCATEGORY[Id='
     *             <id>']
     */
    public void testInsertWithScopeLoadScorecardCategoryFromXPath()
            throws AssertionError, Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        Scorecard scorecard = folder.getScorecards()[ScorecardFixture.INITIAL_NB_SCORECARDS - 1];

        ScorecardCategory scorecardCategory = scorecard
            .getScorecardCategories()[ScorecardFixture.INITIAL_NB_CATEGORIES - 1];

        verifyXPath("/SCORECARDFOLDER[Name='" + folder.getName()
                + "']/SCORECARD[Name='" + scorecard.getName()
                + "']/SCORECARDCATEGORY[Id='" + scorecardCategory.getID()
                + "']", scorecardCategory, mainScope.getScorecards()
            .getScorecardCategories());
        
        deleteFolder(folder);
        
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for /SCORECARDFOLDER[/SCORECARD[Id=' <id>]]
     */
    public void testInsertWithScopeScorecardConditionFromXPath()
            throws AssertionError, Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        Scorecard scorecard = folder.getScorecards()[ScorecardFixture.INITIAL_NB_SCORECARDS - 1];

        verifyXPath("/SCORECARDFOLDER[/SCORECARD[Id='" + scorecard.getID()
                + "']]", folder, mainScope);
        
        deleteFolder(folder);
    }

    /**
     * @throws AssertionError
     * @throws Exception
     *             Test xpath for
     *             /SCORECARDFOLDER[/SCORECARD[/SCORECARDCATEGORY[Id=' <id>']]]
     */
    public void testInsertWithScopeScorecardCategoryConditionFromXPath()
            throws AssertionError, Exception
    {
        ScorecardFolder folder = createScorecardFolder(mainScope);

        Scorecard scorecard = folder.getScorecards()[ScorecardFixture.INITIAL_NB_SCORECARDS - 1];

        ScorecardCategory scorecardCategory = scorecard
            .getScorecardCategories()[ScorecardFixture.INITIAL_NB_CATEGORIES - 1];

        verifyXPath("/SCORECARDFOLDER[/SCORECARD[/SCORECARDCATEGORY[Id='"
                + scorecardCategory.getID() + "']]]", folder, mainScope);
        
        deleteFolder(folder);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(null);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(mainScope);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(null);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(mainScope);
    }

    private void insert(ScorecardFolderScope scope) throws AssertionError,
            Exception
    {
        //insert an element
        ScorecardFolder folder = createScorecardFolder(scope);

        folder = loadFolder(folder, scope);

        //delete the element
        deleteFolder(folder);
    }

    private void update(ScorecardFolderScope scope) throws AssertionError,
            Exception
    {
        //insert an element
        ScorecardFolder folder = createScorecardFolder(scope);

        folder = updateStructure(folder, scope, false);

        // all new objects must be saved using a module
        //folder = updateStructure(folder, scope, true);

        //delete the element
        deleteFolder(folder);
    }

    private ScorecardFolder createScorecardFolder(ScorecardFolderScope scope)
            throws Exception
    {

        ScorecardFolder folder = ScorecardFixture.createSaveFixture(module,
            scope, this);

         //return the folder
        return folder;
    }

    private ScorecardFolder saveFolder(ScorecardFolder folder,
            ScorecardFolderScope scope) throws Exception, AssertionFailedError
    {
        ScorecardFolder savedFolder = ScorecardFixture.saveFixture(folder,
            scope, this);

        return savedFolder;
    }

    private ScorecardFolder loadFolder(ScorecardFolder savedFolder,
            ScorecardFolderScope scope) throws Exception, AssertionFailedError
    {
        //load from an id
        RPMObject obj = new ScorecardFolder();
        obj.setID(savedFolder.getID());
        obj.setContextName(savedFolder.getContextName());
        ScorecardFolder loadedFolder = (ScorecardFolder) loadFromID(obj, scope);

        ScorecardFixture.compare(loadedFolder, savedFolder, scope);
        return loadedFolder;
    }

    private ScorecardFolder updateStructure(ScorecardFolder folder,
            ScorecardFolderScope scope, boolean updateStructure)
            throws AssertionError, Exception
    {
        if (!updateStructure)
        {
            //updates the elements properties
            ScorecardFixture.updateFolder(folder, scope);
        }
        else
        {
            //updates the structure by adding elements
            ScorecardFixture.updateFolderStructure(folder, scope);
        }

        ScorecardFolder savedFolder = saveFolder(folder, scope);

        ScorecardFolder loadedFolder = loadFolder(savedFolder, scope);

        return loadedFolder;

    }

    private void deleteFolder(ScorecardFolder folder) throws Exception
    {
        //delete the element
        delete(folder);

        //reload the deleted element
        ScorecardFolder deletedFolder = loadDeletedFolder(folder);

        //validate deleted element
        assertNull(deletedFolder);
    }

    private ScorecardFolder loadDeletedFolder(ScorecardFolder folder)
            throws Exception
    {
        try
        {
            folder = loadFolder(folder, null);
            return folder;
        }
        catch (Exception e)
        {
            //element does not exist
            logger.debug("**********Good Exception**********");
        }
        return null;
    }

    private void verifyXPath(String xpath, AbstractScorecardEntity element,
            AbstractScorecardScope scope) throws Exception
    {
        RPMObject[] objs = loadFromXpath(xpath, scope);

        Assert.assertNotNull(objs);
        Assert.assertEquals(1, objs.length);
        ScorecardFixture.compare(element, objs[0], scope);

    }

}