/*
 * Created on Jan 25, 2006
 * @author mmatwiy
 * Validation tests for container AssignedScorecardCategory for all contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardCategory;

public abstract class AbstractTestAssignedScorecardCategoryValidation extends
        GenericValidation
{
    private AssignedScorecardCategory scorecardCategory;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void initializeValues(AssignedScorecardCategory category)
            throws Exception
    {
        scorecardCategory = category;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void readOnlyFields() throws AssertionError, Exception
    {
        String previousName = scorecardCategory.getName();
        Integer previousWeight = scorecardCategory.getWeight();
        
        // Set read-only field values and verify that they are not accepted
        scorecardCategory
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        scorecardCategory
            .setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    scorecardCategory.getClass().getName(),
                    ValidationConstants.NAME_FIELD, scorecardCategory.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    scorecardCategory.getClass().getName(),
                    ValidationConstants.WEIGHT_FIELD, scorecardCategory.getID())
        };

        verifySaveMustFail(scorecardCategory, null, null, expectedWarnings);

        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardCategory[id='" + scorecardCategory.getID()
                    + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardCategory);
        AssignedScorecardCategory loadedScorecardCategory = (AssignedScorecardCategory) rpmObjects[0];

        // we can expect these to be equal because they're not calculated fields
        assertEquals(previousName, loadedScorecardCategory.getName());
        assertEquals(previousWeight, loadedScorecardCategory.getWeight());
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are set
        assertNotNull(scorecardCategory.getProbabilityType());
        assertNotNull(scorecardCategory.getWeight());
        assertNotNull(scorecardCategory.getScore());

        // Save again to validate the default values
        scorecardCategory = (AssignedScorecardCategory) saveMustSucceed(
            scorecardCategory, AssignedScorecardCategory.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardCategory[id='" + scorecardCategory.getID()
                    + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardCategory);

        PublishedScorecardFixture.compare(scorecardCategory,
            (AssignedScorecardCategory) rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        // Update the default values to null and verify validation errors;
        // weight is read-only
        scorecardCategory.setProbabilityType(null);
        scorecardCategory.setScore(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    scorecardCategory.getClass().getName(),
                    ValidationConstants.PROBABILITY_TYPE_FIELD,
                    scorecardCategory.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    scorecardCategory.getClass().getName(),
                    ValidationConstants.SCORE_FIELD, scorecardCategory.getID())
        };

        verifySaveMustFail(scorecardCategory, null, expectedExceptions);

    }
}