/*
 * Created on Jan 25, 2006
 * @author mmatwiy
 * Validation tests for container AssignedScorecardQuestion for all contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;

public abstract class AbstractTestAssignedScorecardQuestionValidation extends
        GenericValidation
{
    private AssignedScorecardQuestion question;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void initializeValues(AssignedScorecardQuestion scorecardQuestion)
    {
        question = scorecardQuestion;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void readOnlyFields() throws AssertionError, Exception
    {
        String previousName = question.getName();
        Integer previousWeight = question.getWeight();
        
        // Set read-only field values and verify that they are not accepted
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        question.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, question
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    question.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, question
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    question.getID())
        };

        verifySaveMustFail(question, null, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardQuestion[id='" + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardQuestion);
        
        AssignedScorecardQuestion loadedQuestion = (AssignedScorecardQuestion)rpmObjects[0];
        
        // we can expect these to be equal because they're not calculated fields
        assertEquals(previousName, loadedQuestion.getName());
        assertEquals(previousWeight, loadedQuestion.getWeight());
        
        
    }

    protected void selectedResponses() throws AssertionError, Exception
    {
        ScorecardQuestionScope scope = new ScorecardQuestionScope();
        scope.setScorecardResponses(new ScorecardResponseScope());
        
        // Set all responses to the question as selected and verify
        // that they are not accepted
        AssignedScorecardResponse[] responses = question
            .getAssignedScorecardResponses();

        for (int i = 0; i < responses.length; i++)
        {
            responses[i].setSelectedAnswer(new Boolean(true));
        }

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.scorecard.constants.ErrorCodes.WRONG_NUMBER_OF_RESPONSES_FOR_QUESTION,
                question.getClass().getName())
        };

        verifySaveMustFail(question, scope, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are set
        assertNotNull(question.getResponseDate());
        assertNotNull(question.getWeight());
        assertNotNull(question.getScore());
        assertNotNull(question.getToReport());

        // Save again to validate the default values
        question = (AssignedScorecardQuestion) saveMustSucceed(question,
            AssignedScorecardQuestion.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardQuestion[id='" + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardQuestion);

        PublishedScorecardFixture.compare(question,
            (AssignedScorecardQuestion) rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        // Check that the default values are set
        assertNotNull(question.getWeight());
        assertNotNull(question.getResponseDate());
        assertNotNull(question.getScore());
        assertNotNull(question.getToReport());

        // Update the default values to null and verify validation errors;
        // weight is read-only
        question.setResponseDate(null);
        question.setScore(null);
        question.setToReport(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, question
                    .getClass().getName(),
                    ValidationConstants.RESPONSE_DATE_FIELD, question.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, question
                    .getClass().getName(), ValidationConstants.SCORE_FIELD,
                    question.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, question
                    .getClass().getName(), ValidationConstants.TOREPORT_FIELD,
                    question.getID())
        };

        verifySaveMustFail(question, null, expectedExceptions);

    }
}
