/*
 * Created on Jan 25, 2006
 * @author mmatwiy
 * Validation tests for container AssignedScorecardResponse for all contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;

public abstract class AbstractTestAssignedScorecardResponseValidation extends
        GenericValidation
{
    private AssignedScorecardResponse response;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void initializeValues(AssignedScorecardResponse scorecardResponse)
    {
        response = scorecardResponse;
    }

    protected void readOnlyFields() throws AssertionError, Exception
    {
        String previousName = response.getName();
        Integer previousWeight = response.getWeight();
        
        // Set read-only field values and verify that they are not accepted
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        response.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, response
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    response.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, response
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    response.getID())
        };

        verifySaveMustFail(response, null, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardResponse[id='" + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardResponse);
        AssignedScorecardResponse loadedResponse = (AssignedScorecardResponse)rpmObjects[0];
        
        // we can expect these to be equal because they're not calculated fields
        assertEquals(previousName, loadedResponse.getName());
        assertEquals(previousWeight, loadedResponse.getWeight());        
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set comments field with maxLength + 1 and verify that it is not accepted
        response
            .setComments(getTestString(ValidationConstants.COMMENTS_CHARACTER_LIMIT + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, response.getClass()
                .getName(), ValidationConstants.COMMENTS_FIELD, response
                .getID())
        };

        verifySaveMustFail(response, null, expectedExceptions);

    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        response
            .setComments(getTestString(ValidationConstants.COMMENTS_CHARACTER_LIMIT));

        // Attempt to save the AssignedScorecardResponse object
        response = (AssignedScorecardResponse) saveMustSucceed(response,
            AssignedScorecardResponse.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardResponse[id='" + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardResponse);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are now set
        assertNotNull(response.getSelectedAnswer());
        assertNotNull(response.getWeight());
        assertNotNull(response.getScore());

        // Save again to validate the default values
        response = (AssignedScorecardResponse) saveMustSucceed(response,
            AssignedScorecardResponse.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/AssignedScorecardResponse[id='" + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardResponse);

        PublishedScorecardFixture.compare(response,
            (AssignedScorecardResponse) rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        // Update the default values to null and verify validation errors;
        // weight is read-only
        response.setSelectedAnswer(null);
        response.setScore(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, response
                    .getClass().getName(),
                    ValidationConstants.SELECTED_ANSWER_FIELD, response.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, response
                    .getClass().getName(), ValidationConstants.SCORE_FIELD,
                    response.getID())
        };

        verifySaveMustFail(response, null, expectedExceptions);

    }
}
