/*
 * Created on Dec 8, 2005 @author mmatwiy
 * 
 * Validation tests for container AssignedScorecard for all contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;

public abstract class AbstractTestAssignedScorecardValidation extends
        GenericValidation
{
    private AssignedScorecard scorecard;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void initializeValues(AssignedScorecard assignedScorecard)
    {
        scorecard = assignedScorecard;
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, scorecard.getClass()
                .getName(), ValidationConstants.NAME_FIELD, scorecard.getID())
        };

        verifySaveMustFail(scorecard, null, expectedExceptions);
    }

    public void failureValidationMin() throws AssertionError, Exception
    {
        scorecard.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, scorecard.getClass()
                .getName(), ValidationConstants.NAME_FIELD, scorecard.getID())
        };

        verifySaveMustFail(scorecard, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        scorecard.setName(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, scorecard.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, scorecard
                    .getID()),
        };

        verifySaveMustFail(scorecard, null, expectedExceptions);
    }

    protected void readOnlyFields() throws AssertionError, Exception
    {
        Integer previousWeight = scorecard.getWeight();

        scorecard.setName("test");

        // Set read-only field values and verify that they are not accepted
        scorecard.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, scorecard.getClass()
                .getName(), ValidationConstants.WEIGHT_FIELD, scorecard.getID())
        };

        verifySaveMustFail(scorecard, null, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssignedScorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecard);
        
        AssignedScorecard loadedScorecard = (AssignedScorecard)rpmObjects[0];
        
        // we can expect the weight to be equal because it's not a calculated field
        assertEquals(previousWeight, loadedScorecard.getWeight());
        
    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));

        // Attempt to save the AssignedScorecard object
        scorecard = (AssignedScorecard) saveMustSucceed(scorecard,
            AssignedScorecard.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssignedScorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecard);

        PublishedScorecardFixture.compare(scorecard,
            (AssignedScorecard) rpmObjects[0], null);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are now set
        assertNotNull(scorecard.getWeight());
        assertNotNull(scorecard.getScore());
        assertNotNull(scorecard.getProbabilityType());

        // Save again to validate the default values
        scorecard = (AssignedScorecard) saveMustSucceed(scorecard,
            AssignedScorecard.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssignedScorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecard);

        PublishedScorecardFixture.compare(scorecard,
            (AssignedScorecard) rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        // Update the default values to null and verify validation errors
        // weight is read-only
        scorecard.setScore(null);
        scorecard.setProbabilityType(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, scorecard
                    .getClass().getName(),
                    ValidationConstants.PROBABILITY_TYPE_FIELD, scorecard
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, scorecard
                    .getClass().getName(), ValidationConstants.SCORE_FIELD,
                    scorecard.getID())
        };

        verifySaveMustFail(scorecard, null, expectedExceptions);

    }
}
