/*
 * Created on Dec 6, 2005 @author mmatwiy Validation tests for container
 * ScorecardQuestion.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;

public abstract class AbstractTestScorecardQuestionValidation extends
        GenericValidation
{
    private ScorecardQuestion question;

    private AbstractModule module;

    private RPMObjectScope scope;

    public void setUp(AbstractModule abstractModule,
            RPMObjectScope moduleScope, ScorecardQuestion scorecardQuestion)
            throws Exception
    {
        super.setUp();

        module = abstractModule;
        scope = moduleScope;
        question = scorecardQuestion;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void insertWithoutModule() throws AssertionError, Exception
    {
        // Set mandatory field
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MODULE_REQUIRED, question
                    .getClass().getName()),
                createRPMException(ErrorCodes.NO_PARENT, question.getClass()
                    .getName(), ValidationConstants.PARENT, question.getID())
        };

        verifySaveMustFail(question, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, question.getClass()
                .getName(), ValidationConstants.NAME_FIELD, question.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void insertSuccess() throws AssertionError, Exception
    {
        question.setName(HelperTestUtilities.createName());
        question
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardQuestion[id='"
                + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(question, rpmObjects[0], null);
    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        question
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));
        question.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardQuestion[id='"
                + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(question, rpmObjects[0], null);
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT + 1));
        question
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT + 1));
        question.setWeight(new Integer(ValidationConstants.WEIGHT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, question.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, question
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, question.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD, question
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, question.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, question
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void successValidationMin() throws AssertionError, Exception
    {
        // Set mandatory fields
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Set field values with min value and verify that they are accepted
        question.setWeight(new Integer(ValidationConstants.WEIGHT_MIN));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardQuestion[id='"
                + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(question, rpmObjects[0], null);
    }

    protected void failureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min value - 1 and verify that they are not
        // accepted
        question.setName("");
        question.setWeight(new Integer(ValidationConstants.WEIGHT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, question.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, question
                    .getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, question.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, question
                    .getID())

        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void failureNumberOfDefaultResponses() throws AssertionError,
            Exception
    {
        // Set mandatory field
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        ScorecardResponse response = new ScorecardResponse();
        response.setName("response name");
        
        ScorecardResponse responses[] =
        {
                response, response, response
        };
        question.setScorecardResponses(responses);

        //Set the default flag for all of the responses to the question
        for (int i = 0; i < responses.length; i++)
        {
            responses[i].setDefaultResponse(new Boolean(true));
        }

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_NUMBER_OF_DEFAULTS, question
                .getClass().getName(), ValidationConstants.RESPONSES_FIELD,
                question.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardQuestion[id='"
                + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        // Check that the default values are now set
        ScorecardQuestion savedQuestion = (ScorecardQuestion) rpmObjects[0];
        assertNotNull(savedQuestion.getToReport());
        assertNotNull(savedQuestion.getWeight());

        // Save again to validate the default values
        savedQuestion = (ScorecardQuestion) saveMustSucceed(savedQuestion,
            ScorecardQuestion.class, null);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ScorecardQuestion[id='" + question.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        ScorecardFixture.compare(question, rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        question
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardQuestion[id='"
                + question.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardQuestion);

        // Check that the default values are now set
        ScorecardQuestion savedQuestion = (ScorecardQuestion) rpmObjects[0];
        assertNotNull(question.getWeight());
        assertNotNull(question.getToReport());

        // Update the default values to null and verify validation errors
        savedQuestion.setWeight(null);
        savedQuestion.setToReport(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedQuestion
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    savedQuestion.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedQuestion
                    .getClass().getName(), ValidationConstants.TOREPORT_FIELD,
                    savedQuestion.getID())
        };

        verifySaveMustFail(savedQuestion, null, expectedExceptions);

    }
}
