/*
 * Created on Dec 5, 2005 @author mmatwiy Validation tests for container
 * Scorecard.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.Scorecard;

public abstract class AbstractTestScorecardValidation extends GenericValidation
{
    private Scorecard scorecard;

    private AbstractModule module;

    private RPMObjectScope scope;

    public void setUp(AbstractModule abstractModule,
            RPMObjectScope moduleScope, Scorecard card) throws Exception
    {
        super.setUp();

        module = abstractModule;
        scope = moduleScope;
        scorecard = card;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void insertWithoutModule() throws AssertionError, Exception
    {
        // Set mandatory field
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_PARENT, scorecard.getClass()
                    .getName(), ValidationConstants.PARENT, scorecard.getID()),
                createRPMException(ErrorCodes.MODULE_REQUIRED, scorecard
                    .getClass().getName())
        };

        verifySaveMustFail(scorecard, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, scorecard.getClass()
                .getName(), ValidationConstants.NAME_FIELD, scorecard.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void insertSuccess() throws AssertionError, Exception
    {
        scorecard.setName(HelperTestUtilities.createName());
        scorecard
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Scorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the loaded Scorecard is identical to the one we
        // requested to store
        ScorecardFixture.compare(scorecard, rpmObjects[0], null);
    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));
        scorecard
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));
        scorecard.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Scorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the loaded Scorecard is identical to the one we
        // requested to store
        ScorecardFixture.compare(scorecard, rpmObjects[0], null);
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT + 1));
        scorecard
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT + 1));
        scorecard.setWeight(new Integer(ValidationConstants.WEIGHT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, scorecard.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, scorecard
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, scorecard.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD,
                    scorecard.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, scorecard
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    scorecard.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void successValidationMin() throws AssertionError, Exception
    {
        // Set mandatory fields
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));

        // Set fields with minimum values and verify that they are accepted
        scorecard.setWeight(new Integer(ValidationConstants.WEIGHT_MIN));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Scorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the loaded Scorecard is identical to the one we
        // requested to store
        ScorecardFixture.compare(scorecard, rpmObjects[0], null);
    }

    protected void failureValidationMin() throws AssertionError, Exception
    {
        // Set fields with minimum values - 1 and verify that they are not
        // accepted
        scorecard.setName("");
        scorecard.setWeight(new Integer(ValidationConstants.WEIGHT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, scorecard
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    scorecard.getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, scorecard.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, scorecard
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Scorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the default values are now set
        Scorecard savedScorecard = (Scorecard) rpmObjects[0];
        assertNotNull(scorecard.getWeight());
        assertNotNull(scorecard.getProbabilityType());
        assertNotNull(scorecard.getActive());

        // Save again to validate the default values
        savedScorecard = (Scorecard) saveMustSucceed(savedScorecard,
            Scorecard.class, null);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath(
            "/Scorecard[id='" + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ScorecardFixture.compare(scorecard, rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        scorecard
            .setName(getTestString(ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Scorecard[id='"
                + scorecard.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Scorecard);

        // Check that the default values are now set
        Scorecard savedScorecard = (Scorecard) rpmObjects[0];
        assertNotNull(scorecard.getWeight());
        assertNotNull(scorecard.getProbabilityType());
        assertNotNull(scorecard.getActive());

        // Update the default values to null and verify validation errors
        savedScorecard.setWeight(null);
        savedScorecard.setProbabilityType(null);
        savedScorecard.setActive(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedScorecard.getClass().getName(),
                    ValidationConstants.WEIGHT_FIELD, savedScorecard.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedScorecard.getClass().getName(),
                    ValidationConstants.PROBABILITY_TYPE_FIELD, savedScorecard
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedScorecard.getClass().getName(),
                    ValidationConstants.ACTIVE_FIELD, savedScorecard.getID()),
        };

        verifySaveMustFail(savedScorecard, null, expectedExceptions);

    }
}
