/*
 * Created on Jan 30, 2006
 * @author mmatwiy
 * Utility methods for use with Assets scorecards
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceModuleScope;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;

public class ScorecardAssetsValidationUtility extends GenericValidation
{
    AssetModule module;

    private static final int NB_MAX_LEVELS = 2;

    private static final int NB_ASSETS_BY_LEVEL = 1;

    public Asset prepareScorecardAssignment(AssetModule module)
            throws Exception, AssertionError, RPMException,
            AssertionFailedError
    {
        AssetScope parentScope = AssetFolderFixture.createScope(1,
            NB_MAX_LEVELS, 0, 0, null);

        AssetScope assetScope = AssetFixture.createScope(1, NB_MAX_LEVELS,
            NB_ASSETS_BY_LEVEL, parentScope);

        assetScope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(assetScope));
        Asset asset = AssetFixture.createSaveFixture(assetScope, this);

        ScorecardFolder folder = module.getScorecardFolders()[0];

        Asset assignedAsset = (Asset) PublishedScorecardFixture
            .createAssignmentFixture(asset, assetScope, this,
                ReloadType.ReloadResult, folder, 1);

        return assignedAsset;
    }

    public ScorecardFolder createScorecardFolderFixture()
    {
        ScorecardFolder folder = new ScorecardFolder();
        folder.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - folder");

        Scorecard scorecard = new Scorecard();
        scorecard.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - scorecard");
        scorecard.setActive(new Boolean(true));

        ScorecardCategory category = new ScorecardCategory();
        category.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - category");

        ScorecardQuestion question = new ScorecardQuestion();
        question.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - question");

        ScorecardResponse response1 = new ScorecardResponse();
        response1.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - response 1");
        response1.setDefaultResponse(new Boolean(true));

        ScorecardResponse response2 = new ScorecardResponse();
        response2.setName(HelperTestUtilities.unitTestPrefix + "Test assets scorecards - response 2");
  
        folder.setScorecards(new Scorecard[]
        {
            scorecard
        });
        scorecard.setScorecardCategories(new ScorecardCategory[]
        {
            category
        });
        category.setScorecardQuestions(new ScorecardQuestion[]
        {
            question
        });
        question.setScorecardResponses(new ScorecardResponse[]
        {
                response1, response2
        });

        return folder;
    }
    
    public ResourceModule createResourceModule() throws Exception
    {
        ResourceModule module = new ResourceModule();
        ScorecardFolder folder = createScorecardFolderFixture();
        module.setScorecardFolders(new ScorecardFolder[]
           {
               folder
           });
        
        
        ResourceModuleScope moduleScope = ResourceModuleFixture.createFullScope();
        module = (ResourceModule) save(module, moduleScope).getRpmObject();
        return module;
    }
    
    public AssetModule createAssetModule() throws Exception
    {
        module = new AssetModule();

        ScorecardFolder folder = createScorecardFolderFixture();
        
        module.setScorecardFolders(new ScorecardFolder[]
           {
               folder
           });
        
        
        AssetModuleScope moduleScope = AssetModuleFixture.createFullScope();

        module = (AssetModule) save(module, moduleScope).getRpmObject();

        return module;
    }
}
