/*
 * Created on Jan 27, 2006
 * @author mmatwiy
 * Utility methods for use with WBS scorecard validation
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScorecardWBSValidationUtility extends GenericValidation
{
    public WorkElementScope createScope()
    {
        WorkElementScope scope = WorkElementFixture.createScope(new Project(),
            false, false, false, false, false, false, false, false, false, 
            false, false, false, false, false,false);

        scope.setTimeCodeAssignments(false);
        scope.setAssignedScorecards(PublishedScorecardFixture
            .createScorecardScope(scope));
        scope.setChildren(null);

        return scope;
    }

    public WorkOrganization prepareWorkOrganizationAssignment(WbsModule module)
            throws AssertionError, Exception
    {
        WorkElementScope scope = createScope();

        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);

        WorkOrganization assignedOrganization = (WorkOrganization) PublishedScorecardFixture
            .createAssignmentFixture(workOrganization, scope, this,
                ReloadType.ReloadResult, module.getScorecardFolders()[0], 1);

        return assignedOrganization;
    }

    public Project prepareProjectAssignment(WbsModule module)
            throws AssertionError, Exception
    {
        WorkElementScope scope = createScope();

        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        Project assignedProject = (Project) PublishedScorecardFixture
            .createAssignmentFixture(project, scope, this,
                ReloadType.ReloadResult, module.getScorecardFolders()[0], 1);

        return assignedProject;
    }

    public Proposal prepareProposalAssignment(WbsModule module)
            throws AssertionError, Exception
    {
        WorkElementScope scope = createScope();

        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), scope, this);

        Proposal assignedProposal = (Proposal) PublishedScorecardFixture
            .createAssignmentFixture(proposal, scope, this,
                ReloadType.ReloadResult, module.getScorecardFolders()[0], 1);

        return assignedProposal;
    }

    public WbsModule createModule() throws Exception
    {

        WbsModule module = new WbsModule();

        ScorecardFolder scorecardFolder = new ScorecardFolder();
        scorecardFolder
            .setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - folder");

        Scorecard scorecard = new Scorecard();
        scorecard.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - scorecard");
        scorecard.setActive(new Boolean(true));

        ScorecardCategory category = new ScorecardCategory();
        category.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - category");

        ScorecardQuestion question = new ScorecardQuestion();
        question.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - question");

        ScorecardResponse response1 = new ScorecardResponse();
        response1.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - response 1");
        response1.setDefaultResponse(new Boolean(true));

        ScorecardResponse response2 = new ScorecardResponse();
        response2.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - response 2");

        module.setScorecardFolders(new ScorecardFolder[]
        {
            scorecardFolder
        });
        scorecardFolder.setScorecards(new Scorecard[]
        {
            scorecard
        });
        scorecard.setScorecardCategories(new ScorecardCategory[]
        {
            category
        });
        category.setScorecardQuestions(new ScorecardQuestion[]
        {
            question
        });
        question.setScorecardResponses(new ScorecardResponse[]
        {
                response1, response2
        });

        //Put only scorecards in the scope()
        ScorecardFolderScope scorecardScope = ScorecardFixture.createScope();
        WbsModuleScope moduleScope =  new WbsModuleScope();          
        moduleScope.setScorecardFolders(scorecardScope);

        module = (WbsModule) save(module, moduleScope).getRpmObject();

        return module;
    }

}
