/*
 * Created on Jan 09, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases;

import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.BaseFieldDescr;
import com.ibm.rpm.security.scope.SecurityGroupScope;

/**
 * @author Karim & Marc-Andr
 */
public class TestResourceSecurityFlagsLocal extends BaseResourceSecurityFlagsTestLocal
{

    public void testAllSecurityGroupFlags() throws Exception
    {
        //        System.out.println("TEST - Starting testAllSecurityGroupFlags");
        boolean flagValue = true;
        SecurityGroupScope scope = new SecurityGroupScope();
        SecurityGroup parentSecurityGroup = loadSecurityGroupByName(
            "DEFAULT_SECURITY_ROLE", scope);
        prepareSecurityGroup(parentSecurityGroup, flagValue);
        // reload all the security group and verify the flag values
        String name = getChildName(parentSecurityGroup.getName());
        SecurityGroup securityGroup = loadSecurityGroupByName(name, scope);
        verifyGroupFlags(securityGroup, flagValue);
        // TODO repeat the exercise with flagValue = true
    }

    /**
     * Tests the database's security flags for singularity. Loads all the flag
     * and sets them to false. Then set one flags to true and check if the
     * others have been affected.
     * 
     * @author Marc-Andr
     */
    public void testAllSecurityGroupFlagsSingularity() throws Exception
    {
        //        System.out.println("TEST - Starting testAllSecurityGroupFlagsSingularity");
        SecurityGroupScope scope = new SecurityGroupScope();
        SecurityGroup parentSecurityGroup = loadSecurityGroupByName(
            "DEFAULT_SECURITY_ROLE", scope);
        prepareSecurityGroup(parentSecurityGroup, false);
        verifyGroupFlagsSingularity(parentSecurityGroup);
    }

    protected void verifyGroupFlagsSingularity(SecurityGroup parentSecurityGroup)
            throws Exception
    {
        String name = getChildName(parentSecurityGroup.getName());
        SecurityGroup securityGroup = loadSecurityGroupByName(name,
            new SecurityGroupScope());
        assertNotNull(securityGroup);
        //        System.out.println("TEST - Verifying "+name);
        Map mapMethods = SecurityUtil.getFieldDescrsMap(securityGroup
            .getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                .getValue();
            setGroupFlag(securityGroup, desc, true);
            saveWithoutDelete(securityGroup, null);
            securityGroup = loadSecurityGroupByName(name,
                new SecurityGroupScope());
            verifyModifiedGroupFlags(securityGroup, desc, false);
            setGroupFlags(securityGroup, false);
            saveWithoutDelete(securityGroup, null);
        }
    }

    protected void verifyModifiedGroupFlags(SecurityGroup securityGroup,
            BaseFieldDescr skipFlag, boolean flagValue) throws Exception
    {
        Map mapMethods = SecurityUtil.getFieldDescrsMap(securityGroup
            .getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                .getValue();
            if (desc == skipFlag)
                verifyGroupFlag(securityGroup, desc, !flagValue);
            else
                verifyGroupFlag(securityGroup, desc, flagValue);
        }
    }

}