/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.framework;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.UnitTestSessionUtility;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.security.testcases.BaseRoleSecurityFlagsTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.InternalSecuredArrayResult;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.impl.ContainerSecurityController;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.role.RoleBasedSecurityControllerImpl;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractRequest;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleAssignment;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.FieldDescr;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Marc-Andr Lamothe
 * 
 */
public class BaseSecurityFrameworkTestLocal extends
        BaseRoleSecurityFlagsTestLocal
{

    protected static final Map SECURITYROLECATEGORIES = new HashMap();

    static
    {
        SECURITYROLECATEGORIES.put(GenericProject.class, "SCPROJECTGROUP");
        SECURITYROLECATEGORIES.put(Deliverable.class, "SCDELIVERABLEGROUP");
        SECURITYROLECATEGORIES.put(Task.class, "SCTASKGROUP");
        SECURITYROLECATEGORIES.put(ChangeRequest.class, "SCCHANGEREQUESTGROUP");
        SECURITYROLECATEGORIES.put(Issue.class, "SCISSUEGROUP");
        SECURITYROLECATEGORIES.put(Risk.class, "SCRISKGROUP");
        SECURITYROLECATEGORIES.put(ServiceRequest.class, "SCSERVICEREQUESTGROUP");
        SECURITYROLECATEGORIES.put(Requirement.class, "SCREQUIREMENTGROUP");
        SECURITYROLECATEGORIES.put(Defect.class, "SCDEFECTGROUP");
        SECURITYROLECATEGORIES.put(Action.class, "SCACTIONITEMGROUP");
        SECURITYROLECATEGORIES.put(GenericDocument.class, "SCDOCUMENTGROUP");
        SECURITYROLECATEGORIES.put(NoteMinutes.class, "SCNOTEMINUTESGROUP");
        SECURITYROLECATEGORIES.put(SummaryTask.class, "SCDELIVERABLEGROUP");
        SECURITYROLECATEGORIES.put(WorkProduct.class, "SCDELIVERABLEGROUP");
        SECURITYROLECATEGORIES.put(Milestone.class, "SCTASKGROUP");
        SECURITYROLECATEGORIES.put(DocumentFolder.class, "SCFOLDERGROUP");
//        SECURITYROLECATEGORIES.put(Pool.class, "SCPOOLGROUP");
        SECURITYROLECATEGORIES.put(ScopeFolder.class, "SCFOLDERGROUP");
        

    }

    protected Class getClass(String SecurityRoleCategory)
    {
        Iterator it = SECURITYROLECATEGORIES.entrySet().iterator();
        while (it.hasNext())
        {
            Entry entry = (Entry) it.next();
            if (SecurityRoleCategory.equals((String)entry.getValue()))
                return (Class)entry.getKey();
        }
        return null;
    }
    
    protected String getSecurityRoleCategory(Class objectClass)
    { 
        String category = null;
        do
        {
            category = (String) SECURITYROLECATEGORIES.get(objectClass);
            objectClass = objectClass.getSuperclass();
        }
        while (objectClass != null && category == null);
        return category;
    }
    
    /**
     * Create a new resource assignment for the container.
     * 
     * @param rpmObject
     * @param securityRole
     * @return RPMObject
     */
    protected InternalSecuredArrayResult buildSecurityDescriptor(
            Class rpmObjClass, SecurityRole securityRoleToAssociate,
            boolean compare) throws Exception
    {
        /* Prepares a message context */
        MessageContext msg = new MessageContext(ServerFactory.getInstance());
        msg.setSession(UnitTestSessionUtility.INSTANCE, sessionid, 
            "buildSecurityDescriptor");
        LoadParams params = new LoadParams();
        msg.setLoadParams(params);
//        Map securityRolesByTypeMap = new HashMap();
//        params.setSecurityRolesByType(securityRolesByTypeMap);
//        Map securityRolesMap = new HashMap();
//        securityRolesByTypeMap.put(rpmObjClass, securityRolesMap);
      
        
        RPMObject rpmObj = (RPMObject) rpmObjClass.newInstance();
        rpmObj.setID(rpmObjClass.getName());
//        Set roleSet = new HashSet();
//        roleSet.add(securityRoleToAssociate);
//        securityRolesMap.put(rpmObj.getID(), roleSet);
        msg.addObject(rpmObj);
        /* Builds the descriptor by converting to InternalSecuredArrayResult */
        InternalSecuredArrayResult internalSecuredArrayResult = msg
            .toInternalSecuredArrayResult();
        /* Compares the result with himself and another result */
        if (compare)
        {
            compareSecurityDescriptors(internalSecuredArrayResult,
                internalSecuredArrayResult, true);
            InternalSecuredArrayResult internalSecuredArrayResult2 = buildSecurityDescriptor(
                rpmObjClass, securityRoleToAssociate, false);
            compareSecurityDescriptors(internalSecuredArrayResult,
                internalSecuredArrayResult2, true);
        }
        return internalSecuredArrayResult;
    }

    /**
     * Compares two security descriptors, test fails if not equals.
     * 
     * @param internalSecuredArrayResult_1
     * @param internalSecuredArrayResult_2
     * @param exceptedResult
     */
    protected void compareSecurityDescriptors(
            InternalSecuredArrayResult internalSecuredArrayResult_1,
            InternalSecuredArrayResult internalSecuredArrayResult_2,
            boolean exceptedResult)
    {
        Assert.assertNotNull(internalSecuredArrayResult_1);
        Assert.assertNotNull(internalSecuredArrayResult_2);
        Map map1 = internalSecuredArrayResult_1
            .getContainerSecurityDescriptorsMap();
        Map map2 = internalSecuredArrayResult_2
            .getContainerSecurityDescriptorsMap();
        Assert.assertEquals(exceptedResult, new HashSet(map1.values())
            .equals(new HashSet(map2.values())));
    }

    /**
     * Loads the first security role category from the database and returns
     * the first security role it contains.
     * 
     * @param securityRoleCategory
     * @return SecurityRole
     */
    protected SecurityRole loadSecurityRoleByCategory(String securityRoleCategory)
            throws Exception
    {
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory[id='"
                + securityRoleCategory + "']", scope);
        if ((rpmObjects != null) && (rpmObjects.length > 0))
            return ((SecurityRoleCategory) rpmObjects[0]).getSecurityRoles()[0];
        return null;
    }

    /**
     * Loads a container with its security descriptor from the database.
     * 
     * @param rpmObjectClass
     * @param ID
     * @return InternalSecuredArrayResult
     */
    protected Map loadContainerWithSecurity(
            Class rpmObjectClass, String ID) throws Exception
    {
        if (GenericProject.class.isAssignableFrom(rpmObjectClass))
        {
            WorkElementScope workElementScope = new WorkElementScope();
            workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            workElementScope.setDocumentFolder(new DocumentScope());
            return loadInternalSecuredArrayResultFromXpath("/Proposal[id='"
                    + ID + "']", workElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Deliverable.class.isAssignableFrom(rpmObjectClass))
        {
            WorkElementScope workElementScope = new WorkElementScope();
            workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            workElementScope.setDocumentFolder(new DocumentScope());
            return loadInternalSecuredArrayResultFromXpath("/Deliverable[id='"
                    + ID + "']", workElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Task.class.isAssignableFrom(rpmObjectClass))
        {
            WorkElementScope workElementScope = new WorkElementScope();
            workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            workElementScope.setDocumentFolder(new DocumentScope());
            return loadInternalSecuredArrayResultFromXpath("/Task[id='" + ID
                    + "']", workElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (ChangeRequest.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath(
                "/ChangeRequest[id='" + ID + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Issue.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath("/Issue[id='" + ID
                    + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Risk.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath("/Risk[id='" + ID
                    + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (ServiceRequest.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath(
                "/ServiceRequest[id='" + ID + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Requirement.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath("/Requirement[id='"
                    + ID + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Defect.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath("/Defect[id='" + ID
                    + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        else if (Action.class.isAssignableFrom(rpmObjectClass))
        {
            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            return loadInternalSecuredArrayResultFromXpath("/Action[id='" + ID
                    + "']", scopeElementScope).getContainerSecurityDescriptorsMap();
        }
        return null;
    }
    
    /**
     * Creates a new instance of the container in the database.
     * 
     * @param rpmObjectClass
     * @param scope
     * @return RPMObject
     */
    protected RPMObject prepareContainer(RPMObject rpmObject, RPMObject parent,
            RPMObjectScope scope, Resource resource, SecurityRole securityRole)
            throws Exception
    {
        RPMObject object = null;
        /* Creates the container */

        if (rpmObject != null)
        {
            // WORKELEMENT SECTION
            if (WorkElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                object = processWorkElement((WorkElement)rpmObject, (WorkElement)parent, (WorkElementScope)scope);
            }
            // SCOPE ELEMENT SECTION
            else if (ScopeElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                assertTrue(parent instanceof ScopeElement);
                object = processScopeElements((ScopeElement)rpmObject, (ScopeElement)parent, scope);
            }

            // FINANCIAL SECTION
            else if (AbstractFinancial.class.isAssignableFrom(rpmObject
                .getClass()))
            {
                object = processFinancials((AbstractFinancial)rpmObject, scope);
            }
 
            // DOCUMENT SECTION
            else if (GenericDocument.class.isAssignableFrom(rpmObject
                .getClass()))
            {
                object = processDocuments((GenericDocument)rpmObject, parent, scope);
            }

            // SCOPE SECTION
            /* Add security role assignment to the scope */
            if (WorkElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                ((WorkElementScope) scope)
                    .setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            }
            else if (ScopeElement.class.isAssignableFrom(rpmObject.getClass()) && ((ScopeElementScope) scope).getResourceTaskAssignments() == null)
            {
                ((ScopeElementScope) scope)
                    .setResourceTaskAssignments(new ResourceTaskAssignmentScope());
            }
            else if (DocumentElement.class.isAssignableFrom(rpmObject
                .getClass()))
            {
                ((DocumentScope) scope).setResourceAssignments(true);
            }

            /* Saves the container and the assigned security role */
            if (object != null)
            {
                object = save(object, scope).getRpmObject();
                Assert.assertNotNull(object);
                prepareSecurityRoleAssignment(object, resource, securityRole);
                object = loadFromID(object, scope);
            }
        }
        return object;
    }

    protected RPMObject processDocuments(DocumentElement baseDocumentElement,
            RPMObject parent, RPMObjectScope scope) throws Exception
    {
        RPMObject object = null;
        object = DocumentElementFixture.createFixture(parent,
            baseDocumentElement, 1, (DocumentScope) scope, this);

        return object;
    }

    private RPMObject processFinancials(AbstractFinancial baseAbstractFinancial,
            RPMObjectScope scope) throws Exception
    {
        RPMObject object = null;
        if (baseAbstractFinancial instanceof WbsFinancial)
        {
            object = FinancialFixture.createWbsFixture((FinancialScope) scope,
                this);

        }
        else if (baseAbstractFinancial instanceof WorksheetFinancial)
        {
            object = FinancialFixture.createWorksheetFixture(
                (FinancialScope) scope, this);   
        }
        
        if (object.getParent() != null)
        {
            if (object.getParent() instanceof WorkElement)
            {
                WorkElement we = (WorkElement) object.getParent();
                if (we.getResourceTaskAssignments() == null)
                {
                    we.setResourceTaskAssignments(new ResourceTaskAssignment[]
                    {
                        (ResourceTaskAssignment) getResourceTaskAssignment(we)
                    });
                }
            }
            else
            {
                fail("processFinancials : object.getParent "+object.getParent().getClass()+" not supported yet. Please implement it.");
            }
        }

        return object;
    }

    protected RPMObject processScopeElements(ScopeElement baseScopeElement,
            ScopeElement parent, RPMObjectScope scope)
            throws Exception
    {
        RPMObject object = null;
        if (AggregateScope.class.isAssignableFrom(baseScopeElement.getClass()))
        {
            if (AbstractAggregateScope.class.isAssignableFrom(baseScopeElement
                .getClass()))
            {
                if (AbstractScope.class.isAssignableFrom(baseScopeElement.getClass()))
                {
                    if (AbstractRequest.class.isAssignableFrom(baseScopeElement
                        .getClass()))
                    {
                        if (baseScopeElement instanceof ChangeRequest)
                        {
                            object = ScopeElementFixture
                                .createChangeRequestFixture(
                                    (ScopeElementScope) scope, 1,
                                    parent, this);

                        }
                        else if (baseScopeElement instanceof Defect)
                        {
                            object = ScopeElementFixture.createDefectFixture(
                                (ScopeElementScope) scope, 1,
                                parent, this);
                        }
                        else if (baseScopeElement instanceof ServiceRequest)
                        {
                            object = ScopeElementFixture
                                .createServiceRequestFixture(
                                    (ScopeElementScope) scope, 1,
                                    parent, this);
                        }
                    }
                    else if (Requirement.class.isAssignableFrom(baseScopeElement
                        .getClass()))
                    {
                        if (baseScopeElement instanceof ReqProRequirement)
                        {
                            // todo:verified
                            object = ScopeElementFixture
                                .createScopeElementFixture(
                                    (ScopeElementScope) scope, 0,
                                    new ReqProRequirement(),
                                    parent, this);

                        }
                        else if (baseScopeElement instanceof Requirement)
                        {
                            object = ScopeElementFixture
                                .createRequirementFixture(
                                    (ScopeElementScope) scope, 1,
                                    parent, this);
                        }
                    }
                    else if (baseScopeElement instanceof Action)
                    {
                        object = ScopeElementFixture.createActionFixture(
                            (ScopeElementScope) scope, 1,
                            parent, this);

                    }
                    else if (baseScopeElement instanceof Issue)
                    {
                        object = ScopeElementFixture.createIssueFixture(
                            (ScopeElementScope) scope, 1,
                            parent, this);
                    }
                    else if (baseScopeElement instanceof Risk)
                    {
                        object = ScopeElementFixture.createRiskFixture(
                            (ScopeElementScope) scope, 1,
                            parent, this);
                    }
                }
                else if (baseScopeElement instanceof ScopeDeliverable)
                {
                    object = ScopeElementFixture.createScopeDeliverableFixture(
                        (ScopeElementScope) scope, 0, parent,
                        this);

                }
                else if (baseScopeElement instanceof ScopeSummaryTask)
                {
                    object = ScopeElementFixture.createScopeSummaryTaskFixture(
                        (ScopeElementScope) scope, 0, parent,
                        this);
                }
                else if (baseScopeElement instanceof ScopeWorkProduct)
                {
                    // todo: verified
                    object = ScopeElementFixture.createScopeElementFixture(
                        (ScopeElementScope) scope, 0, new ScopeWorkProduct(),
                        parent, this);
                }
            }
            else if (ScopeFolder.class.isAssignableFrom(baseScopeElement.getClass()))
            {
                if (baseScopeElement instanceof ReqProIntegrationFolder)
                {
                    // ------todo: verified
                    object = ScopeElementFixture.createScopeElementFixture(
                        (ScopeElementScope) scope, 0,
                        new ReqProIntegrationFolder(), parent,
                        this);

                }
                else if (baseScopeElement instanceof ReqProScopeFolder)
                {
                    // ------todo: verified
                    object = ScopeElementFixture.createScopeElementFixture(
                        (ScopeElementScope) scope, 0, new ReqProScopeFolder(),
                        parent, this);
                }
                else if(baseScopeElement instanceof ScopeFolder)
                {
                    object = ScopeElementFixture.createFolderFixture(
                        (ScopeElementScope) scope, 0, parent, this);
                }

            }
        }
        else if (AbstractScopeTask.class.isAssignableFrom(baseScopeElement.getClass()))
        {
            if (baseScopeElement instanceof ScopeMilestone)
            {
                object = ScopeElementFixture.createScopeMilestoneFixture(
                    (ScopeElementScope) scope, 0, parent, this);

            }
            else if (baseScopeElement instanceof ScopeTask)
            {
                object = ScopeElementFixture.createScopeTaskFixture(
                    (ScopeElementScope) scope, 0, parent, this);
            }
        }
        return object;
    }

    protected RPMObject processWorkElement(WorkElement baseWorkElement, WorkElement parent,
            WorkElementScope scope) throws Exception
    {
    	if (parent == null)
        {
            if (baseWorkElement instanceof GenericProject)
            {
                GenericProject we = (GenericProject) baseWorkElement;
                // set the Portfolio attribute and no parent
                we.setPortfolio(PortfolioFixture.loadOrCreateRootPortfolioForSecurityTester(this));
            }
        }
    	return WorkElementFixture.createFixture(parent, baseWorkElement, scope, this);
    }

    
    /**
     * Create a new security role of the appropriate type for the class.
     * 
     * @param rpmObjectClass
     * @param flagValues
     * @return SecurityRole
     */
    protected SecurityRole prepareSecurityRole(Class rpmObjectClass,
            boolean flagValues) throws Exception
    {
        SecurityRole securityRole = null;
        String securityRoleName = getSecurityRoleCategory(rpmObjectClass);
        securityRole = loadSecurityRoleByCategory(securityRoleName);
        
        return prepareSecurityRole(securityRole, flagValues);
    }
    
    protected SecurityRole updateSecurityRole(SecurityRole securityRole)
            throws Exception
    {
        return (SecurityRole) saveWithoutDelete(securityRole,
            createSecurityRoleScope()).getRpmObject();
    }
    
    public SecurityRole getSecurityRole(RPMObject rpmObject) throws Exception
    {
        if (rpmObject != null && rpmObject.getID() != null)
        {
            if (WorkElement.class.isAssignableFrom(rpmObject.getClass())
                    || ScopeElement.class
                        .isAssignableFrom(rpmObject.getClass()))

            {
                ResourceTaskAssignment assignment = (ResourceTaskAssignment) getResourceTaskAssignment(rpmObject);
                return (assignment != null) ? assignment.getSecurityRole()
                        : null;
            }
            else if (DocumentElement.class.isAssignableFrom(rpmObject
                .getClass()))
            {
                SecurityRoleAssignment assignment = getResourceTaskAssignment(rpmObject);
                return (assignment != null) ? assignment.getSecurityRole()
                        : null;
            }
            else
            {
                fail("getSecurityRole : rpmObject of type "
                        + rpmObject.getClass()
                        + " not supported yet. Please implement it.");
            }
        }
        return null;
    }
    
    private SecurityRoleAssignment getResourceTaskAssignment(RPMObject rpmObject)
            throws Exception
    {
        ResourceRoleAssignment[] assignments = null;

        if (rpmObject != null)
        {
            if (WorkElement.class.isAssignableFrom(rpmObject.getClass()) || 
                    ScopeElement.class.isAssignableFrom(rpmObject.getClass()) || 
                    DocumentElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                assignments = getResourceRoleAssignments(rpmObject);
                if (assignments == null)
                {
                    rpmObject = reloadObjectWithResourceTaskAssgiments(rpmObject);
                    assignments = getResourceRoleAssignments(rpmObject);
                }
            }
            
            if (assignments != null && assignments.length > 0)
            {
                for (int i = 0; i < assignments.length; ++i)
                {
                    if (assignments[i].getResource().getID().equals(
                        getCurrentUser().getID()))
                    {
                        return assignments[i];
                    }
                }
            }
        }
        return null;
    }
    
    private ResourceRoleAssignment[] getResourceRoleAssignments(
            RPMObject rpmObject)
    {
        if (rpmObject != null)
        {
            if (WorkElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                WorkElement workElement = (WorkElement) rpmObject;
                return workElement.getResourceTaskAssignments();
            }
            else if (ScopeElement.class.isAssignableFrom(rpmObject.getClass()))
            {
                ScopeElement scopeElement = (ScopeElement) rpmObject;
                return scopeElement.getResourceTaskAssignments();
            }
            else if (DocumentElement.class.isAssignableFrom(rpmObject
                .getClass()))
            {
                DocumentElement documentElement = (DocumentElement) rpmObject;
                return documentElement.getResourceAssignments();
            }
            else
            {
                fail("getResourceRoleAssignments : rpmObject of type "
                        + rpmObject.getClass()
                        + " not supported yet. Please implement it.");
            }
        }
        return null;
    }

    private RPMObject reloadObjectWithResourceTaskAssgiments(RPMObject rpmObject) throws Exception
    {
        if(rpmObject !=null)
        {
            if(rpmObject instanceof WorkElement)
            {
                WorkElementScope scope = new WorkElementScope();
                scope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
                rpmObject = loadFromID(rpmObject, scope);
            }
            else if(rpmObject instanceof ScopeElement)
            {
                ScopeElementScope scope = new ScopeElementScope();
                scope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
                rpmObject = loadFromID(rpmObject, scope);
            }
            else if(rpmObject instanceof DocumentElement)
            {
                DocumentScope scope = new DocumentScope();
                scope.setResourceAssignments(true);
                rpmObject = loadFromID(rpmObject, scope);
            }
            else
            {
                fail("getSecurityRole : rpmObject of type "+rpmObject.getClass()+" not supported yet. Please implement it.");
            }
        }
        return rpmObject;
    }

    /**
     * Create a new instance of the appropriate resource assignment for the container.
     * 
     * @param rpmObject
     * @param resource
     * @param securityRole
     * @return RPMObject
     */
    protected RPMObject prepareSecurityRoleAssignment(RPMObject rpmObject,
            Resource resource, SecurityRole securityRole) throws Exception
    {
        if (WorkElement.class.isAssignableFrom(rpmObject.getClass()))
        {
            WorkElement workElement = (WorkElement)rpmObject;
            ResourceTaskAssignment assignment;
            if (workElement.getResourceTaskAssignments() != null)
            {
                assignment = workElement.getResourceTaskAssignments()[0];
            }
            else
                assignment = new ResourceTaskAssignment();
            assignment.setParent(rpmObject);
            assignment.setResource(resource);
            assignment.setSecurityRole(securityRole);
            return save(assignment, null).getRpmObject();
        }
        else if (ScopeElement.class.isAssignableFrom(rpmObject.getClass()))
        {
            ScopeElement scopeElement = (ScopeElement)rpmObject;
            ResourceTaskAssignment assignment;
            if (scopeElement.getResourceTaskAssignments() != null)
            {
                assignment = scopeElement.getResourceTaskAssignments()[0];
            }
            else
                assignment = new ResourceTaskAssignment();
            assignment.setParent(rpmObject);
            assignment.setResource(resource);
            assignment.setSecurityRole(securityRole);
            return save(assignment, null).getRpmObject();
        }
        else if (DocumentElement.class.isAssignableFrom(rpmObject.getClass()))
        {
            DocumentElement documentElement = (DocumentElement)rpmObject;
            ResourceRoleAssignment assignment;
            if (documentElement.getResourceAssignments() != null)
            {
                assignment = documentElement.getResourceAssignments()[0];
            }
            else
                assignment = new ResourceRoleAssignment();
            assignment.setParent(rpmObject);
            assignment.setResource(resource);
            assignment.setSecurityRole(securityRole);
            return save(assignment, null).getRpmObject();
        }
        return null;
    }

    /**
     * Returns true if at least one of the flags of securityRole
     * is associated to the edition of a field inside rpmObjectClass.
     * 
     * @param securityRole
     * @param fieldDesc
     * @param rpmObjectClass
     * @return boolean
     */
    protected boolean isRoleAssociatedToEditField(SecurityRole securityRole,
            FieldSecurityDescriptor fieldDesc, Class rpmObjectClass)
            throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(fieldDesc);
        /* Get this container's security controller */
        ContainerSecurityController controller = RoleBasedSecurityControllerImpl
            .getRoleBasedContainerSecurityController(rpmObjectClass);
        assertNotNull(controller);
        /* Get this field's mapping entry */
        FieldMappingEntry fieldEntry = (FieldMappingEntry) controller
            .getFieldMappingEntries().get(fieldDesc.getName());
        /* Get this security role's security flags map */
        Map map = SecurityUtil.getFieldDescrsMap(securityRole.getClass());
        /* Find a flag that is both in the field and in the role */
        if (fieldEntry != null && fieldEntry.canWriteSecurityFlags != null)
            for (int i = 0; i < fieldEntry.canWriteSecurityFlags.length; i++)
                if (map.containsKey(fieldEntry.canWriteSecurityFlags[i]))
                    return true;
        return false;
    }

    /**
     * Returns true if at least one of the flags of securityRole
     * is associated to the viewing of a field inside rpmObjectClass.
     * 
     * @param securityRole
     * @param fieldDesc
     * @param rpmObjectClass
     * @return boolean
     */
    protected boolean isRoleAssociatedToViewField(SecurityRole securityRole,
            FieldSecurityDescriptor fieldDesc, Class rpmObjectClass)
            throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(fieldDesc);
        /* Get this container's security controller */
        ContainerSecurityController controller = RoleBasedSecurityControllerImpl
            .getRoleBasedContainerSecurityController(rpmObjectClass);
        assertNotNull(controller);
        /* Get this field's mapping entry */
        FieldMappingEntry fieldEntry = (FieldMappingEntry) controller
            .getFieldMappingEntries().get(fieldDesc.getName());
        /* Get this security role's security flags map */
        Map map = SecurityUtil.getFieldDescrsMap(securityRole.getClass());
        /* Find a flag that is both in the field and in the role */
        if (fieldEntry != null && fieldEntry.canReadSecurityFlags != null)
            for (int i = 0; i < fieldEntry.canReadSecurityFlags.length; i++)
                if (map.containsKey(fieldEntry.canReadSecurityFlags[i]))
                    return true;
        return false;
    }

    /**
     * Returns true if the flag is included inside the internalSecuredArrayResult.
     * 
     * @param securityRole
     * @param internalSecuredArrayResult
     * @param flag
     * @return boolean
     */
    protected boolean isFlagAssociatedToDescriptor(SecurityRole securityRole,
            InternalSecuredArrayResult internalSecuredArrayResult,
            FieldDescr flag) throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(internalSecuredArrayResult);
        Map.Entry entry = (Map.Entry) internalSecuredArrayResult
            .getContainerSecurityDescriptorsMap().entrySet().iterator().next();
        ContainerSecurityDescriptor container = (ContainerSecurityDescriptor) entry
            .getValue();
        FieldSecurityDescriptor[] securityDesc = (FieldSecurityDescriptor[]) container
            .getFieldSecurityDescriptors().values().toArray(
                new FieldSecurityDescriptor[0]);
        for (int i = 0; i < securityDesc.length; i++)
        {
            FieldSecurityDescriptor fieldDesc = securityDesc[i];
            if (isFlagAssociatedToEditField(securityRole.getClass(),
                flag.clearanceLevelId * 32 + flag.mask, fieldDesc, entry
                    .getKey().getClass())
                    || isFlagAssociatedToViewField(securityRole.getClass(),
                        flag.clearanceLevelId * 32 + flag.mask, fieldDesc,
                        entry.getKey().getClass()))
                return true;
        }
        return false;
    }

    /**
     * Returns true if the flag is associated to the edition of the field.
     * 
     * @param securityRole
     * @param flag
     * @param fieldDesc
     * @param rpmObjectClass
     * @return boolean
     */
    protected boolean isFlagAssociatedToEditField(Class securityRole,
            int flag, FieldSecurityDescriptor fieldDesc,
            Class rpmObjectClass)
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(fieldDesc);
        /* Get this container's security controller */
        ContainerSecurityController controller = RoleBasedSecurityControllerImpl
            .getRoleBasedContainerSecurityController(rpmObjectClass);
        assertNotNull(controller);
        /* Get this field's mapping entry */
        FieldMappingEntry fieldEntry = (FieldMappingEntry) controller
            .getFieldMappingEntries().get(fieldDesc.getName());
        /* Search for the flag inside the field entry */
        if (fieldEntry != null && fieldEntry.canWriteSecurityFlags != null)
            for (int i = 0; i < fieldEntry.canWriteSecurityFlags.length; i++)
                if (fieldEntry.canWriteSecurityFlags[i].intValue() == flag)
                    return true;
        return false;
    }

    /**
     * Returns true if the flag is associated to the viewing of the field.
     * 
     * @param securityRole
     * @param flag
     * @param fieldDesc
     * @param rpmObjectClass
     * @return boolean
     */
    protected boolean isFlagAssociatedToViewField(Class securityRole,
            int flag, FieldSecurityDescriptor fieldDesc,
            Class rpmObjectClass)
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(fieldDesc);
        /* Get this container's security controller */
        ContainerSecurityController controller = RoleBasedSecurityControllerImpl
            .getRoleBasedContainerSecurityController(rpmObjectClass);
        assertNotNull(controller);
        /* Get this field's mapping entry */
        FieldMappingEntry fieldEntry = (FieldMappingEntry) controller
            .getFieldMappingEntries().get(fieldDesc.getName());
        /* Search for the flag inside the field entry */
        if (fieldEntry != null && fieldEntry.canReadSecurityFlags != null)
            for (int i = 0; i < fieldEntry.canReadSecurityFlags.length; i++)
                if (fieldEntry.canReadSecurityFlags[i].intValue() == flag)
                    return true;
        return false;
    }

}
