/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.framework;

import java.lang.reflect.Modifier;
import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.framework.InternalSecuredArrayResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.BaseFieldDescr;
import com.ibm.rpm.security.managers.SecurityUtil.FieldDescr;
import com.ibm.rpm.security.scope.SecurityRoleScope;

/**
 * @author Marc-Andr Lamothe
 * 
 */
public class TestSecurityDescriptorFlagsLocal extends
        BaseSecurityFrameworkTestLocal
{
    public void setUp() throws Exception
    {
        // switch to LDAP mode
        SpecificTestController.overrideOption("webServicesSecurityEnabled",
            Boolean.TRUE);
        SpecificTestController.overrideOption(
            RpmOptions.WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME, ROLE_SEC);
        super.setUp();
    }

    //    public void testDescriptorFlags() throws Exception
    //    {
    //        System.out.println("TEST - Starting testDescriptorFlags");
    //        boolean flagValue = true;
    //        SecurityRoleScope scope = new SecurityRoleScope();
    //        scope.setSecurityRoles(scope);
    //        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
    //        assertNotNull(rpmObjects);
    //        for (int i = 0; i < rpmObjects.length; i++)
    //        {
    //            SecurityRoleCategory securityRoleCategory = (SecurityRoleCategory) rpmObjects[i];
    //            SecurityRole securityRole = securityRoleCategory.getSecurityRoles()[0];
    //            Assert.assertNotNull(securityRole);
    //            String ID = prepareTestContainer(securityRoleCategory.getID(), securityRole);
    //            Assert.assertNotNull(ID);
    //            InternalSecuredArrayResult internalSecuredArrayResult = loadSecurityDescriptor(
    //                securityRoleCategory.getID(), ID);
    //            Assert.assertNotNull(internalSecuredArrayResult);
    //            verifyDescriptorFlags(securityRole, internalSecuredArrayResult,
    //                flagValue);
    //        }
    //    }

    public void testDescriptorFlagsInMemory() throws Exception
    {
        System.out.println("TEST - Starting testDescriptorFlagsInMemory");
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
        assertNotNull(rpmObjects);

        SecurityRoleCategory securityRoleCategory = null;
        for (int i = 0; i < rpmObjects.length; i++)
        {
            // prepare child security role
            securityRoleCategory = (SecurityRoleCategory) rpmObjects[i];
            SecurityRole securityRole = prepareInMemorySecurityRole(
                securityRoleCategory.getSecurityRoles()[0], true);
            Assert.assertNotNull(securityRole);
            System.out.println("INFO - Verifying " + securityRole.getName());

            // tests security descriptor
            Class objectClass = getClass(securityRoleCategory.getID());
            if (objectClass==null || Modifier.isAbstract(objectClass.getModifiers()))
            {
                continue;
            }
            InternalSecuredArrayResult internalSecuredArrayResult = buildSecurityDescriptor(
                objectClass, securityRole, false);
            if (internalSecuredArrayResult != null)
            {
                Assert.assertTrue(internalSecuredArrayResult
                    .getContainerSecurityDescriptorsMap().size() == 1);
                verifyDescriptorFlags(securityRole, internalSecuredArrayResult,
                    true);
            }

            // repeat with false
            securityRole = prepareInMemorySecurityRole(securityRoleCategory
                .getSecurityRoles()[0], false);
            Assert.assertNotNull(securityRole);
            System.out.println("INFO - Verifying " + securityRole.getName());

            // tests security descriptor
            objectClass = getClass(securityRoleCategory.getID());
            internalSecuredArrayResult = buildSecurityDescriptor(objectClass,
                securityRole, false);
            if (internalSecuredArrayResult != null)
            {
                Assert.assertTrue(internalSecuredArrayResult
                    .getContainerSecurityDescriptorsMap().size() == 1);
                verifyDescriptorFlags(securityRole, internalSecuredArrayResult,
                    false);
            }
        }
    }

    public void testDescriptorFlagsSingularityInMemory() throws Exception
    {
        System.out
            .println("TEST - Starting testEmulatedInMemoryDescriptorFlagsSingularity");
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
        assertNotNull(rpmObjects);

        SecurityRoleCategory securityRoleCategory = null;
        for (int i = 0; i < rpmObjects.length; i++)
        {
            securityRoleCategory = (SecurityRoleCategory) rpmObjects[i];
            SecurityRole[] parentSecurityRoles = securityRoleCategory
                .getSecurityRoles();
            Assert.assertNotNull(parentSecurityRoles);

            // prepare child security role
            SecurityRole securityRole = prepareInMemorySecurityRole(
                parentSecurityRoles[0], false);
            Assert.assertNotNull(securityRole);            
            if (Modifier.isAbstract(securityRole.getClass().getModifiers()))
            {
                continue;
            }
            System.out.println("INFO - Verifying " + securityRole.getName());
            verifyDescriptorFlagsSingularityInMemory(securityRoleCategory,
                securityRole, true);

            //Don't repeat test with flag set to true because the testing logic
            //won't work since a given field can be associated with more than one flag
            //so, setting one of them to be false and expecting the container field
            //descr (i.e., FieldSecurityDescriptor) to be false as a result won't work 
            //because those other flags are still true and they get ORed when calcualting the
            //values of canEdit/canView of the FieldSecurityDescriptor
        }
    }

    private void verifyDescriptorFlags(SecurityRole securityRole,
            InternalSecuredArrayResult internalSecuredArrayResult,
            boolean flagValue) throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(internalSecuredArrayResult);
        Map.Entry entry = (Map.Entry) internalSecuredArrayResult
            .getContainerSecurityDescriptorsMap().entrySet().iterator().next();
        ContainerSecurityDescriptor container = (ContainerSecurityDescriptor) entry
            .getValue();
        FieldSecurityDescriptor[] securityDesc = (FieldSecurityDescriptor[]) container
            .getFieldSecurityDescriptors().values().toArray(
                new FieldSecurityDescriptor[0]);
        for (int i = 0; i < securityDesc.length; i++)
        {
            FieldSecurityDescriptor fieldDesc = securityDesc[i];
            if (isRoleAssociatedToEditField(securityRole, fieldDesc, entry
                .getKey().getClass()))
                Assert
                    .assertEquals(entry.getKey().getClass().getName() + ": "
                            + fieldDesc.getName(), flagValue, fieldDesc
                        .isCanEdit());
            else if (isRoleAssociatedToViewField(securityRole, fieldDesc, entry
                .getKey().getClass()))
                Assert
                    .assertEquals(entry.getKey().getClass().getName() + ": "
                            + fieldDesc.getName(), flagValue, fieldDesc
                        .isCanView());
        }
    }

    private void verifyDescriptorFlagsSingularityInMemory(
            SecurityRoleCategory securityRoleCategory,
            SecurityRole securityRole, boolean flagValue) throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(securityRoleCategory);

        Class objectClass = getClass(securityRoleCategory.getID());
        if(objectClass==null){
            return;
        }
        InternalSecuredArrayResult internalSecuredArrayResult = buildSecurityDescriptor(
            objectClass, securityRole, false);
        if (internalSecuredArrayResult != null)
        {
            Assert.assertTrue(internalSecuredArrayResult
                .getContainerSecurityDescriptorsMap().size() == 1);
            Assert.assertNotNull(internalSecuredArrayResult);
            Map mapMethods = SecurityUtil.getFieldDescrsMap(securityRole
                .getClass());
            Iterator it = mapMethods.entrySet().iterator();
            while (it.hasNext())
            {
                BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                    .getValue();
                if (desc instanceof FieldDescr)
                    if (isFlagAssociatedToDescriptor(securityRole,
                        internalSecuredArrayResult, (FieldDescr) desc))
                    {
                        setRoleFlag(securityRole, desc, !flagValue);
                        objectClass = getClass(securityRoleCategory.getID());
                        internalSecuredArrayResult = buildSecurityDescriptor(
                            objectClass, securityRole, false);
                        Assert.assertNotNull(internalSecuredArrayResult);
                        verifyModifiedDescriptorFlags(securityRole,
                            internalSecuredArrayResult, (FieldDescr) desc,
                            flagValue);
                        setRoleFlags(securityRole, flagValue);
                    }
            }
        }
    }

    private void verifyModifiedDescriptorFlags(SecurityRole securityRole,
            InternalSecuredArrayResult internalSecuredArrayResult,
            FieldDescr skipFlag, boolean flagValue) throws Exception
    {
        Assert.assertNotNull(securityRole);
        Assert.assertNotNull(internalSecuredArrayResult);
        Map.Entry entry = (Map.Entry) internalSecuredArrayResult
            .getContainerSecurityDescriptorsMap().entrySet().iterator().next();
        ContainerSecurityDescriptor container = (ContainerSecurityDescriptor) entry
            .getValue();
        FieldSecurityDescriptor[] securityDesc = (FieldSecurityDescriptor[]) container
            .getFieldSecurityDescriptors().values().toArray(
                new FieldSecurityDescriptor[0]);
        for (int i = 0; i < securityDesc.length; i++)
        {
            FieldSecurityDescriptor fieldDesc = securityDesc[i];
            if (isFlagAssociatedToEditField(securityRole.getClass(),
                skipFlag.clearanceLevelId * 32 + skipFlag.mask, fieldDesc,
                entry.getKey().getClass()))
                Assert.assertEquals(entry.getKey().getClass().getName() + ": "
                        + fieldDesc.getName(), !flagValue, fieldDesc
                    .isCanEdit());
            else if (isFlagAssociatedToViewField(securityRole.getClass(),
                skipFlag.clearanceLevelId * 32 + skipFlag.mask, fieldDesc,
                entry.getKey().getClass()))
                Assert.assertEquals(entry.getKey().getClass().getName() + ": "
                        + fieldDesc.getName(), !flagValue, fieldDesc
                    .isCanView());
            else if (isRoleAssociatedToEditField(securityRole, fieldDesc, entry
                .getKey().getClass()))
                Assert
                    .assertEquals(entry.getKey().getClass().getName() + ": "
                            + fieldDesc.getName(), flagValue, fieldDesc
                        .isCanEdit());
            else if (isRoleAssociatedToViewField(securityRole, fieldDesc, entry
                .getKey().getClass()))
                Assert
                    .assertEquals(entry.getKey().getClass().getName() + ": "
                            + fieldDesc.getName(), flagValue, fieldDesc
                        .isCanView());
        }
    }

}
