/*
 * Created on Jan 31, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;

/**
 * @author Marc-Andre Lamothe
 */
public class TestChargeCodeSecurityLocal extends BaseResourceSecurityTestLocal
{
    public void testCreateChargeCode() throws Exception
    {
        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Creates a charge code category, should fail
        ChargeCodeCategory chargeCode = (ChargeCodeCategory) ChargeCodeFixture
            .createGenericChargeCode(2, 1, null, false);
        saveAndValidate(chargeCode, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Creates a charge code category, should succeed
        chargeCode = (ChargeCodeCategory) ChargeCodeFixture
            .createGenericChargeCode(2, 1, null, false);
        saveAndValidate(chargeCode, null, true);

    }

    public void testDeleteChargeCode() throws Exception
    {
        // Creates a charge code category, should succeed
        ChargeCodeCategory chargeCode = (ChargeCodeCategory) ChargeCodeFixture
            .createGenericChargeCode(2, 1, null, false);
        chargeCode = (ChargeCodeCategory) saveAndValidate(chargeCode, null,
            true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Deletes the charge code, should fail
        deleteAndValidate(chargeCode, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Deletes the charge code, should succeed
        deleteAndValidate(chargeCode, true);
    }

    public void testUpdateChargeCode() throws Exception
    {
        // Creates a charge code category, should succeed
        ChargeCodeCategory chargeCode = (ChargeCodeCategory) ChargeCodeFixture
            .createGenericChargeCode(2, 1, null, false);
        chargeCode = (ChargeCodeCategory) saveAndValidate(chargeCode, null,
            true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Updates the charge code, should fail
        chargeCode.setDescription(chargeCode.getDescription() + "_MODIFIED");
        saveAndValidate(chargeCode, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditAccountCodes(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Updates the charge code, should succeed
        chargeCode.setDescription(chargeCode.getDescription() + "_MODIFIED");
        saveAndValidate(chargeCode, null, true);
    }

}