/*
 * Created on Feb 1, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 *
 */
public class TestDatafieldSecurityLocal extends BaseResourceSecurityTestLocal
{
    private CustomDatafieldCategory createParent() throws Exception
    {
        CustomDatafieldCategory parent = DatafieldFixture
            .createCustomDatafieldCategory(0);
        SingleResult singleResult = save(parent, null);
        return (CustomDatafieldCategory) singleResult.getRpmObject();
    }

    public void testCreateDatafield() throws Exception
    {
        // Create the parent
        CustomDatafieldCategory parent = createParent();

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create a data field, should fail
        CustomDatafield customDatafield = DatafieldFixture
            .createCustomDatafield(parent);
        customDatafield = (CustomDatafield) saveAndValidate(customDatafield,
            null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create a data field, should succeed
        customDatafield = DatafieldFixture.createCustomDatafield(parent);
        customDatafield = (CustomDatafield) saveAndValidate(customDatafield,
            null, true);
    }

    public void testDeleteDatafield() throws Exception
    {
        // Create the parent
        CustomDatafieldCategory parent = createParent();

        // Create a data field, should succeed
        CustomDatafield customDatafield = DatafieldFixture
            .createCustomDatafield(parent);
        customDatafield = (CustomDatafield) saveAndValidate(customDatafield,
            null, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the container, should fail
        deleteAndValidate(customDatafield, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the container, should succeed
        deleteAndValidate(customDatafield, true);
    }

    public void testUpdateDatafield() throws Exception
    {
        // Create the parent
        CustomDatafieldCategory parent = createParent();

        // Create a data field, should succeed
        CustomDatafield customDatafield = DatafieldFixture
            .createCustomDatafield(parent);
        customDatafield = (CustomDatafield) saveAndValidate(customDatafield,
            null, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the container, should fail
        customDatafield.setModified();
        saveAndValidate(customDatafield, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditDatafieldAdmin(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the container, should succeed
        customDatafield.setModified();
        saveAndValidate(customDatafield, null, true);
    }

}