/*
 * Created on Jan 31, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author Marc-Andre
 *
 */
public class TestProjectSecurityLocal extends BaseResourceSecurityTestLocal
{
    public void testCreateEmptyProject() throws Exception
    {
        // Remove the right to create
        getCurrentUser().getResourceSecurityRights().setCanCreateEmptyProjects(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create a project, should fail
        Project project = (Project) WorkElementFixture.createFixture(
            new Project(), null, this);
        project = (Project) saveAndValidate(project, null, false);

        // Give the right to create
        getCurrentUser().getResourceSecurityRights().setCanCreateEmptyProjects(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create a project, should succeed
        project = (Project) WorkElementFixture.createFixture(new Project(),
            null, this);
        project = (Project) saveAndValidate(project, null, true);
    }

    public void testCreateProjectFromTemplate() throws Exception
    {
        // Create a template
        Project template = loadOrCreateProjectTemplate(getCurrentUser());

        // Remove the right to create
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateProjectsFromWBSTemplate(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Creates new project from template, should fail
        Project newProject = new Project();
        newProject.setPublished(ProjectPublishedType.None);
        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newProject, template, null);
        addObjectToDelete(result.getRpmObject());
        actionMustFail(result);

        // Give the right to create
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateProjectsFromWBSTemplate(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Creates new project from template, should succeed
        newProject = new Project();
        newProject.setPublished(ProjectPublishedType.None);
        result = getApplicationInterface().createObjectFromTemplate(sessionid,
            newProject, template, null);
        addObjectToDelete(result.getRpmObject());
        actionMustSucceed(result, Project.class);
    }

    public void testDeleteProject() throws Exception
    {
        // Create a pproject, should succeed
        WorkElementScope scope = new WorkElementScope();
        scope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        Project project = (Project) WorkElementFixture.createFixture(
            new Project(), scope, this);
        project = (Project) saveAndValidate(project, scope, true);

        // Create a different resource and make it Manager of the Project
        SecurityRole managerRole = project.getResourceTaskAssignments()[0]
            .getSecurityRole();
        Pool pool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(pool, null, this);
        resource = (Resource) save(resource, null).getRpmObject();
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        assignment.setParent(project);
        assignment.setResource(resource);
        assignment.setSecurityRole(managerRole);
        save(assignment, null);

        // Remove the right to delete
        getCurrentUser().getResourceSecurityRights().setCanDeleteProjects(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the fixture, should fail
        deleteAndValidate(project, false);

        // Give the right to delete
        getCurrentUser().getResourceSecurityRights().setCanDeleteProjects(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the fixture, should succeed
        deleteAndValidate(project, true);
    }

    //    TODO Enable this test when we can unlock a project template.
    //    public void testDeleteProjectTemplate() throws Exception
    //    {
    //        // Create a template
    //        Project template = createProjectTemplate();
    //        
    //        // Remove the right to delete
    //        rights.setCanDeleteTemplates(Boolean.FALSE);
    //        saveSecurityRights();
    //
    //        // Unpublish the project from template, should fail
    //        template.setPublished(ProjectPublishedType.None);
    //        SingleResult result = getApplicationInterface().save(sessionid, template, null, ReloadType.ReloadResult);
    //        assertNotNull("Result is null", result);
    //        actionMustFail(result);
    //
    //        // Give the right to delete
    //        rights.setCanDeleteTemplates(Boolean.TRUE);
    //        saveSecurityRights();
    //
    //        // Unpublish the project from template, should succeed
    //        template.setPublished(ProjectPublishedType.None);
    //        result = getApplicationInterface().save(sessionid, template, null, ReloadType.ReloadResult);
    //        assertNotNull("Result is null", result);
    //        actionMustSucceed(result, template.getClass());
    //    }

}