/*
 * Created on Jan 31, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceContactGroupAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceGeographicalAssignmentFixture;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author karim
 *
 */
public class TestResourceSecurityLocal extends BaseResourceSecurityTestLocal
{
    public void testCreateResource() throws Exception
    {
        // Set up the scope
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());

        // Remove the right to create
        getCurrentUser().getResourceSecurityRights().setCanCreateResources(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create a resource, should fail
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        saveAndValidate(resource, scope, false);

        // Give the right to create
        getCurrentUser().getResourceSecurityRights().setCanCreateResources(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create a resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testDeleteResource() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to delete
        getCurrentUser().getResourceSecurityRights().setCanDeleteResources(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the resource, should fail
        deleteAndValidate(resource, false);

        // Give the right to delete
        getCurrentUser().getResourceSecurityRights().setCanDeleteResources(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the resource, should succeed
        deleteAndValidate(resource, true);
    }

    public void testEditResourceAddress() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditResourceAddress(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setAddress1(resource.getAddress1() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditResourceAddress(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceCalendar() throws Exception
    {
        // Create a resource, should succeed
        CalendarScope cs = new CalendarScope();
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCalendar(cs);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCalendar(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setCalendar(RPMCalendarFixture.createSaveFixture(null, this));
        saveAndValidate(resource, scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCalendar(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        resource.setCalendar(RPMCalendarFixture.createSaveFixture(null, this));
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceCompetencyCostsRates() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCompetencyAttributeAssignments(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCompetencyCostsRates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        ResourceAttributeAssignment[] aa = resource
            .getCompetencyAttributeAssignments();
        aa[0].setRates(ResourceFixture.createRates());
        saveAndValidate(resource, scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCompetencyCostsRates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceCompetencySkills() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCompetencyAttributeAssignments(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCompetencySkills(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        ResourceAttributeAssignment[] aa = resource
            .getCompetencyAttributeAssignments();
        aa[0].setDescription("test");
        saveAndValidate(aa[0], null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceCompetencySkills(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(aa[0], null, true);
    }

    public void testCreateResourceDocuments() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        scope.setDocumentFolder(docScope);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        DocumentElement de = DocumentElementFixture.createFixture(resource
            .getDocumentFolder(), new Document(), 0, docScope.getChildren(),
            this);
        resource.getDocumentFolder().setChildren(new DocumentElement[]
        {
            de
        });
        saveAndValidate(de, docScope.getChildren(), false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(de, docScope.getChildren(), true);
    }

    public void testEditResourceDocuments() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        scope.setDocumentFolder(docScope);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Add new document, should succeed
        DocumentElement de = DocumentElementFixture.createFixture(resource
            .getDocumentFolder(), new Document(), 0, docScope.getChildren(),
            this);
        de = (DocumentElement) saveAndValidate(de, docScope.getChildren(), true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the document, should fail
        de.setDocumentRank("test");
        saveAndValidate(de, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the document, should succeed
        saveAndValidate(de, null, true);
    }

    public void testDeleteResourceDocuments() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        scope.setDocumentFolder(docScope);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Add new document
        DocumentElement de = DocumentElementFixture.createFixture(resource
            .getDocumentFolder(), new Document(), 0, docScope.getChildren(),
            this);
        de = (DocumentElement) saveAndValidate(de, docScope.getChildren(), true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the document, should fail
        deleteAndValidate(de, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the document, should succeed
        deleteAndValidate(de, true);
    }

    public void testEditResourceEffectiveDate() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        Calendar ed = resource.getEffectiveStartDate();
        ed.setTimeInMillis(ed.getTimeInMillis() + 100);
        resource.setEffectiveStartDate(ed);
        saveAndValidate(resource, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceDiscontinueDate() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        Calendar dd = resource.getDiscontinueDate();
        if (dd == null)
            dd = new GregorianCalendar();
        dd.setTimeInMillis(dd.getTimeInMillis() + 100);
        resource.setDiscontinueDate(dd);
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceActive() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setActive(new Boolean(false));
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceExecutiveAttributes() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setExecutiveAttributes(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceAttributeAdmin(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        AttributeAssignment[] aa =
        {
            AttributeAssignmentFixture.createAssignmentFixture(resource, this)
        };
        resource.setExecutiveAttributes(aa);
        saveAndValidate(resource, scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceAttributeAdmin(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceFullName() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceFullName(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setFullName(resource.getFullName() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceFullName(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceInformation() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceInformation(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setTitle(resource.getTitle() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceInformation(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceName() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights().setCanEditResourceName(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setGivenName(resource.getGivenName() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights().setCanEditResourceName(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceNickName() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceNickName(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setNickName(resource.getNickName() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceNickName(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceOrganization() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setOrganizationalAssignment(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        OrganizationalAssignment oa = new OrganizationalAssignment();
        oa.setOrganizational(OrganizationalFixture.createSaveFixture(0, this));
        resource.setOrganizationalAssignment(oa);
        saveAndValidate(resource, scope, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceGroup() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setContactGroupAssignments(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        ContactGroupAssignment[] cga =
        {
            ResourceContactGroupAssignmentFixture.createAssignmentFixture(
                resource, this)
        };
        resource.setContactGroupAssignments(cga);
        saveAndValidate(resource, scope, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceGeographical() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setGeographicalAssignment(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        GeographicalAssignment ga = ResourceGeographicalAssignmentFixture
            .createFixture(resource, this);
        resource.setGeographicalAssignment(ga);
        saveAndValidate(resource, scope, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourcePassword() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourcePassword(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setPassword("test");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourcePassword(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceRates() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setRates(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights().setCanEditResourceRates(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the rates, should fail
        ResourceRates rr = resource.getRates();
        rr.setSellingRatePerHour(new Double(rr.getSellingRatePerHour()
            .intValue() + 10));
        saveAndValidate(rr, null, false);

        // Update the resource, should fail
        // Not valid because the Is Modified value is not set
        // resource.setRates(ResourceFixture.createRates());
        // saveAndValidate(resource, scope, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights().setCanEditResourceRates(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the rates, should succeed
        // Not supported by the manager!!!
        // saveAndValidate(rr, null, true); 

        // Update the resource, should succeed
        saveAndValidate(resource, scope, true);
    }

    public void testEditResourceUserName() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceUserNameEmail(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setUserName(resource.getUserName() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceUserNameEmail(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceEmail() throws Exception
    {
        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceUserNameEmail(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        resource.setEmailAddress(resource.getEmailAddress() + "_UPDATED");
        saveAndValidate(resource, null, false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceUserNameEmail(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        saveAndValidate(resource, null, true);
    }

    public void testEditResourceCustomFieldGroups() throws Exception
    {
        Boolean[] flagArray = new Boolean[10];
        Arrays.fill(flagArray, Boolean.TRUE);

        // Create a resource, should succeed
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCustomFieldAssignments(true);
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Create a custom field assignment, should succeed
        CustomFieldAssignment[] cfa2 =
        {
            new CustomFieldAssignment()
        };
        cfa2[0].setCustomField(CustomFieldFixture.createSaveCustomField(
            CustomFieldFixture.CF_RESOURCE_CONTEXT, FieldInputType.Text,
            new Integer(1), this));
        resource.setCustomFieldAssignments(cfa2);
        resource = (Resource) saveAndValidate(resource, scope, true);

        // Remove the rigth to edit
        flagArray[0] = Boolean.FALSE;
        getCurrentUser().getResourceSecurityRights()
            .setCanEditOtherResourcesRecordCustomFieldsGroups(flagArray);
        saveCurrentUserSecurityRights(null);

        // Update the custom field assignment, should fail
        cfa2 = resource.getCustomFieldAssignments();
        cfa2[0].setValue(cfa2[0].getValue() + "_MOD");
        saveAndValidate(cfa2[0], null, false);

        // Give the rigth to edit
        flagArray[0] = Boolean.TRUE;
        getCurrentUser().getResourceSecurityRights()
            .setCanEditOtherResourcesRecordCustomFieldsGroups(flagArray);
        saveCurrentUserSecurityRights(null);

        // Update the custom field assignment, should succeed
        saveAndValidate(cfa2[0], null, true);

        // Clean up
        addObjectToDelete(cfa2[0].getCustomField());
    }

    public void testEditResourceSecurityRights() throws Exception
    {
        // Prepare Scope
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setResourceSecurityRights(true);

        // Create new resource, should succeed
        Pool parentPool = PoolFixture.retrieveStaticFixture(this);
        Resource resource = ResourceFixture.createFixture(parentPool, scope,
            this);
        resource = (Resource) saveAndValidate(resource, scope, true);

        //      Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceSecurity(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the security rights, should succeed
        ResourceSecurityRights securityRights = resource
            .getResourceSecurityRights();
        securityRights.setCanAdoptWhatIfAnalysis(Boolean.FALSE);
        saveAndValidate(resource, scope, true);

        // Update the resource, should succeed
        resource.setResourceSecurityRights(new ResourceSecurityRights());
        saveAndValidate(resource, scope, true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditResourceSecurity(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the security rights, should fail
        securityRights = resource.getResourceSecurityRights();
        securityRights.setCanAdoptWhatIfAnalysis(Boolean.FALSE);
        saveAndValidate(resource, scope, false);

    }

}