/*
 * Created on Feb 1, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.security.containers.SecurityGroup;

/**
 * @author karim
 *
 */
public class TestSecurityGroupSecurityLocal extends
        BaseResourceSecurityTestLocal
{
    private SecurityGroup createFixture() throws Exception
    {
        SecurityGroup parent = (SecurityGroup) loadOneFromXpath(
            "/SecurityGroup[@name='DEFAULT_SECURITY_ROLE']", null);
        SecurityGroup securityGroup = new SecurityGroup();
        securityGroup.setName("TestSecurityGroupSecurityLocal");
        securityGroup.setParent(parent);
        return securityGroup;
    }

    public void testCreateDefaultSecurityGroup() throws Exception
    {
        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create new security group, should fail
        SecurityGroup securityGroup = createFixture();
        securityGroup = (SecurityGroup) saveAndValidate(securityGroup, null,
            false);

        // Give the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create new security group, should succeed
        securityGroup = createFixture();
        securityGroup = (SecurityGroup) saveAndValidate(securityGroup, null,
            true);
    }

    public void testDeleteDefaultSecurityGroup() throws Exception
    {
        // Create new security group, should succeed
        SecurityGroup securityGroup = createFixture();
        securityGroup = (SecurityGroup) saveAndValidate(securityGroup, null,
            true);

        // Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the security group, should fail
        deleteAndValidate(securityGroup, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the security group, should succeed
        deleteAndValidate(securityGroup, true);
    }

    public void testUpdateDefaultSecurityGroup() throws Exception
    {
        // Create new security group, should succeed
        SecurityGroup securityGroup = createFixture();
        securityGroup = (SecurityGroup) saveAndValidate(securityGroup, null,
            true);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the security group, should succeed
        securityGroup.setCanAdoptWhatIfAnalysis(new Boolean(!securityGroup
            .getCanAdoptWhatIfAnalysis().booleanValue()));
        saveAndValidate(securityGroup, null, true);

        //      Remove the right to edit 
        getCurrentUser().getResourceSecurityRights()
            .setCanEditDefaultResourceSecurity(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the security group, should fail
        securityGroup.setCanAdoptWhatIfAnalysis(new Boolean(!securityGroup
            .getCanAdoptWhatIfAnalysis().booleanValue()));
        saveAndValidate(securityGroup, null, false);
    }

}
