package com.ibm.rpm.client.junit.security.testcases.role;

import java.util.ArrayList;

import junit.framework.Assert;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.MitigationTrigger;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.types.PrecisionType;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ScopeElementSecurityType;

public abstract class AbstractPlannedScopeElementSecurityTestLocal extends
        AbstractPlannedSecurityTestLocal
{
    public AbstractPlannedScopeElementSecurityTestLocal(
            Class securityRoleClass, RPMObject containerObject,
            RPMObject grandparentWorkElement, RPMObjectScope testObjectScope,
            String fieldToTest, String flagName)
    {
        super(securityRoleClass, containerObject, testObjectScope, fieldToTest,
            flagName);
        this.grandparentObject = grandparentWorkElement;
    }

    public AbstractPlannedScopeElementSecurityTestLocal(
            Class securityRoleClass, RPMObject containerObject,
            RPMObject grandparentWorkElement, RPMObjectScope testObjectScope,
            ArrayList fieldsToTest, String flagName)
    {
        super(securityRoleClass, containerObject, testObjectScope,
            fieldsToTest, flagName);
        this.grandparentObject = grandparentWorkElement;
    }

    protected void prepareParentHierarchy() throws Exception
    {
        // create and save the grandparent of the ScopeElement
        grandparentObjectSecurityRole = prepareSecurityRole(grandparentObject
            .getClass(), true);
        grandparentObject = (WorkElement) prepareContainer(grandparentObject,
            null, new WorkElementScope(), securityTestUser,
            grandparentObjectSecurityRole);

        // Set the grandparent scope used in the load operation
        WorkElementScope grandparentWorkElementScope = new WorkElementScope();
        grandparentWorkElementScope
            .setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        ScopeElementScope parentScopeFolderScope = new ScopeElementScope();
        parentScopeFolderScope
            .setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        grandparentWorkElementScope.setScopeElement(parentScopeFolderScope);

        // Load the grandparent with the parent ScopeFolder
        grandparentObject = (WorkElement) loadFromID(grandparentObject,
            grandparentWorkElementScope);
        assertNotNull(grandparentObject);
        parentObject = (ScopeFolder) ((WorkElement) grandparentObject)
            .getScopeElement();
        assertNotNull(parentObject);
        parentObjectSecurityRole = prepareSecurityRole(parentObject.getClass(),
            true);
    }

    /**
	 * Change a certain value.
	 * 
	 * @param object
	 *            The object for which we want to change the value.
	 * @return the new object which has been changed
	 * @throws Exception
	 */
	protected Object changeArgValue(Object object) throws Exception {
		boolean isArray = false;

		if (object.getClass().isArray()) {
			object = ((Object[]) object)[0];
			isArray = true;
		}
		if (object instanceof ApprovedCostBenefits) {
			object = getChangedApprovedCostBenefits((ApprovedCostBenefits) object);
		} else if (object instanceof ScopeElementSecurityType) {
			object = getChangedScopeElementSecurityType((ScopeElementSecurityType) object);
		} else if (object instanceof RiskMatrix) {
			object = getChangedRiskMatrix((RiskMatrix) object);
		} else if (object instanceof MitigationFactor) {
			MitigationFactor retVal = getChangedMitigationFactors((MitigationFactor) object);
			if (isArray) {
				object = new MitigationFactor[] { (MitigationFactor) retVal };
			}
		} else if (object instanceof OrderOfMagnitudeCostBenefits) {
			object = getChangedOrderOfMagnitudeCostBenefits((OrderOfMagnitudeCostBenefits) object);
		} else if (object instanceof Impact) {
			object = getChangedImpact((Impact) object);
		} else if(object instanceof ClientCostCenter){
			((ClientCostCenter)object).setName(getReverseString(((ClientCostCenter)object).getName()));
		} else if(object instanceof ResourceTaskAssignment){
			object = getChangedTaskAssignment((ResourceTaskAssignment)object);
			if (isArray) {
				object = new ResourceTaskAssignment[]{(ResourceTaskAssignment)object};
			}
		}else if(object instanceof Duplicate){
			object = getChangedDuplicates((Duplicate)object);
			if (isArray) {
				object = new Duplicate[]{(Duplicate)object};
			}
		} else {
			if(isArray)
			{
				object = new Object[]{object};
			}
			object = super.changeArgValue(object);
		}
		return object;
	}
	private Duplicate getChangedDuplicates(Duplicate value) throws Exception{
		AbstractScope dupSE = null;
		if (testObject instanceof Action)
        {
			dupSE = new Action();
        }
        else if (testObject instanceof ChangeRequest)
        {
        	dupSE = new ChangeRequest();
        }
        else if (testObject instanceof Defect)
        {
        	dupSE = new Defect();
        }
        else if (testObject instanceof Issue)
        {
        	dupSE = new Issue();
        }
        else if (testObject instanceof Requirement)
        {
        	dupSE = new Requirement();
        }
        else if (testObject instanceof Risk)
        {
        	dupSE = new Risk();
        }
        else if (testObject instanceof ServiceRequest)
        {
        	dupSE = new ServiceRequest();
        }
        dupSE.setName(HelperTestUtilities.createName());
        dupSE.setParent(parentObject);
        dupSE = (AbstractScope)save(dupSE, null).getRpmObject();
        assertNotNull(dupSE);
		Duplicate pd = new Duplicate();
        pd.setScopeElement(dupSE);
        pd.setParent(testObject);
    	return pd;
	}
	private ResourceTaskAssignment getChangedTaskAssignment(ResourceTaskAssignment value) throws Exception{
		TaskAssignment taskAssignment = value.getTaskAssignment();
		if(taskAssignment == null)
			taskAssignment = TaskAssignmentFixture.createSaveFixture(new TaskAssignmentScope(), this);
			//taskAssignment = TaskAssignmentFixturecreateTaskAssignmentUnsavableFixture(null, false, this);
		if(taskAssignment != null)
		{
			if(flagName.equalsIgnoreCase("CanEditResourceProfileCost"))
		{
			Double d = taskAssignment.getCostPerHour();
			if(d==null)
			{
				d = new Double(1);
			}
			taskAssignment.setCostPerHour(new Double(d.doubleValue() + 1));
			d = taskAssignment.getSpecialCostPerHour();
			if(d==null)
			{
				d = new Double(1);
			}
			taskAssignment.setSpecialCostPerHour(new Double(d.doubleValue() + 1));
		}
			else if(flagName.equalsIgnoreCase("CanEditResourceProfileRates"))
		{
			Double d = taskAssignment.getSellingRatePerHour();
			if(d==null)
			{
				d = new Double(1);
			}
			taskAssignment.setSellingRatePerHour(new Double(d.doubleValue() + 1));
			d = taskAssignment.getSpecialSellingRatePerHour();
			if(d==null)
			{
				d = new Double(1);
			}
			taskAssignment.setSpecialSellingRatePerHour(new Double(d.doubleValue() + 1));
			}
		}
		else
		{
			taskAssignment = TaskAssignmentFixture.createTaskAssignmentUnsavableFixture(null, false, this);
			value.setTaskAssignment(taskAssignment);
		}
		value.setTaskAssignment(taskAssignment);
		return value;
	}

	private Impact getChangedImpact(Impact value) {
		value.setOrderOfMagnitudeHighWorkInHours(new Integer(value
				.getOrderOfMagnitudeHighWorkInHours().intValue() + 1));
		value.setOrderOfMagnitudeLowWorkInHours(new Integer(value
				.getOrderOfMagnitudeLowWorkInHours().intValue() + 1));
		value.setOrderOfMagnitudeWorkInHours(new Integer(value
				.getOrderOfMagnitudeWorkInHours().intValue() + 1));
		value.setScheduledImpactInDays(new Integer(value.getScheduledImpactInDays().intValue() + 1));
		return value;
	}

	/**
	 * Increase the values of different fields by 1
	 * 
	 * @param value
	 * 			
	 * @return The changed value
	 */
	private OrderOfMagnitudeCostBenefits getChangedOrderOfMagnitudeCostBenefits(
			OrderOfMagnitudeCostBenefits value) {
		value.setOrderOfMagnitudeHigh(new Double(value.getOrderOfMagnitudeHigh().doubleValue() + 1));
		value.setOrderOfMagnitudeLow(new Double(value.getOrderOfMagnitudeLow().doubleValue() + 1));
		value.setOrderOfMagnitudeBillableRevenue(new Double(value
				.getOrderOfMagnitudeBillableRevenue().doubleValue() + 1));
		value.setOrderOfMagnitudeCost(new Double(value.getOrderOfMagnitudeCost().doubleValue() + 1));
		value.setOrderOfMagnitudeNonBillableBenefits(new Double(value
				.getOrderOfMagnitudeNonBillableBenefits().doubleValue() + 1));
		return value;
	}

	/**
	 * Increase the values of different fields by 1
	 * 
	 * @param value
	 * 			
	 * @return The changed value
	 */
	private MitigationFactor getChangedMitigationFactors(MitigationFactor value) {
		if (value.getPriority() != null && value.getPriority().intValue() == 20) {
			value.setPriority(new Integer(40));
		} else {
			value.setPriority(new Integer(20));
		}
		if (value.getDescription() != null
				&& value.getDescription().length() > 0) {
			value.setDescription(getReverseString(value.getDescription()));
		} else {
			value.setDescription("Description Mitigation Factor");
		}
		if (value.getMitigationTriggers() != null
				&& value.getMitigationTriggers()[0] != null) {
			if (value.getMitigationTriggers()[0].getDescription() != null
					&& value.getMitigationTriggers()[0].getDescription()
							.length() > 0) {
				value.getMitigationTriggers()[0]
						.setDescription(getReverseString(value
								.getMitigationTriggers()[0].getDescription()));
			} else {
				value.getMitigationTriggers()[0]
						.setDescription("Description Mitigation Trigger");
			}
			MitigationTrigger mt[] = new MitigationTrigger[value
					.getMitigationTriggers().length + 1];
			mt[value.getMitigationTriggers().length] = new MitigationTrigger();
			value.setMitigationTriggers(mt);
		} else {
			MitigationTrigger m = new MitigationTrigger();
			m.setDescription("Description Mitigation Trigger");
			value.setMitigationTriggers(new MitigationTrigger[] { m });
		}
		if (value.getMitigationCost() != null) {
			value.setMitigationCost(new Double(value.getMitigationCost().doubleValue() + 1));
		} else {
			value.setMitigationCost(new Double(20.0));
		}

		return value;
	}

	/**
	 * Increase the values of different fields by 1
	 * 
	 * @param value
	 * 			
	 * @return The changed value
	 */
	private RiskMatrix getChangedRiskMatrix(RiskMatrix value) {
		value.setProbabilityCost(new Double(value.getProbabilityCost().doubleValue() + 1));
		if (value.getPrecision().equals(PrecisionType.High)) {
			value.setPrecision(PrecisionType.Low);
		} else {
			value.setPrecision(PrecisionType.High);
		}
		value.setExposurePercent(new Integer(value.getExposurePercent().intValue() + 1));

		if (value.getProbability().intValue() == 20) {
			value.setProbability(new Integer(40));
		} else {
			value.setProbability(new Integer(20));
		}
		if (value.getRiskMatrixColor().equals("Green")) {
			value.setRiskMatrixColor("Red");
		} else {
			value.setRiskMatrixColor("Green");
		}
		value.setMitigationCost(new Double(value.getMitigationCost().doubleValue() + 1));
		if (value.getImpact().intValue() == 20) {
			value.setImpact(new Integer(40));
		} else {
			value.setImpact(new Integer(20));
		}
		value.setConsequenceCost(new Double(value.getConsequenceCost().doubleValue() + 1));
		return value;
	}

	/**
	 * Increase the values of different fields by 1
	 * 
	 * @param value
	 * 			
	 * @return The changed value
	 */
	private ScopeElementSecurityType getChangedScopeElementSecurityType(
			ScopeElementSecurityType value) {
		if (ScopeElementSecurityType.ByInvitation
				.equals(value)) {
			value = ScopeElementSecurityType.ProjectLevel;
		} else {
			value = ScopeElementSecurityType.ByInvitation;
		}
		return value;
	}

	/**
	 * Increase the values of different fields by 1
	 * 
	 * @param value
	 * 			
	 * @return The changed value
	 */
	private ApprovedCostBenefits getChangedApprovedCostBenefits(
			ApprovedCostBenefits value) {
		value.setBillableRevenue(new Double(((ApprovedCostBenefits) value)
				.getBillableRevenue().doubleValue() + 1));
		value.setNonBillableBenefits(new Double(((ApprovedCostBenefits) value)
				.getNonBillableBenefits().doubleValue() + 1));
		value
				.setWorkInHours(new Integer(((ApprovedCostBenefits) value).getWorkInHours().intValue() + 1));
		value.setScheduleImpactInDays(new Integer(((ApprovedCostBenefits) value)
				.getScheduleImpactInDays().intValue() + 1));
		return value;
	}
    /**
     * Verify the test must fail for CanChangeType cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForChangeTypes() throws Exception
    {
    	AbstractScope newAbstractScope = null;
    	if(testObject instanceof Risk)
    	{
    		newAbstractScope = new Defect();
    	}
    	else
    	{
    		newAbstractScope = new Risk();
    	}
    	convertContainer(testObject, newAbstractScope, testObjectScope, false);
    }
    /**
     * Verify the test must Succeed for CanChangeType cases.
     * 
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForChangeTypes() throws Exception
    {
    	AbstractScope newAbstractScope = null;
    	if(testObject instanceof Risk)
    	{
    		newAbstractScope = new Defect();
    	}
    	else
    	{
    		newAbstractScope = new Risk();
    	}
    	convertContainer(testObject, newAbstractScope, testObjectScope, true);
    }
}
