package com.ibm.rpm.client.junit.security.testcases.role;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.containers.TaskState;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceRoleAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.DependencyFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.AbstractWbsSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.NotesMinutesSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

/**
 * 
 * @author cobber
 * 
 */

public abstract class AbstractPlannedSecurityTestLocal extends
        BaseRoleSecurityTestLocal
{
    /**
     * The Security Role class which belongs to the test object
     */
    protected Class securityRoleClass;

    /**
     * The field needs to be tested
     */
    protected String fieldToTest;

    /**
     * The fields need to be tested
     */
    protected ArrayList fieldsToTest;

    /**
     * The security flag name
     */
    protected String flagName;

    /**
     * The test object which will be used during the test.
     */
    protected RPMObject testObject;

    /**
     * The scope of the testObjcet
     */
    protected RPMObjectScope testObjectScope;

    /**
     * The Security Role created by prepareSecurityRole which is used during the
     * test
     */
    protected SecurityRole testObjectSecurityRole;

    /**
     * The certain test security flags which needs to be turned off before the
     * test.
     */
    protected ArrayList testObjectRequiredOffFlags;

    /**
     * The grandparent object in the hierarchy.
     */
    protected RPMObject grandparentObject;

    /**
     * The security role for the grandparent object.
     */
    protected SecurityRole grandparentObjectSecurityRole;

    /**
     * The flags in the grandparent security role that need to be turned off
     * before the test.
     */
    protected ArrayList grandparentRequiredOffFlags;

    /**
     * The parent object in the hierarchy. Usually it is a folder. The
     * parentObject is not required for WorkElement Security tests.
     */
    protected RPMObject parentObject;

    /**
     * The security role for the parent object.
     */
    protected SecurityRole parentObjectSecurityRole;

    /**
     * The flags in the parent security role which need to be turned off before
     * the test.
     */
    protected ArrayList parentRequiredOffFlags;

    /**
     * The field value for some test cases. e.g. CanEdit, CanRemove.
     */
    protected Object fieldValue;

    protected static final String SET = "set";

    protected static final String GET = "get";
    
    protected static final String IS = "is";
    
    protected static final String RESOURCE = "Resource";
    
    protected static final String PARTICIPANT = "Participant";
    
    protected static final String PROFILE = "Profile";
    
    protected static final String CAN_REMOVE_RESOURCE_PARTICIPANT = "CanRemoveResourceOrParticipant";
      
    protected static final String DEPENDENCIES = "Dependencies";
    
    protected static final String ATTRIBUTE_ASSIGNMENTS = "AttributeAssignments";

    protected static final String TIMECODE_ASSIGNMENTS = "TimeCodeAssignments";
    
    protected static final String DUPLICATES = "Duplicates";
    /**
     * Constructor.
     * 
     * @param securityRoleClass
     *            The security role class of the test object.
     * @param testObject
     *            The container needs to be tested.
     * @param testObjectScope
     *            The scope of the test object.
     * @param fieldToTest
     *            The field we are testing.
     * @param flagName
     *            The name of the flag that we are testing.
     */
    public AbstractPlannedSecurityTestLocal(Class securityRoleClass,
            RPMObject testObject, RPMObjectScope testObjectScope,
            String fieldToTest, String flagName)
    {
        this.securityRoleClass = securityRoleClass;
        this.testObject = testObject;
        this.testObjectScope = testObjectScope;
        this.fieldToTest = initFieldToTest(fieldToTest);
        this.flagName = initFlagName(flagName);
    }

    /**
     * Constructor.
     * 
     * @param securityRoleClass
     *            The security role class of the test object.
     * @param testObject
     *            The container needs to be tested.
     * @param testObjectScope
     *            The scope of the test object.
     * @param fieldsToTest
     *            The fields we are testing.
     * @param flagName
     *            The name of the flag that we are testing.
     */
    public AbstractPlannedSecurityTestLocal(Class securityRoleClass,
            RPMObject testObject, RPMObjectScope testObjectScope,
            ArrayList fieldsToTest, String flagName)
    {
        this.securityRoleClass = securityRoleClass;
        this.testObject = testObject;
        this.testObjectScope = testObjectScope;
        this.fieldsToTest = fieldsToTest;
        for (int i = 0; i != this.fieldsToTest.size(); ++i)
        {
            this.fieldsToTest.set(i, initFieldToTest((String) this.fieldsToTest
                .get(i)));
        }
        this.flagName = initFlagName(flagName);
    }

    public void setUp() throws Exception
    {
        SpecificTestController.overrideOption(
            RpmOptions.WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT,
            Boolean.FALSE);

        super.setUp();
        testObjectSecurityRole = prepareSecurityRole(testObject.getClass(),
            true);
        prepareParentHierarchy();
        prepareTestContainer();
        initParentSecurityFlags();
    }

    protected void runTest() throws Exception
    {
        setUp();
        flagOff(testObject, flagName);
        initTestContainerOtherRelatedSecurityFlags();
        // This turnOff is not required as it is done in flagOff()
        turnOffTestFlag(testObject, testObjectSecurityRole, flagName);
        verifyMustFailScenario();
        resetInitializedTestContainerOtherRelatedSecurityFlags();
        turnOnTestFlag(testObject, testObjectSecurityRole, flagName);
        verifyMustSuccessScenario();
    }

    /**
     * Verify the test must fail when we turn off a flag. We are expecting a
     * failure in the save operation.
     * 
     * @throws Exception
     */
    protected abstract void verifyMustFailScenario() throws Exception;

    /**
     * Verify the test must success when we turn on a flag. We are expecting a
     * successful in the save operation.
     * 
     * @throws Exception
     */
    protected abstract void verifyMustSuccessScenario() throws Exception;

    /**
     * In this method, we prepare parent hierarchy including grandparent, and
     * parent if needed.
     * 
     * @throws Exception
     */
    protected abstract void prepareParentHierarchy() throws Exception;

    /**
     * Prepare the container which will be used in the test.
     * 
     * @throws Exception
     */
    protected void prepareTestContainer() throws Exception
    {
        testObject = prepareContainer(testObject, parentObject,
            testObjectScope, securityTestUser, testObjectSecurityRole);
        
        // Verify that the field(s) to be tested are popluated
        verifyNotNullValue();
    }

    /**
     * The method turns off some flags in the parent or grandparent before the
     * test.
     * 
     * @throws Exception
     */
    private void initParentSecurityFlags() throws Exception
    {
        if (grandparentRequiredOffFlags != null)
        {
            grandparentRequiredOffFlags = initFlagNames(grandparentRequiredOffFlags);
            turnOffGrandparentSecurityFlags(grandparentObject,
                grandparentRequiredOffFlags);
            grandparentObjectSecurityRole = getSecurityRole(grandparentObject);
        }
        if (parentRequiredOffFlags != null)
        {
            parentRequiredOffFlags = initFlagNames(parentRequiredOffFlags);
            turnOffParentSecurityFlags(parentObject, parentRequiredOffFlags);
            parentObjectSecurityRole = getSecurityRole(parentObject);
        }
    }

    /**
     * The method turns off some flags in the testObjectSecurityRole.
     * 
     * @throws Exception
     */
    private void initTestContainerOtherRelatedSecurityFlags()
            throws Exception
    {
        if (testObjectRequiredOffFlags != null)
        {
            testObjectRequiredOffFlags = initFlagNames(testObjectRequiredOffFlags);
            turnOffTestObjectSecurityFlags(testObject,
                testObjectRequiredOffFlags);
        }
        testObjectSecurityRole = getSecurityRole(testObject);
    }

    /**
     * Reset the testObjectSecurityRole. It turns on the flags which was turned
     * off in initTestContainerOtherRelatedSecurityFlags().
     * 
     * @throws Exception
     */
    private void resetInitializedTestContainerOtherRelatedSecurityFlags()
            throws Exception
    {
        if (testObjectRequiredOffFlags != null)
        {
            testObjectRequiredOffFlags = initFlagNames(testObjectRequiredOffFlags);
            turnOnTestObjectSecurityFlags(testObject,
                testObjectRequiredOffFlags);
        }
        testObjectSecurityRole = getSecurityRole(testObject);
    }

    /**
     * Change a certain value.
     * 
     * @param object
     *            The object for which we want to change the value.
     * @return the new object which has been changed
     * @throws Exception
     */
    protected Object changeArgValue(Object object) throws Exception
    {
        boolean isArray = false;

        if (object.getClass().isArray())
        {
            object = ((Object[]) object)[0];
            isArray = true;
        }

        if (object instanceof String)
        {
            object = getReverseString((String) object);
        }
        else if(object instanceof Integer)
        {
        	object = getChangedIntegerValue((Integer)object);
        }
        else if (object instanceof Boolean)
        {
            object = getChangedBooleanValue((Boolean) object);
        }
        else if (object instanceof Double)
        {
        	object = getChangedDoubleValue((Double) object);
        }
        else if (object instanceof RtfAssignment)
        {
            object = getChangedRtfAssignment((RtfAssignment) object);
            if (isArray)
            {
                object = new RtfAssignment[]
                {
                    (RtfAssignment) object
                };
            }
        }
        else if (object instanceof AssignmentType)
        {
            object = getChangedAssignmentType((AssignmentType) object);
        }
        else if(object instanceof DatafieldState)
        {
        	object = getChangedState((DatafieldState)object);
        }
        else if(object instanceof AttributeAssignment)
        {
        	object = getChangedAttributeAssignment((AttributeAssignment) object);
        	if(isArray)
        	{
        		object = new AttributeAssignment[] { (AttributeAssignment) object };
        	}
        }
        else if (object instanceof Calendar)
        {
        	object = getChangedDate((Calendar)object);
        }
        // TODO add more types if needed.
        else
        {
            object = null;
        }

        return object;
    }

    /**
     * Increase the value by 1
     * 
     * @param value
     * 			
     * @return The increased value
     */
    private Integer getChangedIntegerValue(Integer value)
    {
    	return new Integer(value.intValue() + 1);
    }
    
    /**
     * Increase the value by 1
     * 
     * @param value
     * 			
     * @return The increased value
     */
    private Double getChangedDoubleValue(Double value)
    {
    	return new Double(value.doubleValue() + 1);
    }
    
    /**
     * Change the Boolean object value.
     * 
     * @param value
     *            The boolean object needs to be changed.
     * @return The new boolean object.
     */
    private Boolean getChangedBooleanValue(Boolean value)
    {
        Boolean retVal = Boolean.TRUE;

        if (value.booleanValue())
        {
            retVal = Boolean.FALSE;
        }

        return retVal;
    }

    /**
     * Change the RtfAssignment value.
     * 
     * @param rtfAssignment
     *            The RtfAssignment needs to be changed.
     * @return The new RtfAssignment.
     */
    private RtfAssignment getChangedRtfAssignment(RtfAssignment rtfAssignment)
    {
        rtfAssignment.setValue(getReverseString(rtfAssignment.getValue()));
        return rtfAssignment;
    }
    
    /**
     * Change the AttributeAssignment Value
     * 
     * @param attrAssignment
     * 
     * @return The changed AttributeAssignment
     * @throws ParseException 
     * 
     */
    private AttributeAssignment getChangedAttributeAssignment(AttributeAssignment attrAssignment) throws ParseException
    {
    	Attribute attribute = (Attribute)attrAssignment.getAttribute();
    	InputType inputType = attribute.getInputType();
    	if(InputType.Date.equals(inputType))
    	{
    		Calendar cal = DateUtil.convertToCalendarNoTime((String)attrAssignment.getValue());
    		attrAssignment.setValue(DateUtil.convertToStringNoTime(getChangedDate(cal).getTime()));
    	}
    	else if(InputType.Text.equals(inputType))
    	{
    		attrAssignment.setValue(getReverseString((String)attrAssignment.getValue()));
    	}
    	else if(InputType.Numeric.equals(inputType))
    	{
    		attrAssignment.setValue(getChangedIntegerValue(new Integer((String)attrAssignment.getValue())));
    	}
    	else if(InputType.RiskTolerance.equals(inputType))
    	{
    		attrAssignment.setValue(getChangedDoubleValue(new Double((String)attrAssignment.getValue())));
    	}
    	else if(InputType.RiskAppetite.equals(inputType))
    	{
    		attrAssignment.setValue(getChangedIntegerValue(new Integer((String)attrAssignment.getValue())));
    	}
    	else if(InputType.None.equals(inputType))
    	{
    		attribute.setInputType(InputType.Text);
    		attrAssignment.setAttribute(attribute);
    		attrAssignment.setValue("TEST_VALUE");
    	}
    	else
    	{
    		// TODO: Handle the other InputTypes
    	}
    	return attrAssignment;
    }
    
    /**
     * Change the Datafield State value
     * 
     * @param datafieldState
     * 			The DatafieldState needs to be changed
     * 
     * @return The changed DataFieldState
     */
    private DatafieldState getChangedState(DatafieldState datafieldState)
    {
    	String value = datafieldState.getID();

		if (datafieldState instanceof State) {
			// TODO: Implement For WBS Elements
    		if(DatafieldFixture.PROJECT_STATE_IN_PROGRESS.equals(value))
    		{
    			value = DatafieldFixture.PROJECT_STATE_CLOSED;
    		}
    		else
    		{
    			value = DatafieldFixture.PROJECT_STATE_IN_PROGRESS;
    		}
		} else if (datafieldState instanceof ActionState) {
			if (DatafieldFixture.ACTION_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.ACTION_STATE_CLOSED;
			} else {
				value = DatafieldFixture.ACTION_STATE_OPEN;
			}
		} else if (datafieldState instanceof ChangeRequestState) {
			if (DatafieldFixture.CHANGEREQUEST_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.CHANGEREQUEST_STATE_CLOSED;
			} else {
				value = DatafieldFixture.CHANGEREQUEST_STATE_OPEN;
			}
		} else if (datafieldState instanceof DefectState) {
			if (DatafieldFixture.DEFECT_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.DEFECT_STATE_CLOSED;
			} else {
				value = DatafieldFixture.DEFECT_STATE_OPEN;
			}
		} else if (datafieldState instanceof IssueState) {
			if (DatafieldFixture.ISSUE_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.ISSUE_STATE_CLOSED;
			} else {
				value = DatafieldFixture.ISSUE_STATE_OPEN;
			}
		} else if (datafieldState instanceof RequirementState) {
			if (DatafieldFixture.REQUIREMENT_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.REQUIREMENT_STATE_CLOSED;
			} else {
				value = DatafieldFixture.REQUIREMENT_STATE_OPEN;
			}
		} else if (datafieldState instanceof RiskState) {
			if (DatafieldFixture.RISK_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.RISK_STATE_CLOSED;
			} else {
				value = DatafieldFixture.RISK_STATE_OPEN;
			}
		} else if (datafieldState instanceof ServiceRequestState) {
			if (DatafieldFixture.SERVICEREQUEST_STATE_OPEN.equals(value)) {
				value = DatafieldFixture.SERVICEREQUEST_STATE_CLOSED;
			} else {
				value = DatafieldFixture.SERVICEREQUEST_STATE_OPEN;
			}
		}
		if (DatafieldFixture.DOCUMENT_STATE_CLIENT_APPROVED.equals(value)) {
			value = DatafieldFixture.DOCUMENT_STATE_OPEN;
		} else if (DatafieldFixture.NOTEMINUTES_STATE_CLIENT_APPROVED
				.equals(value)) {
			value = DatafieldFixture.NOTEMINUTES_STATE_OPEN;
		} else {
    		value = Integer.toString(Integer.parseInt(value)+1);
    	}
    	datafieldState.setID(value);
    	datafieldState.setValue(value);
    	return datafieldState;
    }

    /**
     * Change the AssignmentType value.
     * 
     * @param assignmentType
     *            The AssignmentType needs to be changed.
     * @return The new AssignmentType.
     */
    private AssignmentType getChangedAssignmentType(
            AssignmentType assignmentType)
    {
        if (AssignmentType.Duration.equals(assignmentType))
        {
            assignmentType = AssignmentType.Work;
        }
        else
        {
            assignmentType = AssignmentType.Duration;
        }
        return assignmentType;
    }
    
    /**
     * Increment the Day value by 1
     * 
     * @param date
     * 
     * @return The Next day 
     */
    private Calendar getChangedDate(Calendar date)
    {
    	date.add(Calendar.DAY_OF_MONTH, 1);
		return date;
    }

    /**
     * Reverse a String.
     * 
     * @param originalString
     *            The String needs to be changed.
     * @return The new String.
     */
    protected String getReverseString(String originalString)
    {
        StringBuffer retVal = new StringBuffer();

        if (originalString == null || originalString.trim().equals(""))
        {
            return "anyString";
        }

        for (int i = originalString.length() - 1; i >= 0; --i)
        {
            retVal.append(originalString.charAt(i));
        }

        return retVal.toString();
    }

    /**
     * Capitalize the first character of the name of the test field.
     * 
     * @param fieldToTest
     *            The name of the field needs to be tested.
     * @return The correct format of the name.
     */
    private String initFieldToTest(String fieldToTest)
    {
        return convertFirstCharToUpperCase(fieldToTest);
    }

    /**
     * Capitalize the first character of the flag name.
     * 
     * @param flagName
     *            The name of the flag needs to be tested.
     * @return The correct format of the flag name.
     */
    private String initFlagName(String flagName)
    {
        return convertFirstCharToUpperCase(flagName);
    }

    /**
     * Capitalize the first character of the flag names.
     * 
     * @param flagNames
     *            An ArrayList contains all the flag names.
     * @return The correct format of all the flag names.
     */
    private ArrayList initFlagNames(ArrayList flagNames)
    {
        if (flagNames != null && flagNames.size() != 0)
        {
            ArrayList retVal = new ArrayList();
            for (int i = 0; i != flagNames.size(); ++i)
            {
                retVal.add(convertFirstCharToUpperCase((String) flagNames
                    .get(i)));
            }

            return retVal;
        }

        return null;
    }

    /**
     * Capitalize the first character of a given String.
     * 
     * @param string
     *            The given String.
     * @return The correct format of the String.
     */
    private String convertFirstCharToUpperCase(String string)
    {
        if (string != null && !string.trim().equals(""))
        {
            if (Character.isLowerCase(string.charAt(0)))
            {
                string = Character.toUpperCase(string.charAt(0))
                        + string.substring(1);
            }
        }
        return string;
    }

    /**
     * Turns off certain flags in the grandparent security role.
     * 
     * @param grandparentObject
     *            The grandparent object.
     * @param flags
     *            The flags in grandparent security role which needs to be
     *            turned off.
     * @throws Exception
     */
    private void turnOffGrandparentSecurityFlags(RPMObject grandparentObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(grandparentObject, flags, Boolean.FALSE);
    }

    /**
     * Turns on certain flags in the grandparent security role.
     * 
     * @param grandparentObject
     *            The grandparent object.
     * @param flags
     *            The flags in grandparent security role which needs to be
     *            turned on.
     * @throws Exception
     */
    private void turnOnGrandparentSecurityFlags(RPMObject grandparentObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(grandparentObject, flags, Boolean.TRUE);
    }

    /**
     * Turns off certain flags in the parent security role.
     * 
     * @param parentObject
     *            The parent object.
     * @param flags
     *            The flags in parent security role which needs to be turned
     *            off.
     * @throws Exception
     */
    private void turnOffParentSecurityFlags(RPMObject parentObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(parentObject, flags, Boolean.FALSE);
    }

    /**
     * Turns on certain flags in the parent security role.
     * 
     * @param parentObject
     *            The parent object.
     * @param flags
     *            The flags in parent security role which needs to be turned on.
     * @throws Exception
     */
    private void turnOnParentSecurityFlags(RPMObject parentObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(parentObject, flags, Boolean.TRUE);
    }

    /**
     * Turns off certain flags in the test object security role.
     * 
     * @param testObject
     *            The test object.
     * @param flags
     *            The flags in test object security role which needs to be
     *            turned off.
     * @throws Exception
     */
    private void turnOffTestObjectSecurityFlags(RPMObject testObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(testObject, flags, Boolean.FALSE);
    }

    /**
     * Turns on certain flags in the test object security role.
     * 
     * @param testObject
     *            The test object.
     * @param flags
     *            The flags in test object security role which needs to be
     *            turned on.
     * @throws Exception
     */
    private void turnOnTestObjectSecurityFlags(RPMObject testObject,
            ArrayList flags) throws Exception
    {
        switchSecurityFlags(testObject, flags, Boolean.TRUE);
    }

    /**
     * Switches the security flags.
     * 
     * @param rpmObject
     *            The object.
     * @param flags
     *            The flags need to be switched
     * @param value
     *            On or Off value
     * @throws Exception
     */
    private void switchSecurityFlags(RPMObject rpmObject, ArrayList flags,
            Boolean value) throws Exception
    {
        if (rpmObject == null || flags == null)
        {
            return;
        }

        SecurityRole securityRole = null;

        if (rpmObject instanceof WorkElement)
        {
            securityRole = (AbstractWbsSecurityRole) ((WorkElement) rpmObject)
                .getResourceTaskAssignments()[0].getSecurityRole();
        }
        else if (rpmObject instanceof DocumentFolder)
        {
            securityRole = (FolderSecurityRole) ((DocumentFolder) rpmObject)
                .getResourceAssignments()[0].getSecurityRole();
        }
        else if (rpmObject instanceof ScopeFolder)
        {
            securityRole = (FolderSecurityRole) ((ScopeFolder) rpmObject)
                .getResourceTaskAssignments()[0].getSecurityRole();
        }
        else if (rpmObject instanceof NoteMinutes)
        {
            securityRole = (NotesMinutesSecurityRole) ((NoteMinutes) rpmObject)
                .getResourceAssignments()[0].getSecurityRole();
        }

        if (securityRole == null)
        {
            fail("SecurityRole is null");
        }

        for (int i = 0; i != flags.size(); ++i)
        {
            securityRole.getClass().getMethod(SET + flags.get(i), new Class[]
            {
                Boolean.class
            }).invoke(securityRole, new Object[]
            {
                value
            });
        }
        saveWithoutDelete(securityRole, null);
    }

    /**
     * Turns on the test security flag.
     * 
     * @param testObject
     *            The test object.
     * @param testObjectSecurityRole
     *            The security role of the test object.
     * @param flagName
     *            The name of the flag.
     * @throws Exception
     */
    protected void turnOnTestFlag(RPMObject testObject,
            SecurityRole testObjectSecurityRole, String flagName)
            throws Exception
    {
        switchTestFlag(testObject, testObjectSecurityRole, flagName,
            Boolean.TRUE);
    }

    /**
     * Turns off the test security flag.
     * 
     * @param testObject
     *            The test object.
     * @param testObjectSecurityRole
     *            The security role of the test object.
     * @param flagName
     *            The name of the flag.
     * @throws Exception
     */
    protected void turnOffTestFlag(RPMObject testObject,
            SecurityRole testObjectSecurityRole, String flagName)
            throws Exception
    {
        switchTestFlag(testObject, testObjectSecurityRole, flagName,
            Boolean.FALSE);
    }

    /**
     * Switches the test flag.
     * 
     * @param testObject
     *            The test object.
     * @param testObjectSecurityRole
     *            The security role of the test object.
     * @param flagName
     *            The name of the flag.
     * @param value
     *            On or Off value.
     * @throws Exception
     */
    private void switchTestFlag(RPMObject testObject,
            SecurityRole testObjectSecurityRole, String flagName, Boolean value)
            throws Exception
    {
        testObjectSecurityRole = getSecurityRole(testObject);

        testObjectSecurityRole.getClass().getMethod(SET + flagName, new Class[]
        {
            Boolean.class
        }).invoke(testObjectSecurityRole, new Object[]
        {
            value
        });
        testObjectSecurityRole = updateSecurityRole(testObjectSecurityRole);
    }

    /**
     * Verify the test must fail for CanEdit cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForEdit() throws Exception
    {
    	fieldValue = getValue(testObject, GET, fieldToTest, null);
        
    	fieldValue = changeArgValue(fieldValue);

        setValues(testObject, SET, fieldToTest, new Object[] {fieldValue} );
        
        saveAndValidate(testObject, testObjectScope, false);
    }

    /**
     * Verify the test must success for CanEdit cases.
     * 
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForEdit() throws Exception
    {
    	setValues(testObject, SET, fieldToTest, new Object[] {fieldValue} );
        
        saveAndValidate(testObject, testObjectScope, true);
    }

    /**
     * Verify the test must fail for CanView cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForView() throws Exception
    {
        testObject = loadFromID(testObject, testObjectScope);

        verifyNullValue();
    }

    /**
     * Verify the test must success for CanView cases.
     * 
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForView() throws Exception
    {
        testObject = loadFromID(testObject, testObjectScope);

        verifyNotNullValue();
    }

    /**
     * Verify that fieldToTest or fieldsToTest values are not null.
     * 
     * @throws Exception
     */
    private void verifyNotNullValue() throws Exception
    {
    	if (fieldToTest != null)
        {
            verifyNotNullFieldValue(fieldToTest);
        }
        else
        {
            if (fieldsToTest != null && fieldsToTest.size() > 0)
            {
                verifyNotNullFieldsValues();
            }
            else
            {
                fail("No field specified.");
            }
        }
    }
    
    /**
     * Verify that filedToTest or fieldsToTest values are null.
     * 
     * @throws Exception
     */
    private void verifyNullValue() throws Exception
    {
    	if (fieldToTest != null)
        {
            verifyNullFieldValue(fieldToTest);
        }
        else
        {
            if (fieldsToTest != null)
            {
                verifyNullFieldsValues();
            }
        }
    }
    /**
     * Verify the given field must be null.
     * 
     * @param fieldToTest
     *            The field name which we are testing.
     * @throws Exception
     */
    private void verifyNullFieldValue(String fieldToTest) throws Exception
    {
    	Object nullValue = getValue(testObject, GET, fieldToTest, null);
        assertNull(nullValue);
    }

    /**
     * Verify the given field must not be null.
     * 
     * @param fieldToTest
     *            The field name which we are testing.
     * @throws Exception
     */
    private void verifyNotNullFieldValue(String fieldToTest) throws Exception
    {
    	Object notNullValue = getValue(testObject, GET, fieldToTest, null);
        
        assertNotNull(notNullValue);
    }

    /**
     * Verify the test fields must be null.
     * 
     * @throws Exception
     */
    private void verifyNullFieldsValues() throws Exception
    {
        for (int i = 0; i != fieldsToTest.size(); ++i)
        {
            verifyNullFieldValue((String) fieldsToTest.get(i));
        }
    }

    /**
     * Verify the test fields must be null.
     * 
     * @throws Exception
     */
    private void verifyNotNullFieldsValues() throws Exception
    {
        for (int i = 0; i != fieldsToTest.size(); ++i)
        {
            verifyNotNullFieldValue((String) fieldsToTest.get(i));
        }
    }

    /**
     * Verify the test must fail for CanRemove cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForRemove() throws Exception
    {
    	fieldValue = getValue(testObject, GET, fieldToTest, null);
    	
    	if(CAN_REMOVE_RESOURCE_PARTICIPANT.equals(flagName))
    	{
    		boolean found = false;
    		for(int i=0;!found && i < ((Object[]) fieldValue).length; i++)
    		{
    			if (((ResourceTaskAssignment)((Object[]) fieldValue)[i]).getResource() == null || !((ResourceTaskAssignment)((Object[]) fieldValue)[i]).getResource().getID().equals(
    					getCurrentUser().getID()))
    			{
    				fieldValue = ((Object[]) fieldValue)[i];
    				found = true;
    			}
    		}
    		assertTrue("ResourceTaskAssignment to Delete is not found",found);

    	}
    	else
    	{
    		if (fieldValue.getClass().isArray())
    		{
    			fieldValue = ((Object[]) fieldValue)[0];
    		}
    	}
        if(fieldValue instanceof Duplicate)
        {
        	((Duplicate)fieldValue).setDeleted(Boolean.TRUE);
        	((AbstractScope)testObject).setDuplicates(new Duplicate[]{(Duplicate)fieldValue});
        	saveAndValidate((RPMObject)testObject, testObjectScope, false);
        }
        else
        {	
        	deleteAndValidate((RPMObject) fieldValue, false);
        }
    }
    
    /**
     * Verify the test must success for CanRemove cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForRemove() throws Exception
    {
    	 if(fieldValue instanceof Duplicate)
         {
    		((Duplicate)fieldValue).setDeleted(Boolean.TRUE);
         	((AbstractScope)testObject).setDuplicates(new Duplicate[]{(Duplicate)fieldValue});
         	saveAndValidate((RPMObject)testObject, testObjectScope, true);
         }
         else
         {	
        	 deleteAndValidate((RPMObject) fieldValue, true);
         }
    }
    
    /**
     * Verify that test must fail for CanCreate cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForCreate() throws Exception
    {
    	verifyCreateScenario(false);
    }

    /**
     * Verify that test muss success for CanCreate cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForCreate() throws Exception
    {
    	verifyCreateScenario(true);
    }
    
    private void verifyCreateScenario(boolean canSave) throws Exception
    {
    	assertNull("Object" + testObject.getClass().getName() + "is already created",testObject.getID());
    	
    	saveAndValidate(testObject, testObjectScope, canSave);
    }

    /**
     * Verify that test must fail for CanAssign cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForAssign() throws Exception
    {
    	saveAndValidate(testObject,testObjectScope, false);
    }
    
    /**
     * Verify that test must success for CanAssign cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForAssign() throws Exception
    {
    	saveAndValidate(testObject, testObjectScope, true);
    }
    
    protected void assign(String assignValue) throws Exception
    {
    	ResourceScope resourceScope = new ResourceScope();
    	resourceScope.setCalendar(new CalendarScope());
    	Resource resource = ResourceFixture.createSaveFixture(resourceScope, this);
    	addObjectToDelete(resource);
    	
    	ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
    	SecurityRole securityRole = null;
    	ResourceRoleAssignment resourceRoleAssignment = null;
    	if(RESOURCE.equals(assignValue))
    	{
    		rtaScope.setTaskAssignment(new TaskAssignmentScope());
    	}
    	else if(PARTICIPANT.equals(assignValue))
    	{
    		securityRole = SecurityRoleFixture.createParticipantSecurityRole(testObject);
    	}
    	else if(PROFILE.equals(assignValue))
    	{
    		// TODO: Add profiles for AssignProfile and RemoveProfile flags
    	}
    	
    	if(DocumentElement.class.isAssignableFrom(testObject.getClass()))
    	{
    		resourceRoleAssignment = ResourceRoleAssignmentFixture.createFixture(resource, testObject, securityRole);
    		Object oldResourceRoleAssignments = getValue(testObject, GET, fieldToTest, null);
    		ResourceRoleAssignment oldResourceRoleAssignment = (ResourceRoleAssignment)((Object[])oldResourceRoleAssignments)[0];
    		ResourceRoleAssignment resourceRoleAssignments[] = new ResourceRoleAssignment[]{oldResourceRoleAssignment , resourceRoleAssignment};
    		setValues(testObject, SET, fieldToTest, new Object[] {resourceRoleAssignments} );
    		
    		testObjectScope.getClass().getMethod(
        			SET + fieldToTest,
    				new Class[]
    				          {
        					testObjectScope.getClass().getMethod(IS + fieldToTest, null)
    						.getReturnType()
    				          }).invoke(testObjectScope, new Object[] {Boolean.TRUE});
    	}
    	else if(WorkElement.class.isAssignableFrom(testObject.getClass()) || ScopeElement.class.isAssignableFrom(testObject.getClass()))
    	{
    		resourceRoleAssignment = ResourceTaskAssignmentFixture.createFixture(testObject, resource, null, rtaScope, false, this);
    		if(securityRole != null)
        	{
        		resourceRoleAssignment.setSecurityRole(securityRole);
        	}
    		Object oldRtas = getValue(testObject, GET, fieldToTest, null);
    		ResourceTaskAssignment oldRta = (ResourceTaskAssignment)((Object[])oldRtas)[0];
        	ResourceTaskAssignment rtas[] = new ResourceTaskAssignment[]{oldRta , (ResourceTaskAssignment)resourceRoleAssignment};
        	setValues(testObject, SET, fieldToTest, new Object[] {rtas} );
        	setValues(testObjectScope, SET, fieldToTest, new Object[] {rtaScope});
    	}
    }
    
    private void setValues(Object object, String methodHandler, String fieldName, Object[] parameterValues) throws Exception
    {
    	object.getClass().getMethod(
    			methodHandler + fieldName,
				new Class[]
				          {
    					object.getClass().getMethod(GET + fieldName, null)
						.getReturnType()
				          }).invoke(object, parameterValues);
    }
    
    private Object getValue(Object object, String methodHandler, String fieldName, Object[] parameterValues) throws Exception
    {
    	 return object.getClass().getMethod(methodHandler + fieldName, null)
         .invoke(object, parameterValues);
    }
    
    protected void tearDown() throws Exception
    {
        super.tearDown();
        // turn on parent flags if needed
        if (grandparentRequiredOffFlags != null)
        {
            turnOnGrandparentSecurityFlags(grandparentObject,
                grandparentRequiredOffFlags);
        }
        if (parentRequiredOffFlags != null)
        {
            turnOnParentSecurityFlags(parentObject, parentRequiredOffFlags);
        }
        flagOn(testObject, flagName);
    }
    /**
     * Verify the test must fail for CanDelete cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForDelete() throws Exception
    {
    	deleteAndValidate(testObject, false);
    }
    /**
     * Verify the test must Succeed for CanDelete cases.
     * 
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForDelete() throws Exception
    {
    	deleteAndValidate(testObject, true);
    }
    
    /**
     * Change The State To Closed.
     * 
     * @param DatafieldState Object
     *            The object for which we want to change the state to closed.
     * @return the new object which has been changed
     * @throws Exception
     */
    protected DatafieldState changeStateToClosed(DatafieldState datafieldState) throws Exception
    {
    	String value = datafieldState.getID();

		if (datafieldState instanceof State) {
			value = DatafieldFixture.PROJECT_STATE_CLOSED;
		} else if(datafieldState instanceof TaskState ){
			value = DatafieldFixture.TASK_STATE_CLOSED;
		}else if(datafieldState instanceof DeliverableState ){
			value = DatafieldFixture.DELIVERABLE_STATE_CLOSED;
		} else if (datafieldState instanceof ActionState) {
			value = DatafieldFixture.ACTION_STATE_CLOSED;
		} else if (datafieldState instanceof ChangeRequestState) {
			value = DatafieldFixture.CHANGEREQUEST_STATE_CLOSED;
		} else if (datafieldState instanceof DefectState) {
			value = DatafieldFixture.DEFECT_STATE_CLOSED;
		} else if (datafieldState instanceof IssueState) {
			value = DatafieldFixture.ISSUE_STATE_CLOSED;
		} else if (datafieldState instanceof RequirementState) {
			value = DatafieldFixture.REQUIREMENT_STATE_CLOSED;
		} else if (datafieldState instanceof RiskState) {
			value = DatafieldFixture.RISK_STATE_CLOSED;
		} else if (datafieldState instanceof ServiceRequestState) {
			value = DatafieldFixture.SERVICEREQUEST_STATE_CLOSED;
		}
		datafieldState.setID(value);
    	datafieldState.setValue(value);
    	datafieldState.setClosedState(Boolean.TRUE);
    	return datafieldState;
    }
    
    protected void verifyMustFailScenarioForClose() throws Exception
    {
    	fieldValue = getValue(testObject, GET, fieldToTest, null);
        assertTrue(fieldValue instanceof DatafieldState);
        DatafieldState datafieldState = (DatafieldState)fieldValue;
        fieldValue = changeStateToClosed(datafieldState);
        setValues(testObject, SET, fieldToTest,  new Object[]{fieldValue});
        saveAndValidate(testObject, testObjectScope, false);
    }
    
    protected void verifyMustSuccessScenarioForClose() throws Exception
    {
    	setValues(testObject,SET, fieldToTest, new Object[]{fieldValue});
        saveAndValidate(testObject, testObjectScope, true);
    }

    /**
     * Verify that test must fail for CanAdd cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForAdd() throws Exception
    {
    	addElementForAddRemove();
    	saveAndValidate(testObject,testObjectScope,false);
    }

    /**
     * Verify that test muss success for CanAdd cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForAdd() throws Exception
    {
    	testObject = saveAndValidate(testObject,testObjectScope,true);
    	fieldValue = getValue(testObject, GET, fieldToTest, null);
    	assertNotNull(fieldValue);
	    if (fieldValue.getClass().isArray())
	    {
	        fieldValue = ((Object[]) fieldValue)[0];
	        assertNotNull(fieldValue);
	    }    
    }
    
    private void addElementForAddRemove() throws Exception
    {
    	if(DEPENDENCIES.equals(fieldToTest))
    	{
    		ElementDependency successor_dependency = DependencyFixture
    		.createFixture((WorkElement)testObject, new ElementDependencyScope()
    		, this);
    		ElementDependency predecessor_dependency = DependencyFixture
    		.createFixture((WorkElement)testObject, new ElementDependencyScope(), this);
    		((WorkElement)testObject).setDependencies(new ElementDependency[]
    		                                                                {
    				successor_dependency, predecessor_dependency
    		                                                                });
    		((WorkElementScope)testObjectScope).setDependencies(new ElementDependencyScope());

    	}else if(ATTRIBUTE_ASSIGNMENTS.equals(fieldToTest))
    	{  
    		AttributeAssignment attributeAssignment = AttributeAssignmentFixture.createAssignmentFixture(testObject, this);
    		AttributeAssignment[] attributeAssignments = new AttributeAssignment[] {attributeAssignment};
    		setValues(testObject, SET, fieldToTest, new Object[] {attributeAssignments});
            testObjectScope.getClass().getMethod(
                    SET + fieldToTest ,
                    new Class[]
                    {
                        testObjectScope.getClass().getMethod(IS + fieldToTest, null)
                            .getReturnType()
                    }).invoke(testObjectScope, new Object[]
                {
                    		Boolean.TRUE
                });

    	}else if(TIMECODE_ASSIGNMENTS.equals(fieldToTest) && WorkElement.class.isAssignableFrom(testObject.getClass()))
    	{
    		TimeCodeAssignment timeCodeAssigns1 = TimeCodeAssignmentFixture
    		.createFixture((WorkElement)testObject, true, this);
    		TimeCodeAssignment timeCodeAssigns2 = TimeCodeAssignmentFixture
    		.createFixture((WorkElement)testObject, false, this);
    		((WorkElement)testObject).setTimeCodeAssignments(new TimeCodeAssignment[]
    		                                                                        {
    				timeCodeAssigns1, timeCodeAssigns2
    		                                                                        });
    		((WorkElementScope)testObjectScope).setTimeCodeAssignments(true);
    		    		
    	}else if(DUPLICATES.equalsIgnoreCase(fieldToTest) && AbstractScope.class.isAssignableFrom(testObject.getClass()))
    	{
    		AbstractScope scopeElement = (AbstractScope)processScopeElements((ScopeElement)testObject, (ScopeElement)parentObject, null);
    		scopeElement = (AbstractScope)save(scopeElement, null).getRpmObject();
    		assertNotNull(scopeElement);
    		Duplicate duplicate = new Duplicate();
    		duplicate.setParent(testObject);
    		duplicate.setScopeElement(scopeElement);
    		((AbstractScope)testObject).setDuplicates(new Duplicate[]{duplicate});
    		((ScopeElementScope)testObjectScope).setDuplicates(true);
    	}
    }
    /**
     * Verify the test must fail for CanTransferUnderWBS cases.
     * 
     * @throws Exception
     */
    protected void verifyMustFailScenarioForTransfer() throws Exception
    {
    	transferUnderWBS((AbstractScope)testObject, false);
    }
    
    /**
     * Verify the test must success for CanTransferUnderWBS cases.
     * 
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForTransfer() throws Exception
    {
   		transferUnderWBS((AbstractScope)testObject, true);
    }

}
