package com.ibm.rpm.client.junit.security.testcases.role;

import java.util.ArrayList;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimePhasedFinancialFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.security.containers.AbstractDeliverableSecurityRole;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.TaskType;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;

public class TestProjectSecurityLocal extends BaseRoleSecurityTestLocal
{

    ProjectSecurityRole projectSecurityRole;
    WorkElementScope projectScope;
    GenericProject testProject;
    
    protected void setUp() throws Exception
    {
        super.setUp();
        /* Prepare global variables */
        projectSecurityRole = (ProjectSecurityRole) prepareSecurityRole(Project.class, true);
        projectScope = new WorkElementScope();
        
      projectSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE); 
      projectSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE); 
      updateProjectSecurityRole();
      
      /* Prepare test environment*/
      testProject = (GenericProject) prepareContainer(new Project(),
          null, projectScope, securityTestUser, projectSecurityRole);

    }
    
    protected void tearDown() throws Exception
    {
        if (projectSecurityRole != null)
        {
            projectSecurityRole.setAllDocumentPermissionsGranted(Boolean.TRUE);
            projectSecurityRole
                .setAllScopeElementsPermissionsGranted(Boolean.TRUE);
            updateProjectSecurityRole();
        }

        super.tearDown();
    }
    

    protected void updateProjectSecurityRole() throws Exception
    {
        projectSecurityRole = (ProjectSecurityRole) updateSecurityRole(projectSecurityRole);            
    }
    
    protected SecurityRole updateWorkElementSecurityRole(SecurityRole securityRole) throws Exception
    {
        return securityRole = (SecurityRole) updateSecurityRole(securityRole);
    }
    
    
 
    
    public void testEditProjectAssignmentType() throws Exception
    {
        /* Remove the right to edit */
        projectSecurityRole.setCanEditAssignmentType(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setAssignmentType(AssignmentType.Duration);
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanEditAssignmentType(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectCustomField() throws Exception
    {
        /* Prepare test environment*/
        projectScope.setCustomFieldAssignments(true);
        
        /* Remove the right to edit */
        Boolean[] values = new Boolean[10];
        for (int i = 0; i < 10; i++)
            values[i] = Boolean.FALSE;
        projectSecurityRole.setCanEditCustomFieldsGroups(values);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        CustomFieldAssignment[] assignments = {new CustomFieldAssignment()};
        assignments[0].setCustomField(CustomFieldFixture.createSaveCustomField(
            CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, FieldInputType.Text, new Integer(1), this));
        assignments[0].setValue(assignments[0].getValue()+"_MOD");
        testProject.setCustomFieldAssignments(assignments);
        saveAndValidate(testProject, projectScope, false);
        
        /* Give the right to edit */
        for (int i = 0; i < 10; i++)
            values[i] = Boolean.TRUE;
        projectSecurityRole.setCanEditCustomFieldsGroups(values);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, projectScope, true);
    }

    public void testEditProjectDefault() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole.setCanSetAsDefaultProject(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setDefaultProject(new Boolean(
            !testProject.getDefaultProject().booleanValue()));
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanSetAsDefaultProject(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectName() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole.setCanRenameWBSItems(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setName(testProject.getName()+"_MOD");
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanRenameWBSItems(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectPriority() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole.setCanEditWBSPriority(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setPriority(new Integer(testProject.getPriority().intValue()+2));
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanEditWBSPriority(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectProbabilityPercent() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole.setCanEditPursuit(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setProbabilityPercent(new Integer(
            testProject.getProbabilityPercent().intValue()+2));
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanEditPursuit(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectPublished() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole.setCanPublishSecurity(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setPublished(ProjectPublishedType.ToTeam);
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanPublishSecurity(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectReferenceNumber() throws Exception
    {
        /* Remove the right to edit */
        projectSecurityRole.setCanEditReferenceNumber(Boolean.FALSE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */        
        // preventing checkpoint from generating errors if > 19.
        if (testProject.getReferenceNumber().length() < 15)
        {
            testProject.setReferenceNumber(testProject.getReferenceNumber()+"_MOD");
        }
        else
        {
            testProject.setReferenceNumber("ReferenceNumberTT");
        }

        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanEditReferenceNumber(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, null, true);
    }

    public void testEditProjectRTF() throws Exception
    {
        /* Prepare test environment*/
        projectScope.setRtfAssignments(true);
        testProject = (GenericProject) prepareContainer(new Project(),
            null, projectScope, securityTestUser, projectSecurityRole);
        
        /* Remove the right to edit */
        Boolean[] values = new Boolean[20];
        for (int i = 0; i < 20; i++)
            values[i] = Boolean.FALSE;
        projectSecurityRole.setCanEditProjectRTFs(values);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        RtfAssignment[] assignments = testProject.getRtfAssignments();
        if (assignments == null)
        {
            DatafieldRTF rtf = (DatafieldRTF) loadOneFromXpath("/ProjectRTF[name='1']", null);
            assignments = new RtfAssignment[] {
                RtfAssignmentFixture.createAssignmentFixture(testProject, "1", rtf, this)};
        }
        assignments[0].setValue(assignments[0].getValue()+"_MOD");
        testProject.setRtfAssignments(assignments);
        saveAndValidate(testProject, projectScope, false);
        
        /* Give the right to edit */
        for (int i = 0; i < 20; i++)
            values[i] = Boolean.TRUE;
        projectSecurityRole.setCanEditProjectRTFs(values);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        saveAndValidate(testProject, projectScope, true);
    }
    
    
    
    
    
    
    
    
    
    
    
    public void testcanViewAssignmentType() throws Exception
    {

        /* Remove the right to view */
        projectSecurityRole.setCanViewAssignmentType(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getAssignmentType());

        /* Give the right to view */
        projectSecurityRole.setCanViewAssignmentType(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getAssignmentType());
    }
    
    public void testcanViewProjectCalendar() throws Exception
    {

        /* Remove the right to view */
        projectSecurityRole.setCanViewProjectCalendar(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getCalendar());

        /* Give the right to view */
        projectSecurityRole.setCanViewProjectCalendar(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getCalendar());
    }
    
    
    public void testcanViewCurrency() throws Exception
    {

        /* Remove the right to view */
        projectSecurityRole.setCanViewCurrency(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getCurrency());

        /* Give the right to view */
        projectSecurityRole.setCanViewCurrency(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getCurrency());
    }    
    
    public void testcanViewBudgetCostRevenue_WBS() throws Exception
    {
        // Part 1: WorksheetFinancial
        // Part 2: WbsFinancial
        // Part 3: TimePhased (WBS + WORKSHEET)

        /* Prepare test environment */

        TimePhasedFinancialScope tscope = new TimePhasedFinancialScope();
        tscope.setState(true);

        WbsFinancial wbs = new WbsFinancial();
        wbs.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            TimePhasedFinancialFixture.createWbsFixture(tscope, this)
        });
        wbs
            .setChargeCode(ChargeCodeFixture
                .createAndSaveSimpleChargeCode(this));

        testProject.setWbsFinancials(new WbsFinancial[]
        {
            wbs
        });
        projectScope.setWbsFinancials(FinancialFixture.createAllScope());

        SingleResult rs = save(testProject, projectScope);
        assertNotNull(rs.getRpmObject());
        assertTrue(rs.isSuccessful());

        testProject = (Project) rs.getRpmObject();

        /* Remove the right to view */
        projectSecurityRole
            .setCanViewBudgetCostRevenueFinancialsTimePhasing(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getWbsFinancials()[0].getEstimateTimePhased());

        /* Give the right to view */
        projectSecurityRole
            .setCanViewBudgetCostRevenueFinancialsTimePhasing(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getWbsFinancials()[0].getEstimateTimePhased());
    }
   
    public void testcanViewBudgetCostRevenue_WORKSHEET() throws Exception
    {
        // Part 1: WorksheetFinancial
        // Part 2: WbsFinancial
        // Part 3: TimePhased (WBS + WORKSHEET)

        /* Prepare test environment */

        TimePhasedFinancialScope tscope = new TimePhasedFinancialScope();
        tscope.setState(true);

        WorksheetFinancial ws = new WorksheetFinancial();
        ws.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            TimePhasedFinancialFixture.createWorksheetFixture(tscope, this)
        });
        ws.setChargeCode(ChargeCodeFixture.createAndSaveSimpleChargeCode(this));

        testProject.setWorksheetFinancials(new WorksheetFinancial[]
        {
            ws
        });
        projectScope.setWorksheetFinancials(FinancialFixture.createAllScope());

        SingleResult rs = save(testProject, projectScope);
        assertNotNull(rs.getRpmObject());
        assertTrue(rs.isSuccessful());

        testProject = (Project) rs.getRpmObject();

        /* Remove the right to view */
        projectSecurityRole
            .setCanViewBudgetCostRevenueFinancialsTimePhasing(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getWorksheetFinancials()[0].getEstimateTimePhased());

        /* Give the right to view */
        projectSecurityRole
            .setCanViewBudgetCostRevenueFinancialsTimePhasing(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getWorksheetFinancials()[0]
            .getEstimateTimePhased());
    }



    
    //Classes to Test:
    //                  Project         Done
    //                  SummaryTask +
    //                  Deliverable     Done
    //                  WorkProduct +
    //                  Milestone       Done
    //                  Task            Done
    public void testCanCanViewTimeCodesWithWorkProduct() throws Exception
    {
        WorkElementScope allScope = new WorkElementScope();
        allScope.setTimeCodeAssignments(true);
        allScope.setContainingProject(new WorkElementScope());
        allScope.setParent(new WorkElementScope());
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            TaskSecurityRole.class, new WorkProduct(), new Proposal(),
        allScope, "TimeCodeAssignments", "CanViewTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewTimeCodes");
        test.runTest();
    }
    
    public void testCanCanViewTimeCodesWithSummaryTask() throws Exception
    {
        WorkElementScope allScope = new WorkElementScope();
        allScope.setTimeCodeAssignments(true);
        allScope.setContainingProject(new WorkElementScope());
        allScope.setParent(new WorkElementScope());
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            TaskSecurityRole.class, new SummaryTask(), new Proposal(),
        allScope, "TimeCodeAssignments", "CanViewTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewTimeCodes");
        test.runTest();
    }

    public void testcanViewTimeCodesAggregateNodeTask() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodeAssignments(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            TaskSecurityRole.class, new Task(), new Proposal(), projectScope,
            "TimeCodeAssignments", "CanViewTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewTimeCodes");
        test.runTest();

    }
    
    public void testcanViewTimeCodesAggregateNodeMileStone() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodeAssignments(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            SecurityRole.class, new Milestone(), new Proposal(), projectScope,
            "TimeCodeAssignments", "CanViewTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewTimeCodes");
        test.runTest();

    }

    public void testcanViewTimeCodesAggregateNodeDeliverable() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodeAssignments(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            DeliverableSecurityRole.class, new Deliverable(), new Proposal(), projectScope,
            "TimeCodeAssignments", "CanViewTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewTimeCodes");
        test.runTest();

    }
    
    public void testcanViewTimeCodesAggregateNodeProject() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodeAssignments(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            ProjectSecurityRole.class, new Project(), new Proposal(), projectScope,
            "TimeCodeAssignments", "CanViewTimeCodes");
        test.runTest();

    }


    //Classes to Test:
    //                  Defect----------Done
    //                  ActionItem--------Done
    //                  Requirement--------Done
    //                  ServiceRequest------Done
    //                  Risk +
    //                  ChangeRequest +
    //                  Issue +
    public void testCanViewRankWithRisk() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            RiskSecurityRole.class, new Risk(), new Proposal(), allScope,
            "Rank", "CanViewRank");
        
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewWBSPriority");
        test.runTest();
    }
    
    public void testCanViewRankWithChangeRequest() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ChangeRequestSecurityRole.class, new ChangeRequest(), new Proposal(), allScope,
            "Rank", "CanViewRank");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewWBSPriority");
        test.runTest();
    }

    public void testCanViewRankWithChangeIssue() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            IssueSecurityRole.class, new Issue(), new Proposal(), allScope,
            "Rank", "CanViewRank");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewWBSPriority");
        test.runTest();
    }

    public void testcanViewRank() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());

        DefectSecurityRole defectSecurityRole = (DefectSecurityRole) prepareSecurityRole(
            Defect.class, true);
        ActionItemSecurityRole actionItemSecurityRole = (ActionItemSecurityRole) prepareSecurityRole(
            Action.class, true);
        RequirementSecurityRole requirementSecurityRole = (RequirementSecurityRole) prepareSecurityRole(
            Requirement.class, true);
        ServiceRequestSecurityRole serviceRequestSecurityRole = (ServiceRequestSecurityRole) prepareSecurityRole(
            ServiceRequest.class, true);

        // -----ResourceTaskAssignment creations and assignment

        /* Prepare Parent test environment */
        testProject = (GenericProject) prepareContainer(new Project(), null,
            projectScope, securityTestUser, projectSecurityRole);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + testProject.getName() + "']", new ScopeElementScope());
        ScopeFolder folderParent = (ScopeFolder) rpmObjects[0];

        // Creating parents for containers...

        ScopeElementScope scopeAll = new ScopeElementScope();
        scopeAll.setParent(new ScopeElementScope());

        /* Prepare test environment */

        Defect testDefect = (Defect) prepareContainer(new Defect(),
            folderParent, allScope, securityTestUser, defectSecurityRole);

        Action testAction = (Action) prepareContainer(new Action(),
            folderParent, allScope, securityTestUser, actionItemSecurityRole);

        Requirement testRequirement = (Requirement) prepareContainer(
            new Requirement(), folderParent, allScope, securityTestUser,
            requirementSecurityRole);

        ServiceRequest testServiceRequest = (ServiceRequest) prepareContainer(
            new ServiceRequest(), folderParent, allScope, securityTestUser,
            serviceRequestSecurityRole);

        /* Remove the right to view */
        // --SuperFlags
        projectSecurityRole.setCanViewWBSPriority(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        defectSecurityRole.setCanViewRank(Boolean.FALSE);
        defectSecurityRole = (DefectSecurityRole) updateWorkElementSecurityRole(defectSecurityRole);

        actionItemSecurityRole.setCanViewRank(Boolean.FALSE);
        actionItemSecurityRole = (ActionItemSecurityRole) updateWorkElementSecurityRole(actionItemSecurityRole);

        requirementSecurityRole.setCanViewRank(Boolean.FALSE);
        requirementSecurityRole = (RequirementSecurityRole) updateWorkElementSecurityRole(requirementSecurityRole);

        serviceRequestSecurityRole.setCanViewRank(Boolean.FALSE);
        serviceRequestSecurityRole = (ServiceRequestSecurityRole) updateWorkElementSecurityRole(serviceRequestSecurityRole);

        /* Try to view the project, should fail */

        Defect reloadedDefect = (Defect) loadFromID(testDefect, allScope);
        assertNotNull(reloadedDefect);
        assertNull(reloadedDefect.getRank());

        Action reloadedAction = (Action) loadFromID(testAction, allScope);
        assertNotNull(reloadedAction);
        assertNull(reloadedAction.getRank());

        Requirement reloadedRequirement = (Requirement) loadFromID(
            testRequirement, allScope);
        assertNotNull(reloadedRequirement);
        assertNull(reloadedRequirement.getRank());

        ServiceRequest reloadedServiceRequest = (ServiceRequest) loadFromID(
            testServiceRequest, allScope);
        assertNotNull(reloadedServiceRequest);
        assertNull(reloadedServiceRequest.getRank());

        /* Give the right to view */

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        defectSecurityRole.setCanViewRank(Boolean.TRUE);
        defectSecurityRole = (DefectSecurityRole) updateWorkElementSecurityRole(defectSecurityRole);

        actionItemSecurityRole.setCanViewRank(Boolean.TRUE);
        actionItemSecurityRole = (ActionItemSecurityRole) updateWorkElementSecurityRole(actionItemSecurityRole);

        requirementSecurityRole.setCanViewRank(Boolean.TRUE);
        requirementSecurityRole = (RequirementSecurityRole) updateWorkElementSecurityRole(requirementSecurityRole);

        serviceRequestSecurityRole.setCanViewRank(Boolean.TRUE);
        serviceRequestSecurityRole = (ServiceRequestSecurityRole) updateWorkElementSecurityRole(serviceRequestSecurityRole);

        /* Try to view the project, should fail */
        reloadedDefect = (Defect) loadFromID(testDefect, allScope);
        assertNotNull(reloadedDefect);
        assertNotNull(reloadedDefect.getRank());

        reloadedAction = (Action) loadFromID(testAction, allScope);
        assertNotNull(reloadedAction);
        assertNotNull(reloadedAction.getRank());

        reloadedRequirement = (Requirement) loadFromID(testRequirement,
            allScope);
        assertNotNull(reloadedRequirement);
        assertNotNull(reloadedRequirement.getRank());

        reloadedServiceRequest = (ServiceRequest) loadFromID(
            testServiceRequest, allScope);
        assertNotNull(reloadedServiceRequest);
        assertNotNull(reloadedServiceRequest.getRank());

        // --SuperFlags
        projectSecurityRole.setCanViewWBSPriority(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);
    }
        
    
    /*GenericProject.planDate, GenericProject.containingProject, GenericProject.actualDate, GenericProject.referenceNumber, 
     * GenericProject.deliverableAttributeAssignments, GenericProject.changeRequestAttributeAssignments, 
     * GenericProject.referenceID, GenericProject.earnedValuesCalculationState, GenericProject.customFieldAssignments, 
     * GenericProject.workflowProcessesAvailable, GenericProject.documentFolder, GenericProject.assignmentType, 
     * GenericProject.closedByDate, GenericProject.issueScorecard, GenericProject.priority, GenericProject.rtfAssignments, 
     * GenericProject.statusUpdates, GenericProject.assignedScorecards, GenericProject.asset, GenericProject.initialBaselineDate,
     *  GenericProject.portfolio, GenericProject.earnedValues, GenericProject.parent, GenericProject.defaultSteps, 
     *  GenericProject.workflowProcessesRunning, GenericProject.externalID, GenericProject.organizationalAssignments, 
     *  GenericProject.documentAttributeAssignments, GenericProject.defaultProject, GenericProject.minutesPerDay, 
     *  GenericProject.actionScorecard, GenericProject.forecastDate, GenericProject.issueAttributeAssignments, 
     *  GenericProject.closedByName, GenericProject.timeCodeAssignments, GenericProject.scopeElement, GenericProject.state, 
     *  GenericProject.children, GenericProject.attributeAssignments, GenericProject.worksheetFinancials, 
     *  GenericProject.resourceTaskAssignments, GenericProject.expectedDate, GenericProject.assetCode, 
     *  GenericProject.estimatedFinishDate, GenericProject.probabilityPercent, GenericProject.currency, 
     *  GenericProject.dependencies, GenericProject.estimatedStartDate, GenericProject.timeCode2, GenericProject.name, 
     *  GenericProject.timeCode1, GenericProject.taskAttributeAssignments, GenericProject.actionAttributeAssignments,
     *   GenericProject.wbsFinancials, GenericProject.published, GenericProject.requirementScorecard, 
     *   GenericProject.serviceRequestScorecard, GenericProject.calendar, GenericProject.previousBaselineDate, 
     *   GenericProject.wbsSortingRank, GenericProject.serviceRequestAttributeAssignments, GenericProject.defectScorecard, 
     *   GenericProject.riskAttributeAssignments, GenericProject.defectAttributeAssignments, GenericProject.opportunity, 
     *   GenericProject.projectCostCenter, GenericProject.riskScorecard, GenericProject.templateSortingRank, 
     *   GenericProject.proposedDate, GenericProject.requirementAttributeAssignments, GenericProject.baselineDate, 
     *   GenericProject.lastModifiedDate, GenericProject.changeRequestScorecard, GenericProject.workflowRoleMappings*/ 
 public void testcanViewProjectDescription() throws Exception
    {

        FinancialScope fScope = new FinancialScope();
        fScope.setChargeCode(true);

        projectScope.setChildren(new WorkElementScope());
        projectScope.setAssignedScorecards(new ScorecardScope());
        projectScope.setScopeElement(new ScopeElementScope());
        projectScope.setOpportunity(true);
        projectScope.setTimeCodes(true);
        projectScope.setWorkflowRoleMappings(new WorkflowRoleMappingScope());

        Project project = new Project();
        project.setChildren(new WorkElement[]
        {
            new Project()
        });
        project.setScopeElement(new Defect());
        project.setOpportunity(new Opportunity());
        project.setTimeCode1(new TimeCode());
        project.setTimeCode2(new TimeCode());
        project.setWorkflowRoleMappings(new WorkflowRoleMapping[]
        {
            new WorkflowRoleMapping()
        });

        project = (Project) WorkElementFixture.createSaveFixture(null, project,
            projectScope, this);

        /* Prepare test environment*/
        testProject = (GenericProject) prepareContainer(project, null,
            projectScope, securityTestUser, projectSecurityRole);

        /* Remove the right to view */
        projectSecurityRole.setCanViewProjectDescription(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(testProject,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getCurrency());
        assertNull(reloaded.getReferenceID());

        assertNull(reloaded.getDefaultProject());
        assertNull(reloaded.getEstimatedFinishDate());
        assertNull(reloaded.getEstimatedStartDate());
        assertNull(reloaded.getLastModifiedDate());
        assertNull(reloaded.getMinutesPerDay());
        assertNull(reloaded.getPriority());
        assertNull(reloaded.getProbabilityPercent());
        assertNull(reloaded.getPublished());
        assertNull(reloaded.getResourceTaskAssignments());
        assertNull(reloaded.getWbsSortingRank());

        assertNull(reloaded.getChildren());
        assertNull(reloaded.getOpportunity());
        assertNull(reloaded.getTimeCode1());
        assertNull(reloaded.getTimeCode2());
        assertNull(reloaded.getWorkflowRoleMappings());

        /* Give the right to view */
        projectSecurityRole.setCanViewProjectDescription(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(testProject, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getCurrency());
        assertNotNull(reloaded.getReferenceID());

        assertNotNull(reloaded.getDefaultProject());
        assertNotNull(reloaded.getEstimatedFinishDate());
        assertNotNull(reloaded.getEstimatedStartDate());
        assertNotNull(reloaded.getLastModifiedDate());
        assertNotNull(reloaded.getMinutesPerDay());
        assertNotNull(reloaded.getPriority());
        assertNotNull(reloaded.getProbabilityPercent());
        assertNotNull(reloaded.getPublished());
        assertNotNull(reloaded.getResourceTaskAssignments());
        assertNotNull(reloaded.getWbsSortingRank());

        assertNotNull(reloaded.getChildren());
        assertNotNull(reloaded.getOpportunity());
        assertNotNull(reloaded.getTimeCode1());
        assertNotNull(reloaded.getTimeCode2());
        assertNotNull(reloaded.getWorkflowRoleMappings());

    }

 
 //Classes to Test:
 //                     Project     Done
 //                     Deliverable
 //                     Task
 
 public void testCanDeletePublicScopeElements() throws Exception
    {
        // Project, Task, Deliverable <-----------------|
        // ScopeFolder.deleted

        // -----Creating parent project
        Project project = new Project();
        project = (Project) WorkElementFixture.createSaveFixture(project,
            new WorkElementScope(), this);
        project.setScopeElement(new Defect());

        WorkElementScope allScope = new WorkElementScope();
        allScope.setContainingProject(new WorkElementScope());
        allScope.setParent(new WorkElementScope());

        ScopeElementScope parentFolderScope = new ScopeElementScope();
        parentFolderScope.setChildren(new ScopeElementScope());

        ScopeFolder parentFolder = null;

        /* Prepare Project test environment */
        testProject = (GenericProject) prepareContainer(project, null,
            allScope, securityTestUser, projectSecurityRole);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + testProject.getName() + "']", new ScopeElementScope());
        parentFolder = (ScopeFolder) rpmObjects[0];

        ScopeFolder childProjectFolder = new ScopeFolder();
        childProjectFolder.setName("SecurityFolder");
        parentFolder.setChildren(new ScopeElement[]
        {
            childProjectFolder
        });

        SingleResult rs = save(parentFolder, parentFolderScope);
        assertNotNull(rs.getRpmObject());
        assertTrue(rs.isSuccessful());
        childProjectFolder = (ScopeFolder) ((ScopeFolder) rs.getRpmObject())
            .getChildren()[0];

        /* Remove the right to view */
        //Super Flag
        projectSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        projectSecurityRole.setCanDeleteWBSItems(Boolean.FALSE);
        projectSecurityRole.setCanDeletePublicScopeElements(Boolean.FALSE);
        projectSecurityRole.setCanPermanentlyDeleteDocuments(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        deleteAndValidate(childProjectFolder, false);

        /* Give the right to view */

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        projectSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
        projectSecurityRole.setCanDeletePublicScopeElements(Boolean.TRUE);
        projectSecurityRole.setCanPermanentlyDeleteDocuments(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should pass */
        deleteAndValidate(childProjectFolder, true);
        
        //Super Flag
        projectSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

    }

 
 public void testCanViewAssignedOrganizations() throws Exception
    {
        Project project = null;

        /* Prepare test environment */
        project = (Project) prepareContainer(new Project(), null, projectScope,
            securityTestUser, projectSecurityRole);

        projectScope.setOrganizationalAssignments(true);

        //we need to create a projectOrganizationalAssignment object and the
        //required Organizational.
        ProjectOrganizationalAssignment orgz = new ProjectOrganizationalAssignment();
        orgz.setParent(project);
        orgz.setOrganizationType(DatafieldFixture
            .createOrganizationType(DatafieldFixture.ORGANIZATION_TYPE_OWNING));

        Organizational orgzchild = OrganizationalFixture.createFixture(1,
            new OrganizationalScope());
        
       

        SingleResult result = null;
        result = save(orgzchild, new OrganizationalScope());
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Organizational);

        orgzchild = (Organizational) result.getRpmObject();

        orgz.setOrganizational(orgzchild);
        project
            .setOrganizationalAssignments(new ProjectOrganizationalAssignment[]
            {
                orgz
            });

        result = null;
        result = save(project, projectScope);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Project);

        project = (Project) result.getRpmObject();

        /* Remove the right to view */
        projectSecurityRole.setCanViewAssignedOrganizations(Boolean.FALSE);
        projectSecurityRole.setCanViewProjectDescription(Boolean.FALSE);

        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should fail */
        GenericProject reloaded = (GenericProject) loadFromID(project,
            projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getOrganizationalAssignments());
        
 
        /* Give the right to view */
        projectSecurityRole.setCanViewAssignedOrganizations(Boolean.TRUE);
        projectSecurityRole.setCanViewProjectDescription(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        /* Try to view the project, should succeed */
        reloaded = (GenericProject) loadFromID(project, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getOrganizationalAssignments());
        
        ProjectOrganizationalAssignment poa2[] = (ProjectOrganizationalAssignment[])reloaded.getOrganizationalAssignments();
        assertNotNull(poa2[0].getOrganizational());
        assertNotNull(poa2[0].getOrganizationType());

    }

 
 
 public void testEditAssignmentType() throws Exception
 {
     /* Prepare test environment*/
     
     /* Remove the right to edit */
     projectSecurityRole.setCanEditAssignmentType(Boolean.FALSE);
     updateProjectSecurityRole();
     
     /* Try to modify the project, should fail */
     testProject.setAssignmentType(AssignmentType.Duration);
     saveAndValidate(testProject, null, false);
     
     /* Give the right to edit */
     projectSecurityRole.setCanEditAssignmentType(Boolean.TRUE);
     updateProjectSecurityRole();
     
     /* Try to modify the project, should succeed */
     testProject.setAssignmentType(AssignmentType.Duration);
     saveAndValidate(testProject, null, true);
 }

 
//Classes to Test:
 //                     DocumentFolder     
 //                     ScopeFolder         Done

 public void testCanPublishRenderPrivateFolder() throws Exception
    {
        // -----Creating parent project
        WorkElementScope allScope = new WorkElementScope();
        allScope.setContainingProject(new WorkElementScope());
        allScope.setParent(new WorkElementScope());

        // Scope Folder
        ScopeElementScope parentFolderScope = new ScopeElementScope();
        parentFolderScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        parentFolderScope.setChildren(parentFolderScope);
        ScopeFolder parentSFolder = null;

        /* Prepare Project test environment */
        testProject = (GenericProject) prepareContainer(new Project(), null,
            allScope, securityTestUser, projectSecurityRole);

        // ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + testProject.getName() + "']", new ScopeElementScope());
        parentSFolder = (ScopeFolder) rpmObjects[0];

        ScopeFolder childSFolder = new ScopeFolder();
        childSFolder.setName("SecurityFolder");
        parentSFolder.setChildren(new ScopeElement[]
        {
            childSFolder
        });

        SingleResult rs = save(parentSFolder, parentFolderScope);
        assertNotNull(rs.getRpmObject());
        assertTrue(rs.isSuccessful());
        childSFolder = (ScopeFolder) ((ScopeFolder) rs.getRpmObject())
            .getChildren()[0];

        /* Remove the right to view */

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        FolderSecurityRole childScopeFolderSR = (FolderSecurityRole) getSecurityRole(childSFolder);
        childScopeFolderSR.setCanPublishRenderPrivateFolder(Boolean.FALSE);
        childScopeFolderSR = (FolderSecurityRole) updateWorkElementSecurityRole(childScopeFolderSR);

        childSFolder.setPrivateFolder(Boolean.TRUE);

        /* Try to view the project, should fail */
        saveAndValidate(childSFolder, parentFolderScope, false);

        /* Give the right to view */

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        childScopeFolderSR = (FolderSecurityRole) getSecurityRole(childSFolder);
        childScopeFolderSR.setCanPublishRenderPrivateFolder(Boolean.TRUE);
        childScopeFolderSR = (FolderSecurityRole) updateWorkElementSecurityRole(childScopeFolderSR);

        childSFolder.setPrivateFolder(Boolean.TRUE);

        /* Try to view the project, should fail */
        saveAndValidate(childSFolder, parentFolderScope, true);
    }
 

//Classes to Test:
 //                     Project         Done
 //                     Deliverable      Done
 //                     Task

    public void testcanDeleteWBSItemsWithActualAssignments() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        WorkElementScope allScope = new WorkElementScope();
        allScope.setAttributeAssignments(true);
        allScope.setParent(new WorkElementScope());
        allScope.setAttributeAssignments(true);
        allScope.setTimeCodeAssignments(true);
        allScope.setCustomFieldAssignments(true);

        // Extra SecurityRoles
        // When SecurityRoles exist for Object use : prepareSecurityRole(-)
        // Otherwise Call SecurityRole Fixture to get ID and Load it from DB.
        DeliverableSecurityRole deliverableSecurityRole = (DeliverableSecurityRole) prepareSecurityRole(
            Deliverable.class, true);

        /* Prepare test environment */

        // --Project
        testProject = (GenericProject) prepareContainer(new Project(), null,
            allScope, securityTestUser, projectSecurityRole);
        testProject.setAttributeAssignments(new AttributeAssignment[]
        {
            AttributeAssignmentFixture.createAssignmentFixture(testProject,
                this)
        });

        SingleResult rs = null;
        rs = save(testProject, allScope);
        assertNotNull(rs);
        assertTrue(rs.isSuccessful());
        testProject = (Project) rs.getRpmObject();

        // --Deliverable
        Deliverable testDeliverable = (Deliverable) prepareContainer(
            new Deliverable(), testProject, allScope, securityTestUser,
            deliverableSecurityRole);
        testDeliverable.setAttributeAssignments((new AttributeAssignment[]
        {
            AttributeAssignmentFixture.createAssignmentFixture(testDeliverable,
                this)
        }));
        testDeliverable.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
            TimeCodeAssignmentFixture
                .createFixture(testDeliverable, true, this)
        });
        rs = null;
        rs = save(testDeliverable, allScope);
        assertNotNull(rs);
        assertTrue(rs.isSuccessful());
        testDeliverable = (Deliverable) rs.getRpmObject();

        /* Remove the right to view */

        // ---Project does not have a parent so no need to set parent
        // SecurityRole
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
        projectSecurityRole.setCanDeleteWBSItems(Boolean.FALSE);
        projectSecurityRole.setCanDeleteWBSItemsWithActualAssignments(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        // --Deliverable
        deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
        deliverableSecurityRole.setCanDeleteWBSItemsWithActualAssignments(Boolean.FALSE);
        deliverableSecurityRole.setCanDeleteWBSItems(Boolean.FALSE);
        deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

        /* Try to delete the project, should fail */
        deleteAndValidate(testProject, false);
        deleteAndValidate(testDeliverable, false);

        /* Give the right to delete */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
        projectSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
        projectSecurityRole.setCanDeleteWBSItemsWithActualAssignments(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

        // --Deliverable
        deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
        deliverableSecurityRole.setCanDeleteWBSItemsWithActualAssignments(Boolean.TRUE);
        deliverableSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
        deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

        /* Try to delete the project, should fail */
        deleteAndValidate(testDeliverable, true);
        deleteAndValidate(testProject, true);

    }
    
    public void testCanSetAsDefaultProject() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
        projectSecurityRole.setCanSetAsDefaultProject(Boolean.FALSE);
       
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setDefaultProject(Boolean.TRUE);
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanSetAsDefaultProject(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        testProject.setDefaultProject(Boolean.TRUE);
        saveAndValidate(testProject, null, true);
    }
    
    
//--Cannot be done since the CheckPoint prevent you from changing the publishType
//--to template until it is implemented.
//    public void testCanPublishDocumentsToTemplate() throws Exception
//    {
//        /* Prepare test environment*/
//        testProject = (GenericProject) prepareContainer(new Project(),
//            null, projectScope, securityTestUser, projectSecurityRole);
//        
//        /* Remove the right to edit */
//        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
//        projectSecurityRole.setCanPublishDocumentsToTemplate(Boolean.FALSE);
//        updateProjectSecurityRole();
//        
//        /* Try to modify the project, should fail */
//        testProject.setPublished(ProjectPublishedType.ToTemplate);
//        saveAndValidate(testProject, null, false);
//        
//        /* Give the right to edit */
//        projectSecurityRole.setCanPublishDocumentsToTemplate(Boolean.TRUE);
//        updateProjectSecurityRole();
//        
//        /* Try to modify the project, should succeed */
//        testProject.setPublished(ProjectPublishedType.ToTemplate);
//        saveAndValidate(testProject, null, true);
//    }
    
    public void testCanEditTaskType() throws Exception
    {
        projectScope.setParent(new WorkElementScope());

        TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
            Task.class, true);

        /* Prepare test environment*/
        Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
            securityTestUser, taskSecurityRole);

        /* Remove the right to edit */  
        projectSecurityRole = (ProjectSecurityRole)getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDefaultTaskType(Boolean.FALSE);
        updateWorkElementSecurityRole(projectSecurityRole);
        
        
        taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
        taskSecurityRole.setCanEditTaskType(Boolean.FALSE);        
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should fail */
        testTask.setTaskType(TaskType.DefaultTask);
        saveAndValidate(testTask, null, false);

        /* Give the right to edit */
        projectSecurityRole = (ProjectSecurityRole)getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDefaultTaskType(Boolean.TRUE);
        updateWorkElementSecurityRole(projectSecurityRole);

         taskSecurityRole.setCanEditTaskType(Boolean.TRUE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should succeed */
        testTask.setTaskType(TaskType.DefaultTask);
        saveAndValidate(testTask, null, true);
    }
    
    
//  Classes to Test:
    //                     Project       
    //                     Deliverable     
    //                     Task            Done

    public void testCanEditDefaultTimeCodes() throws Exception
    {
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timeCode1 = TimecodeFixture.createSaveTimeCodeFixture(timeCodeScope, true, this);
        TimeCode timeCode2 = TimecodeFixture.createSaveTimeCodeFixture(timeCodeScope, false, this);        

        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);

        TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
            Task.class, true);

        /* Prepare test environment*/
        Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
            securityTestUser, taskSecurityRole);

        /* Remove the right to edit */          
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDefaultTimeCodes(Boolean.FALSE);
        updateWorkElementSecurityRole(projectSecurityRole);

        taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
        taskSecurityRole.setCanEditDefaultTimeCodes(Boolean.FALSE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should fail */                 
        testTask.setTimeCode1(timeCode1);
        saveAndValidate(testTask, projectScope, false);
        
        testTask.setTimeCode2(timeCode2);
        saveAndValidate(testTask, projectScope, false);

        /* Give the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDefaultTimeCodes(Boolean.TRUE);
        updateWorkElementSecurityRole(projectSecurityRole);

         taskSecurityRole.setCanEditDefaultTimeCodes(Boolean.TRUE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should succeed */
        testTask.setTimeCode1(timeCode1);
        saveAndValidate(testTask, projectScope, true);
        
        testTask.setTimeCode2(timeCode2);
        saveAndValidate(testTask, projectScope, true);
    }
    
//  Classes to Test:
    //                     Project     +  
    //                     Deliverable  +   
    //                     Task            Done
    public void testCanViewDefaultTimeCodesWithDeliverable1() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            DeliverableSecurityRole.class, new Deliverable(), new Proposal(), projectScope,
            "TimeCode1", "CanViewDefaultTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewDefaultTimeCodes");
        test.runTest();
    }
    
    public void testCanViewDefaultTimeCodesWithDeliverable2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            DeliverableSecurityRole.class, new Deliverable(), new Proposal(), projectScope,
            "TimeCode2", "CanViewDefaultTimeCodes");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("CanViewDefaultTimeCodes");
        test.runTest();
    }

    public void testCanViewDefaultTimeCodesWithProject1() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            ProjectSecurityRole.class, new Project(), new Project(), projectScope,
            "TimeCode1", "CanViewDefaultTimeCodes");
         test.runTest();
    }
    
    public void testCanViewDefaultTimeCodesWithProject2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        projectScope = new WorkElementScope();
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            ProjectSecurityRole.class, new Project(), new Project(), projectScope,
            "TimeCode2", "CanViewDefaultTimeCodes");
         test.runTest();
    }

    public void testCanViewDefaultTimeCodes() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);

        TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
            Task.class, true);

        /* Prepare test environment*/
        Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
            securityTestUser, taskSecurityRole);

        /* Remove the right to edit */          
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanViewDefaultTimeCodes(Boolean.FALSE);
        updateWorkElementSecurityRole(projectSecurityRole);

        taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
        taskSecurityRole.setCanViewDefaultTimeCodes(Boolean.FALSE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should fail */  
        
        Task reloaded = (Task) loadFromID(testTask, projectScope);
        assertNotNull(reloaded);
        assertNull(reloaded.getTimeCode1());
        assertNull(reloaded.getTimeCode2());     

        /* Give the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanViewDefaultTimeCodes(Boolean.TRUE);
        updateWorkElementSecurityRole(projectSecurityRole);

         taskSecurityRole.setCanViewDefaultTimeCodes(Boolean.TRUE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should succeed */
        reloaded = (Task) loadFromID(testTask, projectScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getTimeCode1());
        assertNotNull(reloaded.getTimeCode2());   
    }

    
   // Project, +
    //Defect, +
    //ActionItem, +
    //Deliverable,      Done 
    //Requirement, 
    //ServiceRequest, + 
    //Risk,     +
    //ChangeRequest, +
    //Issue +
    
    public void testCanViewESTAndESTFinishWithProject() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            ProjectSecurityRole.class, new Project(), new Project(), projectScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.runTest();
    }
    
    public void testCanViewESTAndESTFinishWithRisk() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            RiskSecurityRole.class, new Risk(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }
    
    public void testCanViewESTAndESTFinishWithChangeRequest() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ChangeRequestSecurityRole.class, new ChangeRequest(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithServiceRequest() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ServiceRequestSecurityRole.class, new ServiceRequest(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithIssue() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            IssueSecurityRole.class, new Issue(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithDefect() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            DefectSecurityRole.class, new Defect(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithActionItem() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ActionItemSecurityRole.class, new Action(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithRequirement() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            RequirementSecurityRole.class, new Requirement(), new Proposal(), allScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    
    
    
    public void testCanViewESTAndESTFinishWithProject2() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            ProjectSecurityRole.class, new Project(), new Project(), projectScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.runTest();
    }
    
    public void testCanViewESTAndESTFinishWithRisk2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            RiskSecurityRole.class, new Risk(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }
    
    public void testCanViewESTAndESTFinishWithChangeRequest2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ChangeRequestSecurityRole.class, new ChangeRequest(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithServiceRequest2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ServiceRequestSecurityRole.class, new ServiceRequest(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithIssue2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            IssueSecurityRole.class, new Issue(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithDefect2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            DefectSecurityRole.class, new Defect(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithActionItem2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            ActionItemSecurityRole.class, new Action(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithRequirement2() throws Exception
    {
        // Scope, needs to include parent and timecodeAssignements
        ScopeElementScope allScope = new ScopeElementScope();
        allScope.setParent(new ScopeElementScope());
        allScope.getParent().setParent(new WorkElementScope());
        
        PlannedScopeElementSecurityTestCanViewLocal test = new PlannedScopeElementSecurityTestCanViewLocal(
            RequirementSecurityRole.class, new Requirement(), new Proposal(), allScope,
            "EstimatedStartDate", "CanViewESTStartAndESTFinish");
        test.grandparentRequiredOffFlags = new ArrayList();
        test.grandparentRequiredOffFlags.add("AllScopeElementsPermissionsGranted");
        test.grandparentRequiredOffFlags.add("CanViewESTStartAndESTFinish");
        test.runTest();
    }

    public void testCanViewESTAndESTFinishWithDeliverable() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        
        PlannedWorkElementSecurityTestCanViewLocal test = new PlannedWorkElementSecurityTestCanViewLocal(
            DeliverableSecurityRole.class, new Deliverable(), new Project(), projectScope,
            "EstimatedFinishDate", "CanViewESTStartAndESTFinish");
        test.runTest();
    }


    //  Classes to Test:
    //                     Project       
    //                     Deliverable     
    //                     Task            Done

    public void testCanEditWBSPriority() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        testProject = (GenericProject) prepareContainer(new Project(),
            null, projectScope, securityTestUser, projectSecurityRole);
        
        TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
            Task.class, true);

        /* Prepare test environment*/
        Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
            securityTestUser, taskSecurityRole);

        /* Remove the right to edit */          
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditWBSPriority(Boolean.FALSE);
        updateWorkElementSecurityRole(projectSecurityRole);

        taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
        taskSecurityRole.setCanEditWBSPriority(Boolean.FALSE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should fail */                 
        testTask.setPriority(new Integer(30));
        saveAndValidate(testTask, projectScope, false);
        
        /* Give the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditWBSPriority(Boolean.TRUE);
        updateWorkElementSecurityRole(projectSecurityRole);

         taskSecurityRole.setCanEditWBSPriority(Boolean.TRUE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should succeed */
        testTask.setPriority(new Integer(30));
        saveAndValidate(testTask, projectScope, true);
        
    }

    
    public void testCanPublishSecurity() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
        projectSecurityRole.setCanPublishSecurity(Boolean.FALSE);
         
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        testProject.setPublished(ProjectPublishedType.ToTeam);
        saveAndValidate(testProject, null, false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanPublishSecurity(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        testProject.setPublished(ProjectPublishedType.ToTeam);
        saveAndValidate(testProject, null, true);
    }

    //  Classes to Test:
    //                     Project       Done
    //                     Deliverable     
    //                     Task            
    public void testCanCopyProposedToPlan() throws Exception
    {
        
        /* Remove the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
        projectSecurityRole.setCanCopyProposedToPlan(Boolean.FALSE);
         
        updateProjectSecurityRole();
        
        /* Try to modify the project, should fail */
        copyProposedToPlan(testProject, null,false);
        
        /* Give the right to edit */
        projectSecurityRole.setCanCopyProposedToPlan(Boolean.TRUE);
        updateProjectSecurityRole();
        
        /* Try to modify the project, should succeed */
        copyProposedToPlan(testProject, null,true);
    }

    //  Classes to Test:
    //                     Project       
    //                     Deliverable     
    //                     Task            Done

    public void testCanEditDependency() throws Exception
    {
        projectScope.setParent(new WorkElementScope());
        projectScope.setTimeCodes(true);
        projectScope.setDependencies(new ElementDependencyScope());

        TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
            Task.class, true);

        /* Prepare test environment*/
        Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
            securityTestUser, taskSecurityRole);

        /* Remove the right to edit */          
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDependency(Boolean.FALSE);
        updateWorkElementSecurityRole(projectSecurityRole);

        taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
        taskSecurityRole.setCanEditDependency(Boolean.FALSE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should fail */  
        assertNotNull(testTask.getDependencies());
        assertTrue(testTask.getDependencies().length>=2); //may be we should loop
        testTask.getDependencies()[0].setAmount(new Integer(4));
        testTask.getDependencies()[0].setPredecessor(new WorkElement());
        testTask.getDependencies()[0].setDependencyType(DependencyType.StartFinish);
        saveAndValidate(testTask, projectScope, false);
        
        /* Give the right to edit */
        projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
        projectSecurityRole.setCanEditDependency(Boolean.TRUE);
        updateWorkElementSecurityRole(projectSecurityRole);

         taskSecurityRole.setCanEditDependency(Boolean.TRUE);
        updateWorkElementSecurityRole(taskSecurityRole);

        /* Try to modify the project, should succeed */
        assertNotNull(testTask.getDependencies());
        assertTrue(testTask.getDependencies().length>=2);
        testTask.getDependencies()[0].setAmount(new Integer(4));
        testTask.getDependencies()[0].setPredecessor(new WorkElement());
        testTask.getDependencies()[1].setPredecessor(new WorkElement());
        testTask.getDependencies()[0].setDependencyType(DependencyType.StartFinish);
        saveAndValidate(testTask, projectScope, true);
    }

    
    
//  Classes to Test:
    //                     Project         Done
    //                     Deliverable      Done
    //                     Task

       public void testcanDeleteWBSItems() throws Exception
       {
           // Scope, needs to include parent and timecodeAssignements
           WorkElementScope allScope = new WorkElementScope();
           allScope.setParent(new WorkElementScope());

           // Extra SecurityRoles
           // When SecurityRoles exist for Object use : prepareSecurityRole(-)
           // Otherwise Call SecurityRole Fixture to get ID and Load it from DB.
           DeliverableSecurityRole deliverableSecurityRole = (DeliverableSecurityRole) prepareSecurityRole(
               Deliverable.class, true);

           /* Prepare test environment */

           // --Project
           testProject = (GenericProject) prepareContainer(new Project(), null,
               allScope, securityTestUser, projectSecurityRole);

           // --Deliverable
           Deliverable testDeliverable = (Deliverable) prepareContainer(
               new Deliverable(), testProject, allScope, securityTestUser,
               deliverableSecurityRole);

           /* Remove the right to view */

           // ---Project does not have a parent so no need to set parent
           // SecurityRole
           projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
           projectSecurityRole.setCanDeleteWBSItems(Boolean.FALSE);
           projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

           // --Deliverable
           deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
           deliverableSecurityRole.setCanDeleteWBSItems(Boolean.FALSE);
           deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

           /* Try to delete the project, should fail */
           deleteAndValidate(testProject, false);
           deleteAndValidate(testDeliverable, false);

           /* Give the right to delete */
           projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
           projectSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
           projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

           // --Deliverable
           deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
           deliverableSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
           deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

           /* Try to delete the project, should fail */
           deleteAndValidate(testDeliverable, true);
           deleteAndValidate(testProject, true);

       }

       
       
//     Classes to Test:
       //                     DocumentFolder     
       //                     ScopeFolder         Done

       public void testCanRenameFolder() throws Exception
          {
              // -----Creating parent project
              WorkElementScope allScope = new WorkElementScope();
              allScope.setContainingProject(new WorkElementScope());
              allScope.setParent(new WorkElementScope());

              // Scope Folder
              ScopeElementScope parentFolderScope = new ScopeElementScope();
              parentFolderScope.setChildren(parentFolderScope);
              parentFolderScope.setParent(new RPMObjectScope());
              parentFolderScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
              
              ScopeFolder parentSFolder = null;

              /* Prepare Project test environment */
              testProject = (GenericProject) prepareContainer(new Project(), null,
                  allScope, securityTestUser, projectSecurityRole);
              
              // ScopeFolder
              RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                      + testProject.getName() + "']", parentFolderScope);
              parentSFolder = (ScopeFolder) rpmObjects[0];
              
              ScopeFolder childSFolder = new ScopeFolder();              
              childSFolder.setName("SecurityFolder");
              parentSFolder.setChildren(new ScopeElement[]
              {
                  childSFolder
              });

              SingleResult rs = save(parentSFolder, parentFolderScope);
              assertNotNull(rs.getRpmObject());
              assertTrue(rs.isSuccessful());
              childSFolder = (ScopeFolder) ((ScopeFolder) rs.getRpmObject())
                  .getChildren()[0];

              /* Remove the right to view */

              // ---Project does not have a parent so no need to set parent
              // SecurityRole
              projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
              projectSecurityRole.setCanRenameWBSItems(Boolean.FALSE);
              updateProjectSecurityRole();
              
              FolderSecurityRole childScopeFolderSR = (FolderSecurityRole) getSecurityRole(childSFolder);
              childScopeFolderSR.setCanRenameFolders(Boolean.FALSE);
              childScopeFolderSR = (FolderSecurityRole) updateWorkElementSecurityRole(childScopeFolderSR);

              childSFolder.setName("RenamedFolder");

              /* Try to view the project, should fail */
              saveAndValidate(childSFolder, parentFolderScope, false);

              /* Give the right to view */

              // ---Project does not have a parent so no need to set parent
              // SecurityRole
              projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
              projectSecurityRole.setCanRenameWBSItems(Boolean.TRUE);
              updateProjectSecurityRole();

              childScopeFolderSR = (FolderSecurityRole) getSecurityRole(childSFolder);
              childScopeFolderSR.setCanRenameFolders(Boolean.TRUE);
              childScopeFolderSR = (FolderSecurityRole) updateWorkElementSecurityRole(childScopeFolderSR);

              childSFolder.setName("RenamedFolder");

              /* Try to view the project, should fail */
              saveAndValidate(childSFolder, parentFolderScope, true);
          }

       
       //     Classes to Test:
       //                     Project         Done
       //                     Deliverable      Done
       //                     Task

          public void testcanRenameWBSItems() throws Exception
          {
              // Scope, needs to include parent and timecodeAssignements
              WorkElementScope allScope = new WorkElementScope();
              allScope.setParent(new WorkElementScope());

              // Extra SecurityRoles
              // When SecurityRoles exist for Object use : prepareSecurityRole(-)
              // Otherwise Call SecurityRole Fixture to get ID and Load it from DB.
              DeliverableSecurityRole deliverableSecurityRole = (DeliverableSecurityRole) prepareSecurityRole(
                  Deliverable.class, true);

              /* Prepare test environment */

              // --Project
              testProject = (GenericProject) prepareContainer(new Project(), null,
                  allScope, securityTestUser, projectSecurityRole);

              // --Deliverable
              Deliverable testDeliverable = (Deliverable) prepareContainer(
                  new Deliverable(), testProject, allScope, securityTestUser,
                  deliverableSecurityRole);

              /* Remove the right to view */

              // ---Project does not have a parent so no need to set parent
              // SecurityRole
              projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
              projectSecurityRole.setCanRenameWBSItems(Boolean.FALSE);
              projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

              // --Deliverable
              deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
              deliverableSecurityRole.setCanRenameWBSItems(Boolean.FALSE);
              deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);
              
              testProject.setName("RenamedProject");
              testDeliverable.setName("RenamedDeliverable");

              /* Try to delete the project, should fail */
              saveAndValidate(testProject,allScope, false);
              saveAndValidate(testDeliverable,allScope, false);

              /* Give the right to delete */
              projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
              projectSecurityRole.setCanRenameWBSItems(Boolean.TRUE);
              projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

              // --Deliverable
              deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
              deliverableSecurityRole.setCanRenameWBSItems(Boolean.TRUE);
              deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

              testProject.setName("RenamedProject");
              testDeliverable.setName("RenamedDeliverable");

              /* Try to delete the project, should fail */
              saveAndValidate(testProject,allScope, true);
              saveAndValidate(testDeliverable,allScope, true);

          }
          
          
//        Classes to Test:
          //                     Project         Done
          //                     Deliverable      Done
          //                     Task

             public void testcanChangeWBSItemsType() throws Exception
             {
                 // Scope, needs to include parent and timecodeAssignements
                 WorkElementScope allScope = new WorkElementScope();
                 allScope.setParent(new WorkElementScope());

                 // Extra SecurityRoles
                 // When SecurityRoles exist for Object use : prepareSecurityRole(-)
                 // Otherwise Call SecurityRole Fixture to get ID and Load it from DB.
                 DeliverableSecurityRole deliverableSecurityRole = (DeliverableSecurityRole) prepareSecurityRole(
                     Deliverable.class, true);

                 /* Prepare test environment */

                 // --Project
                 testProject = (GenericProject) prepareContainer(new Project(), null,
                     allScope, securityTestUser, projectSecurityRole);

                 // --Deliverable
                 Deliverable testDeliverable = (Deliverable) prepareContainer(
                     new Deliverable(), testProject, allScope, securityTestUser,
                     deliverableSecurityRole);

                 /* Remove the right to view */

                 // ---Project does not have a parent so no need to set parent
                 // SecurityRole
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
                 projectSecurityRole.setCanChangeWBSItemTypes(Boolean.FALSE);
                 projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

                 // --Deliverable
                 deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
                 deliverableSecurityRole.setCanChangeWBSItemTypes(Boolean.FALSE);    
                 deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);
                 
                  /* Try to delete the project, should fail */
                 convertContainer(testProject,new Proposal(),allScope, false);
                 convertContainer(testDeliverable,new WorkProduct(),allScope, false);

                 /* Give the right to delete */
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testProject);
                 projectSecurityRole.setCanChangeWBSItemTypes(Boolean.TRUE);                 
                 projectSecurityRole = (ProjectSecurityRole) updateWorkElementSecurityRole(projectSecurityRole);

                 // --Deliverable
                 deliverableSecurityRole = (DeliverableSecurityRole) getSecurityRole(testDeliverable);
                 deliverableSecurityRole.setCanChangeWBSItemTypes(Boolean.TRUE);
                 deliverableSecurityRole = (DeliverableSecurityRole) updateWorkElementSecurityRole(deliverableSecurityRole);

                 /* Try to delete the project, should fail */
                 convertContainer(testProject,new Proposal(),allScope, true);
                 convertContainer(testDeliverable,new WorkProduct(),allScope, true);
             }

             public void testCanAddRemoveDependency_remove() throws Exception
             {
                 projectScope.setParent(new WorkElementScope());
                 projectScope.setTimeCodes(true);
                 projectScope.setDependencies(new ElementDependencyScope());                 

                 TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
                     Task.class, true);

                 /* Prepare test environment*/
                 Task testTask = (Task) prepareContainer(new Task(), null, projectScope,
                     securityTestUser, taskSecurityRole);                 
                 
                 /* Remove the right to edit */          
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveDependency(Boolean.FALSE);
                 updateWorkElementSecurityRole(projectSecurityRole);                  
                 
                 taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
                 taskSecurityRole.setCanAddRemoveDependency(Boolean.FALSE);
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);                   
                 updateWorkElementSecurityRole(taskSecurityRole);

                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getDependencies());
                 deleteAndValidate(testTask.getDependencies()[0], false);                 
                 
                 
                 /* Give the right to edit */
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveDependency(Boolean.TRUE);
                  updateWorkElementSecurityRole(projectSecurityRole);

                 taskSecurityRole.setCanAddRemoveDependency(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);

                 
                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getDependencies());                 
                 deleteAndValidate(testTask.getDependencies()[0], true);                 
                 
                 //Setting SuperFlags back to true
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.TRUE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
             }
             
             public void testCanAddRemoveDependency_add() throws Exception
             {
                 ElementDependencyScope depScope = new ElementDependencyScope();
                 depScope.setPredecessor(new WorkElementScope());
                 
                 projectScope.setParent(new WorkElementScope());
                 projectScope.setTimeCodes(true);
                
                 TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
                     Task.class, true);

                 /* Prepare test environment*/
                 Task testTask = (Task) prepareContainer(new Task(), null, projectScope,
                     securityTestUser, taskSecurityRole);                 
                 
                 /* Remove the right to edit */          
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveDependency(Boolean.FALSE);
                 updateWorkElementSecurityRole(projectSecurityRole);                  
                 
                 taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
                 taskSecurityRole.setCanAddRemoveDependency(Boolean.FALSE);
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);                   
                 updateWorkElementSecurityRole(taskSecurityRole);

                 /* Try to modify the project, should fail */                  
                 projectScope.setDependencies(depScope); 
                 
                 WorkElement we = WorkElementFixture.createSaveFixture(new Project(), null, this);
                 ElementDependency eld1 = new ElementDependency();
                 eld1.setDependencyType(DependencyType.FinishStart);
                 eld1.setPredecessor(we);                
                 testTask.setDependencies(new ElementDependency[]{eld1});
                 eld1.setAmount(new Integer(30));             
                 
                 saveAndValidate(testTask,projectScope, false);                  
                 
                 /* Give the right to edit */
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveDependency(Boolean.TRUE);
                  updateWorkElementSecurityRole(projectSecurityRole);

                 taskSecurityRole.setCanAddRemoveDependency(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
                 
                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getDependencies());                 
                 saveAndValidate(testTask,projectScope, true);                
                 
                 //Setting SuperFlags back to true
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.TRUE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
             }

             public void testCanAddRemoveTimeCodeAssignments_remove() throws Exception
             {
                 projectScope.setParent(new WorkElementScope());
                 projectScope.setTimeCodeAssignments(true);

                 TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
                     Task.class, true);

                 /* Prepare test environment*/
                 Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
                     securityTestUser, taskSecurityRole);                 
                 
                 /* Remove the right to edit */          
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveTimeCodes(Boolean.FALSE);
                 updateWorkElementSecurityRole(projectSecurityRole);                  
                 
                 taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
                 taskSecurityRole.setCanAddRemoveTimeCodes(Boolean.FALSE);
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);                   
                 updateWorkElementSecurityRole(taskSecurityRole);

                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getTimeCodeAssignments());
                 deleteAndValidate(testTask.getTimeCodeAssignments()[0], false);                 
                 
                 
                 /* Give the right to edit */
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveTimeCodes(Boolean.TRUE);
                  updateWorkElementSecurityRole(projectSecurityRole);

                 taskSecurityRole.setCanAddRemoveTimeCodes(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);

                 
                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getTimeCodeAssignments());                 
                 deleteAndValidate(testTask.getTimeCodeAssignments()[0], true);                 
                 
                 //Setting SuperFlags back to true
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.TRUE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
             }

             public void testCanAddRemoveTimeCodeAssignments_add() throws Exception
             {                 
                 projectScope.setParent(new WorkElementScope());
                
                 TaskSecurityRole taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(
                     Task.class, true);

                 /* Prepare test environment*/
                 Task testTask = (Task) prepareContainer(new Task(), testProject, projectScope,
                     securityTestUser, taskSecurityRole);                 
                 
                 /* Remove the right to edit */          
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveTimeCodes(Boolean.FALSE);
                 updateWorkElementSecurityRole(projectSecurityRole);                  
                 
                 taskSecurityRole = (TaskSecurityRole) getSecurityRole(testTask);
                 taskSecurityRole.setCanAddRemoveTimeCodes(Boolean.FALSE);
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);                   
                 updateWorkElementSecurityRole(taskSecurityRole);

                 /* Try to modify the project, should fail */                  
                 projectScope.setTimeCodes(true);
                 
                 assertNull(testTask.getTimeCodeAssignments());
                                 
                 TimeCodeAssignment tca1 = TimeCodeAssignmentFixture.createFixture(testTask, false, this);
                 TimeCodeAssignment tca2 = TimeCodeAssignmentFixture.createFixture(testTask, true, this);                 
                 testTask.setTimeCodeAssignments(new TimeCodeAssignment[]{tca1,tca2});           
                 
                 TimeCodeScope scope = new TimeCodeScope();
                 scope.setParent(projectScope);
                 saveAndValidate(tca1,null, false);     
                 saveAndValidate(tca2,null, false); 
                 
                 /* Give the right to edit */
                 projectSecurityRole = (ProjectSecurityRole) getSecurityRole(testTask.getParent());
                 projectSecurityRole.setCanAddRemoveTimeCodes(Boolean.TRUE);
                  updateWorkElementSecurityRole(projectSecurityRole);

                 taskSecurityRole.setCanAddRemoveTimeCodes(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
                 
                 /* Try to modify the project, should fail */  
                 assertNotNull(testTask.getTimeCodeAssignments());                 

                 saveAndValidate(tca1,null, true);   
                 saveAndValidate(tca2,null, true); 
                 
                 //Setting SuperFlags back to true
                 taskSecurityRole.setAllDocumentPermissionsGranted(Boolean.TRUE);
                 taskSecurityRole.setAllScopeElementsPermissionsGranted(Boolean.TRUE);
                 updateWorkElementSecurityRole(taskSecurityRole);
             }
        


}
