package com.ibm.rpm.client.junit.security.testcases.role.document;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.DocumentSecurityRole;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestAbstractDocumentSecurityRoleLocal extends
        BaseRoleSecurityTestLocal
{
    DocumentSecurityRole documentSecurityRole;

    DocumentScope documentScope;

    protected void setUp() throws Exception
    {
        super.setUp();
        documentScope = new DocumentScope();
        documentSecurityRole = (DocumentSecurityRole) prepareSecurityRole(
            AbstractDocument.class, true);
    }

    /**
     * Allows to add or remove values in the Attributes portlet in the Project,
     * Deliverable, Task, Scope, Document and Notes/Minutes description view.
     * 
     * Applies when the container is saved and the field has been modified. Also
     * applies when the field container is created or deleted.
     */
    public void testCanAddRemoveAttributes() throws Exception
    {
//       canAddRemoveAttributesScopeSection();
//       canAddRemoveAttributesWbsSection();
    }

    /**
     * The flag is tested for the WBS section
     */
    private void canAddRemoveAttributesWbsSection() throws Exception
    {
        WorkElementScope parentScope = new WorkElementScope();
        WorkElement parent = WorkElementFixture.createFixture(new Task(), parentScope, this);
        
        parentScope.setDocumentFolder(new DocumentScope());
        parent = (WorkElement) save(parent, parentScope).getRpmObject();
        
        flipCanAddRemoveAttributes(parent);
    }
    
    /**
     * the flag is tested in the Scope section
     */
    private void canAddRemoveAttributesScopeSection() throws Exception
    {
        ScopeElementScope parentScope = new ScopeElementScope();
        Requirement parentScopeElement = ScopeElementFixture.createRequirementFixture (
            parentScope, 
            1, 
            (ScopeElement)ScopeElementFixture.createMainParent(this), 
            this);
        parentScopeElement.setDefaultScope(null); // set in the fixture
        parentScope.setDocumentFolder(new DocumentScope());
        parentScopeElement = (Requirement) save(parentScopeElement, parentScope).getRpmObject();
        
        flipCanAddRemoveAttributes(parentScopeElement);
    }
    
    /**
     * Common test code for all sections to which flipCanAddRemoveAttributes applies
     * @param parent
     */
    private void flipCanAddRemoveAttributes(RPMObject parent) throws Exception
    {
        Document document = (Document) prepareContainer(new Document(), parent,
            documentScope, securityTestUser, documentSecurityRole);
        documentScope.setAttributeAssignments(true);
        
        assertNotNull(document);
        
        /* Remove the right to assign attributes to the document */
        documentSecurityRole.setCanAddRemoveAttributes(Boolean.FALSE);
        updateDocumentSecurityRole();

        document = (Document)loadFromID(document, documentScope);
        
        AttributeAssignment attribute = AttributeAssignmentFixture.createAssignmentFixture(document, this);
        document.setAttributeAssignments(new AttributeAssignment[] {attribute});
        
        saveAndValidate(document, documentScope, false);
        
        /* Allow to assign attributes to the document */
        documentSecurityRole.setCanAddRemoveAttributes(Boolean.TRUE);
        updateDocumentSecurityRole();
        
        document = (Document) loadFromID(document, documentScope);
        attribute = AttributeAssignmentFixture.createAssignmentFixture(document, this);
        document.setAttributeAssignments(new AttributeAssignment[] {attribute});
        
        saveAndValidate(document, documentScope, true);
    }

    protected void updateDocumentSecurityRole() throws Exception
    {
        documentSecurityRole = (DocumentSecurityRole) updateSecurityRole(documentSecurityRole);
    }
}