/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.role.fixes;

import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Yasser Lulu
 *
 */
public class Test65119Local extends BaseRoleSecurityTestLocal
{

    public void testEditProjectPriority() throws Exception
    {
        WorkElementScope projectScope = new WorkElementScope();
        projectScope.setOpportunity(true);//to force saving of probabilityPercent
        ProjectSecurityRole projectSecurityRole = (ProjectSecurityRole) prepareSecurityRole(
            Project.class, true);
        GenericProject testProject = (GenericProject) prepareContainer(
            new Project(), null, projectScope, securityTestUser,
            projectSecurityRole);

        Integer priority = new Integer(testProject.getPriority().intValue() + 2);
        testProject.setPriority(priority);
        Integer probabilityPercent = new Integer(testProject
            .getProbabilityPercent().intValue() + 2);
        testProject.setProbabilityPercent(probabilityPercent);
        saveAndValidate(testProject, projectScope, true);
        GenericProject reloadedProject = (GenericProject) loadFromID(
            testProject, null);
        assertEquals(priority.intValue(), reloadedProject.getPriority()
            .intValue());
        assertEquals(probabilityPercent.intValue(), reloadedProject
            .getProbabilityPercent().intValue());

        projectSecurityRole.setCanViewWBSPriority(Boolean.FALSE);
        projectSecurityRole.setCanEditWBSPriority(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateSecurityRole(projectSecurityRole);

        reloadedProject = (GenericProject) loadFromID(testProject, null);

        assertNull(reloadedProject.getPriority());
        assertNotNull(reloadedProject.getProbabilityPercent());

        probabilityPercent = new Integer(reloadedProject
            .getProbabilityPercent().intValue() + 2);
        reloadedProject.setProbabilityPercent(probabilityPercent);
        saveAndValidate(reloadedProject, projectScope, true);

        reloadedProject = (GenericProject) loadFromID(testProject, null);
        assertNull(reloadedProject.getPriority());
        assertEquals(probabilityPercent.intValue(), reloadedProject
            .getProbabilityPercent().intValue());

        projectSecurityRole.setCanViewWBSPriority(Boolean.TRUE);
        projectSecurityRole.setCanEditWBSPriority(Boolean.TRUE);
        projectSecurityRole = (ProjectSecurityRole) updateSecurityRole(projectSecurityRole);
        reloadedProject = (GenericProject) loadFromID(testProject, null);
        assertEquals(priority.intValue(), reloadedProject.getPriority()
            .intValue());

        probabilityPercent = new Integer(reloadedProject
            .getProbabilityPercent().intValue() + 2);
        reloadedProject.setProbabilityPercent(probabilityPercent);
        saveAndValidate(reloadedProject, projectScope, true);
        assertEquals(priority.intValue(), reloadedProject.getPriority()
            .intValue());
        assertEquals(probabilityPercent.intValue(), reloadedProject
            .getProbabilityPercent().intValue());
    }

}
