package com.ibm.rpm.client.junit.security.testcases.role.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.security.containers.AbstractDeliverableSecurityRole;
import com.ibm.rpm.security.containers.AbstractWbsSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00253170Local extends BaseRoleSecurityTestLocal {

	AggregateNode parent = null;
	SimpleNode task = null;
	WorkElementScope workElementScope = null;
	int i = 0;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		workElementScope = new WorkElementScope();
		workElementScope.setScheduleDates(true);
		workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
		parent = createSaveProject();
		task = createSaveTask();
	}
	
	public void testSaveProposedDate() throws Exception
	{
		runSave(false,false,false);
		runSave(true,false,true);
		runSave(false,true,true);
		runSave(true,true,true);
	}
	
	public void testLoadProposedDate() throws Exception
	{
		runLoad(false);
		runLoad(true);
	}
	
	private void runSave(boolean parentFlagValue, boolean childFlagValue, boolean canSave) throws Exception
	{
		modifyCanEditProposedDatesFlag(parent, parentFlagValue);
		modifyCanEditProposedDatesFlag(task, childFlagValue);
		modifyAndSaveProposedDateForTask(canSave);
	}
	
	private void runLoad(boolean canView) throws Exception
	{
		WorkElementScope workElementScope = createScopeForLoad();
		modifyCanViewScheduleInfoFlag(parent,canView);
		modifyCanViewScheduleInfoFlag(task,canView);
		AggregateNode loadedElement = (AggregateNode)loadFromID(parent, workElementScope);
		validateLoad(loadedElement,canView);
	}
	
	private WorkElementScope createScopeForLoad()
	{
		WorkElementScope workElementScope = new WorkElementScope();
		WorkElementScope taskScope = new WorkElementScope();
		taskScope.setScheduleDates(true);
		workElementScope.setChildren(taskScope);
		workElementScope.setScheduleDates(true);
		return workElementScope;
	}
	
	private Task createSaveTask() throws Exception
	{
		Task task = new Task();
		task.setName(HelperTestUtilities.createName());
		task.setParent(parent);
		task = (Task)save(task, workElementScope).getRpmObject();
		validateWorkElement(task);
		return task;
	}
	
	private void modifyAndSaveProposedDateForTask(boolean canSave) throws Exception
	{
		int minutesPerDay = parent.getMinutesPerDay().intValue();
		int durationInMinutes = (10 + i++) * minutesPerDay;
		WbsScheduleDate proposedDate = task.getProposedDate();
		Calendar startDate = proposedDate.getStartDate();
		startDate.add(Calendar.DATE, 1);
		WbsScheduleDate updatedProposedDate = new WbsScheduleDate();
		updatedProposedDate.setStartDate(startDate);
		updatedProposedDate.setDurationInMinutes(new Integer(durationInMinutes));
		task.setProposedDate(updatedProposedDate);
		Task savedTask = (Task)saveAndValidate(task, workElementScope, canSave);
		if(canSave)
		{
			task = savedTask;
			validateWorkElement(task);
			validateDate(startDate,task.getProposedDate().getStartDate());
			assertEquals(durationInMinutes, task.getProposedDate().getDurationInMinutes().intValue());
		}
	}

	private void validateWorkElement(WorkElement workElement)
	{
		assertNotNull(workElement);
		assertNotNull(workElement.getProposedDate());
		assertNotNull(workElement.getResourceTaskAssignments());
		assertTrue(workElement.getResourceTaskAssignments().length == 1);
		assertNotNull(workElement.getResourceTaskAssignments()[0]);
	}
	
	private void validateLoad(AggregateNode workElement, boolean canView)
	{
		assertNotNull(workElement);
		assertNotNull(workElement.getChildren());
		assertTrue(workElement.getChildren().length == 1);
		assertNotNull(workElement.getChildren()[0]);
		if(canView)
		{
			assertNotNull(workElement.getProposedDate());
			assertNotNull(workElement.getChildren()[0].getProposedDate());
		}
		else
		{
			assertNull(workElement.getProposedDate());
			assertNull(workElement.getChildren()[0].getProposedDate());
		}
	}
	
	private void validateDate(Calendar expected, Calendar actual)
	{
		assertNotNull(expected);
		assertNotNull(actual);
		assertEquals(expected.get(Calendar.YEAR),actual.get(Calendar.YEAR));
		assertEquals(expected.get(Calendar.MONTH),actual.get(Calendar.MONTH));
		assertEquals(expected.get(Calendar.DATE),actual.get(Calendar.DATE));
	}
	
	private void modifyCanEditProposedDatesFlag(WorkElement workElement, boolean value) throws Exception
	{
		AbstractWbsSecurityRole securityRole;
		securityRole = (AbstractWbsSecurityRole)workElement.getResourceTaskAssignments()[0].getSecurityRole();
		if(AggregateNode.class.isAssignableFrom(workElement.getClass()))
		{
			((AbstractDeliverableSecurityRole)securityRole).setCanEditForChildrenDatesProposed(new Boolean(value));
		}
		else
		{
			((TaskSecurityRole)securityRole).setCanEditDatesProposed(new Boolean(value));
		}
		securityRole = (AbstractWbsSecurityRole)save(securityRole, null).getRpmObject();
		assertNotNull(securityRole);
		workElement.getResourceTaskAssignments()[0].setSecurityRole(securityRole);
	}
	
	private void modifyCanViewScheduleInfoFlag(WorkElement workElement, boolean value) throws Exception
	{
		AbstractWbsSecurityRole securityRole;
		securityRole = (AbstractWbsSecurityRole)workElement.getResourceTaskAssignments()[0].getSecurityRole();
		securityRole.setCanViewScheduleInformation(new Boolean(value));
		securityRole = (AbstractWbsSecurityRole)save(securityRole, null).getRpmObject();
		assertNotNull(securityRole);
		assertEquals(value,securityRole.getCanViewScheduleInformation().booleanValue());
		workElement.getResourceTaskAssignments()[0].setSecurityRole(securityRole);
	}
	
	private GenericProject createSaveProject() throws Exception
	{
		GenericProject genericProject = new Proposal();
		genericProject.setName(HelperTestUtilities.createName());
		genericProject = (GenericProject)save(genericProject, workElementScope).getRpmObject();
		validateWorkElement(genericProject);
		return genericProject;
	}
}