/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.role.scopemanagement;

import java.util.Calendar;

import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.AbstractScopeSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ScopeElementSecurityType;

/**
 * @author SandeepS
 *
 */
public class AbstractTestScopeElementSecurityTestLocal extends
        BaseRoleSecurityTestLocal
{
	ScopeFolder parent = null;
	ScopeElementScope scopeElementScope = null;
	ScopeElement testScopeElement = null;
	AbstractScopeSecurityRole scopeElementSecurityRole = null;

    GenericProject parentProject = null;
    private ProjectSecurityRole parentProjectSecurityRole = null;

    String testClass = null;

    /* TYPE OF TEST CLASS */
    protected static final String TEST_ACTION = "ACTION";
    protected static final String TEST_ISSUE = "ISSUE";
    protected static final String TEST_RISK = "RISK";
    protected static final String TEST_REQUIREMENT = "REQUIREMENT";
    protected static final String TEST_DEFECT = "DEFECT";

    /* SCOPE ELEMENT FIELDS */
    protected static final int SCOPE_SECURITYROLE = 0;
    protected static final int SCOPE_RESOURCE = 1;

    /* ABSTRACT AGGREGATE SCOPE FIELDS */
	protected static final int ABSTRACT_AGGREGATESCOPE_PROPOSEDBY = 100;
	protected static final int ABSTRACT_AGGREGATESCOPE_CLOSEDBY = 101;
	protected static final int ABSTRACT_AGGREGATESCOPE_ESTIMATED_DATES = 102;
	protected static final int ABSTRACT_AGGREGATESCOPE_PRIORITY = 103;
	protected static final int ABSTRACT_ATTRIBUTE_ASSIGNMENTS = 104;

	/* ABSTRACT SCOPE FIELDS */
	protected static final int ABSTRACTSCOPE_STATE = 200;
	protected static final int ABSTRACTSCOPE_EXTERNAL_REFERENCE = 201;
	protected static final int ABSTRACTSCOPE_TARGETDATE = 202;
	protected static final int ABSTRACTSCOPE_APPROVEDBY = 203;
	protected static final int ABSTRACTSCOPE_IMPACT = 204;
	protected static final int ABSTRACTSCOPE_RANK = 205;
	protected static final int ABSTRACTSCOPE_ORDEROF_MAGNITUDE_COSTBENEFITS = 206;
	protected static final int ABSTRACTSCOPE_SCORECARD_ASSIGNMENTS = 207;
	protected static final int ABSTRACTSCOPE_DUPLICATES = 208;
	protected static final int ABSTRACTSCOPE_DUPLICATENUMBER = 209;
	protected static final int ABSTRACTSCOPE_DEFAULTSCOPE = 210;
	
	/* FIELDS */
	protected static final int ISSUE_OCCURREDDATETIME = 300;
	protected static final int ISSUE_TIMESREASSIGNED = 301;
	protected static final int RISK_MITIGATIONFACTORS = 302;
	protected static final int RISK_MATRIX = 303;
	protected static final int REQUIREMENT_APPROVEDCOST_BENEFITS = 304;

    protected void prepareParentProject() throws Exception
    {
    	prepareParentProjectSecurityRole();

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setScopeElement(new ScopeElementScope());

        parentProject = (GenericProject) prepareContainer(new Project(), null,
            workElementScope, securityTestUser, parentProjectSecurityRole);
        assertNotNull(parentProject);
        assertNotNull(parentProject.getScopeElement());
        assertTrue(parentProject.getScopeElement() instanceof ScopeFolder);
        parent = (ScopeFolder)parentProject.getScopeElement();
    }

    protected void prepareParentProjectSecurityRole() throws Exception
    {
        //set parent projects flags to false so that when ORed with scope flags they will have no effect
    	parentProjectSecurityRole = (ProjectSecurityRole) prepareSecurityRole(
                Project.class, false);

    	if(TEST_ACTION.equals(testClass))
    	{
    		parentProjectSecurityRole.setCanCreateActions(Boolean.TRUE);
    	}
    	else if(TEST_ISSUE.equals(testClass))
    	{
    		parentProjectSecurityRole.setCanCreateIssues(Boolean.TRUE);
    	}
    	else if(TEST_RISK.equals(testClass))
    	{
    		parentProjectSecurityRole.setCanCreateRisks(Boolean.TRUE);
    	}
    	else if(TEST_REQUIREMENT.equals(testClass))
    	{
    		parentProjectSecurityRole.setCanCreateRequirements(Boolean.TRUE);
    	}
    	else if(TEST_DEFECT.equals(testClass))
    	{
    		parentProjectSecurityRole.setCanCreateDefect(Boolean.TRUE);
    	}

    	parentProjectSecurityRole = (ProjectSecurityRole)updateSecurityRole(parentProjectSecurityRole);
    }

    protected void setUp() throws Exception
    {
        super.setUp();

        prepareParentProject();

        resetScopeElementScope();
    }

    protected void tearDown() throws Exception
    {
    	parentProjectSecurityRole.setCanDeleteWBSItems(Boolean.TRUE);
        parentProjectSecurityRole = (ProjectSecurityRole)updateSecurityRole(parentProjectSecurityRole);

        super.tearDown();
    }

    protected void prepareSecurityRole(Class scopeElementClass) throws Exception
    {
    	 scopeElementSecurityRole = (AbstractScopeSecurityRole) prepareSecurityRole(scopeElementClass,
                true);
    }

    protected void resetScopeElementScope()
    {
    	scopeElementScope = new ScopeElementScope();
        scopeElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
    }

    protected void runUpdatesFor(int fieldType) throws Exception
	{
		/* This Update should Fail */
    	loadOrUpdateField(fieldType, false, false, false);

		/* This update should Succeed */
    	loadOrUpdateField(fieldType, true, false, false);
	}

    protected void runLoadValueFor(int fieldType) throws Exception
    {
    	/* This load should return null value for the field */
    	loadOrUpdateField(fieldType, false, true, false);

    	/* Check for the null value for the field */
    	checkNullValueFor(fieldType,false, new Object[] {
    														new Object()
    													});

    	/* This load should return the value for the field */
    	loadOrUpdateField(fieldType, true, true, false);
    }

   protected void loadOrUpdateField(int fieldType, boolean isSecurityFlagOn, boolean isLoad, boolean found) throws Exception
    {
    	switch(fieldType)
		{
			case SCOPE_SECURITYROLE:
				if(isLoad)
                {
					scopeElementSecurityRole.setCanViewElementSecurity(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditElementSecurity(new Boolean(isSecurityFlagOn));
				}
				else
                {
					scopeElementSecurityRole.setCanEditElementSecurity(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewElementSecurity(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case SCOPE_RESOURCE:
				if(isLoad)
                {
					scopeElementSecurityRole.setCanViewResourceProfileCost(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditResourceProfileCost(new Boolean(isSecurityFlagOn));
				}
				else
                {
					scopeElementSecurityRole.setCanEditResourceProfileCost(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewResourceProfileCost(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
		}
    	if(!found)
    	{
    		throw new Exception("Field to edit is not found");
    	}
    	if(isLoad)
    	{
    		saveSecurityRoleAndloadScopeElement();
    	}
    	else
    	{
    		saveScopeElementAndSecurityRoleUpdates(isSecurityFlagOn);
    	}
    }

    protected void checkNullValueFor(int fieldType, boolean found, Object value[]) throws Exception
    {
    	switch(fieldType)
 	    {
 	   		case SCOPE_SECURITYROLE:
 	   			value = new Object[1];
 	   			value[0] = testScopeElement.getElementSecurityType();
 	   			found = true;
 	   			break;
 	   		case SCOPE_RESOURCE:
 	   			value = new Object[1];
 	   			value[0] = testScopeElement.getResourceTaskAssignments();
 	   			found = true;
 	   			break;
 	    }
 	    if(!found)
 	    {
 		    throw new Exception("Field to view is not found");
 	    }
 	    else
 	    {
 	    	if(value != null)
 	    	{
 	    		for(int i=0; i<value.length; i++)
 	    		{
 	    			assertNull(value[i]);
 	    		}
 	    	}
 	    }
 	}

    protected void createSaveScopeElementWithSecurity(ScopeElement scopeElement) throws Exception
    {
    	testScopeElement = scopeElement;
    	testScopeElement.setName(HelperTestUtilities.createName());
		testScopeElement.setParent(parent);
		testScopeElement = (ScopeElement)save(testScopeElement, scopeElementScope).getRpmObject();
    	assertNotNull(testScopeElement);

    	prepareSecurityRoleAssignment(testScopeElement, securityTestUser, scopeElementSecurityRole);
    }

    protected void updateSaveScopeElement(boolean isSecurityFlagOn) throws Exception
    {
    	ScopeElement returnElement = (ScopeElement)saveAndValidate(testScopeElement, scopeElementScope, isSecurityFlagOn);
    	if(isSecurityFlagOn)
		{
			testScopeElement = returnElement;
			assertNotNull(testScopeElement);
		}
    }

    protected void saveScopeElementAndSecurityRoleUpdates(boolean isSecurityFlagOn) throws Exception
    {
    	scopeElementSecurityRole = (AbstractScopeSecurityRole)updateSecurityRole(scopeElementSecurityRole);

		updateSaveScopeElement(isSecurityFlagOn);
	}

    protected void saveSecurityRoleAndloadScopeElement() throws Exception
    {
    	scopeElementSecurityRole = (AbstractScopeSecurityRole)updateSecurityRole(scopeElementSecurityRole);

    	testScopeElement = (ScopeElement)loadFromID(testScopeElement, scopeElementScope);
    }
    
    protected String modifyString(String stringValue)
	{
		String returnByName = "_MOD";
		if(stringValue != null)
		{
			returnByName = stringValue + returnByName;
		}
		return returnByName;
	}
	
    protected Boolean modifyBooleanValue(Boolean booleanValue)
    {
    	Boolean returnValue = Boolean.FALSE;
    	
    	if(booleanValue != null)
    	{
    		returnValue = new Boolean(!booleanValue.booleanValue());
    	}
    	return returnValue;
    }
    
	protected Calendar modifyDate(Calendar date)
	{
		Calendar returnByDate = Calendar.getInstance();
		if(date != null)
		{
			returnByDate = date;
			returnByDate.add(Calendar.DAY_OF_MONTH, 1);
		}
		return returnByDate;
	}
    
    protected void validateDate(Calendar expectedDate, Calendar actualDate) throws Exception
    {
    	assertEquals(expectedDate.get(Calendar.YEAR), actualDate.get(Calendar.YEAR));
    	assertEquals(expectedDate.get(Calendar.MONTH), actualDate.get(Calendar.MONTH));
    	assertEquals(expectedDate.get(Calendar.DAY_OF_MONTH), actualDate.get(Calendar.DAY_OF_MONTH));
    }

    /* MODIFY AND TEST THE SCOPE ELEMENT FIELDS */
    protected void runEditElementSecurity() throws Exception
    {
    	ScopeElementSecurityType scopeElementSecurityType = testScopeElement.getElementSecurityType();
    	if(ScopeElementSecurityType.ProjectLevel.getValue().equals(scopeElementSecurityType.getValue()))
    	{
    		scopeElementSecurityType = ScopeElementSecurityType.ByInvitation;
    	}
    	else
    	{
    		scopeElementSecurityType = ScopeElementSecurityType.ProjectLevel;
    	}
    	testScopeElement.setElementSecurityType(scopeElementSecurityType);

    	runUpdatesFor(SCOPE_SECURITYROLE);

    	assertEquals(scopeElementSecurityType, testScopeElement.getElementSecurityType());
    }

//    protected void runEditResourceProfileCosts() throws Exception
//    {
//    	for(int i=0; i<testScopeElement.getResourceTaskAssignments().length; i++)
//    	{
//    		if(!testScopeElement.getResourceTaskAssignments()[i].getResource().getFullName().equals(securityTestUser.getFullName()))
//    		{
//    			TaskAssignment taskAssignment= testScopeElement.getResourceTaskAssignments()[i].getTaskAssignment();
//    			double modifiedSpecialCostPerHour = 1.0;
//    			double modifiedRegularCostPerHour = 1.0;
//    			if(taskAssignment.getSpecialCostPerHour() != null)
//    			{
//    				modifiedSpecialCostPerHour += taskAssignment.getSpecialCostPerHour().doubleValue();
//    			}
//    			if(taskAssignment.getCostPerHour() != null)
//    			{
//    				modifiedRegularCostPerHour += taskAssignment.getCostPerHour().doubleValue();
//    			}
//    			taskAssignment.setSpecialCostPerHour(new Double(modifiedSpecialCostPerHour));
//    			taskAssignment.setCostPerHour(new Double(modifiedRegularCostPerHour));
//    			testScopeElement.getResourceTaskAssignments()[i].setTaskAssignment(taskAssignment);
//    		}
//    	}
//
//    	runUpdatesFor(SCOPE_RESOURCE);
//    }
}
