package com.ibm.rpm.client.junit.security.testcases.role.scopemanagement;

import java.util.Calendar;

import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.security.containers.IssueSecurityRole;

/**
 *
 * @author SandeepS
 *
 */
public class TestIssueSecurityTestLocal extends AbstractTestScopeSecurityTestLocal {

	protected void setUp() throws Exception
    {
    	testClass = TEST_ISSUE;
        super.setUp();

        prepareSecurityRole(Issue.class);
    }

    public void testEditIssueOccurredDateTime() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Issue());

    	Calendar issueOccurredDateTime = Calendar.getInstance();
    	((Issue)testScopeElement).setOccuredDateTime(issueOccurredDateTime);

    	runUpdatesFor(ISSUE_OCCURREDDATETIME);

    	assertNotNull(testScopeElement);
    	assertNotNull(((Issue)testScopeElement).getOccuredDateTime());
    	validateDate(issueOccurredDateTime, ((Issue)testScopeElement).getOccuredDateTime());
    }

    public void testEditTimesReassigned() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Issue());

    	int numberOfTimeReassigned = 0;
    	if(((Issue)testScopeElement).getNumberTimesReassigned() != null)
    	{
    		numberOfTimeReassigned += ((Issue)testScopeElement).getNumberTimesReassigned().intValue() + 1;
    	}
    	((Issue)testScopeElement).setNumberTimesReassigned(new Integer(numberOfTimeReassigned));

    	runUpdatesFor(ISSUE_TIMESREASSIGNED);

    	assertNotNull(testScopeElement);
    	assertEquals(new Integer(numberOfTimeReassigned), ((Issue)testScopeElement).getNumberTimesReassigned());
    }

    public void testViewTargetDate() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Issue());

    	runViewTargetDate();
    }

    public void testEditTargetDate() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Issue());

    	runEditTargetDate();
    }

    protected void loadOrUpdateField(int fieldType, boolean isSecurityFlagOn, boolean isLoad, boolean found) throws Exception
	{
    	IssueSecurityRole issueSecurityRole = (IssueSecurityRole)scopeElementSecurityRole;
		switch(fieldType)
		{
			case ISSUE_OCCURREDDATETIME:
				if(isLoad)
				{
					issueSecurityRole.setCanViewOccurredDateTime(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						issueSecurityRole.setCanEditOccurredDateTime(new Boolean(isSecurityFlagOn));
				}
				else
				{
					issueSecurityRole.setCanEditOccurredDateTime(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						issueSecurityRole.setCanViewOccurredDateTime(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ISSUE_TIMESREASSIGNED:
				if(isLoad)
				{
					issueSecurityRole.setCanViewTimesReassigned(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						issueSecurityRole.setCanEditTimesReassigned(new Boolean(isSecurityFlagOn));
				}
				else
				{
					issueSecurityRole.setCanEditTimesReassigned(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						issueSecurityRole.setCanViewTimesReassigned(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
		}
		super.loadOrUpdateField(fieldType, isSecurityFlagOn, isLoad, found);
	}

    protected void checkNullValueFor(int fieldType, boolean found, Object value[]) throws Exception
	{
		Issue issue = (Issue)testScopeElement;
		switch(fieldType)
		{
			case ISSUE_OCCURREDDATETIME:
				value = new Object[1];
				value[0] = issue.getOccuredDateTime();
				found = true;
				break;
			case ISSUE_TIMESREASSIGNED:
				value = new Object[1];
				value[0] = issue.getNumberTimesReassigned();
				found = true;
				break;
		}
		super.checkNullValueFor(fieldType,found,value);
	}
}
