package com.ibm.rpm.client.junit.security.testcases.role.scopemanagement;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.security.containers.RequirementSecurityRole;

/**
 *
 * @author SandeepS
 *
 */
public class TestRequirementSecurityTestLocal extends AbstractTestScopeSecurityTestLocal {

	protected void setUp() throws Exception
    {
		testClass = TEST_REQUIREMENT;
        super.setUp();

        prepareSecurityRole(Requirement.class);
    }

	public void testViewApprovedCostBenefits() throws Exception
	{
		resetScopeElementScope();
    	scopeElementScope.setApprovedCostBenefits(true);
    	createSaveScopeElementWithSecurity(new Requirement());

    	ApprovedCostBenefits approvedCostBenefits = new ApprovedCostBenefits();
        HelperTestUtilities.setByIntrospection(approvedCostBenefits,
            RPMObjectFixture.random(), RPMObjectFixture.skipSetters);
        ((Requirement)testScopeElement).setApprovedCostBenefits(approvedCostBenefits);

        updateSaveScopeElement(true);

        runLoadValueFor(REQUIREMENT_APPROVEDCOST_BENEFITS);

        assertNotNull(((Requirement)testScopeElement).getApprovedCostBenefits());
	}

	public void testEditApprovedCostBenefits() throws Exception
    {
    	resetScopeElementScope();
    	scopeElementScope.setApprovedCostBenefits(true);
    	createSaveScopeElementWithSecurity(new Requirement());

    	ApprovedCostBenefits approvedCostBenefits = new ApprovedCostBenefits();
        HelperTestUtilities.setByIntrospection(approvedCostBenefits,
            RPMObjectFixture.random(), RPMObjectFixture.skipSetters);
        ((Requirement)testScopeElement).setApprovedCostBenefits(approvedCostBenefits);

        runUpdatesFor(REQUIREMENT_APPROVEDCOST_BENEFITS);

        assertNotNull(((Requirement)testScopeElement).getApprovedCostBenefits());
    }

    public void testEditState() throws Exception
    {
    	resetScopeElementScope();
    	scopeElementScope.setState(true);
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditState();
    }

    public void testEditRank() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditRank();
    }

    public void testEditEstimatedDates() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditEstimatedDates();
    }

    public void testEditOrderOfMagnitudeCostBenefits() throws Exception
    {
    	resetScopeElementScope();
    	scopeElementScope.setOrderOfMagnitudeCostBenefits(true);
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditOrderOfMagnitudeCostBenefits();
    }

    public void testEditPriority() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditPriority();
    }

    public void testEditAttributeAssignments() throws Exception
    {
    	resetScopeElementScope();
    	scopeElementScope.setAttributeAssignments(true);
    	createSaveScopeElementWithAttributeAssignments(new Requirement());

    	runEditAttributeAssignments();
    }

    public void testEditElementSecurityType() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Requirement());

    	runEditElementSecurity();
    }

    public void testEditScorecardAssignments() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithScorecardAssignments(new Requirement());

    	runEditScorecardAssignments();
    }
    
    public void testEditDefaultScope() throws Exception
    {
    	resetScopeElementScope();
    	createSaveScopeElementWithSecurity(new Requirement());
    	
    	runEditDefaultScope();
    }
    
    public void testViewDuplicates() throws Exception
    {
    	Duplicate duplicate = prepareScopeAndDuplicateContainers();
    	
    	runViewDuplicates(duplicate);
    }
    
    public void testEditDuplicates() throws Exception
    {
    	Duplicate duplicate = prepareScopeAndDuplicateContainers();
    	
    	runEditDuplicates(duplicate);
    }

    public void testEditDuplicateNumber() throws Exception
    {
    	Duplicate duplicate = prepareScopeAndDuplicateContainers();
    	
    	runEditDuplicateNumbers(duplicate);
    }
    
    /* There is some issue in Client while updating the costs 
     * Check with Cober
     */
//    public void testEditResourceProfileCosts() throws Exception
//    {
//    	resetScopeElementScope();
//
//    	createScopeForEditResourceProfileCost();
//
//    	createSaveScopeElementWithSecurity(new Requirement());
//
//    	createSaveResourceTaskAssignment();
//
//    	runEditResourceProfileCosts();
//    }

    protected void loadOrUpdateField(int fieldType, boolean isSecurityFlagOn, boolean isLoad, boolean found) throws Exception
	{
    	RequirementSecurityRole requirementSecurityRole = (RequirementSecurityRole)scopeElementSecurityRole;
		switch(fieldType)
		{
			case REQUIREMENT_APPROVEDCOST_BENEFITS:
				if(isLoad)
				{
					requirementSecurityRole.setCanViewApprovedCostBenefits(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						requirementSecurityRole.setCanEditApprovedCostBenefits(new Boolean(isSecurityFlagOn));
				}		
				else
				{
					requirementSecurityRole.setCanEditApprovedCostBenefits(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						requirementSecurityRole.setCanViewApprovedCostBenefits(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
		}
		super.loadOrUpdateField(fieldType, isSecurityFlagOn, isLoad, found);
	}

    protected void checkNullValueFor(int fieldType, boolean found, Object value[]) throws Exception
	{
		Requirement requirement = (Requirement)testScopeElement;
		switch(fieldType)
		{
			case REQUIREMENT_APPROVEDCOST_BENEFITS:
				value = new Object[1];
				value[0] = requirement.getApprovedCostBenefits();
				found = true;
				break;
		}
		super.checkNullValueFor(fieldType,found,value);
	}

    /* OPERATIONS */
    public void testTransferScopeElementsToWBSView() throws Exception
    {
    	resetScopeElementScope();
    	scopeElementScope.setOrderOfMagnitudeCostBenefits(true);
    	createSaveScopeElementWithSecurity(new Requirement());

    	runCanTransferScopeElementsToWBSView();
    }

    private Duplicate prepareScopeAndDuplicateContainers() throws Exception
    {
        resetScopeElementScope();
        scopeElementScope.setParent(new RPMObjectScope());
        scopeElementScope.setDuplicates(true);
        createSaveScopeElementWithSecurity(new Requirement());
        Duplicate duplicate = createSaveDuplicate(new Requirement());
    
        return duplicate;
    }
    
//    private void createSaveResourceTaskAssignment() throws Exception
//    {
//    	Resource resource = createResource();
//    	ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture.createFixture(testScopeElement, resource, null, scopeElementScope.getResourceTaskAssignments(), true, this);
//    	testScopeElement.setResourceTaskAssignments(new ResourceTaskAssignment[]{resourceTaskAssignment});
//
//    	testScopeElement = (Requirement)save(testScopeElement,scopeElementScope).getRpmObject();
//
//    	assertNotNull(testScopeElement);
//    	assertNotNull(testScopeElement.getResourceTaskAssignments());
//    	assertEquals(testScopeElement.getResourceTaskAssignments().length, 2);
//    	assertNotNull(testScopeElement.getResourceTaskAssignments()[0]);
//    	assertNotNull(testScopeElement.getResourceTaskAssignments()[1]);
//    }
//
//    private Resource createResource() throws Exception
//    {
//    	ResourceScope resourceScope = new ResourceScope();
//    	resourceScope.setCalendar(new CalendarScope());
//    	Resource resource = ResourceFixture.createSaveFixture(resourceScope, this);
//
//    	assertNotNull(resource);
//    	return resource;
//    }
//
//    private void createScopeForEditResourceProfileCost()
//    {
//    	ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
//    	resourceTaskAssignmentScope.setResource(new ResourceScope());
//    	resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());
//    	scopeElementScope.setResourceTaskAssignments(resourceTaskAssignmentScope);
//    }
}
