/**
 * @author gmeloche
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class CreditTimesheetFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        ////////////
        //Setters
        ////////////
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setWeekOf"); // Primary key

        // Skip these because they have to be in a certain range
        localSkipSetters.add("setRegularFriday");
        localSkipSetters.add("setRegularMonday");
        localSkipSetters.add("setRegularSaturday");
        localSkipSetters.add("setRegularSunday");
        localSkipSetters.add("setRegularThursday");
        localSkipSetters.add("setRegularTuesday");
        localSkipSetters.add("setRegularWednesday");
        localSkipSetters.add("setSpecialFriday");
        localSkipSetters.add("setSpecialMonday");
        localSkipSetters.add("setSpecialSaturday");
        localSkipSetters.add("setSpecialSunday");
        localSkipSetters.add("setSpecialThursday");
        localSkipSetters.add("setSpecialTuesday");
        localSkipSetters.add("setSpecialWednesday");

        // Read-only
        localSkipSetters.add("setBillable");

        ////////////
        //Getters
        ////////////
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        // set in the SP call
        localSkipGetters.add("getBillable");
    }

    public static CreditTimesheet createSaveFixture(Credit parent,
            Calendar weekOf, Calendar creditWeekOf, RPMObjectScope scope, ClientTestController ctc)
            throws Exception
    {
        CreditTimesheet timesheet = createFixture(parent, weekOf, creditWeekOf, scope, ctc);
        timesheet = (CreditTimesheet) save(ctc, timesheet, scope);
        return timesheet;
    }

    public static CreditTimesheet createFixture(Credit parent,
            Calendar weekOf, Calendar creditWeekOf, RPMObjectScope scope, ClientTestController ctc)
            throws Exception
    {
        CreditTimesheet timesheet = new CreditTimesheet();
        timesheet.setParent(parent);
        
        TaskAssignment taskAssignment = parent.getTaskAssignment();
        
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture
            .createSaveFixture(taskAssignment, weekOf, null, false, ctc);
       
        timesheet.setWeekOf(summaryTimesheet.getWeekOf());
        timesheet.setCreditWeekOf(creditWeekOf);

        updateFixture(timesheet);

        return timesheet;
    }

    public static CreditTimesheet updateFixture(CreditTimesheet timesheet)
            throws Exception
    {
       
        if (timesheet.getID() == null)
        {
            TimesheetTestUtil.setWorkedHours(timesheet, 2, 1);
        }
        else
        {
            TimesheetTestUtil.setWorkedHours(timesheet, 3, 2);
        }

        return timesheet;
    }

    public static void compare(CreditTimesheet[] array1,
            CreditTimesheet[] array2, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;        
        
        for (int i = 0; i < array1.length; i++){
            boolean found = false;
            for (int j = 0; j < array2.length; j++)
            {
                if(array1[i].getWeekOf().equals(array2[j].getWeekOf())){
                    CreditTimesheetFixture.compare(array1[i], array2[j], scope);
                    found = true;
                }
            }
            if(!found ){
                Assert.fail("StepTimesheet for weekOf = " + DateUtil.convertToString(array1[i].getWeekOf()) + " not loaded.");
            }
        }
    }

    public static void compare(CreditTimesheet obj1, CreditTimesheet obj2,
            RPMObjectScope scope) throws AssertionFailedError, Exception
    {

        obj1 = (CreditTimesheet)TimesheetTestUtil.initWorkingHours(obj1);
        obj2 = (CreditTimesheet)TimesheetTestUtil.initWorkingHours(obj2);

        RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);

        if (scope != null)
        {
            if (scope.getParent() != null)
            {
                StepFixture.compare((Step) obj1.getParent(),
                    (Step) obj2.getParent(), (StepScope) scope
                        .getParent());
            }
        }
    }

}
