package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;

/**
 * @author gmeloche
 *  
 */
public class DefaultAdministrativeTaskFixture extends RPMObjectFixture
{

    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {

        //Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);

        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
    }

    public static DefaultAdministrativeTask createSaveFixture(Pool parent, ClientTestController ctc)
            throws Exception
    {

        DefaultAdministrativeTask adminTask = createFixture(parent, ctc);
        adminTask = (DefaultAdministrativeTask) save(ctc, adminTask, null);

        return adminTask;

    }

    public static DefaultAdministrativeTask createFixture(Pool parent,
            ClientTestController ctc) throws Exception
    {

        DefaultAdministrativeTask adminTask = new DefaultAdministrativeTask();

        if (parent == null)
        {
            parent = PoolFixture.createSaveFixture(ctc);

        }

        adminTask.setParent(parent);
        HelperTestUtilities.setByIntrospection(adminTask, new Double(Math
            .random() * 100).intValue(), localSkipSetters);

        return adminTask;

    }

    public static DefaultAdministrativeTask updateFixture(DefaultAdministrativeTask adminTask,
            ClientTestController ctc) throws Exception
    {

        HelperTestUtilities.setByIntrospection(adminTask, new Double(Math
            .random() * 100).intValue(), localSkipSetters);

        return adminTask;

    }

    public static void compare(DefaultAdministrativeTask t1, DefaultAdministrativeTask t2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(t1, t2, localSkipGetters);
    }

    public static void compare(DefaultAdministrativeTask[] pArray1,
            DefaultAdministrativeTask[] pArray2)
            throws AssertionFailedError, Exception
    {
        pArray1 = removeDeletedObject(pArray1);

        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            DefaultAdministrativeTaskFixture.compare(pArray1[i], pArray2[i]);
        }
    }

    public static DefaultAdministrativeTask[] removeDeletedObject(
            DefaultAdministrativeTask[] pArray)
    {

        DefaultAdministrativeTask[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new DefaultAdministrativeTask[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

}
