/**
 * Created on Dec 12, 2005
 * 
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.framework.testcases.util.TimeCodeUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class DefaultStepFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        // Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);

        // Getters
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
    }

    public static DefaultStep createSaveFixture(GenericProject parent, 
            DefaultStepScope scope, ClientTestController ctc) 
    	throws Exception 
    {
        DefaultStep defaultStep = createFixture(parent, scope, ctc);        
        defaultStep = (DefaultStep) save(ctc, defaultStep, scope);
        return defaultStep;
    }
    
    public static DefaultStep createFixture(GenericProject parent, 
            DefaultStepScope scope, ClientTestController ctc) 
    	throws Exception
    {
        DefaultStep defaultStep = new DefaultStep();
        defaultStep.setParent(parent);
        
        HelperTestUtilities.setByIntrospection(defaultStep, random(),
            localSkipSetters);
        
        if (scope != null)
        {
            if (scope.isTimeCodes())
                defaultStep = assignTimeCodes(defaultStep, ctc);
        }
        
        return defaultStep;
    }

    public static DefaultStep updateFixture(DefaultStep defaultStep, 
            DefaultStepScope scope, ClientTestController ctc) 
    	throws Exception 
    {
        int index = random();
        HelperTestUtilities.setByIntrospection(defaultStep, index, localSkipSetters);
        
        if (scope != null)
        {
            if (scope.isTimeCodes()) 
                defaultStep = assignTimeCodes(defaultStep, ctc);
        }
        
        return defaultStep;
    }
    
    public static DefaultStepScope createScope(boolean isParent, 
            boolean isTimeCode)
    {
        DefaultStepScope scope = new DefaultStepScope();

        if (isParent)
            scope.setParent(new WorkElementScope());

        if (isTimeCode)
            scope.setTimeCodes(true);

        return scope;
    }

    public static void compare(DefaultStep[] array1, 
            DefaultStep[] array2, DefaultStepScope scope)
    	throws AssertionFailedError, Exception
	{
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null) return;
		
		for(int i = 0; i < array1.length; i++)
		    DefaultStepFixture.compare(array1[i], array2[i], scope);
	}
    
    public static void compare(DefaultStep obj1, DefaultStep obj2, 
            DefaultStepScope scope) throws AssertionFailedError, Exception
	{
//        logger.info("======================");
//        logger.info("ObjectToCompare: " + obj1);
//        logger.info("======================");
//        logger.info("LoadedObject: " + obj2);
        
		RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);
		
        if (scope != null)
        {
            if (scope.getParent() != null) 
            {
                WorkElementFixture.compare((WorkElement)obj1.getParent(), 
                    (WorkElement)obj1.getParent(), (WorkElementScope)scope.getParent());
            }
            if (scope.isTimeCodes())
            {
                TimecodeFixture.compare(obj1.getTimeCode1(), obj2.getTimeCode1(), 
                    null);
                TimecodeFixture.compare(obj1.getTimeCode2(), obj2.getTimeCode2(), 
                    null);
            }
        }
	}
    
    private static DefaultStep assignTimeCodes(DefaultStep defaultStep, 
            ClientTestController ctc) throws Exception
    {
        GenericProject project = (GenericProject) defaultStep.getParent();
        TimeCodeAssignment[] timeCodeAssignments = project
            .getTimeCodeAssignments();

        if (timeCodeAssignments == null)
        {
            // create some TimeCodeAssignmnets
            TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
                .createSaveFixture(project, true, ctc);
            TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
                .createSaveFixture(project, false, ctc);
            timeCodeAssignments = new TimeCodeAssignment[2];
            timeCodeAssignments[0] = timecode1;
            timeCodeAssignments[1] = timecode2;
        }

        boolean timeCode1Assigned = false, timeCode2Assigned = false;
        for (int i = 0; i < timeCodeAssignments.length && !timeCode1Assigned
                && !timeCode2Assigned; i++)
        {
            TimeCode timeCode = timeCodeAssignments[i].getTimeCode();
            
            // re-load timecode with the parent in scope
            TimeCodeScope scope = new TimeCodeScope();
            scope.setParent(scope);
            timeCode = (TimeCode) ctc.loadFromID(timeCode, scope);

            if (!timeCode1Assigned && TimeCodeUtil.isChildOfTimeCode1(timeCode))
            {
                defaultStep.setTimeCode1(timeCode);
                timeCode1Assigned = true;
            }
            else if (!timeCode2Assigned
                    && TimeCodeUtil.isChildOfTimeCode2(timeCode))
            {
                defaultStep.setTimeCode2(timeCode);
                timeCode2Assigned = true;
            }
        }

        return defaultStep;
    }
}
