/**
 * Created on Dec 6, 2005
 * 
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class StepTimesheetFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        ////////////
        //Setters
        ////////////
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setWeekOf"); // Primary key

        // Skip these because they have to be in a certain range
        localSkipSetters.add("setRegularFriday");
        localSkipSetters.add("setRegularMonday");
        localSkipSetters.add("setRegularSaturday");
        localSkipSetters.add("setRegularSunday");
        localSkipSetters.add("setRegularThursday");
        localSkipSetters.add("setRegularTuesday");
        localSkipSetters.add("setRegularWednesday");
        localSkipSetters.add("setSpecialFriday");
        localSkipSetters.add("setSpecialMonday");
        localSkipSetters.add("setSpecialSaturday");
        localSkipSetters.add("setSpecialSunday");
        localSkipSetters.add("setSpecialThursday");
        localSkipSetters.add("setSpecialTuesday");
        localSkipSetters.add("setSpecialWednesday");

        // Read-only
        localSkipSetters.add("setBillable");

        ////////////
        //Getters
        ////////////
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        // set in the SP call
        localSkipGetters.add("getBillable");
    }

    public static StepTimesheet createSaveFixture(Step parent,
            Calendar weekOf, RPMObjectScope scope, ClientTestController ctc)
            throws Exception
    {
        StepTimesheet timesheet = createFixture(parent, weekOf, scope, ctc);
        timesheet = (StepTimesheet) save(ctc, timesheet, scope);
        return timesheet;
    }

    public static StepTimesheet createFixture(Step parent,
            Calendar weekOf, RPMObjectScope scope, ClientTestController ctc)
            throws Exception
    {
        return createFixture(parent, weekOf, scope, null, ctc);
    }
    
    public static StepTimesheet createFixture(Step parent,
            Calendar weekOf, RPMObjectScope scope, SummaryTimesheetScope summaryTimesheetScope, ClientTestController ctc)
            throws Exception
    {
        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setParent(parent);
        
        TaskAssignment taskAssignment = parent.getTaskAssignment();
        
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture
            .createSaveFixture(taskAssignment, weekOf, summaryTimesheetScope, false, ctc);
             
        timesheet.setWeekOf(summaryTimesheet.getWeekOf());

        updateFixture(timesheet);

        // update the step's timesheets
        if (scope != null && scope.getParent() != null)
        {
            StepTimesheet[] temp = parent.getStepTimesheets();
            StepTimesheet[] timesheets = null;
            if (temp == null || temp.length == 0)
            {
                timesheets = new StepTimesheet[1];
            }
            else
            {
                timesheets = new StepTimesheet[temp.length + 1];
                for (int i = 0; i < temp.length; i++)
                    timesheets[i] = temp[i];
            }
            timesheets[timesheets.length - 1] = timesheet;
            parent.setStepTimesheets(timesheets);
        }

        return timesheet;
    }

    public static StepTimesheet updateFixture(StepTimesheet timesheet)
            throws Exception
    {

        if (timesheet.getID() == null)
        {
            TimesheetTestUtil.setWorkedHours(timesheet, 5, 2);
        }
        else
        {
            TimesheetTestUtil.setWorkedHours(timesheet, 6, 3);
        }
        
        return timesheet;
    }

    public static RPMObjectScope createScope(boolean isParent)
    {
        RPMObjectScope scope = new RPMObjectScope();

        if (isParent)
            scope.setParent(StepFixture.createScope(true, false, false,
                false, true, true));

        return scope;
    }

    public static void compare(StepTimesheet[] array1,
            StepTimesheet[] array2, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;        
        
        for (int i = 0; i < array1.length; i++){
            boolean found = false;
            for (int j = 0; j < array2.length; j++)
            {
                if(array1[i].getWeekOf().equals(array2[j].getWeekOf())){
                    StepTimesheetFixture.compare(array1[i], array2[j], scope);
                    found = true;
                }
            }
            if(!found ){
                Assert.fail("StepTimesheet for weekOf = " + DateUtil.convertToString(array1[i].getWeekOf()) + " not loaded.");
            }
        }
    }

    public static void compare(StepTimesheet obj1, StepTimesheet obj2,
            RPMObjectScope scope) throws AssertionFailedError, Exception
    {
        //        logger.info("======================");
        //        logger.info("ObjectToCompare: " + obj1);
        //        logger.info("======================");
        //        logger.info("LoadedObject: " + obj2);

        obj1 = (StepTimesheet)TimesheetTestUtil.initWorkingHours(obj1);
        obj2 = (StepTimesheet)TimesheetTestUtil.initWorkingHours(obj2);

        RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);

        if (scope != null)
        {
            if (scope.getParent() != null)
            {
                StepFixture.compare((Step) obj1.getParent(),
                    (Step) obj2.getParent(), (StepScope) scope
                        .getParent());
            }
        }
    }
}
