package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author gmeloche
 *  
 */
public class TimesheetTestUtil extends RPMObjectFixture
{
    //week index used for automatically generated timesheet in StepFixture
    public static int TIMESHEET1_WEEK = 0;

    //week index used for automatically generated timesheet in StepFixture
    public static int TIMESHEET2_WEEK = 1;

    public static int CREDIT_WEEK_OF = 2;

    public static int CURRENT_WEEK = 3;

    //week index used in validation to valdate timeshet without
    // summaryTimesheet
    public static int OTHER_WEEK = 4;

    private static String PROJECT_FIXTURE_NAME = "STATIC_TestTimesheet_Project";

    private static String POOL_FIXTURE_NAME = "STATIC_TestTimesheet_Pool";

    /**
     * Returns a valid week of according to the assignmentStartDate.
     * 
     * @param assignmentStartDate
     * @param weekIndex
     * @return
     */

    public static Calendar getValidWeekOf(Calendar assignmentStartDate,
            int weekIndex)
    {
        if (assignmentStartDate == null)
        {
            return null;
        }

        int year = assignmentStartDate.get(Calendar.YEAR);
        int month = assignmentStartDate.get(Calendar.MONTH);
        int day = assignmentStartDate.get(Calendar.DAY_OF_MONTH);
        
        Calendar weekOf = new GregorianCalendar(year, month, day);
        weekOf.add(Calendar.WEEK_OF_YEAR, weekIndex);

        logger.debug("StartDate: "
                + DateUtil.convertToString(assignmentStartDate) + ", Week "
                + weekIndex + " : " + DateUtil.convertToString(weekOf));
        
        return weekOf;

    }

    private static Project loadStaticProject(ClientTestController ctc)
            throws Exception
    {
        // Try loading the WorkElement from xpath
        RPMObject[] rpmObjects = null;
        Project project = null;

        rpmObjects = ctc.loadFromXpath("/Project[name='" + PROJECT_FIXTURE_NAME
                + "']", null);

        if (rpmObjects != null && rpmObjects.length > 0
                && rpmObjects[0] != null)
        {
            project = (Project) rpmObjects[0];
        }
        else
        {

            project = new Project();
            project.setName(PROJECT_FIXTURE_NAME);

            Calendar calendar = Calendar.getInstance();
            calendar.set(2006, 06, 01, 00, 00);
            project.setEstimatedStartDate(calendar);

            WbsScheduleDate expected = new WbsScheduleDate();
            expected.setStartDate(calendar);
            project.setExpectedDate(expected);

            WorkElementScope scope = new WorkElementScope();
            scope.setScheduleDates(true);

            project = (Project) saveWithoutDelete(ctc, project, scope);
        }
        return project;
    }

    private static Project createProject(ClientTestController ctc)
            throws Exception
    {

        Project project = new Project();
        project.setName(PROJECT_FIXTURE_NAME);

        Calendar calendar = Calendar.getInstance();
        calendar.set(2006, 06, 01, 00, 00);
        project.setEstimatedStartDate(calendar);

        WbsScheduleDate expected = new WbsScheduleDate();
        expected.setStartDate(calendar);
        project.setExpectedDate(expected);

        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);

        project = (Project) saveWithoutDelete(ctc, project, scope);

        return project;
    }

    private static Task createTask(ClientTestController ctc) throws Exception
    {

        Project project = createProject(ctc);

        Task task = new Task();
        task.setParent(project);
        task.setName(HelperTestUtilities.createName());

        //Dates are set for resource task assignment.
        Calendar calendar = Calendar.getInstance();
        calendar.set(2006, 06, 01, 00, 00);
        task.setEstimatedStartDate(calendar);

        WbsScheduleDate expected = new WbsScheduleDate();
        expected.setStartDate(calendar);
        task.setExpectedDate(expected);

        WbsScheduleDate proposed = new WbsScheduleDate();
        proposed.setStartDate(calendar);
        task.setProposedDate(proposed);

        WorkElementScope scope = new WorkElementScope();
        //containingProject is mandatory for creating a ResourceTaskAssignment
        scope.setContainingProject(new WorkElementScope());
        scope.setScheduleDates(true);

        task = (Task) save(ctc, task, scope);

        return task;
    }

    private static Pool loadStaticPool(ClientTestController ctc)
            throws Exception
    {
        Pool pool = null;

        RPMObject[] rpmObjects = ctc.loadFromXpath("/pool" + "[name='"
                + POOL_FIXTURE_NAME + "']", null);

        if (rpmObjects != null && rpmObjects.length > 0
                && rpmObjects[0] != null)
        {
            pool = (Pool) rpmObjects[0];
        }
        else
        {

            pool = PoolFixture.createFixture(false, 0, null, ctc);
            pool.setName(POOL_FIXTURE_NAME);

            pool = (Pool) saveWithoutDelete(ctc, pool, null);

        }

        return pool;
    }

    private static Resource createResource(ClientTestController ctc)
            throws Exception
    {
        Pool pool = loadStaticPool(ctc);

        Resource resource = null;

        ResourceScope scope = new ResourceScope();
        scope.setCalendar(new CalendarScope());

        resource = ResourceFixture.createFixture(pool, scope, ctc);
        resource.setEffectiveStartDate(new GregorianCalendar(2006, 00, 01));

        resource = (Resource) save(ctc, resource, scope);

        return resource;
    }

    //not used for now because i didn't find a way to reuse either a project
    // or a resource (problem in ResourceTaskAssignment)

    public static TaskAssignment createTaskAssignmentFromStaticObjects(
            ClientTestController ctc) throws Exception
    {

        WorkElement we = createTask(ctc);

        Resource res = createResource(ctc);

        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(we, res, null, ctc);

        return taskAssignment;

    }

    public static void initProject(GenericProject project,
            ClientTestController ctc) throws AssertionError, Exception
    {

        if (!(project.getPublished().equals(ProjectPublishedType.ToTeam)))
        {
            // Publish the project to the team
            project.setPublished(ProjectPublishedType.ToTeam);
            project = (GenericProject) saveWithoutDelete(ctc, project, null);
        }

        WbsScheduleDate planDates = project.getPlanDate();
        if (planDates == null || planDates.getStartDate() == null
                || planDates.getFinishDate() == null)
        {
            // Copy proposed to plan
            project = (GenericProject) ctc.getApplicationInterface()
                .copyProposedToPlan(ClientTestController.sessionid, project,
                    null).getRpmObject();
        }
    }

    public static AbstractTimesheet initWorkingHours(AbstractTimesheet timesheet)
    {
        Double zero = new Double(0);

        if (timesheet == null)
            return null;

        if (timesheet.getRegularFriday() == null)
            timesheet.setRegularFriday(zero);
        if (timesheet.getRegularMonday() == null)
            timesheet.setRegularMonday(zero);
        if (timesheet.getRegularSaturday() == null)
            timesheet.setRegularSaturday(zero);
        if (timesheet.getRegularSunday() == null)
            timesheet.setRegularSunday(zero);
        if (timesheet.getRegularThursday() == null)
            timesheet.setRegularThursday(zero);
        if (timesheet.getRegularTuesday() == null)
            timesheet.setRegularTuesday(zero);
        if (timesheet.getRegularWednesday() == null)
            timesheet.setRegularWednesday(zero);
        if (timesheet.getSpecialFriday() == null)
            timesheet.setSpecialFriday(zero);
        if (timesheet.getSpecialMonday() == null)
            timesheet.setSpecialMonday(zero);
        if (timesheet.getSpecialSaturday() == null)
            timesheet.setSpecialSaturday(zero);
        if (timesheet.getSpecialSunday() == null)
            timesheet.setSpecialSunday(zero);
        if (timesheet.getSpecialThursday() == null)
            timesheet.setSpecialThursday(zero);
        if (timesheet.getSpecialTuesday() == null)
            timesheet.setSpecialTuesday(zero);
        if (timesheet.getSpecialWednesday() == null)
            timesheet.setSpecialWednesday(zero);

        return timesheet;
    }

    public static void compareTotalsWithSummaryTimesheet(
            AbstractTimesheet timesheet, ClientTestController ctc)
            throws AssertionError, Exception
    {

        if (timesheet == null || timesheet instanceof SummaryTimesheet)
        {
            return;
        }

        RPMObjectScope timesheetScope = new RPMObjectScope();
        StepScope stepScope = new StepScope();
        stepScope.setTaskAssignment(true);
        timesheetScope.setParent(stepScope);

        AbstractTimesheet loadedTimesheet = (AbstractTimesheet) ctc.loadFromID(
            timesheet, timesheetScope);

        String weekOf = DateUtil.convertToString(loadedTimesheet.getWeekOf());
        String taskAssignmentId = ((GenericStep) loadedTimesheet.getParent())
            .getTaskAssignment().getID();

        String stepXpath = "/Step[taskAssignment[id = '" + taskAssignmentId
                + "']]/StepTimesheet[weekOf = '" + weekOf + "']";
        String creditXpath = "/Credit[taskAssignment[id = '" + taskAssignmentId
                + "']]/CreditTimesheet[weekOf = '" + weekOf + "']";

        ArrayList timesheets = new ArrayList();
        RPMObject[] stepTimesheets = ctc.loadFromXpath(stepXpath, null);
        RPMObject[] creditTimesheets = ctc.loadFromXpath(creditXpath, null);

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setID(taskAssignmentId);
        summaryTimesheet.setWeekOf(loadedTimesheet.getWeekOf());
        summaryTimesheet = (SummaryTimesheet) ctc.loadFromID(summaryTimesheet,
            null);
        initWorkingHours(summaryTimesheet);

        if (stepTimesheets != null)
        {
            for (int i = 0; i < stepTimesheets.length; i++)
            {
                timesheets.add(stepTimesheets[i]);
            }
        }

        if (creditTimesheets != null)
        {
            for (int i = 0; i < creditTimesheets.length; i++)
            {
                timesheets.add(creditTimesheets[i]);
            }
        }

        double regMonday = 0;
        double regTuesday = 0;
        double regWednesday = 0;
        double regThursday = 0;
        double regfriday = 0;
        double regSaturday = 0;
        double regSunday = 0;
        double specMonday = 0;
        double specTuesday = 0;
        double specWednesday = 0;
        double specThursday = 0;
        double specFriday = 0;
        double specSaturday = 0;
        double specSunday = 0;
        
        if (timesheets != null && !timesheets.isEmpty())
        {
            for (Iterator iter = timesheets.iterator(); iter.hasNext();)
            {
                AbstractTimesheet ts = (AbstractTimesheet) iter.next();
                initWorkingHours(ts);
                double factor = 1;
                if (ts instanceof CreditTimesheet)
                {
                    factor = -1;
                }
                regMonday += factor * ts.getRegularMonday().doubleValue();
                regTuesday += factor * ts.getRegularTuesday().doubleValue();
                regWednesday += factor * ts.getRegularWednesday().doubleValue();
                regThursday += factor * ts.getRegularThursday().doubleValue();
                regfriday += factor * ts.getRegularFriday().doubleValue();
                regSaturday += factor * ts.getRegularSaturday().doubleValue();
                regSunday += factor * ts.getRegularSunday().doubleValue();
                specMonday += factor * ts.getSpecialMonday().doubleValue();
                specTuesday += factor * ts.getSpecialTuesday().doubleValue();
                specWednesday += factor
                        * ts.getSpecialWednesday().doubleValue();
                specThursday += factor * ts.getSpecialThursday().doubleValue();
                specFriday += factor * ts.getSpecialFriday().doubleValue();
                specSaturday += factor * ts.getSpecialSaturday().doubleValue();
                specSunday += factor * ts.getSpecialSunday().doubleValue();
            }
        }

        ClientTestController.assertEquals(regMonday, summaryTimesheet
            .getRegularMonday().doubleValue(), 0);
        ClientTestController.assertEquals(regTuesday, summaryTimesheet
            .getRegularTuesday().doubleValue(), 0);
        ClientTestController.assertEquals(regWednesday, summaryTimesheet
            .getRegularWednesday().doubleValue(), 0);
        ClientTestController.assertEquals(regThursday, summaryTimesheet
            .getRegularThursday().doubleValue(), 0);
        ClientTestController.assertEquals(regfriday, summaryTimesheet
            .getRegularFriday().doubleValue(), 0);
        ClientTestController.assertEquals(regSaturday, summaryTimesheet
            .getRegularSaturday().doubleValue(), 0);
        ClientTestController.assertEquals(regSunday, summaryTimesheet
            .getRegularSunday().doubleValue(), 0);
        ClientTestController.assertEquals(specMonday, summaryTimesheet
            .getSpecialMonday().doubleValue(), 0);
        ClientTestController.assertEquals(specTuesday, summaryTimesheet
            .getSpecialTuesday().doubleValue(), 0);
        ClientTestController.assertEquals(specWednesday, summaryTimesheet
            .getSpecialWednesday().doubleValue(), 0);
        ClientTestController.assertEquals(specThursday, summaryTimesheet
            .getSpecialThursday().doubleValue(), 0);
        ClientTestController.assertEquals(specFriday, summaryTimesheet
            .getSpecialFriday().doubleValue(), 0);
        ClientTestController.assertEquals(specSaturday, summaryTimesheet
            .getSpecialSaturday().doubleValue(), 0);
        ClientTestController.assertEquals(specSunday, summaryTimesheet
            .getSpecialSunday().doubleValue(), 0);

    }

    public static void compareTotalStepWithSummaryTimesheet(
            GenericStep genericStep, ClientTestController ctc)
            throws AssertionError, Exception
    {     
        StepScope stepScope = new StepScope();
        stepScope.setTaskAssignment(true);
    
        AbstractTimesheet loadedTimesheet = (AbstractTimesheet) ctc.loadFromID(
            genericStep, stepScope);
    
        String taskAssignmentId = ((GenericStep) loadedTimesheet.getParent())
            .getTaskAssignment().getID();
    
        String stepXpath = "/Step[taskAssignment[id = '" + taskAssignmentId
                + "']]";
        String creditXpath = "/Credit[taskAssignment[id = '" + taskAssignmentId
                + "']]";
    
        ArrayList timesheets = new ArrayList();
        RPMObject[] stepTimesheets = ctc.loadFromXpath(stepXpath, null);
        RPMObject[] creditTimesheets = ctc.loadFromXpath(creditXpath, null);
    
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setID(taskAssignmentId);
        summaryTimesheet.setWeekOf(loadedTimesheet.getWeekOf());
        summaryTimesheet = (SummaryTimesheet) ctc.loadFromID(summaryTimesheet,
            null);
        initWorkingHours(summaryTimesheet);
    
        if (stepTimesheets != null)
        {
            for (int i = 0; i < stepTimesheets.length; i++)
            {
                timesheets.add(stepTimesheets[i]);
            }
        }
    
        if (creditTimesheets != null)
        {
            for (int i = 0; i < creditTimesheets.length; i++)
            {
                timesheets.add(creditTimesheets[i]);
            }
        }
    
        double regMonday = 0;
        double regTuesday = 0;
        double regWednesday = 0;
        double regThursday = 0;
        double regfriday = 0;
        double regSaturday = 0;
        double regSunday = 0;
        double specMonday = 0;
        double specTuesday = 0;
        double specWednesday = 0;
        double specThursday = 0;
        double specFriday = 0;
        double specSaturday = 0;
        double specSunday = 0;
        
        if (timesheets != null && !timesheets.isEmpty())
        {
            for (Iterator iter = timesheets.iterator(); iter.hasNext();)
            {
                AbstractTimesheet ts = (AbstractTimesheet) iter.next();
                initWorkingHours(ts);
                double factor = 1;
                if (ts instanceof CreditTimesheet)
                {
                    factor = -1;
                }
                regMonday += factor * ts.getRegularMonday().doubleValue();
                regTuesday += factor * ts.getRegularTuesday().doubleValue();
                regWednesday += factor * ts.getRegularWednesday().doubleValue();
                regThursday += factor * ts.getRegularThursday().doubleValue();
                regfriday += factor * ts.getRegularFriday().doubleValue();
                regSaturday += factor * ts.getRegularSaturday().doubleValue();
                regSunday += factor * ts.getRegularSunday().doubleValue();
                specMonday += factor * ts.getSpecialMonday().doubleValue();
                specTuesday += factor * ts.getSpecialTuesday().doubleValue();
                specWednesday += factor
                        * ts.getSpecialWednesday().doubleValue();
                specThursday += factor * ts.getSpecialThursday().doubleValue();
                specFriday += factor * ts.getSpecialFriday().doubleValue();
                specSaturday += factor * ts.getSpecialSaturday().doubleValue();
                specSunday += factor * ts.getSpecialSunday().doubleValue();
            }
        }
    
        ClientTestController.assertEquals(summaryTimesheet.getRegularMonday()
            .doubleValue(), regMonday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getRegularTuesday()
            .doubleValue(), regTuesday, 0);
        ClientTestController.assertEquals(summaryTimesheet
            .getRegularWednesday().doubleValue(), regWednesday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getRegularThursday()
            .doubleValue(), regThursday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getRegularFriday()
            .doubleValue(), regfriday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getRegularSaturday()
            .doubleValue(), regSaturday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getRegularSunday()
            .doubleValue(), regSunday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialMonday()
            .doubleValue(), specMonday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialTuesday()
            .doubleValue(), specTuesday, 0);
        ClientTestController.assertEquals(summaryTimesheet
            .getSpecialWednesday().doubleValue(), specWednesday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialThursday()
            .doubleValue(), specThursday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialFriday()
            .doubleValue(), specFriday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialSaturday()
            .doubleValue(), specSaturday, 0);
        ClientTestController.assertEquals(summaryTimesheet.getSpecialSunday()
            .doubleValue(), specSunday, 0);
    
    }

    public static void setWorkedHours(AbstractTimesheet timesheet,
            double regularHours, double specialHours)
    {

        Double regular = new Double(regularHours);
        timesheet.setRegularMonday(regular);
        timesheet.setRegularTuesday(regular);
        timesheet.setRegularWednesday(regular);
        timesheet.setRegularThursday(regular);
        timesheet.setRegularFriday(regular);
        timesheet.setRegularSaturday(regular);
        timesheet.setRegularSunday(regular);

        Double special = new Double(specialHours);
        timesheet.setSpecialMonday(special);
        timesheet.setSpecialTuesday(special);
        timesheet.setSpecialWednesday(special);
        timesheet.setSpecialThursday(special);
        timesheet.setSpecialFriday(special);
        timesheet.setSpecialSaturday(special);
        timesheet.setSpecialSunday(special);

    }
}
