package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.PersonalTaskFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;

/**
 * @author gmeloche
 *  
 */
public class TestPersonalTaskAssignment extends ClientTestController
{

    RPMObjectScope persoTaskScope = null;

    PersonalTask persoTask = null;

    PersonalProject personalProject = null;

    Resource resource = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        persoTaskScope = new RPMObjectScope();
        RPMObjectScope persoProjectScope = new RPMObjectScope();
        persoProjectScope.setParent(new RPMObjectScope());
        persoTaskScope.setParent(persoProjectScope);

        persoTask = PersonalTaskFixture.createFixture(null, this);

        personalProject = (PersonalProject) persoTask.getParent();

        resource = (Resource) personalProject.getParent();

    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testLoadAssignmentFromXpath() throws Exception
    {

        persoTask = save(persoTask);

        String xpath = "Resource[@id = '" + resource.getID()
                + "']/personalTaskAssignment";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        RPMObject[] objs = loadFromXpath(xpath, scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof PersonalTaskAssignment);
        assertNotNull(objs[0].getParent());
        assertIdEquals(resource, objs[0].getParent());
    }

    public void testLoadResourceFromXpath() throws Exception
    {

        persoTask = save(persoTask);

        String xpath = "Resource[personalTaskAssignment[personaltask[id = '"
                + persoTask.getID() + "']]]";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        RPMObject[] objs = loadFromXpath(xpath, scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Resource);
        assertIdEquals(resource, objs[0]);
    }

    public void testLoadPersonalTaskFromXpath() throws Exception
    {

        persoTask = save(persoTask);

        String xpath = "Resource[@id = '" + resource.getID()
                + "']/personalTaskAssignment/personaltask[@id = '"
                + persoTask.getID() + "']";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        RPMObject[] objs = loadFromXpath(xpath, scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof PersonalTask);
        assertIdEquals(persoTask, objs[0]);
    }

    private PersonalTask save(PersonalTask object) throws Exception
    {
        SingleResult result = save(persoTask, persoTaskScope);

        persoTask = (PersonalTask) result.getRpmObject();

        personalProject = (PersonalProject) persoTask.getParent();

        resource = (Resource) personalProject.getParent();

        return (PersonalTask) result.getRpmObject();
    }

    public void testShownInTimesheet() throws Exception
    {
        persoTask = save(persoTask);
        String xPath = "Resource[@id='" + resource.getID()
                + "']/@personalTaskAssignments";
        GenericTaskAssignment taskAssignment = (GenericTaskAssignment) loadFromXpath(
                xPath, null)[0];
        doTestShownInTimesheet(taskAssignment);
    }

    private void doTestShownInTimesheet(GenericTaskAssignment taskAssignment)
            throws Exception
    {
        Boolean isShownInTimesheet = taskAssignment.getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(Boolean.FALSE, isShownInTimesheet);

        doTestUpdateShownInTimesheet(taskAssignment, Boolean.TRUE);
        doTestUpdateShownInTimesheet(taskAssignment, Boolean.FALSE);
    }

    private void doTestUpdateShownInTimesheet(
            GenericTaskAssignment originalTaskAssignment, Boolean value)
            throws Exception
    {
        Class originalClass = originalTaskAssignment.getClass();
        String typeName = StringUtil.getShortClassName(originalClass);
        originalTaskAssignment.setShownInTimesheet(value);
        save(originalTaskAssignment, null, ReloadType.None)
                .getRpmObject();

        GenericTaskAssignment reloadedTaskAssignment = (GenericTaskAssignment) originalClass
                .newInstance();
        reloadedTaskAssignment.setID(originalTaskAssignment.getID());
        reloadedTaskAssignment = (GenericTaskAssignment) loadFromID(
                reloadedTaskAssignment, null);
        Boolean isShownInTimesheet = reloadedTaskAssignment
                .getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(value, isShownInTimesheet);

        doTestLoadFromXPath(typeName, originalTaskAssignment.getID(), value);
    }

    private void doTestLoadFromXPath(String typeName, String assignmentId,
            Boolean value) throws Exception
    {
        RPMObject[] assignments = loadFromXpath("/" + typeName + "[@id='"
                + assignmentId + "' and @shownInTimesheet='" + value + "']",
                null);
        assertNotNull(assignments);
        assertEquals(1, assignments.length);

        String notValue = Boolean.toString(!value.booleanValue());
        assignments = loadFromXpath("/" + typeName + "[@id='" + assignmentId
                + "' and shownInTimesheet='" + notValue + "']", null);
        assertNull(assignments);
    }
}
