/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class TestStepTimesheet extends ClientTestController
{
    private Step parent = null;
    private RPMObjectScope mainScope = null;
    
    private Calendar weekOf = null;
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        
        mainScope = StepTimesheetFixture.createScope(true);
        
        TaskAssignment taskAssignment = TaskAssignmentFixture.createSaveFixture(
            null, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        parent = StepFixture.createSaveFixture(taskAssignment, 
            (StepScope)mainScope.getParent(), null, this);

    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(mainScope, true);
    }
    
    public void testloadFromXpathWithResourceAndWeekOf() throws Exception
    {

        StepScope parentScope = StepFixture.createScope(false, false, false, true, false, false);
        mainScope.setParent(parentScope);
        
        StepTimesheet timesheet = insertTimesheet(mainScope, true);
        Step step = (Step)timesheet.getParent();

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @resource[id = '"
                + step.getResource().getID() + "']]/StepTimesheet[weekof = '"
                + DateUtil.convertToString(timesheet.getWeekOf()) + "']",
            mainScope);

        assertNotNull(objs);
        assertTrue(objs.length == 1);
        assertTrue(objs[0] instanceof StepTimesheet);

    }

    public void testloadParentFromXpathWithResourceAndWeekOf() throws Exception
    {

        StepScope parentScope = StepFixture.createScope(false, false, false, true, false, false);
        mainScope.setParent(parentScope);
        
        StepTimesheet timesheet = insertTimesheet(mainScope, true);
        Step step = (Step)timesheet.getParent();

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @resource[id = '"
                + step.getResource().getID() + "'] and @stepTimesheets[weekof = '"
                + DateUtil.convertToString(timesheet.getWeekOf()) + "']]",
            mainScope);

        assertNotNull(objs);
        assertTrue(objs.length == 1);
        assertTrue(objs[0] instanceof Step);

    }
    private void insert(RPMObjectScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert an element
        insertTimesheet(scope, loadFromId);

        // Do not attempt to delete it, because you cannot delete a timesheet.
    }

    private void update(RPMObjectScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert an element
        StepTimesheet timesheet = insertTimesheet(scope, loadFromId);

        //update the element
        updateTimesheet(timesheet, scope, loadFromId);

        // Do not attempt to delete it, because you cannot delete a timesheet.
    }

    private StepTimesheet insertTimesheet(RPMObjectScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
               
        StepTimesheet timesheet = StepTimesheetFixture.createFixture(parent, weekOf,
            scope, this);

        //Insert the element into the DB
        SingleResult result = save(timesheet, scope);

        //compare the returned element
        StepTimesheet returnedTimesheet = (StepTimesheet) result.getRpmObject();
        StepTimesheetFixture.compare(timesheet, returnedTimesheet, scope);

        //reload the element and compare it
        returnedTimesheet = (StepTimesheet) load(returnedTimesheet, scope, loadFromId);
        StepTimesheetFixture.compare(timesheet, returnedTimesheet, scope);

        //return the element
        return returnedTimesheet;
    }

    private void updateTimesheet(StepTimesheet timesheet, RPMObjectScope scope,
            boolean loadFromId) throws AssertionError, Exception
    {
        StepTimesheetFixture.updateFixture(timesheet);

        //Insert the element into the DB
        SingleResult result = save(timesheet, scope);

        //compare the returned element
        StepTimesheet returnedTimesheet = (StepTimesheet) result.getRpmObject();
        StepTimesheetFixture.compare(timesheet, returnedTimesheet, scope);

        //reload the element and compare it
        returnedTimesheet = (StepTimesheet) load(returnedTimesheet, scope, loadFromId);
        StepTimesheetFixture.compare(timesheet, returnedTimesheet, scope);
    }
    
    private RPMObject load(StepTimesheet timesheet, RPMObjectScope scope,
            boolean loadFromId) throws Exception
    {
        
        RPMObject object = null;
        
        if (loadFromId)
        {
            // load from an id
            StepTimesheet obj = new StepTimesheet();
            obj.setParent(timesheet.getParent());
            obj.setWeekOf(timesheet.getWeekOf());
            object = loadFromID(obj, scope);
        }
        else
        {
            // load from an xpath
            RPMObject[] objs = loadFromXpath("/STEPTIMESHEET[activityId='" + 
                timesheet.getParent().getID() + "' and weekOf='" + 
                DateUtil.convertToString(timesheet.getWeekOf()) + "']", scope);

            if (objs != null && objs.length > 0)
                object =  objs[0];
        }
        
        return object;
    }
}
