/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;

public class TestSummaryTimesheet extends ClientTestController
{
    private SummaryTimesheetScope mainScope = new SummaryTimesheetScope();
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = SummaryTimesheetFixture.createScope(true, true, true, true, true);
        
    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
        
    public void testInsertWithNoScopeLoadFromXPath() 
    	throws AssertionError, Exception
    {
        insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID()
    	throws AssertionError, Exception
    {
        insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        update(mainScope, true);
    }

    public void testInsertWithForecastStartDate() throws AssertionError, Exception
    {
        SummaryTimesheetScope scope = null;
        
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture.
        	createFixture(null, null, scope, false, this);
        
        summaryTimesheet.setForecastStartDate(TimesheetTestUtil.getValidWeekOf(summaryTimesheet.getWeekOf(), 1));
        // Insert the element into the DB
        SingleResult result = save(summaryTimesheet, scope);

        // compare the returned element
        SummaryTimesheet returnedSummaryTimesheet = 
            (SummaryTimesheet) result.getRpmObject();
               
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // reload the element and compare it
        returnedSummaryTimesheet = 
            (SummaryTimesheet) load(returnedSummaryTimesheet, scope, true);
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

    }
    
    public void testManualCheckoutUpdateWithScopeLoadFromID()
            throws AssertionError, Exception
    {        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(null, true);
        getApplicationInterface().disableAutomaticCheckoutMode(getSessionID());
        checkOut(summaryTimesheet);
        updateSummaryTimesheet(summaryTimesheet, null, true);
    }
    
    public void testLoadFromXPathWithResource() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, false, true, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @resource[id = '"
                + summaryTimesheet.getResource().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(summaryTimesheet.getResource().getID(),
            ((SummaryTimesheet) objs[0]).getResource().getID());
        
    }
    
    public void testLoadChildFromXPath_Resource() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, false, true, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/resource[id = '"
                + summaryTimesheet.getResource().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Resource);
        assertEquals(summaryTimesheet.getResource().getID(),
            ((Resource) objs[0]).getID());
        
    }
    
    public void testLoadFromXPathWithProject() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, true, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @project[id = '"
                + summaryTimesheet.getProject().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(summaryTimesheet
            .getProject().getID(), ((SummaryTimesheet) objs[0]).getProject().getID());
        
    }
    
    public void testLoadFromChildXPath_Project() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, true, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/@project[id = '"
                + summaryTimesheet.getProject().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof GenericProject);
        assertEquals(summaryTimesheet.getProject().getID(),
            ((GenericProject) objs[0]).getID());
        
    }
    
    public void testLoadFromXPathWithWorkElement() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, false, false, true);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @workElement[id = '"
                + summaryTimesheet.getWorkElement().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(summaryTimesheet.getWorkElement().getID(),
            ((SummaryTimesheet) objs[0]).getWorkElement().getID());
        
    }
    
    public void testLoadFromChildFromXPath_WorkElement() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, false, false, true);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/@workElement[id = '"
                + summaryTimesheet.getWorkElement().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof WorkElement);
        assertEquals( summaryTimesheet
            .getWorkElement().getID(), ((WorkElement) objs[0]).getID());
        
    }
    
    public void testLoadFromChildFromXPath_TaskAssignment() throws Exception{
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(null, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/@TaskAssignment[id = '"
                + summaryTimesheet.getTaskAssignment().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TaskAssignment);
        assertEquals( summaryTimesheet
            .getTaskAssignment().getID(), ((TaskAssignment) objs[0]).getID());
        
    }


    public void testLoadFromXPathWithTimeCode1() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(true, false, false, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @timecode1[id = '"
                + summaryTimesheet.getTimeCode1().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(summaryTimesheet.getTimeCode1().getID(),
            ((SummaryTimesheet) objs[0]).getTimeCode1().getID());
        
    }
    
    public void testLoadChildFromXPath_TimeCode1() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(true, false, false, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/@timecode1[id = '"
                + summaryTimesheet.getTimeCode1().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCode);
        assertEquals(summaryTimesheet.getTimeCode1().getID(),
            ((TimeCode) objs[0]).getID());
        
    }
    

    public void testLoadFromXPathWithTimeCode2() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(true, false, false, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @timecode2[id = '"
                + summaryTimesheet.getTimeCode2().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(summaryTimesheet.getTimeCode2().getID(),
            ((SummaryTimesheet) objs[0]).getTimeCode2().getID());
        
    }
    
    public void testLoadChildFromXPath_TimeCode2() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(true, false, false, false, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "']/@timecode2[id = '"
                + summaryTimesheet.getTimeCode2().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCode);
        assertEquals(summaryTimesheet.getTimeCode2().getID(),
            ((TimeCode) objs[0]).getID());
        
    }
    
    private void insert(SummaryTimesheetScope scope, boolean loadFromId) throws AssertionError, Exception
    {
        // insert an element
        insertSummaryTimesheet(scope, loadFromId);

        // Do not delete the element. SummaryTimesheet gets deleted by the
        // TaskAssignment's delete SP.
    }

    private void update(SummaryTimesheetScope scope, boolean loadFromId) throws AssertionError, Exception
    {
        // insert an element
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, 
            loadFromId);

        // update the element
        updateSummaryTimesheet(summaryTimesheet, scope, loadFromId);

        // Do not delete the element. SummaryTimesheet gets deleted by the
        // TaskAssignment's delete SP.
    }

    private SummaryTimesheet insertSummaryTimesheet(SummaryTimesheetScope scope, 
            boolean loadFromId) throws AssertionError, Exception
    {
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture.
        	createFixture(null, null, scope, true, this);

        // Insert the element into the DB
        SingleResult result = save(summaryTimesheet, scope);

        // compare the returned element
        SummaryTimesheet returnedSummaryTimesheet = 
            (SummaryTimesheet) result.getRpmObject();
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // reload the element and compare it
        returnedSummaryTimesheet = 
            (SummaryTimesheet) load(returnedSummaryTimesheet, scope, loadFromId);
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // return the element
        return returnedSummaryTimesheet;
    }

    private void updateSummaryTimesheet(SummaryTimesheet summaryTimesheet, 
            SummaryTimesheetScope scope, boolean loadFromId) 
    	throws AssertionError, Exception
    {
        summaryTimesheet = SummaryTimesheetFixture.updateFixture(summaryTimesheet, scope, true, this);

        // Insert the element into the DB
        SingleResult result = save(summaryTimesheet, scope);

        // compare the returned element
        SummaryTimesheet returnedSummaryTimesheet = 
            (SummaryTimesheet) result.getRpmObject();
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // reload the element and compare it
        returnedSummaryTimesheet = (SummaryTimesheet)
        	load(returnedSummaryTimesheet, scope, loadFromId);
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);
    }

    private RPMObject load(SummaryTimesheet summaryTimesheet, 
            SummaryTimesheetScope scope, boolean loadFromId) throws Exception
    {
        RPMObject object = null;

        if (loadFromId)
        {
            // load from an id
            SummaryTimesheet obj = new SummaryTimesheet();
            obj.setTaskAssignment(summaryTimesheet.getTaskAssignment());
            obj.setWeekOf(summaryTimesheet.getWeekOf());
            object = loadFromID(obj, scope);
        }
        else
        {

            // load from an xpath
            RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "' and @taskassignment[id = '"
                    + summaryTimesheet.getTaskAssignment().getID() + "']]",
                scope);

            if (objs != null && objs.length > 0)
            {
                object = objs[0];
            }
        }

        return object;
    }
    

}
