package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;

public class TestSummaryTimesheetOnAdministrativeTask extends ClientTestController
{
    private SummaryTimesheetScope mainScope = new SummaryTimesheetScope();
    
    private SummaryTimesheet finalSt = null;
    
    AdministrativeTaskAssignment administativeTaskAssignment = null;
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = SummaryTimesheetFixture.createScope(true, true, false, true, false);
        
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setParent(new PoolScope());
        resourceScope.setCalendar(new CalendarScope());
        resourceScope.setAdministrativeTaskAssignments(true);
        Resource resource = ResourceFixture.createSaveFixture(resourceScope, this);
        
        Pool parentPool = (Pool)resource.getParent();
        //add an administrative task to parent pool
        DefaultAdministrativeTaskFixture.createSaveFixture(parentPool, this);
        
        //reload the resource
        resource = (Resource)loadFromID(resource, resourceScope);
        
        administativeTaskAssignment = resource.getAdministrativeTaskAssignments()[0];
        
    }
    
    public void tearDown() throws Exception
    {
        try
        {
            // need to reject the timesheet and reset the actuals before the
            // Resource can be deleted.
            if (finalSt != null)
            {
                //reload with approvalStatus
                
               SummaryTimesheetScope scope = new SummaryTimesheetScope();
               scope.setApprovalStatus(new TimesheetApprovalStatusScope());
                
                finalSt = (SummaryTimesheet)load(finalSt, scope, true);           
                
                if (finalSt.getApprovalStatus() != null
                        && finalSt.getApprovalStatus().getApprovalStatus() == TimesheetStatus.Submitted)
                {
                    finalSt.getApprovalStatus().setApprovalStatus(TimesheetStatus.Rejected);
                    save(finalSt, scope);
                }
            }

            if (finalSt != null)
            {
                //for clean up purpose
                //reset timesheet values
                finalSt.setRegularMonday(null);
                finalSt.setRegularTuesday(null);
                finalSt.setRegularWednesday(null);
                finalSt.setRegularThursday(null);
                finalSt.setRegularThursday(null);
                finalSt.setRegularFriday(null);
                finalSt.setRegularSaturday(null);
                finalSt.setRegularSunday(null);
                finalSt.setSpecialMonday(null);
                finalSt.setSpecialTuesday(null);
                finalSt.setSpecialWednesday(null);
                finalSt.setSpecialThursday(null);
                finalSt.setSpecialThursday(null);
                finalSt.setSpecialFriday(null);
                finalSt.setSpecialSaturday(null);
                finalSt.setSpecialSunday(null);

                save(finalSt, null);
            }
        }
        catch (Exception e)
        {
            //Only print the exception. The test should not fail because of an
            // error in the tearDown method.
            e.printStackTrace();
        }

        super.tearDown();
    }
        
    public void testInsertWithScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        update(mainScope, true);
    }

    public void testLoadFromXPathWithResource() throws Exception{
        
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(false, false, false, true, false);
        
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and @resource[id = '"
                + summaryTimesheet.getResource().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof SummaryTimesheet);
        assertEquals(((SummaryTimesheet) objs[0]).getResource().getID(), summaryTimesheet
            .getResource().getID());
        
    }

    private void insert(SummaryTimesheetScope scope, boolean loadFromId) throws AssertionError, Exception
    {
        // insert an element
        insertSummaryTimesheet(scope, loadFromId);
    }

    private void update(SummaryTimesheetScope scope, boolean loadFromId) throws AssertionError, Exception
    {
        // insert an element
        SummaryTimesheet summaryTimesheet = insertSummaryTimesheet(scope, 
            loadFromId);

        // update the element
        updateSummaryTimesheet(summaryTimesheet, scope, loadFromId);

        // Do not delete the element. SummaryTimesheet gets deleted by the
        // TaskAssignment's delete SP.
    }

    private SummaryTimesheet insertSummaryTimesheet(SummaryTimesheetScope scope, 
            boolean loadFromId) throws AssertionError, Exception
    {
        
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture.
        	createFixture(administativeTaskAssignment, null, scope, true, this);

        // Insert the element into the DB
        SingleResult result = save(summaryTimesheet, scope);

        // compare the returned element
        SummaryTimesheet returnedSummaryTimesheet = 
            (SummaryTimesheet) result.getRpmObject();
        
        //finalSt set for clean up purposes
        finalSt = returnedSummaryTimesheet;
            
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // reload the element and compare it
        returnedSummaryTimesheet = 
            (SummaryTimesheet) load(returnedSummaryTimesheet, scope, loadFromId);
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // return the element
        return returnedSummaryTimesheet;
    }

    private void updateSummaryTimesheet(SummaryTimesheet summaryTimesheet, 
            SummaryTimesheetScope scope, boolean loadFromId) 
    	throws AssertionError, Exception
    {
        summaryTimesheet = SummaryTimesheetFixture.updateFixture(summaryTimesheet, scope, true, this);

        // Insert the element into the DB
        SingleResult result = save(summaryTimesheet, scope);

        // compare the returned element
        SummaryTimesheet returnedSummaryTimesheet = 
            (SummaryTimesheet) result.getRpmObject();
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);

        // reload the element and compare it
        returnedSummaryTimesheet = (SummaryTimesheet)
        	load(returnedSummaryTimesheet, scope, loadFromId);
        SummaryTimesheetFixture.compare(summaryTimesheet, 
            returnedSummaryTimesheet, scope);
    }

    private RPMObject load(SummaryTimesheet summaryTimesheet, 
            SummaryTimesheetScope scope, boolean loadFromId) throws Exception
    {
        RPMObject object = null;

        if (loadFromId)
        {
            // load from an id
            SummaryTimesheet obj = new SummaryTimesheet();
            obj.setTaskAssignment(summaryTimesheet.getTaskAssignment());
            obj.setWeekOf(summaryTimesheet.getWeekOf());
            object = loadFromID(obj, scope);
        }
        else
        {

            // load from an xpath
            RPMObject[] objs = loadFromXpath("/SummaryTimesheet[weekOf='"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "' and @taskassignment[id = '"
                    + summaryTimesheet.getTaskAssignment().getID() + "']]",
                scope);

            if (objs != null && objs.length > 0)
            {
                object = objs[0];
            }
        }

        return object;
    }
    

}
