/*
 * Create on Jan 11, 2007
 * 
 * Defect 57831: WS: RATLC00295383 - Timesheet approval 
 * column returns "NULL"
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.TaskAssignment;

/**
 * @author cobber
 * 
 */
public class Test57831 extends ClientTestController
{
    private TaskAssignment taskAssignment = null;

    private Calendar weekOf1 = null;

    private SummaryTimesheet summaryTimesheet = null;

    private SummaryTimesheetScope summaryTimesheetScope = null;

    private void init() throws Exception
    {
        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
        RPMObject rpmObject = loadFromID(taskAssignment, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) rpmObject;

        weekOf1 = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), 1);
        assertNotNull(weekOf1);

        // build appropriate scope
        summaryTimesheetScope = new SummaryTimesheetScope();
        summaryTimesheetScope
            .setApprovalStatus(new TimesheetApprovalStatusScope());
        
        //create SummaryTimesheet
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf1, summaryTimesheetScope, false, this);
        rpmObject = loadFromID(summaryTimesheet, summaryTimesheetScope);
        assertNotNull(rpmObject);
        validateSummaryTimesheet(rpmObject, TimesheetStatus.Submitted);
    }

    public void testApprovedByProjectManagerApprovalStatus() throws Exception
    {
        init();
        
        TimesheetApprovalStatus approvalStatus = summaryTimesheet
            .getApprovalStatus();
        approvalStatus
            .setApprovalStatus(TimesheetStatus.ApprovedByProjectManager);

        saveAndValidate(TimesheetStatus.ApprovedByProjectManager);
    }

    public void testRejectedByProjectManagerApprovalStatus() throws Exception
    {
        init();
        
        TimesheetApprovalStatus approvalStatus = summaryTimesheet
            .getApprovalStatus();
        approvalStatus
            .setApprovalStatus(TimesheetStatus.RejectedByProjectManager);

        saveAndValidate(TimesheetStatus.RejectedByProjectManager);
    }

    private void saveAndValidate(TimesheetStatus expectedStatus)
            throws Exception
    {
        SaveResult result = this.save(summaryTimesheet, summaryTimesheetScope);
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
        
        validateSummaryTimesheet(result.getRpmObject(), expectedStatus);
    }
    
    private void validateSummaryTimesheet(RPMObject rpmObject, TimesheetStatus expectedStatus)
    {
        assertTrue(rpmObject instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) rpmObject;
        assertNotNull(summaryTimesheet.getApprovalStatus());
        assertNotNull(summaryTimesheet.getApprovalStatus().getApprovalStatus());
        assertEquals(expectedStatus, summaryTimesheet
            .getApprovalStatus().getApprovalStatus());
    }
}
