/*
 * Created on Apr 25, 2007
 * 
 * Defect 59289: WS-59289 - load from xpath is failing for TimesheetApprovalStatus
 * 
 * Description:
 * Using the same XPATH
 * String xpath = "/TimesheetApprovalStatus[@resource[@id='"+resource.getID()+"'] and @project[@id='"+project.getID()+"'] and @weekOf='"+weekOfStringFormat+"']";
 * The following error message is being thrown:
 * :ERROR:: Cannot perform a loadFromXpath on the container type of field <TimesheetApprovalStatus.project>. The type of the field is <RPMObject>.
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

/**
 * @author cobber
 */
import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetApprovalStatusFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class Test59289 extends GenericValidation
{
    public void test() throws Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);

        SummaryTimesheetScope scope = new SummaryTimesheetScope();

        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);

        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        scope = SummaryTimesheetFixture.createScope(false, true, false, false,
            false);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        Resource resource = SummaryTimesheetFixture.loadResource(
            summaryTimesheet, this);

        resource.getResourceSecurityRights().setCanUseWebServicesAPI(
            Boolean.TRUE);

        resource = (Resource) save(resource, null).getRpmObject();

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        summaryTimesheet.setRegularMonday(new Double(2.0));

        // save the object
        summaryTimesheet = (SummaryTimesheet) saveMustSucceed(summaryTimesheet,
            SummaryTimesheet.class, scope);

        // submit the timesheet for approval
        TimesheetApprovalStatus appStatus = new TimesheetApprovalStatus();
        appStatus.setParent(summaryTimesheet);
        appStatus.setResource(resource);
        appStatus.setWeekOf(summaryTimesheet.getWeekOf());

        TimesheetApprovalStatusFixture.submitTimesheetAndSave(appStatus, null,
            true, this);

        // reload the summaryTimesheet
        summaryTimesheet = (SummaryTimesheet) loadFromID(summaryTimesheet,
            scope);
        assertNotNull(summaryTimesheet.getApprovalStatus());

        // load TimesheetApprovalStatus
        String xpath = "/TimesheetApprovalStatus[@resource[@id='"
                + resource.getID()
                + "'] and GenericProject[@id='"
                + project.getID()
                + "'] and @weekof='"
                + DateUtil.convertToString(summaryTimesheet.getApprovalStatus()
                    .getWeekOf()) + "']";
        RPMObject[] rpmObjects = loadFromXpath(xpath, null);
        assertNotNull(rpmObjects);
    }
}
