package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AdministrativeProject;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.scope.AdministrativeProjectScope;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;

public class Test59696 extends ClientTestController
{
    private Resource resource;

    private RPMObject[] rpmObjects;

    private SingleResult result;

    private String personalTaskName = "personalTaskFromAPI";

    private String adminTaskName = "adminTask";

    public void testLoadAdminProject() throws Exception
    {
        createResource();

        AdministrativeProjectScope adminProjectScope = new AdministrativeProjectScope();
        adminProjectScope.setAdministrativeTasks(true);
        rpmObjects = loadFromXpath("/Resource[@id='" + resource.getID()
                + "']/PersonalProject/AdministrativeProject", adminProjectScope);
        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof AdministrativeProject);
        AdministrativeProject adminProject = (AdministrativeProject) rpmObjects[0];
        assertNotNull(adminProject.getAdministrativeTasks());
        assertEquals(1, adminProject.getAdministrativeTasks().length);
        assertEquals(adminTaskName, adminProject.getAdministrativeTasks()[0]
            .getName());
    }

    public void testLoadAdminTask() throws Exception
    {
        createResource();

        rpmObjects = loadFromXpath(
            "/Resource[@id='"
                    + resource.getID()
                    + "']/PersonalProject/AdministrativeProject/AdministrativeTask",
            null);

        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof AdministrativeTask);
        AdministrativeTask administrativeTask = (AdministrativeTask) rpmObjects[0];
        assertEquals(adminTaskName, administrativeTask.getName());
    }

    public void testLoadPersonalTask() throws Exception
    {
        createPersonalTask();

        // load personal task back
        rpmObjects = loadFromXpath("/Resource[@id='" + resource.getID()
                + "']/PersonalProject/PersonalTask", null);
        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof PersonalTask);
        assertEquals(personalTaskName, ((PersonalTask) rpmObjects[0]).getName());
    }

    public void testLoadPersonalProject() throws Exception
    {
        createPersonalTask();

        // load personalProject with personalTask in scope
        PersonalProjectScope personalProjectScope = new PersonalProjectScope();
        personalProjectScope.setPersonalTasks(true);
        rpmObjects = loadFromXpath("/Resource[@id='" + resource.getID()
                + "']/PersonalProject", personalProjectScope);
        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof PersonalProject);
        PersonalProject personalProject = (PersonalProject) rpmObjects[0];
        assertEquals(resource.getFullName(), personalProject.getName());
        assertNotNull(personalProject.getPersonalTasks());
        assertEquals(1, personalProject.getPersonalTasks().length);
        assertEquals(personalTaskName, personalProject.getPersonalTasks()[0]
            .getName());
    }

    private void createPersonalTask() throws Exception
    {
        // create resource
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resource = ResourceFixture.createSaveFixture(resourceScope, this);

        // create personalTask
        PersonalProject personalProject = loadPersonalProject();
        PersonalTask personalTask = new PersonalTask();
        personalTask.setParent(personalProject);
        personalTask.setName(personalTaskName);
        result = this.save(personalTask, null);
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
    }

    private PersonalProject loadPersonalProject() throws Exception
    {
        rpmObjects = loadFromXpath("/Resource[@id='" + resource.getID()
                + "']/PersonalProject", null);
        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof PersonalProject);
        PersonalProject personalProject = (PersonalProject) rpmObjects[0];
        assertEquals(resource.getFullName(), personalProject.getName());
        return personalProject;
    }

    private void createResource() throws Exception
    {
        Pool pool = new Pool();
        pool.setName("newPool");
        DefaultAdministrativeTask administrativeTask = new DefaultAdministrativeTask();
        administrativeTask.setName(adminTaskName);
        pool.setAdministrativeTasks(new DefaultAdministrativeTask[]
        {
            administrativeTask
        });
        PoolScope poolScope = new PoolScope();
        poolScope.setAdministrativeTasks(true);
        result = this.save(pool, poolScope);

        resource = new Resource();
        resource.setUserName("newResource");
        resource.setParent(pool);
        resource.setFullName("newResource");
        RPMCalendar cal = (RPMCalendar) loadFromXpath(
            "/RPMCalendar[@id='DEFAULT_CALENDAR']", null)[0];

        resource.setCalendar(cal);

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        result = this.save(resource, resourceScope);
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Resource);
        resource = (Resource) result.getRpmObject();
    }
}
