/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

/**
 * @author dominiqu
 */
public class Test59836 extends ClientTestController
{
    private int nbChildren = 2;

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    public void testAutomaticCheckinAfterLogout() throws Exception
    {
        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(
            false, false, true, true, true);

        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture
            .createSaveFixture(null, null, scope, false, this);

        // Manually checkout the timesheet
        disableAutomaticCheckout();
        checkOut(summaryTimesheet, false);
        Assert.assertTrue(isCheckedOut(summaryTimesheet));

        // Logging out automatically checks-in what the resource left check out
        getAuthenticateInterface().logout(getSessionID());

        // Verify that the check-in worked
        performLogin();
        disableAutomaticCheckout();
        AlreadyCheckedOutResult result = getApplicationInterface()
            .isCheckedOut(getSessionID(), summaryTimesheet);
        assertFalse(result.isCheckedOut());
    }

    public void testAdminTaskWithHoursWithManualCheckout() throws Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resourceScope.setAdministrativeTaskAssignments(true);

        PoolScope mainScope = new PoolScope();
        mainScope.setAdministrativeTasks(true);
        mainScope.setResources(resourceScope);

        Pool mainPool = PoolFixture.createFixture(false, nbChildren, mainScope,
            this);
        Pool saved = savePool(mainPool, mainScope);
        mainPool.setID(saved.getID());

        mainPool = (Pool) loadFromID(mainPool, mainScope);

        SummaryTimesheetScope stScope = SummaryTimesheetFixture.createScope(
            true, true, false, true, false);

        // admin task 1

        AdministrativeTaskAssignment adminTask1 = mainPool.getResources()[0]
            .getAdministrativeTaskAssignments()[0];

        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture
            .createFixture(adminTask1, null, stScope, true, this);
        SingleResult result = save(summaryTimesheet, stScope);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();

        // admin task 2

        AdministrativeTaskAssignment adminTask2 = mainPool.getResources()[0]
            .getAdministrativeTaskAssignments()[0];

        SummaryTimesheet summaryTimesheet2 = SummaryTimesheetFixture
            .createFixture(adminTask2, null, stScope, true, this);
        SingleResult result2 = save(summaryTimesheet2, stScope);
        summaryTimesheet2 = (SummaryTimesheet) result2.getRpmObject();

        // Manually checkout the timesheets
        disableAutomaticCheckout();
        checkOut(summaryTimesheet, false);
        Assert.assertTrue(isCheckedOut(summaryTimesheet));
        checkOut(summaryTimesheet2, false);
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));

        summaryTimesheet = SummaryTimesheetFixture.updateFixture(
            summaryTimesheet, stScope, true, this);
        SingleResult result3 = save(summaryTimesheet2, stScope);
        summaryTimesheet = (SummaryTimesheet) result3.getRpmObject();

        summaryTimesheet2 = SummaryTimesheetFixture.updateFixture(
            summaryTimesheet2, stScope, true, this);
        SingleResult result4 = save(summaryTimesheet2, stScope);
        summaryTimesheet2 = (SummaryTimesheet) result4.getRpmObject();

        checkIn(summaryTimesheet);
        Assert.assertFalse(isCheckedOut(summaryTimesheet));
        checkIn(summaryTimesheet2);
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
    }

    private Pool savePool(Pool p, PoolScope scope) throws AssertionError,
            Exception
    {
        SingleResult result;
        result = save(p, scope);

        p = (Pool) result.getRpmObject();
        return p;
    }

    public void testStepWithHoursWithManualCheckout() throws Exception
    {
        StepScope scope = StepFixture.createScope(true, true, false, true,
            false, true);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(null, this);

        // step 1

        Step step1 = StepFixture.createFixture(taskAssignment, scope, null,
            this);
        SingleResult result1 = save(step1, scope);
        step1 = (Step) result1.getRpmObject();

        // step 2

        Step step2 = StepFixture.createFixture(taskAssignment, scope, null,
            this);
        step2.setWeight(new Double(100 - step1.getWeight().doubleValue()));
        SingleResult result2 = save(step2, scope);
        step2 = (Step) result2.getRpmObject();

        step1 = StepFixture.updateFixture(step1, scope, this);
        step1.setWeight(new Double(100 - step2.getWeight().doubleValue()));
        step2 = StepFixture.updateFixture(step2, scope, this);
        step2.setWeight(new Double(100 - step1.getWeight().doubleValue()));
        
        // Manually checkout the steps
        disableAutomaticCheckout();
        checkOut(step1, false);
        Assert.assertTrue(isCheckedOut(step1));
        checkOut(step2, false);
        Assert.assertTrue(isCheckedOut(step2));

        SingleResult result3 = save(step1, scope);
        step1 = (Step) result3.getRpmObject();

        SingleResult result4 = save(step2, scope);
        step2 = (Step) result4.getRpmObject();

        checkIn(step1);
        Assert.assertFalse(isCheckedOut(step1));
        checkIn(step2);
        Assert.assertFalse(isCheckedOut(step2));
    }
}
