/*
 * Create on Feb 9, 2007
 * Defect 60965: WS - incorrect %c when adding steps
 * 
 * This is a WebUI defect (50796)
 *
 * Steps to reproduce:
 * =============
 * 1- Create a TaskAssignement with eetc = 8 and save it.
 * 2- Create a SummaryTimesheet for the task assignement and save it.
 * 3- Create a step for the task assignement (don't save it)
 * 4- Create a StepTimesheet for the step using the same weekOf as the one in SummaryTimesheet. Also set regularMonday to 1. 
 * 5- Save the step with stepTimesheet in scope.
 * 6- Reload the SummaryTimesheet and notice how eetc is -1 (it should be 7) and how %c is %13 (it should be 12.5)
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author cobber
 * 
 */
public class Test60965 extends ClientTestController
{
    public void testPercentageCWhenAddingSteps() throws Exception
    {
        Double EETC = new Double(8);
        Double workHour = new Double(1);
        
        // create scope
        TaskAssignmentScope taskAssignmentScope = new TaskAssignmentScope();
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setContainingProject(new WorkElementScope());
        resourceTaskAssignmentScope.setParent(taskScope);
        taskAssignmentScope.setParent(resourceTaskAssignmentScope);

        // 1- Create a TaskAssignement with eetc = 8 and save it.
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        RPMObject rpmObject = this.loadFromID(taskAssignment, taskAssignmentScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) rpmObject;
        taskAssignment.setEETCHours(EETC);
        SingleResult result = this.save(taskAssignment, taskAssignmentScope);
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
        rpmObject = result.getRpmObject();
        assertTrue(rpmObject instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) rpmObject;
        assertNotNull(taskAssignment.getEETCHours());
        assertEquals(EETC.doubleValue(), taskAssignment.getEETCHours().doubleValue(), 0.001);
        rpmObject = taskAssignment.getParent();
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ResourceTaskAssignment);
        rpmObject = rpmObject.getParent();
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Task);
        rpmObject = ((Task) rpmObject).getContainingProject();
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Proposal);
        
        // copyProposedToPlan, PublishToTeam
        Proposal proposal = (Proposal)rpmObject;
        WorkElementScope proposalScope = new WorkElementScope();
        resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());
        taskScope.setResourceTaskAssignments(resourceTaskAssignmentScope);
        proposalScope.setChildren(taskScope);
        
        this.copyProposedToPlan(proposal, new WorkElementScope());
        proposal.setPublished(ProjectPublishedType.ToTeam);
        result = save(proposal, proposalScope);
        assertTrue(result.isSuccessful());
        rpmObject = this.loadFromID(taskAssignment, taskAssignmentScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof TaskAssignment);
        taskAssignment = (TaskAssignment)rpmObject;
        assertEquals(EETC.doubleValue(), taskAssignment.getEETCHours().doubleValue(), 0.001);

        // 2- Create a SummaryTimesheet for the task assignement and save it.
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        result = this.save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        rpmObject = result.getRpmObject();
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) rpmObject;
        assertEquals(EETC.doubleValue(), ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getEETCHours().doubleValue(), 0.001);

        // 3- Create a step for the task assignement (don't save it)
        Step step = new Step();
        step.setTaskAssignment(taskAssignment);

        // 4- Create a StepTimesheet for the step using the same weekOf as the
        // one in SummaryTimesheet. Also set regularMonday to 1.
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setParent(step);
        stepTimesheet.setWeekOf(weekOf);
        stepTimesheet.setRegularMonday(workHour);

        // 5- Save the step with stepTimesheet in scope.
        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);
        step.setStepTimesheets(new StepTimesheet[]
        {
            stepTimesheet
        });
        result = this.save(step, stepScope);
        assertTrue(result.isSuccessful());
        rpmObject = result.getRpmObject();
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Step);

        // 6- Reload the SummaryTimesheet and notice how eetc is -1 (it should
        // be 7) and how %c is %13 (it should be 12.5)
        rpmObject = this.loadFromID(summaryTimesheet, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) rpmObject;
        assertEquals(EETC.doubleValue() - workHour.doubleValue(), ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getEETCHours().doubleValue(), 0.001);
        assertEquals(workHour.doubleValue() / EETC.doubleValue() * 100, ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getPercentageComplete().doubleValue(), 0.001);
    }
}
