/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.PersonalTaskFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author dominiqu
 */
public class Test61344 extends ClientTestController
{
    private PoolScope poolScope;

    private ResourceScope resourceScope;

    private Resource resource;

    // Timesheet items to check-in / check-out

    private AdministrativeTaskAssignment administrativeTaskAssignment1;

    private PersonalTaskAssignment personalTaskAssignment1;

    private SummaryTimesheet summaryTimesheet1;

    private Step step1;

    private StepTimesheet stepTimesheet1;

    private SummaryTimesheet summaryTimesheet2;

    private SummaryTimesheet summaryTimesheet3;

    protected void setUp() throws Exception
    {
        super.setUp();

        // Get a resource to test timesheet with
        poolScope = new PoolScope();
        poolScope.setAdministrativeTasks(true);
        resourceScope = new ResourceScope();
        resourceScope.setParent(poolScope);
        resourceScope.setAdministrativeTaskAssignments(true);
        resourceScope.setPersonalProject(new PersonalProjectScope());
        resourceScope.setPersonalTaskAssignments(true);
        resource = ResourceFixture.loadStaticFixture(this, resourceScope);
        getAuthenticateInterface().logout(sessionid);
        performLogin(resource.getUserName(),
            ResourceFixture.STATIC_RESOURCE_PASSWORD);

        // Create a timesheet will all types of objects for the resource

        // Add administrative tasks to the Pool if there aren't any
        if (resource.getAdministrativeTaskAssignments() == null
                || resource.getAdministrativeTaskAssignments().length == 0)
        {
            DefaultAdministrativeTaskFixture.createSaveFixture((Pool) resource
                .getParent(), this);
            resource = (Resource) loadFromID(resource, resourceScope);
            assertNotNull(resource.getAdministrativeTaskAssignments());
        }
        administrativeTaskAssignment1 = resource
            .getAdministrativeTaskAssignments()[0];

        // Add personal tasks to the Resource if there aren't any
        if (resource.getPersonalTaskAssignments() == null
                || resource.getPersonalTaskAssignments().length == 0)
        {
            assertNotNull(resource.getPersonalProject());
            PersonalTaskFixture.createSaveFixture(
                resource.getPersonalProject(), this);
            resource = (Resource) loadFromID(resource, resourceScope);
            assertNotNull(resource.getPersonalTaskAssignments());
        }
        personalTaskAssignment1 = resource.getPersonalTaskAssignments()[0];

        // Create a SummaryTimesheet
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        WorkElement workElement1 = WorkElementFixture.createSaveFixture(
            new Task(), workElementScope, this);
        TaskAssignment taskAssignment1 = TaskAssignmentFixture
            .createSaveFixture(workElement1, resource, null, this);
        summaryTimesheet1 = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment1, null, null, true, this);

        // Create StepTimesheet for above SummaryTimesheet
        stepTimesheet1 = new StepTimesheet();
        stepTimesheet1.setWeekOf(summaryTimesheet1.getWeekOf());
        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);
        step1 = StepFixture.createSaveFixture(
            (TaskAssignment) summaryTimesheet1.getTaskAssignment(), stepScope,
            new StepTimesheet[]
            {
                stepTimesheet1
            }, this);

        // Create a SummaryTimesheet in the same project as the above summary
        // timesheet
        WorkElement workElement2 = WorkElementFixture.createSaveFixture(
            workElement1.getContainingProject(), new Task(), workElementScope,
            this);
        TaskAssignment taskAssignment2 = TaskAssignmentFixture
            .createSaveFixture(workElement2, resource, null, this);
        summaryTimesheet2 = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment2, null, null, true, this);

        // Create a SummaryTimesheet in an entirely different portfolio
        Portfolio portfolio3 = (Portfolio) PortfolioFixture
            .createSaveFixture(this);
        Project project3 = (Project) WorkElementFixture.createFixture(null,
            new Project(), workElementScope, this);
        project3.setPortfolio(portfolio3);
        save(project3, null);
        WorkElement workElement3 = WorkElementFixture.createSaveFixture(
            project3, new Task(), workElementScope, this);
        TaskAssignment taskAssignment3 = TaskAssignmentFixture
            .createSaveFixture(workElement3, resource, null, this);
        summaryTimesheet3 = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment3, null, null, true, this);
    }

    protected void tearDown() throws Exception
    {
        if (isCheckedOut(administrativeTaskAssignment1))
        {
            checkIn(administrativeTaskAssignment1);
        }

        super.tearDown();
    }

    /**
     * Test that when you check out a single timesheet item it checks out all
     * timesheets for the user in that session.
     */

    public void testCheckOutAdministrativeTaskAssignmentMakesEverythingCheckedOut()
            throws Exception
    {
        // Manually checkout the administrative task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            administrativeTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertTrue(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
        Assert.assertTrue(isCheckedOut(step1));
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));
        Assert.assertTrue(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckOutPersonalTaskAssignmentMakesEverythingCheckedOut()
            throws Exception
    {
        // Manually checkout the personal task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            personalTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertTrue(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
        Assert.assertTrue(isCheckedOut(step1));
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));
        Assert.assertTrue(isCheckedOut(summaryTimesheet3));
    }
    
    public void testCheckOutPersonalTaskMakesEverythingCheckedOut()
            throws Exception
    {
        // Manually checkout the personal task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            personalTaskAssignment1.getPersonalTask(), false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertTrue(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
        Assert.assertTrue(isCheckedOut(step1));
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));
        Assert.assertTrue(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckOutSummaryTimesheetMakesEverythingCheckedOut()
            throws Exception
    {
        // Manually checkout the summary timesheet
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            summaryTimesheet1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertTrue(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
        Assert.assertTrue(isCheckedOut(step1));
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));
        Assert.assertTrue(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckOutStepMakesEverythingCheckedOut() throws Exception
    {
        // Manually checkout the step
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            step1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertTrue(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1));
        Assert.assertTrue(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
        Assert.assertTrue(isCheckedOut(step1));
        Assert.assertTrue(isCheckedOut(summaryTimesheet2));
        Assert.assertTrue(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckOutInAnotherSession() throws Exception
    {
        // Manually checkout
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            summaryTimesheet1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());
        Assert.assertTrue(isCheckedOut(summaryTimesheet1));

        String tempSessionid = sessionid;

        performLogin(resource.getUserName(),
            ResourceFixture.STATIC_RESOURCE_PASSWORD);

        AlreadyCheckedOutResult result2 = getApplicationInterface()
            .isCheckedOut(sessionid, summaryTimesheet1);

        Assert.assertTrue(result2.isCheckedOut());
        Assert
            .assertEquals(resource.getID(), result2.getCheckedOutBy().getID());

        getAuthenticateInterface().logout(sessionid);

        sessionid = tempSessionid;

        Assert.assertTrue(isCheckedOut(summaryTimesheet1));
    }

    /**
     * Test that when you check in a single timesheet item it checks in all
     * timesheets for the user in that session.
     */

    public void testCheckInAdministrativeTaskAssignmentMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the administrative task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            administrativeTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the administrative task assignment
        result = getApplicationInterface().checkIn(getSessionID(),
            administrativeTaskAssignment1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked out
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckInPersonalTaskAssignmentMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the personal task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            personalTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the personal task assignment
        result = getApplicationInterface().checkIn(getSessionID(),
            personalTaskAssignment1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckInSummaryTimesheetMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the summary timesheet
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            summaryTimesheet1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the summary timesheet
        result = getApplicationInterface().checkIn(getSessionID(),
            summaryTimesheet1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testCheckInStepMakesEverythingCheckedIn() throws Exception
    {
        // Manually checkout the step
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            step1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the step
        result = getApplicationInterface().checkIn(getSessionID(), step1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    /**
     * Test that when you force check in a single timesheet item it checks in
     * all timesheets for the user in that session.
     */

    public void testForceCheckInAdministrativeTaskAssignmentMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the administrative task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            administrativeTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the administrative task assignment
        result = getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
            getSessionID(), administrativeTaskAssignment1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testForceCheckInPersonalTaskAssignmentMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the personal task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            personalTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the personal task assignment
        result = getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
            getSessionID(), personalTaskAssignment1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testForceCheckInSummaryTimesheetMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the summary timesheet
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            summaryTimesheet1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the summary timesheet
        result = getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
            getSessionID(), summaryTimesheet1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is not checked out
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }

    public void testForceCheckInStepMakesEverythingCheckedIn() throws Exception
    {
        // Manually checkout the step
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            step1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Manually checkin the step
        result = getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
            getSessionID(), step1);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1.getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }
    
    /** 
     * Check that logging out undoes the checkouts successfully.
     * @throws Exception
     */
    
    public void testLogoutMakesEverythingCheckedIn()
            throws Exception
    {
        // Manually checkout the administrative task assignment
        disableAutomaticCheckout();
        Result result = getApplicationInterface().checkOut(getSessionID(),
            administrativeTaskAssignment1, false);
        Assert.assertNull(result.getErrors());
        Assert.assertNull(result.getWarnings());

        // Log out and log back in the administrative task assignment
        getAuthenticateInterface().logout(sessionid);
        performLogin(resource.getUserName(),
            ResourceFixture.STATIC_RESOURCE_PASSWORD);
        
        // Verify that everything in the resource's timesheet is checked in
        Assert.assertFalse(isCheckedOut(administrativeTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1));
        Assert.assertFalse(isCheckedOut(personalTaskAssignment1
            .getPersonalTask()));
        Assert.assertFalse(isCheckedOut(summaryTimesheet1));
        Assert.assertFalse(isCheckedOut(step1));
        Assert.assertFalse(isCheckedOut(summaryTimesheet2));
        Assert.assertFalse(isCheckedOut(summaryTimesheet3));
    }
}
