package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class Test65224 extends ClientTestController
{
    private SummaryTimesheet summaryTimesheet;
    
    private TaskAssignment taskAssignment;
    
    public void test() throws Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);

        taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        assertNull(taskAssignment.getActualFinishDate());

        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        summaryTimesheet.setRegularFriday(taskAssignment.getEETCHours());
        taskAssignment.setEETCHours(null);
        taskAssignment.setPercentageComplete(null);
        
        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        saveAndValidate();

        assertNotNull(taskAssignment.getActualFinishDate());
        assertEquals(new Double(0.0), taskAssignment.getEETCHours());
        
        // reduce hours
        summaryTimesheet.setRegularFriday(new Double(2.0));
        taskAssignment.setEETCHours(null);
        taskAssignment.setPercentageComplete(null);
        
        saveAndValidate();
        assertEquals(new Double(38.0), taskAssignment.getEETCHours());
        assertNull(taskAssignment.getActualFinishDate());
    }
    
    private void saveAndValidate() throws Exception
    {
        SingleResult result = this.save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
        assertNotNull(summaryTimesheet.getTaskAssignment());
        assertTrue(summaryTimesheet.getTaskAssignment() instanceof TaskAssignment);
        taskAssignment = (TaskAssignment) summaryTimesheet.getTaskAssignment();
    }
}
