/*
 * Created on May 9, 2007
 * 
 * Defect 65274: WS - SummaryTimesheet.timecode* - fails when saving null
 * Description:
 * Discovered in Web UI defect 64606
 * Identify platform versions: API build 1607
 * Overview of problem
 * SummaryTimesheet.timecode* - fails when saving null
 * Steps to reproduce
 * Save a SummaryTimesheet with non null timecode (set scope timecode true)
 * Save a SummaryTimesheet with null timecode (set scope timecode true)
 * => Null pointer exception in the API's validator
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

/**
 * @author cobber
 */
public class Test65274 extends ClientTestController
{
    private SummaryTimesheetScope scope;

    private TaskAssignment taskAssignment;

    private Calendar weekOf;

    private SummaryTimesheet summaryTimesheet;

    public void setUp() throws Exception
    {
        super.setUp();

        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);

        scope = new SummaryTimesheetScope();
        scope.setTimeCodes(true);

        taskAssignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);

        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        summaryTimesheet = new SummaryTimesheet();
    }

    public void test() throws Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(project, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(project, false, this);
        project.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        summaryTimesheet.setTimeCode1(timecode1.getTimeCode());
        summaryTimesheet.setTimeCode2(timecode2.getTimeCode());

        saveAndValidate();
        assertNotNull(summaryTimesheet.getTimeCode1());
        assertNotNull(summaryTimesheet.getTimeCode2());

        // set timeCodes to null
        summaryTimesheet.setTimeCode1(null);
        summaryTimesheet.setTimeCode2(null);

        saveAndValidate();
        assertNull(summaryTimesheet.getTimeCode1());
        assertNull(summaryTimesheet.getTimeCode2());
    }

    private void saveAndValidate() throws Exception
    {
        SingleResult result = this.save(summaryTimesheet, scope);
        assertTrue(result.isSuccessful());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
    }
}
