package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class Test67460 extends ClientTestController {
	
	TaskAssignment taskAssignment;
	protected void setUp() throws Exception
	{
		super.setUp();
		
		taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
		assertNotNull(taskAssignment);
		SummaryTimesheet summaryTimeSheet = createSaveSummaryTimesheet(TimesheetTestUtil.TIMESHEET1_WEEK);
		
		summaryTimeSheet = saveUpdateSummaryTimesheetWithActuals(summaryTimeSheet, 2);
	}
	
	public void loadSummaryTimesheet() throws Exception
	{
		Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(),
				TimesheetTestUtil.TIMESHEET1_WEEK);
		String weekOfValue = weekOf.get(Calendar.MONTH) + 1 + "/"
		+ weekOf.get(Calendar.DATE) + "/"
		+ weekOf.get(Calendar.YEAR);
		
		String xpath = "/SummaryTimesheet[ @TaskAssignment [@id = '" + taskAssignment.getID() +"' ] and @weekOf = '" + weekOfValue + "']";
		
		RPMObject rpmObject[] = loadFromXpath(xpath, null);
		System.out.println();
	}
	
	public void testModifyEETCWithoutActuals() throws Exception
	{
		SummaryTimesheet summaryTimeSheet = createSaveSummaryTimesheet(TimesheetTestUtil.TIMESHEET2_WEEK);
		
		validateAndSaveUpdateSummaryTimeSheetWithModifiedEETC(summaryTimeSheet,true);
	}
	
	public void testModifyEETCWithActuals() throws Exception
	{
		SummaryTimesheet summaryTimeSheet = createSaveSummaryTimesheet(TimesheetTestUtil.TIMESHEET2_WEEK);
	
		summaryTimeSheet = saveUpdateSummaryTimesheetWithActuals(summaryTimeSheet, 2);
		
		validateAndSaveUpdateSummaryTimeSheetWithModifiedEETC(summaryTimeSheet,true);
	}
	
	public void testModifyEETCAndActuals() throws Exception
	{
		SummaryTimesheet summaryTimeSheet = createSaveSummaryTimesheet(TimesheetTestUtil.TIMESHEET2_WEEK);
		
		TimesheetTestUtil.setWorkedHours(summaryTimeSheet, 1, 0);
		
		validateAndSaveUpdateSummaryTimeSheetWithModifiedEETC(summaryTimeSheet,false);
	}
	
	private SummaryTimesheet createSaveSummaryTimesheet(int timeSheetWeek) throws Exception
	{
		Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(),
                timeSheetWeek);
		SummaryTimesheet summaryTimeSheet = SummaryTimesheetFixture.createSaveFixture(taskAssignment, weekOf, null, false, this);
		assertNotNull(summaryTimeSheet);
		assertNotNull(summaryTimeSheet.getTaskAssignment());
		taskAssignment = (TaskAssignment)summaryTimeSheet.getTaskAssignment();
		return summaryTimeSheet;
	}
	
	private SummaryTimesheet saveUpdateSummaryTimesheetWithActuals(SummaryTimesheet summaryTimesheet, int noOfHours) throws Exception
	{
		TimesheetTestUtil.setWorkedHours(summaryTimesheet, noOfHours, 0);
		taskAssignment.setEETCHours(null);
		taskAssignment.setPercentageComplete(null);
		summaryTimesheet.setTaskAssignment(taskAssignment);
		summaryTimesheet = (SummaryTimesheet)save(summaryTimesheet,null).getRpmObject();
		assertNotNull(summaryTimesheet);
		assertNotNull(summaryTimesheet.getTaskAssignment());
		taskAssignment = (TaskAssignment)summaryTimesheet.getTaskAssignment();
		return summaryTimesheet;
	}
	
	private void validateAndSaveUpdateSummaryTimeSheetWithModifiedEETC(SummaryTimesheet summaryTimesheet, boolean validatePC) throws Exception
	{
		Double eetc = taskAssignment.getEETCHours();
		taskAssignment.setEETCHours(new Double(eetc.doubleValue()+eetc.doubleValue()));
		Double percentageCompleted = taskAssignment.getPercentageComplete();
		taskAssignment.setPercentageComplete(null);
		summaryTimesheet.setTaskAssignment(taskAssignment);
		summaryTimesheet = (SummaryTimesheet)save(summaryTimesheet,null).getRpmObject();
		
		assertNotNull(summaryTimesheet);
		assertNotNull(summaryTimesheet.getTaskAssignment());
		if(validatePC) {
			assertEquals(percentageCompleted, ((TaskAssignment)summaryTimesheet.getTaskAssignment()).getPercentageComplete());
		}
	}

}
