package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.CreditTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.TaskAssignment;


public class TestCreditTimesheetValidation extends GenericValidation
{
    private SummaryTimesheet summaryTimesheet = null;

    private SummaryTimesheetScope summaryTimesheetScope = null;

    private StepScope creditScope = StepFixture.createScope(true, true, true, true,
        true, true);

    private Credit credit = new Credit();

    private TaskAssignment taskAssignment = null;
    
    private CreditTimesheet timesheet = new CreditTimesheet();
    
    private Calendar creditWeekOf = null;

    private Calendar weekOf = null;

    public void setUp() throws Exception
    {
        super.setUp();
    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        timesheet.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, 
                timesheet.getClass().getName(), ValidationConstants.PARENT_FIELD, timesheet.getID())
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }
    
   
    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, 
                timesheet.getClass().getName(), ValidationConstants.PARENT_FIELD, timesheet.getID())
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }
    
    public void testInsertWithoutMandatoryFields() throws AssertionError,
    	Exception
    {

        initSummaryTimesheetAndTaskAssignment();

        approvedSummaryTimesheet();
        initCredit();
        initStep();

        timesheet = credit.getCreditTimesheet();
        timesheet.setWeekOf(null);
        timesheet.setCreditWeekOf(null);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, 
                timesheet.getClass().getName(), ValidationConstants.WEEK_OF_FIELD, 
                timesheet.getID()),
            createRPMException(ErrorCodes.MANDATORY, 
                timesheet.getClass().getName(), ValidationConstants.CREDIT_WEEK_OF_FIELD, 
                timesheet.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    public void testFailureValidationMax() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment();
        approvedSummaryTimesheet();
        initCredit();  
        initStep();
        

        Double max_hours = new Double(
            ValidationConstants.MAX_WORKING_HOURS_PER_DAY + 1);
        timesheet
            .setSummaryNotes(getTestString(ValidationConstants.SUMMARY_NOTES_MAX) + 1);
        timesheet.setRegularMonday(max_hours);
        timesheet.setRegularTuesday(max_hours);
        timesheet.setRegularWednesday(max_hours);
        timesheet.setRegularThursday(max_hours);
        timesheet.setRegularFriday(max_hours);
        timesheet.setRegularSaturday(max_hours);
        timesheet.setRegularSunday(max_hours);
        timesheet.setSpecialMonday(max_hours);
        timesheet.setSpecialTuesday(max_hours);
        timesheet.setSpecialWednesday(max_hours);
        timesheet.setSpecialThursday(max_hours);
        timesheet.setSpecialFriday(max_hours);
        timesheet.setSpecialSaturday(max_hours);
        timesheet.setSpecialSunday(max_hours);
        
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, timesheet.getClass()
                    .getName(), ValidationConstants.SUMMARY_NOTES_FIELD,
                    timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    public void testFailureValidationMin() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment();
        initStep();
        approvedSummaryTimesheet();
        initCredit();        

        Double min_hours = new Double(
            ValidationConstants.MIN_WORKING_HOURS_PER_DAY - 1);
        timesheet.setRegularMonday(min_hours);
        timesheet.setRegularTuesday(min_hours);
        timesheet.setRegularWednesday(min_hours);
        timesheet.setRegularThursday(min_hours);
        timesheet.setRegularFriday(min_hours);
        timesheet.setRegularSaturday(min_hours);
        timesheet.setRegularSunday(min_hours);
        timesheet.setSpecialMonday(min_hours);
        timesheet.setSpecialTuesday(min_hours);
        timesheet.setSpecialWednesday(min_hours);
        timesheet.setSpecialThursday(min_hours);
        timesheet.setSpecialFriday(min_hours);
        timesheet.setSpecialSaturday(min_hours);
        timesheet.setSpecialSunday(min_hours);
        
       
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    public void testFailureInvalidValues() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment();
        initStep();
        approvedSummaryTimesheet();
        initCredit();        

        
        Double hours = new Double(1.8);
        timesheet.setRegularMonday(hours);
        timesheet.setRegularTuesday(hours);
        timesheet.setRegularWednesday(hours);
        timesheet.setRegularThursday(hours);
        timesheet.setRegularFriday(hours);
        timesheet.setRegularSaturday(hours);
        timesheet.setRegularSunday(hours);
        timesheet.setSpecialMonday(hours);
        timesheet.setSpecialTuesday(hours);
        timesheet.setSpecialWednesday(hours);
        timesheet.setSpecialThursday(hours);
        timesheet.setSpecialFriday(hours);
        timesheet.setSpecialSaturday(hours);
        timesheet.setSpecialSunday(hours);
        
      
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    private CreditTimesheet createTimesheet() throws Exception
    {
        RPMObjectScope stepScope = StepTimesheetFixture.createScope(true);
        timesheet = new CreditTimesheet();
        timesheet = CreditTimesheetFixture.createFixture(credit,
            weekOf, creditWeekOf,
            stepScope, this);
        return timesheet;
    }

    private void initSummaryTimesheetAndTaskAssignment() throws Exception
    {

        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        creditWeekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CREDIT_WEEK_OF);
        
        summaryTimesheetScope = SummaryTimesheetFixture.createScope(false,
            true, false, false, false);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(taskAssignment,
            creditWeekOf, summaryTimesheetScope, true, this);
       


    }

    private void initStep() throws Exception
    {

        //Create a step with a StepTimesheet for the current week

        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);

        //create a summaryTimesheet for the weekOf
        SummaryTimesheetFixture.createSaveFixture(summaryTimesheet
            .getTaskAssignment(), weekOf, null, false, this);
        
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setWeekOf(weekOf);

        StepFixture.createSaveFixture((TaskAssignment) summaryTimesheet
            .getTaskAssignment(), stepScope, new StepTimesheet[]
        {
            stepTimesheet
        }, this);

    }

    private void approvedSummaryTimesheet() throws Exception
    {

        TimesheetApprovalStatus approvalStatus = summaryTimesheet
            .getApprovalStatus();
        approvalStatus.setApprovalStatus(TimesheetStatus.Approved);

        SingleResult result = save(summaryTimesheet, summaryTimesheetScope);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();

    }
    
    private void initCredit() throws Exception
    {

        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());

    }

}
