package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;

/**
 * @author gmeloche
 *
 */
public class TestPersonalProjectValidation extends GenericValidation
{
    
    PersonalProject personalProject = null;
    Resource resource = null;
       
    
    public void testFailureOnCreate() throws AssertionError, Exception
    {
        personalProject = new PersonalProject();
        personalProject.setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CREATE,
                personalProject.getClass().getName()),
        };

        verifySaveMustFail(personalProject, null, expectedExceptions);
    }
    
    
    public void testFailureReadOnly() throws AssertionError, Exception
    {
        init();
        
        personalProject.setName("new name");

        RPMExceptionMock[] expectedWarning =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalProject.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, personalProject
                    .getID()),
        };

        verifySaveMustFail(personalProject, null, null, expectedWarning);
    }
    
    private void init()throws Exception{
        ResourceScope scope = new ResourceScope();
        scope.setPersonalProject(new PersonalProjectScope());
        
        resource = ResourceFixture.createSaveFixture(scope, this);
        
        personalProject = resource.getPersonalProject();
    }
}
