/**
 * Created on Jan 4, 2006
 * 
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetApprovalStatusFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestSummaryTimesheetValidation extends GenericValidation
{
    private SummaryTimesheet summaryTimesheet = null;

    private TaskAssignment taskAssignment = null;

    private SummaryTimesheetScope scope;
    
    private Calendar weekOf = null;

    public void setUp() throws Exception
    {
        super.setUp();

        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);

        scope = new SummaryTimesheetScope();
        scope.setTimeCodes(true);

        taskAssignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        summaryTimesheet = new SummaryTimesheet();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        summaryTimesheet.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_ALWAYS_BE_NULL, summaryTimesheet
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                summaryTimesheet.getID())
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureProjectNotReady() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.PROJECT_NOT_PUBLISHED_TO_TEAM,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.PUBLISHED_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.WORK_ELEMENT_STILL_PROPOSED,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.PROJECT_FIELD, summaryTimesheet.getID())
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testInsertSuccessfully() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // Attempt to save the object
        summaryTimesheet = (SummaryTimesheet) saveMustSucceed(summaryTimesheet,
            SummaryTimesheet.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTimesheet[weekof='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and taskassignment[id = '"
                + summaryTimesheet.getTaskAssignment().getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTimesheet);

        // Check that the loaded object is identical to the one we
        // requested to store
        SummaryTimesheetFixture.compare(summaryTimesheet,
            (SummaryTimesheet) rpmObjects[0], scope);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        summaryTimesheet.setTaskAssignment(taskAssignment);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, summaryTimesheet
                .getClass().getName(), ValidationConstants.WEEK_OF_FIELD,
                summaryTimesheet.getID())
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // set the Read-only fields
        Double num = new Double(5.3);
        summaryTimesheet.setTotalRegular(num);
        summaryTimesheet.setTotalSpecial(num);
        summaryTimesheet.setTotalWeek(num);

        summaryTimesheet.setResource(new Resource());
        summaryTimesheet.setWorkElement(new Task());
        summaryTimesheet.setProject(new Project());

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.TOTAL_REGULAR_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.TOTAL_SPECIAL_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.TOTAL_WEEK_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(), ValidationConstants.RESOURCE_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(), ValidationConstants.PROJECT_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.WORK_ELEMENT_FIELD, summaryTimesheet
                        .getID())
        };

        verifySaveMustFail(summaryTimesheet, null, null, expectedWarnings);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/SummaryTimesheet[weekof='"
                + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                + "' and taskassignment[id = '"
                + summaryTimesheet.getTaskAssignment().getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTimesheet);

        SummaryTimesheet loadedTimesheet = (SummaryTimesheet) rpmObjects[0];

        // assert that the values we set did not get saved in the db
        assertNotSame(summaryTimesheet.getTotalRegular(), loadedTimesheet
            .getTotalRegular());
        assertNotSame(summaryTimesheet.getTotalSpecial(), loadedTimesheet
            .getTotalSpecial());
        assertNotSame(summaryTimesheet.getTotalWeek(), loadedTimesheet
            .getTotalWeek());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        Double max_hours = new Double(
            ValidationConstants.MAX_WORKING_HOURS_PER_DAY + 1);
        summaryTimesheet
            .setSummaryNotes(getTestString(ValidationConstants.SUMMARY_NOTES_MAX) + 1);
        summaryTimesheet.setRegularMonday(max_hours);
        summaryTimesheet.setRegularTuesday(max_hours);
        summaryTimesheet.setRegularWednesday(max_hours);
        summaryTimesheet.setRegularThursday(max_hours);
        summaryTimesheet.setRegularFriday(max_hours);
        summaryTimesheet.setRegularSaturday(max_hours);
        summaryTimesheet.setRegularSunday(max_hours);
        summaryTimesheet.setSpecialMonday(max_hours);
        summaryTimesheet.setSpecialTuesday(max_hours);
        summaryTimesheet.setSpecialWednesday(max_hours);
        summaryTimesheet.setSpecialThursday(max_hours);
        summaryTimesheet.setSpecialFriday(max_hours);
        summaryTimesheet.setSpecialSaturday(max_hours);
        summaryTimesheet.setSpecialSunday(max_hours);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SUMMARY_NOTES_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        Double min_hours = new Double(
            ValidationConstants.MIN_WORKING_HOURS_PER_DAY - 1);
        summaryTimesheet.setRegularMonday(min_hours);
        summaryTimesheet.setRegularTuesday(min_hours);
        summaryTimesheet.setRegularWednesday(min_hours);
        summaryTimesheet.setRegularThursday(min_hours);
        summaryTimesheet.setRegularFriday(min_hours);
        summaryTimesheet.setRegularSaturday(min_hours);
        summaryTimesheet.setRegularSunday(min_hours);
        summaryTimesheet.setSpecialMonday(min_hours);
        summaryTimesheet.setSpecialTuesday(min_hours);
        summaryTimesheet.setSpecialWednesday(min_hours);
        summaryTimesheet.setSpecialThursday(min_hours);
        summaryTimesheet.setSpecialFriday(min_hours);
        summaryTimesheet.setSpecialSaturday(min_hours);
        summaryTimesheet.setSpecialSunday(min_hours);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureInvalidValues() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        Double hours = new Double(1.8);
        summaryTimesheet.setRegularMonday(hours);
        summaryTimesheet.setRegularTuesday(hours);
        summaryTimesheet.setRegularWednesday(hours);
        summaryTimesheet.setRegularThursday(hours);
        summaryTimesheet.setRegularFriday(hours);
        summaryTimesheet.setRegularSaturday(hours);
        summaryTimesheet.setRegularSunday(hours);
        summaryTimesheet.setSpecialMonday(hours);
        summaryTimesheet.setSpecialTuesday(hours);
        summaryTimesheet.setSpecialWednesday(hours);
        summaryTimesheet.setSpecialThursday(hours);
        summaryTimesheet.setSpecialFriday(hours);
        summaryTimesheet.setSpecialSaturday(hours);
        summaryTimesheet.setSpecialSunday(hours);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, summaryTimesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, summaryTimesheet
                        .getID()),
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureTimeCodesNotAssigned() throws AssertionError,
            Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        summaryTimesheet.setTimeCode1(timecode1);
        summaryTimesheet.setTimeCode2(timecode2);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TIMECODE1_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TIMECODE2_FIELD, summaryTimesheet
                        .getID()),
        };

        verifySaveMustFail(summaryTimesheet, scope, expectedExceptions);
    }

    public void testFailureModifyWhenStepsExist() throws AssertionError,
            Exception
    {
        // set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        Step[] activities = new Step[1];
        StepScope activityScope = new StepScope();
        activityScope.setTimesheets(true);
        activities[0] = StepFixture.createSaveFixture(taskAssignment, activityScope,
            null, this);
        //parentTaskAssignment.setActivities(activities);

        Double hours = new Double(1.75);
        summaryTimesheet.setRegularMonday(hours);
        summaryTimesheet.setRegularTuesday(hours);
        summaryTimesheet.setRegularWednesday(hours);
        summaryTimesheet.setRegularThursday(hours);
        summaryTimesheet.setRegularFriday(hours);
        summaryTimesheet.setRegularSaturday(hours);
        summaryTimesheet.setRegularSunday(hours);
        summaryTimesheet.setSpecialMonday(hours);
        summaryTimesheet.setSpecialTuesday(hours);
        summaryTimesheet.setSpecialWednesday(hours);
        summaryTimesheet.setSpecialThursday(hours);
        summaryTimesheet.setSpecialFriday(hours);
        summaryTimesheet.setSpecialSaturday(hours);
        summaryTimesheet.setSpecialSunday(hours);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD,
                    summaryTimesheet.getID())
        };

        verifySaveMustFail(summaryTimesheet, null, null, expectedWarnings);
    }

    public void testSuccessModifyWhenStepsExistForAnotherWeek()
            throws AssertionError, Exception
    {
        // set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        Step step = new Step();
        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);
        step = StepFixture.createFixture(taskAssignment,
            stepScope, null, this);
        step.setAlwaysAvailable(Boolean.FALSE);
        
        saveMustSucceed(step, Step.class, stepScope);

        Calendar otherWeek = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(),
            TimesheetTestUtil.OTHER_WEEK);
        
        System.out.println("weekOf" + DateUtil.convertToString(weekOf));
        System.out.println("other week" + DateUtil.convertToString(otherWeek));
        
        SummaryTimesheet nextWeekSummaryTimesheet = SummaryTimesheetFixture
            .createFixture(taskAssignment, otherWeek, null, false, this);

        TimesheetTestUtil.setWorkedHours(nextWeekSummaryTimesheet, 1.75, 1.75);

        saveMustSucceed(nextWeekSummaryTimesheet, SummaryTimesheet.class, null);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(project, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(project, false, this);
        project.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        summaryTimesheet.setTimeCode1(timecode2.getTimeCode());
        summaryTimesheet.setTimeCode2(timecode1.getTimeCode());

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TIMECODE1_FIELD, summaryTimesheet
                        .getID()),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    summaryTimesheet.getClass().getName(),
                    ValidationConstants.TIMECODE2_FIELD, summaryTimesheet
                        .getID()),
        };

        verifySaveMustFail(summaryTimesheet, scope, expectedExceptions);
    }
    
    public void testSuccessValidTimeCodes() throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(project, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(project, false, this);
        project.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        summaryTimesheet.setTimeCode1(timecode1.getTimeCode());
        summaryTimesheet.setTimeCode2(timecode2.getTimeCode());

        saveMustSucceed(summaryTimesheet, SummaryTimesheet.class, scope);
    }

    public void testFailureModifyAfterSubmitForApproval()
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        SummaryTimesheetScope scope = SummaryTimesheetFixture.createScope(
            false, true, false, false, false);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);
        
        Resource resource = SummaryTimesheetFixture.loadResource(summaryTimesheet, this);

        resource.getResourceSecurityRights().setCanUseWebServicesAPI(Boolean.TRUE);
        
        resource = (Resource)save(resource, null).getRpmObject();
        
        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);
        
        summaryTimesheet.setRegularMonday(new Double(2.0));

        // save the object
        summaryTimesheet = (SummaryTimesheet) saveMustSucceed(summaryTimesheet,
            SummaryTimesheet.class, scope);
        
        // submit the timesheet for approval
        TimesheetApprovalStatus appStatus = new TimesheetApprovalStatus();
        appStatus.setParent(summaryTimesheet);
        appStatus.setResource(resource);
        appStatus.setWeekOf(summaryTimesheet.getWeekOf());
        
        TimesheetApprovalStatusFixture.submitTimesheetAndSave(appStatus, null, true, this);
        
        // modify the timesheet and save again
        summaryTimesheet.setRegularMonday(new Double(8));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.CANNOT_MODIFY_TIMESHEET,
                StringUtil.getShortClassName(summaryTimesheet.getClass()),
                ValidationConstants.APPROVAL_STATUS_FIELD,
                summaryTimesheet.getID())
        };

        verifySaveMustFail(summaryTimesheet, scope, expectedExceptions);
    }

    public void testInsertWithoutTaskAssignment() throws AssertionError,
            Exception
    {
        summaryTimesheet.setWeekOf(weekOf);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, summaryTimesheet
                .getClass().getName(),
                ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                    .getID())
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }

    public void testFailureTaskAssignmentWithProfile() throws AssertionError,
            Exception
    {
        // Create a TaskAssignmnet with aprofile
        ResourceTaskAssignmentScope resTskScope = ResourceTaskAssignmentFixture
            .createScope(true, true, true, false, false);
        ResourceTaskAssignment resTskAssignmnet = ResourceTaskAssignmentFixture
            .createSaveFixture(resTskScope, false, this);
        taskAssignment = resTskAssignmnet.getTaskAssignment();

        // set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.TASK_ASSIGNMENT_WITHOUT_RESOURCE,
                summaryTimesheet.getClass().getName(),
                ValidationConstants.TASK_ASSIGNMENT_FIELD, summaryTimesheet
                    .getID())
        };

        verifySaveMustFail(summaryTimesheet, null, expectedExceptions);
    }
    
    public void testSuccessValidTimeCodesAssignToTask() throws Exception
    {
        assertNotNull(taskAssignment.getParent());
        assertNotNull(taskAssignment.getParent().getParent());
        assertTrue(taskAssignment.getParent().getParent() instanceof WorkElement);
        WorkElement workElement = (WorkElement)taskAssignment.getParent().getParent();
        
        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);

        // this will do both copy proposed to plan and publish project to team
        TimesheetTestUtil.initProject(project, this);

        // assign the timecodes to the workElement not project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(workElement, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(workElement, false, this);
        workElement.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        summaryTimesheet.setTimeCode1(timecode1.getTimeCode());
        summaryTimesheet.setTimeCode2(timecode2.getTimeCode());

        assertNull(project.getTimeCodeAssignments());
        saveMustSucceed(summaryTimesheet, SummaryTimesheet.class, scope);
    }
}
