/*
 * Created on Sep 2, 2005
 */
package com.ibm.rpm.client.junit.wbs.fixtures;

/**
 * @author vnarayan
 */

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class PortfolioFixture extends RPMObjectFixture
{

    private static final String ROOT_TEST_PORTFOLIO_NAME = "ROOT_TEST_PORTFOLIO";
    
    private static final String ROOT_SECURITY_TEST_PORTFOLIO_NAME = "ROOT_SECURITY_TEST_PORTFOLIO";

    protected static ArrayList localSkipGetters = new ArrayList();

    protected static ArrayList localSkipSetters = new ArrayList();

    static
    {
        // set skipGetters
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getDefaultProject");
        localSkipGetters.add("getRank");

        // set skipSetters
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setDefaultProject");
        localSkipSetters.add("setRank");
    }

    public static Portfolio loadOrCreateRootPortfolio(ClientTestController ctc)
    throws Exception
    {

        RPMObject[] objs = ctc.loadFromXpath("/PORTFOLIO[NAME='"
        + ROOT_TEST_PORTFOLIO_NAME + "']", null);

        if (objs == null || objs.length == 0)
        {
            Portfolio rootPortfolio = createFixture(0, 0, null, ctc);
            rootPortfolio.setName(ROOT_TEST_PORTFOLIO_NAME);
            rootPortfolio = (Portfolio) save(ctc, rootPortfolio, null);
            // we don't want the root portfolio to be removed at the end of the
            // test
            ctc.removeObjectToDelete(rootPortfolio);
            return rootPortfolio;
        }
        else
        {
            return (Portfolio) objs[0];
        }
    }
    
    public static Portfolio loadOrCreateRootPortfolioForSecurityTester(ClientTestController ctc)
    throws Exception
    {

        RPMObject[] objs = ctc.loadFromXpath("/PORTFOLIO[NAME='"
        + ROOT_SECURITY_TEST_PORTFOLIO_NAME + "']", null);

        if (objs == null || objs.length == 0)
        {
            Portfolio rootPortfolio = createFixture(0, 0, null, ctc);
            rootPortfolio.setName(ROOT_SECURITY_TEST_PORTFOLIO_NAME);
            rootPortfolio = (Portfolio) save(ctc, rootPortfolio, null);
            // we don't want the root portfolio to be removed at the end of the
            // test
            ctc.removeObjectToDelete(rootPortfolio);
            return rootPortfolio;
        }
        else
        {
            return (Portfolio) objs[0];
        }
    }

    public static Portfolio createFixture(int nbPortfoliosByLevel, int nbProposalsByLevel, 
            PortfolioScope scope, ClientTestController ctc) throws Exception
    {

        return createPortfolio(random(), nbPortfoliosByLevel,

        nbProposalsByLevel, scope, ctc);

    }

    public static Portfolio createSaveFixture(ClientTestController ctc)
            throws Exception

    {

        Portfolio portfolio = createFixture(0, 0, null, ctc);

        portfolio = (Portfolio) save(ctc, portfolio, null);

        return portfolio;

    }

    private static Portfolio createPortfolio(int index, int nbPortfoliosByLevel, int nbProposalsByLevel, 
            PortfolioScope scope, ClientTestController ctc) throws Exception
    {

        Portfolio portfolio = new Portfolio();

        HelperTestUtilities.setByIntrospection(portfolio, index,

        localSkipSetters);

        if (scope != null)

        {

            // create subportfolios

            if (scope.getPortfolios() != null)

            {

                Portfolio subPortfolio = null;

                Portfolio[] subPortfolios = new Portfolio[nbPortfoliosByLevel];

                for (int i = 1; i <= nbPortfoliosByLevel; i++)

                {

                    subPortfolio = PortfolioFixture.createFixture(

                    nbPortfoliosByLevel, nbProposalsByLevel, scope

                    .getPortfolios(), ctc);

                    subPortfolio.setParent(portfolio);

                    subPortfolios[i - 1] = subPortfolio;

                }

                portfolio.setPortfolios(subPortfolios);

            }

            // create Proposal

            if (scope.getProjects() != null)

            {

                GenericProject project = null;

                GenericProject[] projects = new GenericProject[nbProposalsByLevel * 3];

                for (int i = 0; i < nbProposalsByLevel * 3; i += 3)

                {

                    project = (Proposal) WorkElementFixture.createFixture(

                    new Proposal(), scope.getProjects(), ctc);

                    project.setPortfolio(portfolio);

                    projects[i] = project;

                    project = (Project) WorkElementFixture.createFixture(

                    new Project(), scope.getProjects(), ctc);

                    project.setPortfolio(portfolio);

                    projects[i + 1] = project;

                    project = (WorkOrganization) WorkElementFixture

                    .createFixture(new WorkOrganization(),

                    scope.getProjects(), ctc);

                    project.setPortfolio(portfolio);

                    projects[i + 2] = project;

                }

                portfolio.setProjects(projects);

            }

        }

        return portfolio;

    }

    // //update fields

    public static void updateFixtureFields(Portfolio portfolio,

    PortfolioScope scope, ClientTestController ctc) throws Exception

    {

        updateFields(portfolio, random(), scope, ctc);

    }

    private static void updateFields(Portfolio portfolio, int index,

    PortfolioScope scope, ClientTestController ctc) throws Exception

    {

        HelperTestUtilities.setByIntrospection(portfolio, index, skipSetters);

        if (scope != null)

        {

            // update Sub portfolio fields

            if (scope.getPortfolios() != null)

            {

                for (int i = 0; i < portfolio.getPortfolios().length; i++)

                {

                    PortfolioFixture.updateFixtureFields(portfolio

                    .getPortfolios()[i], scope.getPortfolios(), ctc);

                }

            }

            // update Proposal fields.

            if (scope.getProjects() != null)

            {

                for (int i = 0; i < portfolio.getProjects().length; i++)

                {

                    WorkElementFixture.updateFixtureFields(portfolio

                    .getProjects()[i], scope.getProjects(), ctc);

                }

            }

        }

    }

    public static PortfolioScope createScope(int currentLevel, int maxLevels,

    int nbPortfoliosByLevel, int nbProposalsByLevel)

    {

        PortfolioScope scope = new PortfolioScope();

        if (currentLevel <= maxLevels)

        {

            if (nbPortfoliosByLevel > 0 && currentLevel < maxLevels)

            {

                // create subfolders scope

                PortfolioScope subPortfolioScope = PortfolioFixture

                .createScope(currentLevel + 1, maxLevels,

                nbPortfoliosByLevel, nbProposalsByLevel);

                subPortfolioScope.setParent(scope);

                scope.setPortfolios(subPortfolioScope);

            }

            if (nbProposalsByLevel > 0)

            {

                WorkElementScope projectScope = new WorkElementScope();

                projectScope.setPortfolio(scope);

                scope.setProjects(projectScope);

            }

        }

        return scope;

    }

    private static ArrayList mergeArrays(Object[] array1, Object[] array2)

    {

        ArrayList temp = new ArrayList();

        addArray(temp, array1);

        addArray(temp, array2);

        return temp;

    }

    private static void addArray(ArrayList result, Object[] array)

    {

        if (array != null && array.length != 0)

        {

            for (int i = 0; i < array.length; i++)

            {

                result.add(array[i]);

            }

        }

    }

    // update structure

    public static void updateFixtureStructure(Portfolio portfolio,

    int deltaNbPortfoliosByLevel, int deltaNbProjectsByLevel,

    PortfolioScope scope, ClientTestController ctc) throws Exception

    {

        updateStructure(portfolio, deltaNbPortfoliosByLevel,

        deltaNbProjectsByLevel, scope, ctc);

    }

    private static void updateStructure(Portfolio portfolio,

    int deltaNbPortfoliosByLevel, int deltaNbProjectsByLevel,

    PortfolioScope scope, ClientTestController ctc) throws Exception

    {

        if (scope != null)

        {
            int newNbProjectsByLevel = 0;
            // add new Projects/WorkOrganizations & Proposals.

            if (scope.getProjects() != null)

            {

                if (deltaNbProjectsByLevel > 0)

                {// add new elements

                    GenericProject newGenericProject = null;

                    GenericProject[] newGenericProjects = new GenericProject[deltaNbProjectsByLevel * 3];

                    for (int i = 0; i < deltaNbProjectsByLevel; i += 3)

                    {

                        // No need to create any sub elements under the

                        // projects.

                        newGenericProject = (GenericProject) WorkElementFixture
                            .createFixture(

                            new Proposal(), scope.getProjects(), ctc);

                        newGenericProject.setPortfolio(portfolio);

                        newGenericProjects[i] = newGenericProject;

                        newGenericProject = (GenericProject) WorkElementFixture
                            .createFixture(

                            new Project(), scope.getProjects(), ctc);

                        newGenericProject.setPortfolio(portfolio);

                        newGenericProjects[i++] = newGenericProject;

                        newGenericProject = (GenericProject) WorkElementFixture
                            .createFixture(

                            new WorkOrganization(), scope.getProjects(), ctc);

                        newGenericProject.setPortfolio(portfolio);

                        newGenericProjects[i++] = newGenericProject;

                    }

                    ArrayList temp = mergeArrays(portfolio.getProjects(),

                    newGenericProjects);

                    GenericProject[] totalProjects = new GenericProject[temp

                    .size()];

                    temp.toArray(totalProjects);

                    portfolio.setProjects(totalProjects);

                    newNbProjectsByLevel = totalProjects.length;

                }

            }

            // add new sub portfolios

            if (scope.getPortfolios() != null)

            {

                if (deltaNbPortfoliosByLevel > 0)

                {// add new elements

                    Portfolio newPortfolio = null;

                    Portfolio[] newPortfolios = new Portfolio[deltaNbPortfoliosByLevel];

                    for (int i = 0; i < deltaNbPortfoliosByLevel; i++)

                    {

                        newPortfolio = PortfolioFixture.createFixture(

                        deltaNbPortfoliosByLevel, newNbProjectsByLevel,

                        scope.getPortfolios(), ctc);

                        newPortfolio.setParent(portfolio);

                        newPortfolios[i] = newPortfolio;

                    }

                    ArrayList temp = mergeArrays(portfolio.getPortfolios(),

                    newPortfolios);

                    Portfolio[] totalPortfolios = new Portfolio[temp.size()];

                    temp.toArray(totalPortfolios);

                    portfolio.setPortfolios(totalPortfolios);

                }

            }

        }

    }

    public static void compare(Portfolio p1, Portfolio p2, PortfolioScope scope)

    throws AssertionFailedError, Exception

    {

        RPMObjectFixture.compareWithSkip(p1, p2, localSkipGetters);

        if (scope == null)

            return;

        // validate Portfolios

        if (scope.getPortfolios() != null)

        {

            PortfolioScope s = scope.getPortfolios();

            if (s != null)

            {

                s.setPortfolios(null);

            }

            PortfolioFixture.compare(p1.getPortfolios(), p2.getPortfolios(), s);

        }

        if (scope.getProjects() != null)

        {

            WorkElementScope s = scope.getProjects();

            if (s != null)

            {

                s.setPortfolio(null);

            }

            WorkElementFixture.compare(p1.getProjects(), p2.getProjects(), s);

        }
    }

    public static void compare(Portfolio[] fArray1, Portfolio[] fArray2,

    PortfolioScope scope) throws AssertionFailedError, Exception

    {
        sortWithComparator(fArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(fArray2, RPMObjectNameAndIDComparator.INSTANCE);
        RPMObjectFixture.compare(fArray1, fArray2);

        if (fArray1 == null)

            return;

        for (int i = 0; i < fArray1.length; i++)

        {

            PortfolioFixture.compare(fArray1[i], fArray2[i], scope);

        }

    }

}