/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.wbs.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *  
 */
public class WbsModuleFixture extends AbstractModuleFixture
{
    public static WbsModule createFixture(WbsModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static WbsModule updateFixture(WbsModule module,
            WbsModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static WbsModule createModule(WbsModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        WbsModule module = new WbsModule();
        
        AssetFinancialCategory category = new AssetFinancialCategory();
        category.setID(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_NONE);
        category.setValue("None");

        module.setAssetFinancialCategories(new AssetFinancialCategory[]
        {
            category
        });
        
        if (scope != null)
        {
            if (scope.getPortfolios() != null)
            {
                Portfolio[] portfolios = (Portfolio[]) createElements(scope
                    .getPortfolios(), Portfolio.class, ctc);
                module.setPortfolios(portfolios);
            }
            if (scope.getGenericProjects() != null)
            {
                GenericProject[] genericProjects = (GenericProject[]) createElements(
                    scope.getGenericProjects(), GenericProject.class, ctc);
                module.setGenericProjects(genericProjects);
            }
            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                    scope.getAttributeCategories(), AttributeCategory.class,
                    ctc);
                module.setAttributeCategories(attributeCategories);
            }
            if (scope.getScorecardFolders() != null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) createElements(
                    scope.getScorecardFolders(), ScorecardFolder.class, ctc);
                module.setScorecardFolders(scorecardFolders);
            }
            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) createElements(
                    scope.getCustomFieldCategories(),
                    CustomFieldCategory.class, ctc);
                module.setCustomFieldCategories(customFieldCategories);
            }
        }

        return module;
    }

    private static WbsModule updateModule(WbsModule module,
            WbsModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if (scope.getPortfolios() != null)
            {
                Portfolio[] portfolios = (Portfolio[]) updateElements(module
                    .getPortfolios(), scope.getPortfolios(), ctc);
                module.setPortfolios(portfolios);
            }
            if (scope.getGenericProjects() != null)
            {
                GenericProject[] genericProjects = (GenericProject[]) updateElements(
                    module.getGenericProjects(), scope.getGenericProjects(),
                    ctc);
                module.setGenericProjects(genericProjects);
            }
            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) updateElements(
                    module.getAttributeCategories(), scope
                        .getAttributeCategories(), ctc);
                module.setAttributeCategories(attributeCategories);
            }
            if (scope.getScorecardFolders() != null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) updateElements(
                    module.getScorecardFolders(), scope.getScorecardFolders(),
                    ctc);
                module.setScorecardFolders(scorecardFolders);
            }
            if (scope.getCustomFieldCategories() != null)
            {
                module.setCustomFieldCategories(CustomFieldFixture
                    .modifyCustomFieldCategories(module
                        .getCustomFieldCategories(), scope
                        .getCustomFieldCategories()));
            }
        }
        return module;
    }

    public static WbsModuleScope createFullScope()
    {
        return createScope(true, true, true, true, true, true, true, true,
            true, true, true, true, true, true, true, true, true);
    }

    public static WbsModuleScope createScope(boolean isPortfolios,
            boolean isGenericProjects, boolean isAttributeCategories,
            boolean isAssetFinancialCategories, boolean isAssignementStatus,
            boolean isWorkLocations, boolean isStates,
            boolean isDeliverableStates, boolean isTaskStates,
            boolean isProjectRtfs, boolean isDeliverableRtfs,
            boolean isWorkProductRtfs, boolean isSummaryTaskRtfs,
            boolean isMilestoneRfts, boolean isTaskRtfs,
            boolean isCustomFields, boolean isScorecardFolders)
    {
        WbsModuleScope scope = new WbsModuleScope();

        if (isPortfolios)
        {
            scope.setPortfolios(new PortfolioScope());
        }
        if (isGenericProjects)
        {
            scope.setGenericProjects(new WorkElementScope());
        }
        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }
        if (isAssetFinancialCategories)
        {
            scope.setAssetFinancialCategories(true);
        }
        if (isAssignementStatus)
        {
            scope.setAssignmentStatus(true);
        }
        if (isWorkLocations)
        {
            scope.setWorkLocations(true);
        }
        if (isStates)
        {
            scope.setStates(true);
        }
        if (isDeliverableStates)
        {
            scope.setDeliverableStates(true);
        }
        if (isTaskStates)
        {
            scope.setTaskStates(true);
        }
        if (isProjectRtfs)
        {
            scope.setProjectRtfs(true);
        }
        if (isDeliverableRtfs)
        {
            scope.setDeliverableRtfs(true);
        }
        if (isWorkProductRtfs)
        {
            scope.setWorkProductRtfs(true);
        }
        if (isSummaryTaskRtfs)
        {
            scope.setSummaryTaskRtfs(true);
        }
        if (isMilestoneRfts)
        {
            scope.setMilestoneRtfs(true);
        }
        if (isTaskRtfs)
        {
            scope.setTaskRtfs(true);
        }
        if (isScorecardFolders)
        {
            scope.setScorecardFolders(new ScorecardFolderScope());
        }
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(new CustomFieldScope());
        }
        return scope;
    }

    public static void compare(WbsModule obj1, WbsModule obj2,
            WbsModuleScope scope) throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if (scope.getPortfolios() != null)
            {
                PortfolioFixture.compare(obj1.getPortfolios(), obj2
                    .getPortfolios(), scope.getPortfolios());
            }
            if (scope.getGenericProjects() != null)
            {
                WorkElementFixture.compare(obj1.getGenericProjects(), obj2
                    .getGenericProjects(), scope.getGenericProjects());
            }
            if (scope.getAttributeCategories() != null)
            {
                AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                    .getAttributeCategories(), scope.getAttributeCategories());
            }
            if (scope.isAssetFinancialCategories())
            {
                DatafieldFixture.compare(obj1.getAssetFinancialCategories(),
                    obj2.getAssetFinancialCategories());
            }
            if (scope.isAssignmentStatus())
            {
                DatafieldFixture.compare(obj1.getAssignmentStatus(), obj2
                    .getAssignmentStatus());
            }
            if (scope.isWorkLocations())
            {
                DatafieldFixture.compare(obj1.getWorkLocations(), obj2
                    .getWorkLocations());
            }
            if (scope.isStates())
            {
                DatafieldFixture.compare(obj1.getStates(), obj2.getStates());
            }
            if (scope.isDeliverableStates())
            {
                DatafieldFixture.compare(obj1.getDeliverableStates(), obj2
                    .getDeliverableStates());
            }
            if (scope.isTaskStates())
            {
                DatafieldFixture.compare(obj1.getTaskStates(), obj2
                    .getTaskStates());
            }
            if (scope.isProjectRtfs())
            {
                DatafieldFixture.compare(obj1.getProjectRtfs(), obj2
                    .getProjectRtfs());
            }
            if (scope.isDeliverableRtfs())
            {
                DatafieldFixture.compare(obj1.getDeliverableRtfs(), obj2
                    .getDeliverableRtfs());
            }
            if (scope.isWorkProductRtfs())
            {
                DatafieldFixture.compare(obj1.getWorkProductRtfs(), obj2
                    .getWorkProductRtfs());
            }
            if (scope.isSummaryTaskRtfs())
            {
                DatafieldFixture.compare(obj1.getSummaryTaskRtfs(), obj2
                    .getSummaryTaskRtfs());
            }
            if (scope.isMilestoneRtfs())
            {
                DatafieldFixture.compare(obj1.getMilestoneRtfs(), obj2
                    .getMilestoneRtfs());
            }
            if (scope.isTaskRtfs())
            {
                DatafieldFixture
                    .compare(obj1.getTaskRtfs(), obj2.getTaskRtfs());
            }
            if (scope.getScorecardFolders() != null)
            {
                ScorecardFixture.compare(obj1.getScorecardFolders(), obj2
                    .getScorecardFolders(), scope.getScorecardFolders());
            }
            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldFixture.assertIdEqual(obj1
                    .getCustomFieldCategories(), obj2
                    .getCustomFieldCategories(), scope
                    .getCustomFieldCategories());
            }
        }
    }
}