package com.ibm.rpm.client.junit.wbs.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultStepFixture;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ConstraintType;
import com.ibm.rpm.wbs.types.TaskType;

public class WorkElementFixture extends RPMObjectFixture

{
    private static Project STATIC_PROJECT_FIXTURE = null;
    private static Proposal STATIC_PROPOSAL_FIXTURE = null;
    private static WorkOrganization STATIC_WORK_ORGANIZATION_FIXTURE = null;
    private static Task STATIC_TASK_FIXTURE = null;
    private static WorkProduct STATIC_WORK_PRODUCT_FIXTURE = null;
    private static Milestone STATIC_MILESTONE_FIXTURE = null;
    private static Deliverable STATIC_DELIVERABLE_FIXTURE = null;
    private static SummaryTask STATIC_SUMMARY_TASK_FIXTURE = null;

    public static String XPATH_PROJECT_NAME = "STATIC_PROJECT_FIXTURE";
    private static String XPATH_PROPOSAL_NAME = "STATIC_PROPOSAL_FIXTURE";
    private static String XPATH_WORK_ORGANIZATION_NAME = "STATIC_WORK_ORGANIZATION_FIXTURE";
    private static String XPATH_TASK_NAME = "STATIC_TASK_FIXTURE";
    private static String XPATH_WORK_PRODUCT_NAME = "STATIC_WORK_PRODUCT_FIXTURE";
    private static String XPATH_DELIVERABLE_NAME = "STATIC_DELIVERABLE_FIXTURE";
    private static String XPATH_MILESTONE_NAME = "STATIC_MILESTONE_FIXTURE";
    private static String XPATH_SUMMARY_TASK_NAME = "STATIC_SUMMARY_TASK_FIXTURE";
    
    protected static ArrayList localSkipGetters = new ArrayList();

    protected static ArrayList localSkipSetters = new ArrayList();
    static
    {
        //set skipGetters
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getDefaultProject");
        localSkipGetters.add("getPercentageDurationCompleted");
        localSkipGetters.add("getReferenceNumber");
        localSkipGetters.add("getMinutesPerDay");
        localSkipGetters.add("getLastModifiedDate");
        
        //A specific test must be created for these fields
        localSkipGetters.add("getConstraintDate");
        localSkipGetters.add("getConstraintType");

        localSkipGetters.add("getWbsSortingRank");
        localSkipGetters.add("getTemplateSortingRank");
        
        localSkipGetters.add("getEarnedValuesCalculationState");
        
        localSkipGetters.add("getReferenceID"); // read-only
        localSkipGetters.add("getClosedByDate"); // read-only
        localSkipGetters.add("getClosedByName"); // read-only
        
        //set skipSetters
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setDefaultProject");
        localSkipSetters.add("setRank"); // we set it explicitly
        localSkipSetters.add("setPriority"); // we set it explicitly
        localSkipSetters.add("setPercentageDurationCompleted");
        localSkipSetters.add("setMinutesPerDay");  
        localSkipSetters.add("setProbabilityPercent"); // has to be between 0-100
        localSkipSetters.add("setEstimatedStartDate");
        localSkipSetters.add("setEstimatedFinishDate");
        localSkipSetters.add("setLastModifiedDate"); // read-only
        
        localSkipSetters.add("setWbsSortingRank");
        localSkipSetters.add("setTemplateSortingRank");
        
        localSkipSetters.add("setReferenceID"); // read-only
        localSkipSetters.add("setClosedByDate"); // read-only
        localSkipSetters.add("setClosedByName"); // read-only
        
        //A specific test must be created for these fields
        localSkipSetters.add("setConstraintDate");
        localSkipSetters.add("setConstraintType");
        
        localSkipSetters.add("setEarnedValuesCalculationState");
    }

    public static WorkElement createSaveFixtureFullScope(WorkElement baseWorkElement,
            ClientTestController ctc) throws Exception
    {
        WorkElementScope scope = createScope(baseWorkElement, true, true, true, 
            true, true, true, true, true, true, true, true, true, true, true,true);
        return createSaveFixture(baseWorkElement, scope, ctc);
    }
    
    public static WorkElement createSaveFixture(WorkElement baseWorkElement,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        return createSaveFixture(null, baseWorkElement, scope, ctc);
    }

  
    public static WorkElement createSaveFixture(RPMObject parent,
            WorkElement baseWorkElement, WorkElementScope scope,
            ClientTestController ctc) throws Exception
    {
        //default is proposal
        WorkElement workElement = createFixture(parent, baseWorkElement,
           scope, ctc);
        workElement = (WorkElement) save(ctc, workElement, scope);
        return workElement;
    }

    public static WorkElement createFixture(WorkElement baseWorkElement,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        return createFixture(null, baseWorkElement, scope, ctc);
    }

    public static WorkElement createFixture(RPMObject parent,
            WorkElement baseWorkElement, WorkElementScope scope,
            ClientTestController ctc) throws Exception
    {
        if (parent == null)
        {
            if (baseWorkElement instanceof GenericProject)
            {
                GenericProject we = (GenericProject) baseWorkElement;
                if(we.getPortfolio() == null || we.getPortfolio().getID() == null)
                {
                	// 	set the Portfolio attribute and no parent
                	we.setPortfolio(PortfolioFixture.loadOrCreateRootPortfolio(ctc));
                }
            }
            else
            {
                // set a parent Project
                parent = retrieveStaticFixture(new Project(), ctc);
            }
        }
        return createWorkElement(parent, baseWorkElement, random(),
            scope, ctc);
    }
    
    public static WorkElement retrieveStaticFixture(
            WorkElement baseWorkElement, ClientTestController ctc) 
            throws Exception
    {
        return retrieveStaticFixture(baseWorkElement, null, ctc);
    }

    /*
     * Use this method to create work elements for use as parents or
     * as assigned objects only. The work elements are reused.
     */
    public static WorkElement retrieveStaticFixture(
            WorkElement baseWorkElement, WorkElementScope weScope,
            ClientTestController ctc) throws Exception
    {
        WorkElement workElement = null;
        String xpathName = null;

        if (baseWorkElement instanceof Project)
        {
            xpathName = XPATH_PROJECT_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_PROJECT_FIXTURE = (Project) workElement;
        }
        else if (baseWorkElement instanceof Proposal)
        {
            xpathName = XPATH_PROPOSAL_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_PROPOSAL_FIXTURE = (Proposal) workElement;
        }
        else if (baseWorkElement instanceof WorkOrganization)
        {
            xpathName = XPATH_WORK_ORGANIZATION_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_WORK_ORGANIZATION_FIXTURE = (WorkOrganization) workElement;
        }
        else if (baseWorkElement instanceof Task)
        {
            xpathName = XPATH_TASK_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_TASK_FIXTURE = (Task) workElement;
        }
        else if (baseWorkElement instanceof WorkProduct)
        {
            xpathName = XPATH_WORK_PRODUCT_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_WORK_PRODUCT_FIXTURE = (WorkProduct) workElement;
        }
        else if (baseWorkElement instanceof Deliverable)
        {
            xpathName = XPATH_DELIVERABLE_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_DELIVERABLE_FIXTURE = (Deliverable) workElement;
        }
        else if (baseWorkElement instanceof Milestone)
        {
            xpathName = XPATH_MILESTONE_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_MILESTONE_FIXTURE = (Milestone) workElement;
        }
        else if (baseWorkElement instanceof SummaryTask)
        {
            xpathName = XPATH_SUMMARY_TASK_NAME;

            // Static fixture should always be reloaded from the DB
            workElement = loadFixture(baseWorkElement, weScope, ctc, xpathName);
            STATIC_SUMMARY_TASK_FIXTURE = (SummaryTask) workElement;
        }
        else
        {
            Assert
                .fail("WorkElementFixture.retrieveStaticFixture : Invalid WorkElement ");
        }

        return workElement;

    }

    public static WorkElement loadFixture(WorkElement baseWorkElement,
            ClientTestController ctc, String xpathName)
            throws Exception
    {
        return loadFixture(baseWorkElement, null, ctc, xpathName);
    }
    
    public static WorkElement loadFixture(WorkElement baseWorkElement,
            WorkElementScope weScope, ClientTestController ctc, String xpathName)
            throws Exception
    {
        // Try loading the WorkElement from xpath
        RPMObject[] rpmObjects = null;
        WorkElement workElement = null;
        
        if(weScope == null)
        {
            weScope = new WorkElementScope();
        }
        weScope.setContainingProject(weScope);

        rpmObjects = ctc.loadFromXpath(
            "/" + StringUtil.getShortClassName(baseWorkElement.getClass())
                    + "[name='" + xpathName + "']", weScope);

        if (rpmObjects != null && rpmObjects.length > 0 && rpmObjects[0] != null)
        {
            workElement = (WorkElement) rpmObjects[0];
            if(ctc.isCheckedOut(workElement))
            {
                ctc.forceCheckinOnRPMObjectCheckOut(workElement);
            }
        }
        else
        {
            // Create the WorkElement using the fixture, set its name, then save it
            workElement = createFixture(baseWorkElement, null, ctc);
            workElement.setName(xpathName);

            //Calendar are set for profile and resource task assignment.
            Calendar calendar = Calendar.getInstance();
            calendar.set(2006, 12, 22, 00, 00);
            workElement.setEstimatedStartDate(calendar);
            calendar.set(2008, 10, 20, 00,00);
            workElement.setEstimatedFinishDate(calendar);
            
            workElement = (WorkElement) saveWithoutDelete(ctc, workElement, null);
        }
        return workElement;
    }

    private static WorkElement createWorkElement(RPMObject parent,
            WorkElement baseWorkElement, int index,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        WorkElement workElement = null;
        Currency currency = null;
        Opportunity opportunity = null;
        boolean isAggregateNode = false;

        if (baseWorkElement instanceof Project)
        {
            workElement = new Project();
        }
        else if (baseWorkElement instanceof Proposal)
        {
            workElement = new Proposal();
        }
        else if (baseWorkElement instanceof WorkOrganization)
        {
            workElement = new WorkOrganization();
        }
        else if (baseWorkElement instanceof SummaryTask)
        {
            isAggregateNode = true;
            workElement = new SummaryTask();
        }
        else if (baseWorkElement instanceof Deliverable)
        {
            workElement = new Deliverable();
            isAggregateNode = true;
        }
        else if (baseWorkElement instanceof WorkProduct)
        {
            workElement = new WorkProduct();
            isAggregateNode = true;
        }
        else if (baseWorkElement instanceof Milestone)
        {
            workElement = new Milestone();
        }
        else if (baseWorkElement instanceof Task)
        {
            workElement = new Task();
        }
        else
        {
            Assert
                .fail("WorkElementFixture.createWorkElement : Invalid Element for WBS ");
        }
        
       	workElement.setParent(parent);

        HelperTestUtilities.setByIntrospection(workElement, index,
            localSkipSetters);
        workElement.setPriority(new Integer(55));
        GregorianCalendar startDate = new GregorianCalendar();
        GregorianCalendar endDate = new GregorianCalendar();
        workElement.setEstimatedStartDate(startDate);
        endDate.set(Calendar.YEAR, endDate.get(Calendar.YEAR)+1);
        workElement.setEstimatedFinishDate(endDate);
        
        //rtf assignments
        if (scope != null && scope.isRtfAssignments())
        {
            RtfAssignment[] rtfAssignments = RtfAssignmentFixture
                .createAllAssignmentFixtures(workElement, ctc);
            workElement.setRtfAssignments(rtfAssignments);
        }
        
        if (scope != null && scope.isTimeCodeAssignments())
        {
            TimeCodeAssignment timeCodeAssigns1 = TimeCodeAssignmentFixture
                .createFixture(workElement, true, ctc);
            TimeCodeAssignment timeCodeAssigns2 = TimeCodeAssignmentFixture
                .createFixture(workElement, false, ctc);
            workElement.setTimeCodeAssignments(new TimeCodeAssignment[]
            {
                    timeCodeAssigns1, timeCodeAssigns2
            });
        }
        
        if (scope != null && scope.isTimeCodes())
        {
            TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
            TimeCode timeCode1 = TimecodeFixture.createSaveTimeCodeFixture(
                timeCodeScope, true, ctc);
            TimeCode timeCode2 = TimecodeFixture.createSaveTimeCodeFixture(
                timeCodeScope, false, ctc);
            
            workElement.setTimeCode1(timeCode1);
            workElement.setTimeCode2(timeCode2);
        }

        if (scope != null && scope.getDependencies() != null)
        {
            ElementDependency successor_dependency = DependencyFixture
                .createFixture(workElement, scope.getDependencies(), ctc);
            ElementDependency predecessor_dependency = DependencyFixture
                .createFixture(workElement, scope.getDependencies(), ctc);
            workElement.setDependencies(new ElementDependency[]
            {
                    successor_dependency, predecessor_dependency
            });
        }

        if (scope != null)
        { //TODO update it after attribute fixture is set for wbs attr.
            if (scope.isAttributeAssignments())
            {
                AttributeAssignment attr = AttributeAssignmentFixture
                    .createAssignmentFixture(workElement, ctc);
                workElement.setAttributeAssignments(new AttributeAssignment[]
                {
                    attr
                });
            }
        }

        if (scope != null && scope.isState())
        {
            DatafieldState state = DatafieldFixture.createState(workElement);
            workElement.setState(state);
        }
        
        //ScheduleDates
        
        if (scope != null && scope.isScheduleDates())
        {
            throw new Exception(
                "WorkElement.createWorkElement: you must use the WbsScheduleDateFixture to add valid scheduleDate to a WorkElement");
        }
        
        if (workElement instanceof SimpleNode)
        {
            ((SimpleNode) workElement).setTaskType(TaskType.FixedDuration);
            ((SimpleNode) workElement).setEffortDriven(Boolean.FALSE);
            
            // required by SP_SEARCH_2 to assign resource to task
            ((SimpleNode) workElement).setConstraintDate(DateUtil
                .convertToCalendarNoTime("2009-10-14"));
            ((SimpleNode) workElement)
                .setConstraintType(ConstraintType.FinishNoEarlierThan);
            
        }
        
        if (workElement instanceof GenericProject)
        {
            isAggregateNode = true;
            
            if (scope != null && scope.isOpportunity())
            {
                opportunity = new Opportunity();
                HelperTestUtilities.setByIntrospection(opportunity, index,
                    localSkipSetters);
                ((GenericProject) workElement).setOpportunity(opportunity);
                opportunity.setParent(workElement);
            }

            currency = DatafieldFixture
                .createCurrency(DatafieldFixture.CURRENCY_USD);

            ((GenericProject) workElement).setCurrency(currency);
            RPMCalendar calendar = RPMCalendarFixture.retrieveStaticFixtureForInserts(ctc);
            ((GenericProject) workElement).setCalendar(calendar);
            ((GenericProject) workElement).setProbabilityPercent(new Integer(50));
            
            //AssignmentType set for resourceTaskAssignment and profile
            ((GenericProject) workElement)
                .setAssignmentType(AssignmentType.Work);

            if (scope != null && scope.isRelatedAttributeAssignments())
            {
                //Scope
                AttributeAssignment attr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setActionAttributeAssignments(new AttributeAssignment[]
                    {
                        attr
                    });
                AttributeAssignment defAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setDefectAttributeAssignments(new AttributeAssignment[]
                    {
                        defAttr
                    });
                AttributeAssignment chAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setChangeRequestAttributeAssignments(new AttributeAssignment[]
                    {
                        chAttr
                    });
                AttributeAssignment issueAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setIssueAttributeAssignments(new AttributeAssignment[]
                    {
                        issueAttr
                    });
                AttributeAssignment reqAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setRequirementAttributeAssignments(new AttributeAssignment[]
                    {
                        reqAttr
                    });
                AttributeAssignment riskAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setRiskAttributeAssignments(new AttributeAssignment[]
                    {
                        riskAttr
                    });
                AttributeAssignment servAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.SCOPE_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setServiceRequestAttributeAssignments(new AttributeAssignment[]
                    {
                        servAttr
                    });
                //WBS
                AttributeAssignment deliverableAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.WBS_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setDeliverableAttributeAssignments(new AttributeAssignment[]
                    {
                        deliverableAttr
                    });
                AttributeAssignment taskAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.WBS_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setTaskAttributeAssignments(new AttributeAssignment[]
                    {
                        taskAttr
                    });
                //Document
                AttributeAssignment docAttr = AttributeAssignmentFixture
                    .createAssignmentFixtureWithSpecificContext(
                        AttributeFixture.DOCUMENT_CONTEXT, workElement, ctc);
                ((GenericProject) workElement)
                    .setDocumentAttributeAssignments(new AttributeAssignment[]
                    {
                        docAttr
                    });

            }
            
            if (scope != null && scope.getScopeScorecards() != null)
            {
            	//TODO: Remove all these replace with Fixture
            	ScorecardFolderScope folderScope = new ScorecardFolderScope();
       		 
            	folderScope.setScorecards(new ScorecardScope());
        		
        		//TODO Use AssignedScorecards.
        		
//            	ScorecardFolder  testScoreCardFolder = ScorecardFixture.createSaveFixture(
//                        new ScopeManagementModule(), folderScope, ctc, new int[]{2,2,2,2});
//        		Scorecard[] scoreCards = testScoreCardFolder.getScorecards();
//        		//Action Score Cards
//        		ScoreCardAssignment[] actionScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< actionScorecardAssigns.length; ++i){
//        			actionScorecardAssigns[i] = new ScoreCardAssignment();			
//        			actionScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			actionScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setActionScoreCardAssignments(actionScorecardAssigns);
//
//        		//Change Request Scorecards
//        		ScoreCardAssignment[] changeRequestScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< changeRequestScorecardAssigns.length; ++i){
//        			changeRequestScorecardAssigns[i] = new ScoreCardAssignment();			
//        			changeRequestScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			changeRequestScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setChangeRequestScoreCardAssignments(changeRequestScorecardAssigns);
//
//        		//Defect Scorecards
//        		ScoreCardAssignment[] defectScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< defectScorecardAssigns.length; ++i){
//        			defectScorecardAssigns[i] = new ScoreCardAssignment();			
//        			defectScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			defectScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setDefectScoreCardAssignments(changeRequestScorecardAssigns);
//
//        		//Issue Scorecards
//        		ScoreCardAssignment[] issueScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< actionScorecardAssigns.length; ++i){
//        			issueScorecardAssigns[i] = new ScoreCardAssignment();			
//        			issueScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			issueScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setIssueScoreCardAssignments(issueScorecardAssigns);
//        		
//        		//Requirement Scorecards
//        		ScoreCardAssignment[] requirementScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< requirementScorecardAssigns.length; ++i){
//        			requirementScorecardAssigns[i] = new ScoreCardAssignment();			
//        			requirementScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			requirementScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setRequirementScoreCardAssignments(requirementScorecardAssigns);
//            	
//        		//Risk Scorecards
//        		ScoreCardAssignment[] riskScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< requirementScorecardAssigns.length; ++i){
//        			riskScorecardAssigns[i] = new ScoreCardAssignment();			
//        			riskScorecardAssigns[i].setScoreCard(scoreCards[i]);			
//        			riskScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setRiskScoreCardAssignments(riskScorecardAssigns);
//        		
//        		//Service Request scorecards
//        		ScoreCardAssignment[] serviceRequestScorecardAssigns = new ScoreCardAssignment[1];//testScoreCardFolder.getScorecards().length];
//        		for(int i=0; i< serviceRequestScorecardAssigns.length; ++i){
//        			serviceRequestScorecardAssigns[i] = new ScoreCardAssignment();			
//        			serviceRequestScorecardAssigns[i].setScoreCard(scoreCards[i]);
//        			serviceRequestScorecardAssigns[i].setID(workElement.getID());
//        		}
//        		((GenericProject)workElement).setServiceRequestScoreCardAssignments(riskScorecardAssigns);
            	
            }
            
            if (scope != null && scope.getDefaultSteps() != null)
            {
                GenericProject project = (GenericProject)workElement;
                DefaultStep[] defaultSteps = new DefaultStep[2];
                defaultSteps[0] = DefaultStepFixture.createFixture(
                    project, scope.getDefaultSteps(), ctc);
                defaultSteps[1] = DefaultStepFixture.createFixture(
                    project, scope.getDefaultSteps(), ctc);
                project.setDefaultSteps(defaultSteps);
            }
            
            if(scope != null && scope.isOrganizationalAssignments())
            {
                GenericProject project = (GenericProject)workElement;

                ProjectOrganizationalAssignment assignment = new ProjectOrganizationalAssignment();
                assignment.setParent(project);

                // Set mandatory fields
                assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
                    ctc));

                ProjectOrganizationalAssignment[] assignments =
                {
                    assignment
                };
                project.setOrganizationalAssignments(assignments);
            }
        }

        if (ctc != null && scope != null && isAggregateNode && scope.getChildren() != null)
        {
            setChildren((AggregateNode)workElement, scope, ctc);
        }

        if (ctc != null && scope != null && scope.getWbsFinancials() != null)
        {
            scope.getWbsFinancials().setParent(null);
            WbsFinancial wbsf = FinancialFixture.createWbsFixture(scope
                .getWbsFinancials(), ctc);
            wbsf.setParent(workElement);
            workElement.setWbsFinancials(new WbsFinancial[]
            {
                wbsf
            });
        }

        if (ctc != null && scope != null
                && (workElement instanceof AggregateNode)
                && scope.getWorksheetFinancials() != null)
        {
            scope.getWorksheetFinancials().setParent(null);
            WorksheetFinancial wf = FinancialFixture.createWorksheetFixture(
                scope.getWorksheetFinancials(), ctc);
            wf.setParent(workElement);
            ((AggregateNode) workElement)
                .setWorksheetFinancials(new WorksheetFinancial[]
                {
                    wf
                });
        }

        //Document folder
        if (ctc != null && scope != null && scope.getDocumentFolder() != null)
        {
            DocumentFolder documentFolder = (DocumentFolder) DocumentElementFixture
                .createFixture(workElement, new DocumentFolder(), 0, scope
                    .getDocumentFolder(), ctc);
            workElement.setDocumentFolder(documentFolder);
        }

        //asset
        if(scope != null && scope.isAsset())
        {
            Asset asset = AssetFixture.createSaveFixture(new AssetScope(),ctc);
            workElement.setAsset(asset);

            if(scope.isAssetCode())
            {
                AssetFinancialCategory category = DatafieldFixture.createAssetFinancialCategory(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_APP);
                workElement.setAssetCode(category);
            }
        }
        

        
        return workElement;
    }
    
    private static void setChildren(AggregateNode aggregateNode, WorkElementScope scope,
            ClientTestController ctc) throws Exception
    {
        List workElementList = null;
        if (aggregateNode.getChildren() != null)
        {
            workElementList = Arrays.asList(aggregateNode.getChildren());
        }
        else {
            workElementList = new ArrayList();
        }
        
        if (aggregateNode instanceof GenericProject)
        {
            workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
                new Proposal(), scope.getChildren(), ctc));
            workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
                new Project(), scope.getChildren(), ctc));
            workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
                new WorkOrganization(), scope.getChildren(), ctc));
        }

        workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
            new Deliverable(), scope.getChildren(), ctc));
        workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
            new WorkProduct(), scope.getChildren(), ctc));
        workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
            new SummaryTask(), scope.getChildren(), ctc));
        workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
            new Milestone(), scope.getChildren(), ctc));
        workElementList.add(WorkElementFixture.createFixture(aggregateNode, 
            new Task(), scope.getChildren(), ctc));
        
        WorkElement[] workElements = new WorkElement[workElementList.size()];
        Iterator iterator = workElementList.iterator();
        int i = 0;
        while (iterator.hasNext())
        {
            workElements[i] = (WorkElement) iterator.next();
            i++;
        }
        aggregateNode.setChildren(workElements);
    }

            
    //update fields
    public static void updateFixtureFields(WorkElement workelement,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        updateFields(workelement, random(), scope, ctc);
    }

    private static void updateFields(WorkElement workelement, int index,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        HelperTestUtilities.setByIntrospection(workelement, index,
            localSkipSetters);
        
        //update rtf assignments
        if (scope != null && scope.isRtfAssignments())
        {
            RtfAssignment[] rtfAssignments = workelement.getRtfAssignments();
            if (rtfAssignments != null)
            {
                for (int i = 0; i < rtfAssignments.length; i++)
                {
                    RtfAssignmentFixture
                        .updateAssignmentFixture(rtfAssignments[i]);
                }
            }
        }

        if (scope != null && scope.isState())
        {
            DatafieldFixture.updateState(workelement);
        }

        //TODO remove it later when working on second level tasks
        //       	if(workelement instanceof Task)
        //    		((Task)workelement).setPercentageDurationCompleted(null);

        if (workelement instanceof GenericProject)
        {
            Opportunity opportunity = new Opportunity();
            opportunity.setParent(workelement);
            HelperTestUtilities.setByIntrospection(opportunity, index,
                localSkipSetters);
            ((GenericProject) workelement).setOpportunity(opportunity);
            ((GenericProject) workelement).setDefaultProject(new Boolean(false));
            ((GenericProject) workelement)
                .setAssignmentType(AssignmentType.Work);
        }

        if (scope != null)
        {
            if (workelement instanceof AggregateNode
                    && ((AggregateNode) workelement).getChildren() != null)
            {
                for (int i = 0; i < ((AggregateNode) workelement)
                    .getChildren().length; i++)
                {
                    WorkElementFixture.updateFixtureFields(
                        ((AggregateNode) workelement).getChildren()[i],
                        scope.getChildren(), ctc);
                }
            }
            
            if (scope.isTimeCodeAssignments())
            {
                TimeCodeAssignment[] assignments = workelement
                    .getTimeCodeAssignments();
                assignments = TimeCodeAssignmentFixture
                        .updateFixture(assignments, ctc);
                workelement.setTimeCodeAssignments(assignments);
            }
        }
    }

    public static WorkElement setWorkElementForDependencies(Proposal proposal,
            WorkElementScope scope) throws Exception
    {
        return DependencyFixture.setElementDependency(proposal, scope);
    }
    
    public static WorkElementScope createScope(RPMObject rpmObject,
            boolean includeChildren, boolean includeRtfAndOpportunity,
            boolean includeDependencies, boolean includeFavoriteActivities,
            boolean includeTimeCodeAssignments, boolean includeAssignedScorecards,
            boolean includeScopeScorecards,
            boolean includeScheduleDates, boolean includeAttributeAssignments,
            boolean includeAsset, boolean includeAssetCode, 
            boolean includeWbsFinancials, boolean includeWorksheetFinancials,
            boolean includeTimeCodes, boolean includeEarnedValue)
    {
        WorkElementScope scope = new WorkElementScope();
        
        //NOTE: ScheduleDate will have to explicitely be set in the test.
        scope.setScheduleDates(false);
        
        WorkElementScope subWorkElementScope = null;

        if (includeChildren)
        {
            //create subfolders scope
            subWorkElementScope = WorkElementFixture.createScope(rpmObject,
                false, includeRtfAndOpportunity, includeDependencies,
                includeFavoriteActivities, includeTimeCodeAssignments,
                includeAssignedScorecards, includeScopeScorecards,
                includeScheduleDates, includeAttributeAssignments,
                includeAsset, includeAssetCode, includeWbsFinancials,
                includeWorksheetFinancials, includeTimeCodes,includeEarnedValue);
            subWorkElementScope.setParent(scope);
        }
        
        if (rpmObject instanceof AggregateNode)
        {
            scope.setChildren(subWorkElementScope);
            
            if (includeWorksheetFinancials)
            {
                FinancialScope worksheetFinancialScope = FinancialFixture
                        .createAllScope();
                scope.setWorksheetFinancials(worksheetFinancialScope);
            }
            
            if (rpmObject instanceof GenericProject)
            {   
              	scope.setOpportunity(includeRtfAndOpportunity);
                
                if (includeAssignedScorecards) 
                {
                    scope.setAssignedScorecards(new ScorecardScope());
                }
                if (includeScopeScorecards) 
                {
                    scope.setScopeScorecards(new ScorecardScope());
                }
                if (includeFavoriteActivities)
                {
                    scope.setDefaultSteps(DefaultStepFixture.createScope(
                        true, true));
                }
            }
        }
           
        scope.setAsset(includeAsset);
        scope.setAssetCode(includeAssetCode);
        scope.setAttributeAssignments(includeAttributeAssignments);
        scope.setRtfAssignments(includeRtfAndOpportunity);
        scope.setScheduleDates(includeScheduleDates);
        scope.setState(includeRtfAndOpportunity);
        scope.setTimeCodeAssignments(includeTimeCodeAssignments);
        scope.setEarnedValues(includeEarnedValue);
 
        if (includeDependencies)
        {
            ElementDependencyScope dependencyScope = new ElementDependencyScope();
            dependencyScope.setPredecessor(new WorkElementScope());
            dependencyScope.setCreatedBy(new ResourceScope());
        }
        
        if (includeWbsFinancials)
        {
            FinancialScope wbsFinancialScope = FinancialFixture.createAllScope();
            scope.setWbsFinancials(wbsFinancialScope);
        }
        
        if (includeTimeCodes)
        {
            scope.setTimeCodes(true);
        }
            
        return scope;
    }

    private static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }

    public static void updateFixtureStructure(AggregateNode aggregateNode,
            int deltaNbAggregateNodesByLevel, int deltaNbElementsByLevel,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        updateStructure(aggregateNode, deltaNbAggregateNodesByLevel,
            deltaNbElementsByLevel, scope, ctc);
    }

    private static void updateStructure(AggregateNode aggregateNode,
            int deltaNbAggregateNodesByLevel, int deltaNbElementsByLevel,
            WorkElementScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if (scope.getChildren() != null)
            {
                if (deltaNbElementsByLevel > 0)
                {//add new elements
                    WorkElement newWorkElement = null;
                    WorkElement[] newWorkElements = new WorkElement[deltaNbElementsByLevel * 5];
                    for (int i = 0; i < deltaNbElementsByLevel; i += 5)
                    {
                        newWorkElement = WorkElementFixture
                            .createFixture(new Milestone(), scope
                                .getChildren(), ctc);
                        newWorkElement.setParent(aggregateNode);
                        newWorkElements[i] = newWorkElement;
                        
                        newWorkElement = WorkElementFixture.createFixture(
                            new Task(), scope.getChildren(), ctc);
                        newWorkElement.setParent(aggregateNode);
                        newWorkElements[++i] = newWorkElement;
                        
                        newWorkElement = WorkElementFixture.createFixture(
                            new SummaryTask(), scope.getChildren(),
                            ctc);
                        newWorkElement.setParent(aggregateNode);
                        newWorkElements[++i] = newWorkElement;
                        
                        newWorkElement = WorkElementFixture.createFixture(
                            new Deliverable(), scope.getChildren(),
                            ctc);
                        newWorkElement.setParent(aggregateNode);
                        newWorkElements[++i] = newWorkElement;
                        
                        newWorkElement = WorkElementFixture.createFixture(
                            new WorkProduct(), scope.getChildren(),
                            ctc);
                        newWorkElement.setParent(aggregateNode);
                        newWorkElements[++i] = newWorkElement;
                    }
                    ArrayList temp = mergeArrays(aggregateNode
                        .getChildren(), newWorkElements);
                    WorkElement[] totalWorkElements = new WorkElement[temp
                        .size()];
                    temp.toArray(totalWorkElements);
                    aggregateNode.setChildren(totalWorkElements);
                }

                if (deltaNbAggregateNodesByLevel > 0
                        && aggregateNode instanceof GenericProject)
                {//add new elements
                    WorkElement newGenericProject = null;
                    WorkElement[] newGenericProjects = new GenericProject[deltaNbAggregateNodesByLevel * 3];
                    for (int i = 0; i < deltaNbAggregateNodesByLevel; i += 3)
                    {
                        newGenericProject = WorkElementFixture.createFixture(
                            new Proposal(), scope.getChildren(), ctc);
                        newGenericProject.setParent(aggregateNode);
                        newGenericProjects[i] = newGenericProject;
                        
                        newGenericProject = WorkElementFixture.createFixture(
                            new Project(), scope.getChildren(), ctc);
                        newGenericProject.setParent(aggregateNode);
                        newGenericProjects[++i] = newGenericProject;

                        newGenericProject = WorkElementFixture.createFixture(
                            new WorkOrganization(), scope.getChildren(), ctc);
                        newGenericProject.setParent(aggregateNode);
                        newGenericProjects[++i] = newGenericProject;
                    }
                    ArrayList temp = mergeArrays(aggregateNode
                        .getChildren(), newGenericProjects);
                    WorkElement[] totalAggregateNodes = new WorkElement[temp
                        .size()];
                    temp.toArray(totalAggregateNodes);
                    aggregateNode.setChildren(totalAggregateNodes);
                }
            }
        }
    }

    /**
     * @param w1
     * @return
     */
    private static AttributeAssignment[] getConsolidatedAttributes(
            WorkElement workElement)
    {
        if (workElement instanceof GenericProject)
        {
            return ((GenericProject) workElement).getAttributeAssignments();
        }
        else
        {
            AttributeAssignment[] attrArr = null;
            WorkElement temp = workElement;
            WorkElement nextTemp = null;
            while (true)
            {
                nextTemp = (WorkElement) temp.getParent();
                if (nextTemp != null && !(nextTemp instanceof GenericProject))
                {
                    temp = nextTemp;
                }
                else
                {
                    break;
                }
            }
            if (temp != null)
            {
                if (workElement instanceof Deliverable
                        || workElement instanceof SummaryTask
                        || workElement instanceof WorkProduct)
                {
                    AttributeAssignment[] arr1 = ((GenericProject) temp)
                        .getDeliverableAttributeAssignments();
                    for (int i = 0; i < arr1.length; i++)
                    {
                        arr1[i].setID(workElement.getID());
                        arr1[i].setParent(workElement);
                    }
                    AttributeAssignment[] arr2 = workElement
                        .getAttributeAssignments();
                    ArrayList retVal = mergeArrays(arr1, arr2);
                    if (retVal != null && !retVal.isEmpty())
                    {
                        attrArr = new AttributeAssignment[retVal.size()];
                        retVal.toArray(attrArr);
                        return attrArr;
                    }
                }
                else if (workElement instanceof Task
                        || workElement instanceof Milestone)
                {
                    return workElement.getAttributeAssignments();
                }
            }
        }
        return null;
    }

    public static void compare(WorkElement w1, WorkElement w2,
            WorkElementScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(w1, w2, localSkipGetters);
        if (scope == null)
            return;
        //compare the rtfAssignments
        if (scope.isRtfAssignments())
        {
            RtfAssignment[] rtfAssignments1 = w1.getRtfAssignments();
            RtfAssignment[] rtfAssignments2 = w2.getRtfAssignments();
            RtfAssignmentFixture.compare(rtfAssignments1, rtfAssignments2);
        }
        //compare the dependencies
        if (scope.getDependencies() != null)
        {
            ElementDependency[] dependencies1 = w1.getDependencies();
            ElementDependency[] dependencies2 = w2.getDependencies();
            DependencyFixture.compare(dependencies1, dependencies2);
        }
        
        //Schedule dates
        if (w1 instanceof SimpleNode && scope.isScheduleDates())
        {
            WbsScheduleDateFixture.compareAllWbsScheduleDates((SimpleNode) w1,
                (SimpleNode) w2);            
        }
        
        //EarnedValue
        if (scope.isEarnedValues())
        {
            HelperTestUtilities.assertEqualsByIntrospection(w1
                .getEarnedValues(), w2.getEarnedValues());
        }

        
        if (scope.isAttributeAssignments())
        {
            AttributeAssignment[] attrs = getConsolidatedAttributes(w1);
            AttributeAssignmentFixture.compare(attrs, w2
                .getAttributeAssignments());
            for (int i = 0; i < attrs.length; i++)
            {
                attrs[i].setParent(w1);
                attrs[i].setID(w1.getID());
            }
        }
        
        if (scope.isRelatedAttributeAssignments()
                && w1 instanceof GenericProject)
        {
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getActionAttributeAssignments(), ((GenericProject) w2)
                .getActionAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getChangeRequestAttributeAssignments(), ((GenericProject) w2)
                .getChangeRequestAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getDefectAttributeAssignments(), ((GenericProject) w2)
                .getDefectAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getDeliverableAttributeAssignments(), ((GenericProject) w2)
                .getDeliverableAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getIssueAttributeAssignments(), ((GenericProject) w2)
                .getIssueAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getDocumentAttributeAssignments(), ((GenericProject) w2)
                .getDocumentAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getRequirementAttributeAssignments(), ((GenericProject) w2)
                .getRequirementAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getRiskAttributeAssignments(), ((GenericProject) w2)
                .getRiskAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getServiceRequestAttributeAssignments(), ((GenericProject) w2)
                .getServiceRequestAttributeAssignments());
            AttributeAssignmentFixture.compare(((GenericProject) w1)
                .getTaskAttributeAssignments(), ((GenericProject) w2)
                .getTaskAttributeAssignments());
        }

        //compare documentFolder
        if (scope.getDocumentFolder() != null)
        {
            DocumentFolder documentFolder1 = w1.getDocumentFolder();
            DocumentFolder documentFolder2 = w2.getDocumentFolder();
            DocumentElementFixture.compare(documentFolder1, documentFolder2,
                scope.getDocumentFolder());
        }

        //validate sub workelement
        if (scope.getChildren() != null)
        {
            AggregateNode[] aggregateNodes1 = null, aggregateNodes2 = null;
            scope.getChildren().setParent(null);
            WorkElement[] leaves1 = new WorkElement[2], leaves2 = new WorkElement[2]; //leaves rep milestone and task
            aggregateNodes1 = getAggregateNodes(((AggregateNode) w1)
                .getChildren(), leaves1);
            aggregateNodes2 = getAggregateNodes(((AggregateNode) w2)
                .getChildren(), leaves2);
            WorkElementFixture.compare(aggregateNodes1, aggregateNodes2, scope
                .getChildren());
            WorkElementFixture.compare(leaves1, leaves2, null);
        }
//TODO remove later : not needed anymore as all AggregateNodes will be treated the same        
//        if (scope.getProject() != null)
//        {
//            GenericProject[] gp1 = null, gp2 = null;
//            scope.getProject().setParent(null);
//            if(((GenericProject) w1).getWorkElements() != null && ((GenericProject) w2).getWorkElements()!=null){
//	            gp1 = getGenericProjects(((GenericProject) w1).getWorkElements());
//	            gp2 = getGenericProjects(((GenericProject) w2).getWorkElements());
//	            WorkElementFixture.compare(gp1, gp2, scope.getProject());
//            }
//        }

        if (scope.getAssignedScorecards() != null
                && w1 instanceof GenericProject && w2 instanceof GenericProject)
        {
            PublishedScorecardFixture.compare(((GenericProject) w1)
                .getAssignedScorecards(), ((GenericProject) w2)
                .getAssignedScorecards(), scope.getAssignedScorecards());
        }
        
        //TODO Use AssignedScorecards

//        if(scope.isScoreCardAssignments() && w1 instanceof GenericProject && w2 instanceof GenericProject){
//        	//Compare Action Score Card Assignments
//        	ScoreCardAssignment[] scActionAssigns1 = ((GenericProject)w1).getActionScoreCardAssignments();
//        	ScoreCardAssignment[] scActionAssigns2 = ((GenericProject)w2).getActionScoreCardAssignments();
//        	if(scActionAssigns1 != null && scActionAssigns2 != null)
//        		for(int i=0; i < scActionAssigns1.length; ++i)
//        			ScorecardFixture.compare(scActionAssigns1[i].getScoreCard(), scActionAssigns2[i].getScoreCard());
//        	
//        	//Compare ChangeRequest Score Card Assignments
//        	ScoreCardAssignment[] scChangeRequestAssigns1 = ((GenericProject)w1).getChangeRequestScoreCardAssignments();
//        	ScoreCardAssignment[] scChangeRequestAssigns2 = ((GenericProject)w2).getChangeRequestScoreCardAssignments();
//        	if(scChangeRequestAssigns1 != null && scChangeRequestAssigns2 != null)
//        		for(int i=0; i < scChangeRequestAssigns1.length; ++i)
//        			ScorecardFixture.compare(scChangeRequestAssigns1[i].getScoreCard(), scChangeRequestAssigns2[i].getScoreCard());
//        	
//        	//Compare Defect Score Card Assignments
//        	ScoreCardAssignment[] scDefectAssigns1 = ((GenericProject)w1).getDefectScoreCardAssignments();
//        	ScoreCardAssignment[] scDefectAssigns2 = ((GenericProject)w2).getDefectScoreCardAssignments();
//        	if(scDefectAssigns1 != null && scDefectAssigns2 != null)
//        		for(int i=0; i < scDefectAssigns1.length; ++i)
//        			ScorecardFixture.compare(scDefectAssigns1[i].getScoreCard(), scDefectAssigns2[i].getScoreCard());
//        	
//        	//Compare Issue Score Card Assignments
//        	ScoreCardAssignment[] scIssueAssigns1 = ((GenericProject)w1).getIssueScoreCardAssignments();
//        	ScoreCardAssignment[] scIssueAssigns2 = ((GenericProject)w2).getIssueScoreCardAssignments();
//        	if(scIssueAssigns1 != null && scIssueAssigns2 != null)
//        		for(int i=0; i < scIssueAssigns1.length; ++i)
//        			ScorecardFixture.compare(scIssueAssigns1[i].getScoreCard(), scIssueAssigns2[i].getScoreCard());
//        	
//        	//Compare Requirement Score Card Assignments
//        	ScoreCardAssignment[] scRequirementAssigns1 = ((GenericProject)w1).getRequirementScoreCardAssignments();
//        	ScoreCardAssignment[] scRequirementAssigns2 = ((GenericProject)w2).getRequirementScoreCardAssignments();
//        	if(scRequirementAssigns1 != null && scRequirementAssigns2 != null)
//        		for(int i=0; i < scRequirementAssigns1.length; ++i)
//        			ScorecardFixture.compare(scRequirementAssigns1[i].getScoreCard(), scRequirementAssigns2[i].getScoreCard());
//        	
//        	//Compare Risk Score Card Assignments
//        	ScoreCardAssignment[] scRiskAssigns1 = ((GenericProject)w1).getRiskScoreCardAssignments();
//        	ScoreCardAssignment[] scRiskAssigns2 = ((GenericProject)w2).getRiskScoreCardAssignments();
//        	if(scRiskAssigns1 != null && scRiskAssigns2 != null)
//        		for(int i=0; i < scRiskAssigns1.length; ++i)
//        			ScorecardFixture.compare(scRiskAssigns1[i].getScoreCard(), scRiskAssigns2[i].getScoreCard());
//        	
//        	//Compare Service Request Score Card Assignments
//        	ScoreCardAssignment[] scServiceRequestAssigns1 = ((GenericProject)w1).getServiceRequestScoreCardAssignments();
//        	ScoreCardAssignment[] scServiceRequestAssigns2 = ((GenericProject)w2).getServiceRequestScoreCardAssignments();
//        	if(scServiceRequestAssigns1 != null && scServiceRequestAssigns2 != null)
//        		for(int i=0; i < scServiceRequestAssigns1.length; ++i)
//        			ScorecardFixture.compare(scServiceRequestAssigns1[i].getScoreCard(), scServiceRequestAssigns2[i].getScoreCard());
//        }
        //TODO: Chk if we have to validate parents .. upon uncommenting the below code, it goes into an stack overflow.
        //        if (scope.getParent() != null)
        //        {
        //        	WorkElementScope s = scope.getParent();
        //            if (s != null){
        //                s.setParent(null);
        //            }
        //            WorkElementFixture.compare((AggregateNode) w1.getParent(),
        //                    (AggregateNode) w2.getParent(), s);
        //        }

        //validate if parent portfolio
        if (scope.getPortfolio() != null && w1 instanceof GenericProject
                && w2 instanceof GenericProject)
        {
            PortfolioScope s = scope.getPortfolio();
            if (s != null)
            {
                s.setParent(null);
            }
            PortfolioFixture.compare(((GenericProject) w1).getPortfolio(),
                ((GenericProject) w2).getPortfolio(), s);
        }

        if (scope.getWbsFinancials() != null
                && (w1.getWbsFinancials() != null || w2.getWbsFinancials() != null))
        {

            if (w1.getWbsFinancials() == null)
                Assert
                    .fail("w1 objects getWbsFinancials is null, but w2 getWbsFinancials is not-null");

            if (w2.getWbsFinancials() == null)
                Assert
                    .fail("w2 objects getWbsFinancials is null, but w1 getWbsFinancials is not-null");

            Assert.assertEquals("Array length is not the same", w1
                .getWbsFinancials().length, w2.getWbsFinancials().length);

            sortWithComparator(w1.getWbsFinancials(),
                RPMObjectNameAndIDComparator.INSTANCE);
            sortWithComparator(w2.getWbsFinancials(),
                RPMObjectNameAndIDComparator.INSTANCE);

            scope.getWbsFinancials().setParent(null);

            for (int i = 0; i < w1.getWbsFinancials().length; i++)
            {
                FinancialFixture.compare(w1.getWbsFinancials()[i], w2
                    .getWbsFinancials()[i], scope.getWbsFinancials());
            }
        }

        if(scope!= null && scope.isAsset())
            AssetFixture.compare(w1.getAsset(),w2.getAsset());

        // TODO - Simon Legault: Uncomment this compare when AssetCode loading is implemented.
//        if(scope.isAssetCode()&&w1.getAssetCode()!=null)
//            DatafieldFixture.compare(w1.getAssetCode(),w2.getAssetCode());
        
        if (scope != null & scope.isTimeCodeAssignments())
        {
            TimeCodeAssignmentFixture.compare(w1.getTimeCodeAssignments(), w2
                .getTimeCodeAssignments());
        }
        
        if (scope != null & scope.isTimeCodes())
        {
            TimecodeFixture.compare(w1.getTimeCode1(), w2.getTimeCode1());
            TimecodeFixture.compare(w1.getTimeCode2(), w2.getTimeCode2());
        }

        if (w1 instanceof AggregateNode)
        {
            AggregateNode a1 = (AggregateNode) w1;
            AggregateNode a2 = (AggregateNode) w2;

            if (scope.getWorksheetFinancials() != null
                    && (a1.getWorksheetFinancials() != null || a2
                        .getWorksheetFinancials() != null))
            {
                if (a1.getWorksheetFinancials() == null)
                    Assert
                        .fail("a1 objects getWorksheetFinancials is null, but a2 getWorksheetFinancials is not-null");

                if (a2.getWorksheetFinancials() == null)
                    Assert
                        .fail("a2 objects getWorksheetFinancials is null, but a1 getWorksheetFinancials is not-null");

                Assert.assertEquals("Array length is not the same", a1
                    .getWorksheetFinancials().length, a2
                    .getWorksheetFinancials().length);

                sortWithComparator(a1.getWorksheetFinancials(),
                    RPMObjectNameAndIDComparator.INSTANCE);
                sortWithComparator(a2.getWorksheetFinancials(),
                    RPMObjectNameAndIDComparator.INSTANCE);

                scope.getWorksheetFinancials().setParent(null);

                for (int i = 0; i < a1.getWorksheetFinancials().length; i++)
                {
                    FinancialFixture.compare(a1.getWorksheetFinancials()[i], a2
                        .getWbsFinancials()[i], scope.getWorksheetFinancials());
                }
            }
        }

    }

    /**
     * @param workElements
     * @return
     */
    private static AggregateNode[] getAggregateNodes(
            WorkElement[] workElements, WorkElement[] leaves)
    {
        if (workElements == null)
        {
            return null;
        }
        AggregateNode[] retVal = null;
        ArrayList temp = new ArrayList();
        ArrayList temp2 = new ArrayList();

        for (int i = 0; i < workElements.length; ++i)
        {
            if (workElements[i] instanceof AggregateNode
                    && !(workElements[i] instanceof GenericProject))
            {
                temp.add(workElements[i]);
            }
            if (workElements[i] instanceof Task
                    || workElements[i] instanceof Milestone)
            {
                temp2.add(workElements[i]);
            }
        }
        retVal = new AggregateNode[temp.size()];
        temp.toArray(retVal);
        temp2.toArray(leaves);//size MUST BE 2 !!

        return retVal;
    }

    public static void compare(WorkElement[] fArray1, WorkElement[] fArray2,
            WorkElementScope scope) throws AssertionFailedError, Exception
    {
        sortWithComparator(fArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(fArray2, RPMObjectNameAndIDComparator.INSTANCE);
        RPMObjectFixture.compare(fArray1, fArray2);
        if (fArray1 == null)
            return;
        for (int i = 0; i < fArray1.length; i++)
        {
            WorkElementFixture.compare(fArray1[i], fArray2[i], scope);
        }
    } 
}
