/*
 * Created on Dec 12, 2006
 * @author mmatwiy
 * 
 * Generic methods to be used for testing WorkElements by subclasses.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class AbstractTestWorkElement extends ClientTestController
{
    protected WorkElement updateWorkElementFields(WorkElement workelement,
            WorkElementScope scope, boolean loadFromId,
            int deltaWorkElementByLevel, int deltaElementsByLevel)
            throws Exception
    {
        return updateWorkElement(workelement, scope, loadFromId, false,
            deltaWorkElementByLevel, deltaElementsByLevel);
    }

    protected WorkElement updateAggregateNodeStructure(AggregateNode aggregateNode,
            WorkElementScope scope, boolean loadFromId,
            int deltaWorkElementByLevel, int deltaElementsByLevel)
            throws Exception
    {
        return updateWorkElement(aggregateNode, scope, loadFromId, true,
            deltaWorkElementByLevel, deltaElementsByLevel);
    }

    private WorkElement updateWorkElement(WorkElement workelement,
            WorkElementScope scope, boolean loadFromId,
            boolean updateStructure, int deltaWorkElementByLevel,
            int deltaElementsByLevel) throws Exception
    {
        if (updateStructure && workelement instanceof AggregateNode)
        {
            // update the structure by adding elements
            WorkElementFixture.updateFixtureStructure(
                (AggregateNode) workelement, deltaWorkElementByLevel,
                deltaElementsByLevel, scope, this);
        }
        else
        {
            // update the fields only
            WorkElementFixture.updateFixtureFields(workelement, scope, this);
        }
        // insert the updated workelement into the DB
        SingleResult result = save(workelement, scope);

        // compare the returned workelement
        WorkElement returnedWorkElement = (WorkElement) result.getRpmObject();
        WorkElementFixture.compare(workelement, returnedWorkElement, scope);

        // reload the workelement and compare it
        returnedWorkElement = load(returnedWorkElement, scope, loadFromId);
        WorkElementFixture.compare(workelement, returnedWorkElement, scope);

        return returnedWorkElement;
    }

    protected WorkElement load(WorkElement workElement, WorkElementScope scope,
            boolean loadFromId) throws Exception
    {
        WorkElement returnWorkElement = null;

        if (loadFromId)
        {
            //load from id
            RPMObject obj = new WorkElement();
            obj.setID(workElement.getID());
            returnWorkElement = (WorkElement) loadFromID(obj, scope);
        }
        else
        {
            //load from xpath
            RPMObject[] objs = loadFromXpath("/WorkElement[id='"
                    + workElement.getID() + "']", scope);
            if (objs != null && objs.length > 0)
            {
                returnWorkElement = (WorkElement) objs[0];
            }
        }
        return returnWorkElement;
    }
}
