package com.ibm.rpm.client.junit.wbs.testcases;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.EarnedValues;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ResourceType;

public class TestEarnedValues extends ClientTestController
{
    private WorkElementScope mainScope = null;

    private Proposal proposal = null;

    private Project evProject = null;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new WorkElementScope();
        mainScope.setEarnedValues(true);

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            mainScope, this);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testEarnedValuesCalulationActivationState()
            throws AssertionError, Exception
    {
        // Basic project scoping
        mainScope.setParent(new WorkElementScope());
        mainScope.setContainingProject(new WorkElementScope());

        Calendar startDate = Calendar.getInstance();
        startDate.set(2007, 0, 1);

        Calendar finishDate = Calendar.getInstance();
        finishDate.set(2007, 0, 5);

        //Creating Project
        Project evProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), mainScope, this);

        evProject.setEstimatedStartDate(startDate);
        evProject.setEstimatedFinishDate(finishDate);

        //saving Project
        SingleResult result = null;
        result = getApplicationInterface().save(sessionid, evProject,
            mainScope, ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        evProject = (Project) result.getRpmObject();
        assertNotNull(evProject);
        assertNotNull(evProject.getEarnedValuesCalculationState());

        assertFalse(evProject.getEarnedValuesCalculationState()
            .booleanValue());

        // 2. set Project Setting to activate EV Calculation
        result = getApplicationInterface().enableProjectEVCalculation(
            sessionid, evProject, mainScope);
        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        evProject = (Project) result.getRpmObject();
        assertNotNull(evProject);
        assertNotNull(evProject.getEarnedValuesCalculationState());


        assertTrue(evProject.getEarnedValuesCalculationState()
            .booleanValue());
    }

    public void testLoadProposalWithEarnedValuesAndSave()
            throws AssertionError, Exception
    {
        getApplicationInterface().save(sessionid, proposal, mainScope,
            ReloadType.ReloadResult);

        SingleResult testResult = getApplicationInterface().loadFromID(
            sessionid, proposal, mainScope);
        assertNotNull(testResult);
        assertTrue(testResult.isSuccessful());

        proposal = (Proposal) testResult.getRpmObject();

        SingleResult result = save(proposal, mainScope);

        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
    }

    public void testSaveProposalEarnedValuesWithNullEVObject()
            throws AssertionError, Exception
    {
        proposal.setEarnedValues(null);

        SingleResult result = save(proposal, mainScope);

        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
    }

    public void testSaveProposalWithNullScopeAndEVObject()
            throws AssertionError, Exception
    {
        proposal.setEarnedValues(new EarnedValues());
        proposal.getEarnedValues().setActualCostWorkPerf(new Double(4));

        SingleResult result = save(proposal, null);

        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
    }

    public void testSaveProposalWithNullScopeAndNullEVObject()
            throws AssertionError, Exception
    {
        proposal.setEarnedValues(null);

        SingleResult result = save(proposal, null);

        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
    }

    private Project doEarnedValuesCreationAndLoadingForWeek(int week)
            throws AssertionError, Exception
    {
        //Basic project scoping
        mainScope.setParent(new WorkElementScope());
        mainScope.setContainingProject(new WorkElementScope());

        //Task scoping with rta and  ta  relations        
        WorkElementScope taskScope = new WorkElementScope();
        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());
        taskScope.setResourceTaskAssignments(rtaScope);
        taskScope.setContainingProject(new WorkElementScope());

        //Complete scope of the tree: Project,Task,RTA,TA,POOL,RESOURCE
        WorkElementScope allScope = new WorkElementScope();
        allScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        allScope.getResourceTaskAssignments().setResource(new ResourceScope());
        allScope.getResourceTaskAssignments().setTaskAssignment(
            new TaskAssignmentScope());
        allScope.getResourceTaskAssignments().getResource().setCalendar(
            new CalendarScope());
        allScope.setContainingProject(new WorkElementScope());

        allScope.setChildren(taskScope);
        allScope.setEarnedValues(true);

        //Resource scoping
        CalendarScope calScope = new CalendarScope();
        ResourceScope evResourceScope = new ResourceScope();
        evResourceScope.setParent(new PoolScope());
        evResourceScope.setRates(true);
        evResourceScope.setCalendar(calScope);

        Calendar startDate = Calendar.getInstance();
        startDate.set(2007, 0, 1);

        Calendar finishDate = Calendar.getInstance();
        finishDate.set(2007, 0, 5);

        //1. create Project
        evProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), mainScope, this);

        evProject.setEstimatedStartDate(startDate);
        evProject.setEstimatedFinishDate(finishDate);

        SingleResult result = null;
        result = getApplicationInterface().save(sessionid, evProject,
            mainScope, ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        evProject = (Project) result.getRpmObject();

        //2. set Project Setting to activate EV Calculation
        getApplicationInterface().enableProjectEVCalculation(sessionid,
            evProject, mainScope);

        //3. set appAdmin.EV_Delays(week)=0
        getApplicationInterface().activateEVWeekDelays(sessionid, week);

        //4. Add Task to Project as child
        Task evTask = new Task();
        evTask.setName(HelperTestUtilities.createName());
        evTask.setParent(evProject);

        result = null;
        result = getApplicationInterface().save(sessionid, evTask, taskScope,
            ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Task);
        evTask = (Task) result.getRpmObject();

        // 5. create and save pool
        Pool evPoolParent = new Pool();
        evPoolParent.setName(HelperTestUtilities.createName());

        result = null;
        result = getApplicationInterface().save(sessionid, evPoolParent,
            new PoolScope(), ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.isSuccessful());

        //6. create resource
        Resource evResource = new Resource();
        evResource.setFullName(HelperTestUtilities.createName());
        evResource.setActive(new Boolean(true));
        evResource.setParent(evPoolParent);

        evResource.setEffectiveStartDate(startDate);

        ResourceSecurityRights resourceSecurityRights = new ResourceSecurityRights();
        evResource.setResourceSecurityRights(resourceSecurityRights);

        // 6.1 create, bind and save calendar, rates to resource (horaire)
        RPMCalendar rpmCal = RPMCalendarFixture.createSaveDefaultCalendar(
            calScope, this);
        evResource.setCalendar(rpmCal);

        //5.4 save,bind resource to calendar, rates
        ResourceRates rates = new ResourceRates();
        rates.setRegularCostPerHour(new Double(25));
        rates.setSpecialCostPerHour(new Double(50));
        rates.setSellingRatePerHour(new Double(35));
        rates.setSpecialSellingRatePerHour(new Double(65));

        evResource.setRates(rates);
        evResource.getRates().setParent(evResource);

        result = null;
        result = getApplicationInterface().save(sessionid, evResource,
            evResourceScope, ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.isSuccessful());
        evResource = (Resource) result.getRpmObject();

        //6.2  create RTA and TA, loading SecurityRole and binding all together             
        ResourceTaskAssignment rta = new ResourceTaskAssignment();
        RPMObject[] objs = null;
        objs = loadFromXpath("/SecurityRole[name='Task Responsible']",
            new SecurityRoleScope());
        assertTrue(objs != null);
        assertTrue(objs[0] instanceof SecurityRole);

        rta.setResource(evResource);
        rta.setSecurityRole((SecurityRole) objs[0]);
        rta.setResourceType(ResourceType.Resource);
        rta.setTaskAssignment(new TaskAssignment());
        rta.getTaskAssignment().setEETCHours(new Double(40.00));

        rta.getTaskAssignment().setAssignmentStartDate(startDate);

        evTask.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            rta
        });

        //6.3 saving rta via task
        result = null;
        result = getApplicationInterface().save(sessionid, evTask, allScope,
            ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Task);
        assertTrue(result.isSuccessful());
        evTask = (Task) result.getRpmObject();

        //7. Copy proposed to plan (project)
        result = null;
        result = getApplicationInterface().copyProposedToPlan(sessionid,
            evProject, allScope);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Project);
        assertTrue(result.isSuccessful());
        evProject = (Project) result.getRpmObject();

        //8. Baseline save baseline (project)
        result = null;
        result = getApplicationInterface().saveProjectBaseline(sessionid,
            evProject, allScope);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Project);
        assertTrue(result.isSuccessful());
        evProject = (Project) result.getRpmObject();

        //9. rollUp (project)
        result = null;
        result = getApplicationInterface().rollupProjectEV(sessionid,
            evProject, allScope);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Project);
        assertTrue(result.isSuccessful());

        //9.1 reloading containers
        evProject = (Project) result.getRpmObject();
        evTask = (Task) evProject.getChildren()[0];
        rta = evTask.getResourceTaskAssignments()[0];

        //10. update Task Actuals        
        rta.getTaskAssignment().setActualWork(new Double(50));

        result = null;
        result = getApplicationInterface().save(sessionid, evTask, allScope,
            ReloadType.ReloadResult);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Task);
        assertTrue(result.isSuccessful());
        evTask = (Task) result.getRpmObject();

        //11. rollup (project)
        result = null;
        result = getApplicationInterface().rollupProjectEV(sessionid,
            evProject, allScope);
        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Project);
        assertTrue(result.isSuccessful());
        evProject = (Project) result.getRpmObject();

        return evProject;

    }

    public void testEarnedValuesCreationAndLoading_Week0ToWeek3()
            throws AssertionError, Exception
    {
        WorkElement estimatedWE = new WorkElement();
        WorkElement resultWE = new WorkElement();
        EarnedValues evWeek = new EarnedValues();
        EarnedValues evResult = null;

        // Expected Values
        evWeek.setBudgetAtCompletion(new Double(1000));
        evWeek.setSchedAtCompletion(new Double(5.0));
        evWeek.setBudgetedCostWorkSched(new Double(1000));
        evWeek.setBudgetedCostWorkPerf(new Double(1000));
        evWeek.setActualCostWorkPerf(new Double(1250));
        evWeek.setScheduleVariance_C(new Double(0));
        evWeek.setCostVariance_C(new Double(-250));
        evWeek.setTimeVariance(new Integer(0));
        evWeek.setPercentComplete_C(new Double(100));
        evWeek.setSchedPerformIndex_C(new Double(1));
        evWeek.setCostPerformIndex_C(new Double(0.8));
        evWeek.setCriticalRatio_C(new Double(0.8));
        evWeek.setToCompletePerformIndex_C(new Double(0));
        evWeek.setEstimateToComplete(new Double(0));
        evWeek.setEstimateAtComplete(new Double(1250));
        evWeek.setVarianceAtCompletion_C(new Double(-250));
        evWeek.setSchedPerforIndexAtComp_C(new Double(0));
        evWeek.setCriticalRatioAtComp_C(new Double(-200));
        evWeek.setTimeEstimateAtComp_C(new Double(5));
        evWeek.setTimeVarianceAtComp_C(new Double(0));

        Calendar baseline3FinishCal = Calendar.getInstance();
        baseline3FinishCal.set(2007, 0, 5, 5, 0);
        evWeek.setEstimatedCompletionDate(baseline3FinishCal);

        evWeek.setBudgetedCostWorkSchedForecast(new Double(1000));
        evWeek.setBudgetedCostWorkPerfForecast(new Double(1000));
        evWeek.setActualCostWorkPerfForecast(new Double(1250));
        evWeek.setEstimateToCompleteForecast(new Double(0));
        evWeek.setEstimateAtCompleteForecast(new Double(1250));

        for (int i = 0; i <= 2; i++)
        {
            evResult = doEarnedValuesCreationAndLoadingForWeek(i)
                .getEarnedValues();
            assertNotNull(evResult);

            estimatedWE.setEarnedValues(evWeek);
            resultWE.setEarnedValues(evResult);

            WorkElementFixture.compare(estimatedWE, resultWE, mainScope);
        }
    }

}
