/*
 * Created on Dec 4, 2006
 * @author mmatwiy
 * Test methods for container ElementDependency.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.DependencyFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;

public class TestElementDependency extends ClientTestController
{
    private ElementDependencyScope dependencyScope = null;
    
    public void setUp() throws Exception
    {
        super.setUp();
        dependencyScope = DependencyFixture.createScope();
    }
    
    public void testInsertWithScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insertElementDependency(dependencyScope, false);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insertElementDependency(dependencyScope, true);
    }

    public void testUpdateWithScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ElementDependency dependency = insertElementDependency(dependencyScope,
            false);
        updateElementDependency(dependency, dependencyScope, false);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ElementDependency dependency = insertElementDependency(dependencyScope, true);
        updateElementDependency(dependency, dependencyScope, true);
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insertElementDependency(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insertElementDependency(null, true);
    }

    public void testUpdateWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ElementDependency dependency = insertElementDependency(null, false);
        updateElementDependency(dependency, null, false);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        ElementDependency dependency = insertElementDependency(null, true);
        updateElementDependency(dependency, null, true);
    }

    private ElementDependency insertElementDependency(
            ElementDependencyScope scope, boolean loadFromId) throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        proposal.setMinutesPerDay(new Integer(480));
        save(proposal, null);

        ElementDependency dependency = DependencyFixture.createFixture(
            proposal, scope, this);

        // save and compare
        SingleResult result = save(dependency, scope);
        ElementDependency returnedDependency = (ElementDependency) result
            .getRpmObject();
        
        // createdBy is read-only and generated by the DB
        if (scope != null && scope.getCreatedBy() != null)
        {
            assertNotNull(returnedDependency.getCreatedBy());
        }
        DependencyFixture.compare(dependency, returnedDependency);

        // reload and compare
        returnedDependency = (ElementDependency) load(returnedDependency,
            scope, loadFromId);
        
        if (scope != null && scope.getCreatedBy() != null)
        {
            assertNotNull(returnedDependency.getCreatedBy());
        }
        DependencyFixture.compare(dependency, returnedDependency);

        return dependency;
    }

    private ElementDependency updateElementDependency(
            ElementDependency dependency, ElementDependencyScope scope,
            boolean loadFromId) throws Exception
    {
        DependencyFixture.updateFixture(dependency);

        // save and compare
        SingleResult result = save(dependency, null);
        ElementDependency returnedDependency = (ElementDependency) result
            .getRpmObject();
        DependencyFixture.compare(dependency, returnedDependency);

        // reload and compare
        returnedDependency = (ElementDependency) load(returnedDependency,
            scope, loadFromId);
        DependencyFixture.compare(dependency, returnedDependency);

        return dependency;
    }

    private RPMObject load(ElementDependency dependency,
            ElementDependencyScope scope, boolean loadFromId) throws Exception
    {
        ElementDependency reloaded = null;

        if (loadFromId)
        {
            //load from id
            RPMObject obj = new ElementDependency();
            obj.setID(dependency.getID());
            reloaded = (ElementDependency) loadFromID(obj, scope);
        }
        else
        {
            //load from xpath
            RPMObject[] objs = loadFromXpath("/ElementDependency[id='"
                    + dependency.getID() + "']", scope);
            if (objs != null && objs.length > 0)
            {
                reloaded = (ElementDependency) objs[0];
            }
        }
        return reloaded;
    }
}
