package com.ibm.rpm.client.junit.wbs.testcases;

/**
 * @author vnarayan
 */

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.scope.PortfolioScope;

public class TestPortfolio extends ClientTestController 
{

	private PortfolioScope mainScope = null;
	private Portfolio rootTestPortfolio = null;
	
	//TODO: Confirm these values by couple of runs
    private static final int NB_MAX_LEVELS = 1;
    private static final int NB_PORTFOLIOS_BY_LEVEL = 1;
    private static final int NB_PROPOSALS_BY_LEVEL = 1;
    private static final int DELTA_PORTFOLIOS_BY_LEVEL = 1;
    private static final int DELTA_PROJECTS_BY_LEVEL = 1;
	
	public TestPortfolio(String testName) {
		setName(testName);
	}

	public void setUp() throws Exception {
		logger.info("setUp");
		super.setUp();
		mainScope = PortfolioFixture.createScope(1,NB_MAX_LEVELS,NB_PORTFOLIOS_BY_LEVEL,NB_PROPOSALS_BY_LEVEL);
		
		PortfolioScope rootPortfolioScope = new PortfolioScope();
		rootPortfolioScope.setPortfolios(mainScope);
		mainScope.setParent(rootPortfolioScope);
		rootTestPortfolio = PortfolioFixture.loadOrCreateRootPortfolio(this);
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
			Exception {
	    insertPortfolio(null, false);
	}

	public void testInsertWithNoScopeLoadFromID() throws AssertionError,
			Exception {
	    insertPortfolio(null, true);
	}

	public void testInsertWithScopeLoadFromID() throws AssertionError,
			Exception {
	    insertPortfolio(mainScope, true);
	}

	public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
			Exception {
		update(null, true);
	}

	public void testUpdateWithScopeLoadFromID() throws AssertionError,
			Exception {
		update(mainScope, true);
	}
    
    private Portfolio insertPortfolio(PortfolioScope scope,
			boolean loadFromId) throws Exception
	{
		Portfolio portfolio = PortfolioFixture.createFixture(NB_PORTFOLIOS_BY_LEVEL,
				NB_PROPOSALS_BY_LEVEL, scope,this);
		//set rootPortfolio as parent
		portfolio.setParent(rootTestPortfolio);
		//Insert the portfolio into the DB
		SingleResult result = save(portfolio, scope);
		//compare the returned folder
		Portfolio returnedPortfolio =(Portfolio)result.getRpmObject();
		PortfolioFixture.compare(portfolio, returnedPortfolio, scope);
		//reload the portfolio and compare it
		returnedPortfolio = (Portfolio) load(returnedPortfolio, scope, loadFromId);
		PortfolioFixture.compare(portfolio, returnedPortfolio, scope);
		return portfolio;
	}

	private void update(PortfolioScope scope,
			boolean loadFromId) throws AssertionError, Exception
	{
		//insert a AssetFolder
		Portfolio portfolio = insertPortfolio(scope, loadFromId);		
		portfolio = updatePortfolioFields(portfolio,scope,loadFromId);		
		portfolio = updatePortfolioStructure(portfolio, scope, loadFromId);
	}

	private Portfolio updatePortfolioFields(Portfolio portfolio,PortfolioScope scope, boolean loadFromId) throws Exception
	{
		return updatePortfolio(portfolio, scope, loadFromId, false);
	}

	private Portfolio updatePortfolioStructure(Portfolio portfolio,
			PortfolioScope scope, boolean loadFromId) throws Exception
	{
		return updatePortfolio(portfolio, scope, loadFromId, true);
	}

	private Portfolio updatePortfolio(Portfolio portfolio,
			PortfolioScope scope, boolean loadFromId,
			boolean updateStructure) throws Exception
	{
		if (updateStructure) {
			//updates the structure by adding elements
			PortfolioFixture.updateFixtureStructure(portfolio,
					DELTA_PORTFOLIOS_BY_LEVEL, DELTA_PROJECTS_BY_LEVEL,
					scope, this);
		} else {
			//only updates the fields
			PortfolioFixture.updateFixtureFields(portfolio, scope,this);
		}
		//Insert the updated Portfolio into the DB
		SingleResult result = save(portfolio, scope);
		portfolio = (Portfolio) result.getRpmObject();
		//compare the returned Portfolio
		Portfolio returnedPortfolio = (Portfolio) result.getRpmObject();
		PortfolioFixture.compare(portfolio, returnedPortfolio, scope);
		//reload the Portfolio and compare it
		returnedPortfolio = (Portfolio) load(returnedPortfolio, scope, loadFromId);
		PortfolioFixture.compare(portfolio, returnedPortfolio, scope);
		//return the Portfolio
		return returnedPortfolio;
	}


	private RPMObject load(Portfolio portfolio , PortfolioScope scope,
			boolean loadFromId) throws Exception
	{
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new Portfolio();
            obj.setID(portfolio.getID());
            return loadFromID(obj, scope);
        }
        //load from an xpath
        RPMObject[] objs = loadFromXpath("/PORTFOLIO[id='" + portfolio.getID()
                + "']", scope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }
}

