package com.ibm.rpm.client.junit.wbs.testcases;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ProfileFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ResourceType;

/**
 * 
 * @author gosselin
 */
public class TestResourceTaskAssignmentProfile extends ClientTestController
{
    protected ResourceTaskAssignmentScope mainScope = null;
    protected WorkElement parentWorkElement = null;
    private Profile profile = null;

    public TestResourceTaskAssignmentProfile(String testName)
    {
        setName(testName);
    }
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        mainScope = ResourceTaskAssignmentFixture.createScope(true, false,
            true, false, true);

        parentWorkElement = TaskAssignmentFixture.createProperWorkElement(this);
        save(parentWorkElement, null);
        // parentWorkElement.setProject(null);

        profile = ProfileFixture.createFixture(mainScope.getProfile(), 1,
                null, this);
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    public void testLoadFromWbsPerspective() throws Exception
    {
        ResourceTaskAssignment assignment = ResourceTaskAssignmentFixture
            .createFixture(parentWorkElement, profile, this);
        
        ResourceTaskAssignment saved = (ResourceTaskAssignment) save(
            assignment, mainScope).getRpmObject();

        WorkElement parent = new WorkElement();
        parent.setID(parentWorkElement.getID());

        checkForProperAssignment(parent, saved, mainScope);
    }

    public void testDeleteFromWbsPerspective() throws Exception
    {
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(
            parentWorkElement, profile, this);

        save(assignment, mainScope).getRpmObject();

        WorkElement parent = new WorkElement();
        parent.setID(parentWorkElement.getID());
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(mainScope);
        parent = (WorkElement) loadFromID(parent, workElementScope);
        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array length", parent
            .getResourceTaskAssignments().length > 0);

        ResourceTaskAssignment newAssignment = null;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                assignment.getID()))
            {
                newAssignment = parent.getResourceTaskAssignments()[i];
                break;
            }
        }
        assertNotNull("Proper ResourceTaskAssignment was not loaded",
            newAssignment);
        newAssignment.setDeleted(Boolean.TRUE);

        parent = (WorkElement) save(parent, workElementScope).getRpmObject();
        parent = (WorkElement) loadFromID(parent, workElementScope);
        boolean found = false;
        if (parent.getResourceTaskAssignments() != null
                && parent.getResourceTaskAssignments().length > 0)
        {
            for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
            {
                if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignment.getID()))
                {
                    found = true;
                    break;
                }
            }
        }
        assertFalse("ResourceTaskAssignment should be deleted", found);
    }

    private void checkForProperAssignment(WorkElement parent,
            ResourceTaskAssignment assignmentToCompare,
            ResourceTaskAssignmentScope scope) throws Exception
    {
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(scope);
        parent = (WorkElement) loadFromID(parent, workElementScope);

        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array is empty", parent
            .getResourceTaskAssignments().length > 0);
        boolean found = false;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                assignmentToCompare.getID()))
            {
                ResourceTaskAssignmentFixture.compare(assignmentToCompare,
                    parent.getResourceTaskAssignments()[i], scope);
                found = true;
                break;
            }
        }

        assertTrue("Proper ResourceTaskAssignment was not loaded", found);
    }

    // testInsertWithNoScopeLoadFromXPath()
    // Not applicable because assignment cannot be loaded direclty from
    // XPath. they must be loaded through their parents

    // testInsertWithNoScopeLoadFromID()
    // No point to do it without scope, refer to testInsertWithScopeLoadFromID

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);
        deleteAssignment(assignment);
    }

    public void testReplaceProfileWithResource() throws AssertionError, Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        Resource resource = ResourceFixture.createSaveFixture(resourceScope,
            this);
        
        ResourceAttributeAssignment[] assignments = ResourceAttributeAssignmentFixture
            .createFixtureFromProfile(assignment.getProfile()
                .getProfileCompetencyAssignments());
        resource.setCompetencyAttributeAssignments(assignments);

        assignments = ResourceAttributeAssignmentFixture
        .createFixtureFromProfile(assignment.getProfile()
            .getProfileSkillAssignments());
        resource.setSkillAttributeAssignments(assignments);
        
        resourceScope.setCompetencyAttributeAssignments(true);
        resourceScope.setSkillAttributeAssignments(true);
        save(resource, resourceScope);
       
        ResourceTaskAssignment rtaAssignment = (ResourceTaskAssignment)getApplicationInterface()
            .replaceProfileWithResource(getSessionID(), assignment, mainScope, resource).getRpmObject();
        
        assertNotNull(rtaAssignment);
        assertNotNull(rtaAssignment.getResource());
        assertNull(rtaAssignment.getProfile());
    }
    
    
    /**
     * Create a new task, assign work to it, replace it with a profile created by the SP based on the resource we're deleting,
     * then make sure we can read both the profile and the Administrator RTA.
     * @throws AssertionError
     * @throws Exception
     */
    public void testReplaceResourceWithProfile() throws AssertionError, Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resourceScope.setCompetencyAttributeAssignments(true);
        resourceScope.setSkillAttributeAssignments(true);
 
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        taScope.setParent(mainScope);
        
        mainScope.setResource(resourceScope);
        mainScope.setTaskAssignment(taScope);
        
        // Create the ResourceTaskAssignment's parent
        WorkElementScope taskScope =  new WorkElementScope();
        taskScope.setContainingProject(new WorkElementScope()); // necessary for the fixture
        Task task = (Task) WorkElementFixture.createFixture(new Task(), taskScope, this);
        task = (Task) save(task, taskScope).getRpmObject();
        
        Resource resource = ResourceFixture.createSaveFixture(resourceScope, this);
        
        // Create a ResourceTaskAssignment with a Resource and a TaskAssignment
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        assignment.setParent(task);
        assignment.setSecurityRole(SecurityRoleFixture.createSecurityRole(assignment.getParent()));
        assignment.setResource(resource);
        
        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setAssignmentStartDate(new GregorianCalendar());
        taskAssignment.setParent(assignment);
        
        assignment.setTaskAssignment(taskAssignment);
        
        // Save the ResourceTaskAssignment
        save(assignment, mainScope);
       
        getApplicationInterface().replaceResourceWithProfile(getSessionID(), assignment, mainScope).getRpmObject();

        /*
         * reload the task with all of its RTAs
         */
        taskScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        Task reloadedTask = (Task)loadFromID(task, taskScope);
        
        assertTrue(reloadedTask.getResourceTaskAssignments().length == 2);
        ResourceType first =  reloadedTask.getResourceTaskAssignments()[0].getResourceType();
        ResourceType second =  reloadedTask.getResourceTaskAssignments()[1].getResourceType();
        assertTrue("We are expecting the old resource to be removed, and a new RTA created with a type of Profile",first.equals(ResourceType.Profile) && second.equals(ResourceType.Resource)
            || second.equals(ResourceType.Profile) && first.equals(ResourceType.Resource));
    }
    
    private ResourceTaskAssignment insertAssignment(
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        //create and save a workelement
        ResourceTaskAssignment assignment = null;
        if (mainScope.getProfile() != null)
        {
            assignment = ResourceTaskAssignmentFixture.createFixture(
                parentWorkElement, profile, this);
            assignment.setTaskAssignment(new TaskAssignment());
        }

        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
            .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);
        return returnedAssignment;
    }


    private void deleteAssignment(ResourceTaskAssignment assignment) throws Exception
    {
        delete(assignment);
    }

    private ResourceTaskAssignment load(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new ResourceTaskAssignment();
            obj.setID(assignment.getID());
            return (ResourceTaskAssignment) loadFromID(obj, scope);
        }
        fail("Assignments cannot be loaded directly from XPath. They must be loaded via their parents.");
        return null;
    }
}

