package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.ProfileFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author gosselin
 */
public class TestScopeResourceTaskAssignmentProfile extends
        ClientTestController
{
    protected ResourceTaskAssignmentScope mainScope = null;
    protected ScopeElement scopeParent = null;
    private Profile profile = null;

    public TestScopeResourceTaskAssignmentProfile(String testName)
    {
        setName(testName);
    }
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        mainScope = ResourceTaskAssignmentFixture.createScope(true, true,
            true, false, true);
        ScopeElementScope scopeScope = new ScopeElementScope();
        scopeScope.setProject(new WorkElementScope());
        mainScope.setParent(scopeScope);
        scopeParent = TaskAssignmentFixture.createProperScopeElement(this);
        
        profile = ProfileFixture.createFixture(mainScope.getProfile(), 1,
                null, this);
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    //testInsertWithNoScopeLoadFromXPath()
    //Not applicable because assignment cannot be loaded direclty from
    // XPath
    //they must be loaded through their parents

    // testInsertWithNoScopeLoadFromID()
    // No point to do it without scope, refer to testInsertWithScopeLoadFromID

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);
        delete(assignment);
    }

    public void testInsertWithTaskAssignmentLoadFromID() throws AssertionError,
            Exception
    {
        mainScope.setTaskAssignment(TaskAssignmentFixture.createScope(false,
            false, true, true));
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);
        delete(assignment);
    }

    private ResourceTaskAssignment insertAssignment(
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        //create and save a workelement
        ResourceTaskAssignment assignment = null;
        if (mainScope.getProfile() != null)
        {
            assignment = ResourceTaskAssignmentFixture.createFixture(
                scopeParent, null, profile, mainScope, false, this);
        }

        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
            .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);
        return returnedAssignment;
    }
    
    private ResourceTaskAssignment load(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new ResourceTaskAssignment();
            obj.setID(assignment.getID());
            return (ResourceTaskAssignment) loadFromID(obj, scope);
        }
        fail("Assignments cannot be loaded directly from XPath. They must be loaded via their parents.");
        return null;
    }
}
