/*
 * Created on 2-May-2006
 * @author mmatwiy
 * Test cases for container Task.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTask extends AbstractTestWorkElement
{
    private WorkElementScope mainScope = null;
    
    public void setUp() throws Exception
    {
        super.setUp();
        
        mainScope = WorkElementFixture.createScope(new Task(), true,
            true, false, false, false, false, false, false, false, false,
            false, false, false, false,false);
    }
    
    public void testUpdatePercentDC() throws Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setParent(new WorkElementScope());

        Task task = (Task) WorkElementFixture.createSaveFixture(new Task(),
            scope, this);

        scope.setScheduleDates(true);

        // plan start date must exist in order to edit %DC
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);
        task.setPercentageDurationCompleted(new Double(34.5));

        save(task, scope);

        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        Task reloadedTask = (Task) rpmObjects[0];

        logger.debug("%DC after reload: "
                + reloadedTask.getPercentageDurationCompleted());

        // check that the %DC was properly updated
        assertEquals(reloadedTask.getPercentageDurationCompleted(), new Double(
            34.5));

    }

    public void testModifyPercentDCUpdateActualDuration() throws Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setParent(new WorkElementScope());

        Task task = (Task) WorkElementFixture.createSaveFixture(new Task(),
            scope, this);

        scope.setScheduleDates(true);

        // plan start date must exist in order to edit %DC
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(2400)); // 5 days

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(2400)); // 5 days

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);
        task.setPercentageDurationCompleted(new Double(20.0));

        save(task, scope);

        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        Task reloadedTask = (Task) rpmObjects[0];

        logger.debug("%DC after reload: "
                + reloadedTask.getPercentageDurationCompleted());

        // check that the %DC was properly updated
        // and that the actualDate.durationInMinutes was modified by the proc
        assertEquals(reloadedTask.getPercentageDurationCompleted(), new Double(
            20));
        assertEquals(reloadedTask.getActualDate().getDurationInMinutes(),
            new Integer(480));//1 day

    }

    private Task insertTask(WorkElementScope scope, boolean loadFromId)
            throws Exception
    {
        //create and save a task
        Task task = (Task) WorkElementFixture.createFixture(new Task(), scope,
            this);

        SingleResult result = save(task, scope);

        //compare the returned task
        Task returnedTask = (Task) result.getRpmObject();
        WorkElementFixture.compare(task, returnedTask, scope);

        //reload the Task and compare it
        returnedTask = (Task) load(returnedTask, scope, loadFromId);
        WorkElementFixture.compare(task, returnedTask, scope);
        
        assertNotNull(returnedTask.getReferenceID()); // read-only - always populated by proc

        return returnedTask;
    }

    private void updateTask(WorkElementScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert
        Task task = insertTask(scope, loadFromId);
        
        //update
        task = (Task) updateWorkElementFields(task, scope, loadFromId, 1, 1);
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,Exception
    {
        insertTask(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,Exception
    {
        insertTask(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,Exception
    {
        insertTask(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,Exception
    {
        updateTask(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,Exception
    {
        updateTask(mainScope, true);
    }
    
    public void testCloseTask() throws AssertionError,Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);
        
        //create and close a task
        Task task = (Task) WorkElementFixture.createFixture(new Task(), null,
            this);
        task.setState(DatafieldFixture.createClosedTaskState());

        SingleResult result = save(task, scope);
        Task returnedTask = (Task) result.getRpmObject();
        
        //reload the Task and compare it
        returnedTask = (Task) load(returnedTask, scope, true);
        WorkElementFixture.compare(task, returnedTask, scope);
        
        //read-only, populated when workElement is closed
        assertNotNull(returnedTask.getClosedByDate()); 
        assertNotNull(returnedTask.getClosedByName());
        
        //read-only - always populated by proc
        assertNotNull(returnedTask.getReferenceID()); 
        
        //test load on new closedBy fields
        RPMObject[] loadedResult = loadFromXpath("/Task[@id='"
                + returnedTask.getID() + "' and @closedByDate='"
                + DateUtil.convertToString(returnedTask.getClosedByDate())
                + "' and @closedByName='" + returnedTask.getClosedByName()
                + "']", null);
        
        assertNotNull(loadedResult);
        assertNotNull(loadedResult[0]);
        WorkElementFixture.compare(task, (Task) loadedResult[0], scope);
    }

}