/*
 * Created on 1-Jun-2006
 * @author mmatwiy
 * Test cases for transfer of a WorkElement under scope.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTransferUnderScope extends ClientTestController
{
    WorkElementScope workElementScope;

    ScopeElementScope scopeElementScope;

    public void setUp() throws Exception
    {
        super.setUp();

        workElementScope = new WorkElementScope();
        workElementScope.setScopeElement(new ScopeElementScope());

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setChildren(new ScopeElementScope());
        scopeElementScope.setProject(new WorkElementScope());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testTransferTaskUnderAction() throws Exception
    {
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(project);

        // create an Action
        Action action = new Action();
        action.setName(HelperTestUtilities.createName());
        action.setParent(scopeFolder);
        save(action, scopeElementScope);

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(project);
        save(task, workElementScope);

        // transfer the task to the action
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), action, task);

        assertTrue(result.isSuccessful());

        // reload the action and check if it contains the task
        action = (Action) loadFromID(action, scopeElementScope);

        assertNotNull(action);
        assertNotNull(action.getChildren());
        assertTrue(action.getChildren()[0] instanceof ScopeTask);
    }

    public void testTransferWorkProductUnderRisk() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);

        // create a Risk and reload it to get a populated project field
        Risk risk = new Risk();
        risk.setName(HelperTestUtilities.createName());
        risk.setParent(scopeFolder);
        save(risk, scopeElementScope);

        risk = (Risk) loadFromID(risk, scopeElementScope);

        WorkProduct workProduct = new WorkProduct();
        workProduct.setName(HelperTestUtilities.createName());
        workProduct.setParent(proposal);
        save(workProduct, workElementScope);

        // transfer the WorkProduct to the Risk
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), risk, workProduct);

        assertTrue(result.isSuccessful());

        // reload the risk and check if it contains the WorkProduct
        risk = (Risk) loadFromID(risk, scopeElementScope);

        assertNotNull(risk);
        assertNotNull(risk.getChildren());
        assertTrue(risk.getChildren()[0] instanceof ScopeWorkProduct);
    }

    public void testTransferDeliverableUnderChangeRequest() throws Exception
    {
        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(workOrganization);

        // create a ChangeRequest 
        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setName(HelperTestUtilities.createName());
        changeRequest.setParent(scopeFolder);
        save(changeRequest, scopeElementScope);

        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(workOrganization);
        save(deliverable, workElementScope);

        // transfer the Deliverable to the ChangeRequest
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), changeRequest, deliverable);

        assertTrue(result.isSuccessful());

        // reload the ChangeRequest and check if it contains the Deliverable
        changeRequest = (ChangeRequest) loadFromID(changeRequest,
            scopeElementScope);

        assertNotNull(changeRequest);
        assertNotNull(changeRequest.getChildren());
        assertTrue(changeRequest.getChildren()[0] instanceof ScopeDeliverable);
    }

    public void testTransferSummaryTaskUnderIssue() throws Exception
    {
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(project);

        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(scopeFolder);
        save(issue, scopeElementScope);

        // Create a SummaryTask to transfer under Scope
        SummaryTask summaryTask = new SummaryTask();
        summaryTask.setName(HelperTestUtilities.createName());
        summaryTask.setParent(project);
        save(summaryTask, workElementScope);

        // transfer the SummaryTask to the Issue
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), issue, summaryTask);

        assertTrue(result.isSuccessful());

        // reload the Issue and check if it contains the SummaryTask
        issue = (Issue) loadFromID(issue,
            scopeElementScope);

        assertNotNull(issue);
        assertNotNull(issue.getChildren());
        assertTrue(issue.getChildren()[0] instanceof ScopeSummaryTask);
    }

    public void testTransferMilestoneUnderRequirement() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);

        // create a Requirement 
        Requirement requirement = new Requirement();
        requirement.setName(HelperTestUtilities.createName());
        requirement.setParent(scopeFolder);
        save(requirement, scopeElementScope);

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName());
        milestone.setParent(proposal);
        save(milestone, workElementScope);

        // transfer the Milestone to the Requirement
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), requirement, milestone);

        assertTrue(result.isSuccessful());

        // reload the Requirement and check if it contains the Milestone
        requirement = (Requirement) loadFromID(requirement, scopeElementScope);

        assertNotNull(requirement);
        assertNotNull(requirement.getChildren());
        assertTrue(requirement.getChildren()[0] instanceof ScopeMilestone);
    }

    private ScopeFolder createScopeFolder(GenericProject project)
            throws Exception
    {
        project = (GenericProject) loadFromID(project, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + project.getName() + "']", scopeElementScope);
        ScopeFolder scopeFolder = (ScopeFolder) rpmObjects[0];

        return scopeFolder;
    }

}