package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author gmeloche
 *
 */
public class TestWbsCustomXPath extends ClientTestController
{

    private AttributeCategory category = null;

    private static String CATEGORY_NAME = HelperTestUtilities.unitTestPrefix
            + "Project Specific Attributes";

    private static String CLASSIFICATION_NAME = HelperTestUtilities.unitTestPrefix
            + "RFS & Child DC Numbers";

    private static String ATTRIBUTE_NAME = HelperTestUtilities.unitTestPrefix
            + "RFS Number";

    private static String VALUE = "rfs123";

    public void setUp() throws Exception
    {
        super.setUp();

        AttributeClassification classification = null;
        Attribute attribute = null;

        //Use the WbsModule to save the Attribute structure

        category = new AttributeCategory();
        category.setName(CATEGORY_NAME);

        classification = new AttributeClassification();
        classification.setName(CLASSIFICATION_NAME);

        attribute = new Attribute();
        attribute.setName(ATTRIBUTE_NAME);
        attribute.setInputType(InputType.Text);

        category.setClassifications(new AttributeClassification[]
        {
            classification
        });
        classification.setAttributes(new Attribute[]
        {
            attribute
        });

        WbsModule wbsModule = new WbsModule();
        category.setParent(wbsModule);
        wbsModule.setAttributeCategories(new AttributeCategory[]
        {
            category
        });

        AttributeScope attributeScope = new AttributeScope();
        attributeScope.setClassifications(new AttributeScope());
        attributeScope.getClassifications().setAttributes(new AttributeScope());

        WbsModuleScope moduleScope = new WbsModuleScope();
        moduleScope.setAttributeCategories(attributeScope);

        save(wbsModule, moduleScope);

        //Create a project and assigned the attribute;

        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        AttributeAssignment assignment = new AttributeAssignment();
        assignment.setAttribute(attribute);
        assignment.setValue(VALUE);

        project.setAttributeAssignments(new AttributeAssignment[]
        {
            assignment
        });

        WorkElementScope projectScope = new WorkElementScope();
        projectScope.setAttributeAssignments(true);

        save(project, projectScope);

    }

    public void tearDown() throws Exception
    {

        if (category != null)
        {
            delete(category);
        }

        super.tearDown();

    }

    /**
     * This test is based on a use case from amex
     * @throws Exception
     */
    public void testLoadProjectWithAttributeAssignmentFromXPath_OldWay()
            throws Exception
    {

        String xpath = "/Project[/@attributeAssignments[/AttributeCategory[name = '"
                + CATEGORY_NAME
                + "']/AttributeClassification[name = '"
                + CLASSIFICATION_NAME
                + "']/Attribute[name = '"
                + ATTRIBUTE_NAME + "']][value = '" + VALUE + "']]";

        WorkElementScope scope = new WorkElementScope();
        scope.setAttributeAssignments(true);

        RPMObject[] objects = loadFromXpath(xpath, scope);

        assertNotNull(objects);

    }

    /**
     * This test is based on a use case from amex
     * @throws Exception
     */
    public void testLoadProjectWithAttributeAssignmentFromXPath()
            throws Exception
    {

        String xpath = "/Project[/@attributeAssignments[value = '" + VALUE
                + "' and /AttributeCategory[name = '" + CATEGORY_NAME
                + "']/AttributeClassification[name = '" + CLASSIFICATION_NAME
                + "']/Attribute[name = '" + ATTRIBUTE_NAME + "']]]";

        WorkElementScope scope = new WorkElementScope();
        scope.setAttributeAssignments(true);

        RPMObject[] objects = loadFromXpath(xpath, scope);

        assertNotNull(objects);

    }

    /**
     * This test is based on a use case from amex
     * @throws Exception
     */
    public void testLoadProjectWithAttributeAssignmentFromXPath_2()
            throws Exception
    {

        String xpath = "/Project[/@attributeAssignments[/AttributeCategory[name = '"
                + CATEGORY_NAME
                + "']/AttributeClassification[name = '"
                + CLASSIFICATION_NAME
                + "']/Attribute[name = '"
                + ATTRIBUTE_NAME + "'] and value = '" + VALUE + "']]";

        WorkElementScope scope = new WorkElementScope();
        scope.setAttributeAssignments(true);

        RPMObject[] objects = loadFromXpath(xpath, scope);

        assertNotNull(objects);

    }

}