/*
 * Created on 1-May-2006
 * @author mmatwiy
 * Description: Test methods for converting generic projects to other types
 * of generic projects.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestWorkElementConversion extends ClientTestController
{

    private WorkElementScope scope = null;

    public void setUp() throws Exception
    {
        super.setUp();
        scope = new WorkElementScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testConvertProposalToProject() throws AssertionError, Exception
    {
        Proposal inputProposal = (Proposal) WorkElementFixture
            .createSaveFixture(null, new Proposal(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProposal, new Project(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        assertEquals(result.getRpmObject().getID(), inputProposal.getID());
    }

    public void testConvertProposalToWorkOrganization() throws AssertionError,
            Exception
    {
        Proposal inputProposal = (Proposal) WorkElementFixture
            .createSaveFixture(null, new Proposal(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProposal, new WorkOrganization(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkOrganization);
        assertEquals(result.getRpmObject().getID(), inputProposal.getID());
    }

    public void testConvertProjectToProposal() throws AssertionError, Exception
    {
        Project inputProject = (Project) WorkElementFixture.createSaveFixture(
            null, new Project(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProject, new Proposal(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertEquals(result.getRpmObject().getID(), inputProject.getID());
    }

    public void testConvertProjectToWorkOrganization() throws AssertionError,
            Exception
    {
        Project inputProject = (Project) WorkElementFixture.createSaveFixture(
            null, new Project(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProject, new WorkOrganization(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkOrganization);
        assertEquals(result.getRpmObject().getID(), inputProject.getID());
    }

    public void testConvertWorkOrganizationToProject() throws AssertionError,
            Exception
    {
        WorkOrganization inputWorkOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(null, new WorkOrganization(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputWorkOrganization, new Project(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        assertEquals(result.getRpmObject().getID(), inputWorkOrganization
            .getID());
    }

    public void testConvertWorkOrganizationToProposal() throws AssertionError,
            Exception
    {
        WorkOrganization inputWorkOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(null, new WorkOrganization(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputWorkOrganization, new Proposal(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertEquals(result.getRpmObject().getID(), inputWorkOrganization
            .getID());
    }

    public void testConvertProposalToProposal() throws AssertionError,
            Exception
    {
        Proposal inputProposal = (Proposal) WorkElementFixture
            .createSaveFixture(null, new Proposal(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProposal, new Proposal(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertEquals(result.getRpmObject().getID(), inputProposal.getID());
    }

    public void testConvertWorkProductToSummaryTask() throws AssertionError,
            Exception
    {
        WorkProduct inputWorkProduct = (WorkProduct) WorkElementFixture
            .createSaveFixture(null, new WorkProduct(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputWorkProduct, new SummaryTask(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof SummaryTask);
        assertEquals(result.getRpmObject().getID(), inputWorkProduct.getID());
    }

    public void testConvertWorkProductToDeliverable() throws AssertionError,
            Exception
    {

        WorkProduct inputWorkProduct = (WorkProduct) WorkElementFixture
            .createSaveFixture(null, new WorkProduct(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputWorkProduct, new Deliverable(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Deliverable);
        assertEquals(result.getRpmObject().getID(), inputWorkProduct.getID());
    }

    public void testConvertSummaryTaskToDeliverable() throws AssertionError,
            Exception
    {
        SummaryTask inputSummaryTask = (SummaryTask) WorkElementFixture
            .createSaveFixture(null, new SummaryTask(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputSummaryTask, new Deliverable(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Deliverable);
        assertEquals(result.getRpmObject().getID(), inputSummaryTask.getID());
    }

    public void testConvertSummaryTaskToWorkProduct() throws AssertionError,
            Exception
    {
        SummaryTask inputSummaryTask = (SummaryTask) WorkElementFixture
            .createSaveFixture(null, new SummaryTask(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputSummaryTask, new WorkProduct(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkProduct);
        assertEquals(result.getRpmObject().getID(), inputSummaryTask.getID());
    }

    public void testConvertDeliverableToWorkProduct() throws AssertionError,
            Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new WorkProduct(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkProduct);
        assertEquals(result.getRpmObject().getID(), inputDeliverable.getID());
    }

    public void testConvertDeliverableToSummaryTask() throws AssertionError,
            Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new SummaryTask(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof SummaryTask);
        assertEquals(result.getRpmObject().getID(), inputDeliverable.getID());
    }

    public void testConvertDeliverableToTask() throws AssertionError, Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new Task(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Task);
        assertEquals(result.getRpmObject().getID(), inputDeliverable.getID());
    }
    
    public void testConvertDeliverableToMilestone() throws AssertionError, Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new Milestone(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Milestone);
        assertEquals(result.getRpmObject().getID(), inputDeliverable.getID());
    }
    
    public void testConvertMilestoneToTask() throws AssertionError, Exception
    {
        Milestone inputMilestone = (Milestone) WorkElementFixture
            .createSaveFixture(null, new Milestone(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputMilestone, new Task(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Task);
        assertEquals(result.getRpmObject().getID(), inputMilestone.getID());
    }
    
    public void testConvertTaskToMilestone() throws AssertionError, Exception
    {
        Task inputTask = (Task) WorkElementFixture
            .createSaveFixture(null, new Task(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputTask, new Milestone(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Milestone);
        assertEquals(result.getRpmObject().getID(), inputTask.getID());
    }
    
    public void testConvertTaskToWorkProduct() throws AssertionError, Exception
    {
        Task inputTask = (Task) WorkElementFixture
            .createSaveFixture(null, new Task(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputTask, new WorkProduct(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkProduct);
        assertEquals(result.getRpmObject().getID(), inputTask.getID());
    }
    
    public void testConvertTaskToDeliverable() throws AssertionError, Exception
    {
        Task inputTask = (Task) WorkElementFixture
            .createSaveFixture(null, new Task(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputTask, new Deliverable(), scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Deliverable);
        assertEquals(result.getRpmObject().getID(), inputTask.getID());
    }

}