package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ContourType;

public class AbstractTestContour extends ClientTestController {

	// TODO: Only three ContourTypes are tested
	//			1. ASAP 
	//			2. HoursPerDay
	//			3. PerncetagePerDay
	// Rest of the types are yet to be done
	
	protected ResourceTaskAssignment assignment = null;
	protected WorkElement parentWorkElement = null;
	protected ResourceTaskAssignmentScope mainScope = null;
	
	protected final static int EFFORT_TYPE_NONE = -1;
	protected final static int EFFORT_TYPE_EFFORT = 0;
	protected final static int EFFORT_TYPE_FINISHDATE = 1;
	protected final static int EFFORT_TYPE_OCCURRENCES = 2;
	protected final static int numOfDays = 3;
	protected final static int minutesPerDay = 480;;
	protected final static Calendar startDate = Calendar.getInstance();
	protected final static Calendar finishDate = Calendar.getInstance();
	protected final static Double workHoursPerDay = new Double(3);
	protected final static Double percentilePerDay = new Double(37.5);
	protected final static Integer occurrences = new Integer(8);
	
	public void setUp() throws Exception
	{
		super.setUp();
		parentWorkElement = createSaveParentWorkElement();
	}
	
	public void testAssignResourceWithNoContour() throws Exception
	{
		assignment.getTaskAssignment().setProposedDurationHours(new Double(8.0));
		saveAndValidateContouredAssignment(null,EFFORT_TYPE_NONE);
	}
	
	public void testAssignUsingContourTypeASAPForEffort() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.ASAP, EFFORT_TYPE_EFFORT);
	}
	
	public void testAssignUsingContourTypeASAPForFinishDate() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.ASAP, EFFORT_TYPE_FINISHDATE);
	}
	
	public void testAssignUsingContourTypeHoursPerDayForEffort() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.HoursPerDay, EFFORT_TYPE_EFFORT);
	}
	
	public void testAssignUsingContourTypeHoursPerDayForFinishDate() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.HoursPerDay, EFFORT_TYPE_FINISHDATE);
	}
	
	public void testAssignUsingContourTypeHoursPerDayForOccurrences() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.HoursPerDay, EFFORT_TYPE_OCCURRENCES);
	}
	
	public void testAssignUsingContourTypePercentagePerDayForEffort() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.PercentilePerDay, EFFORT_TYPE_EFFORT);
	}
	
	public void testAssignUsingContourTypePercentagePerDayForFinishDate() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.PercentilePerDay, EFFORT_TYPE_FINISHDATE);
	}
	
	public void testAssignUsingContourTypePercentagePerDayForOccurrences() throws Exception
	{
		saveAndValidateContouredAssignment(ContourType.PercentilePerDay, EFFORT_TYPE_OCCURRENCES);
	}
	
	protected void saveAndValidateContouredAssignment(ContourType contourType, int effortType) throws Exception
	{
		Contour contour = null;
		if(contourType != null)
		{
			contour = createContour(contourType, effortType);
			assignment.setContour(contour);
		}
		
		assignment = (ResourceTaskAssignment)save(assignment, mainScope).getRpmObject();
		assertNotNull(assignment);
		
		assignment = loadAssignment();
		
		validate(contour,assignment.getTaskAssignment());
	}
	
	private Contour createContour(ContourType contourType, int effortType)
	{
		Contour contour = new Contour();
		contour.setContourType(contourType);
		contour.setStartDate(startDate);
		if(contourType.equals(ContourType.HoursPerDay))
		{
			contour.setWork(workHoursPerDay);
		}
		else if(contourType.equals(ContourType.PercentilePerDay))
		{
			contour.setWork(percentilePerDay);
		}
		switch (effortType)
		{
			case EFFORT_TYPE_EFFORT:
				contour.setEffortInMinutes(new Integer(numOfDays * minutesPerDay));
				break;
			case EFFORT_TYPE_FINISHDATE:
				finishDate.add(Calendar.DATE, numOfDays);
				contour.setFinishDate(finishDate);
				break;
			case EFFORT_TYPE_OCCURRENCES:
				if(!contourType.equals(ContourType.ASAP))
				{
					contour.setOccurrences(occurrences);
				}
				else
				{
					fail("Field Occurrences should not be populated for Contour type ASAP");
				}
				break;
		}
		return contour;
	}
	
	private ResourceTaskAssignment loadAssignment() throws Exception
	{
		ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
		rtaScope.setTaskAssignment(new TaskAssignmentScope());
		assignment = (ResourceTaskAssignment)loadFromID(assignment, rtaScope);
		assertNotNull(assignment);
		assertNotNull(assignment.getTaskAssignment());
		assertNotNull(assignment.getTaskAssignment().getEETCHours());
		assertNotNull(assignment.getTaskAssignment().getAssignmentFinishDate());
		assertNotNull(assignment.getTaskAssignment().getAssignmentStartDate());
		assertNotNull(assignment.getTaskAssignment().getUnitsPercentage());
		assertNotNull(assignment.getTaskAssignment().getProposedDurationHours());
		return assignment;
	}
	
	private void validate(Contour contour, TaskAssignment taskAssignment)
	{
		Calendar actualStartDate = taskAssignment.getAssignmentStartDate();
		Calendar actualFinishDate = taskAssignment.getAssignmentFinishDate();
		Integer actualEffortInMinutes = new Integer(new Double(taskAssignment.getEETCHours().doubleValue() * 60).intValue());
		Double actualUnits = taskAssignment.getUnitsPercentage();
		int proposedDurationInMinutes = new Double(taskAssignment.getProposedDurationHours().doubleValue() * 60).intValue();
		
		if(contour != null)
		{
			Calendar expectedStartDate = contour.getStartDate();
			Calendar expectedFinishDate = contour.getFinishDate();
			Integer expectedEffortInMinutes = contour.getEffortInMinutes();
			Double expectedWork = contour.getWork();
			Integer occurrences = contour.getOccurrences();
			ContourType contourType = contour.getContourType();

			validateDate(expectedStartDate, actualStartDate);
			if(expectedEffortInMinutes != null && expectedEffortInMinutes.intValue() > 0)
			{
				assertEquals(expectedEffortInMinutes.intValue(), actualEffortInMinutes.intValue());
			}
			else if(expectedFinishDate != null)
			{
				validateDate(expectedFinishDate, actualFinishDate);
			}

			if(contourType.equals(ContourType.ASAP))
			{
				double expectedValue = 100;
				assertTrue(expectedValue==actualUnits.doubleValue());
			}
			else if(contourType.equals(ContourType.HoursPerDay))
			{
				validateHoursPerDayContour(expectedEffortInMinutes, actualEffortInMinutes, expectedWork, actualUnits, occurrences);
			}
			else if(contourType.equals(ContourType.PercentilePerDay))
			{
				validatePercentageDayContour(expectedEffortInMinutes, actualEffortInMinutes, expectedWork, actualUnits, occurrences);
			}
		}
		else
		{
			assertEquals(proposedDurationInMinutes, actualEffortInMinutes.intValue());
		}
	}
	
	private void validateHoursPerDayContour(Integer expectedEffortInMinutes, Integer actualEffortInMinutes, Double expectedWork, Double actualUnits, Integer occurrences)
	{
		double expectedUnits =(expectedWork.doubleValue() / 8 ) * 100;
		assertTrue(expectedUnits==actualUnits.doubleValue());
		if(occurrences != null)
		{
			expectedEffortInMinutes = new Integer(new Double(expectedWork.doubleValue() * 60 * occurrences.intValue()).intValue());
			assertEquals(expectedEffortInMinutes.intValue(), actualEffortInMinutes.intValue());
		}
	}
	
	private void validatePercentageDayContour(Integer expectedEffortInMinutes, Integer actualEffortInMinutes, Double expectedWork, Double actualUnits, Integer occurrences)
	{
		assertEquals(expectedWork, actualUnits);
		if(occurrences != null)
		{
			double hoursPerDay = (expectedWork.doubleValue() / 100) * 8;
			expectedEffortInMinutes = new Integer(new Double(hoursPerDay * 60 * occurrences.intValue()).intValue());
			assertEquals(expectedEffortInMinutes.intValue(), actualEffortInMinutes.intValue());
		}
	}
	
	private void validateDate(Calendar expectedDate, Calendar actualDate)
	{
		assertNotNull(expectedDate);
		assertNotNull(actualDate);
		int dayOfWeek = expectedDate.get(Calendar.DAY_OF_WEEK); 
		if(dayOfWeek==Calendar.SATURDAY)
		{
			expectedDate.add(Calendar.DATE, -1);
		}
		if(dayOfWeek==Calendar.SUNDAY)
		{
			expectedDate.add(Calendar.DATE, -2);
		}
		assertEquals(expectedDate.get(Calendar.YEAR), actualDate.get(Calendar.YEAR));
		assertEquals(expectedDate.get(Calendar.MONTH), actualDate.get(Calendar.MONTH));
		assertEquals(expectedDate.get(Calendar.DATE), actualDate.get(Calendar.DATE));
	}
	
	private WorkElement createSaveParentWorkElement() throws Exception
	{
		WorkElementScope taskScope = new WorkElementScope();
		taskScope.setContainingProject(new WorkElementScope());
		WorkElement parentWorkElement = WorkElementFixture.createSaveFixture(new Task(), taskScope, this);
		assertNotNull(parentWorkElement);
		assertNotNull(parentWorkElement.getContainingProject());
		return parentWorkElement;
	}
}