package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author gmeloche
 *  
 */
public class Test54785 extends ClientTestController
{

    private static Project project = null;

    private static SummaryTask summaryTask = null;

    private static Resource resource = null;

    private static TaskAssignment taskAssignment = null;

    /**
     * Load a a summaryTimesheet with the parent of the resourceTaskAssignment
     * where the parent is a scopeElement transfered under wbs.
     * 
     * @throws Exception
     */
public void testLoadSummaryTimesheet() throws Exception
    {

        createResource();
        createServiceRequestAndTransferUnderWbsRoot();
        assignResourceToTask();

        String xpath = "/Task/ResourceTaskAssignment[@resource[id = '"
                + resource.getID() + "']]/TaskAssignment";

        RPMObject[] objects = loadFromXpath(xpath, null);
        assertNotNull(objects);

        taskAssignment = (TaskAssignment) objects[0];

        //create a SummaryTimesheet

        System.out.println("\nCreate a SummaryTimesheet");

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        //Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(taskAssignment.getAssignmentStartDate());
        summaryTimesheet.setRegularMonday(new Double(8));

       
        SingleResult result1 = save(summaryTimesheet, null);

        assertNotNull(result1.getRpmObject());
        SummaryTimesheet loadedTimesheet = (SummaryTimesheet) result1
            .getRpmObject();
        taskAssignment = (TaskAssignment)loadedTimesheet.getTaskAssignment();
        assertNotNull(taskAssignment);
        
        //reload taskAssignment with resourceTaskAssignment and workElement in the scope
        ResourceTaskAssignmentScope rtaScope = new  ResourceTaskAssignmentScope();
        rtaScope.setParent(new WorkElementScope());
        
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        taScope.setParent(rtaScope);
        
        taskAssignment = (TaskAssignment)loadFromID(taskAssignment, taScope);
        
        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment)taskAssignment.getParent();
        assertNotNull(resourceTaskAssignment);
        assertNotNull(resourceTaskAssignment.getParent());
        
        assertEquals(summaryTask.getID(),
            resourceTaskAssignment.getParent().getID());
         

    }
    private void createResource() throws Exception
    {
        Pool pool = new Pool();
        pool.setName(HelperTestUtilities.createName());

        resource = new Resource();
        resource.setFullName(HelperTestUtilities.createName());

        //Load the default calendar
        RPMObject[] objects = loadFromXpath(
            "/RPMCalendar[name = 'Standard (Default)']", null);
        assertNotNull(objects);
        RPMCalendar calendar = (RPMCalendar) objects[0];

        //Resource calendar is mandatory to assign a resource
        resource.setCalendar(calendar);

        pool.setResources(new Resource[]
        {
            resource
        });

        PoolScope poolScope = new PoolScope();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        poolScope.setResources(resourceScope);

        SingleResult result2 = save(pool, poolScope);
        assertNotNull(result2.getRpmObject());
        pool = (Pool) result2.getRpmObject();
        resource = pool.getResources()[0];

    }

    private void createServiceRequestAndTransferUnderWbsRoot() throws Exception
    {
        //create a project and retrieve the associated scopeFolder
        project = new Project();
        project.setName(HelperTestUtilities.createName());
        project.setPublished(ProjectPublishedType.ToTeam);

        WorkElementScope projectScope = new WorkElementScope();
        projectScope.setScopeElement(new ScopeElementScope());

        SingleResult result = save(project, projectScope);
        assertNotNull(result.getRpmObject());
        project = (Project) result.getRpmObject();

        ScopeFolder scopeFolder = (ScopeFolder) project.getScopeElement();
        assertNotNull(scopeFolder);

        //create a serviceRequest
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setName(HelperTestUtilities.createName());
        serviceRequest.setParent(scopeFolder);

        SingleResult result1 = save(serviceRequest, null);
        assertNotNull(result1.getRpmObject());
        serviceRequest = (ServiceRequest) result1.getRpmObject();


        //transfert unde wbs root
        transferUnderWBS(null, serviceRequest);

        //get the associated task
        summaryTask = new SummaryTask();
        summaryTask.setID(serviceRequest.getID());

        WorkElementScope taskScope = new WorkElementScope();

        //Contaning project is mandatory to create a ResourceTaskAssignment
        taskScope.setContainingProject(new WorkElementScope());

        summaryTask = (SummaryTask) loadFromID(summaryTask, taskScope);

        assertNotNull(summaryTask);

    }

    private void assignResourceToTask() throws Exception
    {
        //Create resourceTaskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        //Load the security role
        RPMObject[] objects = loadFromXpath(
            "/SecurityRole[name = 'Deliverable Responsible']", null);
        assertNotNull(objects);
        SecurityRole securityRole = (SecurityRole) objects[0];

        resourceTaskAssignment.setSecurityRole(securityRole);
        resourceTaskAssignment.setResource(resource);

        //Create TaskAssignment
        TaskAssignment taskAssignment = new TaskAssignment();
        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        summaryTask.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });

        //Create task scope
        WorkElementScope taskScope = new WorkElementScope();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setResource(new ResourceScope());
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);
        taskScope.setResourceTaskAssignments(rtaScope);

        //Save Task
        save(summaryTask, taskScope);

        //Copy Project proposed dates to plan dates (mandatory for timesheet
        // creation)
        SingleResult result3 = copyProposedToPlan(project, null);
        assertNotNull(result3.getRpmObject());

    }
}
