package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * 
 * Ref No: DB2-SM-700 Date: 2006/11/29 Version: 7.0.4.4 (Build 10) Reported by:
 * Stephane Requirement: Tomcat 5.5.17 (JVM 1.4.2_11-b06) Regression: No
 * 
 * Description I create a milestone with an actual start date and the actual
 * duration in minutes. On the reload the finish date is null and it should be
 * the actual finish date.
 */
public class Test58599 extends ClientTestController
{
    Proposal proposal;

    Milestone milestone;

    String proposalName = "WS-C#-Proposal";

    String milestoneName = "WS-C#-Milestone";

    public void setUp() throws Exception
    {
        super.setUp();

        proposal = new Proposal();
        proposal.setName(proposalName);
        proposal.setPublished(ProjectPublishedType.None);
        proposal.setAssignmentType(AssignmentType.Work);

        SaveResult sr = save(proposal, null);
        proposal = (Proposal) sr.getRpmObject();
        proposal.setPublished(ProjectPublishedType.None);
        proposal.setAssignmentType(AssignmentType.Work);

        milestone = new Milestone();
        milestone.setName(milestoneName);
        proposal.setChildren(new WorkElement[]
        {
            milestone
        });

        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(new WorkElementScope());

        sr = save(proposal, wes);
        proposal = (Proposal) sr.getRpmObject();
        proposal.setPublished(ProjectPublishedType.None);
        proposal.setAssignmentType(AssignmentType.Work);

        assertNotNull(proposal.getChildren());
        assertTrue(proposal.getChildren()[0] instanceof Milestone);
        milestone = (Milestone) proposal.getChildren()[0];
    }

    private void setBack(Calendar input, int numberOfDays)
    {
        int present = 25;
        if(numberOfDays > present + 1)
            Assert.fail("Please use a smaller number of days");
        
        input.set(Calendar.DAY_OF_MONTH, present - numberOfDays);
    }
    
    public void testSetActualDateAfterPlanDate() throws Exception
    {
          Calendar planSD = Calendar.getInstance();
          setBack(planSD, 2);
          
         milestone.setPlanDate(new WbsScheduleDate());
         milestone.getPlanDate().setStartDate(planSD);
         milestone.getPlanDate().setDurationInMinutes(new Integer(1000));
        
         WorkElementScope wes = new WorkElementScope();
         wes.setChildren(new WorkElementScope());
         wes.getChildren().setScheduleDates(true);
                     
         SaveResult sr = save(milestone, wes.getChildren());
         milestone = (Milestone)sr.getRpmObject();
         assertNotNull(milestone.getPlanDate());
//         assertTrue(planSD.getTimeInMillis() == milestone.getPlanDate().getStartDate().getTimeInMillis());
        
          Calendar actualSD = Calendar.getInstance();
          setBack(actualSD, 1);
          
         milestone.setActualDate(new WbsScheduleDate());
         milestone.getActualDate().setStartDate(actualSD);
         milestone.getActualDate().setDurationInMinutes(new Integer(980)); // according to the 7.1.1.1 client, this is read-only under these conditions : output an error since otherwise we .
        
         sr = saveMustFail(milestone, wes.getChildren());
         assertNotNull(sr.getErrors());
         assertFalse(sr.isSuccessful());
         assertTrue(sr.getErrors().length == 1);
         
         RPMException error = sr.getErrors()[0];
         assertTrue(error.getErrorID() == ErrorCodes.READ_ONLY_VALUE_EXISTS);
    }
}
