/*
 * Created on 8-Jan-07
 * @author mmatwiy
 * 
 * Defect 59589: Create dependency with finishToFinish type, 
 * dependency is created with finishToStart type
 * 
 * Resolution: Added FinishFinish and StartFinish types to call to 
 * SP_U_DEPEND and to decodeRow in DependenciesManager. 
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.types.TimeUnitType;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;

public class Test59589 extends ClientTestController
{
    private Proposal savedProposal = null;

    private Task savedTask = null;

    private Deliverable savedDeliverable = null;

    private ElementDependency ed = null;

    private WorkElementScope mainScope = null;

    public void setUp() throws Exception
    {
        super.setUp();

        ElementDependencyScope dependencyScope = new ElementDependencyScope();
        dependencyScope.setPredecessor(new WorkElementScope());

        mainScope = new WorkElementScope();
        WorkElementScope childScope = new WorkElementScope();
        childScope.setDependencies(dependencyScope);
        mainScope.setChildren(childScope);
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());

        //Add deliverable to proposal
        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());

        //Add deliverable to proposal
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        proposal.setChildren(new WorkElement[]
        {
                task, deliverable
        });

        SaveResult result = save(proposal, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNull(result.getErrors());
        assertEquals(result.isSuccessful(), true);

        savedProposal = (Proposal) result.getRpmObject();
        assertNotNull(savedProposal);
        savedDeliverable = (Deliverable) savedProposal.getChildren()[1];
        savedTask = (Task) savedProposal.getChildren()[0];

        //create ElementDependency
        ed = new ElementDependency();
        ed.setLeadlagType(LeadLagType.Lead);
        ed.setAmount(new Integer(23));
        ed.setUnitType(TimeUnitType.Hours);
        ed.setPredecessor(savedDeliverable);
        ed.setParent(savedTask);

        savedTask.setDependencies(new ElementDependency[]
        {
            ed
        });
    }

    public void testFinishFinish() throws Exception
    {
        ed.setDependencyType(DependencyType.FinishFinish);

        SaveResult result1 = save(savedProposal, mainScope);
        assertNotNull(result1);
        savedProposal = (Proposal) result1.getRpmObject();
        assertNotNull(savedProposal);
        savedDeliverable = (Deliverable) savedProposal.getChildren()[1];
        savedTask = (Task) savedProposal.getChildren()[0];
        assertEquals(savedTask.getDependencies()[0].getDependencyType(),
            DependencyType.FinishFinish);
    }

    public void testStartFinish() throws Exception
    {
        ed.setDependencyType(DependencyType.StartFinish);

        SaveResult result1 = save(savedProposal, mainScope);
        assertNotNull(result1);
        savedProposal = (Proposal) result1.getRpmObject();
        assertNotNull(savedProposal);
        savedDeliverable = (Deliverable) savedProposal.getChildren()[1];
        savedTask = (Task) savedProposal.getChildren()[0];
        assertEquals(savedTask.getDependencies()[0].getDependencyType(),
            DependencyType.StartFinish);
    }

    public void testFinishStart() throws Exception
    {
        ed.setDependencyType(DependencyType.FinishStart);

        SaveResult result1 = save(savedProposal, mainScope);
        assertNotNull(result1);
        savedProposal = (Proposal) result1.getRpmObject();
        assertNotNull(savedProposal);
        savedDeliverable = (Deliverable) savedProposal.getChildren()[1];
        savedTask = (Task) savedProposal.getChildren()[0];
        assertEquals(savedTask.getDependencies()[0].getDependencyType(),
            DependencyType.FinishStart);
    }

    public void testStartStart() throws Exception
    {
        ed.setDependencyType(DependencyType.StartStart);

        SaveResult result1 = save(savedProposal, mainScope);
        assertNotNull(result1);
        savedProposal = (Proposal) result1.getRpmObject();
        assertNotNull(savedProposal);
        savedDeliverable = (Deliverable) savedProposal.getChildren()[1];
        savedTask = (Task) savedProposal.getChildren()[0];
        assertEquals(savedTask.getDependencies()[0].getDependencyType(),
            DependencyType.StartStart);
    }

}
