package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

/**
 * The user of the APi should not be able to create a TaskAssignment for a duration-based Task.
 * This is verified in TestDurationBasedTaskAssignmentsValidation. Here, we verify that he actually
 * can assign ResourceTaskAssignments with no TaskAssignments as children.
 * @author andreit
 *
 */
public class Test60180 extends ClientTestController
{
    Task task;
    Project project;
    
    public void setUp() throws Exception
    {
        super.setUp();
        
        /*
         * Create a duration-based portfolio and task
         */
           project =   (Project) WorkElementFixture.createFixture(null, new Project(), new WorkElementScope(), this);

           project.setAssignmentType(AssignmentType.Duration);
           project = (Project)save(project, new WorkElementScope()).getRpmObject();
           assertNotNull(project);
           
           task = new Task();
           task.setName(HelperTestUtilities.createName());
           task.setParent(project);
           task.setPriority(new Integer(500));
           task = (Task)save(task, new WorkElementScope()).getRpmObject();
    }
    
    /*
     * create a resourceTaskAssignment (without a TaskAssignment) - should pass
     */
    public void testcantCreateTaskAssignmentForDurationBasedProjects() throws Exception 
    {
           WorkElementScope taskScope = new WorkElementScope();
           taskScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());

           /*
            * Associate to the task a valid ResourceTaskAssignment with no children TaskAssignments
            */
           ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture.createSaveResourceTaskAssignmentForTask(task, this);
           ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
           resourceTaskAssignmentScope.setResource(new ResourceScope());
           ResourceTaskAssignment savedResourceTaskAssignment = 
               (ResourceTaskAssignment)save(resourceTaskAssignment, resourceTaskAssignmentScope).getRpmObject();
           
           task.setResourceTaskAssignments(new ResourceTaskAssignment[] {savedResourceTaskAssignment});
           task = (Task)save(task, taskScope).getRpmObject();
           
           savedResourceTaskAssignment = (ResourceTaskAssignment) loadFromID(savedResourceTaskAssignment, resourceTaskAssignmentScope);
           
           assertNotNull(task);
           assertNotNull(task.getResourceTaskAssignments());
           assertNull(savedResourceTaskAssignment.getTaskAssignment());
           
           ResourceTaskAssignmentFixture.compare(resourceTaskAssignment, savedResourceTaskAssignment, 
               new ResourceTaskAssignmentScope());
    }

}