package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.TaskState;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test64237 extends ClientTestController {

	Resource resource;

	public void setUp() throws Exception
	{
		super.setUp();
		String userName = RpmTestProperties.getProperty("rpm.test.user.name");
		resource = (Resource)loadFromXpath("/Resource[username = '" + userName + "']", null)[0];
		assertNotNull(resource);
	}

	public void testCloseTaskByValidName() throws Exception
	{
	    closeTask(true, 1);
	}

	public void testCloseTaskByInvalidName() throws Exception
	{
	    closeTask(false, 1);
	}

	public void testCloseTaskByDate() throws Exception
	{
	    boolean dummy = false;
	    closeTask(dummy, 2);
	}

	public void testCloseTaskByState() throws Exception
	{
	    boolean dummy = false;
	    closeTask(dummy, 3);
	}

	private void closeTask(boolean isValidName, int closeOperationType) throws Exception
	{
		Proposal proposal = createAndSaveProposal();

		Task task = createAndSaveTask(proposal);

		task = closeTaskByOperationType(task, isValidName, closeOperationType);

		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setState(true);

		task = (Task)loadFromID(task, workElementScope);

		validateTask(task);
	}

	private Task closeTaskByOperationType(Task task, boolean isValidName, int closeOperationType) throws Exception
	{
	    switch (closeOperationType) 
	    {
	    	case 1 : task = closeTaskByName(task,isValidName);break;
	    	case 2 : task = closeTaskByDate(task);break;
	    	case 3 : task = closeTaskByState(task);break;
	    }
	    return task;
	}

	private Proposal createAndSaveProposal() throws Exception
	{
		Proposal proposal = new Proposal();
		proposal.setName(HelperTestUtilities.createName());

		proposal = (Proposal)save(proposal, null).getRpmObject();
		assertNotNull(proposal);

		return proposal;
	}

	private Task createAndSaveTask(Proposal parent) throws Exception
	{
		Task task = new Task();
		task.setName(HelperTestUtilities.createName());
		task.setParent(parent);

		task = (Task)save(task,null).getRpmObject();
		assertNotNull(task);

		return task;
	}

	private Task closeTaskByState(Task task) throws Exception
	{
		TaskState state = DatafieldFixture.createClosedTaskState();
		task.setState(state);
		WorkElementScope scope = new WorkElementScope();
		scope.setState(true);

		task = (Task)save(task, scope).getRpmObject();

		return task;
	}

	private Task closeTaskByName(Task task, boolean isValid) throws Exception
	{
		if(isValid)
		{
			task.setClosedByName(resource.getFullName());
		}
		else
		{
			task.setClosedByName(HelperTestUtilities.createName());
		}

		task = (Task)save(task,null).getRpmObject();

		assertNotNull(task);

		return task;
	}

	private Task closeTaskByDate(Task task) throws Exception
	{
		task.setClosedByDate(Calendar.getInstance());

		task = (Task)save(task,null).getRpmObject();

		assertNotNull(task);

		return task;
	}

	private void validateTask(Task task) throws Exception
	{
		assertNotNull(task);
		assertNotNull(task.getClosedByName());
		assertEquals(resource.getFullName(), task.getClosedByName());
		assertNotNull(task.getClosedByDate());
		assertNotNull(task.getState());
		assertEquals(DatafieldFixture.TASK_STATE_CLOSED, task.getState().getID());
	}
}
