package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

public class TestDurationBasedTaskAssignmentsValidation extends
        GenericValidation
{
    Task task;
    Project project;
    
    public void setUp() throws Exception
    {
        super.setUp();
        
        /*
         * Create a duration-based portfolio and task
         */
           project =   (Project) WorkElementFixture.createFixture(null, new Project(), new WorkElementScope(), this);

           project.setAssignmentType(AssignmentType.Duration);
           project = (Project)save(project, new WorkElementScope()).getRpmObject();
           assertNotNull(project);
           
           task = new Task();
           task.setName(HelperTestUtilities.createName());
           task.setParent(project);
           task.setPriority(new Integer(500));
           task.setContainingProject(project);
           task = (Task)save(task, new WorkElementScope()).getRpmObject();
    }
    
    /*
     * Associate a TaskAssignment to a resourceTaskAssignment which is attached
     * to a duration-based task. This is an invalid system operation and the
     * save should fail - the TaskAssignment should not be saved
     */
    public void testSaveTaskAssignmentForDurationBasedProjects()
            throws Exception
    {
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope
            .setTaskAssignment(new TaskAssignmentScope());

        ResourceTaskAssignment resourceTaskAssignment = createResourceTaskAssignmentForTask(task);
        TaskAssignment taskAssignment = TaskAssignmentFixture.createFixture(
            new TaskAssignmentScope(), this);
        taskAssignment.setParent(resourceTaskAssignment);
        resourceTaskAssignment.setTaskAssignment(taskAssignment);

        RPMExceptionMock[] expectexExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CREATE_TASKASSIGNMENT_UNDER_DURATION_BASED_PROJECTS,
                resourceTaskAssignment.getClass().getName())
        };
        verifySaveMustFail(resourceTaskAssignment, resourceTaskAssignmentScope,
            expectexExceptions);

        assertNotNull(resourceTaskAssignment);

    }
    
    private ResourceTaskAssignment createResourceTaskAssignmentForTask(WorkElement task) throws Exception
    {
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        resourceTaskAssignment.setParent(task);
        Resource resource  = TaskAssignmentFixture.createProperResource(resourceTaskAssignment,
            this);
        resource = (Resource) save(resource, new ResourceScope()).getRpmObject();
        resourceTaskAssignment.setResource(resource);
        SecurityRole securityRole = SecurityRoleFixture.createSecurityRole(task);
        securityRole = (SecurityRole) loadFromID(securityRole, new RPMObjectScope());
        resourceTaskAssignment.setSecurityRole(securityRole);
        task.setContainingProject(project);
        return resourceTaskAssignment;
    }
}
