/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container Opportunity.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestOpportunityValidation extends GenericValidation
{
    private Opportunity opportunity;

    private WorkElementScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        opportunity = new Opportunity();

        scope = new WorkElementScope();
        scope.setOpportunity(true);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);

        opportunity.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, opportunity
                .getClass().getName(), ValidationConstants.PARENT, opportunity
                .getID())
        };

        verifySaveMustFail(opportunity, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // create and set the parent 
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
                + "parent project");
        opportunity.setParent(parentProject);

        //set the editable fields
        opportunity.setBenefitBeginsMonths(new Integer(
            (int) Math.random() * 200));
        opportunity.setBenefitDurationMonths(new Integer(
            (int) Math.random() * 200));
        opportunity
            .setContactNumber(getTestString(ValidationConstants.CONTACT_NUMBER_CHARACTER_LIMIT));
        opportunity
            .setContactType(getTestString(ValidationConstants.CONTACT_TYPE_CHARACTER_LIMIT));
        opportunity
            .setCustomerNumber(getTestString(ValidationConstants.CUSTOMER_NUMBER_CHARACTER_LIMIT));
        opportunity
            .setExternalClient(getTestString(ValidationConstants.EXTERNAL_CLIENT_CHARACTER_LIMIT));
        opportunity.setForecastClosed(new GregorianCalendar(2006, 12, 12));
        opportunity
            .setOpportunityOwner(getTestString(ValidationConstants.OPPORTUNITY_OWNER_CHARACTER_LIMIT));
        opportunity
            .setOpportunityTrackingNumber(getTestString(ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_CHARACTER_LIMIT));
        opportunity.setOrderOfMagnitudeCost(new Double(Math.random() * 10000));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(
            Math.random() * 10000));
        opportunity.setPursuitFinish(new GregorianCalendar(2006, 10, 10));
        opportunity.setPursuitStart(new GregorianCalendar(2006, 10, 10));

        parentProject.setOpportunity(opportunity);

        // Attempt to save the object
        parentProject = (Project) saveMustSucceed(parentProject, Project.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + parentProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentProject, (Project) rpmObjects[0],
            scope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
            + "parent project");
        opportunity.setParent(parentProject);

        // Set values to max + 1 and verify that they are not accepted
        opportunity.setBenefitBeginsMonths(new Integer(
            ValidationConstants.BENEFIT_BEGINS_MONTHS_MAX + 1));
        opportunity.setBenefitDurationMonths(new Integer(
            ValidationConstants.BENEFIT_DURATION_MONTHS_MAX + 1));
        opportunity
            .setContactNumber(getTestString(ValidationConstants.CONTACT_NUMBER_CHARACTER_LIMIT + 1));
        opportunity
            .setContactType(getTestString(ValidationConstants.CONTACT_TYPE_CHARACTER_LIMIT + 1));
        opportunity
            .setCustomerNumber(getTestString(ValidationConstants.CUSTOMER_NUMBER_CHARACTER_LIMIT + 1));
        opportunity
            .setExternalClient(getTestString(ValidationConstants.EXTERNAL_CLIENT_CHARACTER_LIMIT + 1));
        opportunity
            .setOpportunityOwner(getTestString(ValidationConstants.OPPORTUNITY_OWNER_CHARACTER_LIMIT + 1));
        opportunity
            .setOpportunityTrackingNumber(getTestString(ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_CHARACTER_LIMIT + 1));
        opportunity.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_COST_MAX + 1));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MAX + 1));

        parentProject.setOpportunity(opportunity);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.BENEFIT_BEGINS_MONTHS_FIELD,
                    opportunity.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.BENEFIT_DURATION_MONTHS_FIELD,
                    opportunity.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.CONTACT_NUMBER_FIELD, opportunity
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.CONTACT_TYPE_FIELD, opportunity.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.CUSTOMER_NUMBER_FIELD, opportunity
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.EXTERNAL_CLIENT_FIELD, opportunity
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.OPPORTUNITY_OWNER_FIELD, opportunity
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, opportunity
                    .getClass().getName(),
                    ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_FIELD,
                    opportunity.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_COST_FIELD,
                    opportunity.getID()),
                createRPMException(
                    ErrorCodes.OUT_OF_RANGE,
                    opportunity.getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_FIELD,
                    opportunity.getID()),
        };

        verifySaveMustFail(parentProject, scope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
            + "parent project");
        opportunity.setParent(parentProject);

        // Set numeric values to min - 1 and verify that they are not accepted
        opportunity.setBenefitBeginsMonths(new Integer(
            ValidationConstants.BENEFIT_BEGINS_MONTHS_MIN - 1));
        opportunity.setBenefitDurationMonths(new Integer(
            ValidationConstants.BENEFIT_DURATION_MONTHS_MIN - 1));
        opportunity.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_COST_MIN - 1));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MIN - 1));

        parentProject.setOpportunity(opportunity);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.BENEFIT_BEGINS_MONTHS_FIELD,
                    opportunity.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.BENEFIT_DURATION_MONTHS_FIELD,
                    opportunity.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, opportunity
                    .getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_COST_FIELD,
                    opportunity.getID()),
                createRPMException(
                    ErrorCodes.OUT_OF_RANGE,
                    opportunity.getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_FIELD,
                    opportunity.getID()),
        };

        verifySaveMustFail(parentProject, scope, expectedExceptions);
    }

    public void testReadOnlyFields() throws AssertionError, Exception
    {
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
            + "parent project");
        opportunity.setParent(parentProject);

        // Set read-only fields verify that they are not accepted
        opportunity.setOrderOfMagnitudeGPSavingsPercent(new Double(9.0));

        parentProject.setOpportunity(opportunity);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, opportunity
                .getClass().getName(),
                ValidationConstants.ORDER_OF_MAGNITUDE_GP_SAVINGS_PERCENT,
                opportunity.getID())
        };

        verifySaveMustFail(parentProject, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + parentProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);
        
        Opportunity loadedOpportunity = ((Project)rpmObjects[0]).getOpportunity();
        
        // check that the values we set weren't saved in the db
        assertNotSame(new Double(9.0), loadedOpportunity.getOrderOfMagnitudeGPSavingsPercent());
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        opportunity.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, opportunity.getClass()
                .getName(), ValidationConstants.PARENT, opportunity.getID())
        };

        verifySaveMustFail(opportunity, null, expectedExceptions);

    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        WorkOrganization parentWorkOrganization = new WorkOrganization();
        parentWorkOrganization.setName(HelperTestUtilities.unitTestPrefix 
            + "parent work organization");
        opportunity.setParent(parentWorkOrganization);

        // Set doubles to max value and verify that they are accepted
        opportunity.setBenefitBeginsMonths(new Integer(
            ValidationConstants.BENEFIT_BEGINS_MONTHS_MAX));
        opportunity.setBenefitDurationMonths(new Integer(
            ValidationConstants.BENEFIT_DURATION_MONTHS_MAX));
        opportunity
            .setContactNumber(getTestString(ValidationConstants.CONTACT_NUMBER_CHARACTER_LIMIT));
        opportunity
            .setContactType(getTestString(ValidationConstants.CONTACT_TYPE_CHARACTER_LIMIT));
        opportunity
            .setCustomerNumber(getTestString(ValidationConstants.CUSTOMER_NUMBER_CHARACTER_LIMIT));
        opportunity
            .setExternalClient(getTestString(ValidationConstants.EXTERNAL_CLIENT_CHARACTER_LIMIT));
        opportunity
            .setOpportunityOwner(getTestString(ValidationConstants.OPPORTUNITY_OWNER_CHARACTER_LIMIT));
        opportunity
            .setOpportunityTrackingNumber(getTestString(ValidationConstants.OPPORTUNITY_TRACKING_NUMBER_CHARACTER_LIMIT));
        opportunity.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_COST_MAX));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MAX));

        parentWorkOrganization.setOpportunity(opportunity);

        // Attempt to save the object
        parentWorkOrganization = (WorkOrganization) saveMustSucceed(
            parentWorkOrganization, WorkOrganization.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + parentWorkOrganization.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentWorkOrganization,
            (WorkOrganization) rpmObjects[0], scope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        Proposal parentProposal = new Proposal();
        parentProposal.setName(HelperTestUtilities.unitTestPrefix 
            + "parent proposal");
        opportunity.setParent(parentProposal);

        // Set numerics to min value and verify that they are accepted
        opportunity.setBenefitBeginsMonths(new Integer(
            ValidationConstants.BENEFIT_BEGINS_MONTHS_MIN));
        opportunity.setBenefitDurationMonths(new Integer(
            ValidationConstants.BENEFIT_DURATION_MONTHS_MIN));
        opportunity.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_COST_MIN));
        opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(
            ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_MIN));

        parentProposal.setOpportunity(opportunity);

        // Attempt to save the object
        parentProposal = (Proposal) saveMustSucceed(parentProposal,
            Proposal.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[id='"
                + parentProposal.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentProposal, (Proposal) rpmObjects[0],
            scope);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
            + "parent project");
        opportunity.setParent(parentProject);

        parentProject.setOpportunity(opportunity);

        // Leave default values as null and save
        parentProject = (Project) saveMustSucceed(parentProject, Project.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + parentProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the default values are now set
        Opportunity savedOpportunity = ((Project)rpmObjects[0]).getOpportunity();
        
        assertNotNull(savedOpportunity.getBenefitBeginsMonths());
        assertNotNull(savedOpportunity.getBenefitDurationMonths());
        assertNotNull(savedOpportunity.getOrderOfMagnitudeCost());
        //        assertNotNull(savedOpportunity.getOrderOfMagnitudeGPSavingsPercent());
        //        read-only        
        assertNotNull(savedOpportunity.getOrderOfMagnitudeRevenueBenefits());
        
        // Attempt to save the object
        parentProject = (Project) saveMustSucceed(parentProject, Project.class,
            scope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Project[id='" + parentProject.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentProject, (Project) rpmObjects[0],
            scope);

    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Project parentProject = new Project();
        parentProject.setName(HelperTestUtilities.unitTestPrefix 
            + "parent project");
        opportunity.setParent(parentProject);
        
        parentProject.setOpportunity(opportunity);

        // Leave default values as null and save
        parentProject = (Project) saveMustSucceed(parentProject, Project.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Project[id='"
                + parentProject.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Project);

        // Check that the default values are now set
        Opportunity savedOpportunity = ((Project)rpmObjects[0]).getOpportunity();

        assertNotNull(savedOpportunity.getBenefitBeginsMonths());
        assertNotNull(savedOpportunity.getBenefitDurationMonths());
        assertNotNull(savedOpportunity.getOrderOfMagnitudeCost());
        //        assertNotNull(savedOpportunity.getOrderOfMagnitudeGPSavingsPercent());
        //        read-only
        assertNotNull(savedOpportunity.getOrderOfMagnitudeRevenueBenefits());

        // Update the default values to null
        savedOpportunity.setBenefitBeginsMonths(null);
        savedOpportunity.setBenefitDurationMonths(null);
        savedOpportunity.setOrderOfMagnitudeCost(null);
        //        savedOpportunity.setOrderOfMagnitudeGPSavingsPercent(null);
        //        read-only        
        savedOpportunity.setOrderOfMagnitudeRevenueBenefits(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedOpportunity.getClass().getName(),
                    ValidationConstants.BENEFIT_BEGINS_MONTHS_FIELD,
                    savedOpportunity.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedOpportunity.getClass().getName(),
                    ValidationConstants.BENEFIT_DURATION_MONTHS_FIELD,
                    savedOpportunity.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedOpportunity.getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_COST_FIELD,
                    savedOpportunity.getID()),
                createRPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedOpportunity.getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_REVENUE_BENEFITS_FIELD,
                    savedOpportunity.getID())
        };

        verifySaveMustFail(savedOpportunity, null, expectedExceptions);

    }
}
