/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container Portfolio.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Task;


public class TestPortfolioValidation extends GenericValidation
{
    private Portfolio portfolio;
    
    public void setUp() throws Exception
    {
        super.setUp();

        portfolio = new Portfolio();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
             this);

        portfolio
            .setName(getTestString(ValidationConstants.PORTFOLIO_NAME_CHARACTER_LIMIT));

        portfolio.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, portfolio
                .getClass().getName(), ValidationConstants.PARENT, portfolio
                .getID())
        };

        verifySaveMustFail(portfolio, null, expectedExceptions);
    }

    public void testInsertWithParentPorfolio() throws AssertionError, Exception
    {
        Portfolio parentPorfolio = PortfolioFixture.createSaveFixture(this);
        
        portfolio.setParent(parentPorfolio);
        portfolio
            .setName(getTestString(ValidationConstants.PORTFOLIO_NAME_CHARACTER_LIMIT));

        // Attempt to save the object
        portfolio = (Portfolio) saveMustSucceed(portfolio,
            Portfolio.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Portfolio[id='"
                + portfolio.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Portfolio);

        // Check that the loaded object is identical to the one we
        // requested to store
        PortfolioFixture.compare(portfolio, (Portfolio) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set string values with max length + 1 and verify that they are not
        // accepted
        portfolio
            .setName(getTestString(ValidationConstants.PORTFOLIO_NAME_CHARACTER_LIMIT + 1));
 
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, portfolio.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, portfolio
                    .getID())
        };

        verifySaveMustFail(portfolio, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        portfolio.setName("");
 
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MIN_LENGTH, portfolio.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, portfolio
                    .getID())
        };

        verifySaveMustFail(portfolio, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, portfolio.getClass()
                .getName(), ValidationConstants.NAME_FIELD, portfolio.getID())
        };

        verifySaveMustFail(portfolio, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set the mandatory fields
        portfolio
            .setName(getTestString(ValidationConstants.PORTFOLIO_NAME_CHARACTER_LIMIT));

        portfolio.setParent(null);

        // Attempt to save the object
        portfolio = (Portfolio) saveMustSucceed(portfolio,
            Portfolio.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Portfolio[id='"
                + portfolio.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Portfolio);

        // Check that the loaded object is identical to the one we
        // requested to store
        PortfolioFixture.compare(portfolio, (Portfolio) rpmObjects[0], null);

    }


}
