/*
 * Created on Feb 20, 2006
 * @author mmatwiy 
 * Validation tests for container ProfileSkillAssignment
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.types.ResourceType;

public class TestProfileSkillAssignmentValidation extends GenericValidation
{
    private ProfileSkillAssignment assignment;

    public void setUp() throws Exception
    {
        super.setUp();

        assignment = new ProfileSkillAssignment();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        assignment.setParent(new Task());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, assignment
                .getClass().getName(), ValidationConstants.PARENT, assignment
                .getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }

    public void testInsertWithParentProfile() throws AssertionError, Exception
    {
        ProfileScope profileScope = new ProfileScope();
        profileScope.setProfileSkillAssignments(true);
        profileScope.setProfileCompetencyAssignments(true);
        
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope.setProfile(profileScope);
        resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());
        
        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(resourceTaskAssignmentScope, false, this);

        assignment.setParent(resourceTaskAssignment.getProfile());

        // Attempt to save the object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class,
            resourceTaskAssignmentScope);

        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTaskAssignment.getID() + "']",
            resourceTaskAssignmentScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        assertNotNull(((ResourceTaskAssignment) rpmObjects[0]).getProfile());
        assertNotNull(((ResourceTaskAssignment) rpmObjects[0])
            .getProfile().getProfileSkillAssignments());
        assertNotNull(((ResourceTaskAssignment) rpmObjects[0])
            .getProfile().getProfileSkillAssignments()[0]);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTaskAssignment,
            (ResourceTaskAssignment)rpmObjects[0], resourceTaskAssignmentScope);
    }


    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, assignment.getClass()
                .getName(), ValidationConstants.PARENT, assignment.getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }
    
    public void testInsertWithInvalidSkills() throws AssertionError, Exception
    {
    	ProfileScope profileScope = new ProfileScope();
    	profileScope.setProfileCompetencyAssignments(true);
        
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope.setProfile(profileScope);
        resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());

        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
				.createFixture(resourceTaskAssignmentScope, false, this);
        resourceTaskAssignment.setResourceType(ResourceType.Profile);

        ProfileSkillAssignment psa[] = new ProfileSkillAssignment[1];
		ResourceAttributeProficiencyLevel raProficiencyLevel = resourceTaskAssignment.getProfile().getProfileCompetencyAssignments()[0].getProficiencyLevel();
		psa[0] = new ProfileSkillAssignment();
		psa[0].setProficiencyLevel(raProficiencyLevel);
		resourceTaskAssignment.getProfile().setProfileSkillAssignments(psa);
		
        assignment.setParent(resourceTaskAssignment.getProfile());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.resource.constants.ErrorCodes.INVALID_SKILL_PROFICIENCY_LEVEL, assignment.getClass()
                .getName(), ValidationConstants.PROFILE_PROFICIENCY_LEVEL_FIELD, assignment.getID())
        };        
        
        // Attempt to save the object
        verifySaveMustFail(resourceTaskAssignment, resourceTaskAssignmentScope, expectedExceptions);
    }
}
