/*
 * Created on 31-May-2006
 * @author mmatwiy
 * Validation tests for the ResourceTaskAssignment copyAssignmentProposedToPlan method.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTaskAssignmentCopyProposedToPlanValidation extends
        GenericValidation
{

    ResourceTaskAssignment rta;

    ResourceTaskAssignmentScope rtaScope;

    ResourceScope resourceScope;

    public void setUp() throws Exception
    {
        super.setUp();

        SecurityRole securityRole = new SecurityRole();
        securityRole.setID("SCPROJECTMANAGER");

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        workElementScope.setParent(new WorkElementScope());

        resourceScope = new ResourceScope();
        resourceScope.setParent(new PoolScope());
        resourceScope.setCalendar(new CalendarScope());

        Task parentTask = createTask(workElementScope);

        // create RTA with mandatory fields
        rta = new ResourceTaskAssignment();
        rta.setSecurityRole(securityRole);
        rta.setParent(parentTask);

        rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setParent(workElementScope);
        rtaScope.setResource(resourceScope);
    }

    private ResourceTaskAssignment copyProposedToPlan() throws Exception
    {
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);

        TaskAssignment taskAssignment = new TaskAssignment();
        rta.setTaskAssignment(taskAssignment);

        rta.setResource(createResource(resourceScope));
        
        SecurityRole role = new SecurityRole();
        role.setID("SCTASKRESPONSIBLE");
        rta.setSecurityRole(role);
        
        save(rta, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + rta.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        rta = (ResourceTaskAssignment) rpmObjects[0];
        assertNotNull(rta.getTaskAssignment().getID());

        ResourceTaskAssignment reloaded = (ResourceTaskAssignment) getApplicationInterface()
            .copyAssignmentProposedToPlan(getSessionID(), rta, rtaScope)
            .getRpmObject();

        // now the plannedDurationHours should exist and
        // the actualWork, %work, and %complete fields should be editable
        assertNotNull(reloaded.getTaskAssignment());
        assertNotNull(reloaded.getTaskAssignment().getPlannedDurationHours());

        return reloaded;
    }

    private Resource createResource(ResourceScope resourceScope)
            throws Exception
    {
        Resource resource = new Resource();
        resource.setFullName("RTA Tester");
        resource.setParent(PoolFixture.createSaveFixture(this));
        resource.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(this));
        save(resource, resourceScope);
        return resource;
    }

    private Task createTask(WorkElementScope workElementScope) throws Exception
    {
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal = (Proposal) save(proposal, workElementScope).getRpmObject();

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(proposal);

        task = (Task) save(task, workElementScope).getRpmObject();
        return task;
    }

    public void testErrorMissingIDs() throws Exception
    {
        // attempt to copy proposed to plan without saving the RTA, 
        // the resource, and the TaskAssignment
        SingleResult result = getApplicationInterface()
            .copyAssignmentProposedToPlan(getSessionID(), rta, rtaScope);

        // 3 errors: no ID
        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, rta.getClass().getName()),
                createRPMException(ErrorCodes.NO_ID, rta.getClass().getName(),
                    ValidationConstants.TASK_ASSIGNMENT_FIELD, rta.getID()),
                createRPMException(ErrorCodes.NO_ID, rta.getClass().getName(),
                    ValidationConstants.RESOURCE_FIELD, rta.getID()),
        };
        
        verifyFailure(rta, expectedExceptions, null, result);
    }

    public void testErrorAlreadyCopied() throws Exception
    {
        // copyProposedToPlan twice on the same taskAssignment will lead to an error
        // because the plannedDurationHours field will be populated
        ResourceTaskAssignment reloaded = copyProposedToPlan();

        SingleResult result = getApplicationInterface()
            .copyAssignmentProposedToPlan(getSessionID(), reloaded, rtaScope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.RTA_COPY_PROPOSED_TO_PLAN_INVALID,
                reloaded.getClass().getName())
        };

        verifyFailure(reloaded, expectedExceptions, null, result);
    }

}