/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container TaskAssignment.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTaskAssignmentValidation extends GenericValidation
{
    private TaskAssignment taskAssignment = null;

    private ResourceTaskAssignment resourceTaskAssignment = null;

    private ResourceTaskAssignmentScope rtaScope = null;

    private TaskAssignmentScope taScope = null;

    private Task parentTask;

    private WorkElementScope workElementScope;

    private SecurityRole securityRole = null;

    public void setUp() throws Exception
    {
        super.setUp();

        // create the grandparent
        parentTask = (Task) TaskAssignmentFixture.createProperWorkElement(this);

        // create the grandparent, parent & TaskAssignment scope
        rtaScope = ResourceTaskAssignmentFixture.createScope(true, true, false,
            true, false);
        taScope = TaskAssignmentFixture.createScope(true, false, false, false);
        rtaScope.setTaskAssignment(taScope);

        workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(rtaScope);
        workElementScope.setScheduleDates(true);

        // create a resource to attach to the parent
        Resource resource = TaskAssignmentFixture.createProperResource(
            parentTask, this);

        // create the parent
        resourceTaskAssignment = new ResourceTaskAssignment();
        resourceTaskAssignment.setParent(parentTask);
        resourceTaskAssignment.setResource(resource);

        securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");
        resourceTaskAssignment.setSecurityRole(securityRole);

        ResourceTaskAssignment[] resourceTaskAssignments =
        {
            resourceTaskAssignment
        };
        parentTask.setResourceTaskAssignments(resourceTaskAssignments);

        // create the TaskAssignment container
        taskAssignment = new TaskAssignment();
        taskAssignment.setParent(resourceTaskAssignment);

        resourceTaskAssignment.setTaskAssignment(taskAssignment);

        taskAssignment.setAssignmentStartDate(new GregorianCalendar());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testReadOnlyFields() throws AssertionError, Exception
    {
        GregorianCalendar calendar = new GregorianCalendar();

        // all of these fields are read-only
        taskAssignment.setActualFinishDate(calendar);
        taskAssignment.setActualStartDate(calendar);
        taskAssignment.setAssignmentFinishDate(calendar);
        taskAssignment.setBaselineFinishDate(calendar);
        taskAssignment.setBaselineStartDate(calendar);
        taskAssignment.setBaselineWork(new Double(8.0));
        taskAssignment.setCalculatedLeveledFinishDate(calendar);
        taskAssignment.setCalculatedLeveledStartDate(calendar);
        taskAssignment.setCalculatedLeveledWork(new Double(8.0));
        taskAssignment.setExpectedFinishDate(calendar);
        taskAssignment.setExpectedStartDate(calendar);
        taskAssignment.setExpectedWork(new Double(8.0));
        taskAssignment.setForecastFinishDate(calendar);
        taskAssignment.setForecastStartDate(calendar);
        taskAssignment.setPercentageDurationComplete(new Integer(8));
        taskAssignment.setRemainingHours(new Double(8.0));
        taskAssignment.setLastScheduleWork(new Double(8.0));
        taskAssignment.setComplete(new Boolean(true));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.ACTUAL_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.ACTUAL_START_DATE_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.ASSIGNMENT_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.BASELINE_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.BASELINE_START_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.BASELINE_WORK_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.CALCULATED_LEVELED_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.CALCULATED_LEVELED_START_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.CALCULATED_LEVELED_WORK_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.EXPECTED_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.EXPECTED_START_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.EXPECTED_WORK_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.FORECAST_FINISH_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.FORECAST_START_DATE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_DURATION_COMPLETE_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.REMAINING_HOURS_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.LAST_SCHEDULE_WORK_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.COMPLETE_FIELD,
                    taskAssignment.getID())
        };

        verifySaveMustFail(resourceTaskAssignment, rtaScope, null,
            expectedWarnings);
    }

    public void testConditionalReadOnly() throws AssertionError, Exception
    {
        taskAssignment.setActualWork(new Double(
            ValidationConstants.ACTUAL_WORK_MAX));
        taskAssignment.setEETCHours(new Double(2));
        taskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MAX));
        taskAssignment.setPercentageWork(new Double(
            ValidationConstants.PERCENTAGE_WORK_MAX));

        // actualWork read-only if all of this task's assigned resources
        // have a null actualStartDate
        // actualWork, percentageComplete, and percentageWork cannot
        // be modified and saved at the same time 
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    taskAssignment.getClass().getName(),
                    ValidationConstants.ACTUAL_WORK_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD + " or "
                            + ValidationConstants.PLANNED_DURATION_HOURS_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    taskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_WORK_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD + " or "
                            + ValidationConstants.PLANNED_DURATION_HOURS_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    taskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    ValidationConstants.SCHEDULE_START_FIELD, taskAssignment
                        .getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                    taskAssignment.getClass().getName())
        };
        
        verifySaveMustFail(resourceTaskAssignment, rtaScope, null, expectedWarnings);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);

        taskAssignment.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, taskAssignment
                .getClass().getName(), ValidationConstants.PARENT,
                taskAssignment.getID())
        };

        verifySaveMustFail(taskAssignment, taScope, expectedExceptions);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set values to max + 1 and verify that they are not accepted
        taskAssignment.setCostPerHour(new Double(
            ValidationConstants.COST_PER_HOUR_MAX + 1));
        taskAssignment.setDaysOfDelay(new Integer(
            ValidationConstants.DAYS_OF_DELAY_MAX + 1));
        taskAssignment.setPercentageRateAdjustment(new Integer(
            ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MAX + 1));
        taskAssignment.setSpecialCostPerHour(new Double(
            ValidationConstants.SPECIAL_COST_PER_HOUR_MAX + 1));
        taskAssignment.setSellingRatePerHour(new Double(
            ValidationConstants.SELLING_RATE_PER_HOUR_MAX + 1));
        taskAssignment.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MAX + 1));
        taskAssignment.setUnitsPercentage(new Double(
            ValidationConstants.UNITS_PERCENTAGE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.COST_PER_HOUR_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.DAYS_OF_DELAY_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_COST_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SELLING_RATE_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.UNITS_PERCENTAGE_FIELD, taskAssignment
                        .getID())
        };

        verifySaveMustFail(resourceTaskAssignment, rtaScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set numeric values to min - 1 and verify that they are not accepted
        taskAssignment.setCostPerHour(new Double(
            ValidationConstants.COST_PER_HOUR_MIN - 1));
        taskAssignment.setDaysOfDelay(new Integer(
            ValidationConstants.DAYS_OF_DELAY_MIN - 1));
        taskAssignment.setPercentageRateAdjustment(new Integer(
            ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MIN - 1));
        taskAssignment.setSpecialCostPerHour(new Double(
            ValidationConstants.SPECIAL_COST_PER_HOUR_MIN - 1));
        taskAssignment.setSellingRatePerHour(new Double(
            ValidationConstants.SELLING_RATE_PER_HOUR_MIN - 1));
        taskAssignment.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MIN - 1));
        taskAssignment.setUnitsPercentage(new Double(
            ValidationConstants.UNITS_PERCENTAGE_MIN - 1));
        taskAssignment.setEETCHours(new Double(
            ValidationConstants.EETC_HOURS_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.COST_PER_HOUR_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.DAYS_OF_DELAY_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_COST_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SELLING_RATE_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_FIELD,
                    taskAssignment.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.UNITS_PERCENTAGE_FIELD, taskAssignment
                        .getID()),
                createRPMException(ErrorCodes.GREATER_THAN_OR_EQUAL_VALUE, taskAssignment
                    .getClass().getName(),
                    ValidationConstants.EETC_HOURS_FIELD, taskAssignment
                        .getID())
        };

        verifySaveMustFail(resourceTaskAssignment, rtaScope, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        taskAssignment.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, taskAssignment.getClass()
                .getName(), ValidationConstants.PARENT, taskAssignment.getID())
        };

        verifySaveMustFail(taskAssignment, taScope, expectedExceptions);

    }

    public void testInsertWithParentResourceTaskAssignment()
            throws AssertionError, Exception
    {
        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[id='"
                    + resourceTaskAssignment.getID() + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set doubles to max value and verify that they are accepted
        taskAssignment.setCostPerHour(new Double(
            ValidationConstants.COST_PER_HOUR_MAX));
        taskAssignment.setDaysOfDelay(new Integer(
            ValidationConstants.DAYS_OF_DELAY_MAX));
        taskAssignment.setPercentageRateAdjustment(new Integer(
            ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MAX));
        taskAssignment.setSpecialCostPerHour(new Double(
            ValidationConstants.SPECIAL_COST_PER_HOUR_MAX));
        taskAssignment.setSellingRatePerHour(new Double(
            ValidationConstants.SELLING_RATE_PER_HOUR_MAX));
        taskAssignment.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MAX));
        taskAssignment.setUnitsPercentage(new Double(
            ValidationConstants.UNITS_PERCENTAGE_MAX));

        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[id='"
                    + resourceTaskAssignment.getID() + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);
        TaskAssignmentFixture.compare(taskAssignment, resourceTaskAssignment
            .getTaskAssignment(), null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set numerics to min value and verify that they are accepted
        taskAssignment.setCostPerHour(new Double(
            ValidationConstants.COST_PER_HOUR_MIN));
        taskAssignment.setDaysOfDelay(new Integer(
            ValidationConstants.DAYS_OF_DELAY_MIN));
        taskAssignment.setPercentageRateAdjustment(new Integer(
            ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MIN));
        taskAssignment.setSpecialCostPerHour(new Double(
            ValidationConstants.SPECIAL_COST_PER_HOUR_MIN));
        taskAssignment.setSellingRatePerHour(new Double(
            ValidationConstants.SELLING_RATE_PER_HOUR_MIN));
        taskAssignment.setSpecialSellingRatePerHour(new Double(
            ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MIN));
        taskAssignment.setUnitsPercentage(new Double(
            ValidationConstants.UNITS_PERCENTAGE_MIN));
        
        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[TaskAssignment[id='"
                    + taskAssignment.getID() + "']]]", workElementScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);
        
        RPMObject[] rpmObjects = loadFromXpath(
            "/TaskAssignment[id='" + taskAssignment.getID() + "']", taScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment loadedTaskAssignment = (TaskAssignment) rpmObjects[0];
        assertNotNull(loadedTaskAssignment);

        // Check that the loaded TaskAssignment is identical to the one we
        // requested to store
        TaskAssignmentFixture.compare(taskAssignment, loadedTaskAssignment,
            taScope);
    }

    public void testInvalidArgsAcceptTaskAssignmentChanges()
            throws AssertionError, Exception
    {
        taskAssignment = null;

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.ERROR_TASK_ASSIGNMENT_APPROVE_ARGS,
                    TaskAssignment.class.getName(), null, null),
                createRPMException(ErrorCodes.MANDATORY, TaskAssignment.class
                    .getName(), null, null)
        };

        // pass invalid arguments to the call
        SaveResult saveResult = getApplicationInterface()
            .acceptTaskAssignmentChanges(sessionid, taskAssignment, taScope,
                false, false);

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(taskAssignment.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = verifyExceptions(expectedExceptions, saveResult
            .getErrors());

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

    public void testInsertTaskAssignmentFullScope() throws AssertionError,
            Exception
    {
        taskAssignment = TaskAssignmentFixture.createFixture(ResourceFixture
            .createSaveFixture(this), taScope, this);

        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[id='"
                    + resourceTaskAssignment.getID() + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);

    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        // Leave default values as null and save
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/TaskAssignment[id='" + taskAssignment.getID() + "']", taScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment loadedTaskAssignment = (TaskAssignment) rpmObjects[0];
        assertNotNull(loadedTaskAssignment);

        // Check that the default values are now set
        assertNotNull(loadedTaskAssignment.getActualWork());
        assertNotNull(loadedTaskAssignment.getAssignmentFinishDate());
        assertNotNull(loadedTaskAssignment.getAssignmentStartDate());
        assertNotNull(loadedTaskAssignment.getBillable());
        assertNotNull(loadedTaskAssignment.getCostPerHour());
        assertNotNull(loadedTaskAssignment.getDaysOfDelay());
        assertNotNull(loadedTaskAssignment.getDefaultLabor());
        assertNotNull(loadedTaskAssignment.getEETCHours());
        assertNotNull(loadedTaskAssignment.getExpectedFinishDate());
        assertNotNull(loadedTaskAssignment.getExpectedStartDate());
        assertNotNull(loadedTaskAssignment.getExpectedWork());
        assertNotNull(loadedTaskAssignment.getPercentageRateAdjustment());
        assertNotNull(loadedTaskAssignment.getRemainingHours());
        assertNotNull(loadedTaskAssignment.getSellingRatePerHour());
        assertNotNull(loadedTaskAssignment.getSpecialCostPerHour());
        assertNotNull(loadedTaskAssignment.getSpecialSellingRatePerHour());
        assertNotNull(loadedTaskAssignment.getUnitsPercentage());
        assertNotNull(loadedTaskAssignment.getLastScheduleWork());

        // Save again to validate the default values
        taskAssignment = (TaskAssignment) saveMustSucceed(loadedTaskAssignment,
            TaskAssignment.class, taScope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/TaskAssignment[id='"
                + taskAssignment.getID() + "']", taScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment reLoadedTaskAssignment = (TaskAssignment) rpmObjects[0];

        assertNotNull(reLoadedTaskAssignment);

    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        parentTask = (Task) saveMustSucceed(parentTask, WorkElement.class,
            workElementScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/TaskAssignment[id='" + taskAssignment.getID() + "']", taScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment loadedTaskAssignment = (TaskAssignment) rpmObjects[0];
        assertNotNull(loadedTaskAssignment);

        // Check that the default values are now set
        assertNotNull(loadedTaskAssignment.getAssignmentFinishDate());
        assertNotNull(loadedTaskAssignment.getAssignmentStartDate());
        assertNotNull(loadedTaskAssignment.getBillable());
        assertNotNull(loadedTaskAssignment.getCostPerHour());
        assertNotNull(loadedTaskAssignment.getDaysOfDelay());
        assertNotNull(loadedTaskAssignment.getDefaultLabor());
        assertNotNull(loadedTaskAssignment.getExpectedFinishDate());
        assertNotNull(loadedTaskAssignment.getExpectedStartDate());
        assertNotNull(loadedTaskAssignment.getExpectedWork());
        assertNotNull(loadedTaskAssignment.getLastScheduleWork());
        assertNotNull(loadedTaskAssignment.getPercentageRateAdjustment());
        assertNotNull(loadedTaskAssignment.getRemainingHours());
        assertNotNull(loadedTaskAssignment.getSellingRatePerHour());
        assertNotNull(loadedTaskAssignment.getSpecialCostPerHour());
        assertNotNull(loadedTaskAssignment.getSpecialSellingRatePerHour());
        assertNotNull(loadedTaskAssignment.getUnitsPercentage());

        // Update the editable default values to null
        //loadedTaskAssignment.setAssignmentFinishDate(null); // read-only
        loadedTaskAssignment.setAssignmentStartDate(null);
        loadedTaskAssignment.setBillable(null);
        loadedTaskAssignment.setCostPerHour(null);
        loadedTaskAssignment.setDaysOfDelay(null);
        loadedTaskAssignment.setDefaultLabor(null);
        //loadedTaskAssignment.setExpectedFinishDate(null); // read-only
        //loadedTaskAssignment.setExpectedStartDate(null); // read-only
        //loadedTaskAssignment.setExpectedWork(null); // read-only
        //loadedTaskAssignment.setLastScheduleWork(null); //read-only
        loadedTaskAssignment.setPercentageRateAdjustment(null);
        //loadedTaskAssignment.setRemainingHours(null); // read-only
        loadedTaskAssignment.setSellingRatePerHour(null);
        loadedTaskAssignment.setSpecialCostPerHour(null);
        loadedTaskAssignment.setSpecialSellingRatePerHour(null);
        loadedTaskAssignment.setUnitsPercentage(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.ASSIGNMENT_START_DATE_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.BILLABLE_FIELD, loadedTaskAssignment
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.COST_PER_HOUR_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.DAYS_OF_DELAY_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.DEFAULT_LABOR_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.SELLING_RATE_PER_HOUR_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.SPECIAL_COST_PER_HOUR_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_FIELD,
                    loadedTaskAssignment.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    loadedTaskAssignment.getClass().getName(),
                    ValidationConstants.UNITS_PERCENTAGE_FIELD,
                    loadedTaskAssignment.getID())
        };

        verifySaveMustFail(loadedTaskAssignment, taScope, expectedExceptions);
    }
    

    public void testFailureDelete() throws AssertionError, Exception
    {
        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);
        
        RPMObject[] rpmObjects = loadFromXpath(
            "/TaskAssignment[id='" + taskAssignment.getID() + "']", taScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TaskAssignment);

        TaskAssignment loadedTaskAssignment = (TaskAssignment) rpmObjects[0];
        assertNotNull(loadedTaskAssignment);

        // can't delete a TaskAssignment 
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_DELETE,
                loadedTaskAssignment.getClass().getName())
        };

        verifyDeleteMustFail(loadedTaskAssignment, expectedExceptions, null);
    }
    
    public void testSuccessValidationMaxOnSpecialFields()
            throws AssertionError, Exception
    {
        copyProposedToPlan();
        
        taskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MAX));

        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[ResourceTaskAssignment[id='"
                + resourceTaskAssignment.getID() + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);
        TaskAssignmentFixture.compare(taskAssignment, resourceTaskAssignment
            .getTaskAssignment(), null);
        TaskAssignmentFixture.comparePercentageComplete(taskAssignment,
            resourceTaskAssignment.getTaskAssignment());
    }
    
    public void testSuccessValidationMinOnSpecialFields()
            throws AssertionError, Exception
    {
        copyProposedToPlan();

        taskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MIN));

        // Attempt to save the TaskAssignment object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[id='"
                    + resourceTaskAssignment.getID() + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(parentTask, (WorkElement) rpmObjects[0],
            null);
        TaskAssignmentFixture.compare(taskAssignment, resourceTaskAssignment
            .getTaskAssignment(), null);
        TaskAssignmentFixture.comparePercentageComplete(taskAssignment,
            resourceTaskAssignment.getTaskAssignment());
    }

    public void testFailureValidationMinOnSpecialFields()
            throws AssertionError, Exception
    {
        copyProposedToPlan();

        taskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                .getClass().getName(),
                ValidationConstants.PERCENTAGE_COMPLETE_FIELD, taskAssignment
                    .getID())
        };

        verifySaveMustFail(resourceTaskAssignment, rtaScope, expectedExceptions);
    }

    public void testFailureValidationMaxOnSpecialFields()
            throws AssertionError, Exception
    {
        copyProposedToPlan();

        taskAssignment.setPercentageComplete(new Double(
            ValidationConstants.PERCENTAGE_COMPLETE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, taskAssignment
                .getClass().getName(),
                ValidationConstants.PERCENTAGE_COMPLETE_FIELD, taskAssignment
                    .getID())
        };

        verifySaveMustFail(resourceTaskAssignment, rtaScope, expectedExceptions);
    }

    private void copyProposedToPlan() throws Exception
    {
        // Save first in order to copy proposed to plan
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class, rtaScope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/WorkElement[ResourceTaskAssignment[id='"
                    + resourceTaskAssignment.getID() + "']]", workElementScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        assertTrue(rpmObjects[0] instanceof Task);
        parentTask = (Task) rpmObjects[0];

        SingleResult result = this.getApplicationInterface()
            .copyProposedToPlan(this.getSessionID(), parentTask,
                workElementScope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Task);
        parentTask = (Task) result.getRpmObject();

        // load resource assignment back in order to change task assignment
        // values
        RPMObject rpmObject = loadFromID(resourceTaskAssignment, rtaScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ResourceTaskAssignment);
        resourceTaskAssignment = (ResourceTaskAssignment) rpmObject;

        // reset taskAssignment
        assertNotNull(resourceTaskAssignment.getTaskAssignment());
        taskAssignment = resourceTaskAssignment.getTaskAssignment();
    }
}