/*
 * Created on Dec 2, 2005 @author mmatwiy Validation tests for container Task.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTaskValidation extends AbstractTestSimpleNodeValidation
{
    private Task task;

    private WorkElementScope scope;

    private ProfileScope profileScope;

    private SecurityRole securityRole;

    public void setUp() throws Exception
    {
        super.setUp();

        task = new Task();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();

        profileScope = new ProfileScope();
        profileScope.setProfileCompetencyAssignments(true);
        rtaScope.setProfile(profileScope);

        scope = new WorkElementScope();
        scope.setScheduleDates(true);
        scope.setResourceTaskAssignments(rtaScope);

        securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testReadOnlyActualDateFields() throws AssertionError, Exception
    {
        super.testReadOnlyActualDateFields(task);
    }

    public void testReadOnlyForecastDateFields() throws AssertionError,
            Exception
    {
        super.testReadOnlyForecastDateFields(task);
    }

    public void testReadOnlyPlanDateFields() throws AssertionError, Exception
    {
        super.testReadOnlyPlanDateFields(task);
    }

    public void testReadOnlyProposedDateFields() throws AssertionError,
            Exception
    {
        super.testReadOnlyProposedDateFields(task);
    }

    public void testConditionalReadOnlyAssignedWorkExists()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyAssignedWorkExists(task);
    }

    public void testSuccessConditionalReadOnlyAssignedParticipantsExist()
            throws AssertionError, Exception
    {
        super.testSuccessConditionalReadOnlyAssignedParticipantsExist(task);
    }

    public void testConditionalReadOnlyActualStart() throws AssertionError,
            Exception
    {
        super.testConditionalReadOnlyActualStart(task);
    }

    public void testConditionalReadOnlyActualDurationInMinutes()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyActualDurationInMinutes(task);
    }

    public void testConditionalReadOnlyDurationsSame() throws AssertionError,
            Exception
    {
        super.testConditionalReadOnlyDurationsSame(task);
    }

    public void testConditionalReadOnlyForecastDurationInMinutes()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyForecastDurationInMinutes(task);
    }

    public void testConditionalReadOnlyPlanStartAndDuration()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyPlanStartAndDuration(task);
    }

    public void testConditionalReadOnlyPlanDurationInMinutes()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyPlanDurationInMinutes(task);
    }

    public void testConditionalReadOnlyProposedStartAndDuration()
            throws AssertionError, Exception
    {
        super.testConditionalReadOnlyProposedStartAndDuration(task);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        super.testInsertWithInvalidParent(task);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        super.testInsertSuccess(task);
    }

    public void testInsertScheduleDatesSuccess() throws AssertionError,
            Exception
    {
        super.testInsertScheduleDatesSuccess(task);
    }

    public void testFailureClearProposedStart() throws AssertionError,
            Exception
    {
        super.testFailureClearProposedStart(task);
    }

    public void testFailureClearPlanStart() throws AssertionError, Exception
    {
        super.testFailureClearPlanStart(task);
    }

    public void testFailureCompareDurations() throws AssertionError, Exception
    {
        super.testFailureCompareDurations(task);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        super.testFailureInvalidEstimatedFinishDate(task);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        super.testFailureValidationMax(task);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        super.testFailureValidationMin(task);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        super.testInsertWithoutMandatoryFields(task);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        super.testInsertWithoutParent(task);

    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        super.testInsertWithParentWorkOrganization(task);
    }

    public void testInsertWithParentWorkProduct() throws AssertionError,
            Exception
    {
        super.testInsertWithParentWorkProduct(task);
    }

    public void testInsertWithParentSummaryTask() throws AssertionError,
            Exception
    {
        super.testInsertWithParentSummaryTask(task);
    }

    public void testInsertWithParentDeliverable() throws AssertionError,
            Exception
    {
        super.testInsertWithParentDeliverable(task);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        super.testInsertWithParentProposal(task);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        super.testInsertWithParentProject(task);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        super.testSuccessValidationMax(task);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        super.testSuccessValidationMin(task);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        super.testFailureInvalidState(task);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        super.testSuccessValidState(task);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        super.testFailureInvalidTimeCodes(task);
    }
    
    public void testFailureEffortDrivenWithDefaultTask() throws AssertionError,
            Exception
    {
        super.testFailureEffortDrivenWithDefaultTask(task);
    }
    
    public void testReadOnlyFields() throws Exception
    {
        task
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        task.setParent(WorkElementFixture.retrieveStaticFixture(new Project(),
            this));
        
        super.testReadOnlyFields(task);
    }

    public void testConditionalReadOnlyPercentDurationCompleted()
            throws AssertionError, Exception
    {

        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // percentDurationCompleted read-only when planDate.startDate is null

        // save the workElement so that the above fields aren't set as modified
        task = (Task) save(task, null).getRpmObject();

        task.setPercentageDurationCompleted(new Double(10));

        // percentDurationCompleted read-only if work has been assigned
        // (resourceTaskAssignments exist and include a taskAssignment)
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
        resourceTaskAssignment.setTaskAssignment(new TaskAssignment());
        ResourceTaskAssignment[] resourceTaskAssignments =
        {
            resourceTaskAssignment
        };
        task.setResourceTaskAssignments(resourceTaskAssignments);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    task.getClass().getName(),
                    ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, task
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING, task
                    .getClass().getName(),
                    ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD, task
                        .getID())
        };

        verifySaveMustFail(task, null, null, expectedWarnings);

        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);
        task = (Task) rpmObjects[0];

        Task loadedTask = (Task) rpmObjects[0];

        // assert that the value we set was not saved in the db
        assertNotSame(new Double(10), loadedTask.getPercentageDurationCompleted());

    }

    public void testConditionalReadOnlyPercentDurationCompleted2()
            throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set required dates
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(1995));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(1995));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(1500));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);
        task.setActualDate(actualDate);

        // save the workElement so that the above fields aren't set as modified
        task = (Task) save(task, scope).getRpmObject();

        // percentDurationCompleted read-only when
        // actualDate.durationInMinutes == planDate.durationInMinutes
        task.getActualDate().setDurationInMinutes(new Integer(1995));

        task.setPercentageDurationCompleted(new Double(
            ValidationConstants.PERCENT_DURATION_COMPLETED_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    ErrorCodes.READ_ONLY_CONDITIONAL_VALUES_SAME, task
                        .getClass().getName(),
                    ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    task.getID())
        };

        verifySaveMustFail(task, scope, null, expectedWarnings);
    }

    public void testFailurePercentDurationCompletedMax() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set required dates
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(2000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(2000));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);

        task.setPercentageDurationCompleted(new Double(
            ValidationConstants.PERCENT_DURATION_COMPLETED_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, task.getClass()
                .getName(),
                ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, task
                    .getID())
        };

        verifySaveMustFail(task, null, expectedExceptions);
    }

    public void testFailurePercentDurationCompletedMin() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(2000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(2000));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);

        // Set numerics to min value -1 and verify that they are not accepted
        task.setPercentageDurationCompleted(new Double(
            ValidationConstants.PERCENT_DURATION_COMPLETED_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, task.getClass()
                .getName(),
                ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, task
                    .getID()),
        };

        verifySaveMustFail(task, null, expectedExceptions);
    }

    public void testSuccessPercentDurationCompletedMax() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set required dates
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar(2005, 12, 10));
        proposedDate.setDurationInMinutes(new Integer(2000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2005, 12, 10));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);

        // Set field values with maxLength and verify that they are accepted
        task.setPercentageDurationCompleted(new Double(
            ValidationConstants.PERCENT_DURATION_COMPLETED_MAX));

        // Attempt to save the Task object
        task = (Task) saveMustSucceed(task, Task.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the loaded Task is identical to the one we
        // requested to store
        WorkElementFixture.compare(task, (Task) rpmObjects[0], null);
    }

    public void testSuccessPercentDurationCompletedMin() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set required dates
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar(2005, 12, 10));
        proposedDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2005, 12, 10));

        task.setProposedDate(proposedDate);
        task.setPlanDate(planDate);

        // Set integers to min value and verify that they are accepted
        task.setPercentageDurationCompleted(new Double(
            ValidationConstants.PERCENT_DURATION_COMPLETED_MIN));

        // Attempt to save the Task object
        task = (Task) saveMustSucceed(task, Task.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the loaded Task is identical to the one we
        // requested to store
        WorkElementFixture.compare(task, (Task) rpmObjects[0], null);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        WorkElementScope scope = new WorkElementScope();
        scope.setAssetCode(true);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        task = (Task) saveMustSucceed(task, Task.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the default values are now set
        Task savedTask = (Task) rpmObjects[0];
        assertNotNull(savedTask.getAssetCode());
        assertNotNull(savedTask.getEstimatedStartDate());
        assertNotNull(savedTask.getPriority());

        // Save again to validate the default values
        task = (Task) saveMustSucceed(task, Task.class, scope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Task[id='" + task.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the loaded Task is identical to the one we
        // requested to store
        WorkElementFixture.compare(task, (Task) rpmObjects[0], scope);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        WorkElementScope scope = new WorkElementScope();
        scope.setAssetCode(true);

        // Set the mandatory fields
        task.setParent(parentProject);
        task
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        task = (Task) saveMustSucceed(task, Task.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the default values are now set
        Task savedTask = (Task) rpmObjects[0];
        assertNotNull(savedTask.getAssetCode());
        assertNotNull(savedTask.getEstimatedStartDate());
        assertNotNull(savedTask.getPriority());

        // Update the default values to null
        savedTask.setAssetCode(null);
        savedTask.setEstimatedStartDate(null);
        savedTask.setPriority(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedTask
                    .getClass().getName(),
                    ValidationConstants.ASSET_CODE_FIELD, savedTask.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedTask
                    .getClass().getName(),
                    ValidationConstants.ESTIMATED_START_DATE_FIELD, savedTask
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedTask
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    savedTask.getID()),
        };

        verifySaveMustFail(savedTask, scope, expectedExceptions);

    }

}