/*
 * Created on 1-Jun-2006
 * @author mmatwiy
 * Description
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTransferUnderScopeValidation extends GenericValidation
{
    WorkElementScope workElementScope;

    ScopeElementScope scopeElementScope;

    public void setUp() throws Exception
    {
        super.setUp();

        workElementScope = new WorkElementScope();
        workElementScope.setScopeElement(new ScopeElementScope());

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setChildren(new ScopeElementScope());
        scopeElementScope.setProject(new WorkElementScope());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureCrossTransfer() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);

        // create a ServiceRequest
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setName(HelperTestUtilities.createName());
        serviceRequest.setParent(scopeFolder);
        save(serviceRequest, scopeElementScope);

        // create a WorkElement with a different parent project
        WorkProduct workProduct = new WorkProduct();
        workProduct.setName(HelperTestUtilities.createName());
        workProduct.setParent(WorkElementFixture.createSaveFixture(null,
            new Project(), workElementScope, this));
        save(workProduct, workElementScope);

        // attempt to transfer the WorkProduct to the ServiceRequest
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), serviceRequest, workProduct);

        assertFalse(result.isSuccessful());

        // error: cross transfer not valid
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_CROSS_TRASNFER_UNDER_SCOPE,
                workProduct.getClass().getName()),
        };

        verifyFailure(workProduct, expectedExceptions, null, result);

    }

    public void testFailureMissingIDs() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);

        // create an unsaved ServiceRequest
        Defect defect = new Defect();
        defect.setParent(scopeFolder);

        // create an unsaved SummaryTask
        SummaryTask summaryTask = new SummaryTask();

        // attempt to transfer the unsaved SummaryTask to the unsaved Defect
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), defect, summaryTask);

        assertFalse(result.isSuccessful());

        // error: missing IDs
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, defect.getClass()
                    .getName()),
                createRPMException(ErrorCodes.NO_ID, summaryTask.getClass()
                    .getName()),
        };

        verifyFailure(summaryTask, expectedExceptions, null, result);

    }

    public void testFailureTransferGenericProject() throws Exception
    {
        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(workOrganization);

        // create an Issue
        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(scopeFolder);
        save(issue, scopeElementScope);

        // create a GenericProject 
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());
        project.setParent(workOrganization);
        save(project, workElementScope);

        // attempt to transfer the GenericProject to the Issue
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), issue, project);

        assertFalse(result.isSuccessful());

        // error: transfer of GenericProject not valid
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_TRANSFER_UNDER_SCOPE,
                project.getClass().getName()),
        };

        verifyFailure(project, expectedExceptions, null, result);
    }

    public void testFailureTransferDeliverableToScopeDeliverable()
            throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);
        
        Issue issue = new Issue();
        issue.setName(HelperTestUtilities.createName());
        issue.setParent(scopeFolder);
        save(issue, scopeElementScope);

        // create a ScopeDeliverable and reload it to get a populated project field
        ScopeDeliverable scopeDeliverable = new ScopeDeliverable();
        scopeDeliverable.setName(HelperTestUtilities.createName());
        scopeDeliverable.setParent(issue);
        save(scopeDeliverable, scopeElementScope);

        scopeDeliverable = (ScopeDeliverable) loadFromID(scopeDeliverable,
            scopeElementScope);

        // create a Deliverable to transfer
        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(proposal);
        save(deliverable, workElementScope);

        // attempt to transfer the Deliverable to the ScopeDeliverable
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), scopeDeliverable, deliverable);

        assertFalse(result.isSuccessful());

        // error: transfer from Deliverable to ScopeDeliverable not valid
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.
                CANNOT_TRANSFER_UNDER_SPECIFIC_SCOPE_ELEMENT,
                deliverable.getClass().getName()),
        };

        verifyFailure(deliverable, expectedExceptions, null, result);
    }

    public void testFailureTransferDeliverableToScopeSummaryTask()
            throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        ScopeFolder scopeFolder = createScopeFolder(proposal);
        
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setName(HelperTestUtilities.createName());
        serviceRequest.setParent(scopeFolder);
        save(serviceRequest, scopeElementScope);

        // create a ScopeSummnaryTask and reload it to get a populated project field
        ScopeSummaryTask scopeSummaryTask = new ScopeSummaryTask();
        scopeSummaryTask.setName(HelperTestUtilities.createName());
        scopeSummaryTask.setParent(serviceRequest);
        save(scopeSummaryTask, scopeElementScope);

        scopeSummaryTask = (ScopeSummaryTask) loadFromID(scopeSummaryTask,
            scopeElementScope);

        // create a Deliverable to transfer
        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(proposal);
        save(deliverable, workElementScope);

        // attempt to transfer the Deliverable to the ScopeSummaryTask
        Result result = getApplicationInterface().transferUnderScope(
            getSessionID(), scopeSummaryTask, deliverable);

        assertFalse(result.isSuccessful());

        // error: transfer from Deliverable to ScopeSummaryTask not valid
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.
                CANNOT_TRANSFER_UNDER_SPECIFIC_SCOPE_ELEMENT,
                deliverable.getClass().getName()),
        };

        verifyFailure(deliverable, expectedExceptions, null, result);
    }

    private ScopeFolder createScopeFolder(GenericProject project)
            throws Exception
    {
        project = (GenericProject) loadFromID(project, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + project.getName() + "']", new ScopeElementScope());
        ScopeFolder scopeFolder = (ScopeFolder) rpmObjects[0];

        return scopeFolder;
    }
}