package com.ibm.rpm.client.junit.wbs.testcases.xpath;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WbsModuleFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;

public class TestWbsModuleXPath extends ClientTestController
{
    // This xpath test class cannot extend ObjectXPathTester because loading
    // a module with child conditions is not valid.

    // Testing all fields except Datafields - there are test
    // cases for one Datafield only.

    WbsModule module;

    WbsModuleScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        scope = WbsModuleFixture.createScope(true, true, true, true, true,
            true, true, true, true, true, true, true, true, true, true, true,
            true);

        module = WbsModuleFixture.createFixture(scope, this);
        save(module, scope);
    }

    public void testLoadAttributeCategoryFromID() throws Exception
    {
        // loads the AttributeCategory with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/AttributeCategory[id='"
                    + module.getAttributeCategories()[0].getID() + "']", scope
                .getAttributeCategories());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AttributeCategory);

        AttributeCategory attributeCategory = (AttributeCategory) rpmObjects[0];
        AttributeFixture.compare(module.getAttributeCategories()[0],
            attributeCategory, scope.getAttributeCategories());

    }

    public void testLoadAllWbsAttributeCategories() throws Exception
    {
        // loads all AttributeCategories with context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/AttributeCategory",
            scope.getAttributeCategories());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AttributeCategory);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(
                module.getAttributeCategories()[0].getID()))
            {
                AttributeCategory attributeCategory = (AttributeCategory) rpmObjects[i];

                AttributeFixture.compare(module.getAttributeCategories()[0],
                    attributeCategory, scope.getAttributeCategories());

                compare = true;
            }
        }

        assertTrue("AttributeCategory not loaded", compare);
    }

    public void testLoadCustomFieldCategoryFromID() throws Exception
    {
        // loads the CustomFieldCategory with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/CustomFieldCategory[id='"
                    + module.getCustomFieldCategories()[0].getID() + "']",
            scope.getCustomFieldCategories());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof CustomFieldCategory);

        CustomFieldCategory customFieldCategory = (CustomFieldCategory) rpmObjects[0];
        CustomFieldFixture.assertEquals(module.getCustomFieldCategories()[0],
            customFieldCategory, scope.getCustomFieldCategories());

    }

    public void testLoadAllWbsCustomFieldCategories() throws Exception
    {
        // loads all CustomFieldCategories with context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/CustomFieldCategory", scope.getCustomFieldCategories());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof CustomFieldCategory);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(
                module.getCustomFieldCategories()[0].getID()))
            {
                CustomFieldCategory customFieldCategory = (CustomFieldCategory) rpmObjects[i];

                CustomFieldFixture.assertEquals(module
                    .getCustomFieldCategories()[0], customFieldCategory, scope
                    .getCustomFieldCategories());

                compare = true;
            }
        }

        assertTrue("CustomFieldCategory not loaded", compare);
    }

    public void testFailureLoadParentFromChild() throws Exception
    {
        boolean exception = false;

        try
        {
            // not supported; same result as doing a loadFromXpath on /WbsModule
            loadFromXpath("/WbsModule[AttributeCategory]", scope);
        }
        catch (RPMException e)
        {
            exception = true;
            assertEquals(e.getErrorID(), ErrorCodes.XPATH_UNSUPPORTED_CONDITION);
        }

        if (!exception)
        {
            fail("Expected exception with error code: "
                    + ErrorCodes.XPATH_UNSUPPORTED_CONDITION);
        }

    }

    public void testFailureLoadParentFromChildWithCondition() throws Exception
    {
        boolean exception = false;

        try
        {
            // not supported; same result as doing a loadFromXpath on /WbsModule
            loadFromXpath("/WbsModule[AttributeCategory[id='"
                    + module.getAttributeCategories()[0].getID() + "']]", scope);
        }
        catch (RPMException e)
        {
            exception = true;
            assertEquals(e.getErrorID(), ErrorCodes.XPATH_UNSUPPORTED_CONDITION);
        }

        if (!exception)
        {
            fail("Expected exception with error code: "
                    + ErrorCodes.XPATH_UNSUPPORTED_CONDITION);
        }

    }

    public void testLoadAssetFinancialCategoryFromID() throws Exception
    {
        // loads the AssetFinancialCategory with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/AssetFinancialCategory[id='"
                    + module.getAssetFinancialCategories()[0].getID() + "']",
            null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFinancialCategory);

        AssetFinancialCategory assetFinancialCategory = (AssetFinancialCategory) rpmObjects[0];
        DatafieldFixture.compare(module.getAssetFinancialCategories()[0],
            assetFinancialCategory);

    }

    public void testLoadAllWbsAssetFinancialCategories() throws Exception
    {
        // loads all AssetFinancialCategories with context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/AssetFinancialCategory", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFinancialCategory);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(
                module.getAssetFinancialCategories()[0].getID()))
            {
                AssetFinancialCategory assetFinancialCategory = (AssetFinancialCategory) rpmObjects[i];
                DatafieldFixture.compare(
                    module.getAssetFinancialCategories()[0],
                    assetFinancialCategory);

                compare = true;
            }
        }

        assertTrue("AssetFinancialCategory not loaded", compare);
    }

    public void testLoadGenericProjectFromID() throws Exception
    {
        // loads the GenericProject with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/GenericProject[id='"
                + module.getGenericProjects()[0].getID() + "']", scope
            .getGenericProjects());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);

        GenericProject genericProject = (GenericProject) rpmObjects[0];
        WorkElementFixture.compare(module.getGenericProjects()[0],
            genericProject, scope.getGenericProjects());

    }

    public void testLoadAllWbsGenericProjects() throws Exception
    {
        // loads all GenericProjects with context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/GenericProject",
            scope.getGenericProjects());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(
                module.getGenericProjects()[0].getID()))
            {
                GenericProject genericProject = (GenericProject) rpmObjects[i];
                WorkElementFixture.compare(module.getGenericProjects()[0],
                    genericProject, scope.getGenericProjects());

                compare = true;
            }
        }

        assertTrue("GenericProject not loaded", compare);
    }

    public void testLoadPortfolioFromID() throws Exception
    {
        // loads the Portfolio with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/Portfolio[id='"
                + module.getPortfolios()[0].getID() + "']", scope
            .getPortfolios());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Portfolio);

        Portfolio portfolio = (Portfolio) rpmObjects[0];
        PortfolioFixture.compare(module.getPortfolios()[0], portfolio, scope
            .getPortfolios());

    }

    public void testLoadAllWbsPortfolios() throws Exception
    {
        // loads all Portfolios with context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/Portfolio", scope
            .getPortfolios());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Portfolio);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(module.getPortfolios()[0].getID()))
            {
                Portfolio portfolio = (Portfolio) rpmObjects[i];
                PortfolioFixture.compare(module.getPortfolios()[0], portfolio,
                    scope.getPortfolios());

                compare = true;
            }
        }

        assertTrue("Portfolio not loaded", compare);
    }

    public void testLoadScorecardFolderFromID() throws Exception
    {
        // loads the ScorecardFolder with the specified ID and context WBS
        RPMObject[] rpmObjects = loadFromXpath(
            "/WbsModule/ScorecardFolder[id='"
                    + module.getScorecardFolders()[0].getID() + "']", scope
                .getScorecardFolders());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardFolder);

        ScorecardFolder scorecardFolder = (ScorecardFolder) rpmObjects[0];
        ScorecardFixture.compare(module.getScorecardFolders()[0],
            scorecardFolder, scope.getScorecardFolders());

    }

    public void testLoadAllWbsScorecardFolders() throws Exception
    {
        // loads all ScorecardFolders with context WBS
        RPMObject[] rpmObjects = loadFromXpath("/WbsModule/ScorecardFolder",
            scope.getScorecardFolders());

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardFolder);

        boolean compare = false;

        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (rpmObjects[i].getID().equals(
                module.getScorecardFolders()[0].getID()))
            {
                ScorecardFolder scorecardFolder = (ScorecardFolder) rpmObjects[i];
                ScorecardFixture.compare(module.getScorecardFolders()[0],
                    scorecardFolder, scope.getScorecardFolders());

                compare = true;
            }
        }

        assertTrue("ScorecardFolder not loaded", compare);
    }
}