/*
 * Created on 4-Oct-2006
 * @author mmatwiy
 * Test methods used by subclass WorkflowProcess tests.
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.workflow.fixtures.WorkflowFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class AbstractTestWorkflowProcess extends ClientTestController
{
    public void associateProcess(WorkflowProcess process,
            GenericProject genericProject, boolean defaultFlag)
            throws Exception
    {
        Result result = getApplicationInterface().associateWorkflow(
            getSessionID(), process, genericProject, true, defaultFlag);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }

    public void unAssociateProcess(WorkflowProcess process,
            GenericProject genericProject) throws Exception
    {
        Result result = getApplicationInterface().associateWorkflow(
            getSessionID(), process, genericProject, false, false);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }

    public RunningWorkflowProcess startProcess(WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
            throws Exception
    {
        SingleResult result = getApplicationInterface().startWorkflow(
            sessionid, process, object, scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        return (RunningWorkflowProcess) result.getRpmObject();
    }

    public RunningWorkflowProcess suspendProcess(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws Exception
    {
        SingleResult result = getApplicationInterface().suspendWorkflow(
            sessionid, runningProcess, scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        return (RunningWorkflowProcess) result.getRpmObject();
    }

    public RunningWorkflowProcess resumeProcess(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws Exception
    {
        SingleResult result = getApplicationInterface().resumeWorkflow(
            sessionid, runningProcess, scope);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        return (RunningWorkflowProcess) result.getRpmObject();
    }

    public void cancelProcess(RunningWorkflowProcess runningProcess)
            throws Exception
    {
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            runningProcess);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }

    public void findAndCompareRunningProcess(
            RunningWorkflowProcess runningProcess,
            RPMObject[] runningProcesses, RunningWorkflowProcessScope scope)
            throws Exception
    {
        boolean found = false;

        for (int i = 0; i < runningProcesses.length; i++)
        {
            if (runningProcesses[i].getID().equals(runningProcess.getID()))
            {
                found = true;
                assertTrue(runningProcesses[i] instanceof RunningWorkflowProcess);

                WorkflowFixture
                    .compare(runningProcess,
                        (RunningWorkflowProcess) runningProcesses[i],
                        scope);
            }
        }
        assertTrue("RunningWorkflowProcess with correct ID not loaded", found);
    }

    public void findAndCompareAvailableProcess(WorkflowProcess process,
            RPMObject[] availableProcesses, WorkflowScope workflowScope)
            throws Exception
    {
        boolean found = false;

        for (int i = 0; i < availableProcesses.length; i++)
        {
            if (availableProcesses[i].getID().equals(process.getID()))
            {
                found = true;
                assertTrue(availableProcesses[i] instanceof AvailableWorkflowProcess);

                WorkflowFixture.compare(process,
                    ((AvailableWorkflowProcess) availableProcesses[i])
                        .getWorkflowProcess(), workflowScope);
            }
        }
        assertTrue("AvailableWorkflowProcess with correct ID not loaded", found);
    }
}