/*
 * Created on 5-Oct-2006
 * @author mmatwiy
 * 
 * Xpath test cases for WorkflowProcesses. 
 * This class tests these xpaths, with conditions:
 * - /WorkflowCategory/WorkflowProcess
 * - /WorkflowCategory[WorkflowProcess]
 * - /WorkflowGroup/WorkflowCategory
 * - /WorkflowGroup[WorkflowCategory]
 * - /AvailableWorkflowProcess/WorkflowProcess
 * - /AvailableWorkflowProcess[WorkflowProcess]
 * - /RunningWorkflowProcess/WorkflowProcess
 * - /RunningWorkflowProcess[WorkflowProcess]
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.client.junit.workflow.fixtures.WorkflowFixture;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowCategory;
import com.ibm.rpm.workflow.containers.WorkflowGroup;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestWorkflowProcess extends AbstractTestWorkflowProcess
{

    private WorkElementScope workElementScope = null;

    private WorkflowProcess process = null;

    private static final String PROCESS_NAME = "TaskProcess1";

    private static final String CATEGORY_NAME = "TaskCategory1";

    private static final String GROUP_ID = "TASK";
    
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;
     
    private AvailableWorkflowProcessScope availableScope = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());
        
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
                
        availableScope = new AvailableWorkflowProcessScope();
        availableScope.setChildren(workflowScope);

        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowProcessesAvailable(availableScope);
        workElementScope.setWorkflowProcessesRunning(runningScope);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        process = (WorkflowProcess) objects[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testLoadAvailableProcessWithChildWorkflowProcess()
            throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        // associate the process so that it shows up in the list
        // of available processes
        associateProcess(process, proposal, true);

        RPMObject[] objects = loadFromXpath(
            "/AvailableWorkflowProcess[WorkflowProcess[id='" + process.getID()
                    + "']]", availableScope);
        assertNotNull(objects);

        AvailableWorkflowProcess availableProcess = null;
        boolean found = false;

        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof AvailableWorkflowProcess);
            availableProcess = (AvailableWorkflowProcess) objects[i];

            if (process.getID().equals(availableProcess.getID()))
            {
                found = true;
            }
        }
        assertTrue("AvailableWorkflowProcess with correct ID not loaded", found);
    }

    public void testLoadRunningProcessWithChildWorkflowProcess()
            throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        Task task = (Task) WorkElementFixture.createSaveFixture(proposal,
            new Task(), null, this);

        // associate the process so that it can be started
        associateProcess(process, proposal, true);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, task,
            runningScope);

        RPMObject[] objects = loadFromXpath(
            "/RunningWorkflowProcess[WorkflowProcess[id='" + process.getID()
                    + "']]", runningScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof RunningWorkflowProcess);

        boolean found = false;
        
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof RunningWorkflowProcess);
            if (((RunningWorkflowProcess) objects[i]).getID().equals(
                runningProcess.getID()))
            {
                WorkflowFixture.compare(runningProcess,
                    (RunningWorkflowProcess) objects[i], runningScope);
                found = true;
            }
        }

        assertTrue("RunningWorkflowProcess with correct ID not found.", found);
        cancelRunningProcesses(task);
    }

    public void testLoadWorkflowProcessFromAvailableProcess() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        // associate the process so that it shows up in the list
        // of available processes
        associateProcess(process, proposal, true);

        RPMObject[] objects = loadFromXpath(
            "/AvailableWorkflowProcess[defaultFlag='true']/WorkflowProcess",
            workflowScope);
        assertNotNull(objects);
        compareProcess(objects);
    }

    public void testLoadWorkflowProcessFromRunningProcess() throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        Task task = (Task) WorkElementFixture.createSaveFixture(proposal,
            new Task(), null, this);

        // associate the process
        associateProcess(process, proposal, true);

        // start the process
        startProcess(process, task, runningScope);

        RPMObject[] objects = loadFromXpath("/RunningWorkflowProcess[state='"
                + WorkflowState.Open + "']/WorkflowProcess", workflowScope);
        assertNotNull(objects);
        compareProcess(objects);

        cancelRunningProcesses(task);
    }

    public void testLoadWorkflowProcessFromWorkflowCategory() throws Exception
    {
        RPMObject[] objects = loadFromXpath("/WorkflowCategory[name='"
                + CATEGORY_NAME + "']/WorkflowProcess", workflowScope);
        assertNotNull(objects);
        compareProcess(objects);
    }

    public void testLoadWorkflowCategoryFromWorkflowGroup() throws Exception
    {
        //TODO: replace this load once the API is able to create a WorkflowCategory
        RPMObject[] categories1 = loadFromXpath("/WorkflowCategory[name='"
                + CATEGORY_NAME + "']", workflowScope);
        assertNotNull(categories1);
        WorkflowCategory category = (WorkflowCategory) categories1[0];

        RPMObject[] categories2 = loadFromXpath("/WorkflowGroup[id='"
                + GROUP_ID + "']/WorkflowCategory", workflowScope);
        assertNotNull(categories2);
        compareCategory(categories2, category);
    }

    public void testLoadWorkflowCategoryWithChildProcess() throws Exception
    {
        //TODO: replace this load once the API is able to create a WorkflowCategory
        RPMObject[] categories1 = loadFromXpath("/WorkflowCategory[name='"
                + CATEGORY_NAME + "']", workflowScope);
        assertNotNull(categories1);
        WorkflowCategory category = (WorkflowCategory) categories1[0];

        RPMObject[] categories2 = loadFromXpath(
            "/WorkflowCategory[WorkflowProcess[id='" + process.getID() + "']]",
            workflowScope);
        assertNotNull(categories2);
        compareCategory(categories2, category);
    }

    public void testLoadWorkflowGroupWithChildCategory() throws Exception
    {
        //TODO: replace this load once the API is able to create a WorkflowGroup
        RPMObject[] groups1 = loadFromXpath("/WorkflowGroup[id='" + GROUP_ID
                + "']", workflowScope);
        assertNotNull(groups1);
        WorkflowGroup group = (WorkflowGroup) groups1[0];

        RPMObject[] groups2 = loadFromXpath(
            "/WorkflowGroup[WorkflowCategory[name='" + CATEGORY_NAME + "']]",
            workflowScope);
        assertNotNull(groups2);
        compareGroup(groups2, group);
    }

    private void compareProcess(RPMObject[] objects) throws Exception
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof WorkflowProcess);
            if (process.getID().equals(objects[i].getID()))
            {
                found = true;
                WorkflowFixture.compare(process, (WorkflowProcess) objects[i],
                    workflowScope);
            }
        }
        assertTrue("WorkflowProcess with correct ID not loaded", found);
    }

    private void compareCategory(RPMObject[] objects, WorkflowCategory category)
            throws Exception
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof WorkflowCategory);
            if (category.getID().equals(objects[i].getID()))
            {
                found = true;
                WorkflowFixture.compare(category,
                    (WorkflowCategory) objects[i], workflowScope);
            }
        }
        assertTrue("WorkflowCategory with correct ID not loaded", found);
    }

    private void compareGroup(RPMObject[] objects, WorkflowGroup group)
            throws Exception
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof WorkflowGroup);
            if (group.getID().equals(objects[i].getID()))
            {
                found = true;
                WorkflowFixture.compare(group, (WorkflowGroup) objects[i],
                    workflowScope);
            }
        }
        assertTrue("WorkflowGroup with correct ID not loaded", found);
    }

    private void cancelRunningProcesses(Task task) throws Exception
    {
        // reload the Task to get the running workflows
        RPMObject[] objects = loadFromXpath("/Task[id='" + task.getID() + "']",
            workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof Task);
        task = (Task) objects[0];

        if (task.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < task.getWorkflowProcessesRunning().length; i++)
            {
                getApplicationInterface().cancelWorkflow(sessionid,
                    task.getWorkflowProcessesRunning()[i]);
            }
        }
    }
}