/*
 * Created on 12-Oct-2006
 * @author mmatwiy
 * Test cases for container WorkflowRoleMapping.
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.client.junit.workflow.fixtures.WorkflowRoleMappingFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;

public class TestWorkflowRoleMappings extends AbstractTestWorkflowProcess
{
    private static final String PROCESS_1_NAME = "DocumentProcess1";

    private static final String PROCESS_2_NAME = "DocumentProcess2";

    private static final String ROLE_1_NAME = "Role1";

    private static final String ROLE_2_NAME = "Role2";

    private static final String ROLE_3_NAME = "Role3";

    private WorkElementScope workElementScope = null;

    private WorkOrganization workOrganization;

    WorkflowRoleMappingScope mappingScope = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mappingScope = new WorkflowRoleMappingScope();
        mappingScope.setContactGroup(new ContactGroupScope());
        mappingScope.setResource(new ResourceScope());
        mappingScope.setSecurityRole(new SecurityRoleScope());

        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowRoleMappings(mappingScope);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_1_NAME + "']", null);
        assertNotNull(objects);
        WorkflowProcess process1 = (WorkflowProcess) objects[0];

        //TODO: replace this load once the API is able to create a WorkflowProcess
        objects = loadFromXpath("/WorkflowProcess[name='" + PROCESS_2_NAME
                + "']", null);
        assertNotNull(objects);
        WorkflowProcess process2 = (WorkflowProcess) objects[0];

        // don't use the static fixture because we need the WorkOrganization
        // to be deleted at the end of the tests in order to reset everything
        workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);

        // associate both processes to the WorkOrganization so that
        // their roles show up in the list of WorkflowRoleMappings
        associateProcess(process1, workOrganization, false);
        associateProcess(process2, workOrganization, false);

    }

    public void testLoadEmptyRoleMappings() throws Exception
    {
        // load the process role mappings 
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());

        // DocumentProcess1 Step1 contains Role1, Step2 contains Role1 and Role2.
        // DocumentProcess2 Step1 contains Role1, Role2, and Role3. Step2 contains no roles.
        // The list of WorkflowRoleMappings should contain one of each of the roles.

        checkLoadedRoleMappings(loadedOrganization.getWorkflowRoleMappings());
    }

    public void testUpdateRoleMappingWithResource() throws Exception
    {
        // load the process role mappings 
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());
        assertNotNull(loadedOrganization.getWorkflowRoleMappings()[0]);

        // update the role mapping with a resource then save
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setResource(ResourceFixture.createSaveFixture(this));

        Result result = save(loadedOrganization.getWorkflowRoleMappings()[0],
            workElementScope.getWorkflowRoleMappings());

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // reload to check that the mapping was successful
        objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization reloadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(reloadedOrganization.getWorkflowRoleMappings());
        assertNotNull(reloadedOrganization.getWorkflowRoleMappings()[0]);

        WorkflowRoleMappingFixture.compare(loadedOrganization
            .getWorkflowRoleMappings()[0], reloadedOrganization
            .getWorkflowRoleMappings()[0], workElementScope
            .getWorkflowRoleMappings());

        loadRoleMappingFromWorkOrganization(loadedOrganization
            .getWorkflowRoleMappings()[0]);

        loadParentWithChildWorkflowRoleMappingWithChildResource(loadedOrganization);

    }

    public void testUpdateRoleMappingWithSecurityRole() throws Exception
    {
        // load the process role mappings 
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());
        assertNotNull(loadedOrganization.getWorkflowRoleMappings()[0]);

        // Create a security role to assign to the mapping
        SecurityRole parentSecurityRole = new SecurityRole();
        parentSecurityRole.setID("SCPROJECTMANAGER");
        parentSecurityRole = (SecurityRole) loadFromID(parentSecurityRole, null);

        SecurityRole securityRole = SecurityRoleFixture
            .createFixture(parentSecurityRole);

        securityRole = (SecurityRole) save(securityRole, null).getRpmObject();

        // update the role mapping with the security role then save
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setSecurityRole(securityRole);

        Result result = save(loadedOrganization.getWorkflowRoleMappings()[0],
            workElementScope.getWorkflowRoleMappings());

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // reload to check that the mapping was successful
        objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization reloadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(reloadedOrganization.getWorkflowRoleMappings());
        assertNotNull(reloadedOrganization.getWorkflowRoleMappings()[0]);

        WorkflowRoleMappingFixture.compare(loadedOrganization
            .getWorkflowRoleMappings()[0], reloadedOrganization
            .getWorkflowRoleMappings()[0], workElementScope
            .getWorkflowRoleMappings());

        loadRoleMappingFromWorkOrganization(loadedOrganization
            .getWorkflowRoleMappings()[0]);
        
        loadParentWithChildWorkflowRoleMappingWithChildSecurityRole(loadedOrganization);

    }

    public void testUpdateRoleMappingWithContactGroup() throws Exception
    {
        // load the process role mappings 
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());
        assertNotNull(loadedOrganization.getWorkflowRoleMappings()[0]);

        // update the role mapping with a ContactGroup then save
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setContactGroup(ContactGroupFixture.createSaveFixture(0, this));

        Result result = save(loadedOrganization.getWorkflowRoleMappings()[0],
            workElementScope.getWorkflowRoleMappings());

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // reload to check that the mapping was successful
        objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization reloadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(reloadedOrganization.getWorkflowRoleMappings());
        assertNotNull(reloadedOrganization.getWorkflowRoleMappings()[0]);

        WorkflowRoleMappingFixture.compare(loadedOrganization
            .getWorkflowRoleMappings()[0], reloadedOrganization
            .getWorkflowRoleMappings()[0], workElementScope
            .getWorkflowRoleMappings());

        loadRoleMappingFromWorkOrganization(loadedOrganization
            .getWorkflowRoleMappings()[0]);
        
        loadParentWithChildWorkflowRoleMappingWithChildContactGroup(loadedOrganization);
    }

    public void testUpdateRoleMapping() throws Exception
    {
        // load the process role mappings 
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization loadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(loadedOrganization.getWorkflowRoleMappings());
        assertNotNull(loadedOrganization.getWorkflowRoleMappings()[0]);

        // update the role mapping with a ContactGroup
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setContactGroup(ContactGroupFixture.createSaveFixture(0, this));

        // Create a security role to assign to the mapping
        SecurityRole parentSecurityRole = new SecurityRole();
        parentSecurityRole.setID("SCPROJECTMANAGER");
        parentSecurityRole = (SecurityRole) loadFromID(parentSecurityRole, null);

        SecurityRole securityRole = SecurityRoleFixture
            .createFixture(parentSecurityRole);

        securityRole = (SecurityRole) save(securityRole, null).getRpmObject();

        // update the role mapping with the security role 
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setSecurityRole(securityRole);

        // update the role mapping with a resource
        loadedOrganization.getWorkflowRoleMappings()[0]
            .setResource(ResourceFixture.createSaveFixture(this));

        Result result = save(loadedOrganization.getWorkflowRoleMappings()[0],
            workElementScope.getWorkflowRoleMappings());

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // reload to check that the mapping was successful
        objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkOrganization);
        WorkOrganization reloadedOrganization = (WorkOrganization) objects[0];

        assertNotNull(reloadedOrganization.getWorkflowRoleMappings());
        assertNotNull(reloadedOrganization.getWorkflowRoleMappings()[0]);

        WorkflowRoleMappingFixture.compare(loadedOrganization
            .getWorkflowRoleMappings()[0], reloadedOrganization
            .getWorkflowRoleMappings()[0], workElementScope
            .getWorkflowRoleMappings());

    }

    private void loadRoleMappingFromWorkOrganization(
            WorkflowRoleMapping mapping1) throws Exception
    {
        // load the process role mapping; you can't load empty role mappings - they're
        // only stored in the table if they contain a Resource, a ContactGroup or a SecurityRole
        RPMObject[] objects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']/WorkflowRoleMapping",
            mappingScope);
        assertNotNull(objects);
        assertNotNull(objects[0]);

        assertTrue(objects[0] instanceof WorkflowRoleMapping);
        WorkflowRoleMappingFixture.compare(mapping1,
            (WorkflowRoleMapping) objects[0], mappingScope);
    }

    private void loadParentWithChildWorkflowRoleMappingWithChildResource(
            WorkOrganization workOrganization1) throws Exception
    {
        // load the parent of the role mapping; won't work for empty role mappings - they're
        // only stored in the table if they contain a Resource, a ContactGroup or a SecurityRole
        RPMObject[] objects = loadFromXpath(
            "/WorkOrganization[WorkflowRoleMapping[Resource[id='"
                    + workOrganization1.getWorkflowRoleMappings()[0]
                        .getResource().getID() + "']]]", workElementScope);
        assertNotNull(objects);
        assertNotNull(objects[0]);

        assertTrue(objects[0] instanceof WorkOrganization);
        WorkElementFixture.compare(workOrganization1,
            (WorkOrganization) objects[0], workElementScope);
    }
    
    private void loadParentWithChildWorkflowRoleMappingWithChildSecurityRole(
            WorkOrganization workOrganization1) throws Exception
    {
        // load the parent of the role mapping; won't work for empty role mappings - they're
        // only stored in the table if they contain a Resource, a ContactGroup or a SecurityRole
        RPMObject[] objects = loadFromXpath(
            "/WorkOrganization[WorkflowRoleMapping[SecurityRole[id='"
                    + workOrganization1.getWorkflowRoleMappings()[0]
                        .getSecurityRole().getID() + "']]]", workElementScope);
        assertNotNull(objects);
        assertNotNull(objects[0]);

        assertTrue(objects[0] instanceof WorkOrganization);
        WorkElementFixture.compare(workOrganization1,
            (WorkOrganization) objects[0], workElementScope);
    }
    
    private void loadParentWithChildWorkflowRoleMappingWithChildContactGroup(
            WorkOrganization workOrganization1) throws Exception
    {
        // load the parent of the role mapping; won't work for empty role mappings - they're
        // only stored in the table if they contain a Resource, a ContactGroup or a SecurityRole
        RPMObject[] objects = loadFromXpath(
            "/WorkOrganization[WorkflowRoleMapping[ContactGroup[id='"
                    + workOrganization1.getWorkflowRoleMappings()[0]
                        .getContactGroup().getID() + "']]]", workElementScope);
        assertNotNull(objects);
        assertNotNull(objects[0]);

        assertTrue(objects[0] instanceof WorkOrganization);
        WorkElementFixture.compare(workOrganization1,
            (WorkOrganization) objects[0], workElementScope);
    }

    private void checkLoadedRoleMappings(WorkflowRoleMapping[] roleMappings)
            throws Exception
    {
        // DocumentProcess1 Step1 contains Role1, Step2 contains Role1 and Role2.
        // DocumentProcess2 Step1 contains Role1, Role2, and Role3. Step2 contains no roles.
        // The list of WorkflowRoleMappings should contain one of each of the roles.

        int role1 = 0;
        int role2 = 0;
        int role3 = 0;

        for (int i = 0; i < roleMappings.length; i++)
        {
            WorkflowRoleMapping mapping = roleMappings[i];

            if (mapping != null)
            {
                if (mapping.getWorkflowRole().getName().equals(ROLE_1_NAME))
                {
                    role1++;
                }
                else if (mapping.getWorkflowRole().getName()
                    .equals(ROLE_2_NAME))
                {
                    role2++;
                }
                else if (mapping.getWorkflowRole().getName()
                    .equals(ROLE_3_NAME))
                {
                    role3++;
                }
            }
        }

        assertTrue("Wrong number of instances of " + ROLE_1_NAME
                + " found. Found " + role1 + ".", role1 == 1);
        assertTrue("Wrong number of instances of " + ROLE_2_NAME
                + " found. Found " + role2 + ".", role2 == 1);
        assertTrue("Wrong number of instances of " + ROLE_3_NAME
                + " found. Found " + role3 + ".", role3 == 1);
    }
}