/**
 * Created on Feb 14, 2006
 * @author shahla
 * @author mmatwiy
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestStartWorkflowProcessValidation extends GenericValidation
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;

    private WorkElementScope workElementScope = null;
    
    public static final String PROJECT_PROCESS_NAME = "ProjectProcess1";

    public void setUp() throws Exception
    {
        super.setUp();

        workflowScope = new WorkflowScope();
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
        
        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowProcessesRunning(runningScope);
    }

    private WorkflowProcess loadProcess() throws Exception
    {
        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // then associate the process to the project here
        RPMObject[] objects = loadFromXpath(
            "/WorkflowProcess[name='" + PROJECT_PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkflowProcess);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        return process;
    }

    public void testFailureInvalidWorkflowGroup() throws Exception
    {
        // load a WorkflowProcess 
        WorkflowProcess process = loadProcess();

        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        Deliverable targetDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(project, new Deliverable(), null, this);

        // Start the process.
        Result result = getApplicationInterface().startWorkflow(sessionid,
            process, targetDeliverable, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INVALID_WORKFLOW_PARAMS,
                null)
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    public void testFailureStartNullProcessWithNullObject() throws Exception
    {
        // Start a null process with a null target oject
        Result result = getApplicationInterface().startWorkflow(sessionid,
            null, null, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, "RPMObject"),
                createRPMException(ErrorCodes.NO_ID, "WorkflowProcess")
        };

        verifyProcessFailure(null, expectedExceptions, null, result);
    }

    public void testFailureStartUnsavedProcessWithUnsavedObject()
            throws Exception
    {
        WorkflowProcess process = new WorkflowProcess();
        GenericProject genericProject = new Project();

        // Start an unsaved process with an unsaved target object
        Result result = getApplicationInterface().startWorkflow(sessionid,
            process, genericProject, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, genericProject.getClass()
                    .getName()),
                createRPMException(ErrorCodes.NO_ID, process.getClass()
                    .getName())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    public void testFailureStartInactiveProcess() throws Exception
    {
        // load a target GenericProject
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // make sure it is not activated
        RPMObject[] objects = loadFromXpath(
            "/WorkflowProcess[name='ProjectProcessInactive']", workflowScope);
        assertNotNull(objects);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        // Start the process.
        Result result = getApplicationInterface().startWorkflow(sessionid,
            process, project, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.workflow.constants.ErrorCodes.INACTIVE_WORKFLOW,
                    process.getClass().getName(),
                    ValidationConstants.ACTIVE_FIELD, process.getID()),
                createRPMException(
                    com.ibm.rpm.workflow.constants.ErrorCodes.WORKFLOW_PROCESS_NOT_ASSOCIATED,
                    process.getClass().getName())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    public void testFailureStartUnassociatedProcess() throws Exception
    {
        // load a target GenericProject
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        WorkflowProcess process = loadProcess();

        // Start the process.
        Result result = getApplicationInterface().startWorkflow(sessionid,
            process, project, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.WORKFLOW_PROCESS_NOT_ASSOCIATED,
                process.getClass().getName())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    private void verifyProcessFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, result);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}