package com.ibm.rpm.client.junit.xpath.testcases;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolScope;

public class TestXPathDatesAndTimestamps extends ClientTestController
{

    private static DelDoc documentElement;

    private static Pool parent;

    private static Calendar creationDate;

    private static Calendar lastUpdate;

    private static String creationDateString = "creationdate";

    private static String lastupdatedString = "lastupdated";

    private static SimpleDateFormat timestampFormatWithMilli = new SimpleDateFormat(
        "yyyy-MM-dd HH:mm:ss.SSS");

    private static SimpleDateFormat timestampFormat = new SimpleDateFormat(
        "yyyy-MM-dd HH:mm:ss");

    private static SimpleDateFormat badTimestampFormat = new SimpleDateFormat(
        "MM-dd yyyy HH:mm:ss");

    private static SimpleDateFormat incompleteTimestampFormat = new SimpleDateFormat(
        "yyyy-MM-dd HH:mm");

    private static SimpleDateFormat dateFormat = new SimpleDateFormat(
        "yyyy-MM-dd");

    private static SimpleDateFormat incompleteDateFormat = new SimpleDateFormat(
        "yyyy-MM");

    private static SimpleDateFormat badDateFormat = new SimpleDateFormat(
        "MM-dd yyyy");

    protected void setUp() throws Exception
    {
        super.setUp();

        if (documentElement == null)
        {
            parent = new Pool();
            documentElement = new DelDoc();

            PoolScope poolScope = new PoolScope();
            Pool pool = PoolFixture.createFixture(false, 0, poolScope, this);
            parent = (Pool) saveWithoutDelete(pool, poolScope).getRpmObject();
            DocumentScope documentScope = new DocumentScope();
            poolScope.setDocumentFolder(documentScope);
            parent = (Pool) loadFromID(parent, poolScope);
            RPMObject parentDocumentElement = DocumentElementFixture
                .getParentDocumentFolder(parent);
            documentElement = (DelDoc) DocumentElementFixture.createFixture(
                parentDocumentElement, documentElement, 0,
                (DocumentScope) null, this);

            documentElement = (DelDoc) saveWithoutDelete(documentElement, null)
                .getRpmObject();

            creationDate = documentElement.getCreationDate();
            lastUpdate = documentElement.getLastUpdated();
        }
    }

    public void testDateWithDateInput() throws Exception
    {
        SimpleDateFormat selectedFormatter = dateFormat;
        Calendar calendarToBeUsed = creationDate;
        String fieldToBeUsed = creationDateString;

        xpathMustWork(selectedFormatter, calendarToBeUsed, fieldToBeUsed);

    }

    private void xpathMustWork(SimpleDateFormat selectedFormatter,
            Calendar calendarToBeUsed, String fieldToBeUsed) throws Exception
    {
        String xpath = formXpath(selectedFormatter, calendarToBeUsed,
            fieldToBeUsed);

        RPMObject[] result = loadFromXpath(xpath, null);
        assertNotNull(result);
        assertEquals(1, result.length);
        assertEquals(documentElement.getID(), result[0].getID());
    }

    private void xpathMustFail(SimpleDateFormat selectedFormatter,
            Calendar calendarToBeUsed, String fieldToBeUsed) throws Exception
    {
        String xpath = formXpath(selectedFormatter, calendarToBeUsed,
            fieldToBeUsed);

        ArrayResult failLoadResult = getApplicationInterface().loadFromXpath(
            sessionid, xpath, null);
        RPMException[] errors = failLoadResult.getErrors();
        assertNotNull(errors);
        assertEquals(1, errors.length);
        assertTrue(errors[0].getErrorID() > 400000);
    }

    private String formXpath(SimpleDateFormat selectedFormatter,
            Calendar calendarToBeUsed, String fieldToBeUsed)
    {
        String printDate = selectedFormatter.format(calendarToBeUsed.getTime());
        String xpath = "/DelDoc[@id='" + documentElement.getID() + "' and @"
                + fieldToBeUsed + ">='" + printDate + "']";
        return xpath;
    }

    public void testDateWithTimestampInput() throws Exception
    {
        xpathMustWork(timestampFormat, creationDate, creationDateString);

    }

    public void testDateWithTimestampInputMilli() throws Exception
    {
        xpathMustWork(timestampFormatWithMilli, creationDate,
            creationDateString);
    }

    public void testDateWithBadInputDate() throws Exception
    {
        xpathMustFail(badDateFormat, creationDate, creationDateString);
    }

    public void testDateWithBadInputTimestamp() throws Exception
    {
        xpathMustFail(badTimestampFormat, creationDate, creationDateString);
    }

    public void testDateWithBadInputIncompleteTimestamp() throws Exception
    {
        xpathMustFail(incompleteTimestampFormat, creationDate,
            creationDateString);
    }

    public void testDateWithBadInputIncompleteDate() throws Exception
    {
        xpathMustFail(incompleteDateFormat, creationDate, creationDateString);
    }

    public void testTimestampWithTimestampInput() throws Exception
    {
        xpathMustWork(timestampFormat, lastUpdate, lastupdatedString);
    }

    public void testTimestampWithTimestampInputMilli() throws Exception
    {
        xpathMustWork(timestampFormatWithMilli, lastUpdate, lastupdatedString);
    }
   
    public void testTimestampWithBadInputDate() throws Exception
    {
        xpathMustFail(dateFormat, lastUpdate, lastupdatedString);
    }
    
    public void testTimestampWithBadInputBadDate() throws Exception
    {
        xpathMustFail(badDateFormat, lastUpdate, lastupdatedString);
    }
    
    public void testTimestampWithBadInputBadTimestamp() throws Exception
    {
        xpathMustFail(badTimestampFormat, lastUpdate, lastupdatedString);
    }
    
    public void testTimestampWithBadInputIncompleteTimestamp() throws Exception
    {
        xpathMustFail(incompleteTimestampFormat, lastUpdate, lastupdatedString);
    }
    
    public void testTimestampWithBadInputIncompleteDate() throws Exception
    {
        xpathMustFail(incompleteDateFormat, lastUpdate, lastupdatedString);
    }
}
