/*
 * Created on 9-Jan-07
 * @author mmatwiy
 * 
 * Action 56952:
 * Currently when a user queries a timestamp field but does not include the 
 * time portion in the query, a -180 error is returned by the database. 
 * Add a validation error that tells the user that the time portion is 
 * required for the fields that have a metadata rule of VALIDATION_TO_SECOND 
 * (only for fields where the time is relevant)
 * 
 * This is required for field WorkElement.lastModifiedDate.
 * Refer to Test56934 (remove the timestamp from the loadFromXpath to view the error).
 * 
 * Resolution:
 * Return a validation error if the time is missing.
 * Done in XPathUtil.validateFieldCondition.
 */
package com.ibm.rpm.client.junit.xpath.testcases.fixes;

import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;

public class Test56952 extends GenericValidation
{
    public void test() throws Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // create a task and save it
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(parentProject);
        save(task, null);

        boolean invalid = false;
        try
        {
            // load a task with the lastModifiedDate with no time portion
            loadFromXpath("/Task[lastModifiedDate>'"
                    + XpathTesterUtils.convertToString(new GregorianCalendar())
                    + "']", null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_DATE_FORMAT, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);
    }
}
