package com.ibm.rpm.client.junit.xpath.testcases.validation;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author gmeloche
 *
 */
public class TestXPathParserValidation extends ClientTestController
{
    public void testXPathFailWithInvalidGroup() throws Exception
    {
        String xpath = "Pooldomain[@name = 'a name'] and isnotnull(address1)";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_GROUP, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidSyntax() throws Exception
    {
        String xpath = "Pooldomain[@name = 'a name']]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_STRING, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithUnsupportedFunction() throws Exception
    {
        String xpath = "Pooldomain[count(name)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_OPERATION_CODE, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithNonExistentFunction() throws Exception
    {
        String xpath = "Pooldomain[test(name)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            if (e.getErrorID() == ErrorCodes.XPATH_UNSUPPORTED_OPERATION_CODE
                    || e.getErrorID() == ErrorCodes.XPATH_INVALID_STRING)
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidArguments_1() throws Exception
    {
        String xpath = "Pooldomain[contains(field,a)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_ARGUMENTS, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidArguments_2() throws Exception
    {
        String xpath = "Pooldomain[starts-with(field,a)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_ARGUMENTS, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidNumberOfArguments_1() throws Exception
    {
        String xpath = "Pooldomain[contains(a)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidNumberOfArguments_2() throws Exception
    {
        String xpath = "Pooldomain[starts-with(a)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithInvalidNumberOfArguments_3() throws Exception
    {
        String xpath = "Pooldomain[isnull()]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
}
