package com.ibm.rpm.client.junit.xpath.testcases.validation;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author gmeloche
 *
 */
public class TestXPathUtilValidation extends ClientTestController
{

    public void testXPathFailsWithInvalidField() throws Exception
    {

        String xpath = "/pools[name='beef']";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD, e
                .getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }

    public void testXPathFailsWithInvalidFieldAsContainer() throws Exception
    {

        String xpath = "/pools/pools";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_CONTAINER_OR_FIELD, e
                .getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithParentAsContainer() throws Exception
    {

        String xpath = "/Pool/parent[id = '1234']";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_FIELD, e
                .getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithParentAsField() throws Exception
    {

        String xpath = "/Pool[parent = '1234']";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_FIELD, e.getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithDeletedAsContainer() throws Exception
    {

        String xpath = "/Pool[deleted]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_FIELD, e
                .getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithDeletedAsField() throws Exception
    {

        String xpath = "/Pool[deleted = 1]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_FIELD, e.getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithNonExistentContainer() throws Exception
    {

        String xpath = "/Pool/a_field";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_NON_EXISTENT_FIELD, e
                .getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }
    
    public void testXPathFailWithInvalidField() throws Exception
    {
        String xpath = "Pooldomain[bbb = 'bbb']";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_NON_EXISTENT_FIELD, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }

    public void testXPathFailsWithUnsupportedFieldAsContainer()
            throws Exception
    {

        String xpath = "/Resource[passport]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_FIELD_AS_CONTAINER, e
                .getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithInvalidJoinRelation() throws Exception
    {

        String xpath = "/Resource/Pool/Pool/Pool";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_JOIN, e.getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithInvalidSubTableRelation() throws Exception
    {

        String xpath = "Pool/Resource[Pool[resources]]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_INVALID_SUBJOIN, e.getErrorID());
            invalid = true;
        }
        assertTrue(invalid);
    }

    public void testXPathFailsWithMandatoryFieldName() throws Exception
    {

        String xpath = "/AssetFolder/AssetFolder";

        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_MANDATORY_FIELD_NAME, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);
    }
    
    public void testXPathFailWithUnsupportedOperationOnField() throws Exception
    {
        String xpath = "Pooldomain[@pooldomains = 0]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_OPERATION, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithUnsupportedFunctionOnField() throws Exception
    {
        String xpath = "Pooldomain[isnotnull(@pooldomains)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_OPERATION, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithUnsupportedFunctionOnBoolean() throws Exception
    {
        String xpath = "resource[isnull(@active)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_OPERATION, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithUnsupportedFunctionOnEnumeratedType() throws Exception
    {
        String xpath = "project[isnull(published)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_OPERATION, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    
    public void testXPathFailWithNumericNonExistentField() throws Exception
    {
        String xpath = "Pooldomain[isnull(2)]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_NON_EXISTENT_FIELD, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }
    
    public void testXPathFailWithNonSupportedEnumeratedType() throws Exception
    {
        String xpath = "WbsFinancial[@actualTimePhased[budgettype='actual']]";
        boolean invalid = false;

        try
        {
            loadFromXpathMustFail(xpath, null);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.XPATH_UNSUPPORTED_ENUMERATED_TYPE, e.getErrorID());
            invalid = true;
        }

        assertTrue(invalid);

    }

}
