/*
 * Created on Oct 26, 2005
 * Provides validation for class ClientsManager validation.
 */
package com.ibm.rpm.clientcostcenters.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.clientcostcenters.constants.ValidationConstants;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.checkpoints.DocumentElementCheckpoint;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author mmatwiy
 */
public class ClientCostCenterCheckpoint extends GenericClientCheckpoint
{
    private static final List CLIENTCOSTCENTER_PARENT_LIST = new ArrayList();

    private static final List CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST = new ArrayList();

    protected static ClientCostCenterCheckpoint instance = new ClientCostCenterCheckpoint();

    static
    {
        CLIENTCOSTCENTER_PARENT_LIST.add(ClientFolder.class);
        CLIENTCOSTCENTER_PARENT_LIST.add(Division.class);

        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(DocumentFolder.class);
        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(DelDoc.class);
        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(Document.class);
        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(Software.class);
        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(UrlDocument.class);
        CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST.add(WPDoc.class);
    }

    public static ClientCostCenterCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, false,
            CLIENTCOSTCENTER_PARENT_LIST);

        // Initialize
        ClientCostCenter clientCostCenter = (ClientCostCenter) rpmObject;
        ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;

        // Field validation of description
        if (clientCostCenter.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(clientCostCenter,
                ValidationConstants.DESCRIPTION_FIELD, clientCostCenter
                    .getDescription(), ValidationConstants.DESCRIPTION_MAX,
                msgCtx);
        }

        // DocumentElements validation
        if (scope != null && scope.getDocumentFolder() != null)
        {
            DocumentElementCheckpoint.validateDocumentElements(
                clientCostCenter, new DocumentElement[]
                {
                    clientCostCenter.getDocumentFolder()
                }, CLIENTCOSTCENTER_CHILDREN_DOCUMENT_LIST, msgCtx);
        }
    }

}