/*
 * Created on Aug 22, 2006
 */
package com.ibm.rpm.communications.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import com.ibm.rpm.communications.checkpoints.CommunicationMethodCheckpoint;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.communications.util.CommunicationUtil;
import com.ibm.rpm.communications.util.WorkflowAttachmentUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;

/**
 * @author dominiqu
 */
public class CommunicationManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        CommunicationManager.class);

    static
    {
        containerMap.add(GenericCommunication.class);
        containerMap.add(Notification.class);
        containerMap.add(WorkflowStepNotification.class);
        containerMap.add(StaffingRequest.class);
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_WORKFLOWS.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_WORKFLOWS.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_WORKFLOWS.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_WORKFLOWS.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_WORKFLOWS.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column WKF_DEF_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_DEF_ID = 6;

    public static final int TYPE_WKF_DEF_ID = Types.CHAR;

    public static final String NAME_WKF_DEF_ID = "TMT_WORKFLOWS.WKF_DEF_ID";

    // public static final String PROPERTY_WKF_DEF_ID = "WORKFLOWPROCESS";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_WKF_DEF_ID, NAME_WKF_DEF_ID);
    // }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WORKFLOWS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_WORKFLOWS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_WORKFLOWS.PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column PARENT_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PARENT_TYPE = 10;

    public static final int TYPE_PARENT_TYPE = Types.SMALLINT;

    public static final String NAME_PARENT_TYPE = "TMT_WORKFLOWS.PARENT_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_TYPE,NAME_PARENT_TYPE);}

    /**
     * Column REQUEST_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REQUEST_ID = 11;

    public static final int TYPE_REQUEST_ID = Types.INTEGER;

    public static final String NAME_REQUEST_ID = "TMT_WORKFLOWS.REQUEST_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REQUEST_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REQUEST_ID,NAME_REQUEST_ID);}

    /**
     * Column REVISION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION = 12;

    public static final int TYPE_REVISION = Types.VARCHAR;

    public static final String NAME_REVISION = "TMT_WORKFLOWS.REVISION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REVISION = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION,NAME_REVISION);}

    /**
     * Column WKF_PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WKF_PRIORITY = 13;

    public static final int TYPE_WKF_PRIORITY = Types.SMALLINT;

    public static final String NAME_WKF_PRIORITY = "TMT_WORKFLOWS.WKF_PRIORITY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_PRIORITY = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_PRIORITY,NAME_WKF_PRIORITY);}

    /**
     * Column CREATED_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATE = 14;

    public static final int TYPE_CREATED_DATE = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATE = "TMT_WORKFLOWS.CREATED_DATE";

    public static final String PROPERTY_CREATED_DATE = "RECEIVEDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATE,
            SqlUtil.convertTimestampToDate(NAME_CREATED_DATE));
    }

    /**
     * Column AUTHOR_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_AUTHOR_ID = 15;

    public static final int TYPE_AUTHOR_ID = Types.CHAR;

    public static final String NAME_AUTHOR_ID = "TMT_WORKFLOWS.AUTHOR_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_AUTHOR_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_AUTHOR_ID,NAME_AUTHOR_ID);}

    /**
     * Column EMAIL_ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EMAIL_ADDRESS = 16;

    public static final int TYPE_EMAIL_ADDRESS = Types.VARCHAR;

    public static final String NAME_EMAIL_ADDRESS = "TMT_WORKFLOWS.EMAIL_ADDRESS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EMAIL_ADDRESS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EMAIL_ADDRESS,NAME_EMAIL_ADDRESS);}

    /**
     * Column CLOSED_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CLOSED_DATE = 17;

    public static final int TYPE_CLOSED_DATE = Types.TIMESTAMP;

    public static final String NAME_CLOSED_DATE = "TMT_WORKFLOWS.CLOSED_DATE";

    public static final String PROPERTY_CLOSED_DATE = "CLOSEDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLOSED_DATE,
            SqlUtil.convertTimestampToDate(NAME_CLOSED_DATE));
    }

    /**
     * Column OUTCOME_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_OUTCOME_ID = 18;

    public static final int TYPE_OUTCOME_ID = Types.INTEGER;

    public static final String NAME_OUTCOME_ID = "TMT_WORKFLOWS.OUTCOME_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OUTCOME_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OUTCOME_ID,NAME_OUTCOME_ID);}

    /**
     * Column WKF_STATUS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WKF_STATUS = 19;

    public static final int TYPE_WKF_STATUS = Types.INTEGER;

    public static final String NAME_WKF_STATUS = "TMT_WORKFLOWS.WKF_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_STATUS,NAME_WKF_STATUS);}

    /**
     * Column END_CYCLE of type Types.CHAR mapped to String.
     */
    public static final int ID_END_CYCLE = 20;

    public static final int TYPE_END_CYCLE = Types.CHAR;

    public static final String NAME_END_CYCLE = "TMT_WORKFLOWS.END_CYCLE";

    public static final String PROPERTY_END_CYCLE = "STATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_END_CYCLE, NAME_END_CYCLE);
    }

    /**
     * Column EXPIRES_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_EXPIRES_DATE = 21;

    public static final int TYPE_EXPIRES_DATE = Types.TIMESTAMP;

    public static final String NAME_EXPIRES_DATE = "TMT_WORKFLOWS.EXPIRES_DATE";

    public static final String PROPERTY_EXPIRES_DATE = "DUEDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXPIRES_DATE,
            SqlUtil.convertTimestampToDate(NAME_EXPIRES_DATE));
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 22;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_WORKFLOWS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 23;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_WORKFLOWS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 24;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_WORKFLOWS.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 25;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_WORKFLOWS.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column CHECKLIST of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CHECKLIST = 26;

    public static final int TYPE_CHECKLIST = Types.VARCHAR;

    public static final String NAME_CHECKLIST = "TMT_WORKFLOWS.CHECKLIST";

    public static final String PROPERTY_CHECKLIST = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST, NAME_CHECKLIST);
    }

    /**
     * Column SUBJECT of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SUBJECT = 27;

    public static final int TYPE_SUBJECT = Types.VARCHAR;

    public static final String NAME_SUBJECT = "TMT_WORKFLOWS.SUBJECT";

    public static final String PROPERTY_SUBJECT = "SUBJECT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUBJECT, NAME_SUBJECT);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 28;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_WORKFLOWS.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "COMMENTS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column PROJECT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROJECT_NAME = 29;

    public static final int TYPE_PROJECT_NAME = Types.VARCHAR;

    public static final String NAME_PROJECT_NAME = "TMT_WORKFLOWS.PROJECT_NAME";

    public static final String PROPERTY_PROJECT_NAME = "PROJECTNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_NAME, NAME_PROJECT_NAME);
    }

    /**
     * Column ATTACHED_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ATTACHED_NAME = 30;

    public static final int TYPE_ATTACHED_NAME = Types.VARCHAR;

    public static final String NAME_ATTACHED_NAME = "TMT_WORKFLOWS.ATTACHED_NAME";

    public static final String PROPERTY_ATTACHED_NAME = "ATTACHMENTNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ATTACHED_NAME, NAME_ATTACHED_NAME);
    }

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 31;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "TMT_WORKFLOWS.STAGE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID);}

    /**
     * Column OUTCOME_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OUTCOME_NAME = 32;

    public static final int TYPE_OUTCOME_NAME = Types.VARCHAR;

    public static final String NAME_OUTCOME_NAME = "TMT_WORKFLOWS.OUTCOME_NAME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OUTCOME_NAME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OUTCOME_NAME,NAME_OUTCOME_NAME);}

    /**
     * Column STATE_CHANGE_WKF of type Types.VARCHAR mapped to String.
     */
    public static final int ID_STATE_CHANGE_WKF = 33;

    public static final int TYPE_STATE_CHANGE_WKF = Types.VARCHAR;

    public static final String NAME_STATE_CHANGE_WKF = "TMT_WORKFLOWS.STATE_CHANGE_WKF";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_STATE_CHANGE_WKF = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_STATE_CHANGE_WKF,NAME_STATE_CHANGE_WKF);}

    /**
     * Column WKF_GROUP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WKF_GROUP_ID = 34;

    public static final int TYPE_WKF_GROUP_ID = Types.INTEGER;

    public static final String NAME_WKF_GROUP_ID = "TMT_WORKFLOWS.WKF_GROUP_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_GROUP_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_GROUP_ID,NAME_WKF_GROUP_ID);}

    /**
     * Column PARENT_GROUP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PARENT_GROUP_ID = 35;

    public static final int TYPE_PARENT_GROUP_ID = Types.INTEGER;

    public static final String NAME_PARENT_GROUP_ID = "TMT_WORKFLOWS.PARENT_GROUP_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_GROUP_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PARENT_GROUP_ID,NAME_PARENT_GROUP_ID);}

    public static final String TABLE_NAME = "TMT_WORKFLOWS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_WORKFLOWS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_WKF_DEF_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_PARENT_TYPE,
            NAME_REQUEST_ID, NAME_REVISION, NAME_WKF_PRIORITY,
            NAME_CREATED_DATE, NAME_AUTHOR_ID, NAME_EMAIL_ADDRESS,
            NAME_CLOSED_DATE, NAME_OUTCOME_ID, NAME_WKF_STATUS, NAME_END_CYCLE,
            NAME_EXPIRES_DATE, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_RANK, NAME_CHECKLIST, NAME_SUBJECT,
            NAME_DESCRIPTION, NAME_PROJECT_NAME, NAME_ATTACHED_NAME,
            NAME_STAGE_ID, NAME_OUTCOME_NAME, NAME_STATE_CHANGE_WKF,
            NAME_WKF_GROUP_ID, NAME_PARENT_GROUP_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new CommunicationManager instance.
     * 
     * @return CommunicationManager
     */
    public CommunicationManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D'");
        sql.appendAnd();
        sql.append(NAME_ELEMENT_ID);
        sql.append(" IS NOT NULL ");
        sql.appendAnd();
        sql.append(NAME_LEVEL_ID);
        sql.append(">0 ");
        return sql.toString();
    }

    /**
     * CommunicationManager overloads loadBySelect because getFilter does not
     * have messageContext available to filter based on the current user. Since
     * all select statements for loads go through loadBySelect, this is an
     * alternative.
     */
    protected ArrayList loadBySelect(RPMObject parent, RPMObject[] rpmObjects,
            RPMObjectScope rpmScope, MessageContext messageContext,
            String selectStatement, Object[] args, String contextName,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        // Filter to ensure that only the current logged in resource's
        // communications are loaded
        SqlBuffer userCondition = new SqlBuffer();
        userCondition.append(selectStatement);
        if (selectStatement.toUpperCase().indexOf(" WHERE ") < 0)
        {
            userCondition.appendWhere();
        }
        userCondition.appendSubSelect(NAME_ELEMENT_ID,
            WorkflowResponseManager.NAME_ELEMENT_ID,
            WorkflowResponseManager.TABLE_NAME,
            WorkflowResponseManager.NAME_RESOURCE_ID + "='"
                    + getUser(messageContext).getID() + "' AND "
                    + WorkflowResponseManager.NAME_REPLY_STATUS + " != 3)");

        return super.loadBySelect(parent, rpmObjects, rpmScope, messageContext,
            userCondition.toString(), args, contextName, isDelta);
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericCommunication temp = (GenericCommunication) rpmObject;

        temp.setID(rs.getString(ID_ELEMENT_ID));

        if (isDelta)
        {
            temp.deltaName(rs.getString(ID_ELEMENT_NAME));
            temp.deltaSubject(rs.getString(ID_SUBJECT));
            temp.deltaComments(rs.getString(ID_DESCRIPTION));
            temp.deltaProjectName(rs.getString(ID_PROJECT_NAME));
            temp.deltaAttachmentName(rs.getString(ID_ATTACHED_NAME));
            temp.deltaReceivedDate(Manager.getCalendar(rs, ID_CREATED_DATE));
            temp.deltaState((WorkflowState) EnumeratedTypeUtil.getInstance().getType(
                null, WorkflowState.class,
                new Integer(rs.getString(ID_END_CYCLE))));
        }
        else
        {
            temp.setName(rs.getString(ID_ELEMENT_NAME));
            temp.setSubject(rs.getString(ID_SUBJECT));
            temp.setComments(rs.getString(ID_DESCRIPTION));
            temp.setProjectName(rs.getString(ID_PROJECT_NAME));
            temp.setAttachmentName(rs.getString(ID_ATTACHED_NAME));
            temp.setReceivedDate(Manager.getCalendar(rs, ID_CREATED_DATE));
            temp.setState((WorkflowState) EnumeratedTypeUtil.getInstance().getType(
                null, WorkflowState.class,
                new Integer(rs.getString(ID_END_CYCLE))));
        }

        if (temp instanceof StaffingRequest)
        {
            if (isDelta)
            {
                ((StaffingRequest) temp).deltaDueDate(Manager.getCalendar(rs,
                    ID_EXPIRES_DATE));
            }
            else
            {
                ((StaffingRequest) temp).setDueDate(Manager.getCalendar(rs,
                    ID_EXPIRES_DATE));
            }
        }
        else if (temp instanceof WorkflowStepNotification)
        {
            if (isDelta)
            {
                ((WorkflowStepNotification) temp).deltaDescription(rs.getString(ID_CHECKLIST));
                ((WorkflowStepNotification) temp).deltaClosedDate(Manager.getCalendar(
                    rs, ID_CLOSED_DATE));
            }
            else
            {
                ((WorkflowStepNotification) temp).setDescription(rs.getString(ID_CHECKLIST));
                ((WorkflowStepNotification) temp).setClosedDate(Manager.getCalendar(
                    rs, ID_CLOSED_DATE));
            }
        }

        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String authorId = resultSet.getString(ID_AUTHOR_ID);
        relatedFields.put(index, ID_AUTHOR_ID, authorId);

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        Integer parentTypeId = new Integer(resultSet.getInt(ID_PARENT_TYPE));
        relatedFields.put(index, ID_PARENT_TYPE, parentTypeId);

        String workflowProcessId = resultSet.getString(ID_WKF_DEF_ID);
        relatedFields.put(index, ID_WKF_DEF_ID, workflowProcessId);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject == null)
            return;

        // Validate method arguments and business rules
        CommunicationMethodCheckpoint.getInstance().validateDelete(rpmObject,
            messageContext);

        if (!messageContext.isSuccessful())
            return;

        SP_D_WOKFLOW(rpmObject, messageContext);
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof CommunicationManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?)");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null && rpmScope instanceof GenericCommunicationScope)
        {
            GenericCommunicationScope gcScope = (GenericCommunicationScope) rpmScope;

            loadAuthor((GenericCommunication) rpmObject, gcScope,
                messageContext, relatedFields, index, isDelta);

            loadAttachment((GenericCommunication) rpmObject, gcScope,
                messageContext, relatedFields, index, isDelta);

            loadResponses((GenericCommunication) rpmObject, gcScope,
                messageContext, isDelta);

            loadRunningWorkflowProcess((GenericCommunication) rpmObject,
                gcScope, messageContext, isDelta);
        }

        return rpmObject;
    }

    private void loadAuthor(GenericCommunication rpmObject,
            GenericCommunicationScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!rpmScope.isAuthor())
            return;

        ResourceManager manager = (ResourceManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            Resource.class.getName());

        // Get the correct Resource ID for the author
        String authorID = (String) relatedFields.get(index, ID_AUTHOR_ID);

        Resource author;
        if (rpmObject.getAuthor() == null)
        {
            author = new Resource();
            author.setID(authorID);
        }
        else
        {
            author = rpmObject.getAuthor();
        }

        if (author.getID() != null)
        {
            author = (Resource) manager.loadByPrimaryKey(author, null,
                messageContext, isDelta);

            if (isDelta)
                rpmObject.deltaAuthor(author);
            else
                rpmObject.setAuthor(author);
        }
    }

    private void loadAttachment(GenericCommunication rpmObject,
            GenericCommunicationScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!rpmScope.isWorkflowAttachment())
            return;

        WorkflowAttachmentManager manager = (WorkflowAttachmentManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            WorkflowAttachment.class.getName());

        // Get the workflow ID
        String workflowID = rpmObject.getID();

        WorkflowAttachment workflowAttachment;
        if (rpmObject.getWorkflowAttachment() == null)
        {
            workflowAttachment = new WorkflowAttachment();
            workflowAttachment.setID(workflowID);
        }
        else
        {
            workflowAttachment = rpmObject.getWorkflowAttachment();
        }

        if (workflowAttachment.getAttachment() == null)
        {
            workflowAttachment.setAttachment(WorkflowAttachmentUtil.createAttachmentInstanceFromTableType(
                (Integer) relatedFields.get(index, ID_PARENT_TYPE),
                messageContext));
            workflowAttachment.getAttachment().setID(
                (String) relatedFields.get(index, ID_PARENT_ID));
        }

        if (workflowAttachment.getID() != null
                && workflowAttachment.getAttachment().getID() != null)
        {
            workflowAttachment = (WorkflowAttachment) manager.loadByPrimaryKey(
                workflowAttachment, null, messageContext, isDelta);

            if (isDelta)
                rpmObject.deltaWorkflowAttachment(workflowAttachment);
            else
                rpmObject.setWorkflowAttachment(workflowAttachment);
        }
    }

    private void loadResponses(GenericCommunication rpmObjectParent,
            GenericCommunicationScope rpmScope, MessageContext messageContext,
            boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope.getResponses() == null)
            return;

        WorkflowResponseManager manager = (WorkflowResponseManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            WorkflowResponse.class.getName());

        WorkflowResponse[] oldResponses = rpmObjectParent.getResponses();

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
            rpmObjectParent, oldResponses, rpmScope.getResponses(),
            messageContext, this, new Object[]
            {
                rpmObjectParent.getID()
            }, null, null, isDelta);

        WorkflowResponse[] responses = null;

        if (temp != null)
        {
            responses = new WorkflowResponse[temp.size()];
            temp.toArray(responses);

            for (int i = 0; i < responses.length; i++)
            {
                responses[i].setParent(rpmObjectParent);
            }
        }

        rpmObjectParent.setResponses(responses);
    }

    private void loadRunningWorkflowProcess(GenericCommunication rpmObject,
            GenericCommunicationScope rpmScope, MessageContext messageContext,
            boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!rpmScope.isRunningWorkflowProcess()
                || !(CommunicationUtil.isWorkflowStepNotification(rpmObject)))
            return;

        RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            RunningWorkflowProcess.class.getName());

        String workflowProcessId = rpmObject.getID();

        RunningWorkflowProcess process;
        if (((WorkflowStepNotification) rpmObject).getRunningWorkflowProcess() == null)
        {
            process = new RunningWorkflowProcess();
            process.setID(workflowProcessId);
        }
        else
        {
            process = ((WorkflowStepNotification) rpmObject).getRunningWorkflowProcess();
        }

        if (process.getID() != null)
        {
            try
            {
                process = (RunningWorkflowProcess) manager.loadByPrimaryKey(
                    process, null, messageContext, isDelta);
            }
            catch (RPMException e)
            {
                if (e.getErrorID() != ErrorCodes.AM_INVALID_RPMOBJECTID)
                {
                    throw e;
                }
                process = null;
            }

            if (isDelta)
                ((WorkflowStepNotification) rpmObject).deltaRunningWorkflowProcess(process);
            else
                ((WorkflowStepNotification) rpmObject).setRunningWorkflowProcess(process);
        }
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    // ************************* External functions *************************//

    public void sendNotification(GenericCommunicationScope scope,
            RPMObject attachment, Resource[] recipients, String subject,
            String comments, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Construct the notification

        Notification notification = new Notification();
        notification.setAuthor((Resource) messageContext.getUser());
        notification.setSubject(subject);
        notification.setComments(comments);

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(attachment);
        workflowAttachment.setParent(notification);
        notification.setWorkflowAttachment(workflowAttachment);

        if (recipients != null)
        {
            WorkflowResponse[] responses = new WorkflowResponse[recipients.length];
            for (int i = 0; i < responses.length; ++i)
            {
                responses[i] = new WorkflowResponse();
                responses[i].setResource(recipients[i]);
            }
            notification.setResponses(responses);
        }

        // Method argument validation
        CommunicationMethodCheckpoint.getInstance().validateSendGenericCommunication(
            notification, messageContext);

        if (!messageContext.isSuccessful())
            return;

        notification.setID(SP_SEND_WORKFLOW(notification, messageContext));

        GenericCommunication reloadedCommunication = (GenericCommunication) load(
            notification, scope, messageContext);

        messageContext.addObject(reloadedCommunication);
    }

    public void sendStaffingRequest(GenericCommunicationScope scope,
            RPMObject attachment, Resource[] recipients, String subject,
            String comments, Calendar dueDate, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Construct the staffing request

        StaffingRequest staffingRequest = new StaffingRequest();
        staffingRequest.setAuthor((Resource) messageContext.getUser());
        staffingRequest.setSubject(subject);
        staffingRequest.setComments(comments);
        staffingRequest.setDueDate(dueDate);

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(attachment);
        workflowAttachment.setParent(staffingRequest);
        staffingRequest.setWorkflowAttachment(workflowAttachment);

        WorkflowResponse[] responses = new WorkflowResponse[recipients.length];
        for (int i = 0; i < responses.length; ++i)
        {
            responses[i] = new WorkflowResponse();
            responses[i].setResource(recipients[i]);
        }
        staffingRequest.setResponses(responses);

        // Method argument validation
        CommunicationMethodCheckpoint.getInstance().validateSendGenericCommunication(
            staffingRequest, messageContext);

        if (!messageContext.isSuccessful())
        {
            return;
        }

        staffingRequest.setID(SP_SEND_WORKFLOW(staffingRequest, messageContext));

        GenericCommunication reloadedCommunication = (GenericCommunication) load(
            staffingRequest, scope, messageContext);

        messageContext.addObject(reloadedCommunication);
    }

    /*
     * SP_SEND_WORKFLOW(1,'80C365A9D14346BF97D33DB1C846A17C','2f5b81b4e1b911d49b32
     * 97e29d4369b5',3,'0.3',0,0,null,0,'Dominique Resource 1 Communications
     * Taskfff', 'Comment - Can only specify, TO, Subject and Comment - This is
     * a notification','903CE93A5B00461F96A154CD306BC353,',null,null,0,'5028E
     * 26F018D49769BFED52337E4B079')
     */
    private String SP_SEND_WORKFLOW(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_SEND_WORKFLOW";
        Object args[] = new Object[16];
        args[0] = new Integer(1); // Table type

        // parent_id - The parent is actually the attachment
        if (((GenericCommunication) rpmObject).getWorkflowAttachment() != null
                && ((GenericCommunication) rpmObject).getWorkflowAttachment().getAttachment() != null)
        {
            args[1] = ((GenericCommunication) rpmObject).getWorkflowAttachment().getAttachment().getID();
        }

        args[2] = CommunicationUtil.getRootParentID(rpmObject, messageContext);

        // Attachment type
        if (((GenericCommunication) rpmObject).getWorkflowAttachment() != null
                && ((GenericCommunication) rpmObject).getWorkflowAttachment().getAttachment() != null)
        {
            args[3] = ((GenericCommunication) rpmObject).getWorkflowAttachment().getAttachment().retrieveTypeID();
        }

        args[4] = "0.3"; // Revision
        args[5] = new Integer(0); // request_id
        args[6] = new Integer(0); // Priority - no way to set in client

        // expires_date - can only be set for staffing request
        if (CommunicationUtil.isStaffingRequest(rpmObject))
        {
            args[7] = DateUtil.convertToString(((StaffingRequest) rpmObject).getDueDate())
                    + " 0:00:00";
        }

        // State, always starts as Open and cannot be set in client
        args[8] = new Integer(0);

        args[9] = ((GenericCommunication) rpmObject).getSubject();
        args[10] = ((GenericCommunication) rpmObject).getComments();

        // participants
        args[11] = null;
        if (((GenericCommunication) rpmObject).getResponses() != null)
        {
            args[11] = "";

            for (int i = ((GenericCommunication) rpmObject).getResponses().length - 1; i >= 0; --i)
            {
                if (((GenericCommunication) rpmObject).getResponses()[i].getResource() != null
                        && ((GenericCommunication) rpmObject).getResponses()[i].getResource().getID() != null)
                    args[11] = args[11]
                            + ((GenericCommunication) rpmObject).getResponses()[i].getResource().getID()
                            + ",";
            }
        }

        // groups
        // the ui actually always calls this stored procedure with the id of all
        // resources in a group, and not with a group id
        // therefore, I'm always setting this to null
        args[12] = null;

        args[13] = null; // attachments

        args[14] = new Integer(0); // bitflag

        // do not use rec user, because we can specify the author
        if (((GenericCommunication) rpmObject).getAuthor() != null)
        {
            args[15] = ((GenericCommunication) rpmObject).getAuthor().getID();
        }

        return executeProcedure(messageContext, name, args);
    }

    private void SP_D_WOKFLOW(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_WOKFLOW";
        Object args[] = new Object[4];

        args[0] = rpmObject.getID();

        args[1] = ManagerUtil.getParentId(this, rpmObject, messageContext,
            getTableName());

        args[2] = "X";

        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        executeProcedure(messageContext, name, args);
    }
}