package com.ibm.rpm.communications.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;

import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.util.WorkflowAttachmentUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;

public class WorkflowAttachmentManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        WorkflowAttachmentManager.class);

    static
    {
        containerMap.add(WorkflowAttachment.class);
    }

    /**
     * Column WORKFLOW_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WORKFLOW_ID = 1;

    public static final int TYPE_WORKFLOW_ID = Types.CHAR;

    public static final String NAME_WORKFLOW_ID = "WKF_ATTACHMENTS.WORKFLOW_ID";

    public static final String PROPERTY_WORKFLOW_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WORKFLOW_ID, NAME_WORKFLOW_ID);
    }

    /**
     * Column ATTACHMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ATTACHMENT_ID = 2;

    public static final int TYPE_ATTACHMENT_ID = Types.CHAR;

    public static final String NAME_ATTACHMENT_ID = "WKF_ATTACHMENTS.ATTACHMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ATTACHMENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ATTACHMENT_ID,NAME_ATTACHMENT_ID);}

    /**
     * Column TABLE_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TABLE_TYPE = 3;

    public static final int TYPE_TABLE_TYPE = Types.INTEGER;

    public static final String NAME_TABLE_TYPE = "WKF_ATTACHMENTS.TABLE_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TABLE_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TABLE_TYPE,NAME_TABLE_TYPE);}

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 4;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "WKF_ATTACHMENTS.CONFIG_FLAG";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONFIG_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG,NAME_CONFIG_FLAG);}

    public static final String TABLE_NAME = "WKF_ATTACHMENTS";

    /**
     * Create an array of type string containing all the fields of the
     * WKF_ATTACHMENTS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_WORKFLOW_ID, NAME_ATTACHMENT_ID, NAME_TABLE_TYPE,
            NAME_CONFIG_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new WorkflowAttachmentManager instance.
     * 
     * @return WorkflowAttachmentManager
     */
    public WorkflowAttachmentManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return null;
    }

    protected Object[] getPrimaryKey(ResultSet resultSet) throws RPMException,
            SQLException
    {
        String[] primaryKeyArray = getPrimaryKeyArray(null);
        Object[] primaryKey = new Object[primaryKeyArray.length];
        for (int i = 0; i < primaryKeyArray.length; i++)
        {
            // This line removes the table name
            primaryKeyArray[i] = primaryKeyArray[i].substring(primaryKeyArray[i].indexOf('.') + 1);

            if (i != 2)
            {
                primaryKey[i] = resultSet.getString(primaryKeyArray[i]);
            }
            else
            {
                primaryKey[i] = new Integer(
                    resultSet.getInt(primaryKeyArray[i]));
            }
        }

        return primaryKey;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        // NAME_WORKFLOW_ID is the parent GenericCommunication
        // and NAME_ATTACHMENT_ID is the attachment (Proposal, Project, etc)
        // NAME_TABLE_TYPE is the table that the attachment is stored in
        return new String[]
        {
                NAME_WORKFLOW_ID, NAME_ATTACHMENT_ID, NAME_TABLE_TYPE
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        WorkflowAttachment object = (WorkflowAttachment) rpmObject;

        String parentId = object.getID();

        String attachmentId = null;

        Integer attachmentTableType = null;

        if (object.getAttachment() != null)
        {
            attachmentId = object.getAttachment().getID();
            attachmentTableType = WorkflowAttachmentUtil.getTableType(object.getAttachment());
        }

        return new Object[]
        {
                parentId, attachmentId, attachmentTableType
        };
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject)
    {
        WorkflowAttachment workflowAttachment = (WorkflowAttachment) rpmObject;

        workflowAttachment.setID((String) primaryKey[0]);

        RPMObject attachment = null;
        attachment = WorkflowAttachmentUtil.createAttachmentInstanceFromTableType(
            (Integer) primaryKey[2], null);
        attachment.setID((String) primaryKey[1]);

        workflowAttachment.setAttachment(attachment);

        return workflowAttachment;
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }

        if (((WorkflowAttachment) rpmObject).getAttachment() == null
                || ((WorkflowAttachment) rpmObject).getAttachment().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(", ");
            invalidPrimaryKeys.append("Attachment.ID");
            valid = false;
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                invalidPrimaryKeys.toString());
        }
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new WorkflowAttachment();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rs.getString(ID_WORKFLOW_ID));

        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String attachmentId = resultSet.getString(ID_ATTACHMENT_ID);
        relatedFields.put(index, ID_ATTACHMENT_ID, attachmentId);

        Integer tableType = new Integer(resultSet.getInt(ID_TABLE_TYPE));
        relatedFields.put(index, ID_TABLE_TYPE, tableType);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkflowAttachment workflowAttachment = (WorkflowAttachment) rpmObject;

        loadAttachment(workflowAttachment, messageContext, relatedFields,
            index, isDelta);

        return workflowAttachment;
    }

    private void loadAttachment(WorkflowAttachment workflowAttachment,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        RPMObject attachment = WorkflowAttachmentUtil.createAttachmentInstanceFromTableType(
            (Integer) relatedFields.get(index, ID_TABLE_TYPE), messageContext);
        attachment.setID((String) relatedFields.get(index, ID_ATTACHMENT_ID));

        RPMObjectManager manager = getManagerInstance(attachment.getClass().getName());
        attachment = manager.loadByPrimaryKey(attachment, null, messageContext,
            isDelta);

        if (isDelta)
        {
            workflowAttachment.deltaAttachment(attachment);
        }
        else
        {
            workflowAttachment.setAttachment(attachment);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

}
