package com.ibm.rpm.communications.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.communications.checkpoints.CommunicationMethodCheckpoint;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.GenericResponseType;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.util.CommunicationUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;

/**
 * @author dominiqu
 */
public class WorkflowResponseManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        WorkflowResponseManager.class);

    static
    {
        containerMap.add(WorkflowResponse.class);
    }

    /**
     * Column WKF_REPLY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_REPLY_ID = 1;

    public static final int TYPE_WKF_REPLY_ID = Types.CHAR;

    public static final String NAME_WKF_REPLY_ID = "WORKFLOW_REPLIES.WKF_REPLY_ID";

    public static final String PROPERTY_WKF_REPLY_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WKF_REPLY_ID, NAME_WKF_REPLY_ID);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 2;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "WORKFLOW_REPLIES.ELEMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);}

    /**
     * Column GROUP_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_GROUP_ID = 3;

    public static final int TYPE_GROUP_ID = Types.CHAR;

    public static final String NAME_GROUP_ID = "WORKFLOW_REPLIES.GROUP_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_GROUP_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_GROUP_ID,NAME_GROUP_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 4;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "WORKFLOW_REPLIES.RESOURCE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column REPLY_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REPLY_TYPE = 5;

    public static final int TYPE_REPLY_TYPE = Types.INTEGER;

    public static final String NAME_REPLY_TYPE = "WORKFLOW_REPLIES.REPLY_TYPE";

    public static final String PROPERTY_REPLY_TYPE = "VISIBLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REPLY_TYPE, SqlUtil.getBooleanValue(
            NAME_REPLY_TYPE, 31));
    }

    /**
     * Column REPLY_STATUS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REPLY_STATUS = 6;

    public static final int TYPE_REPLY_STATUS = Types.INTEGER;

    public static final String NAME_REPLY_STATUS = "WORKFLOW_REPLIES.REPLY_STATUS";

    // Note: Folder requires two fields in order to be determined, so we're
    // adding them together
    public static final String PROPERTY_REPLY_STATUS = "FOLDER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REPLY_STATUS, SqlUtil.castAsChar(
            NAME_REPLY_STATUS + "+" + NAME_REPLY_TYPE, 2));
    }

    /**
     * Column REPLY_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REPLY_DATE = 7;

    public static final int TYPE_REPLY_DATE = Types.TIMESTAMP;

    public static final String NAME_REPLY_DATE = "WORKFLOW_REPLIES.REPLY_DATE";

    public static final String PROPERTY_REPLY_DATE = "RESPONSEDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REPLY_DATE,
            SqlUtil.convertTimestampToDate(NAME_REPLY_DATE));
    }

    /**
     * Column COMPLETED of type Types.CHAR mapped to String.
     */
    public static final int ID_COMPLETED = 8;

    public static final int TYPE_COMPLETED = Types.CHAR;

    public static final String NAME_COMPLETED = "WORKFLOW_REPLIES.COMPLETED";

    public static final String PROPERTY_COMPLETED = "STATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPLETED, NAME_COMPLETED);
    }

    /**
     * Column NOTIFIED_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFIED_FLAG = 9;

    public static final int TYPE_NOTIFIED_FLAG = Types.CHAR;

    public static final String NAME_NOTIFIED_FLAG = "WORKFLOW_REPLIES.NOTIFIED_FLAG";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NOTIFIED_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFIED_FLAG,NAME_NOTIFIED_FLAG);}

    /**
     * Column MARKED_READ of type Types.CHAR mapped to String.
     */
    public static final int ID_MARKED_READ = 10;

    public static final int TYPE_MARKED_READ = Types.CHAR;

    public static final String NAME_MARKED_READ = "WORKFLOW_REPLIES.MARKED_READ";

    public static final String PROPERTY_MARKED_READ = "READ";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MARKED_READ, SqlUtil.getBooleanValue(
            NAME_MARKED_READ, "1"));
    }

    /**
     * Column EMAIL_ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EMAIL_ADDRESS = 11;

    public static final int TYPE_EMAIL_ADDRESS = Types.VARCHAR;

    public static final String NAME_EMAIL_ADDRESS = "WORKFLOW_REPLIES.EMAIL_ADDRESS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EMAIL_ADDRESS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EMAIL_ADDRESS,NAME_EMAIL_ADDRESS);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 12;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "WORKFLOW_REPLIES.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "COMMENTS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 13;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "WORKFLOW_REPLIES.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column SECURITY_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SECURITY_ID = 14;

    public static final int TYPE_SECURITY_ID = Types.VARCHAR;

    public static final String NAME_SECURITY_ID = "WORKFLOW_REPLIES.SECURITY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SECURITY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SECURITY_ID,NAME_SECURITY_ID);}

    /**
     * Column LAYOUT_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAYOUT_ID = 15;

    public static final int TYPE_LAYOUT_ID = Types.VARCHAR;

    public static final String NAME_LAYOUT_ID = "WORKFLOW_REPLIES.LAYOUT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAYOUT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAYOUT_ID,NAME_LAYOUT_ID);}

    /**
     * Column WEIGHT_PERCENT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEIGHT_PERCENT = 16;

    public static final int TYPE_WEIGHT_PERCENT = Types.INTEGER;

    public static final String NAME_WEIGHT_PERCENT = "WORKFLOW_REPLIES.WEIGHT_PERCENT";

    public static final String PROPERTY_WEIGHT_PERCENT = "WEIGHT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEIGHT_PERCENT, NAME_WEIGHT_PERCENT);
    }

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 17;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "WORKFLOW_REPLIES.CONFIG_FLAG";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONFIG_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG,NAME_CONFIG_FLAG);}

    /**
     * Column TIMESHEET_MINUTES of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TIMESHEET_MINUTES = 18;

    public static final int TYPE_TIMESHEET_MINUTES = Types.INTEGER;

    public static final String NAME_TIMESHEET_MINUTES = "WORKFLOW_REPLIES.TIMESHEET_MINUTES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TIMESHEET_MINUTES = "EFFORT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMESHEET_MINUTES, "("
                + NAME_TIMESHEET_MINUTES + "/60)");
    }

    public static final String TABLE_NAME = "WORKFLOW_REPLIES";

    /**
     * Create an array of type string containing all the fields of the
     * WORKFLOW_REPLIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_WKF_REPLY_ID, NAME_ELEMENT_ID, NAME_GROUP_ID,
            NAME_RESOURCE_ID, NAME_REPLY_TYPE, NAME_REPLY_STATUS,
            NAME_REPLY_DATE, NAME_COMPLETED, NAME_NOTIFIED_FLAG,
            NAME_MARKED_READ, NAME_EMAIL_ADDRESS, NAME_DESCRIPTION,
            NAME_REC_DATETIME, NAME_SECURITY_ID, NAME_LAYOUT_ID,
            NAME_WEIGHT_PERCENT, NAME_CONFIG_FLAG, NAME_TIMESHEET_MINUTES
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new WorkflowResponseManager instance.
     * 
     * @return WorkflowResponseManager
     */
    public WorkflowResponseManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_WKF_REPLY_ID);
        filter.append(" IS NOT NULL ");

        return filter.toString();
    }

    /**
     * WorkflowResponseManager overloads loadBySelect because getFilter does not
     * have messageContext available to filter based on the current user. Since
     * all select statements for loads go through loadBySelect, this is an
     * alternative.
     */
    protected ArrayList loadBySelect(RPMObject parent, RPMObject[] rpmObjects,
            RPMObjectScope rpmScope, MessageContext messageContext,
            String selectStatement, Object[] args, String contextName,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        // Filter to ensure that only the workflow responses to the current
        // logged in resource's communications are loaded
        SqlBuffer userCondition = new SqlBuffer();
        userCondition.append(selectStatement);
        if (selectStatement.toUpperCase().indexOf(" WHERE ") < 0)
        {
            userCondition.appendWhere();
        }
        userCondition.appendSubSelect(NAME_ELEMENT_ID, NAME_ELEMENT_ID,
            TABLE_NAME, NAME_RESOURCE_ID + "='"
                    + getUser(messageContext).getID() + "' AND "
                    + NAME_REPLY_STATUS + " != 3)");

        return super.loadBySelect(parent, rpmObjects, rpmScope, messageContext,
            userCondition.toString(), args, contextName, isDelta);
    }

    public String getPrimaryKey()
    {
        return NAME_WKF_REPLY_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new WorkflowResponse();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WorkflowResponse temp = (WorkflowResponse) rpmObject;

        temp.setID(rs.getString(ID_WKF_REPLY_ID));

        if (isDelta)
        {
            temp.deltaComments(rs.getString(ID_DESCRIPTION));
            temp.deltaRead(decodeRead(rs.getString(ID_MARKED_READ)));
            temp.deltaResponseDate(Manager.getCalendar(rs, ID_REPLY_DATE));
            temp.deltaState((WorkflowResponseState) EnumeratedTypeUtil.getInstance().getType(
                null, WorkflowResponseState.class,
                new Integer(rs.getString(ID_COMPLETED))));
            temp.deltaFolder((CommunicationsFolder) EnumeratedTypeUtil.getInstance().getType(
                null,
                CommunicationsFolder.class,
                new Integer(rs.getInt(ID_REPLY_STATUS)
                        + rs.getInt(ID_REPLY_TYPE))));
            temp.deltaEffort(decodeEffort(rs.getInt(ID_TIMESHEET_MINUTES)));
            temp.deltaVisible(decodeVisible(rs.getString(ID_REPLY_TYPE)));
            temp.deltaWeight(new Integer(rs.getInt(ID_WEIGHT_PERCENT)));
        }
        else
        {
            temp.setComments(rs.getString(ID_DESCRIPTION));
            temp.setRead(decodeRead(rs.getString(ID_MARKED_READ)));
            temp.setResponseDate(Manager.getCalendar(rs, ID_REPLY_DATE));
            temp.setState((WorkflowResponseState) EnumeratedTypeUtil.getInstance().getType(
                null, WorkflowResponseState.class,
                new Integer(rs.getString(ID_COMPLETED))));
            temp.setFolder((CommunicationsFolder) EnumeratedTypeUtil.getInstance().getType(
                null,
                CommunicationsFolder.class,
                new Integer(rs.getInt(ID_REPLY_STATUS)
                        + rs.getInt(ID_REPLY_TYPE))));
            temp.setEffort(decodeEffort(rs.getInt(ID_TIMESHEET_MINUTES)));
            temp.setVisible(decodeVisible(rs.getString(ID_REPLY_TYPE)));
            temp.setWeight(new Integer(rs.getInt(ID_WEIGHT_PERCENT)));
        }

        return rpmObject;
    }

    private Boolean decodeRead(String readString)
    {
        return (readString != null && (readString.compareTo("1") == 0))
                ? Boolean.TRUE : Boolean.FALSE;
    }

    private Boolean decodeVisible(String visibleString)
    {
        return (visibleString != null && (visibleString.compareTo("31") == 0))
                ? Boolean.TRUE : Boolean.FALSE;
    }

    private Integer decodeEffort(int effort)
    {
        return new Integer(effort / 60);
    }

    private String encodeRead(Boolean read)
    {
        return (read != null && read == Boolean.TRUE) ? "1" : "0";
    }

    private Integer encodeEffort(Integer effort)
    {
        return (effort == null) ? new Integer(0) : new Integer(
            effort.intValue() * 60);
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, parentId);

        String resourceId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, resourceId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null && rpmScope instanceof WorkflowResponseScope)
        {
            WorkflowResponseScope wrScope = (WorkflowResponseScope) rpmScope;

            loadParent((WorkflowResponse) rpmObject, wrScope, messageContext,
                relatedFields, index, isDelta, false);

            loadResource((WorkflowResponse) rpmObject, wrScope, messageContext,
                relatedFields, index, isDelta);

            loadResponse((WorkflowResponse) rpmObject, wrScope, messageContext,
                isDelta);
        }

        return rpmObject;
    }

    private void loadParent(WorkflowResponse rpmObject,
            WorkflowResponseScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta,
            boolean methodLoad) throws RPMException, SQLException,
            ParseException
    {
        if (rpmScope != null && rpmScope.getParent() == null && !methodLoad)
            return;

        CommunicationManager manager = (CommunicationManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            GenericCommunication.class.getName());

        // Get the correct Parent ID
        String parentID;
        if (methodLoad)
        {
            parentID = ManagerUtil.getParentId(this, rpmObject, messageContext,
                TABLE_NAME, NAME_WKF_REPLY_ID, NAME_ELEMENT_ID);
        }
        else
        {
            parentID = (String) relatedFields.get(index, ID_ELEMENT_ID);
        }

        RPMObject parent;
        if (rpmObject.getParent() == null)
        {
            parent = new GenericCommunication();
            parent.setID(parentID);
        }
        else
        {
            parent = rpmObject.getParent();
        }

        if (parent.getID() != null)
        {
            parent = manager.loadByPrimaryKey(parent, null, messageContext,
                isDelta);

            if (isDelta)
                rpmObject.deltaParent(parent);
            else
                rpmObject.setParent(parent);
        }
    }

    private void loadResource(WorkflowResponse rpmObject,
            WorkflowResponseScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!rpmScope.isResource())
            return;

        ResourceManager manager = (ResourceManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            Resource.class.getName());

        // Get the correct Resource ID for the author
        String resourceID = (String) relatedFields.get(index, ID_RESOURCE_ID);

        Resource resource;
        if (rpmObject.getResource() == null)
        {
            resource = new Resource();
            resource.setID(resourceID);
        }
        else
        {
            resource = rpmObject.getResource();
        }

        if (resource.getID() != null)
        {
            resource = (Resource) manager.loadByPrimaryKey(resource, null,
                messageContext, isDelta);

            if (isDelta)
                rpmObject.deltaResource(resource);
            else
                rpmObject.setResource(resource);
        }
    }

    private void loadResponse(WorkflowResponse rpmObject,
            WorkflowResponseScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (!rpmScope.isResponse())
            return;

        WorkflowResponseChecklistManager manager = (WorkflowResponseChecklistManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            GenericResponseType.class.getName());

        if (rpmObject.getID() != null)
        {
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                rpmObject, null, null, messageContext, this, new Object[]
                {
                    rpmObject.getID()
                }, null, null, isDelta);

            GenericResponseType[] response = null;

            if (temp != null)
            {
                response = new GenericResponseType[temp.size()];
                temp.toArray(response);

                for (int i = 0; i < response.length; i++)
                {
                    response[i].setParent(rpmObject);
                }
            }

            rpmObject.setResponse(response);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof CommunicationManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID,
                CommunicationManager.NAME_ELEMENT_ID,
                CommunicationManager.TABLE_NAME,
                CommunicationManager.NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    // ************************* External functions *************************//

    public void sendWorkflowResponse(WorkflowResponse response,
            WorkflowResponseScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Load parent
        if (response != null
                && (response.getParent() == null || response.getParent()
                    .getID() == null) && response.getID() != null)
        {
            loadParent(response, null, messageContext, null, 0, false, true);
        }

        // Validate method arguments
        CommunicationMethodCheckpoint.getInstance().validateSendWorkflowResponse(
            response, messageContext);

        if (!messageContext.isSuccessful())
            return;

        if (response.getParent() instanceof StaffingRequest)
        {
            SP_U_WKF_REPLY(response, messageContext);
        }
        else if (response.getParent() instanceof WorkflowStepNotification)
        {
            if (response.getResponse() != null
                    && response.getResponse().length != 0)
            {
                WorkflowResponseChecklistManager manager = (WorkflowResponseChecklistManager) RPMManagerFactory.getInstance().getRPMObjectManager(
                    GenericResponseType.class.getName());

                if (response.getResponse()[0] instanceof ChecklistItem)
                {
                    for (int i = 0; i < response.getResponse().length; ++i)
                    {
                        manager.SP_U_WKF_CHECKLIST(
                            (ChecklistItem) response.getResponse()[i],
                            messageContext);
                    }
                }
                else
                {
                    String selectedReason = null;
                    int i = 0;
                    for (; i < response.getResponse().length; ++i)
                    {
                        for (int j = 0; ((VoteResponse) response.getResponse()[i]).getReasons() != null
                                && j < ((VoteResponse) response.getResponse()[i]).getReasons().length; ++j)
                        {
                            if (((VoteResponse) response.getResponse()[i]).getReasons()[j].getSelected() == Boolean.TRUE)
                            {
                                selectedReason = ((VoteResponse) response.getResponse()[i]).getReasons()[j].getReason();
                            }
                        }
                    }
                    manager.SP_U_WKF_CHECKLIST(
                        (VoteResponse) response.getResponse()[i - 1],
                        selectedReason, messageContext);
                }
            }

            SP_U_WKF_REPLY_DESC(response, messageContext);

            if (CommunicationMethodCheckpoint.getInstance().validateSaveEffort(
                response, messageContext))
            {
                SP_U_WKF_REPLY_MINUTE(response, messageContext);
            }

            SP_WKF_REPLY_COMPLETED(response, messageContext);
        }

        WorkflowResponse reloadedResponse = (WorkflowResponse) load(response,
            scope, messageContext);

        messageContext.addObject(reloadedResponse);
    }

    public void saveWorkflowResponse(WorkflowResponse response,
            WorkflowResponseScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Load parent
        if (response != null
                && (response.getParent() == null || response.getParent()
                    .getID() == null) && response.getID() != null)
        {
            loadParent(response, null, messageContext, null, 0, false, true);
        }

        // Validate method arguments
        CommunicationMethodCheckpoint.getInstance().validateSaveWorkflowResponse(
            response, messageContext);

        if (!messageContext.isSuccessful())
            return;

        if (response.getResponse() != null
                && response.getResponse().length != 0)
        {
            WorkflowResponseChecklistManager manager = (WorkflowResponseChecklistManager) RPMManagerFactory.getInstance().getRPMObjectManager(
                GenericResponseType.class.getName());

            if (response.getResponse()[0] instanceof ChecklistItem)
            {
                for (int i = 0; i < response.getResponse().length; ++i)
                {
                    manager.SP_U_WKF_CHECKLIST(
                        (ChecklistItem) response.getResponse()[i],
                        messageContext);
                }
            }
        }

        SP_U_WKF_REPLY_DESC(response, messageContext);

        if (CommunicationMethodCheckpoint.getInstance().validateSaveEffort(
            response, messageContext))
        {
            SP_U_WKF_REPLY_MINUTE(response, messageContext);
        }

        WorkflowResponse reloadedResponse = (WorkflowResponse) load(response,
            scope, messageContext);

        messageContext.addObject(reloadedResponse);
    }

    public void updateReadAndFolder(WorkflowResponse response,
            WorkflowResponseScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Validate method arguments and business rules
        CommunicationMethodCheckpoint.getInstance().validateUpdateReadAndFolder(
            response, messageContext);

        if (!messageContext.isSuccessful())
            return;

        if (response.getRead() != null)
        {
            SP_U_REPLY_READ(response, messageContext);
        }

        if (response.getFolder() != null)
        {
            SP_U_WKF_STATUS(response, messageContext);
        }

        WorkflowResponse reloadedResponse = (WorkflowResponse) load(response,
            scope, messageContext);

        messageContext.addObject(reloadedResponse);
    }

    /*
     * Update workflow_replies status (completed and marked_read for
     * notification, marked_read for workflow) completed; '0': not reply '1':
     * yes '2': no
     * 
     * marked_read: '0': not read '1': read
     */
    private void SP_U_REPLY_READ(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_REPLY_READ";
        Object args[] = new Object[3];

        args[0] = ManagerUtil.getParentId(this, workflowResponse,
            messageContext, getTableName());
        args[1] = encodeRead(workflowResponse.getRead());
        args[2] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_WKF_STATUS(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_STATUS";
        Object args[] = new Object[3];

        args[0] = ManagerUtil.getParentId(this, workflowResponse,
            messageContext, getTableName());
        args[1] = EnumeratedTypeUtil.getInstance().getValue(String.class,
            CommunicationsFolder.class, workflowResponse.getFolder().toString());
        args[2] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_WKF_REPLY_DESC(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_REPLY_DESC";
        Object args[] = new Object[4];

        args[0] = workflowResponse.getID();
        args[1] = workflowResponse.getComments();
        args[2] = new Integer(0); // Temporary. 
        // This SP call was modified and support for this new parameter must be added. 
        // See Requirement 62886.
        args[3] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_WKF_REPLY(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_REPLY";
        Object args[] = new Object[4];

        args[0] = workflowResponse.getID();
        args[1] = workflowResponse.getComments();
        args[2] = EnumeratedTypeUtil.getInstance().getValue(null,
            WorkflowResponseState.class, workflowResponse.getState().toString()).toString();
        args[3] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_WKF_REPLY_MINUTE(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_REPLY_MINUTE";
        Object args[] = new Object[3];

        args[0] = workflowResponse.getID();
        args[1] = encodeEffort(workflowResponse.getEffort());
        args[2] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_WKF_REPLY_COMPLETED(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_WKF_REPLY_COMPLETED";
        Object args[] = new Object[3];

        args[0] = workflowResponse.getID();
        args[1] = new Integer(0);
        args[2] = CommunicationUtil.getResourceId(workflowResponse,
            messageContext);

        executeProcedure(messageContext, procedureName, args);
    }
}
