/*
 * Created on Oct 17, 2005
 *
 */
package com.ibm.rpm.contract.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.ContractType;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;

/**
 * @author shiv
 *  
 */
public class ContractManager extends AbstractRPMObjectManager
{

    public static final String DEFAULT_CONTRACT_TYPE = "6701";

    private static Log logger = LogFactory.getLog(ContractManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {

        CONTAINERS.add(GenericContract.class.getName());
        CONTAINERS.add(ContractFolder.class.getName());
        CONTAINERS.add(Contract.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ContractManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */

    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_CONTRACTS.LEVEL_ID";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_CONTRACTS.TYPE_ID";

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_CONTRACTS.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_CONTRACTS.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_CONTRACTS.DELETED_COUNT";

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */

    public static final int ID_RANK = 7;

    public static final int TYPE_RANK = Types.CHAR;

    public static final String NAME_RANK = "TMT_CONTRACTS.RANK";

    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CONTRACTS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_CONTRACTS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_CONTRACTS.PARENT_ID";

    public static final int ID_CONTRACT_TYPE = 10;

    public static final int TYPE_CONTRACT_TYPE = Types.INTEGER;

    public static final String NAME_CONTRACT_TYPE = "TMT_CONTRACTS.CONTRACT_TYPE";

    public static final int ID_CLIENT_ID = 11;

    public static final int TYPE_CLIENT_ID = Types.CHAR;

    public static final String NAME_CLIENT_ID = "TMT_CONTRACTS.CLIENT_ID";

    /**
     * Column MAX_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MAX_AMOUNT = 12;

    public static final int TYPE_MAX_AMOUNT = Types.DECIMAL;

    public static final String NAME_MAX_AMOUNT = "TMT_CONTRACTS.MAX_AMOUNT";
    
    public static final String PROPERTY_MAX_AMOUNT = "MAXIMUMAMOUNT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MAX_AMOUNT, NAME_MAX_AMOUNT);
    }

    public static final int ID_CURRENCY_ID = 13;

    public static final int TYPE_CURRENCY_ID = Types.INTEGER;

    public static final String NAME_CURRENCY_ID = "TMT_CONTRACTS.CURRENCY_ID";

    public static final int ID_CHARGED_AMOUNT = 14;

    public static final int TYPE_CHARGED_AMOUNT = Types.DECIMAL;

    public static final String NAME_CHARGED_AMOUNT = "TMT_CONTRACTS.CHARGED_AMOUNT";

    public static final int ID_LAST_CHARGED_DATE = 15;

    public static final int TYPE_LAST_CHARGED_DATE = Types.DATE;

    public static final String NAME_LAST_CHARGED_DATE = "TMT_CONTRACTS.LAST_CHARGED_DATE";

    /**
     * Column EXTERNAL_REFERENCE of type Types.CHAR mapped to String.
     */
    public static final int ID_EXTERNAL_REFERENCE = 16;

    public static final int TYPE_EXTERNAL_REFERENCE = Types.CHAR;

    public static final String NAME_EXTERNAL_REFERENCE = "TMT_CONTRACTS.EXTERNAL_REFERENCE";
    
    public static final String PROPERTY_EXTERNAL_REFERENCE = "EXTERNALREFERENCE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_REFERENCE, NAME_EXTERNAL_REFERENCE);
    }
    

    public static final int ID_REC_USER = 17;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_CONTRACTS.REC_USER";

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 18;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_CONTRACTS.REC_STATUS";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 19;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_CONTRACTS.REC_DATETIME";

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 20;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_CONTRACTS.DESCRIPTION";
    
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    private static final String TABLE_NAME = "TMT_CONTRACTS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_CONTRACTS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_RANK, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_CONTRACT_TYPE, NAME_CLIENT_ID,
            NAME_MAX_AMOUNT, NAME_CURRENCY_ID, NAME_CHARGED_AMOUNT,
            NAME_LAST_CHARGED_DATE, NAME_EXTERNAL_REFERENCE, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new ContractManager instance.
     * 
     * @return ContractManager
     */
    public ContractManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected HashSet getValidContainerNames()
    {
        return CONTAINERS;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;
        switch (typeId)
        {
        case ContractFolder.TYPE_ID:
            object = new ContractFolder();
            break;

        case Contract.TYPE_ID:
            object = new Contract();
            break;

        default:
            RPMException e = new RPMException("TypeId " + typeId
                    + " is not supported");
            throw e.log();
        }
        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericContract temp = (GenericContract) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);
        String externalReference = rs.getString(ID_EXTERNAL_REFERENCE);

        if (name != null)
        {
            name = name.trim();
        }
        if (externalReference != null)
        {
            externalReference = externalReference.trim();
        }
        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            temp.setName(name);
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        if (isContract(rpmObject))
        {

            Contract contract = (Contract) temp;

            if (isDelta)
            {
                contract.deltaDescription(rs.getString(ID_DESCRIPTION));
                contract.deltaExternalReference(externalReference);
                contract.deltaMaximumAmount(new Double(rs
                    .getDouble(ID_MAX_AMOUNT)));

            }
            else
            {
                contract.setDescription(rs.getString(ID_DESCRIPTION));
                contract.setExternalReference(externalReference);
                contract.setMaximumAmount(new Double(rs
                    .getDouble(ID_MAX_AMOUNT)));

            }
        }
        return temp;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        String contractTypeId = resultSet.getString(ID_CONTRACT_TYPE);
        String currencyTypeId = resultSet.getString(ID_CURRENCY_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        relatedFields.put(index, ID_CONTRACT_TYPE, contractTypeId);
        relatedFields.put(index, ID_CURRENCY_ID, currencyTypeId);
        return relatedFields;
    }

    /**
     *  
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject != null)
        {
            SP_D_CONTRACT(rpmObject, messageContext);
        }
    }

    private String determineTypeId(String containerName) throws RPMException
    {
        StringBuffer typeIdWhere = new StringBuffer();
        int typeId = 0;

        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Contract.class)))
            {
                typeId = Contract.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ContractFolder.class)))
            {
                typeId = ContractFolder.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(GenericContract.class)))
            {
                typeIdWhere.append(NAME_TYPE_ID);
                typeIdWhere.append(" in ( ").append(Contract.TYPE_ID);
                typeIdWhere.append(" , ").append(ContractFolder.TYPE_ID);
                typeIdWhere.append(" )");

                return typeIdWhere.toString();
            }
            else
            {
                ExceptionUtil.handleNoTypeId(this, containerName);
            }
        }

        if (typeId != 0)
            typeIdWhere.append(NAME_TYPE_ID).append(" = ").append(typeId);
        return typeIdWhere.toString();
    }

    /**
     *  
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        String typeIdWhere = determineTypeId(containerName);

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(typeIdWhere);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

            //TODO do not use RANK field in the join condition
        }
        else if (manager instanceof ContractManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            int rankLenght = 4 * level;
            String rank = TABLE_NAME + ".RANK";
            where.append("(LEVEL_ID >= ");
            where.append(level);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ContractChargeScheduleManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            
            where.append(ContractChargeScheduleManager.NAME_CONTRACT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            
            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_CURRENCY_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            
            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_CONTRACT_TYPE);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else
        {
            logger.error(this.getClass().getName()
                    + ".getJoinCondition() is not implemented for manager "
                    + manager.getClass().getName());
        }
        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ContractManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(getPrimaryKey());
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ContractModuleManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {
            ContractScope scope = (ContractScope) rpmScope;

            //Parent
            if (scope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                RPMObject newParent = null;
                if (isDelta)
                {
                    newParent = rpmObject.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new ContractFolder();
                        oldParent.setID(oldParentId);
                        oldParent.setContextName(getContextName(oldParent));
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = loadByPrimaryKey(newParent, scope
                                .getParent(), messageContext, true);
                        }
                        else
                        {
                            oldParent = loadByPrimaryKey(oldParent, null,
                                messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {
                            newParent = new ContractFolder();
                            newParent.setID(oldParentId);
                            newParent.setContextName(getContextName(newParent));
                            newParent = loadByPrimaryKey(newParent, scope
                                .getParent(), messageContext, false);
                        }
                        rpmObject.setParent(newParent);
                    }
                }
            }
            
            //load children
            if (isFolder(rpmObject))
            {
                ContractFolder folder = (ContractFolder) rpmObject;
                if (scope.getChildren() != null)
                {
                    loadChildren(folder, scope.getChildren(), messageContext,
                        isDelta);
                }

            }
            else if (isContract(rpmObject))
            {
                Contract contract = (Contract) rpmObject;
                ContractScope cs = (ContractScope) rpmScope;
                
                if (cs.isCurrency())
                {
                    loadCurrency(contract, rpmScope, messageContext, relatedFields,
                        index, isDelta);
                }
                if (cs.isContractType())
                {
	                loadContractType(contract, rpmScope, messageContext,
	                    relatedFields, index, isDelta);
                }

                if (cs != null && cs.isChargeSchedules())
                {
                    loadContractChargeSchedule(contract, rpmScope,
                        messageContext, relatedFields, index, isDelta);
                }
            }
        }

        return rpmObject;
    }

    private void loadContractChargeSchedule(Contract contract,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ContractChargeScheduleManager manager = (ContractChargeScheduleManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                ContractChargeSchedule.class.getName());

        Object[] args = new Object[]
        {
            contract.getID()
        };

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(contract,
            null, rpmScope, messageContext, this, args, null,
            getContextName(contract), isDelta);
        ContractChargeSchedule[] ccs = null;
        if (temp != null && !temp.isEmpty())
        {
            setParent(temp, contract);
            ccs = new ContractChargeSchedule[temp.size()];
            temp.toArray(ccs);
            contract.setChargeSchedules(ccs);
        }
    }

    public void saveContractChargeSchedule(Contract contract,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ContractScope scope = (ContractScope) rpmScope;

        contract.setChargeSchedules((ContractChargeSchedule[]) updateChildren(
            contract, contract.getChargeSchedules(), scope.isChargeSchedules(),
            messageContext, null));

    }

    private void loadCurrency(Contract contract, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        Currency currency = new Currency();
        currency.setID((String) relatedFields.get(index, ID_CURRENCY_ID));

        if (!currency.getID().equals("0"))
        {
            RPMObjectManager manager = RPMManagerFactory
            	.getInstance().getRPMObjectManager(Currency.class);

            currency = (Currency) manager.loadByPrimaryKey(currency, null,
                messageContext, isDelta);

            if (isDelta)
            {
                contract.deltaCurrency(currency);
            }
            else
            {
                contract.setCurrency(currency);
            }
        }
    }

    private void loadContractType(Contract contract, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ContractType ct = new ContractType();
        ct.setID((String) relatedFields.get(index, ID_CONTRACT_TYPE));

        if (!ct.getID().equals("0"))
        {
            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(ContractType.class.getName());

            ct = (ContractType) manager.loadByPrimaryKey(ct, null,
                messageContext, isDelta);

            if (isDelta)
            {
                contract.deltaContractType(ct);
            }
            else
            {
                contract.setContractType(ct);
            }
        }
    }

    private String getContextName(RPMObject rpmObject) throws RPMException
    {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }

    private void loadChildren(ContractFolder folder, ContractScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        GenericContract[] oldChildren = folder.getChildren();
        ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
            this, new Object[]
            {
                folder.getID()
            }, null, getContextName(folder), isDelta);

        GenericContract[] children = null;
        if (temp != null)
        {
            children = new GenericContract[temp.size()];
            temp.toArray(children);
        }
        folder.setChildren(children);

    }

    public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ContractScope scope = (ContractScope) rpmScope;

        if (scope != null && scope.getParent() != null)
        {
            RPMObject parent = rpmObject.getParent();
            if (parent != null)
            {
                internalSave(parent, scope.getParent(), messageContext);
            }
        }
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        // Set default values
        if (rpmObject instanceof Contract)
        {
            if (((Contract) rpmObject).getContractType() == null)
            {
                ContractType contractType = new ContractType();
                contractType.setID(DEFAULT_CONTRACT_TYPE);
                ((Contract) rpmObject).setContractType(contractType);
            }

        }

        rpmObject.setID(SP_I_CONTRACT((GenericContract) rpmObject,
            messageContext));
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        ContractScope scope = (ContractScope) rpmScope;
        if (((GenericContract) rpmObject).testNameModified())
        {
            SP_U_CONTRACT((GenericContract) rpmObject, messageContext);
        }

        if (!isContract(rpmObject))
        {
            if (scope != null)
            {
                rpmObject = updateChildren(rpmObject, scope, messageContext,
                    false);
            }
        }
        else
        {

            Contract contract = (Contract) rpmObject;
            if (!((GenericContract) rpmObject).testNameModified()
                    && (contract.testExternalReferenceModified() || contract
                        .testDescriptionModified()))
            {
                SP_U_CONTRACT((GenericContract) rpmObject, messageContext);
            }
            if (contract.testCurrencyModified())
            {
                SP_U_CONTRACT_CURRENCY(contract, messageContext);
            }
            if (contract.testContractTypeModified())
            {
                SP_U_CONTRACT_TYPE(contract, messageContext);
            }
            if (contract.testMaximumAmountModified()
                    && (contract.getContractType() != null && "6702"
                        .equals(contract.getContractType().getID())))
            {
                SP_U_CONTRACT_MAX(contract, messageContext);
            }
            // From here for 2nd level

            // ChargeSchedule
            if (scope != null && scope.isChargeSchedules())
            {
                saveContractChargeSchedule(contract, rpmScope, messageContext);
            }

        }
        return rpmObject;
    }

    protected RPMObject updateChildren(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean saveParent) throws RPMException, SQLException,
            ParseException
    {

        ContractScope scope = (ContractScope) rpmScope;

        if (isFolder(rpmObject))
        {
            ContractFolder folder = (ContractFolder) rpmObject;

            // save children
            GenericContract[] children = folder.getChildren();

            updateChildren(folder, children, scope.getChildren(),
                messageContext, GenericContract.class);

        }
        return rpmObject;
    }

    private boolean isFolder(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ContractFolder.TYPE_ID;
    }

    private boolean isContract(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == Contract.TYPE_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getSubTableJoinCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager instanceof TaskAssignmentManager)
        {
            // example: /TaskAssignment[Contract[ID='123']]
            joinCondition = JoinCondition.createSubSelect(
                TaskAssignmentManager.NAME_CONTRACT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, condition);
        }
        else if (manager instanceof ContractManager)
        {
            // example: /ContractFolder[GenericContract[ID='123']]
            joinCondition = JoinCondition.createSubSelect(NAME_ELEMENT_ID,
                NAME_PARENT_ID, TABLE_NAME, condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    private String SP_I_CONTRACT(GenericContract rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_CONTRACT";
        Object args[] = new Object[4];
        args[0] = rpmObject.getName();
        if (rpmObject.getParent() != null)
        {
            args[1] = rpmObject.getParent().getID();
        }
        args[2] = rpmObject.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    /**
     * @param contract
     * @param messageContext
     */
    private String SP_U_CONTRACT_TYPE(Contract contract,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_CONTRACT_TYPE";
        Object args[] = new Object[3];
        args[0] = contract.getID();
        Integer id = new Integer((contract.getContractType()).getID());
        args[1] = id;
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * @param contract
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     * @throws RPMException
     * @throws Exception
     */
    private String SP_U_CONTRACT_CURRENCY(Contract contract,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String elementId = null;

        String name = "SP_U_CONTRACT_CURRENCY";
        Object args[] = new Object[3];
        args[0] = contract.getID();
        Integer id = new Integer((contract.getCurrency()).getID());
        args[1] = id;
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    private String SP_U_CONTRACT(GenericContract rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_CONTRACT";
        Object args[] = new Object[5];
        args[0] = rpmObject.getID();
        args[1] = rpmObject.getName();
        if (isContract(rpmObject))
        {
            args[2] = ((Contract) rpmObject).getExternalReference();
            args[3] = ((Contract) rpmObject).getDescription();
        }
        args[4] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private void SP_D_CONTRACT(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_CONTRACT";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = ManagerUtil.getParentId(this, rpmObject, messageContext,
            getTableName());
        args[2] = "X";
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        executeProcedure(messageContext, name, args);
    }

	//private String SP_U_CONTRACT_NAME(Contract rpmObject,
	//        MessageContext messageContext) throws SQLException, RPMException
	//{
	//
	//    String elementId = null;
	//
	//    String name = "SP_U_CONTRACT_NAME";
	//    Object args[] = new Object[3];
	//    args[0] = rpmObject.getID();
	//    args[1] = rpmObject.getName();
	//    args[2] = getUser(messageContext).getID();
	//    elementId = executeProcedure(messageContext, name, args);
	//
	//    return elementId;
	//}

    private String SP_U_CONTRACT_MAX(Contract rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;

        String name = "SP_U_CONTRACT_MAX";
        Object args[] = new Object[3];
        args[0] = rpmObject.getID();
        args[1] = rpmObject.getMaximumAmount();
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

	//private String SP_I_CONTRACT_CCB(Contract rpmObject,
	//        MessageContext messageContext) throws SQLException, RPMException
	//{
	//    String elementId = null;
	//    String name = "SP_I_CONTRACT_CCB";
	//    Object args[] = new Object[2];
	//    args[0] = rpmObject.getID();
	//    args[1] = getUser(messageContext).getID();
	//    elementId = executeProcedure(messageContext, name, args);
	//    return elementId;
	//}

    // Checkout/Checkin not applicable for Contracts
}