/*
 * Created on Feb 21, 2006
 *
 */
package com.ibm.rpm.contract.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.ContractType;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.contract.containers.ContractModule;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.contract.scope.ContractModuleScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;

/**
 * @author karim
 *
 */
public class ContractModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(ContractModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ContractModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ContractModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.NONE;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new ContractModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof ContractManager)
        {
                joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ContractModule module = (ContractModule) abstractModule;
        ContractModuleScope scope = (ContractModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);


        if(scope!=null)
        {
            if(scope.getContracts()!=null)
            {
                String extraWhere = " LEVEL_ID = 0 ";
            	GenericContract[] genericContracts = (GenericContract[]) loadElements(module, module.getContracts(),
            			scope.getContracts(),GenericContract.class, 
    					null, extraWhere, messageContext, isDelta );        
                module.setContracts(genericContracts);
            }
            if(scope.isContractTypes())
            {
            	ContractType[] contractTypes = (ContractType[]) datafieldManager
                    .loadDatafields(module, module.getContractTypes(),
                        ContractType.class, ContractType.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setContractTypes(contractTypes);
            }
        }
    }
    

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        ContractModuleScope scope = (ContractModuleScope) rpmScope;
        ContractModule module = (ContractModule) rpmObject;
        module.setContextName(localContextName);
        
        if(scope!=null)
        {
            if(scope.getContracts()!=null)
            {
                GenericContract[] contracts = (GenericContract[]) saveElements(module, module.getContracts(),
            			scope.getContracts(),messageContext, GenericContract.class);        
                module.setContracts(contracts);
            }
        }
        return module;
    }
}
