/*
 * Created on Sep 12, 2006
 */
package com.ibm.rpm.customfield.checkpoints;

import com.ibm.rpm.customfield.constants.ValidationConstants;
import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class CustomPortletCheckpoint extends AbstractCheckpoint
{
    private static final Class CUSTOM_PORTLETS_PARENTS = CustomPortletCategory.class;

    protected static CustomPortletCheckpoint instance = new CustomPortletCheckpoint();

    public static CustomPortletCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (rpmObject instanceof CustomPortletCategory)
        {
            isValidCategory(rpmObject, msgCtx);
        }
        else if (rpmObject instanceof CustomPortlet)
        {
            isValidCustomPortlet(rpmObject, msgCtx);
        }
        else
        {
            GenericValidator.validateInstance(rpmObject,
                GenericCustomPortlet.class, msgCtx);
        }
    }
    
    private void isValidCustomPortlet(RPMObject rpmObject, MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, CUSTOM_PORTLETS_PARENTS);
        if (!GenericValidator.validateInstance(rpmObject, CustomPortlet.class,
            msgCtx))
        {
            return;
        }

        CustomPortlet customPortlet = (CustomPortlet) rpmObject;
        isValidName(customPortlet, msgCtx);
    }

    private void isValidCategory(RPMObject rpmObject, MessageContext msgCtx)
    {
        if (!GenericValidator.validateInstance(rpmObject,
            CustomPortletCategory.class, msgCtx))
        {
            return;
        }

        CustomPortletCategory category = (CustomPortletCategory) rpmObject;
        isValidName(category, msgCtx);
        GenericValidator.validateMandatory(category, "groupType",
            category.getGroupType(), msgCtx);
    }

    private void isValidName(GenericCustomPortlet customPortlet,
            MessageContext msgCtx)
    {
        if (customPortlet.testNameModified())
        {
            GenericValidator.validateMaxLength(customPortlet, "name",
                customPortlet.getName(),
                ValidationConstants.CUSTOMFIELD_NAME_MAX, msgCtx);
        }
    }
}
