package com.ibm.rpm.customfield.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputTypeToDBMap;
import com.ibm.rpm.financial.managers.FinancialManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;

/**
 * @author ikusakov
 *  
 */
public class CustomFieldAssignmentManager extends AbstractRPMObjectManager
{
    public static final int SC_CUSTOM_FOLDER = 145;

    protected static Log logger = LogFactory.getLog(FinancialManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        CustomFieldAssignmentManager.class);

    static
    {
        containerMap.add(CustomFieldAssignment.class);
    }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PARENT";
    //static
    //{
    //    FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    //}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 5;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 6;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 7;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 8;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 9;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "ELEMENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column FOREIGN_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_FOREIGN_ID = 12;

    public static final int TYPE_FOREIGN_ID = Types.CHAR;

    public static final String NAME_FOREIGN_ID = "FOREIGN_ID";

    public static final String PROPERTY_FOREIGN_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FOREIGN_ID, NAME_FOREIGN_ID);
    }

    /**
     * Column FIELD_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_FIELD_ID = 13;

    public static final int TYPE_FIELD_ID = Types.CHAR;

    public static final String NAME_FIELD_ID = "FIELD_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FIELD_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FIELD_ID,NAME_FIELD_ID);}

    /**
     * Column FIELD_VALUE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FIELD_VALUE = 14;

    public static final int TYPE_FIELD_VALUE = Types.VARCHAR;

    public static final String NAME_FIELD_VALUE = "FIELD_VALUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_FIELD_VALUE = "VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FIELD_VALUE, NAME_FIELD_VALUE);
    }

    /**
     * Create an array of type string containing all the fields of the
     * TMT_RESOURCE_CUSTOM_FIELDS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_LEVEL_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
            NAME_ELEMENT_NAME, NAME_REC_STATUS, NAME_REC_USER, NAME_FOREIGN_ID,
            NAME_FIELD_ID, NAME_FIELD_VALUE
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public CustomFieldAssignmentManager()
    {
        super();
    }

    public String getTableName()
    {
        ExceptionUtil.handleGetTableName(this);
        return null;
    }

    public String getTableName(RPMObject rpmObject)
    {
        return CustomFieldAssignmentUtil.getTableName(rpmObject
                .getContextName());
    }

    public String getTableName(String contextName)
    {
        return CustomFieldAssignmentUtil.getTableName(contextName);
    }

    protected Integer getTableType(RPMObject rpmObject)
    {

        return CustomFieldAssignmentUtil.getTableType(rpmObject
                .getContextName());
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    public String getFilter(String contextName)
    {
        String table = CustomFieldAssignmentUtil.getTableName(contextName);
        String typeIdFieldName = (table == null) ? NAME_TYPE_ID : table + '.'
                + NAME_TYPE_ID;
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");
        filter.append(" AND " + typeIdFieldName + " <> " + SC_CUSTOM_FOLDER);
        filter.append(" AND " + NAME_ELEMENT_ID + " IS NOT NULL");

        return filter.toString();
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {
        case 0:
            object = new CustomFieldAssignment();
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject assignment, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {

        //Get contextName from the parent if contextName is null.
        String contextName = assignment.getContextName();
        if (contextName == null)
        {
            RPMObject parent = assignment.getParent();
            //parent should never be null because it is validated as mandatory
            if (parent != null)
            {
                String parentContext = parent.getContextName();
                //Get the context from the map if the type is not
                // multiple-context
                if (parentContext == null)
                {
                    String[] contextNames = ContextUtil.getInstance()
                        .getContextNames(parent.getClass().getName());
                    if (contextNames.length == 1)
                    {
                        parentContext = contextNames[0];
                    }
                    else
                    {
                        //Should never happen. Parent with an id should have a contextName.
                        
                    }
                }

                assignment.setContextName(parentContext);
            }
            else
            {
                //Should never happen. parent is mandatory
                RPMException e = new RPMException(ErrorCodes.NO_PARENT, null,
                    assignment.getClass().getName(), "parent", assignment.getID());
                throw e;
            }
        }
        
        
        if (ContextUtil.getInstance().isDocumentContext(
                assignment.getContextName()))
        {
            assignment.setContextName(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        }
        assignment.setID(SP_A_CUSTOM_FIELDS((CustomFieldAssignment) assignment,
                messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        CustomFieldAssignment assignment = (CustomFieldAssignment) rpmObject;

        assignment.setID(rs.getString(ID_ELEMENT_ID));

        return assignment;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        relatedFields = super.decodeRelatedFields(resultSet, relatedFields,
                index);

        relatedFields.put(index, ID_FOREIGN_ID, resultSet
                .getString(ID_FOREIGN_ID));

        relatedFields.put(index, ID_FIELD_ID, resultSet.getString(ID_FIELD_ID));
        relatedFields.put(index, ID_FIELD_VALUE, resultSet
                .getString(ID_FIELD_VALUE));

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        CustomFieldAssignment assignment = (CustomFieldAssignment) rpmObject;

        if (rpmObject.getContextName() == null)
        {
            ExceptionUtil.handleUnsupportedContext(this, rpmObject,
                    "loadRelatedObjects");
        }

        // Parent and CustomField are read-only
        if (!isDelta)
         {
            
            //parent is loaded if it is not already set
            if(assignment.getParent() == null){
                
                String id = (String) relatedFields.get(index, ID_FOREIGN_ID);
                RPMObject parent = CustomFieldAssignmentUtil.createParentInstance(assignment.getContextName());
                parent.setID(id);
                RPMObjectManager manager = getManagerInstance(parent.getClass());                
                parent = manager.loadByPrimaryKey(parent, null, messageContext,
                    isDelta);
                
                //parent can be either a WorkElement or a ScopeElement
                if(parent == null && ContextUtil.WBS_CONTEXT.equals(assignment.getContextName())){
                    parent = CustomFieldAssignmentUtil.createParentInstance(ContextUtil.SCOPE_CONTEXT);
                    parent.setID(id);
                    manager = getManagerInstance(parent.getClass());
                    parent = manager.loadByPrimaryKey(parent, null, messageContext,
                        isDelta);                    
                }
               
                assignment.setParent(parent);            
            }
            
            // CustomField is read-only and is always loaded
            CustomField customField = new CustomField();
            customField.setID((String) relatedFields.get(index, ID_FIELD_ID));
            customField.setContextName(assignment.getContextName());
            CustomFieldScope cfscope = null;

            CustomFieldManager manager = (CustomFieldManager) getManagerInstance(CustomField.class
                    .getName());
            
            // Hack to support broken links and customFieldAssignments with
            // invalid CustomFields in the database
            try {
                customField = (CustomField) manager.loadByPrimaryKey(customField,
                    cfscope, messageContext, isDelta);

                //Check if the customField type matches the parent type (e.g.
                // excludes Resource with a WBS customField)
                Integer parentType = getTableType(rpmObject);

                SqlBuffer sql = new SqlBuffer();
                sql.appendEqualQuestionMark(CustomFieldManager.NAME_ELEMENT_ID);

                Object[] arguments = new Object[]
                {
                    customField.getID()
                };
                
                Integer tableType = (Integer) ManagerUtil.selectColumnValue(
                    Integer.class, customField, null,
                    CustomFieldManager.NAME_TABLE_TYPE, CustomFieldManager.TABLE_NAME, sql, arguments, messageContext);
                
                if (!parentType.equals(tableType))
                {
                    customField = null;
                }
                assignment.setCustomField(customField);
                
            } catch(Exception e) {
                // do nothing, null values will be filtered out by the method 
                // in corresponding Util class
            }
        }
        Object newValue = null;
        
        if (assignment.getCustomField() != null)
        {
            newValue = FieldInputTypeToDBMap.dbStringToObject(assignment,
                "value", assignment.getCustomField().getInputType(),
                (String) relatedFields.get(index, ID_FIELD_VALUE),
                messageContext);
        }

        if (isDelta)
        {
            assignment.deltaValue(newValue);
        }
        else
        {
            assignment.setValue(newValue);
        }

        return assignment;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        SP_UA_CUSTOM_FIELDS((CustomFieldAssignment) rpmObject, messageContext);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (CustomFieldAssignmentUtil.isParentManager(manager))
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            where.append(getFilter(contextName));
            where.append(" AND ");
            where.append(getTableName(contextName));
            where.append(".");
            where.append(NAME_FOREIGN_ID);
            where.append(" = ? ");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                where.append(sb.toString());
            }

            joinCondition.setTableName(getTableName(contextName));
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getSubTableJoinCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getSubTableJoinCondition(
            RPMObjectManager parentManager, String condition, int level,
            String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (CustomFieldAssignmentUtil.isParentManager(parentManager))
        {
            // XPath example: /Resource[CustomField[Id='.....']]
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(CustomFieldAssignmentUtil
                    .getParentPKColumn(contextName));
            where.append(" IN (SELECT ");
            // no need to prefix with table name in subselect
            where.append(NAME_FOREIGN_ID);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            where.append(" WHERE ");
            // no need to prefix with table name in subselect
            where.append(getFilter(contextName));
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, parentManager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        CustomFieldAssignment assignment = (CustomFieldAssignment) rpmObject;
                
        if (assignment.testValueModified())
        {
            SP_U_A_CUSTOM_FIELDS(assignment, messageContext);
        }

        return assignment;
    }
    
    

    /**
     *
     */

    public ArrayList load(Stack tableStack, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        
        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }

        
        ArrayList objects = super.load(tableStack, rpmScope, messageContext);
            
        // Hack to support broken links and customFieldAssignments with
        // invalid parent in the database
        
        //We will keep only objects that have a parent and a custom field.
        if(objects != null && !objects.isEmpty()){
            
            for (Iterator iter = objects.iterator(); iter.hasNext();)
            {
                CustomFieldAssignment element = (CustomFieldAssignment) iter.next();
                if(element.getParent() == null || element.getCustomField() == null){
                    iter.remove();
                }                
            }
        }
        
        
        return objects;

    }
    
    private String SP_A_CUSTOM_FIELDS(CustomFieldAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_A_CUSTOM_FIELDS";

        Object args[] = new Object[4];

        // table_type : SC_LONG_PARAM
        args[0] = CustomFieldAssignmentUtil.getParentTableType(assignment
                .getContextName());

        // element_id : SC_WCHAR (element_id in wbs, cri, doc,
        // resource, client, asset )
        if (assignment.getParent() == null
                || assignment.getParent().getID() == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.OBJECT_IS_NULL);
        }
        args[1] = assignment.getParent().getID();

        // custom_field_id : SC_WCHAR
        if (assignment.getCustomField() == null
                || assignment.getCustomField().getID() == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.OBJECT_IS_NULL);
        }
        args[2] = assignment.getCustomField().getID();

        args[3] = getUser(messageContext).getID();

        String objectId = executeProcedure(messageContext, procedureName, args);

        return objectId;
    }

    private void SP_UA_CUSTOM_FIELDS(CustomFieldAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_UA_CUSTOM_FIELDS";

        Object args[] = new Object[4];

        // table_type : SC_LONG_PARAM
        args[0] = CustomFieldAssignmentUtil.getParentTableType(assignment
                .getContextName());

        // parent_id : SC_WCHAR (element_id in wbs, cri, doc,
        // fresource, client, asset )
        if (assignment.getParent() == null
                || assignment.getParent().getID() == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.OBJECT_IS_NULL);
        }
        args[1] = assignment.getParent().getID();

        // element_id : SC_WCHAR (element_id in
        // TMT_WBS_CRI_CUSTOM_FIELDS )
        if (assignment.getID() == null)
        {
            ExceptionUtil.handleRPMException(this,
                    ErrorCodes.MISSING_PRIMARY_KEY);
        }
        args[2] = assignment.getID();

        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_A_CUSTOM_FIELDS(CustomFieldAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        final String procedureName = "SP_U_A_CUSTOM_FIELDS";

        Object args[] = new Object[4];

        // table_type : SC_LONG_PARAM
        args[0] = CustomFieldAssignmentUtil.getParentTableType(assignment
                .getContextName());

        // element_id : SC_WCHAR (element_id in
        // TMT_WBS_CRI_CUSTOM_FIELDS )
        if (assignment.getID() == null)
        {
            ExceptionUtil.handleRPMException(this,
                    ErrorCodes.MISSING_PRIMARY_KEY);
        }
        args[1] = assignment.getID();

        if (assignment.getCustomField() == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.OBJECT_IS_NULL);
        }
        // value : SC_WCHAR
        args[2] = FieldInputTypeToDBMap.objectToDBString(assignment
                .getCustomField().getInputType(), assignment.getValue());

        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }
}
