package com.ibm.rpm.customfield.managers;

import java.lang.reflect.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.customfield.scope.CustomPortletScope;
import com.ibm.rpm.customfield.types.CustomPortletCategoryGroupType;
import com.ibm.rpm.customfield.types.GroupTypeToDBMap;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.xpathparser.XPathContainer;

public class CustomPortletManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        CustomPortletManager.class);

    static
    {
        containerMap.add(GenericCustomPortlet.class);
        containerMap.add(CustomPortletCategory.class);
        containerMap.add(CustomPortlet.class);
    }

    private static Map GROUP_TYPE_TO_GROUP_ID_MAP;
    static
    {
        GROUP_TYPE_TO_GROUP_ID_MAP = new HashMap();
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._AssetGroup,
            "SC_ASSET_CUSTOM_FIELD___________");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._ClientGroup,
            "SC_CLIENT_CUSTOM_FIELD__________");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._DocumentGroup,
            "SC_DOCUMENT_CUSTOM_FIELD________");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._ResourceGroup,
            "SC_RESOURCE_CUSTOM_FIELD________");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._ScopeGroup,
            "SC_SCOPE_CUSTOM_FIELD___________");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._ProjectGroup,
            "SC_PROJECT_CUSTOM_PORTLET_______");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._DeliverableGroup,
            "SC_DEL_CUSTOM_FIELD_FOLDER______");
        GROUP_TYPE_TO_GROUP_ID_MAP.put(
            CustomPortletCategoryGroupType._TaskGroup,
            "SC_TASK_CUSTOM_FIELD____________");
    }

    private String getGroupId(CustomPortletCategory category)            
    {
        return (String) GROUP_TYPE_TO_GROUP_ID_MAP.get(category
            .getGroupType().getValue());
    }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_CUSTOM_PORTLETS.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_CUSTOM_PORTLETS.PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CUSTOM_PORTLETS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_CUSTOM_PORTLETS.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 5;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_CUSTOM_PORTLETS.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 6;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_CUSTOM_PORTLETS.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 7;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_CUSTOM_PORTLETS.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 8;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_CUSTOM_PORTLETS.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 9;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_CUSTOM_PORTLETS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_CUSTOM_PORTLETS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_CUSTOM_PORTLETS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column GROUP_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_GROUP_TYPE = 12;

    public static final int TYPE_GROUP_TYPE = Types.INTEGER;

    public static final String NAME_GROUP_TYPE = "TMT_CUSTOM_PORTLETS.GROUP_TYPE";

    public static final String PROPERTY_GROUP_TYPE = "GROUPTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_GROUP_TYPE, SqlUtil.castAsChar(
            NAME_GROUP_TYPE, 2));
    }

    public static final String TABLE_NAME = "TMT_CUSTOM_PORTLETS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_CUSTOM_PORTLETS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_LEVEL_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
            NAME_ELEMENT_NAME, NAME_REC_STATUS, NAME_REC_USER, NAME_GROUP_TYPE
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new TmtCustomPortletsManager instance.
     * 
     * @return TmtCustomPortletsManager
     */
    public CustomPortletManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public String getFilter()
    {
        return NAME_ELEMENT_ID + " IS NOT NULL";
    }

    public String getForeignKey(RPMObject object)
    {
        if ((object instanceof CustomPortletCategory)
                || (object instanceof CustomPortlet))
            return NAME_PARENT_ID;
        return null;
    }

    private SqlBuffer getContainerFilter(Class containerClass)
            throws RPMException
    {
        SqlBuffer filter = new SqlBuffer(getFilter());
        if (GenericCustomPortlet.class.equals(containerClass))
        {
            filter.appendCondition(NAME_LEVEL_ID + ">0");
        }
        else if (CustomPortlet.class.equals(containerClass))
        {
            filter.appendEqual(NAME_TYPE_ID, CustomPortlet.TYPE_ID);
        }
        else if (CustomPortletCategory.class.equals(containerClass))
        {
            filter.appendEqual(NAME_TYPE_ID, CustomPortletCategory.TYPE_ID);
        }
        return filter;
    }

    private String getFinalCondition(XPathContainer container)
            throws RPMException
    {
        // Add the container filter, if required.
        SqlBuffer finalCondition = getContainerFilter(container.getType());

        return finalCondition.toString();
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        String finalCondition = getFinalCondition(container);
        JoinCondition joinCondition = new JoinCondition();

        if (manager == null)
        {
            joinCondition.setTableName(getTableName());
            joinCondition.appendCondition(finalCondition);
        }
        else if (manager instanceof CustomPortletManager)
        {
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, finalCondition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        String finalCondition = getFinalCondition(container);

        if (manager instanceof CustomPortletManager)
        {
            joinCondition = JoinCondition.createSubSelect(NAME_ELEMENT_ID,
                NAME_PARENT_ID, TABLE_NAME, finalCondition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericCustomPortlet customPortlet = (GenericCustomPortlet) rpmObject;
        if (isDelta)
        {
            customPortlet.deltaName(rs.getString(ID_ELEMENT_NAME));
            customPortlet.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));            
        }
        else
        {
            customPortlet.setID(rs.getString(ID_ELEMENT_ID));
            customPortlet.setName(rs.getString(ID_ELEMENT_NAME));
            customPortlet.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        return rpmObject;

    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        int groupID = resultSet.getInt(ID_GROUP_TYPE);
        relatedFields.put(index, ID_GROUP_TYPE, GroupTypeToDBMap
            .toGroupType(groupID));
        return super.decodeRelatedFields(resultSet, relatedFields, index);
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {       
        if (rpmObject instanceof CustomPortletCategory)
        {
            CustomPortletCategory current = (CustomPortletCategory) rpmObject;
            CustomPortletCategoryGroupType groupType = (CustomPortletCategoryGroupType) relatedFields
                .get(index, ID_GROUP_TYPE);
            current.setGroupType(groupType);
        }
        
        if (rpmScope != null)
        {
            CustomPortletScope scope = (CustomPortletScope) rpmScope;
            if (rpmObject instanceof CustomPortlet && scope.getParent() != null)
            {
                String parentID = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                RPMObject parent = rpmObject.getParent();
                if (parentID != null)
                {
                    if (parent == null)
                    {
                        parent = new GenericCustomPortlet();
                    }
                    parent.setID(parentID);
                    parent = loadByPrimaryKey(parent, scope.getParent(),
                        messageContext, isDelta);
                }
                rpmObject.setParent(parent);
            }

            if (rpmObject instanceof CustomPortletCategory
                    && scope.isChildren())
            {
                CustomPortletCategory current = (CustomPortletCategory) rpmObject;
                CustomPortletScope childrenScope = new CustomPortletScope();
                childrenScope.setChildren(false);
                current.setChildren((CustomPortlet[]) loadChildren(current,
                    current.getChildren(), childrenScope, CustomPortlet.class,
                    messageContext, isDelta));
            }
        }
        return super.loadRelatedObjects(rpmObject, rpmScope, messageContext,
            relatedFields, index, isDelta);
    }

    private RPMObject[] loadChildren(RPMObject parent, RPMObject[] children,
            CustomPortletScope childrenScope, Class childrenClass,
            MessageContext messageContext, boolean isDelta)
            throws SQLException, RPMException, ParseException
    {
        if (childrenScope == null)
        {
            return null;
        }

        Object[] arguments =
        {
            parent.getID()
        };
        ArrayList childrenList = loadByForeignKey(children, childrenScope,
            messageContext, this, arguments, null, parent.getContextName(),
            isDelta);
        if (childrenList != null)
        {
            int childrenCount = childrenList.size();
            if (childrenCount > 0)
            {
                children = (RPMObject[]) Array.newInstance(childrenClass,
                    childrenCount);
                childrenList.toArray(children);
                for (int i = 0; i < childrenCount; i++)
                {
                    children[i].setParent(parent);
                }
            }
        }
        return children;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof CustomPortletManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(NAME_PARENT_ID + "=?");
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        GenericCustomPortlet genCustomPortlet = (GenericCustomPortlet) rpmObject;
        String parentId = getParentId(genCustomPortlet);
        String id = SP_I_CUSTOM_PORTLET(parentId, genCustomPortlet.getName(),
            genCustomPortlet.retrieveTypeID(), messageContext);
        rpmObject.setID(id);
    }

    private String getParentId(RPMObject rpmObject)
    {
        String parentId = null;
        if (rpmObject instanceof CustomPortletCategory)
        {
            CustomPortletCategory category = (CustomPortletCategory) rpmObject;
            parentId = getGroupId(category);
        }
        else if (rpmObject.getParent() != null)
        {
            parentId = rpmObject.getParent().getID();
        }
        return parentId;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericCustomPortlet customPortlet = (GenericCustomPortlet) rpmObject;
        CustomPortletScope scope = (CustomPortletScope) rpmScope;
        if (customPortlet.testNameModified())
        {
            SP_U_CUSTOM_PORTLET_NAME(customPortlet.getID(), customPortlet
                .getName(), messageContext);
        }

        if (customPortlet instanceof CustomPortletCategory && scope != null)
        {
            CustomPortletCategory category = (CustomPortletCategory) customPortlet;
            if (scope.isChildren())
            {
                CustomPortlet[] children = category.getChildren();
                propagateContextName(category, children);
                category.setChildren((CustomPortlet[]) updateChildren(category,
                    children, scope.isChildren(), messageContext, null));
            }
        }
        return rpmObject;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        String parentId = ManagerUtil.getParentId(this, rpmObject,
            messageContext, TABLE_NAME);
        SP_D_CUSTOM_PORTLET(rpmObject.getID(), parentId, messageContext);
    }

    private void SP_D_CUSTOM_PORTLET(String objectId, String parentId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_D_CUSTOM_PORTLET";
        Object args[] = new Object[4];
        args[0] = objectId;
        args[1] = parentId;
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, procedureName, args);
    }

    private String SP_I_CUSTOM_PORTLET(String parentId, String name,
            Integer typeId, MessageContext messageContext) throws SQLException,
            RPMException
    {
        final String procedureName = "SP_I_CUSTOM_PORTLET";
        Object args[] = new Object[4];
        args[0] = name;
        args[1] = parentId;
        args[2] = typeId;
        args[3] = getUser(messageContext).getID();
        String objectId = executeProcedure(messageContext, procedureName, args);
        return objectId;
    }

    private void SP_U_CUSTOM_PORTLET_NAME(String objectId, String name,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_CUSTOM_PORTLET_NAME";
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = name;
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, procedureName, args);
    }
}
