/*
 * Created on 16-Nov-2005
 */
package com.ibm.rpm.document.checkpoints;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class NoteMinutesCheckpoint extends GenericDocumentCheckpoint
{
    protected static NoteMinutesCheckpoint instance = new NoteMinutesCheckpoint();

    public static NoteMinutesCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        boolean valid = true;

        super.validateSave(rpmObject, rpmScope, msgCtx);

        // Initialize
        NoteMinutes mainNoteMinutes = (NoteMinutes) rpmObject;

        // Context validation (No NoteMinutes allowed for CCCs, Assets and Resources
        String contextName = mainNoteMinutes.getContextName();
        if(contextName==null && mainNoteMinutes.getParent()!=null){
            //retrieve context from parent
            contextName = mainNoteMinutes.getParent().getContextName();
        }
        
        if(contextName != null){
            if (ContextUtil.ASSET_CONTEXT.equals(contextName)
                    || ContextUtil.ASSET_DOCUMENT_CONTEXT.equals(contextName))
            {
                valid &= validateContext(Asset.class, msgCtx);
            }
            else if (ContextUtil.CLIENT_CONTEXT.equals(contextName)
                    || ContextUtil.CLIENT_DOCUMENT_CONTEXT.equals(contextName))
            { 
                valid &= validateContext(ClientCostCenter.class, msgCtx);
            }
            else if (ContextUtil.RESOURCE_CONTEXT.equals(contextName)
                    || ContextUtil.RESOURCE_DOCUMENT_CONTEXT.equals(contextName))
            { 
                valid &= validateContext(Resource.class, msgCtx);
            }
        }
        
        if(valid){
            // Field validation: priority
            if (mainNoteMinutes.testPriorityModified())
            {
                GenericValidator.validateRange(mainNoteMinutes, "priority",
                    mainNoteMinutes.getPriority(),
                    ValidationConstants.NOTEMINUTES_PRIORITY_MIN,
                    ValidationConstants.NOTEMINUTES_PRIORITY_MAX, msgCtx);
            }

            // Field validation: rank
            if (mainNoteMinutes.testRankModified())
            {
                GenericValidator.validateRange(mainNoteMinutes, "rank",
                    mainNoteMinutes.getRank(), ValidationConstants.NOTEMINUTES_RANK_MIN,
                    ValidationConstants.NOTEMINUTES_RANK_MAX, msgCtx);
            }            
        }
    }
    
    private boolean validateContext(Class rootParent, MessageContext msgCtx){
        String currentClassName = StringUtil.getShortClassName(NoteMinutes.class.getName());
        String rootParentClassName = StringUtil.getShortClassName(rootParent.getName());
        String[] errorParams =
        {
                currentClassName, currentClassName, rootParentClassName
        };
        RPMException e = new RPMException(
                ErrorCodes.CANNOT_SAVE_UNDER_A_STRUCTURE, errorParams);
        addException(e, msgCtx);
        return false;
    }
}
