package com.ibm.rpm.document.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.resource.containers.Resource;

public class DocumentHistoryManager extends AbstractRPMObjectManager
{
    private static Log logger = LogFactory.getLog(DocumentHistoryManager.class);
    
    // Columns of the table HISTORY_DOCUMENTS

    public static final int HIST_ID_LEVEL_ID = 1;

    public static final int HIST_TYPE_LEVEL_ID = Types.INTEGER;

    public static final String HIST_NAME_LEVEL_ID = "HISTORY_DOCUMENTS.LEVEL_ID";

    public static final int HIST_ID_TYPE_ID = 2;

    public static final int HIST_TYPE_TYPE_ID = Types.INTEGER;

    public static final String HIST_NAME_TYPE_ID = "HISTORY_DOCUMENTS.TYPE_ID";

    public static final int HIST_ID_RANK = 3;

    public static final int HIST_TYPE_RANK = Types.INTEGER;

    public static final String HIST_NAME_RANK = "HISTORY_DOCUMENTS.RANK";

    public static final int HIST_ID_PARENT_ID = 4;

    public static final int HIST_TYPE_PARENT_ID = Types.CHAR;

    public static final String HIST_NAME_PARENT_ID = "HISTORY_DOCUMENTS.PARENT_ID";

    public static final int HIST_ID_ELEMENT_ID = 5;

    public static final int HIST_TYPE_ELEMENT_ID = Types.CHAR;

    public static final int HIST_ID_ELEMENT_NAME = 6;

    public static final int HIST_TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String HIST_NAME_ELEMENT_NAME = "HISTORY_DOCUMENTS.ELEMENT_NAME";

    public static final int HIST_ID_CHILD_COUNT = 7;

    public static final int HIST_TYPE_CHILD_COUNT = Types.INTEGER;

    public static final int HIST_ID_DELETED_COUNT = 8;

    public static final int HIST_TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String HIST_NAME_DELETED_COUNT = "HISTORY_DOCUMENTS.DELETED_COUNT";

    public static final int HIST_ID_CYCLE_NAME = 9;

    public static final int HIST_TYPE_CYCLE_NAME = Types.VARCHAR;

    public static final int HIST_ID_DESCRIPTION = 10;

    public static final int HIST_TYPE_DESCRIPTION = Types.VARCHAR;

    public static final int HIST_ID_REVISION = 11;

    public static final int HIST_TYPE_REVISION = Types.VARCHAR;

    public static final int HIST_ID_REVISION_IMPACT = 12;

    public static final int HIST_TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String HIST_NAME_REVISION_IMPACT = "HISTORY_DOCUMENTS.REVISION_IMPACT";

    public static final int HIST_ID_REVISION_HISTORY = 13;

    public static final int HIST_TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String HIST_NAME_REVISION_HISTORY = "HISTORY_DOCUMENTS.REVISION_HISTORY";

    public static final int HIST_ID_REVISION_TYPE = 14;

    public static final int HIST_TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String HIST_NAME_REVISION_TYPE = "HISTORY_DOCUMENTS.REVISION_TYPE";

    public static final int HIST_ID_DOCUMENT_SIZE = 15;

    public static final int HIST_TYPE_DOCUMENT_SIZE = Types.INTEGER;

    public static final int HIST_ID_REC_DATETIME = 16;

    public static final int HIST_TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String HIST_NAME_REC_DATETIME = "HISTORY_DOCUMENTS.REC_DATETIME";

    public static final int HIST_ID_AUTHOR = 17;

    public static final int HIST_TYPE_AUTHOR = Types.VARCHAR;

    protected StatusUpdate decodeDocumentNonFolderHistoryRow(ResultSet rs)
            throws RPMException, SQLException, ParseException
    {

        StatusUpdate statusUpdate = new StatusUpdate();
        statusUpdate.setID(rs.getString(HIST_ID_ELEMENT_ID));
        statusUpdate.assignTypeID(new Integer(rs.getInt(HIST_ID_TYPE_ID)));
        statusUpdate.setNameStep(rs.getString(HIST_ID_ELEMENT_NAME));
        statusUpdate.setRevision(rs.getString(HIST_ID_REVISION));
        statusUpdate.setChangeBrief(rs.getString(HIST_ID_REVISION_HISTORY));

        Timestamp tempTimeStamp = rs.getTimestamp(HIST_ID_REC_DATETIME);
        Calendar tempCalendar = DateUtil.getCalendarInstance();

        if (tempTimeStamp != null)
        {
            tempCalendar.setTimeInMillis(tempTimeStamp.getTime());
        }
        statusUpdate.setDateTime(tempCalendar);

        Resource tempResource = new Resource();
        tempResource.setFullName(rs.getString(HIST_ID_AUTHOR));
        statusUpdate.setParent(tempResource);

        return statusUpdate;
    }

//    protected ArrayList convertResultSetToArrayList(ResultSet rs)
//            throws RPMException, SQLException, ParseException
//    {
//
//        StatusUpdate statusUpdate = null;
//        ArrayList statusUpdates = new ArrayList();
//        while (rs.next())
//        {
//            statusUpdate = decodeDocumentNonFolderHistoryRow(rs);
//            statusUpdates.add(statusUpdate);
//        }
//
//        return statusUpdates;
//
//    }

    private ArrayList SP_S_DOC_REVISION(GenericDocument documentNonFolder,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        String name = "SP_S_DOC_REVISION";

        Object args[] = new Object[3];

        args[0] = new Integer(4);
        args[1] = documentNonFolder.getID();
        args[2] = new Integer(0);

        ArrayList arrayList = null; // loadFromProcedure(messageContext, name,
                                    // args);

        return arrayList;
    }

    public String getTableName()
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected String[] getFieldsNames()
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected String getAllFields()
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        // TODO Implement if the supported containers can be loaded
        // as children by another manager
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);

        return joinCondition;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            	 //uncomment if additionnal condition is needed
                //where.append(" AND ");
             }
             //TODO Add additionnal condition if needed
             //where.append(condition on status or type...);

             joinCondition.setCondition(where.toString());
    
        }

        //TODO Implement for managers that handles the children of the supported containers

        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
    
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }
        
    protected HashMap getPropertiesMap()
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        // TODO Auto-generated method stub
        return null;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        // TODO Auto-generated method stub
        return null;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        // TODO Auto-generated method stub

    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TODO Auto-generated method stub

    }
}
