package com.ibm.rpm.document.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.NoteState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.checkpoints.GenericDocumentCheckpoint;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentBlobDetails;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.document.types.DocumentLinkingStatus;
import com.ibm.rpm.document.types.DocumentStatusType;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.managers.ResourceRoleAssignmentManager;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.managers.AvailableWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.util.WorkflowUtil;

public class DocumentManager extends AbstractRPMObjectManager
{
    // private static Log logger = LogFactory.getLog(DocumentManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(GenericDocument.class.getName());
        CONTAINERS.add(DocumentFolder.class.getName());
        CONTAINERS.add(Document.class.getName());
        CONTAINERS.add(WPDoc.class.getName());
        CONTAINERS.add(DelDoc.class.getName());
        CONTAINERS.add(Software.class.getName());
        CONTAINERS.add(UrlDocument.class.getName());
        CONTAINERS.add(NoteMinutes.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                DocumentManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.BIGINT mapped to Long.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.BIGINT;

    public static final String NAME_MAX_RANK = "MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column ELEMENT_NUMBER of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ELEMENT_NUMBER = 8;

    public static final int TYPE_ELEMENT_NUMBER = Types.BIGINT;

    public static final String NAME_ELEMENT_NUMBER = "ELEMENT_NUMBER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NUMBER = "ELEMENTNUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NUMBER, NAME_ELEMENT_NUMBER);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 9;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROJECT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROJECT_ID = "PROJECTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);
    }

    /**
     * Column BLOB_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_BLOB_ID = 10;

    public static final int TYPE_BLOB_ID = Types.CHAR;

    public static final String NAME_BLOB_ID = "BLOB_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BLOB_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_BLOB_ID,NAME_BLOB_ID);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 11;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column PARENT_TYPE_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PARENT_TYPE_ID = 12;

    public static final int TYPE_PARENT_TYPE_ID = Types.SMALLINT;

    public static final String NAME_PARENT_TYPE_ID = "PARENT_TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_TYPE_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PARENT_TYPE_ID,NAME_PARENT_TYPE_ID);}

    /**
     * Column AUTHOR_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_AUTHOR_ID = 13;

    public static final int TYPE_AUTHOR_ID = Types.CHAR;

    public static final String NAME_AUTHOR_ID = "AUTHOR_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_AUTHOR_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_AUTHOR_ID,NAME_AUTHOR_ID);}

    /**
     * Column CREATION_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_CREATION_DATE = 14;

    public static final int TYPE_CREATION_DATE = Types.DATE;

    public static final String NAME_CREATION_DATE = "CREATION_DATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
   public static final String PROPERTY_CREATION_DATE = "CREATIONDATE";
   static{ 
       FIELDPROPERTYMAP.put(PROPERTY_CREATION_DATE,NAME_CREATION_DATE);
   }

    /**
     * Column APPROVED_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_APPROVED_DATE = 15;

    public static final int TYPE_APPROVED_DATE = Types.DATE;

    public static final String NAME_APPROVED_DATE = "APPROVED_DATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_APPROVED_DATE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_APPROVED_DATE,NAME_APPROVED_DATE);}

    /**
     * Column REF_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REF_NUMBER = 16;

    public static final int TYPE_REF_NUMBER = Types.VARCHAR;

    public static final String NAME_REF_NUMBER = "REF_NUMBER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REF_NUMBER = "REFERENCENUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REF_NUMBER, NAME_REF_NUMBER);
    }

    /**
     * Column AUTHOR_CHECKOUT of type Types.CHAR mapped to String.
     */
    public static final int ID_AUTHOR_CHECKOUT = 17;

    public static final int TYPE_AUTHOR_CHECKOUT = Types.CHAR;

    public static final String NAME_AUTHOR_CHECKOUT = "AUTHOR_CHECKOUT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_AUTHOR_CHECKOUT = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_AUTHOR_CHECKOUT,NAME_AUTHOR_CHECKOUT);}

    /**
     * Column PUBLISHED_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PUBLISHED_DT = 18;

    public static final int TYPE_PUBLISHED_DT = Types.TIMESTAMP;

    public static final String NAME_PUBLISHED_DT = "PUBLISHED_DT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PUBLISHED_DT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED_DT,NAME_PUBLISHED_DT);}

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINOR_REVISION = 19;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "MINOR_REVISION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MINOR_REVISION = "MINORREVISIONNUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION, NAME_MINOR_REVISION);
    }

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAJOR_REVISION = 20;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "MAJOR_REVISION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MAJOR_REVISION = "MAJORREVISIONNUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION, NAME_MAJOR_REVISION);
    }

    /**
     * Column MAX_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAX_REVISION = 21;

    public static final int TYPE_MAX_REVISION = Types.SMALLINT;

    public static final String NAME_MAX_REVISION = "MAX_REVISION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_REVISION = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_REVISION,NAME_MAX_REVISION);}

    /**
     * Column MAX_REV_REACHED of type Types.CHAR mapped to String.
     */
    public static final int ID_MAX_REV_REACHED = 22;

    public static final int TYPE_MAX_REV_REACHED = Types.CHAR;

    public static final String NAME_MAX_REV_REACHED = "MAX_REV_REACHED";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_REV_REACHED = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAX_REV_REACHED,NAME_MAX_REV_REACHED);}

    /**
     * Column ATTACHED_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ATTACHED_ID = 23;

    public static final int TYPE_ATTACHED_ID = Types.CHAR;

    public static final String NAME_ATTACHED_ID = "ATTACHED_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ATTACHED_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ATTACHED_ID,NAME_ATTACHED_ID);}

    /**
     * Column LAST_WF_TYPE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_WF_TYPE = 24;

    public static final int TYPE_LAST_WF_TYPE = Types.VARCHAR;

    public static final String NAME_LAST_WF_TYPE = "LAST_WF_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAST_WF_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_TYPE,NAME_LAST_WF_TYPE);}

    /**
     * Column LAST_WF_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_LAST_WF_BY = 25;

    public static final int TYPE_LAST_WF_BY = Types.CHAR;

    public static final String NAME_LAST_WF_BY = "LAST_WF_BY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAST_WF_BY = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_BY,NAME_LAST_WF_BY);}

    /**
     * Column LAST_WF_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_WF_DT = 26;

    public static final int TYPE_LAST_WF_DT = Types.TIMESTAMP;

    public static final String NAME_LAST_WF_DT = "LAST_WF_DT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAST_WF_DT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_DT,NAME_LAST_WF_DT);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 27;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 28;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REC_STATUS = "STATUS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS, NAME_REC_STATUS);
    }

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 29;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REC_DATETIME = "LASTUPDATED";
    static{ 
        FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);
     }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 30;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_RANK = "DOCUMENTRANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RANK, NAME_RANK);
    }
    public static final String PROPERTY_WBSSORTING_RANK = "WBSSORTINGRANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WBSSORTING_RANK, NAME_RANK);
    }
    

    /**
     * Column PARENT_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PARENT_RANK = 31;

    public static final int TYPE_PARENT_RANK = Types.VARCHAR;

    public static final String NAME_PARENT_RANK = "PARENT_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PARENT_RANK = "PARENTSORTINGRANK";
    static{ 
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_RANK,NAME_PARENT_RANK);
    }

    /**
     * Column KEYWORDS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_KEYWORDS = 32;

    public static final int TYPE_KEYWORDS = Types.VARCHAR;

    public static final String NAME_KEYWORDS = "KEYWORDS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_KEYWORDS = "KEYWORDS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_KEYWORDS, NAME_KEYWORDS);
    }

    /**
     * Column LAST_CHECKIN of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_CHECKIN = 33;

    public static final int TYPE_LAST_CHECKIN = Types.TIMESTAMP;

    public static final String NAME_LAST_CHECKIN = "LAST_CHECKIN";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAST_CHECKIN = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_CHECKIN,NAME_LAST_CHECKIN);}

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_TYPE = 34;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "REVISION_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_IMPACT = 35;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "REVISION_IMPACT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION_HISTORY = 36;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "REVISION_HISTORY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

    /**
     * Column SCORECARD_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SCORECARD_ID = 37;

    public static final int TYPE_SCORECARD_ID = Types.CHAR;

    public static final String NAME_SCORECARD_ID = "SCORECARD_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SCORECARD_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_ID,NAME_SCORECARD_ID);}

    /**
     * Column SCORE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SCORE = 38;

    public static final int TYPE_SCORE = Types.SMALLINT;

    public static final String NAME_SCORE = "SCORE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SCORE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SCORE,NAME_SCORE);}

    /**
     * Column WEIGHT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WEIGHT = 39;

    public static final int TYPE_WEIGHT = Types.SMALLINT;

    public static final String NAME_WEIGHT = "WEIGHT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WEIGHT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WEIGHT,NAME_WEIGHT);}

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_SCORE = 40;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "CONVERTED_SCORE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE);}

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 41;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "CONVERTED_WEIGHT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column ACTIVE of type Types.CHAR mapped to String.
     */
    public static final int ID_ACTIVE = 42;

    public static final int TYPE_ACTIVE = Types.CHAR;

    public static final String NAME_ACTIVE = "ACTIVE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTIVE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ACTIVE,NAME_ACTIVE);}

    /**
     * Column LOCAL_FOLDER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LOCAL_FOLDER = 43;

    public static final int TYPE_LOCAL_FOLDER = Types.VARCHAR;

    public static final String NAME_LOCAL_FOLDER = "LOCAL_FOLDER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LOCAL_FOLDER = "LOCALFOLDER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOCAL_FOLDER, NAME_LOCAL_FOLDER);
    }

    /**
     * Column RTF_BITS of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_RTF_BITS = 44;

    public static final int TYPE_RTF_BITS = Types.SMALLINT;

    public static final String NAME_RTF_BITS = "RTF_BITS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RTF_BITS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RTF_BITS,NAME_RTF_BITS);}

    /**
     * Column PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRIORITY = 45;

    public static final int TYPE_PRIORITY = Types.SMALLINT;

    public static final String NAME_PRIORITY = "PRIORITY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PRIORITY = "PRIORITY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIORITY, NAME_PRIORITY);
    }

    /**
     * Column SCOPE_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SCOPE_RANK_ID = 46;

    public static final int TYPE_SCOPE_RANK_ID = Types.SMALLINT;

    public static final String NAME_SCOPE_RANK_ID = "SCOPE_RANK_ID";
    public static final String PROPERTY_DOCUMENT_RANK = "RANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DOCUMENT_RANK, NAME_SCOPE_RANK_ID);
    }
    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SCOPE_RANK_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_RANK_ID,NAME_SCOPE_RANK_ID);}

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 47;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "STAGE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID);}

    /**
     * Column TEMP_FOLDER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TEMP_FOLDER = 48;

    public static final int TYPE_TEMP_FOLDER = Types.VARCHAR;

    public static final String NAME_TEMP_FOLDER = "TEMP_FOLDER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TEMP_FOLDER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TEMP_FOLDER,NAME_TEMP_FOLDER);}

    /**
     * Column LOCAL_MACHINE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LOCAL_MACHINE = 49;

    public static final int TYPE_LOCAL_MACHINE = Types.VARCHAR;

    public static final String NAME_LOCAL_MACHINE = "LOCAL_MACHINE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LOCAL_MACHINE = "LOCALMACHINE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOCAL_MACHINE, NAME_LOCAL_MACHINE);
    }

    /**
     * Column DOCUMENT_SIZE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DOCUMENT_SIZE = 50;

    public static final int TYPE_DOCUMENT_SIZE = Types.INTEGER;

    public static final String NAME_DOCUMENT_SIZE = "DOCUMENT_SIZE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DOCUMENT_SIZE = "DOCUMENTSIZE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DOCUMENT_SIZE, NAME_DOCUMENT_SIZE);
    }

    /**
     * Column COMPRESSED_SIZE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_COMPRESSED_SIZE = 51;

    public static final int TYPE_COMPRESSED_SIZE = Types.INTEGER;

    public static final String NAME_COMPRESSED_SIZE = "COMPRESSED_SIZE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_COMPRESSED_SIZE = "DOCUMENTCOMPRESSEDSIZE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPRESSED_SIZE, NAME_COMPRESSED_SIZE);
    }

    /**
     * Column PUBLISHED of type Types.CHAR mapped to String.
     */
    public static final int ID_PUBLISHED = 52;

    public static final int TYPE_PUBLISHED = Types.CHAR;

    public static final String NAME_PUBLISHED = "PUBLISHED";

    public static final String PROPERTY_PUBLISHED = "PUBLISHED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED, NAME_PUBLISHED);
    }

    /**
     * Column ASSIGN_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSIGN_NAMES = 53;

    public static final int TYPE_ASSIGN_NAMES = Types.VARCHAR;

    public static final String NAME_ASSIGN_NAMES = "ASSIGN_NAMES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ASSIGN_NAMES = "ASSIGNEDRESOURCES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_NAMES, NAME_ASSIGN_NAMES);
    }

    /**
     * Column SUMMARY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SUMMARY = 54;

    public static final int TYPE_SUMMARY = Types.VARCHAR;

    public static final String NAME_SUMMARY = "SUMMARY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SUMMARY = "SUMMARY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUMMARY, NAME_SUMMARY);
    }

    /**
     * Column PREVIOUS_WKF_STEP of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PREVIOUS_WKF_STEP = 55;

    public static final int TYPE_PREVIOUS_WKF_STEP = Types.VARCHAR;

    public static final String NAME_PREVIOUS_WKF_STEP = "PREVIOUS_WKF_STEP";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PREVIOUS_WKF_STEP = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PREVIOUS_WKF_STEP,NAME_PREVIOUS_WKF_STEP);}

    /**
     * Column zPRIVATE of type Types.CHAR mapped to String.
     */
    public static final int ID_ZPRIVATE = 56;

    public static final int TYPE_ZPRIVATE = Types.CHAR;

    public static final String NAME_ZPRIVATE = "PRIVATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ZPRIVATE = "ELEMENTSECURITYTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ZPRIVATE, NAME_ZPRIVATE);
    }

    /**
     * Column VISIBLE_AT_PARENT of type Types.CHAR mapped to String.
     */
    public static final int ID_VISIBLE_AT_PARENT = 57;

    public static final int TYPE_VISIBLE_AT_PARENT = Types.CHAR;

    public static final String NAME_VISIBLE_AT_PARENT = "VISIBLE_AT_PARENT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_VISIBLE_AT_PARENT = "VISIBLEATPARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_VISIBLE_AT_PARENT, SqlUtil
            .getBooleanValue(NAME_VISIBLE_AT_PARENT, "Y"));
    }

    /**
     * Column LINK_DOC of type Types.CHAR mapped to String.
     */
    public static final int ID_LINK_DOC = 58;

    public static final int TYPE_LINK_DOC = Types.CHAR;

    public static final String NAME_LINK_DOC = "LINK_DOC";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LINK_DOC = "LINKINGSTATUS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LINK_DOC, NAME_LINK_DOC);
    }
    
    /**
     * Column T_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_T_RANK = 59;

    public static final int TYPE_T_RANK = Types.VARCHAR;

    public static final String NAME_T_RANK = "T_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_T_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_T_RANK,NAME_T_RANK);}

    /**
     * Column RUP_LINK of type Types.CHAR mapped to String.
     */
    public static final int ID_RUP_LINK = 60;

    public static final int TYPE_RUP_LINK = Types.CHAR;

    public static final String NAME_RUP_LINK = "RUP_LINK";

    public static final String PROPERTY_RUP_LINK = "RUPLINK";
    static
    {
    	FIELDPROPERTYMAP.put(PROPERTY_RUP_LINK,NAME_RUP_LINK);
    }
    
    /**
     * Column TNG_LINK of type Types.CHAR mapped to String.
     */
    public static final int ID_TNG_LINK = 61;

    public static final int TYPE_TNG_LINK = Types.CHAR;

    public static final String NAME_TNG_LINK = "TNG_LINK";

    public static final String PROPERTY_TNG_LINK = "PROCESSLINK";
    static
    {
    	FIELDPROPERTYMAP.put(PROPERTY_TNG_LINK,NAME_TNG_LINK);
    }


    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 62;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "CONFIG_FLAG";




    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONFIG_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG,NAME_CONFIG_FLAG);}


    /**
     * Column T_PARENT_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_T_PARENT_ID = 64;

    public static final int TYPE_T_PARENT_ID = Types.VARCHAR;

    public static final String NAME_T_PARENT_ID = "T_PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_T_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_T_PARENT_ID,NAME_T_PARENT_ID);}

    /**
     * Create an array of type string containing all the fields of the Documents
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_ELEMENT_NUMBER, NAME_PROJECT_ID, NAME_BLOB_ID, NAME_PARENT_ID,
            NAME_PARENT_TYPE_ID, NAME_AUTHOR_ID, NAME_CREATION_DATE,
            NAME_APPROVED_DATE, NAME_REF_NUMBER, NAME_AUTHOR_CHECKOUT,
            NAME_PUBLISHED_DT, NAME_MINOR_REVISION, NAME_MAJOR_REVISION,
            NAME_MAX_REVISION, NAME_MAX_REV_REACHED, NAME_ATTACHED_ID,
            NAME_LAST_WF_TYPE, NAME_LAST_WF_BY, NAME_LAST_WF_DT, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_RANK, NAME_PARENT_RANK,
            NAME_KEYWORDS, NAME_LAST_CHECKIN, NAME_REVISION_TYPE,
            NAME_REVISION_IMPACT, NAME_REVISION_HISTORY, NAME_SCORECARD_ID,
            NAME_SCORE, NAME_WEIGHT, NAME_CONVERTED_SCORE,
            NAME_CONVERTED_WEIGHT, NAME_ACTIVE, NAME_LOCAL_FOLDER,
            NAME_RTF_BITS, NAME_PRIORITY, NAME_SCOPE_RANK_ID, NAME_STAGE_ID,
            NAME_TEMP_FOLDER, NAME_LOCAL_MACHINE, NAME_DOCUMENT_SIZE,
            NAME_COMPRESSED_SIZE, NAME_PUBLISHED, NAME_ASSIGN_NAMES,
            NAME_SUMMARY, NAME_PREVIOUS_WKF_STEP, NAME_ZPRIVATE,
            NAME_VISIBLE_AT_PARENT, NAME_LINK_DOC, NAME_T_RANK,
            NAME_RUP_LINK, NAME_TNG_LINK
    /*
     * Unmapped fields as of now
     * NAME_T_PARENT_ID, NAME_CONFIG_FLAG
     */
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new DocumentsManager instance.
     *
     * @return DocumentsManager
     */
    public DocumentManager()
    {
        super();
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);

    }

    public String getTableName()
    {
        ExceptionUtil.handleGetTableName(this);
        return null;
    }

    public String getTableName(RPMObject rpmObject)
    {
        String contextName = determineContextName(rpmObject);
        return getTableName(contextName);
    }

    public String getTableName(String contextName)
    {
        return DocumentUtil.getInstance().getTableName(contextName);
    }

    protected Integer getTableType(RPMObject rpmObject)
    {
        String contextName = determineContextName(rpmObject);
        return getTableType(contextName);
    }

    protected Integer getTableType(String contextName)
    {
        return DocumentUtil.getInstance().getTableType(contextName);
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject) throws RPMException
    {
    	String tableName = getTableName(rpmObject);
        return new String[]
        {
        	tableName+"."+getPrimaryKey()
        };
    }
    
    private String determineContextName(RPMObject rpmObject)
    {
        if (rpmObject != null)
        {
            if (rpmObject.getContextName() != null)
            {
                return rpmObject.getContextName();
            }
            else if (rpmObject.getParent() != null)
            {
                // try to get contextName from the parent
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof DocumentElement)
                {
                    return determineContextName(parent);
                }
                try
                {
                    return ContextUtil.getInstance().getContextName(parent);
                }
                catch (RPMException e)
                {
                    e.log();
                }
            }
        }
        return null;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {

        case DocumentFolder.TYPE_ID:
            object = new DocumentFolder();
            break;
        case Document.TYPE_ID:
            object = new Document();
            break;
        case DelDoc.TYPE_ID:
            object = new DelDoc();
            break;
        case WPDoc.TYPE_ID:
            object = new WPDoc();
            break;
        case Software.TYPE_ID:
            object = new Software();
            break;
        case UrlDocument.TYPE_ID:
            object = new UrlDocument();
            break;
        case NoteMinutes.TYPE_ID:
            object = new NoteMinutes();
            break;
        case 145: // RootFolder created by SP
            object = new DocumentFolder();
            break;
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }

    protected Object getQualifiedColumnName(HashMap propertiesMap, String propertyName, String contextName)
    {
    	Object o = propertiesMap.get(propertyName.toUpperCase());
    	if(o!=null)
    	{
            if (propertyName.equalsIgnoreCase(PROPERTY_VISIBLE_AT_PARENT))
            {
                return o;
            }
            else
            {
        		String tableName = getTableName(contextName);
        		if(tableName!=null)
        		{
        			return tableName+"."+o;
        		}
            }
    	}
    	return null;
    }
    
    protected RPMObject decodeRow(ResultSet rs, RPMObject targetObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DocumentElement temp = (DocumentElement) targetObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));

        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }

        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            temp.deltaAssignedResources(rs.getString(ID_ASSIGN_NAMES));
            temp.deltaDocumentRank(decodeDocumentRank(rs));
            temp.deltaWbsSortingRank(rs.getString(ID_RANK));
            temp.deltaParentSortingRank(rs.getString(ID_PARENT_RANK));
            temp.deltaTemplateSortingRank(rs.getString(ID_T_RANK));
        }
        else
        {
            temp.setName(name);
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            temp.setAssignedResources(rs.getString(ID_ASSIGN_NAMES));
            temp.setDocumentRank(decodeDocumentRank(rs));
            temp.setWbsSortingRank(rs.getString(ID_RANK));
            temp.setParentSortingRank(rs.getString(ID_PARENT_RANK));
            temp.setTemplateSortingRank(rs.getString(ID_T_RANK));
        }

        if (isDocumentFolder(targetObject))
        {
            DocumentFolder docFolder = (DocumentFolder) targetObject;

            if (isDelta)
            {
                docFolder.deltaPrivateFolder(decodePrivateFolder(rs));
            }
            else
            {
                docFolder.setPrivateFolder(decodePrivateFolder(rs));
            }
        }
        else
        { // Not a DocumentFolder

            GenericDocument genericDocument = (GenericDocument) targetObject;

            if (isDelta)
            {
            	
            	genericDocument.deltaElementNumber(new Integer(rs
                        .getInt(ID_ELEMENT_NUMBER)));
                genericDocument.deltaVisibleAtParent(decodeVisibleAtParent(rs));
                // check if locked
                genericDocument.deltaLocked(decodeIsLocked(rs));
                genericDocument.deltaDocumentStatus(decodeDocumentStatus(rs));
                // Element Security
                genericDocument
                    .deltaElementSecurityType(decodeElementSecurityType(rs));
                genericDocument.deltaHostName(rs.getString(ID_LOCAL_MACHINE));
                genericDocument.deltaCheckoutFolder(rs
                    .getString(ID_LOCAL_FOLDER));
                genericDocument.deltaHasAttachment(decodeHasAttachment(rs));
                genericDocument.deltaDocumentSize(new Integer(rs
                    .getInt(ID_DOCUMENT_SIZE)));
                genericDocument.deltaDocumentCompressedSize(new Integer(rs
                    .getInt(ID_COMPRESSED_SIZE)));
                genericDocument.deltaLastUpdated(Manager.getTimestamp(rs,
                    ID_REC_DATETIME));
                genericDocument.deltaCreationDate(Manager.getCalendar(rs,
                    ID_CREATION_DATE));
                genericDocument.deltaWorkflowCollaborationDate(Manager
                    .getTimestamp(rs, ID_LAST_WF_DT));
                genericDocument.deltaParentSortingRank(rs
                    .getString(ID_PARENT_RANK));
                genericDocument.deltaTemplateSortingRank(rs
                    .getString(ID_T_RANK));
            }
            else
            {
            	genericDocument.setElementNumber(new Integer(rs
                        .getInt(ID_ELEMENT_NUMBER)));
                genericDocument.setVisibleAtParent(decodeVisibleAtParent(rs));
                genericDocument.setLocked(decodeIsLocked(rs));
                genericDocument.setDocumentStatus(decodeDocumentStatus(rs));
                genericDocument
                    .setElementSecurityType(decodeElementSecurityType(rs));
                genericDocument.setHostName(rs.getString(ID_LOCAL_MACHINE));
                genericDocument
                    .setCheckoutFolder(rs.getString(ID_LOCAL_FOLDER));
                genericDocument.setHasAttachment(decodeHasAttachment(rs));
                genericDocument.setDocumentSize(new Integer(rs
                    .getInt(ID_DOCUMENT_SIZE)));
                genericDocument.setDocumentCompressedSize(new Integer(rs
                    .getInt(ID_COMPRESSED_SIZE)));
                genericDocument.setLastUpdated(Manager.getTimestamp(rs,
                    ID_REC_DATETIME));
                genericDocument.setCreationDate(Manager.getCalendar(rs,
                    ID_CREATION_DATE));
                genericDocument.setWorkflowCollaborationDate(Manager
                    .getTimestamp(rs, ID_LAST_WF_DT));
                genericDocument.setParentSortingRank(rs
                    .getString(ID_PARENT_RANK));
                genericDocument.setTemplateSortingRank(rs.getString(ID_T_RANK));
            }

            if (isNoteMinutes(targetObject))
            {

                NoteMinutes noteMin = (NoteMinutes) targetObject;
                if (isDelta)
                {
                    noteMin.deltaPriority(new Integer(rs.getInt(ID_PRIORITY)));
                    noteMin.deltaRank(new Integer(rs.getInt(ID_SCOPE_RANK_ID)));
                }
                else
                {
                    noteMin.setPriority(new Integer(rs.getInt(ID_PRIORITY)));
                    noteMin.setRank(new Integer(rs.getInt(ID_SCOPE_RANK_ID)));
                }

            }
            else
            {

                AbstractDocument docElement = (AbstractDocument) targetObject;

                if (isDelta)
                {
                    docElement.deltaPublished(decodePublished(rs));
                    docElement
                        .deltaReferenceNumber(rs.getString(ID_REF_NUMBER));

                    docElement.deltaKeywords(rs.getString(ID_KEYWORDS));
                    docElement.deltaSummary(rs.getString(ID_SUMMARY));
                    docElement.deltaLinkingStatus(decodeLinkingStatus(rs));
                }
                else
                {
                    docElement.setPublished(decodePublished(rs));
                    docElement.setReferenceNumber(rs.getString(ID_REF_NUMBER));

                    docElement.setKeywords(rs.getString(ID_KEYWORDS));
                    docElement.setSummary(rs.getString(ID_SUMMARY));
                    docElement.setLinkingStatus(decodeLinkingStatus(rs));
                }

                if (isURL(targetObject))
                {
                     UrlDocument url = (UrlDocument)docElement;
                     if (isDelta)
                     {
                    	 url.deltaRUPLink(decodeRUPLink(rs));
                    	 url.deltaProcessLink(decodeProcessLink(rs));
                     }
                     else
                     {
                    	 url.setRUPLink(decodeRUPLink(rs));
                    	 url.setProcessLink(decodeProcessLink(rs));
                     }
                }
                else
                {

                    if (isDelta)
                    {
                        docElement.deltaMajorRevisionNumber(new Integer(rs
                            .getInt(ID_MAJOR_REVISION)));
                        docElement.deltaMinorRevisionNumber(new Integer(rs
                            .getInt(ID_MINOR_REVISION)));
                    }
                    else
                    {
                        docElement.setMajorRevisionNumber(new Integer(rs
                            .getInt(ID_MAJOR_REVISION)));
                        docElement.setMinorRevisionNumber(new Integer(rs
                            .getInt(ID_MINOR_REVISION)));
                    }

                }

            }

        }

        return targetObject;
    }

    private Boolean decodeRUPLink(ResultSet rs) throws SQLException
    {
    	Boolean retVal = Boolean.TRUE;
    	String dbValue = rs.getString(ID_RUP_LINK);
        
        if ("0".equalsIgnoreCase(dbValue))
        {
            retVal = Boolean.FALSE;
        }
        
        return retVal;
    }
    
    private Boolean decodeProcessLink(ResultSet rs) throws SQLException
    {
    	Boolean retVal = Boolean.TRUE;
    	String dbValue = rs.getString(ID_TNG_LINK);
        
        if ("0".equalsIgnoreCase(dbValue))
        {
            retVal = Boolean.FALSE;
        }
        
        return retVal;
    }
    
    private String decodeDocumentRank(ResultSet rs) throws SQLException
    {
        String dbValue = rs.getString(ID_PARENT_RANK);

        // mimicking the related Stored Procedure
        if (dbValue != null && dbValue.startsWith("CRI#"))
        {
            return "Y";
        }

        return "N";
    }

    private DocumentStatusType decodeDocumentStatus(ResultSet rs)
            throws SQLException
    {
        DocumentStatusType result = null;
        String docStatus = rs.getString(ID_REC_STATUS);

        if(docStatus != null)
        {
            if (docStatus.equals("O"))
            {
                result = DocumentStatusType.CheckedOut;
            }
            else if (docStatus.equals("U"))
            {
                result = DocumentStatusType.NotCheckedOut;
            }
            else if (docStatus.equals("L"))
            {
                result = DocumentStatusType.Locked;
            }
            else if (docStatus.equals("D"))
            {
                result = DocumentStatusType.Deleted;
            }
            else if (docStatus.equals("T"))
            {
                result = DocumentStatusType.Template;
            }
        }

        return result;
    }

    private DocumentLinkingStatus decodeLinkingStatus(ResultSet rs)
            throws SQLException
    {
        DocumentLinkingStatus result = null;
        String linkStatus = rs.getString(ID_LINK_DOC);

        if (linkStatus != null)
        {
            if (linkStatus.equals("Y"))
            {
                result = DocumentLinkingStatus.IsLink;
            }
            else if (linkStatus.equals("P"))
            {
                result = DocumentLinkingStatus.IsReferenced;
            }
            else if (linkStatus.equals("N"))
            {
                result = DocumentLinkingStatus.IsNotReferenced;
            }
        }

        return result;
    }

    public static Boolean decodePrivateFolder(ResultSet rs) throws SQLException
    {

        return ("Y").equals(rs.getString(ID_ZPRIVATE)) ? Boolean.TRUE
                : Boolean.FALSE;
    }

    // TODO: Uncomment after ID_RUP_LINK and ID_TNG_LINK is added to all tables

    /*
     * public static Boolean decodeRUPLink(ResultSet rs) throws SQLException {
     * return ("1").equals(rs.getString(ID_RUP_LINK)) ? Boolean.TRUE :
     * Boolean.FALSE; }
     *
     * public static Boolean decodeProcessLink(ResultSet rs) throws SQLException {
     * return ("1").equals(rs.getString(ID_TNG_LINK)) ? Boolean.TRUE :
     * Boolean.FALSE; }
     */

    public static PublishedType decodePublished(ResultSet rs)
            throws SQLException
    {
        return ("Y").equals(rs.getString(ID_PUBLISHED))
                ? PublishedType.ToTemplate : PublishedType.None;
    }

    public static ElementSecurityType decodeElementSecurityType(ResultSet rs)
            throws SQLException
    {
        ElementSecurityType result = null;

        String docSecurityType = rs.getString(NAME_ZPRIVATE);

        if (docSecurityType != null)
        {
            if (docSecurityType.equals("Y"))
            {
                result = ElementSecurityType.ByInvitation;
            }
            else if (docSecurityType.equals("N"))
            {
                result = ElementSecurityType.ProjectLevel;
            }
            else if (docSecurityType.equals("U"))
            {
                result = ElementSecurityType.Unrestricted;
            }
        }

        return result;
    }

    public static Boolean decodeIsLocked(ResultSet rs) throws SQLException
    {
        return ("L").equals(rs.getString(ID_REC_STATUS)) ? Boolean.TRUE
                : Boolean.FALSE;
    }

    public static Boolean decodeVisibleAtParent(ResultSet rs)
            throws SQLException
    {
        return ("Y").equals(rs.getString(ID_VISIBLE_AT_PARENT)) ? Boolean.TRUE
                : Boolean.FALSE;
    }

    public static Boolean decodeHasAttachment(ResultSet rs) throws SQLException
    {
        return (rs.getString(ID_ATTACHED_ID) != null) ? Boolean.TRUE
                : Boolean.FALSE;
    }
    
    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String status = new String(resultSet.getString(ID_STAGE_ID));
        relatedFields.put(index, ID_STAGE_ID, status);

        String authorId = resultSet.getString(ID_AUTHOR_ID);
        relatedFields.put(index, ID_AUTHOR_ID, authorId);

        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);

        String elementId = resultSet.getString(ID_BLOB_ID);
        relatedFields.put(index, ID_BLOB_ID, elementId);

        String resourceId = resultSet.getString(ID_REC_USER);
        relatedFields.put(index, ID_REC_USER, resourceId);

        String parentRank = resultSet.getString(ID_PARENT_RANK);
        relatedFields.put(index, ID_PARENT_RANK, parentRank);

        return relatedFields;
    }

    public static String encodeElementSecurityType(GenericDocument docNonFolder)
	{
    	ElementSecurityType elementSecurityType = docNonFolder.getElementSecurityType();
		if (elementSecurityType == ElementSecurityType.ByInvitation) {
			return "Y";
		} else if (elementSecurityType == ElementSecurityType.ProjectLevel) {
			return "N";
		} else {
			return "U";
		}
	}

    private String encodeFileName(AbstractDocument docElement,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String fileName = null;
        if (isURL(docElement))
        {
            fileName = ((UrlDocument) docElement).getUrl();
        }
        else
        {
        	if(docElement.getDocumentBlobDetails() != null)
        	{
        	    fileName = docElement.getDocumentBlobDetails().getFilename();
        	}
        	else
        	{
        		SqlBuffer condition = new SqlBuffer();
        		condition.append(DocumentBlobDetailManager.NAME_ELEMENT_ID);
        		condition.appendIn();
        		condition.appendSelect();
        		condition.append(NAME_BLOB_ID);
        		condition.appendFrom(getTableName(docElement));
        		condition.appendWhere();
        		condition.appendEqual(NAME_ELEMENT_ID, (Object)docElement.getID());
        		
        		fileName = (String)ManagerUtil.selectColumnValue(String.class, docElement,null, 
        				DocumentBlobDetailManager.NAME_ATTACHMENT,DocumentBlobDetailManager.TABLE_NAME, 
        				condition, messageContext);
        	}
        }
        return fileName;
    }
    
    /**
     *
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        if (rpmObject != null)
        {
            // load parent if necessary as it's used to determine the tableType
            if (rpmObject.getParent() == null)
            {
                RPMObject parent = getParent(rpmObject, messageContext);
                rpmObject.setParent(parent);
            }

            SP_D_DOC(rpmObject, messageContext);

        }
    }

    /**
     *
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName(contextName));
            StringBuffer where = new StringBuffer();
            where.append(getTypeIdWhere(contextName, containerName));
            where.append(" AND ");
            where.append(getFilter(contextName));
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof DocumentManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            int rankLenght = 4 * level;
            String rank = getTableName(contextName) + ".RANK";
            
            where.append("(" + NAME_LEVEL_ID + " >= ");
            where.append(level);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(getTypeIdWhere(contextName, containerName));
            
            if (condition != null)
            {
                where.append(condition);
            }        

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof CustomFieldAssignmentManager)
        {
            if (ContextUtil.UMBRELLA_DOCUMENT_CONTEXT.equalsIgnoreCase(contextName))
            {
                // for XPath queries like
                // Asset[<condition>]/AssignedCustomField
                joinCondition = new JoinCondition();
                StringBuffer where = new StringBuffer();
                where.append(CustomFieldAssignmentUtil
                    .getTableName(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT));
                where.append(".");
                where.append(CustomFieldAssignmentManager.NAME_FOREIGN_ID);
                where.append(" IN (");
                where.append(" SELECT ").append(NAME_ELEMENT_ID);
                where.append(" FROM ").append(
                    getTableName(ContextUtil.WBS_CONTEXT));

                where.append(" WHERE ");
                where.append(getTypeIdWhere(contextName, containerName));
                if (condition != null)
                {
                    where.append(condition);
                }
                joinCondition.setCondition(where.toString());
            }
        }
        else if (manager instanceof AvailableWorkflowProcessManager)
        {
            // for xpath like:
            // /GenericDocument[<condition>]/AvailableWorkflowProcess
            joinCondition = JoinCondition.createSubSelect(
                AvailableWorkflowProcessManager.NAME_PROJECT_ID,
                NAME_PROJECT_ID, getTableName(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT),
                condition);
        }
        else if (manager instanceof RunningWorkflowProcessManager)
        {
            // for xpath like:
            // /GenericDocument[<condition>]/RunningWorkflowProcess
            joinCondition = JoinCondition.createSubSelect(
                RunningWorkflowProcessManager.NAME_PARENT_ID, NAME_ELEMENT_ID,
                getTableName(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT), condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ScopeManager)
        {
            SqlBuffer where = new SqlBuffer();
            where.appendEqual(NAME_TYPE_ID, DocumentFolder.TYPE_ID);
            where.append(condition);
            where.appendEqual(NAME_LEVEL_ID, 2);
            joinCondition = JoinCondition.createSubSelect(
                ScopeManager.NAME_PROJECT_ID, NAME_PROJECT_ID,
                getTableName(ContextUtil.SCOPE_DOCUMENT_CONTEXT),
                where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof DocumentManager)
        {
            // where clause of the join condition
            StringBuffer condition = new StringBuffer();
            
            if (ContextUtil.getInstance().isResourceContext(contextName))
            {
                condition.append("(" + NAME_LEVEL_ID + " >= 1");
                condition.append(" AND " + NAME_PARENT_ID + " = ? ");
                
                joinCondition = new JoinCondition();
                joinCondition.setCondition(condition.toString());
            }
            else
            {
                condition.append(getPrimaryKey());
                condition.append(" = ?");
                
                // In this case, we don't need a condition on the type of the parent
                joinCondition = getJoinCondition(manager, condition.toString(), 1,
                    null, contextName);
            }
            
            int[] types;
            
            if (ContextUtil.getInstance().isResourceContext(contextName)
                    || ContextUtil.getInstance().isAssetContext(contextName))
            {
            	types = new int[]
            	                {
            			DocumentFolder.TYPE_ID, Document.TYPE_ID, WPDoc.TYPE_ID,
            			DelDoc.TYPE_ID, Software.TYPE_ID, UrlDocument.TYPE_ID,
            			NoteMinutes.TYPE_ID, 145//SYSTEM DOCUMENT FOLDER TYPE_ID
            	                };
            }
            else
            {
            	types = new int[]
            	                {
            			DocumentFolder.TYPE_ID, Document.TYPE_ID, WPDoc.TYPE_ID,
            			DelDoc.TYPE_ID, Software.TYPE_ID, UrlDocument.TYPE_ID,
            			NoteMinutes.TYPE_ID
            	                };
            }

            String extraWhereClauseWithTypes = constructLoadDocumentsExtraWhereClause(types);

            StringBuffer sb = new StringBuffer();
            sb.append(joinCondition.getCondition());
            sb.append(" AND ");
            sb.append(extraWhereClauseWithTypes);
            sb.append(" ) ");
            if (extraWhere != null && extraWhere.length() > 0)
            {
                sb.append(extraWhere);
            }

            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(getTableName(contextName));

        }
        else if (manager instanceof AssetManager)
        {
            StringBuffer sb = new StringBuffer();
            String tableName = "TMT_ASSET_DOCUMENTS";
            sb.append(" ELEMENT_ID = ? ");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(tableName);
        }
        else if (manager instanceof WbsManager)
        {

            String tableName = "TMT_DOCUMENTS";

            // where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition
                .append(" PROJECT_ID = (SELECT PROJECT_ID FROM TMT_WBS WHERE ELEMENT_ID = ?)");
            condition.append(" AND LEVEL_ID = 2 ");

            joinCondition = new JoinCondition();
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(tableName);

        }
        else if (manager instanceof ScopeManager)
        {
            SqlBuffer condition = new SqlBuffer();
            condition
                .appendSubSelect(NAME_PROJECT_ID, 
                    ScopeManager.NAME_PROJECT_ID, ScopeManager.TABLE_NAME, "ELEMENT_ID = ?");
            condition.appendCloseParenthesis();
            condition.appendEqual(NAME_LEVEL_ID, 2);
            condition.appendAnd();
            condition.append(" " + extraWhere);
            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(ContextUtil.SCOPE_DOCUMENT_CONTEXT));
        }
        else if (manager instanceof ClientManager)
        {
            StringBuffer sb = new StringBuffer();
            String tableName = "TMT_CLIENT_DOCUMENTS";
            sb.append(" ELEMENT_ID = ? ");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(tableName);
        }
        else if (manager instanceof ResourceManager)
        {

            StringBuffer sb = new StringBuffer();
            String tableName = "TMT_RESOURCE_DOCUMENTS";
            sb.append(" ELEMENT_ID = ? ");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(tableName);

        }
        else if (manager instanceof PoolManager)
        {
            StringBuffer sb = new StringBuffer();
            String tableName = "TMT_POOL_DOCUMENTS";
            sb.append(" ELEMENT_ID = ? ");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(tableName);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    public String getFilter(String contextName)
    {
        String table = getTableName(contextName);
        String recStatusFieldName = (table == null || ContextUtil.NONE
            .compareTo(contextName) == 0) ? NAME_REC_STATUS : table + '.'
                + NAME_REC_STATUS;
        String elementIdFieldName = (table == null || ContextUtil.NONE
            .compareTo(contextName) == 0) ? NAME_ELEMENT_ID : table + '.'
                + NAME_ELEMENT_ID;
        SqlBuffer filter = new SqlBuffer(recStatusFieldName + "!='D'");
        filter.appendIsNotNull(elementIdFieldName);

        return filter.toString();
    }

    private StringBuffer getTypeIdWhere(String contextName, String containerName)
            throws RPMException
    {
        StringBuffer typeIdWhere = null;
        String tableName = getTableName(contextName);

        if (tableName != null)
        {
            typeIdWhere = new StringBuffer(tableName);
            typeIdWhere.append("." + determineTypeId(containerName));
        }

        return typeIdWhere;
    }

    private String determineTypeId(String containerName) throws RPMException
    {
        StringBuffer typeIdWhere = new StringBuffer();
        int typeId = 0;

        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(DocumentFolder.class)))
            {
                typeId = DocumentFolder.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(DelDoc.class)))
            {
                typeId = DelDoc.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Document.class)))
            {
                typeId = Document.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Software.class)))
            {
                typeId = Software.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(UrlDocument.class)))
            {
                typeId = UrlDocument.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(WPDoc.class)))
            {
                typeId = WPDoc.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(NoteMinutes.class)))
            {
                typeId = NoteMinutes.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(AbstractDocument.class)))
            {
                typeIdWhere.append(NAME_TYPE_ID).append(" IN ( ");
                typeIdWhere.append(DelDoc.TYPE_ID);
                typeIdWhere.append(" , ").append(Document.TYPE_ID);
                typeIdWhere.append(" , ").append(Software.TYPE_ID);
                typeIdWhere.append(" , ").append(UrlDocument.TYPE_ID);
                typeIdWhere.append(" , ").append(WPDoc.TYPE_ID);
                typeIdWhere.append(" ) ");

                return typeIdWhere.toString();
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(GenericDocument.class)))
            {
                typeIdWhere.append(NAME_TYPE_ID).append(" IN ( ");
                typeIdWhere.append(DelDoc.TYPE_ID);
                typeIdWhere.append(" , ").append(Document.TYPE_ID);
                typeIdWhere.append(" , ").append(Software.TYPE_ID);
                typeIdWhere.append(" , ").append(UrlDocument.TYPE_ID);
                typeIdWhere.append(" , ").append(WPDoc.TYPE_ID);
                typeIdWhere.append(" , ").append(NoteMinutes.TYPE_ID);
                typeIdWhere.append(" ) ");

                return typeIdWhere.toString();
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(DocumentElement.class)))
            {
                typeIdWhere.append(NAME_TYPE_ID).append(" IN ( ");
                typeIdWhere.append(DelDoc.TYPE_ID);
                typeIdWhere.append(" , ").append(Document.TYPE_ID);
                typeIdWhere.append(" , ").append(Software.TYPE_ID);
                typeIdWhere.append(" , ").append(UrlDocument.TYPE_ID);
                typeIdWhere.append(" , ").append(WPDoc.TYPE_ID);
                typeIdWhere.append(" , ").append(NoteMinutes.TYPE_ID);
                typeIdWhere.append(" , ").append(DocumentFolder.TYPE_ID);
                typeIdWhere.append(" ) ");

                return typeIdWhere.toString();

            }
            else
            {
                ExceptionUtil.handleNoTypeId(this, containerName);
            }
        }

        if (typeId != 0)
            typeIdWhere.append(NAME_TYPE_ID).append(" = ").append(typeId);
        return typeIdWhere.toString();
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {

            // Parent
            if (rpmScope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                RPMObject newParent = null;
                if (isDelta)
                {
                    newParent = rpmObject.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new DocumentElement();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            newParent
                                .setContextName(rpmObject.getContextName());
                            oldParent = loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, true);
                        }
                        else
                        {
                            // In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldParent
                                .setContextName(rpmObject.getContextName());
                            oldParent = loadByPrimaryKey(oldParent, null,
                                messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {// newParent not in cache
                            newParent = new DocumentElement();
                            newParent.setID(oldParentId);
                            newParent
                                .setContextName(rpmObject.getContextName());
                            newParent = loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, false);
                        }
                        rpmObject.setParent(newParent);
                    }
                }
            }

            DocumentElement docIntfElement = (DocumentElement) rpmObject;



            if(rpmScope instanceof DocumentScope)
            {

 				DocumentScope scope = (DocumentScope) rpmScope;

                // load children
                if (scope.getChildren() != null)
                {
                    loadDocuments(docIntfElement, scope.getChildren(),
                        messageContext, isDelta);
                }

                // load second level properties

                if (!isDocumentFolder(rpmObject))
                {
                    GenericDocument genericDocument = (GenericDocument) rpmObject;
                    // Document's Author
                    if (scope.isAuthor())
                    {
                        loadDocumentAuthor(genericDocument, scope, relatedFields,
                            messageContext, isDelta, index);
                    }

                    if (scope.isLastCheckedOutBy())
                    {
                        loadLastCheckedOutBy(genericDocument, scope, relatedFields,
                            messageContext, isDelta, index);
                    }

                    if (scope.isDocumentBlobDetails())
                    {
                        loadDocumentBlobDetails(genericDocument, scope,
                            relatedFields, messageContext, isDelta, index);
                    }

                    // Notes Description
                    if (isNoteMinutes(rpmObject))
                    {

                        NoteMinutes noteMin = (NoteMinutes) genericDocument;

                        if (scope.isRtfAssignments())
                        {
                            loadNoteMinutesRtfAssignments(noteMin, messageContext,
                                isDelta);
                        }
                    }
                    else if (isURL(rpmObject))
                    {
                        loadURLContent((UrlDocument) genericDocument,
                            messageContext, isDelta);
                    }

                    // Attribute Assignments
                    if (scope.isAttributeAssignments())
                    {
                        loadDocumentAttributeAssignments(genericDocument,
                            messageContext, isDelta);
                    }

                    if (scope.isState())
                    {
                        loadState(genericDocument, scope, messageContext,
                            relatedFields, index, isDelta);
                    }

                    if (scope.isCustomFieldAssignments())
                    {
                        genericDocument
                            .setCustomFieldAssignments(CustomFieldAssignmentUtil
                                .loadAssignedCustomFields(this, genericDocument
                                    .getCustomFieldAssignments(), genericDocument,
                                    messageContext, isDelta));
                        propagateContextName(genericDocument, genericDocument
                            .getCustomFieldAssignments());
                    }

                    // always load the history
                    // TODO: Uncomment this when status updates are needed to be
                    // loaded
                    /*
                     * loadGenericDocumentHistory(genericDocument, messageContext,
                     * isDelta);
                     */

                    if (scope.getWorkflowProcessesAvailable() != null)
                    {
                        loadAvailableWorkflowProcesses(genericDocument, scope
                            .getWorkflowProcessesAvailable(), messageContext,
                            isDelta);
                    }

                    if (scope.getWorkflowProcessesRunning() != null)
                    {
                        loadRunningWorkflowProcesses(genericDocument, scope
                            .getWorkflowProcessesRunning(), messageContext, isDelta);
                    }

		        

		            //external parent (proposal, Deliverable, clientcostcenter...)
		            loadExternalParent(docIntfElement, scope, messageContext,
	                relatedFields, index, isDelta);
                }
                
                // containingProject -- since version 37 of the file, it was
                // not loaded correctly. There is no justification why the
                // "fix" at version 37 took place.
                loadContainingProject(docIntfElement, scope, messageContext,
                    relatedFields, index, isDelta);
                
                // DocumentElement resourceAssignement
                loadDocumentResourceAssignments(docIntfElement, scope,
                    messageContext, isDelta);
            }
        }
        return rpmObject;
    }

    private void loadContainingProject(DocumentElement docIntfElement, DocumentScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        if (scope != null && scope.getContainingProject() != null)
        {
        	String contextName = docIntfElement.getContextName();
            boolean isScopeContext = ContextUtil.getInstance().isScopeContext(contextName);
            boolean isWbsContext = ContextUtil.getInstance().isWbsContext(contextName);

            if(isWbsContext || isScopeContext)
            {
                String oldProjectId = (String) relatedFields.get(index,
                        ID_PROJECT_ID);
                GenericProject newProject = null;

                WbsManager wbsManager = (WbsManager)
                	getManagerInstance(GenericProject.class);

                if (isDelta)
                {
                    newProject = docIntfElement.getContainingProject();
                    GenericProject oldProject = null;
                    if (oldProjectId != null)
                    {
                        oldProject = new GenericProject();
                        oldProject.setID(oldProjectId);
                        if (newProject != null && newProject.getID() != null
                                && newProject.getID().equals(oldProjectId))
                        {
                            oldProject = (GenericProject) wbsManager.loadByPrimaryKey(
                                    newProject, scope.getContainingProject(), messageContext, true);
                        }
                        else
                        {
                            //In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldProject = (GenericProject) wbsManager.loadByPrimaryKey(
                                    oldProject, scope.getContainingProject(), messageContext, false);
                        }
                    }
                    docIntfElement.deltaContainingProject(oldProject);
                }
                else
                {
                    if (oldProjectId != null)
                    {
                        if (newProject == null)
                        {//newParent not in cache
                            newProject = new Project();
                            newProject.setID(oldProjectId);
                            newProject = (GenericProject) wbsManager.loadByPrimaryKey(
                                    newProject, scope.getContainingProject(), messageContext, false);
                        }
                        docIntfElement.setContainingProject(newProject);
                    }
                }
            }
        }
    }

    private void loadExternalParent(DocumentElement docIntfElement, DocumentScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        if (scope != null && scope.getExternalParent() != null)
        {
        	String contextName = docIntfElement.getContextName();

            String parentRank = (String) relatedFields.get(index,
                    ID_PARENT_RANK);
            parentRank = parentRank.substring(parentRank.indexOf("#")+1);

            String projectId = (String) relatedFields.get(index,
                    ID_PROJECT_ID);

        	boolean isWbsContext = ContextUtil.getInstance().isWbsContext(contextName);
        	boolean isScopeContext = ContextUtil.getInstance().isScopeContext(contextName);
            boolean isResourceContext = ContextUtil.getInstance().isResourceContext(contextName);
            if(isWbsContext)
            {
            	WbsManager wbsManager = (WbsManager) getManagerInstance(WorkElement.class);

				String extraWhere = " TMT_WBS.RANK = '" + parentRank
						+ "' AND TMT_WBS.PROJECT_ID = '" + projectId + "'";

				ArrayList temp = wbsManager.loadByForeignKey(null, scope
						.getExternalParent(), messageContext, this, null,
						extraWhere, ContextUtil.WBS_CONTEXT, isDelta);

				WorkElement externalParent = null;
				if (temp != null && !temp.isEmpty()) {
					externalParent = (WorkElement) temp.get(0);
				}
				docIntfElement.setExternalParent(externalParent);
            }
            else if(isScopeContext)
            {
            	//TODO clarify this
//            	if(docIntfElement instanceof GenericDocument)
//            	{
                	ScopeManager scopeManager = (ScopeManager) getManagerInstance(ScopeElement.class);

    				String extraWhere = " TMT_CRI.RANK = '" + parentRank
    						+ "' AND TMT_CRI.PROJECT_ID = '" + projectId + "'";

    				ArrayList temp = scopeManager.loadByForeignKey(null, scope
    						.getExternalParent(), messageContext, this, null,
    						extraWhere, ContextUtil.SCOPE_CONTEXT, isDelta);

    				ScopeElement externalParent = null;
    				if (temp != null && !temp.isEmpty()) {
    					externalParent = (ScopeElement) temp.get(0);
    				}
    				docIntfElement.setExternalParent(externalParent);
//            	}
            }
            else if(isResourceContext)
            {
                ResourceManager resourceManager = (ResourceManager) getManagerInstance(Resource.class);

                String extraWhere = ResourceManager.NAME_RESOURCE_ID + " = '"
                        + projectId + "'";

                ArrayList temp = resourceManager.loadByForeignKey(null, scope
                    .getExternalParent(), messageContext, this, null,
                    extraWhere, ContextUtil.RESOURCE_CONTEXT, isDelta);

                WorkElement externalParent = null;
                if (temp != null && !temp.isEmpty()) {
                    externalParent = (WorkElement) temp.get(0);
                }
                docIntfElement.setExternalParent(externalParent);
            }
        }
    }

    private void loadLastCheckedOutBy(GenericDocument genericDocument,
            DocumentScope scope, FieldValueMap relatedFields,
            MessageContext messageContext, boolean isDelta, int index)
            throws RPMException, ParseException, SQLException
    {
        String resourceID = (String) relatedFields.get(index, ID_REC_USER);

        ResourceManager manager = (ResourceManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Resource.class.getName());

        Resource lookingFor = new Resource();
        lookingFor.setID(resourceID);

        lookingFor = (Resource) manager.loadByPrimaryKey(lookingFor, null,
            messageContext, isDelta);

        if (isDelta)
        {
            genericDocument.deltaLastCheckedOutBy(lookingFor);
        }
        else
        {
            genericDocument.setLastCheckedOutBy(lookingFor);
        }
    }

    private void loadDocumentBlobDetails(GenericDocument genericDocument,
            DocumentScope scope, FieldValueMap relatedFields,
            MessageContext messageContext, boolean isDelta, int index)
            throws RPMException, ParseException, SQLException
    {
        String blobID = (String) relatedFields.get(index, ID_BLOB_ID);

        DocumentBlobDetailManager manager = (DocumentBlobDetailManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(DocumentBlobDetailManager.class.getName());

        DocumentBlobDetails key = new DocumentBlobDetails();
        key.setID(blobID);

        key = (DocumentBlobDetails) manager.loadByPrimaryKey(key, null,
            messageContext, isDelta);

        if (isDelta)
        {
            genericDocument.deltaDocumentBlobDetails(key);
        }
        else
        {
            genericDocument.setDocumentBlobDetails(key);
        }
    }

    private void loadDocumentAuthor(GenericDocument genericDocument,
            DocumentScope scope, FieldValueMap relatedFields,
            MessageContext messageContext, boolean isDelta, int index)
            throws RPMException, ParseException, SQLException
    {
        String authorID = (String) relatedFields.get(index, ID_AUTHOR_ID);
        ResourceManager manager = (ResourceManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Resource.class.getName());

        Resource lookingFor = new Resource();
        lookingFor.setID(authorID);

        lookingFor = (Resource) manager.loadByPrimaryKey(lookingFor, null,
            messageContext, isDelta);

        if (isDelta)
        {
            genericDocument.deltaAuthor(lookingFor);
        }
        else
        {
            genericDocument.setAuthor(lookingFor);
        }
    }

    private void loadURLContent(UrlDocument url, MessageContext messageContext,
            boolean isDelta) throws RPMException
    {
        String selectStmt = "SELECT ATTACHEMENT FROM DOCUMENT_BLOBS WHERE ELEMENT_ID = (SELECT BLOB_ID FROM "
                + getTableName(url) + " WHERE ELEMENT_ID  = ?)";

        Object[] args = new Object[]
        {
            url.getID()
        };
        List result = ManagerUtil.select(new Class[]
        {
            String.class
        }, null, null, new SqlBuffer(selectStmt), args, messageContext);

        String urlContent = null;
        if (result != null && result.size() != 0)
        {
            ArrayList row = (ArrayList) (result.get(0));
            if (row != null && row.size() != 0)
            {
                urlContent = (String) row.get(0);
            }
        }

        if (isDelta)
        {
            url.deltaUrl(urlContent);
        }
        else
        {
            url.setUrl(urlContent);
        }
    }

    private void loadNoteMinutesRtfAssignments(NoteMinutes noteMin,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
            .getName());

        Object[] args = new Object[]
        {
            noteMin.getID()
        };

        RtfAssignment[] oldNoteMinutesRtfAssignments = noteMin.getRtfAssignments();

        ArrayList temp = rtfAssignmentManager
            .loadByForeignKeyWithSpecifiedParent(noteMin,
                oldNoteMinutesRtfAssignments, null, messageContext, this, args,
                null, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, isDelta);

        RtfAssignment[] noteMinutesRtfAssignments = null;
        if (temp != null && !temp.isEmpty())
        {
            noteMinutesRtfAssignments = new RtfAssignment[temp.size()];
            temp.toArray(noteMinutesRtfAssignments);
        }
        noteMin.setRtfAssignments(noteMinutesRtfAssignments);
    }

    private void loadDocumentResourceAssignments(
            DocumentElement docIntfElement, DocumentScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if (scope != null && scope.isResourceAssignments())
        {
            ResourceRoleAssignmentManager manager = (ResourceRoleAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceRoleAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(docIntfElement
                .getResourceAssignments(), null, messageContext, this,
                new Object[]
                {
                    docIntfElement.getID()
                }, null, docIntfElement.getContextName(), isDelta);

            if (temp != null)
            {
                ResourceRoleAssignment[] resAss = new ResourceRoleAssignment[temp
                    .size()];
                temp.toArray(resAss);

                for (int i = 0; i < resAss.length; i++)
                {

                    resAss[i].setParent(docIntfElement);

                }

                docIntfElement.setResourceAssignments(resAss);
            }
        }
    }

    private void loadDocumentAttributeAssignments(
            GenericDocument genericDocument, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(AttributeAssignment.class
            .getName());

        Object[] args = new Object[]
        {
            genericDocument.getID()
        };
        ArrayList temp = attributeAssignmentManager
            .loadByForeignKeyWithSpecifiedParent(genericDocument,
                genericDocument.getAttributeAssignments(), null,
                messageContext, this, args, null, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT,
                isDelta);

        AttributeAssignment[] aa = null;
        if (temp != null && !temp.isEmpty())
        {
            aa = new AttributeAssignment[temp.size()];
            temp.toArray(aa);
        }
        genericDocument.setAttributeAssignments(aa);
    }

    private void loadState(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        DocumentScope scope = (DocumentScope) rpmScope;
        if (rpmObject instanceof GenericDocument)
        {
            if (scope != null)
            {
                DatafieldState state = createState(rpmObject);

                state.setID((String) relatedFields.get(index, ID_STAGE_ID));
                DatafieldManager manager = (DatafieldManager) getManagerInstance(state
                    .getClass().getName());
                state = (DatafieldState) manager.loadByPrimaryKey(state, null,
                    messageContext, isDelta);
                if (isDelta)
                {
                    if (isNoteMinutes(rpmObject))
                    {
                        ((NoteMinutes) rpmObject).deltaState((NoteState) state);
                    }
                    else
                    {
                        ((AbstractDocument) rpmObject)
                            .deltaState((DocumentState) state);
                    }
                }
                else
                {
                    if (isNoteMinutes(rpmObject))
                    {
                        ((NoteMinutes) rpmObject).setState((NoteState) state);
                    }
                    else
                    {
                        ((AbstractDocument) rpmObject)
                            .setState((DocumentState) state);
                    }
                }
            }
        } // if
    }

    private void loadRunningWorkflowProcesses(GenericDocument genericDocument,
            RunningWorkflowProcessScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                RunningWorkflowProcess.class.getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
            genericDocument, genericDocument.getWorkflowProcessesRunning(),
            scope, messageContext, this, new Object[]
            {
                genericDocument.getID()
            }, null, genericDocument.getContextName(), isDelta);

        RunningWorkflowProcess[] workflowProcesses = new RunningWorkflowProcess[0];
        if (temp != null)
        {
            workflowProcesses = new RunningWorkflowProcess[temp.size()];
            temp.toArray(workflowProcesses);
            genericDocument.setWorkflowProcessesRunning(workflowProcesses);
        }
    }

    private void loadAvailableWorkflowProcesses(
            GenericDocument genericDocument,
            AvailableWorkflowProcessScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AvailableWorkflowProcessManager manager = (AvailableWorkflowProcessManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                AvailableWorkflowProcess.class.getName());

        Integer groupTypeId = SP_S_WKF_PROCESS_TYPE(genericDocument
            .retrieveTypeID(), messageContext);

        String rank = WorkflowUtil.loadWorkflowProcessRankThroughParent(
            genericDocument, groupTypeId, messageContext);

        if (rank != null) // rank should never be null
        {
            SqlBuffer extraWhere = new SqlBuffer();
            extraWhere.appendSubSelect(
                AvailableWorkflowProcessManager.NAME_PROCESS_ID,
                WorkflowManager.NAME_ELEMENT_ID, WorkflowManager.TABLE_NAME,
                WorkflowManager.NAME_RANK + " like '" + rank + "%')");

            String projectId = WorkflowUtil.loadAssociatedProjectID(
                genericDocument, messageContext);

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                genericDocument, genericDocument
                    .getWorkflowProcessesAvailable(), scope, messageContext,
                this, new Object[]
                {
                    projectId
                }, extraWhere.toString(), genericDocument.getContextName(),
                isDelta);

            AvailableWorkflowProcess[] workflowProcesses = new AvailableWorkflowProcess[0];
            if (temp != null)
            {
                workflowProcesses = new AvailableWorkflowProcess[temp.size()];
                temp.toArray(workflowProcesses);
                genericDocument
                    .setWorkflowProcessesAvailable(workflowProcesses);
            }
        }
    }

    /**
     * @param rpmObject
     * @return
     */
    private DatafieldState createState(RPMObject rpmObject)
    {
        DatafieldState state = null;
        if (isNoteMinutes(rpmObject))
            state = new NoteState();
        else
            state = new DocumentState();

        return state;
    }

    private String constructLoadDocumentsExtraWhereClause(int[] types)
    {
        if (types != null && types.length != 0)
        {
            StringBuffer whereClause = new StringBuffer();
            whereClause.append("TYPE_ID IN (");
            for (int i = 0; i < types.length; i++)
            {
                if (i != 0)
                {
                    whereClause.append(" , ");
                }
                whereClause.append(types[i]);
            }
            whereClause.append(" ) ");
            return whereClause.toString();
        }
        return null;
    }

    private void loadDocuments(DocumentElement docIntfElement,
            DocumentScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        DocumentElement[] oldChildren = docIntfElement.getChildren();

        // int[] types = new int[]{DocumentFolder.TYPE_ID, Document.TYPE_ID,
        // WPDoc.TYPE_ID,
        // DelDoc.TYPE_ID, Software.TYPE_ID, UrlDocument.TYPE_ID,
        // NoteMinutes.TYPE_ID};
        // String extraWhereClause =
        // constructLoadDocumentsExtraWhereClause(types);

        String extraWhereClause = "";

        Object[] args = null;

        RPMObject parent = docIntfElement.getParent();
        String contextName = docIntfElement.getContextName();
        boolean isScopeContext = ContextUtil.getInstance().isScopeContext(
            contextName);
        boolean isWbsContext = ContextUtil.getInstance().isWbsContext(
            contextName);
        if (isScopeContext && parent != null && parent instanceof AbstractScope)
        {
            // Include parent_type_id = 0 for templated elements
            String scopeExtraWhere = " AND PARENT_RANK = (SELECT CONCAT('CRI#', RANK) FROM TMT_CRI WHERE ELEMENT_ID = ?)"
                    + " AND (PARENT_TYPE_ID IN (? , 0)) ";

            extraWhereClause += scopeExtraWhere;

            args = new Object[]
            {
                    docIntfElement.getID(), parent.getID(),
                    parent.retrieveTypeID()
            };
        }
        else if (isWbsContext && parent != null
                && parent instanceof WorkElement)
        {
            String wbsExtraWhere = null;

            if (parent instanceof GenericProject)
            {
                // Generic projects must see all the documents attached to them
                // + the ones attached to their children
                // so no condition on the PARENT_RANK or the PARENT_TYPE_ID
                args = new Object[]
                {
                    docIntfElement.getID()
                };
            }
            else
            {
            	String parentId = null;
            	Integer parentTypeId = null;
            	// Include parent_type_id = 0 for templated elements
            	WbsManager wbsManager = (WbsManager) getManagerInstance(WorkElement.class
                        .getName());
            	WorkElementScope workElementScope = new WorkElementScope();
            	workElementScope.setScopeElement(new ScopeElementScope());
            	WorkElement loadedElement = (WorkElement)wbsManager.load(parent, workElementScope, messageContext);
            	if(loadedElement.getScopeElement()== null)
            	{
            		wbsExtraWhere = " AND PARENT_RANK = (SELECT CONCAT('WBS#', RANK) FROM TMT_WBS WHERE ELEMENT_ID = ?) ";
            		parentId = parent.getID();
            		parentTypeId = parent.retrieveTypeID(); 
                }
            	else
            	{
            		// Implies that this element is transferred from Scope to WBS. 
            		// Hence the documents attached to this element will have the 
            		// reference of its associate scope element. 
            		wbsExtraWhere = " AND PARENT_RANK = (SELECT CONCAT('CRI#', RANK) FROM TMT_CRI WHERE ELEMENT_ID = ?)";
            		ScopeElement scopeElement = loadedElement.getScopeElement();
            		parentId = scopeElement.getID();
            		parentTypeId = scopeElement.retrieveTypeID();
                }
            	wbsExtraWhere += " AND PARENT_TYPE_ID IN (? , 0)";
                extraWhereClause += wbsExtraWhere;
                
                args = new Object[]
                {
                        docIntfElement.getID(), parentId,
                        parentTypeId
                };
            }

        }
        else
        {
            args = new Object[]
            {
                docIntfElement.getID()
            };
        }

        ArrayList temp = loadByForeignKeyWithSpecifiedParent(docIntfElement,
            oldChildren, scope, messageContext, this, args, extraWhereClause,
            contextName, isDelta);

        DocumentElement[] children = null;
        if (temp != null && !temp.isEmpty())
        {
            children = new DocumentElement[temp.size()];
            temp.toArray(children);
        }
        docIntfElement.setChildren(children);
    }

    // private void loadGenericDocumentHistory(GenericDocument genericDocument,
    // MessageContext messageContext, boolean isDelta)
    // throws RPMException, SQLException, ParseException
    // {
    //
    // StatusUpdateManager statusUpdateManager = (StatusUpdateManager)
    // RPMManagerFactory
    // .getInstance()
    // .getRPMObjectManager(
    // StatusUpdate.class.getName());
    // ArrayList statusUpdates = statusUpdateManager
    // .loadStatusUpdates(genericDocument,
    // null, messageContext, this, new Object[]
    // {
    // genericDocument.getID()
    // }, null, ContextUtil.DOCUMENT_CONTEXT, isDelta);
    // if (statusUpdates != null)
    // {
    // StatusUpdate[] statusUpdatesArray = new
    // StatusUpdate[statusUpdates.size()];
    // statusUpdates.toArray(statusUpdatesArray);
    // genericDocument.setStatusUpdate(statusUpdatesArray);
    // }
    //
    // }

    /**
     * @param pObject
     * @throws SQLException
     * @throws RPMException
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        // load parent if necessary as it's used to determine the tableType
        if (rpmObject.getParent() == null)
        {
            RPMObject parent = getParent(rpmObject, messageContext);
            rpmObject.setParent(parent);
        }

        // set context if necessary
        if (rpmObject.getContextName() == null)
        {
            String contextName = determineContextName(rpmObject);
            rpmObject.setContextName(contextName);
        }

        //set default values (if necessary)for generic documents
        if(rpmObject instanceof GenericDocument)
        {
        	GenericDocument genericDocument = (GenericDocument)rpmObject;
            if(genericDocument.getVisibleAtParent()==null)
            {
            	genericDocument.setVisibleAtParent(new Boolean(true));
            }
            if(genericDocument.getElementSecurityType()==null)
            {
            	genericDocument.setElementSecurityType(ElementSecurityType.ProjectLevel);
            }
        }

        rpmObject.setID(SP_I_DOC(rpmObject, messageContext));
    }

    /**
     *
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        DocumentScope scope = (DocumentScope) rpmScope;

        DocumentElement docElement = (DocumentElement) rpmObject;

        // load parent if necessary as it's used to determine the tableType
        if (rpmObject.getParent() == null)
        {
            RPMObject parent = getParent(rpmObject, messageContext);
            rpmObject.setParent(parent);
        }

        // set context if necessary
        if (rpmObject.getContextName() == null)
        {
            String contextName = determineContextName(rpmObject);
            rpmObject.setContextName(contextName);
        }

        // save children part
        if (scope != null)
        {
            rpmObject = updateChildren(rpmObject, scope, messageContext);
        }

        if (docElement.testNameModified())
        {
            SP_U_DOC_NAME(rpmObject, messageContext);
        }

        if (isDocumentFolder(rpmObject))
        {
            DocumentFolder docFolder = (DocumentFolder) rpmObject;
            if (docFolder.testPrivateFolderModified())
            {
                SP_U_DOC_PUBLISHED(docFolder, messageContext);
            }
        }
        else
        {
            GenericDocument docNonFolder = (GenericDocument) rpmObject;

            // ElementSecurity (Published) gets updated in the below procedure
            if (docNonFolder.testVisibleAtParentModified()||docNonFolder.testElementSecurityTypeModified())
            {
                SP_U_DOC_PUBLISHED(docNonFolder, messageContext);
            }

            // TODO: lock needs to be taken care later
            // to lock/unlock, the object should not have been checked out
            // but since at this point the object is always checked out
            // if (!isCheckedOut(messageContext, docNonFolder,
            // getUser(messageContext))
            // && docNonFolder.testLockedModified())
            // {
            // SP_LOCK_DOC(docNonFolder, messageContext);
            // }

            if (isNoteMinutes(rpmObject))
            {
                // NoteMinutes Description
                NoteMinutes noteMin = (NoteMinutes) rpmObject;
                if (scope != null && scope.isRtfAssignments())
                {
                    noteMin = saveNoteMinutesRtfAssignments(noteMin, scope,
                        messageContext);
                }

                if (noteMin.testRankModified()
                        || noteMin.testPriorityModified())
                {
                    SP_U_DOC_PRIORITY_RANK(noteMin, messageContext);
                }

                // change state
                if (noteMin.testStateModified())
                {
                    SP_U_DOC_REVISION(rpmObject, messageContext);
                }

            }
            else
            { // AbstractDocument
                AbstractDocument absDocElement = (AbstractDocument) rpmObject;
                if (absDocElement.testKeywordsModified()
                        || absDocElement.testSummaryModified())
                {
                    SP_U_DOC_DETAIL(absDocElement, messageContext);
                }

                if (absDocElement.testReferenceNumberModified()
                        || (isURL(absDocElement) && isURLContentModified((UrlDocument) absDocElement)))
                {
                    SP_U_DOC(absDocElement, messageContext);
                }

                // change state
                if (absDocElement.testStateModified())
                {
                    SP_U_DOC_REVISION(rpmObject, messageContext);
                }
            }

            // Document Attribute assignments
            if (scope != null)
            {
                saveDocumentAttributeAssignments(docNonFolder, rpmScope,
                    messageContext);

                propagateContextName(docNonFolder, docNonFolder
                    .getCustomFieldAssignments());
                docNonFolder
                    .setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
                        docNonFolder, docNonFolder.getCustomFieldAssignments(),
                        scope.isCustomFieldAssignments(), messageContext, null));
            }
        }
        saveDocumentResourceRoleAssignment(docElement, scope, messageContext);

        return rpmObject;
    }

    public GenericDocument lockDocument(GenericDocument documentToLock, MessageContext messageContext) throws RPMException, SQLException, ParseException
    {
    	return handleDocumentLockAndUnlock(documentToLock, true, messageContext);
    }
    
    public GenericDocument unLockDocument(GenericDocument documentToUnlock, MessageContext messageContext) throws RPMException, SQLException, ParseException
    {
    	return handleDocumentLockAndUnlock(documentToUnlock, false, messageContext);
    }
    
    private GenericDocument handleDocumentLockAndUnlock(GenericDocument document, boolean isLock, MessageContext messageContext) throws RPMException, SQLException, ParseException
    {
    	
    	GenericDocument reloaded = (GenericDocument) loadByPrimaryKey(document, null, messageContext, false);
    	
    	GenericDocument outputElement = null;
    	
    	GenericDocumentCheckpoint.validateDocumentLockAndUnlock(reloaded, isLock, messageContext);
    	
    	if(messageContext.isSuccessful())
    	{
    		if(GenericDocumentCheckpoint.canLockUnlock(reloaded, isLock, messageContext))
    		{
    			SP_LOCK_DOC(reloaded, isLock, messageContext);

    			//To be safe, remove inputGenericDocumentElement from messageContext cache -if it was there-
    			//since now it has been converted which means the instance that 
    			//-might- exist in the cache is not valid anymore
    			messageContext.getCache().removeContainerFromCache(document);

    			outputElement = (GenericDocument) loadByPrimaryKey(
    					reloaded, null, messageContext, false);
    		}
    	}
    	
    	return outputElement;
    }
    
    private boolean isURLContentModified(UrlDocument url)
    {
        return url.testUrlModified() || url.testRUPLinkModified()
                || url.testProcessLinkModified();
    }

    protected RPMObject updateChildren(RPMObject rpmObject,
            DocumentScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        DocumentElement docElement = (DocumentElement) rpmObject;
        propagateContextName(docElement, docElement.getChildren());
        docElement.setChildren((DocumentElement[]) updateChildren(docElement,
            docElement.getChildren(), scope.getChildren(), messageContext,
            DocumentElement.class));
        return rpmObject;
    }

    private boolean isDocumentFolder(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == DocumentFolder.TYPE_ID
                || rpmObject.retrieveTypeID().intValue() == 145;
    }

    private boolean isURL(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == UrlDocument.TYPE_ID;
    }

    private boolean isNoteMinutes(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == NoteMinutes.TYPE_ID;
    }

    private NoteMinutes saveNoteMinutesRtfAssignments(NoteMinutes noteMin,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        DocumentScope scope = (DocumentScope) rpmScope;
        noteMin.setRtfAssignments((RtfAssignment[]) updateChildren(noteMin,
            noteMin.getRtfAssignments(), scope.isRtfAssignments(),
            messageContext, RtfAssignment.class));
        
        return noteMin;
    }

    private GenericDocument saveDocumentAttributeAssignments(
            GenericDocument docNonFolder, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        DocumentScope scope = (DocumentScope) rpmScope;
        docNonFolder
            .setAttributeAssignments((AttributeAssignment[]) updateChildren(
                docNonFolder, docNonFolder.getAttributeAssignments(), scope
                    .isAttributeAssignments(), messageContext, null));
        return docNonFolder;
    }

    private DocumentElement saveDocumentResourceRoleAssignment(
            DocumentElement pObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        DocumentScope scope = (DocumentScope) rpmScope;
        if (scope != null && scope.isResourceAssignments())
        {
            pObject
                .setResourceAssignments((ResourceRoleAssignment[]) updateChildren(
                    pObject, pObject.getResourceAssignments(), scope
                        .isResourceAssignments(), messageContext,
                    ResourceRoleAssignment.class));
        }
        return pObject;
    }

    private RPMObject getParent(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        if (rpmObject.getParent() != null)
        {
            return rpmObject.getParent();
        }
        if (rpmObject.getID() != null)
        {
            // reload the object with its parent in the scope, to be able to
            // retrieve the parent ID
            RPMObject current = new DocumentElement();
            current.setID(rpmObject.getID());
            current.setContextName(rpmObject.getContextName());
            DocumentScope scope = new DocumentScope();
            scope.setParent(new DocumentScope());
            current = loadByPrimaryKey(current, scope, messageContext, false);
            if (current != null)
            {
                return current.getParent();
            }
        }
        return null;
    }

    private String getParentId(RPMObject rpmObject)
    {
        if (rpmObject != null && rpmObject.getParent() != null)
        {
            return rpmObject.getParent().getID();
        }
        return null;
    }

    private void SP_D_DOC(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_DOC";

        Object args[] = new Object[5];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = ManagerUtil.getParentId(this, rpmObject, messageContext,
            getTableName(rpmObject.getContextName()));

        args[3] = "X";
        args[4] = getUser(messageContext).getID();

        // The needed parameters for the SP are:
        // TABLE, ELEMENT_ID, PARENT_ID, REC_STATUS, REC_USER
        executeProcedure(messageContext, name, args);
    }

    private boolean isSpecialScopeCase(RPMObject rpmObject)
    {
        // special scope case, is when we add a documentElement in a Scope
        // context
        // and we're adding it directly below the root document folder
        boolean isScopeContext = ContextUtil.getInstance().isScopeContext(
            rpmObject.getContextName());
        if (isScopeContext)
        {
            if (rpmObject.getParent() != null
                    && rpmObject.getParent() instanceof DocumentFolder)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent.getParent() != null
                        && parent.getParent() instanceof AbstractScope)
                {
                    return true;
                }
            }
        }
        return false;
    }

    private boolean isSpecialWbsCase(RPMObject rpmObject)
    {
        // special wbs case, is when we add a documentElement in a wbs context
        // and we're adding it directly below an element that is a workElement
        // but not a GenericProject
        boolean isScopeContext = ContextUtil.getInstance().isWbsContext(
            rpmObject.getContextName());
        if (isScopeContext)
        {
            if (rpmObject.getParent() != null
                    && rpmObject.getParent() instanceof DocumentFolder)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent.getParent() != null
                        && parent.getParent() instanceof WorkElement
                        && !(parent.getParent() instanceof GenericProject))
                {
                    return true;
                }
            }
        }
        return false;
    }

    private String loadScopeDocumentRootFolderId(DocumentFolder documentFolder,
            MessageContext messageContext) throws RPMException
    {
        String whereClause = " ELEMENT_NAME = 'CRI' AND LEVEL_ID = 1 AND TYPE_ID = 145 AND PROJECT_ID = (SELECT PROJECT_ID FROM TMT_DOCUMENTS WHERE ELEMENT_ID  = ?)";
        return loadSpecialDocumentFolderId(documentFolder.getID(), whereClause,
            messageContext);
    }

    private String loadWbsDocumentRootFolderId(DocumentFolder documentFolder,
            MessageContext messageContext) throws RPMException
    {
        String whereClause = " ELEMENT_NAME = 'WBS' AND LEVEL_ID = 1 AND TYPE_ID = 145 AND PROJECT_ID = (SELECT PROJECT_ID FROM TMT_WBS WHERE ELEMENT_ID  = ?)";
        // validation is done in the is* method so no exception will be thrown
        // here
        return loadSpecialDocumentFolderId(documentFolder.getParent().getID(),
            whereClause, messageContext);
    }

    private String loadSpecialDocumentFolderId(String parentId,
            String whereClause, MessageContext messageContext)
            throws RPMException
    {
        String selectStmt = "SELECT ELEMENT_ID FROM TMT_DOCUMENTS WHERE "
                + whereClause;

        Object[] args = new Object[]
        {
            parentId
        };
        List result = ManagerUtil.select(new Class[]
        {
            String.class
        }, null, null, new SqlBuffer(selectStmt), args, messageContext);

        if (result != null && result.size() != 0)
        {
            ArrayList row = (ArrayList) (result.get(0));
            if (row != null && row.size() != 0)
            {
                return (String) row.get(0);
            }
        }
        return null;
    }

    private String SP_I_DOC(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_I_DOC";

        Object args[] = new Object[6];

        if (isSpecialScopeCase(rpmObject))
        {
            // TMT_CRI table type
            args[0] = new Integer(3);
            args[1] = ((DocumentElement) rpmObject).getName();

            DocumentFolder documentFolder = (DocumentFolder) rpmObject
                .getParent();
            AbstractScope abstractScope = (AbstractScope) documentFolder
                .getParent();

            args[2] = loadScopeDocumentRootFolderId(documentFolder,
                messageContext);
            args[3] = abstractScope.getID();
        }
        else if (isSpecialWbsCase(rpmObject))
        {
            // TMT_WBS table type
            args[0] = new Integer(1);
            args[1] = ((DocumentElement) rpmObject).getName();

            DocumentFolder documentFolder = (DocumentFolder) rpmObject
                .getParent();
            WorkElement workElement = (WorkElement) documentFolder.getParent();

            args[2] = loadWbsDocumentRootFolderId(documentFolder,
                messageContext);
            args[3] = workElement.getID();
        }
        else
        {
            // TMT_*_DOCUMENTS table type
            args[0] = getTableType(rpmObject);
            args[1] = ((DocumentElement) rpmObject).getName();
            args[2] = getParentId(rpmObject);
            args[3] = null;
        }

        args[4] = rpmObject.retrieveTypeID();
        args[5] = getUser(messageContext).getID();
        // The needed parameters for the SP are:
        // TABLE, ELEMENT_NAME, PARENT_ID, FOREIGN_ID, TYPE_ID, REC_USER
        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_U_DOC_NAME(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_NAME";

        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = ((DocumentElement) rpmObject).getName();
        args[3] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_U_DOC_PUBLISHED(DocumentFolder docFolder,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_PUBLISHED";

        Object args[] = new Object[5];
        args[0] = getTableType(docFolder);
        args[1] = docFolder.getID();
        args[2] = docFolder.getPrivateFolder().booleanValue() == true ? "Y"
                : "N";
        args[3] = "Y"; // not applicable for DocumentFolder
        args[4] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_U_DOC_PUBLISHED(GenericDocument docNonFolder,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_PUBLISHED";

        Object args[] = new Object[5];
        args[0] = getTableType(docNonFolder);
        args[1] = docNonFolder.getID();
        args[2] = encodeElementSecurityType(docNonFolder);
        args[3] = docNonFolder.getVisibleAtParent().booleanValue() == true
                ? "Y" : "N";
        args[4] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    // private String SP_LOCK_DOC(GenericDocument docNonFolder,
    // MessageContext messageContext) throws SQLException, RPMException
    // {
    // String name = "SP_LOCK_DOC";
    //
    //
    //
    // Object args[] = new Object[5];
    // args[0] = getTableType(docNonFolder);
    // args[1] = docNonFolder.getID();
    // args[2] = docNonFolder.getLocked().booleanValue() == true ? "L" : "U";
    // args[3] = docNonFolder.getLocked().booleanValue() == true ? new
    // Integer(0) : new Integer(1);
    // args[4] = getUser(messageContext).getID();
    //
    // String elementId = executeProcedure(messageContext, name, args);
    // return elementId;
    // }

    private String SP_U_DOC_DETAIL(AbstractDocument docElement,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_DETAIL";

        Object args[] = new Object[5];
        args[0] = getTableType(docElement);
        args[1] = docElement.getID();
        args[2] = docElement.getKeywords();
        args[3] = docElement.getSummary();
        args[4] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }


    
    private String SP_U_DOC(AbstractDocument docElement,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC";

        Object args[] = new Object[9];
        args[0] = getTableType(docElement);
        args[1] = docElement.getID();
        args[2] = encodeFileName(docElement, messageContext);        
        args[3] = docElement.getReferenceNumber();
        args[4] = isCheckedOut(messageContext, docElement) ? "Y" : "N";
        args[5] = docElement.getPublished() == PublishedType.ToTemplate ? "Y"
                : "N";
        
         if (isURL(docElement)) 
         { 
        	 args[6] = Boolean.TRUE.equals(((UrlDocument)docElement).getRUPLink())? "1" : "0"; 
        	 args[7] = Boolean.TRUE.equals(((UrlDocument)docElement).getProcessLink())? "1" : "0"; 
        }
        else 
        {
        	args[6] = "0";
        	args[7] = "0";
         } 
        args[8] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_U_DOC_PRIORITY_RANK(NoteMinutes noteMin,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_PRIORITY_RANK";

        Object args[] = new Object[5];
        args[0] = getTableType(noteMin);
        args[1] = noteMin.getID();
        args[2] = noteMin.getRank();
        args[3] = noteMin.getPriority();
        args[4] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_U_DOC_REVISION(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {

        GenericDocument docNonFolder = (GenericDocument) rpmObject;

        String name = "SP_U_DOC_REVISION";

        Object args[] = new Object[8];
        args[0] = getTableType(rpmObject);
        args[1] = docNonFolder.getID();
        args[2] = new Integer(0);
        args[3] = new Integer(137);
        args[4] = new Integer(4768);

        StatusUpdate[] statusUpdates = docNonFolder.getStatusUpdates();
        if (statusUpdates != null && statusUpdates.length != 0)
        {
            StatusUpdate statusUpdate = statusUpdates[statusUpdates.length - 1]; // getting
                                                                                    // the
                                                                                    // last
                                                                                    // one
            args[5] = statusUpdate.getChangeBrief();
        }

        // state
        if (isNoteMinutes(rpmObject))
        {
            args[6] = ManagerUtil.getIntId(((NoteMinutes) rpmObject).getState());
        }
        else
        {
            args[6] = ManagerUtil.getIntId(((AbstractDocument) rpmObject).getState());
        }
        args[7] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private void SP_CHECK_IN_DOC(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_CHECK_IN_DOC";

        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = new Integer(0);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_OUT_WKF(RPMObject rpmObject, String workflowID,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_CHECK_OUT_WKF";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = workflowID;
        args[2] = new Integer(4); // table type for document.
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_OUT_DOC(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_CHECK_OUT_DOC";

        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = new Integer(0);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_UNDO_DOC(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_UNDO_DOC";

        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = new Integer(0);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }
    
    //SP_LOCK_DOC( 4, 'DOCUMENT_ID','L',0,'USER_ID')
    //SP_LOCK_DOC( 4, 'DOCUMENT_ID', 'U', 1, 'USER_ID')
    private void SP_LOCK_DOC(GenericDocument document, boolean isLock, MessageContext messageContext) throws SQLException, RPMException
    {
    	final String procedureName = "SP_LOCK_DOC";
    	
    	Object args[] = new Object[5];
    
    	args[0] = getTableType(document);
    	
    	//Document Id
    	args[1] = document.getID();
    	
    	if(isLock)
    	{
    		args[2] = "L";
    		args[3] = new Integer(0);
    	}
    	else
    	{
    		args[2] = "U";
    		args[3] = new Integer(1);
    	}
    	
    	args[4] = getUser(messageContext).getID();
    	
    	executeProcedure(messageContext, procedureName, args);
    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_CHECK_IN_DOC(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }

    }

    /**
     *
     */
    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_UNDO_DOC(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    /**
     * @param rpmObject
     *
     */
    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            String workflowId = messageContext.getWorkflowId();
            if (workflowId == null)
                SP_CHECK_OUT_DOC(rpmObject, messageContext);
            else
                SP_CHECK_OUT_WKF(rpmObject, workflowId, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {

        //set context if necessary
        if (rpmObject.getContextName() == null)
        {
            String contextName = determineContextName(rpmObject);
            rpmObject.setContextName(contextName);
        }

        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }

    private Integer SP_S_WKF_PROCESS_TYPE(Integer typeId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_S_WKF_PROCESS_TYPE";
        Integer value = null;
        Object args[] = new Object[2];

        args[0] = typeId;
        args[1] = getUser(messageContext).getID();

        // we cannot call framework method executeProcedure directly
        // because SP_S_* procedures do not return an error code
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);
            if (rs.next())
            {
                value = new Integer(rs.getInt(1));
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return value;
    }

    protected String getQualifiedFieldsNames(String tableName)
    {
        // TODO improve performance by constructing the qualified names for all
        // the possible tables (contexts)
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(tableName + "." + FIELD_NAMES[i]);
        }
        return fieldcat.toString();
    }
    
    protected void decodeSecurityRoleIds(MessageContext msgContext,
            ResultSet rs, String contextName) throws RPMException, SQLException
    {
        boolean isScopeContext = ContextUtil.getInstance().isScopeContext(
            contextName);
        boolean isWbsContext = ContextUtil.getInstance().isWbsContext(
            contextName);

        // decode securityRole Ids only for the contexts supporting the
        // security SP (WBS and SCOPE)
        if (isWbsContext || isScopeContext)
        {
            String objectId = rs.getString(ID_ELEMENT_ID);

            String securityRoleIds = null;

            // CRI security
            securityRoleIds = (String) rs.getObject("CRI_SECURITY_ID");
            ManagerUtil.setSecurityRoleIds(msgContext, DocumentElement.class,
                objectId, securityRoleIds);

            // WBS security
            securityRoleIds = (String) rs.getObject("WBS_SECURITY_ID");
            ManagerUtil.setSecurityRoleIds(msgContext, DocumentElement.class,
                objectId, securityRoleIds);

            // DOC security
            securityRoleIds = (String) rs.getObject("DOC_SECURITY_ID");
            ManagerUtil.setSecurityRoleIds(msgContext, DocumentElement.class,
                objectId, securityRoleIds);
        }
    }
    
    protected Object loadWithSecurity(MessageContext messageContext, Connection c,
            String selectStmtStr, Object[] argValues, String contextName)
            throws SQLException, RPMException
    {
        boolean isScopeContext = ContextUtil.getInstance().isScopeContext(contextName);
        boolean isWbsContext = ContextUtil.getInstance().isWbsContext(contextName);

        if(!isWbsContext && !isScopeContext)
        {
        	//use direct select query for contexts other than WBS and CRI while
			// security is not yet implemented for them
        	return super.loadWithSecurity(messageContext, c, selectStmtStr,
					argValues, contextName);
        }
        else
        {
            String name = "SP_S_DOC_PAGES";

            Object args[] = new Object[15];

            //TODO here the table name depends on the context, code must be adjusted after clarifying with Amir

            String[] output = ManagerUtil.processSelectStatementBeforeSecurityCall(selectStmtStr, argValues);
            //In this manager we must use the qualified names instead
            String tableName = output[1];
            String columnList = getQualifiedFieldsNames(tableName);
            String whereClause = output[2];

            //Get loadParams from the messageContext
            LoadParams loadParams = messageContext.getLoadParams();
            Integer startIndex = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getStartIndex()!=null)?loadParams.getStartIndex():new Integer(1);
            // endIndex Default '0' = load all
            Integer endIndex = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getEndIndex()!=null)?loadParams.getEndIndex():new Integer(0);
            String orderBy = (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getOrderBy():null;

            //for internal security mechanism, the loadParams.isForceBuildSecurity is used
//            Integer buildSecurity = (loadParams!=null && ((!loadParams.isAlreadySet() && loadParams.isBuildSecurity())||(loadParams.isForceBuildSecurity())))?new Integer(1):new Integer(0);
            //To improve performance, we will always build the security
            
            // idsType Default '1' = All Project (values are mutually exclusive even if separate arguments exist for projectIDs/portfolioIDs and orgFilterIds)
            //      1=return all projects that are visible by resource
            //      2=ids is a list of project IDs
            //      3=ids is a list of portfolio IDs
            //      4=ids is a organization filter id
            Integer idsType =  (loadParams!=null && !loadParams.isAlreadySet() && loadParams.getIdsType()!=null)?loadParams.getIdsType():new Integer(1);
            String idsList =  (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getIdsList():null;
            String attributesIdsList =  (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getAttributesIdsList():null;
            // viewType Default to All (0:all documents, 1: my documents)
            Integer viewType =  (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getViewType()!=null)?loadParams.getViewType():new Integer(0);
            // inclusions Default to All(0)
            //                  0   -> Include documents attached to both WBS and CRI elements
            //                  128 -> Include documents attached to WBS
            //                  256 -> Include documents attached to CRI
            Integer inclusions = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getInclusions()!=null)?loadParams.getInclusions():new Integer(0);

            whereClause = (loadParams!=null && !loadParams.isAlreadySet() && loadParams.getExtraWhere()!=null)?(whereClause+" "+loadParams.getExtraWhere()+" "):whereClause;

            boolean isProjectIdListUseful = (idsType.intValue()==2)||(idsType.intValue()==3);
            
            if (!isProjectIdListUseful)
            {
                idsList = null;
            }
            
            if (idsList == null || idsList.length() == 0)
            {
                ManagerUtil.IDStorage storage = ManagerUtil.getRestrictedProjectIdList(messageContext,
                    name, whereClause, idsType, idsList, tableName);
                idsList = storage.idsList;
                idsType = storage.idsType;
            }            
            
            args[0] = idsType; // p_bitflag
            args[1] = idsList; // p_ids
            args[2] = (idsType.intValue()==4) ? idsList:null; // p_org_filter_id
            args[3] = getUser(messageContext).getID(); // p_resource_id
            args[4] = viewType;//p_view_type
            args[5] = new Integer(0);//p_stage_id  0 : all stages(filtering will be in the where clause)
            args[6] = new Integer(0);//p_type_id   0 : all types (filtering will be in the where clause)
            args[7] = attributesIdsList; //filter_string
            args[8] = new Integer(1);//Always build_security
            args[9] = inclusions; // p_active_record
            args[10] = whereClause; // p_where_clause
            args[11] = orderBy;//p_order_by
            args[12] = columnList;
            args[13] = startIndex;//p_start_from
            args[14] = endIndex;//p_end_to

            return Manager.executeStoredProcedure(c, name, args, spParamsTypes, true, messageContext, true, useDBStatementsForLOBS);
        }
    }

    private static final int[] spParamsTypes = new int[]{
        java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,
        java.sql.Types.INTEGER,java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER,
        java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER
    }; 
}

