/*
 * Created on Oct 19, 2005
 */
package com.ibm.rpm.document.transfer;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ErrorCodeProperties;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.interfaces.impl.ManagerCaller;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * This code is used to upload files (documents into BLOBs) and download them
 * back. DocumentTransfer class in particular is dedicated to database
 * operations, while DocumentTransferServlet class is dedicated for HTTP Servlet
 * upload/download operations
 *
 * If you are using this class, do not forget to call releaseAll() after you
 * have done your job
 *
 * @author dexmar00
 * @author ikusakov
 */
public class DocumentTransfer
{
    private static Log logger = LogFactory.getLog(DocumentTransfer.class);

    private static String DEFAULT_ATTACHMENT_NAME = "attachment.zip";

    private Connection connection = null;

    private PreparedStatement ps = null;

    private ResultSet rs = null;

    private MessageContext messageContext = null;
    
    private InputStream downloadInputStream = null;
    
    private String attachmentName = null;
    
    private long fileSize = -1;
    
    private FileInputStream tempFileInputStream = null;


    /**
     * <code>ACTION</code><br>
     * Form parameter that determine the operation to be executed by the
     * servlet.
     */
    public static final String ACTION = "ACTION";

    /**
     * <code>ACTION_DOWNLOADZIP</code><br>
     * Download a zipped version of a document from the document blob table to
     * the HTTP Client. The WebService client will have to unzip the document.
     */
    public static final String ACTION_DOWNLOADZIP = "DOWNLOADZIPPED";

    /**
     * <code>ACTION_UPLOADZIP</code><br>
     * Upload a zipped version of a document from the HTTP Client to the
     * document blob table. The WebService client have to zip the document with
     * the right version of GZip before uploading the document.
     */
    public static final String ACTION_UPLOADZIP = "UPLOADZIPPED";

    /**
     * <code>ACTION_DOWNLOADUNZIP</code><br>
     * Download an unzipped document from the document blob table to the HTTP
     * Client. The server will unzip the document for the WebService client.
     */
    public static final String ACTION_DOWNLOADUNZIP = "DOWNLOADUNZIPPED";

    /**
     * <code>ACTION_UPLOADUNZIP</code><br>
     * Upload an unzipped document from the HTTP Client to the document blob
     * table. The server will zip the document before saving it to the document
     * blob table.
     */
    public static final String ACTION_UPLOADUNZIP = "UPLOADUNZIPPED"; // From
                                                                      // the

    // HTTP
    // Client to
    // the
    // document
    // blob
    // table.

    public static final String HEADER_SESSION_ID = "SESSION_ID";

    public static final String HEADER_DOCUMENT_CONTEXT_NAME = "CONTEXT_NAME";

    public static final String HEADER_DOCUMENT_ID = "DOCUMENT_ID";

    // back door. Can be used to download an attachment without
    // checking for the container document.
    public static final String HEADER_UNCOMPRESSED_LENGTH = "UNCOMPRESSED_LENGTH";

    public static final String HEADER_PARENT_ID = "PARENT_ID";

    public static final String HEADER_FILE = "FILE";

    private final int BUFFER_SIZE = 2048;

    private File tempZipFile = null;

    private ServerFactory factory;

    public DocumentTransfer(ServerFactory factoryToUse, String sessionID, String operationName) throws RPMException
    {
        super();
        factory = factoryToUse;
        messageContext = new MessageContext(factory, sessionID, operationName);
        try
        {
            connection = Manager.getConnection(messageContext);
        }
        catch (SQLException e)
        {
            throw new RPMException(e).log();
        }
    }

    //------------ Metadata methods

    public static String getDocumentTableNameForContext(String contextName)
    {
        if (contextName == null)
            return null;
        return getDocumentTableName(getTypeIdForContext(contextName));
    }

    public static String getHistoryDocumentTableNameFromContext(
            String contextName)
    {
        return getHistoryDocumentTableName(""
                + getTypeIdForContext(contextName));
    }

    public static int getTypeIdForContext(String contextName)
    {
        int result = -1;
        if (contextName != null)
        {
            if (contextName.equalsIgnoreCase(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT))
                result = 4;
            else if (contextName.equalsIgnoreCase(ContextUtil.POOL_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.POOL_DOCUMENT_CONTEXT))
                result = 12;
            else if (contextName.equalsIgnoreCase(ContextUtil.RESOURCE_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.RESOURCE_DOCUMENT_CONTEXT))
                result = 13;
            else if (contextName.equalsIgnoreCase(ContextUtil.ASSET_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.ASSET_DOCUMENT_CONTEXT))
                result = 14;
            else if (contextName.equalsIgnoreCase(ContextUtil.CLIENT_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.CLIENT_DOCUMENT_CONTEXT))
                result = 15;
            else if (contextName.equalsIgnoreCase(ContextUtil.WBS_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.WBS_DOCUMENT_CONTEXT))
                result = 4;
            else if (contextName.equalsIgnoreCase(ContextUtil.SCOPE_CONTEXT)
                    || contextName
                        .equalsIgnoreCase(ContextUtil.SCOPE_DOCUMENT_CONTEXT))
                result = 4;

            // (contextName.equalsIgnoreCase(ContextUtil.VENDOR_CONTEXT))
            //				result = 17;
            //			else if
            // (contextName.equalsIgnoreCase(ContextUtil.WORKFLOW_CONTEXT))
            //				result = 22;
            //			else if
            // (contextName.equalsIgnoreCase(ContextUtil.LAYOUT_CONTEXT))
            //				result = 23;
        }
        return result;
    }

    public static String getDocumentTableName(int type)
    {
        String result = "";
        if (type > 0)
        {
            if (type == 4)
                result = "TMT_DOCUMENTS";
            else if (type == 12)
                result = "TMT_POOL_DOCUMENTS";
            else if (type == 13)
                result = "TMT_RESOURCE_DOCUMENTS";
            else if (type == 14)
                result = "TMT_ASSET_DOCUMENTS";
            else if (type == 15)
                result = "TMT_CLIENT_DOCUMENTS";
            else if (type == 17)
                result = "TMT_VENDOR_DOCUMENTS";
            else if (type == 22)
                result = "TMT_WORKFLOW_CYCLES";
            else if (type == 23)
                result = "TMT_LAYOUTS";
        }
        return result;
    }

    public static String getHistoryDocumentTableName(String type)
    {
        String result = "";
        if (type != null)
        {
            int iType = Integer.parseInt(type);
            if (iType == 4)
                result = "HISTORY_DOCUMENTS";
            else if (iType == 12)
                result = "HISTORY_POOL_DOCUMENTS";
            else if (iType == 13)
                result = "HISTORY_RESOURCE_DOCUMENTS";
            else if (iType == 14)
                result = "HISTORY_ASSET_DOCUMENTS";
            else if (iType == 15)
                result = "HISTORY_CLIENT_DOCUMENTS";
            else if (iType == 17)
                result = "HISTORY_VENDOR_DOCUMENTS";
        }
        return result;
    }

    // ----------------

    /**
     * Closes used resultset and statement objects
     */
    public void releaseResultSetAndStatement()
    {
        if (rs != null)
        {
            Manager.close(null, rs);
        }
    }

    /**
     * Closes used resultset, statement and connection objects
     * Deletes temporary zip file
     */
    public void releaseAll()
    {
        releaseResultSetAndStatement();
        if (connection != null)
        {
            Manager.commitAndCloseConnection(messageContext, connection);
            connection = null;
        }
        
        closeInputStream(downloadInputStream);
        closeInputStream(tempFileInputStream);
        downloadInputStream = null;
        tempFileInputStream = null;

        deleteTemporaryFile();
    }

    public static boolean isSessionIdValid(ManagerCaller caller, String sessionID)
    {
        boolean valid;
        try
        {
            // Get the session information from the session database.
            // A little simplistic test? Maybe. We need to know if the
            // session is expired.
            caller.getSystemInfo(sessionID);
            valid = true;
        }
        catch (RPMException e)
        {
            // session is certainly invalid, do we really need to know why.
            valid = false;
        }
        return valid;
    }

    /**
     * @param messageContext
     * @return Resource object representing the current user
     */
    public Resource getUser()
    {
        return ((Resource) messageContext.getUser());
    }

    public String getAttachmentName()
            throws RPMException
    {
        return attachmentName;
    }

    private static void logException(Exception e1)
    {
        logger.error(e1);
        e1.printStackTrace();
    }

    public long getAttachmentSize()
            throws RPMException
    {
        return fileSize;
    }

    public void downloadZipped(OutputStream outputStream, ResultSet rs)
            throws RPMException
    {
        InputStream inputStream = null;
        try
        {
            Blob blob = rs.getBlob("BLOB_DATA");
            if ( blob != null )
            {
	            // Casting to int is OK if compressed size is <= 2,147,483,647 bytes
	            inputStream = blob.getBinaryStream();
	            // Get the optimum buffer size and use this to create the read/write
	            // buffer
	            byte[] buffer = new byte[1024];
	            int length = -1;
	            // Transfer the data
	            while ((length = inputStream.read(buffer)) != -1)
	            {
	                outputStream.write(buffer, 0, length);
	            }
	            outputStream.flush();
            }
        }
        catch (SQLException e)
        {
            throw (new RPMException(e)).log();
        }
        catch (IOException e)
        {
            throw (new RPMException(e)).log();
        }
        finally
        {
            // Close everything down
            closeInputStream(inputStream);
            closeOutputStream(outputStream);
        }
        return;
    }

    private void cacheDownloadUnzipped() throws RPMException
    {
        InputStream inputStream = null;
        BufferedOutputStream bof = null;
        ZipInputStream zis = null;
        FileOutputStream fos = null;
        
        try
        {
            tempZipFile = File.createTempFile("readblob", ".tmp");
            fos = new FileOutputStream(tempZipFile);
            Blob blob = rs.getBlob("BLOB_DATA");
            if( blob != null )
            {
	            InputStream blobBinaryStream = blob.getBinaryStream();
                zis = new ZipInputStream(new BufferedInputStream(
                    blobBinaryStream));

	            ZipEntry entry =  zis.getNextEntry();
	            if (entry != null)
	            {
	                // We will return only the first entry!
	                int count;
	                byte data[] = new byte[BUFFER_SIZE];
	
	                // write the files to the servlet output stream
	                bof = new BufferedOutputStream(fos, BUFFER_SIZE);
	                while ((count = zis.read(data, 0, BUFFER_SIZE)) != -1)
	                {
	                    bof.write(data, 0, count);
	                }
	                
	                bof.flush();
	                fos.flush();
	                
	                closeOutputStream(bof);
	                closeOutputStream(fos);
	                bof = null;
	                fos = null;
	                
	                fileSize = tempZipFile.length();
	            }
            }
        }
        catch (SQLException e)
        {
            throw (new RPMException(e)).log();
        }
        catch (IOException e)
        {
            throw (new RPMException(e)).log();
        }
        finally
        {
            // Close everything down
            closeInputStream(inputStream);
            closeOutputStream(bof);
            closeOutputStream(fos);
            closeInputStream(zis);
        }
    }

    private void getBLOB(String contextName, String id) throws SQLException
    {
        releaseResultSetAndStatement();

        String selectStmt = null;

        if (contextName == null)
        {
            // This is a back door to get a blob when we don't know the Doc ID
            // and the context name but the blob id is known.
            selectStmt = "SELECT BLOB_DATA, ATTACHEMENT FROM DOCUMENT_BLOBS WHERE ELEMENT_ID='"
                    + id + "'";
        }
        else
        {
            // This is the normal usage of the method.
            selectStmt = "SELECT BLOB_DATA, ATTACHEMENT FROM DOCUMENT_BLOBS WHERE ELEMENT_ID=(Select Blob_id from "
                    + getDocumentTableNameForContext(contextName)
                    + " where element_id ='" + id + "') ";
        }
        selectStmt = Manager
                .makeStatementUncommitedRead(selectStmt, connection);
        ps = connection.prepareStatement(selectStmt,
                ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        rs = ManagerUtil.executeSqlQuery(messageContext, ps, selectStmt, null);

        if (!rs.next())
        {
            releaseResultSetAndStatement();
            rs = null;
        }
    }

    /**
     * check if a document exist
     * @param contextName
     * @param id
     * @return true if the document exists
     */
    public void validateDocumentID(
            String contextName, String id) throws RPMException
    {
        releaseResultSetAndStatement();

        try
        {
            String selectStmt = null;

            selectStmt = "SELECT TYPE_ID FROM "
                    + getDocumentTableNameForContext(contextName)
                    + " where element_id ='" + id + "'";

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt,
                connection);
            ps = connection.prepareStatement(selectStmt,
                ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            rs = ManagerUtil.executeSqlQuery(messageContext, ps, selectStmt, null);

            if (rs.next())
            {
                int typeId = rs.getInt(DocumentManager.NAME_TYPE_ID);
                if(typeId == NoteMinutes.TYPE_ID || typeId == UrlDocument.TYPE_ID)
                {
                	String errorMessage = "Upload|Downlaod for the Document (ID =" + id + ") of type Note/Minutes or UrlDocument is not supported."; 
                	logger.error(errorMessage);
                	throw new RPMException(errorMessage);
                }
            }
            else
            {
            	throw new RPMException("Document not found with ID=" + id + " for context "
                            + contextName);
            }
        }
        catch (SQLException e)
        {
            // ignore sql errors
        }
        finally
        {
            releaseResultSetAndStatement();
        }
    }
    
    private ZipInfo readZipInfo(File zipFile) throws RPMException
    {
        ZipInfo retVal = new ZipInfo();
        int numberOfFiles = 0;
        FileInputStream fis=null;
        BufferedInputStream bis=null;
        try
        {
            fis = new FileInputStream(zipFile);
            bis = new BufferedInputStream(fis);
            ZipInputStream zis = new ZipInputStream(bis);
            ZipEntry entry;
            while ((entry = zis.getNextEntry()) != null)
            {
                numberOfFiles++;
                retVal.size = entry.getSize();
                String filename = entry.getName();

                if (filename != null)
                {
                    int index = filename.lastIndexOf('/');
                    if (index != -1)
                    {
                        filename = filename.substring(index + 1, filename
                            .length());
                    }
                }

                retVal.fileName = filename;
            }
        }
        catch (Exception e)
        {
            logException(e);
            throw new RPMException("Failed to read zip file information:"
                    + e.getMessage());
        }
        finally
        {
            closeInputStream(fis);
            closeInputStream(bis);
        }

        if(numberOfFiles != 1)
        {
            throw new RPMException("Invalid zip file. The zip file must contain only one document.");
        }
        return retVal;
    }

    public void uploadZipped(String contextName, String docID,
            String zipFileName, BufferedInputStream is)
            throws SQLException, RPMException
    {
        try
        {
            saveTempZipFile(is);
            closeInputStream(is);
            long unzippedLength = findUnzippedLength();
            uploadZipped(contextName, docID, zipFileName, (int)unzippedLength);
        }
        finally
        {
            deleteTemporaryFile();
        }
    }
    
    /**
     * @return the unzipped length
     * @throws RPMException
     */
    private long findUnzippedLength() throws RPMException
    {
        long retVal = -1; 
        
        ZipInputStream zis = null;
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        
        try
        {
            fis = new FileInputStream(tempZipFile);
            bis = new BufferedInputStream(fis);
            zis = new ZipInputStream(bis);
            
            ZipEntry entry = zis.getNextEntry();
            if (entry != null)
            {
                retVal = 0;
                while (zis.read() != -1)
                {
                    retVal++;
                }
            }
        }
        catch (Exception e)
        {
            logException(e);
            throw new RPMException(
                "Failed to read zip file uncompressed length:" + e.getMessage());
        }
        finally
        {
            closeInputStream(zis);
            closeInputStream(fis);
            closeInputStream(bis);
        }
        return retVal;
    }

    private static class ZipInfo
    {
        public long size=0;
        public String fileName="";
    }

    /**
     * Saves given input stream in the database BLOB as-is, expecting it to be a
     * zip stream. In case it is not a zip stream, zips it first, then saves in
     * BLOB.
     *
     * @param contextName
     * @param docID
     * @param zipFileName
     * @param contentLength
     * @param bis
     * @param uncompressedLength
     * @throws SQLException
     * @throws IOException
     * @throws RPMException
     */
    protected void uploadZipped(String contextName, String docID,
            String zipFileName, int readSize ) throws 
            RPMException, SQLException
    {
    	long uncompressedLength = 0;
    	int contentLength = readContentLength();

    	if (contentLength > 0)
    	{
    		//validate zip file
    		ZipInfo readZipInfo = readZipInfo(tempZipFile);

    		// extract the filename from the zip file if filename is null;
    		String filename = readZipInfo.fileName;

    		// in case this is not a zip - convert to zip
    		if (filename == null)
    		{
    			readSize = convertToZipFile(zipFileName);

    			// read back the zip information after we re-zipped it.
    			readZipInfo = readZipInfo(tempZipFile);
    		}

    		if  (readSize < 0 )
    		{
    			uncompressedLength = readZipInfo.size;
    		}
    		else
    		{
    			uncompressedLength = readSize;
    		}

    		if ( uncompressedLength  < 0 )
    		{
    			throw new RPMException("Failed to read file size: "
    					+ tempZipFile.getAbsolutePath());
    		}

    		DocumentElement documentElement = new DocumentElement();
    		documentElement.setContextName(contextName);
    		documentElement.setID(docID);
    		documentElement.setName(filename);
    		if ((docID != null) && (contextName != null))
    		{
    			try
    			{
    				// Checkout the doc
    				SP_CHECK_OUT_DOC(documentElement);
    			}
    			catch (RPMException e2)
    			{
    				String codeString = e2.getMessage();
    				codeString = codeString.substring(codeString
    						.indexOf("code: ") + 6);
    				codeString = codeString.substring(0, codeString
    						.indexOf(' '));
    				if ((codeString != null) && (codeString.length() > 0)
    						&& (Integer.parseInt(codeString) == 200070))
    				{
    					// Already checked out. That's ok
    					// TODO Check who did it. If it's the administrator
    					// that's OK and go for it, else we'll have to return an
    					// error
    					// For now, we don't care. Overwrite the file and
    					// checkin.
    				}
    				else
    				{
    					logException(e2);
    					throw e2;
    				}
    			}
    			try
    			{
    				// Update Doc Revision
    				SP_U_DOC_REVISION(documentElement);
    			}
    			catch (RPMException e3)
    			{
    				logException(e3);
    			}
    		}
    		// Upload the document attachment to the database
    		boolean success = false;
    		PreparedStatement preparedStatement1 = null;
    		PreparedStatement preparedStatement2 = null;
    		PreparedStatement preparedStatement3 = null;
    		FileInputStream fis = null;
    		BufferedInputStream bis = null;
    		try
    		{
    			fis = new FileInputStream(tempZipFile);
    			bis = new BufferedInputStream(fis);

    			logger.debug("Uploading document. File name = " + filename
    					+ ", compressed size = " + contentLength
    					+ ", uncompressed size = " + uncompressedLength);

    			long timeStamp = System.currentTimeMillis();

    			String updateFileNameStatement = "UPDATE DOCUMENT_BLOBS SET (ATTACHEMENT) =(?) "
    				+ "WHERE ELEMENT_ID=(Select Blob_id from "
    				+ getDocumentTableNameForContext(contextName)
    				+ " where element_id ='" + docID + "')";

    			logger.info(updateFileNameStatement);

    			Object[] args = new Object[]{filename};
    			preparedStatement1 = connection.prepareStatement(updateFileNameStatement);
    			preparedStatement1.setString(1, filename);
    			ManagerUtil.executeSqlUpdate(messageContext, preparedStatement1, updateFileNameStatement, args);

    			if (Manager.isOracle(connection)) {
    				String initializeBlobStatement = "UPDATE DOCUMENT_BLOBS SET (BLOB_DATA) = (empty_blob()) "
    					+ "WHERE ELEMENT_ID=(Select Blob_id from "
    					+ getDocumentTableNameForContext(contextName)
    					+ " where element_id ='" + docID + "')";
    				logger.info(initializeBlobStatement);

    				preparedStatement3 = connection.prepareStatement(initializeBlobStatement);
    				ManagerUtil.executeSqlUpdate(messageContext,
    						preparedStatement3, initializeBlobStatement,
    						null);

    				String sQuery = "SELECT BLOB_DATA FROM DOCUMENT_BLOBS WHERE ELEMENT_ID = (Select Blob_id from "
    					+ getDocumentTableNameForContext(contextName)
    					+ " where element_id ='"
    					+ docID
    					+ "')"
    					+ " FOR UPDATE";
    				logger.info(sQuery);
    				ResultSet rs = ManagerUtil.executeSqlQuery(
    						messageContext, connection, sQuery, null);

    				byte[] buf = new byte[contentLength];
    				try {
    					int count = fis.read(buf, 0, contentLength);
    					if (count == -1) {
    						logger.debug("\nEmpty Stream\n");
    					}
    					if (count < contentLength) {
    						logger.debug("\nIncomplete Stream:" + count
    								+ "\n");
    					}
    				} catch (IOException e1) {
    					logger.error(e1);
    					throw new RPMException(e1);
    				}

    				try {
    					if (rs.next()) {
    						Blob oblob = rs.getBlob(1);
    						if (oblob == null) {
    							String errorMessage = "["
    							                        + docID
    							                        + "][DATABASE] [ERROR] Error :BLOB was NOT updated, EMPTY BLOB LOCATOR";
    							logger.error(errorMessage);
    							throw new RPMException(errorMessage);
    						}

    						((oracle.sql.BLOB) oblob)
    						.open(oracle.sql.BLOB.MODE_READWRITE);
    						OutputStream oStream = ((oracle.sql.BLOB) oblob)
    						.getBinaryOutputStream();

    						oStream.write(buf);
    						oStream.close();
    						((oracle.sql.BLOB) oblob).close();
    					}
    				} catch (RPMException e) {
    					logger.error(e, e);
    					throw e;
    				} catch (IOException e) {
    					logger.error(e, e);
    					throw new RPMException(e);
    				}
    			} else {
    				String updateBlobStatement = "UPDATE DOCUMENT_BLOBS SET (BLOB_DATA) =(?) "
    					+ "WHERE ELEMENT_ID=(Select Blob_id from "
    					+ getDocumentTableNameForContext(contextName)
    					+ " where element_id ='" + docID + "')";

    				logger.info(updateBlobStatement);
    				Object[] args2 = new Object[] { "Binary stream of length: "
    						+ contentLength };
    				preparedStatement2 = connection.prepareStatement(updateBlobStatement);
    				preparedStatement2.setBinaryStream(1, bis,
    						contentLength);
    				ManagerUtil.executeSqlUpdate(messageContext,
    						preparedStatement2, updateBlobStatement, args2);
    			}
    			connection.commit();
    			success = true;

    			long executionTime = System.currentTimeMillis() - timeStamp;

    			logger.info("Document upload successful. File name = "
    					+ filename + ", compressed size = " + contentLength
    					+ ", uncompressed size = " + uncompressedLength
    					+ ", execution time = " + executionTime + " seconds");
    		}
    		catch (SQLException e)
    		{
    			logException(e);
    			if (connection != null)
    			{
    				connection.rollback();
    			}
    		}
    		catch (FileNotFoundException e)
    		{
    			logException(e);
    			throw new RPMException("Failed to read internal zip file:" + e);
    		}
    		finally
    		{
    			closeInputStream(bis);
    			closeInputStream(fis);
    			closePreparedStatement(preparedStatement1);
    			closePreparedStatement(preparedStatement2);
    		}

    		if ((docID != null) && (contextName != null))
    		{
    			try
    			{
    				// Update the doc
    				if (success)
    				{
    					SP_U_DOC(documentElement);
    				}
    			}
    			catch (RPMException e2)
    			{
    				logException(e2);
    			}
    			try
    			{
    				// Update doc local folder
    				if (success)
    				{
    					SP_U_DOC_LOCAL_FOLDER(documentElement,
    							(int)uncompressedLength);
    				}
    			}
    			catch (RPMException e3)
    			{
    				logException(e3);
    			}
    			try
    			{
    				if(messageContext.isAutomaticCheckout())
    				{
    					// checkin the doc
    					SP_CHECK_IN_DOC(documentElement);                        
    				}
    			}
    			catch (RPMException e4)
    			{
    				logException(e4);
    			}
    		}
    	}
    	else
    	{
    		throw new RPMException(
    				"Received an empty document or server was unable to read the input stream.");
    	}
    }
    
    /**
     * @param preparedStatement
     * @throws SQLException
     */
    private void closePreparedStatement(PreparedStatement preparedStatement)
            throws SQLException
    {
        if (preparedStatement != null)
        {
            preparedStatement.close();
        }
    }

    /**
     * @param zipFileName
     * @throws FileNotFoundException
     * @throws IOException
     * @throws RPMException
     */
    private int convertToZipFile(String zipFileName) throws RPMException
    {
        int readSize = -1;
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        try
        {
            fis = new FileInputStream(tempZipFile);
            bis = new BufferedInputStream(fis);
            readSize = internalConvertToZipFile(zipFileName, bis);
        }
        catch (FileNotFoundException e)
        {
            logException(e);
            throw new RPMException("Failed to save internal zip file:" + e);
        }
        finally
        {
            closeInputStream(fis);
            closeInputStream(bis);
        }
        return readSize;
    }

    /**
     * @return
     * @throws FileNotFoundException
     * @throws IOException
     */
    private int readContentLength() throws RPMException
    {
        int contentLength = 0;
        FileInputStream lengthFis=null;
        try
        {
            lengthFis = new FileInputStream(tempZipFile);
            contentLength = lengthFis.available();
            closeInputStream(lengthFis);
        }
        catch(Exception e)
        {
            logException(e);
            throw new RPMException("Failed to read zip file length:" + e);
        }
        finally
        {
            closeInputStream(lengthFis);
        }
        return contentLength;
    }

    public static void closeInputStream(InputStream streamToClose)
    {
        try
        {
            if ( streamToClose != null )
            {
                streamToClose.close();
            }
        }
        catch (IOException e)
        {
            logException(e);
        }
    }

    /**
     * uploadUnzipped is designed to receive via the stream a non compressed
     * file. The file will be zipped and saved in the blob cell of the table
     * row.
     * 
     * @param messageContext
     * @param contextName
     * @param docID
     * @param blobID
     * @param parentID
     * @param filename
     * @param contentLength
     * @param bis
     * @throws RPMException
     * @throws SQLException
     */
    public void uploadUnzipped(String contextName, String docID,
            String filename, BufferedInputStream bis)
            throws RPMException, SQLException
    {
        try
        {
            int readSize = internalConvertToZipFile(filename, bis);
            
            closeInputStream(bis);

            uploadZipped(contextName, docID, filename, readSize);
        }
        finally
        {
            deleteTemporaryFile();
        }
    }

    private int internalConvertToZipFile(String filename, BufferedInputStream bis)
            throws RPMException
    {
        int readSize = 0;
        deleteTemporaryFile();
        File tempUnzippedFile = null;

        
        FileOutputStream fos = null;
        FileOutputStream unzippedfos = null;
        BufferedOutputStream unzippedbos = null;
        ZipOutputStream zipOutputStream = null;
        FileInputStream unzippedfis = null;
        BufferedInputStream unzippedbis = null;
        
        try
        {
            // First, save the file in uncompressed state
            tempUnzippedFile = File.createTempFile("rpmupload", "ziptmp");
            unzippedfos = new FileOutputStream(tempUnzippedFile);
            unzippedbos = new BufferedOutputStream(unzippedfos);
            int b = 0;
            while (-1 != (b = bis.read()))
            {
                unzippedbos.write(b);
            }
            unzippedbos.flush();
            
            closeOutputStream(unzippedfos);
            unzippedfos = null;
            closeOutputStream(unzippedbos);
            unzippedbos = null;
            
            // read the total file size
            readSize = (int)tempUnzippedFile.length();
            
            // transform the uncompressed file into a compressed one
            unzippedfis = new FileInputStream(tempUnzippedFile);
            unzippedbis = new BufferedInputStream(unzippedfis);
            
            tempZipFile = File.createTempFile("rpmupload", "ziptmp");
            fos = new FileOutputStream(tempZipFile);
            zipOutputStream = new ZipOutputStream(fos);
            ZipEntry entry = new ZipEntry(filename);
            entry.setMethod(ZipEntry.DEFLATED);
            // entry.setSize(contentLength); // Don't think of it, it doesn't
            // work!
            // Look at the different forums on the internet. The problem is that
            // it
            // works but the zip spec is unclear about that
            // and the different implementations are not always compatible. This
            // is
            // not really a problem usually because
            // "uncompressed size" and "comment" are strictly informative.
            // entry.setComment("Unploaded as an unzipped file from a batch
            // process
            // using the RPM WS API."); // This doesn't work.
            zipOutputStream.putNextEntry(entry);

            while (-1 != (b = unzippedbis.read()))
            {
                zipOutputStream.write(b);
            }
            zipOutputStream.flush();
            zipOutputStream.closeEntry();
            zipOutputStream.finish();
        }
        catch (Exception e)
        {
            logException(e);
            throw new RPMException("Failed to create internal zip file:" + e);
        }
        finally
        {
            closeOutputStream(unzippedfos);
            closeOutputStream(unzippedbos);
            closeOutputStream(fos);
            closeOutputStream(zipOutputStream);
            
            closeInputStream(unzippedbis);
            closeInputStream(unzippedfis);
            
            deleteTempFile(tempUnzippedFile);
        }
        return readSize;
    }

    private void saveTempZipFile(BufferedInputStream bis)
            throws RPMException
    {
        FileOutputStream fos = null;
        try
        {
            deleteTemporaryFile();
            tempZipFile = File.createTempFile("rpmupload", "ziptmp");
            fos = new FileOutputStream(tempZipFile);
            int b;
            while (-1 != (b = bis.read()))
            {
                fos.write(b);
            }
            fos.flush();
            fos.close();
        }
        catch (Exception e)
        {
            logException(e);
            throw new RPMException("Failed to save internal zip file:" + e);
        }
        finally
        {
            if ( fos != null )
            {
                closeOutputStream(fos);
            }
        }
    }
    
    private void deleteTemporaryFile()
    {
        File fileToDelete = tempZipFile;
        
        deleteTempFile(fileToDelete);
        
        tempZipFile = null;
    }

    /**
     * @param fileToDelete
     */
    public static void deleteTempFile(File fileToDelete)
    {
        if (fileToDelete != null)
        {
            if (!fileToDelete.delete())
            {
                fileToDelete.deleteOnExit();

                String filePath = fileToDelete.getAbsolutePath();
                
                try
                {
                    filePath = fileToDelete.getCanonicalPath();
                }
                catch (IOException e)
                {

                }
                
                logger.warn("Failed to delete temporary file: "
                        + filePath);
            }
        }
    }

    /**
     * @param fos
     */
    public static void closeOutputStream(OutputStream fos)
    {
        try
        {
            if ( fos != null )
            {
                fos.close();
            }
        }
        catch (Exception e)
        {
            logException(e);
        }
    }

    /**
     * @param procedureName
     * @param args
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    private final String executeProcedure(String procedureName, Object[] args)
            throws SQLException, RPMException
    {
        releaseResultSetAndStatement();
        try
        {
            // logging
            StringBuffer callStr = new StringBuffer(procedureName + "(");
            for (int i = 0; i < args.length; i++)
            {
                callStr.append(args[i]);
                callStr.append(", ");
            }
            callStr.append(")");
            logger.info(callStr);

            rs = Manager
                    .executeStoredProcedure(connection, procedureName, args, true, messageContext);
            rs.next();
            int error = 0;
            try
            {
                error = Manager.checkForError(rs);
            }
            catch (SQLException sqle)
            {
                RPMException e = new RPMException(
                        procedureName
                                + " didn't return a result set or the result set didn't include an error column.");
                //error will be logged in saveInternal method.
                throw e;
            }
            if (error != 0)
            {
                String message = ErrorCodeProperties.getInstance().getString(
                        String.valueOf(error));
                message = "code: " + error + " " + message;
                RPMException e = new RPMException(message);
                //error will be logged in saveInternal method.
                throw e;
            }
            return rs.getString(2);

        }
        finally
        {
            releaseResultSetAndStatement();
        }
    }

    private void SP_CHECK_IN_DOC(DocumentElement docElement)
            throws SQLException, RPMException
    {
        String name = "SP_CHECK_IN_DOC";
        Object args[] = new Object[4];
        args[0] = new Integer(getTypeIdForContext(docElement.getContextName()));
        args[1] = docElement.getID();
        args[2] = new Integer(0); // bit_flag
        args[3] = getUser().getID();
        executeProcedure(name, args);
    }

    private void SP_CHECK_OUT_DOC(DocumentElement docElement)
            throws SQLException, RPMException
    {
        String name = "SP_CHECK_OUT_DOC";
        Object args[] = new Object[4];

        args[0] = new Integer(getTypeIdForContext(docElement.getContextName()));
        args[1] = docElement.getID();
        args[2] = new Integer(0); // bit_flag
        args[3] = getUser().getID();
        executeProcedure(name, args);
    }

    private String SP_U_DOC(DocumentElement docElement) throws SQLException,
            RPMException
    {

        String elementId = null;

        String name = "SP_U_DOC";
        Object args[] = new Object[9];

        // table type
        args[0] = new Integer(getTypeIdForContext(docElement.getContextName()));

        // doc_id
        args[1] = docElement.getID();

        // attach Name of the file compressed in the zip
        args[2] = docElement.getName();

        // ref-number ???
        args[3] = null;

        // author_checkout
        args[4] = "Y"; // In the context of DocumentTransfer Servet, the
        // document will always be checked out before this call.
        // published
        args[5] = null;

        // rup_link
        args[6] = "0"; // Seems not used, but must be not-null

        // tng_link
        args[7] = "0"; // Seems not used, but must be not-null

        args[8] = getUser().getID();

        elementId = executeProcedure(name, args);

        return elementId;
    }

    private String SP_U_DOC_REVISION(DocumentElement docElement)
            throws SQLException, RPMException
    {

        String name = "SP_U_DOC_REVISION";

        Object args[] = new Object[8];

        args[0] = new Integer(getTypeIdForContext(docElement.getContextName()));
        args[1] = docElement.getID();
        args[2] = new Integer(0); // constant from DocumentManager
        args[3] = new Integer(137); // constant from DocumentManager
        args[4] = new Integer(4768); // constant from DocumentManager

        args[5] = null; // StatusUpdate

        args[6] = new Integer(4953); // constant from DocumentManager

        args[7] = getUser().getID();

        String elementId = executeProcedure(name, args);

        return elementId;
    }

    private void SP_U_DOC_LOCAL_FOLDER(DocumentElement docElement,
            int uncompressedFileSize) throws SQLException, RPMException
    {
        String name = "SP_U_DOC_LOCAL_FOLDER";
        Object args[] = new Object[6];

        args[0] = new Integer(getTypeIdForContext(docElement.getContextName()));
        args[1] = docElement.getID();
        // Change machine name and local folder; RPM Client will have to reload
        // from the blob table the document
        // and this is the expected behaviour
        args[2] = null; // Machine name
        args[3] = null; // Local folder
        args[4] = new Integer(uncompressedFileSize); // File size uncompressed
        args[5] = getUser().getID();
        executeProcedure(name, args);
    }

    /**
     * @param contextName
     * @param docID
     * @param isZipped
     * @throws SQLException
     * @throws RPMException
     */
    public boolean prepareDownload(String contextName, String docID,
            boolean isZipped) throws RPMException
    {
        try
        {
            getBLOB(contextName, docID);
    
            if ( rs == null )
            {
                return false;
            }
            
            // read the attachement name
            if (isZipped)
            {
                attachmentName = DEFAULT_ATTACHMENT_NAME;
            }
            else
            {
                attachmentName = rs.getString("ATTACHEMENT");
            }
            
            // fill the input stream and file size
            if( isZipped )
            {
                Blob blob = rs.getBlob("BLOB_DATA");
                if ( blob != null )
                {
                    fileSize = blob.length();
                    downloadInputStream = blob.getBinaryStream();
                }else{
                    throw new RPMException("There is no document attached");
                }
                
            }
            else
            {
                cacheDownloadUnzipped();
                
                tempFileInputStream = new FileInputStream(tempZipFile);
                downloadInputStream = new BufferedInputStream(
                    tempFileInputStream);
            }
        }
        catch (SQLException e2)
        {
            logException(e2);
            throw new RPMException(e2);
        }
        catch (FileNotFoundException e)
        {
            logException(e);
            throw new RPMException(e);
        }
        
        return true;
    }

    /**
     * @param dos
     * @throws RPMException
     */
    public void executeDownload(OutputStream outputStream) throws RPMException
    {
        try
        {
            // Get the optimum buffer size and use this to create the read/write
            // buffer
            byte[] buffer = new byte[1024];
            int length = -1;
            // Transfer the data
            while ((length = downloadInputStream.read(buffer)) != -1)
            {
                outputStream.write(buffer, 0, length);
            }
            outputStream.flush();
        }
        catch (IOException e)
        {
            throw (new RPMException(e)).log();
        }
        finally
        {
            // Close everything down
            closeOutputStream(outputStream);
        }
        return;
    }
}
