/**
 * Created on Mar 13, 2006
 * @author shahla
 */
package com.ibm.rpm.financial.checkpoints;

import java.util.ArrayList;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;


public class GenericChargeCodeCheckpoint extends AbstractCheckpoint
{
    private final static ArrayList DEFAULT_CHARGE_CODE_ID = new ArrayList();
    static
    {
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_CHARGE_CODES_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_EXPENSE_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_AVERAGE_COST");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_CAPITAL_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_AVERAGE_COST");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_BENEFIT_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_AVERAGE_RATE");
        DEFAULT_CHARGE_CODE_ID.add("BILLABLE_BENEFIT_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("PMO_PROJECT_TIMEPHASE_CALENDAR__");
        DEFAULT_CHARGE_CODE_ID.add("CUSTOM_CHARGE_CODES_FOLDER______");
        
    }
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (rpmObject instanceof GenericChargeCode)
        {
            // Validate nesting level
            GenericValidator.validateNestingLevel(rpmObject, "TMT_CHARGECODES",
                ValidationConstants.CHARGE_CODE_NESTINGLEVEL_MAX, msgCtx);
            
            // validate the common fields
            GenericChargeCode chargeCode = (GenericChargeCode) rpmObject;
            validateName(chargeCode, msgCtx);
            validateDescription(chargeCode, msgCtx);
            validateExternalID(chargeCode, msgCtx);
        }
    }
    
    private void validateName(GenericChargeCode chargeCode,
            MessageContext msgCtx)
    {
        boolean valid = GenericValidator.validateMandatory(chargeCode,
            ValidationConstants.NAME_FIELD, chargeCode.getName(), msgCtx);
        if (valid && chargeCode.testNameModified())
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode, "name", msgCtx);
            }
            else
            {
                GenericValidator.validateMaxLength(chargeCode,
                    ValidationConstants.NAME_FIELD, chargeCode.getName(),
                    ValidationConstants.CHARGE_CODE_NAME_MAX, msgCtx);
                GenericValidator.validateMinLength(chargeCode,
                    ValidationConstants.NAME_FIELD, chargeCode.getName(),
                    ValidationConstants.CHARGE_CODE_NAME_MIN, msgCtx);
            }
        }
    }
    
    private void validateDescription(GenericChargeCode chargeCode,
            MessageContext msgCtx)
    {
        if (chargeCode.testDescriptionModified())
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode, "description",
                    msgCtx);
            }
            else
            {
                GenericValidator.validateMaxLength(chargeCode, "description",
                    chargeCode.getDescription(),
                    ValidationConstants.CHARGE_CODE_DESCRIPTION_MAX, msgCtx);
            }
        }
    }
    
    private void validateExternalID(GenericChargeCode chargeCode,
            MessageContext msgCtx)
    {
        if (chargeCode.testExternalIDModified())
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode, "externalID", msgCtx);
            }
            else
            {
                GenericValidator.validateMaxLength(chargeCode, "externalID",
                    chargeCode.getExternalID(),
                    ValidationConstants.CHARGE_CODE_EXTERNAL_ID_MAX, msgCtx);
            }
        }
    }

    public static boolean isChargeCodePartOfDefaultChargeCodeList(
            String chargeCode)
    {
        boolean isDefaultChargeCode = false;
        String chargeCodeID = null;

        if (chargeCode != null)
        {
            chargeCodeID = chargeCode.trim();

            for (int i = 0; i < DEFAULT_CHARGE_CODE_ID.size(); i++)
            {
                if (((String) DEFAULT_CHARGE_CODE_ID.get(i))
                    .equals(chargeCodeID))
                {
                    isDefaultChargeCode = true;
                    break;
                }
            }
        }

        return isDefaultChargeCode;
    }
}
