/*
 * Created on Nov 18, 2005
 */
package com.ibm.rpm.financial.checkpoints;

import com.ibm.rpm.financial.constants.ErrorCodes;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.framework.util.ValidateSerializer;

/**
 * @author ikusakov
 */
public class TimePhasedFinancialCheckpoint extends AbstractCheckpoint
{
    private static final Class FINANCIAL_PARENTS = AbstractFinancial.class;

    protected static TimePhasedFinancialCheckpoint instance = new TimePhasedFinancialCheckpoint();

    public static TimePhasedFinancialCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        boolean validParent = super.isValidParent(rpmObject, msgCtx, false,
            true, FINANCIAL_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            TimePhasedFinancial.class, msgCtx))
            return;

        TimePhasedFinancial timephased = (TimePhasedFinancial) rpmObject;
        
        if (timephased.getID() == null && validParent)
        {
            // for inserts, the parent's parent ID is mandatory
            // for use in the call to SP_U_PBB
            GenericValidator.validateMandatoryID(timephased,
                ValidationConstants.PARENT_PARENT_FIELD, timephased.getParent()
                    .getParent(), msgCtx);
        }

        if (validParent)
        {
            // the parent's charge code ID is mandatory for inserts and updates
            // for use in the call to SP_U_PBB
            GenericValidator.validateMandatoryID(timephased,
                ValidationConstants.PARENT_CHARGE_CODE_FIELD,
                ((AbstractFinancial) timephased.getParent()).getChargeCode(),
                msgCtx);
        }   
        

        if (timephased.testYearModified())
        {
            // TODO implement year validation from business logic perspective
        }

        if (timephased.getID() == null
                && (timephased.getParent() == null
                        || timephased.getParent().getParent() == null || timephased
                    .getParent().getParent().getID() == null))
        {
            ValidateSerializer vs = msgCtx.getObjectsToValidate();
            if (vs.before(timephased.getParent().getParent(), rpmObject))
            {
            
	            // element ID is a part of primary key,
	            // it is stored in either timephased ID or it
	            // has to be retreived from parent of financial
	            String[] errorParams =
	            {
	                StringUtil.getShortClassName(timephased.getClass()) + " ID "
	            };
	
	            RPMException e = new RPMException(
	                ErrorCodes.TIMEPHASED_ID_REQUIRED, errorParams, timephased
	                    .getClass().getName(), "ID", timephased.getID());
	            addException(e, msgCtx);
            }
        }

        GenericValidator.validateMandatory(timephased, "budgetType", timephased
            .getBudgetType(), msgCtx);

        if (timephased.testMonth01ValueModified())
        {
            GenericValidator.validateRange(timephased, "month01", timephased
                .getMonth01Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth02ValueModified())
        {
            GenericValidator.validateRange(timephased, "month02", timephased
                .getMonth02Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth03ValueModified())
        {
            GenericValidator.validateRange(timephased, "month03", timephased
                .getMonth03Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth04ValueModified())
        {
            GenericValidator.validateRange(timephased, "month04", timephased
                .getMonth04Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth05ValueModified())
        {
            GenericValidator.validateRange(timephased, "month05", timephased
                .getMonth05Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth06ValueModified())
        {
            GenericValidator.validateRange(timephased, "month06", timephased
                .getMonth06Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth07ValueModified())
        {
            GenericValidator.validateRange(timephased, "month07", timephased
                .getMonth07Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth08ValueModified())
        {
            GenericValidator.validateRange(timephased, "month08", timephased
                .getMonth08Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth09ValueModified())
        {
            GenericValidator.validateRange(timephased, "month09", timephased
                .getMonth09Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth10ValueModified())
        {
            GenericValidator.validateRange(timephased, "month10", timephased
                .getMonth10Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth11ValueModified())
        {
            GenericValidator.validateRange(timephased, "month11", timephased
                .getMonth11Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }

        if (timephased.testMonth12ValueModified())
        {
            GenericValidator.validateRange(timephased, "month12", timephased
                .getMonth12Value(), ValidationConstants.FINANCIAL_MIN,
                ValidationConstants.FINANCIAL_MAX, msgCtx);
        }
        
        if (timephased.testTotalForYearModified())
        {
            GenericValidator.validateReadOnly(timephased, "totalForYear",
                msgCtx);
        }
        
        if (validParent)
        {
            AbstractFinancial financialParent = (AbstractFinancial) timephased.getParent();
            
            if ((financialParent.testActualModified()
                    || financialParent.testEstimateModified()
                    || financialParent.testETCModified()
                    || financialParent.testEACModified() || financialParent
                .testRemainingModified())
                    && FinancialUtil.isTimePhasedModified(timephased))
            {
                // cannot update the timephased amounts and the worksheet financial
                // values at the same time
                RPMException e = new RPMException(
                    ErrorCodes.TIMEPHASED_INDIVIDUAL_MODIFICATION_REQUIRED,
                    financialParent.getClass().getName());
                e.setSeverity(SeverityLevel.Error);
                addException(e, msgCtx);
            }
        }
    }
}
