package com.ibm.rpm.financial.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.financial.types.RevenueRecognitionMethodType;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.financial.util.BudgetTypeToDBMap;
import com.ibm.rpm.financial.util.ChargeCodeUtil;
import com.ibm.rpm.financial.util.FinancialBusinessLogicUtil;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * Holds commonalities between WorksheetFinancialManager and WBSFinancialManager
 * (see them for more comments)
 * 
 * @see WorksheetFinancialManager
 * @see WbsFinancialManager
 * @see TimePhasedFinancialManager
 * 
 * @author legauts
 * @author ikusakov
 */
public abstract class FinancialManager extends AbstractRPMObjectManager
{
    protected static final int UPDATE_WORK_ELEMENT = 1;

    protected static final int UPDATE_SCOPE_ELEMENT = 3;

    protected static final int PROB_PERCENT_DIV = 100;

    protected static Log logger = LogFactory.getLog(FinancialManager.class);

    protected static final HashMap FIELDPROPERTYMAP = new HashMap();

    // CONTAINTERS are implemented in subclasses

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "BUDGETS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 2;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "BUDGETS.ELEMENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column CHARGECODE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CHARGECODE_ID = 3;

    public static final int TYPE_CHARGECODE_ID = Types.CHAR;

    public static final String NAME_CHARGECODE_ID = "BUDGETS.CHARGECODE_ID";

    public static final String PROPERTY_CHARGECODE_ID = "CHARGECODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CHARGECODE_ID, NAME_CHARGECODE_ID);
    }

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGECODE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGECODE_ID,NAME_CHARGECODE_ID);}

    /**
     * Column BUDGET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_BUDGET_ID = 4;

    public static final int TYPE_BUDGET_ID = Types.CHAR;

    public static final String NAME_BUDGET_ID = "BUDGETS.BUDGET_ID";

    public static final String PROPERTY_BUDGET_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_ID, NAME_BUDGET_ID);
    }

    /**
     * Column ASSOCIATE_BUDGET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSOCIATE_BUDGET_ID = 5;

    public static final int TYPE_ASSOCIATE_BUDGET_ID = Types.CHAR;

    public static final String NAME_ASSOCIATE_BUDGET_ID = "BUDGETS.ASSOCIATE_BUDGET_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSOCIATE_BUDGET_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ASSOCIATE_BUDGET_ID,NAME_ASSOCIATE_BUDGET_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 6;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "BUDGETS.TYPE_ID";

    //public static final String PROPERTY_TYPE_ID = "TYPE";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 7;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "BUDGETS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 8;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "BUDGETS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 9;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "BUDGETS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 10;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "BUDGETS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column TOTAL_ESTIMATION_PROB_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_TOTAL_ESTIMATION_PROB_AMOUNT = 11;

    public static final int TYPE_TOTAL_ESTIMATION_PROB_AMOUNT = Types.DECIMAL;

    public static final String NAME_TOTAL_ESTIMATION_PROB_AMOUNT = "BUDGETS.TOTAL_ESTIMATION_PROB_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOTAL_ESTIMATION_PROB_AMOUNT =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TOTAL_ESTIMATION_PROB_AMOUNT,NAME_TOTAL_ESTIMATION_PROB_AMOUNT);}

    /**
     * Column TOTAL_BUDGET_PROB_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_TOTAL_BUDGET_PROB_AMOUNT = 12;

    public static final int TYPE_TOTAL_BUDGET_PROB_AMOUNT = Types.DECIMAL;

    public static final String NAME_TOTAL_BUDGET_PROB_AMOUNT = "BUDGETS.TOTAL_BUDGET_PROB_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOTAL_BUDGET_PROB_AMOUNT =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TOTAL_BUDGET_PROB_AMOUNT,NAME_TOTAL_BUDGET_PROB_AMOUNT);}

    /**
     * Column PROB_PERCENT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROB_PERCENT = 13;

    public static final int TYPE_PROB_PERCENT = Types.INTEGER;

    public static final String NAME_PROB_PERCENT = "BUDGETS.PROB_PERCENT";

    public static final String PROPERTY_PROB_PERCENT = "PROBABILITYPERCENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROB_PERCENT, NAME_PROB_PERCENT + " / 100");
    }

    /**
     * Column PROB_AMOUNT of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_PROB_AMOUNT = 14;

    public static final int TYPE_PROB_AMOUNT = Types.DECIMAL;

    public static final String NAME_PROB_AMOUNT = "BUDGETS.PROB_AMOUNT";

    // gone to WBSFinancial

    /**
     * Column ESTIMATED_BASELINE1 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE1 = 15;

    public static final int TYPE_ESTIMATED_BASELINE1 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE1 = "BUDGETS.ESTIMATED_BASELINE1";

    // gone to WorksheetFinancial

    /**
     * Column ESTIMATED_BASELINE2 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE2 = 16;

    public static final int TYPE_ESTIMATED_BASELINE2 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE2 = "BUDGETS.ESTIMATED_BASELINE2";

    // gone to WorksheetFinancial

    /**
     * Column ESTIMATED_BASELINE3 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE3 = 17;

    public static final int TYPE_ESTIMATED_BASELINE3 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE3 = "BUDGETS.ESTIMATED_BASELINE3";

    // gone to WorksheetFinancial

    /**
     * Column ESTIMATED_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_TOTAL = 18;

    public static final int TYPE_ESTIMATED_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_TOTAL = "BUDGETS.ESTIMATED_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_TOTAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_TOTAL,NAME_ESTIMATED_TOTAL);}

    /**
     * Column ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BUDGET = 19;

    public static final int TYPE_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BUDGET = "BUDGETS.ESTIMATED_BUDGET";

    // gone to WorksheetFinancial

    /**
     * Column ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL = 20;

    public static final int TYPE_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL = "BUDGETS.ESTIMATED_ACTUAL";

    // gone to WorksheetFinancial

    /**
     * Column ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ETC = 21;

    public static final int TYPE_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ETC = "BUDGETS.ESTIMATED_ETC";

    // gone to WorksheetFinancial

    /**
     * Column BUDGET_BASELINE1 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BUDGET_BASELINE1 = 22;

    public static final int TYPE_BUDGET_BASELINE1 = Types.DECIMAL;

    public static final String NAME_BUDGET_BASELINE1 = "BUDGETS.BUDGET_BASELINE1";

    //  gone to WBSFinancial

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.

    /**
     * Column BUDGET_BASELINE2 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BUDGET_BASELINE2 = 23;

    public static final int TYPE_BUDGET_BASELINE2 = Types.DECIMAL;

    public static final String NAME_BUDGET_BASELINE2 = "BUDGETS.BUDGET_BASELINE2";

    //  gone to WBSFinancial

    /**
     * Column BUDGET_BASELINE3 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BUDGET_BASELINE3 = 24;

    public static final int TYPE_BUDGET_BASELINE3 = Types.DECIMAL;

    public static final String NAME_BUDGET_BASELINE3 = "BUDGETS.BUDGET_BASELINE3";

    //  gone to WBSFinancial

    /**
     * Column BUDGET of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BUDGET = 25;

    public static final int TYPE_BUDGET = Types.DECIMAL;

    public static final String NAME_BUDGET = "BUDGETS.BUDGET";

    //  gone to WBSFinancial

    /**
     * Column ACTUAL of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ACTUAL = 26;

    public static final int TYPE_ACTUAL = Types.DECIMAL;

    public static final String NAME_ACTUAL = "BUDGETS.ACTUAL";

    //  gone to WBSFinancial

    /**
     * Column ETC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ETC = 27;

    public static final int TYPE_ETC = Types.DECIMAL;

    public static final String NAME_ETC = "BUDGETS.ETC";

    //  gone to WBSFinancial

    /**
     * Column PAID_TODATE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_PAID_TODATE = 28;

    public static final int TYPE_PAID_TODATE = Types.DECIMAL;

    public static final String NAME_PAID_TODATE = "BUDGETS.PAID_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PAID_TODATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PAID_TODATE,NAME_PAID_TODATE);}

    /**
     * Column AUTO_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_AUTO_FLAG = 29;

    public static final int TYPE_AUTO_FLAG = Types.SMALLINT;

    public static final String NAME_AUTO_FLAG = "BUDGETS.AUTO_FLAG";

    public static final String PROPERTY_AUTO_FLAG = "REVRECTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_AUTO_FLAG, SqlUtil.castAsChar(NAME_AUTO_FLAG, 1));
    }

    /**
     * Column ACTUAL_EDITED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ACTUAL_EDITED = 30;

    public static final int TYPE_ACTUAL_EDITED = Types.SMALLINT;

    public static final String NAME_ACTUAL_EDITED = "BUDGETS.ACTUAL_EDITED";

    //public static final String PROPERTY_ACTUAL_EDITED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_EDITED,NAME_ACTUAL_EDITED);}

    /**
     * Column ROLLUP_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ROLLUP_BITS = 31;

    public static final int TYPE_ROLLUP_BITS = Types.INTEGER;

    public static final String NAME_ROLLUP_BITS = "BUDGETS.ROLLUP_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLLUP_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_BITS,NAME_ROLLUP_BITS);}

    /**
     * Column TIMEPHASED_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TIMEPHASED_BITS = 32;

    public static final int TYPE_TIMEPHASED_BITS = Types.INTEGER;

    public static final String NAME_TIMEPHASED_BITS = "BUDGETS.TIMEPHASED_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMEPHASED_BITS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TIMEPHASED_BITS,NAME_TIMEPHASED_BITS);}

    /**
     * Column TIMEPHASED_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TIMEPHASED_FLAG = 33;

    public static final int TYPE_TIMEPHASED_FLAG = Types.SMALLINT;

    public static final String NAME_TIMEPHASED_FLAG = "BUDGETS.TIMEPHASED_FLAG";

    // moved to WorksheetFinancialManager

    /**
     * Column DATAFIELD_1 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DATAFIELD_1 = 34;

    public static final int TYPE_DATAFIELD_1 = Types.INTEGER;

    public static final String NAME_DATAFIELD_1 = "BUDGETS.DATAFIELD_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DATAFIELD_1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DATAFIELD_1,NAME_DATAFIELD_1);}

    /**
     * Column DATAFIELD_2 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DATAFIELD_2 = 35;

    public static final int TYPE_DATAFIELD_2 = Types.INTEGER;

    public static final String NAME_DATAFIELD_2 = "BUDGETS.DATAFIELD_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DATAFIELD_2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DATAFIELD_2,NAME_DATAFIELD_2);}

    /**
     * Column TXT_FIELD_01 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_01 = 36;

    public static final int TYPE_TXT_FIELD_01 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_01 = "BUDGETS.TXT_FIELD_01";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_01 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_01,NAME_TXT_FIELD_01);}

    /**
     * Column TXT_FIELD_02 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_02 = 37;

    public static final int TYPE_TXT_FIELD_02 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_02 = "BUDGETS.TXT_FIELD_02";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_02 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_02,NAME_TXT_FIELD_02);}

    /**
     * Column TXT_FIELD_03 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_03 = 38;

    public static final int TYPE_TXT_FIELD_03 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_03 = "BUDGETS.TXT_FIELD_03";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_03 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_03,NAME_TXT_FIELD_03);}

    /**
     * Column TXT_FIELD_04 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_04 = 39;

    public static final int TYPE_TXT_FIELD_04 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_04 = "BUDGETS.TXT_FIELD_04";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_04 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_04,NAME_TXT_FIELD_04);}

    /**
     * Column TXT_FIELD_05 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_05 = 40;

    public static final int TYPE_TXT_FIELD_05 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_05 = "BUDGETS.TXT_FIELD_05";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_05 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_05,NAME_TXT_FIELD_05);}

    /**
     * Column TXT_FIELD_06 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_06 = 41;

    public static final int TYPE_TXT_FIELD_06 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_06 = "BUDGETS.TXT_FIELD_06";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_06 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_06,NAME_TXT_FIELD_06);}

    /**
     * Column TXT_FIELD_07 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_07 = 42;

    public static final int TYPE_TXT_FIELD_07 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_07 = "BUDGETS.TXT_FIELD_07";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_07 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_07,NAME_TXT_FIELD_07);}

    /**
     * Column TXT_FIELD_08 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_08 = 43;

    public static final int TYPE_TXT_FIELD_08 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_08 = "BUDGETS.TXT_FIELD_08";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_08 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_08,NAME_TXT_FIELD_08);}

    /**
     * Column TXT_FIELD_09 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_09 = 44;

    public static final int TYPE_TXT_FIELD_09 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_09 = "BUDGETS.TXT_FIELD_09";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_09 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_09,NAME_TXT_FIELD_09);}

    /**
     * Column TXT_FIELD_10 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TXT_FIELD_10 = 45;

    public static final int TYPE_TXT_FIELD_10 = Types.VARCHAR;

    public static final String NAME_TXT_FIELD_10 = "BUDGETS.TXT_FIELD_10";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TXT_FIELD_10 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TXT_FIELD_10,NAME_TXT_FIELD_10);}

    /**
     * Column TOLERANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TOLERANCE = 46;

    public static final int TYPE_TOLERANCE = Types.INTEGER;

    public static final String NAME_TOLERANCE = "BUDGETS.TOLERANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOLERANCE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TOLERANCE,NAME_TOLERANCE);}

    /**
     * Column ESTIMATED_ACTUAL_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL_TOTAL = 47;

    public static final int TYPE_ESTIMATED_ACTUAL_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL_TOTAL = "BUDGETS.ESTIMATED_ACTUAL_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ACTUAL_TOTAL =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL_TOTAL,NAME_ESTIMATED_ACTUAL_TOTAL);}

    /**
     * Column ESTIMATED_ETC_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ETC_TOTAL = 48;

    public static final int TYPE_ESTIMATED_ETC_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ETC_TOTAL = "BUDGETS.ESTIMATED_ETC_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ETC_TOTAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ETC_TOTAL,NAME_ESTIMATED_ETC_TOTAL);}

    /**
     * Column ESTIMATED_PAID_TODATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_PAID_TODATE = 49;

    public static final int TYPE_ESTIMATED_PAID_TODATE = Types.DECIMAL;

    public static final String NAME_ESTIMATED_PAID_TODATE = "BUDGETS.ESTIMATED_PAID_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_PAID_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_PAID_TODATE,NAME_ESTIMATED_PAID_TODATE);}

    /**
     * Column ESTIMATED_PAID_TODATE_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_PAID_TODATE_TOTAL = 50;

    public static final int TYPE_ESTIMATED_PAID_TODATE_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_PAID_TODATE_TOTAL = "BUDGETS.ESTIMATED_PAID_TODATE_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_PAID_TODATE_TOTAL =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_PAID_TODATE_TOTAL,NAME_ESTIMATED_PAID_TODATE_TOTAL);}

    /**
     * Column ESTIMATED_PROB_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_PROB_AMOUNT = 51;

    public static final int TYPE_ESTIMATED_PROB_AMOUNT = Types.DECIMAL;

    public static final String NAME_ESTIMATED_PROB_AMOUNT = "BUDGETS.ESTIMATED_PROB_AMOUNT";

    // gone to WorksheetFinancialManager

    /**
     * Column ESTIMATED_PROB_AMOUNT_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_PROB_AMOUNT_TOTAL = 52;

    public static final int TYPE_ESTIMATED_PROB_AMOUNT_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_PROB_AMOUNT_TOTAL = "BUDGETS.ESTIMATED_PROB_AMOUNT_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_PROB_AMOUNT_TOTAL =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_PROB_AMOUNT_TOTAL,NAME_ESTIMATED_PROB_AMOUNT_TOTAL);}

    /**
     * Column BASELINE3_SCH_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BASELINE3_SCH_FLAG = 53;

    public static final int TYPE_BASELINE3_SCH_FLAG = Types.SMALLINT;

    public static final String NAME_BASELINE3_SCH_FLAG = "BUDGETS.BASELINE3_SCH_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_SCH_FLAG = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_SCH_FLAG,NAME_BASELINE3_SCH_FLAG);}

    /**
     * Column BASELINE3_EST_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BASELINE3_EST_FLAG = 54;

    public static final int TYPE_BASELINE3_EST_FLAG = Types.SMALLINT;

    public static final String NAME_BASELINE3_EST_FLAG = "BUDGETS.BASELINE3_EST_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_EST_FLAG = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_EST_FLAG,NAME_BASELINE3_EST_FLAG);}

    /**
     * Column DATAFIELD_3 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DATAFIELD_3 = 55;

    public static final int TYPE_DATAFIELD_3 = Types.INTEGER;

    public static final String NAME_DATAFIELD_3 = "BUDGETS.DATAFIELD_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DATAFIELD_3 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DATAFIELD_3,NAME_DATAFIELD_3);}

    /**
     * Column DATAFIELD_4 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DATAFIELD_4 = 56;

    public static final int TYPE_DATAFIELD_4 = Types.INTEGER;

    public static final String NAME_DATAFIELD_4 = "BUDGETS.DATAFIELD_4";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DATAFIELD_4 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DATAFIELD_4,NAME_DATAFIELD_4);}

    /**
     * Column ACTUAL_TIMEPHASED_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ACTUAL_TIMEPHASED_FLAG = 57;

    public static final int TYPE_ACTUAL_TIMEPHASED_FLAG = Types.SMALLINT;

    public static final String NAME_ACTUAL_TIMEPHASED_FLAG = "BUDGETS.ACTUAL_TIMEPHASED_FLAG";


    /**
     * Column CAPITAL_EXPENSE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CAPITAL_EXPENSE = 58;

    public static final int TYPE_CAPITAL_EXPENSE = Types.SMALLINT;

    public static final String NAME_CAPITAL_EXPENSE = "BUDGETS.CAPITAL_EXPENSE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_EXPENSE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_EXPENSE,NAME_CAPITAL_EXPENSE);}

    /**
     * Column BILLABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BILLABLE = 59;

    public static final int TYPE_BILLABLE = Types.SMALLINT;

    public static final String NAME_BILLABLE = "BUDGETS.BILLABLE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLABLE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BILLABLE,NAME_BILLABLE);}

    /**
     * Column COST_CENTER_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_COST_CENTER_BITS = 60;

    public static final int TYPE_COST_CENTER_BITS = Types.INTEGER;

    public static final String NAME_COST_CENTER_BITS = "BUDGETS.COST_CENTER_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_CENTER_BITS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_BITS,NAME_COST_CENTER_BITS);}

    /**
     * Column CHARGED of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_CHARGED = 61;

    public static final int TYPE_CHARGED = Types.DECIMAL;

    public static final String NAME_CHARGED = "BUDGETS.CHARGED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGED,NAME_CHARGED);}

    /**
     * Column LEDGER of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_LEDGER = 62;

    public static final int TYPE_LEDGER = Types.DECIMAL;

    public static final String NAME_LEDGER = "BUDGETS.LEDGER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEDGER,NAME_LEDGER);}

    /**
     * Column ESTIMATED_CHARGED of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_CHARGED = 63;

    public static final int TYPE_ESTIMATED_CHARGED = Types.DECIMAL;

    public static final String NAME_ESTIMATED_CHARGED = "BUDGETS.ESTIMATED_CHARGED";

    public static final String PROPERTY_ESTIMATED_CHARGED = "BILLED";
    static
    {
        FIELDPROPERTYMAP
                .put(PROPERTY_ESTIMATED_CHARGED, NAME_ESTIMATED_CHARGED);
    }

    /**
     * Column ESTIMATED_LEDGER of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_LEDGER = 64;

    public static final int TYPE_ESTIMATED_LEDGER = Types.DECIMAL;

    public static final String NAME_ESTIMATED_LEDGER = "BUDGETS.ESTIMATED_LEDGER";

    public static final String PROPERTY_ESTIMATED_LEDGER = "GL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_LEDGER, NAME_ESTIMATED_LEDGER);
    }

    /**
     * Column ESTIMATED_CHARGED_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_CHARGED_TOTAL = 65;

    public static final int TYPE_ESTIMATED_CHARGED_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_CHARGED_TOTAL = "BUDGETS.ESTIMATED_CHARGED_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_CHARGED_TOTAL =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_CHARGED_TOTAL,NAME_ESTIMATED_CHARGED_TOTAL);}

    /**
     * Column ESTIMATED_LEDGER_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_LEDGER_TOTAL = 66;

    public static final int TYPE_ESTIMATED_LEDGER_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_LEDGER_TOTAL = "BUDGETS.ESTIMATED_LEDGER_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_LEDGER_TOTAL =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_LEDGER_TOTAL,NAME_ESTIMATED_LEDGER_TOTAL);}

    public static final String TABLE_NAME = "BUDGETS";

    /**
     * Create an array of type string containing all the fields of the BUDGETS
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_ELEMENT_ID, NAME_CHARGECODE_ID,
            NAME_BUDGET_ID, NAME_ASSOCIATE_BUDGET_ID, NAME_TYPE_ID,
            NAME_CREATED_BY, NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_TOTAL_ESTIMATION_PROB_AMOUNT, NAME_TOTAL_BUDGET_PROB_AMOUNT,
            NAME_PROB_PERCENT, NAME_PROB_AMOUNT, NAME_ESTIMATED_BASELINE1,
            NAME_ESTIMATED_BASELINE2, NAME_ESTIMATED_BASELINE3,
            NAME_ESTIMATED_TOTAL, NAME_ESTIMATED_BUDGET, NAME_ESTIMATED_ACTUAL,
            NAME_ESTIMATED_ETC, NAME_BUDGET_BASELINE1, NAME_BUDGET_BASELINE2,
            NAME_BUDGET_BASELINE3, NAME_BUDGET, NAME_ACTUAL, NAME_ETC,
            NAME_PAID_TODATE, NAME_AUTO_FLAG, NAME_ACTUAL_EDITED,
            NAME_ROLLUP_BITS, NAME_TIMEPHASED_BITS, NAME_TIMEPHASED_FLAG,
            NAME_DATAFIELD_1, NAME_DATAFIELD_2, NAME_TXT_FIELD_01,
            NAME_TXT_FIELD_02, NAME_TXT_FIELD_03, NAME_TXT_FIELD_04,
            NAME_TXT_FIELD_05, NAME_TXT_FIELD_06, NAME_TXT_FIELD_07,
            NAME_TXT_FIELD_08, NAME_TXT_FIELD_09, NAME_TXT_FIELD_10,
            NAME_TOLERANCE, NAME_ESTIMATED_ACTUAL_TOTAL,
            NAME_ESTIMATED_ETC_TOTAL, NAME_ESTIMATED_PAID_TODATE,
            NAME_ESTIMATED_PAID_TODATE_TOTAL, NAME_ESTIMATED_PROB_AMOUNT,
            NAME_ESTIMATED_PROB_AMOUNT_TOTAL, NAME_BASELINE3_SCH_FLAG,
            NAME_BASELINE3_EST_FLAG, NAME_DATAFIELD_3, NAME_DATAFIELD_4,
            NAME_ACTUAL_TIMEPHASED_FLAG, NAME_CAPITAL_EXPENSE, NAME_BILLABLE,
            NAME_COST_CENTER_BITS, NAME_CHARGED, NAME_LEDGER,
            NAME_ESTIMATED_CHARGED, NAME_ESTIMATED_LEDGER,
            NAME_ESTIMATED_CHARGED_TOTAL, NAME_ESTIMATED_LEDGER_TOTAL
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new FinancialManager instance.
     * 
     * @return FinancialManager
     */
    public FinancialManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_BUDGET_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public String getFilter()
    {
        // Ignoring calendar for now (Not ment to be addressed in this release)
        return NAME_CHARGECODE_ID + " <> 'PMO_PROJECT_TIMEPHASE_CALENDAR__'";
    }

    
    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            // XPath example: /WorkElement/WorksheetFinancial
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(getFilter());
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof TimePhasedFinancialManager)
        {
            // XPath example: /WorksheetFinancial/TimePhasedFinancial
            joinCondition = new JoinCondition();

            // adding BUDGETS table to the condition
            joinCondition.setTableName(getTableName());
            StringBuffer where = new StringBuffer();
            where.append(getFilter());
            where.append(" AND ");
            where.append(TimePhasedFinancialManager.NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(NAME_ELEMENT_ID);
            where.append(" AND ");
            where.append(TimePhasedFinancialManager.NAME_CHARGECODE_ID);
            where.append(" = ");
            where.append(NAME_CHARGECODE_ID);
            where.append(" AND ");
            where.append(TimePhasedFinancialManager.NAME_BUDGET_TYPE);
            where.append(" = ");
            where.append(BudgetTypeToDBMap.getIntegerBudgetTypeFromFieldname(
                container.getType(), container.getFieldName()));

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ChargeCodeManager)
        {
            // XPath example: /WorksheetFinancial/ChargeCode
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(ChargeCodeManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_CHARGECODE_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager || manager instanceof ScopeManager)
        {
            // XPath example: /WorkElement[WorksheetFinancial[Id='.....']]
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            if (manager instanceof WbsManager)
                where.append(WbsManager.NAME_ELEMENT_ID);
            else if (manager instanceof ScopeManager)
                where.append(ScopeManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(0);
    }

    abstract protected RPMObject createContainer(int typeId)
            throws RPMException;

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String chargeCodeID = null;
        relatedFields = super.decodeRelatedFields(resultSet, relatedFields,
                index);

        relatedFields.put(index, ID_ELEMENT_ID, resultSet
                .getString(ID_ELEMENT_ID));

        chargeCodeID = resultSet.getString(ID_CHARGECODE_ID);
        
        if(chargeCodeID!=null)
        {
            chargeCodeID = chargeCodeID.trim();
        }
        relatedFields.put(index, ID_CHARGECODE_ID, chargeCodeID);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // TODO SLegault: skip calculateFTE for objects that are not in both
        // TMT_WBS and TMT_CRI.
        if (!FinancialUtil.DEFAULT_CHARGE_CODE_ID.contains(((String) relatedFields
                .get(index, ID_CHARGECODE_ID))))

        {
            FinancialBusinessLogicUtil
                .decodeCustomChargeCodeBusinessRules((AbstractFinancial) rpmObject);
        }
        
        if (rpmScope != null)
        {

            AbstractFinancial financial = (AbstractFinancial) rpmObject;            

            // Parent is read-only
            if (rpmScope.getParent() != null && !isDelta)
            {
                // we check the type of the scope, because object itself gets
                // cleaned by
                // the framework sometimes
                if (rpmScope.getParent() instanceof WorkElementScope)
                {
                    WorkElement newParent = new WorkElement();
                    newParent.setID((String) relatedFields.get(index,
                            ID_ELEMENT_ID));

                    WbsManager manager = (WbsManager) RPMManagerFactory
                            .getInstance().getRPMObjectManager(
                                    WorkElement.class.getName());

                    newParent = (WorkElement) manager.loadByPrimaryKey(
                            newParent, rpmScope.getParent(),
                            messageContext, false);
                    financial.setParent(newParent);
                }
                else if (rpmScope.getParent() instanceof ScopeElementScope)
                {
                    // TODO remove rpmObject.getParent() dependency, cause we
                    // can rely on ScopeElementScope only
                    if (rpmObject.getParent() instanceof AbstractAggregateScope)
                    {
                        AbstractAggregateScope newParent = (AbstractAggregateScope) rpmObject
                                .getParent();
                        newParent.setID((String) relatedFields.get(index,
                                ID_ELEMENT_ID));

                        ScopeManager manager = (ScopeManager) RPMManagerFactory
                                .getInstance().getRPMObjectManager(
                                        AbstractAggregateScope.class.getName());

                        newParent = (AbstractAggregateScope) manager
                                .loadByPrimaryKey(newParent, rpmScope
                                        .getParent(), messageContext, false);
                        financial.setParent(newParent);
                    }
                    else if (rpmObject.getParent() instanceof AbstractScopeTask)
                    {
                        AbstractScopeTask newParent = (AbstractScopeTask) rpmObject
                                .getParent();
                        newParent.setID((String) relatedFields.get(index,
                                ID_ELEMENT_ID));

                        ScopeManager manager = (ScopeManager) RPMManagerFactory
                                .getInstance().getRPMObjectManager(
                                        AbstractScopeTask.class.getName());

                        newParent = (AbstractScopeTask) manager
                                .loadByPrimaryKey(newParent, rpmScope
                                        .getParent(), messageContext, false);
                        financial.setParent(newParent);
                    }
                }
                else
                {
                    ExceptionUtil.handleRPMException(this, 200003);
                }
            }

            if(rpmScope instanceof FinancialScope){
            
	            FinancialScope financialScope = (FinancialScope) rpmScope;
	            
	            // ChargeCode is read-only
	            if (financialScope.isChargeCode() && !isDelta)
	            {
	                ChargeCode cc = null;
	                cc = new ChargeCode();
	                cc.setID((String) relatedFields.get(index, ID_CHARGECODE_ID));
	
	                ChargeCodeManager manager = (ChargeCodeManager) RPMManagerFactory
	                        .getInstance().getRPMObjectManager(
	                                ChargeCode.class.getName());
	                cc = (ChargeCode) manager.loadByPrimaryKey(cc, null,
	                        messageContext, isDelta);
	                financial.setChargeCode(cc);
	
	                /*
	                 * if isSystemChargeCode(Id) then begin AText :=
	                 * Translate('N/A'); end
	                 */
	                if (cc != null && FinancialCategoryUtil.isSystemFinancialCategory(cc
	                        .getFinancialCategory()))
	                {
	                    financial.setRevRecType(RevenueRecognitionMethodType.NA);
	                }
	            }
	
                TimePhasedFinancialManager manager = (TimePhasedFinancialManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        TimePhasedFinancial.class.getName());
                
                if (financialScope.isActualTimePhased())
                {
                    financial.setActualTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial.getActualTimePhased(),
                        BudgetType.Actual, financialScope, messageContext,
                        isDelta));
                }
                if (financialScope.isEstimateTimePhased())
                {
                    financial.setEstimateTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial.getEstimateTimePhased(),
                        BudgetType.Estimate, financialScope, messageContext,
                        isDelta));
                }
                if (financialScope.isBaselineTimePhased())
                {
                    financial.setBaselineTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial.getBaselineTimePhased(),
                        BudgetType.Baseline, financialScope, messageContext,
                        isDelta));
                }
                if (financialScope.isBaselinePreviousTimePhased())
                {
                    financial.setBaselinePreviousTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial
                            .getBaselinePreviousTimePhased(),
                        BudgetType.BaselinePrevious, financialScope,
                        messageContext, isDelta));
                }
                if (financialScope.isBaselineInitialTimePhased())
                {
                    financial.setBaselineInitialTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial
                            .getBaselineInitialTimePhased(),
                        BudgetType.BaselineInitial, financialScope,
                        messageContext, isDelta));
                }
                if (financialScope.isGLTimePhased())
                {
                    financial
                        .setGLTimePhased(loadTimePhased((String) relatedFields
                            .get(index, ID_ELEMENT_ID), (String) relatedFields
                            .get(index, ID_CHARGECODE_ID), manager, financial,
                            financial.getGLTimePhased(), BudgetType.GL,
                            financialScope, messageContext, isDelta));
                }
                if (financialScope.isBilledTimePhased())
                {
                    financial.setBilledTimePhased(loadTimePhased(
                        (String) relatedFields.get(index, ID_ELEMENT_ID),
                        (String) relatedFields.get(index, ID_CHARGECODE_ID),
                        manager, financial, financial.getBilledTimePhased(),
                        BudgetType.Billed, financialScope, messageContext,
                        isDelta));
                }
	            
            }
        }
        return rpmObject;
    }

    /**
     * 
     * @param timePhasedFinancialManager
     * @param financial
     * @param budgetType
     * @param financialScope
     * @param messageContext
     * @param isDelta
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    private TimePhasedFinancial[] loadTimePhased(String parentID,
            String chargeCodeID,
            TimePhasedFinancialManager timePhasedFinancialManager,
            AbstractFinancial financial, RPMObject[] objectsToCompare,
            BudgetType budgetType, FinancialScope financialScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        Object args[] = new Object[3];
        args[0] = parentID;
        args[1] = chargeCodeID;
        args[2] = BudgetTypeToDBMap.getIntegerBudgetType(
                financial instanceof WorksheetFinancial, budgetType);

        ArrayList found = timePhasedFinancialManager
                .loadByForeignKeyWithSpecifiedParent(financial,
                        objectsToCompare, null, messageContext, this, args,
                        null, null, isDelta);

        TimePhasedFinancial[] results = null;
        if (found != null)
        {
            results = new TimePhasedFinancial[found.size()];
            results = (TimePhasedFinancial[]) found.toArray(results);
        }

        return results;
    }

    /**
     * FTE (Full Time Equivalent) line is treated exclusively in RPM Client UI
     * The values from the DB are never displayed, but calculated values appear,
     * based on TMT_WBS table, SP_S_FTE() results and Financial object for Hours
     * 
     * Business rules are located in AssignBudgetViewFrame.pas see methods
     * LoadDataEx() and updateFTE()
     * 
     * @param rpmObject
     * @param messageContext
     * @param relatedFields
     * @param index
     * @return
     */
    abstract protected RPMObject calculateFTE(boolean isWorkElementParent,
            RPMObject rpmObject, MessageContext messageContext,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException;

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager || manager instanceof ScopeManager)
        {
            // called from WBSManager to get WBSFinancials
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID + " = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }

            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof TimePhasedFinancialManager
                || manager instanceof FinancialManager)
        {
            // Called from TimePhasedFinancialManager to get its parent
            // Called from FinancialManager to check if the row is in the
            // database already

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" = ? AND ");
            where.append(NAME_CHARGECODE_ID);
            where.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }

            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        AbstractFinancial financial = (AbstractFinancial) rpmObject;

        // The row might already be there - with an opposite element
        AbstractFinancial opposite = financial instanceof WorksheetFinancial
                ? (AbstractFinancial) new WbsFinancial()
                : (AbstractFinancial) new WorksheetFinancial();

        FinancialManager oppositeManager = (FinancialManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                        opposite.getClass().getName());

        ArrayList found = oppositeManager.loadByForeignKey(null, null,
                messageContext, this, new Object[]
                {
                        financial.getParent().getID(),
                        financial.getChargeCode().getID()
                }, null, null, false);

        if (found != null && found.size() > 0)
        {
            // in case the database row is already there
            financial.setID(((AbstractFinancial) found.get(0)).getID());
        }
        else
        {
            // in case new row is required
            financial.setID(SP_A_BUDGET(financial, messageContext));
        }

        // in case user has defined timephased breakdowns - use manual
        // timephased
        if ((financial.getActualTimePhased() != null && financial
                .getActualTimePhased().length > 0)
                || (financial.getEstimateTimePhased() != null && financial
                        .getEstimateTimePhased().length > 0))
        {
            financial.setTimePhased(TimePhasedType.ManuallyTimephased);
        }
        else
        {
            financial.setTimePhased(TimePhasedType.AutoTimephased);
        }
    }

    private String SP_A_BUDGET(AbstractFinancial financial,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_A_BUDGET";

        Object args[] = new Object[4];
        // Set the table type: 1 for a WorkElement and 3 for a ScopeElement.
        if (ContextUtil.SCOPE_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof ScopeElement))
        {
            args[0] = new Long(UPDATE_SCOPE_ELEMENT);
        }
        else if (ContextUtil.WBS_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof WorkElement))
        {
            args[0] = new Long(UPDATE_WORK_ELEMENT);
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, financial,
                    "SP_A_BUDGET");
        }

        if (financial.getParent() == null
                || financial.getParent().getID() == null)
        {
            RPMException e = new RPMException(
                    "This type of object can't be saved without a parent");
            throw e.log();
        }

        // Set the parent element ID.
        args[1] = financial.getParent().getID();

        if (financial.getChargeCode() == null
                || financial.getChargeCode().getID() == null)
        {
            RPMException e = new RPMException(
                    "New Financial object requires CargeCode object to be attached.");
            throw e.log();
        }

        args[2] = financial.getChargeCode().getID();
        args[3] = getUser(messageContext).getID();

        String budgetId = executeProcedure(messageContext, procedureName, args);

        return budgetId;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        AbstractFinancial financial = (AbstractFinancial) rpmObject;
        FinancialScope scope = (FinancialScope) rpmScope;

        boolean timephasedModified = false;
        if (!FinancialUtil.isFTEFinancial(financial))
        {

        // Case 1: User modified TimePhased manually,and is saving via the parent
        // so Actuals and Estimates have to be updated in consequence
        if (scope != null)
        {
            if (scope.isActualTimePhased() || scope.isEstimateTimePhased())
            {
                // check if timephased actuals or estimates have been modified
                boolean actualTimephasedModified = FinancialUtil
                    .isTimePhasedActualAllYearsModified(financial);
                boolean estimateTimephasedModified = FinancialUtil
                    .isTimePhasedEstimateAllYearsModified(financial);

                timephasedModified = actualTimephasedModified
                        || estimateTimephasedModified;

                boolean includeActuals = false;
                boolean includeEstimates = false;

                if (timephasedModified)
                {
                    if (actualTimephasedModified && scope.isActualTimePhased()
                            && financial.getActualTimePhased() != null)
                    {
                        includeActuals = true;
                    }

                    if (estimateTimephasedModified
                            && scope.isEstimateTimePhased()
                            && financial.getEstimateTimePhased() != null)
                    {
                        includeEstimates = true;
                    }

                    // applying business rule - when user modifies timephased,
                    // types becomes 'Manual'
                    financial.setTimePhased(TimePhasedType.ManuallyTimephased);

                    if (financial instanceof WorksheetFinancial)
                    {
                        double totalTimephasedEstimate = 0.0;
                        double totalTimephasedActual = 0.0;
                        double differenceActuals = 0.0;
                        double differenceEstimates = 0.0;
                        AbstractFinancial existingFinancial = null;

                        //-- Loading current financial DB values
                        
                        if (includeActuals || includeEstimates)
                        {
                            // we need to check if the actual and estimate
                            // fields already had
                            // values and remove the values of the existing
                            // timephased totals
                            // then add the new timephased values to the total
                            existingFinancial = FinancialUtil
                                .loadExistingWorksheetValues(
                                    (WorksheetFinancial) financial, this,
                                    messageContext);

                            differenceActuals = existingFinancial.getActual() == null
                                    ? 0
                                    : existingFinancial.getActual()
                                        .doubleValue()
                                            - FinancialUtil
                                                .calculateTimephasedActualTotal(existingFinancial);

                            differenceEstimates = existingFinancial
                                .getEstimate() == null
                                    ? 0
                                    : existingFinancial.getEstimate()
                                        .doubleValue()
                                            - FinancialUtil
                                                .calculateTimephasedEstimateTotal(existingFinancial);
                            
                            
                            
                            differenceActuals = differenceActuals >= 0
                                    ? differenceActuals : 0;
                            differenceEstimates = differenceEstimates >= 0
                                    ? differenceEstimates : 0;

                        }
                        
                        //--Updating worksheet financial ESTIMATE and ACTUALS based on scope.
                        
                        if (includeActuals)
                        {
                            // manually add up the totals in order to update the
                            // worksheet
                            totalTimephasedActual = FinancialUtil
                                .calculateTimephasedActualTotal(financial);
                            
                            financial.setActual(new Double(totalTimephasedActual
                                    + differenceActuals));                            

                            // check if actuals were modified while estimates
                            // are NOT in the scope
                            if (!scope.isEstimateTimePhased()
                                    || financial.getEstimateTimePhased() == null)
                            {
                              financial.setEstimate(existingFinancial
                                    .getEstimate() == null ? new Double(0.0)
                                        : existingFinancial.getEstimate());
                            }
                        }

                        if (includeEstimates)
                        {
                            // manually add up the totals in order to update the
                            // worksheet
                            totalTimephasedEstimate = FinancialUtil
                                .calculateTimephasedEstimateTotal(financial);

                            financial.setEstimate(new Double(totalTimephasedEstimate
                                    + differenceEstimates));

                            // check if estimates were modified while actuals
                            // are NOT in the scope
                            if (!scope.isActualTimePhased()
                                    || financial.getActualTimePhased() == null)
                            {
                                financial.setActual(existingFinancial
                                    .getActual() == null ? new Double(0.0)
                                        : existingFinancial.getActual());                                
                            }
                        }
                        
                        //--Updating worksheet financial ETC
                        
                        double estimateMinusActual = FinancialUtil.nullToZero(
                            financial.getEstimate()).doubleValue()
                                - FinancialUtil.nullToZero(
                                    financial.getActual()).doubleValue();

                        financial.setETC(estimateMinusActual > 0 ? new Double(
                            estimateMinusActual) : new Double(0));                        

                        // Calculate and set new Actuals and Estimates
                    }                  
                    
                    //--Updating TimePhased financial ESTIMATE and ACTUALS based on scope.
                    
                    if (includeActuals)
                    {
                        // insert/update/delete timephased actuals on mutable
                        // fields
                        financial
                            .setActualTimePhased((TimePhasedFinancial[]) updateChildren(
                                financial, financial.getActualTimePhased(),
                                new TimePhasedFinancialScope(), messageContext,
                                null));
                    }
                    
                    if (includeEstimates)
                    {
                        // insert/update/delete timephased estimates on mutable fields
                        financial
                            .setEstimateTimePhased((TimePhasedFinancial[]) updateChildren(
                                financial, financial
                                    .getEstimateTimePhased(),
                                new TimePhasedFinancialScope(),
                                messageContext, null));
                    }
               }
            }
            
            
            // Case 2: TimePhased is updated automatically using the 
            // values for Actuals and Estimates that were provided by the user
            // In this case, we create a default TimePhased container
            
            if (financial.getActual() != null && financial.testActualModified()
                    && !timephasedModified)
            {
                setValidTimePhased(financial, false);

                financial
                    .setActualTimePhased((TimePhasedFinancial[]) updateChildren(
                        financial, financial.getActualTimePhased(),
                        new TimePhasedFinancialScope(), messageContext, null));

                // Reloading the financial data that was updated in the
                // TimePhasedManager
                if (!FinancialCategoryUtil.isSystemFinancialCategory(financial
                    .getChargeCode().getFinancialCategory()))
                {

                    AbstractFinancial reloaded = null;

                    if (financial instanceof WorksheetFinancial)
                    {
                        reloaded = FinancialUtil.loadExistingWorksheetValues(
                            (WorksheetFinancial) financial, this,
                            messageContext);

                    }
                    else
                    {
                        reloaded = FinancialUtil.loadExistingWbsValues(
                            (WbsFinancial) financial, this, messageContext);

                    }

                    
                    if (reloaded != null)
                    {
                        financial.setActual(reloaded.getActual());
                        financial.setRemaining(reloaded.getRemaining());
                        financial.setETC(reloaded.getETC());
                        financial.setEAC(reloaded.getEAC());
                    }

                }

            }

            if (financial.getEstimate() != null
                    && financial.testEstimateModified() && !timephasedModified)
            {
                setValidTimePhased(financial, true);

                financial
                    .setEstimateTimePhased((TimePhasedFinancial[]) updateChildren(
                        financial, financial.getEstimateTimePhased(),
                        new TimePhasedFinancialScope(), messageContext, null));

                // Reloading the financial data that was updated in the
                // TimePhasedManager
                if (!FinancialCategoryUtil.isSystemFinancialCategory(financial
                    .getChargeCode().getFinancialCategory()))
                {
                    AbstractFinancial reloaded = null;

                    if (financial instanceof WorksheetFinancial)
                    {
                        reloaded = FinancialUtil.loadExistingWorksheetValues(
                            (WorksheetFinancial) financial, this,
                            messageContext);

                    }
                    else
                    {
                        reloaded = FinancialUtil.loadExistingWbsValues(
                            (WbsFinancial) financial, this, messageContext);

                    }
                    
                    if (reloaded != null)
                    {
                        financial.setEstimate(reloaded.getEstimate());
                        financial.setRemaining(reloaded.getRemaining());
                        financial.setETC(reloaded.getETC());
                        financial.setEAC(reloaded.getEAC());

                    }

                }

            }
        }
        

        if (financial.testActualModified() || financial.testETCModified()
                || financial.testEstimateModified()
                || financial.testProbabilityPercentModified()
                || timephasedModified)
        {
            //we are not saving FTE anymore
                SP_U_BUDGET(financial, messageContext);
            
        }
    }
        return rpmObject;   
        
        
    }
    
    private void setValidTimePhased(AbstractFinancial financial,
            boolean isEstimate)
    {
        // When worksheetFinancials are created without timephased we must
        // create
        // basic timephased infos with the values of actuals or estimates and
        // add them to
        // the current month total of the current year.
        // If the financial has timephased we must reset them to zero and add
        // the basic
        // timephased with them. Following UI logic until we know otherwise. :)

        // If financial has timephased we must reset them to zero
        TimePhasedFinancial currentTimePhasedTable[] = (isEstimate) ? financial
            .getEstimateTimePhased() : financial.getActualTimePhased();

        Calendar cal = DateUtil.getCalendarInstance();
        int currentYear = cal.get(Calendar.YEAR);
        int currentMonth = cal.get(Calendar.MONTH);

        int nbTimePhasedEntries = 0;
        boolean isSameYearTimephased = true;

        if (currentTimePhasedTable != null && currentTimePhasedTable.length > 0)
        {
            nbTimePhasedEntries = currentTimePhasedTable.length;

            for (int i = 0; i < nbTimePhasedEntries; i++)
            {
                currentTimePhasedTable[i].setMonth01Value(new Double(0));
                currentTimePhasedTable[i].setMonth02Value(new Double(0));
                currentTimePhasedTable[i].setMonth03Value(new Double(0));
                currentTimePhasedTable[i].setMonth04Value(new Double(0));
                currentTimePhasedTable[i].setMonth05Value(new Double(0));
                currentTimePhasedTable[i].setMonth06Value(new Double(0));
                currentTimePhasedTable[i].setMonth07Value(new Double(0));
                currentTimePhasedTable[i].setMonth08Value(new Double(0));
                currentTimePhasedTable[i].setMonth09Value(new Double(0));
                currentTimePhasedTable[i].setMonth10Value(new Double(0));
                currentTimePhasedTable[i].setMonth11Value(new Double(0));
                currentTimePhasedTable[i].setMonth12Value(new Double(0));

                currentTimePhasedTable[i].setTotalForYear(new Double(0));
                currentTimePhasedTable[i].setModified();
                
                if (FinancialUtil.nullToZero(
                    currentTimePhasedTable[i].getYear()).intValue() != currentYear)
                {
                    isSameYearTimephased = false;
                }
            }
        }

        nbTimePhasedEntries = (isSameYearTimephased)? 0:nbTimePhasedEntries;
        
        // Creating default timephased for worksheetfinancials and adding total
        // actuals or estimates to current month of current year
        TimePhasedFinancial defaultTimePhased[] = new TimePhasedFinancial[1 + nbTimePhasedEntries];
        defaultTimePhased[0] = new TimePhasedFinancial();

        final int START_COPY_SOURCE = 0;
        final int START_COPY_DESTINATION = 1;

        Double total = (isEstimate) ? FinancialUtil.nullToZero(financial
            .getEstimate()) : FinancialUtil.nullToZero(financial.getActual());

        defaultTimePhased[0].setYear(new Integer(currentYear));
        defaultTimePhased[0].setBudgetType((isEstimate) ? BudgetType.Estimate
                : BudgetType.Actual);

        switch (currentMonth)
        {
        case Calendar.JANUARY:
            defaultTimePhased[0].setMonth01Value(total);
            break;
        case Calendar.FEBRUARY:
            defaultTimePhased[0].setMonth02Value(total);
            break;
        case Calendar.MARCH:
            defaultTimePhased[0].setMonth03Value(total);
            break;
        case Calendar.APRIL:
            defaultTimePhased[0].setMonth04Value(total);
            break;
        case Calendar.MAY:
            defaultTimePhased[0].setMonth05Value(total);
            break;
        case Calendar.JUNE:
            defaultTimePhased[0].setMonth06Value(total);
            break;
        case Calendar.JULY:
            defaultTimePhased[0].setMonth07Value(total);
            break;
        case Calendar.AUGUST:
            defaultTimePhased[0].setMonth08Value(total);
            break;
        case Calendar.SEPTEMBER:
            defaultTimePhased[0].setMonth09Value(total);
            break;
        case Calendar.OCTOBER:
            defaultTimePhased[0].setMonth10Value(total);
            break;
        case Calendar.NOVEMBER:
            defaultTimePhased[0].setMonth11Value(total);
            break;
        case Calendar.DECEMBER:
            defaultTimePhased[0].setMonth12Value(total);
            break;
        }

        defaultTimePhased[0].setTotalForYear(total);

        // If timephased exist we add them to the new one
        if (currentTimePhasedTable != null && !isSameYearTimephased)
        {
            System.arraycopy(currentTimePhasedTable, START_COPY_SOURCE,
                defaultTimePhased, START_COPY_DESTINATION, nbTimePhasedEntries);
        }

        // Attaching Timephased to financial
        if (isEstimate)
        {
            financial.setEstimateTimePhased(defaultTimePhased);
        }
        else
        {
            financial.setActualTimePhased(defaultTimePhased);
        }
        
    }

    protected abstract void SP_U_BUDGET(AbstractFinancial financial,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException;

    
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.OBJECT_IS_NULL);
        }

        if (!(rpmObject instanceof AbstractFinancial))
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.INVALID_RPMOBJECT_TYPE, new String[]
            {
                    StringUtil.getShortClassName(AbstractFinancial.class),
                    StringUtil.getShortClassName(rpmObject.getClass())
            });
        }

        if (rpmObject.getID() == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.NO_ID);
        }

        AbstractFinancial financial = (AbstractFinancial) rpmObject;

        // To delete a financial, all values should be updated to 0
        // also opposite financial is still stored at the same row and might not
        // be 0 ... which means the object will not be deleted, but 0-z will be
        // there
        financial.setActual(new Double(0));
        financial.setEstimate(new Double(0));
        financial.setETC(new Double(0));
        SP_U_BUDGET(financial, messageContext);

        // now we can delete
        SP_UA_BUDGET(financial, messageContext);
    }

    private void SP_UA_BUDGET(AbstractFinancial financial,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_UA_BUDGET";

        Object args[] = new Object[4];
        // Set the table type: 1 for a WorkElement and 3 for a ScopeElement.
        if (ContextUtil.SCOPE_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof ScopeElement))
        {
            args[0] = new Long(UPDATE_SCOPE_ELEMENT);
        }
        else if (ContextUtil.WBS_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof WorkElement))
        {
            args[0] = new Long(UPDATE_WORK_ELEMENT);
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, financial,
                    "SP_U_BUDGET");
        }

        if (financial.getParent() == null
                || financial.getParent().getID() == null)
        {
            RPMException e = new RPMException(
                    "This type of object can't be deleted without a parent");
            logger.error(e);
            throw e;
        }

        // Set the parent element ID.
        args[1] = financial.getParent().getID();

        if (financial.getID() == null)
        {
            RPMException e = new RPMException(
                    "Object without ID cannot be deleted");
            logger.error(e);
            throw e;
        }

        args[2] = financial.getID();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    /**
     * FTE calcualtion requires certain data from TMT_WBS and TMT_CRI, which is not fetched
     * to WorkElement container. It was decided to do a direct SQL call to get
     * this data
     * 
     * @param workElementID
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    protected Map loadWbsDataForFTE(String workElementID,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        String selectStmt = "SELECT TMT_WBS.START_FINISH_DURATION, TMT_CRI.ESTIMATED_DURATION, TMT_WBS.ESTIMATED_BASELINE1_DURATION, "
                + "TMT_WBS.ESTIMATED_BASELINE2_DURATION, TMT_WBS.ESTIMATED_BASELINE3_DURATION, TMT_CRI.UNRESOLVED_DURATION, "
                + "TMT_WBS.ACTUAL_DURATION, TMT_WBS.BASELINE1_DURATION, TMT_WBS.BASELINE2_DURATION, TMT_WBS.BASELINE3_DURATION "
                + "FROM TMT_WBS, TMT_CRI "
                + "WHERE TMT_WBS.ELEMENT_ID = ? AND TMT_CRI.ELEMENT_ID = TMT_WBS.ELEMENT_ID";

        Connection c = null;
        ResultSet rs = null;
        Map resultMap = null;

        try
        {
            c = Manager.getConnection(messageContext);

            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);

            Object[] arguments = new Object[]
            {
                workElementID
            };

            try
            {
                rs = ManagerUtil.executeSqlQuery(messageContext, c, selectStmt, arguments);
                if (rs.next())
                {
                    resultMap = new HashMap();
                    resultMap.put("START_FINISH_DURATION", new Double(rs
                            .getDouble("START_FINISH_DURATION")));
                    resultMap.put("ESTIMATED_DURATION", new Double(rs
                            .getDouble("ESTIMATED_DURATION")));
                    resultMap.put("ESTIMATED_BASELINE1_DURATION", new Double(rs
                            .getDouble("ESTIMATED_BASELINE1_DURATION")));
                    resultMap.put("ESTIMATED_BASELINE2_DURATION", new Double(rs
                            .getDouble("ESTIMATED_BASELINE2_DURATION")));
                    resultMap.put("ESTIMATED_BASELINE3_DURATION", new Double(rs
                            .getDouble("ESTIMATED_BASELINE3_DURATION")));
                    resultMap.put("UNRESOLVED_DURATION", new Double(rs
                            .getDouble("UNRESOLVED_DURATION")));
                    resultMap.put("ACTUAL_DURATION", new Double(rs
                            .getDouble("ACTUAL_DURATION")));
                    resultMap.put("BASELINE1_DURATION", new Double(rs
                            .getDouble("BASELINE1_DURATION")));
                    resultMap.put("BASELINE2_DURATION", new Double(rs
                            .getDouble("BASELINE2_DURATION")));
                    resultMap.put("BASELINE3_DURATION", new Double(rs
                            .getDouble("BASELINE3_DURATION")));
                }
                else
                {
                    // Note: this situation is normal for certain objects, 
                    // like task for instance.
                    logger.debug(
                            "Rows not found, in both TMT_WBS and TMT_CRI, with ELEMENT_ID '"
                            + workElementID + "'. Cannot compute FTE for related financial object.");
                }
            }
            catch (Exception e)
            {
                ExceptionUtil.handleException(this, e);
            }
        }
        finally
        {
            Manager.close(messageContext,rs);
            Manager.releaseConnection(messageContext,c);
        }

        return resultMap;
    }
    
    /**
     * Called from calculateFTE, to get corresponding HOURS financials object 
     * 
     * @param manager either WorksheetFinancialManager or WbsFinancialManager
     * @param workElementID
     * @param chargeCodeID
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    protected AbstractFinancial loadHoursFinancial(FinancialManager manager,
            String workElementID, String chargeCodeID,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        WorkElementScope parentScope = new WorkElementScope();
        FinancialScope scope = new FinancialScope();
        scope.setParent(parentScope);
        scope.setChargeCode(true);

        ArrayList array = manager.loadByForeignKey(null, scope, messageContext,
            this, new Object[]
            {
                    workElementID,
                    ChargeCodeUtil.translateFTEChargeCodeToHours(chargeCodeID)
            }, null, null, false);

        if (array == null || array.size() != 1)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.NOT_EXISTING_ID,
                new String[]
                {
                        StringUtil.getShortClassName(this.getClass()),
                        "BUDGETS", workElementID + "/" + chargeCodeID,
                        "ELEMENT_ID/CHARGECODE_ID"
                });
        }

        return (AbstractFinancial) array.get(0);
    }
    
    /**
     * FTE calculation requires certain data retirieved by SP_S_FTE We have to
     * call this SP directy to get what we need
     * 
     * @param isWorkElementParent
     * @param workElementID
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     */
    public static Map SP_S_FTE(boolean isWorkElementParent, String workElementID,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        final String procedureName = "SP_S_FTE";
        Map resultMap = null;

        Object args[] = new Object[3];

        // type : SC_TABLE_WBS or SC_TABLE_CRI
        args[0] = new Long(isWorkElementParent ? UPDATE_WORK_ELEMENT
                : UPDATE_SCOPE_ELEMENT);

        // element_id : SC_WCHAR
        args[1] = workElementID;

        // ieflag : unused
        args[2] = new Long(0);//null

        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, procedureName, args, true, messageContext);
            if (rs.next())
            {
                resultMap = new HashMap();
                resultMap.put("EST_REMAINING", new Double(rs
                        .getDouble("EST_REMAINING")));
                resultMap.put("SCHED_REMAINING", new Double(rs
                        .getDouble("SCHED_REMAINING")));
            }
            else
            {
                ExceptionUtil.handleRPMException(FinancialManager.class,
                        ErrorCodes.NOT_EXISTING_ID, new String[]
                        {
                                "TMT_WBS", workElementID, "ELEMENT_ID"
                        });
            }
        }
        finally
        {
            Manager.close(messageContext,rs);
            Manager.releaseConnection(messageContext,c);
        }

        return resultMap;
    }
        
}
