/**
 * Created on Feb 3, 2006
 * @author shahla
 */
package com.ibm.rpm.financial.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.WbsManager;


public class TimeCodeAssignmentManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(TimeCodeAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                TimeCodeAssignmentManager.class);
        }
    }
    
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_TIMECODES.LEVEL_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_TIMECODES.TYPE_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_TIMECODES.MAX_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_TIMECODES.CHILD_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_TIMECODES.DELETED_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_TIMECODES.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_TIMECODES.ELEMENT_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ELEMENT_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_TIMECODES.PARENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_TIMECODES.REC_USER";
    // uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}


    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_TIMECODES.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 11;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_TIMECODES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_FLAG = 12;
    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;
    public static final String NAME_DEFAULT_FLAG = "TMT_TIMECODES.DEFAULT_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DEFAULT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_TIMECODES.RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FS_CODE = 14;
    public static final int TYPE_FS_CODE = Types.VARCHAR;
    public static final String NAME_FS_CODE = "TMT_TIMECODES.FS_CODE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_FS_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FS_CODE,NAME_FS_CODE);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 15;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_TIMECODES.DESCRIPTION";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DESCRIPTION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    public static final String TABLE_NAME = "TMT_TIMECODES";

    /**
     * Create an array of type string containing all the fields of the TMT_TIMECODES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_DEFAULT_FLAG
        ,NAME_RANK
        ,NAME_FS_CODE
        ,NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };
    
    /**
     * Creates a new TimeCodeAssignmentManager instance.
     * @return TimeCodeAssignmentManager
     */   
    public TimeCodeAssignmentManager()
    {
        super();
    }

    public String getTableName() 
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields() 
    {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() 
    {
        return null;
    }  
    
    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        return new String[]
        {
                NAME_ELEMENT_ID, NAME_PARENT_ID
        };
    }
    
    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        TimeCodeAssignment object = (TimeCodeAssignment) rpmObject;

        String parentId = object.getID();
        String timeCodelId = null;

        if (object.getTimeCode() != null)
        {
            timeCodelId = object.getTimeCode().getID();
        }

        return new Object[]
        {
                parentId, timeCodelId
        };
    }
    
    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject){
    	
        TimeCodeAssignment object = (TimeCodeAssignment)rpmObject;
        
        object.setID((String)primaryKey[0]);
        
        WorkElement parent = new WorkElement();
        parent.setID((String)primaryKey[0]);
        object.setParent(parent);
        
        TimeCode timeCode = new TimeCode();
        timeCode.setID((String)primaryKey[1]);
    	object.setTimeCode(timeCode);
    	
    	return rpmObject;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }
    
    protected HashMap getPropertiesMap() 
    {
        return FIELDPROPERTYMAP;
    }  
    
    
    public String getFilter()
    {
        SqlBuffer sb = new SqlBuffer();
        sb.append(NAME_REC_STATUS + " != 'D'");
        sb.appendAnd();
        sb.append(NAME_ELEMENT_ID + " IS NOT NULL");
        sb.appendAnd();
        sb.append(NAME_PARENT_ID + " IS NOT NULL");
        sb.appendAnd();
        sb.append(NAME_TYPE_ID);
        sb.append(" NOT IN ");
        sb.appendOpenParenthesis();
        sb.append(TimeCode.TYPE_ID);
        sb.appendComma();
        sb.append(TimeCodeCategory.TYPE_ID);
        sb.appendCloseParenthesis();
        
        return sb.toString();
    }
    
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {

            joinCondition = JoinCondition.createSubSelect(
                WbsManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, getTableName(), getFilter());
            joinCondition.setTableName(getTableName());
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append("=?");
            joinCondition.setCondition(where.toString());
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;

    }
    
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition(getTableName(), getFilter());
        }

        //TODO Implement for managers that handles the children of the
        //     supported containers

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
       }
    
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        
        if (manager instanceof WbsManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                WbsManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFilter());

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }
    
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {
        	case Project.TYPE_ID:
        	case Proposal.TYPE_ID:
        	case WorkOrganization.TYPE_ID:
        	case Task.TYPE_ID:
        	case Milestone.TYPE_ID:
        	case SummaryTask.TYPE_ID:
        	case WorkProduct.TYPE_ID:
        	case Deliverable.TYPE_ID:
        	    object = new TimeCodeAssignment();
        		break;
        	default:
        	    ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }
    
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        TimeCodeAssignment assignment = (TimeCodeAssignment) rpmObject;

        loadWorkElement(assignment, messageContext, relatedFields, index,
            isDelta);

        loadTimeCode(assignment, messageContext, relatedFields, index, isDelta);

        return rpmObject;
    }
    
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rs.getString(ID_ELEMENT_ID));

        return rpmObject;
    }
    
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, parentId);
        
        String timeCodeId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, timeCodeId);
       
        return relatedFields;
    }
    
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        SP_UA_TC_WBS((TimeCodeAssignment) rpmObject, messageContext);
    }
    
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        saveTimeCode((TimeCodeAssignment)rpmObject, messageContext);
        
        return rpmObject;
    }
    
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rpmObject.getParent().getID());
        SP_A_TC_WBS((TimeCodeAssignment) rpmObject, messageContext);
    }
    
    private void loadTimeCode(TimeCodeAssignment assignment,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        TimecodeManager timecodeManager = (TimecodeManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(TimeCode.class.getName());
        String oldTimeCodeId = (String) relatedFields.get(index, ID_PARENT_ID);
        
        RPMObject newTimeCode = null;
        if (isDelta)
        {
            newTimeCode = assignment.getTimeCode();
            RPMObject oldTimeCode = null;
            if (oldTimeCodeId != null)
            {
                oldTimeCode = new TimeCode();
                oldTimeCode.setID(oldTimeCodeId);
                if (newTimeCode != null && newTimeCode.getID() != null
                        && newTimeCode.getID().equals(oldTimeCodeId))
                {
                    newTimeCode = timecodeManager.loadByPrimaryKey(newTimeCode,
                        null, messageContext, true);
                }
                else
                {
                    oldTimeCode = timecodeManager.loadByPrimaryKey(oldTimeCode,
                        null, messageContext, true);
                }
            }
            assignment.deltaTimeCode((TimeCode) oldTimeCode);
        }
        else
        {
            if (oldTimeCodeId != null)
            {
                if (newTimeCode == null)
                { //newParent not in cache
                    newTimeCode = new TimeCode();
                    newTimeCode.setID(oldTimeCodeId);
                    newTimeCode = timecodeManager.loadByPrimaryKey(newTimeCode,
                        null, messageContext, false);
                }
                assignment.setTimeCode((TimeCode) newTimeCode);
            }
        }
    }
    
    private void saveTimeCode(TimeCodeAssignment assignment,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        TimecodeManager timecodeManager = (TimecodeManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(TimeCode.class.getName());

        assignment.setTimeCode((TimeCode) timecodeManager.internalSave(
            assignment.getTimeCode(), null, messageContext));
    }
    
    private void loadWorkElement(TimeCodeAssignment assignment, 
            MessageContext messageContext, FieldValueMap relatedFields, 
            int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_ELEMENT_ID);
        
        WbsManager wbsManager = (WbsManager) RPMManagerFactory.getInstance()
            .getRPMObjectManager(WorkElement.class.getName());
        
        RPMObject newParent = null;
        if (isDelta)
        {
            newParent = assignment.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new WorkElement();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    newParent = wbsManager.loadByPrimaryKey(newParent, null,
                        messageContext, true);
                }
                else
                {
                    oldParent = wbsManager.loadByPrimaryKey(oldParent, null,
                        messageContext, true);
                }
            }
            assignment.setParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {   // newTimeCode not in cache						
                    newParent = new WorkElement();
                    newParent.setID(oldParentId);
                    newParent = wbsManager.loadByPrimaryKey(newParent, null,
                        messageContext, false);
                }
                assignment.setParent(newParent);
            }
        }
    }
    
    public void isValidPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();
        
        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }
        
        if(((TimeCodeAssignment) rpmObject).getTimeCode() == null
                || ((TimeCodeAssignment) rpmObject).getTimeCode().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(" and ");
            invalidPrimaryKeys.append("timeCode.ID");
            valid = false;
        }
        
        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, invalidPrimaryKeys.toString());
        }
    }
    
    private void SP_A_TC_WBS(TimeCodeAssignment assignment, 
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_TC_WBS";
        Object args[] = new Object[3];
        
        args[0] = assignment.getParent().getID();  // element_id
        args[1] = assignment.getTimeCode().getID();  // id
        args[2] = getUser(messageContext).getID();  // rec_user
        
        executeProcedure(messageContext, name, args);
    }
    
    private void SP_UA_TC_WBS(TimeCodeAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_UA_TC_WBS";
        Object args[] = new Object[3];
        
        args[0] = assignment.getParent().getID();  // element_id
        args[1] = assignment.getTimeCode().getID();  // id
        args[2] = getUser(messageContext).getID();  // rec_user
        
        executeProcedure(messageContext, name, args);
    }
}
