/*
 * Created on Apr 5, 2005
 *
 */
package com.ibm.rpm.framework;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContainerClassMap;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.StringUtil;


/**
 * @author Genevieve
 *
 */
public class RPMManagerFactory {
    
	private static Log logger = LogFactory.getLog(RPMManagerFactory.class);
	
    private HashMap MANAGERCONTAINERMAP = new HashMap();
    private HashMap MANAGERMAP = new HashMap();
    
    private static RPMManagerFactory singleton = new RPMManagerFactory();

    /**
     * 
     */
    
    public RPMManagerFactory() 
    {
        // Nothing to initialize.
    }
       
    synchronized public static RPMManagerFactory getInstance()
    {
        return singleton;
    }
    
    /**
     * @param containerName
     * @param managerIsMandatory if true, an exception is thrown if the manager is null
     * @return
     * @throws RPMException
     */
    public RPMObjectManager getRPMObjectManager(String containerName, boolean managerIsMandatory)
            throws RPMException {

        RPMObjectManager manager = null;
        
        if(containerName != null)
        {           
            Object o = MANAGERCONTAINERMAP.get(containerName.toUpperCase());
            if(o == null)
                o = MANAGERCONTAINERMAP.get(containerName);
            if(o != null)
            {
                Class managerClass = (Class)o;
                try
                {                        
                    manager = (RPMObjectManager)MANAGERMAP.get(managerClass);
                    if(manager == null)
                    {
                        manager = (RPMObjectManager)managerClass.newInstance();
                        MANAGERMAP.put(managerClass, manager);
                    }
                    
                } catch (InstantiationException s) {
                    ExceptionUtil.handleMethodException(this, ErrorCodes.MF_NO_CONSTRUCTOR, StringUtil.getShortClassName(managerClass));
                } catch (IllegalAccessException s) {
                    ExceptionUtil.handleMethodException(this, ErrorCodes.MF_CONSTRUCTOR_NOT_PUBLIC, StringUtil.getShortClassName(managerClass));
                } catch (Exception s) {
                    ExceptionUtil.handleMethodException(this, ErrorCodes.MF_CANNOT_CREATE_INSTANCE, StringUtil.getShortClassName(managerClass));
                }               
            }
        }
        
        if (managerIsMandatory && manager == null)
        {
            ExceptionUtil.handleUnsupportedAPICall(this, containerName);
        }

        return manager;      
    }

    
    public RPMObjectManager getRPMObjectManager(String containerName)
            throws RPMException 
    {
        RPMObjectManager manager = getRPMObjectManager(containerName, true);
        return manager;      
    }
    
    public RPMObjectManager getRPMObjectManager(RPMObject rpmObject)
    throws RPMException {
        return getRPMObjectManager(rpmObject.getClass().getName());
    }
    
    public RPMObjectManager getRPMObjectManager(Class rpmObjectClass)
    throws RPMException {
        return getRPMObjectManager(rpmObjectClass.getName());
    }    
    
    public void addContainerToMap(Class containerClass, Class managerClass)
    {
        addContainerToMap(containerClass.getName(), managerClass);
    }
    
    public void addContainerToMap(String containerName, Class managerClass)
    {
        
        if(containerName != null){
            Class o = (Class) MANAGERCONTAINERMAP.get(containerName.toUpperCase());
            if(o != null){
                if(o != managerClass ){
                    logger.error("The container name " + containerName + " is already mapped to " + managerClass.getName());
                }                
            }else{
                if (managerClass != null)
                {
                    MANAGERCONTAINERMAP.put(containerName.toUpperCase(),
                            managerClass);

                    //Add container without the package name
                    MANAGERCONTAINERMAP.put(StringUtil.getShortClassName(
                            containerName).toUpperCase(), managerClass);
                    
                    //map container without package name to container with package name
                    ContainerClassMap.getInstance().addContainerClass(convertToClass(containerName));
                    
                }                
            }
        }
                
    }
    
    private Class convertToClass(String containerType)
    {

        Class containerClass = null;
        try
        {
            containerClass = Class.forName(containerType);
        }
        catch(NoClassDefFoundError e){
            //Note: We cannot throw an RPMException at this point. The error message will logged.
            RPMException ex = new RPMException(ErrorCodes.INVALID_CONTAINER_NAME, new String[]{containerType});
            logger.error(ex);
            //The original exception is thrown
            e.printStackTrace();
            throw e;
        }
        catch (ClassNotFoundException e)
        {
            
            //Note: We cannot throw an exception at this point. The error message will logged.
            RPMException ex = new RPMException(ErrorCodes.INVALID_CONTAINER_NAME, new String[]{containerType});
            logger.error(ex);
        }

        return containerClass;

    }
     
}
