/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security;

import com.ibm.rpm.framework.RPMException;

/**
 * @author Yasser Lulu
 * 
 */
public class OperationSecurityDescriptor extends BaseSecurityDescriptor
{

    private boolean canPerform;

    /**
     * @return the canPerform
     */
    public boolean isCanPerform()
    {
        return canPerform;
    }

    /**
     * @param canPerform the canPerform to set
     */
    public void setCanPerform(boolean canPerform)
    {
        this.canPerform = canPerform;
    }

    /**
     * @author Marc-Andr Lamothe
     * @return this == parameter
     */
    public boolean equals(Object obj)
    {
        return obj instanceof OperationSecurityDescriptor
                && super.equals(obj)
                && isCanPerform() == ((OperationSecurityDescriptor) obj)
                    .isCanPerform();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.AbstractSecurityDescriptor#combine(com.ibm.rpm.framework.security.AbstractSecurityDescriptor)
     */
    AbstractSecurityDescriptor combine(AbstractSecurityDescriptor other,
            boolean usingOr) throws RPMException
    {
        if (!(other instanceof OperationSecurityDescriptor))
        {
            throw new RPMException(
                "Cannot combine an object that is not a OperationSecurityDescriptor");
        }
        OperationSecurityDescriptor otherDescr = (OperationSecurityDescriptor) other;
        if (!getName().equals(otherDescr.getName()))
        {
            throw new RPMException(
                "Cannot combine OperationSecurityDescriptor of different operations");
        }
        OperationSecurityDescriptor result = new OperationSecurityDescriptor();
        result.setName(getName());
        if (usingOr)
        {
            result.setCanPerform(isCanPerform() || otherDescr.isCanPerform());
        }
        else
        {
            result.setCanPerform(isCanPerform() && otherDescr.isCanPerform());
        }
        return result;
    }

}
