package com.ibm.rpm.framework.security.controller.impl;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.interfaces.Application;

public class CreationOperationMappingEntry extends OperationMappingEntry
{
    public Class childrenClass;

    public CreationOperationMappingEntry(String operationName, Class container,
            Class childrenClass)
    {
        super(operationName, container);
        this.childrenClass = childrenClass;
    }

    /**
     * Returns the value of the associated security flag.
     */
    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        
        return SecurityControllerUtil.calculateFlagResult(securityFlag,
            securityFlags, getErrorMessage(), rpmObject.getClass());
    }

    /**
     * Returns a string describing the action on the container.
     */
    protected String getErrorMessage()
    {
        if (errorMessage == null)
        {
            if (container == Application.class)
                errorMessage = "create {0}"; // {0} will be replaced with container type
            else
                errorMessage = "create {0} in " + StringUtil.getShortClassName(container);
        }
        return errorMessage;
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(name);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil
            .calculateFlagValue(securityFlag, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(name, operationSecurityDescr);
    }
}
