/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl;

import java.text.MessageFormat;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.StringUtil;

public class OperationMappingEntry extends AbstractSecurityMappingEntry
{
    protected String errorMessage;

    public Integer securityFlag;

    public OperationMappingEntry(String operationName, Class container)
    {
        super(operationName, container);
    }

    /**
     * Returns the value of the associated security flag.
     */
    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (securityFlags.isALL_FALSE())
        {
            return SecurityValidationResult.makeFalseResult(MessageFormat
                .format(getErrorMessage(), new Object[]
                {
                    StringUtil.getShortClassName(rpmObject.getClass())
                }));
        }
        if (securityFlags.isALL_TRUE())
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }
        return SecurityControllerUtil.calculateFlagResult(securityFlag,
            securityFlags, getErrorMessage(), rpmObject.getClass());
    }

    /**
     * Returns a string describing the action on the container.
     */
    protected String getErrorMessage()
    {
        if (errorMessage == null)
        {
            errorMessage = name + " on {0}"; // {0} will be replaced with container type
        }
        return errorMessage;
    }

}