package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.types.PublishedType;

class ResourceBasedAbstractDocumentSecurityController extends
        ResourceBasedGenericDocumentSecurityController
{

    public Class getContainerClass()
    {
        return AbstractDocument.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry("keywords",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry(
            "minorRevisionNumber", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry("summary",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry(
            "majorRevisionNumber", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry(
            "elementNumber", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry("state",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry("published",
            getContainerClass())
        {

            public SecurityValidationResult canWrite(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                AbstractDocument abstractDocument = (AbstractDocument) rpmObject;
                PublishedType type = SecurityControllerUtil
                    .loadAbstractDocumentPublishedType(abstractDocument,
                        messageContext);
                if (type == PublishedType.ToTemplate
                        && abstractDocument.getPublished() != PublishedType.ToTemplate)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanDeleteTemplates, securityFlags,
                        getEditErrorMessage(), rpmObject.getClass());
                }
                return super.canWrite(rpmObject, securityFlags, messageContext);
            }
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry(
            "referenceNumber", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedDocumentElementMappingEntry(
            "linkingStatus", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishLocalWBSTemplates, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanPublishLocalWBSTemplates;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishToMethodWBSTemplates, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanPublishToMethodWBSTemplates;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        final DeletionOperationMappingEntry parentDeletionEntry = super
            .generateDeletionOperationMappingEntry();

        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                AbstractDocument abstractDocument = (AbstractDocument) rpmObject;
                PublishedType type = SecurityControllerUtil
                    .loadAbstractDocumentPublishedType(abstractDocument,
                        messageContext);
                if (type == PublishedType.ToTemplate)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanDeleteTemplates, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }

                return (parentDeletionEntry != null) ? parentDeletionEntry
                    .canPerform(rpmObject, securityFlags, messageContext)
                        : SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return deletionEntry;
    }

}
