package com.ibm.rpm.framework.security.controller.impl.resource;

import java.text.MessageFormat;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;

class ResourceBasedCreateGenericAssetMappingEntry extends
        CreationOperationMappingEntry
{

    public ResourceBasedCreateGenericAssetMappingEntry(
            String operationName, Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        if (SecurityControllerUtil.calculateFlagValue(RESOURCE.CanAccessAssetManagement,
            securityFlags)
                && SecurityControllerUtil.calculateFlagValue(
                    RESOURCE.CanCreateDeleteAssetsAndFolders,
                    securityFlags))
        {
            return SecurityValidationResult.TRUE_RESULT;
        }

        return SecurityValidationResult.makeFalseResult(MessageFormat.format(
            getErrorMessage(), new Object[]
            {
                StringUtil.getShortClassName(rpmObject.getClass())
            }));
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(name);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil.calculateFlagValue(
            RESOURCE.CanAccessAssetManagement, combinedSecurityFlags)
                && SecurityControllerUtil.calculateFlagValue(
                    RESOURCE.CanCreateDeleteAssetsAndFolders,
                    combinedSecurityFlags));
        creationOperationSecurityDescrs.put(name,
            operationSecurityDescr);
    }
}
