package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedEducationSecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return Education.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry("honoursAwards",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry("specialization",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry("appointedDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry("degreeProgram",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry(
            "certificationGraduateDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEducationMappingEntry("organization",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof Resource)
                {
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewPersonalBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewResourceBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}
