package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.util.SqlBuffer;

class ResourceBasedGenericAssetSecurityController extends
        ResourceBasedContainerSecurityController
{

    protected Integer[] readSecurityFlags = new Integer[]
    {
        RESOURCE.CanAccessAssetManagement,
    };
    protected Integer[] writeSecurityFlags = new Integer[]
    {
        RESOURCE.CanAccessAssetManagement,
    };
        
    public Class getContainerClass()
    {
        return GenericAsset.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateGenericAssetMappingEntry(
            SecuredOperations.CreateGenericAsset, getContainerClass(),
            GenericAsset.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = SecurityControllerUtil
                    .calculateFlagResult(RESOURCE.CanAccessAssetManagement,
                        securityFlags, "Access Asset Management", rpmObject.getClass());
                if (result.type == SecurityValidationResult.FALSE)
                    return result;
                // Finds out who created the asset
                String createdByID = null;
                try
                {
                    SqlBuffer condition = new SqlBuffer();
                    condition.appendEqualQuestionMark(AssetManager.NAME_ELEMENT_ID);
                    
                    Object[] arguments = new Object[]
                    {
                        rpmObject.getID()
                    };
                    
                    createdByID = (String) ManagerUtil.selectColumnValue(
                        String.class, null, null, AssetManager.NAME_CREATED_BY,
                        RPMManagerFactory.getInstance().getRPMObjectManager(
                            rpmObject).getTableName(), condition, arguments,
                        messageContext);
                }
                catch (Exception e)
                {
                }
                if (createdByID != null
                        && createdByID.equals(messageContext.getUser().getID()))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanCreateDeleteAssetsAndFolders,
                        securityFlags, getErrorMessage(), rpmObject.getClass());
                }
                else
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanDeleteAssetsAndFoldersFromOthers,
                        securityFlags, getErrorMessage(), rpmObject.getClass());
                }
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assets", getContainerClass());
        fieldInfo.canReadSecurityFlags = readSecurityFlags;
        fieldInfo.canWriteSecurityFlags = writeSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canReadSecurityFlags = readSecurityFlags;
        fieldInfo.canWriteSecurityFlags = writeSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
