package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;

class ResourceBasedGeographicalSecurityController extends
        ResourceBasedContainerSecurityController
{

    protected Integer[] writeSecurityFlags = new Integer[]
    {
        RESOURCE.CanEditLocationGeographies,
    };
        
    public Class getContainerClass()
    {
        return Geographical.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGeographical, getContainerClass(),
            Geographical.class);
        creationInfo.securityFlag = RESOURCE.CanEditLocationGeographies;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass());
        deletionEntry.securityFlag = RESOURCE.CanEditLocationGeographies;
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("description", getContainerClass());
        fieldInfo.canWriteSecurityFlags = writeSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = writeSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("children", getContainerClass());
        fieldInfo.canWriteSecurityFlags = writeSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
