package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.CalendarDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;

class ResourceBasedRPMCalendarSecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return RPMCalendar.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateRPMCalendar, getContainerClass(),
            RPMCalendar.class);
        creationInfo.securityFlag = RESOURCE.CanEditCalendarAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateCalendarDay, getContainerClass(),
            CalendarDay.class);
        creationInfo.securityFlag = RESOURCE.CanEditCalendarAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass());
        deletionEntry.securityFlag = RESOURCE.CanEditCalendarAdmin;
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        Integer[] canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanEditCalendarAdmin
        };

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardMonday", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardTuesday",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardSunday", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardSaturday",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardWednesday",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardThursday",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("exceptions", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("standardFriday", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("startDate", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("hoursPerWeek", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calendars", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("hoursPerDay", getContainerClass());
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    //    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    //    {
    //        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
    //            getContainerClass())
    //        {
    //            public SecurityValidationResult canPerform(RPMObject rpmObject,
    //                    CombinedSecurityFlags securityFlags,
    //                    MessageContext messageContext)
    //            {
    //                RPMObject parent = rpmObject.getParent();
    //                if (parent instanceof Resource)
    //                {
    //                    if (parent.getID() != null
    //                            && messageContext.getUser().getID().equals(
    //                                parent.getID()))
    //                    {
    //                        return SecurityControllerUtil.calculateFlagResult(
    //                            RESOURCE.CanViewPersonalCalendar, securityFlags,
    //                            getErrorMessage(), rpmObject.getClass());
    //                    }
    //                    else
    //                    { // TODO Should we return true?
    //                        return SecurityValidationResult
    //                            .makeFalseResult(MessageFormat.format(
    //                                getErrorMessage(), new Object[]
    //                                {
    //                                    StringUtil.getShortClassName(rpmObject
    //                                        .getClass())
    //                                }));
    //                    }
    //                }
    //                return SecurityValidationResult.UNDEFINED_RESULT;
    //            }
    //        };
    //        return viewEntry;
    //    }

}
