package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;

class ResourceBasedResourceAttributeAssignmentSecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return ResourceAttributeAssignment.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                RPMObject parent = SecurityControllerUtil.loadParent(
                    messageContext, rpmObject);
                if (parent instanceof Resource)
                {
                    if (parent.getID().equals(messageContext.getUser().getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanEditPersonalCompetencySkills,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanEditResourceCompetencySkills,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "proficiencyLevel", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "acquiredOn", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "preferenceRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "strategicFlag", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "mandatory", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "defaultFlag", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "mandatoryDetails", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "description", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "rates", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "experience", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "lastUsedLocation", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceAttributeAssignmentMappingEntry(
            "lastUsedDateTime", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof Resource)
                {
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewPersonalCompetencySkills,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewResourceCompetencySkills,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}
