/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.resource;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.resource.containers.ResourceRates;

class ResourceBasedResourceRatesMappingEntry extends FieldMappingEntry
{
    public ResourceBasedResourceRatesMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        String resourceID = SecurityControllerUtil.loadResourceIdWithRates(
            messageContext, (ResourceRates) rpmObject);
        if (resourceID != null)
        {
            if (messageContext.getUser().getID().equals(resourceID))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanViewPersonalRates, securityFlags,
                    getViewErrorMessage(), rpmObject.getClass());
            }
            else
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanViewResourceRates, securityFlags,
                    getViewErrorMessage(), rpmObject.getClass());
            }
        }
        else
        {
            resourceID = SecurityControllerUtil
                .loadResourceIdWithResourceAttributeAssignmentWithRates(
                    messageContext, (ResourceRates) rpmObject);
            if (resourceID != null)
            {
                if (messageContext.getUser().getID().equals(resourceID))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanViewPersonalCompetencyCostsRates,
                        securityFlags, getEditErrorMessage(), rpmObject
                            .getClass());
                }
                else
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanViewResourceCompetencyCostsRates,
                        securityFlags, getEditErrorMessage(), rpmObject
                            .getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        String resourceID = SecurityControllerUtil.loadResourceIdWithRates(
            messageContext, (ResourceRates) rpmObject);
        if (resourceID != null)
        {
            if (messageContext.getUser().getID().equals(resourceID))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditPersonalRates, securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
            else
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditResourceRates, securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
        }
        else
        {
            resourceID = SecurityControllerUtil
                .loadResourceIdWithResourceAttributeAssignmentWithRates(
                    messageContext, (ResourceRates) rpmObject);
            if (resourceID != null)
            {
                if (messageContext.getUser().getID().equals(resourceID))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanEditPersonalCompetencyCostsRates,
                        securityFlags, getEditErrorMessage(), rpmObject
                            .getClass());
                }
                else
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanEditResourceCompetencyCostsRates,
                        securityFlags, getEditErrorMessage(), rpmObject
                            .getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}