package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;

public class ResourceBasedResourceSecurityRightsSecurityController extends
        ResourceBasedAbstractSecurityGroupSecurityController
{

    public Class getContainerClass()
    {
        return ResourceSecurityRights.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        ResourceBasedResourceSecurityRightsMappingEntry fieldInfo;

        Integer[] canReadSecurityFlags = new Integer[]
        {
            RESOURCE.CanViewPersonalSecurity,
            RESOURCE.CanViewResourceSecurity,
        };
        Integer[] canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanEditResourceSecurity,
        };
            
        /*
         * Replace all the FieldMappingEntry with
         * ResourceBasedResourceSecurityRightsMappingEntry because
         * different flags applies to ResourceSecurityRights
         */
        Iterator it = mappingEntryMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry mapEntry = (Entry) it.next();
            FieldMappingEntry fieldEntry = (FieldMappingEntry) mapEntry.getValue();
            fieldInfo = new ResourceBasedResourceSecurityRightsMappingEntry(
                fieldEntry.name, getContainerClass());
            fieldInfo.canReadSecurityFlags = canReadSecurityFlags;
            fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
            mapEntry.setValue(fieldInfo);
        }
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof Resource)
                {
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewPersonalSecurity,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewResourceSecurity,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}