package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.security.containers.SecurityGroup;

class ResourceBasedSecurityGroupSecurityController extends
        ResourceBasedAbstractSecurityGroupSecurityController
{

    public Class getContainerClass()
    {
        return SecurityGroup.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateSecurityGroup, getContainerClass(),
            SecurityGroup.class);
        creationInfo.securityFlag = RESOURCE.CanEditDefaultResourceSecurity;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass());
        deletionEntry.securityFlag = RESOURCE.CanEditDefaultResourceSecurity;
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("description", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("securityGroups", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        /*
         * Changes all the FieldMappingEntry to use the appropriate flag
         */
        Integer[] canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanEditDefaultResourceSecurity,
        };
        Iterator it = mappingEntryMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry mapEntry = (Entry) it.next();
            fieldInfo = (FieldMappingEntry) mapEntry.getValue();
            fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
            mapEntry.setValue(fieldInfo);
        }
    }

}
