package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

class CreationOperationMappingEntryWithSuperFlag extends
        CreationOperationMappingEntry
{
    final Integer superFlag;

    final CreationOperationMappingEntry entry;

    CreationOperationMappingEntryWithSuperFlag(Integer superFlag,
            CreationOperationMappingEntry entry)
    {
        super(entry.name, entry.container, entry.childrenClass);
        this.superFlag = superFlag;
        this.entry = entry;
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        SecurityValidationResult result = SecurityControllerUtil
            .calculateFlagResult(superFlag, securityFlags, getErrorMessage(), rpmObject.getClass());
        if (result.type == SecurityValidationResult.TRUE)
        {
            return result; // super flag is true, then allow it
        }
        return entry.canPerform(rpmObject, securityFlags, messageContext);
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry#buildCreationOperationSecurityDescriptors(java.util.Map, com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
     */
    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        SecurityValidationResult result = SecurityControllerUtil
            .calculateFlagResult(superFlag, combinedSecurityFlags,
                getErrorMessage(), objectInfo.rpmObject.getClass());
        if (result.type == SecurityValidationResult.TRUE)
        {
            Map map = new HashMap();
            entry.buildCreationOperationSecurityDescriptors(map, objectInfo,
                combinedSecurityFlags, messageContext);
            if (!map.isEmpty())
            {
                Iterator it = map.values().iterator();
                while (it.hasNext())
                {
                    OperationSecurityDescriptor operationSecurityDescr = (OperationSecurityDescriptor) it
                        .next();
                    operationSecurityDescr.setCanPerform(true);
                }
                creationOperationSecurityDescrs.putAll(map);
            }
        }
        else
        {
            entry.buildCreationOperationSecurityDescriptors(
                creationOperationSecurityDescrs, objectInfo,
                combinedSecurityFlags, messageContext);
        }

    }
}
