package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.types.PublishedType;

class RoleBasedAbstractDocumentSecurityController extends
        RoleBasedGenericDocumentSecurityController
{

    public Class getContainerClass()
    {
        return AbstractDocument.class;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry("keywords",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewKeywords,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditKeywords,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("minorRevisionNumber",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry("summary",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDocumentSummary,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDocumentSummary,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("majorRevisionNumber",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("elementNumber", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditState,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("published", getContainerClass())
        {
            public SecurityValidationResult canWrite(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (canWriteSecurityFlags != null)
                {
                    AbstractDocument abstractDocument = (AbstractDocument) rpmObject;
                    PublishedType type = SecurityControllerUtil
                        .loadAbstractDocumentPublishedType(abstractDocument,
                            messageContext);
                    if (type != PublishedType.ToTemplate
                            && abstractDocument.getPublished() == PublishedType.ToTemplate)
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            ROLE.CanPublishDocumentsToTemplate, securityFlags,
                            getEditErrorMessage(), rpmObject.getClass());
                    }

                    return SecurityValidationResult.TRUE_RESULT;
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("referenceNumber",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewReferenceNumber,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditReferenceNumber,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("linkingStatus", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
