package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;

class RoleBasedAbstractFinancialSecurityController extends
        RoleBasedContainerSecurityController
{
    public Class getContainerClass()
    {
        return AbstractFinancial.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);

        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new RoleBasedCreateBudgetCostRevenueChargeCodeMappingEntry(
            ControllerSecuredOperations.CreateChargeCode,
            getContainerClass(), ChargeCode.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        final Map canReadSecurityFlagsMap_Actual = new HashMap();
        final Map canWriteSecurityFlagsMap_Actual = new HashMap();

        //Benefits
        Integer[] flags_Actual = new Integer[]
        {
            ROLE.CanViewProfitBudgetCostRevenue,
        };
        canReadSecurityFlagsMap_Actual.put(FinancialCategory._Benefit,
            flags_Actual);
        flags_Actual = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsBenefitCapitalActual,
        };
        canWriteSecurityFlagsMap_Actual.put(FinancialCategory._Benefit,
            flags_Actual);

        //Expense & Capital
        flags_Actual = new Integer[]
        {
            ROLE.CanViewProfitBudgetCostRevenue,
        };
        canReadSecurityFlagsMap_Actual.put(FinancialCategory._Expense,
            flags_Actual);
        canReadSecurityFlagsMap_Actual.put(FinancialCategory._Capital,
            flags_Actual);
        flags_Actual = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsExpenseCapitalActual,
        };
        canWriteSecurityFlagsMap_Actual.put(FinancialCategory._Expense,
            flags_Actual);
        canWriteSecurityFlagsMap_Actual.put(FinancialCategory._Capital,
            flags_Actual);

        final Map canReadSecurityFlagsMap_Estimate = new HashMap();
        final Map canWriteSecurityFlagsMap_Estimate = new HashMap();

        //Benefits
        Integer[] flags_Estimate = new Integer[]
        {
            ROLE.CanViewProfitBudgetCostRevenue,
        };
        canReadSecurityFlagsMap_Estimate.put(FinancialCategory._Benefit,
            flags_Estimate);
        flags_Estimate = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsBenefitCapitalEstimate,
        };
        canWriteSecurityFlagsMap_Estimate.put(FinancialCategory._Benefit,
            flags_Estimate);

        //Expense & Capital
        flags_Estimate = new Integer[]
        {
            ROLE.CanViewProfitBudgetCostRevenue,
        };
        canReadSecurityFlagsMap_Estimate.put(FinancialCategory._Expense,
            flags_Estimate);
        canReadSecurityFlagsMap_Estimate.put(FinancialCategory._Capital,
            flags_Estimate);
        flags_Estimate = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsExpenseCapitalEstimate,
        };
        canWriteSecurityFlagsMap_Estimate.put(FinancialCategory._Expense,
            flags_Estimate);
        canWriteSecurityFlagsMap_Estimate.put(FinancialCategory._Capital,
            flags_Estimate);

        Map canReadSecurityFlagsMap_Estimate_2 = new HashMap();
        Integer[] flags_Estimate_2 = new Integer[]
        {
            ROLE.CanViewEarnedValueBudgetCostRevenue,
        };
        canReadSecurityFlagsMap_Estimate_2.put(FinancialCategory._Benefit,
            flags_Estimate_2);
        canReadSecurityFlagsMap_Estimate_2.put(FinancialCategory._Expense,
            flags_Estimate_2);
        canReadSecurityFlagsMap_Estimate_2.put(FinancialCategory._Capital,
            flags_Estimate_2);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("billed", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "baselinePreviousTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "actualTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("revRecType", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("estimate",
            getContainerClass(), canReadSecurityFlagsMap_Estimate,
            canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry("timePhased",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------        
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("EACVariance",
            getContainerClass(), canReadSecurityFlagsMap_Estimate_2,
            canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "GLTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("EAC",
            getContainerClass(), canReadSecurityFlagsMap_Estimate_2,
            canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        Map canReadSecurityFlagsMap_2 = new HashMap();
        Map canWriteSecurityFlagsMap_2 = new HashMap();

        //Benefits
        Integer[] flags_2 = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsBenefits,
        };
        canReadSecurityFlagsMap_2.put(FinancialCategory._Benefit, flags_2);
        flags_2 = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsBenefits,
        };
        canWriteSecurityFlagsMap_2.put(FinancialCategory._Benefit, flags_2);

        //Expense & Capital
        flags_2 = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsExpenseCapital,
        };
        canReadSecurityFlagsMap_2.put(FinancialCategory._Expense, flags_2);
        canReadSecurityFlagsMap_2.put(FinancialCategory._Capital, flags_2);
        flags_2 = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsExpenseCapital,
        };
        canWriteSecurityFlagsMap_2.put(FinancialCategory._Expense, flags_2);
        canWriteSecurityFlagsMap_2.put(FinancialCategory._Capital, flags_2);

        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("chargeCode",
            getContainerClass(), canReadSecurityFlagsMap_2,
            canWriteSecurityFlagsMap_2);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry(
            "baselineInitial", getContainerClass(),
            canReadSecurityFlagsMap_Estimate, canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "baselineTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry(
            "probabilityAmount", getContainerClass(),
            canReadSecurityFlagsMap_Estimate, canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("GL", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry(
            "probabilityPercent", getContainerClass(),
            canReadSecurityFlagsMap_Estimate, canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry(
            "baselinePrevious", getContainerClass(),
            canReadSecurityFlagsMap_Actual, canWriteSecurityFlagsMap_Actual);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("ETC",
            getContainerClass(), canReadSecurityFlagsMap_Estimate_2,
            canWriteSecurityFlagsMap_Estimate);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("remaining", getContainerClass())
        {
            final FieldMappingEntry fieldInfo_Estimate = new RoleBasedAbstractFinancialMappingEntry(
                "remaining", getContainerClass(),
                canReadSecurityFlagsMap_Estimate,
                canWriteSecurityFlagsMap_Estimate);

            final FieldMappingEntry fieldInfo_Actual = new RoleBasedAbstractFinancialMappingEntry(
                "remaining", getContainerClass(),
                canReadSecurityFlagsMap_Actual, canWriteSecurityFlagsMap_Actual);

            public SecurityValidationResult canRead(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = fieldInfo_Estimate.canRead(
                    rpmObject, securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
                return fieldInfo_Actual.canRead(rpmObject, securityFlags,
                    messageContext);
            }

            public SecurityValidationResult canWrite(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = fieldInfo_Estimate.canWrite(
                    rpmObject, securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
                return fieldInfo_Actual.canWrite(rpmObject, securityFlags,
                    messageContext);
            }
        };

        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------       
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("baseline",
            getContainerClass(), canReadSecurityFlagsMap_Actual,
            canWriteSecurityFlagsMap_Actual);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialMappingEntry("actual",
            getContainerClass(), canReadSecurityFlagsMap_Actual,
            canWriteSecurityFlagsMap_Actual);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "estimateTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "billedTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------        
        fieldInfo = new FieldMappingEntry("baselineVariance",
            getContainerClass())
        {
            final FieldMappingEntry fieldInfo_Estimate = new RoleBasedAbstractFinancialMappingEntry(
                "baselineVariance", getContainerClass(),
                canReadSecurityFlagsMap_Estimate,
                canWriteSecurityFlagsMap_Estimate);

            final FieldMappingEntry fieldInfo_Actual = new RoleBasedAbstractFinancialMappingEntry(
                "baselineVariance", getContainerClass(),
                canReadSecurityFlagsMap_Actual, canWriteSecurityFlagsMap_Actual);

            /* (non-Javadoc)
             * @see com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry#canRead(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
             */
            public SecurityValidationResult canRead(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = fieldInfo_Estimate.canRead(
                    rpmObject, securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
                return fieldInfo_Actual.canRead(rpmObject, securityFlags,
                    messageContext);
            }

            /* (non-Javadoc)
             * @see com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry#canWrite(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
             */
            public SecurityValidationResult canWrite(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = fieldInfo_Estimate.canWrite(
                    rpmObject, securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
                return fieldInfo_Actual.canWrite(rpmObject, securityFlags,
                    messageContext);
            }

        };

        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            "baselineInitialTimePhased", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsTimePhasing,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsTimePhasing,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        OperationMappingEntry operationInfo = new OperationMappingEntry(
            SecuredOperations.ViewBudgetCostRevenueFinancialsTotal,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanViewBudgetCostRevenueFinancialsTotal;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.AcceptRejectAssignmentEETCWBSBudgetEETC,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanAcceptRejectAssignmentEETCWBSBudgetEETC;

        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

}
