package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;

class RoleBasedAbstractScopeSecurityController extends
        RoleBasedAbstractAggregateScopeSecurityController
{

    public Class getContainerClass()
    {
        return AbstractScope.class;
    }

    protected void doGenerateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------

        creationInfo = new RoleBasedCreateScopeElementChildMappingEntry(
            ControllerSecuredOperations.CreateScopeElement,
            getContainerClass(), ScopeElement.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAssignedScorecard,
            getContainerClass(), AssignedScorecard.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveScorecards;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateDuplicates,
            getContainerClass(), Duplicate.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveDuplicates;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("children", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanBreakdownScopeElements,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("task", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAssignedScorecard,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assetCode", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesAvailable",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("impact", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewQuantificationImpact,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditQuantificationImpact,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("duplicates", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDuplicates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDuplicates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("orderOfMagnitudeCostBenefits",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewOrderOfMagnitudeCostBenefits,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditOrderOfMagnitudeCostBenefits,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentFolder", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedByName", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewApprovedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditApprovedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalReferenceNumber",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewExternalReference,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditExternalReference,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("published", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("duplicateNumber",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDuplicateNumber,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDuplicateNumbers,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedAbstractScopeMappingEntry("state",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditState,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedByDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewApprovedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditApprovedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesRunning",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("clientCostCenter",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rank", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewRank,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditRank,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("targetDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewTargetDate,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditTargetDate,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDatesApproved,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDatesApproved,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultScope", getContainerClass());
        fieldInfo.canReadSecurityFlags = null;
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanSetAsOrCancelDefault,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("asset", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void doGenerateOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ChangeElementType, getContainerClass());
        operationInfo.securityFlag = ROLE.CanChangeScopeElementsItemTypes;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.AssignScoreCard, getContainerClass());
        operationInfo.securityFlag = ROLE.CanAddRemoveScorecards;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    // Adds the CreationOperation descriptors of the scope's DocumentFolder 
    // to the scope's CreationOperation descriptors because the WebUI does not
    // receive the scope's DocumentFolder when it is empty but still requires security
    protected Map buildCreationOperationSecurityDescriptors(ReferencedObjectInfo objectInfo, CombinedSecurityFlags combinedSecurityFlags, MessageContext messageContext)
    {
        Map map = super.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        RoleBasedDocumentFolderSecurityController controller =
            (RoleBasedDocumentFolderSecurityController) getContainerSecurityController(
                DocumentFolder.class);
        if (controller != null)
        {
            map.putAll(controller.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext));
        }
        return map;
    }
    
}
