package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;

class RoleBasedAvailableWorkflowProcessSecurityController extends
        RoleBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return AvailableWorkflowProcess.class;
    }

    public SecurityValidationResult canCreate(RPMObject parent,
            RPMObject object, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        //when a workflow project gets associated, it will be wrapped in
        //an AvailableWorkflowProcess. So the only way to create an AvailableWorkflowProcess
        //is through associating it with a WorkElement, which is controlled by this flag
        return SecurityValidationResult.UNDEFINED_RESULT;
    }    

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcess",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWorkflowElementAssociations,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditWorkflowElementsAssociations,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultFlag",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWorkflowElementAssociations,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditWorkflowElementsAssociations,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass());
        viewEntry.securityFlag = ROLE.CanViewWorkflowElementAssociations;
        return viewEntry;
    }

}
