package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

class RoleBasedClientCostCenterSecurityController extends
        RoleBasedGenericClientSecurityController
{

    public Class getContainerClass()
    {
        return ClientCostCenter.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAttributeAssignment,
            getContainerClass(), AttributeAssignment.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveAttributes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        //      --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateDocumentsFolders,
            getContainerClass(), DocumentFolder.class);
        creationInfo.securityFlag = ROLE.CanCreateDocumentsFolders;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("statusUpdates",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalID", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentFolder",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultFlag", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("customFieldAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("attributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rtfAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map customFieldSecurityDescrs = new HashMap();
        FieldSecurityDescriptor customFieldSecurityDescr = null;

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
            customFieldSecurityDescr);

        return customFieldSecurityDescrs;
    }

}
