package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

class RoleBasedCreateChargeCodeCategoryChargeCodeMappingEntry extends
        CreationOperationMappingEntry
{

    public RoleBasedCreateChargeCodeCategoryChargeCodeMappingEntry(
            String operationName, Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        FinancialCategory category = ((ChargeCode) rpmObject)
            .getFinancialCategory();
        if (category != null)
        {
            if (category.getValue().equals(FinancialCategory._Benefit))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    ROLE.CanAddRemoveBudgetCostRevenueFinancialsBenefits,
                    securityFlags, "create {0} of type "
                            + category.getValue(), rpmObject.getClass());
            }
            else if (category.getValue().equals(
                FinancialCategory._Expense)
                    || category.getValue().equals(
                        FinancialCategory._Capital))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    ROLE.CanAddRemoveBudgetCostRevenueFinancialsExpenseCapital,
                    securityFlags, "create {0} of type "
                            + category.getValue(), rpmObject.getClass());
            }
        }

        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateBudgetCostRevenueBenefits);
        operationSecurityDescr
            .setCanPerform(SecurityControllerUtil.calculateFlagValue(
                ROLE.CanAddRemoveBudgetCostRevenueFinancialsBenefits,
                combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
        //--------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateBudgetCostRevenueExpenseCapital);
        operationSecurityDescr
            .setCanPerform(SecurityControllerUtil.calculateFlagValue(
                ROLE.CanAddRemoveBudgetCostRevenueFinancialsExpenseCapital,
                combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
    }
}
