package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

class RoleBasedCreateProjectResourceTaskAssignmentMappingEntry extends
        CreationOperationMappingEntry
{

    public RoleBasedCreateProjectResourceTaskAssignmentMappingEntry(
            String operationName, Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;
        if (assignment.getResource() != null)
        {
            return SecurityControllerUtil.calculateFlagResult(
                ROLE.CanAssignOrAffectResourceToProjectWithEffort,
                securityFlags, getErrorMessage(), rpmObject.getClass());
        }
        return SecurityControllerUtil.calculateFlagResult(
            ROLE.CanAssignOrAffectProfileToProject, securityFlags,
            getErrorMessage(), rpmObject.getClass());
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateResourceTaskAssignment);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil
            .calculateFlagValue(
                ROLE.CanAssignOrAffectResourceToProjectWithEffort,
                combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateProfileTaskAssignment);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanAssignOrAffectProfileToProject,
                combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
    }
}
