package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

class RoleBasedCreateWorkElementResourceTaskAssignmentMappingEntry extends
        CreationOperationMappingEntry
{

    public RoleBasedCreateWorkElementResourceTaskAssignmentMappingEntry(
            String operationName, Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;
        if (assignment.getResource() != null)
        {
            if (assignment.getTaskAssignment() != null)
            {
                return SecurityControllerUtil.calculateFlagResult(
                    ROLE.CanAssignResource, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
            return SecurityControllerUtil.calculateFlagResult(
                ROLE.CanAssignParticipant, securityFlags,
                getErrorMessage(), rpmObject.getClass());
        }
        return SecurityControllerUtil.calculateFlagResult(ROLE.CanAssignProfile,
            securityFlags, getErrorMessage(), rpmObject.getClass());
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateResourceTaskAssignment);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanAssignResource, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
        //--------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateParticipantTaskAssignment);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanAssignParticipant, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
        //--------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr
            .setName(SecuredOperations.CreateProfileTaskAssignment);
        operationSecurityDescr.setCanPerform(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanAssignProfile, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr
            .getName(), operationSecurityDescr);
    }
}
