/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

/**
 * @author Yasser Lulu
 * 
 */
class RoleBasedDocumentElementDeletedMappingEntry extends FieldMappingEntry
{
    public RoleBasedDocumentElementDeletedMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        return SecurityValidationResult.TRUE_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null)
        {
            String resourceID = SecurityControllerUtil
                .loadResourceIdWithDocument(messageContext,
                    (DocumentElement) rpmObject);
            if (resourceID != null)
            {
                if (messageContext.getUser().getID().equals(resourceID))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        canWriteSecurityFlags[0], securityFlags,
                        getEditErrorMessage(), rpmObject.getClass());
                }
            }
            return SecurityControllerUtil.calculateFlagResult(
                canWriteSecurityFlags[1], securityFlags, getEditErrorMessage(),
                rpmObject.getClass());
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}
