package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CheckOutOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;

class RoleBasedDocumentElementSecurityController extends
        RoleBasedContainerSecurityControllerWithSuperFlag
{

    public Class getContainerClass()
    {
        return DocumentElement.class;
    }
    
    protected final Integer getSuperFlag()
    {
        return ROLE.AllDocumentPermissionsGranted;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);

        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("templateSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("containingProject",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("resourceAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalParent", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanRename,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedResources",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsSortingRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("parentSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("children", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void doGenerateOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateOperationMappingEntries(mappingEntryMap);

        OperationMappingEntry operationInfo = new OperationMappingEntry(
            SecuredOperations.SaveAs, getContainerClass());
        operationInfo.securityFlag = ROLE.CanSaveAs;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CheckOutOrCheckOutTo, getContainerClass());
        operationInfo.securityFlag = ROLE.CanCheckOutOrCheckOutTo;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected CheckOutOperationMappingEntry doGenerateCheckOutOperationMappingEntry()
    {
        CheckOutOperationMappingEntry checkOutEntry = new CheckOutOperationMappingEntry(
            getContainerClass());
        checkOutEntry.securityFlag = ROLE.CanCheckOutOrCheckOutTo;
        return checkOutEntry;
    }

}
