package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

class RoleBasedGenericProjectSecurityController extends
        RoleBasedAggregateNodeSecurityController
{

    //The excluded field entries that should not be subjected to description flag security
    private static final String[] PROJECT_DESCR_EXCLUDED_FIELD_NAMES = new String[]
    {
            "name", "scopeElement", "documentFolder", "portfolio", "calendar"
    };

    public Class getContainerClass()
    {
        return GenericProject.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAssignedScorecard, getContainerClass(),
            AssignedScorecard.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveProjectScorecard;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

//        // --------------------------------------------------------
//        creationInfo = new CreationOperationMappingEntry(
//            SecuredOperations.CreateCalendarExceptions, getContainerClass(),
//            CalendarException.class);
//        creationInfo.securityFlag = ROLE.CanCreateCalendarExceptions;
//        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        creationInfo = new RoleBasedCreateProjectResourceTaskAssignmentMappingEntry(
            ControllerSecuredOperations.CreateResourceTaskAssignment,
            getContainerClass(), ResourceTaskAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        //      --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateProjectOrganizationalAssignment,
            getContainerClass(), ProjectOrganizationalAssignment.class);
        creationInfo.securityFlag = ROLE.CanEditAssignedOrganizations;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        fieldInfo = new RoleBasedGenericProjectMappingEntry("published",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanPublishSecurity,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("parent", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanMoveProjects,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defectScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defectAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calendar", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewProjectCalendar,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("riskAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("requirementAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowRoleMappings",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWorkflowRoleMapping,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("portfolio", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedScorecards",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewProjectScorecard,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveProjectScorecard,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("deliverableAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actionScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("riskScorecard", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("earnedValuesCalculationState",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("probabilityPercent",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewPursuit,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditPursuit,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultSteps", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("issueAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("changeRequestScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("currency", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewCurrency,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("issueScorecard", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("taskAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("requirementScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("serviceRequestScorecard",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("projectCostCenter",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultProject", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanSetAsDefaultProject,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actionAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("changeRequestAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("serviceRequestAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("opportunity", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("organizationalAssignments",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAssignedOrganizations,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignmentType", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAssignmentType,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditAssignmentType,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        /* Adds additional security for the CanViewProjectDescription flag
         * because it has priority over the other flags.
         * We add a parent entry that will call the child entry when
         * necessary.
         */

        //REMOVE -temporarily- the excluded field entries because it should not be subjected to description flag
        FieldMappingEntry[] fields = new FieldMappingEntry[PROJECT_DESCR_EXCLUDED_FIELD_NAMES.length];
        for (int i = 0; i < PROJECT_DESCR_EXCLUDED_FIELD_NAMES.length; ++i)
        {
            fields[i] = (FieldMappingEntry) mappingEntryMap
                .remove(PROJECT_DESCR_EXCLUDED_FIELD_NAMES[i]);
        }

        try
        {
            Integer[] flags = new Integer[]
            {
                ROLE.CanViewProjectDescription,
            };
            Iterator it = mappingEntryMap.entrySet().iterator();
            while (it.hasNext())
            {
                Entry entry = (Entry) it.next();
                fieldInfo = new RoleBasedProjectDescriptionMappingEntry(
                    (String) entry.getKey(), getContainerClass(),
                    (FieldMappingEntry) entry.getValue());
                fieldInfo.canReadSecurityFlags = flags;
                entry.setValue(fieldInfo);
            }
        }
        finally
        {
            //RESTORE excluded fields
            for (int i = 0; i < fields.length; ++i)
            {
                if (fields[i] != null)
                {
                    mappingEntryMap.put(fields[i].name, fields[i]);
                }
            }
        }
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.MergeProjects, getContainerClass());
        operationInfo.securityFlag = ROLE.CanMergeProjects;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ImportExport, getContainerClass());
        operationInfo.securityFlag = ROLE.CanImportExport;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.SendToArchive, getContainerClass());
        operationInfo.securityFlag = ROLE.CanSendToArchive;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishBubble, getContainerClass());
        operationInfo.securityFlag = ROLE.CanPublishBubble;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.DecommissionProject, getContainerClass());
        operationInfo.securityFlag = ROLE.CanProjectDecommission;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.LevelWithIBMRationalPortfolioManager,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanLevelWithIBMRationalPortfolioManager;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CopyCalculatedResults, getContainerClass());
        operationInfo.securityFlag = ROLE.CanCopyCalculatedResults;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CopyDatesAndSchedulesCommitAffectedResources,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanCopyDatesAndSchedulesCommitAffectedResources;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(SecuredOperations.RollupEV,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanRollupEV;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.SaveBaselineEstDatesWBSFinancialsAndAffectedAssignments,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanSaveBaselineEstDatesWSFinancialsAndAffectedAssignments;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.SaveBaselineWBSDatesWBSFinancialsAndAssignments,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanSaveBaselineWBSDatesWBSFinancialsAndAssignments;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.SaveBaselineCrossCharges, getContainerClass());
        operationInfo.securityFlag = ROLE.CanSaveBaselineCrossCharges;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.SaveBaselineUnlimited, getContainerClass());
        operationInfo.securityFlag = ROLE.CanBaselineUnlimited;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.AssignScoreCard, getContainerClass());//for AssignProjectScoreCard
        operationInfo.securityFlag = ROLE.CanAddRemoveProjectScorecard;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map rtfSecurityDescrs = new HashMap();
        FieldSecurityDescriptor rtfSecurityDescr = null;

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF1);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs1, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs1, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[1], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF2);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs2, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs2, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[2], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF3);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs3, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs3, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[3], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF4);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs4, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs4, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[4], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF5);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs5, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs5, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[5], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF6);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs6, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs6, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[6], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF7);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs7, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs7, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[7], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF8);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs8, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs8, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[8], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF9);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs9, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs9, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[9], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF10);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs10, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs10, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[10], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF11);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs11, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs11, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[11], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF12);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs12, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs12, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[12], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF13);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs13, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs13, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[13], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF14);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs14, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs14, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[14], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF15);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs15, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs15, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[15], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF16);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs16, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs16, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[16], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF17);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs17, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs17, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[17], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF18);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs18, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs18, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[18], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF19);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs19, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs19, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[19], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF20);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewProjectRTFs20, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditProjectRTFs20, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[20], rtfSecurityDescr);

        return rtfSecurityDescrs;
    }

}
