package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkProduct;

class RoleBasedRtfAssignmentSecurityController extends
        RoleBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return RtfAssignment.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {

        // --------------------------------------------------------
        // Readonly field
        mappingEntryMap.put("rtf", new FieldMappingEntry("rtf",
            getContainerClass()));

        // --------------------------------------------------------
        // This mapping entry uses its own map of arrays, never use the default arrays.
        RoleBasedRtfAssignmentMappingEntry fieldInfo = new RoleBasedRtfAssignmentMappingEntry(
            "value", getContainerClass());
        fieldInfo.canReadSecurityFlags = null;
        fieldInfo.canWriteSecurityFlags = null;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        // GenericProject
        Integer[] flags = new Integer[]
        {
                ROLE.CanViewProjectRTFs1,
                ROLE.CanViewProjectRTFs2,
                ROLE.CanViewProjectRTFs3,
                ROLE.CanViewProjectRTFs4,
                ROLE.CanViewProjectRTFs5,
                ROLE.CanViewProjectRTFs6,
                ROLE.CanViewProjectRTFs7,
                ROLE.CanViewProjectRTFs8,
                ROLE.CanViewProjectRTFs9,
                ROLE.CanViewProjectRTFs10,
                ROLE.CanViewProjectRTFs11,
                ROLE.CanViewProjectRTFs12,
                ROLE.CanViewProjectRTFs13,
                ROLE.CanViewProjectRTFs14,
                ROLE.CanViewProjectRTFs15,
                ROLE.CanViewProjectRTFs16,
                ROLE.CanViewProjectRTFs17,
                ROLE.CanViewProjectRTFs18,
                ROLE.CanViewProjectRTFs19,
                ROLE.CanViewProjectRTFs20
        };
        fieldInfo.canReadSecurityFlagsMap.put(GenericProject.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditProjectRTFs1,
                ROLE.CanEditProjectRTFs2,
                ROLE.CanEditProjectRTFs3,
                ROLE.CanEditProjectRTFs4,
                ROLE.CanEditProjectRTFs5,
                ROLE.CanEditProjectRTFs6,
                ROLE.CanEditProjectRTFs7,
                ROLE.CanEditProjectRTFs8,
                ROLE.CanEditProjectRTFs9,
                ROLE.CanEditProjectRTFs10,
                ROLE.CanEditProjectRTFs11,
                ROLE.CanEditProjectRTFs12,
                ROLE.CanEditProjectRTFs13,
                ROLE.CanEditProjectRTFs14,
                ROLE.CanEditProjectRTFs15,
                ROLE.CanEditProjectRTFs16,
                ROLE.CanEditProjectRTFs17,
                ROLE.CanEditProjectRTFs18,
                ROLE.CanEditProjectRTFs19,
                ROLE.CanEditProjectRTFs20
        };
        fieldInfo.canWriteSecurityFlagsMap.put(GenericProject.class, flags);

        // --------------------------------------------------------
        // Action
        flags = new Integer[]
        {
                ROLE.CanViewActionRTFs1,
                ROLE.CanViewActionRTFs2,
                ROLE.CanViewActionRTFs3,
                ROLE.CanViewActionRTFs4,
                ROLE.CanViewActionRTFs5,
                ROLE.CanViewActionRTFs6,
                ROLE.CanViewActionRTFs7,
                ROLE.CanViewActionRTFs8,
                ROLE.CanViewActionRTFs9,
                ROLE.CanViewActionRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Action.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditActionRTFs1,
                ROLE.CanEditActionRTFs2,
                ROLE.CanEditActionRTFs3,
                ROLE.CanEditActionRTFs4,
                ROLE.CanEditActionRTFs5,
                ROLE.CanEditActionRTFs6,
                ROLE.CanEditActionRTFs7,
                ROLE.CanEditActionRTFs8,
                ROLE.CanEditActionRTFs9,
                ROLE.CanEditActionRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Action.class, flags);
        // --------------------------------------------------------
        // ChangeRequest
        flags = new Integer[]
        {
                ROLE.CanViewChangeRequestRTFs1,
                ROLE.CanViewChangeRequestRTFs2,
                ROLE.CanViewChangeRequestRTFs3,
                ROLE.CanViewChangeRequestRTFs4,
                ROLE.CanViewChangeRequestRTFs5,
                ROLE.CanViewChangeRequestRTFs6,
                ROLE.CanViewChangeRequestRTFs7,
                ROLE.CanViewChangeRequestRTFs8,
                ROLE.CanViewChangeRequestRTFs9,
                ROLE.CanViewChangeRequestRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(ChangeRequest.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditChangeRequestRTFs1,
                ROLE.CanEditChangeRequestRTFs2,
                ROLE.CanEditChangeRequestRTFs3,
                ROLE.CanEditChangeRequestRTFs4,
                ROLE.CanEditChangeRequestRTFs5,
                ROLE.CanEditChangeRequestRTFs6,
                ROLE.CanEditChangeRequestRTFs7,
                ROLE.CanEditChangeRequestRTFs8,
                ROLE.CanEditChangeRequestRTFs9,
                ROLE.CanEditChangeRequestRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(ChangeRequest.class, flags);

        // --------------------------------------------------------
        // Defect
        flags = new Integer[]
        {
                ROLE.CanViewDefectRTFs1,
                ROLE.CanViewDefectRTFs2,
                ROLE.CanViewDefectRTFs3,
                ROLE.CanViewDefectRTFs4,
                ROLE.CanViewDefectRTFs5,
                ROLE.CanViewDefectRTFs6,
                ROLE.CanViewDefectRTFs7,
                ROLE.CanViewDefectRTFs8,
                ROLE.CanViewDefectRTFs9,
                ROLE.CanViewDefectRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Defect.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditDefectRTFs1,
                ROLE.CanEditDefectRTFs2,
                ROLE.CanEditDefectRTFs3,
                ROLE.CanEditDefectRTFs4,
                ROLE.CanEditDefectRTFs5,
                ROLE.CanEditDefectRTFs6,
                ROLE.CanEditDefectRTFs7,
                ROLE.CanEditDefectRTFs8,
                ROLE.CanEditDefectRTFs9,
                ROLE.CanEditDefectRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Defect.class, flags);

        // --------------------------------------------------------
        // Deliverable
        flags = new Integer[]
        {
                ROLE.CanViewDeliverableRTFs1,
                ROLE.CanViewDeliverableRTFs2,
                ROLE.CanViewDeliverableRTFs3,
                ROLE.CanViewDeliverableRTFs4,
                ROLE.CanViewDeliverableRTFs5,
                ROLE.CanViewDeliverableRTFs6,
                ROLE.CanViewDeliverableRTFs7,
                ROLE.CanViewDeliverableRTFs8,
                ROLE.CanViewDeliverableRTFs9,
                ROLE.CanViewDeliverableRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Deliverable.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditDeliverableRTFs1,
                ROLE.CanEditDeliverableRTFs2,
                ROLE.CanEditDeliverableRTFs3,
                ROLE.CanEditDeliverableRTFs4,
                ROLE.CanEditDeliverableRTFs5,
                ROLE.CanEditDeliverableRTFs6,
                ROLE.CanEditDeliverableRTFs7,
                ROLE.CanEditDeliverableRTFs8,
                ROLE.CanEditDeliverableRTFs9,
                ROLE.CanEditDeliverableRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Deliverable.class, flags);

        // --------------------------------------------------------
        // Issue
        flags = new Integer[]
        {
                ROLE.CanViewIssueRTFs1,
                ROLE.CanViewIssueRTFs2,
                ROLE.CanViewIssueRTFs3,
                ROLE.CanViewIssueRTFs4,
                ROLE.CanViewIssueRTFs5,
                ROLE.CanViewIssueRTFs6,
                ROLE.CanViewIssueRTFs7,
                ROLE.CanViewIssueRTFs8,
                ROLE.CanViewIssueRTFs9,
                ROLE.CanViewIssueRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Issue.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditIssueRTFs1,
                ROLE.CanEditIssueRTFs2,
                ROLE.CanEditIssueRTFs3,
                ROLE.CanEditIssueRTFs4,
                ROLE.CanEditIssueRTFs5,
                ROLE.CanEditIssueRTFs6,
                ROLE.CanEditIssueRTFs7,
                ROLE.CanEditIssueRTFs8,
                ROLE.CanEditIssueRTFs9,
                ROLE.CanEditIssueRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Issue.class, flags);

        // --------------------------------------------------------
        // Milestone
        flags = new Integer[]
        {
                ROLE.CanViewMilestoneRTFs1,
                ROLE.CanViewMilestoneRTFs2,
                ROLE.CanViewMilestoneRTFs3,
                ROLE.CanViewMilestoneRTFs4,
                ROLE.CanViewMilestoneRTFs5,
                ROLE.CanViewMilestoneRTFs6,
                ROLE.CanViewMilestoneRTFs7,
                ROLE.CanViewMilestoneRTFs8,
                ROLE.CanViewMilestoneRTFs9,
                ROLE.CanViewMilestoneRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Milestone.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditMilestoneRTFs1,
                ROLE.CanEditMilestoneRTFs2,
                ROLE.CanEditMilestoneRTFs3,
                ROLE.CanEditMilestoneRTFs4,
                ROLE.CanEditMilestoneRTFs5,
                ROLE.CanEditMilestoneRTFs6,
                ROLE.CanEditMilestoneRTFs7,
                ROLE.CanEditMilestoneRTFs8,
                ROLE.CanEditMilestoneRTFs9,
                ROLE.CanEditMilestoneRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Milestone.class, flags);

        // --------------------------------------------------------
        // Requirement
        flags = new Integer[]
        {
                ROLE.CanViewRequirementRTFs1,
                ROLE.CanViewRequirementRTFs2,
                ROLE.CanViewRequirementRTFs3,
                ROLE.CanViewRequirementRTFs4,
                ROLE.CanViewRequirementRTFs5,
                ROLE.CanViewRequirementRTFs6,
                ROLE.CanViewRequirementRTFs7,
                ROLE.CanViewRequirementRTFs8,
                ROLE.CanViewRequirementRTFs9,
                ROLE.CanViewRequirementRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Requirement.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditRequirementRTFs1,
                ROLE.CanEditRequirementRTFs2,
                ROLE.CanEditRequirementRTFs3,
                ROLE.CanEditRequirementRTFs4,
                ROLE.CanEditRequirementRTFs5,
                ROLE.CanEditRequirementRTFs6,
                ROLE.CanEditRequirementRTFs7,
                ROLE.CanEditRequirementRTFs8,
                ROLE.CanEditRequirementRTFs9,
                ROLE.CanEditRequirementRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Requirement.class, flags);

        // --------------------------------------------------------
        // Risk
        flags = new Integer[]
        {
                ROLE.CanViewRiskRTFs1, ROLE.CanViewRiskRTFs2,
                ROLE.CanViewRiskRTFs3, ROLE.CanViewRiskRTFs4,
                ROLE.CanViewRiskRTFs5, ROLE.CanViewRiskRTFs6,
                ROLE.CanViewRiskRTFs7, ROLE.CanViewRiskRTFs8,
                ROLE.CanViewRiskRTFs9, ROLE.CanViewRiskRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Risk.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditRiskRTFs1, ROLE.CanEditRiskRTFs2,
                ROLE.CanEditRiskRTFs3, ROLE.CanEditRiskRTFs4,
                ROLE.CanEditRiskRTFs5, ROLE.CanEditRiskRTFs6,
                ROLE.CanEditRiskRTFs7, ROLE.CanEditRiskRTFs8,
                ROLE.CanEditRiskRTFs9, ROLE.CanEditRiskRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Risk.class, flags);

        // --------------------------------------------------------
        // ServiceRequest
        flags = new Integer[]
        {
                ROLE.CanViewServiceRequestRTFs1,
                ROLE.CanViewServiceRequestRTFs2,
                ROLE.CanViewServiceRequestRTFs3,
                ROLE.CanViewServiceRequestRTFs4,
                ROLE.CanViewServiceRequestRTFs5,
                ROLE.CanViewServiceRequestRTFs6,
                ROLE.CanViewServiceRequestRTFs7,
                ROLE.CanViewServiceRequestRTFs8,
                ROLE.CanViewServiceRequestRTFs9,
                ROLE.CanViewServiceRequestRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(ServiceRequest.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditServiceRequestRTFs1,
                ROLE.CanEditServiceRequestRTFs2,
                ROLE.CanEditServiceRequestRTFs3,
                ROLE.CanEditServiceRequestRTFs4,
                ROLE.CanEditServiceRequestRTFs5,
                ROLE.CanEditServiceRequestRTFs6,
                ROLE.CanEditServiceRequestRTFs7,
                ROLE.CanEditServiceRequestRTFs8,
                ROLE.CanEditServiceRequestRTFs9,
                ROLE.CanEditServiceRequestRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(ServiceRequest.class, flags);

        // --------------------------------------------------------
        // SummaryTask
        flags = new Integer[]
        {
                ROLE.CanViewSummaryTaskRTFs1,
                ROLE.CanViewSummaryTaskRTFs2,
                ROLE.CanViewSummaryTaskRTFs3,
                ROLE.CanViewSummaryTaskRTFs4,
                ROLE.CanViewSummaryTaskRTFs5,
                ROLE.CanViewSummaryTaskRTFs6,
                ROLE.CanViewSummaryTaskRTFs7,
                ROLE.CanViewSummaryTaskRTFs8,
                ROLE.CanViewSummaryTaskRTFs9,
                ROLE.CanViewSummaryTaskRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(SummaryTask.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditSummaryTaskRTFs1,
                ROLE.CanEditSummaryTaskRTFs2,
                ROLE.CanEditSummaryTaskRTFs3,
                ROLE.CanEditSummaryTaskRTFs4,
                ROLE.CanEditSummaryTaskRTFs5,
                ROLE.CanEditSummaryTaskRTFs6,
                ROLE.CanEditSummaryTaskRTFs7,
                ROLE.CanEditSummaryTaskRTFs8,
                ROLE.CanEditSummaryTaskRTFs9,
                ROLE.CanEditSummaryTaskRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(SummaryTask.class, flags);

        // --------------------------------------------------------
        // Task
        flags = new Integer[]
        {
                ROLE.CanViewTaskRTFs1, ROLE.CanViewTaskRTFs2,
                ROLE.CanViewTaskRTFs3, ROLE.CanViewTaskRTFs4,
                ROLE.CanViewTaskRTFs5, ROLE.CanViewTaskRTFs6,
                ROLE.CanViewTaskRTFs7, ROLE.CanViewTaskRTFs8,
                ROLE.CanViewTaskRTFs9, ROLE.CanViewTaskRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(Task.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditTaskRTFs1, ROLE.CanEditTaskRTFs2,
                ROLE.CanEditTaskRTFs3, ROLE.CanEditTaskRTFs4,
                ROLE.CanEditTaskRTFs5, ROLE.CanEditTaskRTFs6,
                ROLE.CanEditTaskRTFs7, ROLE.CanEditTaskRTFs8,
                ROLE.CanEditTaskRTFs9, ROLE.CanEditTaskRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(Task.class, flags);

        // --------------------------------------------------------
        // WorkProduct
        flags = new Integer[]
        {
                ROLE.CanViewWorkProductRTFs1,
                ROLE.CanViewWorkProductRTFs2,
                ROLE.CanViewWorkProductRTFs3,
                ROLE.CanViewWorkProductRTFs4,
                ROLE.CanViewWorkProductRTFs5,
                ROLE.CanViewWorkProductRTFs6,
                ROLE.CanViewWorkProductRTFs7,
                ROLE.CanViewWorkProductRTFs8,
                ROLE.CanViewWorkProductRTFs9,
                ROLE.CanViewWorkProductRTFs10
        };
        fieldInfo.canReadSecurityFlagsMap.put(WorkProduct.class, flags);
        flags = new Integer[]
        {
                ROLE.CanEditWorkProductRTFs1,
                ROLE.CanEditWorkProductRTFs2,
                ROLE.CanEditWorkProductRTFs3,
                ROLE.CanEditWorkProductRTFs4,
                ROLE.CanEditWorkProductRTFs5,
                ROLE.CanEditWorkProductRTFs6,
                ROLE.CanEditWorkProductRTFs7,
                ROLE.CanEditWorkProductRTFs8,
                ROLE.CanEditWorkProductRTFs9,
                ROLE.CanEditWorkProductRTFs10
        };
        fieldInfo.canWriteSecurityFlagsMap.put(WorkProduct.class, flags);

        // --------------------------------------------------------
        // NoteMinutes
        flags = new Integer[]
        {
            ROLE.CanViewDescription,
        };
        fieldInfo.canReadSecurityFlagsMap.put(NoteMinutes.class, flags);
        flags = new Integer[]
        {
            ROLE.CanEditDescription,
        };
        fieldInfo.canWriteSecurityFlagsMap.put(NoteMinutes.class, flags);
    }

}
