package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult.ValueType;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;

public class RoleBasedRunningWorkflowProcessSecurityController extends
        RoleBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return RunningWorkflowProcess.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);

        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("startDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWorkflowScheduling,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditWorkflowScheduling,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessName",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("currentStep", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcess",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);

        //--------------------------------------------------------
        OperationMappingEntry operationInfo = new OperationMappingEntry(
            SecuredOperations.ManageWorkflow, getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                ValueType valueType = SecurityControllerUtil.isRunningWorkflowProcessManual(
                    (RunningWorkflowProcess) rpmObject, messageContext);

                if (valueType == SecurityValidationResult.TRUE)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanManageManualWorkflows, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                else if (valueType == SecurityValidationResult.FALSE)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanManageScheduledWorkflows, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                //if an exception happend, we'll get an UNDEFINED result, therefore
                //disable evey thing
                return SecurityValidationResult.makeFalseResult(name);
            }
        };
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }
}
