package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

class RoleBasedScorecardFolderSecurityController extends
        RoleBasedAbstractScorecardEntitySecurityController
{

    public Class getContainerClass()
    {
        return ScorecardFolder.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateScorecard, getContainerClass(),
            Scorecard.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveDefaultScorecards;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);

        Integer[] canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultScorecards,

        };
        Integer[] canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDefaultScorecards,
        };

        SecurityControllerUtil.replaceFieldMappingEntryFlags(
            canReadSecurityFlags, canWriteSecurityFlags, mappingEntryMap);

        FieldMappingEntry fieldInfo = null;
        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("scorecards", getContainerClass());
        fieldInfo.canReadSecurityFlags = canReadSecurityFlags;
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass());
        viewEntry.securityFlag = ROLE.CanViewDefaultScorecards;
        return viewEntry;
    }

}
