package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.SimpleNode;

class RoleBasedSimpleNodeSecurityController extends
        RoleBasedWorkElementSecurityController
{

    public Class getContainerClass()
    {
        return SimpleNode.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = SecurityValidationResult.UNDEFINED_RESULT;
//                SecurityValidationResult result = super.canDelete(rpmObject, securityFlags, messageContext);
//                if (result.type == SecurityValidationResult.FALSE)
//                {
//                    return result;
//                }
                Boolean mandatory = ((SimpleNode) rpmObject).getMandatory();
                if (mandatory != null && mandatory.booleanValue())
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanModifyFlagDeleteMandatoryTask, securityFlags,
                        "Delete mandatory Task", rpmObject.getClass());
                }
                return result;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        //--------------------------------------------------------
        fieldInfo = new FieldMappingEntry("proposedDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
        		ROLE.CanViewScheduleInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
        	  ROLE.CanEditDatesProposed,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
        
        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("constraintDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewConstraints,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditConstraints,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("taskType", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditTaskType,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("effortDriven", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("constraintType", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewConstraints,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditConstraints,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("mandatory", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanModifyFlagDeleteMandatoryTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
