package com.ibm.rpm.framework.util;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.ResourceBundle;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.Property;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author Genevieve
 */
public class AboutProperties
{
    private ResourceBundle props = null;

    private ArrayList propsList = null;

    private static AboutProperties instance = null;

    public static AboutProperties getInstance() throws RPMException
    {
        if (instance == null)
        {
            instance = new AboutProperties();
        }

        return instance;
    }

    public String getString(String key, MessageContext messageContext)
            throws RPMException
    {
        getProps(messageContext);
        String result = null;
        if (props != null)
            result = props.getString(key);
        return result;
    }

    /**
     * Returns the list of RPM's properties.
     * 
     * @return A list of Property.
     */
    public ArrayList getAllProperties(MessageContext messageContext)
            throws RPMException
    {
        if (propsList == null)
        {
            try
            {
                getProps(messageContext);
                propsList = new ArrayList();
                if (props != null)
                {
                    Enumeration keys = props.getKeys();
                    while (keys.hasMoreElements())
                    {
                        String key = (String) keys.nextElement();
                        String value = props.getString(key);
                        Property prop = new Property(key, value);
                        propsList.add(prop);
                    }
                }
            }
            catch (Exception e)
            {
                String[] errorParams =
                {
                    "Property file about.properties does not exist in the classpath"
                };
                messageContext.addException(new RPMException(
                        ErrorCodes.INTERNAL_ERROR, errorParams, e));
            }
        }

        return propsList;
    }

    private ResourceBundle getProps(MessageContext messageContext)
            throws RPMException
    {
        if (props == null)
        {
            try
            {
                props = ResourceBundle.getBundle("about");
            }
            catch (Exception e)
            {
                String[] errorParams =
                {
                    "Property file about.properties does not exist in the classpath"
                };
                messageContext.addException(new RPMException(
                        ErrorCodes.INTERNAL_ERROR, errorParams, e));
            }
        }
        return props;
    }
}
