package com.ibm.rpm.framework.util;

import java.lang.reflect.Array;
import java.util.ArrayList;

public class ArrayUtil
{
    /**
     * Translates a list to a typed array. Returns null if the list is null or
     * if the list is empty. Each object in the list must be instanceof the type
     * or null.
     * 
     * @param type
     *            Type of an element of the array.
     * @param list
     *            List of objects to be translated.
     * @return
     */
    public static Object[] listToArray(Class type, ArrayList list)
    {
        Object[] array = null;
        if (list != null && !list.isEmpty())
        {
            array = (Object[]) Array.newInstance(type, list.size());
            array = list.toArray(array);
        }
        return array;
    }

    /**
     * Returns <code>true</code> if <code>array</code> is <code>null</code>
     * or empty.
     * 
     * @param array
     *            Array to be tested.
     * @return True if empty.
     */
    public static boolean isEmpty(Object[] array)
    {
        return array == null || array.length == 0;
    }
    
    /**
     * Searches for the first occurence of the given argument, testing 
     * for equality using the <tt>==</tt> operator. 
     *
     * @param   array 
     * @param   object
     * @return  the index of the first occurrence of the argument in this
     *          list; returns <tt>-1</tt> if the object is not found.
     */
    public static int indexOfReference(ArrayList array, Object object)
    {

        if (array == null)
        {
            return -1;
        }
        int size = array.size();
        if (object == null)
        {
            for (int i = 0; i < size; i++)
                if (array.get(i) == null)
                    return i;
        }
        else
        {
            for (int i = 0; i < size; i++)
                if (object == array.get(i))
                    return i;
        }
        return -1;
    }
    
    /**
     * Returns <tt>true</tt> if this list contains a reference to the specified element.
     *
     * @param object element whose presence in this List is to be tested.
     * @return  <code>true</code> if the specified element is present;
     *          <code>false</code> otherwise.
     */
    public static boolean containsReference(ArrayList array, Object object)
    {
        return indexOfReference(array, object) >= 0;
    }
}
