package com.ibm.rpm.framework.util;

/**
 * Set of static methods to manipulate <code>int</code> bit fields.
 */
public class BitField
{
    /**
     * Returns <code>true</code> when the bit(s) specified by
     * <code>bitMask</code> is(are) set to 1 in <code>bitField</code>.
     * Otherwise, returns false.
     * 
     * @param bitField
     * @param bitMask
     * @return <code>true</code> when the bit(s) is(are) set to 1.
     */
    public static boolean isSet(int bitField, int bitMask)
    {
        return (bitField & bitMask) == bitMask;
    }

    /**
     * Returns <code>Boolean.TRUE</code> when the bit(s) specified by
     * <code>bitMask</code> is(are) set to 1 in <code>bitField</code>.
     * Otherwise, returns false.
     * 
     * @param bitField
     * @param bitMask
     * @return <code>Boolean.TRUE</code> when the bit(s) is(are) set to 1.
     */
    public static Boolean isSetBoolean(int bitField, int bitMask)
    {
        return new Boolean(isSet(bitField, bitMask));
    }

    /**
     * Sets the bit(s) specified by <code>bitMask</code>. Sets the bit to 1
     * when <code>isSet</code> is <code>true</code> and sets the bit to 0
     * when <code>isSet</code> is <code>false</code>.
     * 
     * @param bitField
     * @param bitMask
     * @return The modified <code>bitField</code>.
     */
    public static int set(int bitField, int bitMask, boolean isSet)
    {
        if (isSet)
        {
            bitField |= bitMask;
        }
        else
        {
            bitField &= ~bitMask;
        }
        return bitField;
    }

    /**
     * Sets the bit(s) specified by <code>bitMask</code>. Sets the bit to 1
     * when <code>isSet</code> is <code>Boolean.TRUE</code> and sets the bit
     * to 0 when <code>isSet</code> is <code>Boolean.FALSE</code>.
     * 
     * @param bitField
     * @param bitMask
     * @return The modified <code>bitField</code>.
     */
    public static int set(int bitField, int bitMask, Boolean isSet)
    {
        return set(bitField, bitMask, isSet.booleanValue());
    }
}
