/*
 * Created on Jul 13, 2005
 *
 */

package com.ibm.rpm.framework.util;

import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

/**
 * @author Genevieve
 *  
 */
public class DateUtil {

    public static TimeZone TIMEZONE_GMT = TimeZone.getTimeZone("GMT");
    
    /**
     *  
     */
    public DateUtil() {
        super();
    }

    public static Calendar getCalendarInstance()
    {
        return Calendar.getInstance(TIMEZONE_GMT);
    }
    
    public static Calendar convertToCalendar(Date date) {

        Calendar calendar = null;

        if (date != null) {
            calendar = Calendar.getInstance(TIMEZONE_GMT);
            calendar.setTime(date);
        }

        return calendar;
    }
    
    /**
     * Convert a Calendar to a YYYY-MM-DD format.
     * @param calendar
     * @return
     */
    public static String convertToString(Calendar calendar){
        
        String date = null;
        if(calendar != null){
            Calendar localTimezone = Calendar.getInstance(TIMEZONE_GMT);
            int timezoneOffset = localTimezone.get(Calendar.ZONE_OFFSET);
            long setTimeInMillis = calendar.getTimeInMillis();
            Date setDate = new Date(setTimeInMillis - timezoneOffset);
            date = convertToStringNoTime(setDate);
        }
        return date;
    }
    
    /**
     * Convert a Date to a YYYY-MM-DD format.
     * @param calendar
     * @return
     */
    public static String convertToStringNoTime(Date date){
        
        String result = null;
        if(date != null){
    		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
    		result = format.format(date);
        }
        return result;
    }
    
    /**
     * Convert a String (YYYY-MM-DD) to Calendar.
     * @param String
     * @return
     * @throws ParseException
     */
    public static Calendar convertToCalendarNoTime(String str)
            throws ParseException
    {
        if (str != null)
        {
            SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
            Date date = dateFormatter.parse(str);
            return DateUtil.convertToCalendar(date);
        }
        return null;
    }
    
    /**
     * Convert a Calendar to a YYYY-MM-DD HH:mm:ss format.
     * @param calendar
     * @return
     */
    public static String convertToStringWithTime(Calendar calendar){
        
        String date = null;
        
        if(calendar != null){
    		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    		date = format.format(calendar.getTime());
        }
        
        return date;
        
    }
    
    /**
     * Convert a Calendar to a YYYY-MM-DD 00:00:00 format.
     * @param calendar
     * @return
     */
    public static String convertToStringWithTimeZero(Calendar calendar)
    {
        String date = null;
        
        if (calendar != null)
        {
            date = convertToString(calendar) + " 00:00:00";
        }
        
        return date;
    }
    
    /**
     * Convert from hours to minutes and return the result as Double
     * @param hours	
     * @return
     */
    public static Integer convertHoursToMinutes(Double hours) 
    {
        int minutes = 0;
        if (hours != null)
            minutes = (int) Math.round(hours.doubleValue() * 60);
        return new Integer(minutes);
    }
    
    /**
     * Convert from minutes to hours and return the result as Double
     * @param mins	
     * @return
     */
    public static Double convertMinutesToHours(long minutes) 
    {
        double hours = 0;
        if (minutes != 0)
            hours = (double) minutes / 60;
        return new Double(hours);
    }
    
    /**
     * Convert types that extends java.util.Date, and Oracle timestamp types
     * (oracle.jdbc.Timestamp, oracle.sql.TIMESTAMP) to Calendar
     * 
     * @param obj
     * @return
     * @throws Exception
     */
   
    public static Calendar convertObjectToCalendar(Object obj) throws Exception
    {
        if (obj == null)
        {
            return null;
        }
        
        Calendar result = null;
        try
        {
            if (java.util.Date.class.isAssignableFrom(obj.getClass()))
            {
                result = DateUtil.convertToCalendar((Date) obj);
            }
            else if (obj.getClass().getName().equals("oracle.jdbc.Timestamp") || obj.getClass().getName().equals("oracle.sql.TIMESTAMP"))
            {
                Method timestampValue = obj.getClass().getMethod(
                    "timestampValue", null);
                result = DateUtil.convertToCalendar((Date) timestampValue
                    .invoke(obj, null));
            }
            else{
                //Unsupported type
                throw new Exception();
            }
        }
        catch (Exception e)
        {
            throw new Exception("Error converting Timestamp to Calendar. Object: " + obj.toString(), e);
        }
        return result;
    }
}
