/*
 * Created on Sep 15, 2005
 */
package com.ibm.rpm.framework.util;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author gosselin
 */
public class ErrorCodeProperties
{
    protected static final String ERROR_CODE_PROPERTIES_FILE = "errorCode";

    private static final Log logger = LogFactory.getLog(ErrorCodeProperties.class);

    protected static ErrorCodeProperties instance = new ErrorCodeProperties(null);

    protected ResourceBundle bundle;
    
    protected String fileName;

    private ErrorCodeProperties(String fileName)
    {
        try
        {
            if (StringUtil.isEmpty(fileName))
                fileName = ERROR_CODE_PROPERTIES_FILE;
            bundle = ResourceBundle.getBundle(fileName);
            this.fileName = fileName;
        }
        catch (MissingResourceException e)
        {
            logger.error("Unable to access the properties file: "
                + ERROR_CODE_PROPERTIES_FILE, e);
        }
        catch (NullPointerException e)
        {
            logger.error("No file name specified"
                + " to retrieve the error code properties", e);
        }
    }

    public static ErrorCodeProperties getInstance()
    {
        if (instance.fileName != null)
            instance = new ErrorCodeProperties(null);
        return instance;
    }
    
    public static ErrorCodeProperties getInstance(String fileName)
    {
        if (instance.fileName == null || !instance.fileName.equals(fileName))
            instance = new ErrorCodeProperties(fileName);
        return instance;
    }

    public String getString(String key)
    {
        String value = null;
        try
        {
            value = getErrorDescriptionFromBundle(key);
            return value;
        }
        catch (MissingResourceException e)
        {
            logger.error("No error code value could be retrieved for the following key: "
                + key);
        }
        catch (NullPointerException e)
        {
            logger.error("No property key specified"
                + " to retrieve the error code value", e);
        }
        catch (ClassCastException e)
        {

        }
        return new String(
            "No error code value could be retrieved for the following key: " + key);
    }

	public String getErrorDescriptionFromBundle(String key) 
	{
		return bundle.getString(key);
	}

    public String getString(String key, String[] param)
    {
        String ret = getString(key);
        
        if( param != null )
        {
            for (int i = 0; i < param.length; i++)
            {
                ret = replaceAll(ret, "{" + i + "}", param[i]);
            }
        }      
        return ret;
    }

    // java.lang.String does not have a replaceAll function that does not
    // use regular expressions - use this function to avoid problems
    // with special characters
    private String replaceAll(String str, String substring, String replacement)
    {
        StringBuffer ret = new StringBuffer();
        
        int strIndex = 0;
        int substringIndex = 0;
        while ((substringIndex = str.indexOf(substring, strIndex)) >= 0)
        {
            ret.append(str.substring(strIndex, substringIndex));
            ret.append(replacement);
            strIndex = substringIndex + substring.length();
        }
        ret.append(str.substring(strIndex));
        
        return ret.toString();
    }
}
