/*
 * Created on 24-Aug-2006
 * @author mmatwiy
 * 
 * Generates a list of containers with their type IDs sorted by container name
 * and a second list sorted by type ID.
 * 
 * A null type ID is listed as 0.
 */
package com.ibm.rpm.framework.util;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;

import sun.misc.Launcher;

import com.ibm.rpm.framework.RPMObject;

public class GenerateListofContainerTypeIds
{
    private static ArrayList list = new ArrayList();

    public static void main(String[] args)
    {
        getPackages("com.ibm.rpm.applicationadministration.containers");
        getPackages("com.ibm.rpm.asset.containers");
        getPackages("com.ibm.rpm.clientcostcenters.containers");
        getPackages("com.ibm.rpm.communications.containers");
        getPackages("com.ibm.rpm.contract.containers");
        getPackages("com.ibm.rpm.customfield.containers");
        getPackages("com.ibm.rpm.document.containers");
        getPackages("com.ibm.rpm.expense.containers");
        getPackages("com.ibm.rpm.financial.containers");
        getPackages("com.ibm.rpm.framework.containers");
        getPackages("com.ibm.rpm.resource.containers");
        getPackages("com.ibm.rpm.scopemanagement.containers");
        getPackages("com.ibm.rpm.scorecard.containers");
        getPackages("com.ibm.rpm.security.containers");
        getPackages("com.ibm.rpm.timesheet.containers");
        getPackages("com.ibm.rpm.wbs.containers");
        getPackages("com.ibm.rpm.workflow.containers");

        System.out.println("Type IDs sorted by container name: ");
        printSortByContainerName();

        System.out.println("\n\n****************************************");
        System.out.println("Type IDs sorted by type ID: ");
        printSortByTypeId();
    }

    public static void getPackages(String packageName)
    {
        String name = new String(packageName);
        if (!name.startsWith("/"))
        {
            name = "/" + name;
        }
        name = name.replace('.', '/');

        // Get a File object for the package
        URL url = Launcher.class.getResource(name);

        if (url != null)
        {
            File entry = new File(url.getFile());

            listFiles(entry, packageName);

            if (entry.isDirectory() && entry.getName() != null)
            {
                // Create list of entries for this directory
                String[] fileName = entry.list();
                if (fileName != null)
                {
                    for (int i = 0; i < fileName.length; i++)
                    {
                        File item = new File(entry.getPath(), fileName[i]);
                        listFiles(item, packageName);
                    }
                }
            }
        }

    }

    public static void listFiles(File directory, String packageName)
    {
        if (directory.exists())
        {
            // Get the list of the files contained in the package
            String[] files = directory.list();
            if (files != null)
            {
                for (int i = 0; i < files.length; i++)
                {

                    // we are only interested in .class files
                    if (files[i].endsWith(".class"))
                    {
                        // removes the .class extension
                        String classname = files[i].substring(0, files[i]
                            .length() - 6);
                        try
                        {
                            // Try to create an instance of the object
                            Object o = Class.forName(
                                packageName + "." + classname).newInstance();
                            if (o instanceof RPMObject)
                            {
                                Integer typeId = ((RPMObject) o).retrieveTypeID();
                                ContainerTypeId container = new ContainerTypeId();
                                container.setContainerName(classname);
                                container.setTypeId(typeId == null ? 0 : typeId
                                    .intValue());
                                list.add(container);
                            }
                        }
                        catch (ClassNotFoundException cnfex)
                        {
                            System.err.println(cnfex);
                        }
                        catch (InstantiationException iex)
                        {
                        }
                        catch (IllegalAccessException iaex)
                        {
                        }
                    }
                }
            }
        }
    }

    private static void printSortByContainerName()
    {
        Collections.sort(list, new CompareContainerNames());

        Iterator iterator = list.iterator();

        while (iterator.hasNext())
        {
            ContainerTypeId container = (ContainerTypeId) iterator.next();
            System.out.println(container.getContainerName() + " "
                    + container.getTypeId());
        }
    }

    private static void printSortByTypeId()
    {
        CompareTypeIds comparator = new CompareTypeIds();
        Collections.sort(list, comparator);

        Iterator iterator = list.iterator();

        while (iterator.hasNext())
        {
            ContainerTypeId container = (ContainerTypeId) iterator.next();
            if (container.getTypeId() != 0)
            {
                System.out.println(container.getTypeId() + " "
                        + container.getContainerName());
            }
        }
    }

}

class CompareContainerNames implements Comparator
{
    public int compare(Object obj1, Object obj2)
    {
        int result = 0;
        try
        {
            if (((ContainerTypeId) obj1).getContainerName() != null)
            {
                result = ((ContainerTypeId) obj1).getContainerName().compareTo(
                    ((ContainerTypeId) obj2).getContainerName());
            }
        }
        catch (Exception e)
        {
            return 0;
        }
        return result;
    }
}

class CompareTypeIds implements Comparator
{
    public int compare(Object obj1, Object obj2)
    {
        int result = 0;
        try
        {
            if (((ContainerTypeId) obj1).getTypeId() > ((ContainerTypeId) obj2)
                .getTypeId())
            {
                return 1;
            }
        }
        catch (Exception e)
        {
            return 0;
        }
        return result;
    }
}

class ContainerTypeId
{
    private String containerName;
    private int typeId;

    /**
     * @return Returns the containerName.
     */
    public String getContainerName()
    {
        return containerName;
    }
    /**
     * @param containerName The containerName to set.
     */
    public void setContainerName(String containerName)
    {
        this.containerName = containerName;
    }
    /**
     * @return Returns the typeId.
     */
    public int getTypeId()
    {
        return typeId;
    }
    /**
     * @param typeId The typeId to set.
     */
    public void setTypeId(int typeId)
    {
        this.typeId = typeId;
    }
}