package com.ibm.rpm.framework.util;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;

public class PackageUtil
{
    public static ArrayList getClasses(Package thePackage) throws Exception
    {
        return getClasses(thePackage, Object.class);
    }

    public static ArrayList getClasses(Package thePackage, Class baseClass)
            throws Exception
    {
        ArrayList classesInPackage = new ArrayList();

        String thePackageName = thePackage.getName();

        File directory = null;
        try
        {
            ClassLoader cld = Thread.currentThread().getContextClassLoader();
            if (cld == null)
            {
                throw new Exception("Can't get class loader.");
            }

            String path = thePackageName.replace('.', '/');
            URL resource = cld.getResource(path);
            if (resource == null)
            {
                throw new Exception("No resource for " + path);
            }
            
            String filename = resource.getFile().replaceAll("%20", " ");
            directory = new File(filename);

            if (directory.exists())
            {
                // Get the list of the files contained in the package
                String[] files = directory.list();
                for (int i = 0; i < files.length; i++)
                {
                    // Get class files from the directory
                    String file = files[i];
                    if (file.endsWith(".class"))
                    {
                        // Removes the .class extension
                        String shortClassName = file.substring(0, files[i]
                            .length() - 6);
                        String fullClassName = thePackageName + '.'
                                + shortClassName;
                        Class theClass = Class.forName(fullClassName);
                        if (baseClass.isAssignableFrom(theClass))
                        {
                            classesInPackage.add(theClass);
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw e;
        }

        return classesInPackage;
    }
    
    public static ArrayList getAllContainers() throws Exception
    {
        ArrayList rpmObjectClasses = new ArrayList();

        Package[] allPackages = Package.getPackages();

        for (int i = 0; i < allPackages.length; ++i)
        {
            if (allPackages[i].getName().indexOf("containers") != -1){
                rpmObjectClasses.addAll(getClasses(allPackages[i]));
            }
        }

        return rpmObjectClasses;
    }
}
