package com.ibm.rpm.framework.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.StringTokenizer;
import java.util.TreeSet;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;

/**
 * @author Genevieve
 */
public class PreparedStatementHelper 
{
    public static int nbRecyclings = 0;
    
    private MessageContext messageContext = null;
    private Connection c = null;
    private boolean createCallableStatement = false;
    
    public PreparedStatementHelper(MessageContext messageContext, Connection c) 
    {
        super();
        this.messageContext = messageContext;
        this.c = c;
    }

    public PreparedStatementHelper(MessageContext messageContext, Connection c, boolean createCallableStatement) 
    {
        super();
        this.messageContext = messageContext;
        this.c = c;
        this.createCallableStatement = createCallableStatement;
    }
    
    public PreparedStatement createPreparedStatement(String where, Object[] args)
            throws RPMException, SQLException 
    {
        return createPreparedStatement(where, args, null, false);
    }

    public PreparedStatement createPreparedStatement(String where, Object[] args, int[] spParamsTypes,
            boolean isForCLOBs)
			throws RPMException, SQLException 
    {
        PreparedStatement stmt = null;

        //Oracle Patch to avoid triming issue in 'SELECT' queries
        //We explicitly replace all ? related to NON NULL 'String' arguments
        //by their values
		if(Manager.isOracle(c) && args!=null && args.length!=0 && where.length()>6 && where.substring(0,6).toUpperCase().equals("SELECT"))
		{
		    validateArguments(where, args.length);

		    TreeSet nonNullStringIndexesInArgs = getNonNullStringIndexesInArgs(args);
		    where = replaceNonNullStringArgumentsByValues(where,args,nonNullStringIndexesInArgs);
		    args = removeNonNullStringArgumentsFromArray(args,nonNullStringIndexesInArgs);
		}
		    
        boolean isCachePreparedStatement = MessageContext.isConnectionCachingEnabled(messageContext);
        if(isCachePreparedStatement)
        {
            //try to retrieve stmt from the cache
            stmt = messageContext.getStatementFromCache(c,where);
//TODO clean up later            
if(stmt!=null)
{
    nbRecyclings++;
//System.out.println("RECYCLING STATEMENT, TOTAL = "+nbRecyclings);   
}
            
        }

        if(stmt==null)
        {
            if(createCallableStatement)
            {
                if (isForCLOBs)
                {
                    stmt = c.prepareCall(where);
                }
                else
                {
                    stmt = c.prepareCall(where,
                            ResultSet.TYPE_FORWARD_ONLY,
                            ResultSet.CONCUR_READ_ONLY);
                }                
            }
            else
            {
                if (isForCLOBs)
                {
                    stmt = c.prepareStatement(where);
                }
                else
                {
                    stmt = c.prepareStatement(where,
                            ResultSet.TYPE_FORWARD_ONLY,
                            ResultSet.CONCUR_READ_ONLY);
                }                
            }

            if(isCachePreparedStatement)
            {
//TODO clean up later
//Manager.openStatements++;      
//System.out.println(" OPENED A STATEMENT. TOTAL = "+Manager.openStatements);
                //add stmt to the cache
                messageContext.addStatementToCache(c,where, stmt);
            }
        }
		
        if (! ArrayUtil.isEmpty(args))
        {
			validateArguments(where, args.length);
			setArguments(stmt, args, spParamsTypes);
        }
		return stmt;
	}

//TODO clean up later    
//    private boolean isCachePreparedStatement()
//    {
//        return messageContext!=null && messageContext.isSingleConnectionPerContext();
//    }
    
    private Object[] removeNonNullStringArgumentsFromArray(Object[] args, TreeSet nonNullStringIndexesInArgs)
    {
        Object[] newArgs = null;
        if(args!=null)
        {
            if(nonNullStringIndexesInArgs==null||nonNullStringIndexesInArgs.size()==0)
            {
                newArgs = args;
            }
            else //at least one element
            {
                int newLength = args.length-nonNullStringIndexesInArgs.size();
                newArgs = new Object[newLength];
                int index = 0;
                for(int i=0;i<args.length;i++)
                {
                    if(!nonNullStringIndexesInArgs.contains(new Integer(i)))
                    {
                        newArgs[index] = args[i];
                        index++;
                    }
                }
            }
        }
        return newArgs;
    }

    private String replaceNonNullStringArgumentsByValues(String selectStmt,
            Object[] args, TreeSet nonNullStringIndexesInArgs)
    {    	
        if(args!=null && nonNullStringIndexesInArgs!=null && nonNullStringIndexesInArgs.size()!=0)
        {
            StringBuffer sb = new StringBuffer();
            String[] splits = selectStmt.trim().split("\\?");
            for(int i=0;i<splits.length;i++)
            {
                if(splits[i]!=null&&splits[i].length()!=0)
                {
                    sb.append(splits[i]);
                    if(nonNullStringIndexesInArgs.contains(new Integer(i)))
                    {
                        sb.append("'");
                        sb.append((String)args[i]);
                        sb.append("'");
                    }
                    else
                    {
//                      //not the last chunck and not a null param in the last position (?)
                        if(i+1<splits.length)
                        {
                            sb.append("?");                            
                        }
                        else //if last and null parameter
                        {
                            if(i<args.length&& (args[i]==null||!(args[i] instanceof String)))
                            {
                                sb.append("?");
                            }
                        }
                    }
                }
            }
            return sb.toString();
        }
        return selectStmt;	
    }
    

    private TreeSet getNonNullStringIndexesInArgs(Object[] args)
    {
        TreeSet stringIndexes = new TreeSet();
        for (int i = 0; i < args.length; i++)
        {
            if (args[i] instanceof String && args[i]!=null)
            {
                stringIndexes.add(new Integer(i));
            }
        }
        return stringIndexes;
    }
    
    private void setArguments(PreparedStatement stmt, Object[] args, int[] spParamsTypes)
            throws RPMException, SQLException
    {
        if(args!=null)
        {
            for (int i = 0; i < args.length; i++)
            {

                if (args[i] instanceof String)
                {
                    stmt.setString(i + 1, (String) args[i]);
                    // Patch for Oracle support
                }
                else if (args[i] instanceof Integer)
                {
                    stmt.setString(i + 1, String.valueOf(args[i]));
                }
                else if (args[i] instanceof Long)
                {
                    stmt.setString(i + 1, String.valueOf(args[i]));
                }
                else if(args[i] instanceof Boolean)
                {
                    stmt.setBoolean(i + 1, ((Boolean)args[i]).booleanValue());
                }
                else if(args[i] instanceof Timestamp)
                {
                    stmt.setTimestamp(i+1, ((Timestamp)args[i]));
                }
                else if (args[i] == null)
                {
//                    throw new RPMException(
//                        "PreparedStatementHelper doesn't support null arguments.");
//                    int 
                    stmt.setNull(i+1, spParamsTypes[i]);
                }
                else
                {
                    throw new RPMException(
                        "PreparedStatementHelper doesn't support type "
                                + args[i].getClass().getName() + ".");
                }
            }            
        }
    }

    private void validateArguments(String where, int numberOfArguments)
            throws RPMException {

        StringTokenizer st = new StringTokenizer(where, "?");
        
        int number = st.countTokens();
        
        if(!where.endsWith("?")) number -= 1 ; 

        if (number != numberOfArguments) {
            throw new RPMException(
                    "Number of arguments is not equal to the number of ? in the where clause. Where: "
                            + where);
        }
    }
}
