package com.ibm.rpm.framework.util;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;

/**
 * @author gmeloche
 *  
 */
public class RPMObjectSerializer
{
    private ArrayList objects = new ArrayList();

    private Map objectMap = new HashMap();

    private int index = -1;

    private class ObjectInfo
    {
        int level = 0;

        RPMObject rpmObject = null;

        RPMObjectScope rpmScope = null;

        public ObjectInfo(RPMObject rpmobject, RPMObjectScope rpmScope)
        {
            this.rpmObject = rpmobject;
            this.rpmScope = rpmScope;
        }

        /**
         * @return Returns the rpmObject.
         */
        public RPMObject getRpmObject()
        {
            return rpmObject;
        }

        /**
         * @return Returns the rpmScope.
         */
        public RPMObjectScope getRpmScope()
        {
            return rpmScope;
        }
    }

    public RPMObjectSerializer(RPMObject rpmObject, RPMObjectScope rpmScope)
            throws RPMException
    {
        this();
        if (rpmObject == null)
        {
            throw new RPMException(this.getClass().getName()
                    + " cannot be instanciated with a null object");
        }
        serialize(rpmObject, rpmScope);

    }

    private RPMObjectSerializer()
    {
        super();
    }

    /**
     * This method will serialize the object graph into an arrayList of
     * RPMObject
     *  
     */
    protected void serialize(RPMObject rpmObject, RPMObjectScope rpmScope)
            throws RPMException
    {
        try
        {
            if (rpmObject != null)
            {
                addObject(rpmObject, rpmScope);

                FieldInfo fieldInfo = null;
                Method fieldGetter = null;
                Method scopeGetter = null;

                FieldInfo[] children = ContainerFieldInfoMap.getInstance()
                    .getFieldInfos(rpmObject.getClass());

                if (children != null)
                {
                    for (int i = 0; i < children.length; i++)
                    {
                        fieldInfo = children[i];

                        if (fieldInfo.isComposite())
                        {
                            fieldGetter = fieldInfo.getFieldGetter();

                            Object childValue = null;
                            if (fieldGetter != null)
                            {
                                childValue = fieldGetter
                                    .invoke(rpmObject, null);
                            }

                            boolean alwaysInScope = fieldInfo.isAlwaysInScope();
                            Object scopeValue = null;

                            if (childValue != null)
                            {
                                scopeGetter = fieldInfo.getScopeGetter();
                                if (scopeGetter != null)
                                {
                                    if (rpmScope != null)
                                    {
                                        alwaysInScope = false;
                                        scopeValue = scopeGetter.invoke(
                                            rpmScope, null);

                                        if (scopeValue != null
                                                && Boolean.class
                                                    .isAssignableFrom(scopeValue
                                                        .getClass()))
                                        {
                                            // check if scope is a boolean and
                                            // equals false
                                            if (!((Boolean) scopeValue)
                                                .booleanValue())
                                            {
                                                scopeValue = null;
                                            }
                                        }
                                    }
                                }

                                if (alwaysInScope || scopeValue != null)
                                {
                                    RPMObjectScope scopeParam = null;
                                    if (scopeValue != null
                                            && RPMObjectScope.class
                                                .isAssignableFrom(scopeValue
                                                    .getClass()))
                                    {
                                        scopeParam = (RPMObjectScope) scopeValue;
                                    }

                                    boolean isArray = false;
                                    RPMObject[] childArray = null;
                                    isArray = childValue.getClass().isArray();

                                    // create an array with the children
                                    if (isArray)
                                    {
                                        int length = Array
                                            .getLength(childValue);
                                        childArray = new RPMObject[length];
                                        for (int j = 0; j < length; j++)
                                        {

                                            childArray[j] = (RPMObject) Array
                                                .get(childValue, j);

                                            // By definition, the parent of an
                                            // object should always be its
                                            // preceding object in the structure
                                            if (childArray[j] != null)
                                            {
                                                // For some object, the parent
                                                // object of the structure is
                                                // not a valid parent
                                                // (eg. PortFolio.projects)

                                                childArray[j]
                                                    .setParent(rpmObject);

                                            }
                                        }
                                    }
                                    else
                                    {
                                        ((RPMObject) childValue)
                                            .setParent(rpmObject);
                                        childArray = new RPMObject[]
                                        {
                                            (RPMObject) childValue
                                        };
                                    }

                                    // Serialize each child
                                    for (int j = 0; j < childArray.length; j++)
                                    {
                                        serialize(childArray[j], scopeParam);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            throw new RPMException(e);
        }
    }

    protected void addObject(RPMObject rpmObject, RPMObjectScope rpmScope)
    {
        index++;
        ObjectInfo info = new ObjectInfo(rpmObject, rpmScope);
        objects.add(rpmObject);
        objectMap.put(rpmObject, info);
    }

    public ArrayList getArrayList()
    {

        return objects;
    }

    public Iterator iterator()
    {

        return objects.iterator();
    }

    public RPMObjectScope getScope(RPMObject rpmObject) throws RPMException
    {
        RPMObjectScope scope = null;

        if (rpmObject != null)
        {

            ObjectInfo info = (ObjectInfo) objectMap.get(rpmObject);
            if (info != null)
            {
                scope = info.getRpmScope();
            }
            else
            {
                throw new RPMException(this.getClass().getName()
                        + " Object not found in the ArrayList");
            }

        }

        return scope;

    }

    /**
     * @param object1
     * @param object2
     * @return true if both objects are in the list and index of object1 is
     *         smaller than index of object2
     */
    public boolean before(RPMObject object1, RPMObject object2)
    {
        boolean before = false;

        int index1 = ArrayUtil.indexOfReference(objects, object1);
        int index2 = ArrayUtil.indexOfReference(objects, object2);

        if (index1 >= 0 && index2 >= 0 && index1 < index2)
        {
            before = true;
        }
        return before;
    }

}
