package com.ibm.rpm.framework.util;

import java.lang.reflect.Method;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;

/**
 * 
 * @author cobber
 * 
 */
public class RPMObjectUtil
{
    /**
     * Set the new value and change the modified flag if the new value is
     * different with the old value
     * 
     * @param rpmObject
     * @param fieldName
     * @param newValue
     * @throws RPMException
     */
    public static void deltaAndSet(RPMObject rpmObject, String fieldName,
            Object newValue) throws RPMException
    {
        try
        {
            Container container = MetadataStore.getInstance()
                .getContainerByJavaClass(rpmObject.getClass());
            if (container == null)
            {
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, rpmObject
                    .getClass().getName());
            }
            Field field = container.getField(fieldName);
            if (field == null)
            {
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, rpmObject
                    .getClass().getName(), fieldName);
            }
            Method setter = field.getSetterMethod();
            Method delta = field.getDeltaMethod();
            if (setter == null || delta == null)
            {
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, rpmObject
                    .getClass().getName(), fieldName);
            }

            delta.invoke(rpmObject, new Object[]
            {
                newValue
            });
            setter.invoke(rpmObject, new Object[]
            {
                newValue
            });
        }
        catch (Exception e)
        {
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, new String[]
            {
                e.getMessage()
            }, e);
        }
    }
}
