package com.ibm.rpm.framework.util;


/**
 * @author cobber
 */
public class SqlUtil
{
    /**
     * Char to boolean conversion in SQL.
     * @param fieldName
     * @param truthValue
     * @return the statement can be put in the HashMap.
     */
    public static String getBooleanValue(String fieldName, String truthValue)
    {
        StringBuffer returnString = new StringBuffer();
        
        returnString.append(" CASE ");
        returnString.append(fieldName);
        returnString.append(" WHEN '");
        returnString.append(truthValue);
        returnString.append("' THEN 'true'");
        returnString.append(" ELSE 'false' END ");
        
        return returnString.toString();
    }
    
    /**
     * Integer to boolean conversion in SQL.
     * @param fieldName
     * @param truthValue
     * @return the statement can be put in the HashMap.
     */
    public static String getBooleanValue(String fieldName, int truthValue)
    {
        StringBuffer returnString = new StringBuffer();
        
        returnString.append(" CASE ");
        returnString.append(fieldName);
        returnString.append(" WHEN ");
        returnString.append(truthValue);
        returnString.append(" THEN 'true'");
        returnString.append(" ELSE 'false' END ");
        
        return returnString.toString();
    }
    
    /**
     * Cast integer to character
     * @param fieldName
     * @param numberOfChars
     * @return the statement can be put when we do the mapping.
     */
    public static String castAsChar(String fieldName, int numberOfChars)
    {
        StringBuffer returnString = new StringBuffer();
        
        returnString.append(" CAST( ");
        returnString.append(fieldName);
        returnString.append(" AS ");
        returnString.append(" CHAR(");
        returnString.append(numberOfChars);
        returnString.append(")) ");
        
        return returnString.toString();
    }

    /**
     * Cast char to double
     * @param fieldName
     * @return the statement can be put when we do the mapping.
     */
    public static String castAsDouble(String fieldName)
    {
        StringBuffer returnString = new StringBuffer();
        
        returnString.append("char_to_dec(");
        returnString.append(fieldName);
        returnString.append(") "); 
        
        return returnString.toString();
    }
    
    /**
     * Convert a timestamp to a date
     * @param fieldName
     * @return the statement can be put when we do the mapping.
     */
    public static String convertTimestampToDate(String fieldName)
    {
        StringBuffer returnString = new StringBuffer();
        
        returnString.append("timestamp_to_date(");
        returnString.append(fieldName);
        returnString.append(") "); 
        
        return returnString.toString();
    }
    
    /**
     * Escape characters % and _ so that they're treated like normal characters.
     * This works in both DB2 and Oracle.
     * @param input - the input String
     * @return the input string modified to include escape characters
     * as well as an end wildcard
     */
    public static String escapeSpecialCharactersAddWildcard(String input)
    {
        // remove the end '
        input = input.substring(0, input.length() - 1);
        
        if (input.indexOf("%") != -1 || input.indexOf("_") != -1)
        {
            // escape occurrences of the escape character
            input = input.replaceAll("/", "//");
            
            // escape all occurrences of % and _
            input = input.replaceAll("%", "/%");
            input = input.replaceAll("_", "/_");
            
            // add the wildcard and the end ' prior to the escape clause
            input = input + "%'";
            
            // add the escape clause
            input =  input + " ESCAPE '/'";
        }
        else
        {
            // nothing to escape, add the end wildcard and the '
            input = input + "%'";
        }
        
        return input;
    }
}
