package com.ibm.rpm.framework.util;

import java.io.UnsupportedEncodingException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author gmeloche
 */
public class StringUtil
{
    public static String toString(Object object)
    {
        return (object == null) ? null : object.toString();
    }
    
    /**
     * True when <code>str</code> is null or its length is zero. False
     * otherwise.
     * 
     * @param str
     *            String to be tested for emptyness.
     * @return True when <code>str</code> is empty.
     */
    public static boolean isEmpty(String str)
    {
        return (str == null) || (str.length() == 0);
    }

    /**
     * True when <code>buffer</code> is null or its length is zero. False
     * otherwise.
     * 
     * @param buffer
     *            StringBuffer to be tested for emptyness.
     * @return True when <code>buffer</code> is empty.
     */
    public static boolean isEmpty(StringBuffer buffer)
    {
        return (buffer == null) || (buffer.length() == 0);
    }

    /**
     * True when <code>str</code> is null or contains non-blank characters.
     * False otherwise.
     * 
     * @param str
     *            String to be tested for blankness.
     * @return True when <code>str</code> is blank.
     */
    public static boolean isBlank(String str)
    {
        return (str == null) || (str.trim().length() == 0);
    }

    /**
     * Returns a class name without the package name.
     * 
     * @param aClass
     * @return
     */
    public static String getShortClassName(Class aClass)
    {
        String className = null;

        if (aClass != null)
        {
            className = getShortClassName(aClass.getName());
        }
        return className;
    }

    /**
     * Returns a class name without the package name.
     * 
     * @param className
     * @return
     */
    public static String getShortClassName(String className)
    {        
        return substringAfterOccurenceOf(className, ".");
    }

    /**
     * @param source
     * @param regexp
     * @return count the occurence of a string that matches a regular expression
     *         in a string.
     */
    public static int countOccurence(String source, String regexp)
    {
        int count = 0;
        Pattern lp = Pattern.compile(regexp);
        Matcher mlp = lp.matcher(source);
        while (mlp.find())
        {
            count++;
        }

        return count;
    }
    
    /**
     * @param source
     * @param regexp
     * @return count the occurence of a string that matches a regular expression
     *         for text that is between ' in a string.
     */
    public static int countOccurenceBetweenQuotes(String source, String regexp)
    {
        
        String exp = "'[^']*?'";
       
        int count = 0;
        Pattern lp = Pattern.compile(exp);
        Matcher mlp = lp.matcher(source);
        while (mlp.find())
        {
            count = count + countOccurence(mlp.group(), regexp);
        }

        return count;
    }
    
    public static String removeQuotationMarks(String source)
    {
        return source.replaceAll("'", "");
    }
    
    public static String replaceApostrophesWithQuotes(Object source)
    {
        if(source != null){
            
            return source.toString().replaceAll("&apos;", "''");
        }
        
        return null;
    }
    
    public static String convertUnicodeToUtf8(String unicode)
            throws UnsupportedEncodingException
    {       
        // Convert from Unicode to UTF-8
        if (unicode != null)
        {
            byte [] utf8 = unicode.getBytes("UTF-8");
            return new String(utf8);
        }
        
        return null;
    }
    
    public static String substringAfterOccurenceOf(String input, String stringToMatch)
    {
        if (input != null)
        {
            int index;
            index = input.lastIndexOf(stringToMatch);
            if (index != -1)
            {
                input = input.substring(index + 1, input.length());
            }
        }
        return input;
    }
    
    public static String removeQuotedContent(String input)
    {
        if (input != null)
        {
            int index = input.indexOf("'", 0);
            int nextIndex = input.indexOf("'", index + 1);
            
            while (nextIndex != -1)
            {
                String input1 = input.substring(0, index);
                String input2 = input.substring(nextIndex + 1, input.length());
                
                input = input1 + input2;
                
                index = input.indexOf("'", 0);
                nextIndex = input.indexOf("'", index + 1);
             }
        }
        
        return input;
    }
}
