package com.ibm.rpm.init;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ContainerFieldInfoMap;
import com.ibm.rpm.framework.util.FieldInfo;
import com.ibm.rpm.framework.util.PackageUtil;
import com.ibm.rpm.framework.util.RPMObjectScopeUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author gmeloche
 *
 */
public class FieldsBizMetaInfoInitializer
{
    private static String ALWAYS_IN_SCOPE = "AlwaysInScope";

    private static String NOT_IMPLEMENTED = "NotImplemented";

    private static Map scopeFieldExceptions = new HashMap();

    private static Map compositeExceptions = new HashMap();
    
    private static Log logger = LogFactory
        .getLog(FieldsBizMetaInfoInitializer.class);

    static int nbExceptions;
    static
    {
        fillScopeFieldExceptionlist();
        fillCompositeExceptionList();
    }

    public static void main(String[] args) throws Exception
    {
        RPMInitializer.init();
        init();
        logger.debug("Number of exceptions " + nbExceptions);
    }

   
    /**
     * Makes sure that each compiled container's getters match up to its scope
     * object's methods
     * 
     */
    public static void init()
    {

        try
        {
            ArrayList rpmObjects = PackageUtil.getAllContainers();
            //rpmObjects.clear();
            //rpmObjects.add(Project.class);

            int counter = 0;
            int noScopeFieldCounter = 0;
            //ArrayList scopeGetterNotFoundFields = new ArrayList();

            for (int i = 0; i < rpmObjects.size(); ++i)
            {
                Class container = (Class) rpmObjects.get(i);

                if (!Modifier.isAbstract(container.getModifiers())
                        && RPMObject.class.isAssignableFrom(container))
                {

                    boolean scopeGetterFound = false;
                    Method containerFieldGetter = null;
                    Method scopeFieldGetter = null;
                    boolean isComposite = false;
                    boolean isAlwaysInScope = false;

                    Class scope = RPMObjectScopeUtil.getInstance().getScope(
                        container);

                    HashMap containerFieldGetters = createGettersMap(container);

                    if (!containerFieldGetters.isEmpty())
                    {
                        //System.out.println("\n//" + container.getName());
                    }
                    else
                    {
                        continue;
                    }

                    for (Iterator iter = containerFieldGetters.keySet().iterator(); iter
                        .hasNext();)
                    {
                        counter++;
                        containerFieldGetter = (Method) iter.next();
                        scopeFieldGetter = null;
                        scopeGetterFound = false;
                        isAlwaysInScope = false;

                        if(scope == RPMObjectScope.class){
                            isAlwaysInScope = true;
                        }
                        
                        
                        if (scope != null && scope != RPMObjectScope.class)
                        {
                            
                            //check for exception
                            if (!scopeGetterFound)
                            {
                                String scopeFieldGetterName = getScopeFieldException(
                                    container, containerFieldGetter.getName());

                                if (scopeFieldGetterName != null)
                                {
                                    if (scopeFieldGetterName
                                        .equals(ALWAYS_IN_SCOPE))
                                    {
                                        isAlwaysInScope = true;
                                        scopeGetterFound = true;

                                    }
                                    else if (scopeFieldGetterName
                                        .equals(NOT_IMPLEMENTED))
                                    {
                                        scopeGetterFound = true;
                                    }
                                    else
                                    {
                                        try
                                        {
                                            scopeFieldGetter = scope.getMethod(
                                                scopeFieldGetterName, null);
                                            scopeGetterFound = true;
                                        }
                                        catch (Exception e)
                                        {
                                            //method does not exist
                                        }
                                    }
                                }
                            }
                            
                            //ContainerFieldGetter matches scopeFieldGetter
                            try
                            {
                                scopeFieldGetter = scope.getMethod(
                                    containerFieldGetter.getName(), null);
                                scopeGetterFound = true;
                            }
                            catch (Exception e)
                            {
                                //method does not exist
                            }

                            //Try with is (scopeFieldGetter returns a boolean)
                            if (!scopeGetterFound)
                            {
                                String methodName = containerFieldGetter.getName();
                                String newMethodName = methodName.substring(3);
                                newMethodName = "is" + newMethodName;

                                try
                                {
                                    scopeFieldGetter = scope.getMethod(
                                        newMethodName, null);
                                    scopeGetterFound = true;
                                }
                                catch (Exception e)
                                {
                                    //method does not exist
                                }
                            }


                            if (!scopeGetterFound)
                            {
                                noScopeFieldCounter++;
                                scopeFieldGetter = null;
                            }

                        }
                        else
                        {
                            //no associated scope class
                            scopeGetterFound = true;
                        }

                        isComposite = isComposite(container, containerFieldGetter);


                        if (scopeGetterFound)
                        {
                            
                            if (scopeFieldGetter != null && !scopeFieldGetter.equals(NOT_IMPLEMENTED))
                            {
                                validatePredefinedMapping(
                                    container,
                                    getFieldName(containerFieldGetter.getName()),
                                    containerFieldGetter, scope,
                                    scopeFieldGetter, isComposite,
                                    isAlwaysInScope);
                            }
                            
                            
                            //TODO mapping is now done using the
                            // PredefinedFieldsBizMetaInfo class until we found
                            // a better way to load it dynamically using the
                            // class loader ()
                            
                            /*
                            ContainerFieldInfoMap.getInstance().addFieldInfo(
                                container,
                                getFieldName(containerFieldGetter.getName()),
                                containerFieldGetter, scopeFieldGetter,
                                isComposite, isAlwaysInScope, isChild);
                            */

                            //TODO Remove comments to generate the call needed to
                            // fill out PredefinedFieldsBizMetaInfo class
                            
                            /*
                            generatePredefinedMappingMethod(counter, container,
                                containerFieldGetter, scope, scopeFieldGetter,
                                isComposite, isAlwaysInScope, isChild);
                             */
                            
                            //TODO uncomment to extract a list container with
                            // composite status for verification
                             /*
                              * printCompositionInfo(container,
                              * containerFieldGetter, isComposite);
                              */


                        }
/*                        else
                        {
                            scopeGetterNotFoundFields
                                .add("\n***Scope getter not found*** \nContainer "
                                        + container.getName() + "\nMethod: "
                                        + containerFieldGetter.getName()
                                        + "\nScope Class:" + scope.getName());

                        }*/
                    }
                }
            }

/*            if (noScopeFieldCounter > 0)
            {
                StringBuffer sb = new StringBuffer();
                for (Iterator iter = scopeGetterNotFoundFields.iterator(); iter
                    .hasNext();)
                {
                    String element = (String) iter.next();
                    sb.append(element.toString() + "\n");

                }
                logger.error(sb.toString());
                throw new RPMException(FieldsBizMetaInfoInitializer.class.getName()
                        + "- Incomplete scope field mapping.");
            }

            logger.debug(FieldsBizMetaInfoInitializer.class.getName()
                    + ": Number of fields mapped: " + counter);*/
        }

        catch (Exception e)
        {
            logger.error(e.getMessage());
        }

    }

    /**
     * Associate each getter to its return type
     * 
     * @param containerType
     * @return a map of all the class' getters mapped to their return types
     * @throws Exception
     */
    private static HashMap createGettersMap(Class containerType) throws Exception
    {
        ArrayList excludeList = new ArrayList();
        excludeList.add("getParent");

        HashMap getters = new HashMap();

        Method[] methods = containerType.getMethods();

        for (int i = 0; i < methods.length; i++)
        {
            Method method = methods[i];
            String methodName = method.getName();
            if ((methodName.startsWith("get") || methodName.startsWith("is"))
                    && !excludeList.contains(methodName))
            {
                Class fieldType = method.getReturnType();

                if (fieldType.isArray())
                {
                    fieldType = fieldType.getComponentType();
                }

                //include only RPMObject fields
                if (RPMObject.class.isAssignableFrom(fieldType))
                {
                    getters.put(method, fieldType);
                }
            }
        }

        return getters;
    }

    private static void fillScopeFieldExceptionlist()
    {
        //exceptions can be retrieve with or without a container name
        scopeFieldExceptions.put("getBaselineDate", "isScheduleDates");
        scopeFieldExceptions.put("getActualDate", "isScheduleDates");
        scopeFieldExceptions.put("getExpectedDate", "isScheduleDates");
        scopeFieldExceptions.put("getForecastDate", "isScheduleDates");
        scopeFieldExceptions.put("getInitialBaselineDate", "isScheduleDates");
        scopeFieldExceptions.put("getPlanDate", "isScheduleDates");
        scopeFieldExceptions.put("getPreviousBaselineDate", "isScheduleDates");
        scopeFieldExceptions.put("getProposedDate", "isScheduleDates");
        scopeFieldExceptions.put("getApprovedDate", "isScheduleDates");

        scopeFieldExceptions.put("getIssueAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getTaskAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getDeliverableAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getServiceRequestAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getActionAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getRequirementAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getChangeRequestAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getRiskAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getDocumentAttributeAssignments",
            "isRelatedAttributeAssignments");
        scopeFieldExceptions.put("getDefectAttributeAssignments",
            "isRelatedAttributeAssignments");

        scopeFieldExceptions.put("getChangeRequestScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getRiskScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getDefectScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getIssueScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getRequirementScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getServiceRequestScorecard",
            "getScopeScorecards");
        scopeFieldExceptions.put("getActionScorecard",
            "getScopeScorecards");

        scopeFieldExceptions
            .put("getAssignedScorecardQuestions", "getScorecardQuestions");
        scopeFieldExceptions
            .put("getAssignedScorecardResponses", "getScorecardResponses");
        scopeFieldExceptions.put("getAssignedScorecardCategories",
            "getScorecardCategories");

        scopeFieldExceptions.put("getTimeCode1", "isTimeCodes");
        scopeFieldExceptions.put("getTimeCode2", "isTimeCodes");
        
        scopeFieldExceptions.put("getStepTimesheets", "isTimesheets");
        scopeFieldExceptions.put("getCreditTimesheet", "isTimesheets");

        //Fields are always loaded without scope
        scopeFieldExceptions.put(
            AssignedScorecard.class.getName() + ".getScorecardState",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(CustomField.class.getName() + ".getDatafieldCategory",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardMonday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardFriday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardSaturday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardSunday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardThursday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardTuesday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(RPMCalendar.class.getName() + ".getStandardWednesday",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Asset.class.getName() + ".getCategory", ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Asset.class.getName() + ".getState", ALWAYS_IN_SCOPE);

        scopeFieldExceptions.put(GenericProject.class.getName() + ".getCalendar",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Proposal.class.getName() + ".getCalendar",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(WorkOrganization.class.getName() + ".getCalendar",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Project.class.getName() + ".getCalendar",
            ALWAYS_IN_SCOPE);

        scopeFieldExceptions.put(GenericProject.class.getName() + ".getCurrency",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Proposal.class.getName() + ".getCurrency",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(WorkOrganization.class.getName() + ".getCurrency",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Project.class.getName() + ".getCurrency",
            ALWAYS_IN_SCOPE);

        scopeFieldExceptions.put(ResourceTaskAssignment.class.getName()
                + ".getSecurityRole", ALWAYS_IN_SCOPE);

        scopeFieldExceptions.put(Resource.class.getName() + ".getVerifiedBy",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Resource.class.getName() + ".getReVerifiedBy",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(SummaryTimesheet.class.getName() + ".getTaskAssignment",
            ALWAYS_IN_SCOPE);
        
        //Field exist but is not in the scope because it is not implemented
        //All
        scopeFieldExceptions.put("getStatusUpdates", NOT_IMPLEMENTED);

        //GenericProject
        scopeFieldExceptions.put(
            GenericProject.class.getName() + ".getProjectCostCenter",
            NOT_IMPLEMENTED);
        scopeFieldExceptions.put(Proposal.class.getName() + ".getProjectCostCenter",
            NOT_IMPLEMENTED);
        scopeFieldExceptions.put(WorkOrganization.class.getName()
                + ".getProjectCostCenter", NOT_IMPLEMENTED);
        scopeFieldExceptions.put(Project.class.getName() + ".getProjectCostCenter",
            NOT_IMPLEMENTED);

        //Pool
        scopeFieldExceptions.put(Pool.class.getName() + ".getClientCostCenter",
            NOT_IMPLEMENTED);

        //Resource
        scopeFieldExceptions.put(Resource.class.getName() + ".getExecutiveRole",
            NOT_IMPLEMENTED);
        scopeFieldExceptions.put(Resource.class.getName() + ".getWorkPackages",
            NOT_IMPLEMENTED);
        scopeFieldExceptions.put(Resource.class.getName() + ".getResourceSecurityRights",
            NOT_IMPLEMENTED);
        scopeFieldExceptions.put(Resource.class.getName() + ".getSecurityRole",
            ALWAYS_IN_SCOPE);
        scopeFieldExceptions.put(Resource.class.getName() + ".getTaskAssignments",
            NOT_IMPLEMENTED);
    }

    private static String getScopeFieldException(Class container, String methodName)
    {

        String scopeMethodName = null;

        if (scopeFieldExceptions.containsKey(methodName))
        {
            scopeMethodName = (String) scopeFieldExceptions.get(methodName);
        }
        else if (scopeFieldExceptions.containsKey(container.getName() + "." + methodName))
        {
            scopeMethodName = (String) scopeFieldExceptions.get(container.getName() + "."
                    + methodName);
        }

        return scopeMethodName;

    }

    private static void fillCompositeExceptionList()
    {

        //based on return type
        compositeExceptions.put(WbsScheduleDate.class, Boolean.TRUE);
        compositeExceptions.put(ScopeScheduleDate.class, Boolean.TRUE);
        compositeExceptions.put(AttributeAssignment.class, Boolean.TRUE);
        compositeExceptions.put(RtfAssignment.class, Boolean.TRUE);
        compositeExceptions.put(CalendarWeekDay.class, Boolean.TRUE);
        compositeExceptions.put(ResourceRates.class, Boolean.TRUE);
        compositeExceptions.put(ResourceSecurityRights.class, Boolean.TRUE);
        compositeExceptions.put(GeographicalAssignment.class, Boolean.TRUE);
        compositeExceptions.put(OrganizationalAssignment.class, Boolean.TRUE);
        compositeExceptions.put(AssetOrganizationalAssignment.class, Boolean.TRUE);
        compositeExceptions.put(AssignedScorecard.class, Boolean.TRUE);
        compositeExceptions.put(RiskMatrix.class, Boolean.TRUE);

        compositeExceptions.put(OrderOfMagnitudeCostBenefits.class, Boolean.TRUE);
        compositeExceptions.put(Impact.class, Boolean.TRUE);
        compositeExceptions.put(ApprovedCostBenefits.class, Boolean.TRUE);
        compositeExceptions.put(Opportunity.class, Boolean.TRUE);

        compositeExceptions.put(TimesheetApprovalStatus.class, Boolean.TRUE);
        compositeExceptions.put(CreditTimesheet.class, Boolean.TRUE);
        compositeExceptions.put(PersonalProject.class, Boolean.TRUE);

        //Specific field
        compositeExceptions.put(ResourceTaskAssignment.class.getName()
                + ".getTaskAssignment", Boolean.TRUE);
        compositeExceptions.put(Asset.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(ClientCostCenter.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(AbstractScope.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(WorkElement.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Action.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Deliverable.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(GenericProject.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Milestone.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Project.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Proposal.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(SimpleNode.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(SummaryTask.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Task.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(WorkOrganization.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(WorkProduct.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(ChangeRequest.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Defect.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Issue.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(ReqProRequirement.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Requirement.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Risk.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(ServiceRequest.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Pool.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        compositeExceptions.put(Resource.class.getName()
            + ".getDocumentFolder", Boolean.TRUE);
        
        compositeExceptions.put(Resource.class.getName()
            + ".getAdministrativeTaskAssignments", Boolean.FALSE);
        compositeExceptions.put(Resource.class.getName()
            + ".getPersonalTaskAssignments", Boolean.FALSE);  
        
    }

  
    private static boolean isComposite(Class container,
            Method containerFieldGetter)
    {

        boolean isComposite = false;

        Class returnType = containerFieldGetter.getReturnType();

        //generic rule
        if (containerFieldGetter.getReturnType().isArray())
        {
            isComposite = true;
        }

        //exceptions based on return type
        if (compositeExceptions.containsKey(returnType))
        {
            isComposite = ((Boolean) compositeExceptions.get(returnType))
                .booleanValue();
        }
        else if (compositeExceptions.containsKey(container.getName() + "."
                + containerFieldGetter.getName()))
        {
            isComposite = ((Boolean) compositeExceptions.get(container
                .getName()
                    + "." + containerFieldGetter.getName())).booleanValue();
        }

        return isComposite;

    }

    public static void printCompositionInfo(Class containerType,
            Method containerFieldGetter, boolean isComposite)
    {

        StringBuffer sb = new StringBuffer();

        //container
        sb.append(StringUtil.getShortClassName(containerType));
        sb.append(", ");

        //container field name
        sb.append(getFieldName(containerFieldGetter.getName()));

        //container field type
        sb.append(", TYPE:" + StringUtil.getShortClassName(containerFieldGetter.getReturnType()));

        //isArray
        sb.append(", ARRAY:" + containerFieldGetter.getReturnType().isArray());

        //isComposite
        sb.append(", COMPOSITE:" + isComposite);

        System.out.println(sb.toString());

    }

    private static String getFieldName(String methodName)
    {
        String fieldname = null;
        if (methodName != null)
        {
            if (methodName.startsWith("get"))
            {
                fieldname = methodName.substring(3);
            }
            else
            {
                fieldname = methodName.substring(2);
            }
        }
        return fieldname;
    }

    private static String generatePredefinedMappingMethod(int counter, Class containerType,
            Method containerFieldGetter, Class scopeType,
            Method scopeFieldMethod, boolean isComposite,
            boolean isAlwaysInScope)
    {

        //TODO create a separate generator to generate the complete class
        
        /*map.addFieldInfo(
                                container,
                                getFieldName(containerFieldGetter.getName()),
                                containerFieldGetter, scopeFieldGetter,
                                isComposite, isAlwaysInScope, isChild);*/

        if(counter == 400){            
            System.out.println("\n\n******Next calls be put in initPart2() method******\n\n");
            
        }else if((counter == 800)){
            System.out.println("\n\n******Next calls be put in initPart3() method******\n\n");            
        }
        
        StringBuffer sb = new StringBuffer();
        sb.append("map.addFieldInfo(");

        //container
        sb.append(containerType.getName() + ".class");
        sb.append(", ");

        //container field name
        sb.append("\"");
        sb.append(getFieldName(containerFieldGetter.getName()));
        sb.append("\", ");

        //container field getter
        sb.append("getMethod(");
        sb.append(containerType.getName() + ".class,\"");
        sb.append(containerFieldGetter.getName());
        sb.append("\")");
        sb.append(", ");

        //scope field getter
        if (scopeFieldMethod != null)
        {
            sb.append("getMethod(");
            sb.append(scopeType.getName() + ".class,\"");
            sb.append(scopeFieldMethod.getName());
            sb.append("\")");
        }
        else
        {
            sb.append(scopeFieldMethod);
        }
        sb.append(", ");

        //isComposite
        sb.append(isComposite);
        sb.append(", ");

        //isAlwaysInScope
        sb.append(isAlwaysInScope);
        sb.append(", ");



        sb.append(");");

        return sb.toString();

    }
    
    
    /**
     * Verifies that all fields of containerType are properly mapped to metadata
     */
    private static void validatePredefinedMapping(Class containerType,
            String fieldName, Method containerFieldGetter, Class scopeType,
            Method scopeFieldMethod, boolean isComposite,
            boolean isAlwaysInScope)
    {

        boolean missingInfo = false;
        boolean unsynchronized = false;

        FieldInfo info = null;
        // the actual compiled container's field name is mapped to
        // value in the "old" metadata
        Map fields = ContainerFieldInfoMap.getInstance().getFieldMap(
            containerType);
        if (fields != null)
        {
            info = (FieldInfo) fields.get(fieldName.toUpperCase());
            if (info != null)
            {

                if ((containerFieldGetter != null && !containerFieldGetter
                    .equals(info.getFieldGetter()))
                        || (containerFieldGetter == null && info
                            .getFieldGetter() != null))
                {
                    unsynchronized = true;
                }
                if ((scopeFieldMethod != null && !scopeFieldMethod.equals(info
                    .getScopeGetter()))
                        || (scopeFieldMethod == null && info.getScopeGetter() != null))
                {
                    unsynchronized = true;
                }
                if (isComposite != info.isComposite())
                {
                    unsynchronized = true;
                }
                if (isAlwaysInScope != info.isAlwaysInScope())
                {
                    unsynchronized = true;
                }
            }
            else
            {
                missingInfo = true;
            }
        }
        else
        {
            missingInfo = true;
        }

        if (missingInfo)
        {
            logger.debug("Missing field in "
                    + StringUtil
                        .getShortClassName(PredefinedFieldsBizMetaInfo.class)
                    + ": " + StringUtil.getShortClassName(containerType) + "."
                    + containerFieldGetter.getName());
            nbExceptions++;
        }
        if (missingInfo || unsynchronized)
        {
            String msg = "Generated  : ";
            if(missingInfo){
                msg = "Add line   : ";
            }
            
            logger.debug(msg
                    + generatePredefinedMappingMethod(0, containerType,
                        containerFieldGetter, scopeType, scopeFieldMethod,
                        isComposite, isAlwaysInScope));
            nbExceptions++;

        }
        if (unsynchronized)
        {
            Class scope = null;
            if (info.getScopeGetter() != null)
            {
                scope = info.getScopeGetter().getDeclaringClass();
            }
            logger
                .debug("Predefined : "
                        + generatePredefinedMappingMethod(0, containerType,
                            info.getFieldGetter(), scope,
                            info.getScopeGetter(), info.isComposite(), info
                                .isAlwaysInScope()) + "\n");
            nbExceptions++;

        }
    }

}
