/*
 * Created on Jun 6, 2006
 *
 */
package com.ibm.rpm.interfaces;

import java.util.Calendar;

import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.PropertyArray;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectArray;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author eturcot
 */
public class ApplicationLiteral
{
    private Application application;

    public ApplicationLiteral()
    {
        super();
        application = new Application();
    }
    
    public Result checkOut(String sessionID, RPMObject rpmObject,
            boolean isTimeless)
    {
        Result result = application.checkOut(sessionID, rpmObject, isTimeless);
        return result;
    }

    public Result checkIn(String sessionID, RPMObject rpmObject)
    {
        Result result = application.checkIn(sessionID, rpmObject);
        return result;
    }

    public Calendar getDatabaseTimestamp(String sessionID) throws RPMException
    {
        return application.getDatabaseTimestamp(sessionID); 
    }
    
    public AlreadyCheckedOutResult isCheckedOut(String sessionID,
            RPMObject rpmObject)
    {
        AlreadyCheckedOutResult result = application.isCheckedOut(sessionID,
            rpmObject);
        return result;
    }
    
    public Result forceCheckinOnSessionsCheckOut(String sessionID,
            String targetSessionID)
    {
        return application.forceCheckinOnSessionsCheckOut(sessionID,
            targetSessionID);
    }

    public Result forceCheckinOnUsersCheckOut(String sessionID,
            Resource user)
    {
        return application.forceCheckinOnUsersCheckOut(sessionID, user);
    }
    
    public Result forceCheckinOnRPMObjectCheckOut(String sessionID,
            RPMObject rpmObject)
    {
        return application.forceCheckinOnRPMObjectCheckOut(sessionID, rpmObject);
    }

    public SingleResult save(String sessionID, RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
    {
        SingleResult result = application.save(sessionID, rpmObject,
            rpmObjectScope, reloadType);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public Result delete(String sessionID, RPMObject rpmObject)
    {
        Result result = application.delete(sessionID, rpmObject);
        return result;
    }

    public ArrayResult loadFromXpath(String sessionID, String xpath,
            RPMObjectScope scope)
    {
        ArrayResult result = application.loadFromXpath(sessionID, xpath, scope);
        CyclicReferenceRemover.remove(result.getRpmObjectList());
        return result;
    }

    public SingleResult loadFromID(String sessionID, RPMObject rpmObject,
            RPMObjectScope scope)
    {
        SingleResult result = application.loadFromID(sessionID, rpmObject,
            scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public ArrayResult loadArrayFromID(String sessionID,
            RPMObjectArray rpmObjectArray, RPMObjectScope scope)
    {
        ArrayResult result = application.loadArrayFromID(sessionID,
            rpmObjectArray, scope);
        CyclicReferenceRemover.remove(result.getRpmObjectList());
        return result;
    }

    public PropertyArray getSystemInfo(String sessionID) throws RPMException
    {
        return application.getSystemInfo(sessionID);
    }
    
    public Result disableAutomaticCheckoutMode(String sessionID)
    {
        return application.disableAutomaticCheckoutMode(sessionID);
    }
    
    public Result enableAutomaticCheckoutMode(String sessionID)
    {
        return application.enableAutomaticCheckoutMode(sessionID);
    }

    public SingleResult startWorkflow(String sessionID,
            WorkflowProcess process, RPMObject object,
            RunningWorkflowProcessScope scope)
    {
        SingleResult result = application.startWorkflow(sessionID, process,
            object, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public Result associateWorkflow(String sessionID, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
    {
        Result result = application.associateWorkflow(sessionID, process,
            genericProject, associate, isDefault);
        return result;
    }

    public SingleResult suspendWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        SingleResult result = application.suspendWorkflow(sessionID,
            runningProcess, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult resumeWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        SingleResult result = application.resumeWorkflow(sessionID,
            runningProcess, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public Result cancelWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess)
    {
        Result result = application.cancelWorkflow(sessionID, runningProcess);
        return result;
    }

    public SingleResult sendNotification(String sessionID, 
            GenericCommunicationScope scope, RPMObject attachment, ResourceArray recipients, 
            String subject, String comments)
    {
        SingleResult result = application.sendNotification(sessionID, scope, attachment, recipients, 
            subject, comments);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult sendStaffingRequest(String sessionID,
            GenericCommunicationScope scope, RPMObject attachment, ResourceArray recipients, 
            String subject, String comments, Calendar dueDate)
    {
        SingleResult result = application.sendStaffingRequest(sessionID, scope, attachment, recipients, 
            subject, comments, dueDate);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult saveWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        SingleResult result = application.saveWorkflowResponse(sessionID, response, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult sendWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        SingleResult result = application.sendWorkflowResponse(sessionID, response, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult updateReadAndFolder(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        SingleResult result = application.updateReadAndFolder(sessionID, response, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    /**
     * The assignedScorecard method will assign a Scorecard object to an
     * RPMObject. There are three types of RPMObjects that are amenable for
     * scorecard assignment: 1) Asset; 2) GenericProject and its subobjects; 3)
     * AbstractScope and its subobjects. For more information see those objects
     * documentation. The assumption for the assignment is that the Scorecard is
     * already been created in the design phase and that the RPMObject is
     * already been saved. After the assignment, any change in the Scorecard
     * will not affect the AssignedScorecard.
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param objectToAssign -
     *            the object that you want to assign the scorecard.
     * @param scoreCardToAssign -
     *            the scorecard to assign
     * @return - the object that you wanted to assign with the scorecard
     *         assigned to it
     * @throws RPMException
     */
    public SingleResult assignScoreCard(String sessionID,
            RPMObject objectToAssign, RPMObjectScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
    {
        SingleResult result = application.assignScoreCard(sessionID,
            objectToAssign, objectToAssignScope, scoreCardToAssign, reloadType);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult assignScoreCardToResource(String sessionID, GenericProject project, 
            Resource objectToAssign, ResourceScope objectToAssignScope, 
            Scorecard scoreCardToAssign, ReloadType reloadType) 
    {
        SingleResult result = application.assignScoreCardToResource(sessionID,
            project, objectToAssign, objectToAssignScope, scoreCardToAssign, reloadType);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    /**
     * The publishScorecard publishes a an assignedScorecard. This has two
     * effects: 1) the assignedScorecard is available for portfolio analysis; 2)
     * the assignedScorecard values are been versioned and a user can return to
     * history assignedScorecard values and compare between versions. A user can
     * publish an assignedScorecard as much times as desired, each such publish
     * marks new version. Once an assignedScorecard is been published, it will
     * continue to be as such. A user can deactivate the assignedScorecard using
     * the AssignedScorecardState.
     * 
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param assignedScorecard -
     *            The assignedScorecard to publish.
     * @param scorecardScope -
     *            This scope defines the assignedScorecard elements that will be
     *            loaded and returned. A null will load only the
     *            assignedScorecard.
     * @return AssignedScorecard object. This object is the assignedScorecard
     *         after the publish was done
     * @throws RPMException
     */
    public SingleResult publishScorecard(String sessionID,
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope)
    {
        SingleResult result = application.publishScorecard(sessionID,
            assignedScorecard, scorecardScope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult acceptTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean acceptEETC, boolean acceptForecastDates)
    {
        SingleResult result = application.acceptTaskAssignmentChanges(
            sessionID, taskAssignment, scope, acceptEETC, acceptForecastDates);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult rejectTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean rejectEETC, boolean rejectForecastDates)
    {
        SingleResult result = application.rejectTaskAssignmentChanges(
            sessionID, taskAssignment, scope, rejectEETC, rejectForecastDates);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult copyProposedToPlan(String sessionID,
            WorkElement workElement, WorkElementScope workElementScope)
    {
        SingleResult result = application.copyProposedToPlan(sessionID,
            workElement, workElementScope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult copyAssignmentProposedToPlan(String sessionID,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope)
    {
        SingleResult result = application.copyAssignmentProposedToPlan(
            sessionID, resourceTaskAssignment, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult commitAffectedResources(String sessionID,
            GenericProject genericProject, WorkElementScope workElementScope)
    {
        SingleResult result = application.commitAffectedResources(sessionID,
            genericProject, workElementScope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult replaceProfileWithResource(String sessionID,
            ResourceTaskAssignment rta, ResourceTaskAssignmentScope rtaScope,
            Resource resource)
    {
        SingleResult result = application.replaceProfileWithResource(sessionID,
            rta, rtaScope, resource);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult replaceResourceWithProfile(String sessionID,
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope)
    {
        SingleResult result = application.replaceResourceWithProfile(sessionID,
            assignment, rtaScope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public Result transferUnderScope(String sessionID,
            AbstractAggregateScope scopeElement, WorkElement workElement)
    {
        return application.transferUnderScope(sessionID, scopeElement,
            workElement);
    }

    public Result transferUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope)
    {
        return application.transferUnderWBS(sessionID, workElement,
            abstractScope);
    }

    public Result moveUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope)
    {
        return application.moveUnderWBS(sessionID, workElement, abstractScope);
    }

    public Result moveAndTransferUnderWBS(String sessionID,
            WorkElement workElement, AbstractScope abstractScope)
    {
        return application.moveAndTransferUnderWBS(sessionID, workElement,
            abstractScope);
    }

    public SingleResult convertContainer(String sessionID,
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope) 
    {
        SingleResult result = application.convertContainer(sessionID,
            inputContainer, 
            outputContainer, 
            rpmScope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }

    public SingleResult createObjectFromTemplate(String sessionID,
        RPMObject inputObject, RPMObject template, RPMObjectScope scope)
    {
        SingleResult result = application.createObjectFromTemplate(sessionID,
            inputObject, 
            template, 
            scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public SingleResult saveProjectBaseline(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        SingleResult result = application.saveProjectBaseline(sessionID,
            projectObject, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }
    
    public Result activateEVWeekDelays(String sessionID, int evDelays)
    {
        Result result = application.activateEVWeekDelays(sessionID, evDelays);
        //CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }


    public SingleResult rollupProjectEV(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        SingleResult result = application.rollupProjectEV(sessionID,
            projectObject, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }    
    
    public SingleResult enableProjectEVCalculation(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        SingleResult result = application.enableProjectEVCalculation(sessionID,
            projectObject, scope);
        CyclicReferenceRemover.remove(result.getRpmObject());
        return result;
    }    
 
    
    
    public Result moveBeforeSibling(String sessionID, ScopeElement objecToMove,
            ScopeElement nextSibling)
    {
        return application.moveBeforeSibling(sessionID, objecToMove,
            nextSibling);
    }
    
    public SingleResult lockDocument(String sessionID, GenericDocument documentToLock)
    {
    	SingleResult result = application.lockDocument(sessionID, documentToLock);
    	CyclicReferenceRemover.remove(result.getRpmObject());
    	return result;
    }
    
    public SingleResult unLockDocument(String sessionID, GenericDocument documentToUnlock)
    {
    	SingleResult result = application.unLockDocument(sessionID, documentToUnlock);
    	CyclicReferenceRemover.remove(result.getRpmObject());
    	return result;
    }
}