package com.ibm.rpm.interfaces;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.interfaces.checkpoints.AuthenticateCheckpoint;
import com.ibm.rpm.interfaces.impl.ServerFactory;

/**
 * Design note: deleguates each method to the Authentication implementation
 * chosen by configuration option.
 * 
 * @author legaults
 */
public class Authenticate
{
    private ServerFactory factory;

    public Authenticate(ServerFactory factoryToUse)
    {
        factory = factoryToUse;
    }
    
    private SessionUtility getSessionUtility()
    {
        return factory.getSession();
    }
    
    private Authentication getAuthenticateImplementation()
    {
        return factory.getAuthenticationImplementation();
    }
    
    public Authenticate()
    {
        this(ServerFactory.getInstance());
    }

    public String login(String username, String password, String dsn)
            throws RPMException
    {
        getSessionUtility().validateWebServicesEnabledAndLog("login");
    	AuthenticateCheckpoint.getInstance().validateLogin(username, password);
		
        return getAuthenticateImplementation().login(username, password, dsn);
    }

    public void assignWorkflowToSession(String sessionId, String workflowId)
    throws RPMException
	{
        getSessionUtility().validateWebServicesEnabledAndLog("assignWorkflowToSession");
	    getAuthenticateImplementation().assignWorkflowToSession(sessionId, workflowId);
	}

    public void logout(String sessionID) throws RPMException
    {
        getSessionUtility().validateWebServicesEnabledAndLog("sessionID");
        getAuthenticateImplementation().logout(sessionID);
    }

    public void changePassword(String username, String oldPw, String newPw,
            String dsn) throws RPMException
    {
        getSessionUtility().validateWebServicesEnabledAndLog("changePassword");
        getAuthenticateImplementation().changePassword(username, oldPw, newPw, dsn);
    }
}
