package com.ibm.rpm.interfaces;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.ContractType;
import com.ibm.rpm.applicationadministration.containers.ReconciliationReason;
import com.ibm.rpm.applicationadministration.containers.ScorecardState;
import com.ibm.rpm.applicationadministration.containers.ValueDatafield;
import com.ibm.rpm.applicationadministration.scope.AdministrationModuleScope;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.communications.containers.VoteResponseReason;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.containers.ContractModule;
import com.ibm.rpm.contract.scope.ContractModuleScope;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.scope.CustomPortletScope;
import com.ibm.rpm.financial.containers.FinancialModule;
import com.ibm.rpm.financial.scope.FinancialModuleScope;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scorecard.containers.ResourceAssignedScorecard;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.DocumentSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.NotesMinutesSecurityRole;
import com.ibm.rpm.security.containers.PoolSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.containers.TemplateFolderSecurityRole;
import com.ibm.rpm.security.scope.SecurityGroupScope;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.security.types.ActionItemSecurityRoleType;
import com.ibm.rpm.security.types.ChangeRequestSecurityRoleType;
import com.ibm.rpm.security.types.DefectSecurityRoleType;
import com.ibm.rpm.security.types.DeliverableSecurityRoleType;
import com.ibm.rpm.security.types.DocumentSecurityRoleType;
import com.ibm.rpm.security.types.FolderSecurityRoleType;
import com.ibm.rpm.security.types.IssueSecurityRoleType;
import com.ibm.rpm.security.types.NotesMinutesSecurityRoleType;
import com.ibm.rpm.security.types.PoolSecurityRoleType;
import com.ibm.rpm.security.types.ProjectSecurityRoleType;
import com.ibm.rpm.security.types.RequirementSecurityRoleType;
import com.ibm.rpm.security.types.RiskSecurityRoleType;
import com.ibm.rpm.security.types.ServiceRequestSecurityRoleType;
import com.ibm.rpm.security.types.TaskSecurityRoleType;
import com.ibm.rpm.security.types.TemplateFolderSecurityRoleType;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.wbs.containers.EarnedValues;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.ExternalAction;
import com.ibm.rpm.workflow.containers.ExternalActionCategory;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowCategory;
import com.ibm.rpm.workflow.containers.WorkflowGroup;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.ExternalActionScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;



public class ContainerList2
{
    //TODO Keep declared containers grouped by packages
    //TODO Always have a getter and a setter for each declared container for
    // compatibility purposes

    //Custom Fields
    private CustomField declareCustomField;
    private CustomFieldCategory declareCustomFieldCategory;
    private CustomPortlet declareCustomPortlet;
    private CustomPortletCategory declareCustomPortletCategory;
    private CustomFieldScope declareCustomFieldScope;
    private CustomPortletScope declareCustomPortletScope;
    private CustomFieldAssignment declareAssignedCustomField;
    private TimeCodeScope declaredTimeCodeScope;

    //Contract
    private ContractFolder declareContractFolder;
    private Contract declareContract;
    private ContractChargeSchedule declareContractChargeSchedule;
    private ContractScope declareContractScope;
    private ContractType declareContractType;

    //Additional Modules
    private ContractModule declareContractModule;
    private ContractModuleScope declareContractModuleScope;
    private FinancialModule declareFinancialModule;
    private FinancialModuleScope declareFinancialModuleScope;

    //ResourceTaskAssignment
    private ResourceTaskAssignment declareResourceTaskAssignment;
    private ResourceTaskAssignmentScope declareResourceTaskAssignmentScope;

    //Profile
    private Profile declareProfile;
    private ProfileCompetencyAssignment declareProfileCompetencyAssignment;
    private ProfileSkillAssignment declareProfileSkillAssignment;
    private ProfileScope declareProfileScope;

    //Security
    private SecurityGroup declareSecurityGroup;
    private SecurityGroupScope declareSecurityGroupScope;
    private SecurityRoleCategory declareSecurityRoleCategory;
    private SecurityRole declareSecurityRole;
    private SecurityRoleScope declareSecurityRoleScope;
    private ActionItemSecurityRole declareActionItemSecurityRole;
    private ActionItemSecurityRoleType declareActionItemSecurityRoleType;
    private ChangeRequestSecurityRole declareChangeRequestSecurityRole;
    private ChangeRequestSecurityRoleType declareChangeRequestSecurityRoleType;
    private DefectSecurityRole declareDefectSecurityRole;
    private DefectSecurityRoleType declareDefectSecurityRoleType;
    private DeliverableSecurityRole declareDeliverableSecurityRole;
    private DeliverableSecurityRoleType declareDeliverableSecurityRoleType;
    private DocumentSecurityRole declareDocumentSecurityRole;
    private DocumentSecurityRoleType declareDocumentSecurityRoleType;
    private FolderSecurityRole declareFolderSecurityRole;
    private FolderSecurityRoleType declareFolderSecurityRoleType;
    private IssueSecurityRole declareIssueSecurityRole;
    private IssueSecurityRoleType declareIssueSecurityRoleType;
    private NotesMinutesSecurityRole declareNotesMinutesSecurityRole;
    private NotesMinutesSecurityRoleType declareNotesMinutesSecurityRoleType;
    private PoolSecurityRole declarePoolSecurityRole;
    private PoolSecurityRoleType declarePoolSecurityRoleType;
    private ProjectSecurityRole declareProjectSecurityRole;
    private ProjectSecurityRoleType declareProjectSecurityRoleType;
    private RequirementSecurityRole declareRequirementSecurityRole;
    private RequirementSecurityRoleType declareRequirementSecurityRoleType;
    private RiskSecurityRole declareRiskSecurityRole;
    private RiskSecurityRoleType declareRiskSecurityRoleType;
    private ServiceRequestSecurityRole declareServiceRequestSecurityRole;
    private ServiceRequestSecurityRoleType declareServiceRequestSecurityRoleType;
    private TaskSecurityRole declareTaskSecurityRole;
    private TaskSecurityRoleType declareTaskSecurityRoleType;
    private TemplateFolderSecurityRole declareTemplateFolderSecurityRole;
    private TemplateFolderSecurityRoleType declareTemplateFolderSecurityRoleType;
    

    //Adminsistration
    private AdministrationModule declareAdministrationModule;
    private AdministrationModuleScope declareAdministrationModuleScope;
    private ValueDatafield declareValueDatafield;

    // Timesheet
    private SummaryTimesheetScope declareSummaryTimesheetScope;
    private StepScope declareStepScope;
    private DefaultStepScope declareDefaultStepScope;
    private TimesheetApprovalStatusScope declareTimesheetApprovalStatusScope;
    private PersonalTaskAssignment declarePersonalTaskAssignment;
    private AdministrativeTaskAssignment declareAdministrativeTaskAssignment;
    private StepTimesheet declareStepTimesheet;
    private Step declareStep;
    private SummaryTimesheet declareSummaryTimesheet;
    private Credit declareCredit;
    private CreditTimesheet declareCreditTimesheet;

    // Scorecards
    private ResourceAssignedScorecard declareResourceAssignedScorecard;

    // WBS
    private SimpleNode declareSimpleNode;
    private EarnedValues declareEarnedValues;

    // Workflow
    private WorkflowScope declareWorkflowScope;
    private WorkflowProcess declareWorkflowProcess;
    private WorkflowCategory declareWorkflowCategory;
    private WorkflowGroup declareWorkflowGroup;
    private RunningWorkflowProcess declareRunningWorkflowProcess;
    private AvailableWorkflowProcess declareAvailableWorkflowProcess;
    
    
    //patni specific
    private ExternalActionCategory declareExternalActionCategory;
    private ExternalAction declareExternalAction;
    private ExternalActionScope declareExternalActionScope;

    // Framework.
    private SeverityLevel severityLevel;
    private RPMException rpmException;

    // Financials
    private ReconciliationReason declareReconciliationReason;

    // Scope Management
    private ScopeWorkProduct declareScopeWorkProduct;

    // Communications
    private GenericCommunication declareGenericCommunication;
    private Notification declareNotification;
    private WorkflowStepNotification declareWorkflowStepNotification;
    private StaffingRequest declareStaffingRequest;
    private WorkflowResponse declareWorkflowResponse;
    private ChecklistItem declareChecklistItem;
    private VoteResponse declareVoteResponse;
    private VoteResponseReason declareVoteResponseReason;
    private WorkflowAttachment declareWorkflowAttachment;
    private CommunicationsFolder declareCommunicationsFolder;
    private WorkflowResponseState declareWorkflowResponseState;
    private WorkflowState declareWorkflowState;
    private GenericCommunicationScope declareGenericCommunicationScope;
    private WorkflowResponseScope declareWorkflowResponseScope;

    //ReqPro Containers
    private ReqProConnection declareReqProConnection;
    private ReqProImportStatus declareReqProImportStatus;
    private ReqProIntegrationFolder declareReqProIntegrationFolder;
    private ReqProRequirement declareReqProRequirement;
    private ReqProScopeFolder declareReqProScopeFolder;
    
    /**
     * @return Returns the declareResourceSummaryTimesheetTask
     */
    public PersonalTaskAssignment getResourceTimesheetTaskAssignment()
    {
        return declarePersonalTaskAssignment;
    }

    /**
     * @param declareWbsResource
     *            The declareWbsResource to set.
     */
    public void setResourceTimesheetTaskAssignment(PersonalTaskAssignment declareWbsResource)
    {
        this.declarePersonalTaskAssignment = declareWbsResource;
    }

    /**
     * @return Returns the declareCustomField.
     */
    public CustomField getDeclareCustomField()
    {
        return declareCustomField;
    }

    /**
     * @param declareCustomField
     *            The declareCustomField to set.
     */
    public void setDeclareCustomField(CustomField declareCustomField)
    {
        this.declareCustomField = declareCustomField;
    }

    /**
     * @return Returns the declareCustomFieldCategory.
     */
    public CustomFieldCategory getDeclareCustomFieldCategory()
    {
        return declareCustomFieldCategory;
    }

    /**
     * @param declareCustomFieldCategory
     *            The declareCustomFieldCategory to set.
     */
    public void setDeclareCustomFieldCategory(
            CustomFieldCategory declareCustomFieldCategory)
    {
        this.declareCustomFieldCategory = declareCustomFieldCategory;
    }

    /**
     * @return Returns the declareCustomPortlet.
     */
    public CustomPortlet getDeclareCustomPortlet()
    {
        return declareCustomPortlet;
    }

    /**
     * @param declareCustomPortlet
     *            The declareCustomPortlet to set.
     */
    public void setDeclareCustomPortlet(CustomPortlet declareCustomPortlet)
    {
        this.declareCustomPortlet = declareCustomPortlet;
    }

    /**
     * @return Returns the declareCustomPortletCategory.
     */
    public CustomPortletCategory getDeclareCustomPortletCategory()
    {
        return declareCustomPortletCategory;
    }

    /**
     * @param declareCustomPortletCategory
     *            The declareCustomPortletCategory to set.
     */
    public void setDeclareCustomPortletCategory(
            CustomPortletCategory declareCustomPortletCategory)
    {
        this.declareCustomPortletCategory = declareCustomPortletCategory;
    }

    /**
     * @return Returns the declareCustomFieldScope.
     */
    public CustomFieldScope getDeclareCustomFieldScope()
    {
        return declareCustomFieldScope;
    }

    /**
     * @param declareCustomFieldScope
     *            The declareCustomFieldScope to set.
     */
    public void setDeclareCustomFieldScope(
            CustomFieldScope declareCustomFieldScope)
    {
        this.declareCustomFieldScope = declareCustomFieldScope;
    }


    /**
     * @return Returns the .
     */
    public CustomPortletScope getDeclareCustomPortletScope()
    {
        return declareCustomPortletScope;
    }

    /**
     * @param
     *            The  to set.
     */
    public void setDeclareCustomPortletScope(
            CustomPortletScope declareCustomPortletScope)
    {
        this.declareCustomPortletScope = declareCustomPortletScope;
    }

    /**
     * @return Returns the declareAssignedCustomField.
     */
    public CustomFieldAssignment getDeclareAssignedCustomField()
    {
        return declareAssignedCustomField;
    }
    /**
     * @param declareAssignedCustomField The declareAssignedCustomField to set.
     */
    public void setDeclareAssignedCustomField(
            CustomFieldAssignment declareAssignedCustomField)
    {
        this.declareAssignedCustomField = declareAssignedCustomField;
    }

    /**
     * @return Returns the declaredTimeCodeScope.
     */
    public TimeCodeScope getDeclaredTimeCodeScope()
    {
        return declaredTimeCodeScope;
    }

    /**
     * @param declaredTimeCodeScope
     *            The declaredTimeCodeScope to set.
     */
    public void setDeclaredTimeCodeScope(
            TimeCodeScope declaredTimeCodeScope)
    {
        this.declaredTimeCodeScope = declaredTimeCodeScope;
    }

    /**
     * @return Returns the declareContract.
     */
    public Contract getDeclareContract()
    {
        return declareContract;
    }

    /**
     * @param declareContract
     *            The declareContract to set.
     */
    public void setDeclareContract(Contract declareContract)
    {
        this.declareContract = declareContract;
    }

    /**
     * @return Returns the declareContractChargeSchedule.
     */
    public ContractChargeSchedule getDeclareContractChargeSchedule()
    {
        return declareContractChargeSchedule;
    }

    /**
     * @param declareContractChargeSchedule
     *            The declareContractChargeSchedule to set.
     */
    public void setDeclareContractChargeSchedule(
            ContractChargeSchedule declareContractChargeSchedule)
    {
        this.declareContractChargeSchedule = declareContractChargeSchedule;
    }

    /**
     * @return Returns the declareContractFolder.
     */
    public ContractFolder getDeclareContractFolder()
    {
        return declareContractFolder;
    }

    /**
     * @param declareContractFolder
     *            The declareContractFolder to set.
     */
    public void setDeclareContractFolder(ContractFolder declareContractFolder)
    {
        this.declareContractFolder = declareContractFolder;
    }

    /**
     * @return Returns the declareContractScope.
     */
    public ContractScope getDeclareContractScope()
    {
        return declareContractScope;
    }

    /**
     * @param declareContractScope
     *            The declareContractScope to set.
     */
    public void setDeclareContractScope(ContractScope declareContractScope)
    {
        this.declareContractScope = declareContractScope;
    }

    /**
     * @return Returns the declareContractType.
     */
    public ContractType getDeclareContractType()
    {
        return declareContractType;
    }

    /**
     * @param declareContractType
     *            The declareContractType to set.
     */
    public void setDeclareContractType(
            ContractType declareContractType)
    {
        this.declareContractType = declareContractType;
    }


    public Profile getDeclareProfile()
    {
        return declareProfile;
    }
    public void setDeclareProfile(Profile declareProfile)
    {
        this.declareProfile = declareProfile;
    }
    public ProfileScope getDeclareProfileScope()
    {
        return declareProfileScope;
    }
    public void setDeclareProfileScope(ProfileScope declareProfileScope)
    {
        this.declareProfileScope = declareProfileScope;
    }


    public ResourceTaskAssignment getDeclareResourceTaskAssignment()
    {
        return declareResourceTaskAssignment;
    }
    public void setDeclareResourceTaskAssignment(
            ResourceTaskAssignment declareResourceTaskAssignment)
    {
        this.declareResourceTaskAssignment = declareResourceTaskAssignment;
    }
    public ResourceTaskAssignmentScope getDeclareResourceTaskAssignmentScope()
    {
        return declareResourceTaskAssignmentScope;
    }
    public void setDeclareResourceTaskAssignmentScope(
            ResourceTaskAssignmentScope declareResourceTaskAssignmentScope)
    {
        this.declareResourceTaskAssignmentScope = declareResourceTaskAssignmentScope;
    }

    /**
     * @return Returns the declareDocumentSecurityRole.
     */
    public DocumentSecurityRole getDeclareDocumentSecurityRole()
    {
        return declareDocumentSecurityRole;
    }

    /**
     * @param declareDocumentSecurityRole
     *            The declareDocumentSecurityRole to set.
     */
    public void setDeclareDocumentSecurityRole(DocumentSecurityRole declareDocumentSecurityRole)
    {
        this.declareDocumentSecurityRole = declareDocumentSecurityRole;
    }

    /**
     * @return Returns the declareDocumentSecurityRoleType.
     */
    public DocumentSecurityRoleType getDeclareDocumentSecurityRoleType()
    {
        return declareDocumentSecurityRoleType;
    }

    /**
     * @param declareDocumentSecurityRoleType
     *            The declareDocumentSecurityRoleType to set.
     */
    public void setDeclareDocumentSecurityRoleType(DocumentSecurityRoleType declareDocumentSecurityRoleType)
    {
        this.declareDocumentSecurityRoleType = declareDocumentSecurityRoleType;
    }

    /**
     * @return Returns the declareClientModule.
     */
    public AdministrationModule getDeclareAdministrationModule()
    {
        return declareAdministrationModule;
    }

    /**
     * @param declareAdministrationModule
     *            The declareAdministrationModule to set.
     */
    public void setDeclareAdministrationModule(AdministrationModule declareAdministrationModule)
    {
        this.declareAdministrationModule = declareAdministrationModule;
    }

    /**
     * @return Returns the declareAdministrationModuleScope.
     */
    public AdministrationModuleScope getDeclareAdministrationModuleScope()
    {
        return declareAdministrationModuleScope;
    }

    /**
     * @param declareAdministrationModuleScope
     *            The declareAdministrationModuleScope to set.
     */
    public void setDeclareAdministrationModuleScope(
            AdministrationModuleScope declareAdministrationModuleScope)
    {
        this.declareAdministrationModuleScope = declareAdministrationModuleScope;
    }
    /**
     * @return Returns the declareActivityScope.
     */
    public StepScope getDeclareStepScope()
    {
        return declareStepScope;
    }
    /**
     * @param declareActivityScope The declareActivityScope to set.
     */
    public void setDeclareStepScope(StepScope declareActivityScope)
    {
        this.declareStepScope = declareActivityScope;
    }
    /**
     * @return Returns the declareFavoriteActivityScope.
     */
    public DefaultStepScope getDeclareDefaultStepScope()
    {
        return declareDefaultStepScope;
    }
    /**
     * @param declareFavoriteActivityScope The declareFavoriteActivityScope to set.
     */
    public void setDeclareDefaultStepScope(
            DefaultStepScope declareFavoriteActivityScope)
    {
        this.declareDefaultStepScope = declareFavoriteActivityScope;
    }
    /**
     * @return Returns the declareSummaryTimesheetScope.
     */
    public SummaryTimesheetScope getDeclareSummaryTimesheetScope()
    {
        return declareSummaryTimesheetScope;
    }
    /**
     * @param declareSummaryTimesheetScope The declareSummaryTimesheetScope to set.
     */
    public void setDeclareSummaryTimesheetScope(
            SummaryTimesheetScope declareSummaryTimesheetScope)
    {
        this.declareSummaryTimesheetScope = declareSummaryTimesheetScope;
    }
    /**
     * @return Returns the declareTimesheetApprovalStatusScope.
     */
    public TimesheetApprovalStatusScope getDeclareTimesheetApprovalStatusScope()
    {
        return declareTimesheetApprovalStatusScope;
    }
    /**
     * @param declareTimesheetApprovalStatusScope The declareTimesheetApprovalStatusScope to set.
     */
    public void setDeclareTimesheetApprovalStatusScope(
            TimesheetApprovalStatusScope declareTimesheetApprovalStatusScope)
    {
        this.declareTimesheetApprovalStatusScope = declareTimesheetApprovalStatusScope;
    }


    public ProfileCompetencyAssignment getDeclareProfileCompetencyAssignment()
    {
        return declareProfileCompetencyAssignment;
    }
    public void setDeclareProfileCompetencyAssignment(
            ProfileCompetencyAssignment declareProfileCompetencyAssignment)
    {
        this.declareProfileCompetencyAssignment = declareProfileCompetencyAssignment;
    }
    public ProfileSkillAssignment getDeclareProfileSkillAssignment()
    {
        return declareProfileSkillAssignment;
    }
    public void setDeclareProfileSkillAssignment(
            ProfileSkillAssignment declareProfileSkillAssignment)
    {
        this.declareProfileSkillAssignment = declareProfileSkillAssignment;
    }
    /**
     * @return Returns the declareResourceTimesheetTaskAssignment.
     */
    public PersonalTaskAssignment getDeclarePersonalTaskAssignment()
    {
        return declarePersonalTaskAssignment;
    }
    /**
     * @param declareResourceTimesheetTaskAssignment The declareResourceTimesheetTaskAssignment to set.
     */
    public void setDeclarePersonalTaskAssignment(
            PersonalTaskAssignment declareResourceTimesheetTaskAssignment)
    {
        this.declarePersonalTaskAssignment = declareResourceTimesheetTaskAssignment;
    }

    /**
     * @return Returns the declareSimpleNode.
     */
    public SimpleNode getDeclareSimpleNode()
    {
        return declareSimpleNode;
    }
    /**
     * @param declareSimpleNode The declareSimpleNode to set.
     */
    public void setDeclareSimpleNode(SimpleNode declareSimpleNode)
    {
        this.declareSimpleNode = declareSimpleNode;
    }
    
    /**
     * @return Returns the earnedValues.
     */
    public EarnedValues getEarnedValues()
    {
        return declareEarnedValues;
    }
    /**
     * @param earnedValues The earnedValues to set.
     */
    public void setEarnedValues(EarnedValues earnedValues)
    {
        this.declareEarnedValues = earnedValues;
    }
    /**
     * @return Returns the declareWorkflow.
     */
    public WorkflowProcess getDeclareWorkflowProcess()
    {
        return declareWorkflowProcess;
    }
    /**
     * @param declareWorkflow The declareWorkflow to set.
     */
    public void setDeclareWorkflowProcess(WorkflowProcess declareWorkflowProcess)
    {
        this.declareWorkflowProcess = declareWorkflowProcess;
    }
    /**
     * @return Returns the declareWorkflowCategory.
     */
    public WorkflowCategory getDeclareWorkflowCategory()
    {
        return declareWorkflowCategory;
    }
    /**
     * @param declareWorkflowCategory The declareWorkflowCategory to set.
     */
    public void setDeclareWorkflowCategory(
            WorkflowCategory declareWorkflowCategory)
    {
        this.declareWorkflowCategory = declareWorkflowCategory;
    }
    /**
     * @return Returns the declareWorkflowScope.
     */
    public WorkflowScope getDeclareWorkflowScope()
    {
        return declareWorkflowScope;
    }
    /**
     * @param declareWorkflowScope The declareWorkflowScope to set.
     */
    public void setDeclareWorkflowScope(WorkflowScope declareWorkflowScope)
    {
        this.declareWorkflowScope = declareWorkflowScope;
    }
    /**
     * @return Returns the declareWorkflowGroup.
     */
    public WorkflowGroup getDeclareWorkflowGroup()
    {
        return declareWorkflowGroup;
    }
    /**
     * @param declareWorkflowGroup The declareWorkflowGroup to set.
     */
    public void setDeclareWorkflowGroup(WorkflowGroup declareWorkflowGroup)
    {
        this.declareWorkflowGroup = declareWorkflowGroup;
    }


    public ContractModule getDeclareContractModule()
    {
        return declareContractModule;
    }
    public void setDeclareContractModule(ContractModule declareContractModule)
    {
        this.declareContractModule = declareContractModule;
    }
    public ContractModuleScope getDeclareContractModuleScope()
    {
        return declareContractModuleScope;
    }
    public void setDeclareContractModuleScope(
            ContractModuleScope declareContractModuleScope)
    {
        this.declareContractModuleScope = declareContractModuleScope;
    }
    public FinancialModule getDeclareFinancialModule()
    {
        return declareFinancialModule;
    }
    public void setDeclareFinancialModule(FinancialModule declareFinancialModule)
    {
        this.declareFinancialModule = declareFinancialModule;
    }
    public FinancialModuleScope getDeclareFinancialModuleScope()
    {
        return declareFinancialModuleScope;
    }
    public void setDeclareFinancialModuleScope(
            FinancialModuleScope declareFinancialModuleScope)
    {
        this.declareFinancialModuleScope = declareFinancialModuleScope;
    }
    public SeverityLevel getSeverityLevel()
    {
        return severityLevel;
    }
    public void setSeverityLevel(SeverityLevel securityLevel)
    {
        this.severityLevel = securityLevel;
    }
    public RPMException getRpmException()
    {
        return rpmException;
    }
    public void setRpmException(RPMException rpmException)
    {
        this.rpmException = rpmException;
    }

    /**
     * @return Returns the declareSecurityGroup.
     */
    public SecurityGroup getDeclareSecurityGroup()
    {
        return declareSecurityGroup;
    }

    /**
     * @param declareSecurityGroup
     *            The declareSecurityGroup to set.
     */
    public void setDeclareSecurityGroup(SecurityGroup declareSecurityGroup)
    {
        this.declareSecurityGroup = declareSecurityGroup;
    }

    /**
     * @return Returns the declareSecurityGroupScope.
     */
    public SecurityGroupScope getDeclareSecurityGroupScope()
    {
        return declareSecurityGroupScope;
    }

    /**
     * @param declareSecurityGroupScope
     *            The declareSecurityGroupScope to set.
     */
    public void setDeclareSecurityGroupScope(
            SecurityGroupScope declareSecurityGroupScope)
    {
        this.declareSecurityGroupScope = declareSecurityGroupScope;
    }

    /**
     * @return Returns the declareSecurityRole.
     */
    public SecurityRole getDeclareSecurityRole()
    {
        return declareSecurityRole;
    }

    /**
     * @param declareSecurityRole
     *            The declareSecurityRole to set.
     */
    public void setDeclareSecurityRole(SecurityRole declareSecurityRole)
    {
        this.declareSecurityRole = declareSecurityRole;
    }

    /**
     * @return Returns the declareSecurityRoleCategory.
     */
    public SecurityRoleCategory getDeclareSecurityRoleCategory()
    {
        return declareSecurityRoleCategory;
    }

    /**
     * @param declareSecurityRoleCategory
     *            The declareSecurityRoleCategory to set.
     */
    public void setDeclareSecurityRoleCategory(SecurityRoleCategory declareSecurityRoleCategory)
    {
        this.declareSecurityRoleCategory = declareSecurityRoleCategory;
    }

    /**
     * @return Returns the declareSecurityRoleScope.
     */
    public SecurityRoleScope getDeclareSecurityRoleScope()
    {
        return declareSecurityRoleScope;
    }

    /**
     * @param declareSecurityRoleScope
     *            The declareSecurityRoleScope to set.
     */
    public void setDeclareSecurityRoleScope(
            SecurityRoleScope declareSecurityRoleScope)
    {
        this.declareSecurityRoleScope = declareSecurityRoleScope;
    }
    /**
     * @return Returns the declareReconciliationReason.
     */
    public ReconciliationReason getDeclareReconciliationReason()
    {
        return declareReconciliationReason;
    }
    /**
     * @param declareReconciliationReason The declareReconciliationReason to set.
     */
    public void setDeclareReconciliationReason(
            ReconciliationReason declareReconciliationReason)
    {
        this.declareReconciliationReason = declareReconciliationReason;
    }

    public ValueDatafield getDeclareValueDatafield()
    {
        return declareValueDatafield;
    }
    public void setDeclareValueDatafield(ValueDatafield declareValueDatafield)
    {
        this.declareValueDatafield = declareValueDatafield;
    }

    public ScopeWorkProduct getDeclareScopeWorkProduct()
    {
        return declareScopeWorkProduct;
    }
    public void setDeclareScopeWorkProduct(
            ScopeWorkProduct declareScopeWorkProduct)
    {
        this.declareScopeWorkProduct = declareScopeWorkProduct;
    }
    public AdministrativeTaskAssignment getDeclareAdministrativeTaskAssignment()
    {
        return declareAdministrativeTaskAssignment;
    }
    public void setDeclareAdministrativeTaskAssignment(
            AdministrativeTaskAssignment declareAdministrativeTaskAssignment)
    {
        this.declareAdministrativeTaskAssignment = declareAdministrativeTaskAssignment;
    }
    public StepTimesheet getDeclareStepTimesheet()
    {
        return declareStepTimesheet;
    }
    public void setDeclareStepTimesheet(
            StepTimesheet declareActivityTimesheet)
    {
        this.declareStepTimesheet = declareActivityTimesheet;
    }
    public Step getDeclareStep()
    {
        return declareStep;
    }
    public void setDeclareStep(
            Step declareActivity)
    {
        this.declareStep = declareActivity;
    }
    public SummaryTimesheet getDeclareSummaryTimesheet()
    {
        return declareSummaryTimesheet;
    }
    public void setDeclareSummaryTimesheet(
            SummaryTimesheet declareSummaryTimesheet)
    {
        this.declareSummaryTimesheet = declareSummaryTimesheet;
    }

    public TaskSecurityRole getDeclareTaskSecurityRole()
    {
        return declareTaskSecurityRole;
    }

    public void setDeclareTaskSecurityRole(
        TaskSecurityRole declareTaskSecurityRole)
    {
        this.declareTaskSecurityRole = declareTaskSecurityRole;
    }
    public Credit getDeclareCredit()
    {
        return declareCredit;
    }
    public void setDeclareCredit(Credit declareCredit)
    {
        this.declareCredit = declareCredit;
    }
    public CreditTimesheet getDeclareCreditTimesheet()
    {
        return declareCreditTimesheet;
    }
    public void setDeclareCreditTimesheet(CreditTimesheet declareCreditTimesheet)
    {
        this.declareCreditTimesheet = declareCreditTimesheet;
    }

    public GenericCommunication getDeclareGenericCommunication()
    {
        return declareGenericCommunication;
    }

    public void setDeclareGenericCommunication(GenericCommunication declareGenericCommunication)
    {
        this.declareGenericCommunication = declareGenericCommunication;
    }

    public Notification getDeclareNotification()
    {
        return declareNotification;
    }

    public void setDeclareNotification(Notification declareNotification)
    {
        this.declareNotification = declareNotification;
    }

    public WorkflowStepNotification getDeclareWorkflowStepNotification()
    {
        return declareWorkflowStepNotification;
    }

    public void setDeclareWorkflowSteNotification(WorkflowStepNotification declareWorkflowStepNotification)
    {
        this.declareWorkflowStepNotification = declareWorkflowStepNotification;
    }

    public StaffingRequest getDeclareStaffingRequest()
    {
        return declareStaffingRequest;
    }

    public void setDeclareStaffingRequest(StaffingRequest declareStaffingRequest)
    {
        this.declareStaffingRequest = declareStaffingRequest;
    }

    public WorkflowResponse getDeclareWorkflowResponse()
    {
        return declareWorkflowResponse;
    }

    public void setDeclareWorkflowResponse(WorkflowResponse declareWorkflowResponse)
    {
        this.declareWorkflowResponse = declareWorkflowResponse;
    }
    
    public ChecklistItem getDeclareChecklistItem()
    {
        return declareChecklistItem;
    }

    public void setDeclareChecklistItem(ChecklistItem declareChecklistItem)
    {
        this.declareChecklistItem = declareChecklistItem;
    }
    
    public VoteResponse getDeclareVoteResponse()
    {
        return declareVoteResponse;
    }

    public void setDeclareVoteResponse(VoteResponse declareVoteResponse)
    {
        this.declareVoteResponse = declareVoteResponse;
    }
    
    public VoteResponseReason getDeclareVoteResponseReason()
    {
        return declareVoteResponseReason;
    }

    public void setDeclareVoteResponseReason(VoteResponseReason declareVoteResponseReason)
    {
        this.declareVoteResponseReason = declareVoteResponseReason;
    }

    public WorkflowAttachment getDeclareWorkflowAttachment()
    {
        return declareWorkflowAttachment;
    }

    public void setDeclareWorkflowAttachment(WorkflowAttachment declareWorkflowAttachment)
    {
        this.declareWorkflowAttachment = declareWorkflowAttachment;
    }

    public CommunicationsFolder getDeclareCommunicationsFolder()
    {
        return declareCommunicationsFolder;
    }

    public void setDeclareCommunicationsFolder(CommunicationsFolder declareCommunicationsFolder)
    {
        this.declareCommunicationsFolder = declareCommunicationsFolder;
    }

    public WorkflowResponseState getDeclareWorkflowResponseState()
    {
        return declareWorkflowResponseState;
    }

    public void setDeclareWorkflowResponseState(WorkflowResponseState declareWorkflowResponseState)
    {
        this.declareWorkflowResponseState = declareWorkflowResponseState;
    }

    public WorkflowState getDeclareWorkflowState()
    {
        return declareWorkflowState;
    }

    public void setDeclareWorkflowState(WorkflowState declareWorkflowState)
    {
        this.declareWorkflowState = declareWorkflowState;
    }

    public WorkflowResponseScope getDeclareWorkflowResponseScope()
    {
        return declareWorkflowResponseScope;
    }

    public void setDeclareWorkflowResponseScope(WorkflowResponseScope declareWorkflowResponseScope)
    {
        this.declareWorkflowResponseScope = declareWorkflowResponseScope;
    }

    public GenericCommunicationScope getDeclareGenericCommunicationScope()
    {
        return declareGenericCommunicationScope;
    }

    public void setDeclareGenericCommunicationScope(GenericCommunicationScope declareGenericCommunicationScope)
    {
        this.declareGenericCommunicationScope = declareGenericCommunicationScope;
    }
    /**
     * @return Returns the declareRunningWorkflowProcess.
     */
    public RunningWorkflowProcess getDeclareRunningWorkflowProcess()
    {
        return declareRunningWorkflowProcess;
    }
    /**
     * @param declareRunningWorkflowProcess The declareRunningWorkflowProcess to set.
     */
    public void setDeclareRunningWorkflowProcess(
            RunningWorkflowProcess declareRunningWorkflowProcess)
    {
        this.declareRunningWorkflowProcess = declareRunningWorkflowProcess;
    }
    /**
     * @return Returns the declareAvailableWorkflowProcess.
     */
    public AvailableWorkflowProcess getDeclareAvailableWorkflowProcess()
    {
        return declareAvailableWorkflowProcess;
    }
    /**
     * @param declareAvailableWorkflowProcess The declareAvailableWorkflowProcess to set.
     */
    public void setDeclareAvailableWorkflowProcess(
            AvailableWorkflowProcess declareAvailableWorkflowProcess)
    {
        this.declareAvailableWorkflowProcess = declareAvailableWorkflowProcess;
    }
    
	/**
	 * @return Returns the declareReqProConnection.
	 */
	public ReqProConnection getDeclareReqProConnection() {
		return declareReqProConnection;
	}
	/**
	 * @param declareReqProConnection The declareReqProConnection to set.
	 */
	public void setDeclareReqProConnection(
			ReqProConnection declareReqProConnection) {
		this.declareReqProConnection = declareReqProConnection;
	}
	/**
	 * @return Returns the declareReqProImportStatus.
	 */
	public ReqProImportStatus getDeclareReqProImportStatus() {
		return declareReqProImportStatus;
	}
	/**
	 * @param declareReqProImportStatus The declareReqProImportStatus to set.
	 */
	public void setDeclareReqProImportStatus(
			ReqProImportStatus declareReqProImportStatus) {
		this.declareReqProImportStatus = declareReqProImportStatus;
	}
	/**
	 * @return Returns the declareReqProIntegrationFolder.
	 */
	public ReqProIntegrationFolder getDeclareReqProIntegrationFolder() {
		return declareReqProIntegrationFolder;
	}
	/**
	 * @param declareReqProIntegrationFolder The declareReqProIntegrationFolder to set.
	 */
	public void setDeclareReqProIntegrationFolder(
			ReqProIntegrationFolder declareReqProIntegrationFolder) {
		this.declareReqProIntegrationFolder = declareReqProIntegrationFolder;
	}
	/**
	 * @return Returns the declareReqProRequirement.
	 */
	public ReqProRequirement getDeclareReqProRequirement() {
		return declareReqProRequirement;
	}
	/**
	 * @param declareReqProRequirement The declareReqProRequirement to set.
	 */
	public void setDeclareReqProRequirement(
			ReqProRequirement declareReqProRequirement) {
		this.declareReqProRequirement = declareReqProRequirement;
	}
	/**
	 * @return Returns the declareReqProScopeFolder.
	 */
	public ReqProScopeFolder getDeclareReqProScopeFolder() {
		return declareReqProScopeFolder;
	}
	/**
	 * @param declareReqProScopeFolder The declareReqProScopeFolder to set.
	 */
	public void setDeclareReqProScopeFolder(
			ReqProScopeFolder declareReqProScopeFolder) {
		this.declareReqProScopeFolder = declareReqProScopeFolder;
	}
	
	
	/**
	 * @return Returns the declareActionItemSecurityRole.
	 */
	public ActionItemSecurityRole getDeclareActionItemSecurityRole() {
		return declareActionItemSecurityRole;
	}
	/**
	 * @param declareActionItemSecurityRole The declareActionItemSecurityRole to set.
	 */
	public void setDeclareActionItemSecurityRole(
			ActionItemSecurityRole declareActionItemSecurityRole) {
		this.declareActionItemSecurityRole = declareActionItemSecurityRole;
	}
	/**
	 * @return Returns the declareActionItemSecurityRoleType.
	 */
	public ActionItemSecurityRoleType getDeclareActionItemSecurityRoleType() {
		return declareActionItemSecurityRoleType;
	}
	/**
	 * @param declareActionItemSecurityRoleType The declareActionItemSecurityRoleType to set.
	 */
	public void setDeclareActionItemSecurityRoleType(
			ActionItemSecurityRoleType declareActionItemSecurityRoleType) {
		this.declareActionItemSecurityRoleType = declareActionItemSecurityRoleType;
	}
	/**
	 * @return Returns the declareChangeRequestSecurityRole.
	 */
	public ChangeRequestSecurityRole getDeclareChangeRequestSecurityRole() {
		return declareChangeRequestSecurityRole;
	}
	/**
	 * @param declareChangeRequestSecurityRole The declareChangeRequestSecurityRole to set.
	 */
	public void setDeclareChangeRequestSecurityRole(
			ChangeRequestSecurityRole declareChangeRequestSecurityRole) {
		this.declareChangeRequestSecurityRole = declareChangeRequestSecurityRole;
	}
	/**
	 * @return Returns the declareChangeRequestSecurityRoleType.
	 */
	public ChangeRequestSecurityRoleType getDeclareChangeRequestSecurityRoleType() {
		return declareChangeRequestSecurityRoleType;
	}
	/**
	 * @param declareChangeRequestSecurityRoleType The declareChangeRequestSecurityRoleType to set.
	 */
	public void setDeclareChangeRequestSecurityRoleType(
			ChangeRequestSecurityRoleType declareChangeRequestSecurityRoleType) {
		this.declareChangeRequestSecurityRoleType = declareChangeRequestSecurityRoleType;
	}
	/**
	 * @return Returns the declareDefectSecurityRole.
	 */
	public DefectSecurityRole getDeclareDefectSecurityRole() {
		return declareDefectSecurityRole;
	}
	/**
	 * @param declareDefectSecurityRole The declareDefectSecurityRole to set.
	 */
	public void setDeclareDefectSecurityRole(
			DefectSecurityRole declareDefectSecurityRole) {
		this.declareDefectSecurityRole = declareDefectSecurityRole;
	}
	/**
	 * @return Returns the declareDefectSecurityRoleType.
	 */
	public DefectSecurityRoleType getDeclareDefectSecurityRoleType() {
		return declareDefectSecurityRoleType;
	}
	/**
	 * @param declareDefectSecurityRoleType The declareDefectSecurityRoleType to set.
	 */
	public void setDeclareDefectSecurityRoleType(
			DefectSecurityRoleType declareDefectSecurityRoleType) {
		this.declareDefectSecurityRoleType = declareDefectSecurityRoleType;
	}
	/**
	 * @return Returns the declareDeliverableSecurityRole.
	 */
	public DeliverableSecurityRole getDeclareDeliverableSecurityRole() {
		return declareDeliverableSecurityRole;
	}
	/**
	 * @param declareDeliverableSecurityRole The declareDeliverableSecurityRole to set.
	 */
	public void setDeclareDeliverableSecurityRole(
			DeliverableSecurityRole declareDeliverableSecurityRole) {
		this.declareDeliverableSecurityRole = declareDeliverableSecurityRole;
	}
	/**
	 * @return Returns the declareDeliverableSecurityRoleType.
	 */
	public DeliverableSecurityRoleType getDeclareDeliverableSecurityRoleType() {
		return declareDeliverableSecurityRoleType;
	}
	/**
	 * @param declareDeliverableSecurityRoleType The declareDeliverableSecurityRoleType to set.
	 */
	public void setDeclareDeliverableSecurityRoleType(
			DeliverableSecurityRoleType declareDeliverableSecurityRoleType) {
		this.declareDeliverableSecurityRoleType = declareDeliverableSecurityRoleType;
	}
	/**
	 * @return Returns the declareFolderSecurityRole.
	 */
	public FolderSecurityRole getDeclareFolderSecurityRole() {
		return declareFolderSecurityRole;
	}
	/**
	 * @param declareFolderSecurityRole The declareFolderSecurityRole to set.
	 */
	public void setDeclareFolderSecurityRole(
			FolderSecurityRole declareFolderSecurityRole) {
		this.declareFolderSecurityRole = declareFolderSecurityRole;
	}
	/**
	 * @return Returns the declareFolderSecurityRoleType.
	 */
	public FolderSecurityRoleType getDeclareFolderSecurityRoleType() {
		return declareFolderSecurityRoleType;
	}
	/**
	 * @param declareFolderSecurityRoleType The declareFolderSecurityRoleType to set.
	 */
	public void setDeclareFolderSecurityRoleType(
			FolderSecurityRoleType declareFolderSecurityRoleType) {
		this.declareFolderSecurityRoleType = declareFolderSecurityRoleType;
	}
	/**
	 * @return Returns the declareIssueSecurityRole.
	 */
	public IssueSecurityRole getDeclareIssueSecurityRole() {
		return declareIssueSecurityRole;
	}
	/**
	 * @param declareIssueSecurityRole The declareIssueSecurityRole to set.
	 */
	public void setDeclareIssueSecurityRole(
			IssueSecurityRole declareIssueSecurityRole) {
		this.declareIssueSecurityRole = declareIssueSecurityRole;
	}
	/**
	 * @return Returns the declareIssueSecurityRoleType.
	 */
	public IssueSecurityRoleType getDeclareIssueSecurityRoleType() {
		return declareIssueSecurityRoleType;
	}
	/**
	 * @param declareIssueSecurityRoleType The declareIssueSecurityRoleType to set.
	 */
	public void setDeclareIssueSecurityRoleType(
			IssueSecurityRoleType declareIssueSecurityRoleType) {
		this.declareIssueSecurityRoleType = declareIssueSecurityRoleType;
	}
	/**
	 * @return Returns the declareNotesMinutesSecurityRole.
	 */
	public NotesMinutesSecurityRole getDeclareNotesMinutesSecurityRole() {
		return declareNotesMinutesSecurityRole;
	}
	/**
	 * @param declareNotesMinutesSecurityRole The declareNotesMinutesSecurityRole to set.
	 */
	public void setDeclareNotesMinutesSecurityRole(
			NotesMinutesSecurityRole declareNotesMinutesSecurityRole) {
		this.declareNotesMinutesSecurityRole = declareNotesMinutesSecurityRole;
	}
	/**
	 * @return Returns the declareNotesMinutesSecurityRoleType.
	 */
	public NotesMinutesSecurityRoleType getDeclareNotesMinutesSecurityRoleType() {
		return declareNotesMinutesSecurityRoleType;
	}
	/**
	 * @param declareNotesMinutesSecurityRoleType The declareNotesMinutesSecurityRoleType to set.
	 */
	public void setDeclareNotesMinutesSecurityRoleType(
			NotesMinutesSecurityRoleType declareNotesMinutesSecurityRoleType) {
		this.declareNotesMinutesSecurityRoleType = declareNotesMinutesSecurityRoleType;
	}
	/**
	 * @return Returns the declarePoolSecurityRole.
	 */
	public PoolSecurityRole getDeclarePoolSecurityRole() {
		return declarePoolSecurityRole;
	}
	/**
	 * @param declarePoolSecurityRole The declarePoolSecurityRole to set.
	 */
	public void setDeclarePoolSecurityRole(
			PoolSecurityRole declarePoolSecurityRole) {
		this.declarePoolSecurityRole = declarePoolSecurityRole;
	}
	/**
	 * @return Returns the declarePoolSecurityRoleType.
	 */
	public PoolSecurityRoleType getDeclarePoolSecurityRoleType() {
		return declarePoolSecurityRoleType;
	}
	/**
	 * @param declarePoolSecurityRoleType The declarePoolSecurityRoleType to set.
	 */
	public void setDeclarePoolSecurityRoleType(
			PoolSecurityRoleType declarePoolSecurityRoleType) {
		this.declarePoolSecurityRoleType = declarePoolSecurityRoleType;
	}
	/**
	 * @return Returns the declareProjectSecurityRole.
	 */
	public ProjectSecurityRole getDeclareProjectSecurityRole() {
		return declareProjectSecurityRole;
	}
	/**
	 * @param declareProjectSecurityRole The declareProjectSecurityRole to set.
	 */
	public void setDeclareProjectSecurityRole(
			ProjectSecurityRole declareProjectSecurityRole) {
		this.declareProjectSecurityRole = declareProjectSecurityRole;
	}
	/**
	 * @return Returns the declareProjectSecurityRoleType.
	 */
	public ProjectSecurityRoleType getDeclareProjectSecurityRoleType() {
		return declareProjectSecurityRoleType;
	}
	/**
	 * @param declareProjectSecurityRoleType The declareProjectSecurityRoleType to set.
	 */
	public void setDeclareProjectSecurityRoleType(
			ProjectSecurityRoleType declareProjectSecurityRoleType) {
		this.declareProjectSecurityRoleType = declareProjectSecurityRoleType;
	}
	/**
	 * @return Returns the declareRequirementSecurityRole.
	 */
	public RequirementSecurityRole getDeclareRequirementSecurityRole() {
		return declareRequirementSecurityRole;
	}
	/**
	 * @param declareRequirementSecurityRole The declareRequirementSecurityRole to set.
	 */
	public void setDeclareRequirementSecurityRole(
			RequirementSecurityRole declareRequirementSecurityRole) {
		this.declareRequirementSecurityRole = declareRequirementSecurityRole;
	}
	/**
	 * @return Returns the declareRequirementSecurityRoleType.
	 */
	public RequirementSecurityRoleType getDeclareRequirementSecurityRoleType() {
		return declareRequirementSecurityRoleType;
	}
	/**
	 * @param declareRequirementSecurityRoleType The declareRequirementSecurityRoleType to set.
	 */
	public void setDeclareRequirementSecurityRoleType(
			RequirementSecurityRoleType declareRequirementSecurityRoleType) {
		this.declareRequirementSecurityRoleType = declareRequirementSecurityRoleType;
	}
	/**
	 * @return Returns the declareRiskSecurityRole.
	 */
	public RiskSecurityRole getDeclareRiskSecurityRole() {
		return declareRiskSecurityRole;
	}
	/**
	 * @param declareRiskSecurityRole The declareRiskSecurityRole to set.
	 */
	public void setDeclareRiskSecurityRole(
			RiskSecurityRole declareRiskSecurityRole) {
		this.declareRiskSecurityRole = declareRiskSecurityRole;
	}
	/**
	 * @return Returns the declareRiskSecurityRoleType.
	 */
	public RiskSecurityRoleType getDeclareRiskSecurityRoleType() {
		return declareRiskSecurityRoleType;
	}
	/**
	 * @param declareRiskSecurityRoleType The declareRiskSecurityRoleType to set.
	 */
	public void setDeclareRiskSecurityRoleType(
			RiskSecurityRoleType declareRiskSecurityRoleType) {
		this.declareRiskSecurityRoleType = declareRiskSecurityRoleType;
	}
	/**
	 * @return Returns the declareServiceRequestSecurityRole.
	 */
	public ServiceRequestSecurityRole getDeclareServiceRequestSecurityRole() {
		return declareServiceRequestSecurityRole;
	}
	/**
	 * @param declareServiceRequestSecurityRole The declareServiceRequestSecurityRole to set.
	 */
	public void setDeclareServiceRequestSecurityRole(
			ServiceRequestSecurityRole declareServiceRequestSecurityRole) {
		this.declareServiceRequestSecurityRole = declareServiceRequestSecurityRole;
	}
	/**
	 * @return Returns the declareServiceRequestSecurityRoleType.
	 */
	public ServiceRequestSecurityRoleType getDeclareServiceRequestSecurityRoleType() {
		return declareServiceRequestSecurityRoleType;
	}
	/**
	 * @param declareServiceRequestSecurityRoleType The declareServiceRequestSecurityRoleType to set.
	 */
	public void setDeclareServiceRequestSecurityRoleType(
			ServiceRequestSecurityRoleType declareServiceRequestSecurityRoleType) {
		this.declareServiceRequestSecurityRoleType = declareServiceRequestSecurityRoleType;
	}
	/**
	 * @return Returns the declareTaskSecurityRoleType.
	 */
	public TaskSecurityRoleType getDeclareTaskSecurityRoleType() {
		return declareTaskSecurityRoleType;
	}
	/**
	 * @param declareTaskSecurityRoleType The declareTaskSecurityRoleType to set.
	 */
	public void setDeclareTaskSecurityRoleType(
			TaskSecurityRoleType declareTaskSecurityRoleType) {
		this.declareTaskSecurityRoleType = declareTaskSecurityRoleType;
	}
	/**
	 * @return Returns the declareTemplateFolderSecurityRole.
	 */
	public TemplateFolderSecurityRole getDeclareTemplateFolderSecurityRole() {
		return declareTemplateFolderSecurityRole;
	}
	/**
	 * @param declareTemplateFolderSecurityRole The declareTemplateFolderSecurityRole to set.
	 */
	public void setDeclareTemplateFolderSecurityRole(
			TemplateFolderSecurityRole declareTemplateFolderSecurityRole) {
		this.declareTemplateFolderSecurityRole = declareTemplateFolderSecurityRole;
	}
	/**
	 * @return Returns the declareTemplateFolderSecurityRoleType.
	 */
	public TemplateFolderSecurityRoleType getDeclareTemplateFolderSecurityRoleType() {
		return declareTemplateFolderSecurityRoleType;
	}
	/**
	 * @param declareTemplateFolderSecurityRoleType The declareTemplateFolderSecurityRoleType to set.
	 */
	public void setDeclareTemplateFolderSecurityRoleType(
			TemplateFolderSecurityRoleType declareTemplateFolderSecurityRoleType) {
		this.declareTemplateFolderSecurityRoleType = declareTemplateFolderSecurityRoleType;
	}

    public ExternalAction getDeclareExternalAction()
    {
        return declareExternalAction;
    }

    public void setDeclareExternalAction(ExternalAction declareExternalAction)
    {
        this.declareExternalAction = declareExternalAction;
    }

    public ExternalActionCategory getDeclareExternalActionCategory()
    {
        return declareExternalActionCategory;
    }

    public void setDeclareExternalActionCategory(
            ExternalActionCategory declareExternalActionCategory)
    {
        this.declareExternalActionCategory = declareExternalActionCategory;
    }

    public ExternalActionScope getDeclareExternalActionScope()
    {
        return declareExternalActionScope;
    }

    public void setDeclareExternalActionScope(
            ExternalActionScope declareExternalActionScope)
    {
        this.declareExternalActionScope = declareExternalActionScope;
    }

    public ResourceAssignedScorecard getDeclareResourceAssignedScorecard()
    {
        return declareResourceAssignedScorecard;
    }

    public void setDeclareResourceAssignedScorecard(
            ResourceAssignedScorecard declareResourceAssignedScorecard)
    {
        this.declareResourceAssignedScorecard = declareResourceAssignedScorecard;
    }
}
