package com.ibm.rpm.interfaces.impl;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.util.EnvironmentUtil;
import com.ibm.rpm.framework.util.Manager;

public class ConnectionTestServlet extends HttpServlet implements Servlet
{
    private static final long serialVersionUID = -6144421124032309035L;

    private static final String DATASOURCE_NAME = "jdbc/RPMDATASOURCE";

    private static Log logger = LogFactory.getLog(ConnectionTestServlet.class);

    private final static boolean ENABLE_VARIABLE_STATUS_DISPLAY = false;
    
    public ConnectionTestServlet()
    {
        super();
    }

    /**
     * Accesses application datasource and tries to obtain a connection from it.
     * Also checks database schema version. Sends the result as a simple report
     * in HTML format on the response outputsream.
     */
    protected void service(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        PrintWriter pw = new PrintWriter(response.getOutputStream());
        try
        {
            response.setContentType("text/html");
            pw.write("<HTML><HEAD></HEAD><BODY>");
            runSelfTest(pw);
            if(ENABLE_VARIABLE_STATUS_DISPLAY) 
            {
                getConnectionParameters(pw);
            }
        }
        finally
        {
            pw.write("</BODY></HTML>");
            pw.flush();
            pw.close();
            response.getOutputStream().flush();
            response.getOutputStream().close();
        }
    }

    private void getConnectionParameters(PrintWriter pw)
    {
        RpmOptions options = ServerFactory.getInstance().getCurrentOptions();
        
         printOption("webServicesSessionTimeoutDelayInSeconds", ""  + options.getSessionTimeoutInSeconds(), pw);
         printOption("webServicesUseSessionTimeout", "" + options.isSessionTimeoutUsed(), pw);
         printOption("webServicesUseApiSecurityFlagOnLogin", "" + options.isSecurityFlagUsed(), pw);
         printOption("webServicesEnabled", "" + options.isWebServicesEnabled(), pw);
         printOption("webServicesSecurityEnabled", "" + options.isWebServicesSecurityEnabled(), pw);
         printOption("useLdapAuthentication", "" + options.isLdapUsed(), pw);
         printOption("ldapConfiguration", options.getLdapConfiguration(), pw);
    }
    
    private void printOption(String name, String value, PrintWriter pw)
    {
        pw.write("<p>" + name + " = " + value + "</p>");
    }

    private void runSelfTest(PrintWriter pw)
    {
        Connection connection = null;
        try
        {
            MessageContext messageContext = new MessageContext(
                ServerFactory.getInstance());
            // Checks connection to database.
            DataSource datasource = (DataSource) EnvironmentUtil
                    .read(DATASOURCE_NAME);
            messageContext.addDatasource(datasource);
            connection = Manager.getConnection(messageContext);
            if (connection != null)
            {
                // Reports connection success.
                pw.write("Connection to DataSource '" + DATASOURCE_NAME
                        + "' is successful.");
                // Checks database compatibility.
                DatabaseCompatibility compatibility = new DatabaseCompatibility(
                        messageContext);
                // Reports database compatibility.
                pw.write("<BR>");
                pw.write(compatibility.getMessage());
            }
        }
        catch (Exception e)
        {
            String msg = "Error connecting to the datasource '"
                + DATASOURCE_NAME + "': " + e.toString() + '.';
            pw.write(msg);
        }
        finally
        {
            try
            {
                if (connection != null)
                    Manager.commitAndCloseConnection(null, connection);
            }
            catch (Exception e)
            {
                logger.error(e.getMessage(), e);
            }
        }
    }

    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        this.service(request, response);
    }

    protected void doPost(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException
    {
        this.service(request, response);
    }
}