package com.ibm.rpm.interfaces.impl;

import com.ibm.rpm.document.transfer.DocumentTransferServlet;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.SessionCheckoutsDAO;
import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.interfaces.Authenticate;
import com.ibm.rpm.interfaces.Authentication;
import com.ibm.rpm.interfaces.InternalServices;
import com.ibm.rpm.interfaces.SessionUtility;


public class ServerFactory
{

    private static ServerFactory instance = null;

    private SessionUtility session = null;
    
    private ManagerCaller managerCaller;
    
    private RpmOptions currentOptions;
    
    private Application applicationInterface;

    private InternalServices internalServices;
    
    private Authenticate authenticateInterface;

    private Authentication authenticationImplementation;
    
    private SessionCheckoutsDAO sessionCheckoutDAO;
    
    private DocumentTransferServlet transferServlet;

    public ServerFactory()
    {
        this(RpmOptions.getDefaultOptions());
    }
    
    public static void restartServerFactory()
    {
        instance = new ServerFactory();
    }

    public ServerFactory(RpmOptions optionsToUse)
    {
        currentOptions = optionsToUse;
        currentOptions.getClass();
        session = new DefaultSessionUtility(this);
        managerCaller = new ManagerCaller(this);
        sessionCheckoutDAO = new SessionCheckoutsDAO(this);
        transferServlet = new DocumentTransferServlet(this);
        applicationInterface = new Application(this);
        authenticateInterface = new Authenticate(this);
        internalServices = new InternalServices(this); 
    }
    
    public final static ServerFactory getInstance()
    {
        if (instance == null)
            instance = new ServerFactory();
        return instance;
    }

    public Application getApplicationInterface()
    {
        return applicationInterface;
    }

    public Authenticate getAuthenticateInterface()
    {
        return authenticateInterface;
    }

    public InternalServices getInternalServices()
    {
    	return internalServices;
    }
    
    public RpmOptions getCurrentOptions()
    {
        return currentOptions;
    }

    public ManagerCaller getManagerCaller()
    {
        return managerCaller;
    }

    public SessionUtility getSession()
    {
        return session;
    }

    public Authentication getAuthenticationImplementation()
    {
        // Initializes the default implementation.
        if( authenticationImplementation == null )
        {
            if (getCurrentOptions().isLdapUsed() == true)
            {
                authenticationImplementation = new LdapAuthentication(this);
            }
            else
            {
                authenticationImplementation = new DatabaseAuthentication(this);
            }
        }

        return authenticationImplementation;
    }

    public SessionCheckoutsDAO getSessionCheckoutDAO()
    {
        return sessionCheckoutDAO;
    }

    public DocumentTransferServlet getTransferServlet()
    {
        return transferServlet;
    }

    public void register(SessionUtility overrideSessionUtility)
    {
        session = overrideSessionUtility;
    }

}
