/*
 * Created on 7-Oct-2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class LanguageProficiencyCheckpoint extends AbstractCheckpoint
{
    protected static LanguageProficiencyCheckpoint instance = new LanguageProficiencyCheckpoint();

    public static LanguageProficiencyCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, Resource.class);

        if (!GenericValidator.validateInstance(rpmObject, LanguageProficiency.class,
            msgCtx))
            return;

        // Initialize
        LanguageProficiency mainLanguageProficiency = (LanguageProficiency) rpmObject;
        
        // Check for mandatory field
        GenericValidator.validateMandatory(mainLanguageProficiency,
            ValidationConstants.LANGUAGEPROFICIENCY_LANGUAGE_FIELD,
            mainLanguageProficiency.getLanguage(), msgCtx);
        GenericValidator.validateMandatory(mainLanguageProficiency,
            ValidationConstants.LANGUAGEPROFICIENCY_READ_LEVEL_FIELD,
            mainLanguageProficiency.getReadLevel(), msgCtx);
        GenericValidator.validateMandatory(mainLanguageProficiency,
            ValidationConstants.LANGUAGEPROFICIENCY_SPEAK_LEVEL_FIELD,
            mainLanguageProficiency.getSpeakLevel(), msgCtx);
        GenericValidator.validateMandatory(mainLanguageProficiency,
            ValidationConstants.LANGUAGEPROFICIENCY_WRITE_LEVEL_FIELD,
            mainLanguageProficiency.getWriteLevel(), msgCtx);
        GenericValidator.validateMandatory(mainLanguageProficiency,
            ValidationConstants.LANGUAGEPROFICIENCY_DEFAULT_LANGUAGE_FIELD,
            mainLanguageProficiency.getDefaultLanguage(), msgCtx);

        // Field validation: readLevel
        if (mainLanguageProficiency.testReadLevelModified())
        {
            GenericValidator.validateRange(mainLanguageProficiency,
                "readLevel", mainLanguageProficiency.getReadLevel(),
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN,
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX, msgCtx);
        }

        // Field validation: speakLevel
        if (mainLanguageProficiency.testSpeakLevelModified())
        {
            GenericValidator.validateRange(mainLanguageProficiency,
                "speakLevel", mainLanguageProficiency.getSpeakLevel(),
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN,
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX, msgCtx);
        }

        // Field validation: writeLevel
        if (mainLanguageProficiency.testWriteLevelModified())
        {
            GenericValidator.validateRange(mainLanguageProficiency,
                "writeLevel", mainLanguageProficiency.getWriteLevel(),
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN,
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX, msgCtx);
        }
    }
}