/*
 * Created on Sep 15, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;

/**
 * @author dominiqu
 */
public class PoolCheckpoint extends AbstractCheckpoint
{
    private static final List POOL_PARENTS = new ArrayList();
    static
    {
        POOL_PARENTS.add(PoolModule.class);
        POOL_PARENTS.add(Pool.class);
    }
    
    protected static PoolCheckpoint instance = new PoolCheckpoint();

    public static PoolCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, POOL_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, Pool.class, msgCtx))
            return;

        // Initialize
        Pool mainPool = (Pool) rpmObject;

        // Validate nesting level - not yet tested thus commented
        //GenericValidator.validateNestingLevel(mainPool, "TMT_POOLS",
        //    ValidationConstants.RESOURCEMANAGEMENT_NESTINGLEVEL_MAX, msgCtx);
        
        // Field validation: address
        if (mainPool.testAddressModified())
        {
            GenericValidator.validateMaxLength(mainPool, "address",
                mainPool.getAddress(), ValidationConstants.POOL_ADDRESS_MAX, msgCtx);
        }

        // Field validation: companyBackground
        if (mainPool.testCompanyBackgroundModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "companyBackground", mainPool.getCompanyBackground(),
                ValidationConstants.POOL_COMPANYBACKGROUND_MAX, msgCtx);
        }

        // Field validation: contactName
        if (mainPool.testContactNameModified())
        {
            GenericValidator.validateMaxLength(mainPool, "contactName",
                mainPool.getContactName(), ValidationConstants.POOL_CONTACTNAME_MAX,
                msgCtx);
        }

        // Field validation: description
        if (mainPool.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainPool, "description",
                mainPool.getDescription(), ValidationConstants.POOL_DESCRIPTION_MAX,
                msgCtx);
        }

        // Field validation: email
        if (mainPool.testEmailModified())
        {
            GenericValidator.validateMaxLength(mainPool, "email",
                mainPool.getEmail(), ValidationConstants.POOL_EMAIL_MAX, msgCtx);
        }

        // Field validation: fax
        if (mainPool.testFaxModified())
        {
            GenericValidator.validateMaxLength(mainPool, "fax", mainPool
                .getFax(), ValidationConstants.POOL_FAX_MAX, msgCtx);
        }

        // Field validation: industriesServed
        if (mainPool.testIndustriesServedModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "industriesServed", mainPool.getIndustriesServed(),
                ValidationConstants.POOL_INDUSTRIESSERVED_MAX, msgCtx);
        }

        // Field validation: locationsServed
        if (mainPool.testLocationsServedModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "locationsServed", mainPool.getLocationsServed(),
                ValidationConstants.POOL_LOCATIONSSERVED_MAX, msgCtx);
        }

        // Field validation: name
        if (mainPool.testNameModified())
        {
            GenericValidator.validateMaxLength(mainPool, "name", mainPool
                .getName(), ValidationConstants.POOL_NAME_MAX, msgCtx);
            GenericValidator.validateCharacters(mainPool, "name",
                mainPool.getName(),
                com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS, msgCtx);
        }

        // Field validation: organizationName
        if (mainPool.testOrganizationNameModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "organizationName", mainPool.getOrganizationName(),
                ValidationConstants.POOL_ORGANIZATIONNAME_MAX, msgCtx);
        }

        // Field validation: ownership
        if (mainPool.testOwnershipModified())
        {
            GenericValidator.validateMaxLength(mainPool, "ownership",
                mainPool.getOwnership(), ValidationConstants.POOL_OWNERSHIP_MAX, msgCtx);
        }

        // Field validation: productsExpertiseUsed
        if (mainPool.testProductsExpertiseUsedModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "productsExpertiseUsed", mainPool.getProductsExpertiseUsed(),
                ValidationConstants.POOL_PRODUCTSEXPERTISEUSED_MAX, msgCtx);
        }

        // Field validation: servicesOffered
        if (mainPool.testServicesOfferedModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "servicesOffered", mainPool.getServicesOffered(),
                ValidationConstants.POOL_SERVICESOFFERED_MAX, msgCtx);
        }

        // Field validation: servicesSpecialized
        if (mainPool.testServicesSpecializedModified())
        {
            GenericValidator.validateMaxLength(mainPool,
                "servicesSpecialized", mainPool.getServicesSpecialized(),
                ValidationConstants.POOL_SERVICESSPECIALIZED_MAX, msgCtx);
        }

        // Field validation: telephone
        if (mainPool.testTelephoneModified())
        {
            GenericValidator.validateMaxLength(mainPool, "telephone",
                mainPool.getTelephone(), ValidationConstants.POOL_TELEPHONE_MAX, msgCtx);
        }
        
        // on update, the maskEmployeeInfo and publish fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(mainPool,
            ValidationConstants.POOL_MASK_EMPLOYEE_INFO_FIELD, mainPool
                .getMaskEmployeeInfo(), msgCtx);
        
        GenericValidator.validateDefaultValueUpdate(mainPool,
            ValidationConstants.POOL_PUBLISH_FIELD, mainPool
                .getPublish(), msgCtx);

        // TODO Verify that pools == null if this pool is the 28th in the
        // hierarchy - Requires a select from the database
    }
    
}
