/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;

/**
 * @author dominiqu
 */
public class PoolReferenceCheckpoint extends AbstractCheckpoint
{
    private static final Class REFERENCE_PARENTS = Pool.class;

    protected static PoolReferenceCheckpoint instance = new PoolReferenceCheckpoint();

    public static PoolReferenceCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, REFERENCE_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, PoolReference.class, msgCtx))
            return;

        // Initialize
        PoolReference mainRef = (PoolReference) rpmObject;

        // Field validation: contactEmail
        if (mainRef.testContactEmailModified())
        {
            GenericValidator.validateMaxLength(mainRef, "contactEmail",
                mainRef.getContactEmail(),
                ValidationConstants.REFERENCE_CONTACTEMAIL_MAX, msgCtx);
        }

        // Field validation: contactFax
        if (mainRef.testContactFaxModified())
        {
            GenericValidator.validateMaxLength(mainRef, "contactFax",
                mainRef.getContactFax(), ValidationConstants.REFERENCE_CONTACTFAX_MAX,
                msgCtx);
        }

        // Field validation: contactName
        if (mainRef.testContactNameModified())
        {
            GenericValidator.validateMaxLength(mainRef, "contactName",
                mainRef.getContactName(), ValidationConstants.REFERENCE_CONTACTNAME_MAX,
                msgCtx);
        }

        // Field validation: contactTelephone
        if (mainRef.testContactTelephoneModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "contactTelephone", mainRef.getContactTelephone(),
                ValidationConstants.REFERENCE_CONTACTTELEPHONE_MAX, msgCtx);
        }

        // Field validation: organizationAddress
        if (mainRef.testOrganizationAddressModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "organizationAddress", mainRef.getOrganizationAddress(),
                ValidationConstants.REFERENCE_ORGANIZATIONADDRESS_MAX, msgCtx);
        }

        // Field validation: organizationFax
        if (mainRef.testOrganizationFaxModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "organizationFax", mainRef.getOrganizationFax(),
                ValidationConstants.REFERENCE_ORGANIZATIONFAX_MAX, msgCtx);
        }

        // Field validation: organizationName
        if (mainRef.testOrganizationNameModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "organizationName", mainRef.getOrganizationName(),
                ValidationConstants.REFERENCE_ORGANIZATIONNAME_MAX, msgCtx);
        }

        // Field validation: organizationTelephone
        if (mainRef.testOrganizationTelephoneModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "organizationTelephone", mainRef.getOrganizationTelephone(),
                ValidationConstants.REFERENCE_ORGANIZATIONTELEPHONE_MAX, msgCtx);
        }

        // Field validation: organizationWebsite
        if (mainRef.testOrganizationWebsiteModified())
        {
            GenericValidator.validateMaxLength(mainRef,
                "organizationWebsite", mainRef.getOrganizationWebsite(),
                ValidationConstants.REFERENCE_ORGANIZATIONWEBSITE_MAX, msgCtx);
        }
    }
}