/*
 * Created on 6-Oct-2005
 */
package com.ibm.rpm.resource.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;

/**
 * @author dominiqu
 */
public class ResourceRatesCheckpoint extends AbstractCheckpoint
{
    private static final List RESOURCERATES_PARENTS = new ArrayList();

    static
    {
        RESOURCERATES_PARENTS.add(Resource.class);
        RESOURCERATES_PARENTS.add(ResourceAttributeAssignment.class);
        RESOURCERATES_PARENTS.add(ResourceAttributeProficiencyLevel.class);
    }

    protected static ResourceRatesCheckpoint instance = new ResourceRatesCheckpoint();

    public static ResourceRatesCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, false, RESOURCERATES_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, ResourceRates.class, msgCtx))
            return;

        // Initialize
        ResourceRates mainRates = (ResourceRates) rpmObject;

        // Note: setModified does not set the inner fields of children classes
        // as modified so we cannot check if they have been modified before
        // validating

        // Field validation: regularCostPerHour
        GenericValidator.validateRange(mainRates, "regularCostPerHour",
            mainRates.getRegularCostPerHour(), ValidationConstants.RESOURCERATES_MIN,
            ValidationConstants.RESOURCERATES_MAX, msgCtx);

        // Field validation: sellingRatePerHour
        GenericValidator.validateRange(mainRates, "sellingRatePerHour",
            mainRates.getSellingRatePerHour(), ValidationConstants.RESOURCERATES_MIN,
            ValidationConstants.RESOURCERATES_MAX, msgCtx);

        // Field validation: specialCostPerHour
        GenericValidator.validateRange(mainRates, "specialCostPerHour",
            mainRates.getSpecialCostPerHour(), ValidationConstants.RESOURCERATES_MIN,
            ValidationConstants.RESOURCERATES_MAX, msgCtx);

        // Field validation: specialSellingRatePerHour
        GenericValidator.validateRange(mainRates,
            "specialSellingRatePerHour", mainRates.getSpecialSellingRatePerHour(),
            ValidationConstants.RESOURCERATES_MIN, ValidationConstants.RESOURCERATES_MAX,
            msgCtx);
    }
}