/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author gosselin
 */
public class ResourceRoleAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final List RESOURCEROLEASSIGNMENT_PARENTS = new ArrayList();

    static
    {
        RESOURCEROLEASSIGNMENT_PARENTS.add(AbstractDocument.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(DocumentElement.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(DocumentFolder.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(NoteMinutes.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Pool.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Action.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ChangeRequest.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Defect.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Issue.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Requirement.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Risk.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeDeliverable.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeElement.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeFolder.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeMilestone.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeSummaryTask.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ScopeTask.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(ServiceRequest.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Deliverable.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(GenericProject.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Milestone.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(SummaryTask.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(Task.class);
        RESOURCEROLEASSIGNMENT_PARENTS.add(WorkProduct.class);
    }

    protected static ResourceRoleAssignmentCheckpoint instance = new ResourceRoleAssignmentCheckpoint();

    public static ResourceRoleAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, RESOURCEROLEASSIGNMENT_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, ResourceRoleAssignment.class,
            msgCtx))
            return;

        // Initialize
        ResourceRoleAssignment mainRRA = (ResourceRoleAssignment) rpmObject;

        // Field validation: resource
        GenericValidator.validateMandatory(mainRRA, "resource", mainRRA
            .getResource(), msgCtx);

//        // Field validation: securityRole
//        GenericValidator.validateMandatory(mainRRA, "securityRole", mainRRA
//            .getSecurityRole(), msgCtx);
//
//        // Parent verification
//        RPMObject parent = mainRRA.getParent();
//
//        if (mainRRA.testParentModified() || mainRRA.testSecurityRoleModified())
//        {
//            // Resource management
//            if (parent instanceof Pool)
//            {
//                GenericValidator.validateType(mainRRA, PoolSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            // Documents
//            else if (parent instanceof NoteMinutes)
//            {
//                GenericValidator.validateType(mainRRA,
//                    NotesMinutesSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof AbstractDocument)
//            {
//                GenericValidator.validateType(mainRRA,
//                    DocumentSecurityRole.class, "securityRole",
//                    mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof DocumentFolder)
//            {
//                GenericValidator.validateType(mainRRA, FolderSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            // Scope management
//            else if (parent instanceof Requirement)
//            {
//                GenericValidator.validateType(mainRRA,
//                    RequirementSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof Action)
//            {
//                GenericValidator.validateType(mainRRA,
//                    ActionItemSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof Issue)
//            {
//                GenericValidator.validateType(mainRRA, IssueSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof Risk)
//            {
//                GenericValidator.validateType(mainRRA, RiskSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof Defect)
//            {
//                GenericValidator.validateType(mainRRA, DefectSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof ChangeRequest)
//            {
//                GenericValidator.validateType(mainRRA,
//                    ChangeRequestSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof ServiceRequest)
//            {
//                GenericValidator.validateType(mainRRA,
//                    ServiceRequestSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof ScopeDeliverable
//                || parent instanceof ScopeSummaryTask)
//            {
//                GenericValidator.validateType(mainRRA,
//                    DeliverableSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof ScopeMilestone || parent instanceof ScopeTask)
//            {
//                GenericValidator.validateType(mainRRA, TaskSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof ScopeFolder)
//            {
//                GenericValidator.validateType(mainRRA, FolderSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            // WBS
//            else if (parent instanceof Milestone || parent instanceof Task)
//            {
//                GenericValidator.validateType(mainRRA, TaskSecurityRole.class,
//                    "securityRole", mainRRA.getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof Deliverable || parent instanceof WorkProduct
//                || parent instanceof SummaryTask)
//            {
//                GenericValidator.validateType(mainRRA,
//                    DeliverableSecurityRole.class, "securityRole", mainRRA
//                        .getSecurityRole(), msgCtx);
//            }
//            else if (parent instanceof GenericProject)
//            {
//                GenericValidator.validateType(mainRRA,
//                    ProjectSecurityRole.class, "securityRole", mainRRA.getSecurityRole(),
//                    msgCtx);
//            }
//            // TODO: TemplateFolderSecurityRole allowed parent types
//        }
    }
}
