package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.PoolDomainsManager;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.managers.SecurityGroupManager;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.managers.DefaultAdministrativeTaskManager;

public class PoolManager extends AbstractRPMObjectManager
{
	
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(Pool.class.getName());
        
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    PoolManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_POOLS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_POOLS.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_POOLS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_POOLS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_POOLS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_POOLS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_POOLS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_POOLS.PARENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_POOLS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_POOLS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_POOLS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column LABEL of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LABEL = 12;

    public static final int TYPE_LABEL = Types.VARCHAR;

    public static final String NAME_LABEL = "TMT_POOLS.LABEL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LABEL = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LABEL,NAME_LABEL);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 13;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_POOLS.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DEFAULT_FLAG = 14;

    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;

    public static final String NAME_DEFAULT_FLAG = "TMT_POOLS.DEFAULT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DEFAULT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_RANK = 15;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_POOLS.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DOMAIN_ID of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DOMAIN_ID = 16;

    public static final int TYPE_DOMAIN_ID = Types.CHAR;

    public static final String NAME_DOMAIN_ID = "TMT_POOLS.DOMAIN_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DOMAIN_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DOMAIN_ID,NAME_DOMAIN_ID);}

    /**
     * Column MANAGER_ID of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MANAGER_ID = 17;

    public static final int TYPE_MANAGER_ID = Types.CHAR;

    public static final String NAME_MANAGER_ID = "TMT_POOLS.MANAGER_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MANAGER_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MANAGER_ID,NAME_MANAGER_ID);}

    /**
     * Column COMPANY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COMPANY_NAME = 18;

    public static final int TYPE_COMPANY_NAME = Types.VARCHAR;

    public static final String NAME_COMPANY_NAME = "TMT_POOLS.COMPANY_NAME";

    public static final String PROPERTY_COMPANY_NAME = "ORGANIZATIONNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_NAME, NAME_COMPANY_NAME);
    }

    /**
     * Column ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ADDRESS = 19;

    public static final int TYPE_ADDRESS = Types.VARCHAR;

    public static final String NAME_ADDRESS = "TMT_POOLS.ADDRESS";

    public static final String PROPERTY_ADDRESS = "ADDRESS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ADDRESS, NAME_ADDRESS);
    }

    /**
     * Column PHONE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PHONE = 20;

    public static final int TYPE_PHONE = Types.VARCHAR;

    public static final String NAME_PHONE = "TMT_POOLS.PHONE";

    public static final String PROPERTY_PHONE = "TELEPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PHONE, NAME_PHONE);
    }

    /**
     * Column FAX of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FAX = 21;

    public static final int TYPE_FAX = Types.VARCHAR;

    public static final String NAME_FAX = "TMT_POOLS.FAX";

    public static final String PROPERTY_FAX = "FAX";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FAX, NAME_FAX);
    }

    /**
     * Column WEBSITE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_WEBSITE = 22;

    public static final int TYPE_WEBSITE = Types.VARCHAR;

    public static final String NAME_WEBSITE = "TMT_POOLS.WEBSITE";

    public static final String PROPERTY_WEBSITE = "ORGANIZATIONWEBSITE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEBSITE, NAME_WEBSITE);
    }

    /**
     * Column CONTACT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACT_NAME = 23;

    public static final int TYPE_CONTACT_NAME = Types.VARCHAR;

    public static final String NAME_CONTACT_NAME = "TMT_POOLS.CONTACT_NAME";

    public static final String PROPERTY_CONTACT_NAME = "CONTACTNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACT_NAME, NAME_CONTACT_NAME);
    }

    /**
     * Column OTHER_LOCATION of type Types.VARCHAR mapped to String. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_OTHER_LOCATION = 24;

    public static final int TYPE_OTHER_LOCATION = Types.VARCHAR;

    public static final String NAME_OTHER_LOCATION = "TMT_POOLS.OTHER_LOCATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_OTHER_LOCATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_OTHER_LOCATION,NAME_OTHER_LOCATION);}

    /**
     * Column YEAR_IN_BUSINESS of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_YEAR_IN_BUSINESS = 25;

    public static final int TYPE_YEAR_IN_BUSINESS = Types.INTEGER;

    public static final String NAME_YEAR_IN_BUSINESS = "TMT_POOLS.YEAR_IN_BUSINESS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_YEAR_IN_BUSINESS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_YEAR_IN_BUSINESS,NAME_YEAR_IN_BUSINESS);}

    /**
     * Column CLIENT_MORE_100_EMPLOYEES of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLIENT_MORE_100_EMPLOYEES = 26;

    public static final int TYPE_CLIENT_MORE_100_EMPLOYEES = Types.INTEGER;

    public static final String NAME_CLIENT_MORE_100_EMPLOYEES = "TMT_POOLS.CLIENT_MORE_100_EMPLOYEES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLIENT_MORE_100_EMPLOYEES =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_100_EMPLOYEES,NAME_CLIENT_MORE_100_EMPLOYEES);}

    /**
     * Column CLIENT_MORE_1000_EMPLOYEES of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_CLIENT_MORE_1000_EMPLOYEES = 27;

    public static final int TYPE_CLIENT_MORE_1000_EMPLOYEES = Types.INTEGER;

    public static final String NAME_CLIENT_MORE_1000_EMPLOYEES = "TMT_POOLS.CLIENT_MORE_1000_EMPLOYEES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLIENT_MORE_1000_EMPLOYEES =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_1000_EMPLOYEES,NAME_CLIENT_MORE_1000_EMPLOYEES);}

    /**
     * Column CLIENT_MORE_10000_EMPLOYEES of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_CLIENT_MORE_10000_EMPLOYEES = 28;

    public static final int TYPE_CLIENT_MORE_10000_EMPLOYEES = Types.INTEGER;

    public static final String NAME_CLIENT_MORE_10000_EMPLOYEES = "TMT_POOLS.CLIENT_MORE_10000_EMPLOYEES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLIENT_MORE_10000_EMPLOYEES =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_10000_EMPLOYEES,NAME_CLIENT_MORE_10000_EMPLOYEES);}

    /**
     * Column NUMBER_EMPLOYEES of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_NUMBER_EMPLOYEES = 29;

    public static final int TYPE_NUMBER_EMPLOYEES = Types.INTEGER;

    public static final String NAME_NUMBER_EMPLOYEES = "TMT_POOLS.NUMBER_EMPLOYEES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_NUMBER_EMPLOYEES = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_NUMBER_EMPLOYEES,NAME_NUMBER_EMPLOYEES);}

    /**
     * Column NUMBER_POOLS of type Types.INTEGER mapped to Integer.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_NUMBER_POOLS = 30;

    public static final int TYPE_NUMBER_POOLS = Types.INTEGER;

    public static final String NAME_NUMBER_POOLS = "TMT_POOLS.NUMBER_POOLS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_NUMBER_POOLS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NUMBER_POOLS,NAME_NUMBER_POOLS);}

    /**
     * Column NUMBER_CLIENTS of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_NUMBER_CLIENTS = 31;

    public static final int TYPE_NUMBER_CLIENTS = Types.INTEGER;

    public static final String NAME_NUMBER_CLIENTS = "TMT_POOLS.NUMBER_CLIENTS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_NUMBER_CLIENTS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_NUMBER_CLIENTS,NAME_NUMBER_CLIENTS);}

    /**
     * Column ANNUAL_REVENUES of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     * in upper case.
     */
    //ANNUAL_REVENUES property not available
    public static final int ID_ANNUAL_REVENUES = 32;

    public static final int TYPE_ANNUAL_REVENUES = Types.DECIMAL;

    public static final String NAME_ANNUAL_REVENUES = "TMT_POOLS.ANNUAL_REVENUES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ANNUAL_REVENUES = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ANNUAL_REVENUES,NAME_ANNUAL_REVENUES);}

    /**
     * Column BILLING_ADDRESS of type Types.VARCHAR mapped to String. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_BILLING_ADDRESS = 33;

    public static final int TYPE_BILLING_ADDRESS = Types.VARCHAR;

    public static final String NAME_BILLING_ADDRESS = "TMT_POOLS.BILLING_ADDRESS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLING_ADDRESS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BILLING_ADDRESS,NAME_BILLING_ADDRESS);}

    /**
     * Column BILLING_PHONE of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_BILLING_PHONE = 34;

    public static final int TYPE_BILLING_PHONE = Types.VARCHAR;

    public static final String NAME_BILLING_PHONE = "TMT_POOLS.BILLING_PHONE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLING_PHONE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_PHONE,NAME_BILLING_PHONE);}

    /**
     * Column BILLING_FAX of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_BILLING_FAX = 35;

    public static final int TYPE_BILLING_FAX = Types.VARCHAR;

    public static final String NAME_BILLING_FAX = "TMT_POOLS.BILLING_FAX";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLING_FAX = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_FAX,NAME_BILLING_FAX);}

    /**
     * Column BILLING_EMAIL of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_BILLING_EMAIL = 36;

    public static final int TYPE_BILLING_EMAIL = Types.VARCHAR;

    public static final String NAME_BILLING_EMAIL = "TMT_POOLS.BILLING_EMAIL";

    public static final String PROPERTY_BILLING_EMAIL = "EMAIL";
    static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_EMAIL,NAME_BILLING_EMAIL);}

    /**
     * Column EQUAL_OPPORTUNITY of type Types.CHAR mapped to String. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_EQUAL_OPPORTUNITY = 37;

    public static final int TYPE_EQUAL_OPPORTUNITY = Types.CHAR;

    public static final String NAME_EQUAL_OPPORTUNITY = "TMT_POOLS.EQUAL_OPPORTUNITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EQUAL_OPPORTUNITY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EQUAL_OPPORTUNITY,NAME_EQUAL_OPPORTUNITY);}

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAJOR_REVISION = 38;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "TMT_POOLS.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MINOR_REVISION = 39;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "TMT_POOLS.MINOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION);}

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_IMPACT = 40;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "TMT_POOLS.REVISION_IMPACT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_TYPE = 41;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "TMT_POOLS.REVISION_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_HISTORY = 42;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "TMT_POOLS.REVISION_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

    /**
     * Column PUBLIC_FLAG of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PUBLIC_FLAG = 43;

    public static final int TYPE_PUBLIC_FLAG = Types.CHAR;

    public static final String NAME_PUBLIC_FLAG = "TMT_POOLS.PUBLIC_FLAG";

    public static final String PROPERTY_PUBLIC_FLAG = "PUBLISH";
    static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLIC_FLAG, SqlUtil.getBooleanValue(NAME_PUBLIC_FLAG, "1"));}

    /**
     * Column COMPANY_BACKGROUND of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COMPANY_BACKGROUND = 44;

    public static final int TYPE_COMPANY_BACKGROUND = Types.VARCHAR;

    public static final String NAME_COMPANY_BACKGROUND = "TMT_POOLS.COMPANY_BACKGROUND";

    public static final String PROPERTY_COMPANY_BACKGROUND = "COMPANYBACKGROUND";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_BACKGROUND,
                NAME_COMPANY_BACKGROUND);
    }

    /**
     * Column OWNERSHIP of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OWNERSHIP = 45;

    public static final int TYPE_OWNERSHIP = Types.VARCHAR;

    public static final String NAME_OWNERSHIP = "TMT_POOLS.OWNERSHIP";

    public static final String PROPERTY_OWNERSHIP = "OWNERSHIP";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_OWNERSHIP, NAME_OWNERSHIP);
    }

    /**
     * Column SERVICES_OFFERED of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SERVICES_OFFERED = 46;

    public static final int TYPE_SERVICES_OFFERED = Types.VARCHAR;

    public static final String NAME_SERVICES_OFFERED = "TMT_POOLS.SERVICES_OFFERED";

    public static final String PROPERTY_SERVICES_OFFERED = "SERVICESOFFERED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SERVICES_OFFERED, NAME_SERVICES_OFFERED);
    }

    /**
     * Column SERVICES_SPECIALIZED of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SERVICES_SPECIALIZED = 47;

    public static final int TYPE_SERVICES_SPECIALIZED = Types.VARCHAR;

    public static final String NAME_SERVICES_SPECIALIZED = "TMT_POOLS.SERVICES_SPECIALIZED";

    public static final String PROPERTY_SERVICES_SPECIALIZED = "SERVICESSPECIALIZED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SERVICES_SPECIALIZED,
                NAME_SERVICES_SPECIALIZED);
    }

    /**
     * Column INDUSTRIES_SERVED of type Types.VARCHAR mapped to String.
     */
    public static final int ID_INDUSTRIES_SERVED = 48;

    public static final int TYPE_INDUSTRIES_SERVED = Types.VARCHAR;

    public static final String NAME_INDUSTRIES_SERVED = "TMT_POOLS.INDUSTRIES_SERVED";

    public static final String PROPERTY_INDUSTRIES_SERVED = "INDUSTRIESSERVED";
    static
    {
        FIELDPROPERTYMAP
                .put(PROPERTY_INDUSTRIES_SERVED, NAME_INDUSTRIES_SERVED);
    }

    /**
     * Column HIGHLIGHT of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_HIGHLIGHT = 49;

    public static final int TYPE_HIGHLIGHT = Types.VARCHAR;

    public static final String NAME_HIGHLIGHT = "TMT_POOLS.HIGHLIGHT";

    public static final String PROPERTY_HIGHLIGHT = "PRODUCTSEXPERTISEUSED";
    static{ FIELDPROPERTYMAP.put(PROPERTY_HIGHLIGHT,NAME_HIGHLIGHT);}

    /**
     * Column LOCATION_SERVED of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LOCATION_SERVED = 50;

    public static final int TYPE_LOCATION_SERVED = Types.VARCHAR;

    public static final String NAME_LOCATION_SERVED = "TMT_POOLS.LOCATION_SERVED";

    public static final String PROPERTY_LOCATION_SERVED = "LOCATIONSSERVED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOCATION_SERVED, NAME_LOCATION_SERVED);
    }

    /**
     * Column SCORECARD_COUNT of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_SCORECARD_COUNT = 51;

    public static final int TYPE_SCORECARD_COUNT = Types.INTEGER;

    public static final String NAME_SCORECARD_COUNT = "TMT_POOLS.SCORECARD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_COUNT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_COUNT,NAME_SCORECARD_COUNT);}

    /**
     * Column DOC_COUNT of type Types.INTEGER mapped to Integer.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DOC_COUNT = 52;

    public static final int TYPE_DOC_COUNT = Types.INTEGER;

    public static final String NAME_DOC_COUNT = "TMT_POOLS.DOC_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DOC_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DOC_COUNT,NAME_DOC_COUNT);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CREATED_BY = 53;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_POOLS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column ASSIGN_NAMES of type Types.VARCHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ASSIGN_NAMES = 54;

    public static final int TYPE_ASSIGN_NAMES = Types.VARCHAR;

    public static final String NAME_ASSIGN_NAMES = "TMT_POOLS.ASSIGN_NAMES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_NAMES = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_NAMES,NAME_ASSIGN_NAMES);}

    /**
     * Column CLEARANCE_LEVEL_1 of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_1 = 55;

    public static final int TYPE_CLEARANCE_LEVEL_1 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_1 = "TMT_POOLS.CLEARANCE_LEVEL_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_1 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_1,NAME_CLEARANCE_LEVEL_1);}

    /**
     * Column CLEARANCE_LEVEL_2 of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_2 = 56;

    public static final int TYPE_CLEARANCE_LEVEL_2 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_2 = "TMT_POOLS.CLEARANCE_LEVEL_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_2 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_2,NAME_CLEARANCE_LEVEL_2);}

    /**
     * Column CLEARANCE_LEVEL_3 of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_3 = 57;

    public static final int TYPE_CLEARANCE_LEVEL_3 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_3 = "TMT_POOLS.CLEARANCE_LEVEL_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_3 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_3,NAME_CLEARANCE_LEVEL_3);}

    /**
     * Column CLEARANCE_LEVEL_4 of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_4 = 58;

    public static final int TYPE_CLEARANCE_LEVEL_4 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_4 = "TMT_POOLS.CLEARANCE_LEVEL_4";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_4 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_4,NAME_CLEARANCE_LEVEL_4);}

    /**
     * Column USER_GROUP_ID of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_USER_GROUP_ID = 59;

    public static final int TYPE_USER_GROUP_ID = Types.CHAR;

    public static final String NAME_USER_GROUP_ID = "TMT_POOLS.USER_GROUP_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_USER_GROUP_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_USER_GROUP_ID,NAME_USER_GROUP_ID);}

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_5 = 60;

    public static final int TYPE_CLEARANCE_LEVEL_5 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_5 = "TMT_POOLS.CLEARANCE_LEVEL_5";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_5 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_5,NAME_CLEARANCE_LEVEL_5);}

    /**
     * Column MASK_EMPLOYEE_INFO of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MASK_EMPLOYEE_INFO = 61;

    public static final int TYPE_MASK_EMPLOYEE_INFO = Types.SMALLINT;

    public static final String NAME_MASK_EMPLOYEE_INFO = "TMT_POOLS.MASK_EMPLOYEE_INFO";

    public static final String PROPERTY_MASK_EMPLOYEE_INFO = "MASKEMPLOYEEINFO";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MASK_EMPLOYEE_INFO,
                SqlUtil.getBooleanValue(NAME_MASK_EMPLOYEE_INFO, 1));
    }

    /**
     * Column COST_CENTER_ID of type Types.CHAR mapped to String.
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_COST_CENTER_ID = 62;

    public static final int TYPE_COST_CENTER_ID = Types.CHAR;

    public static final String NAME_COST_CENTER_ID = "TMT_POOLS.COST_CENTER_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_CENTER_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_ID,NAME_COST_CENTER_ID);}

    /**
     * Column EXTERNAL_REFERENCE of type Types.CHAR mapped to String. 
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_EXTERNAL_REFERENCE = 63;

    public static final int TYPE_EXTERNAL_REFERENCE = Types.CHAR;

    public static final String NAME_EXTERNAL_REFERENCE = "TMT_POOLS.EXTERNAL_REFERENCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXTERNAL_REFERENCE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_REFERENCE,NAME_EXTERNAL_REFERENCE);}

    public static final String TABLE_NAME = "TMT_POOLS";

    /**
     * Create an array of type string containing all the fields of the TMT_POOLS
     * table.
     */
    private static final String[] FIELD_NAMES = { NAME_LEVEL_ID, NAME_TYPE_ID,
            NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
            NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_LABEL, NAME_DESCRIPTION,
            NAME_DEFAULT_FLAG, NAME_RANK, NAME_DOMAIN_ID, NAME_MANAGER_ID,
            NAME_COMPANY_NAME, NAME_ADDRESS, NAME_PHONE, NAME_FAX,
            NAME_WEBSITE, NAME_CONTACT_NAME, NAME_OTHER_LOCATION,
            NAME_YEAR_IN_BUSINESS, NAME_CLIENT_MORE_100_EMPLOYEES,
            NAME_CLIENT_MORE_1000_EMPLOYEES, NAME_CLIENT_MORE_10000_EMPLOYEES,
            NAME_NUMBER_EMPLOYEES, NAME_NUMBER_POOLS, NAME_NUMBER_CLIENTS,
            NAME_ANNUAL_REVENUES, NAME_BILLING_ADDRESS, NAME_BILLING_PHONE,
            NAME_BILLING_FAX, NAME_BILLING_EMAIL, NAME_EQUAL_OPPORTUNITY,
            NAME_MAJOR_REVISION, NAME_MINOR_REVISION, NAME_REVISION_IMPACT,
            NAME_REVISION_TYPE, NAME_REVISION_HISTORY, NAME_PUBLIC_FLAG,
            NAME_COMPANY_BACKGROUND, NAME_OWNERSHIP, NAME_SERVICES_OFFERED,
            NAME_SERVICES_SPECIALIZED, NAME_INDUSTRIES_SERVED, NAME_HIGHLIGHT,
            NAME_LOCATION_SERVED, NAME_SCORECARD_COUNT, NAME_DOC_COUNT,
            NAME_CREATED_BY, NAME_ASSIGN_NAMES, NAME_CLEARANCE_LEVEL_1,
            NAME_CLEARANCE_LEVEL_2, NAME_CLEARANCE_LEVEL_3,
            NAME_CLEARANCE_LEVEL_4, NAME_USER_GROUP_ID, NAME_CLEARANCE_LEVEL_5,
            NAME_MASK_EMPLOYEE_INFO, NAME_COST_CENTER_ID,
            NAME_EXTERNAL_REFERENCE };

    private static final String[] UNSUPPORTED_FIELD_NAMES = {
            "asdf"
    };
    
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new PoolManager instance.
     * 
     * @return PoolManager
     */
    public PoolManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getUnsupportedFieldsNames()
     */
    protected String[] getUnsupportedFieldsNames()
    {
        return UNSUPPORTED_FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    
    /**
     *
     */

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");
        filter.append(" AND ");
        filter.append(NAME_TYPE_ID);
        filter.append(" = ");
        filter.append(Pool.TYPE_ID);

        return filter.toString();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();

            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof PoolManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof PoolReferenceManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(PoolReferenceManager.NAME_POOL_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof PoolDomainsManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(PoolDomainsManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_DOMAIN_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceManager)
        {
            
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            
            //Add the table and the condition to join the 2 tables
            joinCondition.setTableName(getTableName());            
            where.append(ResourceManager.NAME_RESOURCE_ID);
            where.append(" = ");
            where.append(NAME_ELEMENT_ID);
            where.append(" AND ");
            
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }            
                       
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof PoolModuleManager){

        	joinCondition = new JoinCondition();

        	

        }
        else if (manager instanceof DefaultAdministrativeTaskManager)
        {
            SqlBuffer where = new SqlBuffer();
            where.appendSubSelect(
                DefaultAdministrativeTaskManager.NAME_POOL_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFilter());

            joinCondition = JoinCondition.createSubSelect(
                DefaultAdministrativeTaskManager.NAME_PARENT_ID,
                DefaultAdministrativeTaskManager.NAME_ELEMENT_ID,
                DefaultAdministrativeTaskManager.TABLE_NAME, where.toString());

        }        
        else
        {            
    	    {
                ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                        ExceptionUtil.JOIN);
            }
        }
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        
        else if(manager instanceof PoolModuleManager){

            

            joinCondition = new JoinCondition();
        }

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }
        
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.Object[], java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof PoolManager)
        {
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            int level = 1;
            int rankLenght = 4 * level;
            String rank = getTableName() + ".RANK";

            where.append("(LEVEL_ID >= ");
            where.append(1);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(getTableName());            
            where.append(" WHERE ");
            where.append(getPrimaryKey() + " = ?");
            where.append(")");
            joinCondition.setCondition(where.toString());            
            
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        
        }else if (manager instanceof ResourceManager)
            
        {                
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
    
            where.append(NAME_ELEMENT_ID);
            where.append(" IN ");
            where.append("(SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(13);
            where.append(" AND ");
            where.append(NAME_ELEMENT_ID);
            where.append(" = ?");
            where.append(")");
            
            joinCondition.setTableName(getTableName());
           
            joinCondition.setCondition(where.toString());
            
        }
        else if (manager instanceof PoolModuleManager){

        	joinCondition = getJoinCondition(manager,null,2, null, contextName);

            if(extraWhere != null && extraWhere.length() > 0){

                joinCondition.setCondition(extraWhere);

            }

        	joinCondition.setTableName(getTableName(contextName));

        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {

            case Pool.TYPE_ID:
                object = new Pool();
                break;
            default:
                ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet,
     *      com.ibm.rpm.scope.RPMObjectScope)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
        
        if (rpmScope != null){
            
            Pool object = (Pool)rpmObject;

            
            //Parent
            if (rpmScope.getParent() != null) {
                String oldParentId = (String) relatedFields
                .get(index, ID_PARENT_ID);
                Pool newParent = null;
                
                if (isDelta) {
                    newParent = (Pool)object.getParent();
                    Pool oldParent = null;
                    if (oldParentId != null) {
                        oldParent = new Pool();
                        oldParent.setID(oldParentId);
                        oldParent.setContextName(getContextName(oldParent));
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId)) {
                            oldParent = (Pool) loadByPrimaryKey(newParent,
                                rpmScope.getParent(), messageContext, true);
                        } else {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                    } else {
                        if (newParent != null) {
                            ExceptionUtil.handleParentReference(this, object);
                        }
                    }
                    object.deltaParent(oldParent);
                } else {
                    if (oldParentId != null) {
                        newParent = new Pool();
                        newParent.setID(oldParentId);
                        newParent.setContextName(getContextName(newParent));
                        newParent = (Pool) loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, false);
                        object.setParent(newParent);
                    }
                }
            }
            
            if (rpmScope instanceof PoolScope)
            {
                PoolScope scope = ((PoolScope) rpmScope);
                
                //children and siblings
                object = loadPool(object, scope, messageContext, relatedFields,
                    index, isDelta);
            }
        }
        return rpmObject;
    }

    private ArrayList setParent(ArrayList children, Pool parent)
    {
        if (children != null && !children.isEmpty())
        {
            for (int i = 0; i < children.size(); i++)
            {
                ((RPMObject)children.get(i)).setParent(parent);
            }
        }
        return children;
    }

    /**
     * @param pool
     * @param scope
     * @return
     * @throws ParseException
     * @throws SQLException
     * @throws RPMException
     */
    private Pool loadPool(Pool pool, PoolScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException {

        if (scope != null) {

            //Pools
            if (scope.getPools() != null) {
                ArrayList temp = this.loadByForeignKey(pool.getPools(), scope
                        .getPools(), messageContext, this, new Object[] { pool
                        .getID() }, "TYPE_ID = " + Pool.TYPE_ID, getContextName(pool), isDelta);
                if (temp != null) {
                    setParent(temp, pool);
                    Pool[] pools = new Pool[temp.size()];
                    temp.toArray(pools);
                    pool.setPools(pools);
                }
            }

            //Resources
            if (scope.getResources() != null) {
                ResourceManager manager = (ResourceManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                Resource.class.getName());
                ArrayList temp = manager.loadByForeignKey(pool.getResources(),
                        scope.getResources(), messageContext, this,
                        new Object[] { pool.getID() }, null, null, isDelta);

                Resource[] res = new Resource[0];
                if (temp != null) {
                    setParent(temp, pool);
                    res = new Resource[temp.size()];
                    temp.toArray(res);
                    pool.setResources(res);
                }
            }

            //References
            if (scope.getPoolReferences() != null) {
                PoolReferenceManager poolReferenceManager = (PoolReferenceManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                PoolReference.class.getName());
                ArrayList temp = poolReferenceManager.loadByForeignKey(pool
                        .getPoolReferences(), scope.getPoolReferences(),
                        messageContext, this, new Object[] { pool.getID() },
                        null, null, isDelta);
                if (temp != null) {
                    setParent(temp, pool);
                    PoolReference[] ref = new PoolReference[temp.size()];
                    temp.toArray(ref);
                    pool.setPoolReferences(ref);
                }
            }

            //Resource Assignments
            if (scope != null && scope.isResourceAssignments())
            {
                ResourceRoleAssignmentManager manager = (ResourceRoleAssignmentManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        ResourceRoleAssignment.class.getName());

                ArrayList temp = manager.loadByForeignKey(pool
                    .getResourceAssignments(), null, messageContext, this,
                    new Object[]
                    {
                		pool.getID()
                    }, null, pool.getContextName(), isDelta);

                if (temp != null)
                {
                    ResourceRoleAssignment[] resAss = new ResourceRoleAssignment[temp
                        .size()];
                    temp.toArray(resAss);
                    
                    for (int i = 0; i < resAss.length; i++) {
                    	
                    	resAss[i].setParent(pool);
                    	
                    }
                    
                    pool.setResourceAssignments(resAss);
                }
            }

            //Attribute Assignments
            if (scope.isAttributeAssignments()) {
                AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                AttributeAssignment.class.getName());
                ArrayList temp = attributeAssignmentManager
                        .loadByForeignKey(pool.getAttributeAssignments(), null,
                                messageContext, this, new Object[] { pool
                                        .getID() }, null, getContextName(pool), isDelta);
                if (temp != null) {
                    setParent(temp, pool);
                    AttributeAssignment[] aa = new AttributeAssignment[temp
                            .size()];
                    temp.toArray(aa);
                    pool.setAttributeAssignments(aa);
                }
            }

            //Pool Domain
            if (scope.getDomain() != null) {
                
                PoolDomain actualDomain = new PoolDomain();
                actualDomain.setID((String) relatedFields.get(index, ID_DOMAIN_ID));

                if (actualDomain.getID() != null)
                {
                    PoolDomainsManager manager = (PoolDomainsManager) RPMManagerFactory.getInstance().getRPMObjectManager(
                        PoolDomain.class.getName());

                    actualDomain = (PoolDomain) manager.loadByPrimaryKey(
                        actualDomain, scope.getDomain(), messageContext, false);

                    if (isDelta)
                    {
                        pool.deltaDomain(actualDomain);
                    }
                    else
                    {
                        pool.setDomain(actualDomain);
                    }
                }
            }            
        }

        loadSecurityGroup(pool, scope, messageContext, relatedFields, index, isDelta);
        loadDocumentFolder(pool, scope, messageContext, isDelta);
        
        loadAdministrativeTasks(pool, scope, messageContext, isDelta);

        
        
        //TODO load ClientCostCenter
        //TODO load StatusUpdate

        return pool;

    }

    private void loadSecurityGroup(Pool pool, PoolScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        if (scope != null && scope.getSecurityGroup() != null)
        {
            SecurityGroup securityGroup = new SecurityGroup();
            securityGroup.setID((String) relatedFields.get(index,
                    ID_USER_GROUP_ID));

            SecurityGroupManager manager = (SecurityGroupManager) getManagerInstance(SecurityGroup.class);

            securityGroup = (SecurityGroup) manager.loadByPrimaryKey(
                    securityGroup, scope.getSecurityGroup(),
                    messageContext, isDelta);

            if (isDelta)
            {
                pool.deltaSecurityGroup(securityGroup);
            }
            else
            {
                pool.setSecurityGroup(securityGroup);
            }
        }
    }
    
    private void loadDocumentFolder(Pool pool, PoolScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if(scope.getDocumentFolder()!=null){
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                    pool.getDocumentFolder()
            };
            Object[] args = new Object[]
            {
                    pool.getID()
            };
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(pool,
                    oldDocumentFolder, scope.getDocumentFolder(), messageContext,
                    this, args, null, ContextUtil.POOL_DOCUMENT_CONTEXT, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder.setContextName(ContextUtil.POOL_DOCUMENT_CONTEXT);
                pool.setDocumentFolder(documentFolder);
            }            
        }
    }
    
   
    private Pool saveDocumentFolder(Pool pool, PoolScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getDocumentFolder()!=null && pool.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

            DocumentFolder documentFolder = pool.getDocumentFolder();
            documentFolder.setParent(pool);
            manager.internalSave(documentFolder, scope.getDocumentFolder(), messageContext);
            documentFolder.setContextName(ContextUtil.POOL_DOCUMENT_CONTEXT);
            pool.setDocumentFolder(documentFolder);
        }
        return pool;
    }
    
    private void loadAdministrativeTasks(Pool pool, PoolScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if (scope.isAdministrativeTasks())
        {

            RPMObjectManager manager = getManagerInstance(DefaultAdministrativeTask.class);

            ArrayList temp = manager.loadByForeignKey(pool
                .getAdministrativeTasks(), null, messageContext,
                this, new Object[]
                {
                    pool.getID()
                }, null, null, isDelta);

            pool
                .setAdministrativeTasks((DefaultAdministrativeTask[]) ArrayUtil
                    .listToArray(DefaultAdministrativeTask.class, temp));

        }
    
    }
    
    private String getContextName(RPMObject rpmObject)throws RPMException{
		return ContextUtil.getInstance().getContextName(rpmObject);
	}
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
            throws SQLException
    {
        Pool temp = (Pool) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));

        if (isDelta)
        {
            temp.deltaName(rs.getString(ID_ELEMENT_NAME));
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));

            temp.deltaAddress(rs.getString(ID_ADDRESS));
            temp.deltaCompanyBackground(rs.getString(ID_COMPANY_BACKGROUND));
            temp.deltaContactName(rs.getString(ID_CONTACT_NAME));
            temp.deltaDescription(rs.getString(ID_DESCRIPTION));
            temp.deltaEmail(rs.getString(ID_BILLING_EMAIL));
            temp.deltaFax(rs.getString(ID_FAX));
            temp.deltaIndustriesServed(rs.getString(ID_INDUSTRIES_SERVED));
            temp.deltaLocationsServed(rs.getString(ID_LOCATION_SERVED));
            temp.deltaMaskEmployeeInfo(new Boolean(rs
                    .getBoolean(ID_MASK_EMPLOYEE_INFO)));
            temp.deltaOrganizationName(rs.getString(ID_COMPANY_NAME));
            temp.deltaOwnership(rs.getString(ID_OWNERSHIP));
            temp.deltaProductsExpertiseUsed(rs.getString(ID_HIGHLIGHT));
            temp.deltaPublish(new Boolean(rs
                    .getBoolean(ID_PUBLIC_FLAG)));
            temp.deltaServicesOffered(rs.getString(ID_SERVICES_OFFERED));
            temp
                    .deltaServicesSpecialized(rs
                            .getString(ID_SERVICES_SPECIALIZED));
            temp.deltaTelephone(rs.getString(ID_PHONE));
        }
        else
        {
            temp.setName(rs.getString(ID_ELEMENT_NAME));
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));

            temp.setAddress(rs.getString(ID_ADDRESS));
            temp.setCompanyBackground(rs.getString(ID_COMPANY_BACKGROUND));
            temp.setContactName(rs.getString(ID_CONTACT_NAME));
            temp.setDescription(rs.getString(ID_DESCRIPTION));
            temp.setEmail(rs.getString(ID_BILLING_EMAIL));
            temp.setFax(rs.getString(ID_FAX));
            temp.setIndustriesServed(rs.getString(ID_INDUSTRIES_SERVED));
            temp.setLocationsServed(rs.getString(ID_LOCATION_SERVED));
            temp.setMaskEmployeeInfo(new Boolean(rs
                    .getBoolean(ID_MASK_EMPLOYEE_INFO)));
            temp.setOrganizationName(rs.getString(ID_COMPANY_NAME));
            temp.setOwnership(rs.getString(ID_OWNERSHIP));
            temp.setProductsExpertiseUsed(rs.getString(ID_HIGHLIGHT));
            temp.setPublish(new Boolean(rs
                    .getBoolean(ID_PUBLIC_FLAG)));
            temp.setServicesOffered(rs.getString(ID_SERVICES_OFFERED));
            temp.setServicesSpecialized(rs.getString(ID_SERVICES_SPECIALIZED));
            temp.setTelephone(rs.getString(ID_PHONE));
        }
        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        
        String domainId = resultSet.getString(ID_DOMAIN_ID);
        relatedFields.put(index, ID_DOMAIN_ID, domainId);
        
        String userGroupId = resultSet.getString(ID_USER_GROUP_ID);
        //TODO clean up later
//        if(userGroupId!=null)
//        {
//            userGroupId = userGroupId.trim();
//        }
        relatedFields.put(index, ID_USER_GROUP_ID, userGroupId);
        
        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException,
            RPMException
    {
        Pool savedObj = (Pool) rpmObject;
        
        // set default values
        if (savedObj.getMaskEmployeeInfo() == null)
        {
            savedObj.setMaskEmployeeInfo(new Boolean(true));
        }
        if (savedObj.getPublish() == null)
        {
            savedObj.setPublish(new Boolean(false));
        }
        
        savedObj.setID(SP_I_POOL(savedObj, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException,
            RPMException, ParseException
    {
        Pool savedObj = (Pool) rpmObject;
        PoolScope scope = (PoolScope)rpmScope;
        if (savedObj.testNameModified())
            SP_U_POOL_NAME(savedObj, messageContext);
        if (savedObj.testDescriptionModified())
            SP_U_POOL_DESC(savedObj, messageContext);
        if (savedObj.testDomainModified()
                || savedObj.testOrganizationNameModified()
                || savedObj.testContactNameModified()){
            saveCompanyInformation(savedObj, scope, messageContext);
        }            
        if (savedObj.testAddressModified() || savedObj.testTelephoneModified()
                || savedObj.testFaxModified() || savedObj.testEmailModified()
                || savedObj.testCompanyBackgroundModified()
                || savedObj.testOwnershipModified()
                || savedObj.testServicesOfferedModified()
                || savedObj.testServicesSpecializedModified()
                || savedObj.testIndustriesServedModified()
                || savedObj.testProductsExpertiseUsedModified()
                || savedObj.testLocationsServedModified())
            SP_U_POOL_BKG(savedObj, messageContext);
        if (savedObj.testPublishModified()
                || savedObj.testMaskEmployeeInfoModified())
            SP_U_POOL_PUBLIC(savedObj, messageContext);
        if (savedObj.testClientCostCenterModified())
            SP_U_POOL_COST_CENTER(savedObj, messageContext);
            
        //TODO uncomment after implementation of SP_U_POOL_REVISION is done
        //SP_U_POOL_REVISION(savedObj, messageContext);
        
        if (scope != null)
        {
            savePools(savedObj, rpmScope, messageContext);
            saveSecurityGroup(savedObj, rpmScope, messageContext);
            saveReferences(savedObj, rpmScope, messageContext);
            saveResourceAssignments(savedObj, rpmScope, messageContext);
            saveAttributeAssignments(savedObj, rpmScope, messageContext);
            saveResources(savedObj, rpmScope, messageContext);
            saveAdministrativeTasks(savedObj, rpmScope, messageContext);
            saveDocumentFolder(savedObj, scope, messageContext);
        }
        return savedObj;
    }

    private void saveSecurityGroup(Pool pObject, RPMObjectScope scope, MessageContext messageContext)
    throws RPMException, SQLException
    {
        if (((PoolScope)scope).getSecurityGroup() != null
                && pObject.getSecurityGroup() != null && pObject.testSecurityGroupModified())
        {
            SP_U_UG_ID(pObject, messageContext);
        }
    }
    
    private Pool savePools(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        PoolScope scope = (PoolScope) rpmScope;
        pObject.setPools((Pool[]) updateChildren(pObject, pObject.getPools(),
                scope.getPools(), messageContext, null));
        return pObject;
    }

    private Pool saveReferences(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        PoolScope scope = (PoolScope) rpmScope;
        pObject.setPoolReferences((PoolReference[]) 
                updateChildren(pObject, pObject.getPoolReferences(),
		                scope.getPoolReferences(), 
		                messageContext, null));
        return pObject;
    }

    private Pool saveCompanyInformation(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException{
    
        PoolScope scope = (PoolScope) rpmScope;
        
        if(scope == null || scope.getDomain() == null){
            
            //reload missing information
            PoolScope tempScope = new PoolScope();
            tempScope.setDomain(new PoolDomainScope());
            Pool tempPool = new Pool();
            tempPool.setID(pObject.getID());
            tempPool = (Pool)loadByPrimaryKey(tempPool, tempScope, messageContext, false);
            
            pObject.setDomain(tempPool.getDomain());
        }

        SP_U_POOL_CIE(pObject, messageContext);
    
        return pObject;
    }
    
    private Pool saveResources(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        PoolScope scope = (PoolScope) rpmScope;
        pObject.setResources((Resource[]) 
                updateChildren(pObject, pObject.getResources(),
                        scope.getResources(), 
                messageContext, null));
        return pObject;
    }
    

    private Pool saveAttributeAssignments(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        PoolScope scope = (PoolScope) rpmScope;
        AttributeAssignment[] children = pObject.getAttributeAssignments();
        children = (AttributeAssignment[]) updateChildren(pObject, 
                children, scope.isAttributeAssignments(), 
                messageContext, null);
        pObject.setAttributeAssignments(children);
        return pObject;
    }

    private Pool saveResourceAssignments(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException{
        
    	PoolScope scope = (PoolScope) rpmScope;
    	if (scope != null && scope.isResourceAssignments()) {
	        pObject.setResourceAssignments((ResourceRoleAssignment[]) 
	                updateChildren(pObject, 
	                        pObject.getResourceAssignments(), 
	                        scope.isResourceAssignments(), 
	                        messageContext, ResourceRoleAssignment.class));
    	}
        return pObject;
    }

    private Pool saveAdministrativeTasks(Pool pObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        PoolScope scope = (PoolScope) rpmScope;
        DefaultAdministrativeTask[] children = pObject.getAdministrativeTasks();
        children = (DefaultAdministrativeTask[]) updateChildren(pObject, children,
            scope.isAdministrativeTasks(), messageContext, null);
        pObject.setAdministrativeTasks(children);
        return pObject;

    }

    private Pool saveClientCostCenter(Pool pObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException{
        
                //TODO Implements method PoolManager.saveClientCostCenter()
                //TODO Add getClientCostCenters() to PoolScope
      
                return pObject;
   }


    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException {
        Pool pool = null;
        if (rpmObject != null) {
            pool = (Pool) rpmObject;
            SP_D_POOL(pool, messageContext);
        }
    }

    private String SP_U_POOL_PUBLIC(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {

        String elementId = null;

        String name = "SP_U_POOL_PUBLIC";
        Object args[] = new Object[4];
        args[0] = pool.getID();

        if (pool.getPublish() != null
                && pool.getPublish().booleanValue() == true)
            args[1] = "1";
        else
            args[1] = "0";

        args[2] = pool.getMaskEmployeeInfo();
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;

    }

    private String SP_U_POOL_BKG(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {

        String elementId = null;

        String name = "SP_U_POOL_BKG";
        //CALL
        // SP_U_POOL_BKG('8D6A0B6C465A47269AA160A2C621EBB8',0e0,null,null,null,'email',null,'company
        // background','ownership','service offered','services
        // specialized','industries served','highlight','locations
        // servered','PMO_SUPERVISOR')}]
        Object args[] = new Object[15];
        args[0] = pool.getID();
        args[1] = new Double(0);
        args[2] = pool.getAddress();
        args[3] = pool.getTelephone();
        args[4] = pool.getFax();
        args[5] = pool.getEmail();
        args[6] = null;
        args[7] = pool.getCompanyBackground();
        args[8] = pool.getOwnership();
        args[9] = pool.getServicesOffered();
        args[10] = pool.getServicesSpecialized();
        args[11] = pool.getIndustriesServed();
        args[12] = pool.getProductsExpertiseUsed();
        args[13] = pool.getLocationsServed();
        args[14] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;

    }

    //    SC_SQL_API SP_U_POOL_CIE
    //    (
    //        SC_WCHAR *p_id,
    //        SC_WCHAR *p_domain_id,
    //        SC_WCHAR *p_company_name,
    //        SC_WCHAR *p_address,
    //        SC_WCHAR *p_phone,
    //        SC_WCHAR *p_fax,
    //        SC_WCHAR *p_website,
    //        SC_WCHAR *p_other_location,
    //        SC_LONG_PARAM *p_year_in_business,
    //        SC_LONG_PARAM *p_client_more_100_employees,
    //        SC_LONG_PARAM *p_client_more_1000_employees,
    //        SC_LONG_PARAM *p_client_more_10000_employees,
    //        SC_LONG_PARAM *p_number_employees,
    //        SC_LONG_PARAM *p_number_pools,
    //        SC_LONG_PARAM *p_number_clients,
    //        SC_WCHAR *p_contact_name,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
        private String SP_U_POOL_CIE(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {

        String elementId = null;

        String name = "SP_U_POOL_CIE";
        Object args[] = new Object[17];
        args[0] = pool.getID();
        if (pool.getDomain() != null) {
            args[1] = pool.getDomain().getID();
        }

        args[2] = pool.getOrganizationName();
        args[3] = pool.getAddress();
        args[4] = pool.getTelephone();
        args[5] = pool.getFax();
        args[6] = null;
        args[7] = null;
        args[8] = null;
        args[9] = null;
        args[10] = null;
        args[11] = null;
        args[12] = null;
        args[13] = null;
        args[14] = null;
        args[15] = pool.getContactName();
        args[16] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;

    }

        private String SP_U_POOL_DESC(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {

        String elementId = null;

        String name = "SP_U_POOL_DESC";
        Object args[] = new Object[4];
        args[0] = pool.getID();
        args[1] = null;
        args[2] = pool.getDescription();
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;
    }

    private String SP_U_POOL_NAME(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {

        String elementId = null;

        String name = "SP_U_POOL_NAME";
        Object args[] = new Object[3];
        args[0] = pool.getID();
        args[1] = pool.getName();
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;
    }

    private String SP_I_POOL(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {
        String elementId = null;
        String name = "SP_I_POOL";
        Object args[] = new Object[4];
        args[0] = pool.getName();

        if ((args[1] == null) && pool.getParent() != null) {
            args[1] = pool.getParent().getID();
        }

        args[2] = pool.retrieveTypeID();

        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext,name, args);

        return elementId;
    }

    private void SP_D_POOL(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException {
        //SP_D_POOL('DCB47764B6164F978A32217A11D5140C',null,'X','PMO_SUPERVISOR')
        String name = "SP_D_POOL";
        Object args[] = new Object[4];
        args[0] = pool.getID();
        args[1] = ManagerUtil.getParentId(this, pool, messageContext, TABLE_NAME);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_U_POOL_COST_CENTER(Pool pool, MessageContext messageContext)
            throws RPMException {
        //TODO complete SP_U_POOL_COST_CENTER for clientCostCenters
        ExceptionUtil.handleMethodException(this, ErrorCodes.NOT_IMPLEMENTED, ExceptionUtil.SP_U_POOL_COST_CENTER);
    }

	private void SP_U_UG_ID(Pool pool, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_UG_ID";
        Object args[] = new Object[3];
        args[0] = pool.getID();
        args[1] = pool.getSecurityGroup().getID();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_POOL_REVISION(Pool pool, MessageContext messageContext)
            throws RPMException {
        //TODO complete SP_U_POOL_REVISION for statusUpdates
        ExceptionUtil.handleMethodException(this, ErrorCodes.NOT_IMPLEMENTED, ExceptionUtil.SP_U_POOL_REVISION);
    }

    private void SP_CKIN_POOL(Pool res, MessageContext messageContext)
            throws SQLException, RPMException {
        String name = "SP_CKIN_POOL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_CKOUT_POOL(Pool res, MessageContext messageContext)
            throws SQLException, RPMException {
        String name = "SP_CKOUT_POOL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext,name, args);
    }
    
    private void SP_UCKOUT_POOL(Pool res, MessageContext messageContext)
            throws SQLException, RPMException {
        String name = "SP_UCKOUT_POOL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext,name, args);
    }
    
    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_CKIN_POOL((Pool) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }

    }

    /**
     *  
     */
    public void undoCheckoutObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_UCKOUT_POOL((Pool) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_CKOUT_POOL((Pool) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }
    
    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }   
}
