/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.resource.managers;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.PoolDomainsManager;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolModuleScope;
import com.ibm.rpm.resource.scope.PoolScope;
;

/**
 * @author karim
 *
 */
public class PoolModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(PoolModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(PoolModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                PoolModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.POOL_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new PoolModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)        	{
        		if (localContextName.equalsIgnoreCase(contextName))
        		{
        			joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger.error("PoolModuleManager.getJoinCondition: contextName "
                        + contextName + " is not linked with this manager.");
            }
        }        else if (manager instanceof PoolManager)        {        	joinCondition = new JoinCondition();        }    else if (manager instanceof PoolDomainsManager)        {        	joinCondition = new JoinCondition();            	        }                
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        PoolModule module = (PoolModule) abstractModule;
        PoolModuleScope scope = (PoolModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);

        //loadPools
        if (scope.getPools() != null)
        {
			Pool[] pools = loadPools(module, module.getPools(), scope.getPools(), messageContext, isDelta);
			module.setPools(pools);
		}
        //loadAttributes
        if (scope.getAttributeCategories() != null)
        {
            AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

            AttributeCategory[] attributeCategory = attributeManager
                .loadAttributeCategories(module, module.getAttributeCategories(), scope
                    .getAttributeCategories(), this, localContextName,
                    messageContext, isDelta);
            module.setAttributeCategories(attributeCategory);
        }
        
        //loadPoolDomains
        if (scope.getPoolDomains() != null)
        {
			PoolDomain[] poolDomains = loadPoolDomains(module, module.getPoolDomains(), scope.getPoolDomains(), messageContext, isDelta);
			module.setPoolDomains(poolDomains);
		}
		//loadCompanySizes
		if (scope.isCompanySizes())
        {
            CompanySize[] companySizes = (CompanySize[]) datafieldManager
                .loadDatafields(module, module.getCompanySizes(), CompanySize.class,
                    CompanySize.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setCompanySizes(companySizes);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        PoolModuleScope scope = (PoolModuleScope) rpmScope;
        PoolModule module = (PoolModule) rpmObject;
        
        if(scope!=null)
        {
            //save Pools
            if (scope.getPools() != null)
            {
    			Pool[] pools = savePools(module, module.getPools(), scope.getPools(), messageContext);
    			module.setPools(pools);
    		}
    		//save AttributeCagories
            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                    module, module.getAttributeCategories(), scope.getAttributeCategories(), messageContext,
                    AttributeCategory.class);
                module.setAttributeCategories(attributeCategory);
            }
            //save PoolDomains
            if (scope.getPoolDomains() != null)
            {
    			PoolDomain[] poolDomains = savePoolDomains(module, module.getPoolDomains(), scope.getPoolDomains(), messageContext);
    			module.setPoolDomains(poolDomains);
    		}
        }
        return module;
    }

    protected Pool[] loadPools(AbstractModule abstractModule, Pool[] pools, PoolScope scope, MessageContext messageContext, boolean isDelta)
    throws RPMException, SQLException, ParseException
    {
		String extraWhere = " LEVEL_ID = 0 AND (" + " TYPE_ID = "+ Pool.TYPE_ID + " )";
		PoolManager poolManager = (PoolManager) getManagerInstance(Pool.class.getName());
		ArrayList temp = poolManager.loadByForeignKeyWithSpecifiedParent(abstractModule, pools, scope, messageContext, this , null, extraWhere, localContextName, isDelta);
		Pool[] loadedPools = null;
		if (temp != null)
		{
			loadedPools = new Pool[temp.size()];
			temp.toArray(loadedPools);
		}
		return loadedPools;
	}

	protected PoolDomain[] loadPoolDomains(AbstractModule abstractModule,
		PoolDomain[] poolDomains, PoolDomainScope scope,
		MessageContext messageContext, boolean isDelta)
		throws RPMException, SQLException, ParseException
	{
		String extraWhere = " LEVEL_ID = 0 AND (" + " TYPE_ID = "+ PoolDomain.TYPE_ID + " )";
		PoolDomainsManager poolDomainManager = (PoolDomainsManager) getManagerInstance(PoolDomain.class.getName());
		ArrayList temp = poolDomainManager.loadByForeignKeyWithSpecifiedParent(abstractModule, poolDomains,
		scope, messageContext, this, null, extraWhere, localContextName, isDelta);
		PoolDomain[] loadedPoolDomains = null;
		if (temp != null)
		{
			loadedPoolDomains = new PoolDomain[temp.size()];
			temp.toArray(loadedPoolDomains);
		}
		return loadedPoolDomains;
	}

	protected Pool[] savePools(AbstractModule module, Pool[] pools,
            PoolScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        propagateContextName(module, pools);
        Pool[] savedPools = (Pool[]) updateChildren(module, pools, scope,
                messageContext, null);
        return savedPools;
    }

	protected PoolDomain[] savePoolDomains(AbstractModule module,
            PoolDomain[] poolDomains, PoolDomainScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        propagateContextName(module, poolDomains);
        PoolDomain[] savedPoolDomains = (PoolDomain[]) updateChildren(module,
                poolDomains, scope, messageContext, null);
        return savedPoolDomains;
    }
}
