package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.scope.PoolReferenceScope;

public class PoolReferenceManager extends AbstractRPMObjectManager
{
	private static Log logger = LogFactory.getLog(PoolReferenceManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(PoolReference.class.getName());


        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    PoolReferenceManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "POOL_REFERENCES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column POOL_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_POOL_ID = 2;

    public static final int TYPE_POOL_ID = Types.CHAR;

    public static final String NAME_POOL_ID = "POOL_REFERENCES.POOL_ID";

    public static final String PROPERTY_POOL_ID = "POOLID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_POOL_ID, NAME_POOL_ID);
    }

    /**
     * Column COMPANY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COMPANY_NAME = 3;

    public static final int TYPE_COMPANY_NAME = Types.VARCHAR;

    public static final String NAME_COMPANY_NAME = "POOL_REFERENCES.COMPANY_NAME";

    public static final String PROPERTY_COMPANY_NAME = "ORGANIZATIONNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_NAME, NAME_COMPANY_NAME);
    }

    /**
     * Column ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ADDRESS = 4;

    public static final int TYPE_ADDRESS = Types.VARCHAR;

    public static final String NAME_ADDRESS = "POOL_REFERENCES.ADDRESS";

    public static final String PROPERTY_ADDRESS = "ORGANIZATIONADDRESS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ADDRESS, NAME_ADDRESS);
    }

    /**
     * Column PHONE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PHONE = 5;

    public static final int TYPE_PHONE = Types.VARCHAR;

    public static final String NAME_PHONE = "POOL_REFERENCES.PHONE";

    public static final String PROPERTY_PHONE = "ORGANIZATIONTELEPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PHONE, NAME_PHONE);
    }

    /**
     * Column FAX of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FAX = 6;

    public static final int TYPE_FAX = Types.VARCHAR;

    public static final String NAME_FAX = "POOL_REFERENCES.FAX";

    public static final String PROPERTY_FAX = "ORGANIZATIONFAX";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FAX, NAME_FAX);
    }

    /**
     * Column WEBSITE of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_WEBSITE = 7;

    public static final int TYPE_WEBSITE = Types.VARCHAR;

    public static final String NAME_WEBSITE = "POOL_REFERENCES.WEBSITE";

    public static final String PROPERTY_WEBSITE = "ORGANIZATIONWEBSITE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEBSITE, NAME_WEBSITE);
    }

    /**
     * Column COMPANY_SIZE of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_COMPANY_SIZE = 8;

    public static final int TYPE_COMPANY_SIZE = Types.VARCHAR;

    public static final String NAME_COMPANY_SIZE = "POOL_REFERENCES.COMPANY_SIZE";

    public static final String PROPERTY_COMPANY_SIZE = "COMPANYSIZE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_SIZE, NAME_COMPANY_SIZE);
    }

    /**
     * Column CONTACT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACT_NAME = 9;

    public static final int TYPE_CONTACT_NAME = Types.VARCHAR;

    public static final String NAME_CONTACT_NAME = "POOL_REFERENCES.CONTACT_NAME";

    public static final String PROPERTY_CONTACT_NAME = "CONTACTNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACT_NAME, NAME_CONTACT_NAME);
    }

    /**
     * Column CONTACT_PHONE of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CONTACT_PHONE = 10;

    public static final int TYPE_CONTACT_PHONE = Types.VARCHAR;

    public static final String NAME_CONTACT_PHONE = "POOL_REFERENCES.CONTACT_PHONE";

    public static final String PROPERTY_CONTACT_PHONE = "CONTACTTELEPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACT_PHONE, NAME_CONTACT_PHONE);
    }

    /**
     * Column CONTACT_FAX of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CONTACT_FAX = 11;

    public static final int TYPE_CONTACT_FAX = Types.VARCHAR;

    public static final String NAME_CONTACT_FAX = "POOL_REFERENCES.CONTACT_FAX";

    public static final String PROPERTY_CONTACT_FAX = "CONTACTFAX";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACT_FAX, NAME_CONTACT_FAX);
    }

    /**
     * Column CONTACT_EMAIL of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACT_EMAIL = 12;

    public static final int TYPE_CONTACT_EMAIL = Types.VARCHAR;

    public static final String NAME_CONTACT_EMAIL = "POOL_REFERENCES.CONTACT_EMAIL";

    public static final String PROPERTY_CONTACT_EMAIL = "CONTACTEMAIL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONTACT_EMAIL, NAME_CONTACT_EMAIL);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 13;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "POOL_REFERENCES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 14;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "POOL_REFERENCES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_USER = 15;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "POOL_REFERENCES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    private static final String TABLE_NAME = "POOL_REFERENCES";

    /**
     * Create an array of type string containing all the fields of the
     * POOL_REFERENCES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_POOL_ID, NAME_COMPANY_NAME, NAME_ADDRESS,
            NAME_PHONE, NAME_FAX, NAME_WEBSITE, NAME_COMPANY_SIZE,
            NAME_CONTACT_NAME, NAME_CONTACT_PHONE, NAME_CONTACT_FAX,
            NAME_CONTACT_EMAIL, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_REC_USER
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }


    public PoolReferenceManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        return new Integer(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            
            if (condition != null)
            {
                joinCondition.setCondition(condition.toString());
            }
        }
        else if (manager instanceof PoolManager)
        {
            if (condition != null)
            {
                joinCondition = new JoinCondition();
                joinCondition.setCondition(condition.toString());
            }
        }
        else if (manager instanceof PoolReferenceManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            int rankLenght = 4 * level;
            String rank = getTableName() + ".RANK";

            where.append("(LEVEL_ID >= ");
            where.append(level);
            where.append(") AND SUBSTR(");
            where.append(rank);
            where.append(",1,LENGTH(");
            where.append(rank);
            where.append(")- ");
            where.append(rankLenght);
            where.append(") IN (SELECT ");
            where.append(rank);
            where.append(" FROM ");
            where.append(getTableName());
            if (containerName != null || condition != null)
            {
                where.append(" WHERE ");
                if (condition != null)
                {
                    where.append(condition);
                }
            }

            joinCondition.setCondition(where.toString());
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(PoolManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_POOL_ID);
            where.append(" FROM ");
            where.append(getTableName());

            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.SUB_TABLE_JOIN);
        }
        
        return joinCondition;
    }
        
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;
        if (typeId != 0)
        {
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        object = new PoolReference();
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof PoolReferenceManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(getPrimaryKey() + " = ?");
            joinCondition = getJoinCondition(manager, joinCondition.toString(),
                    1, null, null);
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof PoolManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" ");
            condition.append(NAME_POOL_ID);
            condition.append(" ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                    null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet,
     *      com.ibm.rpm.scope.RPMObjectScope)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException,
            SQLException, ParseException {

        
        PoolReference object = (PoolReference)rpmObject;
        PoolReferenceScope scope = (PoolReferenceScope)rpmScope;
        
        //Pool is always loaded without scope
        
        String poolId = (String) relatedFields.get(index, ID_POOL_ID);
        PoolManager poolManager = (PoolManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(Pool.class.getName());

        Pool pool = new Pool();
        pool.setID(poolId);
        pool = (Pool) poolManager.loadByPrimaryKey(pool,
                null, messageContext, isDelta);
        if (isDelta) {
            object.deltaParent(pool);
        } else {
            object.setParent(pool);
        }

        if (scope != null)
        {
            //CompanySize, Datafield
            if (scope.isCompanySize())
            {
                CompanySize status = new CompanySize();
                status.setID((String)relatedFields.get(index, ID_COMPANY_SIZE));
                
                DatafieldManager datafieldManager = (DatafieldManager)RPMManagerFactory.getInstance().getRPMObjectManager(CompanySize.class.getName());                
                RPMObject loadedSize = datafieldManager.loadByPrimaryKey(status,null, messageContext, isDelta);
                if (loadedSize instanceof CompanySize)
                {
                    CompanySize loadedCompanySize = (CompanySize) loadedSize;
                    if (isDelta)
                    {
                        object.deltaCompanySize(loadedCompanySize);
                    }
                    else
                    {
                        object.setCompanySize(loadedCompanySize);
                    }
                }
            }

        }
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
            throws SQLException
    {
        PoolReference ref = (PoolReference) rpmObject;

        ref.setID(rs.getString(ID_ELEMENT_ID));

        if (isDelta)
        {
            ref.deltaContactEmail(rs.getString(ID_CONTACT_EMAIL));
            ref.deltaContactFax(rs.getString(ID_CONTACT_FAX));
            ref.deltaContactName(rs.getString(ID_CONTACT_NAME));
            ref.deltaContactTelephone(rs.getString(ID_CONTACT_PHONE));
            ref.deltaOrganizationAddress(rs.getString(ID_ADDRESS));
            ref.deltaOrganizationName(rs.getString(ID_COMPANY_NAME));
            ref.deltaOrganizationFax(rs.getString(ID_FAX));
            ref.deltaOrganizationTelephone(rs.getString(ID_PHONE));
            ref.deltaOrganizationWebsite(rs.getString(ID_WEBSITE));
        }
        else
        {
            ref.setContactEmail(rs.getString(ID_CONTACT_EMAIL));
            ref.setContactFax(rs.getString(ID_CONTACT_FAX));
            ref.setContactName(rs.getString(ID_CONTACT_NAME));
            ref.setContactTelephone(rs.getString(ID_CONTACT_PHONE));
            ref.setOrganizationAddress(rs.getString(ID_ADDRESS));
            ref.setOrganizationName(rs.getString(ID_COMPANY_NAME));
            ref.setOrganizationFax(rs.getString(ID_FAX));
            ref.setOrganizationTelephone(rs.getString(ID_PHONE));
            ref.setOrganizationWebsite(rs.getString(ID_WEBSITE));
        }

        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String companySize = resultSet.getString(ID_COMPANY_SIZE);
        relatedFields.put(index, ID_COMPANY_SIZE, companySize);

        String poolId = resultSet.getString(ID_POOL_ID);
        relatedFields.put(index, ID_POOL_ID, poolId);
        
        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException,
            RPMException
    {
        PoolReference savedObj = null;
        savedObj = (PoolReference) rpmObject;
        savedObj.setID(SP_I_POOL_REF(savedObj, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException,
            RPMException
    {
        PoolReference savedObj = null;
        savedObj = (PoolReference) rpmObject;
        if (savedObj != null)
        {
            SP_U_POOL_REF(savedObj, messageContext);
        }
        return savedObj;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext) throws RPMException, SQLException
    {
        PoolReference ref = null;
        if (rpmObject != null)
        {
            ref = (PoolReference) rpmObject;
            SP_D_POOL_REF(ref, messageContext);
        }
    }

    private String SP_I_POOL_REF(PoolReference ref, MessageContext messageContext) throws SQLException,
            RPMException
    {

        String elementId = null;
        String name = "SP_I_POOL_REF";
        Object args[] = new Object[4];
        args[0] = ref.getOrganizationName();

        if ((args[1] == null) && ref.getParent() != null)
        {
            args[1] = ref.getParent().getID();
        }

        //TODO How come that in the previous version the type_id equals 93 ?
        args[2] = ref.retrieveTypeID();
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    private String SP_U_POOL_REF(PoolReference ref, MessageContext messageContext) throws SQLException,
            RPMException
    {
        //SP_U_POOL_REF('0BEE1EE553914144B84FEC5BC8D48615','toto','addresse','779','777',null,'132','tuut','779','666',null,'PMO_SUPERVISOR')
        String elementId = null;
        String name = "SP_U_POOL_REF";
        Object args[] = new Object[12];
        args[0] = ref.getID();
        args[1] = ref.getOrganizationName();
        args[2] = ref.getOrganizationAddress();
        args[3] = ref.getOrganizationTelephone();
        args[4] = ref.getOrganizationFax();
        args[5] = ref.getOrganizationWebsite();

        if (ref.getCompanySize() != null)
            args[6] = ref.getCompanySize().getID();

        args[7] = ref.getContactName();
        args[8] = ref.getContactTelephone();
        args[9] = ref.getContactFax();
        args[10] = ref.getContactEmail();

        if (getUser(messageContext) != null)
            args[11] = getUser(messageContext).getID();

        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private void SP_D_POOL_REF(PoolReference ref, MessageContext messageContext) throws SQLException, RPMException
    {
        // SP_D_POOL_REF('6968AF9B597C46A9BF115499BE8FA259','1AB36621EA8C478EB6D379C7DC678901','X','PMO_SUPERVISOR')
        String name = "SP_D_POOL_REF";
        Object args[] = new Object[4];
        args[0] = ref.getID();
        if (ref.getParent() != null)
        {
            args[1] = ref.getParent().getID();
        }
        args[2] = "X";
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        executeProcedure(messageContext, name, args);
    }
}
