package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

public class ResourceAttributeManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ResourceAttributeCompetencyCategory.class.getName());
        CONTAINERS.add(ResourceAttributeCompetencyClassification.class
            .getName());
        CONTAINERS.add(ResourceAttributeCompetency.class.getName());
        CONTAINERS.add(ResourceAttributeCompetencyProficiencyLevel.class
            .getName());
        CONTAINERS.add(ResourceAttributeSkillCategory.class.getName());
        CONTAINERS.add(ResourceAttributeSkillClassification.class.getName());
        CONTAINERS.add(ResourceAttributeSkill.class.getName());
        CONTAINERS.add(ResourceAttributeSkillProficiencyLevel.class.getName());
        CONTAINERS.add(ResourceAttributeProficiencyLevel.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceAttributeManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_COMPETENCIES.LEVEL_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_LEVEL_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_COMPETENCIES.TYPE_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_TYPE_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_COMPETENCIES.MAX_RANK";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_MAX_RANK = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_COMPETENCIES.CHILD_COUNT";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_CHILD_COUNT = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_COMPETENCIES.DELETED_COUNT";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_DELETED_COUNT = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_COMPETENCIES.ELEMENT_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_COMPETENCIES.ELEMENT_NAME";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_COMPETENCIES.PARENT_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_PARENT_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 9;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "TMT_COMPETENCIES.RESOURCE_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_RESOURCE_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column PROFICIENCY_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROFICIENCY_LEVEL_ID = 10;

    public static final int TYPE_PROFICIENCY_LEVEL_ID = Types.INTEGER;

    public static final String NAME_PROFICIENCY_LEVEL_ID = "TMT_COMPETENCIES.PROFICIENCY_LEVEL_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_PROFICIENCY_LEVEL_ID =
    // "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROFICIENCY_LEVEL_ID,NAME_PROFICIENCY_LEVEL_ID);}

    /**
     * Column EXPERIENCE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPERIENCE_ID = 11;

    public static final int TYPE_EXPERIENCE_ID = Types.INTEGER;

    public static final String NAME_EXPERIENCE_ID = "TMT_COMPETENCIES.EXPERIENCE_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_EXPERIENCE_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_ID,NAME_EXPERIENCE_ID);}

    /**
     * Column PREFERENCE_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PREFERENCE_LEVEL_ID = 12;

    public static final int TYPE_PREFERENCE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_PREFERENCE_LEVEL_ID = "TMT_COMPETENCIES.PREFERENCE_LEVEL_ID";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_PREFERENCE_LEVEL_ID = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PREFERENCE_LEVEL_ID,NAME_PREFERENCE_LEVEL_ID);}

    /**
     * Column RATE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RATE_ID = 13;

    public static final int TYPE_RATE_ID = Types.CHAR;

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    public static final String NAME_RATE_ID = "TMT_COMPETENCIES.RATE_ID";

    // public static final String PROPERTY_RATE_ID = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RATE_ID,NAME_RATE_ID);}

    /**
     * Column LAST_USED_DATETIME of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LAST_USED_DATETIME = 14;

    public static final int TYPE_LAST_USED_DATETIME = Types.DATE;

    public static final String NAME_LAST_USED_DATETIME = "TMT_COMPETENCIES.LAST_USED_DATETIME";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_LAST_USED_DATETIME = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_USED_DATETIME,NAME_LAST_USED_DATETIME);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 15;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_COMPETENCIES.CREATED_BY";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_CREATED_BY = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column CREATED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATETIME = 16;

    public static final int TYPE_CREATED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATETIME = "TMT_COMPETENCIES.CREATED_DATETIME";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_CREATED_DATETIME = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATETIME,NAME_CREATED_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_FLAG = 17;

    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;

    public static final String NAME_DEFAULT_FLAG = "TMT_COMPETENCIES.DEFAULT_FLAG";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_DEFAULT_FLAG = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 18;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_COMPETENCIES.REC_USER";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_REC_USER = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 19;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_COMPETENCIES.REC_STATUS";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_REC_STATUS = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 20;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_COMPETENCIES.REC_DATETIME";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_REC_DATETIME = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 21;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_COMPETENCIES.RANK";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_RANK = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column LAST_USED_LOCATION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_USED_LOCATION = 22;

    public static final int TYPE_LAST_USED_LOCATION = Types.VARCHAR;

    public static final String NAME_LAST_USED_LOCATION = "TMT_COMPETENCIES.LAST_USED_LOCATION";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_LAST_USED_LOCATION = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_USED_LOCATION,NAME_LAST_USED_LOCATION);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 23;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_COMPETENCIES.DESCRIPTION";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column STRATEGIC_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_STRATEGIC_FLAG = 24;

    public static final int TYPE_STRATEGIC_FLAG = Types.CHAR;

    public static final String NAME_STRATEGIC_FLAG = "TMT_COMPETENCIES.STRATEGIC_FLAG";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_STRATEGIC_FLAG = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_STRATEGIC_FLAG,NAME_STRATEGIC_FLAG);}

    /**
     * Column TARGET_ATTAINMENT_DATE of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_TARGET_ATTAINMENT_DATE = 25;

    public static final int TYPE_TARGET_ATTAINMENT_DATE = Types.DATE;

    public static final String NAME_TARGET_ATTAINMENT_DATE = "TMT_COMPETENCIES.TARGET_ATTAINMENT_DATE";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_TARGET_ATTAINMENT_DATE =
    // "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_TARGET_ATTAINMENT_DATE,NAME_TARGET_ATTAINMENT_DATE);}

    /**
     * Column ACQUIRED_ON of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_ACQUIRED_ON = 26;

    public static final int TYPE_ACQUIRED_ON = Types.DATE;

    public static final String NAME_ACQUIRED_ON = "TMT_COMPETENCIES.ACQUIRED_ON";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_ACQUIRED_ON = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ACQUIRED_ON,NAME_ACQUIRED_ON);}

    /**
     * Column MANDATORY of type Types.CHAR mapped to String.
     */
    public static final int ID_MANDATORY = 27;

    public static final int TYPE_MANDATORY = Types.CHAR;

    public static final String NAME_MANDATORY = "TMT_COMPETENCIES.MANDATORY";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_MANDATORY = "propertyname";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MANDATORY,NAME_MANDATORY);}

    /**
     * Column MANDATORY_DETAILS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MANDATORY_DETAILS = 28;

    public static final int TYPE_MANDATORY_DETAILS = Types.VARCHAR;

    public static final String NAME_MANDATORY_DETAILS = "TMT_COMPETENCIES.MANDATORY_DETAILS";

    // uncomment and replace propertyname with the container property loaded
    // from this column if it applies.
    // public static final String PROPERTY_MANDATORY_DETAILS = "propertyname";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MANDATORY_DETAILS,NAME_MANDATORY_DETAILS);}

    private static final String TABLE_NAME = "TMT_COMPETENCIES";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_COMPETENCIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_RESOURCE_ID, NAME_PROFICIENCY_LEVEL_ID,
            NAME_EXPERIENCE_ID, NAME_PREFERENCE_LEVEL_ID, NAME_RATE_ID,
            NAME_LAST_USED_DATETIME, NAME_CREATED_BY, NAME_CREATED_DATETIME,
            NAME_DEFAULT_FLAG, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_RANK, NAME_LAST_USED_LOCATION,
            NAME_DESCRIPTION, NAME_STRATEGIC_FLAG, NAME_TARGET_ATTAINMENT_DATE,
            NAME_ACQUIRED_ON, NAME_MANDATORY, NAME_MANDATORY_DETAILS
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    public ResourceAttributeManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {

        case ResourceAttributeCompetencyCategory.TYPE_ID:
            object = new ResourceAttributeCompetencyCategory();
            break;
        case ResourceAttributeCompetencyClassification.TYPE_ID:
            object = new ResourceAttributeCompetencyClassification();
            break;
        case ResourceAttributeCompetency.TYPE_ID:
            object = new ResourceAttributeCompetency();
            break;
        case ResourceAttributeCompetencyProficiencyLevel.TYPE_ID:
            object = new ResourceAttributeCompetencyProficiencyLevel();
            break;
        case ResourceAttributeSkillCategory.TYPE_ID:
            object = new ResourceAttributeSkillCategory();
            break;
        case ResourceAttributeSkillClassification.TYPE_ID:
            object = new ResourceAttributeSkillClassification();
            break;
        case ResourceAttributeSkill.TYPE_ID:
            object = new ResourceAttributeSkill();
            break;
        case ResourceAttributeSkillProficiencyLevel.TYPE_ID:
            object = new ResourceAttributeSkillProficiencyLevel();
            break;
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject pObject,
            boolean isDelta) throws SQLException
    {

        ResourceAttribute resourceAttribute = (ResourceAttribute) pObject;
        resourceAttribute.setID(rs.getString(ID_ELEMENT_ID));

        if (isProficiencyLevel(resourceAttribute))
        {
            ResourceAttributeProficiencyLevel proficiencyLevel = (ResourceAttributeProficiencyLevel) resourceAttribute;
            if (proficiencyLevel.getRates() != null
                    && proficiencyLevel.getRates().getID() == null)
            {
                proficiencyLevel.getRates().setID(rs.getString(ID_RATE_ID));
            }
        }

        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }
        if (isDelta)
        {
            resourceAttribute.deltaName(name);
            resourceAttribute.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            resourceAttribute.setName(name);
            resourceAttribute.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }

        return resourceAttribute;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String rateId = resultSet.getString(ID_RATE_ID);
        relatedFields.put(index, ID_RATE_ID, rateId);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        ResourceAttribute resourceAttribute = null;
        if (rpmObject != null)
        {
            resourceAttribute = (ResourceAttribute) rpmObject;
            // delete the element
            SP_D_COMPET(resourceAttribute, messageContext);
        }
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        int typeId = 0;

        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeCompetency.class)))
            {
                typeId = ResourceAttributeCompetency.TYPE_ID;
            }
            else if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeCompetencyClassification.class)))
            {
                typeId = ResourceAttributeCompetencyClassification.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeCompetencyCategory.class)))
            {
                typeId = ResourceAttributeCompetencyCategory.TYPE_ID;
            }
            else if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeCompetencyProficiencyLevel.class)))
            {
                typeId = ResourceAttributeCompetencyProficiencyLevel.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeSkill.class)))
            {
                typeId = ResourceAttributeSkill.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeSkillClassification.class)))
            {
                typeId = ResourceAttributeSkillClassification.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeSkillCategory.class)))
            {
                typeId = ResourceAttributeSkillCategory.TYPE_ID;
            }
            else if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeSkillProficiencyLevel.class)))
            {
                typeId = ResourceAttributeSkillProficiencyLevel.TYPE_ID;
            }
            else
            {
                ExceptionUtil.handleNoTypeId(this, containerName);
            }
        }

        StringBuffer baseCondition = new StringBuffer();
        baseCondition.append(NAME_ELEMENT_ID);
        baseCondition.append(" IS NOT NULL ");
        if (typeId != 0)
        {
            baseCondition.append(" AND ");
            baseCondition.append(NAME_TYPE_ID);
            baseCondition.append(" = ");
            baseCondition.append(typeId);
        }

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                baseCondition.append(" AND ");
                baseCondition.append(condition);
            }
            joinCondition.setCondition(baseCondition.toString());

        }
        else if (manager instanceof ResourceAttributeManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            where.append(" WHERE ");
            where.append(baseCondition);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ResourceModuleManager)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof ResourceRatesManager)
        {
            joinCondition = new JoinCondition();

            joinCondition.setTableName(manager.getTableName());

            StringBuffer where = new StringBuffer();
            where.append(manager.getPrimaryKey());
            where.append(" IN (SELECT ");
            where.append(NAME_RATE_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            where.append(" WHERE ");
            where.append(baseCondition);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = new JoinCondition();

        if (manager instanceof ResourceAttributeAssignmentManager)
        {
            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(manager.getTableName());
            // condition on proficiencyLevel types
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" IN (");
            where.append(ResourceAttributeCompetencyProficiencyLevel.TYPE_ID);
            where.append(",");
            where.append(ResourceAttributeSkillProficiencyLevel.TYPE_ID);
            where.append(")");
            // end condition on proficiencyLevel types
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceAttributeManager)
        {
            StringBuffer where = new StringBuffer();

            where.append(NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(manager.getTableName());

            int typeId = 0;

            if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeCompetencyClassification.class)))
            {
                typeId = ResourceAttributeCompetencyClassification.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeCompetencyCategory.class)))
            {
                typeId = ResourceAttributeCompetencyCategory.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeCompetency.class)))
            {
                typeId = ResourceAttributeCompetency.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ResourceAttributeSkill.class)))
            {
                typeId = ResourceAttributeSkill.TYPE_ID;
            }
            else if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeCompetencyProficiencyLevel.class)))
            {
                typeId = ResourceAttributeCompetencyProficiencyLevel.TYPE_ID;
            }
            else if (containerName
                .equalsIgnoreCase(StringUtil
                    .getShortClassName(ResourceAttributeSkillProficiencyLevel.class)))
            {
                typeId = ResourceAttributeSkillProficiencyLevel.TYPE_ID;
            }

            if (typeId != 0)
            {
                // condition on proficiencyLevel types
                where.append(" WHERE ");
                where.append(NAME_TYPE_ID);
                where.append(" IN (");
                where.append(typeId);
                where.append(")");
            }

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceAttributeManager)
        {

            // where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(getPrimaryKey());
            condition.append(" = ?)");

            // In this case, we don't need a condition on the type of the parent
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());

        }
        else if (manager instanceof ResourceModuleManager)
        {
            joinCondition = getJoinCondition(manager, null, 2, null,
                contextName);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                joinCondition.setCondition(extraWhere);
            }
            joinCondition.setTableName(getTableName(contextName));
        }

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);

        }

        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        // Rates and ProficiencyLevel are always loaded for
        // ResourceAttributeProficiencyLevel objects.
        if (isProficiencyLevel(rpmObject))
        {
            loadRates((ResourceAttributeProficiencyLevel) rpmObject,
                messageContext, relatedFields, index, isDelta);
            loadProficiencyLevel((ResourceAttribute) rpmObject, messageContext,
                isDelta);
        }

        if (rpmScope != null)
        {

            // Parent
            if (rpmScope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                ResourceAttribute newParent = null;
                if (isDelta)
                {
                    newParent = getParentResourceAttribute((ResourceAttribute) rpmObject);
                    ResourceAttribute oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new ResourceAttribute();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = (ResourceAttribute) loadByPrimaryKey(
                                newParent, rpmScope.getParent(),
                                messageContext, true);
                        }
                        else
                        {
                            // In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldParent = (ResourceAttribute) loadByPrimaryKey(
                                oldParent, null, messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {// newParent not in cache
                            newParent = new ResourceAttribute();
                            newParent.setID(oldParentId);
                            newParent = (ResourceAttribute) loadByPrimaryKey(
                                newParent, rpmScope.getParent(),
                                messageContext, false);
                        }
                        setParent(rpmObject, newParent);
                    }
                }
            }

            if (rpmScope instanceof ResourceAttributeScope)
            {

                ResourceAttributeScope scope = (ResourceAttributeScope) rpmScope;

                // ProficiencyLevels
                if (scope.isProficiencyLevels())
                {
                    if (isCompetency(rpmObject))
                    {
                        loadCompetencyProficiencyLevels(rpmObject, scope,
                            messageContext, isDelta);
                    }
                    else if (isSkill(rpmObject))
                    {
                        loadSkillProficiencyLevels(rpmObject, scope,
                            messageContext, isDelta);
                    }
                }
                // Attributes
                if (scope.getAttributes() != null)
                {
                    if (isCompetencyClassification(rpmObject))
                    {
                        loadCompetencyAttributes(rpmObject, scope,
                            messageContext, isDelta);
                    }
                    else if (isSkillClassification(rpmObject))
                    {
                        loadSkillAttributes(rpmObject, scope, messageContext,
                            isDelta);
                    }
                }

                // Classifications
                if (scope.getClassifications() != null)
                {
                    if (isCompetencyCategory(rpmObject))
                    {
                        loadCompetencyClassifications(rpmObject, scope,
                            messageContext, isDelta);
                    }
                    else if (isSkillCategory(rpmObject))
                    {
                        loadSkillClassifications(rpmObject, scope,
                            messageContext, isDelta);
                    }
                }
            }

        }
        return rpmObject;
    }

    private void loadRates(ResourceAttributeProficiencyLevel proficiencyLevel,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceRates rates = null;
        if (proficiencyLevel.getRates() != null)
        {
            rates = proficiencyLevel.getRates();
        }
        else
        {
            rates = new ResourceRates();
            rates.setID((String) relatedFields.get(index, ID_RATE_ID));
        }

        if (rates.getID() != null)
        {
            ResourceRatesManager manager = (ResourceRatesManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceRates.class.getName());

            rates = (ResourceRates) manager.loadByPrimaryKey(rates, null,
                messageContext, isDelta);

            if (isDelta)
            {
                proficiencyLevel.deltaRates(rates);
            }
            else
            {
                proficiencyLevel.setRates(rates);

            }
        }

    }

    private ResourceAttribute setParent(RPMObject rpmChild, RPMObject rpmParent)
            throws RPMException
    {
        ResourceAttribute child = (ResourceAttribute) rpmChild;
        // TODO add validation call here
        child.setParent(rpmParent);
        return child;
    }

    private void loadCompetencyProficiencyLevels(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeCompetency competency = (ResourceAttributeCompetency) rpmObject;
        ResourceAttributeCompetencyProficiencyLevel[] children = competency
            .getProficiencyLevels();
        ArrayList temp = loadChildren(rpmObject, children, null,
            messageContext,
            ResourceAttributeCompetencyProficiencyLevel.TYPE_ID, isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeCompetencyProficiencyLevel[temp
                .size()];
            temp.toArray(children);
            competency.setProficiencyLevels(children);
        }
    }

    private void loadSkillProficiencyLevels(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeSkill skill = (ResourceAttributeSkill) rpmObject;
        ResourceAttributeSkillProficiencyLevel[] children = skill
            .getProficiencyLevels();
        ArrayList temp = loadChildren(rpmObject, children, null,
            messageContext, ResourceAttributeSkillProficiencyLevel.TYPE_ID,
            isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeSkillProficiencyLevel[temp.size()];
            temp.toArray(children);
            skill.setProficiencyLevels(children);
        }
    }

    private void loadCompetencyAttributes(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeCompetencyClassification classification = (ResourceAttributeCompetencyClassification) rpmObject;
        ResourceAttributeCompetency[] children = classification.getAttributes();
        ArrayList temp = loadChildren(rpmObject, children, scope
            .getAttributes(), messageContext,
            ResourceAttributeCompetency.TYPE_ID, isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeCompetency[temp.size()];
            temp.toArray(children);
            classification.setAttributes(children);
        }
    }

    private void loadSkillAttributes(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeSkillClassification classification = (ResourceAttributeSkillClassification) rpmObject;
        ResourceAttributeSkill[] children = classification.getAttributes();
        ArrayList temp = loadChildren(rpmObject, children, scope
            .getAttributes(), messageContext, ResourceAttributeSkill.TYPE_ID,
            isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeSkill[temp.size()];
            temp.toArray(children);
            classification.setAttributes(children);
        }
    }

    private void loadCompetencyClassifications(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeCompetencyCategory category = (ResourceAttributeCompetencyCategory) rpmObject;
        ResourceAttributeCompetencyClassification[] children = category
            .getClassifications();
        ArrayList temp = loadChildren(rpmObject, children, scope
            .getClassifications(), messageContext,
            ResourceAttributeCompetencyClassification.TYPE_ID, isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeCompetencyClassification[temp
                .size()];
            temp.toArray(children);
            category.setClassifications(children);
        }
    }

    private void loadSkillClassifications(RPMObject rpmObject,
            ResourceAttributeScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceAttributeSkillCategory category = (ResourceAttributeSkillCategory) rpmObject;
        ResourceAttributeSkillClassification[] children = category
            .getClassifications();
        ArrayList temp = loadChildren(rpmObject, children, scope
            .getClassifications(), messageContext,
            ResourceAttributeSkillClassification.TYPE_ID, isDelta);
        if (temp != null)
        {
            children = new ResourceAttributeSkillClassification[temp.size()];
            temp.toArray(children);
            category.setClassifications(children);
        }
    }

    /**
     * @deprecated use loadByForeignKey()
     */
    private ArrayList loadChildren(RPMObject parent, RPMObject[] oldChildren,
            ResourceAttributeScope childrenScope,
            MessageContext messageContext, int whereTypeId, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        return loadByForeignKey(oldChildren, childrenScope, messageContext,
            this, new Object[]
            {
                parent.getID()
            }, "TYPE_ID = " + whereTypeId, null, isDelta);
    }

    private boolean isCompetency(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeCompetency.TYPE_ID;
    }

    private boolean isCompetencyCategory(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeCompetencyCategory.TYPE_ID;
    }

    private boolean isCompetencyClassification(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeCompetencyClassification.TYPE_ID;
    }

    private boolean isCompetencyProficiencyLevel(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeCompetencyProficiencyLevel.TYPE_ID;
    }

    private boolean isSkill(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeSkill.TYPE_ID;
    }

    private boolean isSkillCategory(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeSkillCategory.TYPE_ID;
    }

    private boolean isSkillClassification(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeSkillClassification.TYPE_ID;
    }

    private boolean isSkillProficiencyLevel(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == ResourceAttributeSkillProficiencyLevel.TYPE_ID;
    }

    private boolean isProficiencyLevel(RPMObject rpmObject)
    {
        return isCompetencyProficiencyLevel(rpmObject)
                || isSkillProficiencyLevel(rpmObject);
    }

    public boolean isRelatedToCompetency(RPMObject rpmObject)
    {
        return isCompetency(rpmObject) || isCompetencyCategory(rpmObject)
                || isCompetencyClassification(rpmObject)
                || isCompetencyProficiencyLevel(rpmObject);
    }

    public boolean isRelatedToSkill(RPMObject rpmObject)
    {
        return isSkill(rpmObject) || isSkillCategory(rpmObject)
                || isSkillClassification(rpmObject)
                || isSkillProficiencyLevel(rpmObject);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        ResourceAttribute resourceAttribute = (ResourceAttribute) rpmObject;

        if (!isCompetencyProficiencyLevel(resourceAttribute)
                && !isSkillProficiencyLevel(resourceAttribute))
        {// not a ProficiencyLevel
            // insert the resource attribute
            resourceAttribute.setID(SP_I_COMPET(resourceAttribute,
                messageContext));
        }
        else
        {// proficiencyLevel
            ProficiencyLevel proficiencyLevel = null;

            if (isSkillProficiencyLevel(resourceAttribute)
                    && ((ResourceAttributeSkillProficiencyLevel) resourceAttribute)
                        .getProficiencyLevel() != null
                    && ((ResourceAttributeSkillProficiencyLevel) resourceAttribute)
                        .getProficiencyLevel().getID() != null)
            {
                proficiencyLevel = ((ResourceAttributeSkillProficiencyLevel) resourceAttribute)
                    .getProficiencyLevel();
            }
            else if (isCompetencyProficiencyLevel(resourceAttribute)
                    && ((ResourceAttributeCompetencyProficiencyLevel) resourceAttribute)
                        .getProficiencyLevel() != null
                    && ((ResourceAttributeCompetencyProficiencyLevel) resourceAttribute)
                        .getProficiencyLevel().getID() != null)
            {
                proficiencyLevel = ((ResourceAttributeCompetencyProficiencyLevel) resourceAttribute)
                    .getProficiencyLevel();
            }

            // If no ProficiencyLevel Datafield provided, check if the provided
            // name of the proficiency level exists as a Datafield
            if (proficiencyLevel == null)
            {
                proficiencyLevel = loadProficiencyLevel(resourceAttribute,
                    messageContext, false);
            }
            else
            {
                // Send a warning if the name of the profiency level was
                // overriden
                if (resourceAttribute.getName() != null
                        && resourceAttribute.getName() != proficiencyLevel
                            .getValue())
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(resourceAttribute
                                .getClass()), proficiencyLevel.getValue()
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.resource.constants.ErrorCodes.WARNING_NAME_OVERRIDEN_BY_PROFICIENCY_LEVEL,
                        errorParams,
                        rpmObject.getClass().getName(),
                        com.ibm.rpm.resource.constants.ValidationConstants.RESOURCEATTRIBUTE_NAME_FIELD,
                        rpmObject.getID());
                    e.setSeverity(SeverityLevel.Warning);
                    messageContext.addExceptionNoThrow(e);
                }
            }

            if (proficiencyLevel != null)
            {
                String proficiencyLevelId = proficiencyLevel.getID();

                // insert the proficiency level
                resourceAttribute.setID(SP_I_COMPET(resourceAttribute,
                    messageContext));
                // update the name and proficiency_level_ID
                SP_U_COMP_PROF(resourceAttribute, proficiencyLevelId,
                    messageContext);

                if (isRatesNotNull((ResourceAttributeProficiencyLevel) resourceAttribute))
                {
                    // update the rates
                    SP_U_COMPET_RATE(
                        (ResourceAttributeProficiencyLevel) resourceAttribute,
                        messageContext);
                }
            }
            else
            {
                ExceptionUtil.handleRPMException(this,
                    ErrorCodes.INVALID_PROFICIENCY_LEVEL, resourceAttribute,
                    "name", resourceAttribute.getName());
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ResourceAttribute resourceAttribute = (ResourceAttribute) rpmObject;
        ResourceAttributeScope scope = (ResourceAttributeScope) rpmScope;

        if (rpmObject != null)
        {
            if (isCompetencyCategory(resourceAttribute))
            {
                resourceAttribute = saveCompetencyCategory(resourceAttribute,
                    scope, messageContext);
            }
            else if (isCompetencyClassification(resourceAttribute))
            {
                resourceAttribute = saveCompetencyClassification(
                    resourceAttribute, scope, messageContext);
            }
            else if (isCompetency(resourceAttribute))
            {
                resourceAttribute = saveCompetency(resourceAttribute, scope,
                    messageContext);
            }
            else if (isCompetencyProficiencyLevel(resourceAttribute))
            {
                resourceAttribute = saveProficiencyLevel(
                    (ResourceAttributeProficiencyLevel) resourceAttribute,
                    messageContext);
            }
            else if (isSkillCategory(resourceAttribute))
            {
                resourceAttribute = saveSkillCategory(resourceAttribute, scope,
                    messageContext);
            }
            else if (isSkillClassification(resourceAttribute))
            {
                resourceAttribute = saveSkillClassification(resourceAttribute,
                    scope, messageContext);
            }
            else if (isSkill(resourceAttribute))
            {
                resourceAttribute = saveSkill(resourceAttribute, scope,
                    messageContext);
            }
            else if (isSkillProficiencyLevel(resourceAttribute))
            {
                resourceAttribute = saveProficiencyLevel(
                    (ResourceAttributeProficiencyLevel) resourceAttribute,
                    messageContext);
            }
        }
        return resourceAttribute;
    }

    private ProficiencyLevel loadProficiencyLevel(
            ResourceAttribute resourceAttribute, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DatafieldManager datafieldManager = (DatafieldManager) RPMManagerFactory
            .getInstance()
            .getRPMObjectManager(ProficiencyLevel.class.getName());
        Object[] args = new Object[]
        {
            resourceAttribute.getName()
        };
        ArrayList proficiencyLevels = datafieldManager.loadByForeignKey(
            new RPMObject[]
            {
                null
            }, null, messageContext, this, args, null, null, isDelta);
        if (proficiencyLevels != null && proficiencyLevels.size() != 0)
        {
            if (isSkillProficiencyLevel(resourceAttribute))
            {
                ((ResourceAttributeSkillProficiencyLevel) resourceAttribute)
                    .setProficiencyLevel((ProficiencyLevel) proficiencyLevels
                        .get(0));
            }
            else
            {
                ((ResourceAttributeCompetencyProficiencyLevel) resourceAttribute)
                    .setProficiencyLevel((ProficiencyLevel) proficiencyLevels
                        .get(0));
            }

            return (ProficiencyLevel) proficiencyLevels.get(0);
        }
        return null;
    }

    private boolean isRatesNotNull(
            ResourceAttributeProficiencyLevel proficiencyLevel)
    {
        return proficiencyLevel.getRates() != null
                && (proficiencyLevel.getRates().getRegularCostPerHour() != null
                        || proficiencyLevel.getRates().getSellingRatePerHour() != null
                        || proficiencyLevel.getRates().getSpecialCostPerHour() != null || proficiencyLevel
                    .getRates().getSpecialSellingRatePerHour() != null);
    }

    private ResourceAttribute saveCompetencyCategory(
            ResourceAttribute resourceAttribute, ResourceAttributeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceAttributeCompetencyCategory category = (ResourceAttributeCompetencyCategory) resourceAttribute;

        if (category.testNameModified())
        {
            // update the name
            SP_U_COMPET(category, messageContext);
        }

        if (scope != null)
        {
            category
                .setClassifications((ResourceAttributeCompetencyClassification[]) updateChildren(
                    category, category.getClassifications(), scope
                        .getClassifications(), messageContext, null));
        }
        return category;
    }

    private ResourceAttribute saveCompetencyClassification(
            ResourceAttribute resourceAttribute, ResourceAttributeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceAttributeCompetencyClassification classification = (ResourceAttributeCompetencyClassification) resourceAttribute;

        if (classification.testNameModified())
        {
            // update the name
            SP_U_COMPET(classification, messageContext);
        }

        if (scope != null)
        {
            classification
                .setAttributes((ResourceAttributeCompetency[]) updateChildren(
                    classification, classification.getAttributes(), scope
                        .getAttributes(), messageContext, null));
        }

        return classification;
    }

    private ResourceAttribute saveCompetency(
            ResourceAttribute resourceAttribute, ResourceAttributeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceAttributeCompetency competency = (ResourceAttributeCompetency) resourceAttribute;

        if (competency.testNameModified())
        {
            // update the name
            SP_U_COMPET(competency, messageContext);
        }

        if (scope != null)
        {
            competency
                .setProficiencyLevels((ResourceAttributeCompetencyProficiencyLevel[]) updateChildren(
                    competency, competency.getProficiencyLevels(), scope
                        .isProficiencyLevels(), messageContext, null));
        }

        return competency;
    }

    private ResourceAttribute saveSkillCategory(
            ResourceAttribute resourceAttribute, ResourceAttributeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceAttributeSkillCategory category = (ResourceAttributeSkillCategory) resourceAttribute;

        if (category.testNameModified())
        {
            // update the name
            SP_U_COMPET(category, messageContext);
        }

        if (scope != null)
        {
            category
                .setClassifications((ResourceAttributeSkillClassification[]) updateChildren(
                    category, category.getClassifications(), scope
                        .getClassifications(), messageContext, null));
        }

        return category;
    }

    private ResourceAttribute saveSkillClassification(
            ResourceAttribute resourceAttribute, ResourceAttributeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceAttributeSkillClassification classification = (ResourceAttributeSkillClassification) resourceAttribute;

        if (classification.testNameModified())
        {
            // update the name
            SP_U_COMPET(classification, messageContext);
        }

        if (scope != null)
        {
            classification
                .setAttributes((ResourceAttributeSkill[]) updateChildren(
                    classification, classification.getAttributes(), scope
                        .getAttributes(), messageContext, null));
        }

        return classification;
    }

    private ResourceAttribute saveSkill(ResourceAttribute resourceAttribute,
            ResourceAttributeScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceAttributeSkill skill = (ResourceAttributeSkill) resourceAttribute;

        if (skill.testNameModified())
        {
            // update the name
            SP_U_COMPET(skill, messageContext);
        }

        if (scope != null)
        {
            skill
                .setProficiencyLevels((ResourceAttributeSkillProficiencyLevel[]) updateChildren(
                    skill, skill.getProficiencyLevels(), scope
                        .isProficiencyLevels(), messageContext, null));
        }

        return skill;
    }

    private ResourceAttribute saveProficiencyLevel(
            ResourceAttributeProficiencyLevel proficiencyLevel,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        if (proficiencyLevel.getRates() != null)
        {
            if (proficiencyLevel.getRates().testRegularCostPerHourModified()
                    || proficiencyLevel.getRates()
                        .testSellingRatePerHourModified()
                    || proficiencyLevel.getRates()
                        .testSpecialCostPerHourModified()
                    || proficiencyLevel.getRates()
                        .testSpecialSellingRatePerHourModified())
            {
                // update the rates
                SP_U_COMPET_RATE(proficiencyLevel, messageContext);
            }
        }
        return proficiencyLevel;
    }

    private String SP_I_COMPET(ResourceAttribute resourceAttribute,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        String sp_name = "SP_I_COMPET";
        Object args[] = new Object[4];

        args[0] = resourceAttribute.getName();
        args[1] = getParentId(resourceAttribute, messageContext);
        args[2] = resourceAttribute.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, sp_name, args);
        return elementId;
    }

    private void SP_U_COMPET(ResourceAttribute resourceAttribute,
            MessageContext messageContext) throws SQLException, ParseException,
            RPMException
    {
        String sp_name = "SP_U_COMPET";

        Object args[] = new Object[13];
        args[0] = resourceAttribute.getID();
        args[1] = getParentId(resourceAttribute, messageContext);
        args[2] = resourceAttribute.getName();
        args[3] = null;
        args[4] = new Integer(2771);
        args[5] = new Integer(2759);
        args[6] = null;
        args[7] = null;
        args[8] = null;
        args[9] = null;
        args[10] = null;
        args[11] = null;
        args[12] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    private void SP_U_COMP_PROF(ResourceAttribute resourceAttribute,
            String proficiencyLevelId, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        String sp_name = "SP_U_COMP_PROF";

        Object args[] = new Object[3];
        args[0] = resourceAttribute.getID();
        args[1] = new Integer(proficiencyLevelId);
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    private void SP_U_COMPET_RATE(
            ResourceAttributeProficiencyLevel proficiencyLevel,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        String sp_name = "SP_U_COMPET_RATE";

        Object args[] = new Object[7];
        args[0] = proficiencyLevel.getID();
        args[1] = getParentId(proficiencyLevel, messageContext);
        args[2] = (proficiencyLevel.getRates() != null && proficiencyLevel
            .getRates().getRegularCostPerHour() != null) ? proficiencyLevel
            .getRates().getRegularCostPerHour() : new Double(0);
        args[3] = (proficiencyLevel.getRates() != null && proficiencyLevel
            .getRates().getSellingRatePerHour() != null) ? proficiencyLevel
            .getRates().getSellingRatePerHour() : new Double(0);
        args[4] = (proficiencyLevel.getRates() != null && proficiencyLevel
            .getRates().getSpecialCostPerHour() != null) ? proficiencyLevel
            .getRates().getSpecialCostPerHour() : new Double(0);
        args[5] = (proficiencyLevel.getRates() != null && proficiencyLevel
            .getRates().getSpecialSellingRatePerHour() != null)
                ? proficiencyLevel.getRates().getSpecialSellingRatePerHour()
                : new Double(0);
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    private void SP_D_COMPET(ResourceAttribute resourceAttribute,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        String sp_name = "SP_D_COMPET";

        Object args[] = new Object[4];
        args[0] = resourceAttribute.getID();
        args[1] = getParentId(resourceAttribute, messageContext);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    private ResourceAttribute getParentResourceAttribute(
            ResourceAttribute childResourceAttribute)
    {
        return (ResourceAttribute) childResourceAttribute.getParent();
    }

    private String getParentId(ResourceAttribute resourceAttribute,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        if (resourceAttribute.getParent() != null)
        {
            return resourceAttribute.getParent().getID();
        }
        else
        {
            if (!isCompetencyCategory(resourceAttribute)
                    && !isSkillCategory(resourceAttribute)
                    && resourceAttribute.getID() != null)
            {
                // reload the object with its parent in the scope, to be able to
                // retrieve the parent ID
                RPMObjectScope scope = null;
                RPMObject current = new ResourceAttribute();
                current.setID(resourceAttribute.getID());
                if (resourceAttribute instanceof ResourceAttributeCompetencyProficiencyLevel
                        || resourceAttribute instanceof ResourceAttributeSkillProficiencyLevel)
                {
                    scope = new RPMObjectScope();
                }
                else
                {
                    scope = new ResourceAttributeScope();
                }
                scope.setParent(new ResourceAttributeScope());
                current = loadByPrimaryKey(current, scope, messageContext,
                    false);

                if (current != null && current.getParent() != null
                        && current.getParent().getID() != null)
                {
                    return current.getParent().getID();
                }
            }
        }
        return null;
    }
}
