package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;


public class ResourceCostCenterManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {

        CONTAINERS.add(ResourceCostCenterAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ResourceCostCenterManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "RESOURCE_COST_CENTERS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_COST_CENTERS.RESOURCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column COST_CENTER_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_COST_CENTER_ID = 3;

    public static final int TYPE_COST_CENTER_ID = Types.CHAR;

    public static final String NAME_COST_CENTER_ID = "RESOURCE_COST_CENTERS.COST_CENTER_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_CENTER_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_ID,NAME_COST_CENTER_ID);}

    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 4;

    public static final int TYPE_START_DATE = Types.DATE;

    public static final String NAME_START_DATE = "RESOURCE_COST_CENTERS.START_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_START_DATE,NAME_START_DATE);}

    /**
     * Column FINISH_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FINISH_DATE = 5;

    public static final int TYPE_FINISH_DATE = Types.DATE;

    public static final String NAME_FINISH_DATE = "RESOURCE_COST_CENTERS.FINISH_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FINISH_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FINISH_DATE,NAME_FINISH_DATE);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 6;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "RESOURCE_COST_CENTERS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 7;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "RESOURCE_COST_CENTERS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 8;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "RESOURCE_COST_CENTERS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    private static final String TABLE_NAME = "RESOURCE_COST_CENTERS";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_COST_CENTERS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_RESOURCE_ID, NAME_COST_CENTER_ID,
            NAME_START_DATE, NAME_FINISH_DATE, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new ResourceCostCentersManager instance.
     * 
     * @return ResourceCostCentersManager
     */
    public ResourceCostCenterManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected HashSet getValidContainerNames()
    {
        return CONTAINERS;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        object = new ResourceCostCenterAssignment();
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceCostCenterAssignment rccObject = (ResourceCostCenterAssignment) rpmObject;

        rccObject.setID(rs.getString(ID_ELEMENT_ID));
        
        if(isDelta) {
            rccObject.deltaFinishDate(Manager.getCalendar(rs,ID_FINISH_DATE));
            rccObject.deltaStartDate(Manager.getCalendar(rs,ID_START_DATE));
        } else {
            rccObject.setFinishDate(Manager.getCalendar(rs,ID_FINISH_DATE));
            rccObject.setStartDate(Manager.getCalendar(rs,ID_START_DATE));
        }

        return rccObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);

        String cccId = resultSet.getString(ID_COST_CENTER_ID);
        relatedFields.put(index, ID_COST_CENTER_ID, cccId);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        SP_D_RES_COST_CENTER(
                (ResourceCostCenterAssignment) rpmObject,
                messageContext);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else if (manager instanceof ResourceManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_RESOURCE_ID);
            where.append(" = ");
            where.append(ResourceManager.NAME_RESOURCE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

            StringBuffer tables = new StringBuffer();
            tables.append(getTableName());
            tables.append(",");
            tables.append(manager.getTableName());

            joinCondition.setTableName(tables.toString());

        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.JOIN);
		}            

        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
    
        if (manager instanceof ResourceManager)
        {
    
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(ResourceManager.NAME_RESOURCE_ID);
            condition.append(" = ?");
    
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                    null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
    
            }
    
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}            
    
        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        //Everything is loaded without scope
    
        ResourceCostCenterAssignment object = (ResourceCostCenterAssignment) rpmObject;
    
        //Related resource
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                    .getInstance()
                    .getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                    null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }
    
        //Related ClientCostCenter
        String cccId = (String) relatedFields.get(index, ID_COST_CENTER_ID);
        if (cccId != null)
        {
            ClientManager clientsManager = (ClientManager) RPMManagerFactory
                    .getInstance()
                    .getRPMObjectManager(ClientCostCenter.class.getName());
            ClientCostCenter ccc = new ClientCostCenter();
            ccc.setID(cccId);
            ccc = (ClientCostCenter) clientsManager.loadByPrimaryKey(ccc,
                    null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaClientCostCenter(ccc);
            }
            else
            {
                object.setClientCostCenter(ccc);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.COST_CENTER_ID);
        }
    
        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        rpmObject.setID(SP_A_RES_COST_CENTER(
                (ResourceCostCenterAssignment) rpmObject, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // nothing
    
        return rpmObject;
    }

    /**
     * @param assignment
     * @param messageContext
     * @throws RPMException
     * @throws SQLException
     */
    private void SP_D_RES_COST_CENTER(
            ResourceCostCenterAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_D_RES_COST_CENTER";
        Object args[] = new Object[3];
        args[0] = assignment.getID();
        args[1] = ManagerUtil.getParentId(this, assignment, messageContext,
            TABLE_NAME, "ELEMENT_ID", "RESOURCE_ID");
        args[2] = getUser(messageContext).getID();
    
        executeProcedure(messageContext, name, args);    
    }

    /**
     * @param object
     * @param messageContext
     * @return
     */
    private String SP_A_RES_COST_CENTER(
            ResourceCostCenterAssignment object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_RES_COST_CENTER";
        Object args[] = new Object[3];
        args[0] = object.getParent().getID();
        args[1] = object.getClientCostCenter().getID();
        args[2] = getUser(messageContext).getID();
    
        return executeProcedure(messageContext, name, args);
    }
}