/*
 * Created on Apr 29, 2005
 *
 */

package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;



/**
 * @author Simon
 *
 */
public final class ResourceEducationManager extends AbstractRPMObjectManager
{
    
	private static final HashSet CONTAINERS = new HashSet();
    
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    
    static
    {
        CONTAINERS.add(Education.class.getName());
        
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ResourceEducationManager.class);
        }        
    } 
    
    /**
     * Column EDUCATION_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_EDUCATION_ID = 1;
    public static final int TYPE_EDUCATION_ID = Types.CHAR;
    public static final String NAME_EDUCATION_ID = "RESOURCE_EDUCATION.EDUCATION_ID";
    public static final String PROPERTY_EDUCATION_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EDUCATION_ID, NAME_EDUCATION_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;
    public static final int TYPE_RESOURCE_ID = Types.CHAR;
    public static final String NAME_RESOURCE_ID = "RESOURCE_EDUCATION.RESOURCE_ID";
    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column DEGREE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DEGREE = 3;
    public static final int TYPE_DEGREE = Types.VARCHAR;
    public static final String NAME_DEGREE = "RESOURCE_EDUCATION.DEGREE";
    public static final String PROPERTY_DEGREE = "DEGREEPROGRAM";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEGREE, NAME_DEGREE);
    }

    /**
     * Column GRADUATION of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case. 
     */
    public static final int ID_GRADUATION = 4;
    public static final int TYPE_GRADUATION = Types.CHAR;
    public static final String NAME_GRADUATION = "RESOURCE_EDUCATION.GRADUATION";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_GRADUATION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_GRADUATION,NAME_GRADUATION);}

    /**
     * Column GRADUATION_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_GRADUATION_DATE = 5;
    public static final int TYPE_GRADUATION_DATE = Types.DATE;
    public static final String NAME_GRADUATION_DATE = "RESOURCE_EDUCATION.GRADUATION_DATE";
    public static final String PROPERTY_GRADUATION_DATE = "CERTIFICATIONGRADUATEDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_GRADUATION_DATE, NAME_GRADUATION_DATE);
    }

    /**
     * Column APPOINTED of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case. 
     */
    public static final int ID_APPOINTED = 6;
    public static final int TYPE_APPOINTED = Types.CHAR;
    public static final String NAME_APPOINTED = "RESOURCE_EDUCATION.APPOINTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_APPOINTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_APPOINTED,NAME_APPOINTED);}

    /**
     * Column APPOINTED_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_APPOINTED_DATE = 7;
    public static final int TYPE_APPOINTED_DATE = Types.DATE;
    public static final String NAME_APPOINTED_DATE = "RESOURCE_EDUCATION.APPOINTED_DATE";
    public static final String PROPERTY_APPOINTED_DATE = "APPOINTEDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_APPOINTED_DATE, NAME_APPOINTED_DATE);
    }

    /**
     * Column SPECIALIZATION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SPECIALIZATION = 8;
    public static final int TYPE_SPECIALIZATION = Types.VARCHAR;
    public static final String NAME_SPECIALIZATION = "RESOURCE_EDUCATION.SPECIALIZATION";
    public static final String PROPERTY_SPECIALIZATION = "SPECIALIZATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SPECIALIZATION, NAME_SPECIALIZATION);
    }

    /**
     * Column ORGANIZATION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ORGANIZATION = 9;
    public static final int TYPE_ORGANIZATION = Types.VARCHAR;
    public static final String NAME_ORGANIZATION = "RESOURCE_EDUCATION.ORGANIZATION";
    public static final String PROPERTY_ORGANIZATION = "ORGANIZATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ORGANIZATION, NAME_ORGANIZATION);
    }

    /**
     * Column HONORS_AWARDS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_HONORS_AWARDS = 10;
    public static final int TYPE_HONORS_AWARDS = Types.VARCHAR;
    public static final String NAME_HONORS_AWARDS = "RESOURCE_EDUCATION.HONORS_AWARDS";
    public static final String PROPERTY_HONORS_AWARDS = "HONOURSAWARDS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_HONORS_AWARDS, NAME_HONORS_AWARDS);
    }

    private static final String TABLE_NAME = "RESOURCE_EDUCATION";

    /**
     * Create an array of type string containing all the fields of the RESOURCE_EDUCATION table.
     */
    private static final String[] FIELD_NAMES = 
    {
            NAME_EDUCATION_ID, NAME_RESOURCE_ID, NAME_DEGREE, NAME_GRADUATION,
            NAME_GRADUATION_DATE, NAME_APPOINTED, NAME_APPOINTED_DATE,
            NAME_SPECIALIZATION, NAME_ORGANIZATION, NAME_HONORS_AWARDS
    };

    /**
     * Field that contains the comma separated fields of the RESOURCE_EDUCATION table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

  
    public ResourceEducationManager()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_EDUCATION_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
                
        return new Integer(0);
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        
    	JoinCondition joinCondition = null;
        
        if (manager instanceof ResourceManager)
        {
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" RESOURCE_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ResourceEducationManager)
        {
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" EDUCATION_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }

        }        
        else if(manager instanceof ResourceEducationManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
        }
        else if(manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }            	
        
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }
        
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new Education();
        return object;
    }

    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

    	RPMObject pObject = rpmObject;

        pObject.setID(rs.getString(ID_EDUCATION_ID));
        if (isDelta)
        {
			((Education) pObject).deltaOrganization(rs
					.getString(ID_ORGANIZATION));
			((Education) pObject).deltaDegreeProgram(rs.getString(ID_DEGREE));
			((Education) pObject).deltaSpecialization(rs
					.getString(ID_SPECIALIZATION));
            ((Education) pObject).deltaCertificationGraduateDate(DateUtil
                .convertToCalendar(rs.getDate(ID_GRADUATION_DATE)));
            ((Education) pObject).deltaAppointedDate(DateUtil
                .convertToCalendar(rs.getDate(ID_APPOINTED_DATE)));
			((Education) pObject).deltaHonoursAwards(rs
					.getString(ID_HONORS_AWARDS));
		}
        else
        {
            ((Education) pObject)
                .setOrganization(rs.getString(ID_ORGANIZATION));
            ((Education)pObject).setDegreeProgram(rs.getString(ID_DEGREE));
            ((Education) pObject).setSpecialization(rs
                .getString(ID_SPECIALIZATION));
            ((Education) pObject).setCertificationGraduateDate(DateUtil
                .convertToCalendar(rs.getDate(ID_GRADUATION_DATE)));
            ((Education) pObject).setAppointedDate(DateUtil
                .convertToCalendar(rs.getDate(ID_APPOINTED_DATE)));
            ((Education) pObject).setHonoursAwards(rs
                .getString(ID_HONORS_AWARDS));
        }
        
        return pObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
    
        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);
        
        return relatedFields;
    }

    
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        
	    //Everything is loaded without scope
        Education object = (Education)rpmObject;

        //Related resource
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                    null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }
        
        return rpmObject;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Education pObject = (Education)rpmObject;
    	pObject.setID(SP_I_EDUCATION(pObject, messageContext));        
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Education pObject = (Education)rpmObject;
        if(pObject.testOrganizationModified()
                || pObject.testDegreeProgramModified()
                || pObject.testSpecializationModified()
                || pObject.testCertificationGraduateDateModified()
                || pObject.testAppointedDateModified()
                || pObject.testParentModified()
                || pObject.testHonoursAwardsModified())

        	SP_U_EDUCATION(pObject, messageContext);
        
        return pObject;
    }
    


    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        
        if (rpmObject != null)
        {
        	SP_D_EDUCATION((Education)rpmObject, messageContext);
        }	         
    }  

    private String SP_I_EDUCATION(Education education,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_EDUCATION";
        Object args[] = new Object[4];
        args[0] = null;
        if (education.getParent() != null)
        {
            args[1] = education.getParent().getID(); 
        }
        args[2] = new Integer(102);  	
        args[3] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_EDUCATION(Education education,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_EDUCATION";
        Object args[] = new Object[10];
        args[0] = education.getID();
        args[1] = education.getOrganization();
        args[2] = education.getDegreeProgram();
        args[3] = education.getSpecialization();
        args[4] = null;
        Calendar certificationGraduateDate = education
            .getCertificationGraduateDate();
        java.sql.Date dt;
        if (certificationGraduateDate != null)
        {
            dt = new java.sql.Date(certificationGraduateDate.getTimeInMillis());
        args[5] = dt.toString();
        }
        else
        {
            args[5] = null;
        }
        args[6] = null;
        Calendar appointedDate = education.getAppointedDate();
        if (appointedDate != null)
        {
            dt = new java.sql.Date(appointedDate.getTimeInMillis());
        args[7] = dt.toString();
        }
        else
        {
            args[7] = null;
        }
        args[8] = education.getHonoursAwards();
        args[9] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext, name, args);
    }
    
    private void SP_D_EDUCATION(Education education,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_EDUCATION";
        Object args[] = new Object[4];
        args[0] = education.getID();
        args[1] = ManagerUtil.getParentId(this, education, messageContext,
            getTableName(), "EDUCATION_ID", "RESOURCE_ID");
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
        
    }
}
