/*
 * Created on Apr 29, 2005
 *
 */

package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;



/**
 * @author Simon
 *
 */
public final class ResourceEmploymentHistoryManager extends
        AbstractRPMObjectManager
{
    
	private static final HashSet CONTAINERS = new HashSet();
    
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    
    static
    {
        CONTAINERS.add(EmploymentHistory.class.getName());
        
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ResourceEmploymentHistoryManager.class);
        }        
    } 
    
    /**
     * Column EMPLOYMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_EMPLOYMENT_ID = 1;
    public static final int TYPE_EMPLOYMENT_ID = Types.CHAR;
    public static final String NAME_EMPLOYMENT_ID = "RESOURCE_EMPLOYMENT_HISTORY.EMPLOYMENT_ID";
    public static final String PROPERTY_EMPLOYMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EMPLOYMENT_ID, NAME_EMPLOYMENT_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;
    public static final int TYPE_RESOURCE_ID = Types.CHAR;
    public static final String NAME_RESOURCE_ID = "RESOURCE_EMPLOYMENT_HISTORY.RESOURCE_ID";
    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column COMPANY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_COMPANY_ID = 3;
    public static final int TYPE_COMPANY_ID = Types.CHAR;
    public static final String NAME_COMPANY_ID = "RESOURCE_EMPLOYMENT_HISTORY.COMPANY_ID";
    public static final String PROPERTY_COMPANY_ID = "COMPANY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_ID, NAME_COMPANY_ID);
    }

    /**
     * Column SALARY of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SALARY = 4;
    public static final int TYPE_SALARY = Types.DECIMAL;
    public static final String NAME_SALARY = "RESOURCE_EMPLOYMENT_HISTORY.SALARY";
    public static final String PROPERTY_SALARY = "SALARY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SALARY, NAME_SALARY);
    }

    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 5;
    public static final int TYPE_START_DATE = Types.DATE;
    public static final String NAME_START_DATE = "RESOURCE_EMPLOYMENT_HISTORY.START_DATE";
    public static final String PROPERTY_START_DATE = "STARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_START_DATE, NAME_START_DATE);
    }

    /**
     * Column END_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_END_DATE = 6;
    public static final int TYPE_END_DATE = Types.DATE;
    public static final String NAME_END_DATE = "RESOURCE_EMPLOYMENT_HISTORY.END_DATE";
    public static final String PROPERTY_END_DATE = "ENDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_END_DATE, NAME_END_DATE);
    }

    /**
     * Column LOCATION of type Types.CHAR mapped to String.
     */
    public static final int ID_LOCATION = 7;
    public static final int TYPE_LOCATION = Types.CHAR;
    public static final String NAME_LOCATION = "RESOURCE_EMPLOYMENT_HISTORY.LOCATION";
    public static final String PROPERTY_LOCATION = "LOCATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOCATION, NAME_LOCATION);
    }

    /**
     * Column TITLE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TITLE = 8;
    public static final int TYPE_TITLE = Types.VARCHAR;
    public static final String NAME_TITLE = "RESOURCE_EMPLOYMENT_HISTORY.TITLE";
    public static final String PROPERTY_TITLE = "TITLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TITLE, NAME_TITLE);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 9;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "RESOURCE_EMPLOYMENT_HISTORY.DESCRIPTION";
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column RESPONSIBILITIES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RESPONSIBILITIES = 10;
    public static final int TYPE_RESPONSIBILITIES = Types.VARCHAR;
    public static final String NAME_RESPONSIBILITIES = "RESOURCE_EMPLOYMENT_HISTORY.RESPONSIBILITIES";
    public static final String PROPERTY_RESPONSIBILITIES = "RESPONSIBILITIES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESPONSIBILITIES, NAME_RESPONSIBILITIES);
    }

    private static final String TABLE_NAME = "RESOURCE_EMPLOYMENT_HISTORY";

    /**
     * Create an array of type string containing all the fields of the RESOURCE_EMPLOYMENT_HISTORY table.
     */
    private static final String[] FIELD_NAMES = 
    {
            NAME_EMPLOYMENT_ID, NAME_RESOURCE_ID, NAME_COMPANY_ID, NAME_SALARY,
            NAME_START_DATE, NAME_END_DATE, NAME_LOCATION, NAME_TITLE,
            NAME_DESCRIPTION, NAME_RESPONSIBILITIES
    };

    /**
     * Field that contains the comma separated fields of the RESOURCE_EMPLOYMENT_HISTORY table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };


    public ResourceEmploymentHistoryManager()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {

        return NAME_EMPLOYMENT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
              
        return new Integer(0);
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }  

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        
    	JoinCondition joinCondition = null;
        
        if (manager instanceof ResourceManager)
        {
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" RESOURCE_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ResourceEmploymentHistoryManager)
        {
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" EMPLOYMENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }

        }        
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceEmploymentHistoryManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }        
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }
        
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new EmploymentHistory();
        return object;
    }

    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

    	RPMObject pObject = rpmObject;
        String company = rs.getString(ID_COMPANY_ID);
        String location = rs.getString(ID_LOCATION);
        if (company != null)
        {
            company = company.trim();
        }
        if (location != null)
        {
            location = location.trim();
        }
		
        pObject.setID(rs.getString(ID_EMPLOYMENT_ID));
        if (isDelta)
        {
            ((EmploymentHistory) pObject).deltaCompany(company);
			((EmploymentHistory) pObject).deltaTitle(rs.getString(ID_TITLE));
			((EmploymentHistory) pObject).deltaDescription(rs
					.getString(ID_DESCRIPTION));
            ((EmploymentHistory) pObject).deltaStartDate(DateUtil
                .convertToCalendar(rs.getDate(ID_START_DATE)));
            ((EmploymentHistory) pObject).deltaEndDate(DateUtil
                .convertToCalendar(rs.getDate(ID_END_DATE)));
            ((EmploymentHistory) pObject).deltaLocation(location);
			((EmploymentHistory) pObject).deltaResponsibilities(rs
					.getString(ID_RESPONSIBILITIES));
			((EmploymentHistory) pObject).deltaSalary(Double.valueOf(rs
					.getString(ID_SALARY)));
		}
        else
        {
            ((EmploymentHistory) pObject).setCompany(company);
            ((EmploymentHistory)pObject).setTitle(rs.getString(ID_TITLE));
            ((EmploymentHistory) pObject).setDescription(rs
                .getString(ID_DESCRIPTION));
            ((EmploymentHistory) pObject).setStartDate(DateUtil
                .convertToCalendar(rs.getDate(ID_START_DATE)));
            ((EmploymentHistory) pObject).setEndDate(DateUtil
                .convertToCalendar(rs.getDate(ID_END_DATE)));
            ((EmploymentHistory) pObject).setLocation(location);
            ((EmploymentHistory) pObject).setResponsibilities(rs
                .getString(ID_RESPONSIBILITIES));
            ((EmploymentHistory) pObject).setSalary(Double.valueOf(rs
                .getString(ID_SALARY)));
        }
        
        return pObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
    
        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);
        
        return relatedFields;
    }
        
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

	    //Everything is loaded without scope
        EmploymentHistory object = (EmploymentHistory)rpmObject;

        //Related resource
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                    null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }
        
        return rpmObject;
    }    

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        EmploymentHistory pObject = (EmploymentHistory)rpmObject;
    	pObject.setID(SP_I_EMPLYMENTHIST(pObject, messageContext));      
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException
    {
    	EmploymentHistory pObject = (EmploymentHistory)rpmObject;
        
        boolean defaultSalarySet = false;
        
        if (pObject.getSalary() == null)
        {
            pObject.setSalary(new Double(0));
            defaultSalarySet = true;
        }
       	
        //if (pObject.isModified()) 
        if (pObject.testCompanyModified() || pObject.testTitleModified()
                || pObject.testSalaryModified() || defaultSalarySet
    	        || pObject.testStartDateModified()
    	        || pObject.testEndDateModified()
    	        || pObject.testLocationModified()
    	        || pObject.testDescriptionModified()
    	        || pObject.testResponsibilitiesModified())
    	{
        	SP_U_EMPLYMENTHIST(pObject, messageContext);
    	}
        
        return pObject;
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        
        if (rpmObject != null)
        {
        	SP_D_EMPLYMENTHIST((EmploymentHistory)rpmObject, messageContext);
        }	         
    }  

    private String SP_I_EMPLYMENTHIST(EmploymentHistory employmenthistory,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_EMPLYMENTHIST";
        Object args[] = new Object[4];
        args[0] = null;  	
        if (employmenthistory.getParent() != null)
        {
            args[1] = employmenthistory.getParent().getID();
        } 	
        args[2] = new Integer(99);  	
        args[3] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_EMPLYMENTHIST(EmploymentHistory employmenthistory,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_EMPLYMENTHIST";
        Object args[] = new Object[10];
        args[0] = employmenthistory.getID();
        args[1] = employmenthistory.getCompany();
        args[2] = employmenthistory.getTitle();
        args[3] = employmenthistory.getSalary();
            
        if(employmenthistory.getStartDate() != null)
        {
            java.sql.Date dt = new java.sql.Date(employmenthistory
                    .getStartDate().getTimeInMillis());
            args[4] = dt.toString();
        }
        
        if(employmenthistory.getEndDate() != null)
        {
            java.sql.Date dt = new java.sql.Date(employmenthistory.getEndDate()
                .getTimeInMillis());
            args[5] = dt.toString();
        }
        
        args[6] = employmenthistory.getLocation();
        args[7] = employmenthistory.getDescription();
        args[8] = employmenthistory.getResponsibilities();
        args[9] = getUser(messageContext).getID();		
        
        executeProcedure(messageContext, name, args);
    }
    
    private void SP_D_EMPLYMENTHIST(EmploymentHistory employmenthistory,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_EMPLYMENTHIST";
        Object args[] = new Object[4];
        args[0] = employmenthistory.getID();
        args[1] = ManagerUtil.getParentId(this, employmenthistory,
            messageContext, TABLE_NAME, "EMPLOYMENT_ID", "RESOURCE_ID");
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }
}
