package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.EmployeeStatus;
import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkingStatus;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.ContactGroupAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.GeographicalAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.OrganizationalAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.RPMCalendarManager;
import com.ibm.rpm.asset.managers.AssetResourcesManager;
import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.managers.WorkflowResponseManager;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DatafieldUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.containers.Visa;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.managers.SecurityGroupManager;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.managers.AdministrativeTaskAssignmentManager;
import com.ibm.rpm.timesheet.managers.PersonalTaskAssignmentManager;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.managers.TimesheetTaskManager;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.workflow.managers.WorkflowRoleMappingManager;
import com.ibm.rpm.xpathparser.XPathContainer;

public class ResourceManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    private static final String DEFAULT_PASSWORD = "ibmprm";

    static
    {
        CONTAINERS.add(Resource.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceManager.class);
        }
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 1;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCES.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column FULL_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FULL_NAME = 2;

    public static final int TYPE_FULL_NAME = Types.VARCHAR;

    public static final String NAME_FULL_NAME = "RESOURCES.FULL_NAME";

    public static final String PROPERTY_FULL_NAME = "FULLNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FULL_NAME, NAME_FULL_NAME);
    }

    /**
     * Column FIRST_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FIRST_NAME = 3;

    public static final int TYPE_FIRST_NAME = Types.VARCHAR;

    public static final String NAME_FIRST_NAME = "RESOURCES.FIRST_NAME";

    public static final String PROPERTY_FIRST_NAME = "GIVENNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FIRST_NAME, NAME_FIRST_NAME);
    }

    /**
     * Column MIDDLE_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MIDDLE_NAME = 4;

    public static final int TYPE_MIDDLE_NAME = Types.VARCHAR;

    public static final String NAME_MIDDLE_NAME = "RESOURCES.MIDDLE_NAME";

    public static final String PROPERTY_MIDDLE_NAME = "MIDDLENAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MIDDLE_NAME, NAME_MIDDLE_NAME);
    }

    /**
     * Column LAST_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_NAME = 5;

    public static final int TYPE_LAST_NAME = Types.VARCHAR;

    public static final String NAME_LAST_NAME = "RESOURCES.LAST_NAME";

    public static final String PROPERTY_LAST_NAME = "FAMILYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LAST_NAME, NAME_LAST_NAME);
    }

    /**
     * Column E_MAIL_ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_E_MAIL_ADDRESS = 6;

    public static final int TYPE_E_MAIL_ADDRESS = Types.VARCHAR;

    public static final String NAME_E_MAIL_ADDRESS = "RESOURCES.E_MAIL_ADDRESS";

    public static final String PROPERTY_E_MAIL_ADDRESS = "EMAILADDRESS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_E_MAIL_ADDRESS, NAME_E_MAIL_ADDRESS);
    }

    public static final int ID_LOGON_NAME = 7;

    public static final int TYPE_LOGON_NAME = Types.VARCHAR;

    public static final String NAME_LOGON_NAME = "RESOURCES.LOGON_NAME";

    public static final String PROPERTY_LOGON_NAME = "USERNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOGON_NAME, NAME_LOGON_NAME);
    }

    public static final int ID_LOGON_PASSWORD = 8;

    public static final int TYPE_LOGON_PASSWORD = Types.VARBINARY;

    public static final String NAME_LOGON_PASSWORD = "RESOURCES.LOGON_PASSWORD";

    public static final String PROPERTY_LOGON_PASSWORD = "PASSWORD";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOGON_PASSWORD, NAME_LOGON_PASSWORD);
    }

    public static final int ID_KEY_HISTORY = 9;

    public static final int TYPE_KEY_HISTORY = Types.VARBINARY;

    public static final String NAME_KEY_HISTORY = "RESOURCES.KEY_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_KEY_HISTORY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_KEY_HISTORY,NAME_KEY_HISTORY);}

    /**
     * Column LOGON_REMINDER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LOGON_REMINDER = 10;

    public static final int TYPE_LOGON_REMINDER = Types.VARCHAR;

    public static final String NAME_LOGON_REMINDER = "RESOURCES.LOGON_REMINDER";

    public static final String PROPERTY_LOGON_REMINDER = "LOGONREMINDER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LOGON_REMINDER, NAME_LOGON_REMINDER);
    }

    /**
     * Column CLEARANCE_LEVEL_1 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_1 = 11;

    public static final int TYPE_CLEARANCE_LEVEL_1 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_1 = "RESOURCES.CLEARANCE_LEVEL_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_1 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_1,NAME_CLEARANCE_LEVEL_1);}

    /**
     * Column CLEARANCE_LEVEL_2 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_2 = 12;

    public static final int TYPE_CLEARANCE_LEVEL_2 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_2 = "RESOURCES.CLEARANCE_LEVEL_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_2 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_2,NAME_CLEARANCE_LEVEL_2);}

    /**
     * Column CLEARANCE_LEVEL_3 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_3 = 13;

    public static final int TYPE_CLEARANCE_LEVEL_3 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_3 = "RESOURCES.CLEARANCE_LEVEL_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_3 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_3,NAME_CLEARANCE_LEVEL_3);}

    /**
     * Column CLEARANCE_LEVEL_4 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_4 = 14;

    public static final int TYPE_CLEARANCE_LEVEL_4 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_4 = "RESOURCES.CLEARANCE_LEVEL_4";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_4 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_4,NAME_CLEARANCE_LEVEL_4);}

    /**
     * Column MULTIPLE_LOGINS of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_MULTIPLE_LOGINS = 15;

    public static final int TYPE_MULTIPLE_LOGINS = Types.CHAR;

    public static final String NAME_MULTIPLE_LOGINS = "RESOURCES.MULTIPLE_LOGINS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MULTIPLE_LOGINS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MULTIPLE_LOGINS,NAME_MULTIPLE_LOGINS);}

    /**
     * Column PASSWORD_EXPIRES of type Types.DATE mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PASSWORD_EXPIRES = 16;

    public static final int TYPE_PASSWORD_EXPIRES = Types.DATE;

    public static final String NAME_PASSWORD_EXPIRES = "RESOURCES.PASSWORD_EXPIRES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PASSWORD_EXPIRES = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PASSWORD_EXPIRES,NAME_PASSWORD_EXPIRES);}

    /**
     * Column WEBSITE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_WEBSITE = 17;

    public static final int TYPE_WEBSITE = Types.VARCHAR;

    public static final String NAME_WEBSITE = "RESOURCES.WEBSITE";

    public static final String PROPERTY_WEBSITE = "MYWEBSITE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEBSITE, NAME_WEBSITE);
    }

    /**
     * Column PRIMARY_ROLE of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_PRIMARY_ROLE = 18;

    public static final int TYPE_PRIMARY_ROLE = Types.CHAR;

    public static final String NAME_PRIMARY_ROLE = "RESOURCES.PRIMARY_ROLE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PRIMARY_ROLE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRIMARY_ROLE,NAME_PRIMARY_ROLE);}

    /**
     * Column TITLE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TITLE = 19;

    public static final int TYPE_TITLE = Types.VARCHAR;

    public static final String NAME_TITLE = "RESOURCES.TITLE";

    public static final String PROPERTY_TITLE = "TITLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TITLE, NAME_TITLE);
    }

    /**
     * Column ADDRESS1 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ADDRESS1 = 20;

    public static final int TYPE_ADDRESS1 = Types.VARCHAR;

    public static final String NAME_ADDRESS1 = "RESOURCES.ADDRESS1";

    public static final String PROPERTY_ADDRESS1 = "ADDRESS1";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ADDRESS1, NAME_ADDRESS1);
    }

    /**
     * Column ADDRESS2 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ADDRESS2 = 21;

    public static final int TYPE_ADDRESS2 = Types.VARCHAR;

    public static final String NAME_ADDRESS2 = "RESOURCES.ADDRESS2";

    public static final String PROPERTY_ADDRESS2 = "ADDRESS2";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ADDRESS2, NAME_ADDRESS2);
    }

    /**
     * Column ZIP_AREA_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ZIP_AREA_CODE = 22;

    public static final int TYPE_ZIP_AREA_CODE = Types.VARCHAR;

    public static final String NAME_ZIP_AREA_CODE = "RESOURCES.ZIP_AREA_CODE";

    public static final String PROPERTY_ZIP_AREA_CODE = "ZIPPOSTALCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ZIP_AREA_CODE, NAME_ZIP_AREA_CODE);
    }

    /**
     * Column CITIZENSHIP of type Types.INTEGER mapped to Integer. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CITIZENSHIP = 23;

    public static final int TYPE_CITIZENSHIP = Types.INTEGER;

    public static final String NAME_CITIZENSHIP = "RESOURCES.CITIZENSHIP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CITIZENSHIP = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CITIZENSHIP,NAME_CITIZENSHIP);}

    /**
     * Column CALENDAR_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CALENDAR_ID = 24;

    public static final int TYPE_CALENDAR_ID = Types.CHAR;

    public static final String NAME_CALENDAR_ID = "RESOURCES.CALENDAR_ID";

    public static final String PROPERTY_CALENDAR_ID = "CALENDAR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CALENDAR_ID, NAME_CALENDAR_ID);
    }

    /**
     * Column ACTIVE of type Types.CHAR mapped to String.
     */
    public static final int ID_ACTIVE = 25;

    public static final int TYPE_ACTIVE = Types.CHAR;

    public static final String NAME_ACTIVE = "RESOURCES.ACTIVE";

    public static final String PROPERTY_ACTIVE = "ACTIVE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTIVE, SqlUtil.getBooleanValue(
            NAME_ACTIVE, "1"));
    }

    /**
     * Column EFFECTIVE_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_EFFECTIVE_DATE = 26;

    public static final int TYPE_EFFECTIVE_DATE = Types.DATE;

    public static final String NAME_EFFECTIVE_DATE = "RESOURCES.EFFECTIVE_DATE";

    public static final String PROPERTY_EFFECTIVE_DATE = "EFFECTIVESTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EFFECTIVE_DATE, NAME_EFFECTIVE_DATE);
    }

    /**
     * Column DISCONTINUE_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_DISCONTINUE_DATE = 27;

    public static final int TYPE_DISCONTINUE_DATE = Types.DATE;

    public static final String NAME_DISCONTINUE_DATE = "RESOURCES.DISCONTINUE_DATE";

    public static final String PROPERTY_DISCONTINUE_DATE = "DISCONTINUEDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DISCONTINUE_DATE, NAME_DISCONTINUE_DATE);
    }

    /**
     * Column STATUS1_ID of type Types.INTEGER mapped to Integer. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_STATUS1_ID = 28;

    public static final int TYPE_STATUS1_ID = Types.INTEGER;

    public static final String NAME_STATUS1_ID = "RESOURCES.STATUS1_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STATUS1_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STATUS1_ID,NAME_STATUS1_ID);}

    /**
     * Column STATUS2_ID of type Types.INTEGER mapped to Integer. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_STATUS2_ID = 29;

    public static final int TYPE_STATUS2_ID = Types.INTEGER;

    public static final String NAME_STATUS2_ID = "RESOURCES.STATUS2_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STATUS2_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STATUS2_ID,NAME_STATUS2_ID);}

    /**
     * Column CREATED_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CREATED_DATE = 30;

    public static final int TYPE_CREATED_DATE = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATE = "RESOURCES.CREATED_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATE,NAME_CREATED_DATE);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_CREATED_BY = 31;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "RESOURCES.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column CHARGE_CODE of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_CHARGE_CODE = 32;

    public static final int TYPE_CHARGE_CODE = Types.CHAR;

    public static final String NAME_CHARGE_CODE = "RESOURCES.CHARGE_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGE_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGE_CODE,NAME_CHARGE_CODE);}

    /**
     * Column MAXDAYHRS of type Types.SMALLINT mapped to Integer. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_MAXDAYHRS = 33;

    public static final int TYPE_MAXDAYHRS = Types.SMALLINT;

    public static final String NAME_MAXDAYHRS = "RESOURCES.MAXDAYHRS";

    public static final String PROPERTY_MAXDAYHRS = "WEEKLYMAXHOURS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MAXDAYHRS, NAME_MAXDAYHRS);
    }

    /**
     * Column MINDAYHRS of type Types.SMALLINT mapped to Integer. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_MINDAYHRS = 34;

    public static final int TYPE_MINDAYHRS = Types.SMALLINT;

    public static final String NAME_MINDAYHRS = "RESOURCES.MINDAYHRS";

    public static final String PROPERTY_MINDAYHRS = "WEEKLYMINHOURS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MINDAYHRS, NAME_MINDAYHRS);
    }

    /**
     * Column PERCENT_TO_ASSIGN of type Types.DECIMAL mapped to
     * java.math.BigDecimal. in upper case.
     */
    public static final int ID_PERCENT_TO_ASSIGN = 35;

    public static final int TYPE_PERCENT_TO_ASSIGN = Types.DECIMAL;

    public static final String NAME_PERCENT_TO_ASSIGN = "RESOURCES.PERCENT_TO_ASSIGN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PERCENT_TO_ASSIGN = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PERCENT_TO_ASSIGN,NAME_PERCENT_TO_ASSIGN);}

    /**
     * Column RATE_ID of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_RATE_ID = 36;

    public static final int TYPE_RATE_ID = Types.CHAR;

    public static final String NAME_RATE_ID = "RESOURCES.RATE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RATE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RATE_ID,NAME_RATE_ID);}

    /**
     * Column HOME_OFFICE of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_HOME_OFFICE = 37;

    public static final int TYPE_HOME_OFFICE = Types.VARCHAR;

    public static final String NAME_HOME_OFFICE = "RESOURCES.HOME_OFFICE";

    public static final String PROPERTY_HOME_OFFICE = "HOMEPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_HOME_OFFICE, NAME_HOME_OFFICE);
    }

    /**
     * Column WORK_LOCATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WORK_LOCATION = 38;

    public static final int TYPE_WORK_LOCATION = Types.INTEGER;

    public static final String NAME_WORK_LOCATION = "RESOURCES.WORK_LOCATION";

    public static final String PROPERTY_WORK_LOCATION = "WOKRLOCATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WORK_LOCATION, NAME_WORK_LOCATION);
    }

    /**
     * Column WORK_PHONE of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WORK_PHONE = 39;

    public static final int TYPE_WORK_PHONE = Types.VARCHAR;

    public static final String NAME_WORK_PHONE = "RESOURCES.WORK_PHONE";

    public static final String PROPERTY_WORK_PHONE = "WORKPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WORK_PHONE, NAME_WORK_PHONE);
    }

    /**
     * Column ALTERNATE_PHONE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ALTERNATE_PHONE = 40;

    public static final int TYPE_ALTERNATE_PHONE = Types.VARCHAR;

    public static final String NAME_ALTERNATE_PHONE = "RESOURCES.ALTERNATE_PHONE";

    public static final String PROPERTY_ALTERNATE_PHONE = "ALTERNATIVEPHONE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ALTERNATE_PHONE, NAME_ALTERNATE_PHONE);
    }

    /**
     * Column FAX_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FAX_NUMBER = 41;

    public static final int TYPE_FAX_NUMBER = Types.VARCHAR;

    public static final String NAME_FAX_NUMBER = "RESOURCES.FAX_NUMBER";

    public static final String PROPERTY_FAX_NUMBER = "FAX";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FAX_NUMBER, NAME_FAX_NUMBER);
    }

    /**
     * Column CELL_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CELL_NUMBER = 42;

    public static final int TYPE_CELL_NUMBER = Types.VARCHAR;

    public static final String NAME_CELL_NUMBER = "RESOURCES.CELL_NUMBER";

    public static final String PROPERTY_CELL_NUMBER = "MOBILE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CELL_NUMBER, NAME_CELL_NUMBER);
    }

    /**
     * Column PAGER_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PAGER_NUMBER = 43;

    public static final int TYPE_PAGER_NUMBER = Types.VARCHAR;

    public static final String NAME_PAGER_NUMBER = "RESOURCES.PAGER_NUMBER";

    public static final String PROPERTY_PAGER_NUMBER = "PAGER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PAGER_NUMBER, NAME_PAGER_NUMBER);
    }

    /**
     * Column ICQ_MESSAGING of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_ICQ_MESSAGING = 44;

    public static final int TYPE_ICQ_MESSAGING = Types.VARCHAR;

    public static final String NAME_ICQ_MESSAGING = "RESOURCES.ICQ_MESSAGING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ICQ_MESSAGING = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ICQ_MESSAGING,NAME_ICQ_MESSAGING);}

    /**
     * Column NOTES_MAIL_ID of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_NOTES_MAIL_ID = 45;

    public static final int TYPE_NOTES_MAIL_ID = Types.VARCHAR;

    public static final String NAME_NOTES_MAIL_ID = "RESOURCES.NOTES_MAIL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_NOTES_MAIL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTES_MAIL_ID,NAME_NOTES_MAIL_ID);}

    /**
     * Column NOTES_ID of type Types.VARCHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_NOTES_ID = 46;

    public static final int TYPE_NOTES_ID = Types.VARCHAR;

    public static final String NAME_NOTES_ID = "RESOURCES.NOTES_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_NOTES_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTES_ID,NAME_NOTES_ID);}

    /**
     * Column ASSIGN_STATUS of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_ASSIGN_STATUS = 47;

    public static final int TYPE_ASSIGN_STATUS = Types.VARCHAR;

    public static final String NAME_ASSIGN_STATUS = "RESOURCES.ASSIGN_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_STATUS,NAME_ASSIGN_STATUS);}

    /**
     * Column EXPERIENCE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPERIENCE_ID = 48;

    public static final int TYPE_EXPERIENCE_ID = Types.INTEGER;

    public static final String NAME_EXPERIENCE_ID = "RESOURCES.EXPERIENCE_ID";

    public static final String PROPERTY_EXPERIENCE_ID = "EXPERIENCE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_ID, NAME_EXPERIENCE_ID);
    }

    /**
     * Column ALIAS of type Types.VARCHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_ALIAS = 49;

    public static final int TYPE_ALIAS = Types.VARCHAR;

    public static final String NAME_ALIAS = "RESOURCES.ALIAS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ALIAS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ALIAS,NAME_ALIAS);}

    /**
     * Column LAST_USED of type Types.DATE mapped to java.util.Date. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LAST_USED = 50;

    public static final int TYPE_LAST_USED = Types.DATE;

    public static final String NAME_LAST_USED = "RESOURCES.LAST_USED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_USED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_USED,NAME_LAST_USED);}

    /**
     * Column EXEC_LEVEL of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_EXEC_LEVEL = 51;

    public static final int TYPE_EXEC_LEVEL = Types.VARCHAR;

    public static final String NAME_EXEC_LEVEL = "RESOURCES.EXEC_LEVEL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXEC_LEVEL = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXEC_LEVEL,NAME_EXEC_LEVEL);}

    /**
     * Column VERIFIED_1 of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_VERIFIED_1 = 52;

    public static final int TYPE_VERIFIED_1 = Types.CHAR;

    public static final String NAME_VERIFIED_1 = "RESOURCES.VERIFIED_1";

    public static final String PROPERTY_VERIFIED_1 = "VERIFIEDBY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_VERIFIED_1, NAME_VERIFIED_1);
    }

    /**
     * Column VERIFIED_2 of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_VERIFIED_2 = 53;

    public static final int TYPE_VERIFIED_2 = Types.CHAR;

    public static final String NAME_VERIFIED_2 = "RESOURCES.VERIFIED_2";

    public static final String PROPERTY_VERIFIED_2 = "REVERIFIEDBY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_VERIFIED_2, NAME_VERIFIED_2);
    }

    /**
     * Column WILLING_RELOC of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WILLING_RELOC = 54;

    public static final int TYPE_WILLING_RELOC = Types.CHAR;

    public static final String NAME_WILLING_RELOC = "RESOURCES.WILLING_RELOC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WILLING_RELOC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WILLING_RELOC,NAME_WILLING_RELOC);}

    /**
     * Column RELOC_DURATION of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_RELOC_DURATION = 55;

    public static final int TYPE_RELOC_DURATION = Types.VARCHAR;

    public static final String NAME_RELOC_DURATION = "RESOURCES.RELOC_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RELOC_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_RELOC_DURATION,NAME_RELOC_DURATION);}

    /**
     * Column WORK_RELOC_1 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WORK_RELOC_1 = 56;

    public static final int TYPE_WORK_RELOC_1 = Types.CHAR;

    public static final String NAME_WORK_RELOC_1 = "RESOURCES.WORK_RELOC_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_RELOC_1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_RELOC_1,NAME_WORK_RELOC_1);}

    /**
     * Column WORK_RELOC_2 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WORK_RELOC_2 = 57;

    public static final int TYPE_WORK_RELOC_2 = Types.CHAR;

    public static final String NAME_WORK_RELOC_2 = "RESOURCES.WORK_RELOC_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_RELOC_2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_RELOC_2,NAME_WORK_RELOC_2);}

    /**
     * Column WORK_RELOC_3 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WORK_RELOC_3 = 58;

    public static final int TYPE_WORK_RELOC_3 = Types.CHAR;

    public static final String NAME_WORK_RELOC_3 = "RESOURCES.WORK_RELOC_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_RELOC_3 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_RELOC_3,NAME_WORK_RELOC_3);}

    /**
     * Column WILLING_TRAVEL of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_WILLING_TRAVEL = 59;

    public static final int TYPE_WILLING_TRAVEL = Types.CHAR;

    public static final String NAME_WILLING_TRAVEL = "RESOURCES.WILLING_TRAVEL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WILLING_TRAVEL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_WILLING_TRAVEL,NAME_WILLING_TRAVEL);}

    /**
     * Column TRAVEL_DURATION of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_TRAVEL_DURATION = 60;

    public static final int TYPE_TRAVEL_DURATION = Types.VARCHAR;

    public static final String NAME_TRAVEL_DURATION = "RESOURCES.TRAVEL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRAVEL_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TRAVEL_DURATION,NAME_TRAVEL_DURATION);}

    /**
     * Column TRAVEL_DEST_1 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_TRAVEL_DEST_1 = 61;

    public static final int TYPE_TRAVEL_DEST_1 = Types.CHAR;

    public static final String NAME_TRAVEL_DEST_1 = "RESOURCES.TRAVEL_DEST_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRAVEL_DEST_1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TRAVEL_DEST_1,NAME_TRAVEL_DEST_1);}

    /**
     * Column TRAVEL_DEST_2 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_TRAVEL_DEST_2 = 62;

    public static final int TYPE_TRAVEL_DEST_2 = Types.CHAR;

    public static final String NAME_TRAVEL_DEST_2 = "RESOURCES.TRAVEL_DEST_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRAVEL_DEST_2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TRAVEL_DEST_2,NAME_TRAVEL_DEST_2);}

    /**
     * Column TRAVEL_DEST_3 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_TRAVEL_DEST_3 = 63;

    public static final int TYPE_TRAVEL_DEST_3 = Types.CHAR;

    public static final String NAME_TRAVEL_DEST_3 = "RESOURCES.TRAVEL_DEST_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRAVEL_DEST_3 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TRAVEL_DEST_3,NAME_TRAVEL_DEST_3);}

    /**
     * Column PASPORT_VALID of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_PASPORT_VALID = 64;

    public static final int TYPE_PASPORT_VALID = Types.CHAR;

    public static final String NAME_PASPORT_VALID = "RESOURCES.PASPORT_VALID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PASPORT_VALID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PASPORT_VALID,NAME_PASPORT_VALID);}

    /**
     * Column PASPORT_EXPIRATION of type Types.DATE mapped to java.util.Date.
     *  
     */
    public static final int ID_PASPORT_EXPIRATION = 65;

    public static final int TYPE_PASPORT_EXPIRATION = Types.DATE;

    public static final String NAME_PASPORT_EXPIRATION = "RESOURCES.PASPORT_EXPIRATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PASPORT_EXPIRATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PASPORT_EXPIRATION,NAME_PASPORT_EXPIRATION);}

    /**
     * Column COUNTRY_VISA_1 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_COUNTRY_VISA_1 = 66;

    public static final int TYPE_COUNTRY_VISA_1 = Types.CHAR;

    public static final String NAME_COUNTRY_VISA_1 = "RESOURCES.COUNTRY_VISA_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COUNTRY_VISA_1 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COUNTRY_VISA_1,NAME_COUNTRY_VISA_1);}

    /**
     * Column VISA_EXPIRES_1 of type Types.DATE mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_VISA_EXPIRES_1 = 67;

    public static final int TYPE_VISA_EXPIRES_1 = Types.DATE;

    public static final String NAME_VISA_EXPIRES_1 = "RESOURCES.VISA_EXPIRES_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VISA_EXPIRES_1 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_VISA_EXPIRES_1,NAME_VISA_EXPIRES_1);}

    /**
     * Column COUNTRY_VISA_2 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_COUNTRY_VISA_2 = 68;

    public static final int TYPE_COUNTRY_VISA_2 = Types.CHAR;

    public static final String NAME_COUNTRY_VISA_2 = "RESOURCES.COUNTRY_VISA_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COUNTRY_VISA_2 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COUNTRY_VISA_2,NAME_COUNTRY_VISA_2);}

    /**
     * Column VISA_EXPIRES_2 of type Types.DATE mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_VISA_EXPIRES_2 = 69;

    public static final int TYPE_VISA_EXPIRES_2 = Types.DATE;

    public static final String NAME_VISA_EXPIRES_2 = "RESOURCES.VISA_EXPIRES_2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VISA_EXPIRES_2 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_VISA_EXPIRES_2,NAME_VISA_EXPIRES_2);}

    /**
     * Column COUNTRY_VISA_3 of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_COUNTRY_VISA_3 = 70;

    public static final int TYPE_COUNTRY_VISA_3 = Types.CHAR;

    public static final String NAME_COUNTRY_VISA_3 = "RESOURCES.COUNTRY_VISA_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COUNTRY_VISA_3 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COUNTRY_VISA_3,NAME_COUNTRY_VISA_3);}

    /**
     * Column VISA_EXPIRES_3 of type Types.DATE mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_VISA_EXPIRES_3 = 71;

    public static final int TYPE_VISA_EXPIRES_3 = Types.DATE;

    public static final String NAME_VISA_EXPIRES_3 = "RESOURCES.VISA_EXPIRES_3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VISA_EXPIRES_3 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_VISA_EXPIRES_3,NAME_VISA_EXPIRES_3);}

    /**
     * Column CURRENCY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CURRENCY_ID = 72;

    public static final int TYPE_CURRENCY_ID = Types.INTEGER;

    public static final String NAME_CURRENCY_ID = "RESOURCES.CURRENCY_ID";

    public static final String PROPERTY_CURRENCY_ID = "CURRENCY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CURRENCY_ID, NAME_CURRENCY_ID);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_REC_STATUS = 73;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "RESOURCES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_REC_USER = 74;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "RESOURCES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     *  
     */
    public static final int ID_REC_DATETIME = 75;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "RESOURCES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAJOR_REVISION = 76;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "RESOURCES.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MINOR_REVISION = 77;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "RESOURCES.MINOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION);}

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_IMPACT = 78;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "RESOURCES.REVISION_IMPACT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_TYPE = 79;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "RESOURCES.REVISION_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REVISION_HISTORY = 80;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "RESOURCES.REVISION_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

    /**
     * Column LAST_LOGIN of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LAST_LOGIN = 81;

    public static final int TYPE_LAST_LOGIN = Types.TIMESTAMP;

    public static final String NAME_LAST_LOGIN = "RESOURCES.LAST_LOGIN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_LOGIN = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_LOGIN,NAME_LAST_LOGIN);}

    /**
     * Column LANGUAGE_ISO_CODE of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LANGUAGE_ISO_CODE = 82;

    public static final int TYPE_LANGUAGE_ISO_CODE = Types.CHAR;

    public static final String NAME_LANGUAGE_ISO_CODE = "RESOURCES.LANGUAGE_ISO_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LANGUAGE_ISO_CODE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LANGUAGE_ISO_CODE,NAME_LANGUAGE_ISO_CODE);}

    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_FS_CODE = 83;

    public static final int TYPE_FS_CODE = Types.VARCHAR;

    public static final String NAME_FS_CODE = "RESOURCES.FS_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FS_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FS_CODE,NAME_FS_CODE);}

    /**
     * Column FS_LABOR_CODE of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_FS_LABOR_CODE = 84;

    public static final int TYPE_FS_LABOR_CODE = Types.VARCHAR;

    public static final String NAME_FS_LABOR_CODE = "RESOURCES.FS_LABOR_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FS_LABOR_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FS_LABOR_CODE,NAME_FS_LABOR_CODE);}

    /**
     * Column EXPERIENCE_TXT of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_EXPERIENCE_TXT = 85;

    public static final int TYPE_EXPERIENCE_TXT = Types.VARCHAR;

    public static final String NAME_EXPERIENCE_TXT = "RESOURCES.EXPERIENCE_TXT";

    public static final String PROPERTY_EXPERIENCE_TXT = "INTERVIEWEXPERIENCE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_TXT, NAME_EXPERIENCE_TXT);
    }

    /**
     * Column ACHIVEMENTS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ACHIVEMENTS = 86;

    public static final int TYPE_ACHIVEMENTS = Types.VARCHAR;

    public static final String NAME_ACHIVEMENTS = "RESOURCES.ACHIVEMENTS";

    public static final String PROPERTY_ACHIVEMENTS = "MYACHIEVEMENTS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACHIVEMENTS, NAME_ACHIVEMENTS);
    }

    /**
     * Column PASSIONS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PASSIONS = 87;

    public static final int TYPE_PASSIONS = Types.VARCHAR;

    public static final String NAME_PASSIONS = "RESOURCES.PASSIONS";

    public static final String PROPERTY_PASSIONS = "MYPERSONALPASSIONS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PASSIONS, NAME_PASSIONS);
    }

    /**
     * Column SUMMARY_SKILLS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SUMMARY_SKILLS = 88;

    public static final int TYPE_SUMMARY_SKILLS = Types.VARCHAR;

    public static final String NAME_SUMMARY_SKILLS = "RESOURCES.SUMMARY_SKILLS";

    public static final String PROPERTY_SUMMARY_SKILLS = "MYSUMMARYOFSKILLS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUMMARY_SKILLS, NAME_SUMMARY_SKILLS);
    }

    /**
     * Column MISCELLANEOUS of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_MISCELLANEOUS = 89;

    public static final int TYPE_MISCELLANEOUS = Types.VARCHAR;

    public static final String NAME_MISCELLANEOUS = "RESOURCES.MISCELLANEOUS";

    public static final String PROPERTY_MISCELLANEOUS = "WHATIDOREALLYWELL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MISCELLANEOUS, NAME_MISCELLANEOUS);
    }

    /**
     * Column MAX_PASSWORD_FAILURE of type Types.SMALLINT mapped to Integer.
     *  
     */
    public static final int ID_MAX_PASSWORD_FAILURE = 90;

    public static final int TYPE_MAX_PASSWORD_FAILURE = Types.SMALLINT;

    public static final String NAME_MAX_PASSWORD_FAILURE = "RESOURCES.MAX_PASSWORD_FAILURE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_PASSWORD_FAILURE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAX_PASSWORD_FAILURE,NAME_MAX_PASSWORD_FAILURE);}

    /**
     * Column NICK_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NICK_NAME = 91;

    public static final int TYPE_NICK_NAME = Types.VARCHAR;

    public static final String NAME_NICK_NAME = "RESOURCES.NICK_NAME";

    public static final String PROPERTY_NICK_NAME = "NICKNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_NICK_NAME, NAME_NICK_NAME);
    }

    /**
     * Column PORTAL_PAGE of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_PORTAL_PAGE = 92;

    public static final int TYPE_PORTAL_PAGE = Types.VARCHAR;

    public static final String NAME_PORTAL_PAGE = "RESOURCES.PORTAL_PAGE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PORTAL_PAGE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PORTAL_PAGE,NAME_PORTAL_PAGE);}

    /**
     * Column DASHBOARD of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DASHBOARD = 93;

    public static final int TYPE_DASHBOARD = Types.VARCHAR;

    public static final String NAME_DASHBOARD = "RESOURCES.DASHBOARD";

    public static final String PROPERTY_DASHBOARD = "DASHBOARD";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DASHBOARD, NAME_DASHBOARD);
    }

    /**
     * Column OVERWRITEFILE of type Types.CHAR mapped to String.
     */
    public static final int ID_OVERWRITEFILE = 94;

    public static final int TYPE_OVERWRITEFILE = Types.CHAR;

    public static final String NAME_OVERWRITEFILE = "RESOURCES.OVERWRITEFILE";

    public static final String PROPERTY_OVERWRITEFILE = "OVERRIDEORIGINALDOCUMENTONCHECKOUT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_OVERWRITEFILE, SqlUtil.getBooleanValue(
            NAME_OVERWRITEFILE, "Y"));
    }

    /**
     * Column RES_BLOB of type Types.BLOB mapped to null. PROPERTYNAME with
     * container property in upper case.
     */
    public static final int ID_RES_BLOB = 95;

    public static final int TYPE_RES_BLOB = Types.BLOB;

    public static final String NAME_RES_BLOB = "RESOURCES.RES_BLOB";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RES_BLOB = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RES_BLOB,NAME_RES_BLOB);}

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CONVERTED_SCORE = 96;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "RESOURCES.CONVERTED_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE);}

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CONVERTED_WEIGHT = 97;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "RESOURCES.CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column EXECUTIVE_SECURITY of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_EXECUTIVE_SECURITY = 98;

    public static final int TYPE_EXECUTIVE_SECURITY = Types.CHAR;

    public static final String NAME_EXECUTIVE_SECURITY = "RESOURCES.EXECUTIVE_SECURITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXECUTIVE_SECURITY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXECUTIVE_SECURITY,NAME_EXECUTIVE_SECURITY);}

    /**
     * Column MY_PORTAL_URL_1 of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MY_PORTAL_URL_1 = 99;

    public static final int TYPE_MY_PORTAL_URL_1 = Types.VARCHAR;

    public static final String NAME_MY_PORTAL_URL_1 = "RESOURCES.MY_PORTAL_URL_1";

    public static final String PROPERTY_MY_PORTAL_URL_1 = "MYPORTALURL1";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MY_PORTAL_URL_1, NAME_MY_PORTAL_URL_1);
    }

    /**
     * Column MY_PORTAL_URL_2 of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MY_PORTAL_URL_2 = 100;

    public static final int TYPE_MY_PORTAL_URL_2 = Types.VARCHAR;

    public static final String NAME_MY_PORTAL_URL_2 = "RESOURCES.MY_PORTAL_URL_2";

    public static final String PROPERTY_MY_PORTAL_URL_2 = "MYPORTALURL2";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MY_PORTAL_URL_2, NAME_MY_PORTAL_URL_2);
    }

    /**
     * Column MY_PORTAL_URL_3 of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MY_PORTAL_URL_3 = 101;

    public static final int TYPE_MY_PORTAL_URL_3 = Types.VARCHAR;

    public static final String NAME_MY_PORTAL_URL_3 = "RESOURCES.MY_PORTAL_URL_3";

    public static final String PROPERTY_MY_PORTAL_URL_3 = "MYPORTALURL3";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MY_PORTAL_URL_3, NAME_MY_PORTAL_URL_3);
    }

    /**
     * Column MY_PORTAL_URL_4 of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MY_PORTAL_URL_4 = 102;

    public static final int TYPE_MY_PORTAL_URL_4 = Types.VARCHAR;

    public static final String NAME_MY_PORTAL_URL_4 = "RESOURCES.MY_PORTAL_URL_4";

    public static final String PROPERTY_MY_PORTAL_URL_4 = "MYPORTALURL4";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MY_PORTAL_URL_4, NAME_MY_PORTAL_URL_4);
    }

    /**
     * Column MY_PORTAL_URL_5 of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MY_PORTAL_URL_5 = 103;

    public static final int TYPE_MY_PORTAL_URL_5 = Types.VARCHAR;

    public static final String NAME_MY_PORTAL_URL_5 = "RESOURCES.MY_PORTAL_URL_5";

    public static final String PROPERTY_MY_PORTAL_URL_5 = "MYPORTALURL5";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MY_PORTAL_URL_5, NAME_MY_PORTAL_URL_5);
    }

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_5 = 104;

    public static final int TYPE_CLEARANCE_LEVEL_5 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_5 = "RESOURCES.CLEARANCE_LEVEL_5";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_5 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_5,NAME_CLEARANCE_LEVEL_5);}

    /**
     * Column USER_GROUP_ID of type Types.CHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_USER_GROUP_ID = 105;

    public static final int TYPE_USER_GROUP_ID = Types.CHAR;

    public static final String NAME_USER_GROUP_ID = "RESOURCES.USER_GROUP_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_USER_GROUP_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_USER_GROUP_ID,NAME_USER_GROUP_ID);}

    /**
     * Column COUNTRY_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COUNTRY_CODE = 106;

    public static final int TYPE_COUNTRY_CODE = Types.VARCHAR;

    public static final String NAME_COUNTRY_CODE = "RESOURCES.COUNTRY_CODE";

    public static final String PROPERTY_COUNTRY_CODE = "COUNTRYCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COUNTRY_CODE, NAME_COUNTRY_CODE);
    }

    /**
     * Column COMPANY_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COMPANY_CODE = 107;

    public static final int TYPE_COMPANY_CODE = Types.VARCHAR;

    public static final String NAME_COMPANY_CODE = "RESOURCES.COMPANY_CODE";

    public static final String PROPERTY_COMPANY_CODE = "COMPANYCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COMPANY_CODE, NAME_COMPANY_CODE);
    }

    /**
     * Column EMPLOYEE_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EMPLOYEE_CODE = 108;

    public static final int TYPE_EMPLOYEE_CODE = Types.VARCHAR;

    public static final String NAME_EMPLOYEE_CODE = "RESOURCES.EMPLOYEE_CODE";

    public static final String PROPERTY_EMPLOYEE_CODE = "EMPLOYEECODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EMPLOYEE_CODE, NAME_EMPLOYEE_CODE);
    }

    /**
     * Column CUSTOM_01 of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CUSTOM_01 = 109;

    public static final int TYPE_CUSTOM_01 = Types.VARCHAR;

    public static final String NAME_CUSTOM_01 = "RESOURCES.CUSTOM_01";

    public static final String PROPERTY_CUSTOM_01 = "CUSTOM1";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_01, NAME_CUSTOM_01);
    }

    /**
     * Column CUSTOM_02 of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CUSTOM_02 = 110;

    public static final int TYPE_CUSTOM_02 = Types.VARCHAR;

    public static final String NAME_CUSTOM_02 = "RESOURCES.CUSTOM_02";

    public static final String PROPERTY_CUSTOM_02 = "CUSTOM2";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_02, NAME_CUSTOM_02);
    }

    /**
     * Column CUSTOM_03 of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CUSTOM_03 = 111;

    public static final int TYPE_CUSTOM_03 = Types.VARCHAR;

    public static final String NAME_CUSTOM_03 = "RESOURCES.CUSTOM_03";

    public static final String PROPERTY_CUSTOM_03 = "CUSTOM3";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_03, NAME_CUSTOM_03);
    }

    /**
     * Column CUSTOM_04 of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CUSTOM_04 = 112;

    public static final int TYPE_CUSTOM_04 = Types.VARCHAR;

    public static final String NAME_CUSTOM_04 = "RESOURCES.CUSTOM_04";

    public static final String PROPERTY_CUSTOM_04 = "CUSTOM4";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_04, NAME_CUSTOM_04);
    }

    /**
     * Column CUSTOM_05 of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_CUSTOM_05 = 113;

    public static final int TYPE_CUSTOM_05 = Types.VARCHAR;

    public static final String NAME_CUSTOM_05 = "RESOURCES.CUSTOM_05";

    public static final String PROPERTY_CUSTOM_05 = "CUSTOM5";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_05, NAME_CUSTOM_05);
    }

    /**
     * Column DIVISION_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DIVISION_CODE = 114;

    public static final int TYPE_DIVISION_CODE = Types.VARCHAR;

    public static final String NAME_DIVISION_CODE = "RESOURCES.DIVISION_CODE";

    public static final String PROPERTY_DIVISION_CODE = "DIVISIONCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DIVISION_CODE, NAME_DIVISION_CODE);
    }

    /**
     * Column DEPARTMENT_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DEPARTMENT_CODE = 115;

    public static final int TYPE_DEPARTMENT_CODE = Types.VARCHAR;

    public static final String NAME_DEPARTMENT_CODE = "RESOURCES.DEPARTMENT_CODE";

    public static final String PROPERTY_DEPARTMENT_CODE = "DEPARTMENTCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPARTMENT_CODE, NAME_DEPARTMENT_CODE);
    }

    /**
     * Column EXTERNAL_ID of type Types.VARCHAR mapped to String. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_EXTERNAL_ID = 116;

    public static final int TYPE_EXTERNAL_ID = Types.VARCHAR;

    public static final String NAME_EXTERNAL_ID = "RESOURCES.EXTERNAL_ID";

    public static final String PROPERTY_EXTERNAL_ID = "EXTERNALID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_ID, NAME_EXTERNAL_ID);
    }

    /**
     * Column ASSIGNED_PLUGINS of type Types.BIGINT mapped to Long. PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_ASSIGNED_PLUGINS = 117;

    public static final int TYPE_ASSIGNED_PLUGINS = Types.BIGINT;

    public static final String NAME_ASSIGNED_PLUGINS = "RESOURCES.ASSIGNED_PLUGINS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGNED_PLUGINS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ASSIGNED_PLUGINS,NAME_ASSIGNED_PLUGINS);}

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_6 = 118;

    public static final int TYPE_CLEARANCE_LEVEL_6 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_6 = "RESOURCES.CLEARANCE_LEVEL_6";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_6 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_6,NAME_CLEARANCE_LEVEL_6);}

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CLEARANCE_LEVEL_7 = 119;

    public static final int TYPE_CLEARANCE_LEVEL_7 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_7 = "RESOURCES.CLEARANCE_LEVEL_7";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLEARANCE_LEVEL_7 = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CLEARANCE_LEVEL_7,NAME_CLEARANCE_LEVEL_7);}

    public static final String TABLE_NAME = "RESOURCES";

    /**
     * Create an array of type string containing all the fields of the RESOURCES
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RESOURCE_ID, NAME_FULL_NAME, NAME_FIRST_NAME,
            NAME_MIDDLE_NAME, NAME_LAST_NAME, NAME_E_MAIL_ADDRESS,
            NAME_LOGON_NAME, NAME_LOGON_PASSWORD, NAME_KEY_HISTORY,
            NAME_LOGON_REMINDER, NAME_CLEARANCE_LEVEL_1,
            NAME_CLEARANCE_LEVEL_2, NAME_CLEARANCE_LEVEL_3,
            NAME_CLEARANCE_LEVEL_4, NAME_MULTIPLE_LOGINS,
            NAME_PASSWORD_EXPIRES, NAME_WEBSITE, NAME_PRIMARY_ROLE, NAME_TITLE,
            NAME_ADDRESS1, NAME_ADDRESS2, NAME_ZIP_AREA_CODE, NAME_CITIZENSHIP,
            NAME_CALENDAR_ID, NAME_ACTIVE, NAME_EFFECTIVE_DATE,
            NAME_DISCONTINUE_DATE, NAME_STATUS1_ID, NAME_STATUS2_ID,
            NAME_CREATED_DATE, NAME_CREATED_BY, NAME_CHARGE_CODE,
            NAME_MAXDAYHRS, NAME_MINDAYHRS, NAME_PERCENT_TO_ASSIGN,
            NAME_RATE_ID, NAME_HOME_OFFICE, NAME_WORK_LOCATION,
            NAME_WORK_PHONE, NAME_ALTERNATE_PHONE, NAME_FAX_NUMBER,
            NAME_CELL_NUMBER, NAME_PAGER_NUMBER, NAME_ICQ_MESSAGING,
            NAME_NOTES_MAIL_ID, NAME_NOTES_ID, NAME_ASSIGN_STATUS,
            NAME_EXPERIENCE_ID, NAME_ALIAS, NAME_LAST_USED, NAME_EXEC_LEVEL,
            NAME_VERIFIED_1, NAME_VERIFIED_2, NAME_WILLING_RELOC,
            NAME_RELOC_DURATION, NAME_WORK_RELOC_1, NAME_WORK_RELOC_2,
            NAME_WORK_RELOC_3, NAME_WILLING_TRAVEL, NAME_TRAVEL_DURATION,
            NAME_TRAVEL_DEST_1, NAME_TRAVEL_DEST_2, NAME_TRAVEL_DEST_3,
            NAME_PASPORT_VALID, NAME_PASPORT_EXPIRATION, NAME_COUNTRY_VISA_1,
            NAME_VISA_EXPIRES_1, NAME_COUNTRY_VISA_2, NAME_VISA_EXPIRES_2,
            NAME_COUNTRY_VISA_3,
            NAME_VISA_EXPIRES_3,
            NAME_CURRENCY_ID,
            NAME_REC_STATUS,
            NAME_REC_USER,
            NAME_REC_DATETIME,
            NAME_MAJOR_REVISION,
            NAME_MINOR_REVISION,
            NAME_REVISION_IMPACT,
            NAME_REVISION_TYPE,
            NAME_REVISION_HISTORY,
            NAME_LAST_LOGIN,
            NAME_LANGUAGE_ISO_CODE,
            NAME_FS_CODE,
            NAME_FS_LABOR_CODE,
            NAME_EXPERIENCE_TXT,
            NAME_ACHIVEMENTS,
            NAME_PASSIONS,
            NAME_SUMMARY_SKILLS,
            NAME_MISCELLANEOUS,
            NAME_MAX_PASSWORD_FAILURE,
            NAME_NICK_NAME,
            NAME_PORTAL_PAGE,
            NAME_DASHBOARD,
            NAME_OVERWRITEFILE
            //NAME_CLEARANCE_LEVEL_5 appears twice in the list
            //don't remove it unless you know what you're doing
            , NAME_CLEARANCE_LEVEL_5, NAME_CONVERTED_SCORE,
            NAME_CONVERTED_WEIGHT, NAME_EXECUTIVE_SECURITY,
            NAME_MY_PORTAL_URL_1, NAME_MY_PORTAL_URL_2, NAME_MY_PORTAL_URL_3,
            NAME_MY_PORTAL_URL_4, NAME_MY_PORTAL_URL_5, NAME_CLEARANCE_LEVEL_5,
            NAME_USER_GROUP_ID, NAME_COUNTRY_CODE, NAME_COMPANY_CODE,
            NAME_EMPLOYEE_CODE, NAME_CUSTOM_01, NAME_CUSTOM_02, NAME_CUSTOM_03,
            NAME_CUSTOM_04, NAME_CUSTOM_05, NAME_DIVISION_CODE,
            NAME_DEPARTMENT_CODE, NAME_EXTERNAL_ID, NAME_ASSIGNED_PLUGINS,
            NAME_CLEARANCE_LEVEL_6, NAME_CLEARANCE_LEVEL_7
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    public ResourceManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_RESOURCE_ID;
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");

        return filter.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(Resource.TYPE_ID);

    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /**
     *  
     */

    public String getTreeTableName(String contextName)
    {
        return PoolManager.TABLE_NAME;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        //context and containerName are not applicable for this manager

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof AssetResourcesManager)
        {
            // for XPath queries like
            // Resource[<condition>]/AssetResourceAssignment
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(AssetResourcesManager.NAME_RESOURCE_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_RESOURCE_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");

            if (condition != null)
            {
                where.append(condition);
                where.append(" AND ");
            }
            where.append(NAME_REC_STATUS);
            where.append("!= 'D'");
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof CustomFieldAssignmentManager)
        {
            if (ContextUtil.RESOURCE_CONTEXT.equalsIgnoreCase(contextName))
            {
                // for XPath queries like
                // Resource[<condition>]/AssignedCustomField
                joinCondition = new JoinCondition();
                StringBuffer where = new StringBuffer();
                where.append(CustomFieldAssignmentUtil
                    .getTableName(ContextUtil.RESOURCE_CONTEXT));
                where.append(".");
                where.append(CustomFieldAssignmentManager.NAME_FOREIGN_ID);
                where.append(" IN (");
                where.append(" SELECT ").append(NAME_RESOURCE_ID);
                where.append(" FROM ").append(getTableName());
                where.append(" WHERE ");

                if (condition != null)
                {
                    where.append(condition);
                    where.append(" AND ");
                }
                where.append(getFilter());
                joinCondition.setCondition(where.toString());
            }
        }
        else if (manager instanceof ResourceIndustryExperienceManager)
        {
            // for XPath queries like
            // Resource[<condition>]/IndustryExperience
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(ResourceIndustryExperienceManager.NAME_RESOURCE_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_RESOURCE_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");

            if (condition != null)
            {
                where.append(condition);
                where.append(" AND ");
            }
            where.append(NAME_REC_STATUS);
            where.append("!= 'D'");
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceLanguageProficiencyManager)
        {
            // for XPath queries like
            // Resource[<condition>]/LanguageProficiency
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(ResourceLanguageProficiencyManager.NAME_RESOURCE_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_RESOURCE_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");

            if (condition != null)
            {
                where.append(condition);
                where.append(" AND ");
            }
            where.append(NAME_REC_STATUS);
            where.append("!= 'D'");
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof PersonalTaskAssignmentManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                PersonalTaskAssignmentManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());
        }
        else if (manager instanceof AdministrativeTaskAssignmentManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                AdministrativeTaskAssignmentManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());
        }
        else if (manager instanceof TimesheetTaskManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                TimesheetTaskManager.NAME_CREATED_BY, NAME_RESOURCE_ID,
                TABLE_NAME, getFilter());
        }
        else if (manager instanceof PublishedScorecardManager)
        {
            // For XPath like: /ResourceAssignedScorecard/Resource
            joinCondition = JoinCondition.createSubSelect(
                PublishedScorecardManager.NAME_CREATED_BY,
                NAME_REC_USER, TABLE_NAME, null);
        }
        else if (manager instanceof SecurityGroupManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                SecurityGroupManager.NAME_ELEMENT_ID,
                NAME_USER_GROUP_ID, TABLE_NAME, getFilter());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            //Add the table and the condition to join the 2 tables

            where.append(PoolManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(PoolManager.NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(PoolManager.TABLE_NAME);
            where.append(",");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(NAME_RESOURCE_ID);
            where.append(" = ");
            where.append(PoolManager.NAME_ELEMENT_ID);
            where.append(" AND ");
            where.append(getFilter());
            where.append(" AND ");
            where.append(PoolManager.NAME_TYPE_ID);
            where.append(" = ");
            where.append(Resource.TYPE_ID);

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof AssetResourcesManager)
        {
            // for XPath queries like
            // /AssetResourceAssignment[Resource[<condition>]]
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(AssetResourcesManager.NAME_RESOURCE_ID)
                .append(" IN (");
            where.append(" SELECT ").append(NAME_RESOURCE_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");

            where.append(NAME_REC_STATUS);
            where.append("!= 'D'");
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();

            if (container.getPreviousFieldName().equalsIgnoreCase("VERIFIEDBY"))
            {
                where.append(ResourceManager.NAME_VERIFIED_1);
            }
            else if (container.getPreviousFieldName().equalsIgnoreCase(
                "REVERIFIEDBY"))
            {
                where.append(ResourceManager.NAME_VERIFIED_2);
            }

            where.append(" IN (");
            where.append(" SELECT ").append(NAME_RESOURCE_ID);
            where.append(" FROM ").append(getTableName());
            where.append(" WHERE ");

            where.append(NAME_REC_STATUS);
            where.append("!= 'D'");
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceTaskAssignmentManager)
        {
            //    /ResourceTaskAssignment[Resource[ID='123']]
            joinCondition = JoinCondition.createSubSelect(
                ResourceTaskAssignmentManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());

            // (Igor) fix this one day...
            // HACK: horrible hack because of the evil nature of SP_SEARCH_2 
            // and both joinCondition and WbsManager limitations
            // 1) SP_SEARCH_2 creates 3 resourceTaskAssignments on 1 call
            // 2) WbsManager does not support one of them (TYPE_ID = 13 - see - 13 is an evil number too!)
            // 3) joinCondition does not have addTable() to have multiple tables
            // so finally I am doing this hack:

            //            joinCondition.setTableName(WbsManager.TABLE_NAME); // because we are lucky! it was not used!
            //            joinCondition.appendCondition(WbsManager.NAME_ELEMENT_ID 
            //                    + " = " + ResourceTaskAssignmentManager.NAME_ELEMENT_ID);
            //            joinCondition.appendCondition(WbsManager.NAME_TYPE_ID + " <> 13");
            //            
            // because SP_SEARCH_2 is involved, I also suggest:
            // joinCondition.appendCondition(WbsManager.NAME_TYPE_ID + " <> 666"); 
        }
        else if (manager instanceof SummaryTimesheetManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                SummaryTimesheetManager.NAME_RESOURCE_ID, NAME_RESOURCE_ID,
                TABLE_NAME, getFilter());

        }
        else if (manager instanceof TimesheetApprovalStatusManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                TimesheetApprovalStatusManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());

        }
        else if (manager instanceof StepManager)
        {
            SqlBuffer subCondition = new SqlBuffer();
            subCondition.appendSubSelect(
                TaskAssignmentManager.NAME_RESOURCE_ID, NAME_RESOURCE_ID,
                TABLE_NAME, getFilter());

            joinCondition = JoinCondition.createSubSelect(
                StepManager.NAME_TSK_ASSIGNMENT_ID,
                TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                TaskAssignmentManager.TABLE_NAME, subCondition.toString());
        }
        else if (manager instanceof CommunicationManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                CommunicationManager.NAME_AUTHOR_ID, NAME_RESOURCE_ID,
                TABLE_NAME, getFilter());
        }
        else if (manager instanceof WorkflowResponseManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                WorkflowResponseManager.NAME_RESOURCE_ID, NAME_RESOURCE_ID,
                TABLE_NAME, getFilter());
        }
        else if (manager instanceof ResourceRoleAssignmentManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ResourceRoleAssignmentManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());
        }
//        else if (manager instanceof PublishedScorecardManager)
//        {
//            joinCondition = new JoinCondition();
//
//            StringBuffer where = new StringBuffer();
//            where.append(PublishedScorecardManager.NAME_TYPE_ID);
//            where.append(" = ").append(Scorecard.TYPE_ID);
//            where.append(" AND ");
//            where.append(PublishedScorecardManager.NAME_PROJECT_ID);
////            where.append(" IN (SELECT ");
////            where.append(NAME_RESOURCE_ID);
////            where.append(" FROM ");
//            where.append(getTableName());
//            if (joinCondition != null)
//            {
//                where.append(" WHERE " + joinCondition);
//            }
//            joinCondition.setCondition(where.toString());
//        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {

        case Resource.TYPE_ID:
            object = new Resource();
            break;
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet,
     *      com.ibm.rpm.scope.RPMObjectScope)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        Resource object = (Resource) rpmObject;

        loadParent(object, rpmScope, messageContext, isDelta);

        if (rpmScope != null && rpmScope instanceof ResourceScope)
        {

            ResourceScope scope = (ResourceScope) rpmScope;

            loadCalendar(object, scope, messageContext, isDelta);
            loadClientCostCenter(object, scope, messageContext, isDelta);
            loadCompetenciesAssignment(object, scope, messageContext, isDelta);
            loadContactGroup(object, scope, messageContext, isDelta);

            loadDocumentFolder(object, scope, messageContext, isDelta);

            loadEducation(object, scope, messageContext, isDelta);
            loadEmployeeStatus(object, scope, messageContext, relatedFields,
                index, isDelta);
            loadEmploymentHistory(object, scope, messageContext, isDelta);
            loadExecutiveAttributes(object, scope, messageContext, isDelta);
            loadExecutiveAttributeClassifications(object, scope,
                messageContext, isDelta);
            loadExecutiveSecurityRole(object, scope, messageContext,
                relatedFields, index, isDelta);
            loadExperience(object, scope, messageContext, relatedFields, index,
                isDelta);
            loadGeographical(object, scope, messageContext, isDelta);
            loadIndustryExperience(object, scope, messageContext, isDelta);
            loadLanguageProficiency(object, scope, messageContext, isDelta);
            loadLocationPreferences(object, scope, relatedFields, index,
                isDelta);
            loadOrganizational(object, scope, messageContext, isDelta);
            loadPassport(object, scope, relatedFields, index, isDelta);
            loadWorkPackages(object, scope, messageContext, relatedFields,
                index, isDelta);
            loadProjectHistory(object, scope, messageContext, isDelta);
            loadSecurityGroup(object, scope, messageContext, relatedFields,
                index, isDelta);
            loadSkillAssignment(object, scope, messageContext, isDelta);
            loadWorkingStatus(object, scope, messageContext, relatedFields,
                index, isDelta);
            loadWorkLocation(object, scope, messageContext, relatedFields,
                index, isDelta);
            loadPersonalProject(object, scope, messageContext, isDelta);
            loadPersonalTaskAssignments(object, scope, messageContext, isDelta);
            loadAdministrativeTaskAssignments(object, scope, messageContext,
                isDelta);

            if (scope.isCustomFieldAssignments())
            {
                object.setCustomFieldAssignments(CustomFieldAssignmentUtil
                    .loadAssignedCustomFields(this, object
                        .getCustomFieldAssignments(), object, messageContext,
                        isDelta));

                propagateContextName(object, object.getCustomFieldAssignments());
            }

            loadCurrency(object, scope, messageContext, relatedFields, index,
                isDelta);
            loadRates(object, scope, messageContext, relatedFields, index,
                isDelta);

            loadReVerifiedBy(object, messageContext, relatedFields, index,
                isDelta, scope);
            loadVerifiedBy(object, messageContext, relatedFields, index,
                isDelta, scope);
            loadAssignedScorecard(object, messageContext, relatedFields, index,
                isDelta, scope);

        }

        return rpmObject;
    }

    private void loadAssignedScorecard(Resource resource, MessageContext messageContext, 
            FieldValueMap relatedFields, int index, boolean isDelta, ResourceScope scope) throws RPMException,
            SQLException, ParseException
    {
        if(scope.getAssignedScorecard() != null)
        {
            PublishedScorecardManager manager = (PublishedScorecardManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                    AssignedScorecard.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                resource, new RPMObject[]
            {
                        resource.getAssignedScorecard()
            }, scope.getAssignedScorecard(), messageContext, this,
            new Object[]
            {
            resource.getID()
            }, null, resource.getContextName(), isDelta);
        
            // there can be only 1 assignedScorecard
            if (temp != null && temp.size() > 0)
            {
                AssignedScorecard scorecard = (AssignedScorecard) temp.get(0);
                if (isDelta)
                    resource.deltaAssignedScorecard(scorecard);
                else
                    resource.setAssignedScorecard(scorecard);
                scorecard.setParent(resource);
            }//if
        }
    }

    private void loadVerifiedBy(Resource resource,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta, ResourceScope scope)
            throws RPMException, ParseException, SQLException
    {
        if (!scope.isVerifiedBy())
        {
            return;
        }

        Resource verifiedByResource = new Resource();
        verifiedByResource.setID((String) relatedFields.get(index,
            ID_VERIFIED_1));

        if (verifiedByResource.getID() != null)
        {
            verifiedByResource = (Resource) loadByPrimaryKey(
                verifiedByResource, null, messageContext, isDelta);
            if (isDelta)
            {
                resource.deltaVerifiedBy(verifiedByResource);
            }
            else
            {
                resource.setVerifiedBy(verifiedByResource);
            }
        }
    }

    private void loadReVerifiedBy(Resource resource,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta, ResourceScope scope)
            throws RPMException, ParseException, SQLException
    {
        if (!scope.isReVerifiedBy())
        {
            return;
        }

        Resource reVerifiedByResource = new Resource();
        reVerifiedByResource.setID((String) relatedFields.get(index,
            ID_VERIFIED_2));

        if (reVerifiedByResource.getID() != null)
        {
            reVerifiedByResource = (Resource) loadByPrimaryKey(
                reVerifiedByResource, null, messageContext, isDelta);
            if (isDelta)
            {
                resource.deltaReVerifiedBy(reVerifiedByResource);
            }
            else
            {
                resource.setReVerifiedBy(reVerifiedByResource);
            }
        }
    }

    private void loadParent(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        if (rpmScope != null && rpmScope.getParent() != null)
        {

            PoolManager manager = (PoolManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(Pool.class.getName());

            //Actual parent
            ArrayList temp = manager.loadByForeignKey(null, rpmScope
                .getParent(), messageContext, this, new Object[]
            {
                resource.getID()
            }, null, null, isDelta);

            Pool actualParent = null;
            if (temp != null && temp.size() > 0)
            {
                Pool[] pools = new Pool[temp.size()];
                temp.toArray(pools);
                actualParent = pools[0];
            }

            if (isDelta)
            {
                resource.deltaParent(actualParent);
            }
            else
            {
                resource.setParent(actualParent);
            }

        }
    }

    private void loadCalendar(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.getCalendar() != null)
        {

            RPMCalendarManager manager = (RPMCalendarManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(RPMCalendar.class.getName());

            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                resource.getCalendar()
            }, scope.getCalendar(), messageContext, this, new Object[]
            {
                resource.getID()
            }, null, null, isDelta);

            RPMCalendar actualCalendar = null;
            if (temp != null)
            {
                RPMCalendar[] calendars = new RPMCalendar[temp.size()];
                temp.toArray(calendars);
                actualCalendar = calendars[0];
            }

            if (isDelta)
            {
                resource.deltaCalendar(actualCalendar);
            }
            else
            {
                resource.setCalendar(actualCalendar);
            }

        }
    }

    private void loadCompetenciesAssignment(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isCompetencyAttributeAssignments())
        {

            ResourceAttributeAssignmentManager manager = (ResourceAttributeAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceAttributeAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getCompetencyAttributeAssignments(), null, messageContext,
                this, new Object[]
                {
                    resource.getID()
                }, " TYPE_ID = "
                        + ResourceAttributeCompetencyProficiencyLevel.TYPE_ID,
                null, isDelta);

            if (temp != null)
            {
                ResourceAttributeAssignment[] competencies = new ResourceAttributeAssignment[temp
                    .size()];
                temp.toArray(competencies);
                resource.setCompetencyAttributeAssignments(competencies);
            }
        }
    }

    private void loadContactGroup(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isContactGroupAssignments())

        {

            ContactGroupAssignmentManager manager = (ContactGroupAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ContactGroupAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getContactGroupAssignments(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                ContactGroupAssignment[] groups = new ContactGroupAssignment[temp
                    .size()];
                temp.toArray(groups);
                resource.setContactGroupAssignments(groups);
            }
        }
    }

    private void loadClientCostCenter(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isResourceCostCenterAssignments())
        {
            ResourceCostCenterManager manager = (ResourceCostCenterManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceCostCenterAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getResourceCostCenterAssignments(), null, messageContext,
                this, new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                ResourceCostCenterAssignment[] costCenters = new ResourceCostCenterAssignment[temp
                    .size()];
                temp.toArray(costCenters);
                resource.setResourceCostCenterAssignments(costCenters);
            }
        }
    }

    private void loadCurrency(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isCurrency())
        {
            Currency currency = new Currency();
            currency.setID((String) relatedFields.get(index, ID_CURRENCY_ID));

            if (!currency.getID().equals("0"))
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(Currency.class);

                currency = (Currency) manager.loadByPrimaryKey(currency, null,
                    messageContext, isDelta);

                if (isDelta)
                {
                    resource.deltaCurrency(currency);
                }
                else
                {
                    resource.setCurrency(currency);
                }
            }
        }
    }

    private void loadRates(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isRates())
        {

            ResourceRates rates = new ResourceRates();
            rates.setID((String) relatedFields.get(index, ID_RATE_ID));

            ResourceRatesManager manager = (ResourceRatesManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceRates.class.getName());

            if (manager.getPrimaryKeyString(rates) != null)
            {
                rates = (ResourceRates) manager.loadByPrimaryKey(rates, null,
                    messageContext, isDelta);
            }

            // Design note: rates.getID() never changes. => deltaRates is useless.
            // Design note: rates.*_is_modified flags are NOT available on update.
            if (!isDelta)
            {
                resource.setRates(rates);
            }
        }
    }

    private void loadDocumentFolder(Resource resource, ResourceScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if (scope.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                resource.getDocumentFolder()
            };
            Object[] args = new Object[]
            {
                resource.getID()
            };
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                resource, oldDocumentFolder, scope.getDocumentFolder(),
                messageContext, this, args, null,
                ContextUtil.RESOURCE_DOCUMENT_CONTEXT, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder
                    .setContextName(ContextUtil.RESOURCE_DOCUMENT_CONTEXT);
                resource.setDocumentFolder(documentFolder);
            }
        }
    }

    private Resource saveDocumentFolder(Resource resource, ResourceScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (scope.getDocumentFolder() != null
                && resource.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                .getName());

            DocumentFolder documentFolder = resource.getDocumentFolder();
            documentFolder.setParent(resource);
            manager.internalSave(documentFolder, scope.getDocumentFolder(),
                messageContext);
            documentFolder
                .setContextName(ContextUtil.RESOURCE_DOCUMENT_CONTEXT);
            resource.setDocumentFolder(documentFolder);
        }
        return resource;
    }

    private Resource savePersonalProject(Resource pObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        ResourceScope scope = (ResourceScope) rpmScope;
        PersonalProject child = pObject.getPersonalProject();
        child = (PersonalProject) updateChild(pObject, child, scope
            .getPersonalProject(), messageContext);
        pObject.setPersonalProject(child);
        return pObject;

    }

    private void loadEducation(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isEducations())
        {

            ResourceEducationManager manager = (ResourceEducationManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Education.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource.getEducations(),
                null, messageContext, this, new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                Education[] educations = new Education[temp.size()];
                temp.toArray(educations);
                resource.setEducations(educations);

                //set Resource in Education
                Education edu = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    edu = (Education) iter.next();
                    edu.setParent(resource);
                }
            }
        }
    }

    private void loadEmployeeStatus(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isEmployeeStatus())
        {

            EmployeeStatus status = null;
            String employeeStatusId = (String) relatedFields.get(index,
                ID_STATUS2_ID);

            if (employeeStatusId != null)
            {
                status = new EmployeeStatus();
                status.setID(employeeStatusId);

                DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        EmployeeStatus.class.getName());

                status = (EmployeeStatus) manager.loadByPrimaryKey(status,
                    null, messageContext, false);
            }

            if (isDelta)
            {
                resource.deltaEmployeeStatus(status);
            }
            else
            {
                resource.setEmployeeStatus(status);
            }
        }
    }

    private void loadEmploymentHistory(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isEmploymentHistories())
        {

            ResourceEmploymentHistoryManager manager = (ResourceEmploymentHistoryManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    EmploymentHistory.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getEmploymentHistories(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                EmploymentHistory[] eh = new EmploymentHistory[temp.size()];
                temp.toArray(eh);
                resource.setEmploymentHistories(eh);

                //set Resource in the container
                EmploymentHistory hist = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    hist = (EmploymentHistory) iter.next();
                    hist.setParent(resource);
                }
            }
        }
    }

    private String getContextName(RPMObject rpmObject) throws RPMException
    {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }

    private void loadExecutiveAttributes(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isExecutiveAttributes())

        {
            AttributeAssignmentManager manager = (AttributeAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AttributeAssignment.class.getName());

            String extrawhere = " ELEMENT_ID IN (SELECT RESOURCE_ID FROM RESOURCES) AND "
                    + " PARENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS_ATTRIBUTES WHERE TYPE_ID = "
                    + Attribute.TYPE_ID
                    + " OR TYPE_ID = "
                    + Attribute.TYPE_ID2
                    + ")";
            ArrayList temp = manager.loadByForeignKey(resource
                .getExecutiveAttributes(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, extrawhere, getContextName(resource), isDelta);

            if (temp != null)
            {
                AttributeAssignment[] executiveAttributes = new AttributeAssignment[temp
                    .size()];
                temp.toArray(executiveAttributes);
                resource.setExecutiveAttributes(executiveAttributes);

                //set Resource in the container
                AttributeAssignment rwa = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    rwa = (AttributeAssignment) iter.next();
                    rwa.setParent(resource);
                }
            }
        }
    }

    private void loadExecutiveAttributeClassifications(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isExecutiveAttributeClassifications())

        {
            AttributeAssignmentManager manager = (AttributeAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AttributeAssignment.class.getName());

            String extrawhere = " ELEMENT_ID IN (SELECT RESOURCE_ID FROM RESOURCES) AND "
                    + " PARENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS_ATTRIBUTES WHERE TYPE_ID = "
                    + AttributeClassification.TYPE_ID
                    + " OR TYPE_ID = "
                    + AttributeClassification.TYPE_ID2 + ")";
            ArrayList temp = manager.loadByForeignKey(resource
                .getExecutiveAttributeClassifications(), null, messageContext,
                this, new Object[]
                {
                    resource.getID()
                }, extrawhere, getContextName(resource), isDelta);

            if (temp != null)
            {
                AttributeAssignment[] executiveClassifications = new AttributeAssignment[temp
                    .size()];
                temp.toArray(executiveClassifications);
                resource
                    .setExecutiveAttributeClassifications(executiveClassifications);

                //set Resource in the container
                AttributeAssignment rwa = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    rwa = (AttributeAssignment) iter.next();
                    rwa.setParent(resource);
                }
            }
        }
    }

    private void loadExecutiveSecurityRole(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        if (scope != null && scope.getExecutiveSecurityRole() != null)
        {
            ProjectSecurityRole projectSecurityRole = null;
            String executiveId = (String) relatedFields.get(index,
                ID_EXECUTIVE_SECURITY);
            if (executiveId != null)
            {
                projectSecurityRole = new ProjectSecurityRole();
                projectSecurityRole.setID(executiveId);

                SecurityRoleManager manager = (SecurityRoleManager) getManagerInstance(ProjectSecurityRole.class);

                projectSecurityRole = (ProjectSecurityRole) manager
                    .loadByPrimaryKey(projectSecurityRole, scope
                        .getExecutiveSecurityRole(), messageContext, isDelta);
            }

            if (isDelta)
            {
                resource.deltaExecutiveSecurityRole(projectSecurityRole);
            }
            else
            {
                resource.setExecutiveSecurityRole(projectSecurityRole);
            }
        }
    }

    private void loadExperience(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isExperience())
        {
            Experience experience = null;
            String experienceId = (String) relatedFields.get(index,
                ID_EXPERIENCE_ID);

            if (experienceId != null)
            {
                experience = new Experience();
                experience.setID(experienceId);

                DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        Experience.class.getName());

                experience = (Experience) manager.loadByPrimaryKey(experience,
                    null, messageContext, isDelta);
            }

            if (isDelta)
            {
                resource.deltaExperience(experience);
            }
            else
            {
                resource.setExperience(experience);
            }
        }
    }

    private void loadGeographical(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isGeographicalAssignment())
        {

            GeographicalAssignmentManager manager = (GeographicalAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    GeographicalAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                resource.getGeographicalAssignment()
            }, null, messageContext, this, new Object[]
            {
                resource.getID()
            }, null, null, isDelta);

            GeographicalAssignment geoAssignment = null;
            if (temp != null)
            {
                GeographicalAssignment[] geos = new GeographicalAssignment[temp
                    .size()];
                temp.toArray(geos);
                geoAssignment = geos[0];
            }
            if (isDelta)
            {
                resource.deltaGeographicalAssignment(geoAssignment);
            }
            else
            {
                resource.setGeographicalAssignment(geoAssignment);
            }
        }
    }

    private void loadIndustryExperience(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isIndustryExperiences())
        {

            ResourceIndustryExperienceManager manager = (ResourceIndustryExperienceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    IndustryExperience.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getIndustryExperiences(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                IndustryExperience[] ies = new IndustryExperience[temp.size()];
                temp.toArray(ies);
                resource.setIndustryExperiences(ies);

                //set Resource in the container
                IndustryExperience ie = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    ie = (IndustryExperience) iter.next();
                    ie.setParent(resource);
                }
            }
        }
    }

    private void loadLanguageProficiency(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isLanguageProficiencies())
        {

            ResourceLanguageProficiencyManager manager = (ResourceLanguageProficiencyManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    LanguageProficiency.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getLanguageProficiencies(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                LanguageProficiency[] lps = new LanguageProficiency[temp.size()];
                temp.toArray(lps);
                resource.setLanguageProficiencies(lps);

                //set Resource in the container
                LanguageProficiency lp = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    lp = (LanguageProficiency) iter.next();
                    lp.setParent(resource);
                }
            }
        }
    }

    private void loadLocationPreferences(Resource resource,
            RPMObjectScope rpmScope, FieldValueMap relatedFields, int index,
            boolean isDelta) throws RPMException
    {

        ResourceScope scope = (ResourceScope) rpmScope;
        if (scope.isLocationPreferences())
        {

            LocationPreferences lp = new LocationPreferences();
            ArrayList locations = new ArrayList();
            String relocId1 = (String) relatedFields
                .get(index, ID_WORK_RELOC_1);
            if (relocId1 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(relocId1);
                locations.add(cc);
            }
            String relocId2 = (String) relatedFields
                .get(index, ID_WORK_RELOC_2);
            if (relocId2 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(relocId2);
                locations.add(cc);
            }
            String relocId3 = (String) relatedFields
                .get(index, ID_WORK_RELOC_3);
            if (relocId3 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(relocId3);
                locations.add(cc);
            }

            CitizenshipCountryCode[] reloc = null;
            if (locations.size() > 0)
            {
                reloc = new CitizenshipCountryCode[locations.size()];
                locations.toArray(reloc);
            }

            ArrayList travelsLoc = new ArrayList();
            String travelId1 = (String) relatedFields.get(index,
                ID_TRAVEL_DEST_1);
            if (travelId1 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(travelId1);
                travelsLoc.add(cc);
            }
            String travelId2 = (String) relatedFields.get(index,
                ID_TRAVEL_DEST_2);
            if (travelId2 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(travelId2);
                travelsLoc.add(cc);
            }
            String travelId3 = (String) relatedFields.get(index,
                ID_TRAVEL_DEST_3);
            if (travelId3 != null)
            {
                CitizenshipCountryCode cc = new CitizenshipCountryCode();
                cc.setID(travelId3);
                travelsLoc.add(cc);
            }

            CitizenshipCountryCode[] trloc = null;
            if (travelsLoc.size() > 0)
            {
                trloc = new CitizenshipCountryCode[travelsLoc.size()];
                travelsLoc.toArray(trloc);
            }

            if (isDelta)
            {
                lp.deltaAcceptableRelocateDuration((Integer) relatedFields.get(
                    index, ID_RELOC_DURATION));
                lp.deltaAcceptableTravelDuration((Integer) relatedFields.get(
                    index, ID_TRAVEL_DURATION));
                lp.deltaWillingtoRelocate((Boolean) relatedFields.get(index,
                    ID_WILLING_RELOC));
                lp.deltaWillingtoTravel((Boolean) relatedFields.get(index,
                    ID_WILLING_TRAVEL));
                //RelocateLocations and TravelLocations are not set because
                // modified values would be lost
                resource.deltaLocationPreferences(lp);

            }
            else
            {
                lp.setAcceptableRelocateDuration((Integer) relatedFields.get(
                    index, ID_RELOC_DURATION));
                lp.setAcceptableTravelDuration((Integer) relatedFields.get(
                    index, ID_TRAVEL_DURATION));
                lp.setWillingtoRelocate((Boolean) relatedFields.get(index,
                    ID_WILLING_RELOC));
                lp.setWillingtoTravel((Boolean) relatedFields.get(index,
                    ID_WILLING_TRAVEL));
                lp.setRelocateLocations(reloc);
                lp.setTravelLocations(trloc);
                resource.setLocationPreferences(lp);
            }
        }
    }

    private void loadOrganizational(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isOrganizationalAssignment())
        {

            OrganizationalAssignmentManager manager = (OrganizationalAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    OrganizationalAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                resource.getOrganizationalAssignment()
            }, null, messageContext, this, new Object[]
            {
                resource.getID()
            }, null, null, isDelta);

            OrganizationalAssignment orgAssignment = null;
            if (temp != null)
            {
                OrganizationalAssignment[] orgs = new OrganizationalAssignment[temp
                    .size()];
                temp.toArray(orgs);
                orgAssignment = orgs[0];
            }
            if (isDelta)
            {
                resource.deltaOrganizationalAssignment(orgAssignment);
            }
            else
            {
                resource.setOrganizationalAssignment(orgAssignment);
            }
        }
    }

    private void loadPassport(Resource resource, RPMObjectScope rpmScope,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException
    {

        ResourceScope scope = (ResourceScope) rpmScope;
        if (scope != null && scope.isPassport())
        {

            Passport passport = resource.getPassport();
            if (passport == null)
            {
                passport = new Passport();
            }

            ArrayList visas = new ArrayList();
            String country1 = (String) relatedFields.get(index,
                ID_COUNTRY_VISA_1);
            if (country1 != null)
            {
                CitizenshipCountryCode ccc1 = new CitizenshipCountryCode();
                ccc1.setID(country1);
                Visa visa = new Visa();
                visa.setCountry(ccc1);
                visa.setExpiryDate((Calendar) relatedFields.get(index,
                    ID_VISA_EXPIRES_1));
                visas.add(visa);
            }

            String country2 = (String) relatedFields.get(index,
                ID_COUNTRY_VISA_2);
            if (country2 != null)
            {
                CitizenshipCountryCode ccc2 = new CitizenshipCountryCode();
                ccc2.setID(country2);
                Visa visa = new Visa();
                visa.setCountry(ccc2);
                visa.setExpiryDate((Calendar) relatedFields.get(index,
                    ID_VISA_EXPIRES_2));
                visas.add(visa);
            }
            String country3 = (String) relatedFields.get(index,
                ID_COUNTRY_VISA_3);
            if (country3 != null)
            {
                CitizenshipCountryCode ccc3 = new CitizenshipCountryCode();
                ccc3.setID(country3);
                Visa visa = new Visa();
                visa.setCountry(ccc3);
                visa.setExpiryDate((Calendar) relatedFields.get(index,
                    ID_VISA_EXPIRES_3));
                visas.add(visa);
            }

            Visa[] vi = null;
            if (visas.size() > 0)
            {
                vi = new Visa[visas.size()];
                visas.toArray(vi);
            }

            if (isDelta)
            {
                passport.deltaHasValidPassport((Boolean) relatedFields.get(
                    index, ID_PASPORT_VALID));
                passport.deltaExpiryDate((Calendar) relatedFields.get(index,
                    ID_PASPORT_EXPIRATION));
                //visas are not set because modified visas would be lost
                resource.deltaPassport(passport);
            }
            else
            {
                passport.setHasValidPassport((Boolean) relatedFields.get(index,
                    ID_PASPORT_VALID));
                passport.setExpiryDate((Calendar) relatedFields.get(index,
                    ID_PASPORT_EXPIRATION));
                passport.setVisas(vi);
                resource.setPassport(passport);
                passport.setParent(resource);
            }
        }
    }

    private void loadProjectHistory(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isProjectHistories())
        {

            ResourceProjectHistoryManager manager = (ResourceProjectHistoryManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ProjectHistory.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getProjectHistories(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                ProjectHistory[] phs = new ProjectHistory[temp.size()];
                temp.toArray(phs);
                resource.setProjectHistories(phs);

                //set Resource in the container
                ProjectHistory ph = null;
                Iterator iter = temp.iterator();
                while (iter.hasNext())
                {
                    ph = (ProjectHistory) iter.next();
                    ph.setParent(resource);
                }
            }
        }
    }

    private void loadSecurityGroup(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        if (scope != null && scope.getSecurityGroup() != null)
        {
            SecurityGroup securityGroup = new SecurityGroup();
            securityGroup.setID((String) relatedFields.get(index,
                ID_USER_GROUP_ID));

            SecurityGroupManager manager = (SecurityGroupManager) getManagerInstance(SecurityGroup.class);

            securityGroup = (SecurityGroup) manager.loadByPrimaryKey(
                securityGroup, scope.getSecurityGroup(), messageContext,
                isDelta);

            if (isDelta)
            {
                resource.deltaSecurityGroup(securityGroup);
            }
            else
            {
                resource.setSecurityGroup(securityGroup);
            }
        }
    }

    private void loadSkillAssignment(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isSkillAttributeAssignments())
        {

            ResourceAttributeAssignmentManager manager = (ResourceAttributeAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceAttributeAssignment.class.getName());

            ArrayList temp = manager.loadByForeignKey(resource
                .getSkillAttributeAssignments(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, " TYPE_ID = "
                        + ResourceAttributeSkillProficiencyLevel.TYPE_ID,
                getContextName(resource), isDelta);

            if (temp != null)
            {
                ResourceAttributeAssignment[] skills = new ResourceAttributeAssignment[temp
                    .size()];
                temp.toArray(skills);
                resource.setSkillAttributeAssignments(skills);
            }
        }
    }

    private void loadWorkingStatus(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isWorkingStatus())
        {
            WorkingStatus status = null;
            String workingStatusId = (String) relatedFields.get(index,
                ID_STATUS1_ID);

            if (workingStatusId != null)
            {
                status = new WorkingStatus();
                status.setID(workingStatusId);

                DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        WorkingStatus.class.getName());

                status = (WorkingStatus) manager.loadByPrimaryKey(status, null,
                    messageContext, isDelta);
            }

            if (isDelta)
            {
                resource.deltaWorkingStatus(status);
            }
            else
            {
                resource.setWorkingStatus(status);
            }
        }
    }

    private void loadWorkPackages(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        //TODO Implements method ResourceManager.loadWorkPackages

    }

    private void loadWorkLocation(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        if (scope != null && scope.isWorkLocation())
        {
            WorkLocation workLocation = null;
            String workLocationId = (String) relatedFields.get(index,
                ID_WORK_LOCATION);

            if (workLocationId != null)
            {
                workLocation = new WorkLocation();
                workLocation.setID(workLocationId);

                DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        WorkLocation.class.getName());

                workLocation = (WorkLocation) manager.loadByPrimaryKey(
                    workLocation, null, messageContext, isDelta);
            }

            if (isDelta)
            {
                resource.deltaWorkLocation(workLocation);
            }
            else
            {
                resource.setWorkLocation(workLocation);
            }
        }
    }

    private void loadPersonalTaskAssignments(Resource resource,
            ResourceScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        if (scope.isPersonalTaskAssignments())
        {

            RPMObjectManager manager = getManagerInstance(PersonalTaskAssignment.class);

            ArrayList temp = manager.loadByForeignKey(resource
                .getPersonalTaskAssignments(), null, messageContext, this,
                new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            resource
                .setPersonalTaskAssignments((PersonalTaskAssignment[]) ArrayUtil
                    .listToArray(PersonalTaskAssignment.class, temp));

        }
    }

    private void loadPersonalProject(Resource resource, ResourceScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        if (scope.getPersonalProject() != null)
        {

            RPMObjectManager manager = getManagerInstance(PersonalProject.class);

            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                resource.getPersonalProject()
            }, scope.getPersonalProject(), messageContext, this, new Object[]
            {
                resource.getID()
            }, null, null, isDelta);

            //temp should always have one PersonelProject since PersonalProject
            // is created automatically
            if (temp != null && !temp.isEmpty())
            {
                resource.setPersonalProject((PersonalProject) temp.get(0));
            }
        }
    }

    private void loadAdministrativeTaskAssignments(Resource resource,
            ResourceScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        if (scope.isAdministrativeTaskAssignments())
        {

            RPMObjectManager manager = getManagerInstance(AdministrativeTaskAssignment.class);

            ArrayList temp = manager.loadByForeignKey(resource
                .getAdministrativeTaskAssignments(), null, messageContext,
                this, new Object[]
                {
                    resource.getID()
                }, null, null, isDelta);

            resource
                .setAdministrativeTaskAssignments((AdministrativeTaskAssignment[]) ArrayUtil
                    .listToArray(AdministrativeTaskAssignment.class, temp));

        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException, RPMException
    {
        Resource resObj = (Resource) rpmObject;
        resObj.setID(rs.getString(ID_RESOURCE_ID));

        if (isDelta)
        {
            resObj.deltaActive(Manager.getBoolean(rs, ID_ACTIVE));
            resObj.deltaAddress1(rs.getString(ID_ADDRESS1));
            resObj.deltaAddress2(rs.getString(ID_ADDRESS2));
            resObj.deltaAlternativePhone(rs.getString(ID_ALTERNATE_PHONE));
            resObj.deltaCompanyCode(rs.getString(ID_COMPANY_CODE));
            resObj.deltaCountryCode(rs.getString(ID_COUNTRY_CODE));
            resObj.deltaCustom1(rs.getString(ID_CUSTOM_01));
            resObj.deltaCustom2(rs.getString(ID_CUSTOM_02));
            resObj.deltaCustom3(rs.getString(ID_CUSTOM_03));
            resObj.deltaCustom4(rs.getString(ID_CUSTOM_04));
            resObj.deltaCustom5(rs.getString(ID_CUSTOM_05));
            resObj.deltaDashboard(rs.getString(ID_DASHBOARD));
            resObj.deltaDepartmentCode(rs.getString(ID_DEPARTMENT_CODE));
            resObj.deltaDiscontinueDate(Manager.getCalendar(rs,
                ID_DISCONTINUE_DATE));
            resObj.deltaDivisionCode(rs.getString(ID_DIVISION_CODE));
            resObj.deltaEffectiveStartDate(Manager.getCalendar(rs,
                ID_EFFECTIVE_DATE));
            resObj.deltaEmailAddress(rs.getString(ID_E_MAIL_ADDRESS));
            resObj.deltaEmployeeCode(rs.getString(ID_EMPLOYEE_CODE));
            resObj.deltaExternalID(rs.getString(ID_EXTERNAL_ID));
            resObj.deltaFax(rs.getString(ID_FAX_NUMBER));
            resObj.deltaFamilyName(rs.getString(ID_LAST_NAME));
            resObj.deltaFullName(rs.getString(ID_FULL_NAME));
            resObj.deltaGivenName(rs.getString(ID_FIRST_NAME));
            resObj.deltaHomePhone(rs.getString(ID_HOME_OFFICE));
            resObj.deltaInterviewExperience(rs.getString(ID_EXPERIENCE_TXT));
            resObj.deltaLogonReminder(rs.getString(ID_LOGON_REMINDER));
            resObj.deltaMiddleName(rs.getString(ID_MIDDLE_NAME));
            resObj.deltaMobile(rs.getString(ID_CELL_NUMBER));
            resObj.deltaMyAchievements(rs.getString(ID_ACHIVEMENTS));
            resObj.deltaMyPersonalPassions(rs.getString(ID_PASSIONS));
            resObj.deltaMyPortalURL1(rs.getString(ID_MY_PORTAL_URL_1));
            resObj.deltaMyPortalURL2(rs.getString(ID_MY_PORTAL_URL_2));
            resObj.deltaMyPortalURL3(rs.getString(ID_MY_PORTAL_URL_3));
            resObj.deltaMyPortalURL4(rs.getString(ID_MY_PORTAL_URL_4));
            resObj.deltaMyPortalURL5(rs.getString(ID_MY_PORTAL_URL_5));
            resObj.deltaMySummaryOfSkills(rs.getString(ID_SUMMARY_SKILLS));
            resObj.deltaMyWebsite(rs.getString(ID_WEBSITE));
            //        resObj.deltaName(rs.getString())
            resObj.deltaNickName(rs.getString(ID_NICK_NAME));
            resObj.deltaOverrideOriginalDocumentOnCheckout(rs.getString(
                ID_OVERWRITEFILE).equals("Y") ? Boolean.TRUE : Boolean.FALSE);
            resObj.deltaPager(rs.getString(ID_PAGER_NUMBER));
            resObj.deltaPassword(rs.getString(ID_LOGON_PASSWORD));
            resObj.deltaTitle(rs.getString(ID_TITLE));
            resObj.deltaUserName(rs.getString(ID_LOGON_NAME));
            resObj.deltaWeeklyMaxHours(new Integer(rs.getInt(ID_MAXDAYHRS)));
            resObj.deltaWeeklyMinHours(new Integer(rs.getInt(ID_MINDAYHRS)));
            resObj.deltaWhatIDoReallyWell(rs.getString(ID_MISCELLANEOUS));
            resObj.deltaWorkPhone(rs.getString(ID_WORK_PHONE));
            resObj.deltaZipPostalCode(rs.getString(ID_ZIP_AREA_CODE));
            deltaClearanceLevel(rs, resObj);
        }
        else
        {
            resObj.setActive(Manager.getBoolean(rs, ID_ACTIVE));
            resObj.setAddress1(rs.getString(ID_ADDRESS1));
            resObj.setAddress2(rs.getString(ID_ADDRESS2));
            resObj.setAlternativePhone(rs.getString(ID_ALTERNATE_PHONE));
            resObj.setCompanyCode(rs.getString(ID_COMPANY_CODE));
            resObj.setCountryCode(rs.getString(ID_COUNTRY_CODE));
            resObj.setCustom1(rs.getString(ID_CUSTOM_01));
            resObj.setCustom2(rs.getString(ID_CUSTOM_02));
            resObj.setCustom3(rs.getString(ID_CUSTOM_03));
            resObj.setCustom4(rs.getString(ID_CUSTOM_04));
            resObj.setCustom5(rs.getString(ID_CUSTOM_05));
            resObj.setDashboard(rs.getString(ID_DASHBOARD));
            resObj.setDepartmentCode(rs.getString(ID_DEPARTMENT_CODE));
            resObj.setDiscontinueDate(Manager.getCalendar(rs,
                ID_DISCONTINUE_DATE));
            resObj.setDivisionCode(rs.getString(ID_DIVISION_CODE));
            resObj.setEffectiveStartDate(Manager.getCalendar(rs,
                ID_EFFECTIVE_DATE));
            resObj.setEmailAddress(rs.getString(ID_E_MAIL_ADDRESS));
            resObj.setEmployeeCode(rs.getString(ID_EMPLOYEE_CODE));
            resObj.setExternalID(rs.getString(ID_EXTERNAL_ID));
            resObj.setFax(rs.getString(ID_FAX_NUMBER));
            resObj.setFamilyName(rs.getString(ID_LAST_NAME));
            resObj.setFullName(rs.getString(ID_FULL_NAME));
            resObj.setGivenName(rs.getString(ID_FIRST_NAME));
            resObj.setHomePhone(rs.getString(ID_HOME_OFFICE));
            resObj.setInterviewExperience(rs.getString(ID_EXPERIENCE_TXT));
            resObj.setLogonReminder(rs.getString(ID_LOGON_REMINDER));
            resObj.setMiddleName(rs.getString(ID_MIDDLE_NAME));
            resObj.setMobile(rs.getString(ID_CELL_NUMBER));
            resObj.setMyAchievements(rs.getString(ID_ACHIVEMENTS));
            resObj.setMyPersonalPassions(rs.getString(ID_PASSIONS));
            resObj.setMyPortalURL1(rs.getString(ID_MY_PORTAL_URL_1));
            resObj.setMyPortalURL2(rs.getString(ID_MY_PORTAL_URL_2));
            resObj.setMyPortalURL3(rs.getString(ID_MY_PORTAL_URL_3));
            resObj.setMyPortalURL4(rs.getString(ID_MY_PORTAL_URL_4));
            resObj.setMyPortalURL5(rs.getString(ID_MY_PORTAL_URL_5));
            resObj.setMySummaryOfSkills(rs.getString(ID_SUMMARY_SKILLS));
            resObj.setMyWebsite(rs.getString(ID_WEBSITE));
            //        resObj.setName(rs.getString())
            resObj.setNickName(rs.getString(ID_NICK_NAME));
            resObj.setOverrideOriginalDocumentOnCheckout(rs.getString(
                ID_OVERWRITEFILE).equals("Y") ? Boolean.TRUE : Boolean.FALSE);
            resObj.setPager(rs.getString(ID_PAGER_NUMBER));
            resObj.setPassword(rs.getString(ID_LOGON_PASSWORD));
            resObj.setTitle(rs.getString(ID_TITLE));
            resObj.setUserName(rs.getString(ID_LOGON_NAME));
            resObj.setWeeklyMaxHours(new Integer(rs.getInt(ID_MAXDAYHRS)));
            resObj.setWeeklyMinHours(new Integer(rs.getInt(ID_MINDAYHRS)));
            resObj.setWhatIDoReallyWell(rs.getString(ID_MISCELLANEOUS));
            resObj.setWorkPhone(rs.getString(ID_WORK_PHONE));
            resObj.setZipPostalCode(rs.getString(ID_ZIP_AREA_CODE));
            setClearanceLevel(rs, resObj);
        }

        return resObj;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String experienceId = resultSet.getString(ID_EXPERIENCE_ID);
        relatedFields.put(index, ID_EXPERIENCE_ID, experienceId);

        String status1Id = resultSet.getString(ID_STATUS1_ID);
        relatedFields.put(index, ID_STATUS1_ID, status1Id);

        String status2Id = resultSet.getString(ID_STATUS2_ID);
        relatedFields.put(index, ID_STATUS2_ID, status2Id);

        String currencyId = resultSet.getString(ID_CURRENCY_ID);
        relatedFields.put(index, ID_CURRENCY_ID, currencyId);

        String rateId = resultSet.getString(ID_RATE_ID);
        relatedFields.put(index, ID_RATE_ID, rateId);

        String executiveSecurityId = resultSet.getString(ID_EXECUTIVE_SECURITY);
        if (executiveSecurityId != null)
        {
            executiveSecurityId = executiveSecurityId.trim();
        }
        relatedFields.put(index, ID_EXECUTIVE_SECURITY, executiveSecurityId);

        String userGroupId = resultSet.getString(ID_USER_GROUP_ID);
        if (userGroupId != null)
        {
            userGroupId = userGroupId.trim();
        }
        relatedFields.put(index, ID_USER_GROUP_ID, userGroupId);

        String workLocation = resultSet.getString(ID_WORK_LOCATION);
        relatedFields.put(index, ID_WORK_LOCATION, workLocation);

        String workReloc1 = resultSet.getString(ID_WORK_RELOC_1);
        relatedFields.put(index, ID_WORK_RELOC_1, workReloc1);

        String workReloc2 = resultSet.getString(ID_WORK_RELOC_2);
        relatedFields.put(index, ID_WORK_RELOC_2, workReloc2);

        String workReloc3 = resultSet.getString(ID_WORK_RELOC_3);
        relatedFields.put(index, ID_WORK_RELOC_3, workReloc3);

        String travelDest1 = resultSet.getString(ID_TRAVEL_DEST_1);
        relatedFields.put(index, ID_TRAVEL_DEST_1, travelDest1);

        String travelDest2 = resultSet.getString(ID_TRAVEL_DEST_2);
        relatedFields.put(index, ID_TRAVEL_DEST_2, travelDest2);

        String travelDest3 = resultSet.getString(ID_TRAVEL_DEST_3);
        relatedFields.put(index, ID_TRAVEL_DEST_3, travelDest3);

        Integer relocDuration = new Integer(resultSet.getInt(ID_RELOC_DURATION));
        relatedFields.put(index, ID_RELOC_DURATION, relocDuration);

        Integer travelDuration = new Integer(resultSet
            .getInt(ID_TRAVEL_DURATION));
        relatedFields.put(index, ID_TRAVEL_DURATION, travelDuration);

        Boolean willingReloc = new Boolean(resultSet
            .getBoolean(ID_WILLING_RELOC));
        relatedFields.put(index, ID_WILLING_RELOC, willingReloc);

        Boolean willingTravel = new Boolean(resultSet
            .getBoolean(ID_WILLING_TRAVEL));
        relatedFields.put(index, ID_WILLING_TRAVEL, willingTravel);

        String countryVisa1 = resultSet.getString(ID_COUNTRY_VISA_1);
        relatedFields.put(index, ID_COUNTRY_VISA_1, countryVisa1);

        String countryVisa2 = resultSet.getString(ID_COUNTRY_VISA_2);
        relatedFields.put(index, ID_COUNTRY_VISA_2, countryVisa2);

        String countryVisa3 = resultSet.getString(ID_COUNTRY_VISA_3);
        relatedFields.put(index, ID_COUNTRY_VISA_3, countryVisa3);

        Boolean isPassportValid = new Boolean(resultSet
            .getBoolean(ID_PASPORT_VALID));
        relatedFields.put(index, ID_PASPORT_VALID, isPassportValid);

        Calendar passportExpiration = DateUtil.convertToCalendar(resultSet
            .getDate(ID_PASPORT_EXPIRATION));
        relatedFields.put(index, ID_PASPORT_EXPIRATION, passportExpiration);

        Calendar visaExpires1 = DateUtil.convertToCalendar(resultSet
            .getDate(ID_VISA_EXPIRES_1));
        relatedFields.put(index, ID_VISA_EXPIRES_1, visaExpires1);

        Calendar visaExpires2 = DateUtil.convertToCalendar(resultSet
            .getDate(ID_VISA_EXPIRES_2));
        relatedFields.put(index, ID_VISA_EXPIRES_2, visaExpires2);

        Calendar visaExpires3 = DateUtil.convertToCalendar(resultSet
            .getDate(ID_VISA_EXPIRES_3));
        relatedFields.put(index, ID_VISA_EXPIRES_3, visaExpires3);

        String verifiedById1 = resultSet.getString(ID_VERIFIED_1);
        relatedFields.put(index, ID_VERIFIED_1, verifiedById1);

        String verifiedById2 = resultSet.getString(ID_VERIFIED_2);
        relatedFields.put(index, ID_VERIFIED_2, verifiedById2);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        if (rpmObject == null)
            return;
        Resource savedObj = null;
        savedObj = (Resource) rpmObject;

        // set default values
        if (savedObj.getActive() == null)
        {
            savedObj.setActive(new Boolean(true));
        }
        if (savedObj.getCurrency() == null)
        {
            // set the ID; the currency will be loaded after the save
            Currency currency = new Currency();
            currency.setID(DatafieldUtil.getDefaultCurrencyID(messageContext));
            savedObj.setCurrency(currency);
        }
        if (savedObj.getEffectiveStartDate() == null)
        {
            savedObj.setEffectiveStartDate(DateUtil.getCalendarInstance());
        }
        if (savedObj.getExperience() == null)
        {
            List experiences = DatafieldUtil.getDatafieldValue(StringUtil
                .getShortClassName(Experience.class.getName()), "7", null,
                messageContext);
            savedObj.setExperience((Experience) experiences.get(0));
        }
        if (savedObj.getOverrideOriginalDocumentOnCheckout() == null)
        {
            savedObj.setOverrideOriginalDocumentOnCheckout(new Boolean(true));
        }
        if (savedObj.getPassword() == null)
        {
            savedObj.setPassword(DEFAULT_PASSWORD);
        }
        if (savedObj.getWeeklyMaxHours() == null)
        {
            savedObj.setWeeklyMaxHours(new Integer(40));
        }
        if (savedObj.getWorkLocation() == null)
        {
            List workLocations = DatafieldUtil.getDatafieldValue(StringUtil
                .getShortClassName(WorkLocation.class.getName()), "1", null,
                messageContext);
            savedObj.setWorkLocation((WorkLocation) workLocations.get(0));
        }

        savedObj.setID(SP_I_POOL(savedObj, messageContext));
        rpmObject.resetParentModified();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        Resource resource = (Resource) rpmObject;
        ResourceScope scope = (ResourceScope) rpmScope;

        // Move the Resource into a new pool

        if (scope != null && scope.getParent() != null
                && resource.testParentModified())
        {
            SP_M_POOL(resource, messageContext);
        }

        //Employee Identification

        if ((resource.getUserName() != null)
                && (resource.testUserNameModified() || resource
                    .testLogonReminderModified()))
        {
            SP_U_RES_LOGON(resource, messageContext);
        }

        if (resource.testGivenNameModified()
                || resource.testMiddleNameModified()
                || resource.testFamilyNameModified()
                || resource.testEmailAddressModified()
                || resource.testNickNameModified()
                || resource.testExternalIDModified())
        {

            SP_U_RES_NAME(resource, messageContext);
        }

        if (resource.testFullNameModified())
        {
            SP_U_RES_FULLNAME(resource, messageContext);
        }

        if (resource.testPasswordModified())
        {
            SP_U_RES_PASSWORD(resource, messageContext);
        }

        if (resource.testEffectiveStartDateModified()
                || resource.testDiscontinueDateModified())
        {
            SP_U_COMPANY_SPEC(resource, messageContext);
        }

        if (resource.testActiveModified())
        {
            SP_U_RES_ACTIVE(resource, messageContext);
        }

        // Calendar must be updated before weeklyMaxHours, otherwise the value of weeklyMaxHours
        // gets overridden by the calendar settings
        if (scope != null && scope.getCalendar() != null
                && resource.testCalendarModified())
        {
            SP_U_RES_CAL(resource, messageContext);
        }

        // Address && Employee Information
        if (resource.testHomePhoneModified()
                || resource.testWorkPhoneModified()
                || resource.testAlternativePhoneModified()
                || resource.testFaxModified() || resource.testMobileModified()
                || resource.testPagerModified()
                || resource.testWeeklyMaxHoursModified()
                || resource.testWeeklyMinHoursModified()
                || resource.testExperienceModified()
                || resource.testWorkLocationModified()
                || resource.testWorkingStatusModified()
                || resource.testEmployeeStatusModified())
        {

            saveEmployeeInformation(resource, scope, messageContext);

        }

        if (resource.testMyWebsiteModified())
        {
            SP_U_COMPANY_INFO(resource, messageContext);
        }

        if (resource.testTitleModified() || resource.testCountryCodeModified()
                || resource.testCompanyCodeModified()
                || resource.testDivisionCodeModified()
                || resource.testDepartmentCodeModified()
                || resource.testEmployeeCodeModified()
                || resource.testAddress1Modified()
                || resource.testAddress2Modified()
                || resource.testZipPostalCodeModified()
                || resource.testCustom1Modified()
                || resource.testCustom2Modified()
                || resource.testCustom3Modified()
                || resource.testCustom4Modified()
                || resource.testCustom5Modified())
        {

            SP_U_WORK_SPEC(resource, messageContext);
        }

        // Rates
        // Design note: rates.*_is_modified flags are NOT available to optimize update.
        saveResourceRatesAndCurrency(resource, scope, messageContext);

        //Verified by
        if (resource.testVerifiedByModified()
                || resource.testReVerifiedByModified())
        {
            SP_U_VERIFIED_BY(resource, messageContext);
        }

        //System Security Rights
        if (SecurityUtil.isAnyClearanceLevelModified(resource
            .getResourceSecurityRights()))
        {
            SP_U_RES_SECURITY(resource, messageContext);
        }

        //My Settings
        if (resource.testMyPortalURL1Modified()
                || resource.testMyPortalURL2Modified()
                || resource.testMyPortalURL3Modified()
                || resource.testMyPortalURL4Modified()
                || resource.testMyPortalURL5Modified()
                || resource.testDashboardModified()
                || resource.testOverrideOriginalDocumentOnCheckoutModified())
        {

            SP_U_RES_CAPTION(resource, messageContext);
        }

        //Interview (Background)
        if (resource.testMyAchievementsModified()
                || resource.testMyPersonalPassionsModified()
                || resource.testMySummaryOfSkillsModified()
                || resource.testWhatIDoReallyWellModified()
                || resource.testInterviewExperienceModified())
        {
            SP_U_EXPERIENCE(resource, messageContext);
        }

        if (scope != null)
        {
            saveResourceGeographicalAssignment(resource, rpmScope,
                messageContext);

            saveResourceOrganizationalAssignment(resource, rpmScope,
                messageContext);

            saveSecurityGroup(resource, scope, messageContext);

            saveExecutiveSecurityRole(resource, scope, messageContext);

            saveExecutiveAttributes(resource, rpmScope, messageContext);

            saveExecutiveAttributeClassifications(resource, rpmScope,
                messageContext);

            saveEducation(resource, rpmScope, messageContext);

            saveLanguageProficiencies(resource, rpmScope, messageContext);

            saveProjectHistory(resource, rpmScope, messageContext);

            saveIndustryExperience(resource, rpmScope, messageContext);

            saveEmploymentHistory(resource, rpmScope, messageContext);

            saveContactGroups(resource, rpmScope, messageContext);

            saveCompetencies(resource, rpmScope, messageContext);

            saveSkills(resource, rpmScope, messageContext);

            savePassportandLocationPreferences(resource, rpmScope,
                messageContext);

            saveDocumentFolder(resource, scope, messageContext);

            saveClientCostCenter(resource, rpmScope, messageContext);

            saveWorkPackages(resource, rpmScope, messageContext);

            savePersonalProject(resource, rpmScope, messageContext);

            propagateContextName(resource, resource.getCustomFieldAssignments());
            resource
                .setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
                    resource, resource.getCustomFieldAssignments(), scope
                        .isCustomFieldAssignments(), messageContext, null));
        }
        return resource;
    }

    private void saveSecurityGroup(Resource resource, ResourceScope scope,
            MessageContext messageContext) throws RPMException, SQLException
    {
        if (scope.getSecurityGroup() != null
                && resource.getSecurityGroup() != null
                && resource.testSecurityGroupModified())
        {
            SP_U_UG_ID(resource, messageContext);
        }
    }

    private void saveExecutiveSecurityRole(Resource resource,
            ResourceScope scope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        //TODO add in the validation only the possible types
        //treat case NONE
        if (scope.getExecutiveSecurityRole() != null
                && resource.testExecutiveSecurityRoleModified())
        {
            SP_U_EXECUTIVE_SECURITY(resource, messageContext);
        }
    }

    private Resource saveEmployeeInformation(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        //reload missing information

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendEqualQuestionMark(NAME_RESOURCE_ID);

        Object[] arguments = new Object[]
        {
            resource.getID()
        };
        
        if (scope == null || !scope.isWorkLocation())
        {
            Integer wlId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null, NAME_WORK_LOCATION, TABLE_NAME,
                stmt, arguments, messageContext);
            if (wlId != null)
            {
                WorkLocation wl = new WorkLocation();
                wl.setID(wlId.toString());
                resource.setWorkLocation(wl);
            }
            else
            {
                resource.setWorkLocation(null);
            }
        }

        if (scope == null || !scope.isWorkingStatus())
        {

            Integer wsId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null, NAME_STATUS1_ID, TABLE_NAME, stmt,
                arguments, messageContext);
            if (wsId != null)
            {
                WorkingStatus ws = new WorkingStatus();
                ws.setID(wsId.toString());
                resource.setWorkingStatus(ws);
            }
            else
            {
                resource.setWorkingStatus(null);
            }

        }
        if (scope == null || !scope.isEmployeeStatus())
        {
            Integer esId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null, NAME_STATUS2_ID, TABLE_NAME, stmt,
                arguments, messageContext);
            if (esId != null)
            {
                EmployeeStatus es = new EmployeeStatus();
                es.setID(esId.toString());
                resource.setEmployeeStatus(es);
            }
            else
            {
                resource.setEmployeeStatus(null);
            }
        }

        if (scope == null || !scope.isExperience())
        {
            Integer exId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null, NAME_EXPERIENCE_ID, TABLE_NAME,
                stmt, arguments, messageContext);
            if (exId != null)
            {
                Experience ex = new Experience();
                ex.setID(exId.toString());
                resource.setExperience(ex);
            }
            else
            {
                resource.setExperience(null);
            }
        }

        SP_U_EMPLOYEE_SPEC(resource, messageContext);

        return resource;
    }

    private Resource saveResourceRatesAndCurrency(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        if (scope != null && (scope.isRates() || scope.isCurrency()))
        {
            String resourceId = resource.getID();
            String currencyIdStr = ManagerUtil.getId(resource.getCurrency());
            // reload missing currency information 
            // or if currency is not in scope, reload the previous currency in order to not overwrite it
            if (currencyIdStr == null || !scope.isCurrency())
            {
                SqlBuffer condition = new SqlBuffer();
                condition.appendEqualQuestionMark(NAME_RESOURCE_ID);

                Object[] arguments = new Object[]
                {
                    resourceId
                };
                
                Object currencyIdObj = ManagerUtil.selectColumnValue(
                    String.class, resource, "currency", NAME_CURRENCY_ID,
                    TABLE_NAME, condition, arguments, messageContext);
                currencyIdStr = currencyIdObj.toString();
            }
            Integer currencyId = Integer.valueOf(currencyIdStr);

            ResourceRates rates = null;

            if (scope.isRates())
            {
                rates = resource.getRates();
            }

            if (rates == null)
            {
                ResourceRatesManager manager = (ResourceRatesManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        ResourceRates.class.getName());

                rates = new ResourceRates();
                rates.setID(getRateID(resource, messageContext));
                rates = (ResourceRates) manager.loadByPrimaryKey(rates, null,
                    messageContext, false);
            }

            Double regularCost = null;
            Double regularSelling = null;
            Double specialCost = null;
            Double specialSelling = null;

            if (rates != null)
            {
                regularCost = rates.getRegularCostPerHour();
                regularSelling = rates.getSellingRatePerHour();
                specialCost = rates.getSpecialCostPerHour();
                specialSelling = rates.getSpecialSellingRatePerHour();
            }

            if ((scope.isRates() && rates != null)
                    || (scope.isCurrency() && resource.testCurrencyModified()))
            {
                SP_U_RES_RATES(resourceId, currencyId, regularCost,
                    regularSelling, specialCost, specialSelling, messageContext);
            }
        }

        return resource;
    }

    private String getRateID(Resource resource, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(NAME_RESOURCE_ID);
        
        Object[] arguments = new Object[]
        {
            resource.getID()
        };
        
        String rateID = (String) ManagerUtil.selectColumnValue(String.class,
            resource, "rates", NAME_RATE_ID, TABLE_NAME, condition, arguments,
            messageContext);
        return rateID;
    }

    private Resource savePassportandLocationPreferences(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        ResourceScope scope = (ResourceScope) rpmScope;

        /*
         * resource.testPassportModified() is not used because it will be false
         * if only visas were modified
         */
        /*
         * resource.testLocationPreferencesModified() is not used because it
         * will be false if only RelocateLocations and TravelLocations were
         * modified
         */
        if (((scope.isPassport()) && (resource.getPassport() != null))
                || ((scope.isLocationPreferences()) && (resource
                    .getLocationPreferences() != null)))
        {

            //reload missing information
            if (!scope.isPassport() || !scope.isLocationPreferences())
            {

                //Reload missing information
                ResourceScope tempScope = new ResourceScope();
                tempScope.setPassport(true);
                tempScope.setLocationPreferences(true);

                Resource tempRes = new Resource();
                tempRes.setID(resource.getID());

                //remove from cache before reloading it
                messageContext.getCache().removeContainerFromCache(tempRes);

                tempRes = (Resource) loadByPrimaryKey(tempRes, tempScope,
                    messageContext, false);

                if (!scope.isPassport())
                {
                    resource.setPassport(tempRes.getPassport());
                    if (resource.getPassport() != null)
                    {
                        resource.getPassport().setParent(resource);
                    }
                }

                if (!scope.isLocationPreferences())
                {
                    resource.setLocationPreferences(tempRes
                        .getLocationPreferences());
                }
            }

            SP_U_RESLOCPREF(resource, messageContext);
        }
        return resource;
    }

    private Resource saveEducation(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        Education[] children = resource.getEducations();
        children = (Education[]) updateChildren(resource, children, scope
            .isEducations(), messageContext, null);
        resource.setEducations(children);
        return resource;
    }

    private Resource saveLanguageProficiencies(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setLanguageProficiencies((LanguageProficiency[]) updateChildren(
                resource, resource.getLanguageProficiencies(), scope
                    .isLanguageProficiencies(), messageContext, null));
        return resource;
    }

    private Resource saveProjectHistory(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource.setProjectHistories((ProjectHistory[]) updateChildren(
            resource, resource.getProjectHistories(), scope
                .isProjectHistories(), messageContext, null));
        return resource;
    }

    private Resource saveIndustryExperience(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource.setIndustryExperiences((IndustryExperience[]) updateChildren(
            resource, resource.getIndustryExperiences(), scope
                .isIndustryExperiences(), messageContext, null));
        return resource;
    }

    private Resource saveEmploymentHistory(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource.setEmploymentHistories((EmploymentHistory[]) updateChildren(
            resource, resource.getEmploymentHistories(), scope
                .isEmploymentHistories(), messageContext, null));
        return resource;
    }

    private Resource saveContactGroups(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setContactGroupAssignments((ContactGroupAssignment[]) updateChildren(
                resource, resource.getContactGroupAssignments(), scope
                    .isContactGroupAssignments(), messageContext, null));
        return resource;
    }

    private Resource saveCompetencies(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setCompetencyAttributeAssignments((ResourceAttributeAssignment[]) updateChildren(
                resource, resource.getCompetencyAttributeAssignments(), scope
                    .isCompetencyAttributeAssignments(), messageContext, null));
        return resource;
    }

    private Resource saveExecutiveAttributes(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource.setExecutiveAttributes((AttributeAssignment[]) updateChildren(
            resource, resource.getExecutiveAttributes(), scope
                .isExecutiveAttributes(), messageContext, null));
        return resource;
    }

    private Resource saveExecutiveAttributeClassifications(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setExecutiveAttributeClassifications((AttributeAssignment[]) updateChildren(
                resource, resource.getExecutiveAttributeClassifications(),
                scope.isExecutiveAttributeClassifications(), messageContext,
                null));
        return resource;
    }

    private Resource saveResourceGeographicalAssignment(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setGeographicalAssignment((GeographicalAssignment) updateChild(
                resource, resource.getGeographicalAssignment(), scope
                    .isGeographicalAssignment(), messageContext));
        return resource;
    }

    private Resource saveResourceOrganizationalAssignment(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setOrganizationalAssignment((OrganizationalAssignment) updateChild(
                resource, resource.getOrganizationalAssignment(), scope
                    .isOrganizationalAssignment(), messageContext));
        return resource;
    }

    private Resource saveSkills(Resource resource, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setSkillAttributeAssignments((ResourceAttributeAssignment[]) updateChildren(
                resource, resource.getSkillAttributeAssignments(), scope
                    .isSkillAttributeAssignments(), messageContext, null));
        return resource;

    }

    private Resource saveClientCostCenter(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceScope scope = (ResourceScope) rpmScope;
        resource
            .setResourceCostCenterAssignments((ResourceCostCenterAssignment[]) updateChildren(
                resource, resource.getResourceCostCenterAssignments(), scope
                    .isResourceCostCenterAssignments(), messageContext, null));
        return resource;
    }

    private Resource saveWorkPackages(Resource resource,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        //TODO Implements method ResourceManager.saveWorkPackages()
        //TODO Add getWorkPackages() to ResourceScope

        return resource;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        Resource res = null;
        if (rpmObject != null)
        {
            res = (Resource) rpmObject;
            SP_D_POOL(res, messageContext);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {

            PoolManager poolManager = (PoolManager) manager;
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(ResourceManager.NAME_RESOURCE_ID);
            where.append(" IN ");
            where.append("(SELECT ");
            where.append(PoolManager.NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(",");
            where.append(poolManager.getTableName());
            where.append(" WHERE ");
            where.append(PoolManager.NAME_TYPE_ID);
            where.append(" = ");
            where.append(Resource.TYPE_ID);
            where.append(" AND ");
            where.append(PoolManager.NAME_PARENT_ID);
            where.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                where.append(joinCondition.getCondition());
                where.append(" AND ");
                where.append(extraWhere);
            }

            joinCondition.setCondition(where.toString());

            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof WorkflowRoleMappingManager)
        {
            // for loading WorkflowRoleMapping.resource
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_RESOURCE_ID,
                WorkflowRoleMappingManager.NAME_RESOURCE_ID,
                WorkflowRoleMappingManager.TABLE_NAME,
                WorkflowRoleMappingManager.NAME_PROJECT_ID + "=? AND "
                        + WorkflowRoleMappingManager.NAME_WKF_ROLE_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
//        else if (manager instanceof PublishedScorecardManager)
//        {
//            joinCondition = new JoinCondition();
//
//            StringBuffer where = new StringBuffer();
//            where.append(PublishedScorecardManager.NAME_TYPE_ID);
//            where.append(" = ").append(Scorecard.TYPE_ID);
//            where.append(" AND ");
//            where.append(PublishedScorecardManager.NAME_CREATED_BY);
//            where.append(" IN (SELECT ");
//            where.append(NAME_REC_USER);
//            where.append(" FROM ");
//            where.append(getTableName());
//            if (joinCondition != null)
//            {
//                where.append(" WHERE " + joinCondition);
//            }
//            joinCondition.setCondition(where.toString());
//        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    private void setClearanceLevel(ResultSet rs, Resource resource)
            throws SQLException, RPMException
    {

        ResourceSecurityRights resourceSecurityRights = new ResourceSecurityRights();
        resource.setResourceSecurityRights(resourceSecurityRights);
        resourceSecurityRights.setParent(resource);

        // Set the clearance levels in the security group object
        // from the values in the result set
        int clearanceLevel1 = rs.getInt(ID_CLEARANCE_LEVEL_1);
        int clearanceLevel2 = rs.getInt(ID_CLEARANCE_LEVEL_2);
        int clearanceLevel3 = rs.getInt(ID_CLEARANCE_LEVEL_3);
        int clearanceLevel4 = rs.getInt(ID_CLEARANCE_LEVEL_4);
        int clearanceLevel5 = rs.getInt(ID_CLEARANCE_LEVEL_5);
        int clearanceLevel6 = rs.getInt(ID_CLEARANCE_LEVEL_6);
        int clearanceLevel7 = rs.getInt(ID_CLEARANCE_LEVEL_7);

        try
        {
            SecurityUtil.setClearanceLevels(new int[]
            {
                    clearanceLevel1, clearanceLevel2, clearanceLevel3,
                    clearanceLevel4, clearanceLevel5, clearanceLevel6,
                    clearanceLevel7
            }, resourceSecurityRights);
        }
        catch (Exception e)
        {
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, null, e);
        }
    }

    private void deltaClearanceLevel(ResultSet rs, Resource resource)
            throws SQLException, RPMException
    {
        ResourceSecurityRights resourceSecurityRights = resource
            .getResourceSecurityRights();
        
        if (resourceSecurityRights != null)
        {
            // Set the clearance levels in the security group object
            // from the values in the result set
            int clearanceLevel1 = rs.getInt(ID_CLEARANCE_LEVEL_1);
            int clearanceLevel2 = rs.getInt(ID_CLEARANCE_LEVEL_2);
            int clearanceLevel3 = rs.getInt(ID_CLEARANCE_LEVEL_3);
            int clearanceLevel4 = rs.getInt(ID_CLEARANCE_LEVEL_4);
            int clearanceLevel5 = rs.getInt(ID_CLEARANCE_LEVEL_5);
            int clearanceLevel6 = rs.getInt(ID_CLEARANCE_LEVEL_6);
            int clearanceLevel7 = rs.getInt(ID_CLEARANCE_LEVEL_7);

            // Map clearance levels to "can*" attributes
            // The commented lines relates to bits that are not delta/redelta with
            // the
            // RPM Client or that not currently used.

            SecurityUtil.deltaClearanceLevels(new int[]
            {
                    clearanceLevel1, clearanceLevel2, clearanceLevel3,
                    clearanceLevel4, clearanceLevel5, clearanceLevel6,
                    clearanceLevel7
            }, resourceSecurityRights);
        }
    }

    //    SC_SQL_API SP_U_RES_PASSWORD
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_password,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_RES_PASSWORD(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_U_RES_PASSWORD";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = res.getPassword();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_RES_ACTIVE(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_U_RES_ACTIVE";
        Object args[] = new Object[3];
        args[0] = res.getID();
        if (res.getActive() != null && res.getActive().booleanValue())
        {
            args[1] = "1";
        }
        else
        {
            args[1] = "0";
        }
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_UG_ID(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_UG_ID";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = res.getSecurityGroup().getID();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_EXECUTIVE_SECURITY(Resource res,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_EXECUTIVE_SECURITY";
        Object args[] = new Object[3];
        args[0] = res.getID();
        if (res.getExecutiveSecurityRole() != null)
        {
            args[1] = res.getExecutiveSecurityRole().getID();
        }
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_RES_CAL
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_calendar_id,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_RES_CAL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RES_CAL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        if (res.getCalendar() != null)
        {
            args[1] = res.getCalendar().getID();
        }
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_WCHAR *p_resource_id,
    //    SC_WCHAR *p_portal_page,
    //    SC_WCHAR *p_dashboard,
    //    SC_WCHAR *p_portal_url1,
    //    SC_WCHAR *p_portal_url2,
    //    SC_WCHAR *p_portal_url3,
    //    SC_WCHAR *p_portal_url4,
    //    SC_WCHAR *p_portal_url5,
    //    SC_WCHAR *p_overwritefile,
    //    SC_WCHAR *p_rec_user,
    //    SC_LONG_PARAM *nullIndicator
    private void SP_U_RES_CAPTION(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_U_RES_CAPTION";
        Object args[] = new Object[11];
        args[0] = res.getID();

        //TODO SP_U_RES_CAPTION (to what portal_page should map?)
        //args[1] = res.get
        args[2] = res.getDashboard();

        args[3] = res.getMyPortalURL1();
        args[4] = res.getMyPortalURL2();
        args[5] = res.getMyPortalURL3();
        args[6] = res.getMyPortalURL4();
        args[7] = res.getMyPortalURL5();

        if (res.getOverrideOriginalDocumentOnCheckout() != null
                && res.getOverrideOriginalDocumentOnCheckout().booleanValue())
        {
            args[8] = "Y";
        }
        else
        {
            args[8] = "N";
        }
        
        // TODO args[9] is for navbar_access_control flag
        // In the Client it matches the following two flags in the resource settings
        // - Can Access Documents from Left Navigator and All Documents in My Portal
        // - Can Access Scope Management from Left Navigator All Scope Elements in My Portal
        // those settings can be found in Resource Management - Application/My Settings Portlet
        // The value needs to be adjusted later.
        args[9] = new Integer(3);
        
        args[10] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_RES_FULLNAME
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_full_name,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_RES_FULLNAME(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RES_FULLNAME";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = res.getFullName();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_RES_NAME
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_first_name,
    //        SC_WCHAR *p_middle_name,
    //        SC_WCHAR *p_last_name,
    //        SC_WCHAR *p_nick_name,
    //        SC_WCHAR *p_email_address,
    //        SC_WCHAR *p_alias,
    //        SC_WCHAR *p_sin_number,
    //        SC_WCHAR *p_external_id,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_RES_NAME(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RES_NAME";
        Object args[] = new Object[10];
        args[0] = res.getID();
        args[1] = res.getGivenName();
        args[2] = res.getMiddleName();
        args[3] = res.getFamilyName();
        args[4] = res.getNickName();
        args[5] = res.getEmailAddress();
        args[6] = res.getFullName(); //Alias and fullname are the same
        //TODO SP_U_RES_NAME (to what sin_number should map?)
        //        args[7] = res.get
        args[8] = res.getExternalID();
        args[9] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_EXPERIENCE
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_experience,
    //        SC_WCHAR *p_achivements,
    //        SC_WCHAR *p_passions,
    //        SC_WCHAR *p_summarySkills,
    //        SC_WCHAR *p_miscellaneous,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_EXPERIENCE(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_U_EXPERIENCE";
        Object args[] = new Object[7];
        args[0] = res.getID();
        args[1] = res.getInterviewExperience();
        args[2] = res.getMyAchievements();
        args[3] = res.getMyPersonalPassions();
        args[4] = res.getMySummaryOfSkills();
        args[5] = res.getWhatIDoReallyWell();
        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_RES_RATES(String resourceId, Integer currencyId,
            Double regularCost, Double regularSelling, Double specialCost,
            Double specialSelling, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[7];
        args[0] = resourceId;
        args[1] = currencyId;
        args[2] = ManagerUtil.nullToZero(regularCost);
        args[3] = ManagerUtil.nullToZero(regularSelling);
        args[4] = ManagerUtil.nullToZero(specialCost);
        args[5] = ManagerUtil.nullToZero(specialSelling);
        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_RES_RATES", args);
    }

    //    SC_SQL_API SP_U_COMPANY_SPEC
    //    (
    //         SC_WCHAR *p_resource_id,
    //         SC_WCHAR *p_exec_level,
    //         SC_WCHAR *p_effective_date,
    //         SC_WCHAR *p_discontinue_date,
    //         SC_WCHAR *p_rec_user,
    //         SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_COMPANY_SPEC(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //SP_U_COMPANY_SPEC('140986558B644FE0BB6338F4B239F20A',null,'2005-05-25','2005-05-26','PMO_SUPERVISOR')
        String name = "SP_U_COMPANY_SPEC";
        Object args[] = new Object[5];
        args[0] = res.getID();
        //TODO SP_U_COMPANY_SPEC (to what exec_level should map?)
        args[1] = null;
        args[2] = DateUtil.convertToString(res.getEffectiveStartDate());
        args[3] = DateUtil.convertToString(res.getDiscontinueDate());
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_COMPANY_INFO
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_website,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_COMPANY_INFO(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_COMPANY_INFO";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = res.getMyWebsite();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_RES_LOGON
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_logon,
    //        SC_WCHAR *p_reminder,
    //        SC_WCHAR *p_multiple_logins,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_RES_LOGON(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RES_LOGON";
        Object args[] = new Object[5];
        args[0] = res.getID();
        args[1] = res.getUserName();
        args[2] = res.getLogonReminder();
        //TODO SP_U_RES_LOGON (to what multiple_logins should map?)
        //        args[3] = res.get
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CKIN_POOL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKIN_POOL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CKOUT_POOL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKOUT_POOL";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_UCKOUT_RESOURCE(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_UCKOUT_RESOURCE";
        Object args[] = new Object[3];
        args[0] = res.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_RESLOCPREF
    //    (
    //    SC_WCHAR *p_resource_id,
    //    SC_WCHAR *p_reloc,
    //    SC_WCHAR *p_reloc_duration,
    //    SC_WCHAR *p_reloc1,
    //    SC_WCHAR *p_reloc2,
    //    SC_WCHAR *p_reloc3,
    //            SC_WCHAR *p_travel,
    //    SC_WCHAR *p_duration,
    //    SC_WCHAR *p_travel_dest1,
    //    SC_WCHAR *p_travel_dest2,
    //            SC_WCHAR *p_travel_dest3,
    //    SC_WCHAR *p_passport,
    //    SC_WCHAR *p_passport_exp,
    //    SC_WCHAR *p_visa1,
    //            SC_WCHAR *p_visa_exp1,
    //    SC_WCHAR *p_visa2,
    //    SC_WCHAR *p_visa_exp2,
    //    SC_WCHAR *p_visa3,
    //    SC_WCHAR *p_visa_exp3,
    //            SC_WCHAR *p_rec_user,
    //    SC_LONG_PARAM *nullIndicator
    //    )

    private void SP_U_RESLOCPREF(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_U_RESLOCPREF";
        Object args[] = new Object[20];
        args[0] = res.getID();

        //Location Preferences
        if (res.getLocationPreferences() != null)
        {

            if (res.getLocationPreferences().getWillingtoRelocate() != null
                    && res.getLocationPreferences().getWillingtoRelocate()
                        .booleanValue())
                args[1] = "1";
            else
                args[1] = "0";
            if (res.getLocationPreferences().getAcceptableRelocateDuration() != null)
            {
                args[2] = res.getLocationPreferences()
                    .getAcceptableRelocateDuration().toString();
            }

            if (res.getLocationPreferences().getRelocateLocations() != null)
            {
                if (res.getLocationPreferences().getRelocateLocations().length >= 1
                        && res.getLocationPreferences().getRelocateLocations()[0] != null)
                {
                    args[3] = res.getLocationPreferences()
                        .getRelocateLocations()[0].getID();
                }
                if (res.getLocationPreferences().getRelocateLocations().length >= 2
                        && res.getLocationPreferences().getRelocateLocations()[1] != null)
                {
                    args[4] = res.getLocationPreferences()
                        .getRelocateLocations()[1].getID();
                }
                if (res.getLocationPreferences().getRelocateLocations().length == 3
                        && res.getLocationPreferences().getRelocateLocations()[2] != null)
                {
                    args[5] = res.getLocationPreferences()
                        .getRelocateLocations()[2].getID();
                }
            }

            if (res.getLocationPreferences().getWillingtoTravel()
                .booleanValue())
                args[6] = "1";
            else
                args[6] = "0";
            if (res.getLocationPreferences().getAcceptableTravelDuration() != null)
            {
                args[7] = res.getLocationPreferences()
                    .getAcceptableTravelDuration().toString();
            }

            if (res.getLocationPreferences().getTravelLocations() != null)
            {
                if (res.getLocationPreferences().getTravelLocations().length >= 1
                        && res.getLocationPreferences().getTravelLocations()[0] != null)
                {
                    args[8] = res.getLocationPreferences().getTravelLocations()[0]
                        .getID();
                }
                if (res.getLocationPreferences().getTravelLocations().length >= 2
                        && res.getLocationPreferences().getTravelLocations()[1] != null)
                {
                    args[9] = res.getLocationPreferences().getTravelLocations()[1]
                        .getID();
                }
                if (res.getLocationPreferences().getTravelLocations().length == 3
                        && res.getLocationPreferences().getTravelLocations()[2] != null)
                {
                    args[10] = res.getLocationPreferences()
                        .getTravelLocations()[2].getID();
                }
            }
        }
        else
        {
            args[1] = "0";
        }

        //Passport
        if (res.getPassport() != null)
        {

            if (res.getPassport().getHasValidPassport() != null
                    && res.getPassport().getHasValidPassport().booleanValue())
                args[11] = "1";
            else
                args[11] = "0";

            args[12] = DateUtil.convertToString(res.getPassport()
                .getExpiryDate());
            if (res.getPassport().getVisas() != null)
            {
                if (res.getPassport().getVisas().length >= 1
                        && res.getPassport().getVisas()[0] != null
                        && res.getPassport().getVisas()[0].getCountry() != null)
                {
                    args[13] = res.getPassport().getVisas()[0].getCountry()
                        .getID();
                    args[14] = DateUtil.convertToString(res.getPassport()
                        .getVisas()[0].getExpiryDate());
                }
                if (res.getPassport().getVisas().length >= 2
                        && res.getPassport().getVisas()[1] != null
                        && res.getPassport().getVisas()[1].getCountry() != null)
                {
                    args[15] = res.getPassport().getVisas()[1].getCountry()
                        .getID();
                    args[16] = DateUtil.convertToString(res.getPassport()
                        .getVisas()[1].getExpiryDate());
                }
                if (res.getPassport().getVisas().length == 3
                        && res.getPassport().getVisas()[2] != null
                        && res.getPassport().getVisas()[2].getCountry() != null)
                {
                    args[17] = res.getPassport().getVisas()[2].getCountry()
                        .getID();
                    args[18] = DateUtil.convertToString(res.getPassport()
                        .getVisas()[2].getExpiryDate());
                }
            }
        }
        else
        {
            args[11] = "0";
        }
        args[19] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_VERIFIED_BY
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_verified_by1,
    //        SC_WCHAR *p_verified_by2,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_VERIFIED_BY(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_VERIFIED_BY";
        Object args[] = new Object[4];
        args[0] = res.getID();

        if (res.getVerifiedBy() != null)
        {
            args[1] = res.getVerifiedBy().getID();
        }
        if (res.getReVerifiedBy() != null)
        {
            args[2] = res.getReVerifiedBy().getID();
        }

        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_WORK_SPEC
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_role_id,
    //        SC_WCHAR *p_title,
    //        SC_WCHAR *p_address1,
    //        SC_WCHAR *p_address2,
    //        SC_WCHAR *p_zip,
    //        SC_LONG_PARAM *p_citizenship,
    //        SC_WCHAR *p_country_code,
    //        SC_WCHAR *p_company_code,
    //        SC_WCHAR *p_division_code,
    //        SC_WCHAR *p_department_code,
    //        SC_WCHAR *p_employee_code,
    //        SC_WCHAR *p_custom_01,
    //        SC_WCHAR *p_custom_02,
    //        SC_WCHAR *p_custom_03,
    //        SC_WCHAR *p_custom_04,
    //        SC_WCHAR *p_custom_05,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_WORK_SPEC(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //SP_U_WORK_SPEC('140986558B644FE0BB6338F4B239F20A',null,'toto','4522',null,null,2726,null,null,null,null,null,null,null,null,null,null,'PMO_SUPERVISOR')
        String name = "SP_U_WORK_SPEC";
        Object args[] = new Object[18];
        args[0] = res.getID();
        //TODO SP_U_WORK_SPEC (to what role_id should map?)
        //                args[1] = res.get
        args[2] = res.getTitle();
        args[3] = res.getAddress1();
        args[4] = res.getAddress2();
        args[5] = res.getZipPostalCode();
        //TODO SP_U_WORK_SPEC (to what citizenship should map?)
        //BK: qui sait?
        args[6] = new Integer(2726);
        args[7] = res.getCountryCode();
        args[8] = res.getCompanyCode();
        args[9] = res.getDivisionCode();
        args[10] = res.getDepartmentCode();
        args[11] = res.getEmployeeCode();
        args[12] = res.getCustom1();
        args[13] = res.getCustom2();
        args[14] = res.getCustom3();
        args[15] = res.getCustom4();
        args[16] = res.getCustom5();
        args[17] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SC_SQL_API SP_U_EMPLOYEE_SPEC
    //    (
    //        SC_WCHAR *p_resource_id,
    //        SC_WCHAR *p_home_office,
    //        SC_LONG_PARAM *p_work_loc_id,
    //        SC_WCHAR *p_work_phone,
    //        SC_WCHAR *p_alt_phone,
    //        SC_WCHAR *p_cell_phone,
    //        SC_LONG_PARAM *p_status1,
    //        SC_LONG_PARAM *p_status2,
    //        SC_LONG_PARAM *p_experience,
    //        SC_LONG_PARAM *p_maxdayhours,
    //        SC_LONG_PARAM *p_mindayhours,
    //        double *p_percent_assign,
    //        SC_WCHAR *p_icq_id,
    //        SC_WCHAR *p_pager_num,
    //        SC_WCHAR *p_fax_num,
    //        SC_WCHAR *p_rec_user,
    //        SC_LONG_PARAM *nullIndicator
    //    )
    private void SP_U_EMPLOYEE_SPEC(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //SP_U_EMPLOYEE_SPEC('140986558B644FE0BB6338F4B239F20A','779',265,'779',null,null,312,318,2771,40,0,0e0,null,null,null,'PMO_SUPERVISOR')
        String name = "SP_U_EMPLOYEE_SPEC";
        Object args[] = new Object[16];
        args[0] = res.getID();
        args[1] = res.getHomePhone();

        if (res.getWorkLocation() != null)
        {
            args[2] = new Integer(res.getWorkLocation().getID());
        }

        args[3] = res.getWorkPhone();
        args[4] = res.getAlternativePhone();
        args[5] = res.getMobile();

        if (res.getWorkingStatus() != null)
        {
            args[6] = new Integer(res.getWorkingStatus().getID());
        }

        if (res.getEmployeeStatus() != null)
        {
            args[7] = new Integer(res.getEmployeeStatus().getID());
        }

        if (res.getExperience() != null)
        {
            args[8] = new Integer(res.getExperience().getID());
        }

        args[9] = res.getWeeklyMaxHours();
        args[10] = res.getWeeklyMinHours();
        args[11] = null; //not used
        args[12] = null; //not used
        args[13] = res.getPager();
        args[14] = res.getFax();
        args[15] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private String SP_I_POOL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_I_POOL";
        Object args[] = new Object[4];
        args[0] = res.getFullName();
        args[1] = res.getParent().getID();
        args[2] = res.retrieveTypeID();
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private String SP_M_POOL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_M_POOL";
        Object args[] = new Object[4];
        args[0] = res.getParent().getID();
        args[1] = res.getID();
        args[2] = new Integer(4);
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    private void SP_D_POOL(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        if (res.getID() == null)
            return;

        String name = "SP_D_POOL";
        Object args[] = new Object[4];
        args[0] = res.getID();
        args[1] = ManagerUtil.getParentId(this, res, messageContext,
            PoolManager.TABLE_NAME);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    /*
     * CALL
     * SP_U_RES_SECURITY('2D8FBEC3CFE341CE8B3806E845D70EFE',2084554750,2105540607,1023148023,2147483391,520192,'PMO_SUPERVISOR')
     * Used to change the security of a Resource. SP_U_RES_REVISION must be
     * called before SP_U_RES_SECURITY. Make sure the Resource is checked out
     * before calling SP_U_RES_REVISION and SP_U_RES_SECURITY.
     */
    private void SP_U_RES_SECURITY(Resource res, MessageContext messageContext)
            throws SQLException, RPMException
    {
        int[] clearanceLevels = SecurityUtil.getClearanceLevels(res
            .getResourceSecurityRights(),
            new int[SecurityUtil.SECURITY_GROUP_LEVELS_COUNT]);

        String name = "SP_U_RES_SECURITY";
        Object args[] = new Object[9];
        args[0] = res.getID();
        args[1] = new Integer(clearanceLevels[0]);
        args[2] = new Integer(clearanceLevels[1]);
        args[3] = new Integer(clearanceLevels[2]);
        args[4] = new Integer(clearanceLevels[3]);
        args[5] = new Integer(clearanceLevels[4]);
        args[6] = new Integer(clearanceLevels[5]);
        args[7] = new Integer(clearanceLevels[6]);
        args[8] = getUser(messageContext).getID();

        //XXX TODO Yasser: This comment was copied from SecurityGroupManager: revisit this thing and figure out whay it has been disabled, maybe we have to enable it.        
        // There is a potential to incorrectly set the bit fields in this class
        // so we don't use it
        // SecurityRulesMaps.applyBusinessRules(args);

        executeProcedure(messageContext, name, args);
    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_CKIN_POOL((Resource) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }

    }

    /**
     *  
     */
    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_UCKOUT_RESOURCE((Resource) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    /**
     * @param rpmObject
     *  
     */
    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_CKOUT_POOL((Resource) rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }
}
