/*
 * Created on Apr 29, 2005
 *
 */

package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * @author Simon
 *
 */
public final class ResourceProjectHistoryManager extends
        AbstractRPMObjectManager
{

    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(ProjectHistory.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceProjectHistoryManager.class);
        }
    }

    /**
     * Column RES_PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RES_PROJECT_ID = 1;

    public static final int TYPE_RES_PROJECT_ID = Types.CHAR;

    public static final String NAME_RES_PROJECT_ID = "RESOURCE_PROJECT_HISTORY.RES_PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_RES_PROJECT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RES_PROJECT_ID, NAME_RES_PROJECT_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_PROJECT_HISTORY.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column CLIENT of type Types.CHAR mapped to String.
     */
    public static final int ID_CLIENT = 3;

    public static final int TYPE_CLIENT = Types.CHAR;

    public static final String NAME_CLIENT = "RESOURCE_PROJECT_HISTORY.CLIENT";

    public static final String PROPERTY_CLIENT = "CLIENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLIENT, NAME_CLIENT);
    }

    /**
     * Column CLASSIFICATION of type Types.CHAR mapped to String.
     */
    public static final int ID_CLASSIFICATION = 4;

    public static final int TYPE_CLASSIFICATION = Types.CHAR;

    public static final String NAME_CLASSIFICATION = "RESOURCE_PROJECT_HISTORY.CLASSIFICATION";

    public static final String PROPERTY_CLASSIFICATION = "CLASSIFICATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLASSIFICATION, NAME_CLASSIFICATION);
    }

    /**
     * Column INDUSTRY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INDUSTRY_ID = 5;

    public static final int TYPE_INDUSTRY_ID = Types.INTEGER;

    public static final String NAME_INDUSTRY_ID = "RESOURCE_PROJECT_HISTORY.INDUSTRY_ID";

    public static final String PROPERTY_INDUSTRY_ID = "INDUSTRY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_INDUSTRY_ID, NAME_INDUSTRY_ID);
    }

    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 6;

    public static final int TYPE_START_DATE = Types.DATE;

    public static final String NAME_START_DATE = "RESOURCE_PROJECT_HISTORY.START_DATE";

    public static final String PROPERTY_START_DATE = "STARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_START_DATE, NAME_START_DATE);
    }

    /**
     * Column END_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_END_DATE = 7;

    public static final int TYPE_END_DATE = Types.DATE;

    public static final String NAME_END_DATE = "RESOURCE_PROJECT_HISTORY.END_DATE";

    public static final String PROPERTY_END_DATE = "ENDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_END_DATE, NAME_END_DATE);
    }

    /**
     * Column PROJECT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROJECT_NAME = 8;

    public static final int TYPE_PROJECT_NAME = Types.VARCHAR;

    public static final String NAME_PROJECT_NAME = "RESOURCE_PROJECT_HISTORY.PROJECT_NAME";

    public static final String PROPERTY_PROJECT_NAME = "PROJECTNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_NAME, NAME_PROJECT_NAME);
    }

    /**
     * Column PROJECT_LENGTH of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROJECT_LENGTH = 9;

    public static final int TYPE_PROJECT_LENGTH = Types.VARCHAR;

    public static final String NAME_PROJECT_LENGTH = "RESOURCE_PROJECT_HISTORY.PROJECT_LENGTH";

    public static final String PROPERTY_PROJECT_LENGTH = "PROJECTLENGTH";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_LENGTH, NAME_PROJECT_LENGTH);
    }

    /**
     * Column TIME_SPENT of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIME_SPENT = 10;

    public static final int TYPE_TIME_SPENT = Types.VARCHAR;

    public static final String NAME_TIME_SPENT = "RESOURCE_PROJECT_HISTORY.TIME_SPENT";

    public static final String PROPERTY_TIME_SPENT = "TIMESPENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIME_SPENT, NAME_TIME_SPENT);
    }

    /**
     * Column ROLE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ROLE = 11;

    public static final int TYPE_ROLE = Types.VARCHAR;

    public static final String NAME_ROLE = "RESOURCE_PROJECT_HISTORY.ROLE";

    public static final String PROPERTY_ROLE = "ROLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ROLE, NAME_ROLE);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 12;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "RESOURCE_PROJECT_HISTORY.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column RESPONSIBILITIES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RESPONSIBILITIES = 13;

    public static final int TYPE_RESPONSIBILITIES = Types.VARCHAR;

    public static final String NAME_RESPONSIBILITIES = "RESOURCE_PROJECT_HISTORY.RESPONSIBILITIES";

    public static final String PROPERTY_RESPONSIBILITIES = "SKILLSRESPONSIBILITIES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESPONSIBILITIES, NAME_RESPONSIBILITIES);
    }

    private static final String TABLE_NAME = "RESOURCE_PROJECT_HISTORY";

    /**
     * Create an array of type string containing all the fields of the RESOURCE_PROJECT_HISTORY table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RES_PROJECT_ID, NAME_RESOURCE_ID, NAME_CLIENT,
            NAME_CLASSIFICATION, NAME_INDUSTRY_ID, NAME_START_DATE,
            NAME_END_DATE, NAME_PROJECT_NAME, NAME_PROJECT_LENGTH,
            NAME_TIME_SPENT, NAME_ROLE, NAME_DESCRIPTION, NAME_RESPONSIBILITIES
    };

    /**
     * Field that contains the comma separated fields of the RESOURCE_EMPLOYMENT_HISTORY table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new ResourceProjectHistoryManager instance.
     * @param Resource
     * @return ResourceProjectHistoryManager
     */
    public ResourceProjectHistoryManager()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_RES_PROJECT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(0);
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /**
     * @param containerName
     * @return
     */
    private String getFieldFilter(String containerName)
    {
        StringBuffer fieldFilter = new StringBuffer();

        if (containerName.equalsIgnoreCase("Classification"))
        {
            fieldFilter.append(" cast(");
            fieldFilter
                .append(ResourceProjectHistoryManager.NAME_CLASSIFICATION);
            fieldFilter.append(" AS INTEGER)");
        }
        else if (containerName.equalsIgnoreCase("IndustryType")
                || containerName.equalsIgnoreCase("ProjectHistory"))
        {
            fieldFilter.append(NAME_INDUSTRY_ID);
        }
        return fieldFilter.toString();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition(getTableName(), getFilter());

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" RESOURCE_ID ");
            condition.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setCondition(condition.toString());
        }
        else if (manager instanceof ResourceProjectHistoryManager)
        {
            joinCondition = new JoinCondition(getTableName(), getFilter());

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" RES_PROJECT_ID ");
            condition.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setCondition(condition.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
     */
    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ResourceManager
                || manager instanceof ResourceProjectHistoryManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(getFieldFilter(container.getPreviousType()));
            where.append(" FROM ");
            where.append(getTableName());

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new ProjectHistory();
        return object;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(RPMObject[], java.sql.ResultSet, com.ibm.rpm.scope.RPMObjectScope, boolean)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        ProjectHistory object = (ProjectHistory) rpmObject;

        //Related resource is always loaded without scope
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }

        DatafieldManager dtManager = (DatafieldManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Classification.class.getName());

        //Classification
        String classificationId = (String) relatedFields.get(index,
            ID_CLASSIFICATION);

        if (classificationId != null)
        {
            Classification classification = new Classification();
            classification.setID(classificationId);

            classification = (Classification) dtManager.loadByPrimaryKey(
                classification, null, messageContext, isDelta);

            if (isDelta)
            {
                object.deltaClassification(classification);
            }
            else
            {
                object.setClassification(classification);
            }
        }

        //IndustryType
        String industryTypeId = (String) relatedFields.get(index,
            ID_INDUSTRY_ID);

        if (industryTypeId != null)
        {
            IndustryType industry = new IndustryType();
            industry.setID(industryTypeId);

            industry = (IndustryType) dtManager.loadByPrimaryKey(industry, null,
                messageContext, isDelta);
            if (isDelta)
            {
                object.deltaIndustry(industry);
            }
            else
            {
                object.setIndustry(industry);
            }
        }

        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

        RPMObject pObject = rpmObject;
        Classification cla = new Classification();
        cla.setID(rs.getString(ID_CLASSIFICATION));
        IndustryType ind = new IndustryType();
        ind.setID(rs.getString(ID_INDUSTRY_ID));
        String client = rs.getString(ID_CLIENT);
        if (client != null)
        {
            client = client.trim();
        }

        pObject.setID(rs.getString(ID_RES_PROJECT_ID));
        if (isDelta)
        {
            ((ProjectHistory) pObject).deltaClient(client);
            ((ProjectHistory) pObject).deltaProjectName(rs
                .getString(ID_PROJECT_NAME));
            ((ProjectHistory) pObject).deltaDescription(rs
                .getString(ID_DESCRIPTION));
            ((ProjectHistory) pObject).deltaStartDate(DateUtil
                .convertToCalendar(rs.getDate(ID_START_DATE)));
            ((ProjectHistory) pObject).deltaEndDate(DateUtil
                .convertToCalendar(rs.getDate(ID_END_DATE)));
            ((ProjectHistory) pObject).deltaProjectLength(rs
                .getString(ID_PROJECT_LENGTH));
            ((ProjectHistory) pObject).deltaTimeSpent(rs
                .getString(ID_TIME_SPENT));
            ((ProjectHistory) pObject).deltaRole(rs.getString(ID_ROLE));
            ((ProjectHistory) pObject).deltaSkillsResponsibilities(rs
                .getString(ID_RESPONSIBILITIES));
        }
        else
        {
            ((ProjectHistory) pObject).setClient(client);
            ((ProjectHistory) pObject).setProjectName(rs
                .getString(ID_PROJECT_NAME));
            ((ProjectHistory) pObject).setDescription(rs
                .getString(ID_DESCRIPTION));
            ((ProjectHistory) pObject).setStartDate(DateUtil
                .convertToCalendar(rs.getDate(ID_START_DATE)));
            ((ProjectHistory) pObject).setEndDate(DateUtil.convertToCalendar(rs
                .getDate(ID_END_DATE)));
            ((ProjectHistory) pObject).setProjectLength(rs
                .getString(ID_PROJECT_LENGTH));
            ((ProjectHistory) pObject)
                .setTimeSpent(rs.getString(ID_TIME_SPENT));
            ((ProjectHistory) pObject).setRole(rs.getString(ID_ROLE));
            ((ProjectHistory) pObject).setSkillsResponsibilities(rs
                .getString(ID_RESPONSIBILITIES));
        }

        return pObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);

        String classificationId = resultSet.getString(ID_CLASSIFICATION);
        classificationId = (classificationId == null ? null : (classificationId
            .trim().equals("") ? null : classificationId.trim()));
        relatedFields.put(index, ID_CLASSIFICATION, classificationId);

        String industryTypeId = resultSet.getString(ID_INDUSTRY_ID);
        relatedFields.put(index, ID_INDUSTRY_ID, industryTypeId);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        ProjectHistory pObject = (ProjectHistory) rpmObject;

        //  beforeInsert(pObject); // listener callback

        pObject.setID(SP_I_PROJHIST(pObject, messageContext));
        //pObject.resetIsModified();

    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ProjectHistory pObject = (ProjectHistory) rpmObject;

        if (pObject.testClientModified()
                || pObject.testClassificationModified()
                || pObject.testIndustryModified()
                || pObject.testDescriptionModified()
                || pObject.testEndDateModified()
                || pObject.testParentModified()
                || pObject.testProjectLengthModified()
                || pObject.testProjectNameModified()
                || pObject.testRoleModified()
                || pObject.testSkillsResponsibilitiesModified()
                || pObject.testStartDateModified()
                || pObject.testTimeSpentModified())
        {
            SP_U_PROJHIST(pObject, messageContext);
        }

        return pObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {

        if (rpmObject != null)
        {
            SP_D_PROJHIST((ProjectHistory) rpmObject, messageContext);
        }
    }

    private String SP_I_PROJHIST(ProjectHistory projecthistory,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_PROJHIST";
        Object args[] = new Object[4];
        args[0] = null;
        if (projecthistory.getParent() != null)
        {
            args[1] = projecthistory.getParent().getID();
        }
        args[2] = new Integer(99);
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_PROJHIST(ProjectHistory projecthistory,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_PROJHIST";
        Object args[] = new Object[13];
        args[0] = projecthistory.getID();
        args[1] = projecthistory.getClient();
        if (projecthistory.getClassification() != null)
            args[2] = projecthistory.getClassification().getID();
        else
            args[2] = null;

        if (projecthistory.getIndustry() != null)
        {
            args[3] = new Integer(projecthistory.getIndustry().getID());
        }

        args[4] = projecthistory.getProjectName();
        
        java.sql.Date startDate = null;
        if (projecthistory.getStartDate() != null)
        {
            startDate = new java.sql.Date(projecthistory.getStartDate()
               .getTimeInMillis());
            args[5] = startDate.toString();
        }
        
        java.sql.Date endDate = null;
        if (projecthistory.getEndDate() != null)
        {
            endDate = new java.sql.Date(projecthistory.getEndDate().getTimeInMillis());
            args[6] = endDate.toString();  
        }

        args[7] = projecthistory.getProjectLength();
        args[8] = projecthistory.getTimeSpent();
        args[9] = projecthistory.getRole();
        args[10] = projecthistory.getDescription();
        args[11] = projecthistory.getSkillsResponsibilities();
        if (getUser(messageContext) != null)
        {
            args[12] = getUser(messageContext).getID();
        }

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_PROJHIST(ProjectHistory projecthistory,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_PROJHIST";
        Object args[] = new Object[4];
        args[0] = projecthistory.getID();
        args[1] = ManagerUtil.getParentId(this, projecthistory, messageContext,
            TABLE_NAME, NAME_RES_PROJECT_ID, NAME_RESOURCE_ID);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
}
