package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.resource.containers.ResourceRates;


public class ResourceRatesManager extends AbstractRPMObjectManager{

    private static Log logger = LogFactory.getLog(ResourceRatesManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(ResourceRates.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceRatesManager.class);
        }
    }
    /**
     * Column RATE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RATE_ID = 1;
    public static final int TYPE_RATE_ID = Types.CHAR;
    public static final String NAME_RATE_ID = "RATES.RATE_ID";   
    public static final String PROPERTY_RATE_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_RATE_ID,NAME_RATE_ID);}

    /**
     * Column REGULAR_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_REGULAR_COST = 2;
    public static final int TYPE_REGULAR_COST = Types.DECIMAL;
    public static final String NAME_REGULAR_COST = "RATES.REGULAR_COST";
    public static final String PROPERTY_REGULAR_COST = "REGULARCOSTPERHOUR";
    static{ FIELDPROPERTYMAP.put(PROPERTY_REGULAR_COST,NAME_REGULAR_COST);}

    /**
     * Column REGULAR_SALE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_REGULAR_SALE = 3;
    public static final int TYPE_REGULAR_SALE = Types.DECIMAL;
    public static final String NAME_REGULAR_SALE = "RATES.REGULAR_SALE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REGULAR_SALE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REGULAR_SALE,NAME_REGULAR_SALE);}

    /**
     * Column SPECIAL_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_COST = 4;
    public static final int TYPE_SPECIAL_COST = Types.DECIMAL;
    public static final String NAME_SPECIAL_COST = "RATES.SPECIAL_COST";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SPECIAL_COST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_COST,NAME_SPECIAL_COST);}

    /**
     * Column SPECIAL_SALE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_SALE = 5;
    public static final int TYPE_SPECIAL_SALE = Types.DECIMAL;
    public static final String NAME_SPECIAL_SALE = "RATES.SPECIAL_SALE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SPECIAL_SALE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_SALE,NAME_SPECIAL_SALE);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 6;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "RATES.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 7;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "RATES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 8;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "RATES.ELEMENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    private static final String TABLE_NAME = "RATES";

    /**
     * Create an array of type string containing all the fields of the RATES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_RATE_ID
        ,NAME_REGULAR_COST
        ,NAME_REGULAR_SALE
        ,NAME_SPECIAL_COST
        ,NAME_SPECIAL_SALE
        ,NAME_REC_USER
        ,NAME_REC_DATETIME
        ,NAME_ELEMENT_ID
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /**
     * Creates a new ResourceRatesManager instance.
     * @return ResourceRatesManager
     */   
    public ResourceRatesManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() {
        return NAME_RATE_ID;
    }  

    protected Integer getContainerType(ResultSet resultSet) throws RPMException, SQLException {


    	return new Integer(0);
    }  

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }  

    protected RPMObject createContainer(int typeId) throws RPMException {

         return new ResourceRates();
     }
    /**
     *
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException {
        ExceptionUtil.handleMethodException(this, ErrorCodes.UNSUPPORTED_METHOD, ExceptionUtil.CREATE_OBJECT);
    }
    /**
     *
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException {

        ResourceRates rates = (ResourceRates)rpmObject;
        
        rates.setID(rs.getString(ID_RATE_ID));
        
        if(isDelta){
            rates.deltaRegularCostPerHour(new Double(rs.getDouble(ID_REGULAR_COST)));
            rates.deltaSellingRatePerHour(new Double(rs.getDouble(ID_REGULAR_SALE)));
            rates.deltaSpecialCostPerHour(new Double(rs.getDouble(ID_SPECIAL_COST)));
            rates.deltaSpecialSellingRatePerHour(new Double(rs.getDouble(ID_SPECIAL_SALE)));
        }else{
            rates.setRegularCostPerHour(new Double(rs.getDouble(ID_REGULAR_COST)));
            rates.setSellingRatePerHour(new Double(rs.getDouble(ID_REGULAR_SALE)));
            rates.setSpecialCostPerHour(new Double(rs.getDouble(ID_SPECIAL_COST)));
            rates.setSpecialSellingRatePerHour(new Double(rs.getDouble(ID_SPECIAL_SALE)));
        }
         
        return rpmObject;
    }
    /**
     *
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }
    /**
     *
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        
        
        return null;

    }
    /**
     *
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName) throws RPMException {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            
            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            	 //uncomment if additionnal condition is needed
                //where.append(" AND ");
             }
             //TODO Add additionnal condition if needed
             //where.append(condition on status or type...);

             joinCondition.setCondition(where.toString());
    
        }

        //TODO Implement for managers that handles the children of the supported containers

        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
    
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceAttributeManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(ResourceAttributeManager.NAME_RATE_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_RATE_ID);
            where.append(" FROM ");
            where.append(getTableName());

            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(ResourceManager.NAME_RATE_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_RATE_ID);
            where.append(" FROM ");
            where.append(getTableName());

            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        //TODO Implement method getSubTableJoinCondition if needed
        else
        {
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.SUB_TABLE_JOIN);
        }
        
        return joinCondition;
    }
            
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException {

        ExceptionUtil.handleMethodException(this, ErrorCodes.UNSUPPORTED_METHOD, ExceptionUtil.UPDATE_OBJECT);
        return null;
    }
}
