package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.managers.DocumentUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * @author kuschel
 */
public class ResourceRoleAssignmentManager extends AbstractRPMObjectManager
{

    private static Log logger = LogFactory.getLog(ResourceRoleAssignmentManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {

        CONTAINERS.add(ResourceRoleAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceRoleAssignmentManager.class);
        }
    }

    /**
     * Column RES_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RES_ASSIGNMENT_ID = 1;

    public static final int TYPE_RES_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_RES_ASSIGNMENT_ID = "RESOURCE_ASSIGNMENTS.RES_ASSIGNMENT_ID";

    public static final String PROPERTY_RES_ASSIGNMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RES_ASSIGNMENT_ID, NAME_RES_ASSIGNMENT_ID);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PROJECT_ID = 2;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "RESOURCE_ASSIGNMENTS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "RESOURCE_ASSIGNMENTS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_TYPE of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ELEMENT_TYPE = 4;

    public static final int TYPE_ELEMENT_TYPE = Types.INTEGER;

    public static final String NAME_ELEMENT_TYPE = "RESOURCE_ASSIGNMENTS.ELEMENT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_TYPE,NAME_ELEMENT_TYPE);}

    /**
     * Column PROFILE_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PROFILE_ID = 5;

    public static final int TYPE_PROFILE_ID = Types.CHAR;

    public static final String NAME_PROFILE_ID = "RESOURCE_ASSIGNMENTS.PROFILE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROFILE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROFILE_ID,NAME_PROFILE_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 6;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_ASSIGNMENTS.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column RESOURCE_TYPE of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_RESOURCE_TYPE = 7;

    public static final int TYPE_RESOURCE_TYPE = Types.CHAR;

    public static final String NAME_RESOURCE_TYPE = "RESOURCE_ASSIGNMENTS.RESOURCE_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_TYPE,NAME_RESOURCE_TYPE);}

    /**
     * Column WORK_LOCATION of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_WORK_LOCATION = 8;

    public static final int TYPE_WORK_LOCATION = Types.INTEGER;

    public static final String NAME_WORK_LOCATION = "RESOURCE_ASSIGNMENTS.WORK_LOCATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_LOCATION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_LOCATION,NAME_WORK_LOCATION);}

    /**
     * Column PARTICIPANT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PARTICIPANT = 9;

    public static final int TYPE_PARTICIPANT = Types.INTEGER;

    public static final String NAME_PARTICIPANT = "RESOURCE_ASSIGNMENTS.PARTICIPANT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARTICIPANT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARTICIPANT,NAME_PARTICIPANT);}

    /**
     * Column SECURITY_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_SECURITY_ID = 10;

    public static final int TYPE_SECURITY_ID = Types.CHAR;

    public static final String NAME_SECURITY_ID = "RESOURCE_ASSIGNMENTS.SECURITY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SECURITY_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SECURITY_ID,NAME_SECURITY_ID);}

    /**
     * Column ASSIGN_STATUS of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ASSIGN_STATUS = 11;

    public static final int TYPE_ASSIGN_STATUS = Types.INTEGER;

    public static final String NAME_ASSIGN_STATUS = "RESOURCE_ASSIGNMENTS.ASSIGN_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_STATUS,NAME_ASSIGN_STATUS);}

    /**
     * Column TIMECODE_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_TIMECODE_ID = 12;

    public static final int TYPE_TIMECODE_ID = Types.CHAR;

    public static final String NAME_TIMECODE_ID = "RESOURCE_ASSIGNMENTS.TIMECODE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_ID,NAME_TIMECODE_ID);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 13;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "RESOURCE_ASSIGNMENTS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_REC_USER = 14;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "RESOURCE_ASSIGNMENTS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 15;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "RESOURCE_ASSIGNMENTS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column TABLE_TYPE of type Types.SMALLINT mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_TABLE_TYPE = 16;

    public static final int TYPE_TABLE_TYPE = Types.SMALLINT;

    public static final String NAME_TABLE_TYPE = "RESOURCE_ASSIGNMENTS.TABLE_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TABLE_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TABLE_TYPE,NAME_TABLE_TYPE);}

    /**
     * Column CRI_SECURITY_ID of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CRI_SECURITY_ID = 17;

    public static final int TYPE_CRI_SECURITY_ID = Types.VARCHAR;

    public static final String NAME_CRI_SECURITY_ID = "RESOURCE_ASSIGNMENTS.CRI_SECURITY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CRI_SECURITY_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CRI_SECURITY_ID,NAME_CRI_SECURITY_ID);}

    public static final String TABLE_NAME = "RESOURCE_ASSIGNMENTS";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_ASSIGNMENTS table.
     */
    private static final String[] FIELD_NAMES =
    {
        NAME_RES_ASSIGNMENT_ID, NAME_PROJECT_ID, NAME_ELEMENT_ID, NAME_ELEMENT_TYPE,
        NAME_PROFILE_ID, NAME_RESOURCE_ID, NAME_RESOURCE_TYPE, NAME_WORK_LOCATION,
        NAME_PARTICIPANT, NAME_SECURITY_ID, NAME_ASSIGN_STATUS, NAME_TIMECODE_ID,
        NAME_REC_STATUS, NAME_REC_USER, NAME_REC_DATETIME, NAME_TABLE_TYPE,
        NAME_CRI_SECURITY_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public ResourceRoleAssignmentManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_RES_ASSIGNMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException
    {

        return new Integer(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else if (manager instanceof ResourceRoleAssignmentManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_TYPE);
            where.append(" = ");
            where.append(" 11 ");
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof PoolManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(PoolManager.NAME_ELEMENT_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

            StringBuffer tables = new StringBuffer();
            tables.append(getTableName());
            tables.append(",");
            tables.append(manager.getTableName());

            joinCondition.setTableName(tables.toString());
        }
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_RESOURCE_ID);
            where.append(" = ");
            where.append(ResourceManager.NAME_RESOURCE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
            StringBuffer tables = new StringBuffer();
            tables.append(getTableName());
            tables.append(",");
            tables.append(manager.getTableName());
            joinCondition.setTableName(tables.toString());
        }
        else if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(ScopeManager.NAME_ELEMENT_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
            StringBuffer tables = new StringBuffer();
            tables.append(getTableName());
            tables.append(",");
            tables.append(manager.getTableName());
            joinCondition.setTableName(tables.toString());
        }
        else if (manager instanceof DocumentManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(manager.getTableName(contextName) + "." + DocumentManager.NAME_ELEMENT_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
            StringBuffer tables = new StringBuffer();
            tables.append(getTableName());
            tables.append(",");
            tables.append(manager.getTableName(contextName));
            joinCondition.setTableName(tables.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        if (manager instanceof DocumentManager)
        {
            joinCondition = new JoinCondition();    
            SqlBuffer where = new SqlBuffer();
            where.appendSubSelect(DocumentManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID, 
                getTableName(), null);
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof PoolManager)
        {
            joinCondition = new JoinCondition();
            SqlBuffer where = new SqlBuffer();
            where.appendSubSelect(PoolManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID, 
                getTableName(), null);
            joinCondition.setCondition(where.toString());
        }
        else
        {
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	            ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof PoolManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(PoolManager.NAME_ELEMENT_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
        }
        else if (manager instanceof ResourceManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(ResourceManager.NAME_RESOURCE_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
        }
        else if (manager instanceof ResourceRoleAssignmentManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_RES_ASSIGNMENT_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        }
        else if(manager instanceof ScopeManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(ScopeManager.NAME_ELEMENT_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
        }
        else if(manager instanceof DocumentManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(manager.getTableName(contextName) + "." + DocumentManager.NAME_ELEMENT_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, contextName);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new ResourceRoleAssignment();
        return object;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields, int index,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        //Everything is loaded without scope

        ResourceRoleAssignment object = (ResourceRoleAssignment) rpmObject;

        //Related resource
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource, null,
                messageContext, isDelta);
            if (isDelta)
            {
                object.deltaResource(resource);
            }
            else
            {
                object.setResource(resource);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }

        // load SecurityRole without scope
        String securityRoleId = (String)relatedFields.get(index, ID_SECURITY_ID);
        if (securityRoleId != null)
        {
            SecurityRoleManager securityRoleManager = (SecurityRoleManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(SecurityRole.class);
            SecurityRole securityRole = new SecurityRole();
            securityRole.setID(securityRoleId.trim());            
            if (isDelta)
            {
                object.deltaSecurityRole(securityRole);
            }
            else
            {
                securityRole = (SecurityRole) securityRoleManager
                    .loadByPrimaryKey(securityRole, null, messageContext,
                        isDelta);
                object.setSecurityRole(securityRole);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, rpmObject, NAME_SECURITY_ID);
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta)
            throws SQLException
    {

        RPMObject pObject = rpmObject;

        pObject.setID(rs.getString(ID_RES_ASSIGNMENT_ID));

        return pObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException
    {

        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);
        
        String securityRoleId = resultSet.getString(ID_SECURITY_ID);
        relatedFields.put(index, ID_SECURITY_ID, securityRoleId);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        ResourceRoleAssignment pObject = (ResourceRoleAssignment) rpmObject;
        RPMObject parent = null;
        String id = null;
        parent = pObject.getParent();

        if (parent instanceof Pool)
        {
            id = SP_A_POOLRES(pObject, messageContext);
        }
        else if(parent instanceof ScopeElement)
        {
            id = SP_A_CRIRES(pObject, messageContext);
        }
        else if(parent instanceof DocumentElement)
        {
            id = SP_A_DOCRES(pObject, messageContext);
        }
        //TODO add other conditions on the parent
        pObject.setID(id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        ResourceRoleAssignment pObject = (ResourceRoleAssignment) rpmObject;
        if (pObject.testSecurityRoleModified())
        {
            SP_U_PARTICIPANT(pObject, messageContext);
        }

        return pObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject != null)
        {
            if(rpmObject.getParent() instanceof ScopeElement
            	|| rpmObject.getParent() instanceof DocumentElement)
                SP_D_TSKRES_ACT((ResourceRoleAssignment)rpmObject, messageContext);
            else
                SP_D_TSKRES((ResourceRoleAssignment) rpmObject, messageContext);
        }
    }
    
    private String SP_A_CRIRES(ResourceRoleAssignment res,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_CRIRES";
        Object args[] = new Object[3];
        if(res.getParent() != null)
            args[0] = res.getParent().getID();
        if(res.getResource() != null)
            args[1] = res.getResource().getID();
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private String SP_A_POOLRES(ResourceRoleAssignment poolres,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_POOLRES";
        Object args[] = new Object[3];
        args[0] = poolres.getParent().getID();
        args[1] = poolres.getResource().getID();
        args[2] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private String SP_A_DOCRES(ResourceRoleAssignment res,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_DOCRES";
        Object args[] = new Object[4];
        if(res.getParent() != null)
            args[0] = DocumentUtil.getInstance().getTableType(res.getParent().getContextName());
        if(res.getParent() != null)
            args[1] = res.getParent().getID();
        if(res.getResource() != null)
            args[2] = res.getResource().getID();
        args[3] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_PARTICIPANT(ResourceRoleAssignment poolres,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_PARTICIPANT";
        Object args[] = new Object[9];
        args[0] = poolres.getID();
        args[1] = poolres.getSecurityRole().getID();
        args[2] = null;
        args[3] = null;
        args[4] = new Integer(0);
        args[5] = new Integer(0);
        args[6] = new Integer(0);
        args[7] = new Integer(0);
        args[8] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_TSKRES(ResourceRoleAssignment poolres, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_TSKRES";
        Object args[] = new Object[4];
        args[0] = poolres.getID();
        args[1] = new Integer(0);
        args[2] = poolres.getResource().getID();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
    
    private void SP_D_TSKRES_ACT(ResourceRoleAssignment res,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_TSKRES_ACT";
        Object args[] = new Object[5];
        args[0] = res.getID();
        args[1] = new Integer(0);
        if(res.getResource() != null)
            args[2] = res.getResource().getID();
        args[3] = getUser(messageContext).getID();
        args[4] = new Integer(1);

        executeProcedure(messageContext, name, args);
    }
}
