/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;

/**
 * @author dominiqu
 */
public class AbstractScopeTaskCheckpoint extends ScopeElementCheckpoint
{
    private static final List ABSTRACTSCOPETASK_PARENTS = new ArrayList();

    static
    {
        ABSTRACTSCOPETASK_PARENTS.add(Action.class);
        ABSTRACTSCOPETASK_PARENTS.add(Issue.class);
        ABSTRACTSCOPETASK_PARENTS.add(ChangeRequest.class);
        ABSTRACTSCOPETASK_PARENTS.add(Defect.class);
        ABSTRACTSCOPETASK_PARENTS.add(Requirement.class);
        ABSTRACTSCOPETASK_PARENTS.add(Risk.class);
        ABSTRACTSCOPETASK_PARENTS.add(ServiceRequest.class);
        ABSTRACTSCOPETASK_PARENTS.add(ScopeSummaryTask.class);
        ABSTRACTSCOPETASK_PARENTS.add(Deliverable.class);
    }

    protected static AbstractScopeTaskCheckpoint instance = new AbstractScopeTaskCheckpoint();

    public static AbstractScopeTaskCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true, ABSTRACTSCOPETASK_PARENTS);

        // Initialize
        AbstractScopeTask mainAbstractScopeTask = (AbstractScopeTask) rpmObject;

        // Field validation: closedByName
        if (mainAbstractScopeTask.testClosedByNameModified())
        {
        	if(mainAbstractScopeTask.getClosedByName() != null)
        	{
        	 GenericValidator.validateMaxLength(mainAbstractScopeTask,
                     ValidationConstants.ABSTRACTSCOPETASK_CLOSED_BY_NAME_FIELD, mainAbstractScopeTask
                         .getClosedByName(), ValidationConstants.ABSTRACTSCOPETASK_CLOSED_BY_NAME_MAX, msgCtx);
        	}
        }

        // Field validation: priority
        if (mainAbstractScopeTask.testPriorityModified())
        {
            GenericValidator.validateRange(mainAbstractScopeTask,
                ValidationConstants.ABSTRACTSCOPETASK_PRIORITY_FIELD,
                mainAbstractScopeTask.getPriority(),
                ValidationConstants.PRIORITY_MIN,
                ValidationConstants.PRIORITY_MAX, msgCtx);
        }

        // Field validation: proposedByName
        if (mainAbstractScopeTask.testProposedByNameModified())
        {
        	if(mainAbstractScopeTask.getProposedByName() != null)
        	{
        	 GenericValidator.validateMaxLength(mainAbstractScopeTask,
                     ValidationConstants.ABSTRACTSCOPETASK_PROPOSED_BY_NAME_FIELD, mainAbstractScopeTask
                         .getClosedByName(), ValidationConstants.ABSTRACTSCOPETASK_PROPOSED_BY_NAME_MAX, msgCtx);
        	}
        }
    }
}