/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author dominiqu
 */
public class ActionCheckpoint extends AbstractScopeCheckpoint
{
    protected static ActionCheckpoint instance = new ActionCheckpoint();

    public static ActionCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx);

        // Initialize
        Action mainAction = (Action) rpmObject;
        ScopeElementScope scope = (ScopeElementScope) rpmScope;

        // Field validation: state
        if (mainAction.testStateModified() && mainAction.getState() != null)
        {
            GenericValidator.validateType(mainAction, ActionState.class,
                ValidationConstants.ABSTRACTSCOPE_STATE_FIELD, mainAction
                    .getState(), msgCtx);
        }

        // Container validation: approvedCostBenefits
        if (scope != null && scope.isApprovedCostBenefits())
        {
            ApprovedCostBenefits approvedCostBenefits = mainAction.getApprovedCostBenefits();
            if( approvedCostBenefits == null )
            {
                if ( rpmObject.getID() != null )
                {
                    // field can't be reset to null after object creation
                    GenericValidator.validateNotNull(rpmObject, "approvedCostBenefits", approvedCostBenefits, msgCtx);
                }
            }
            else
            {
                ApprovedCostBenefitsCheckpoint.getInstance().validateSave(
                    approvedCostBenefits, msgCtx);
            }
        }
    }
}