/*
 * Created on 19-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.util.ScopeManagementValidator;

/**
 * @author dominiqu
 */
public class DuplicateCheckpoint extends AbstractCheckpoint
{
    private static final List DUPLICATE_PARENTS = new ArrayList();

    static
    {
        DUPLICATE_PARENTS.add(Action.class);
        DUPLICATE_PARENTS.add(Issue.class);
        DUPLICATE_PARENTS.add(ChangeRequest.class);
        DUPLICATE_PARENTS.add(Defect.class);
        DUPLICATE_PARENTS.add(Requirement.class);
        DUPLICATE_PARENTS.add(Risk.class);
        DUPLICATE_PARENTS.add(ServiceRequest.class);
    }

    protected static DuplicateCheckpoint instance = new DuplicateCheckpoint();

    public static DuplicateCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        boolean valid = false;

        valid = super.isValidParent(rpmObject, msgCtx, true, false,
            DUPLICATE_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, Duplicate.class,
            msgCtx))
            return;

        // Initialize
        Duplicate mainDuplicate = (Duplicate) rpmObject;

        // Note: setModified does not set the inner fields of children classes
        // as modified so we cannot check if they have been modified before
        // validating

        // Field validation: scopeElement
        valid &= GenericValidator.validateMandatoryID(mainDuplicate,
            ValidationConstants.DUPLICATE_SCOPE_ELEMENT_FIELD, mainDuplicate
                .getScopeElement(), msgCtx);

        if (valid)
        {
            valid &= GenericValidator.validateExactType(mainDuplicate,
                mainDuplicate.getParent().getClass(),
                ValidationConstants.DUPLICATE_SCOPE_ELEMENT_FIELD,
                mainDuplicate.getScopeElement(), msgCtx);

            if (mainDuplicate.getParent().getID() != null
                    && mainDuplicate.getParent().getID().equals(
                        mainDuplicate.getScopeElement().getID()))
            {
                String[] errorParams =
                {
                        mainDuplicate.getParent().getID()
                };
                RPMException e = new RPMException(
                    ErrorCodes.DUPLICATE_CANNOT_HAVE_SAME_ID, errorParams,
                    mainDuplicate.getClass().getName());
                addException(e, msgCtx);
                valid = false;
            }

            if (!ScopeManagementValidator.isInSameProject(msgCtx, mainDuplicate
                .getParent(), mainDuplicate.getScopeElement()))
            {
                RPMException e = new RPMException(
                    ErrorCodes.DUPLICATE_MUST_BE_IN_SAME_PROJECT, mainDuplicate
                        .getClass().getName());
                addException(e, msgCtx);
                valid = false;
            }
        }
    }
    
}
