/*
 * Created on Aug 18, 2006
 *
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;

/**
 * @author karim
 *
 */
public class ReqProConnectionCheckpoint extends AbstractCheckpoint{

	//TODO add read only constraint on ReqProID
    private static final List REQPROCONNECTION_PARENTS = new ArrayList();

    static
    {
    	REQPROCONNECTION_PARENTS.add(ReqProIntegrationFolder.class);
    }
    
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // Initialize
        ReqProConnection reqProConnection = (ReqProConnection) rpmObject;
    	
        boolean isNew = reqProConnection.getID()==null;
        
        if(isNew)
        {
            super.isValidParent(rpmObject, msgCtx, true, REQPROCONNECTION_PARENTS);        	
        }
        else
        {
        	//parent is always required in the create mode
            super.isValidParent(rpmObject, msgCtx, false, REQPROCONNECTION_PARENTS);
        }

        // Field validation: ReqProID
        if(isNew || reqProConnection.testReqProIDModified())
        {
        	if(!isNew)
        	{
        		GenericValidator.validateReadOnly(reqProConnection,
                        ValidationConstants.REQPRO_ID_FIELD,
                        msgCtx);
        	}
        	GenericValidator.validateMandatory(reqProConnection,
					ValidationConstants.REQPRO_ID_FIELD, reqProConnection
							.getReqProID(), msgCtx);
        	GenericValidator.validateMinLength(reqProConnection,
                    ValidationConstants.REQPRO_ID_FIELD,
                    reqProConnection.getReqProID(), ValidationConstants.REQPRO_ID_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProConnection,
                    ValidationConstants.REQPRO_ID_FIELD,
                    reqProConnection.getReqProID(), ValidationConstants.REQPRO_ID_MAX,
                    msgCtx);
        }
        // Field validation: ReqProServerURL
        if(isNew || reqProConnection.testReqProServerURLModified())
        {
        	GenericValidator.validateMandatory(reqProConnection,
					ValidationConstants.REQPRO_SERVER_URL_FIELD,
					reqProConnection.getReqProServerURL(), msgCtx);
        	GenericValidator.validateMinLength(reqProConnection,
                    ValidationConstants.REQPRO_SERVER_URL_FIELD,
                    reqProConnection.getReqProServerURL(), ValidationConstants.REQPRO_SERVER_URL_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProConnection,
                    ValidationConstants.REQPRO_SERVER_URL_FIELD,
                    reqProConnection.getReqProServerURL(), ValidationConstants.REQPRO_SERVER_URL_MAX,
                    msgCtx);
        }
        // Field validation: ReqProProjectPath
        if(isNew || reqProConnection.testReqProProjectPathModified())
        {
        	GenericValidator.validateMandatory(reqProConnection,
					ValidationConstants.REQPRO_PROJECT_PATH_FIELD,
					reqProConnection.getReqProProjectPath(), msgCtx);
        	GenericValidator.validateMinLength(reqProConnection,
                    ValidationConstants.REQPRO_PROJECT_PATH_FIELD,
                    reqProConnection.getReqProProjectPath(), ValidationConstants.REQPRO_PROJECT_PATH_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProConnection,
                    ValidationConstants.REQPRO_PROJECT_PATH_FIELD,
                    reqProConnection.getReqProProjectPath(), ValidationConstants.REQPRO_PROJECT_PATH_MAX,
                    msgCtx);
        }
        // Field validation: ReqProLogon
        if(isNew || reqProConnection.testReqProLogonModified())
        {
        	GenericValidator.validateMandatory(reqProConnection,
					ValidationConstants.REQPRO_LOGON_FIELD, reqProConnection
							.getReqProLogon(), msgCtx);
        	GenericValidator.validateMinLength(reqProConnection,
                    ValidationConstants.REQPRO_LOGON_FIELD,
                    reqProConnection.getReqProLogon(), ValidationConstants.REQPRO_LOGON_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProConnection,
                    ValidationConstants.REQPRO_LOGON_FIELD,
                    reqProConnection.getReqProLogon(), ValidationConstants.REQPRO_LOGON_MAX,
                    msgCtx);
        }
        // Field validation: ReqProPassword
        if(isNew || reqProConnection.testReqProPasswordModified())
        {
        	GenericValidator.validateMandatory(reqProConnection,
					ValidationConstants.REQPRO_PASSWORD_FIELD, reqProConnection
							.getReqProPassword(), msgCtx);
        	GenericValidator.validateMinLength(reqProConnection,
                    ValidationConstants.REQPRO_PASSWORD_FIELD,
                    reqProConnection.getReqProPassword(), ValidationConstants.REQPRO_PASSWORD_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProConnection,
                    ValidationConstants.REQPRO_PASSWORD_FIELD,
                    reqProConnection.getReqProPassword(), ValidationConstants.REQPRO_PASSWORD_MAX,
                    msgCtx);
        }
    }

}
