/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author dominiqu
 */
public class RequirementCheckpoint extends AbstractScopeCheckpoint
{
    protected static RequirementCheckpoint instance = new RequirementCheckpoint();

    public static RequirementCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx);

        // Initialize
        Requirement mainRequirement = (Requirement) rpmObject;
        ScopeElementScope scope = (ScopeElementScope) rpmScope;

        // Field validation: state
        if (mainRequirement.testStateModified()
                && mainRequirement.getState() != null)
        {
            GenericValidator.validateType(mainRequirement,
                RequirementState.class,
                ValidationConstants.ABSTRACTSCOPE_STATE_FIELD, mainRequirement
                    .getState(), msgCtx);
        }
        
        // Container validation: approvedCostBenefits
        if (scope != null && scope.isApprovedCostBenefits())
        {
            ApprovedCostBenefits approvedCostBenefits = mainRequirement.getApprovedCostBenefits();
            if( approvedCostBenefits == null )
            {
                if ( rpmObject.getID() != null )
                {
                    // field can't be reset to null after object creation
                    GenericValidator.validateNotNull(rpmObject, "approvedCostBenefits", approvedCostBenefits, msgCtx);
                }
            }
            else
            {
                ApprovedCostBenefitsCheckpoint.getInstance().validateSave(
                    approvedCostBenefits, msgCtx);
            }
        }
    }
}