/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;

/**
 * @author dominiqu
 */
public class RiskMatrixCheckpoint extends AbstractCheckpoint
{
    private static final Class RISKMATRIX_PARENTS = Risk.class;

    protected static RiskMatrixCheckpoint instance = new RiskMatrixCheckpoint();

    public static RiskMatrixCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super
            .isValidParent(rpmObject, msgCtx, false, false, RISKMATRIX_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, RiskMatrix.class,
            msgCtx))
            return;

        // Initialize
        RiskMatrix mainRiskMatrix = (RiskMatrix) rpmObject;

        // Note: setModified does not set the inner fields of children classes
        // as modified so we cannot check if they have been modified before
        // validating

        // Field validation: consequenceCost
        GenericValidator.validateRange(mainRiskMatrix,
            ValidationConstants.RISKMATRIX_CONSEQUENCE_COST_FIELD,
            mainRiskMatrix.getConsequenceCost(),
            ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MIN,
            ValidationConstants.RISKMATRIX_CONSEQUENCECOST_MAX, msgCtx);

        // Field validation: probability
        GenericValidator.validatePossibleIntegerValues(mainRiskMatrix,
            ValidationConstants.RISKMATRIX_PROBABILITY_FIELD, mainRiskMatrix
                .getProbability(), ValidationConstants.RISKMATRIX_VALUES,
            msgCtx);

        // Field validation: impact
        GenericValidator.validatePossibleIntegerValues(mainRiskMatrix,
            ValidationConstants.RISKMATRIX_IMPACT_FIELD, mainRiskMatrix
                .getImpact(), ValidationConstants.RISKMATRIX_VALUES, msgCtx);

    }
}