/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author dominiqu
 */
public class ScopeElementCheckpoint extends AbstractCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx,
            boolean parentRequired, List validParentTypes)
    {
        super
            .isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        // Initialize
        ScopeElement mainScopeElement = (ScopeElement) rpmObject;

        GenericValidator.validateMandatory(mainScopeElement,
            ValidationConstants.NAME_FIELD, mainScopeElement.getName(), msgCtx);

        // Field validation: name
        if (mainScopeElement.testNameModified())
        {
            GenericValidator.validateMaxLength(mainScopeElement,
                ValidationConstants.SCOPEELEMENT_NAME_FIELD, mainScopeElement
                    .getName(), ValidationConstants.NAME_MAX, msgCtx);
        }
        
        // Field validation: wbsParent 
        // If we wish to save the related wbsParent, it must be in the same
        // project as the current rpmObject. 
        if(mainScopeElement.testWbsParentModified())
        {
            WorkElement wbsParent = mainScopeElement.getWbsParent();
            RPMManagerFactory managerFactory = msgCtx.getFactory().getManagerCaller().getManagerFactory();
            String scopeProjectID, wbsProjectID;
            scopeProjectID = null;
            wbsProjectID = null;
            
            try
            {
                if(mainScopeElement.getProject() != null && mainScopeElement.getProject().getID() != null) 
                {
                    scopeProjectID = mainScopeElement.getProject().getID();
                } else
                {
                    ScopeManager scopeManager = (ScopeManager) managerFactory.getRPMObjectManager(mainScopeElement.getParent().getClass());
                    ScopeElementScope scopeParentScope = new ScopeElementScope();
                    scopeParentScope.setProject(new WorkElementScope());
                    ScopeElement scopeParent = (ScopeElement) scopeManager.load(
                        mainScopeElement.getParent(), scopeParentScope, msgCtx);
                    scopeProjectID = scopeParent.getProject().getID();
                }

                WbsManager workManager = (WbsManager)managerFactory.getRPMObjectManager(
                    WorkElement.class);
                WorkElementScope wbsScope = new WorkElementScope();
                wbsScope.setContainingProject(new WorkElementScope());
                WorkElement workElement = (WorkElement) workManager.load(wbsParent, wbsScope, msgCtx);
                wbsProjectID = workElement.getContainingProject().getID();
                
                if(!scopeProjectID.equals(wbsProjectID))
                {
                    RPMException exception = new RPMException(
                        ErrorCodes.PROJECTID_DOES_NOT_MATCH ,
                        ScopeElement.class.getName(), 
                        ValidationConstants.NAME_WBSPARENT );
                    addException(exception, msgCtx);
                }
            } 
            catch (RPMException e)
            {
                addException(e, msgCtx);
            }
            catch (SQLException e)
            {
                e.printStackTrace();
            }
            catch (ParseException e)
            {
                e.printStackTrace();
            }
        }
        
        if(mainScopeElement.getID() != null)
        {
        	validateUpdate(mainScopeElement, msgCtx);
        }
    }
    
    private void validateUpdate(ScopeElement mainScopeElement, MessageContext msgCtx)
    {
        //Field Validation: ElementSecurityType
        if(mainScopeElement.testElementSecurityTypeModified())
        {
        	GenericValidator.validateMandatory(mainScopeElement, 
        			ValidationConstants.ELEMENT_SECURITY_TYPE, 
        			mainScopeElement.getElementSecurityType(), msgCtx);
        }
        
    }
    
    public void validateDelete(RPMObject rpmObject, MessageContext msgCtx)
    {
        super.validateDelete(rpmObject, msgCtx);

        ScopeElement scopeElement = (ScopeElement) rpmObject;

        try
        {
            // if the ScopeElement was transferred, must delete the transferred
            // element prior to deleting the original container
            if (ScopeManagementUtil.isTransferredScopeElement(scopeElement,
                msgCtx))
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(scopeElement.getClass()),
                        scopeElement.getID()
                };
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_DELETE_TRANSFERRED_ELEMENTS, errorParams,
                    scopeElement.getClass().getName());
                addException(e, msgCtx);
            }
        }
        catch (RPMException e)
        {
            msgCtx.addExceptionNoThrow(e);
        }
    }
    
    protected boolean validateReadOnlyWorkAssigned(ScopeElement scopeElement,
            String fieldName, MessageContext msgCtx) 
    {
        boolean valid = true;
        WorkElement genericProject = null;
        
        // load the GenericProject that's associated to this ScopeElement
        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(rtaScope);

        genericProject = ScopeManagementUtil.loadScopeProject(scopeElement,
            workElementScope, msgCtx);

        // check if work has been assigned to this project
        if (WbsUtil.isWorkAssigned(genericProject))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(scopeElement.getClass()) + " "
                            + fieldName, fieldName,
                    StringUtil.getShortClassName(scopeElement.getClass())
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED, errorParams,
                scopeElement.getClass().getName(), fieldName, scopeElement
                    .getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            valid = false;
        }

       return valid;
    }
        
}