package com.ibm.rpm.scopemanagement.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.MitigationTrigger;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;

//Copy next line in RPMInitializer.loadManagers()
//Class.forName(MitigationFactorAssignmentManager.class.getName());

public class MitigationFactorManager extends AbstractRPMObjectManager{

    private static Log logger = LogFactory.getLog(MitigationFactorManager.class);
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    private static final HashSet CONTAINERS = new HashSet();

    static{
    	CONTAINERS.add(MitigationFactor.class.getName());
    	CONTAINERS.add(MitigationTrigger.class.getName());
        //Add the complete class name of all the containers that could be loaded by this managers.
        //CONTAINERS.add(Container.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i], MitigationFactorManager.class);
        }
    }
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
    */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_FACTORS.LEVEL_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_FACTORS.TYPE_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_FACTORS.MAX_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_FACTORS.CHILD_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_FACTORS.DELETED_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column MITIGATION_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_MITIGATION_COST = 6;
    public static final int TYPE_MITIGATION_COST = Types.DECIMAL;
    public static final String NAME_MITIGATION_COST = "TMT_FACTORS.MITIGATION_COST";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MITIGATION_COST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MITIGATION_COST,NAME_MITIGATION_COST);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_FACTORS.ELEMENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_FACTORS.ELEMENT_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_FACTORS.PARENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 10;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_FACTORS.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 11;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_FACTORS.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 12;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_FACTORS.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_FACTORS.RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 14;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_FACTORS.DESCRIPTION";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    /**
     * Column PARENT_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PARENT_RANK = 15;
    public static final int TYPE_PARENT_RANK = Types.VARCHAR;
    public static final String NAME_PARENT_RANK = "TMT_FACTORS.PARENT_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PARENT_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_RANK,NAME_PARENT_RANK);}

    /**
     * Column PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRIORITY = 16;
    public static final int TYPE_PRIORITY = Types.SMALLINT;
    public static final String NAME_PRIORITY = "TMT_FACTORS.PRIORITY";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRIORITY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRIORITY,NAME_PRIORITY);}

    public static final String TABLE_NAME = "TMT_FACTORS";

    /**
     * Create an array of type string containing all the fields of the TMT_FACTORS table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_MITIGATION_COST
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_RANK
        ,NAME_DESCRIPTION
        ,NAME_PARENT_RANK
        ,NAME_PRIORITY
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++) {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /**
     * Creates a new MitigationFactorAssignmentManager instance.
     * @return MitigationFactorAssignmentManager
     */   
    public MitigationFactorManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() {
        // validate the primary key field name.
        return NAME_ELEMENT_ID;
        //return null;
    }  

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }  
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException {
    	int typeId = 0;
        if (containerName != null){
        	if (containerName.equalsIgnoreCase(StringUtil.getShortClassName(MitigationFactor.class))){
                    typeId = MitigationFactor.TYPE_ID;
                }
                else if (containerName.equalsIgnoreCase(StringUtil.getShortClassName(MitigationTrigger.class))) {
                    typeId = MitigationTrigger.TYPE_ID;
                }
                else {
                    ExceptionUtil.handleNoTypeId(this, containerName);
                }
        }
        StringBuffer baseCondition = new StringBuffer();
        baseCondition.append(NAME_ELEMENT_ID);
        baseCondition.append(" IS NOT NULL ");
        if (typeId != 0) {
            baseCondition.append(" AND ");
            baseCondition.append(NAME_TYPE_ID);
            baseCondition.append(" = ");
            baseCondition.append(typeId);
        }
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                baseCondition.append(" AND ");
                baseCondition.append(condition);
            }
            joinCondition.setCondition(baseCondition.toString());
        }
        else if(manager instanceof MitigationFactorManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            where.append(" WHERE ");
            where.append(baseCondition);
            if (condition != null) {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        } else {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException {
        //  Implement for managers that handles the parents of the
        //     supported containers
        JoinCondition joinCondition = null;
        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        
        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet) throws RPMException, SQLException {
        // validate the container type field name.
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }  

    protected RPMObject createContainer(int typeId) throws RPMException {
         RPMObject object = null;
         switch (typeId) {

         case MitigationFactor.TYPE_ID :
         	object = new MitigationFactor();
         	break;
         case MitigationTrigger.TYPE_ID :
         	object = new MitigationTrigger();
         	break;
         default:
     		RPMException e = new RPMException("TypeId " + typeId + " is not supported");
     		logger.error(e);
     		throw e;
         }
         return object;
     }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
    	if (isMitigationFactor(rpmObject)){
    		MitigationFactor mitigationFactor = (MitigationFactor) rpmObject;
        	mitigationFactor.setID(rs.getString(ID_ELEMENT_ID));
        	  if (isDelta){
        	  	mitigationFactor.deltaDescription(rs.getString(ID_DESCRIPTION));
        	  	mitigationFactor.deltaMitigationCost(Double.valueOf(rs.getString(ID_MITIGATION_COST)));
        	  	mitigationFactor.deltaPriority(new Integer(rs.getInt(ID_PRIORITY)));        	  	
        	  }else{
        	  	mitigationFactor.setDescription(rs.getString(ID_DESCRIPTION));
        	  	mitigationFactor.setMitigationCost(Double.valueOf(rs.getString(ID_MITIGATION_COST)));
        	  	mitigationFactor.setPriority(new Integer(rs.getInt(ID_PRIORITY)));
        	  }
    	} else if (isMitigationTrigger(rpmObject)){
    		MitigationTrigger mitigationTrigger = (MitigationTrigger) rpmObject;
    		mitigationTrigger.setID(rs.getString(ID_ELEMENT_ID));
    		if (isDelta){
    			mitigationTrigger.deltaDescription(rs.getString(ID_DESCRIPTION));
    		}else{
    			mitigationTrigger.setDescription(rs.getString(ID_DESCRIPTION));
    		}
    	}
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException {
    	//  Implement this method if you need values from the result set
        // to load the related objects.
    	String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
    	 if (rpmScope != null) {

             //Parent
             if (rpmScope.getParent() != null) {
                 String oldParentId = (String) relatedFields.get(index,ID_PARENT_ID);
                 RPMObject newParent = null;
                 if (isDelta) {
                     newParent = rpmObject.getParent();
                     RPMObject oldParent = null;
                     if (oldParentId != null) {
                         oldParent = new Risk();
                         oldParent.setID(oldParentId);
                         if (newParent != null && newParent.getID() != null
                                 && newParent.getID().equals(oldParentId)) {
                         	
                         	ScopeManager scopeManager = (ScopeManager) RPMManagerFactory
							.getInstance().getRPMObjectManager(Risk.class.getName());
                         	oldParent = (Risk) scopeManager.loadByPrimaryKey(newParent, rpmScope
                            .getParent(),messageContext, true);
                         } else  {
                             //In this case, old parent is loaded without the
                             // scope to avoid problem with the cache
                         	ScopeManager scopeManager = (ScopeManager) RPMManagerFactory
							.getInstance().getRPMObjectManager(Risk.class.getName());
                         	oldParent = (Risk) scopeManager.loadByPrimaryKey(oldParent, null,
							messageContext, false);
                         }
                     }
                     rpmObject.deltaParent(oldParent);
                 } else  {
                     if (oldParentId != null)
                     {
                         if (newParent == null)
                         {//newParent not in cache
                             newParent = new Risk();
                             newParent.setID(oldParentId);
                             
                             ScopeManager scopeManager = (ScopeManager) RPMManagerFactory
								.getInstance().getRPMObjectManager(Risk.class.getName());
                             newParent = (Risk) scopeManager.loadByPrimaryKey(newParent, rpmScope
                                .getParent(),
								messageContext, false);
                             
                            // newParent = loadByPrimaryKey(newParent, scope
                              //   .getParent(), messageContext, false);
                         }
                         rpmObject.setParent(newParent);
                     }
                 }
             }
             
            if (rpmScope instanceof MitigationFactorScope)
            {

                MitigationFactorScope scope = (MitigationFactorScope) rpmScope;
             //   load children
                if (isMitigationFactor(rpmObject))
                {
                    MitigationFactor mitigationFactor = (MitigationFactor) rpmObject;
                    if (scope.isMitigationTriggers())
                    {
                        loadMitigationTrigger(mitigationFactor,
                            mitigationFactor.getMitigationTriggers(),
                            messageContext, isDelta);
                    }
                }
            }
      }
    	 return rpmObject;
    }
    	 
    private void loadMitigationTrigger ( RPMObject rpmObject, MitigationTrigger[] oldChildren, MessageContext messageContext, boolean isDelta) throws RPMException, SQLException, ParseException {
    	ArrayList temp = loadByForeignKey(oldChildren, null, messageContext,
				this, new Object[] { rpmObject.getID() }, "TYPE_ID = "
						+ MitigationFactor.TYPE_ID, getContextName(rpmObject), isDelta);
    		if ((temp != null) && (temp.size() > 0)) {
    			MitigationTrigger[] children = new MitigationTrigger[temp.size()];
    			temp.toArray(children);
    		    ((MitigationFactor)rpmObject).setMitigationTriggers(children);
    		}
    }
    
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {
    	// Implement for managers that can load the supported
        // container as children
    	JoinCondition joinCondition = null;
        if (manager instanceof MitigationFactorManager)  {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(getPrimaryKey());
            condition.append(" = ?");

            //In this case, we don't need a condition on the type of the parent
            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0) {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        } else if (manager instanceof ScopeManager) {
            String tableName = "TMT_FACTORS";
            StringBuffer sb = new StringBuffer();
            sb.append(" PARENT_ID = ? ");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(sb.toString());
            joinCondition.setTableName(tableName);
        } 
        return joinCondition;	
    }
    
    private String getContextName(RPMObject rpmObject) throws RPMException {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }
    
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException {
    	rpmObject.setID(SP_I_FAC( rpmObject, messageContext));
    }
    
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException  {
    	MitigationFactorScope scope = (MitigationFactorScope)rpmScope;
    	if (isMitigationFactor(rpmObject)){
	    	MitigationFactor mitFactor = (MitigationFactor)rpmObject;
	    	if ( (mitFactor.testDescriptionModified()) ||(mitFactor.testMitigationCostModified()) || (mitFactor.testPriorityModified())){
	    		SP_U_FAC(rpmObject, messageContext);	
	    	}
			if (scope != null) {
	            rpmObject = updateChildren(rpmObject, scope, messageContext);
	        }
    	} else if (isMitigationTrigger(rpmObject)){
    		MitigationTrigger mitTrigger = (MitigationTrigger)rpmObject;
	    	if (mitTrigger.testDescriptionModified()){
	    		SP_U_FAC(rpmObject, messageContext);	
	    	}
    	}
        return rpmObject;
    }
    
	protected RPMObject updateChildren(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext)
			throws RPMException, SQLException, ParseException {
		MitigationFactorScope scope = (MitigationFactorScope) rpmScope;

		//save all Children of -Mitigation-Factor
		MitigationFactor mitigationFactor = (MitigationFactor) rpmObject;
		if (  (scope.isMitigationTriggers()) && (mitigationFactor.getMitigationTriggers() != null) ){
			MitigationTrigger[] children = (MitigationTrigger[])saveChildren(mitigationFactor, mitigationFactor.getMitigationTriggers(),scope.isMitigationTriggers() , messageContext);
			mitigationFactor.setMitigationTriggers(children);
		}
	   return rpmObject;		
	}

	private RPMObject[] saveChildren(RPMObject parent, RPMObject[] children,
			boolean childrenScope, MessageContext messageContext)
			throws RPMException, SQLException,
			ParseException {
	    children = updateChildren(parent, children,childrenScope, messageContext, null);
		return children;
	}

	protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException   {
        //  Implement if the manager can delete objects
    	if (rpmObject != null)  {
            SP_D_FAC(rpmObject, messageContext);
        }
    }

    private boolean isMitigationFactor(RPMObject rpmObject) {
        return rpmObject.retrieveTypeID().intValue() == MitigationFactor.TYPE_ID;
    }

    private boolean isMitigationTrigger(RPMObject rpmObject) {
        return rpmObject.retrieveTypeID().intValue() == MitigationTrigger.TYPE_ID;
    }

    //Code for Stored Procedures
    //SP_I_FAC('Factor','FBC937703E0744299271E59808E8927A',68,'PMO_SUPERVISOR')}
    private String SP_I_FAC(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException {
		final String procedureName = "SP_I_FAC";
		Object args[] = new Object[4];
		//name
		if (rpmObject instanceof MitigationFactor){
			args[0] = "Factor";
		}else if (rpmObject instanceof MitigationTrigger){
			args[0] = "Trigger";
		}
		//parent_id
		if (rpmObject.getParent() != null){
            args[1] = rpmObject.getParent().getID();
        }
		//type_id
		args[2] = Long.valueOf(rpmObject.retrieveTypeID().toString());
		//rec_user
		args[3] = getUser(messageContext).getID();
		String objectId = executeProcedure(messageContext, procedureName, args);		
		return objectId;
	}
    
    private void SP_D_FAC(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException {
		final String procedureName = "SP_D_FAC";
		Object args[] = new Object[4];
		// ID : SC_WCHAR
		args[0] = rpmObject.getID();
		// Parent_id : SC_WCHAR
        args[1] = ManagerUtil.getParentId( this, rpmObject, messageContext, getTableName());
		//  del_flag : SC_WCHAR
		args[2] = "X";
		args[3] = getUser(messageContext).getID();
		executeProcedure(messageContext, procedureName, args);
	}

    private void SP_U_FAC(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException {
		final String procedureName = "SP_U_FAC";
		Object args[] = new Object[6];
		// id : SC_WCHAR
		args[0] = rpmObject.getID();
		//	name
		if (rpmObject instanceof MitigationFactor){
			args[1] = "Factor";
			MitigationFactor mitigationFactor = (MitigationFactor) rpmObject;
			//	description : SC_WCHAR
			args[2]=mitigationFactor.getDescription();
			//	migration_cost : double
			args[3] = mitigationFactor.getMitigationCost();
			args[4] = mitigationFactor.getPriority();
		}else if (rpmObject instanceof MitigationTrigger){
			args[1] = "Trigger";
			MitigationTrigger mitigationTrigger = (MitigationTrigger) rpmObject;
			// description 
			args[2] = mitigationTrigger.getDescription();
			// migration_cost : double
			args[3] = new Double(0);
			args[4] = new Long(0);
		}
		args[5] = getUser(messageContext).getID();
		executeProcedure(messageContext, procedureName, args);
	}
}