package com.ibm.rpm.scopemanagement.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;

public class ScopeScheduleDateManager extends AbstractRPMObjectManager
{
    //approvedDate
    private static final HashMap APPROVEDPROPERTYMAP = new HashMap();

    //expectedDate
    private static final HashMap PROPOSEDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ScopeScheduleDate.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ScopeScheduleDateManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CRI.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        APPROVEDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        PROPOSEDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column PLANNED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_START = 2;

    public static final int TYPE_PLANNED_START = Types.TIMESTAMP;

    public static final String NAME_PLANNED_START = "TMT_CRI.PLANNED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PLANNED_START,NAME_PLANNED_START);}

    /**
     * Column PLANNED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_FINISH = 3;

    public static final int TYPE_PLANNED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PLANNED_FINISH = "TMT_CRI.PLANNED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_FINISH,NAME_PLANNED_FINISH);}

    /**
     * Column PLANNED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PLANNED_EFFORT = 4;

    public static final int TYPE_PLANNED_EFFORT = Types.BIGINT;

    public static final String NAME_PLANNED_EFFORT = "TMT_CRI.PLANNED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_EFFORT,NAME_PLANNED_EFFORT);}

    /**
     * Column PLANNED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_DURATION = 5;

    public static final int TYPE_PLANNED_DURATION = Types.INTEGER;

    public static final String NAME_PLANNED_DURATION = "TMT_CRI.PLANNED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_DURATION,NAME_PLANNED_DURATION);}

    /**
     * Column ESTIMATED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ESTIMATED_START = 6;

    public static final int TYPE_ESTIMATED_START = Types.TIMESTAMP;

    public static final String NAME_ESTIMATED_START = "TMT_CRI.ESTIMATED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_START,NAME_ESTIMATED_START);}

    /**
     * Column ESTIMATED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ESTIMATED_FINISH = 7;

    public static final int TYPE_ESTIMATED_FINISH = Types.TIMESTAMP;

    public static final String NAME_ESTIMATED_FINISH = "TMT_CRI.ESTIMATED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_FINISH,NAME_ESTIMATED_FINISH);}

    /**
     * Column APPROVED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_APPROVED_START = 8;

    public static final int TYPE_APPROVED_START = Types.TIMESTAMP;

    public static final String NAME_APPROVED_START = "TMT_CRI.APPROVED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_START,NAME_APPROVED_START);}

    /**
     * Column APPROVED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_APPROVED_FINISH = 9;

    public static final int TYPE_APPROVED_FINISH = Types.TIMESTAMP;

    public static final String NAME_APPROVED_FINISH = "TMT_CRI.APPROVED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_FINISH,NAME_APPROVED_FINISH);}

    /**
     * Column ACTUAL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_START = 10;

    public static final int TYPE_ACTUAL_START = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_START = "TMT_CRI.ACTUAL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_START,NAME_ACTUAL_START);}

    /**
     * Column ACTUAL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_FINISH = 11;

    public static final int TYPE_ACTUAL_FINISH = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_FINISH = "TMT_CRI.ACTUAL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_FINISH,NAME_ACTUAL_FINISH);}

    /**
     * Column ACTUAL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_DURATION = 12;

    public static final int TYPE_ACTUAL_DURATION = Types.INTEGER;

    public static final String NAME_ACTUAL_DURATION = "TMT_CRI.ACTUAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_DURATION,NAME_ACTUAL_DURATION);}

    /**
     * Column SCHED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_START = 13;

    public static final int TYPE_SCHED_START = Types.TIMESTAMP;

    public static final String NAME_SCHED_START = "TMT_CRI.SCHED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_START,NAME_SCHED_START);}

    /**
     * Column SCHED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_FINISH = 14;

    public static final int TYPE_SCHED_FINISH = Types.TIMESTAMP;

    public static final String NAME_SCHED_FINISH = "TMT_CRI.SCHED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_FINISH,NAME_SCHED_FINISH);}

    /**
     * Column SCHED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_DURATION = 15;

    public static final int TYPE_SCHED_DURATION = Types.INTEGER;

    public static final String NAME_SCHED_DURATION = "TMT_CRI.SCHED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_DURATION,NAME_SCHED_DURATION);}

    /**
     * Column PROPOSED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_START = 16;

    public static final int TYPE_PROPOSED_START = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_START = "TMT_CRI.PROPOSED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_START,NAME_PROPOSED_START);}

    /**
     * Column PROPOSED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_FINISH = 17;

    public static final int TYPE_PROPOSED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_FINISH = "TMT_CRI.PROPOSED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_FINISH,NAME_PROPOSED_FINISH);}

    /**
     * Column PROPOSED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_DURATION = 18;

    public static final int TYPE_PROPOSED_DURATION = Types.INTEGER;

    public static final String NAME_PROPOSED_DURATION = "TMT_CRI.PROPOSED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_DURATION,NAME_PROPOSED_DURATION);}

    /**
     * Column FORECAST_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_START = 19;

    public static final int TYPE_FORECAST_START = Types.TIMESTAMP;

    public static final String NAME_FORECAST_START = "TMT_CRI.FORECAST_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_START,NAME_FORECAST_START);}

    /**
     * Column FORECAST_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_FINISH = 20;

    public static final int TYPE_FORECAST_FINISH = Types.TIMESTAMP;

    public static final String NAME_FORECAST_FINISH = "TMT_CRI.FORECAST_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_FINISH,NAME_FORECAST_FINISH);}

    /**
     * Column FORECAST_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_DURATION = 21;

    public static final int TYPE_FORECAST_DURATION = Types.INTEGER;

    public static final String NAME_FORECAST_DURATION = "TMT_CRI.FORECAST_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_DURATION,NAME_FORECAST_DURATION);}

    /**
     * Column BASELINE1_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_START = 22;

    public static final int TYPE_BASELINE1_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_START = "TMT_CRI.BASELINE1_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_START,NAME_BASELINE1_START);}

    /**
     * Column BASELINE1_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_FINISH = 23;

    public static final int TYPE_BASELINE1_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_FINISH = "TMT_CRI.BASELINE1_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_FINISH,NAME_BASELINE1_FINISH);}

    /**
     * Column BASELINE1_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_DURATION = 24;

    public static final int TYPE_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE1_DURATION = "TMT_CRI.BASELINE1_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_DURATION,NAME_BASELINE1_DURATION);}

    /**
     * Column BASELINE1_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_EFFORT = 25;

    public static final int TYPE_BASELINE1_EFFORT = Types.INTEGER;

    public static final String NAME_BASELINE1_EFFORT = "TMT_CRI.BASELINE1_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_EFFORT,NAME_BASELINE1_EFFORT);}

    /**
     * Column SCHED_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_EFFORT = 26;

    public static final int TYPE_SCHED_EFFORT = Types.INTEGER;

    public static final String NAME_SCHED_EFFORT = "TMT_CRI.SCHED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_EFFORT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_EFFORT,NAME_SCHED_EFFORT);}

    /**
     * Column FORECAST_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_EFFORT = 27;

    public static final int TYPE_FORECAST_EFFORT = Types.INTEGER;

    public static final String NAME_FORECAST_EFFORT = "TMT_CRI.FORECAST_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_EFFORT,NAME_FORECAST_EFFORT);}

    /**
     * Column PROPOSED_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_EFFORT = 28;

    public static final int TYPE_PROPOSED_EFFORT = Types.INTEGER;

    public static final String NAME_PROPOSED_EFFORT = "TMT_CRI.PROPOSED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_EFFORT,NAME_PROPOSED_EFFORT);}

    /**
     * Column APPROVED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_APPROVED_EFFORT = 29;

    public static final int TYPE_APPROVED_EFFORT = Types.BIGINT;

    public static final String NAME_APPROVED_EFFORT = "TMT_CRI.APPROVED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_EFFORT,NAME_APPROVED_EFFORT);}

    /**
     * Column APPROVED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_APPROVED_DURATION = 30;

    public static final int TYPE_APPROVED_DURATION = Types.INTEGER;

    public static final String NAME_APPROVED_DURATION = "TMT_CRI.APPROVED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_DURATION,NAME_APPROVED_DURATION);}

    /**
     * Column BASELINE2_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_START = 31;

    public static final int TYPE_BASELINE2_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_START = "TMT_CRI.BASELINE2_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_START,NAME_BASELINE2_START);}

    /**
     * Column BASELINE2_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_FINISH = 32;

    public static final int TYPE_BASELINE2_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_FINISH = "TMT_CRI.BASELINE2_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_FINISH,NAME_BASELINE2_FINISH);}

    /**
     * Column BASELINE2_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_DURATION = 33;

    public static final int TYPE_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE2_DURATION = "TMT_CRI.BASELINE2_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_DURATION,NAME_BASELINE2_DURATION);}

    /**
     * Column BASELINE2_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_EFFORT = 34;

    public static final int TYPE_BASELINE2_EFFORT = Types.INTEGER;

    public static final String NAME_BASELINE2_EFFORT = "TMT_CRI.BASELINE2_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_EFFORT,NAME_BASELINE2_EFFORT);}

    private static final String TABLE_NAME = "TMT_CRI";

    /**
     * Create an array of type string containing all the fields of the TMT_CRI
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID,
            NAME_PLANNED_START, NAME_PLANNED_FINISH, NAME_PLANNED_EFFORT, NAME_PLANNED_DURATION,
            NAME_ESTIMATED_START, NAME_ESTIMATED_FINISH,
            NAME_APPROVED_START, NAME_APPROVED_FINISH,            
            NAME_ACTUAL_START, NAME_ACTUAL_FINISH, NAME_ACTUAL_DURATION,
            NAME_SCHED_START, NAME_SCHED_FINISH, NAME_SCHED_DURATION,
            NAME_PROPOSED_START, NAME_PROPOSED_FINISH, NAME_PROPOSED_DURATION,
            NAME_FORECAST_START, NAME_FORECAST_FINISH, NAME_FORECAST_DURATION,
            NAME_BASELINE1_START, NAME_BASELINE1_FINISH, NAME_BASELINE1_DURATION, NAME_BASELINE1_EFFORT,
            NAME_SCHED_EFFORT,
            NAME_FORECAST_EFFORT,
            NAME_PROPOSED_EFFORT,
            NAME_APPROVED_EFFORT, NAME_APPROVED_DURATION,
            NAME_BASELINE2_START, NAME_BASELINE2_START, NAME_BASELINE2_DURATION, NAME_BASELINE2_EFFORT
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new TmtCriManager instance.
     * 
     * @return TmtCriManager
     */
    public ScopeScheduleDateManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }
    
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap(String fieldName, String containerName) throws RPMException
    {

        HashMap fieldPropertyMap = null;

        if (fieldName.equalsIgnoreCase(ScopeScheduleDate.APPROVED))
        {
            fieldPropertyMap = APPROVEDPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(ScopeScheduleDate.PROPOSED))
        {
            fieldPropertyMap = PROPOSEDPROPERTYMAP;
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, fieldName,
                ExceptionUtil.GET_PROPERTIES);
        }

        return fieldPropertyMap;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new ScopeScheduleDate();
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {

        //Nothing to do
    }

   
    public boolean containerExistInCache(Object[] primaryKey,
            MessageContext messageContext)
    {
        //The cache cannot be used for this type of container because the key
        // is not unique.
        return false;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        ScopeScheduleDate object = (ScopeScheduleDate) rpmObject;
        object.setID(rs.getString(ID_ELEMENT_ID));

        //Use the contextName field to keep the name of the field we are
        // loading
        String contextName = rpmObject.getContextName();

        int startDateIndex = 0;
        int finishDateIndex = 0;
        int durationIndex = 0;
        int effortIndex = 0;

        if (contextName.equalsIgnoreCase(ScopeScheduleDate.APPROVED))
        {
            startDateIndex = ID_APPROVED_START;
            finishDateIndex = ID_APPROVED_FINISH;
            durationIndex = ID_APPROVED_DURATION;
            effortIndex = ID_APPROVED_EFFORT;

        }
        else if (contextName.equalsIgnoreCase(ScopeScheduleDate.PROPOSED))
        {
            startDateIndex = ID_PLANNED_START;
            finishDateIndex = ID_PLANNED_FINISH;
            durationIndex = ID_PLANNED_DURATION;
            effortIndex = ID_PLANNED_EFFORT;
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, contextName,
                ExceptionUtil.GET_PROPERTIES);
        }

        Calendar startDate = null;
        Calendar finishDate = null;
        startDate = Manager.getCalendar(rs, startDateIndex);
        finishDate = Manager.getCalendar(rs, finishDateIndex);
        
        //TODO check if this value is hardcoded in the ui for scope management
        //int mins = rs.getInt(ID_MINUTE_PER_DAY);
        int mins = 480;
        
        int duration = rs.getInt(durationIndex);
        int effort = rs.getInt(effortIndex);
        
        double days = duration / mins;
        int hours = (duration % mins) / 60;
        
        double workEffort = (double)effort / 60;
        
        
        if (isDelta)
        {
            object.deltaStartDate(startDate);
            object.deltaFinishDate(finishDate);
            object.deltaDurationInMinutes(new Integer(duration));
            object.deltaDays(new Double(days));
            object.deltaHours(new Double(hours));
            object.deltaWorkHours(new Double(workEffort));
        }
        else
        {
            object.setStartDate(startDate);
            object.setFinishDate(finishDate);
            object.setDurationInMinutes(new Integer(duration));
            object.setDays(new Double(days));
            object.setHours(new Double(hours));
            object.setWorkHours(new Double(workEffort));
        }
        
        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return null;

    }

    /**
     *  
     */

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            //TODO validate
            if (condition != null)
            {
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return null;
    }

    private boolean testScheduleDateModified(ScopeScheduleDate scopeScheduleDate)
    {
        if (scopeScheduleDate != null)
        {
            return scopeScheduleDate.testDaysModified()
                    || scopeScheduleDate.testFinishDateModified()
                    || scopeScheduleDate.testHoursModified()
                    || scopeScheduleDate.testStartDateModified()
                    || scopeScheduleDate.testWorkHoursModified();
        }
        return false;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        ScopeScheduleDate object = (ScopeScheduleDate) rpmObject;

        if (object != null && testScheduleDateModified(object))
        {

            String contextName = rpmObject.getContextName();

            //TODO Validate that plannedDate is read-only

            //SP_U_APPROVED
            if (contextName.equalsIgnoreCase(ScopeScheduleDate.APPROVED))
            {
                SP_U_CRI_APPROVED(object, messageContext);
            }

        }

        return rpmObject;
    }

    private void SP_U_CRI_APPROVED(ScopeScheduleDate object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        
        String name = "SP_U_CRI_APPROVED";

        Object args[] = new Object[7];
        args[0] = object.getParent().getID();
        args[1]= new Integer(0);
        
	    args[2] = DateUtil.convertToString(object.getStartDate());
	    
	    //finishDate is a read-only field
	    //when the parameter is null, the finishDate is calculated from the startDate and the duration.	
	    args[3] = null;
              
        //The fields days and hours are read-only
	    args[4] = object.getDurationInMinutes(); 
        
        //effort read-only
	    //reload the value from the database
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(NAME_ELEMENT_ID);
        Object[] arguments = new Object[]{object.getParent().getID()};
        Long workHours = (Long) ManagerUtil.selectColumnValue(Long.class,
            object, "workHours", NAME_APPROVED_EFFORT, getTableName(),
            condition, arguments, messageContext);

	    args[5] = workHours;    
        
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }    
    
    public RPMObject getContainerInCacheWithScope(
            RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext)
    {
        // disable the cache since the primary key is only the ID
        // but the context is also important.
        return null;
    }
}
