/**
 * @author ragoler
 * Checkpoint validation of container Scorecard.
 */

package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

public class ScorecardCheckpoint extends AbstractScorecardElementCheckpoint
{

    protected static ScorecardCheckpoint instance = new ScorecardCheckpoint();

    private static final List SCORECARD_PARENT_LIST = new ArrayList();

    static
    {
        SCORECARD_PARENT_LIST.add(ScorecardFolder.class);
    }

    public static ScorecardCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true, SCORECARD_PARENT_LIST);

        Scorecard scorecard = (Scorecard) rpmObject;

        // On update, the active and probabilityType fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(scorecard,
            ValidationConstants.ACTIVE_FIELD, scorecard.getActive(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(scorecard,
            ValidationConstants.PROBABILITY_TYPE_FIELD, scorecard
                .getProbabilityType(), msgCtx);
        
        // Field validation of weight
        if (scorecard.testWeightModified())
        {
            GenericValidator.validateRange(scorecard,
                ValidationConstants.WEIGHT_FIELD, scorecard.getWeight(),
                ValidationConstants.WEIGHT_MIN, ValidationConstants.WEIGHT_MAX,
                msgCtx);
        }
    }

}