package com.ibm.rpm.scorecard.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.checkpoints.ScorecardMethodCheckpoint;
import com.ibm.rpm.scorecard.containers.AbstractScorecardCategory;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.AssignedScorecardCategory;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;
import com.ibm.rpm.scorecard.containers.ResourceAssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.managers.ScorecardUtil.ProbabilityTypeID;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class PublishedScorecardManager extends AbstractRPMObjectManager
{
    //private static Log logger = LogFactory
    //    .getLog(PublishedScorecardManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {

        CONTAINERS.add(ResourceAssignedScorecard.class.getName());
        CONTAINERS.add(AssignedScorecard.class.getName());
        CONTAINERS.add(AssignedScorecardCategory.class.getName());
        CONTAINERS.add(AssignedScorecardQuestion.class.getName());
        CONTAINERS.add(AssignedScorecardResponse.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                PublishedScorecardManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_PUBLISHED_SCORECARDS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_PUBLISHED_SCORECARDS.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_PUBLISHED_SCORECARDS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_PUBLISHED_SCORECARDS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_PUBLISHED_SCORECARDS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 6;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "TMT_PUBLISHED_SCORECARDS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_PUBLISHED_SCORECARDS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_PUBLISHED_SCORECARDS.ELEMENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_PUBLISHED_SCORECARDS.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 10;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_PUBLISHED_SCORECARDS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_PUBLISHED_SCORECARDS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 12;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_PUBLISHED_SCORECARDS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 13;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_PUBLISHED_SCORECARDS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 14;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_PUBLISHED_SCORECARDS.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column PARENT_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PARENT_RANK = 15;

    public static final int TYPE_PARENT_RANK = Types.VARCHAR;

    public static final String NAME_PARENT_RANK = "TMT_PUBLISHED_SCORECARDS.PARENT_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_RANK,NAME_PARENT_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 16;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_PUBLISHED_SCORECARDS.DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column ORIGINAL_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ORIGINAL_ID = 17;

    public static final int TYPE_ORIGINAL_ID = Types.CHAR;

    public static final String NAME_ORIGINAL_ID = "TMT_PUBLISHED_SCORECARDS.ORIGINAL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ORIGINAL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ORIGINAL_ID,NAME_ORIGINAL_ID);}

    /**
     * Column SCORECARD_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SCORECARD_ID = 18;

    public static final int TYPE_SCORECARD_ID = Types.CHAR;

    public static final String NAME_SCORECARD_ID = "TMT_PUBLISHED_SCORECARDS.SCORECARD_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_ID,NAME_SCORECARD_ID);}

    /**
     * Column CATEGORY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CATEGORY_ID = 19;

    public static final int TYPE_CATEGORY_ID = Types.CHAR;

    public static final String NAME_CATEGORY_ID = "TMT_PUBLISHED_SCORECARDS.CATEGORY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CATEGORY_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CATEGORY_ID,NAME_CATEGORY_ID);}

    /**
     * Column SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCORE = 20;

    public static final int TYPE_SCORE = Types.BIGINT;

    public static final String NAME_SCORE = "TMT_PUBLISHED_SCORECARDS.SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SCORE = "SCORE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SCORE, NAME_SCORE);
    }

    /**
     * Column WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_WEIGHT = 21;

    public static final int TYPE_WEIGHT = Types.BIGINT;

    public static final String NAME_WEIGHT = "TMT_PUBLISHED_SCORECARDS.WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_WEIGHT = "WEIGHT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEIGHT, NAME_WEIGHT);
    }

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_SCORE = 22;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "TMT_PUBLISHED_SCORECARDS.CONVERTED_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE);}

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 23;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "TMT_PUBLISHED_SCORECARDS.CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column SELECTED_ANSWER of type Types.CHAR mapped to String.
     */
    public static final int ID_SELECTED_ANSWER = 24;

    public static final int TYPE_SELECTED_ANSWER = Types.CHAR;

    public static final String NAME_SELECTED_ANSWER = "TMT_PUBLISHED_SCORECARDS.SELECTED_ANSWER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SELECTED_ANSWER = "SELECTEDANSWER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SELECTED_ANSWER, NAME_SELECTED_ANSWER);
    }

    /**
     * Column PUBLISHED_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PUBLISHED_DT = 25;

    public static final int TYPE_PUBLISHED_DT = Types.TIMESTAMP;

    public static final String NAME_PUBLISHED_DT = "TMT_PUBLISHED_SCORECARDS.PUBLISHED_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PUBLISHED_DT = "PUBLISHEDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED_DT, NAME_PUBLISHED_DT);
    }

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAJOR_REVISION = 26;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "TMT_PUBLISHED_SCORECARDS.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

    /**
     * Column ANSWER_DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ANSWER_DESCRIPTION = 27;

    public static final int TYPE_ANSWER_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_ANSWER_DESCRIPTION = "TMT_PUBLISHED_SCORECARDS.ANSWER_DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ANSWER_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ANSWER_DESCRIPTION,
            NAME_ANSWER_DESCRIPTION);
    }

    /**
     * Column SCOPE_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCOPE_SCORE = 28;

    public static final int TYPE_SCOPE_SCORE = Types.BIGINT;

    public static final String NAME_SCOPE_SCORE = "TMT_PUBLISHED_SCORECARDS.SCOPE_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCOPE_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_SCORE,NAME_SCOPE_SCORE);}

    /**
     * Column SCOPE_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCOPE_WEIGHT = 29;

    public static final int TYPE_SCOPE_WEIGHT = Types.BIGINT;

    public static final String NAME_SCOPE_WEIGHT = "TMT_PUBLISHED_SCORECARDS.SCOPE_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCOPE_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_WEIGHT,NAME_SCOPE_WEIGHT);}

    /**
     * Column SCHEDULE_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHEDULE_SCORE = 30;

    public static final int TYPE_SCHEDULE_SCORE = Types.BIGINT;

    public static final String NAME_SCHEDULE_SCORE = "TMT_PUBLISHED_SCORECARDS.SCHEDULE_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHEDULE_SCORE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHEDULE_SCORE,NAME_SCHEDULE_SCORE);}

    /**
     * Column SCHEDULE_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHEDULE_WEIGHT = 31;

    public static final int TYPE_SCHEDULE_WEIGHT = Types.BIGINT;

    public static final String NAME_SCHEDULE_WEIGHT = "TMT_PUBLISHED_SCORECARDS.SCHEDULE_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHEDULE_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHEDULE_WEIGHT,NAME_SCHEDULE_WEIGHT);}

    /**
     * Column QUALITY_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_QUALITY_SCORE = 32;

    public static final int TYPE_QUALITY_SCORE = Types.BIGINT;

    public static final String NAME_QUALITY_SCORE = "TMT_PUBLISHED_SCORECARDS.QUALITY_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_QUALITY_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_QUALITY_SCORE,NAME_QUALITY_SCORE);}

    /**
     * Column QUALITY_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_QUALITY_WEIGHT = 33;

    public static final int TYPE_QUALITY_WEIGHT = Types.BIGINT;

    public static final String NAME_QUALITY_WEIGHT = "TMT_PUBLISHED_SCORECARDS.QUALITY_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_QUALITY_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_QUALITY_WEIGHT,NAME_QUALITY_WEIGHT);}

    /**
     * Column COST_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_COST_SCORE = 34;

    public static final int TYPE_COST_SCORE = Types.BIGINT;

    public static final String NAME_COST_SCORE = "TMT_PUBLISHED_SCORECARDS.COST_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COST_SCORE,NAME_COST_SCORE);}

    /**
     * Column COST_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_COST_WEIGHT = 35;

    public static final int TYPE_COST_WEIGHT = Types.BIGINT;

    public static final String NAME_COST_WEIGHT = "TMT_PUBLISHED_SCORECARDS.COST_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COST_WEIGHT,NAME_COST_WEIGHT);}

    /**
     * Column BENEFIT_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BENEFIT_SCORE = 36;

    public static final int TYPE_BENEFIT_SCORE = Types.BIGINT;

    public static final String NAME_BENEFIT_SCORE = "TMT_PUBLISHED_SCORECARDS.BENEFIT_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BENEFIT_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BENEFIT_SCORE,NAME_BENEFIT_SCORE);}

    /**
     * Column BENEFIT_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BENEFIT_WEIGHT = 37;

    public static final int TYPE_BENEFIT_WEIGHT = Types.BIGINT;

    public static final String NAME_BENEFIT_WEIGHT = "TMT_PUBLISHED_SCORECARDS.BENEFIT_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BENEFIT_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BENEFIT_WEIGHT,NAME_BENEFIT_WEIGHT);}

    /**
     * Column PROBABILITY_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PROBABILITY_TYPE = 38;

    public static final int TYPE_PROBABILITY_TYPE = Types.SMALLINT;

    public static final String NAME_PROBABILITY_TYPE = "TMT_PUBLISHED_SCORECARDS.PROBABILITY_TYPE";

    public static final String PROPERTY_PROBABILITY_TYPE = "PROPERTYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY_TYPE, SqlUtil.castAsChar(NAME_PROBABILITY_TYPE, 1));
    }

    /**
     * Column REPORT_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_REPORT_FLAG = 39;

    public static final int TYPE_REPORT_FLAG = Types.SMALLINT;

    public static final String NAME_REPORT_FLAG = "TMT_PUBLISHED_SCORECARDS.REPORT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REPORT_FLAG = "REPORT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REPORT_FLAG, NAME_REPORT_FLAG);
    }

    public static final String TABLE_NAME = "TMT_PUBLISHED_SCORECARDS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_PUBLISHED_SCORECARDS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_PROJECT_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_CREATED_BY, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_RANK, NAME_PARENT_RANK,
            NAME_DESCRIPTION, NAME_ORIGINAL_ID, NAME_SCORECARD_ID,
            NAME_CATEGORY_ID, NAME_SCORE, NAME_WEIGHT, NAME_CONVERTED_SCORE,
            NAME_CONVERTED_WEIGHT, NAME_SELECTED_ANSWER, NAME_PUBLISHED_DT,
            NAME_MAJOR_REVISION, NAME_ANSWER_DESCRIPTION, NAME_SCOPE_SCORE,
            NAME_SCOPE_WEIGHT, NAME_SCHEDULE_SCORE, NAME_SCHEDULE_WEIGHT,
            NAME_QUALITY_SCORE, NAME_QUALITY_WEIGHT, NAME_COST_SCORE,
            NAME_COST_WEIGHT, NAME_BENEFIT_SCORE, NAME_BENEFIT_WEIGHT,
            NAME_PROBABILITY_TYPE, NAME_REPORT_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new PublishedScorecardManager instance.
     *
     * @return PublishedScorecardManager
     */
    public PublishedScorecardManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        switch (typeId)
        {

        case AssignedScorecard.TYPE_ID:
            return new AssignedScorecard();

        case AssignedScorecardCategory.TYPE_ID:
            return new AssignedScorecardCategory();

        case AssignedScorecardQuestion.TYPE_ID:
            return new AssignedScorecardQuestion();

        case AssignedScorecardResponse.TYPE_ID:
            return new AssignedScorecardResponse();

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
            return null;
        }
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
    {
        // TODO Auto-generated method stub
        // I think this should not be implemented ???
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException
    {
        int typeId = rpmObject.retrieveTypeID().intValue();

        rpmObject.setID(rs.getString(ID_ELEMENT_ID));

        switch (typeId)
        {
        case AssignedScorecard.TYPE_ID:
            decodeScorecard(rs, (AssignedScorecard) rpmObject, isDelta);
            break;

        case AssignedScorecardCategory.TYPE_ID:
            decodeCategory(rs, (AssignedScorecardCategory) rpmObject, isDelta);
            break;

        case AssignedScorecardQuestion.TYPE_ID:
            decodeQuestion(rs, (AssignedScorecardQuestion) rpmObject, isDelta);
            break;

        case AssignedScorecardResponse.TYPE_ID:
            decodeResponse(rs, (AssignedScorecardResponse) rpmObject, isDelta);
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return rpmObject;
    }

    private void decodeScorecard(ResultSet rs, AssignedScorecard rpmObject,
            boolean isDelta) throws RPMException, SQLException
    {

        if (isDelta)
        {
            rpmObject.deltaProbabilityType(ProbabilityTypeID.getCategory(this,
                rs.getInt(ID_PROBABILITY_TYPE)));
            rpmObject.deltaScore(new Integer(rs.getInt(ID_SCORE)));

        }
        else
        {
            rpmObject.setProbabilityType(ProbabilityTypeID.getCategory(this, rs
                .getInt(ID_PROBABILITY_TYPE)));
            rpmObject.setScore(new Integer(rs.getInt(ID_SCORE)));
        }
        decodeScorecardElement(rs, rpmObject, isDelta);

    }

    private void decodeCategory(ResultSet rs,
            AssignedScorecardCategory rpmObject, boolean isDelta)
            throws RPMException, SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaProbabilityType(ProbabilityTypeID.getCategory(this,
                rs.getInt(ID_PROBABILITY_TYPE)));
            rpmObject.deltaScore(new Integer(rs.getInt(ID_SCORE)));

        }
        else
        {
            rpmObject.setProbabilityType(ProbabilityTypeID.getCategory(this, rs
                .getInt(ID_PROBABILITY_TYPE)));
            rpmObject.setScore(new Integer(rs.getInt(ID_SCORE)));
        }

        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeQuestion(ResultSet rs,
            AssignedScorecardQuestion rpmObject, boolean isDelta)
            throws SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaToReport(rs.getInt(ID_REPORT_FLAG) == 0
                    ? new Boolean(false) : new Boolean(true));
            rpmObject.deltaResponseAuthor(rs.getString(ID_REC_USER));
            rpmObject.deltaResponseDate(DateUtil.convertToCalendar(rs
                .getDate(ID_REC_DATETIME)));
            rpmObject.deltaScore(new Integer(rs.getInt(ID_SCORE)));
        }
        else
        {
            rpmObject.setToReport(rs.getInt(ID_REPORT_FLAG) == 0 ? new Boolean(
                false) : new Boolean(true));
            rpmObject.setResponseAuthor(rs.getString(ID_REC_USER));
            rpmObject.setResponseDate(DateUtil.convertToCalendar(rs
                .getDate(ID_REC_DATETIME)));
            rpmObject.setScore(new Integer(rs.getInt(ID_SCORE)));
        }
        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeResponse(ResultSet rs,
            AssignedScorecardResponse rpmObject, boolean isDelta)
            throws SQLException
    {
        Boolean selectedAnswer = new Boolean(rs.getString(ID_SELECTED_ANSWER)
            .charAt(0) == 'Y' ? true : false);
        if (isDelta)
        {
            rpmObject.deltaDescription(rs.getString(ID_DESCRIPTION));
            rpmObject.deltaSelectedAnswer(selectedAnswer);
            rpmObject.deltaComments(rs.getString(ID_DESCRIPTION));
            rpmObject.deltaScore(new Integer(rs.getInt(ID_SCORE)));
        }
        else
        {
            rpmObject.setDescription(rs.getString(ID_DESCRIPTION));
            rpmObject.setSelectedAnswer(selectedAnswer);
            rpmObject.setComments(rs.getString(ID_DESCRIPTION));
            rpmObject.setScore(new Integer(rs.getInt(ID_SCORE)));
        }
        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeScorecardElement(ResultSet rs,
            AbstractScorecardElement rpmObject, boolean isDelta)
            throws SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaName(rs.getString(ID_ELEMENT_NAME));
            rpmObject.deltaWeight(new Integer(rs.getInt(ID_WEIGHT)));
        }
        else
        {
            rpmObject.setName(rs.getString(ID_ELEMENT_NAME));
            rpmObject.setWeight(new Integer(rs.getInt(ID_WEIGHT)));
        }

    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.framework.util.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {
    	String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);
        return relatedFields;
    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext,
     *      com.ibm.rpm.framework.util.FieldValueMap, int, boolean)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        int typeId = rpmObject.retrieveTypeID().intValue();

        switch (typeId)
        {

        case AssignedScorecard.TYPE_ID:
        	loadParent(rpmObject, rpmScope, messageContext, relatedFields, index, isDelta);
            loadCategories((AssignedScorecard) rpmObject,
                (ScorecardScope) rpmScope, messageContext, isDelta);
            loadScorecardState((AssignedScorecard) rpmObject, messageContext,
                isDelta);

            break;

        case AssignedScorecardCategory.TYPE_ID:
            loadQuestions((AssignedScorecardCategory) rpmObject,
                (ScorecardCategoryScope) rpmScope, messageContext, isDelta);
            break;

        case AssignedScorecardQuestion.TYPE_ID:
            loadResponses((AssignedScorecardQuestion) rpmObject,
                (ScorecardQuestionScope) rpmScope, messageContext, isDelta);
            break;

        case ScorecardResponse.TYPE_ID:
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return rpmObject;
    }

    private void loadParent(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if (rpmScope != null && rpmScope.getParent() != null)
        {
            String contextName = rpmObject.getContextName();

            if (contextName != null)
            {
                RPMObject parent = rpmObject.getParent();
                AbstractRPMObjectManager manager = null;

                if(ContextUtil.ASSET_CONTEXT
                        .equalsIgnoreCase(contextName))
                {
                    manager = (AssetManager) getManagerInstance(Asset.class);
                    parent = new Asset();
                }
                else if (ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName))
                {
                    manager = (AbstractRPMObjectManager) getManagerInstance(GenericProject.class);
                    parent = new GenericProject();
                }
                else if (ContextUtil.SCOPE_CONTEXT
                    .equalsIgnoreCase(contextName))
                {
                    manager = (AbstractRPMObjectManager) getManagerInstance(ScopeElement.class);
                    ArrayList temp = manager.loadByForeignKey(new RPMObject[]
                    {
                        rpmObject.getParent()
                    }, rpmScope.getParent(), messageContext, this, new Object[]
                    {
                        rpmObject.getID()
                    }, null, contextName, isDelta);
                    // There can be only 1 parent
                    if (temp != null && temp.size() > 0)
                    {
                        parent = (RPMObject) temp.get(0);
                    }
                    else
                    {
                        throw new RPMException(
                            "loadParent : SCOPE context, cannot load parent of object with ID "
                                    + rpmObject.getID());   
                    }
                }
                else
                {
                    throw new RPMException("loadParent : Unsupported context "
                            + contextName);
                }
               
                //set parent ID
                if (manager != null
                        && parent!=null && parent.getID()==null)
                {
                    String ID = (String) relatedFields
                        .get(index, ID_PROJECT_ID);
                    parent.setID(ID);
                    parent = manager.loadByPrimaryKey(parent, rpmScope
                        .getParent(), messageContext, isDelta);
                }
                
                //delta or set parent
                if (parent != null)
                {
                    if (isDelta)
                    {
                        rpmObject.deltaParent(parent);
                    }
                    else
                    {
                        rpmObject.setParent(parent);
                    }
                }
            }
        }
    }
    
    private void loadScorecardState(AssignedScorecard rpmObject,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        ScorecardAssignmentManager manager = (ScorecardAssignmentManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(AssignedScorecardState.class);

        AssignedScorecardState state = rpmObject.getScorecardState();
        if (state == null)
        {
            state = new AssignedScorecardState();
            state.setID(rpmObject.getID());
        }

        state = (AssignedScorecardState) manager.loadByPrimaryKey(state, null,
            messageContext, isDelta);

        rpmObject.setScorecardState(state);
    }

    private void loadCategories(AssignedScorecard rpmObject,
            ScorecardScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardCategories() == null)
            return;

        ArrayList children = loadByForeignKey(rpmObject
            .getAssignedScorecardCategories(), rpmScope
            .getScorecardCategories(), messageContext, this, new Object[]
        {
            rpmObject.getID()
        }, NAME_TYPE_ID + "=" + AssignedScorecardCategory.TYPE_ID, rpmObject
            .getContextName(), isDelta);

        if (children != null)
        {
            Iterator it = children.iterator();
            AssignedScorecardCategory[] categories = new AssignedScorecardCategory[children
                .size()];
            for (int i = 0; i < categories.length; i++)
            {
                AssignedScorecardCategory element = (AssignedScorecardCategory) it
                    .next();
                element.setParent(rpmObject);
                categories[i] = element;
            }

            rpmObject.setAssignedScorecardCategories(categories);
        }
    }

    private void loadQuestions(AssignedScorecardCategory rpmObject,
            ScorecardCategoryScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardQuestions() == null)
            return;

        ArrayList children = loadByForeignKey(rpmObject
            .getAssignedScorecardQuestions(), rpmScope.getScorecardQuestions(),
            messageContext, this, new Object[]
            {
                rpmObject.getID()
            }, NAME_TYPE_ID + "=" + AssignedScorecardQuestion.TYPE_ID,
            rpmObject.getContextName(), isDelta);
        if (children != null)
        {
            Iterator it = children.iterator();
            AssignedScorecardQuestion[] questions = new AssignedScorecardQuestion[children
                .size()];
            for (int i = 0; i < questions.length; i++)
            {
                AssignedScorecardQuestion element = (AssignedScorecardQuestion) it
                    .next();
                element.setParent(rpmObject);
                questions[i] = element;
            }
            rpmObject.setAssignedScorecardQuestions(questions);
        }
    }

    private void loadResponses(AssignedScorecardQuestion rpmObject,
            ScorecardQuestionScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardResponses() == null)
            return;

        ArrayList children = loadByForeignKey(rpmObject
            .getAssignedScorecardResponses(), rpmScope.getScorecardResponses(),
            messageContext, this, new Object[]
            {
                rpmObject.getID()
            }, NAME_TYPE_ID + "=" + AssignedScorecardResponse.TYPE_ID,
            rpmObject.getContextName(), isDelta);
        if (children != null)
        {
            Iterator it = children.iterator();
            AssignedScorecardResponse[] responses = new AssignedScorecardResponse[children
                .size()];
            for (int i = 0; i < responses.length; i++)
            {
                AssignedScorecardResponse element = (AssignedScorecardResponse) it
                    .next();
                element.setParent(rpmObject);
                responses[i] = element;
            }
            rpmObject.setAssignedScorecardResponses(responses);
        }
    }

    /**
     *
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject instanceof AssignedScorecard)
        {
            if (rpmObject.getParent() instanceof AbstractScope)
                SP_UA_SC((AssignedScorecard) rpmObject, messageContext);
            else
                SP_D_SC((AssignedScorecard) rpmObject, messageContext);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = new JoinCondition();

        if (manager instanceof PublishedScorecardManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PARENT_ID);
            condition.append(" = ?");
            if (extraWhere != null)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }

            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }
        else if (manager instanceof AssetManager
                || manager instanceof WbsManager )
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PROJECT_ID);
            condition.append(" = ? AND ");
            condition.append(NAME_TYPE_ID);
            condition.append(" = ").append(Scorecard.TYPE_ID);

            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }
        else if (manager instanceof ScopeManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_TYPE_ID);
            condition.append(" = ").append(Scorecard.TYPE_ID);
            condition.append(" AND ").append(NAME_PARENT_RANK);
            condition.append(" IN (SELECT 'CRI#'||").append(
                ScopeManager.NAME_RANK);
            condition.append(" from TMT_CRI where element_id = ?)");
            condition.append(extraWhere);
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }
        else if(manager instanceof ResourceManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PROJECT_ID);
            condition.append(" = ? AND ");
            condition.append(NAME_PARENT_ID);
            condition.append(" = ? AND ");
            condition.append(NAME_TYPE_ID);
            condition.append(" = ").append(AbstractScorecardCategory.TYPE_ID);
           
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // if this is not a legal context for scorecards, we should return null
        if (!ScorecardUtil.isLegalContext(contextName))
            return null;

        JoinCondition joinCondition = new JoinCondition();

        int typeId = getTypeId(containerName, ExceptionUtil.JOIN);

        if (manager == null)
        {
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
            joinCondition.setTableName(getTableName(contextName));

        }
        else if (manager instanceof PublishedScorecardManager)
        {
            StringBuffer where = new StringBuffer();

            where.append(NAME_PARENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ELEMENT_ID);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    /**
     * @param containerName
     * @return
     * @throws RPMException
     */
    private int getTypeId(String containerName, String methodName)
            throws RPMException
    {
        int typeId = 0;

        if (containerName == null)
        {
            ExceptionUtil.handleMethodException(this,
                ErrorCodes.NO_CONTAINER_NAME, methodName);
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(AssignedScorecard.class))
            || containerName.equalsIgnoreCase(StringUtil.
                getShortClassName(ResourceAssignedScorecard.class)))
        {
            typeId = AssignedScorecard.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(AssignedScorecardCategory.class)))
        {
            typeId = AssignedScorecardCategory.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(AssignedScorecardQuestion.class)))
        {
            typeId = AssignedScorecardQuestion.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(AssignedScorecardResponse.class)))
        {
            typeId = AssignedScorecardResponse.TYPE_ID;
        }
        else
        {
            ExceptionUtil.handleNoTypeId(this, containerName);
        }
        return typeId;
    }

    /**
     *
     */

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // if this is not a legal context for scorecards, we should return null
        if (!ScorecardUtil.isLegalContext(contextName))
            return null;

        JoinCondition joinCondition = new JoinCondition();

        if (manager instanceof PublishedScorecardManager)
        {
            int typeId = getTypeId(containerName, ExceptionUtil.SUB_TABLE_JOIN);

            StringBuffer where = new StringBuffer();

            where.append(NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_PARENT_ID);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof AssetManager
                || manager instanceof WbsManager)
        {
            String joinColumn;
            if (manager instanceof AssetManager)
                joinColumn = AssetManager.NAME_ELEMENT_ID;
            else
                joinColumn = WbsManager.NAME_ELEMENT_ID;

            StringBuffer where = new StringBuffer();

            where.append(joinColumn).append(" IN (");
            where.append(" SELECT ").append(NAME_PROJECT_ID);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(AssignedScorecard.TYPE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ScopeManager)
        {
            String joinColumn = ScopeManager.NAME_RANK;

            StringBuffer where = new StringBuffer();

            where.append(joinColumn).append(" IN (");
            where.append(" SELECT ").append(NAME_RANK);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(AssignedScorecard.TYPE_ID);

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    /**
     * @throws RPMException
     * @throws SQLException
     * @throws ParseException
     *
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        int typeId = rpmObject.retrieveTypeID().intValue();

        switch (typeId)
        {
        case AssignedScorecard.TYPE_ID:
            rpmObject = updateScorecard((AssignedScorecard) rpmObject,
                (ScorecardScope) rpmScope, messageContext);
            break;

        case AssignedScorecardCategory.TYPE_ID:
            rpmObject = updateCategory((AssignedScorecardCategory) rpmObject,
                (ScorecardCategoryScope) rpmScope, messageContext);
            break;

        case AssignedScorecardQuestion.TYPE_ID:
            rpmObject = updateQuestion((AssignedScorecardQuestion) rpmObject,
                (ScorecardQuestionScope) rpmScope, messageContext);
            break;

        case AssignedScorecardResponse.TYPE_ID:
            rpmObject = updateResponse((AssignedScorecardResponse) rpmObject,
                messageContext);
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);

        }

        return rpmObject;

    }

    public RPMObject updateScorecard(AssignedScorecard rpmObject,
            ScorecardScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject.testNameModified())
        {
            SP_U_SC_NAME(rpmObject, messageContext);
        }

        if (scope != null)
        {
            rpmObject
                .setAssignedScorecardCategories((AssignedScorecardCategory[]) updateChildren(
                    rpmObject, rpmObject.getAssignedScorecardCategories(),
                    scope.getScorecardCategories(), messageContext, null));
        }

        // the ScorecardState is not really a child, it is more like a sibling.
        // in any case it should always be loaded once the AssignedScorecard is loaded
        // thus we need to pass a none null scope to updateChild
        rpmObject.setScorecardState((AssignedScorecardState) updateChild(
            rpmObject, rpmObject.getScorecardState(), scope == null
                    ? new ScorecardScope() : scope, messageContext));

        return rpmObject;

    }

    public RPMObject updateCategory(AssignedScorecardCategory rpmObject,
            ScorecardCategoryScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope != null)
        {
            rpmObject
                .setAssignedScorecardQuestions((AssignedScorecardQuestion[]) updateChildren(
                    rpmObject, rpmObject.getAssignedScorecardQuestions(), scope
                        .getScorecardQuestions(), messageContext, null));
        }
        return rpmObject;
    }

    public RPMObject updateQuestion(AssignedScorecardQuestion question,
            ScorecardQuestionScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope != null)
        {
            question
                .setAssignedScorecardResponses((AssignedScorecardResponse[]) updateChildren(
                    question, question.getAssignedScorecardResponses(), scope
                        .getScorecardResponses(), messageContext, null));
        }
        return question;
    }

    public RPMObject updateResponse(AssignedScorecardResponse response,
            MessageContext messageContext) throws SQLException, RPMException
    {

        if (response.getSelectedAnswer().booleanValue())
        {

            if (((AssignedScorecardQuestion) response.getParent())
                .testToReportModified()
                    || response.testCommentsModified()
                    || response.testSelectedAnswerModified())
            {
                SP_ANS_SC((AssignedScorecardQuestion) response.getParent(),
                    response, messageContext);
            }
        }
        return response;
    }

    public RPMObject assignScoreCard(RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        RPMObject outputObject = null;
        
        boolean valid = ScorecardMethodCheckpoint.getInstance()
            .validateScorecardAssignment(objectToAssign, objectToAssignScope,
                scoreCardToAssign, reloadType, messageContext)
                && ScorecardMethodCheckpoint.getInstance().canAssignScorecard(
                    objectToAssign, objectToAssignScope, scoreCardToAssign,
                    reloadType, messageContext);
        
        if (valid)
        {
            String assignedScorecardID = null;
            objectToAssign.setContextName(scoreCardToAssign.getContextName());
            
            if (objectToAssign instanceof GenericProject)
            {
                assignedScorecardID = SP_I_SC((GenericProject) objectToAssign,
                    scoreCardToAssign, messageContext);
            }

            SP_A_SC_440(objectToAssign, assignedScorecardID, scoreCardToAssign, null,
                messageContext);

            assignedScorecardID = scoreCardToAssign.getID();

            //Sets the default reloadType
            if (reloadType == null)
            {
                reloadType = ReloadType.ReloadResult;
            }

            outputObject = objectToAssign;
            if (reloadType != ReloadType.None)
            {
                RPMObjectScope loadingScope = null;
                if (reloadType == ReloadType.SavedResult)
                {
                    loadingScope = createScope(objectToAssign);
                }
                else if (reloadType == ReloadType.ReloadResult)
                {
                    loadingScope = objectToAssignScope;
                }

                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(objectToAssign);

                outputObject = manager.load(objectToAssign, loadingScope,
                    messageContext);
            }
        }
        return outputObject;
    }
    
    public RPMObject assignScoreCardToResource(Resource objectToAssign,
            GenericProject project, ResourceScope objectToAssignScope, 
            Scorecard scoreCardToAssign, ReloadType reloadType, MessageContext messageContext) 
            throws RPMException, SQLException, ParseException
    {
       Resource outputObject = null;
        
        boolean valid = ScorecardMethodCheckpoint.getInstance()
            .validateResourceScorecardAssignment(objectToAssign, objectToAssignScope,
                scoreCardToAssign, reloadType, project, messageContext)
                && ScorecardMethodCheckpoint.getInstance().canAssignScorecard(
                    objectToAssign, objectToAssignScope, scoreCardToAssign,
                    reloadType, messageContext);
        
        if (valid)
        {
            String assignedScorecardID = null;

            SP_A_SC_440(objectToAssign, assignedScorecardID, scoreCardToAssign, project,
                messageContext);

            assignedScorecardID = scoreCardToAssign.getID();

            //Sets the default reloadType
            if (reloadType == null)
            {
                reloadType = ReloadType.ReloadResult;
            }

            
            if (reloadType != ReloadType.None)
            {
                ResourceScope loadingScope = null;
                if (reloadType == ReloadType.SavedResult)
                {
                    loadingScope = new ResourceScope();
                    loadingScope.setAssignedScorecard(new ScorecardScope());
                }
                else if (reloadType == ReloadType.ReloadResult)
                {
                    loadingScope = objectToAssignScope;
                }

                
                ArrayList results = messageContext.getFactory().getManagerCaller().load(messageContext.getSession(), 
                    "/Resource[@id='" + objectToAssign.getID() + 
                    "']/ResourceAssignedScorecard[@project[@id='" + project.getID() + "']]", null).getObjects();
                
                if(results != null && results.size() == 1)
                {
                    outputObject = objectToAssign;
                    outputObject.setAssignedScorecard((AssignedScorecard) results.get(0));
                }
            }
        }
        
        return outputObject;
    }

    private RPMObjectScope createScope(RPMObject objectToAssign)
    {
        ScorecardResponseScope rScope = new ScorecardResponseScope();
        ScorecardQuestionScope qScope = new ScorecardQuestionScope();
        ScorecardCategoryScope cScope = new ScorecardCategoryScope();
        ScorecardScope sScope = new ScorecardScope();

        rScope.setParent(qScope);
        qScope.setParent(cScope);
        cScope.setParent(sScope);

        sScope.setScorecardCategories(cScope);
        cScope.setScorecardQuestions(qScope);
        qScope.setScorecardResponses(rScope);

        if (objectToAssign instanceof Asset)
        {
            AssetScope scope = new AssetScope();
            scope.setAssignedScorecard(sScope);
            return scope;
        }
        else if (objectToAssign instanceof GenericProject)
        {
            WorkElementScope scope = new WorkElementScope();
            scope.setAssignedScorecards(sScope);
            return scope;
        }
        else if (objectToAssign instanceof AbstractScope)
        {
            ScopeElementScope scope = new ScopeElementScope();
            scope.setAssignedScorecard(sScope);
            return scope;
        }

        // we can't get here because it have been validated before
        return null;
    }

    private String SP_I_SC(GenericProject project, Scorecard scorecard,
            MessageContext context) throws RPMException
    {
        String name = "SP_I_SC";
        Object args[] = new Object[6];
        args[0] = ScorecardUtil.getTableType(scorecard.getContextName());
        args[1] = scorecard.getName();
        args[2] = project.getID();
        args[3] = project.getID();
        args[4] = scorecard.retrieveTypeID();
        args[5] = getUser(context).getID();

        try
        {
            return executeProcedure(context, name, args);
        }
        catch (SQLException e)
        {
            ExceptionUtil.handleException(this, e);
            return null;
        }
    }

    private String SP_A_SC_440(RPMObject obj, String assignScorecardID,
            Scorecard scorecard, GenericProject project, 
            MessageContext context) throws RPMException
    {
        String name = "SP_A_SC_440";
        Object args[] = new Object[5];
        args[0] = ScorecardUtil.getTableType(scorecard.getContextName());
        args[1] = (project != null ? project.getID() : obj.getID());

        //      for projects there is a scorecard id already and it should be set
        // here, for assets/CRI there is no scorecard id yet thus the asset id
        // should be set
        args[2] = (assignScorecardID == null || assignScorecardID.equals(""))
                ? obj.getID() : assignScorecardID;
        args[3] = scorecard.getID();
        args[4] = getUser(context).getID();

        try
        {
            return executeProcedure(context, name, args);
        }
        catch (SQLException e)
        {
            ExceptionUtil.handleException(this, e);
            return null;
        }
    }

    private void SP_ANS_SC(AssignedScorecardQuestion question,
            AssignedScorecardResponse response, MessageContext context)
            throws SQLException, RPMException
    {
        String name = "SP_ANS_SC";
        Object args[] = new Object[7];
        args[0] = ScorecardUtil.getTableType(question.getContextName());
        args[1] = question.getID();
        args[2] = response.getID();
        args[3] = response.getComments();
        args[4] = question.getToReport().booleanValue() ? new Integer(1)
                : new Integer(0);
        args[5] = new Integer(0);
        args[6] = getUser(context).getID();

        executeProcedure(context, name, args);
    }

    private void SP_D_SC(AssignedScorecard aScorecard, MessageContext context)
            throws SQLException, RPMException
    {
        String name = "SP_D_SC";
        Object args[] = new Object[5];
        args[0] = ScorecardUtil.getTableType(aScorecard.getContextName());

        RPMObject parent = aScorecard.getParent();

        if (parent instanceof Asset)
        {
            args[1] = aScorecard.getID();
            args[2] = "ASSET_FOLDER____________________";
        }
        else if (parent instanceof GenericProject)
        {
            args[1] = aScorecard.getID();
            args[2] = parent.getID();
        }
        else if (parent instanceof AbstractScope)
        {
            args[1] = parent.getID();
            args[2] = aScorecard.getID();
        }

        args[3] = "X";
        args[4] = getUser(context).getID();

        executeProcedure(context, name, args);
    }

    private void SP_UA_SC(AssignedScorecard scorecard, MessageContext context)
            throws SQLException, RPMException
    {
        String name = "SP_UA_SC";
        Object args[] = new Object[4];
        args[0] = ScorecardUtil.getTableType(scorecard.getParent()
            .getContextName());
        args[1] = scorecard.getParent().getID();
        args[2] = scorecard.getID();
        args[3] = getUser(context).getID();

        executeProcedure(context, name, args);
    }

    private void SP_U_SC_NAME(AssignedScorecard scorecard,
            MessageContext context) throws SQLException, RPMException
    {
        String name = "SP_U_SC_NAME";
        Object args[] = new Object[3];
        args[0] = scorecard.getID();
        args[1] = scorecard.getName();
        args[2] = getUser(context).getID();

        executeProcedure(context, name, args);
    }



}