package com.ibm.rpm.scorecard.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import com.ibm.rpm.asset.managers.AssetModuleManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.managers.ResourceModuleManager;
import com.ibm.rpm.scopemanagement.managers.ScopeManagementModuleManager;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.containers.AbstractScorecardEntity;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.managers.ScorecardUtil.ProbabilityTypeID;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.managers.WbsModuleManager;

public class ScorecardManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();
    static
    {

        CONTAINERS.add(ScorecardFolder.class.getName());
        CONTAINERS.add(Scorecard.class.getName());
        CONTAINERS.add(ScorecardCategory.class.getName());
        CONTAINERS.add(ScorecardQuestion.class.getName());
        CONTAINERS.add(ScorecardResponse.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ScorecardManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "ELEMENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 12;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 13;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column WEIGHT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WEIGHT = 14;

    public static final int TYPE_WEIGHT = Types.SMALLINT;

    public static final String NAME_WEIGHT = "WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_WEIGHT = "WEIGHT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEIGHT, NAME_WEIGHT);
    }

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 15;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column DEFAULT_ANSWER of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_ANSWER = 16;

    public static final int TYPE_DEFAULT_ANSWER = Types.CHAR;

    public static final String NAME_DEFAULT_ANSWER = "DEFAULT_ANSWER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DEFAULT_ANSWER = "DEFAULTRESPONSE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_ANSWER, NAME_DEFAULT_ANSWER);
    }

    /**
     * Column ANSWER_DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ANSWER_DESCRIPTION = 17;

    public static final int TYPE_ANSWER_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_ANSWER_DESCRIPTION = "ANSWER_DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ANSWER_DESCRIPTION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ANSWER_DESCRIPTION,NAME_ANSWER_DESCRIPTION);}

    /**
     * Column PROBABILITY_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PROBABILITY_TYPE = 18;

    public static final int TYPE_PROBABILITY_TYPE = Types.SMALLINT;

    public static final String NAME_PROBABILITY_TYPE = "PROBABILITY_TYPE";

    public static final String PROPERTY_PROBABILITY_TYPE = "PROBABILITYTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY_TYPE, SqlUtil.castAsChar(NAME_PROBABILITY_TYPE, 1));
    }

    /**
     * Column PROBABILITY_TYPE_CHANGED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PROBABILITY_TYPE_CHANGED = 19;

    public static final int TYPE_PROBABILITY_TYPE_CHANGED = Types.SMALLINT;

    public static final String NAME_PROBABILITY_TYPE_CHANGED = "PROBABILITY_TYPE_CHANGED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROBABILITY_TYPE_CHANGED =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY_TYPE_CHANGED,NAME_PROBABILITY_TYPE_CHANGED);}

    /**
     * Column REPORT_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_REPORT_FLAG = 20;

    public static final int TYPE_REPORT_FLAG = Types.SMALLINT;

    public static final String NAME_REPORT_FLAG = "REPORT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REPORT_FLAG = "TOREPORT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REPORT_FLAG, NAME_REPORT_FLAG);
    }

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 21;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "CONFIG_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONFIG_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG,NAME_CONFIG_FLAG);}

    /**
     * Create an array of type string containing all the fields of the
     * TMT_SCORECARDS_DEF_WBS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_RANK, NAME_DESCRIPTION, NAME_WEIGHT, NAME_CONVERTED_WEIGHT,
            NAME_DEFAULT_ANSWER, NAME_ANSWER_DESCRIPTION,
            NAME_PROBABILITY_TYPE, NAME_PROBABILITY_TYPE_CHANGED,
            NAME_REPORT_FLAG, NAME_CONFIG_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new WBSScorecardManager instance.
     * 
     * @return WBSScorecardManager
     */
    public ScorecardManager()
    {
        super();
    }

    public String getTableName()
    {
        ExceptionUtil.handleGetTableName(this);
        return null;
    }

    public String getTableName(RPMObject rpmObject)
    {
        if (rpmObject.getContextName() == null)
        {
            // find out the context name by examining the module type
            rpmObject.setContextName(ScorecardUtil
                .getContextNameFromModule((AbstractScorecardEntity) rpmObject));
        }
        return ScorecardUtil.getTableName(rpmObject.getContextName());
    }

    public String getTableName(String contextName)
    {
        return ScorecardUtil.getTableName(contextName);
    }

    protected Integer getTableType(RPMObject rpmObject)
    {
        if (rpmObject.getContextName() == null)
        {
            // find out the context name by examining the module type
            rpmObject.setContextName(ScorecardUtil
                .getContextNameFromModule((AbstractScorecardEntity) rpmObject));
        }
        return ScorecardUtil.getTableType(rpmObject.getContextName());
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {

        int typeId = resultSet.getInt(NAME_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        switch (typeId)
        {

        case ScorecardFolder.TYPE_ID:
            return new ScorecardFolder();

        case Scorecard.TYPE_ID:
            return new Scorecard();

        case ScorecardCategory.TYPE_ID:
            return new ScorecardCategory();

        case ScorecardQuestion.TYPE_ID:
            return new ScorecardQuestion();

        case ScorecardResponse.TYPE_ID:
            return new ScorecardResponse();

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
            return null;
        }

    }

    /**
     *  
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        // Set default field values
        if (rpmObject instanceof Scorecard)
        {
            if (((Scorecard) rpmObject).getActive() == null)
            {
                ((Scorecard) rpmObject).setActive(new Boolean(false));
            }
            if (((Scorecard) rpmObject).getProbabilityType() == null)
            {
                ((Scorecard) rpmObject)
                    .setProbabilityType(ProbabilityType.None);
            }
            if (((Scorecard) rpmObject).getWeight() == null)
            {
                ((Scorecard) rpmObject).setWeight(new Integer(1));
            }
        }
        if (rpmObject instanceof ScorecardCategory)
        {
            if (((ScorecardCategory) rpmObject).getProbabilityType() == null)
            {
                ((ScorecardCategory) rpmObject)
                    .setProbabilityType(ProbabilityType.None);
            }
            if (((ScorecardCategory) rpmObject).getWeight() == null)
            {
                ((ScorecardCategory) rpmObject).setWeight(new Integer(1));
            }
        }
        if (rpmObject instanceof ScorecardQuestion)
        {
            if (((ScorecardQuestion) rpmObject).getToReport() == null)
            {
                ((ScorecardQuestion) rpmObject).setToReport(new Boolean(false));
            }
            if (((ScorecardQuestion) rpmObject).getWeight() == null)
            {
                ((ScorecardQuestion) rpmObject).setWeight(new Integer(1));
            }
        }
        if (rpmObject instanceof ScorecardResponse)
        {
            if (((ScorecardResponse) rpmObject).getDefaultResponse() == null)
            {
                ((ScorecardResponse) rpmObject).setDefaultResponse(new Boolean(
                    false));
            }
            if (((ScorecardResponse) rpmObject).getWeight() == null)
            {
                ((ScorecardResponse) rpmObject).setWeight(new Integer(0));
            }
        }

        rpmObject.setID(SP_I_SCORECARD_ENTITY(
            (AbstractScorecardEntity) rpmObject, messageContext));
    }

    /**
     *  
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException
    {
        int typeId = rpmObject.retrieveTypeID().intValue();

        rpmObject.setID(rs.getString(ID_ELEMENT_ID));

        switch (typeId)
        {
        case ScorecardFolder.TYPE_ID:
            decodeFolder(rs, (ScorecardFolder) rpmObject, isDelta);
            break;

        case Scorecard.TYPE_ID:
            decodeScorecard(rs, (Scorecard) rpmObject, isDelta);
            break;

        case ScorecardCategory.TYPE_ID:
            decodeCategory(rs, (ScorecardCategory) rpmObject, isDelta);
            break;

        case ScorecardQuestion.TYPE_ID:
            decodeQuestion(rs, (ScorecardQuestion) rpmObject, isDelta);
            break;

        case ScorecardResponse.TYPE_ID:
            decodeResponse(rs, (ScorecardResponse) rpmObject, isDelta);
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return rpmObject;
    }

    private void decodeFolder(ResultSet rs, ScorecardFolder rpmObject,
            boolean isDelta) throws SQLException
    {
        decodeScorecardEntity(rs, rpmObject, isDelta);
    }

    private void decodeScorecard(ResultSet rs, Scorecard rpmObject,
            boolean isDelta) throws RPMException, SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaActive(rs.getInt(NAME_CONFIG_FLAG) == 0
                    ? new Boolean(false) : new Boolean(true));
            rpmObject.deltaProbabilityType(ProbabilityTypeID.getCategory(this,
                rs.getInt(ID_PROBABILITY_TYPE)));
        }
        else
        {
            rpmObject.setActive(rs.getInt(NAME_CONFIG_FLAG) == 0 ? new Boolean(
                false) : new Boolean(true));
            rpmObject.setProbabilityType(ProbabilityTypeID.getCategory(this, rs
                .getInt(ID_PROBABILITY_TYPE)));
        }
        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeCategory(ResultSet rs, ScorecardCategory rpmObject,
            boolean isDelta) throws RPMException, SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaProbabilityType(ProbabilityTypeID.getCategory(this,
                rs.getInt(ID_PROBABILITY_TYPE)));
        }
        else
        {
            rpmObject.setProbabilityType(ProbabilityTypeID.getCategory(this, rs
                .getInt(ID_PROBABILITY_TYPE)));
        }

        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeQuestion(ResultSet rs, ScorecardQuestion rpmObject,
            boolean isDelta) throws SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaToReport(rs.getInt(ID_REPORT_FLAG) == 0
                    ? new Boolean(false) : new Boolean(true));
        }
        else
        {
            rpmObject.setToReport(rs.getInt(ID_REPORT_FLAG) == 0 ? new Boolean(
                false) : new Boolean(true));
        }
        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeResponse(ResultSet rs, ScorecardResponse rpmObject,
            boolean isDelta) throws SQLException
    {
        Boolean defAnswer = rs.getString(ID_DEFAULT_ANSWER).toCharArray()[0] == 'Y'
                ? new Boolean(true) : new Boolean(false);
        if (isDelta)
        {
            rpmObject.deltaDefaultResponse(defAnswer);
        }
        else
        {
            rpmObject.setDefaultResponse(defAnswer);
        }
        decodeScorecardElement(rs, rpmObject, isDelta);
    }

    private void decodeScorecardElement(ResultSet rs,
            AbstractScorecardElement rpmObject, boolean isDelta)
            throws SQLException
    {
        decodeScorecardEntity(rs, rpmObject, isDelta);
        if (isDelta)
        {
            rpmObject.deltaWeight(new Integer(rs.getInt(ID_WEIGHT)));
        }
        else
        {
            rpmObject.setWeight(new Integer(rs.getInt(ID_WEIGHT)));
        }

    }

    private void decodeScorecardEntity(ResultSet rs,
            AbstractScorecardEntity rpmObject, boolean isDelta)
            throws SQLException
    {
        if (isDelta)
        {
            rpmObject.deltaDescription(rs.getString(ID_DESCRIPTION));
            rpmObject.deltaName(rs.getString(ID_ELEMENT_NAME));
        }
        else
        {
            rpmObject.setDescription(rs.getString(ID_DESCRIPTION));
            rpmObject.setName(rs.getString(ID_ELEMENT_NAME));
        }

    }

    /**
     *  
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject != null)
        {
            SP_D_SCORECARD_ELEMENT(rpmObject, messageContext);
        }

    }

    /**
     *  
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = new JoinCondition();

        if (manager instanceof ScorecardManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PARENT_ID);
            condition.append(" = ?");

            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }
        else if (isSupportedModuleManager(manager))
        {
            joinCondition = new JoinCondition(getTableName(contextName));
            joinCondition.appendEqual(NAME_LEVEL_ID, 0);
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;

    }

    private boolean isSupportedModuleManager(RPMObjectManager manager)
    {
        return manager instanceof WbsModuleManager
                || manager instanceof AssetModuleManager
                || manager instanceof ScopeManagementModuleManager
                || manager instanceof ResourceModuleManager;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // if this is not a legal context for scorecards, we should return null
        if (!ScorecardUtil.isLegalContext(contextName))
            return null;

        JoinCondition joinCondition = new JoinCondition();

        int typeId = getTypeId(containerName, ExceptionUtil.JOIN);

        if (manager == null)
        {
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
            joinCondition.setTableName(getTableName(contextName));

        }
        else if (manager instanceof ScorecardManager)
        {
            StringBuffer where = new StringBuffer();

            where.append(NAME_PARENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ELEMENT_ID);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (isSupportedModuleManager(manager))
        {
            joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // if this is not a legal context for scorecards, we should return null
        if (!ScorecardUtil.isLegalContext(contextName))
            return null;

        JoinCondition joinCondition = new JoinCondition();

        int typeId = getTypeId(containerName, ExceptionUtil.SUB_TABLE_JOIN);

        if (manager instanceof ScorecardManager)
        {
            StringBuffer where = new StringBuffer();

            where.append(NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_PARENT_ID);
            where.append(" FROM ").append(getTableName(contextName));
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(typeId);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    /**
     * @param containerName
     * @return
     * @throws RPMException
     */
    private int getTypeId(String containerName, String methodName)
            throws RPMException
    {
        int typeId = 0;

        if (containerName == null)
        {
            ExceptionUtil.handleMethodException(this,
                ErrorCodes.NO_CONTAINER_NAME, methodName);
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(ScorecardFolder.class)))
        {
            typeId = ScorecardFolder.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(Scorecard.class)))
        {
            typeId = Scorecard.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(ScorecardCategory.class)))
        {
            typeId = ScorecardCategory.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(ScorecardQuestion.class)))
        {
            typeId = ScorecardQuestion.TYPE_ID;
        }
        else if (containerName.equalsIgnoreCase(StringUtil
            .getShortClassName(ScorecardResponse.class)))
        {
            typeId = ScorecardResponse.TYPE_ID;
        }
        else
        {
            ExceptionUtil.handleNoTypeId(this, containerName);
        }
        return typeId;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        int typeId = rpmObject.retrieveTypeID().intValue();

        switch (typeId)
        {
        case ScorecardFolder.TYPE_ID:
            rpmObject = updateFolder((ScorecardFolder) rpmObject,
                (ScorecardFolderScope) rpmScope, messageContext);
            break;

        case Scorecard.TYPE_ID:
            rpmObject = updateScorecard((Scorecard) rpmObject,
                (ScorecardScope) rpmScope, messageContext);
            break;

        case ScorecardCategory.TYPE_ID:
            rpmObject = updateCategory((ScorecardCategory) rpmObject,
                (ScorecardCategoryScope) rpmScope, messageContext);
            break;

        case ScorecardQuestion.TYPE_ID:
            rpmObject = updateQuestion((ScorecardQuestion) rpmObject,
                (ScorecardQuestionScope) rpmScope, messageContext);
            break;

        case ScorecardResponse.TYPE_ID:
            rpmObject = updateResponse((ScorecardResponse) rpmObject,
                messageContext);
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return rpmObject;

    }

    public boolean updateEntity(AbstractScorecardEntity rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        if (rpmObject.testDescriptionModified() || rpmObject.testNameModified())
        {
            SP_U_SCORECARD(rpmObject, messageContext);
            return true;
        }

        return false;
    }

    private RPMObject updateFolder(ScorecardFolder object,
            ScorecardFolderScope scope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        updateEntity(object, messageContext);
        if (scope != null)
        {

            propagateContextName(object, object.getScorecards());
            object.setScorecards((Scorecard[]) updateChildren(object, object
                .getScorecards(), scope.getScorecards(), messageContext, null));
        }
        return object;

    }

    private RPMObject updateScorecard(Scorecard object, ScorecardScope scope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // if it was not updated and the probability type modified
        if (!updateEntity(object, messageContext)
                && object.testProbabilityTypeModified())
        {
            SP_U_SCORECARD(object, messageContext);
        }

        if (object.testActiveModified())
        {
            SP_U_SC_CONFIG_FLAG(object, messageContext);
        }

        if (object.testWeightModified())
        {
            SP_U_SC_WEIGHT(object, messageContext);
        }

        if (scope != null)
        {
            propagateContextName(object, object.getScorecardCategories());
            object.setScorecardCategories((ScorecardCategory[]) updateChildren(
                object, object.getScorecardCategories(), scope
                    .getScorecardCategories(), messageContext, null));
        }

        return object;

    }

    public RPMObject updateCategory(ScorecardCategory object,
            ScorecardCategoryScope scope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (!updateEntity(object, messageContext)
                && object.testProbabilityTypeModified())
        {
            SP_U_SCORECARD(object, messageContext);
        }

        if (object.testWeightModified())
        {
            SP_U_SC_WEIGHT(object, messageContext);
        }

        if (scope != null)
        {
            propagateContextName(object, object.getScorecardQuestions());
            object.setScorecardQuestions((ScorecardQuestion[]) updateChildren(
                object, object.getScorecardQuestions(), scope
                    .getScorecardQuestions(), messageContext, null));
        }

        return object;

    }

    public RPMObject updateQuestion(ScorecardQuestion object,
            ScorecardQuestionScope scope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (!updateEntity(object, messageContext)
                || object.testToReportModified())
        {
            SP_U_SCORECARD(object, messageContext);
        }

        if (object.testWeightModified())
        {
            SP_U_SC_WEIGHT(object, messageContext);
        }

        if (scope != null)
        {
            propagateContextName(object, object.getScorecardResponses());
            object.setScorecardResponses((ScorecardResponse[]) updateChildren(
                object, object.getScorecardResponses(), scope
                    .getScorecardResponses(), messageContext, null));
        }

        return object;

    }

    public RPMObject updateResponse(ScorecardResponse rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        updateEntity(rpmObject, messageContext);

        if (rpmObject.testWeightModified())
        {
            SP_U_SC_WEIGHT(rpmObject, messageContext);
        }

        // we run it only when the default is changed from false to true
        if (rpmObject.testDefaultResponseModified()
                && rpmObject.getDefaultResponse().booleanValue())
        {
            SP_U_SC_DEFAULT(rpmObject, messageContext);
        }

        return rpmObject;
    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.framework.util.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext,
     *      com.ibm.rpm.framework.util.FieldValueMap, int, boolean)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        int typeId = rpmObject.retrieveTypeID().intValue();

        AbstractScorecardScope scScope = (AbstractScorecardScope) rpmScope;

        String parentId = (String) relatedFields.get(index, ID_PARENT_ID);
        RPMObject parentObject = null;
        switch (typeId)
        {
        case ScorecardFolder.TYPE_ID:
            parentObject = null;
            break;

        case Scorecard.TYPE_ID:
            parentObject = new ScorecardFolder();
            break;

        case ScorecardCategory.TYPE_ID:
            parentObject = new Scorecard();
            break;

        case ScorecardQuestion.TYPE_ID:
            parentObject = new ScorecardCategory();
            break;

        case ScorecardResponse.TYPE_ID:
            parentObject = new ScorecardQuestion();
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        loadParent(rpmObject, parentObject, parentId, messageContext, isDelta,
            scScope);

        switch (typeId)
        {
        case ScorecardFolder.TYPE_ID:
            loadScorecards((ScorecardFolder) rpmObject,
                (ScorecardFolderScope) rpmScope, messageContext, isDelta);
            break;

        case Scorecard.TYPE_ID:
            loadCategories((Scorecard) rpmObject, (ScorecardScope) rpmScope,
                messageContext, isDelta);
            break;

        case ScorecardCategory.TYPE_ID:
            loadQuestions((ScorecardCategory) rpmObject,
                (ScorecardCategoryScope) rpmScope, messageContext, isDelta);
            break;

        case ScorecardQuestion.TYPE_ID:
            loadResponses((ScorecardQuestion) rpmObject,
                (ScorecardQuestionScope) rpmScope, messageContext, isDelta);
            break;

        case ScorecardResponse.TYPE_ID:
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return rpmObject;
    }

    private void loadScorecards(ScorecardFolder rpmObject,
            ScorecardFolderScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        if (rpmScope == null || rpmScope.getScorecards() == null)
            return;

        ArrayList children = loadByForeignKey(rpmObject.getScorecards(),
            rpmScope.getScorecards(), messageContext, this, new Object[]
            {
                rpmObject.getID()
            }, null, rpmObject.getContextName(), isDelta);
        if (children != null)
        {
            Iterator it = children.iterator();
            Scorecard[] scorcards = new Scorecard[children.size()];
            for (int i = 0; i < scorcards.length; i++)
            {
                Scorecard element = (Scorecard) it.next();
                element.setParent(rpmObject);
                scorcards[i] = element;
            }

            rpmObject.setScorecards(scorcards);
        }

    }

    private void loadCategories(Scorecard rpmObject, ScorecardScope rpmScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardCategories() == null)
            return;

        ArrayList children = loadByForeignKey(rpmObject
            .getScorecardCategories(), rpmScope.getScorecardCategories(),
            messageContext, this, new Object[]
            {
                rpmObject.getID()
            }, null, rpmObject.getContextName(), isDelta);

        if (children != null)
        {
            Iterator it = children.iterator();
            ScorecardCategory[] categories = new ScorecardCategory[children
                .size()];
            for (int i = 0; i < categories.length; i++)
            {
                ScorecardCategory element = (ScorecardCategory) it.next();
                element.setParent(rpmObject);
                categories[i] = element;
            }

            rpmObject.setScorecardCategories(categories);
        }
    }

    private void loadQuestions(ScorecardCategory rpmObject,
            ScorecardCategoryScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardQuestions() == null)
            return;

        ArrayList children = loadByForeignKey(
            rpmObject.getScorecardQuestions(),
            rpmScope.getScorecardQuestions(), messageContext, this,
            new Object[]
            {
                rpmObject.getID()
            }, null, rpmObject.getContextName(), isDelta);
        if (children != null)
        {
            Iterator it = children.iterator();
            ScorecardQuestion[] questions = new ScorecardQuestion[children
                .size()];
            for (int i = 0; i < questions.length; i++)
            {
                ScorecardQuestion element = (ScorecardQuestion) it.next();
                element.setParent(rpmObject);
                questions[i] = element;
            }
            rpmObject.setScorecardQuestions(questions);
        }
    }

    private void loadResponses(ScorecardQuestion rpmObject,
            ScorecardQuestionScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null || rpmScope.getScorecardResponses() == null)
            return;

        ArrayList children = loadByForeignKey(
            rpmObject.getScorecardResponses(),
            rpmScope.getScorecardResponses(), messageContext, this,
            new Object[]
            {
                rpmObject.getID()
            }, null, rpmObject.getContextName(), isDelta);
        if (children != null)
        {
            Iterator it = children.iterator();
            ScorecardResponse[] responses = new ScorecardResponse[children
                .size()];
            for (int i = 0; i < responses.length; i++)
            {
                ScorecardResponse element = (ScorecardResponse) it.next();
                element.setParent(rpmObject);
                responses[i] = element;
            }
            rpmObject.setScorecardResponses(responses);
        }
    }

    private void loadParent(RPMObject rpmObject, RPMObject parentObject,
            String parentId, MessageContext messageContext, boolean isDelta,
            AbstractScorecardScope scScope) throws RPMException,
            ParseException, SQLException
    {
        if (scScope != null && scScope.getParent() != null
                && parentObject != null)
        {
            parentObject.setID(parentId);
            parentObject.setContextName(rpmObject.getContextName());
            parentObject = loadByPrimaryKey(parentObject, scScope.getParent(),
                messageContext, isDelta);
            if (isDelta)
            {
                rpmObject.deltaParent(parentObject);
            }
            else
            {
                rpmObject.setParent(parentObject);
            }
        }
    }

    private String SP_I_SCORECARD_ENTITY(AbstractScorecardEntity rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_I_SCORECARD";
        Object args[] = new Object[5];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getName();
        args[2] = (rpmObject.getParent() != null) ? rpmObject.getParent()
            .getID() : null;

        args[3] = rpmObject.retrieveTypeID();
        args[4] = getUser(messageContext).getID();

        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    private void SP_D_SCORECARD_ELEMENT(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_D_SCORECARD";
        Object args[] = new Object[5];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = ManagerUtil.getParentId(this, rpmObject, messageContext,
            getTableName(rpmObject));
        args[3] = "X";
        args[4] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);

    }

    private void SP_U_SC_WEIGHT(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String name = "SP_U_SC_WEIGHT";
        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = ((AbstractScorecardElement) rpmObject).getWeight();
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_SCORECARD(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        AbstractScorecardEntity scObject = (AbstractScorecardEntity) rpmObject;

        int typeId = scObject.retrieveTypeID().intValue();

        String name = "SP_U_SCORECARD";
        Object args[] = new Object[7];
        args[0] = getTableType(scObject);
        args[1] = scObject.getID();
        args[2] = scObject.getName();
        args[3] = scObject.getDescription();

        switch (typeId)
        {

        case ScorecardFolder.TYPE_ID: // same values for folder and response
        case ScorecardResponse.TYPE_ID:
            args[4] = new Integer(0);
            args[5] = new Integer(0);
            break;

        case Scorecard.TYPE_ID:
            args[4] = ProbabilityTypeID.getId(this, ((Scorecard) scObject)
                .getProbabilityType());
            args[5] = new Integer(0);
            break;

        case ScorecardCategory.TYPE_ID:
            args[4] = ProbabilityTypeID.getId(this,
                ((ScorecardCategory) scObject).getProbabilityType());
            args[5] = new Integer(0);
            break;

        case ScorecardQuestion.TYPE_ID:
            args[4] = new Integer(0);
            args[5] = ((ScorecardQuestion) scObject).getToReport()
                .booleanValue() ? new Integer(1) : new Integer(0);
            break;

        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_SC_CONFIG_FLAG(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        if (!(rpmObject instanceof Scorecard))
            throw new RPMException("Not a legal type:"
                    + rpmObject.getClass().getName() + " Should be Scorecard!");

        String name = "SP_U_SC_CONFIG_FLAG";
        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        args[2] = ((Scorecard) rpmObject).getActive().booleanValue()
                ? new Integer(1) : new Integer(0);
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);

    }

    private void SP_U_SC_DEFAULT(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String name = "SP_U_SC_DEFAULT";
        Object args[] = new Object[4];
        args[0] = getTableType(rpmObject);
        args[1] = rpmObject.getID();
        if (((ScorecardResponse) rpmObject).getDefaultResponse().booleanValue())
            args[2] = "1"; // only when it is the default it should be updated
        else
            return;
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);

    }

}