/*
 * Created on Sep 16, 2005
 *
 */
package com.ibm.rpm.scorecard.managers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scorecard.containers.AbstractScorecardEntity;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.containers.WbsModule;

/**
 * @author karim
 *
 */
public class ScorecardUtil
{

    private static Log logger = LogFactory.getLog(ScorecardUtil.class); 

    private static final Map tableNameByContext = new HashMap();

    private static final Map tableTypeByContext = new HashMap();
    
    private static final List VALID_SCORECARDFOLDER_ROOT_LIST = new ArrayList();

    static
    {
        tableNameByContext.put(ContextUtil.ASSET_CONTEXT,
                "TMT_SCORECARDS_DEF_AST");
        tableTypeByContext.put(ContextUtil.ASSET_CONTEXT, new Integer(9));
        
        tableNameByContext.put(ContextUtil.SCOPE_CONTEXT,
                "TMT_SCORECARDS_DEF_CRI");
        tableTypeByContext.put(ContextUtil.SCOPE_CONTEXT, new Integer(3));
        
        tableNameByContext.put(ContextUtil.WBS_CONTEXT,
                "TMT_SCORECARDS_DEF_WBS");
        tableTypeByContext.put(ContextUtil.WBS_CONTEXT, new Integer(1));
        
        tableNameByContext.put(ContextUtil.RESOURCE_CONTEXT,
                "TMT_SCORECARDS_DEF_RES");
        tableTypeByContext.put(ContextUtil.RESOURCE_CONTEXT, new Integer(7));
        
        // valid module
        VALID_SCORECARDFOLDER_ROOT_LIST.add(AssetModule.class);
        VALID_SCORECARDFOLDER_ROOT_LIST.add(ScopeManagementModule.class);
        VALID_SCORECARDFOLDER_ROOT_LIST.add(WbsModule.class);
        VALID_SCORECARDFOLDER_ROOT_LIST.add(ResourceModule.class);
    }

    static String getTableName(String contextName)
    {
        return (String) tableNameByContext.get(contextName);
    }

    static Integer getTableType(String contextName)
    {
        return (Integer) tableTypeByContext.get(contextName);
    }
    
    static boolean isLegalContext(String contextName)
    {
        return getTableType(contextName) != null;
    }
    
    static RPMObject getScorecardRoot(AbstractScorecardEntity scorecard)
    {
        RPMObject parent = scorecard.getParent();
        while (parent != null && parent.getParent() != null)
        {
            parent = parent.getParent();
        }
        
        return parent;
    }
    
    static String getContextNameFromModule(AbstractScorecardEntity scorecard)
    {
        RPMObject root = getScorecardRoot(scorecard);
        String contextName = root.getContextName();
        if (contextName == null)
        {
            contextName = getContextNameFromModule(root);
        }
        return contextName;
    }
    
    static String getContextNameFromModule(RPMObject root)
    {
        if (root == null)
            return null;
        if (root instanceof WbsModule)
            return ContextUtil.WBS_CONTEXT;
        if (root instanceof AssetModule)
            return ContextUtil.ASSET_CONTEXT;
        if (root instanceof ScopeManagementModule)
            return ContextUtil.SCOPE_CONTEXT;
        if (root instanceof ResourceModule)
            return ContextUtil.RESOURCE_CONTEXT;
        return null;
    }
    
    static AbstractModule getModuleFromContextName(String contextName)
    {
        AbstractModule module = null;
        
        if(contextName == null)
            return null;
        if (contextName.equals(ContextUtil.WBS_CONTEXT))
            module = new WbsModule();
        if (contextName.equals(ContextUtil.ASSET_CONTEXT))
            module = new AssetModule();
        if (contextName.equals(ContextUtil.SCOPE_CONTEXT))
            module = new ScopeManagementModule();
        if (contextName.equals(ContextUtil.RESOURCE_CONTEXT))
            module = new ResourceModule();

        return module;
    }
    
    static List getValidAttributeRoot()
    {
        return VALID_SCORECARDFOLDER_ROOT_LIST;
    }

    /**
     * Mapping between probability types and the ID in the database
     *
     */
    static class ProbabilityTypeID
    {

        private static final int _NONE = 0;

        private static final int _PROJECT_BENEFITS_ID = 1;

        private static final int _PROJECT_COSTS_ID = 2;

        private static final int _CUSTOMER_SATISFACTION_ID = 3;

        private static final int _PROJECT_SCHEDULE_ID = 4;

        private static final int _PROJECT_RISK_ID = 5;

        static Integer getId(RPMObjectManager manager, ProbabilityType type) throws RPMException
        {
            if (type == ProbabilityType.None)
                return new Integer(_NONE);

            if (type == ProbabilityType.CustomerSatisfaction)
                return new Integer(_CUSTOMER_SATISFACTION_ID);

            if (type == ProbabilityType.ProjectBenefits)
                return new Integer(_PROJECT_BENEFITS_ID);

            if (type == ProbabilityType.ProjectCosts)
                return new Integer(_PROJECT_COSTS_ID);

            if (type == ProbabilityType.ProjectRisk)
                return new Integer(_PROJECT_RISK_ID);

            if (type == ProbabilityType.ProjectSchedule)
                return new Integer(_PROJECT_SCHEDULE_ID);

            ExceptionUtil.handleNoTypeId(manager, type.getValue());
            return null;
        }

        static ProbabilityType getCategory(RPMObjectManager manager, int id) throws RPMException
        {
            switch (id)
            {
            case _NONE:
                return ProbabilityType.None;

            case _CUSTOMER_SATISFACTION_ID:
                return ProbabilityType.CustomerSatisfaction;

            case _PROJECT_BENEFITS_ID:
                return ProbabilityType.ProjectBenefits;

            case _PROJECT_COSTS_ID:
                return ProbabilityType.ProjectCosts;

            case _PROJECT_RISK_ID:
                return ProbabilityType.ProjectRisk;

            case _PROJECT_SCHEDULE_ID:
                return ProbabilityType.ProjectSchedule;
            }

            ExceptionUtil.handleUnsupportedTypeId(manager, id);
            return null;

        }

    }
}