package com.ibm.rpm.security.managers;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.scope.SecurityGroupScope;

/**
 * @author Marcel Dextraze
 */
public class SecurityGroupManager extends AbstractRPMObjectManager
{
    // TODO remove logger later
    private static Log logger = LogFactory.getLog(SecurityRoleManager.class);

    private static final String TABLE_NAME = "TMT_POOLS";

    private static final String SECURITY_GROUP_CONTAINER_NAME = StringUtil
        .getShortClassName(SecurityGroup.class);

    private static final String ROOT_SECURITY_GROUP_ID = "DEFAULT_SECURITY_ROLE";

    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(SecurityGroup.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                SecurityGroupManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = TABLE_NAME + ".ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 2;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = TABLE_NAME + ".ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column DESCRIPTION of type Types.CHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 3;

    public static final int TYPE_DESCRIPTION = Types.CHAR;

    public static final String NAME_DESCRIPTION = TABLE_NAME + ".DESCRIPTION";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = TABLE_NAME + ".TYPE_ID";

    /**
     * Column CLEARANCE_LEVEL_1 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_1 = 5;

    public static final int TYPE_CLEARANCE_LEVEL_1 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_1 = TABLE_NAME
            + ".CLEARANCE_LEVEL_1";

    /**
     * Column CLEARANCE_LEVEL_2 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_2 = 6;

    public static final int TYPE_CLEARANCE_LEVEL_2 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_2 = TABLE_NAME
            + ".CLEARANCE_LEVEL_2";

    /**
     * Column CLEARANCE_LEVEL_3 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_3 = 7;

    public static final int TYPE_CLEARANCE_LEVEL_3 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_3 = TABLE_NAME
            + ".CLEARANCE_LEVEL_3";

    /**
     * Column CLEARANCE_LEVEL_4 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_4 = 8;

    public static final int TYPE_CLEARANCE_LEVEL_4 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_4 = TABLE_NAME
            + ".CLEARANCE_LEVEL_4";

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_5 = 9;

    public static final int TYPE_CLEARANCE_LEVEL_5 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_5 = TABLE_NAME
            + ".CLEARANCE_LEVEL_5";

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_6 = 10;

    public static final int TYPE_CLEARANCE_LEVEL_6 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_6 = TABLE_NAME
            + ".CLEARANCE_LEVEL_6";

    /**
     * Column CLEARANCE_LEVEL_5 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CLEARANCE_LEVEL_7 = 11;

    public static final int TYPE_CLEARANCE_LEVEL_7 = Types.INTEGER;

    public static final String NAME_CLEARANCE_LEVEL_7 = TABLE_NAME
            + ".CLEARANCE_LEVEL_7";

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 12;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = TABLE_NAME + ".RANK";

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 13;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = TABLE_NAME + ".PARENT_ID";

    /**
     * Create an array of type string containing all the fields of the TMT_POOLS
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_DESCRIPTION, NAME_TYPE_ID,
            NAME_CLEARANCE_LEVEL_1, NAME_CLEARANCE_LEVEL_2,
            NAME_CLEARANCE_LEVEL_3, NAME_CLEARANCE_LEVEL_4,
            NAME_CLEARANCE_LEVEL_5, NAME_CLEARANCE_LEVEL_6,
            NAME_CLEARANCE_LEVEL_7, NAME_RANK, NAME_PARENT_ID
    };

    /**
     * Field that contains the comma separated fields of the TMT_POOLS table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * @param resource
     */
    public SecurityGroupManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof SecurityGroupManager)
        {
            // where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PARENT_ID);
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;

    }

    private String determineTypeId(String containerName)
    {
        String typeId = null;
        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(SECURITY_GROUP_CONTAINER_NAME))
            {
                typeId = String.valueOf(SecurityGroup.TYPE_ID);
            }
        }
        return typeId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        String typeId = determineTypeId(containerName);

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            boolean isWhereCondition = false;

            StringBuffer where = new StringBuffer();
            if (typeId != null)
            {
                where.append(NAME_TYPE_ID);
                where.append(" = ");
                where.append(typeId);

                isWhereCondition = true;
            }

            if (condition != null)
            {
                if (isWhereCondition)
                {
                    where.append(" AND ");
                }
                where.append(condition);
                isWhereCondition = true;
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof SecurityGroupManager)
        {
            // TODO complete later
            joinCondition = new JoinCondition();
            joinCondition.setCondition(condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    public RPMObject createContainer(int typeId)
    {
        RPMObject object = new SecurityGroup();
        return object;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null)
        {
            SecurityGroup securityGroup = (SecurityGroup) rpmObject;

            if (rpmScope.getParent() != null)
            {
                loadParent(securityGroup, rpmScope, messageContext,
                    relatedFields, index, isDelta);
            }

            if (rpmScope != null && rpmScope instanceof SecurityGroupScope)
            {

                SecurityGroupScope scope = (SecurityGroupScope) rpmScope;

                if (scope.getSecurityGroups() != null)
                {
                    // security groups
                    loadSecurityGroups(securityGroup, scope, messageContext,
                        isDelta);
                }
            }
        }
        return rpmObject;
    }

    private void loadSecurityGroups(SecurityGroup securityGroup,
            SecurityGroupScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String extraWhere = " TYPE_ID = " + SecurityGroup.TYPE_ID + " ";
        SecurityGroup[] children = securityGroup.getSecurityGroups();
        ArrayList temp = loadByForeignKeyWithSpecifiedParent(securityGroup,
            children, scope.getSecurityGroups(), messageContext, this,
            new Object[]
            {
                securityGroup.getID()
            }, extraWhere, null, isDelta);
        if (temp != null)
        {
            children = new SecurityGroup[temp.size()];
            temp.toArray(children);
            securityGroup.setSecurityGroups(children);
        }
    }

    private void loadParent(SecurityGroup securityGroup,
            RPMObjectScope rpmscope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);

        SecurityGroup newParent = null;

        if (isDelta)
        {
            newParent = (SecurityGroup) securityGroup.getParent();
            SecurityGroup oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new SecurityGroup();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = (SecurityGroup) loadByPrimaryKey(newParent,
                        rpmscope.getParent(), messageContext, true);
                }
                else
                {
                    ExceptionUtil.handleParentReference(this, securityGroup);
                }
            }
            else
            {
                if (newParent != null)
                {
                    ExceptionUtil.handleParentReference(this, securityGroup);
                }
            }
            securityGroup.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {
                    newParent = new SecurityGroup();
                    newParent.setID(oldParentId);
                    newParent = (SecurityGroup) loadByPrimaryKey(newParent,
                        rpmscope.getParent(), messageContext, false);
                }
                securityGroup.setParent(newParent);
            }
        }
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException, RPMException
    {
        SecurityGroup temp = (SecurityGroup) rpmObject;

        // Verifier si on a plus d'un groupe dans le result set. C'est le cas si
        // on demandait
        // le Default Security Group et ses enfants.

        // Pour chaque groupe recu
        String str = rs.getString(ID_ELEMENT_ID);
        if (str != null)
        {
            str = str.trim();
        }
        temp.setID(str);
        if (isDelta)
        {
            str = rs.getString(ID_ELEMENT_NAME);
            if (str != null)
            {
                str = str.trim();
            }
            temp.deltaName(str);

            str = rs.getString(ID_DESCRIPTION);
            if (str != null)
            {
                str = str.trim();
            }
            temp.deltaDescription(str);
            deltaClearanceLevel(rs, temp);
        }
        else
        {
            str = rs.getString(ID_ELEMENT_NAME);
            if (str != null)
            {
                str = str.trim();
            }
            temp.setName(str);

            str = rs.getString(ID_DESCRIPTION);
            if (str != null)
            {
                str = str.trim();
            }
            temp.setDescription(str);
            setClearanceLevel(rs, temp);
        }
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        if (parentId != null)
            parentId = parentId.trim();
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    private void setClearanceLevel(ResultSet rs, SecurityGroup sg)
            throws SQLException, RPMException
    {
        // Set the clearance levels in the security group object
        // from the values in the result set
        int clearanceLevel1 = rs.getInt(ID_CLEARANCE_LEVEL_1);
        int clearanceLevel2 = rs.getInt(ID_CLEARANCE_LEVEL_2);
        int clearanceLevel3 = rs.getInt(ID_CLEARANCE_LEVEL_3);
        int clearanceLevel4 = rs.getInt(ID_CLEARANCE_LEVEL_4);
        int clearanceLevel5 = rs.getInt(ID_CLEARANCE_LEVEL_5);
        int clearanceLevel6 = rs.getInt(ID_CLEARANCE_LEVEL_6);
        int clearanceLevel7 = rs.getInt(ID_CLEARANCE_LEVEL_7);

        if (sg != null)
        {
            try
            {
                SecurityUtil.setClearanceLevels(new int[]
                {
                        clearanceLevel1, clearanceLevel2, clearanceLevel3,
                        clearanceLevel4, clearanceLevel5, clearanceLevel6,
                        clearanceLevel7
                }, sg);
            }
            catch (Exception e)
            {
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, new String[]
                {
                    "Error reading bitmasks : " + e
                }, e);
            }
        }
    }

    private void deltaClearanceLevel(ResultSet rs, SecurityGroup sg)
            throws SQLException, RPMException
    {

        if (sg != null)
        {
            //Set the clearance levels in the security group object
            // from the values in the result set
            int clearanceLevel1 = rs.getInt(ID_CLEARANCE_LEVEL_1);
            int clearanceLevel2 = rs.getInt(ID_CLEARANCE_LEVEL_2);
            int clearanceLevel3 = rs.getInt(ID_CLEARANCE_LEVEL_3);
            int clearanceLevel4 = rs.getInt(ID_CLEARANCE_LEVEL_4);
            int clearanceLevel5 = rs.getInt(ID_CLEARANCE_LEVEL_5);
            int clearanceLevel6 = rs.getInt(ID_CLEARANCE_LEVEL_6);
            int clearanceLevel7 = rs.getInt(ID_CLEARANCE_LEVEL_7);

            // Map clearance levels to "can*" attributes
            // The commented lines relates to bits that are not delta/redelta with
            // the
            // RPM Client or that not currently used.

            SecurityUtil.deltaClearanceLevels(new int[]
            {
                    clearanceLevel1, clearanceLevel2, clearanceLevel3,
                    clearanceLevel4, clearanceLevel5, clearanceLevel6,
                    clearanceLevel7
            }, sg);
        }
    }

    // /* (non-Javadoc)
    // * @see
    // com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
    // */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        SecurityGroup pObject = (SecurityGroup) rpmObject;
        pObject.setID(SP_I_POOL(pObject, messageContext));
    }

    // /* (non-Javadoc)
    // * @see
    // com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
    // */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        SecurityGroup pObject = (SecurityGroup) rpmObject;
        if (pObject.testNameModified())
        {
            SP_U_POOL_NAME(pObject, messageContext);
        }
        if (pObject.testDescriptionModified())
        {
            SP_U_POOL_DESC(pObject, messageContext);
        }
        if (SecurityUtil.isAnyClearanceLevelModified(pObject))
        {
            SP_U_UG_SECURITY(pObject, messageContext);
        }
        return pObject;
    }

    //
    //
    // /* (non-Javadoc)
    // * @see
    // com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
    // */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        SecurityGroup sg = null;
        if (rpmObject != null)
        {
            sg = (SecurityGroup) rpmObject;
            SP_D_POOL(sg, messageContext);
        }
    }

    public static int[] getClearanceLevelFromID(MessageContext messageContext,
            String id) throws SQLException
    {
        int[] clearanceLevels = new int[7];
        Connection c = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String selectStmt = "SELECT " + ALL_FIELDS + " FROM " + TABLE_NAME
                + " WHERE (" + NAME_ELEMENT_ID + "='" + id + "')";

        try
        {
            c = Manager.getConnection(messageContext);

            ps = c.prepareStatement(selectStmt, ResultSet.TYPE_FORWARD_ONLY,
                ResultSet.CONCUR_READ_ONLY);
            try
            {
                rs = ManagerUtil.executeSqlQuery(messageContext, ps,
                    selectStmt, null);

            }
            catch (Exception e)
            {
                logger.error(e);
            }
            if (rs.next())
            {
                for (int i = SecurityGroupManager.ID_CLEARANCE_LEVEL_1; i <= SecurityGroupManager.ID_CLEARANCE_LEVEL_7; i++)
                {
                    clearanceLevels[i
                            - SecurityGroupManager.ID_CLEARANCE_LEVEL_1] = rs
                        .getInt(i);
                }
            }
        }
        catch (SQLException e)
        {
            logger.error(e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return clearanceLevels;
    }

    /*
     * CALL SP_I_POOL('Marcel''s
     * Group','DEFAULT_SECURITY_ROLE',307,'PMO_SUPERVISOR') Used to create a new
     * Security/Settings Group.
     */
    private String SP_I_POOL(SecurityGroup securityGroup,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String name = "SP_I_POOL";

        Object args[] = new Object[4];
        args[0] = securityGroup.getName();

        args[1] = ROOT_SECURITY_GROUP_ID;
        args[2] = securityGroup.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    private void SP_D_POOL(SecurityGroup securityGroup,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_D_POOL";
        Object args[] = new Object[4];
        args[0] = securityGroup.getID(); // ID of the Resource for which we
        // want to change something
        args[1] = ROOT_SECURITY_GROUP_ID;
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_UG_SECURITY(SecurityGroup securityGroup,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_UG_SECURITY";
        Object args[] = new Object[10];
        args[0] = securityGroup.getID(); // 

        int[] clearanceLevels = SecurityUtil.getClearanceLevels(securityGroup,
            new int[SecurityUtil.SECURITY_GROUP_LEVELS_COUNT]);

        args[1] = new Integer(clearanceLevels[0]);
        args[2] = new Integer(clearanceLevels[1]);
        args[3] = new Integer(clearanceLevels[2]);
        args[4] = new Integer(clearanceLevels[3]);
        args[5] = new Integer(clearanceLevels[4]);
        args[6] = new Integer(clearanceLevels[5]);
        args[7] = new Integer(clearanceLevels[6]);

        // TODO args[8] is for navbar_access_control flag
        // In the Client it matches the following two flags in the resource setting
        // - Can Access Documents from Left Navigator and All Documents in My Portal
        // - Can Access Scope Management from Left Navigator All Scope Elements in My Portal
        // those settings can be found in Resource Management - Application/My Settings Portlet
        // The value needs to be adjusted later.
        args[8] = new Integer(3);
        
        args[9] = getUser(messageContext).getID();

        //XXX TODO Yasser: revisit this thing and figure out whay it has been disabled, maybe we have to enable it.        
        // There is a potential to incorrectly set the bit fields in this class
        // so we don't use it
        // SecurityRulesMaps.applyBusinessRules(args);

        executeProcedure(messageContext, name, args);
    }

    // /*
    // * CALL
    // SP_U_SECURITY('9349773DBD244D9A9EE75D9A333A64D4',18878463,32961662,2101247,1606434748,2147352479,1341980575,2147483647,639,1073741823,1073741823,1073741823,1073741823,1048575,1073741823,1073741823,1073741823,1073741823,1048575,9437183,0,0,0,0,0,'PMO_SUPERVISOR')
    // * for Application Administration/Security/WBS
    // */
    // /*
    // private void SP_U_SECURITY(SecurityGroup securityGroup)
    // throws SQLException, RPMException {
    // String name = "SP_U_SECURITY";
    // Object args[] = new Object[26];
    // args[0] = securityGroup.getID();
    // ...
    // args[25] = getUser().getID();
    // executeProcedure(messageContext, name, args);
    // }
    // */

    private void SP_U_POOL_NAME(SecurityGroup securityGroup,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_POOL_NAME";
        Object args[] = new Object[3];
        args[0] = securityGroup.getID();
        args[1] = securityGroup.getName();
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_POOL_DESC(SecurityGroup securityGroup,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_POOL_DESC";
        Object args[] = new Object[4];
        args[0] = securityGroup.getID();
        args[1] = null;
        args[2] = securityGroup.getDescription();
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    // /*
    // * CALL
    // SP_U_RES_REVISION('2D8FBEC3CFE341CE8B3806E845D70EFE',0,137,4747,null,'PMO_SUPERVISOR')
    // * This is called prior to calling SP_U_RES_SECURITY
    // */
    // /*
    // private void SP_U_RES_REVISION(SecurityGroup securityGroup)
    // throws SQLException, RPMException {
    // String name = "SP_U_RES_REVISION";
    // Object args[] = new Object[6];
    // args[0] = securityGroup.getID(); // ID of the Resource for which we want
    // to change something
    // args[1] = new Integer(0); // value to be defined
    // args[2] = new Integer(137); // value to be defined
    // args[3] = new Integer(4747); // value to be defined
    // args[4] = null; // value to be defined
    // args[5] = getUser().getID();
    // executeProcedure(messageContext, name, args);
    // }
    // */
}
