/*
 * Created on Nov 17, 2005 
 * @author mmatwiy 
 * Checkpoint validation of Timesheet container PersonalTask.
 */
package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.managers.TimesheetTaskManager;


public class PersonalTaskCheckpoint extends TimesheetElementCheckpoint
{
    private static final List PERSONALTASK_PARENT_LIST = new ArrayList();

    protected static PersonalTaskCheckpoint instance = new PersonalTaskCheckpoint();

    static
    {
        PERSONALTASK_PARENT_LIST.add(PersonalProject.class);
    }

    public static PersonalTaskCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        PersonalTask personalTask = (PersonalTask) rpmObject;

        super.validateSave(rpmObject, msgCtx, true, PERSONALTASK_PARENT_LIST);
        
        //Parent must have an id if it is a new PersonalTask
        if (personalTask.getParent()!= null && personalTask.getID() == null)
        {
            GenericValidator.validateMandatoryID(personalTask,
                ValidationConstants.PARENT_FIELD, personalTask.getParent(),
                msgCtx);
        }
        
        
        if(!msgCtx.isSuccessful()){
            return;
        }
        
        super.validateReadOnly(personalTask, msgCtx);
        
        //resource must have a calendar;
        if (personalTask.getID() == null)
        {
            if (!validateCalendar((PersonalProject) personalTask.getParent(), msgCtx))
            {
                return;
            }
        }
        
        
        if (personalTask.testNameModified())
        {
            // Check that the name doesn't exceed the max length
            GenericValidator.validateMaxLength(personalTask,
                ValidationConstants.NAME_FIELD, personalTask.getName(),
                ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT, msgCtx);
        }

    }
    
    private static boolean validateCalendar(
            PersonalProject parent, MessageContext messageContext)
    {
       
        boolean valid = false;

            SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(ResourceManager.NAME_CALENDAR_ID);
        stmt.appendFrom(ResourceManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendSubSelect(ResourceManager.NAME_RESOURCE_ID,
            TimesheetTaskManager.NAME_CREATED_BY,
            TimesheetTaskManager.TABLE_NAME,
            TimesheetTaskManager.NAME_ELEMENT_ID + " = ? ");

        List result = null;
            ;
            try
            {
                result = ManagerUtil.select(null, null, null, stmt,
                    new Object[]
                    {
                        parent.getID()
                    }, messageContext);
            }
            catch (RPMException e1)
            {
                addException(e1, messageContext);
            }

            if (result != null && result.size() > 0)
            {
                ArrayList columns = (ArrayList) result.get(0);
                String calendarId = (String) columns.get(0);
                if (calendarId != null)
                {
                    valid = true;
                }
            }
            else
            {
                //invalid resource
                String[] errorParams =
                {
                        StringUtil.getShortClassName(Resource.class),
                        parent.getID()

                };
                RPMException e = new RPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                    errorParams);
                addException(e, messageContext);

            }
        

        if(!valid){
            //invalid resource
            String[] errorParams =
            {
                    parent.getID()

            };
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.MANDATORY_CALENDAR,
                errorParams, StringUtil
                    .getShortClassName(PersonalTask.class),
                ValidationConstants.PARENT_FIELD);
            addException(e, messageContext);
        }
        return valid;
    }

}
