package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * @author gmeloche
 *
 */
public class AdministrativeTaskAssignmentManager extends AbstractTimesheetTaskAssignmentManager
{
    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(AdministrativeTaskAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    AdministrativeTaskAssignmentManager.class);
        }
    }
    
    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendSubSelect(NAME_TASK_ID, WbsManager.NAME_ELEMENT_ID,
                WbsManager.TABLE_NAME, WbsManager.NAME_TYPE_ID + '='
                        + AdministrativeTask.TYPE_ID);
        sql.appendCloseParenthesis();
        sql.appendSubSelect(NAME_RESOURCE_ID, ResourceManager.NAME_RESOURCE_ID,
                ResourceManager.TABLE_NAME, ResourceManager.NAME_REC_STATUS
                        + "!='D'");
        sql.appendCloseParenthesis();

        return sql.toString();
    }
    
    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new AdministrativeTaskAssignment();
    }
	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext,
			FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {
		
	    AdministrativeTaskAssignment assignment = (AdministrativeTaskAssignment)rpmObject;
		
		//load the administrative task
		TimesheetTaskManager mgr = (TimesheetTaskManager)getManagerInstance(AdministrativeTask.class);
		AdministrativeTask task = new AdministrativeTask();
        task.setID((String) relatedFields.get(index, ID_TASK_ID));
        task = (AdministrativeTask) mgr.loadByPrimaryKey(task, null,messageContext, isDelta);
        if (isDelta) {
            assignment.deltaAdministrativeTask(task);
        } else {
            assignment.setAdministrativeTask(task);
        }
        
        //load the parent
        loadParent(assignment, relatedFields, index, isDelta, messageContext);

        return assignment;
	}
	
	protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, getFilter());
            joinCondition.appendCondition(condition);
        }
        else if (manager instanceof TimesheetTaskManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                    TimesheetTaskManager.NAME_ELEMENT_ID, NAME_TASK_ID,
                    TABLE_NAME, getFilter());
            joinCondition.appendEqualFields(
                    TimesheetTaskManager.NAME_CREATED_BY, NAME_RESOURCE_ID);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
}
