package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.RPMObjectUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.AbstractTimesheetTaskAssignment;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.xpathparser.XPathContainer;

public class SummaryTimesheetManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(SummaryTimesheet.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                SummaryTimesheetManager.class);
        }
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "SUMMARY_TIMESHEETS.PROJECT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "SUMMARY_TIMESHEETS.RESOURCE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column TSK_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TSK_ASSIGNMENT_ID = 3;

    public static final int TYPE_TSK_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_TSK_ASSIGNMENT_ID = "SUMMARY_TIMESHEETS.TSK_ASSIGNMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TSK_ASSIGNMENT_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_TSK_ASSIGNMENT_ID,NAME_TSK_ASSIGNMENT_ID);}
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_TSK_ASSIGNMENT_ID);
    }

    /**
     * Column WEEK_OF of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_WEEK_OF = 4;

    public static final int TYPE_WEEK_OF = Types.DATE;

    public static final String NAME_WEEK_OF = "SUMMARY_TIMESHEETS.WEEK_OF";

    public static final String PROPERTY_WEEK_OF = "WEEKOF";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEEK_OF, NAME_WEEK_OF);
    }

    /**
     * Column MONDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONDAY_REGULAR = 5;

    public static final int TYPE_MONDAY_REGULAR = Types.INTEGER;

    public static final String NAME_MONDAY_REGULAR = "SUMMARY_TIMESHEETS.MONDAY_REGULAR";

    public static final String PROPERTY_MONDAY_REGULAR = "REGULARMONDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONDAY_REGULAR, NAME_MONDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column MONDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONDAY_SPECIAL = 6;

    public static final int TYPE_MONDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_MONDAY_SPECIAL = "SUMMARY_TIMESHEETS.MONDAY_SPECIAL";

    public static final String PROPERTY_MONDAY_SPECIAL = "SPECIALMONDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONDAY_SPECIAL, NAME_MONDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column TUESDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TUESDAY_REGULAR = 7;

    public static final int TYPE_TUESDAY_REGULAR = Types.INTEGER;

    public static final String NAME_TUESDAY_REGULAR = "SUMMARY_TIMESHEETS.TUESDAY_REGULAR";

    public static final String PROPERTY_TUESDAY_REGULAR = "REGULARTUESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_REGULAR, NAME_TUESDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column TUESDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TUESDAY_SPECIAL = 8;

    public static final int TYPE_TUESDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_TUESDAY_SPECIAL = "SUMMARY_TIMESHEETS.TUESDAY_SPECIAL";

    public static final String PROPERTY_TUESDAY_SPECIAL = "SPECIALTUESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_SPECIAL, NAME_TUESDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column WEDNESDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEDNESDAY_REGULAR = 9;

    public static final int TYPE_WEDNESDAY_REGULAR = Types.INTEGER;

    public static final String NAME_WEDNESDAY_REGULAR = "SUMMARY_TIMESHEETS.WEDNESDAY_REGULAR";

    public static final String PROPERTY_WEDNESDAY_REGULAR = "REGULARWEDNESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_REGULAR, NAME_WEDNESDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column WEDNESDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEDNESDAY_SPECIAL = 10;

    public static final int TYPE_WEDNESDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_WEDNESDAY_SPECIAL = "SUMMARY_TIMESHEETS.WEDNESDAY_SPECIAL";

    public static final String PROPERTY_WEDNESDAY_SPECIAL = "SPECIALWEDNESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_SPECIAL, NAME_WEDNESDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column THURSDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_THURSDAY_REGULAR = 11;

    public static final int TYPE_THURSDAY_REGULAR = Types.INTEGER;

    public static final String NAME_THURSDAY_REGULAR = "SUMMARY_TIMESHEETS.THURSDAY_REGULAR";

    public static final String PROPERTY_THURSDAY_REGULAR = "REGULARTHURSDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_REGULAR, NAME_THURSDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column THURSDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_THURSDAY_SPECIAL = 12;

    public static final int TYPE_THURSDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_THURSDAY_SPECIAL = "SUMMARY_TIMESHEETS.THURSDAY_SPECIAL";

    public static final String PROPERTY_THURSDAY_SPECIAL = "SPECIALTHURSDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_SPECIAL, NAME_THURSDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column FRIDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FRIDAY_REGULAR = 13;

    public static final int TYPE_FRIDAY_REGULAR = Types.INTEGER;

    public static final String NAME_FRIDAY_REGULAR = "SUMMARY_TIMESHEETS.FRIDAY_REGULAR";

    public static final String PROPERTY_FRIDAY_REGULAR = "REGULARFRIDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_REGULAR, NAME_FRIDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column FRIDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FRIDAY_SPECIAL = 14;

    public static final int TYPE_FRIDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_FRIDAY_SPECIAL = "SUMMARY_TIMESHEETS.FRIDAY_SPECIAL";

    public static final String PROPERTY_FRIDAY_SPECIAL = "SPECIALFRIDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_SPECIAL, NAME_FRIDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column SATURDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SATURDAY_REGULAR = 15;

    public static final int TYPE_SATURDAY_REGULAR = Types.INTEGER;

    public static final String NAME_SATURDAY_REGULAR = "SUMMARY_TIMESHEETS.SATURDAY_REGULAR";

    public static final String PROPERTY_SATURDAY_REGULAR = "REGULARSATURDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_REGULAR, NAME_SATURDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column SATURDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SATURDAY_SPECIAL = 16;

    public static final int TYPE_SATURDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_SATURDAY_SPECIAL = "SUMMARY_TIMESHEETS.SATURDAY_SPECIAL";

    public static final String PROPERTY_SATURDAY_SPECIAL = "SPECIALSATURDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_SPECIAL, NAME_SATURDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column SUNDAY_REGULAR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUNDAY_REGULAR = 17;

    public static final int TYPE_SUNDAY_REGULAR = Types.INTEGER;

    public static final String NAME_SUNDAY_REGULAR = "SUMMARY_TIMESHEETS.SUNDAY_REGULAR";

    public static final String PROPERTY_SUNDAY_REGULAR = "REGULARSUNDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_REGULAR, NAME_SUNDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column SUNDAY_SPECIAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUNDAY_SPECIAL = 18;

    public static final int TYPE_SUNDAY_SPECIAL = Types.INTEGER;

    public static final String NAME_SUNDAY_SPECIAL = "SUMMARY_TIMESHEETS.SUNDAY_SPECIAL";

    public static final String PROPERTY_SUNDAY_SPECIAL = "SPECIALSUNDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_SPECIAL, NAME_SUNDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column WEEKLY_REGULAR_TOTAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEEKLY_REGULAR_TOTAL = 19;

    public static final int TYPE_WEEKLY_REGULAR_TOTAL = Types.INTEGER;

    public static final String NAME_WEEKLY_REGULAR_TOTAL = "SUMMARY_TIMESHEETS.WEEKLY_REGULAR_TOTAL";

    public static final String PROPERTY_WEEKLY_REGULAR_TOTAL = "TOTALREGULAR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_REGULAR_TOTAL,
            NAME_WEEKLY_REGULAR_TOTAL + " /60.0 ");
    }

    /**
     * Column WEEKLY_SPECIAL_TOTAL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEEKLY_SPECIAL_TOTAL = 20;

    public static final int TYPE_WEEKLY_SPECIAL_TOTAL = Types.INTEGER;

    public static final String NAME_WEEKLY_SPECIAL_TOTAL = "SUMMARY_TIMESHEETS.WEEKLY_SPECIAL_TOTAL";

    public static final String PROPERTY_WEEKLY_SPECIAL_TOTAL = "TOTALSPECIAL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_SPECIAL_TOTAL,
            NAME_WEEKLY_SPECIAL_TOTAL + " /60.0 ");
    }

    static
    {
        FIELDPROPERTYMAP.put("TOTALWEEK", "(" + NAME_WEEKLY_REGULAR_TOTAL
                + " + " + NAME_WEEKLY_SPECIAL_TOTAL + ")" + " /60.0 ");
    }

    /**
     * Column WEEKLY_BILLABLE_REGULAR_TOTAL of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_WEEKLY_BILLABLE_REGULAR_TOTAL = 21;

    public static final int TYPE_WEEKLY_BILLABLE_REGULAR_TOTAL = Types.INTEGER;

    public static final String NAME_WEEKLY_BILLABLE_REGULAR_TOTAL = "SUMMARY_TIMESHEETS.WEEKLY_BILLABLE_REGULAR_TOTAL";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WEEKLY_BILLABLE_REGULAR_TOTAL =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_BILLABLE_REGULAR_TOTAL,NAME_WEEKLY_BILLABLE_REGULAR_TOTAL);}

    /**
     * Column WEEKLY_BILLABLE_SPECIAL_TOTAL of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_WEEKLY_BILLABLE_SPECIAL_TOTAL = 22;

    public static final int TYPE_WEEKLY_BILLABLE_SPECIAL_TOTAL = Types.INTEGER;

    public static final String NAME_WEEKLY_BILLABLE_SPECIAL_TOTAL = "SUMMARY_TIMESHEETS.WEEKLY_BILLABLE_SPECIAL_TOTAL";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WEEKLY_BILLABLE_SPECIAL_TOTAL =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_WEEKLY_BILLABLE_SPECIAL_TOTAL,NAME_WEEKLY_BILLABLE_SPECIAL_TOTAL);}

    /**
     * Column TOTAL_SCHED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_TOTAL_SCHED_EFFORT = 23;

    public static final int TYPE_TOTAL_SCHED_EFFORT = Types.BIGINT;

    public static final String NAME_TOTAL_SCHED_EFFORT = "SUMMARY_TIMESHEETS.TOTAL_SCHED_EFFORT";

    // public static final String PROPERTY_TOTAL_SCHED_EFFORT = "PROPERTYNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_TOTAL_SCHED_EFFORT,
    // NAME_TOTAL_SCHED_EFFORT);
    // }

    /**
     * Column BASELINE_EFFORT_WEEK of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE_EFFORT_WEEK = 24;

    public static final int TYPE_BASELINE_EFFORT_WEEK = Types.BIGINT;

    public static final String NAME_BASELINE_EFFORT_WEEK = "SUMMARY_TIMESHEETS.BASELINE_EFFORT_WEEK";

    // public static final String PROPERTY_BASELINE_EFFORT_WEEK =
    // "PROPERTYNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_EFFORT_WEEK,
    // NAME_BASELINE_EFFORT_WEEK);
    // }

    /**
     * Column WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WORK_TODATE = 25;

    public static final int TYPE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_WORK_TODATE = "SUMMARY_TIMESHEETS.WORK_TODATE";

    // public static final String PROPERTY_WORK_TODATE = "PROPERTYNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE, NAME_WORK_TODATE);
    // }

    /**
     * Column BILLABLE_WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BILLABLE_WORK_TODATE = 26;

    public static final int TYPE_BILLABLE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_BILLABLE_WORK_TODATE = "SUMMARY_TIMESHEETS.BILLABLE_WORK_TODATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BILLABLE_WORK_TODATE =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_BILLABLE_WORK_TODATE,NAME_BILLABLE_WORK_TODATE);}

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 27;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "SUMMARY_TIMESHEETS.ETC";
    
    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ETC = "EETCHOURS";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_ETC, NAME_ETC + "/60.0");
    // }

    /**
     * Column MILESTONE_PERCENT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MILESTONE_PERCENT = 28;

    public static final int TYPE_MILESTONE_PERCENT = Types.INTEGER;

    public static final String NAME_MILESTONE_PERCENT = "SUMMARY_TIMESHEETS.MILESTONE_PERCENT";
    
    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MILESTONE_PERCENT =
    // "PERCENTAGECOMPLETE";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_MILESTONE_PERCENT, NAME_MILESTONE_PERCENT
    // + "/100.0");
    // }
    static
    {
        FIELDPROPERTYMAP.put("COMPLETE", SqlUtil.getBooleanValue(
            NAME_MILESTONE_PERCENT, 10000));
    }

    /**
     * Column BASELINE_REGULAR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE_REGULAR_COST = 29;

    public static final int TYPE_BASELINE_REGULAR_COST = Types.DECIMAL;

    public static final String NAME_BASELINE_REGULAR_COST = "SUMMARY_TIMESHEETS.BASELINE_REGULAR_COST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BASELINE_REGULAR_COST =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_REGULAR_COST,NAME_BASELINE_REGULAR_COST);}

    /**
     * Column BASELINE_SPECIAL_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE_SPECIAL_COST = 30;

    public static final int TYPE_BASELINE_SPECIAL_COST = Types.DECIMAL;

    public static final String NAME_BASELINE_SPECIAL_COST = "SUMMARY_TIMESHEETS.BASELINE_SPECIAL_COST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BASELINE_SPECIAL_COST =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_SPECIAL_COST,NAME_BASELINE_SPECIAL_COST);}

    /**
     * Column BASELINE_REGULAR_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE_REGULAR_RATE = 31;

    public static final int TYPE_BASELINE_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_BASELINE_REGULAR_RATE = "SUMMARY_TIMESHEETS.BASELINE_REGULAR_RATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BASELINE_REGULAR_RATE =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_REGULAR_RATE,NAME_BASELINE_REGULAR_RATE);}

    /**
     * Column BASELINE_SPECIAL_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE_SPECIAL_RATE = 32;

    public static final int TYPE_BASELINE_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_BASELINE_SPECIAL_RATE = "SUMMARY_TIMESHEETS.BASELINE_SPECIAL_RATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BASELINE_SPECIAL_RATE =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_SPECIAL_RATE,NAME_BASELINE_SPECIAL_RATE);}

    /**
     * Column ACTUAL_REGULAR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_REGULAR_COST = 33;

    public static final int TYPE_ACTUAL_REGULAR_COST = Types.DECIMAL;

    public static final String NAME_ACTUAL_REGULAR_COST = "SUMMARY_TIMESHEETS.ACTUAL_REGULAR_COST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTUAL_REGULAR_COST = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_REGULAR_COST,NAME_ACTUAL_REGULAR_COST);}

    /**
     * Column ACTUAL_SPECIAL_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_SPECIAL_COST = 34;

    public static final int TYPE_ACTUAL_SPECIAL_COST = Types.DECIMAL;

    public static final String NAME_ACTUAL_SPECIAL_COST = "SUMMARY_TIMESHEETS.ACTUAL_SPECIAL_COST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTUAL_SPECIAL_COST = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_SPECIAL_COST,NAME_ACTUAL_SPECIAL_COST);}

    /**
     * Column ACTUAL_REGULAR_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_REGULAR_RATE = 35;

    public static final int TYPE_ACTUAL_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_ACTUAL_REGULAR_RATE = "SUMMARY_TIMESHEETS.ACTUAL_REGULAR_RATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTUAL_REGULAR_RATE = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_REGULAR_RATE,NAME_ACTUAL_REGULAR_RATE);}

    /**
     * Column ACTUAL_SPECIAL_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_SPECIAL_RATE = 36;

    public static final int TYPE_ACTUAL_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_ACTUAL_SPECIAL_RATE = "SUMMARY_TIMESHEETS.ACTUAL_SPECIAL_RATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTUAL_SPECIAL_RATE = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_SPECIAL_RATE,NAME_ACTUAL_SPECIAL_RATE);}

    /**
     * Column SYSTEM_EXCHANGE_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SYSTEM_EXCHANGE_RATE = 37;

    public static final int TYPE_SYSTEM_EXCHANGE_RATE = Types.DECIMAL;

    public static final String NAME_SYSTEM_EXCHANGE_RATE = "SUMMARY_TIMESHEETS.SYSTEM_EXCHANGE_RATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SYSTEM_EXCHANGE_RATE =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_SYSTEM_EXCHANGE_RATE,NAME_SYSTEM_EXCHANGE_RATE);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 38;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "SUMMARY_TIMESHEETS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 39;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "SUMMARY_TIMESHEETS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 40;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "SUMMARY_TIMESHEETS.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column TIMECODE_1_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_ID = 41;

    public static final int TYPE_TIMECODE_1_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_1_ID = "SUMMARY_TIMESHEETS.TIMECODE_1_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TIMECODE_1_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_ID,NAME_TIMECODE_1_ID);}

    /**
     * Column TIMECODE_2_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_ID = 42;

    public static final int TYPE_TIMECODE_2_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_2_ID = "SUMMARY_TIMESHEETS.TIMECODE_2_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TIMECODE_2_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_ID,NAME_TIMECODE_2_ID);}

    /**
     * Column ACTIVITY_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ACTIVITY_ID = 43;

    public static final int TYPE_ACTIVITY_ID = Types.VARCHAR;

    public static final String NAME_ACTIVITY_ID = "SUMMARY_TIMESHEETS.ACTIVITY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTIVITY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ACTIVITY_ID,NAME_ACTIVITY_ID);}

    /**
     * Column NOTES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NOTES = 44;

    public static final int TYPE_NOTES = Types.VARCHAR;

    public static final String NAME_NOTES = "SUMMARY_TIMESHEETS.NOTES";

    public static final String PROPERTY_NOTES = "SUMMARYNOTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_NOTES, NAME_NOTES);
    }

    /**
     * Column BASELINE_WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE_WORK_TODATE = 45;

    public static final int TYPE_BASELINE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_BASELINE_WORK_TODATE = "SUMMARY_TIMESHEETS.BASELINE_WORK_TODATE";

    // public static final String PROPERTY_BASELINE_WORK_TODATE =
    // "PROPERTYNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_WORK_TODATE,
    // NAME_BASELINE_WORK_TODATE + "/60.0");
    // }

    /**
     * Column BAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BAC = 46;

    public static final int TYPE_BAC = Types.DECIMAL;

    public static final String NAME_BAC = "SUMMARY_TIMESHEETS.BAC";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BAC = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_BAC,NAME_BAC);}

    /**
     * Column EAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EAC = 47;

    public static final int TYPE_EAC = Types.DECIMAL;

    public static final String NAME_EAC = "SUMMARY_TIMESHEETS.EAC";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EAC = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EAC,NAME_EAC);}

    /**
     * Column BCWS of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWS = 48;

    public static final int TYPE_BCWS = Types.DECIMAL;

    public static final String NAME_BCWS = "SUMMARY_TIMESHEETS.BCWS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BCWS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_BCWS,NAME_BCWS);}

    /**
     * Column BCWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWP = 49;

    public static final int TYPE_BCWP = Types.DECIMAL;

    public static final String NAME_BCWP = "SUMMARY_TIMESHEETS.BCWP";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BCWP = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_BCWP,NAME_BCWP);}

    /**
     * Column ACWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ACWP = 50;

    public static final int TYPE_ACWP = Types.DECIMAL;

    public static final String NAME_ACWP = "SUMMARY_TIMESHEETS.ACWP";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACWP = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ACWP,NAME_ACWP);}

    /**
     * Column PDA_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_PDA_FLAG = 51;

    public static final int TYPE_PDA_FLAG = Types.CHAR;

    public static final String NAME_PDA_FLAG = "SUMMARY_TIMESHEETS.PDA_FLAG";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PDA_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PDA_FLAG,NAME_PDA_FLAG);}

    /**
     * Column EV_ETC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EV_ETC = 52;

    public static final int TYPE_EV_ETC = Types.DECIMAL;

    public static final String NAME_EV_ETC = "SUMMARY_TIMESHEETS.EV_ETC";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EV_ETC = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EV_ETC,NAME_EV_ETC);}

    /**
     * Column MONTH1_REGULAR_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH1_REGULAR_HRS = 53;

    public static final int TYPE_MONTH1_REGULAR_HRS = Types.INTEGER;

    public static final String NAME_MONTH1_REGULAR_HRS = "SUMMARY_TIMESHEETS.MONTH1_REGULAR_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH1_REGULAR_HRS = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH1_REGULAR_HRS,NAME_MONTH1_REGULAR_HRS);}

    /**
     * Column MONTH1_SPECIAL_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH1_SPECIAL_HRS = 54;

    public static final int TYPE_MONTH1_SPECIAL_HRS = Types.INTEGER;

    public static final String NAME_MONTH1_SPECIAL_HRS = "SUMMARY_TIMESHEETS.MONTH1_SPECIAL_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH1_SPECIAL_HRS = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH1_SPECIAL_HRS,NAME_MONTH1_SPECIAL_HRS);}

    /**
     * Column MONTH2_REGULAR_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH2_REGULAR_HRS = 55;

    public static final int TYPE_MONTH2_REGULAR_HRS = Types.INTEGER;

    public static final String NAME_MONTH2_REGULAR_HRS = "SUMMARY_TIMESHEETS.MONTH2_REGULAR_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH2_REGULAR_HRS = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH2_REGULAR_HRS,NAME_MONTH2_REGULAR_HRS);}

    /**
     * Column MONTH2_SPECIAL_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH2_SPECIAL_HRS = 56;

    public static final int TYPE_MONTH2_SPECIAL_HRS = Types.INTEGER;

    public static final String NAME_MONTH2_SPECIAL_HRS = "SUMMARY_TIMESHEETS.MONTH2_SPECIAL_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH2_SPECIAL_HRS = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH2_SPECIAL_HRS,NAME_MONTH2_SPECIAL_HRS);}

    /**
     * Column MONTH1_B_REGULAR_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH1_B_REGULAR_HRS = 57;

    public static final int TYPE_MONTH1_B_REGULAR_HRS = Types.INTEGER;

    public static final String NAME_MONTH1_B_REGULAR_HRS = "SUMMARY_TIMESHEETS.MONTH1_B_REGULAR_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH1_B_REGULAR_HRS =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH1_B_REGULAR_HRS,NAME_MONTH1_B_REGULAR_HRS);}

    /**
     * Column MONTH1_B_SPECIAL_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH1_B_SPECIAL_HRS = 58;

    public static final int TYPE_MONTH1_B_SPECIAL_HRS = Types.INTEGER;

    public static final String NAME_MONTH1_B_SPECIAL_HRS = "SUMMARY_TIMESHEETS.MONTH1_B_SPECIAL_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH1_B_SPECIAL_HRS =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH1_B_SPECIAL_HRS,NAME_MONTH1_B_SPECIAL_HRS);}

    /**
     * Column MONTH2_B_REGULAR_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH2_B_REGULAR_HRS = 59;

    public static final int TYPE_MONTH2_B_REGULAR_HRS = Types.INTEGER;

    public static final String NAME_MONTH2_B_REGULAR_HRS = "SUMMARY_TIMESHEETS.MONTH2_B_REGULAR_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH2_B_REGULAR_HRS =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH2_B_REGULAR_HRS,NAME_MONTH2_B_REGULAR_HRS);}

    /**
     * Column MONTH2_B_SPECIAL_HRS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MONTH2_B_SPECIAL_HRS = 60;

    public static final int TYPE_MONTH2_B_SPECIAL_HRS = Types.INTEGER;

    public static final String NAME_MONTH2_B_SPECIAL_HRS = "SUMMARY_TIMESHEETS.MONTH2_B_SPECIAL_HRS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MONTH2_B_SPECIAL_HRS =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_MONTH2_B_SPECIAL_HRS,NAME_MONTH2_B_SPECIAL_HRS);}

    /**
     * Column DEPARTMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_DEPARTMENT_ID = 61;

    public static final int TYPE_DEPARTMENT_ID = Types.CHAR;

    public static final String NAME_DEPARTMENT_ID = "SUMMARY_TIMESHEETS.DEPARTMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DEPARTMENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DEPARTMENT_ID,NAME_DEPARTMENT_ID);}

    /**
     * Column COST_CENTER_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_COST_CENTER_BITS = 62;

    public static final int TYPE_COST_CENTER_BITS = Types.INTEGER;

    public static final String NAME_COST_CENTER_BITS = "SUMMARY_TIMESHEETS.COST_CENTER_BITS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_COST_CENTER_BITS = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_BITS,NAME_COST_CENTER_BITS);}

    public static final String TABLE_NAME = "SUMMARY_TIMESHEETS";

    /**
     * Create an array of type string containing all the fields of the
     * SUMMARY_TIMESHEETS table.
     */
    private static final String[] FIELD_NAMES =
    {
        NAME_PROJECT_ID, NAME_RESOURCE_ID, NAME_TSK_ASSIGNMENT_ID,
        NAME_WEEK_OF, NAME_MONDAY_REGULAR, NAME_MONDAY_SPECIAL,
        NAME_TUESDAY_REGULAR, NAME_TUESDAY_SPECIAL, NAME_WEDNESDAY_REGULAR,
        NAME_WEDNESDAY_SPECIAL, NAME_THURSDAY_REGULAR,
        NAME_THURSDAY_SPECIAL, NAME_FRIDAY_REGULAR, NAME_FRIDAY_SPECIAL,
        NAME_SATURDAY_REGULAR, NAME_SATURDAY_SPECIAL, NAME_SUNDAY_REGULAR,
        NAME_SUNDAY_SPECIAL, NAME_WEEKLY_REGULAR_TOTAL,
        NAME_WEEKLY_SPECIAL_TOTAL, NAME_WEEKLY_BILLABLE_REGULAR_TOTAL,
        NAME_WEEKLY_BILLABLE_SPECIAL_TOTAL, NAME_TOTAL_SCHED_EFFORT,
        NAME_BASELINE_EFFORT_WEEK, NAME_WORK_TODATE,
        NAME_BILLABLE_WORK_TODATE, NAME_ETC, NAME_MILESTONE_PERCENT,
        NAME_BASELINE_REGULAR_COST, NAME_BASELINE_SPECIAL_COST,
        NAME_BASELINE_REGULAR_RATE, NAME_BASELINE_SPECIAL_RATE,
        NAME_ACTUAL_REGULAR_COST, NAME_ACTUAL_SPECIAL_COST,
        NAME_ACTUAL_REGULAR_RATE, NAME_ACTUAL_SPECIAL_RATE,
        NAME_SYSTEM_EXCHANGE_RATE, NAME_REC_USER, NAME_REC_STATUS,
        NAME_REC_DATETIME, NAME_TIMECODE_1_ID, NAME_TIMECODE_2_ID,
        NAME_ACTIVITY_ID, NAME_NOTES, NAME_BASELINE_WORK_TODATE, NAME_BAC,
        NAME_EAC, NAME_BCWS, NAME_BCWP, NAME_ACWP, NAME_PDA_FLAG,
        NAME_EV_ETC, NAME_MONTH1_REGULAR_HRS, NAME_MONTH1_SPECIAL_HRS,
        NAME_MONTH2_REGULAR_HRS, NAME_MONTH2_SPECIAL_HRS,
        NAME_MONTH1_B_REGULAR_HRS, NAME_MONTH1_B_SPECIAL_HRS,
        NAME_MONTH2_B_REGULAR_HRS, NAME_MONTH2_B_SPECIAL_HRS,
        NAME_DEPARTMENT_ID, NAME_COST_CENTER_BITS
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new SummaryTimesheetManager instance.
     */
    public SummaryTimesheetManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return null;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        return new String[]
        {
                NAME_TSK_ASSIGNMENT_ID, NAME_WEEK_OF
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        String taskAssignmentId = null;
        if (summaryTimesheet.getTaskAssignment() != null)
        {
            taskAssignmentId = summaryTimesheet.getTaskAssignment().getID();
        }
        else
        {
            taskAssignmentId = summaryTimesheet.getID();
        }

        String weekOf = DateUtil.convertToStringNoTime(summaryTimesheet.getWeekOf().getTime());

        return new Object[]
        {
                taskAssignmentId, weekOf
        };
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        if (summaryTimesheet.getID() == null
                && (summaryTimesheet.getTaskAssignment() == null || (summaryTimesheet
                    .getTaskAssignment() != null && summaryTimesheet
                    .getTaskAssignment().getID() == null)))
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }

        if (summaryTimesheet.getWeekOf() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(" and ");
            invalidPrimaryKeys.append(ValidationConstants.WEEK_OF_FIELD);
            valid = false;
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                invalidPrimaryKeys.toString());
        }
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject) throws RPMException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;
        summaryTimesheet.setID((String) primaryKey[0]);

        // set the week-of
        Calendar weekOf = null;
        try
        {
            weekOf = DateUtil.convertToCalendarNoTime((String) primaryKey[1]);
        }
        catch (ParseException e)
        {
            ExceptionUtil.handleException(Manager.class, e);
        }
        summaryTimesheet.setWeekOf(weekOf);

        return summaryTimesheet;
    }

    public String getCheckInCheckOutPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        String taskAssignmentId = null;
        if (summaryTimesheet.getTaskAssignment() != null
                && summaryTimesheet.getTaskAssignment().getID() != null)
        {
            taskAssignmentId = summaryTimesheet.getTaskAssignment().getID();
        }
        else
        {
            taskAssignmentId = summaryTimesheet.getID();
        }

        return TimesheetManagerUtil.getCheckInCheckOutPrimaryKey(
            taskAssignmentId, messageContext).toString();
    }

    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_TSK_ASSIGNMENT_ID + " IN ");
        sql.appendOpenParenthesis();
        sql.append("SELECT " + TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
        sql.append(" FROM " + TaskAssignmentManager.TABLE_NAME + " WHERE ");
        sql.append(new TaskAssignmentManager().getFilter());
        sql.appendOr();
        sql.append(new PersonalTaskAssignmentManager().getFilter());
        sql.appendOr();
        sql.append(new AdministrativeTaskAssignmentManager().getFilter());
        sql.appendCloseParenthesis();

        return sql.toString();
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            // example: /SummaryTimesheet
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            joinCondition.append(getFilter());

        }

        else if (manager instanceof ResourceManager)
        {
            // example: /SummaryTimesheet/Resource
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(ResourceManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());
        }

        else if (manager instanceof WbsManager)
        {

            joinCondition = new JoinCondition();
            if ("project".equalsIgnoreCase(container.getFieldName()))
            {
                // example: /SummaryTimesheet/@project
                joinCondition.appendSubSelect(WbsManager.NAME_ELEMENT_ID,
                    NAME_PROJECT_ID, TABLE_NAME, getFilter());
            }
            else
            {
                // example: /SummaryTimesheet/@workElement
                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    NAME_TSK_ASSIGNMENT_ID, TABLE_NAME, getFilter());

                joinCondition = JoinCondition.createSubSelect(
                    WbsManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_TASK_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }
        }
        else if (manager instanceof TaskAssignmentManager)
        {
            // example: /SummaryTimesheet/taskAssignment
            joinCondition = new JoinCondition();
            joinCondition = JoinCondition.createSubSelect(
                TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                NAME_TSK_ASSIGNMENT_ID, TABLE_NAME, getFilter());
        }
        else if (manager instanceof GenericTaskAssignmentManager)
        {
            // example: /SummaryTimesheet/genericTaskAssignment
            joinCondition = new JoinCondition();
            joinCondition = JoinCondition.createSubSelect(
                TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                NAME_TSK_ASSIGNMENT_ID, TABLE_NAME, getFilter());
        }
        else if (manager instanceof TimecodeManager)
        {

            joinCondition = new JoinCondition();
            if ("timecode1".equalsIgnoreCase(container.getFieldName()))
            {
                // example: /SummaryTimesheet/@timecode1
                joinCondition.appendSubSelect(TimecodeManager.NAME_ELEMENT_ID,
                    NAME_TIMECODE_1_ID, TABLE_NAME, getFilter());
            }
            else
            {
                // example: /SummaryTimesheet/@timecode2
                joinCondition.appendSubSelect(TimecodeManager.NAME_ELEMENT_ID,
                    NAME_TIMECODE_2_ID, TABLE_NAME, getFilter());
            }
        }

        else if (manager instanceof TimesheetApprovalStatusManager)
        {

            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.append(TimesheetApprovalStatusManager.NAME_WEEK_OF);
            joinCondition.appendEqual();
            joinCondition.append(NAME_WEEK_OF);
            joinCondition.appendAnd();
            joinCondition
                .append(TimesheetApprovalStatusManager.NAME_RESOURCE_ID);
            joinCondition.appendEqual();
            joinCondition.append(NAME_RESOURCE_ID);
            joinCondition.appendAnd();
            joinCondition
                .append(TimesheetApprovalStatusManager.NAME_ELEMENT_ID);
            joinCondition.appendEqual();
            joinCondition.append(NAME_PROJECT_ID);

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {

            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(ResourceManager.NAME_RESOURCE_ID,
                NAME_RESOURCE_ID, TABLE_NAME, getFilter());
        }

        else if (manager instanceof WbsManager)
        {

            joinCondition = new JoinCondition();
            if ("project".equalsIgnoreCase(container.getPreviousFieldName()))
            {
                joinCondition.appendSubSelect(WbsManager.NAME_ELEMENT_ID,
                    NAME_PROJECT_ID, TABLE_NAME, getFilter());
            }
            else
            {

                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TASK_ID,
                    WbsManager.NAME_ELEMENT_ID, WbsManager.TABLE_NAME, manager
                        .getFilter(contextName));

                joinCondition = JoinCondition.createSubSelect(
                    WbsManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_TASK_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {

        return new SummaryTimesheet();

    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        double regularTotal = DateUtil.convertMinutesToHours(
            rs.getInt(ID_WEEKLY_REGULAR_TOTAL)).doubleValue();
        double specialTotal = DateUtil.convertMinutesToHours(
            rs.getInt(ID_WEEKLY_SPECIAL_TOTAL)).doubleValue();
        double totalHours = regularTotal + specialTotal;

        if (isDelta)
        {
            summaryTimesheet.deltaWeekOf(Manager.getCalendar(rs, ID_WEEK_OF));
            summaryTimesheet.deltaRegularFriday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_FRIDAY_REGULAR)));
            summaryTimesheet.deltaRegularMonday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_MONDAY_REGULAR)));
            summaryTimesheet.deltaRegularSaturday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SATURDAY_REGULAR)));
            summaryTimesheet.deltaRegularSunday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SUNDAY_REGULAR)));
            summaryTimesheet.deltaRegularThursday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_THURSDAY_REGULAR)));
            summaryTimesheet.deltaRegularTuesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_TUESDAY_REGULAR)));
            summaryTimesheet.deltaRegularWednesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_WEDNESDAY_REGULAR)));
            summaryTimesheet.deltaSpecialFriday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_FRIDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialMonday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_MONDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialSaturday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SATURDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialSunday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SUNDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialThursday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_THURSDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialTuesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_TUESDAY_SPECIAL)));
            summaryTimesheet.deltaSpecialWednesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_WEDNESDAY_SPECIAL)));
            summaryTimesheet.deltaSummaryNotes(rs.getString(ID_NOTES));
            summaryTimesheet.deltaTotalRegular(new Double(regularTotal));
            summaryTimesheet.deltaTotalSpecial(new Double(specialTotal));
            summaryTimesheet.deltaTotalWeek(new Double(totalHours));
            summaryTimesheet.deltaCostPerHour(new Double(rs
                .getDouble(ID_ACTUAL_REGULAR_COST)));
            summaryTimesheet.deltaSellingRatePerHour(new Double(rs
                .getDouble(ID_ACTUAL_REGULAR_RATE)));
            summaryTimesheet.deltaSpecialCostPerHour(new Double(rs
                .getDouble(ID_ACTUAL_SPECIAL_COST)));
            summaryTimesheet.deltaSpecialSellingRatePerHour(new Double(rs
                .getDouble(ID_ACTUAL_SPECIAL_RATE)));
        }
        else
        {
            summaryTimesheet.setWeekOf(Manager.getCalendar(rs, ID_WEEK_OF));
            summaryTimesheet.setRegularFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_REGULAR)));
            summaryTimesheet.setRegularMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_REGULAR)));
            summaryTimesheet.setRegularSaturday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SATURDAY_REGULAR)));
            summaryTimesheet.setRegularSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_REGULAR)));
            summaryTimesheet.setRegularThursday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_THURSDAY_REGULAR)));
            summaryTimesheet.setRegularTuesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_TUESDAY_REGULAR)));
            summaryTimesheet.setRegularWednesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_WEDNESDAY_REGULAR)));
            summaryTimesheet.setSpecialFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_SPECIAL)));
            summaryTimesheet.setSpecialMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_SPECIAL)));
            summaryTimesheet.setSpecialSaturday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_SATURDAY_SPECIAL)));
            summaryTimesheet.setSpecialSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_SPECIAL)));
            summaryTimesheet.setSpecialThursday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_THURSDAY_SPECIAL)));
            summaryTimesheet.setSpecialTuesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_TUESDAY_SPECIAL)));
            summaryTimesheet.setSpecialWednesday(DateUtil
                .convertMinutesToHours(rs.getInt(ID_WEDNESDAY_SPECIAL)));
            summaryTimesheet.setSummaryNotes(rs.getString(ID_NOTES));
            summaryTimesheet.setTotalRegular(new Double(regularTotal));
            summaryTimesheet.setTotalSpecial(new Double(specialTotal));
            summaryTimesheet.setTotalWeek(new Double(totalHours));
            summaryTimesheet.setCostPerHour(new Double(rs
                .getDouble(ID_ACTUAL_REGULAR_COST)));
            summaryTimesheet.setSellingRatePerHour(new Double(rs
                .getDouble(ID_ACTUAL_REGULAR_RATE)));
            summaryTimesheet.setSpecialCostPerHour(new Double(rs
                .getDouble(ID_ACTUAL_SPECIAL_COST)));
            summaryTimesheet.setSpecialSellingRatePerHour(new Double(rs
                .getDouble(ID_ACTUAL_SPECIAL_RATE)));
        }

        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String timeCode1Id = resultSet.getString(ID_TIMECODE_1_ID);
        relatedFields.put(index, ID_TIMECODE_1_ID, timeCode1Id);

        String timeCode2Id = resultSet.getString(ID_TIMECODE_2_ID);
        relatedFields.put(index, ID_TIMECODE_2_ID, timeCode2Id);

        String taskAssignmentId = resultSet.getString(ID_TSK_ASSIGNMENT_ID);
        relatedFields.put(index, ID_TSK_ASSIGNMENT_ID, taskAssignmentId);

        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);

        String resourceId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, resourceId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        // always load the taskAssignment
        summaryTimesheet = loadTaskAssignment(summaryTimesheet, null,
            messageContext, isDelta);

        if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {

            // note: cannot be set in the decode row method because it is not in
            // the same table
            Calendar forecastFinishDate = ((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastFinishDate();
            if (isDelta)
            {
                summaryTimesheet.deltaForecastFinishDate(forecastFinishDate);
            }
            else
            {
                summaryTimesheet.setForecastFinishDate(forecastFinishDate);
            }

            Calendar forecastStartDate = ((TaskAssignment) summaryTimesheet
                .getTaskAssignment()).getForecastStartDate();
            if (isDelta)
            {
                summaryTimesheet.deltaForecastStartDate(forecastStartDate);
            }
            else
            {
                summaryTimesheet.setForecastStartDate(forecastStartDate);
            }

        }

        // always load expectedWorkPerWeek

        Double expectedWorkPerWeek = TimesheetManagerUtil
            .retrieveSummaryTimesheet_EffortPerWeek(summaryTimesheet,
                messageContext);

        if (isDelta)
        {
            summaryTimesheet.deltaExpectedWorkPerWeek(expectedWorkPerWeek);
        }
        else
        {
            summaryTimesheet.setExpectedWorkPerWeek(expectedWorkPerWeek);
        }

        if (rpmScope != null && rpmScope instanceof SummaryTimesheetScope)
        {
            SummaryTimesheetScope scope = (SummaryTimesheetScope) rpmScope;

            // Timecodes and Project are used only for summaryTimesheets related
            // to a TaskAssignment
            if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
            {
                if (scope.isTimeCodes())
                {
                    loadTimeCodes(summaryTimesheet, scope, messageContext,
                        relatedFields, index, isDelta);
                }

                if (scope.isProject())
                {
                    loadProject(summaryTimesheet, scope, messageContext,
                        relatedFields, index, isDelta);
                }

                if (scope.getWorkElement() != null)
                {
                    loadWorkElement(summaryTimesheet, scope, messageContext,
                        relatedFields, index, isDelta);
                }
            }

            if (scope.getApprovalStatus() != null)
            {
                loadApprovalStatus(summaryTimesheet, scope, messageContext,
                    isDelta);
            }

            if (scope.isResource())
            {
                loadResource(summaryTimesheet, messageContext, relatedFields,
                    index, isDelta);
            }

        }

        return summaryTimesheet;
    }

    private SummaryTimesheet loadTaskAssignment(
            SummaryTimesheet summaryTimesheet, SummaryTimesheetScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        GenericTaskAssignment taskAssignment = summaryTimesheet
            .getTaskAssignment();

        if (taskAssignment == null)
        {
            taskAssignment = (GenericTaskAssignment) TimesheetManagerUtil
                .createAbstractTaskAssignment(summaryTimesheet, messageContext);
            taskAssignment.setID(summaryTimesheet.getID());
        }

        boolean isGeneric = GenericTaskAssignment.class.equals(taskAssignment.getClass());
        
        RPMObjectManager manager = getManagerInstance(taskAssignment.getClass());

        taskAssignment = (GenericTaskAssignment) manager.loadByPrimaryKey(
            taskAssignment, null, messageContext, isDelta);

        // If the user passes GenericTaskAssignment, we should calculate EETC
        // and PercentageComplete all the time by setting them to null
        if (isGeneric && isDelta && taskAssignment instanceof TaskAssignment)
        {
            ((TaskAssignment)taskAssignment).setEETCHours(null);
            ((TaskAssignment)taskAssignment).setPercentageComplete(null);
        }
        
        // note: we are not using the delta method because taskAssignmentId
        // cannot be changed (it is part of the
        // primaryKey).
        summaryTimesheet.setTaskAssignment(taskAssignment);

        return summaryTimesheet;
    }

    private void loadTimeCodes(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!scope.isTimeCodes())
            return;

        String timeCode1Id = (String) relatedFields
            .get(index, ID_TIMECODE_1_ID);
        TimeCode oldTimeCode1 = summaryTimesheet.getTimeCode1();
        TimeCode loadedTimeCode1 = loadTimeCode(null, messageContext,
            timeCode1Id, oldTimeCode1, isDelta);

        String timeCode2Id = (String) relatedFields
            .get(index, ID_TIMECODE_2_ID);
        TimeCode oldTimeCode2 = summaryTimesheet.getTimeCode2();
        TimeCode loadedTimeCode2 = loadTimeCode(null, messageContext,
            timeCode2Id, oldTimeCode2, isDelta);

        if (isDelta)
        {
            summaryTimesheet.deltaTimeCode1(loadedTimeCode1);
            summaryTimesheet.deltaTimeCode2(loadedTimeCode2);
        }
        else
        {
            summaryTimesheet.setTimeCode1(loadedTimeCode1);
            summaryTimesheet.setTimeCode2(loadedTimeCode2);
        }
    }

    private TimeCode loadTimeCode(TimeCodeScope timeCodeScope,
            MessageContext messageContext, String oldTimeCodeID,
            TimeCode previousTimeCode, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
        TimecodeManager manager = (TimecodeManager) getManagerInstance(TimeCode.class
            .getName());

        TimeCode newTimeCode = null;
        if (isDelta)
        {
            newTimeCode = previousTimeCode;
            TimeCode oldTimeCode = null;
            if (oldTimeCodeID != null)
            {
                oldTimeCode = new TimeCode();
                oldTimeCode.setID(oldTimeCodeID);
                if (newTimeCode != null && newTimeCode.getID() != null
                        && newTimeCode.getID().equals(oldTimeCodeID))
                {
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        newTimeCode, timeCodeScope, messageContext, true);
                }
                else
                {
                    // In this case, old timecode is loaded without the
                    // scope to avoid problem with the cache
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        oldTimeCode, timeCodeScope, messageContext, false);
                }
            }
            return oldTimeCode;
        }

        if (oldTimeCodeID != null)
        {
            if (newTimeCode == null)
            { // newTimecode not in cache
                newTimeCode = new TimeCode();
                newTimeCode.setID(oldTimeCodeID);
                newTimeCode = (TimeCode) manager.loadByPrimaryKey(newTimeCode,
                    timeCodeScope, messageContext, false);
            }
            return newTimeCode;
        }

        return null;
    }

    private void loadProject(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        GenericProject project = null;

        String projectId = (String) relatedFields.get(index, ID_PROJECT_ID);

        if (projectId != null)
        {
            project = new GenericProject();
            project.setID(projectId);

            RPMObjectManager manager = getManagerInstance(GenericProject.class);

            project = (GenericProject) manager.loadByPrimaryKey(project, null,
                messageContext, isDelta);
        }

        if (isDelta)
        {
            summaryTimesheet.deltaProject(project);
        }
        else
        {
            summaryTimesheet.setProject(project);
        }

    }

    private void loadResource(SummaryTimesheet summaryTimesheet,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        Resource resource = null;

        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);

        if (resourceId != null)
        {
            resource = new Resource();
            resource.setID(resourceId);

            RPMObjectManager manager = getManagerInstance(Resource.class);

            resource = (Resource) manager.loadByPrimaryKey(resource, null,
                messageContext, isDelta);
        }

        if (isDelta)
        {
            summaryTimesheet.deltaResource(resource);
        }
        else
        {
            summaryTimesheet.setResource(resource);
        }

    }

    private void loadWorkElement(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        WorkElement workElement = summaryTimesheet.getWorkElement();

        String taskAssignmentId = (String) relatedFields.get(index,
            ID_TSK_ASSIGNMENT_ID);

        String workElementId = TimesheetManagerUtil
            .retrieveTaskAssignment_TaskId(taskAssignmentId, messageContext);

        if (workElement == null)
        {
            workElement = new WorkElement();
            workElement.setID(workElementId);
        }

        RPMObjectManager manager = getManagerInstance(workElement.getClass());

        workElement = (WorkElement) manager.loadByPrimaryKey(workElement, scope
            .getWorkElement(), messageContext, isDelta);

        if (isDelta)
        {
            summaryTimesheet.deltaWorkElement(workElement);
        }
        else
        {
            summaryTimesheet.setWorkElement(workElement);
        }

    }

    private void loadApprovalStatus(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        String projectId = TimesheetManagerUtil
            .retrieveTaskAssignment_ProjectId(summaryTimesheet
                .getTaskAssignment().getID(), messageContext);
        String resourceId = TimesheetManagerUtil
            .retrieveTaskAssignment_ResourceId(summaryTimesheet
                .getTaskAssignment().getID(), messageContext);

        TimesheetApprovalStatus appStatus = null;
        TimesheetApprovalStatusManager manager = (TimesheetApprovalStatusManager) getManagerInstance(TimesheetApprovalStatus.class
            .getName());

        ArrayList temp = manager.loadByForeignKey(new RPMObject[]
        {
            summaryTimesheet.getApprovalStatus()
        }, scope.getApprovalStatus(), messageContext, this, new Object[]
        {
                resourceId, projectId,
                DateUtil.convertToString(summaryTimesheet.getWeekOf())
        }, null, null, isDelta);

        // There cannot be more than one TimesheetApprovalStatus with the same
        // resource, project, and weekOf.
        if (temp != null && temp.size() == 1)
            appStatus = (TimesheetApprovalStatus) temp.get(0);

        // note: cannot use isDelta on this field
        summaryTimesheet.setApprovalStatus(appStatus);
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;

        // Retrieve the start of that week
        TimesheetManagerUtil.setValidWeekOf(summaryTimesheet, messageContext);
        summaryTimesheet.setID(summaryTimesheet.getTaskAssignment().getID());

        summaryTimesheet = (SummaryTimesheet) TimesheetManagerUtil
            .initWorkingHours(summaryTimesheet);

    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        SummaryTimesheet summaryTimesheet = (SummaryTimesheet) rpmObject;
        SummaryTimesheetScope scope = (SummaryTimesheetScope) rpmScope;

        if (TimesheetManagerUtil.hasNewActualReported(summaryTimesheet)
                || (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment && ((TaskAssignment) summaryTimesheet
                    .getTaskAssignment()).testEETCHoursModified())
                || (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment && ((TaskAssignment) summaryTimesheet
                    .getTaskAssignment()).testPercentageCompleteModified())
                || summaryTimesheet.testTimeCode1Modified()
                || summaryTimesheet.testTimeCode2Modified()
                || summaryTimesheet.testSummaryNotesModified()
                || summaryTimesheet.testWeekOfModified()
                || summaryTimesheet.testForecastStartDateModified()
                || summaryTimesheet.testForecastFinishDateModified())
        {
            SP_U_SRV_TS(summaryTimesheet, scope, false, messageContext);
        }

        if (isApprovalStatusModified(summaryTimesheet, messageContext)
                || (scope != null && scope.getApprovalStatus() != null))
        {
            saveApprovalStatus(summaryTimesheet, scope, messageContext);
        }

        return rpmObject;
    }
    
    private boolean isApprovalStatusModified(SummaryTimesheet summaryTimesheet, 
            MessageContext msgCtx) throws RPMException
    {
        TimesheetApprovalStatus currentApprovalStatus = summaryTimesheet.getApprovalStatus();
        if (currentApprovalStatus == null)
        {
            return false;
        }

        TimesheetApprovalStatus loadedApprovalStatus = TimesheetManagerUtil.loadTimesheetApprovalStatus(
            summaryTimesheet, msgCtx);
        if (loadedApprovalStatus == null)
        {
            return true;
        }
        
        // check the ApprovalStatus
        if (currentApprovalStatus.getApprovalStatus().equals(loadedApprovalStatus.getApprovalStatus()))
        {
            return false;
        }
        
        return true;
    }

    private void saveApprovalStatus(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        TimesheetApprovalStatus appStatus = summaryTimesheet
            .getApprovalStatus();
        if (appStatus == null)
            return;

        // set the mandatory fields if they were missing

        if (appStatus.getResource() == null)
        {
            String resourceId = TimesheetManagerUtil
                .retrieveTaskAssignment_ResourceId(summaryTimesheet
                    .getTaskAssignment().getID(), messageContext);
            Resource resource = new Resource();
            resource.setID(resourceId);
            appStatus.setResource(resource);
        }

        // Always reset the weekOf to the summaryTimesheet weekOf
        appStatus.setWeekOf(summaryTimesheet.getWeekOf());

        RPMObjectManager manager = getManagerInstance(TimesheetApprovalStatus.class);

        TimesheetApprovalStatusScope approvalStatusScope = null;
        if (scope != null)
            approvalStatusScope = scope.getApprovalStatus();

        appStatus = (TimesheetApprovalStatus) manager.internalSave(appStatus,
            approvalStatusScope, messageContext);
        summaryTimesheet.setApprovalStatus(appStatus);
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // do nothing
    }

    protected boolean isCheckingOutOnCreate()
    {
        // Required to disable checking out/in on create because checkin
        // has a side effect on all other objects of that timesheet for
        // that user.
        return false;
    }
    
    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        String taskAssignmentId = rpmObject.getID();
        TimesheetManagerUtil.checkinObject(this, taskAssignmentId,
            messageContext);
    }

    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        String taskAssignmentId = rpmObject.getID();
        TimesheetManagerUtil.undoCheckOut(this, taskAssignmentId,
            messageContext);

    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        String taskAssignmentId = rpmObject.getID();
        TimesheetManagerUtil.checkoutObject(this, taskAssignmentId,
            messageContext);

    }

    public boolean isCheckedOut(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {
        String taskAssignmentID = rpmObject.getID();
        if (taskAssignmentID == null)
        {
            if (((SummaryTimesheet) rpmObject).getTaskAssignment() != null)
            {
                taskAssignmentID = ((SummaryTimesheet) rpmObject)
                    .getTaskAssignment().getID();
            }
        }

        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setID(taskAssignmentID);

        return TimesheetManagerUtil
            .isCheckedOut(messageContext, taskAssignment);
    }

    /**
     * 
     * @param summaryTimesheet
     * @param scope
     * @param childUpdate
     *            indicates that we are updating summaryTimesheet values with
     *            the summation of StepTimesheets and CreditTimesheets
     * @param messageContext
     * @return
     * @throws SQLException
     * @throws RPMException
     * @throws ParseException 
     */
    private String SP_U_SRV_TS(SummaryTimesheet summaryTimesheet,
            SummaryTimesheetScope scope, boolean childUpdate,
            MessageContext messageContext) throws SQLException, RPMException, ParseException
    {

        // Timecodes
        String timecode1Id = null;
        String timecode2Id = null;
        TimeCode timecode1 = summaryTimesheet.getTimeCode1();
        if (timecode1 != null)
        {
            timecode1Id = timecode1.getID();
        }
        TimeCode timecode2 = summaryTimesheet.getTimeCode2();
        if (timecode2 != null)
        {
            timecode2Id = timecode2.getID();
        }

        if (scope == null || !scope.isTimeCodes())
        {
            timecode1Id = TimesheetManagerUtil
                .retrieveSummaryTimesheetTimecode1(summaryTimesheet,
                    messageContext);
            timecode2Id = TimesheetManagerUtil
                .retrieveSummaryTimesheetTimecode2(summaryTimesheet,
                    messageContext);

        }

        GenericTaskAssignment taskAssignment = summaryTimesheet
            .getTaskAssignment();
        
        Calendar startDate = null;
        Calendar finishDate = null;
        String forecastStartDate = null;
        String forecastFinishDate = null;
        int workBaseline = 0;
        double eetc = 0.0;
        double percentageCompleted = 0.0;

        if (taskAssignment instanceof TaskAssignment)
        {
            TaskAssignment ta = (TaskAssignment) taskAssignment;
            startDate = ta.getAssignmentStartDate();
            finishDate = ta.getAssignmentFinishDate();
            if (ta.getBaselineWork() != null)
            {
                workBaseline = ta.getBaselineWork().intValue();
            }

            if (summaryTimesheet.testForecastStartDateModified())
            {
                forecastStartDate = DateUtil.convertToString(summaryTimesheet
                    .getForecastStartDate());
            }
            else
            {
                forecastStartDate = DateUtil.convertToString(ta
                    .getForecastStartDate());
            }

            if (summaryTimesheet.testForecastFinishDateModified())
            {
                forecastFinishDate = DateUtil.convertToString(summaryTimesheet
                    .getForecastFinishDate());
            }
            else
            {
                forecastFinishDate = DateUtil.convertToString(ta
                    .getForecastFinishDate());
            }

            eetc = TimesheetManagerUtil.calculateEETC(summaryTimesheet, scope,
                messageContext);
            percentageCompleted = TimesheetManagerUtil
                    .calculatePercentageComplete(summaryTimesheet, scope,
                        messageContext);
        }
        else if (taskAssignment instanceof AbstractTimesheetTaskAssignment)
        {
            AbstractTimesheetTaskAssignment atta = (AbstractTimesheetTaskAssignment) taskAssignment;
            startDate = atta.getStartDate();
            finishDate = atta.getFinishDate();
        }

        TimesheetManagerUtil.initWorkingHours(summaryTimesheet);
        double billableRegular = TimesheetManagerUtil
            .calculateBillableRegular(summaryTimesheet);
        double billableSpecial = TimesheetManagerUtil
            .calculateBillableSpecial(summaryTimesheet);

        String name = "SP_U_SRV_TS";
        Object args[] = new Object[31];
        args[0] = summaryTimesheet.getTaskAssignment().getID(); // p_tsk_assignment_id
        args[1] = DateUtil.convertToStringNoTime(summaryTimesheet.getWeekOf().getTime()); // p_week_of
        args[2] = DateUtil.convertToString(startDate); // p_actual_start
        args[3] = DateUtil.convertToString(finishDate); // p_actual_finish
        args[4] = forecastStartDate; // p_forecast_start
        args[5] = forecastFinishDate; // p_forecast_finish
        args[6] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularMonday());
        args[7] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialMonday());
        args[8] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularTuesday());
        args[9] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialTuesday());
        args[10] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularWednesday());
        args[11] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialWednesday());
        args[12] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularThursday());
        args[13] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialThursday());
        args[14] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularFriday());
        args[15] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialFriday());
        args[16] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularSaturday());
        args[17] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialSaturday());
        args[18] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getRegularSunday());
        args[19] = DateUtil.convertHoursToMinutes(summaryTimesheet
            .getSpecialSunday());
        args[20] = DateUtil.convertHoursToMinutes(new Double(billableRegular)); // p_billable_regular
        args[21] = DateUtil.convertHoursToMinutes(new Double(billableSpecial)); // p_billable_special
        args[22] = new Integer(workBaseline); // p_baseline_effort_week
        args[23] = DateUtil.convertHoursToMinutes(new Double(eetc)); // p_etc
        args[24] = TimesheetManagerUtil.toRoundedPerTenThousand(percentageCompleted); // p_milestone_percent
        args[25] = summaryTimesheet.getSummaryNotes(); // p_notes
        args[26] = timecode1Id;
        args[27] = timecode2Id;
        args[28] = null; // p_activity_id ...looks like this parameter is not
        // used anymore...
        args[29] = "P"; // p_pda_flag
        args[30] = getUser(messageContext).getID(); // p_rec_user

        return executeProcedure(messageContext, name, args);
    }

    /**
     * Update the fields of SummaryTimeshet
     * 
     * @param timesheet
     * @param scope
     * @param stepDeletion
     * @param messageContext
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    public void updateSummaryTimesheet(AbstractTimesheet timesheet,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {

        String taskAssignmentId = TimesheetManagerUtil
            .retrieveStep_TaskAssignmentId(timesheet.getID(), messageContext);

        AbstractTimesheet[] timesheets = loadAbstractTimesheets(
            taskAssignmentId, timesheet.getWeekOf(), messageContext);

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setID(taskAssignmentId);
        summaryTimesheet.setID(taskAssignmentId);
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(timesheet.getWeekOf());

        // reload the summaryTimesheet
        summaryTimesheet = TimesheetManagerUtil.loadSummaryTimesheet(
            summaryTimesheet, null, messageContext);

        double regularMonday = 0, regularTuesday = 0, regularWednesday = 0;
        double regularThursday = 0, regularFriday = 0, regularSaturday = 0;
        double regularSunday = 0, specialMonday = 0, specialTuesday = 0;
        double specialWednesday = 0, specialThursday = 0, specialFriday = 0;
        double specialSaturday = 0, specialSunday = 0;

        for (int i = 0; i < timesheets.length; i++)
        {
            AbstractTimesheet reloadedTs = timesheets[i];

            double factor = 1;
            // credits values are substracted from the summaryTimesheet values
            if (reloadedTs instanceof CreditTimesheet)
            {
                factor = -1;
            }

            regularMonday = regularMonday + factor
                    * reloadedTs.getRegularMonday().doubleValue();
            regularTuesday = regularTuesday + factor
                    * reloadedTs.getRegularTuesday().doubleValue();
            regularWednesday = regularWednesday + factor
                    * reloadedTs.getRegularWednesday().doubleValue();
            regularThursday = regularThursday + factor
                    * reloadedTs.getRegularThursday().doubleValue();
            regularFriday = regularFriday + factor
                    * reloadedTs.getRegularFriday().doubleValue();
            regularSaturday = regularSaturday + factor
                    * reloadedTs.getRegularSaturday().doubleValue();
            regularSunday = regularSunday + factor
                    * reloadedTs.getRegularSunday().doubleValue();
            specialMonday = specialMonday + factor
                    * reloadedTs.getSpecialMonday().doubleValue();
            specialTuesday = specialTuesday + factor
                    * reloadedTs.getSpecialTuesday().doubleValue();
            specialWednesday = specialWednesday + factor
                    * reloadedTs.getSpecialWednesday().doubleValue();
            specialThursday = specialThursday + factor
                    * reloadedTs.getSpecialThursday().doubleValue();
            specialFriday = specialFriday + factor
                    * reloadedTs.getSpecialFriday().doubleValue();
            specialSaturday = specialSaturday + factor
                    * reloadedTs.getSpecialSaturday().doubleValue();
            specialSunday = specialSunday + factor
                    * reloadedTs.getSpecialSunday().doubleValue();

        }

        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.REGULAR_MONDAY_FIELD, new Double(
                    regularMonday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.REGULAR_TUESDAY_FIELD, new Double(
                regularTuesday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.REGULAR_WEDNESDAY_FIELD, new Double(
                regularWednesday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.REGULAR_THURSDAY_FIELD, new Double(
                regularThursday));
        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.REGULAR_FRIDAY_FIELD, new Double(
                    regularFriday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.REGULAR_SATURDAY_FIELD, new Double(
                regularSaturday));
        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.REGULAR_SUNDAY_FIELD, new Double(
                    regularSunday));
        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.SPECIAL_MONDAY_FIELD, new Double(
                    specialMonday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.SPECIAL_TUESDAY_FIELD, new Double(
                specialTuesday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.SPECIAL_WEDNESDAY_FIELD, new Double(
                specialWednesday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.SPECIAL_THURSDAY_FIELD, new Double(
                specialThursday));
        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.SPECIAL_FRIDAY_FIELD, new Double(
                    specialFriday));
        RPMObjectUtil.deltaAndSet(summaryTimesheet,
            ValidationConstants.SPECIAL_SATURDAY_FIELD, new Double(
                specialSaturday));
        RPMObjectUtil
            .deltaAndSet(summaryTimesheet,
                ValidationConstants.SPECIAL_SUNDAY_FIELD, new Double(
                    specialSunday));
        
        // clear the EETC and %C in TaskAssignment in order to recalculate
        if (summaryTimesheet.getTaskAssignment() != null && summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {
            TaskAssignment ta = (TaskAssignment)summaryTimesheet.getTaskAssignment();
            ta.setEETCHours(null);
            ta.setPercentageComplete(null);
        }

        SP_U_SRV_TS(summaryTimesheet, null, true, messageContext);

    }
    
    private AbstractTimesheet[] loadAbstractTimesheets(String taskAssignmentId,
            Calendar weekOf, MessageContext messageContext) throws RPMException
    {

        String[] args = new String[]
        {
                taskAssignmentId, DateUtil.convertToString(weekOf)
        };

        ArrayList temp;
        try
        {
            RPMObjectManager manager = getManagerInstance(StepTimesheet.class);

            temp = manager.loadByForeignKey(null, null, messageContext, this,
                args, null, null, false);
        }
        catch (Exception e)
        {
            throw new RPMException(e);
        }

        return (AbstractTimesheet[]) ArrayUtil.listToArray(
            AbstractTimesheet.class, temp);
    }
}
