package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.types.ProjectSecurityRoleType;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.timesheet.util.RoleTypeUtility;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.timesheet.util.TimesheetStatusUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TimesheetApprovalStatusManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        TimesheetApprovalStatusManager.class);

    static
    {
        containerMap.add(TimesheetApprovalStatus.class);
    }

    // Enumeration for actions
    private static final int INVALID = 0;

    private static final int SUBMIT = 1;

    private static final int APPROVE = 2;

    private static final int REJECT = 3;

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 1;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "TIMESHEET_APPROVAL_STATUS.RESOURCE_ID";

    /*
     * public static final String PROPERTY_RESOURCE_ID = "RESOURCEID"; static {
     * FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID); }
     */

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 2;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TIMESHEET_APPROVAL_STATUS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column WEEK_OF of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_WEEK_OF = 3;

    public static final int TYPE_WEEK_OF = Types.DATE;

    public static final String NAME_WEEK_OF = "TIMESHEET_APPROVAL_STATUS.WEEK_OF";

    public static final String PROPERTY_WEEK_OF = "WEEKOF";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEEK_OF, NAME_WEEK_OF);
    }

    /**
     * Column CURRENT_STATE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CURRENT_STATE = 4;

    public static final int TYPE_CURRENT_STATE = Types.SMALLINT;

    public static final String NAME_CURRENT_STATE = "TIMESHEET_APPROVAL_STATUS.CURRENT_STATE";

    public static final String PROPERTY_CURRENT_STATE = "APPROVALSTATUS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CURRENT_STATE, SqlUtil.castAsChar(
            NAME_CURRENT_STATE, 2));
    }

    /**
     * Column MANAGER_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_MANAGER_ID = 5;

    public static final int TYPE_MANAGER_ID = Types.CHAR;

    public static final String NAME_MANAGER_ID = "TIMESHEET_APPROVAL_STATUS.MANAGER_ID";

    public static final String PROPERTY_MANAGER_ID = "LASTAPPROVEDREJECTEDBYID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MANAGER_ID, NAME_MANAGER_ID);
    }

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 6;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TIMESHEET_APPROVAL_STATUS.REC_DATETIME";

    public static final String PROPERTY_REC_DATETIME = "APPROVALDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME, NAME_REC_DATETIME);
    }

    /**
     * Column NOTE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NOTE = 7;

    public static final int TYPE_NOTE = Types.VARCHAR;

    public static final String NAME_NOTE = "TIMESHEET_APPROVAL_STATUS.NOTE";

    public static final String PROPERTY_NOTE = "APPROVALNOTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_NOTE, NAME_NOTE);
    }

    public static final String TABLE_NAME = "TIMESHEET_APPROVAL_STATUS";

    /**
     * Create an array of type string containing all the fields of the
     * TIMESHEET_APPROVAL_STATUS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RESOURCE_ID, NAME_ELEMENT_ID, NAME_WEEK_OF,
            NAME_CURRENT_STATE, NAME_MANAGER_ID, NAME_REC_DATETIME, NAME_NOTE
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new TimesheetApprovalStatusManager instance.
     * 
     * @return TimesheetApprovalStatusManager
     */
    public TimesheetApprovalStatusManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return null;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        return new String[]
        {
                NAME_RESOURCE_ID, NAME_ELEMENT_ID, NAME_WEEK_OF
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        TimesheetApprovalStatus appStatus = (TimesheetApprovalStatus) rpmObject;
        String resourceId = null;
        if (appStatus.getResource() != null)
        {
            resourceId = appStatus.getResource().getID();
        }
        
        // ID can be either a ProjectId or a ResourceId. Project cannot be used
        // as a part of the PrimaryKey because it is null when the
        // SummaryTimesheet refers to a PersonalTask or a TimesheetTask.
        String elementID = appStatus.getID();

        String weekOf = DateUtil.convertToString(appStatus.getWeekOf());

        return new Object[]
        {
                resourceId, elementID, weekOf
        };
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean isValid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        TimesheetApprovalStatus timesheetApprovalStatus = (TimesheetApprovalStatus) rpmObject;

        if (timesheetApprovalStatus.getResource() == null
                && timesheetApprovalStatus.getResource().getID() == null)
        {
            invalidPrimaryKeys.append(ValidationConstants.RESOURCE_FIELD);
            isValid = false;
        }
        
        if (timesheetApprovalStatus.getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
            {
                invalidPrimaryKeys.append(" and ");
            }
            invalidPrimaryKeys.append("ID");
            isValid = false;
        }
        
        if (timesheetApprovalStatus.getWeekOf() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
            {
                invalidPrimaryKeys.append(" and ");
            }
            invalidPrimaryKeys.append(ValidationConstants.WEEK_OF_FIELD);
            isValid = false;
        }
        
        if (!isValid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                invalidPrimaryKeys.toString());
        }
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject) throws RPMException
    {
        TimesheetApprovalStatus appStatus = (TimesheetApprovalStatus) rpmObject;
        // Set the ID to the element_id
        appStatus.setID((String) primaryKey[1]);

        // Set the resource
        Resource resource = new Resource();
        resource.setID((String) primaryKey[0]);
        appStatus.setResource(resource);

        // Set week-of
        Calendar weekOf = null;
        try
        {
            weekOf = DateUtil
                .convertToCalendarNoTime((String) primaryKey[2]);
        }
        catch (ParseException e)
        {
            ExceptionUtil.handleException(Manager.class, e);
        }
        appStatus.setWeekOf(weekOf);

        return appStatus;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof SummaryTimesheetManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.append(NAME_WEEK_OF);
            joinCondition.appendEqual();
            joinCondition.append(SummaryTimesheetManager.NAME_WEEK_OF);
            joinCondition.appendAnd();
            joinCondition.append(NAME_RESOURCE_ID);
            joinCondition.appendEqual();
            joinCondition.append(SummaryTimesheetManager.NAME_RESOURCE_ID);
            joinCondition.appendAnd();
            joinCondition.append(NAME_ELEMENT_ID);
            joinCondition.appendEqual();
            joinCondition.append(SummaryTimesheetManager.NAME_PROJECT_ID);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new TimesheetApprovalStatus();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        TimesheetApprovalStatus appStatus = (TimesheetApprovalStatus) rpmObject;

        Integer inputTypeIndex = new Integer(rs.getInt(ID_CURRENT_STATE));
        TimesheetStatus status = TimesheetStatusUtil
            .getTimesheetStatusByIndex(inputTypeIndex);

        if (isDelta)
        {
            appStatus.deltaApprovalDate(Manager
                .getCalendar(rs, ID_REC_DATETIME));
            appStatus.deltaApprovalNotes(rs.getString(ID_NOTE));
            appStatus.deltaApprovalStatus(status);
        }
        else
        {
            appStatus.setApprovalDate(Manager.getCalendar(rs, ID_REC_DATETIME));
            appStatus.setApprovalNotes(rs.getString(ID_NOTE));
            appStatus.setApprovalStatus(status);
        }

        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String managerId = resultSet.getString(ID_MANAGER_ID);
        relatedFields.put(index, ID_MANAGER_ID, managerId);

        String resourceId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, resourceId);

        String elementId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, elementId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        TimesheetApprovalStatus appStatus = (TimesheetApprovalStatus) rpmObject;
        TimesheetApprovalStatusScope scope = (TimesheetApprovalStatusScope) rpmScope;

        if (scope != null)
        {
            if (scope.getResource() != null)
            {
                loadResource(appStatus, scope, messageContext, relatedFields,
                    index, isDelta);
            }
            if (scope.getProject() != null)
            {
                if (TimesheetManagerUtil.isBasedOnWbsProject(appStatus,
                    messageContext))
                {

                    loadProject(appStatus, scope, messageContext,
                        relatedFields, index, isDelta);
                }
            }
            if (scope.getLastApprovedRejectedBy() != null)
            {
                loadLastAppovedRejectedBy(appStatus, scope, messageContext,
                    relatedFields, index, isDelta);
            }
        }

        return rpmObject;
    }

    private void loadLastAppovedRejectedBy(TimesheetApprovalStatus appStatus,
            TimesheetApprovalStatusScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldResourceId = (String) relatedFields.get(index, ID_MANAGER_ID);
        RPMObject newResource = null;

        ResourceManager manager = (ResourceManager) getManagerInstance(Resource.class
            .getName());

        if (isDelta)
        {
            newResource = appStatus.getLastApprovedRejectedBy();
            RPMObject oldResource = null;
            if (oldResourceId != null)
            {
                oldResource = new Resource();
                oldResource.setID(oldResourceId);
                if (newResource != null && newResource.getID() != null
                        && newResource.getID().equals(oldResourceId))
                {
                    oldResource = manager.loadByPrimaryKey(newResource, scope
                        .getLastApprovedRejectedBy(), messageContext, true);
                }
                else
                {
                    // In this case, old resource is loaded without the
                    // scope to avoid problem with the cache
                    oldResource = manager.loadByPrimaryKey(oldResource, scope
                        .getLastApprovedRejectedBy(), messageContext, false);
                }
            }
            appStatus.deltaLastApprovedRejectedBy((Resource) oldResource);
        }
        else
        {
            if (oldResourceId != null)
            {
                if (newResource == null)
                {// newParent not in cache
                    newResource = new Resource();
                    newResource.setID(oldResourceId);
                    newResource = manager.loadByPrimaryKey(newResource, scope
                        .getLastApprovedRejectedBy(), messageContext, false);
                }
                appStatus.setLastApprovedRejectedBy((Resource) newResource);
            }
        }
    }

    private void loadResource(TimesheetApprovalStatus appStatus,
            TimesheetApprovalStatusScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldResourceId = (String) relatedFields
            .get(index, ID_RESOURCE_ID);
        RPMObject newResource = null;

        ResourceManager manager = (ResourceManager) getManagerInstance(Resource.class
            .getName());

        if (isDelta)
        {
            newResource = appStatus.getResource();
            RPMObject oldResource = null;
            if (oldResourceId != null)
            {
                oldResource = new Resource();
                oldResource.setID(oldResourceId);
                if (newResource != null && newResource.getID() != null
                        && newResource.getID().equals(oldResourceId))
                {
                    oldResource = manager.loadByPrimaryKey(newResource, scope
                        .getResource(), messageContext, true);
                }
                else
                {
                    // In this case, old resource is loaded without the
                    // scope to avoid problem with the cache
                    oldResource = manager.loadByPrimaryKey(oldResource, scope
                        .getResource(), messageContext, false);
                }
            }
            appStatus.deltaResource((Resource) oldResource);
        }
        else
        {
            if (oldResourceId != null)
            {
                if (newResource == null)
                {// newParent not in cache
                    newResource = new Resource();
                    newResource.setID(oldResourceId);
                    newResource = manager.loadByPrimaryKey(newResource, scope
                        .getResource(), messageContext, false);
                }
                appStatus.setResource((Resource) newResource);
            }
        }
    }

    private void loadProject(TimesheetApprovalStatus appStatus,
            TimesheetApprovalStatusScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // element_id can be either a project or a resource. Load only if it's
        // a project
        String projectId = (String) relatedFields.get(index, ID_ELEMENT_ID);
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (projectId.equals(resourceId))
            return;

        GenericProject project = null;

        if (projectId != null)
        {
            project = new GenericProject();
            project.setID(projectId);

            if (isDelta)
            {
                // if isDelta = true, we only need to cmpare the Ids
                appStatus.deltaProject(project);
            }
            else
            {
                RPMObjectManager manager = getManagerInstance(GenericProject.class);

                project = (GenericProject) manager.loadByPrimaryKey(project,
                    null, messageContext, isDelta);

                appStatus.setProject(project);
            }
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof SummaryTimesheetManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(NAME_RESOURCE_ID);
            where.append(" = ? AND ");
            where.append(NAME_ELEMENT_ID);
            where.append(" = ? AND ");
            where.append(NAME_WEEK_OF);
            where.append(" = ?");

            joinCondition.setCondition(where.toString());
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        TimesheetApprovalStatus timesheetApprovalStatus = (TimesheetApprovalStatus) rpmObject;

        // Retrieve the start of that week
        if (timesheetApprovalStatus.getWeekOf() != null)
        {
            String weekOf = SP_S_WEEKOF(timesheetApprovalStatus, messageContext);
            timesheetApprovalStatus.setWeekOf(DateUtil
                .convertToCalendarNoTime(weekOf));
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        TimesheetApprovalStatus appStatus = (TimesheetApprovalStatus) rpmObject;
        TimesheetApprovalStatusScope scope = (TimesheetApprovalStatusScope) rpmScope;

        if (appStatus.getID() != null && scope != null
                && scope.getResource() == null)
        {
            reloadApprovalStatusWithFullScope(appStatus, messageContext);
        }

        if (appStatus.testApprovalStatusModified())
        {
            int action = getAction(appStatus.getApprovalStatus());

            if (action != INVALID)
            {

                // This SP inserts the status in the DB.
                SP_U_TS_APPROVAL(appStatus, action, messageContext);

                if (appStatus.getID() == null)
                {
                    // note: parent with id is mandatory when id is null
                    String newId = TimesheetManagerUtil
                        .retrieveTaskAssignment_ProjectId(appStatus.getParent()
                            .getID(), messageContext);
                    appStatus.setID(newId);
                }

            }

        }

        if (appStatus.testApprovalNotesModified())
            SP_U_TS_NOTE(appStatus, messageContext);

        return appStatus;
    }

    /**
     * Load the TimesheetApprovalStatus including it's resource and project.
     * 
     * @param appStatus
     * @param messageContext
     * @return
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    private TimesheetApprovalStatus reloadApprovalStatusWithFullScope(
            TimesheetApprovalStatus appStatus, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        TimesheetApprovalStatusScope fullScope = new TimesheetApprovalStatusScope();
        fullScope.setResource(new ResourceScope());
        fullScope.setProject(new WorkElementScope());

        return (TimesheetApprovalStatus) loadByPrimaryKey(appStatus, fullScope,
            messageContext, false);
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    private String SP_U_TS_APPROVAL(TimesheetApprovalStatus appStatus,
            int action, MessageContext messageContext) throws SQLException,
            RPMException
    {
        Resource resource = (Resource) getUser(messageContext);
        // TODO activate this once ExecutiveSecurityRole is implemented in
        // ResourceManager

        ProjectSecurityRoleType type = null;
        if (action == SUBMIT)
            type = RoleTypeUtility.getRoleTypeByIndex(new Integer(1));
        else
            type = RoleTypeUtility.getRoleTypeByIndex(new Integer(2));

        String elementID = (appStatus.getID() == null || action == SUBMIT)
                ? null : appStatus.getID();

        final String procedureName = "SP_U_TS_APPROVAL";
        Object args[] = new Object[6];
        args[0] = appStatus.getResource().getID(); // resource_id
        args[1] = DateUtil.convertToString(appStatus.getWeekOf()); // week of
        args[2] = elementID; // element_id
        args[3] = RoleTypeUtility.getIndexByRoleType(type); // role_type
        args[4] = new Long(action); // action_type
        args[5] = resource.getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    private String SP_U_TS_NOTE(TimesheetApprovalStatus appStatus,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_TS_NOTE";
        Object args[] = new Object[5];

        args[0] = appStatus.getResource().getID(); // resource_id
        args[1] = DateUtil.convertToString(appStatus.getWeekOf()); // week_of
        args[2] = (appStatus.getID() == null) ? null : appStatus.getID(); // element_id
        args[3] = appStatus.getApprovalNotes(); // note
        args[4] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    private String SP_S_WEEKOF(TimesheetApprovalStatus timesheetApprovalStatus,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_S_WEEKOF";
        Object args[] = new Object[1];
        args[0] = DateUtil.convertToString(timesheetApprovalStatus.getWeekOf()); // p_weekof

        return executeProcedure(messageContext, name, args);
    }

    private int getAction(TimesheetStatus timesheetStatus) throws RPMException
    {
        if (timesheetStatus == TimesheetStatus.Submitted)
            return SUBMIT;

        // TODO Once PersonalTask is implemented, the system should figure out
        // whether it is pm_approved or rm_approved
        if (timesheetStatus == TimesheetStatus.Approved
                || timesheetStatus == TimesheetStatus.ApprovedByProjectManager
                || timesheetStatus == TimesheetStatus.ApprovedByResourceManager)
            return APPROVE;

        // TODO Once PersonalTask is implemented, the system should figure out
        // if it's rm-rejected or pm-rejected
        if (timesheetStatus == TimesheetStatus.Rejected
                || timesheetStatus == TimesheetStatus.RejectedByProjectManager
                || timesheetStatus == TimesheetStatus.RejectedByResourceManager)
            return REJECT;

        // invalid value
        String[] errorParams =
        {
                StringUtil.getShortClassName(getClass().getName()),
                "of field approvalStatus ",
                TimesheetStatus._Approved + ", " + TimesheetStatus._Rejected
                        + " or " + TimesheetStatus._Submitted

        };
        throw new RPMException(ErrorCodes.INVALID_VALUE, errorParams,
            getClass().getName(), NAME_CURRENT_STATE);
    }

}
