/**
 * Created on Feb 2, 2006
 * @author shahla
 */
package com.ibm.rpm.timesheet.util;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;

public class TimesheetManagerUtil
{
    // Constants for the table TASK_DAY_OF_YEAR, since there is currently no
    // plan to
    // implement a manager for this table

    public static final String TASK_DAY_OF_YEAR_TABLE_NAME = "TASK_DAY_OF_YEAR";

    public static final String TASK_DAY_OF_YEAR_NAME_TSK_ASSIGNMENT_ID = "TASK_DAY_OF_YEAR.TSK_ASSIGNMENT_ID";

    public static final String TASK_DAY_OF_YEAR_NAME_BIT_FLAG = "TASK_DAY_OF_YEAR.BIT_FLAG";

    public static final String TASK_DAY_OF_YEAR_NAME_DAY_YEAR = "TASK_DAY_OF_YEAR.DAY_YEAR";

    public static final String TASK_DAY_OF_YEAR_NAME_SCHED_MIN = "TASK_DAY_OF_YEAR.SCHED_MIN";

    public static AbstractTimesheet initWorkingHours(AbstractTimesheet timesheet)
    {
        Double zero = new Double(0);

        if (timesheet == null)
            return null;

        if (timesheet.getRegularFriday() == null)
            timesheet.setRegularFriday(zero);
        if (timesheet.getRegularMonday() == null)
            timesheet.setRegularMonday(zero);
        if (timesheet.getRegularSaturday() == null)
            timesheet.setRegularSaturday(zero);
        if (timesheet.getRegularSunday() == null)
            timesheet.setRegularSunday(zero);
        if (timesheet.getRegularThursday() == null)
            timesheet.setRegularThursday(zero);
        if (timesheet.getRegularTuesday() == null)
            timesheet.setRegularTuesday(zero);
        if (timesheet.getRegularWednesday() == null)
            timesheet.setRegularWednesday(zero);
        if (timesheet.getSpecialFriday() == null)
            timesheet.setSpecialFriday(zero);
        if (timesheet.getSpecialMonday() == null)
            timesheet.setSpecialMonday(zero);
        if (timesheet.getSpecialSaturday() == null)
            timesheet.setSpecialSaturday(zero);
        if (timesheet.getSpecialSunday() == null)
            timesheet.setSpecialSunday(zero);
        if (timesheet.getSpecialThursday() == null)
            timesheet.setSpecialThursday(zero);
        if (timesheet.getSpecialTuesday() == null)
            timesheet.setSpecialTuesday(zero);
        if (timesheet.getSpecialWednesday() == null)
            timesheet.setSpecialWednesday(zero);

        return timesheet;
    }

    public static double calculateBillableRegular(AbstractTimesheet timesheet)
    {
    	double calculateBillableRegular = 0;
    	if(timesheet.getRegularMonday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularMonday().doubleValue();
    	}
    	if(timesheet.getRegularTuesday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularTuesday().doubleValue();
    	}
    	if(timesheet.getRegularWednesday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularWednesday().doubleValue();
    	}
    	if(timesheet.getRegularThursday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularThursday().doubleValue();
    	}
    	if(timesheet.getRegularFriday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularFriday().doubleValue();
    	}
    	if(timesheet.getRegularSaturday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularSaturday().doubleValue();
    	}
    	if(timesheet.getRegularSunday() != null)
    	{
    		calculateBillableRegular += timesheet.getRegularSunday().doubleValue();
    	}
        return calculateBillableRegular;
    }

    public static double calculateBillableSpecial(AbstractTimesheet timesheet)
    {
    	double calculateBillableSpecial = 0;
    	if(timesheet.getSpecialMonday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialMonday().doubleValue();
    	}
    	if(timesheet.getSpecialTuesday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialTuesday().doubleValue();
    	}
    	if(timesheet.getSpecialWednesday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialWednesday().doubleValue();
    	}
    	if(timesheet.getSpecialThursday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialThursday().doubleValue();
    	}
    	if(timesheet.getSpecialFriday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialFriday().doubleValue();
    	}
    	if(timesheet.getSpecialSaturday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialSaturday().doubleValue();
    	}
    	if(timesheet.getSpecialSunday() != null)
    	{
    		calculateBillableSpecial += timesheet.getSpecialSunday().doubleValue();
    	}
        return calculateBillableSpecial;
    }

    public static double calculateTotalBillable(AbstractTimesheet timesheet)
    {
        return calculateBillableRegular(timesheet)
                + calculateBillableSpecial(timesheet);
    }

    public static String retrieveTaskAssignment_ProjectId(
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {

        return (String) retrieveTaskAssignmentField(taskAssignmentId,
            TaskAssignmentManager.NAME_PROJECT_ID, messageContext);

    }

    public static String retrieveTaskAssignment_ResourceId(
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {

        return (String) retrieveTaskAssignmentField(taskAssignmentId,
            TaskAssignmentManager.NAME_RESOURCE_ID, messageContext);
    }

    public static String retrieveTaskAssignment_TaskId(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveTaskAssignmentField(taskAssignmentId,
            TaskAssignmentManager.NAME_TASK_ID, messageContext);
    }

    public static int retrieveTaskAssignment_Billable(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {

        int billable = 0;

        Number number = (Number) retrieveTaskAssignmentField(taskAssignmentId,
            TaskAssignmentManager.NAME_BILLABLE, messageContext);

        if (number != null)
        {
            billable = number.intValue();
        }

        return billable;
    }
    
    public static long retrieveTaskAssignment_EETC(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {
        long eetc = 0;

        Number number = (Number) retrieveTaskAssignmentField(taskAssignmentId,
            TaskAssignmentManager.NAME_ETC, messageContext);
        if (number != null)
        {
            eetc = number.longValue();
        }

        return eetc;
    }

    private static Object retrieveTaskAssignmentField(String taskAssignmentId,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {

        Object value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(fieldName);
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            taskAssignmentId
        }, messageContext);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = columns.get(0);
        }

        return value;
    }

    public static SummaryTimesheet loadSummaryTimesheet(
            SummaryTimesheet summaryTimesheet, RPMObjectScope rpmScope,
            MessageContext msgCtx) throws RPMException
    {
        SummaryTimesheet loadedTimesheet = null;

        if (summaryTimesheet.getID() != null)
        {
            try
            {
                // remove from cache
                msgCtx.getCache().removeContainerFromCache(summaryTimesheet);

                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(SummaryTimesheet.class);
                loadedTimesheet = (SummaryTimesheet) manager.loadByPrimaryKey(
                    summaryTimesheet, rpmScope, msgCtx, false);
            }
            catch (Exception e)
            {
                throw new RPMException(e);
            }
        }
        if (loadedTimesheet == null)
        {
            loadedTimesheet = new SummaryTimesheet();
        }

        return loadedTimesheet;
    }

    public static String retrieveSummaryTimesheetTimecode1(
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
            throws RPMException
    {

        return (String) retrieveSummaryTimesheetField(summaryTimesheet,
            SummaryTimesheetManager.NAME_TIMECODE_1_ID, messageContext);
    }

    private static Object retrieveStepField(String stepId, String fieldName,
            MessageContext messageContext) throws RPMException
    {

        Object value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(fieldName);
        stmt.appendFrom(StepManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(StepManager.NAME_ELEMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            stepId
        }, messageContext);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = columns.get(0);
        }

        return value;
    }

    public static String retrieveSummaryTimesheetTimecode2(
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
            throws RPMException
    {

        return (String) retrieveSummaryTimesheetField(summaryTimesheet,
            SummaryTimesheetManager.NAME_TIMECODE_2_ID, messageContext);
    }

    private static Object retrieveSummaryTimesheetField(
            SummaryTimesheet summaryTimesheet, String fieldname,
            MessageContext messageContext) throws RPMException
    {

        if (summaryTimesheet.getID() == null)
        {
            return null;
        }

        Object value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(fieldname);
        stmt.appendFrom(SummaryTimesheetManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqual(SummaryTimesheetManager.NAME_WEEK_OF, DateUtil
            .convertToString(summaryTimesheet.getWeekOf()));
        stmt
            .appendEqualQuestionMark(SummaryTimesheetManager.NAME_TSK_ASSIGNMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            summaryTimesheet.getID()
        }, messageContext);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = columns.get(0);
        }

        return value;
    }

    public static String retrieveStep_TaskAssignmentId(String stepId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveStepField(stepId,
            StepManager.NAME_TSK_ASSIGNMENT_ID, messageContext);
    }

    public static String retrieveStep_Timecode1Id(String stepId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveStepField(stepId,
            StepManager.NAME_TIMECODE_1_ID, messageContext);
    }

    public static String retrieveStep_Timecode1Name(String stepId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveStepField(stepId,
            StepManager.NAME_TIMECODE_1_NAME, messageContext);
    }

    public static String retrieveStep_Timecode2Id(String stepId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveStepField(stepId,
            StepManager.NAME_TIMECODE_2_ID, messageContext);
    }

    public static int retrieveStep_TypeId(String stepId,
            MessageContext messageContext) throws RPMException
    {

        int typeId = 0;

        Number number = (Number) retrieveStepField(stepId,
            StepManager.NAME_TYPE_ID, messageContext);

        if (number != null)
        {
            typeId = number.intValue();
        }

        return typeId;

    }

    public static long retrieveStep_ActualWork(String stepId,
            MessageContext messageContext) throws RPMException
    {

        long actualWork = 0;

        Number number = (Number) retrieveStepField(stepId,
            StepManager.NAME_WORK_TODATE, messageContext);

        if (number != null)
        {
            actualWork = number.longValue();
        }

        return actualWork;
    }
    
    public static long retrieveStep_eetc(String stepId, MessageContext messageContext) throws RPMException
    {
        long eetc = 0;
        
        Number number = (Number) retrieveStepField(stepId, StepManager.NAME_ETC, messageContext);
        if (number != null)
        {
            eetc = number.longValue();
        }

        return eetc;
    }
    
    public static boolean retrieveStep_eetcChanged(String stepId, MessageContext messageContext) throws RPMException
    {
        boolean eetcModified = false;
        
        String value = (String) retrieveStepField(stepId, StepManager.NAME_ETC_MODIFIED, messageContext);
        if (value.equalsIgnoreCase("Y"))
        {
            eetcModified = true;
        }
        
        return eetcModified;
    }
    
    public static int retrieveStep_weight(String stepId, MessageContext messageContext) throws RPMException
    {
        int weight = 0;
        
        Number number = (Number)retrieveStepField(stepId, StepManager.NAME_EDPERCENT, messageContext);
        if (number != null)
        {
            weight = number.intValue();
        }
        return weight;
    }

    public static String retrieveTimecode_Name(String timecodeId,
            MessageContext messageContext) throws RPMException
    {

        return (String) retrieveTimecodeField(timecodeId,
            TimecodeManager.NAME_ELEMENT_NAME, messageContext);
    }

    private static Object retrieveTimecodeField(String timecodeId,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {

        Object value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(fieldName);
        stmt.appendFrom(TimecodeManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(TimecodeManager.NAME_ELEMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            timecodeId
        }, messageContext);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = columns.get(0);
        }

        return value;
    }

    public static Double retrieveSummaryTimesheet_EffortPerWeek(
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
            throws RPMException, SQLException
    {
        // if (summaryTimesheet.getID() == null || summaryTimesheet.getWeekOf()
        // == null)
        // return null;

        Double expectedWorkPerWeek = null;

        SqlBuffer condition = new SqlBuffer();
        condition.append(TASK_DAY_OF_YEAR_NAME_TSK_ASSIGNMENT_ID + "='"
                + summaryTimesheet.getID());
        condition.append("' AND " + TASK_DAY_OF_YEAR_NAME_BIT_FLAG + "= '2'");
        condition.append(" AND (" + TASK_DAY_OF_YEAR_NAME_DAY_YEAR
                + " BETWEEN ");

        if (Manager.isOracle(messageContext))
        {
            condition.append("TO_DATE('"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "') AND ");
            condition.append("TO_DATE('"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "') + 6)");
        }
        else
        {
            condition.append("DATE('"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "') AND ");
            condition.append("DATE('"
                    + DateUtil.convertToString(summaryTimesheet.getWeekOf())
                    + "') + 6 DAYS)");
        }

        Integer expectedWorkPerWeekInMinutes = (Integer) ManagerUtil
            .selectColumnValue(Integer.class, summaryTimesheet,
                "expectedWorkPerWeek", "SUM(" + TASK_DAY_OF_YEAR_NAME_SCHED_MIN
                        + ")", TASK_DAY_OF_YEAR_TABLE_NAME, condition,
                messageContext);

        if (expectedWorkPerWeekInMinutes != null)
        {
            expectedWorkPerWeek = DateUtil
                .convertMinutesToHours(expectedWorkPerWeekInMinutes.longValue());
        }

        return expectedWorkPerWeek;
    }

    public static GenericTaskAssignment reloadAbstractTaskAssignment(
            GenericTaskAssignment taskAssignment, MessageContext messageContext)
    {

        GenericTaskAssignment reloadedTa = null;

        try
        {
            // remove from cache
            messageContext.getCache().removeContainerFromCache(taskAssignment);

            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(taskAssignment.getClass());
            reloadedTa = (GenericTaskAssignment) manager.loadByPrimaryKey(
                taskAssignment, null, messageContext, false);

        }
        catch (Exception e)
        {
            messageContext.addExceptionNoThrow(e);
        }

        return reloadedTa;
    }

    public static Calendar SP_S_WEEKOF(Calendar weekOf,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        String name = "SP_S_WEEKOF";
        Object args[] = new Object[1];
        args[0] = DateUtil.convertToStringNoTime(weekOf.getTime()); // p_weekof

        String week = ManagerUtil.executeProcedure(TimesheetManagerUtil.class,
            messageContext, name, args);

        return DateUtil.convertToCalendarNoTime(week);
    }

    /**
     * replace weekof with a date that matches the beginning of the week
     * 
     * @param timesheet
     * @param messageContext
     */
    public static void setValidWeekOf(AbstractTimesheet timesheet,
            MessageContext messageContext)
    {
        try
        {
            Calendar validWeekOf = TimesheetManagerUtil.SP_S_WEEKOF(timesheet
                .getWeekOf(), messageContext);
            timesheet.setWeekOf(validWeekOf);

            if (timesheet instanceof CreditTimesheet)
            {
                CreditTimesheet ct = (CreditTimesheet) timesheet;
                Calendar validCreditWeekOf = TimesheetManagerUtil.SP_S_WEEKOF(
                    ct.getCreditWeekOf(), messageContext);
                ct.setCreditWeekOf(validCreditWeekOf);
            }
        }
        catch (Exception e)
        {
            messageContext.addExceptionNoThrow(e);
        }

    }

    /**
     * replace weekof with a date that matches the beginning of the week
     * 
     * @param timesheet
     * @param messageContext
     */
    public static void setValidWeekOf(TimesheetApprovalStatus approvalStatus,
            MessageContext messageContext)
    {
        try
        {
            Calendar validWeekOf = TimesheetManagerUtil.SP_S_WEEKOF(
                approvalStatus.getWeekOf(), messageContext);
            approvalStatus.setWeekOf(validWeekOf);
        }
        catch (Exception e)
        {
            messageContext.addExceptionNoThrow(e);
        }

    }

    public static void SP_U_RES_STATUS(String taskAssignmentId, String status,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String resourceId = null;
        resourceId = TimesheetManagerUtil.retrieveTaskAssignment_ResourceId(
            taskAssignmentId, messageContext);

        if (resourceId == null)
        {
            // Load the resourceId from the session table
            ArrayList checkOutRPMObjects = messageContext.getFactory()
                .getSessionCheckoutDAO().retrieveCK_IDFromPartialCK_ID(
                    messageContext.getSession(),
                    messageContext.getUser().getID(), messageContext);

            if (checkOutRPMObjects != null && !checkOutRPMObjects.isEmpty())
            {
                // Get the resource from the first checked out object
                Object[] checkInCheckOutPK = (Object[]) checkOutRPMObjects
                    .get(0);
                if (checkInCheckOutPK != null)
                {
                    resourceId = (String) checkInCheckOutPK[0];
                }
            }
        }

        final String procedureName = "SP_U_RES_STATUS";
        Object args[] = new Object[4];

        args[0] = resourceId; // resource_id
        args[1] = null; // project_id - the Delphi UI does not seem to use the
        // project id
        // when a project id is used, instead of checking out/in everything for
        // a specific resource's timesheet for the resource that is logged in,
        // it checks in/out all timesheet objects related to a particular
        // project
        // or any children tasks of that project
        args[2] = status; // rec_status
        args[3] = messageContext.getUser().getID();

        ManagerUtil.executeProcedure(TimesheetManagerUtil.class,
            messageContext, procedureName, args);
    }

    public static void checkinObject(RPMObjectManager manager,
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_U_RES_STATUS(taskAssignmentId, "U", messageContext);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(manager, e);
        }
    }

    public static void checkoutObject(RPMObjectManager manager,
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_U_RES_STATUS(taskAssignmentId, "E", messageContext);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(manager, e);
        }
    }

    public static void undoCheckOut(RPMObjectManager manager,
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            TaskAssignment taskAssignment = new TaskAssignment();
            taskAssignment.setID(taskAssignmentId);
            if (isCheckedOut(messageContext, taskAssignment))
            {
                SP_U_RES_STATUS(taskAssignmentId, "U", messageContext);
            }
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(manager, e);
        }
    }

    public static boolean isCheckedOut(MessageContext context,
            GenericTaskAssignment taskAssignment) throws RPMException,
            SQLException
    {

        String status = ManagerUtil.retrieveRecStatus(context, taskAssignment);

        if (status != null)
        {
            return status.equalsIgnoreCase("E");
        }
        return false;
    }

    public static String getCheckInCheckOutPrimaryKey(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {
        String resourceID = TimesheetManagerUtil
            .retrieveTaskAssignment_ResourceId(taskAssignmentId, messageContext);

        StringBuffer primaryKeyString = new StringBuffer();
        primaryKeyString.append(resourceID);
        primaryKeyString.append(AbstractRPMObjectManager.STRING_PK_SEPARATOR);
        primaryKeyString.append(messageContext.getUser().getID());
        primaryKeyString.append(AbstractRPMObjectManager.STRING_PK_SEPARATOR);
        return primaryKeyString.toString();
    }

    public static RPMObject createAbstractTaskAssignment(
            SummaryTimesheet summaryTimesheet, MessageContext messageContext)
            throws RPMException
    {

        RPMObject object = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_TYPE_ID);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendSubSelect(WbsManager.NAME_ELEMENT_ID,
            TaskAssignmentManager.NAME_TASK_ID,
            TaskAssignmentManager.TABLE_NAME,
            TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID + " = ? ");
        stmt.appendCloseParenthesis();

        List result = ManagerUtil.select(new Class[]
        {
            Integer.class
        }, null, null, stmt, new Object[]
        {
            summaryTimesheet.getID()
        }, messageContext);

        // result should not be null or empty at this point
        Integer typeId = null;
        if (result != null || result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            typeId = (Integer) (columns.get(0));
        }

        if (typeId != null)
        {
            switch (typeId.intValue())
            {
            case PersonalTask.TYPE_ID:
                object = new PersonalTaskAssignment();
                break;
            case DefaultAdministrativeTask.TYPE_ID:
                object = new AdministrativeTaskAssignment();
                break;
            default:
                object = new TaskAssignment();
                break;
            }
        }

        return object;
    }

    public static GenericTaskAssignment createConcreteTaskAssignment(
            String assignmentId, MessageContext messageContext)
            throws RPMException
    {

        GenericTaskAssignment object = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_TYPE_ID);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendSubSelect(WbsManager.NAME_ELEMENT_ID,
            TaskAssignmentManager.NAME_TASK_ID,
            TaskAssignmentManager.TABLE_NAME,
            TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID + " = ? ");
        stmt.appendCloseParenthesis();

        List result = ManagerUtil.select(new Class[]
        {
            Integer.class
        }, null, null, stmt, new Object[]
        {
            assignmentId
        }, messageContext);

        // result should not be null or empty at this point
        Integer typeId = null;
        if (result != null || result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            typeId = (Integer) (columns.get(0));
        }

        if (typeId != null)
        {
            switch (typeId.intValue())
            {
            case PersonalTask.TYPE_ID:
                object = new PersonalTaskAssignment();
                break;
            case DefaultAdministrativeTask.TYPE_ID:
                object = new AdministrativeTaskAssignment();
                break;
            default:
                object = new TaskAssignment();
                break;
            }

            object.setID(assignmentId);
        }

        return object;
    }

    public static boolean isBasedOnWbsProject(
            TimesheetApprovalStatus approvalStatus,
            MessageContext messageContext) throws RPMException
    {

        boolean isWbs = false;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_ELEMENT_ID);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);
        stmt.appendIn(WbsManager.NAME_TYPE_ID, new int[]
        {
                Project.TYPE_ID, Proposal.TYPE_ID, WorkOrganization.TYPE_ID
        });

        List result = ManagerUtil.select(new Class[]
        {
            String.class
        }, null, null, stmt, new Object[]
        {
            approvalStatus.getID()
        }, messageContext);

        if (result != null && !result.isEmpty())
        {
            isWbs = true;
        }

        return isWbs;
    }

    private static double deltaSummaryTimesheetActualWork(
            SummaryTimesheet summaryTimesheet, RPMObjectScope rpmScope,
            MessageContext msgCtx) throws RPMException
    {
        SummaryTimesheet loadedSummaryTimesheet = null;
        
        double currentSummaryTimesheetActual = 0;
        if (summaryTimesheet != null)
        {
            try
            {
                loadedSummaryTimesheet = loadSummaryTimesheet(
                    summaryTimesheet, rpmScope, msgCtx);
            }
            catch (RPMException e)
            {
                if (!(e.getCause() instanceof RPMException)
                        || ((RPMException) e.getCause()).getErrorID() != ErrorCodes.AM_INVALID_RPMOBJECTID)
                {
                    throw e;
                }
            }
            currentSummaryTimesheetActual = calculateTotalBillable(summaryTimesheet);
        }
                
        double loadedSummaryTimesheetActual = 0;
        if (loadedSummaryTimesheet != null)
        {
            loadedSummaryTimesheetActual = calculateTotalBillable(loadedSummaryTimesheet);
        }
        
        return currentSummaryTimesheetActual - loadedSummaryTimesheetActual;
    }

    
    
    public static double calculateEETC(SummaryTimesheet summaryTimesheet,
            RPMObjectScope rpmScope, MessageContext msgCtx) throws RPMException
    {
        // Calculates the eetc if the eetc in TaskAssignment is null or there is
        // no changes; returns the eetc in TaskAssingment otherwise.
        // Two cases for calculating EETC.
        // Case 1: There is no changes in Actuals.
        // eetc = TaskAssignment.getExpectedWork() -
        // TaskAssignment.getActualWork()
        // Case 2: Actuals has been modified in SummaryTimesheet.
        // eetc = TaskAssignment.getExpectedWork() -
        // (TaskAssignment.getActualWork() + deltaActualWork).

        double eetc = 0;
        if (summaryTimesheet != null
                && summaryTimesheet.getTaskAssignment() != null
                && summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {
            TaskAssignment taskAssignment = (TaskAssignment) summaryTimesheet
                .getTaskAssignment();
     
            //If EETC is null or less than zero: Calculate EETC.
            //If EETC is zero and if actulas are being updated or already present: Calcualte EETC. 
            if (hasToCaluculateEETC(summaryTimesheet))
            {
                if (taskAssignment.getExpectedWork() != null)
                {
                    double taskAssignmentExpectedWork = taskAssignment
                        .getExpectedWork().doubleValue();
                    double taskAssignmentActualWork = 0;
                    if (taskAssignment.getActualWork() != null)
                    {
                        taskAssignmentActualWork = taskAssignment
                            .getActualWork().doubleValue();
                    }

                    if (!hasNewActualReported(summaryTimesheet))
                    {
                        // Case 1
                        eetc = taskAssignmentExpectedWork
                                - taskAssignmentActualWork;
                    }
                    else
                    {
                        // Case 2
                        eetc = taskAssignmentExpectedWork
                                - (taskAssignmentActualWork + deltaSummaryTimesheetActualWork(
                                    summaryTimesheet, rpmScope, msgCtx));
                    }
                }
            }
            else
            {
                eetc = taskAssignment.getEETCHours().doubleValue();
            }

            // EETC cannot be negative
            if (eetc < 0)
            {
                eetc = 0;
            }

            ((TaskAssignment) summaryTimesheet.getTaskAssignment())
                .setEETCHours(new Double(eetc));
        }

        return eetc;
    }
        
    private static boolean hasToCaluculateEETC(SummaryTimesheet summaryTimesheet)
    {
    	boolean caluculateEETC = false;
    	TaskAssignment taskAssignment = (TaskAssignment) summaryTimesheet.getTaskAssignment();
    	Double EETC = taskAssignment.getEETCHours();
    	if(EETC == null || EETC.doubleValue() < 0)
    	{
    		caluculateEETC = true;
    	}
    	else if(EETC.doubleValue() == 0)
    	{
    		Double actualWork = taskAssignment.getActualWork();
    		if((actualWork != null && actualWork.doubleValue() > 0) || 
    				TimesheetManagerUtil.doesSummaryTimesheetHaveAnyModifiedNonZeroActuals(summaryTimesheet))
    		{
    			caluculateEETC = true;
    		}
    	}
		return caluculateEETC;
    }

    public static double calculatePercentageComplete(
            SummaryTimesheet summaryTimesheet, RPMObjectScope rpmScope,
            MessageContext msgCtx) throws RPMException, ParseException, SQLException

    {
        // Calculates PercentageComplete if taskAssignmentPercentageComplete is
        // null; returns the taskAssignmentPercentageComplete otherwise.
        // Two cases for calculating PercentageComplete using the following:
        // percentageComplete = ((Planned - Remaining) x 100) / (Planned + EETC
        // - Remaining)
        // Assume EETC has been calculated.
        // Case 1: There is no Actuals reported.
        // percentageComplete = (TaskAssignment.getActualWork() x 100) /
        // (TaskAssignment.getActualWork() + TaskAssignment.getEETCHours())
        // Case 2: There are Actuals reported.
        // percentageComplete = ((TaskAssignment.getActualWork() +
        // deltaSummaryTimesheetActualWork)
        // x 100) / (TaskAssignment.getActualWork() +
        // deltaSummaryTimesheetActualWork
        // + TaskAssignment.getEETCHours())

        double percentageComplete = 0;
        TaskAssignment taskAssignment = null;

        if (summaryTimesheet != null
                && summaryTimesheet.getTaskAssignment() != null
                && summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {
            taskAssignment = (TaskAssignment) summaryTimesheet
                .getTaskAssignment();
            
            if (taskAssignment.getPercentageComplete() == null)
            {
                double taskAssignmentActualWork = 0;
                if (taskAssignment.getActualWork() != null)
                {
                    taskAssignmentActualWork = taskAssignment.getActualWork()
                        .doubleValue();
                }

                double taskAssignmentEETC = 0;
                if (taskAssignment.getEETCHours() != null)
                {
                    taskAssignmentEETC = taskAssignment.getEETCHours()
                        .doubleValue();
                }

                if (!hasNewActualReported(summaryTimesheet))
                {
                    // Case 1
                		// If EETC is modified then percentageComplete should not be updated/calculated
                		// using the new EETC value. The value of percentageComplete should not be 
                		// modified in this case. It has to be calculated using the old EETC value.
                	if (taskAssignment.testEETCHoursModified()) {
						taskAssignmentEETC = 0;
						RPMObjectManager manager = RPMManagerFactory
								.getInstance().getRPMObjectManager(
										taskAssignment.getClass());
						TaskAssignment loadedTaskAssignment = (TaskAssignment) manager
								.load(taskAssignment, null, msgCtx);
						if (loadedTaskAssignment.getEETCHours() != null) {
							taskAssignmentEETC = loadedTaskAssignment
									.getEETCHours().doubleValue();
						}
					}
					percentageComplete = (taskAssignmentActualWork * 100)
							/ (taskAssignmentActualWork + taskAssignmentEETC);
				}
                else
                {
                    // Case 2
                    percentageComplete = ((taskAssignmentActualWork + deltaSummaryTimesheetActualWork(
                        summaryTimesheet, rpmScope, msgCtx)) * 100)
                            / (taskAssignmentActualWork
                                    + deltaSummaryTimesheetActualWork(
                                        summaryTimesheet, rpmScope, msgCtx) + taskAssignmentEETC);
                }
            }
            else
            {
                percentageComplete = taskAssignment.getPercentageComplete()
                    .doubleValue();
            }
        }

        return percentageComplete;
    }
    
    public static boolean hasNewActualReported(AbstractTimesheet timesheet)
    {
        return timesheet.testRegularMondayModified()
                || timesheet.testRegularTuesdayModified()
                || timesheet.testRegularWednesdayModified()
                || timesheet.testRegularThursdayModified()
                || timesheet.testRegularFridayModified()
                || timesheet.testRegularSaturdayModified()
                || timesheet.testRegularSundayModified()
                || timesheet.testSpecialMondayModified()
                || timesheet.testSpecialTuesdayModified()
                || timesheet.testSpecialWednesdayModified()
                || timesheet.testSpecialThursdayModified()
                || timesheet.testSpecialFridayModified()
                || timesheet.testSpecialSaturdayModified()
                || timesheet.testSpecialSundayModified();
    }

    /**
     * This method returns true if we have any modified actuals that are Non Zero
     * @param timesheet
     * @return true if modified
     */  
    public static boolean doesSummaryTimesheetHaveAnyModifiedNonZeroActuals(AbstractTimesheet timesheet)
    {
    	return ((timesheet.testRegularMondayModified() && timesheet.getRegularMonday() != null && timesheet.getRegularMonday().doubleValue() > 0)
    			||(timesheet.testRegularTuesdayModified() && timesheet.getRegularTuesday() != null && timesheet.getRegularTuesday().doubleValue() > 0)
    			||(timesheet.testRegularWednesdayModified() && timesheet.getRegularWednesday() != null  && timesheet.getRegularWednesday().doubleValue() > 0)
    			||(timesheet.testRegularThursdayModified() && timesheet.getRegularThursday() != null && timesheet.getRegularThursday().doubleValue() > 0)
    			||(timesheet.testRegularFridayModified() && timesheet.getRegularFriday()!= null && timesheet.getRegularFriday().doubleValue() > 0)
    			||(timesheet.testRegularSaturdayModified() && timesheet.getRegularSaturday()!= null && timesheet.getRegularSaturday().doubleValue() > 0)
    			||(timesheet.testRegularSundayModified() && timesheet.getRegularSunday()!= null && timesheet.getRegularSunday().doubleValue() > 0)
    			||(timesheet.testSpecialMondayModified() && timesheet.getSpecialMonday()!= null && timesheet.getSpecialMonday().doubleValue() > 0)
    			||(timesheet.testSpecialTuesdayModified() && timesheet.getSpecialTuesday()!= null && timesheet.getSpecialTuesday().doubleValue() > 0)
    			||(timesheet.testSpecialWednesdayModified() && timesheet.getSpecialWednesday()!= null && timesheet.getSpecialWednesday().doubleValue() > 0)
    			||(timesheet.testSpecialThursdayModified() && timesheet.getSpecialThursday() != null && timesheet.getSpecialThursday().doubleValue() > 0)
    			||(timesheet.testSpecialFridayModified() && timesheet.getSpecialFriday() != null && timesheet.getSpecialFriday().doubleValue() > 0)
    			||(timesheet.testSpecialSaturdayModified() && timesheet.getSpecialSaturday() != null&& timesheet.getSpecialSaturday().doubleValue() > 0)
    			||(timesheet.testSpecialSundayModified() && timesheet.getSpecialSunday() != null && timesheet.getSpecialSunday().doubleValue() > 0));
    }
    
    public static double calculatePercentageCompleteWithSteps(
            GenericStep step, MessageContext msgCtx) throws RPMException
    {
        // The result will override the value in taskAssignment.
        // %C = Sum(StepActualHours/(StepActualHours+StepEETC)*StepWeight)
        
        double percentageComplete = 0;
        TaskAssignment taskAssignment = null;

        if (step != null && step.getTaskAssignment() != null)
        {
            taskAssignment = step.getTaskAssignment();
            String taskAssignmentID = taskAssignment.getID();
            if (taskAssignmentID != null)
            {
                String[] stepIds = retrieveStepIDs(taskAssignmentID, msgCtx);
                if (stepIds != null && stepIds.length > 0)
                {
                    for (int i = 0; i != stepIds.length; ++i)
                    {
                        double eetc = DateUtil.convertMinutesToHours(
                            retrieveStep_eetc(stepIds[i], msgCtx))
                            .doubleValue();
                        double actualWork = DateUtil.convertMinutesToHours(
                            retrieveStep_ActualWork(stepIds[i], msgCtx))
                            .doubleValue();
                        if (eetc + actualWork != 0)
                        {
                            percentageComplete += actualWork
                                    / (actualWork + eetc)
                                    * calculateStepWeight(taskAssignmentID,
                                        stepIds[i], eetc, actualWork, msgCtx);
                        }
                    }
                }
            }
        }

        return percentageComplete;
    }
    
    private static double calculateStepWeight(String taskAssignmentId, String currentStepId, double eetcCurrentStep, double actualWorkCurrentStep, MessageContext messageContext) throws RPMException
    {
        // The weight field has non-zero value only if the user manually changes
        // it; otherwise the field will be zero.
        // For this reason, the weight will be calculated for those steps which
        // contain zero value in the weight field.
        // Three cases when we calculate the weight.
        // Case 1: when eetc and actualHours are both zero
        // weight = 0
        // Case 2: if the weight contains non-zero value
        // return the value as weight
        // Case 3: if the weight contains zero value
        // weight = (ActualHoursStepU + EETCStepU)/(Sum(ActualHoursStepU) +
        // (TaskAssignmentEETC - Sum(EETCStepM)))*(100-Sum(WeightStepM))
        // where StepU is a Step whose Weight is not changed manually and StepM
        // is a Step whose Weight is changed manually.
        double weight = 0;

        if (eetcCurrentStep == 0 && actualWorkCurrentStep == 0)
        {
            weight = 0;
        }
        else
        {
            weight = retrieveStep_weight(currentStepId, messageContext) / 100.0;
            if (weight == 0)
            {
                // Case 3
                weight = (eetcCurrentStep + actualWorkCurrentStep)
                        / DateUtil.convertMinutesToHours(
                            sumActualForZeroWeightSteps(taskAssignmentId,
                                messageContext)
                                    + retrieveTaskAssignment_EETC(
                                        taskAssignmentId, messageContext)
                                    - sumEETCForNoneZeroWeightSteps(
                                        taskAssignmentId, messageContext))
                            .doubleValue()
                        * (100 - sumWeightForNoneZeroWeightSteps(
                            taskAssignmentId, messageContext) / 100.0);
            }
        }

        return weight;
    }
    
    private static long sumActualForZeroWeightSteps(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {
        long total = 0;

        Number number = sumStepNumberFields(taskAssignmentId, new String[]
        {
            StepManager.NAME_WORK_TODATE
        }, StepManager.NAME_EDPERCENT + "=0", messageContext);
        if (number != null)
        {
            total = number.longValue();
        }

        return total;
    }
    
    private static long sumEETCForNoneZeroWeightSteps(String taskAssignmentId,
            MessageContext messageContext) throws RPMException
    {
        long eetc = 0;

        Number number = sumStepNumberFields(taskAssignmentId, new String[]
        {
            StepManager.NAME_ETC
        }, StepManager.NAME_EDPERCENT + "!=0", messageContext);
        if (number != null)
        {
            eetc = number.longValue();
        }

        return eetc;
    }
    
    private static int sumWeightForNoneZeroWeightSteps(
            String taskAssignmentId, MessageContext messageContext)
            throws RPMException
    {
        int total = 0;

        Number number = sumStepNumberFields(taskAssignmentId, new String[]
        {
            StepManager.NAME_EDPERCENT
        }, StepManager.NAME_EDPERCENT + "!=0", messageContext);
        if (number != null)
        {
            total = number.intValue();
        }

        return total;
    }

    private static Number sumStepNumberFields(String taskAssignmentId,
            String[] fields, String extraWhere, MessageContext messageContext)
            throws RPMException
    {
        Number value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        StringBuffer sumFields = new StringBuffer();
        sumFields.append("SUM(");
        for (int i = 0; i != fields.length; ++i)
        {
            sumFields.append(fields[i]);
            if (i != fields.length - 1)
            {
                sumFields.append("+");
            }
        }
        sumFields.append(")");
        stmt.append(sumFields.toString());
        stmt.appendFrom(StepManager.TABLE_NAME);
        stmt.appendWhere();
        if (extraWhere != null)
        {
            stmt.append(extraWhere);
        }
        stmt.appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            taskAssignmentId
        }, messageContext);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList rows = (ArrayList) result.get(0);
            value = (Number) rows.get(0);
        }

        return value;
    }
    
    private static String[] retrieveStepIDs(String taskAssignmentID,
            MessageContext msgCtx) throws RPMException
    {
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(StepManager.NAME_ELEMENT_ID);
        stmt.appendFrom(StepManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            taskAssignmentID
        }, msgCtx);

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            String[] stepIds = new String[result.size()];
            for (int i = 0; i != result.size(); ++i)
            {
                ArrayList rows = (ArrayList)result.get(i);
                stepIds[i] = (String) rows.get(0);
            }

            return stepIds;
        }
        return null;
    }
    
    /**
     * Converts the given percentage floating point value to an integer "per ten
     * thousand" value, rounded to the closest 10.
     */
    public static Integer toRoundedPerTenThousand(double percent)
    {
        // Converts percent to per thousand and rounds.
        int perThousand = (int) ((percent * 10.0) + 0.5);
        // Converts per thousand to per ten thousand.
        // => the last digit of the result will always be 0.
        int perTenThousand = perThousand * 10;
        return new Integer(perTenThousand);
    }
    
    public static TimesheetApprovalStatus loadTimesheetApprovalStatus(
            SummaryTimesheet summaryTimesheet, MessageContext msgCtx)
            throws RPMException
    {
        TimesheetApprovalStatus loadedStatus = null;

        // load the TimesheetApprovalStatus from the db all the time
        // load the resource id and the project id
        GenericTaskAssignment assignment = summaryTimesheet.getTaskAssignment();
        String resourceID = null;
        String projectID = null;

        SqlBuffer condition = new SqlBuffer();
        condition
            .appendEqualQuestionMark(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(TaskAssignmentManager.NAME_RESOURCE_ID);
        stmt.appendComma();
        stmt.append(TaskAssignmentManager.NAME_PROJECT_ID);
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere(condition);

        List result = null;
        result = ManagerUtil.select(new Class[]
        {
                String.class, String.class
        }, assignment, null, stmt, new Object[]
        {
            assignment.getID()
        }, msgCtx);

        if (result != null && result.size() == 1)
        {
            ArrayList row = (ArrayList) result.get(0);
            resourceID = (String) row.get(0);
            projectID = (String) row.get(1);
        }

        // check the db for an approval status
        stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(TimesheetApprovalStatusManager.NAME_CURRENT_STATE);
        stmt.appendComma();
        stmt.append(TimesheetApprovalStatusManager.NAME_REC_DATETIME);
        stmt.appendFrom(TimesheetApprovalStatusManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_ELEMENT_ID);
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_RESOURCE_ID);
        stmt
            .appendEqualQuestionMark(TimesheetApprovalStatusManager.NAME_WEEK_OF);

        result = ManagerUtil.select(new Class[]
        {
                Integer.class, Calendar.class
        }, null, null, stmt, new Object[]
        {
                projectID, resourceID,
                DateUtil.convertToString(summaryTimesheet.getWeekOf())
        }, msgCtx);

        if (result != null && result.size() > 0)
        {
            ArrayList row = (ArrayList) result.get(0);
            loadedStatus = new TimesheetApprovalStatus();
            TimesheetStatus status = TimesheetStatusUtil
                .getTimesheetStatusByIndex((Integer) row.get(0));
            loadedStatus.setApprovalStatus(status);
            loadedStatus.setApprovalDate((Calendar) row.get(1));
        }

        return loadedStatus;
    }
}
