/*
 * Created on Feb 20, 2005 
 * @author mmatwiy 
 * Checkpoint validation of the read-only WbsScheduleDates common to 
 * AggregateNode containers.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public abstract class AbstractAggregateNodeCheckpoint extends
        WorkElementCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, parentRequired,
            validParentTypes);

        WorkElement workElement = (WorkElement) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (scope != null && scope.isScheduleDates())
        {
            WbsScheduleDate actualDate = null;
            WbsScheduleDate forecastDate = null;
            WbsScheduleDate planDate = null;
            WbsScheduleDate proposedDate = null;
            
            Object[] loadedValues = new Object[4];
            if (workElement.testActualDateModified()
                    || workElement.testForecastDateModified()
                    || workElement.testPlanDateModified()
                    || workElement.testProposedDateModified())
            {
                loadedValues = loadWbsScheduleDatesFromDB(workElement, msgCtx);            
                actualDate = (WbsScheduleDate)loadedValues[0];
                forecastDate = (WbsScheduleDate)loadedValues[1];
                planDate = (WbsScheduleDate)loadedValues[2];
                proposedDate = (WbsScheduleDate)loadedValues[3];
            }

            if (workElement.testActualDateModified())
            {
                GenericValidator.validateReadOnly(workElement,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD, msgCtx);

                // reset read-only field value
                workElement.setActualDate(actualDate);
            }

            if (workElement.testForecastDateModified())
            {
                GenericValidator.validateReadOnly(workElement,
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD, msgCtx);

                // reset read-only field value
                workElement
                    .setForecastDate(forecastDate);
            }

            if (workElement.testPlanDateModified())
            {
                GenericValidator.validateReadOnly(workElement,
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD, msgCtx);

                // reset read-only field value
                workElement.setPlanDate(planDate);
            }

            if (workElement.testProposedDateModified())
            {
                GenericValidator.validateReadOnly(workElement,
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD, msgCtx);

                // reset read-only field value
                workElement
                    .setProposedDate(proposedDate);
            }
        }
    }
}