/*
 * Created on Nov 24, 2005 
 * @author mmatwiy 
 * Checkpoint validation for container ElementDependency.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;

public class ElementDependencyCheckpoint extends AbstractCheckpoint
{

    private static final List ELEMENT_DEPENDENCY_PARENT_LIST = new ArrayList();

    protected static ElementDependencyCheckpoint instance = new ElementDependencyCheckpoint();

    static
    {
        ELEMENT_DEPENDENCY_PARENT_LIST.add(Proposal.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(Project.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(WorkOrganization.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(Deliverable.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(WorkProduct.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(SummaryTask.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(Milestone.class);
        ELEMENT_DEPENDENCY_PARENT_LIST.add(Task.class);
    }

    public static ElementDependencyCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true,
            ELEMENT_DEPENDENCY_PARENT_LIST);

        ElementDependency elementDependency = (ElementDependency) rpmObject;

        // cannot save an element dependency with no predecessor
        GenericValidator.validateMandatory(elementDependency,
            ValidationConstants.ELEMENT_DEPENDENCY_PREDECESSOR_FIELD,
            elementDependency.getPredecessor(), msgCtx);
        
        if (elementDependency.testCreatedByModified())
        {
            // createdBy is set by the DB; read-only
            GenericValidator
                .validateReadOnly(elementDependency,
                    ValidationConstants.ELEMENT_DEPENDENCY_CREATED_BY_FIELD,
                    msgCtx);
        }

        if (elementDependency.testAmountModified())
        {
            // Check that the amount is in the correct range
            // of values
            GenericValidator.validateRange(elementDependency,
                ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_FIELD,
                elementDependency.getAmount(),
                ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MIN,
                ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MAX, msgCtx);
        }

        // On update, the amount, dependencyType, leadLagType, and unitType
        // fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(elementDependency,
            ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_FIELD,
            elementDependency.getAmount(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(elementDependency,
            ValidationConstants.ELEMENT_DEPENDENCY_TYPE_FIELD,
            elementDependency.getDependencyType(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(elementDependency,
            ValidationConstants.ELEMENT_DEPENDENCY_LEAD_LAG_FIELD,
            elementDependency.getLeadlagType(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(elementDependency,
            ValidationConstants.ELEMENT_DEPENDENCY_UNIT_TYPE_FIELD,
            elementDependency.getUnitType(), msgCtx);
    }

}
