/*
 * Created on Nov 17, 2005 
 * @author mmatwiy 
 * Checkpoint validation of WBS container Portfolio.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.WbsModule;

public class PortfolioCheckpoint extends AbstractCheckpoint
{
    private static final List PORTFOLIO_PARENT_LIST = new ArrayList();

    protected static PortfolioCheckpoint instance = new PortfolioCheckpoint();

    static
    {
        PORTFOLIO_PARENT_LIST.add(WbsModule.class);
        PORTFOLIO_PARENT_LIST.add(Portfolio.class);
    }

    public static PortfolioCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        boolean valid = true;

        valid = super.isValidParent(rpmObject, msgCtx, false,
            PORTFOLIO_PARENT_LIST);

        Portfolio portfolio = (Portfolio) rpmObject;

        // Check that the mandatory name exists
        valid = valid
                & GenericValidator
                    .validateMandatory(portfolio,
                        ValidationConstants.NAME_FIELD, portfolio.getName(),
                        msgCtx);

        if (portfolio.testNameModified() && valid)
        {
            // If the name exists, check that it doesn't exceed the max length
            // and that it is at least 1 character long
            valid = GenericValidator.validateMinLength(portfolio,
                ValidationConstants.NAME_FIELD, portfolio.getName(),
                ValidationConstants.PORTFOLIO_NAME_MIN, msgCtx);
            
            valid = GenericValidator.validateMaxLength(portfolio,
                ValidationConstants.NAME_FIELD, portfolio.getName(),
                ValidationConstants.PORTFOLIO_NAME_CHARACTER_LIMIT, msgCtx);
        }

    }
}
