/*
 * Created on Nov 17, 2005
 * @author mmatwiy
 * Checkpoint validation for WorkProduct container.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;


public class WorkProductCheckpoint extends AbstractProjectActivityCheckpoint
{
    private static final List WORKPRODUCT_PARENT_LIST = new ArrayList();

    protected static WorkProductCheckpoint instance = new WorkProductCheckpoint();

    static
    {
        WORKPRODUCT_PARENT_LIST.add(Proposal.class);
        WORKPRODUCT_PARENT_LIST.add(Project.class);
        WORKPRODUCT_PARENT_LIST.add(WorkOrganization.class);
        WORKPRODUCT_PARENT_LIST.add(Deliverable.class);
        WORKPRODUCT_PARENT_LIST.add(WorkProduct.class);
        WORKPRODUCT_PARENT_LIST.add(SummaryTask.class);
    }

    public static WorkProductCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true, WORKPRODUCT_PARENT_LIST);

        WorkProduct workProduct = (WorkProduct) rpmObject;
        WorkElementScope scope = (WorkElementScope)rpmScope;
        
        if (scope != null && scope.isState() && workProduct.testStateModified())
        {
            // workProduct.state must be of type DeliverableState
            GenericValidator.validateType(rpmObject,
                DeliverableState.class, ValidationConstants.STATE_FIELD,
                workProduct.getState(), msgCtx);
        }
    }

}
