package com.ibm.rpm.wbs.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.TimeUnitType;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;

//TODO Copy next line in RPMInitializer.loadManagers()
//Class.forName(DependenciesManager.class.getName());

public class DependenciesManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        //TODO Add the complete class name of all the containers that could be loaded by this managers.
        CONTAINERS.add(ElementDependency.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                DependenciesManager.class);
        }
    }

    /**
     * Column DEPEND_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_DEPEND_ID = 1;

    public static final int TYPE_DEPEND_ID = Types.CHAR;

    public static final String NAME_DEPEND_ID = "DEPENDENCIES.DEPEND_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_DEPEND_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEPEND_ID, NAME_DEPEND_ID);
    }

    /**
     * Column PRED_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PRED_ID = 2;

    public static final int TYPE_PRED_ID = Types.CHAR;

    public static final String NAME_PRED_ID = "DEPENDENCIES.PRED_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_ID,NAME_PRED_ID);}

    /**
     * Column PRED_PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PRED_PROJECT_ID = 3;

    public static final int TYPE_PRED_PROJECT_ID = Types.CHAR;

    public static final String NAME_PRED_PROJECT_ID = "DEPENDENCIES.PRED_PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_PROJECT_ID,NAME_PRED_PROJECT_ID);}

    /**
     * Column PRED_OBJECT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PRED_OBJECT_TYPE = 4;

    public static final int TYPE_PRED_OBJECT_TYPE = Types.INTEGER;

    public static final String NAME_PRED_OBJECT_TYPE = "DEPENDENCIES.PRED_OBJECT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_OBJECT_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_OBJECT_TYPE,NAME_PRED_OBJECT_TYPE);}

    /**
     * Column SUCC_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SUCC_ID = 5;

    public static final int TYPE_SUCC_ID = Types.CHAR;

    public static final String NAME_SUCC_ID = "DEPENDENCIES.SUCC_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUCC_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUCC_ID,NAME_SUCC_ID);}

    /**
     * Column SUCC_PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SUCC_PROJECT_ID = 6;

    public static final int TYPE_SUCC_PROJECT_ID = Types.CHAR;

    public static final String NAME_SUCC_PROJECT_ID = "DEPENDENCIES.SUCC_PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUCC_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUCC_PROJECT_ID,NAME_SUCC_PROJECT_ID);}

    /**
     * Column SUCC_OBJECT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUCC_OBJECT_TYPE = 7;

    public static final int TYPE_SUCC_OBJECT_TYPE = Types.INTEGER;

    public static final String NAME_SUCC_OBJECT_TYPE = "DEPENDENCIES.SUCC_OBJECT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUCC_OBJECT_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUCC_OBJECT_TYPE,NAME_SUCC_OBJECT_TYPE);}

    /**
     * Column DEP_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DEP_TYPE = 8;

    public static final int TYPE_DEP_TYPE = Types.INTEGER;

    public static final String NAME_DEP_TYPE = "DEPENDENCIES.DEP_TYPE";

    public static final String PROPERTY_DEP_TYPE = "DEPENDENCYTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEP_TYPE, SqlUtil.castAsChar(
            NAME_DEP_TYPE, 1));
    }

    /**
     * Column LEAD_OR_LAG_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEAD_OR_LAG_TYPE = 9;

    public static final int TYPE_LEAD_OR_LAG_TYPE = Types.INTEGER;

    public static final String NAME_LEAD_OR_LAG_TYPE = "DEPENDENCIES.LEAD_OR_LAG_TYPE";

    public static final String PROPERTY_LEAD_OR_LAG_TYPE = "LEADLAGTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LEAD_OR_LAG_TYPE, SqlUtil.castAsChar(
            NAME_LEAD_OR_LAG_TYPE, 1));
    }

    /**
     * Column AMOUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_AMOUNT = 10;

    public static final int TYPE_AMOUNT = Types.INTEGER;

    public static final String NAME_AMOUNT = "DEPENDENCIES.AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_AMOUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_AMOUNT,NAME_AMOUNT);}

    /**
     * Column LAG_DISPLAY_UNIT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LAG_DISPLAY_UNIT = 11;

    public static final int TYPE_LAG_DISPLAY_UNIT = Types.INTEGER;

    public static final String NAME_LAG_DISPLAY_UNIT = "DEPENDENCIES.LAG_DISPLAY_UNIT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LAG_DISPLAY_UNIT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAG_DISPLAY_UNIT,NAME_LAG_DISPLAY_UNIT);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 12;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "DEPENDENCIES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 13;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "DEPENDENCIES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 14;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "DEPENDENCIES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column PRED_START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PRED_START_DT = 15;

    public static final int TYPE_PRED_START_DT = Types.TIMESTAMP;

    public static final String NAME_PRED_START_DT = "DEPENDENCIES.PRED_START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_START_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_START_DT,NAME_PRED_START_DT);}

    /**
     * Column PRED_FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PRED_FINISH_DT = 16;

    public static final int TYPE_PRED_FINISH_DT = Types.TIMESTAMP;

    public static final String NAME_PRED_FINISH_DT = "DEPENDENCIES.PRED_FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_FINISH_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_FINISH_DT,NAME_PRED_FINISH_DT);}

    /**
     * Column SUCC_START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SUCC_START_DT = 17;

    public static final int TYPE_SUCC_START_DT = Types.TIMESTAMP;

    public static final String NAME_SUCC_START_DT = "DEPENDENCIES.SUCC_START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUCC_START_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUCC_START_DT,NAME_SUCC_START_DT);}

    /**
     * Column SUCC_FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SUCC_FINISH_DT = 18;

    public static final int TYPE_SUCC_FINISH_DT = Types.TIMESTAMP;

    public static final String NAME_SUCC_FINISH_DT = "DEPENDENCIES.SUCC_FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUCC_FINISH_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUCC_FINISH_DT,NAME_SUCC_FINISH_DT);}

    /**
     * Column PRED_UPDATED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRED_UPDATED = 19;

    public static final int TYPE_PRED_UPDATED = Types.SMALLINT;

    public static final String NAME_PRED_UPDATED = "DEPENDENCIES.PRED_UPDATED";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PRED_UPDATED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRED_UPDATED,NAME_PRED_UPDATED);}

    private static final String TABLE_NAME = "DEPENDENCIES";

    /**
     * Create an array of type string containing all the fields of the DEPENDENCIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_DEPEND_ID, NAME_PRED_ID, NAME_PRED_PROJECT_ID,
            NAME_PRED_OBJECT_TYPE, NAME_SUCC_ID, NAME_SUCC_PROJECT_ID,
            NAME_SUCC_OBJECT_TYPE, NAME_DEP_TYPE, NAME_LEAD_OR_LAG_TYPE,
            NAME_AMOUNT, NAME_LAG_DISPLAY_UNIT, NAME_REC_STATUS, NAME_REC_USER,
            NAME_REC_DATETIME, NAME_PRED_START_DT, NAME_PRED_FINISH_DT,
            NAME_SUCC_START_DT, NAME_SUCC_FINISH_DT, NAME_PRED_UPDATED
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new DependenciesManager instance.
     * @return DependenciesManager
     */
    public DependenciesManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        //TODO validate the primary key field name.
        return NAME_DEPEND_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        //TODO validate the container type field name.
        //        int typeId = resultSet.getInt(ID_TYPE_ID);
        //        return new Integer(typeId);

        //If container type doesn't apply to this manager
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected HashSet getValidContainerNames()
    {
        return CONTAINERS;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new ElementDependency();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (rpmObject != null)
        {

            ElementDependency dependency = (ElementDependency) rpmObject;

            // Set default values
            if (dependency.getDependencyType() == null)
                dependency.setDependencyType(DependencyType.FinishStart);

            if (dependency.getAmount() == null)
                dependency.setAmount(new Integer(0));

            if (dependency.getLeadlagType() == null)
                dependency.setLeadlagType(LeadLagType.Lead);

            if (dependency.getUnitType() == null)
                dependency.setUnitType(TimeUnitType.Days);

            rpmObject.setID(SP_A_DEPEND(rpmObject, messageContext));
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        ElementDependency temp = (ElementDependency) rpmObject;
        temp.setID(rs.getString(ID_DEPEND_ID));
        LeadLagType type = (rs.getInt(ID_LEAD_OR_LAG_TYPE) == 0)
                ? LeadLagType.Lag : LeadLagType.Lead;
        TimeUnitType unit = (rs.getInt(ID_LAG_DISPLAY_UNIT) == 0)
                ? TimeUnitType.Days : TimeUnitType.Hours;
        //cal amount.

        // we need to load the parent's minutes_per_day field in order
        // to calculate the amount field, so decoding the amount is done in
        // loadRelatedObjects

        if (isDelta)
        {
            temp.deltaDependencyType((DependencyType) EnumeratedTypeUtil
                .getInstance().getType(null, DependencyType.class,
                    new Integer(rs.getInt(ID_DEP_TYPE))));
            temp.deltaLeadlagType(type);
            temp.deltaUnitType(unit);
        }
        else
        {
            temp.setDependencyType((DependencyType) EnumeratedTypeUtil
                .getInstance().getType(null, DependencyType.class,
                    new Integer(rs.getInt(ID_DEP_TYPE))));
            temp.setLeadlagType(type);
            temp.setUnitType(unit);
        }

        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope, com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.util.FieldValueMap, int, boolean)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
    throws RPMException, SQLException, ParseException
    {
        ElementDependency dependency = (ElementDependency) rpmObject;
        
        WbsManager wbsMgr = (WbsManager) RPMManagerFactory.getInstance()
            .getRPMObjectManager(Proposal.class.getName());
        
        WorkElement predecessor = null;
        Resource resource = null;
        
        if (rpmScope != null)
        {
            ElementDependencyScope scope = (ElementDependencyScope) rpmScope;

            if (scope.getPredecessor() != null)
            {
            	String predecessorId = (String) relatedFields.get(index, ID_PRED_ID);
                //successor is the parent; set predecessor
                predecessor = new WorkElement();
                predecessor.setID(predecessorId);
                predecessor = (WorkElement) wbsMgr.loadByPrimaryKey(
                    predecessor, scope.getPredecessor(), messageContext, isDelta);
                
                //check if predecessor has been loaded
                if(predecessor==null && messageContext.isWebServicesSecurityEnabled())
                {
                	//the predecessor might have not been loaded because the
					//user is not assigned to it. Therefore, we only the
					// predecessor name to allow the WEBUI to display it as it's
					// the case in the delphi client
                	predecessor = new WorkElement();
                	predecessor.setName(loadPredecessorName(predecessorId, messageContext));
                }
            }

            if (scope.getCreatedBy() != null)
            {
                ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                    .getInstance()
                    .getRPMObjectManager(Resource.class.getName());
                resource = new Resource();
                resource.setID((String) relatedFields.get(index, ID_REC_USER));
                resource = (Resource) resourceManager.loadByPrimaryKey(
                    resource, scope.getCreatedBy(), messageContext, isDelta);
            }
        }

        // always load the parent; we need it to calculate the amount field
        WorkElement parent = new WorkElement();
        parent.setID((String) relatedFields.get(index, ID_SUCC_ID));
        parent = (WorkElement) wbsMgr.loadByPrimaryKey(parent, null,
            messageContext, isDelta);

        int loadedUnit = ((Integer) relatedFields.get(index,
            ID_LAG_DISPLAY_UNIT)) == null ? 0 : ((Integer) relatedFields.get(
            index, ID_LAG_DISPLAY_UNIT)).intValue();
        TimeUnitType unit = loadedUnit == 0 ? TimeUnitType.Days
                : TimeUnitType.Hours;

        // we have to calculate the amount here instead of in decodeRow because
        // we need the minutesPerDay field from the parent
        int amount = 0;
        int minutesPerDay = (parent == null || parent.getMinutesPerDay() == null)
                ? 0 : parent.getMinutesPerDay().intValue();
        int loadedAmount = ((Integer) relatedFields.get(index, ID_AMOUNT)) == null
                ? 0 : ((Integer) relatedFields.get(index, ID_AMOUNT))
                    .intValue();

        if (unit.equals(TimeUnitType.Hours))
        {
            amount = loadedAmount / 60;
        }
        else if (unit.equals(TimeUnitType.Days))
        {
            amount = minutesPerDay == 0 ? 0 : loadedAmount / minutesPerDay;
        }

        if (isDelta)
        {
            dependency.deltaPredecessor(predecessor);
            dependency.deltaCreatedBy(resource);
            dependency.deltaParent(parent);
            dependency.deltaAmount(new Integer(amount));
        }
        else
        {
            dependency.setPredecessor(predecessor);
            dependency.setCreatedBy(resource);
            dependency.setParent(parent);
            dependency.setAmount(new Integer(amount));
        }
        return dependency;
    }

    /**
	 * @param predecessorId
	 * @param messageContext
	 * @return
	 */
	private String loadPredecessorName(String predecessorId,
			MessageContext messageContext) throws RPMException, SQLException {
 
		String selectStmt = "SELECT ELEMENT_NAME FROM TMT_WBS WHERE ELEMENT_ID = '"
				+ predecessorId + "'";
		Connection c = null;
		ResultSet rs = null;
		try {
			c = Manager.getConnection(messageContext);
			selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);

			rs = ManagerUtil.executeSqlQuery(messageContext, c, selectStmt,
					null);

			String predecessorName = null;
			if (rs.next()) {
				predecessorName = rs.getString(1);
			}
			return predecessorName;
		} finally {
			Manager.close(messageContext, rs);
			Manager.releaseConnection(messageContext, c);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRelatedFields(java.sql.ResultSet,
	 *      com.ibm.rpm.framework.util.FieldValueMap, int)
	 */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String succId = resultSet.getString(ID_SUCC_ID);
        relatedFields.put(index, ID_SUCC_ID, succId);

        String predId = resultSet.getString(ID_PRED_ID);
        relatedFields.put(index, ID_PRED_ID, predId);

        String resourceId = resultSet.getString(ID_REC_USER);
        relatedFields.put(index, ID_REC_USER, resourceId);

        int amount = resultSet.getInt(ID_AMOUNT);
        relatedFields.put(index, ID_AMOUNT, new Integer(amount));

        int unit = resultSet.getInt(ID_LAG_DISPLAY_UNIT);
        relatedFields.put(index, ID_LAG_DISPLAY_UNIT, new Integer(unit));

        return relatedFields;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.MessageContext)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject != null)
        {
            SP_UA_DEPEND(rpmObject, messageContext);
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof DependenciesManager)
        {
            //where clause of the join condition
            //			StringBuffer condition = new StringBuffer();
            //			condition.append(getPrimaryKey());
            //			condition.append(" = ?");
            ////			joinCondition = getJoinCondition(manager, condition.toString(), 1,null,null);
            //			if (extraWhere != null && extraWhere.length() > 0) {
            //				StringBuffer sb = new StringBuffer();
            //				sb.append(joinCondition.getCondition());
            //				sb.append(" AND ");
            //				sb.append(extraWhere);
            //				joinCondition.setCondition(sb.toString());
            //			}
            //			joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof WbsManager)
        {
            joinCondition = getJoinCondition(manager, null, 0, null, null);
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }
        }
        else if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(" ( ");
            where.append(NAME_SUCC_ID);
            where.append(" = ? )");
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getSubTableJoinCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#saveObjectParents(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope, com.ibm.rpm.framework.MessageContext)
     */
    public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        // TODO Auto-generated method stub
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope, com.ibm.rpm.framework.MessageContext)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ElementDependency dependency = (ElementDependency) rpmObject;
        if (dependency.testDependencyTypeModified()
                || dependency.testAmountModified()
                || dependency.testLeadlagTypeModified()
                || dependency.testUnitTypeModified())
        {
            SP_U_DEPEND(rpmObject, messageContext);
        }
        return rpmObject;
    }

    private String SP_A_DEPEND(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_DEPEND";
        ElementDependency dependency = (ElementDependency) rpmObject;
        Object args[] = new Object[3];
        args[0] = (dependency.getParent() != null) ? dependency.getParent()
            .getID() : null;//successor
        args[1] = (dependency.getPredecessor() != null) ? dependency
            .getPredecessor().getID() : null;//predecessor
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private String SP_UA_DEPEND(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_UA_DEPEND";
        ElementDependency dependency = (ElementDependency) rpmObject;
        Object args[] = new Object[3];
        args[0] = (dependency.getParent() != null) ? dependency.getParent()
            .getID() : null;
        args[1] = dependency.getID();
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private String SP_U_DEPEND(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DEPEND";
        ElementDependency dependency = (ElementDependency) rpmObject;
        
        Integer dependencyType = (Integer) EnumeratedTypeUtil.getInstance().getValue(
            null, DependencyType.class, dependency.getDependencyType().getValue());
        
        Object args[] = new Object[6];
        args[0] = dependency.getID();
        args[1] = dependencyType;
        args[2] = (dependency.getLeadlagType().getValue()
            .equals(LeadLagType._Lead)) ? new Integer(1) : new Integer(0);
        args[3] = dependency.getAmount();
        args[4] = (dependency.getUnitType().getValue()
            .equals(TimeUnitType._Days)) ? new Integer(0) : new Integer(1);
        args[5] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }

}
