package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Proposal;

public class OpportunityManager extends AbstractRPMObjectManager{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(Opportunity.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                OpportunityManager.class);
        }
    }
    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;
    public static final int TYPE_PROJECT_ID = Types.CHAR;
    public static final String NAME_PROJECT_ID = "PROJECT_DETAILS.PROJECT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_PROJECT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column PROJECT_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROJECT_RTF_BITS = 2;
    public static final int TYPE_PROJECT_RTF_BITS = Types.INTEGER;
    public static final String NAME_PROJECT_RTF_BITS = "PROJECT_DETAILS.PROJECT_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PROJECT_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_RTF_BITS,NAME_PROJECT_RTF_BITS);}

    /**
     * Column DELIVERABLE_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELIVERABLE_RTF_BITS = 3;
    public static final int TYPE_DELIVERABLE_RTF_BITS = Types.INTEGER;
    public static final String NAME_DELIVERABLE_RTF_BITS = "PROJECT_DETAILS.DELIVERABLE_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DELIVERABLE_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELIVERABLE_RTF_BITS,NAME_DELIVERABLE_RTF_BITS);}

    /**
     * Column WORK_PRODUCT_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WORK_PRODUCT_RTF_BITS = 4;
    public static final int TYPE_WORK_PRODUCT_RTF_BITS = Types.INTEGER;
    public static final String NAME_WORK_PRODUCT_RTF_BITS = "PROJECT_DETAILS.WORK_PRODUCT_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_WORK_PRODUCT_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_PRODUCT_RTF_BITS,NAME_WORK_PRODUCT_RTF_BITS);}

    /**
     * Column SUMMARY_TASK_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUMMARY_TASK_RTF_BITS = 5;
    public static final int TYPE_SUMMARY_TASK_RTF_BITS = Types.INTEGER;
    public static final String NAME_SUMMARY_TASK_RTF_BITS = "PROJECT_DETAILS.SUMMARY_TASK_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SUMMARY_TASK_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SUMMARY_TASK_RTF_BITS,NAME_SUMMARY_TASK_RTF_BITS);}

    /**
     * Column MILESTONE_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MILESTONE_RTF_BITS = 6;
    public static final int TYPE_MILESTONE_RTF_BITS = Types.INTEGER;
    public static final String NAME_MILESTONE_RTF_BITS = "PROJECT_DETAILS.MILESTONE_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MILESTONE_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MILESTONE_RTF_BITS,NAME_MILESTONE_RTF_BITS);}

    /**
     * Column TASK_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TASK_RTF_BITS = 7;
    public static final int TYPE_TASK_RTF_BITS = Types.INTEGER;
    public static final String NAME_TASK_RTF_BITS = "PROJECT_DETAILS.TASK_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TASK_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TASK_RTF_BITS,NAME_TASK_RTF_BITS);}

    /**
     * Column REQUIREMENT_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REQUIREMENT_RTF_BITS = 8;
    public static final int TYPE_REQUIREMENT_RTF_BITS = Types.INTEGER;
    public static final String NAME_REQUIREMENT_RTF_BITS = "PROJECT_DETAILS.REQUIREMENT_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REQUIREMENT_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REQUIREMENT_RTF_BITS,NAME_REQUIREMENT_RTF_BITS);}

    /**
     * Column CHANGE_REQUEST_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHANGE_REQUEST_RTF_BITS = 9;
    public static final int TYPE_CHANGE_REQUEST_RTF_BITS = Types.INTEGER;
    public static final String NAME_CHANGE_REQUEST_RTF_BITS = "PROJECT_DETAILS.CHANGE_REQUEST_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHANGE_REQUEST_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHANGE_REQUEST_RTF_BITS,NAME_CHANGE_REQUEST_RTF_BITS);}

    /**
     * Column SERVICE_REQUEST_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SERVICE_REQUEST_RTF_BITS = 10;
    public static final int TYPE_SERVICE_REQUEST_RTF_BITS = Types.INTEGER;
    public static final String NAME_SERVICE_REQUEST_RTF_BITS = "PROJECT_DETAILS.SERVICE_REQUEST_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SERVICE_REQUEST_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SERVICE_REQUEST_RTF_BITS,NAME_SERVICE_REQUEST_RTF_BITS);}

    /**
     * Column DEFECT_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DEFECT_RTF_BITS = 11;
    public static final int TYPE_DEFECT_RTF_BITS = Types.INTEGER;
    public static final String NAME_DEFECT_RTF_BITS = "PROJECT_DETAILS.DEFECT_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DEFECT_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFECT_RTF_BITS,NAME_DEFECT_RTF_BITS);}

    /**
     * Column ISSUE_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ISSUE_RTF_BITS = 12;
    public static final int TYPE_ISSUE_RTF_BITS = Types.INTEGER;
    public static final String NAME_ISSUE_RTF_BITS = "PROJECT_DETAILS.ISSUE_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ISSUE_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ISSUE_RTF_BITS,NAME_ISSUE_RTF_BITS);}

    /**
     * Column RISK_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_RISK_RTF_BITS = 13;
    public static final int TYPE_RISK_RTF_BITS = Types.INTEGER;
    public static final String NAME_RISK_RTF_BITS = "PROJECT_DETAILS.RISK_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RISK_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RISK_RTF_BITS,NAME_RISK_RTF_BITS);}

    /**
     * Column CRI_DELIVERABLE_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CRI_DELIVERABLE_RTF_BITS = 14;
    public static final int TYPE_CRI_DELIVERABLE_RTF_BITS = Types.INTEGER;
    public static final String NAME_CRI_DELIVERABLE_RTF_BITS = "PROJECT_DETAILS.CRI_DELIVERABLE_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CRI_DELIVERABLE_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CRI_DELIVERABLE_RTF_BITS,NAME_CRI_DELIVERABLE_RTF_BITS);}

    /**
     * Column CRI_SUMMARY_TASK_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CRI_SUMMARY_TASK_RTF_BITS = 15;
    public static final int TYPE_CRI_SUMMARY_TASK_RTF_BITS = Types.INTEGER;
    public static final String NAME_CRI_SUMMARY_TASK_RTF_BITS = "PROJECT_DETAILS.CRI_SUMMARY_TASK_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CRI_SUMMARY_TASK_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CRI_SUMMARY_TASK_RTF_BITS,NAME_CRI_SUMMARY_TASK_RTF_BITS);}

    /**
     * Column CRI_MILESTONE_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CRI_MILESTONE_RTF_BITS = 16;
    public static final int TYPE_CRI_MILESTONE_RTF_BITS = Types.INTEGER;
    public static final String NAME_CRI_MILESTONE_RTF_BITS = "PROJECT_DETAILS.CRI_MILESTONE_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CRI_MILESTONE_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CRI_MILESTONE_RTF_BITS,NAME_CRI_MILESTONE_RTF_BITS);}

    /**
     * Column CRI_TASK_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CRI_TASK_RTF_BITS = 17;
    public static final int TYPE_CRI_TASK_RTF_BITS = Types.INTEGER;
    public static final String NAME_CRI_TASK_RTF_BITS = "PROJECT_DETAILS.CRI_TASK_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CRI_TASK_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CRI_TASK_RTF_BITS,NAME_CRI_TASK_RTF_BITS);}

    /**
     * Column DOCUMENT_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DOCUMENT_RTF_BITS = 18;
    public static final int TYPE_DOCUMENT_RTF_BITS = Types.INTEGER;
    public static final String NAME_DOCUMENT_RTF_BITS = "PROJECT_DETAILS.DOCUMENT_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DOCUMENT_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DOCUMENT_RTF_BITS,NAME_DOCUMENT_RTF_BITS);}

    /**
     * Column ACTIONITEM_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTIONITEM_RTF_BITS = 19;
    public static final int TYPE_ACTIONITEM_RTF_BITS = Types.INTEGER;
    public static final String NAME_ACTIONITEM_RTF_BITS = "PROJECT_DETAILS.ACTIONITEM_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ACTIONITEM_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTIONITEM_RTF_BITS,NAME_ACTIONITEM_RTF_BITS);}

    /**
     * Column URL_1 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_URL_1 = 20;
    public static final int TYPE_URL_1 = Types.VARCHAR;
    public static final String NAME_URL_1 = "PROJECT_DETAILS.URL_1";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_URL_1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_URL_1,NAME_URL_1);}

    /**
     * Column URL_2 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_URL_2 = 21;
    public static final int TYPE_URL_2 = Types.VARCHAR;
    public static final String NAME_URL_2 = "PROJECT_DETAILS.URL_2";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_URL_2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_URL_2,NAME_URL_2);}

    /**
     * Column URL_3 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_URL_3 = 22;
    public static final int TYPE_URL_3 = Types.VARCHAR;
    public static final String NAME_URL_3 = "PROJECT_DETAILS.URL_3";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_URL_3 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_URL_3,NAME_URL_3);}

    /**
     * Column URL_4 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_URL_4 = 23;
    public static final int TYPE_URL_4 = Types.VARCHAR;
    public static final String NAME_URL_4 = "PROJECT_DETAILS.URL_4";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_URL_4 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_URL_4,NAME_URL_4);}

    /**
     * Column URL_5 of type Types.VARCHAR mapped to String.
     */
    public static final int ID_URL_5 = 24;
    public static final int TYPE_URL_5 = Types.VARCHAR;
    public static final String NAME_URL_5 = "PROJECT_DETAILS.URL_5";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_URL_5 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_URL_5,NAME_URL_5);}

    /**
     * Column BUDGET_SETTING_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BUDGET_SETTING_BITS = 25;
    public static final int TYPE_BUDGET_SETTING_BITS = Types.INTEGER;
    public static final String NAME_BUDGET_SETTING_BITS = "PROJECT_DETAILS.BUDGET_SETTING_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_BUDGET_SETTING_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BUDGET_SETTING_BITS,NAME_BUDGET_SETTING_BITS);}

    /**
     * Column TIMEPHASE_PROJECT_ASSIGNMENT of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMEPHASE_PROJECT_ASSIGNMENT = 26;
    public static final int TYPE_TIMEPHASE_PROJECT_ASSIGNMENT = Types.CHAR;
    public static final String NAME_TIMEPHASE_PROJECT_ASSIGNMENT = "PROJECT_DETAILS.TIMEPHASE_PROJECT_ASSIGNMENT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TIMEPHASE_PROJECT_ASSIGNMENT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TIMEPHASE_PROJECT_ASSIGNMENT,NAME_TIMEPHASE_PROJECT_ASSIGNMENT);}

    /**
     * Column NOTEMIN_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_NOTEMIN_RTF_BITS = 27;
    public static final int TYPE_NOTEMIN_RTF_BITS = Types.INTEGER;
    public static final String NAME_NOTEMIN_RTF_BITS = "PROJECT_DETAILS.NOTEMIN_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTEMIN_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTEMIN_RTF_BITS,NAME_NOTEMIN_RTF_BITS);}

    /**
     * Column SCORECARD_RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCORECARD_RTF_BITS = 28;
    public static final int TYPE_SCORECARD_RTF_BITS = Types.INTEGER;
    public static final String NAME_SCORECARD_RTF_BITS = "PROJECT_DETAILS.SCORECARD_RTF_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCORECARD_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_RTF_BITS,NAME_SCORECARD_RTF_BITS);}

    /**
     * Column LEVEL_MOVE_PROJ of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_MOVE_PROJ = 29;
    public static final int TYPE_LEVEL_MOVE_PROJ = Types.INTEGER;
    public static final String NAME_LEVEL_MOVE_PROJ = "PROJECT_DETAILS.LEVEL_MOVE_PROJ";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_MOVE_PROJ = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_MOVE_PROJ,NAME_LEVEL_MOVE_PROJ);}

    /**
     * Column LEVEL_START_FINISH_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LEVEL_START_FINISH_DATE = 30;
    public static final int TYPE_LEVEL_START_FINISH_DATE = Types.DATE;
    public static final String NAME_LEVEL_START_FINISH_DATE = "PROJECT_DETAILS.LEVEL_START_FINISH_DATE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_START_FINISH_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_START_FINISH_DATE,NAME_LEVEL_START_FINISH_DATE);}

    /**
     * Column ENABLE_EV_CALCULATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ENABLE_EV_CALCULATION = 31;
    public static final int TYPE_ENABLE_EV_CALCULATION = Types.INTEGER;
    public static final String NAME_ENABLE_EV_CALCULATION = "PROJECT_DETAILS.ENABLE_EV_CALCULATION";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ENABLE_EV_CALCULATION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ENABLE_EV_CALCULATION,NAME_ENABLE_EV_CALCULATION);}

    /**
     * Column CHARGEBACK_FORMULA of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CHARGEBACK_FORMULA = 32;
    public static final int TYPE_CHARGEBACK_FORMULA = Types.SMALLINT;
    public static final String NAME_CHARGEBACK_FORMULA = "PROJECT_DETAILS.CHARGEBACK_FORMULA";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHARGEBACK_FORMULA = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGEBACK_FORMULA,NAME_CHARGEBACK_FORMULA);}

    /**
     * Column ROLLUP_PROJECT_ESTIMATED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ROLLUP_PROJECT_ESTIMATED = 33;
    public static final int TYPE_ROLLUP_PROJECT_ESTIMATED = Types.SMALLINT;
    public static final String NAME_ROLLUP_PROJECT_ESTIMATED = "PROJECT_DETAILS.ROLLUP_PROJECT_ESTIMATED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ROLLUP_PROJECT_ESTIMATED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_PROJECT_ESTIMATED,NAME_ROLLUP_PROJECT_ESTIMATED);}

    /**
     * Column ROLLUP_DELIVERABLE_ESTIMATED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ROLLUP_DELIVERABLE_ESTIMATED = 34;
    public static final int TYPE_ROLLUP_DELIVERABLE_ESTIMATED = Types.SMALLINT;
    public static final String NAME_ROLLUP_DELIVERABLE_ESTIMATED = "PROJECT_DETAILS.ROLLUP_DELIVERABLE_ESTIMATED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ROLLUP_DELIVERABLE_ESTIMATED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_DELIVERABLE_ESTIMATED,NAME_ROLLUP_DELIVERABLE_ESTIMATED);}

    /**
     * Column NOTIFY_TM_REJECTED of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFY_TM_REJECTED = 35;
    public static final int TYPE_NOTIFY_TM_REJECTED = Types.CHAR;
    public static final String NAME_NOTIFY_TM_REJECTED = "PROJECT_DETAILS.NOTIFY_TM_REJECTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTIFY_TM_REJECTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFY_TM_REJECTED,NAME_NOTIFY_TM_REJECTED);}

    /**
     * Column NOTIFY_TM_ACCEPTED of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFY_TM_ACCEPTED = 36;
    public static final int TYPE_NOTIFY_TM_ACCEPTED = Types.CHAR;
    public static final String NAME_NOTIFY_TM_ACCEPTED = "PROJECT_DETAILS.NOTIFY_TM_ACCEPTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTIFY_TM_ACCEPTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFY_TM_ACCEPTED,NAME_NOTIFY_TM_ACCEPTED);}

    /**
     * Column NOTIFY_PM_SUBMITTED of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFY_PM_SUBMITTED = 37;
    public static final int TYPE_NOTIFY_PM_SUBMITTED = Types.CHAR;
    public static final String NAME_NOTIFY_PM_SUBMITTED = "PROJECT_DETAILS.NOTIFY_PM_SUBMITTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTIFY_PM_SUBMITTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFY_PM_SUBMITTED,NAME_NOTIFY_PM_SUBMITTED);}

    /**
     * Column NOTIFY_RM_SUBMITTED of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFY_RM_SUBMITTED = 38;
    public static final int TYPE_NOTIFY_RM_SUBMITTED = Types.CHAR;
    public static final String NAME_NOTIFY_RM_SUBMITTED = "PROJECT_DETAILS.NOTIFY_RM_SUBMITTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTIFY_RM_SUBMITTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFY_RM_SUBMITTED,NAME_NOTIFY_RM_SUBMITTED);}

    /**
     * Column ASSIGNMENT_BILLABLE_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ASSIGNMENT_BILLABLE_FLAG = 39;
    public static final int TYPE_ASSIGNMENT_BILLABLE_FLAG = Types.SMALLINT;
    public static final String NAME_ASSIGNMENT_BILLABLE_FLAG = "PROJECT_DETAILS.ASSIGNMENT_BILLABLE_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ASSIGNMENT_BILLABLE_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGNMENT_BILLABLE_FLAG,NAME_ASSIGNMENT_BILLABLE_FLAG);}

    /**
     * Column NOTIFY_RM_ACCEPTED of type Types.CHAR mapped to String.
     */
    public static final int ID_NOTIFY_RM_ACCEPTED = 40;
    public static final int TYPE_NOTIFY_RM_ACCEPTED = Types.CHAR;
    public static final String NAME_NOTIFY_RM_ACCEPTED = "PROJECT_DETAILS.NOTIFY_RM_ACCEPTED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_NOTIFY_RM_ACCEPTED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_NOTIFY_RM_ACCEPTED,NAME_NOTIFY_RM_ACCEPTED);}

    /**
     * Column SCOPE_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCOPE_SCORE = 41;
    public static final int TYPE_SCOPE_SCORE = Types.BIGINT;
    public static final String NAME_SCOPE_SCORE = "PROJECT_DETAILS.SCOPE_SCORE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCOPE_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_SCORE,NAME_SCOPE_SCORE);}

    /**
     * Column SCOPE_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCOPE_WEIGHT = 42;
    public static final int TYPE_SCOPE_WEIGHT = Types.BIGINT;
    public static final String NAME_SCOPE_WEIGHT = "PROJECT_DETAILS.SCOPE_WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCOPE_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_WEIGHT,NAME_SCOPE_WEIGHT);}

    /**
     * Column SCHEDULE_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHEDULE_SCORE = 43;
    public static final int TYPE_SCHEDULE_SCORE = Types.BIGINT;
    public static final String NAME_SCHEDULE_SCORE = "PROJECT_DETAILS.SCHEDULE_SCORE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCHEDULE_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHEDULE_SCORE,NAME_SCHEDULE_SCORE);}

    /**
     * Column SCHEDULE_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHEDULE_WEIGHT = 44;
    public static final int TYPE_SCHEDULE_WEIGHT = Types.BIGINT;
    public static final String NAME_SCHEDULE_WEIGHT = "PROJECT_DETAILS.SCHEDULE_WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCHEDULE_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHEDULE_WEIGHT,NAME_SCHEDULE_WEIGHT);}

    /**
     * Column QUALITY_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_QUALITY_SCORE = 45;
    public static final int TYPE_QUALITY_SCORE = Types.BIGINT;
    public static final String NAME_QUALITY_SCORE = "PROJECT_DETAILS.QUALITY_SCORE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_QUALITY_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_QUALITY_SCORE,NAME_QUALITY_SCORE);}

    /**
     * Column QUALITY_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_QUALITY_WEIGHT = 46;
    public static final int TYPE_QUALITY_WEIGHT = Types.BIGINT;
    public static final String NAME_QUALITY_WEIGHT = "PROJECT_DETAILS.QUALITY_WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_QUALITY_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_QUALITY_WEIGHT,NAME_QUALITY_WEIGHT);}

    /**
     * Column BENEFIT_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BENEFIT_SCORE = 47;
    public static final int TYPE_BENEFIT_SCORE = Types.BIGINT;
    public static final String NAME_BENEFIT_SCORE = "PROJECT_DETAILS.BENEFIT_SCORE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_BENEFIT_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BENEFIT_SCORE,NAME_BENEFIT_SCORE);}

    /**
     * Column BENEFIT_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BENEFIT_WEIGHT = 48;
    public static final int TYPE_BENEFIT_WEIGHT = Types.BIGINT;
    public static final String NAME_BENEFIT_WEIGHT = "PROJECT_DETAILS.BENEFIT_WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_BENEFIT_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BENEFIT_WEIGHT,NAME_BENEFIT_WEIGHT);}

    /**
     * Column COST_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_COST_SCORE = 49;
    public static final int TYPE_COST_SCORE = Types.BIGINT;
    public static final String NAME_COST_SCORE = "PROJECT_DETAILS.COST_SCORE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_COST_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COST_SCORE,NAME_COST_SCORE);}

    /**
     * Column COST_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_COST_WEIGHT = 50;
    public static final int TYPE_COST_WEIGHT = Types.BIGINT;
    public static final String NAME_COST_WEIGHT = "PROJECT_DETAILS.COST_WEIGHT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_COST_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COST_WEIGHT,NAME_COST_WEIGHT);}

    /**
     * Column LEVEL_LOCK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_LOCK = 51;
    public static final int TYPE_LEVEL_LOCK = Types.INTEGER;
    public static final String NAME_LEVEL_LOCK = "PROJECT_DETAILS.LEVEL_LOCK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_LOCK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_LOCK,NAME_LEVEL_LOCK);}

    /**
     * Column CALCULATE_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CALCULATE_FLAG = 52;
    public static final int TYPE_CALCULATE_FLAG = Types.SMALLINT;
    public static final String NAME_CALCULATE_FLAG = "PROJECT_DETAILS.CALCULATE_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CALCULATE_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CALCULATE_FLAG,NAME_CALCULATE_FLAG);}

    /**
     * Column ROLLUP_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ROLLUP_FLAG = 53;
    public static final int TYPE_ROLLUP_FLAG = Types.SMALLINT;
    public static final String NAME_ROLLUP_FLAG = "PROJECT_DETAILS.ROLLUP_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ROLLUP_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_FLAG,NAME_ROLLUP_FLAG);}

    /**
     * Column COPY_CRI_CONSIDER_ASSIGNMENTS of type Types.CHAR mapped to String.
     */
    public static final int ID_COPY_CRI_CONSIDER_ASSIGNMENTS = 54;
    public static final int TYPE_COPY_CRI_CONSIDER_ASSIGNMENTS = Types.CHAR;
    public static final String NAME_COPY_CRI_CONSIDER_ASSIGNMENTS = "PROJECT_DETAILS.COPY_CRI_CONSIDER_ASSIGNMENTS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_COPY_CRI_CONSIDER_ASSIGNMENTS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COPY_CRI_CONSIDER_ASSIGNMENTS,NAME_COPY_CRI_CONSIDER_ASSIGNMENTS);}

    /**
     * Column COPY_CRI_ADOPT_PARTICIPANTS of type Types.CHAR mapped to String.
     */
    public static final int ID_COPY_CRI_ADOPT_PARTICIPANTS = 55;
    public static final int TYPE_COPY_CRI_ADOPT_PARTICIPANTS = Types.CHAR;
    public static final String NAME_COPY_CRI_ADOPT_PARTICIPANTS = "PROJECT_DETAILS.COPY_CRI_ADOPT_PARTICIPANTS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_COPY_CRI_ADOPT_PARTICIPANTS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COPY_CRI_ADOPT_PARTICIPANTS,NAME_COPY_CRI_ADOPT_PARTICIPANTS);}

    /**
     * Column PROJECT_RTF_BITS2 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROJECT_RTF_BITS2 = 56;
    public static final int TYPE_PROJECT_RTF_BITS2 = Types.INTEGER;
    public static final String NAME_PROJECT_RTF_BITS2 = "PROJECT_DETAILS.PROJECT_RTF_BITS2";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PROJECT_RTF_BITS2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_RTF_BITS2,NAME_PROJECT_RTF_BITS2);}

    /**
     * Column DLV_BUDGET_SETTING_BITS of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_DLV_BUDGET_SETTING_BITS = 57;
    public static final int TYPE_DLV_BUDGET_SETTING_BITS = Types.SMALLINT;
    public static final String NAME_DLV_BUDGET_SETTING_BITS = "PROJECT_DETAILS.DLV_BUDGET_SETTING_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DLV_BUDGET_SETTING_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DLV_BUDGET_SETTING_BITS,NAME_DLV_BUDGET_SETTING_BITS);}

    /**
     * Column OPPORTUNITY_TRACKING_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OPPORTUNITY_TRACKING_NUMBER = 58;
    public static final int TYPE_OPPORTUNITY_TRACKING_NUMBER = Types.VARCHAR;
    public static final String NAME_OPPORTUNITY_TRACKING_NUMBER = "PROJECT_DETAILS.OPPORTUNITY_TRACKING_NUMBER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_OPPORTUNITY_TRACKING_NUMBER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_OPPORTUNITY_TRACKING_NUMBER,NAME_OPPORTUNITY_TRACKING_NUMBER);}

    /**
     * Column OPPORTUNITY_OWNER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OPPORTUNITY_OWNER = 59;
    public static final int TYPE_OPPORTUNITY_OWNER = Types.VARCHAR;
    public static final String NAME_OPPORTUNITY_OWNER = "PROJECT_DETAILS.OPPORTUNITY_OWNER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_OPPORTUNITY_OWNER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_OPPORTUNITY_OWNER,NAME_OPPORTUNITY_OWNER);}

    /**
     * Column CONTRACT_TYPE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTRACT_TYPE = 60;
    public static final int TYPE_CONTRACT_TYPE = Types.VARCHAR;
    public static final String NAME_CONTRACT_TYPE = "PROJECT_DETAILS.CONTRACT_TYPE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CONTRACT_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTRACT_TYPE,NAME_CONTRACT_TYPE);}

    /**
     * Column CONTACT_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTACT_NUMBER = 61;
    public static final int TYPE_CONTACT_NUMBER = Types.VARCHAR;
    public static final String NAME_CONTACT_NUMBER = "PROJECT_DETAILS.CONTACT_NUMBER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CONTACT_NUMBER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTACT_NUMBER,NAME_CONTACT_NUMBER);}

    /**
     * Column CUSTOMER_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CUSTOMER_NUMBER = 62;
    public static final int TYPE_CUSTOMER_NUMBER = Types.VARCHAR;
    public static final String NAME_CUSTOMER_NUMBER = "PROJECT_DETAILS.CUSTOMER_NUMBER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CUSTOMER_NUMBER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CUSTOMER_NUMBER,NAME_CUSTOMER_NUMBER);}

    /**
     * Column OPPORTUNITY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OPPORTUNITY_NAME = 63;
    public static final int TYPE_OPPORTUNITY_NAME = Types.VARCHAR;
    public static final String NAME_OPPORTUNITY_NAME = "PROJECT_DETAILS.OPPORTUNITY_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_OPPORTUNITY_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_OPPORTUNITY_NAME,NAME_OPPORTUNITY_NAME);}

    /**
     * Column EXPECTED_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EXPECTED_COST = 64;
    public static final int TYPE_EXPECTED_COST = Types.DECIMAL;
    public static final String NAME_EXPECTED_COST = "PROJECT_DETAILS.EXPECTED_COST";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_EXPECTED_COST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_COST,NAME_EXPECTED_COST);}

    /**
     * Column EXPECTED_REVENUE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EXPECTED_REVENUE = 65;
    public static final int TYPE_EXPECTED_REVENUE = Types.DECIMAL;
    public static final String NAME_EXPECTED_REVENUE = "PROJECT_DETAILS.EXPECTED_REVENUE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_EXPECTED_REVENUE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_REVENUE,NAME_EXPECTED_REVENUE);}

    /**
     * Column EXPECTED_PAYBACK_PERIOD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPECTED_PAYBACK_PERIOD = 66;
    public static final int TYPE_EXPECTED_PAYBACK_PERIOD = Types.INTEGER;
    public static final String NAME_EXPECTED_PAYBACK_PERIOD = "PROJECT_DETAILS.EXPECTED_PAYBACK_PERIOD";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_EXPECTED_PAYBACK_PERIOD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_PAYBACK_PERIOD,NAME_EXPECTED_PAYBACK_PERIOD);}

    /**
     * Column EXPECTED_PAYBACK_MONTH of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPECTED_PAYBACK_MONTH = 67;
    public static final int TYPE_EXPECTED_PAYBACK_MONTH = Types.INTEGER;
    public static final String NAME_EXPECTED_PAYBACK_MONTH = "PROJECT_DETAILS.EXPECTED_PAYBACK_MONTH";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_EXPECTED_PAYBACK_MONTH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_PAYBACK_MONTH,NAME_EXPECTED_PAYBACK_MONTH);}

    /**
     * Column PURSUIT_START of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_PURSUIT_START = 68;
    public static final int TYPE_PURSUIT_START = Types.DATE;
    public static final String NAME_PURSUIT_START = "PROJECT_DETAILS.PURSUIT_START";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PURSUIT_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PURSUIT_START,NAME_PURSUIT_START);}

    /**
     * Column PURSUIT_FINISH of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_PURSUIT_FINISH = 69;
    public static final int TYPE_PURSUIT_FINISH = Types.DATE;
    public static final String NAME_PURSUIT_FINISH = "PROJECT_DETAILS.PURSUIT_FINISH";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PURSUIT_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PURSUIT_FINISH,NAME_PURSUIT_FINISH);}

    /**
     * Column FORECAST_CLOSED of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FORECAST_CLOSED = 70;
    public static final int TYPE_FORECAST_CLOSED = Types.DATE;
    public static final String NAME_FORECAST_CLOSED = "PROJECT_DETAILS.FORECAST_CLOSED";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_FORECAST_CLOSED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FORECAST_CLOSED,NAME_FORECAST_CLOSED);}

    /**
     * Column ATTACHMENT_VIEW_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ATTACHMENT_VIEW_BITS = 71;
    public static final int TYPE_ATTACHMENT_VIEW_BITS = Types.INTEGER;
    public static final String NAME_ATTACHMENT_VIEW_BITS = "PROJECT_DETAILS.ATTACHMENT_VIEW_BITS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ATTACHMENT_VIEW_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ATTACHMENT_VIEW_BITS,NAME_ATTACHMENT_VIEW_BITS);}

    /**
     * Column LEVEL_LAST_CALL_OPTION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_LAST_CALL_OPTION = 72;
    public static final int TYPE_LEVEL_LAST_CALL_OPTION = Types.INTEGER;
    public static final String NAME_LEVEL_LAST_CALL_OPTION = "PROJECT_DETAILS.LEVEL_LAST_CALL_OPTION";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_LAST_CALL_OPTION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_LAST_CALL_OPTION,NAME_LEVEL_LAST_CALL_OPTION);}

    /**
     * Column SCENARIO_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_SCENARIO_NAME = 73;
    public static final int TYPE_SCENARIO_NAME = Types.CHAR;
    public static final String NAME_SCENARIO_NAME = "PROJECT_DETAILS.SCENARIO_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCENARIO_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCENARIO_NAME,NAME_SCENARIO_NAME);}

    /**
     * Column SCENARIO_CREATOR_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SCENARIO_CREATOR_NAME = 74;
    public static final int TYPE_SCENARIO_CREATOR_NAME = Types.VARCHAR;
    public static final String NAME_SCENARIO_CREATOR_NAME = "PROJECT_DETAILS.SCENARIO_CREATOR_NAME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCENARIO_CREATOR_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCENARIO_CREATOR_NAME,NAME_SCENARIO_CREATOR_NAME);}

    /**
     * Column SCENARIO_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCENARIO_DATETIME = 75;
    public static final int TYPE_SCENARIO_DATETIME = Types.TIMESTAMP;
    public static final String NAME_SCENARIO_DATETIME = "PROJECT_DETAILS.SCENARIO_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCENARIO_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCENARIO_DATETIME,NAME_SCENARIO_DATETIME);}

    /**
     * Column SCENARIO_USER_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SCENARIO_USER_ID = 76;
    public static final int TYPE_SCENARIO_USER_ID = Types.CHAR;
    public static final String NAME_SCENARIO_USER_ID = "PROJECT_DETAILS.SCENARIO_USER_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SCENARIO_USER_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCENARIO_USER_ID,NAME_SCENARIO_USER_ID);}

    /**
     * Column RUP_HOME_PATH of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RUP_HOME_PATH = 77;
    public static final int TYPE_RUP_HOME_PATH = Types.VARCHAR;
    public static final String NAME_RUP_HOME_PATH = "PROJECT_DETAILS.RUP_HOME_PATH";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RUP_HOME_PATH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RUP_HOME_PATH,NAME_RUP_HOME_PATH);}

    /**
     * Column TNG_HOME_PATH of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TNG_HOME_PATH = 78;
    public static final int TYPE_TNG_HOME_PATH = Types.VARCHAR;
    public static final String NAME_TNG_HOME_PATH = "PROJECT_DETAILS.TNG_HOME_PATH";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TNG_HOME_PATH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TNG_HOME_PATH,NAME_TNG_HOME_PATH);}

    /**
     * Column PUBLISH_TO_PJC of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PUBLISH_TO_PJC = 79;
    public static final int TYPE_PUBLISH_TO_PJC = Types.INTEGER;
    public static final String NAME_PUBLISH_TO_PJC = "PROJECT_DETAILS.PUBLISH_TO_PJC";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PUBLISH_TO_PJC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLISH_TO_PJC,NAME_PUBLISH_TO_PJC);}

    private static final String TABLE_NAME = "PROJECT_DETAILS";

    /**
     * Create an array of type string containing all the fields of the PROJECT_DETAILS table.
     */
    private static final String[] FIELD_NAMES =
    {
         NAME_PROJECT_ID
        ,NAME_PROJECT_RTF_BITS
        ,NAME_DELIVERABLE_RTF_BITS
        ,NAME_WORK_PRODUCT_RTF_BITS
        ,NAME_SUMMARY_TASK_RTF_BITS
        ,NAME_MILESTONE_RTF_BITS
        ,NAME_TASK_RTF_BITS
        ,NAME_REQUIREMENT_RTF_BITS
        ,NAME_CHANGE_REQUEST_RTF_BITS
        ,NAME_SERVICE_REQUEST_RTF_BITS
        ,NAME_DEFECT_RTF_BITS
        ,NAME_ISSUE_RTF_BITS
        ,NAME_RISK_RTF_BITS
        ,NAME_CRI_DELIVERABLE_RTF_BITS
        ,NAME_CRI_SUMMARY_TASK_RTF_BITS
        ,NAME_CRI_MILESTONE_RTF_BITS
        ,NAME_CRI_TASK_RTF_BITS
        ,NAME_DOCUMENT_RTF_BITS
        ,NAME_ACTIONITEM_RTF_BITS
        ,NAME_URL_1
        ,NAME_URL_2
        ,NAME_URL_3
        ,NAME_URL_4
        ,NAME_URL_5
        ,NAME_BUDGET_SETTING_BITS
        ,NAME_TIMEPHASE_PROJECT_ASSIGNMENT
        ,NAME_NOTEMIN_RTF_BITS
        ,NAME_SCORECARD_RTF_BITS
        ,NAME_LEVEL_MOVE_PROJ
        ,NAME_LEVEL_START_FINISH_DATE
        ,NAME_ENABLE_EV_CALCULATION
        ,NAME_CHARGEBACK_FORMULA
        ,NAME_ROLLUP_PROJECT_ESTIMATED
        ,NAME_ROLLUP_DELIVERABLE_ESTIMATED
        ,NAME_NOTIFY_TM_REJECTED
        ,NAME_NOTIFY_TM_ACCEPTED
        ,NAME_NOTIFY_PM_SUBMITTED
        ,NAME_NOTIFY_RM_SUBMITTED
        ,NAME_ASSIGNMENT_BILLABLE_FLAG
        ,NAME_NOTIFY_RM_ACCEPTED
        ,NAME_SCOPE_SCORE
        ,NAME_SCOPE_WEIGHT
        ,NAME_SCHEDULE_SCORE
        ,NAME_SCHEDULE_WEIGHT
        ,NAME_QUALITY_SCORE
        ,NAME_QUALITY_WEIGHT
        ,NAME_BENEFIT_SCORE
        ,NAME_BENEFIT_WEIGHT
        ,NAME_COST_SCORE
        ,NAME_COST_WEIGHT
        ,NAME_LEVEL_LOCK
        ,NAME_CALCULATE_FLAG
        ,NAME_ROLLUP_FLAG
        ,NAME_COPY_CRI_CONSIDER_ASSIGNMENTS
        ,NAME_COPY_CRI_ADOPT_PARTICIPANTS
        ,NAME_PROJECT_RTF_BITS2
        ,NAME_DLV_BUDGET_SETTING_BITS
        ,NAME_OPPORTUNITY_TRACKING_NUMBER
        ,NAME_OPPORTUNITY_OWNER
        ,NAME_CONTRACT_TYPE
        ,NAME_CONTACT_NUMBER
        ,NAME_CUSTOMER_NUMBER
        ,NAME_OPPORTUNITY_NAME
        ,NAME_EXPECTED_COST
        ,NAME_EXPECTED_REVENUE
        ,NAME_EXPECTED_PAYBACK_PERIOD
        ,NAME_EXPECTED_PAYBACK_MONTH
        ,NAME_PURSUIT_START
        ,NAME_PURSUIT_FINISH
        ,NAME_FORECAST_CLOSED
        ,NAME_ATTACHMENT_VIEW_BITS
        ,NAME_LEVEL_LAST_CALL_OPTION
        ,NAME_SCENARIO_NAME
        ,NAME_SCENARIO_CREATOR_NAME
        ,NAME_SCENARIO_DATETIME
        ,NAME_SCENARIO_USER_ID
        ,NAME_RUP_HOME_PATH
        ,NAME_TNG_HOME_PATH
        ,NAME_PUBLISH_TO_PJC
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /**
     * Creates a new OpportunityManager instance.
     * @return OpportunityManager
     */
    public OpportunityManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }

    public String getPrimaryKey() {
    //TODO validate the primary key field name.
//        return NAME_PROJECT_ID;
    	return null;
    }
    public String[] getPrimaryKeyArray(RPMObject rpmObject) {

        /* ELEMENT_ID is mapped to the Pool and
         * PARENT_ID is mapped to the PoolAttribute*/
    	//
        return new String[]{NAME_PROJECT_ID,NAME_PROJECT_ID};
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject){
        Opportunity opportunity = (Opportunity)rpmObject;
        String parentId = opportunity.getID();
    	return new Object[]{parentId, parentId};
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject){
    	Opportunity opportunity = (Opportunity)rpmObject;
    	opportunity.setID((String)primaryKey[0]);
    	return opportunity;
    }

    protected Integer getContainerType(ResultSet resultSet) throws RPMException, SQLException {
    	//If container type doesn't apply to this manager
    	return new Integer(0); //TODO validate this
    }

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }

    protected HashSet getValidContainerNames() {
        return CONTAINERS;
    }

    protected RPMObject createContainer(int typeId) throws RPMException {
         return new Opportunity();
     }

    /* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.MessageContext)
	 */
	public void createObject(RPMObject rpmObject, MessageContext messageContext)
			throws SQLException, RPMException, ParseException 
    {

	    Opportunity opportunity = (Opportunity)rpmObject;
	    
	    // Set default field values
	    if (opportunity.getBenefitBeginsMonths() == null)
	       opportunity.setBenefitBeginsMonths(new Integer(0));
	    
	    if (opportunity.getBenefitDurationMonths() == null)
		       opportunity.setBenefitDurationMonths(new Integer(0));
	    
	    if (opportunity.getOrderOfMagnitudeCost() == null)
		       opportunity.setOrderOfMagnitudeCost(new Double(0.00));
	    
	    if (opportunity.getOrderOfMagnitudeRevenueBenefits() == null)
		       opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(0.00));


	    rpmObject.setID(SP_U_WBS_OPPORTUN(opportunity, messageContext));
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.framework.RPMObject, boolean)
	 */
	protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
			boolean isDelta) throws RPMException, SQLException, ParseException 
    {
        Opportunity opportunity = (Opportunity) rpmObject;
        
	    if (isDelta)
        {
	        opportunity.deltaExternalClient(rs.getString(ID_OPPORTUNITY_NAME));
            opportunity.deltaOrderOfMagnitudeCost(new Double(rs
                .getDouble(ID_EXPECTED_COST)));
            opportunity.deltaOrderOfMagnitudeRevenueBenefits(new Double(rs
                .getDouble(ID_EXPECTED_REVENUE)));
            opportunity.deltaBenefitBeginsMonths(new Integer(rs
                .getInt(ID_EXPECTED_PAYBACK_PERIOD)));
            opportunity.deltaBenefitDurationMonths(new Integer(rs
                .getInt(ID_EXPECTED_PAYBACK_MONTH)));
            Date sd = rs.getDate(ID_PURSUIT_START);
            if (sd != null)
            {
                opportunity.deltaPursuitStart(DateUtil.convertToCalendar(sd));
            }
            Date fd = rs.getDate(ID_PURSUIT_FINISH);
            if (fd != null)
            {
                opportunity.deltaPursuitFinish(DateUtil.convertToCalendar(fd));
            }
            Date fcd = rs.getDate(ID_FORECAST_CLOSED);
            if (fcd != null)
            {
                opportunity.deltaForecastClosed(DateUtil.convertToCalendar(fcd));
            }
            opportunity.deltaOpportunityTrackingNumber(rs
                .getString(ID_OPPORTUNITY_TRACKING_NUMBER));
            opportunity.deltaOpportunityOwner(rs.getString(ID_OPPORTUNITY_OWNER));
            opportunity.deltaContactType(rs.getString(ID_CONTRACT_TYPE));
            opportunity.deltaContactNumber(rs.getString(ID_CONTACT_NUMBER));
            opportunity.deltaCustomerNumber(rs.getString(ID_CUSTOMER_NUMBER));
        }
        else
        {
            opportunity.setExternalClient(rs.getString(ID_OPPORTUNITY_NAME));
            opportunity.setOrderOfMagnitudeCost(new Double(rs
                .getDouble(ID_EXPECTED_COST)));
            opportunity.setOrderOfMagnitudeRevenueBenefits(new Double(rs
                .getDouble(ID_EXPECTED_REVENUE)));
            opportunity.setBenefitBeginsMonths(new Integer(rs
                .getInt(ID_EXPECTED_PAYBACK_PERIOD)));
            opportunity.setBenefitDurationMonths(new Integer(rs
                .getInt(ID_EXPECTED_PAYBACK_MONTH)));
            Date sd = rs.getDate(ID_PURSUIT_START);
            if (sd != null)
            {
                opportunity.setPursuitStart(DateUtil.convertToCalendar(sd));
            }
            Date fd = rs.getDate(ID_PURSUIT_FINISH);
            if (fd != null)
            {
                opportunity.setPursuitFinish(DateUtil.convertToCalendar(fd));
            }
            Date fcd = rs.getDate(ID_FORECAST_CLOSED);
            if (fcd != null)
            {
                opportunity.setForecastClosed(DateUtil.convertToCalendar(fcd));
            }
            opportunity.setOpportunityTrackingNumber(rs
                .getString(ID_OPPORTUNITY_TRACKING_NUMBER));
            opportunity.setOpportunityOwner(rs.getString(ID_OPPORTUNITY_OWNER));
            opportunity.setContactType(rs.getString(ID_CONTRACT_TYPE));
            opportunity.setContactNumber(rs.getString(ID_CONTACT_NUMBER));
            opportunity.setCustomerNumber(rs.getString(ID_CUSTOMER_NUMBER));
        }
	    
		return opportunity;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.MessageContext)
	 */
	protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
			throws RPMException, SQLException, ParseException {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, java.lang.String)
	 */
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
			String extraWhere, String contextName) throws RPMException {
		JoinCondition joinCondition = null;
		if (manager instanceof OpportunityManager) {
//			joinCondition = new JoinCondition();
//			//where clause of the join condition
//			StringBuffer condition = new StringBuffer();
//			condition.append(" PROJECT_ID = ?");
//			joinCondition.setCondition(condition.toString());
//			joinCondition = getJoinCondition(manager, condition.toString(), 1,null,null);
		} else if (manager instanceof WbsManager){
			joinCondition = new JoinCondition();
			joinCondition = getJoinCondition(manager, null, 1,null,null);
			joinCondition.setTableName(getTableName());
		}
		return joinCondition;
	}
	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, int, java.lang.String, java.lang.String)
	 */
	protected JoinCondition getJoinCondition(RPMObjectManager manager,
			String condition, int level, String containerName,
			String contextName) throws RPMException {
		// TODO chk if it is right
		JoinCondition joinCondition = null;
        if (manager == null){
	        joinCondition = new JoinCondition();
	        joinCondition.setTableName(getTableName());
	        if (condition != null){
	            joinCondition.setCondition(condition);
            }
        } else if (manager instanceof OpportunityManager){
			joinCondition = new JoinCondition();
	        joinCondition.setTableName(getTableName());
	        if (condition != null){
	            joinCondition.setCondition(condition);
            }
      	} else if (manager instanceof WbsManager) {
			joinCondition = new JoinCondition();
			StringBuffer where = new StringBuffer();
			where.append(" ( ");
			where.append(NAME_PROJECT_ID);
			where.append(" = ? )");
			joinCondition.setCondition(where.toString());
		}
		return joinCondition;
	}
	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getSubTableJoinCondition(com.ibm.rpm.framework.RPMObjectManager, java.lang.String, int, java.lang.String, java.lang.String)
	 */
	protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
			String condition, int level, String containerName,
			String contextName) throws RPMException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope, com.ibm.rpm.framework.MessageContext)
	 */
	public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
			MessageContext messageContext) throws SQLException, RPMException,
			ParseException {
		// TODO Auto-generated method stub
		SP_U_WBS_OPPORTUN(rpmObject, messageContext);
		return rpmObject;
	}


//	public void saveObjectParents(RPMObject rpmObject, RPMObjectScope rpmScope,
//			MessageContext messageContext) throws RPMException, SQLException,
//			ParseException {
//	    WorkElementScope scope = (WorkElementScope) rpmScope;
//		if (scope != null && scope.getParent() != null) {
//			RPMObject parent = rpmObject.getParent();
//			if (parent != null && parent instanceof GenericProject){
//		        WBSManager wbsMgr = (WBSManager)RPMManagerFactory.getInstance().getRPMObjectManager(
//		                Proposal.class.getName());
//		        wbsMgr.internalSave(parent, scope.getParent(), messageContext, false);
//			}
//		}
//	}


	/* (non-Javadoc)
	 * @see com.ibm.rpm.framework.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope, com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.util.FieldValueMap, int, boolean)
	 */
	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext,
			FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {
		String oldParentId = (String) relatedFields.get(index,ID_PROJECT_ID);
		RPMObject newParent = null;
		if (isDelta) {
			newParent = rpmObject.getParent();
			RPMObject oldParent = null;
			if (oldParentId != null && newParent != null) {
			    oldParent = createContainer(newParent.retrieveTypeID().intValue());
				oldParent.setID(oldParentId);
				if (newParent != null && newParent.getID() != null
						&& newParent.getID().equals(oldParentId) ) {
				        WbsManager wbsMgr = (WbsManager)RPMManagerFactory.getInstance().getRPMObjectManager(
				                Proposal.class.getName());
						oldParent = wbsMgr.loadByPrimaryKey(newParent, null, messageContext, true);
				}
			}
			rpmObject.deltaParent(oldParent);
		} else {
			if (oldParentId != null) {
				if (newParent == null) {//newParent not in cache
						newParent = new GenericProject();
						newParent.setID(oldParentId);
				        WbsManager wbsMgr = (WbsManager)RPMManagerFactory.getInstance().getRPMObjectManager(
				                Proposal.class.getName());
						newParent = wbsMgr.loadByPrimaryKey(newParent,null, messageContext, false);
				}
				rpmObject.setParent(newParent);
			}
		}
		return rpmObject;
	}

	protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
			FieldValueMap relatedFields, int index) throws RPMException,
			SQLException {

		String parentId = resultSet.getString(ID_PROJECT_ID);
		relatedFields.put(index, ID_PROJECT_ID, parentId);

		return relatedFields;
	}

	private String SP_U_WBS_OPPORTUN(RPMObject rpmObject,
			MessageContext messageContext) throws SQLException, RPMException {
		String name = "SP_U_WBS_OPPORTUN";
		Opportunity opportunity= (Opportunity) rpmObject;
		Object args[] = new Object[15];
		args[0] = rpmObject.getParent().getID();
	    args[1] = opportunity.getExternalClient();
	    args[2] = opportunity.getOrderOfMagnitudeCost();
	    args[3] = opportunity.getOrderOfMagnitudeRevenueBenefits();
	    args[4] = opportunity.getBenefitBeginsMonths();
	    args[5] = opportunity.getBenefitDurationMonths();
		args[6] = DateUtil.convertToString(opportunity.getPursuitStart());
	    args[7] = DateUtil.convertToString(opportunity.getPursuitFinish());
	    args[8] = DateUtil.convertToString(opportunity.getForecastClosed());
	    args[9] = opportunity.getOpportunityTrackingNumber();
	    args[10] = opportunity.getOpportunityOwner();
	    args[11] = opportunity.getContactType();
	    args[12] = opportunity.getContactNumber();
	    args[13] = opportunity.getCustomerNumber();
	    args[14] = getUser(messageContext).getID();
		return executeProcedure(messageContext, name, args);
	}


}
