package com.ibm.rpm.wbs.managers;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.PreparedStatementHelper;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.xpathparser.XPathContainer;

public class PortfoliosManager extends AbstractRPMObjectManager
{
    //private static Log logger = LogFactory.getLog(PortfoliosManager.class);
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();
    static
    {
        CONTAINERS.add(Portfolio.class.getName());
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    PortfoliosManager.class);
        }
    }

    /**
     * 
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_PORTFOLIOS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}
    /**
     * 
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_PORTFOLIOS.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}
    /**
     * 
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_PORTFOLIOS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}
    /**
     * 
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_PORTFOLIOS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}
    /**
     * 
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_PORTFOLIOS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}
    /**
     * 
     * Column PARENT_ID of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_PARENT_ID = 6;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_PORTFOLIOS.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}
    /**
     * 
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_PORTFOLIOS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * 
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     *  
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_PORTFOLIOS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * 
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_RESOURCE_ID = 9;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "TMT_PORTFOLIOS.RESOURCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}
    /**
     * 
     * Column REC_STATUS of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_PORTFOLIOS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}
    /**
     * 
     * Column RANK of type Types.VARCHAR mapped to String.
     *  
     */
    public static final int ID_RANK = 11;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_PORTFOLIOS.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}
    public static final String TABLE_NAME = "TMT_PORTFOLIOS";

    /**
     * 
     * Create an array of type string containing all the fields of the
     * TMT_PORTFOLIOS table.
     *  
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_PARENT_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_RESOURCE_ID, NAME_REC_STATUS, NAME_RANK
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * 
     * Creates a new PortfoliosManager instance.
     * 
     * @return PortfoliosManager
     *  
     */
    public PortfoliosManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
        //If container type doesn't apply to this manager
        //return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject obj = new Portfolio();
        if (typeId != Portfolio.TYPE_ID)
        {
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return obj;
    }

    /**
     * 
     * @param rpmObject
     * 
     * @return true if the param obj is of type Portfolio.
     *  
     */
    private boolean isPortfolio(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == Portfolio.TYPE_ID;
    }

    /**
     * 
     * @param rpmObject
     * 
     * @return true if the param obj is of type Portfolio.
     *  
     */
    private boolean isGenericProject(RPMObject rpmObject)
    {
        return rpmObject instanceof GenericProject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     *  
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (isPortfolio(rpmObject))
        {
            rpmObject.setID(SP_I_WBS(rpmObject, messageContext));
            if (rpmObject.getParent() != null)
            { // creating under a portfolio
                SP_M_WBS(rpmObject, messageContext);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     *  
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        Portfolio temp = (Portfolio) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }
        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        else
        {
            temp.setName(name);
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        }
        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     *  
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject != null)
        {
            //cascade delete
            //delete portfolio content
            Portfolio portfolio = (Portfolio) rpmObject;
            //delete subportfolios
            Portfolio[] subPorfolios = portfolio.getPortfolios();
            if (subPorfolios != null)
            {
                for (int i = 0; i < subPorfolios.length; i++)
                {
                    try
                    {
                        delete(subPorfolios[i], messageContext);
                    }
                    catch (Exception e)
                    {
                        //do nothing
                    }
                }
            }
            //delete generic projects
            GenericProject[] genericProjects = portfolio.getProjects();
            if (genericProjects != null)
            {
                WbsManager wbsManager = (WbsManager) getManagerInstance(Project.class
                        .getName());
                for (int i = 0; i < genericProjects.length; i++)
                {
                    try
                    {
                        wbsManager.doDelete(genericProjects[i], messageContext);
                    }
                    catch (Exception e)
                    {
                        //do nothing
                    }
                }
            }
            //delete porfolio itself
            SP_D_WBS(rpmObject, messageContext);
        }
    }

    /** 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof PortfoliosManager)
        {
            // For loading: Portfolio.portfolios
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqual(NAME_TYPE_ID, Portfolio.TYPE_ID);
            joinCondition.appendSubSelect(NAME_PARENT_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, NAME_ELEMENT_ID + "=?");
        }
        else if (manager instanceof WbsManager)
        {
            // For loading: GenericProject.portfolio
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID, 
                    NAME_PARENT_ID, TABLE_NAME, NAME_ELEMENT_ID + "=?");
        }
        else if (manager instanceof WbsModuleManager)
        {
            // For loading: WbsModule.portfolios
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqual(NAME_LEVEL_ID, 1);
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            // For ???:
            // For XPath like: /Portfolio
            joinCondition = new JoinCondition(TABLE_NAME); 
            appendFinalCondition(joinCondition, container);
        }
        else if (manager instanceof PortfoliosManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                    NAME_PARENT_ID, NAME_ELEMENT_ID, TABLE_NAME, 
                    null);
        }
        else if (manager instanceof WbsManager)
        { 
            // For XPath like: /Portfolio[<condition>]/projects
            // Equivalent to (same code!): /GenericProject[portfolio[<condition>]]
            joinCondition = JoinCondition.createSubSelect(
			        WbsManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID,
			        TABLE_NAME, null);
            joinCondition.appendWhere();
            joinCondition.appendSubSelect(
                    NAME_PARENT_ID, NAME_ELEMENT_ID, TABLE_NAME, 
			        null);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof WbsManager)
        {
            // For XPath like: /GenericProject[portfolio[<condition>]]
            // Equivalent to (same code!): /Portfolio[<condition>]/projects
            joinCondition = JoinCondition.createSubSelect(
                    WbsManager.NAME_ELEMENT_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, null);
            joinCondition.appendWhere();
            joinCondition.appendSubSelect(
                    NAME_PARENT_ID, NAME_ELEMENT_ID, TABLE_NAME, 
                    null);
        }
        else if (manager instanceof PortfoliosManager)
        {
            // For XPath like: /Portfolio[portfolios[<condition>]]
            joinCondition = JoinCondition.createSubSelect(
                    NAME_ELEMENT_ID, NAME_PARENT_ID, TABLE_NAME, 
                    null);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }
        return joinCondition;
    }
    
    private void appendFinalCondition(JoinCondition joinCondition, 
            XPathContainer container)
    {
        if (container.getTypeId() == null)
        {
            joinCondition.appendCondition(getFilter());
        }
        else
        {
            joinCondition.appendEqual(NAME_TYPE_ID, container.getTypeId());
            joinCondition.appendCondition(getFilter());
        }
    }
    
    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_ELEMENT_ID);
        filter.append(" IS NOT NULL ");

        filter.append(" AND ");

        filter.append(NAME_PARENT_ID);
        filter.append(" IS NOT NULL ");

        filter.append(" AND ");
        
        filter.append(NAME_REC_STATUS);
        filter.append(" != 'D'");

        return filter.toString();
    }
    
    /** 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope scope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        Portfolio temp = (Portfolio) rpmObject;
        if (temp.testNameModified())
        {
            SP_U_WBS_NAME(temp, messageContext);
        }
        if (scope != null)
        {
            rpmObject = updateChildren(rpmObject, (PortfolioScope) scope,
                    messageContext);
        }
        return rpmObject;
    }

    protected RPMObject updateChildren(RPMObject rpmObject,
            PortfolioScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Resolving merge
        if (isPortfolio(rpmObject))
        {
            Portfolio portfolio = (Portfolio) rpmObject;
            // save all children-Portfolio
            if (canSaveChildren(scope.getPortfolios(), portfolio
                    .getPortfolios()))
            {
                Portfolio[] children = (Portfolio[]) saveChildren(portfolio,
                        portfolio.getPortfolios(), scope.getPortfolios(),
                        messageContext);
                portfolio.setPortfolios(children);
            }
            GenericProject[] children = null;
            // save all children-Proposals,WorkOrgz,Projects
            if (canSaveChildren(scope.getProjects(), portfolio.getProjects()))
            {
                children = (GenericProject[]) saveChildren(portfolio, portfolio
                        .getProjects(), scope.getProjects(), messageContext);
                portfolio.setProjects(children);
            }
        }
        return rpmObject;
    }

    private boolean canSaveChildren(RPMObjectScope childrenScope,
            RPMObject[] children)
    {
        return (childrenScope != null && children != null);
    }

    private RPMObject[] saveChildren(RPMObject parent, RPMObject[] children,
            RPMObjectScope childrenScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ArrayList temp = new ArrayList();
        WbsManager wbsMgr = (WbsManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(Proposal.class.getName());//performance..
        // moved it out
        // of for loop.
        for (int i = 0; i < children.length; i++)
        {
            if (children[i] != null)
            {
                if (isGenericProject(children[i]))
                {
                    ((GenericProject) children[i])
                            .setPortfolio((Portfolio) parent);
                    wbsMgr.internalSave(children[i], childrenScope,
                            messageContext);
                }
                else
                {
                    children[i].setParent(parent);
                    internalSave(children[i], childrenScope, messageContext);
                }
                temp.add(children[i]);
            }
        }
        if (temp.size() > 0)
        {
            children = instantiateChildrenArray((RPMObject) temp.get(0), temp
                    .size());
            temp.toArray(children);
        }
        return children;
    }

    private RPMObject[] instantiateChildrenArray(RPMObject child, int length)
    {
        if (isPortfolio(child))
        {
            return new Portfolio[length];
        }
        else if (isGenericProject(child))
        {
            return new GenericProject[length];
        }
        return null;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        PortfolioScope scope = (PortfolioScope) rpmScope;
        if (scope != null)
        {
            //Parent
            if (scope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                        ID_PARENT_ID);
                RPMObject newParent = null;
                if (isDelta)
                {
                    newParent = rpmObject.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new Portfolio();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = loadByPrimaryKey(newParent, scope
                                    .getParent(), messageContext, true);
                        }
                        else
                        {
                            //In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldParent = loadByPrimaryKey(oldParent, null,
                                    messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        if (newParent == null)
                        {//newParent not in cache
                            newParent = new Portfolio();
                            newParent.setID(oldParentId);
                            newParent = loadByPrimaryKey(newParent, scope
                                    .getParent(), messageContext, false);
                        }
                        rpmObject.setParent(newParent);
                    }
                }
            }
            //load children
            if (isPortfolio(rpmObject))
            {
                Portfolio portfolio = (Portfolio) rpmObject;
                if (scope.getPortfolios() != null)
                {
                    loadPortfolios(portfolio, portfolio.getPortfolios(), scope
                            .getPortfolios(), messageContext, isDelta);
                }
                if (scope.getProjects() != null)
                {
                    int types[] = new int[]
                    {
                            Project.TYPE_ID, WorkOrganization.TYPE_ID,
                            Proposal.TYPE_ID
                    };
                    loadGenericProjects(portfolio, portfolio.getProjects(),
                            scope.getProjects(), messageContext, types, isDelta);
                }
            }
        }
        return rpmObject;
    }

    private void loadGenericProjects(Portfolio portfolio,
            GenericProject[] oldChildren, WorkElementScope scope,
            MessageContext messageContext, int[] typeIds, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WbsManager wbsManager = (WbsManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(Proposal.class.getName());
        ArrayList temp = wbsManager.loadByForeignKey(oldChildren, scope,
                messageContext, this, new Object[]
                {
                    portfolio.getID()
                }, null, null, isDelta);
        if (temp != null && !temp.isEmpty())
        {
            GenericProject[] children = new GenericProject[temp.size()];
            temp.toArray(children);
            for (int i = 0; i < children.length; ++i)
            {
                children[i].setPortfolio(portfolio);
            }
            portfolio.setProjects(children);
        }
    }

    private void loadPortfolios(Portfolio portfolio, Portfolio[] oldChildren,
            RPMObjectScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
                this, new Object[]
                {
                    portfolio.getID()
                }, null, null, isDelta);
        if (temp != null)
        {
            Portfolio[] children = new Portfolio[temp.size()];
            temp.toArray(children);
            for (int i = 0; i < children.length; ++i)
            {
                children[i].setParent(portfolio);
            }
            portfolio.setPortfolios(children);
        }
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);
        return relatedFields;
    }

    private String SP_I_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_I_WBS";
        Object args[] = new Object[4];
        args[0] = ((Portfolio) rpmObject).getName();
        if (rpmObject.getParent() != null)
        {
            args[1] = rpmObject.getParent().getID();
        }
        args[2] = rpmObject.retrieveTypeID();
        args[3] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private void SP_M_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_M_WBS";
        Object args[] = new Object[4];
        args[0] = rpmObject.getParent().getID(); //this must exist
        args[1] = ((Portfolio) rpmObject).getID();
        /*
         * p_bitflag: the first bit used to indicate moving operation (moving as
         * child (0) or sibling move (1))
         * 
         * second bit used to indicate which object will be moved (WBS (0) or
         * PORTFOLIO (1))
         * 
         * third bit used to indicate varify external dependencies.
         */
        args[2] = new Integer(6);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_D_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_WBS";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        if (rpmObject.getParent() != null)
        {
            args[1] = rpmObject.getParent().getID();
        }
        else
        {
            args[1] = getUser(messageContext).getID().trim();
        }
        args[2] = "X";
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID().trim();
        }
        executeProcedure(messageContext, name, args);
    }

    private String SP_U_WBS_NAME(Portfolio portfolio,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_WBS_NAME";
        Object args[] = new Object[3];
        args[0] = portfolio.getID();
        args[1] = portfolio.getName();
        args[2] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }
    
    protected Object loadWithSecurity(MessageContext messageContext, Connection c,
            PreparedStatement ps, String selectStmtStr, Object[] argValues, String contextName)
            throws SQLException, RPMException
    {
        SqlBuffer newSelect = new SqlBuffer();
        int indexOf = selectStmtStr.indexOf("WITH UR");
        if(indexOf!=-1)
        {
            newSelect.append(selectStmtStr.substring(0, indexOf));
        }
        else
        {
            newSelect = new SqlBuffer(selectStmtStr);
        }
        
        Resource currentUser = (Resource)messageContext.getUser();
        
        newSelect.appendEqual(NAME_PARENT_ID, currentUser.getID());
        newSelect.append(" WITH UR");

        ResultSet rs = ManagerUtil.executeSqlQuery(messageContext, ps,
            newSelect.toString(), argValues);
        return rs;
    }
    protected Object loadWithSecurity(MessageContext messageContext, Connection c,
            String selectStmtStr, Object[] argValues, String contextName)
            throws SQLException, RPMException
    {       
    	SqlBuffer newSelect = new SqlBuffer();
    	int indexOf = selectStmtStr.indexOf("WITH UR");
    	if (indexOf != -1) {
    		newSelect.append(selectStmtStr.substring(0, indexOf));
    	} else {
    		newSelect = new SqlBuffer(selectStmtStr);
    	}

    	Resource currentUser = (Resource) messageContext.getUser();

    	StringBuffer subCondition = new StringBuffer();
    	subCondition.append(NAME_ELEMENT_ID + "= '" + currentUser.getID() + "'");
    	subCondition.append(" AND ");
    	subCondition.append(NAME_REC_STATUS + " != 'D'");
    	subCondition.append(" AND ");
    	subCondition.append(NAME_TYPE_ID + "=" + Resource.TYPE_ID);
    	newSelect.appendSubSelect("substr(" + NAME_RANK +", 1, 8)", NAME_RANK, getTableName(), subCondition.toString());
    	newSelect.appendCloseParenthesis();
    	
    	//Add "WITH UR" if it was found in the original query string.
    	if (indexOf != -1) {
    		newSelect.append(" WITH UR");
    	}
    	
    	PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
    			messageContext, c);
    	PreparedStatement ps = preparedStatementHelper.createPreparedStatement(
    			newSelect.toString(), argValues, null, useDBStatementsForLOBS);


    	ResultSet rs = ManagerUtil.executeSqlQuery(messageContext, ps, newSelect
    			.toString(), null);
        return rs;
    }
}
