package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Country;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.managers.GeographicalManager;
import com.ibm.rpm.applicationadministration.managers.OrganizationalManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.ResourceAttributeManager;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ProfileScope;


public class ProfileManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {

        CONTAINERS.add(Profile.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ProfileManager.class);
        }
    }
    
    
    /**
     * The following is taken from SP code (pmo_cnst.h)
     */
    //  PROJECT TYPES
//    private static int SC_PROJECT  = 1;
//    private static int SC_PROPOSAL = 154;
//    //     WBS deliverable types
//    private static int SC_DELIVERABLE = 2;
//    private static int SC_WBS_CHANGE_REQUEST = 36;
//    private static int SC_WBS_RISK = 37;
//    private static int SC_WBS_ISSUE = 39;
//    private static int SC_WORK_PRODUCT = 58;
//    private static int SC_SUMMARY_TASK = 60;
//    private static int SC_WBS_REQUIREMENT = 225;
//    private static int SC_WBS_DEFECT = 230;
//    private static int SC_WBS_SERVICE_REQUEST = 231;
//    private static int SC_WBS_ACTION_ITEM = 259;
//    #define SC_DELIVERABLE_TYPE1                      291           //Life Cycle
//    #define SC_DELIVERABLE_TYPE2                      292
//    #define SC_DELIVERABLE_TYPE3                      293
//
////     WBS task types
//    private static int SC_TASK = 3;
//    private static int SC_MILESTONE = 59;
//    #define SC_WBS_CHANGE_REQUEST_TASK                236
//    #define SC_WBS_RISK_TASK                          237
//    #define SC_WBS_ISSUE_TASK                         238
//    #define SC_WBS_DEFECT_TASK                        239
//    #define SC_WBS_SERVICE_REQUEST_TASK               240
//    #define SC_WBS_REQUIREMENT_TASK                   241
//    #define SC_WBS_ACTION_ITEM_TASK                   26

    /**
     * Column PROFILE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROFILE_ID = 1;

    public static final int TYPE_PROFILE_ID = Types.CHAR;

    public static final String NAME_PROFILE_ID = "PROFILES.PROFILE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROFILE_ID = "ID";
    static{ 
        FIELDPROPERTYMAP.put(PROPERTY_PROFILE_ID,NAME_PROFILE_ID);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 2;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROFILES.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ROLE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ROLE_ID = 3;

    public static final int TYPE_ROLE_ID = Types.CHAR;

    public static final String NAME_ROLE_ID = "PROFILES.ROLE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLE_ID,NAME_ROLE_ID);}

    /**
     * Column PROF_NUMBER of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROF_NUMBER = 4;

    public static final int TYPE_PROF_NUMBER = Types.INTEGER;

    public static final String NAME_PROF_NUMBER = "PROFILES.PROF_NUMBER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROF_NUMBER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROF_NUMBER,NAME_PROF_NUMBER);}

    /**
     * Column TEMPLATE of type Types.CHAR mapped to String.
     */
    public static final int ID_TEMPLATE = 5;

    public static final int TYPE_TEMPLATE = Types.CHAR;

    public static final String NAME_TEMPLATE = "PROFILES.TEMPLATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TEMPLATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TEMPLATE,NAME_TEMPLATE);}

    /**
     * Column PROFILE_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROFILE_NAME = 6;

    public static final int TYPE_PROFILE_NAME = Types.VARCHAR;

    public static final String NAME_PROFILE_NAME = "PROFILES.PROFILE_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROFILE_NAME = "PROFILENAME";
    static{ 
        FIELDPROPERTYMAP.put(PROPERTY_PROFILE_NAME,NAME_PROFILE_NAME);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 7;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "PROFILES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column ORG_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ORG_ID = 8;

    public static final int TYPE_ORG_ID = Types.CHAR;

    public static final String NAME_ORG_ID = "PROFILES.ORG_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ORG_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ORG_ID,NAME_ORG_ID);}

    /**
     * Column GEO_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_GEO_ID = 9;

    public static final int TYPE_GEO_ID = Types.CHAR;

    public static final String NAME_GEO_ID = "PROFILES.GEO_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_GEO_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_GEO_ID,NAME_GEO_ID);}

    /**
     * Column POOL_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_POOL_ID = 10;

    public static final int TYPE_POOL_ID = Types.CHAR;

    public static final String NAME_POOL_ID = "PROFILES.POOL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_POOL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_POOL_ID,NAME_POOL_ID);}

    /**
     * Column STAFFING_STATE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_STAFFING_STATE = 11;

    public static final int TYPE_STAFFING_STATE = Types.SMALLINT;

    public static final String NAME_STAFFING_STATE = "PROFILES.STAFFING_STATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STAFFING_STATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_STAFFING_STATE,NAME_STAFFING_STATE);}

    public static final String TABLE_NAME = "PROFILES";

    /**
     * Create an array of type string containing all the fields of the PROFILES
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROFILE_ID, NAME_PROJECT_ID, NAME_ROLE_ID, NAME_PROF_NUMBER,
            NAME_TEMPLATE, NAME_PROFILE_NAME, NAME_REC_STATUS, NAME_ORG_ID,
            NAME_GEO_ID, NAME_POOL_ID, NAME_STAFFING_STATE
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new ProfileManager instance.
     * 
     * @return ProfileManager
     */
    public ProfileManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_PROFILE_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new Profile();
    }

    /**
     *  
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        Profile profile = (Profile) rpmObject;
        profile.setID(rs.getString(ID_PROFILE_ID));
        String name = rs.getString(ID_PROFILE_NAME);

        if (name != null)
        {
            name = name.trim();
        }
        if (isDelta)
        {
            profile.deltaProfileName(name);
        }
        else
        {
            profile.setProfileName(name);
        }
        
        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PROFILE_ID);
        relatedFields.put(index, ID_PROFILE_ID, parentId);

        String geographicalId = resultSet.getString(ID_GEO_ID);
        relatedFields.put(index, ID_GEO_ID, geographicalId);

        String organizationalId = resultSet.getString(ID_ORG_ID);
        relatedFields.put(index, ID_ORG_ID, organizationalId);

        String poolId = resultSet.getString(ID_POOL_ID);
        relatedFields.put(index, ID_POOL_ID, poolId);

        String competencyId = resultSet.getString(ID_ROLE_ID);
        relatedFields.put(index, ID_ROLE_ID, competencyId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {
            Profile profile = (Profile) rpmObject;
            ProfileScope scope = (ProfileScope) rpmScope;

            //Parent
            if (scope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                    ID_PROFILE_ID);
                RPMObject newParent = null;

                ResourceTaskAssignmentManager parentManager = 
                    (ResourceTaskAssignmentManager) getManagerInstance(ResourceTaskAssignment.class
                    .getName());

                if (isDelta)
                {
                    newParent = rpmObject.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null)
                    {
                        oldParent = new ResourceTaskAssignment();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = parentManager.loadByPrimaryKey(
                                newParent, scope.getParent(), messageContext,
                                true);
                        }
                        else
                        {
                            //In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldParent = parentManager.loadByPrimaryKey(
                                oldParent, null, messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null)
                    {
                        ArrayList temp = parentManager.loadByForeignKey(null,
                            scope.getParent(), messageContext, this,
                            new Object[]
                            {
                                profile.getID()
                            }, null, null, isDelta);
                        rpmObject.setParent((ResourceTaskAssignment) temp
                            .get(0));
                    }
                }
            }

            if (scope.getGeographical() != null)
            {
                loadGeographical(profile, scope, messageContext,
                    relatedFields, index, isDelta);
            }

            if (scope.getOrganizational() != null)
            {
                loadOrganizational(profile, scope, messageContext,
                    relatedFields, index, isDelta);
            }

            if (scope.getPool() != null)
            {
                loadPool(profile, scope, messageContext, relatedFields,
                    index, isDelta);
            }

            if (scope.isProfileCompetencyAssignments())
            {
                loadCompetencyAssignments(profile, scope, messageContext,
                    relatedFields, index, isDelta);
            }

            if (scope.isProfileSkillAssignments())
            {
                loadSkillAssignments(profile, scope, messageContext,
                    relatedFields, index, isDelta);
            }
        }
        
        return rpmObject;
    }

    private void loadGeographical(Profile profile, ProfileScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        
        Geographical geo = new Geographical();
        geo.setID((String) relatedFields.get(index, ID_GEO_ID));

        GeographicalManager manager = (GeographicalManager) getManagerInstance(Country.class
            .getName());
        geo = (Geographical) manager.loadByPrimaryKey(geo, scope
            .getGeographical(), messageContext, isDelta);

        if (isDelta)
        {
            profile.deltaGeographical(geo);
        }
        else
        {
            profile.setGeographical(geo);
        }
    }

    private void loadOrganizational(Profile profile, ProfileScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        Organizational org = new Organizational();
        org.setID((String) relatedFields.get(index, ID_ORG_ID));

        OrganizationalManager manager = (OrganizationalManager) getManagerInstance(Organization.class
            .getName());
        org = (Organizational) manager.loadByPrimaryKey(org, scope
            .getOrganizational(), messageContext, isDelta);

        if (isDelta)
        {
            profile.deltaOrganizational(org);
        }
        else
        {
            profile.setOrganizational(org);
        }
    }

    private void loadPool(Profile profile, ProfileScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        Pool pool = new Pool();
        pool.setID((String) relatedFields.get(index, ID_POOL_ID));

        PoolManager manager = (PoolManager) getManagerInstance(Pool.class
            .getName());
        pool = (Pool) manager.loadByPrimaryKey(pool, scope.getPool(),
            messageContext, isDelta);

        if (isDelta)
        {
            profile.deltaPool(pool);
        }
        else
        {
            profile.setPool(pool);
        }
    }

        private void loadCompetencyAssignments(Profile profile, ProfileScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        //Load the single competencyAssignment from PROFILES table
        //and build a ProfileCompetencyAssignment from a competency
        String proLevelID = (String) relatedFields.get(index, ID_ROLE_ID);
        ProfileCompetencyAssignment assignment = 
            new ProfileCompetencyAssignment();
        if (proLevelID != null)
        {
            ResourceAttributeManager RAManager = (ResourceAttributeManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(
                    ResourceAttributeCompetencyProficiencyLevel.class.getName());
            
            ResourceAttributeCompetencyProficiencyLevel proLevel = 
                new ResourceAttributeCompetencyProficiencyLevel();
            
            proLevel.setID(proLevelID);
            
            proLevel = 
                (ResourceAttributeCompetencyProficiencyLevel)RAManager.
                loadByPrimaryKey(proLevel, null, messageContext, isDelta);
            
            assignment.setID(profile.getID());
            assignment.setParent(profile);
            if(isDelta)
            {
                assignment.deltaProficiencyLevel(proLevel);
            }
            else
            {
                assignment.setProficiencyLevel(proLevel);
            }
        }

        //Load the others from RES_ROLES_ASS
        ProfileCompetencyAssignmentManager manager = (ProfileCompetencyAssignmentManager) getManagerInstance(ProfileCompetencyAssignment.class
            .getName());
        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(profile,
            profile.getProfileCompetencyAssignments(), null, messageContext,
            this, new Object[]
            {
                profile.getID()
            }, null, null, isDelta);

        if(temp == null)
        {
            temp = new ArrayList();
        }
        
        temp.add(assignment);
        if (temp != null && temp.size() > 0)
        {
            ProfileCompetencyAssignment[] competencies = new ProfileCompetencyAssignment[temp
                .size()];
            temp.toArray(competencies);
            profile.setProfileCompetencyAssignments(competencies);
        }
    }
        
        private void loadSkillAssignments(Profile profile, ProfileScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        ProfileSkillAssignmentManager manager = (ProfileSkillAssignmentManager) getManagerInstance(ProfileSkillAssignment.class
            .getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(profile,
            profile.getProfileSkillAssignments(), null, messageContext, this,
            new Object[]
            {
                profile.getID()
            }, null, null, isDelta);

        if (temp != null)
        {
            ProfileSkillAssignment[] skills = new ProfileSkillAssignment[temp
                .size()];
            temp.toArray(skills);
            profile.setProfileSkillAssignments(skills);
        }
    }



    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        // TODO Implement if the supported containers can be loaded
        // as children by another manager
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return joinCondition;
    }

    /**
     *  
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            // example: /Proposal[ID='123']/ResourceTaskAssignment/Profile
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }
        } 
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
        
    }

    /**
     *  
     */
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof ResourceTaskAssignmentManager)
        {
            // example: /ResourceTaskAssignment[Profile[ID='123']]
            // NOTE: NAME_RESOURCE_ID is used instead of NAME_PROFILE_ID
            // because they are very evil in DB team...
            joinCondition = JoinCondition.createSubSelect(
                    ResourceTaskAssignmentManager.NAME_RESOURCE_ID, 
                    NAME_PROFILE_ID,
                    TABLE_NAME, condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;        
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {        
        throw new RPMException(ErrorCodes.CANNOT_SAVE_PROFILES_DIRECTLY);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {        
        return null;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject.getParent() != null)
        {
            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceTaskAssignment.class.getName());
            manager.delete(rpmObject.getParent(), messageContext);
        }
    }
    
	public static String retrieveCompetenciesID(Profile profile)
	{
        StringBuffer rolesID = new StringBuffer();
        
        ProfileCompetencyAssignment[] competencies = profile
            .getProfileCompetencyAssignments();
        if(competencies != null)
        {
            for(int i=0; i < competencies.length; i++)
            {
                ResourceAttributeProficiencyLevel p = competencies[i]
                    .getProficiencyLevel();
                if(p != null && p.getID() != null)
                {
                    rolesID.append(p.getID());
                }
            }
        }
        
        if (rolesID.toString().trim().length() > 0)
        {
            return rolesID.toString();	    
        }
        
        return null;
	}
	
	public static String retrieveSkillsID(Profile profile)
    {
        ProfileSkillAssignment[] skills = profile.getProfileSkillAssignments();
        StringBuffer skillsID = new StringBuffer();
        if (skills != null)
        {
            for (int i = 0; i < skills.length; i++)
            {
                ResourceAttributeProficiencyLevel p = skills[i]
                    .getProficiencyLevel();
                if (p != null && p.getID() != null)
                {
                    skillsID.append(p.getID());
                }
            }
        }
        
        if (skillsID.toString().trim().length() > 0)
        {
            return skillsID.toString();
        }
        
        return null;
    }		

}
