package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.managers.ContractManager;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.managers.ChargeCodeManager;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.managers.TempReqProOverride;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.wbs.checkpoints.WbsMethodCheckpoint;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.xpathparser.XPathContainer;

public class TaskAssignmentManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(TaskAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                TaskAssignmentManager.class);
        }
    }

    /**
     * Column TSK_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TSK_ASSIGNMENT_ID = 1;

    public static final int TYPE_TSK_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_TSK_ASSIGNMENT_ID = "TASK_ASSIGNMENTS.TSK_ASSIGNMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TSK_ASSIGNMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_TSK_ASSIGNMENT_ID, NAME_TSK_ASSIGNMENT_ID);
    }

    /**
     * Column RES_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RES_ASSIGNMENT_ID = 2;

    public static final int TYPE_RES_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_RES_ASSIGNMENT_ID = "TASK_ASSIGNMENTS.RES_ASSIGNMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RES_ASSIGNMENT_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_RES_ASSIGNMENT_ID,NAME_RES_ASSIGNMENT_ID);}

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 3;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "TASK_ASSIGNMENTS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column TASK_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TASK_ID = 4;

    public static final int TYPE_TASK_ID = Types.CHAR;

    public static final String NAME_TASK_ID = "TASK_ASSIGNMENTS.TASK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TASK_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TASK_ID,NAME_TASK_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 5;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "TASK_ASSIGNMENTS.RESOURCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column RESOURCE_TYPE of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_TYPE = 6;

    public static final int TYPE_RESOURCE_TYPE = Types.CHAR;

    public static final String NAME_RESOURCE_TYPE = "TASK_ASSIGNMENTS.RESOURCE_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_TYPE,NAME_RESOURCE_TYPE);}

    /**
     * Column ACCEPTED_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_ACCEPTED_DATE = 7;

    public static final int TYPE_ACCEPTED_DATE = Types.DATE;

    public static final String NAME_ACCEPTED_DATE = "TASK_ASSIGNMENTS.ACCEPTED_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACCEPTED_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACCEPTED_DATE,NAME_ACCEPTED_DATE);}

    /**
     * Column ESTIMATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ESTIMATE = 8;

    public static final int TYPE_ESTIMATE = Types.INTEGER;

    public static final String NAME_ESTIMATE = "TASK_ASSIGNMENTS.ESTIMATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ESTIMATE,NAME_ESTIMATE);}

    /**
     * Column BEST of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BEST = 9;

    public static final int TYPE_BEST = Types.INTEGER;

    public static final String NAME_BEST = "TASK_ASSIGNMENTS.BEST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BEST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BEST,NAME_BEST);}

    /**
     * Column BIT_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_BIT_FLAG = 10;

    public static final int TYPE_BIT_FLAG = Types.CHAR;

    public static final String NAME_BIT_FLAG = "TASK_ASSIGNMENTS.BIT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BIT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BIT_FLAG,NAME_BIT_FLAG);}

    /**
     * Column ASSIGN_STRING of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSIGN_STRING = 11;

    public static final int TYPE_ASSIGN_STRING = Types.VARCHAR;

    public static final String NAME_ASSIGN_STRING = "TASK_ASSIGNMENTS.ASSIGN_STRING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_STRING = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_STRING,NAME_ASSIGN_STRING);}

    /**
     * Column SD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SD = 12;

    public static final int TYPE_SD = Types.SMALLINT;

    public static final String NAME_SD = "TASK_ASSIGNMENTS.SD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SD,NAME_SD);}

    /**
     * Column FD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FD = 13;

    public static final int TYPE_FD = Types.SMALLINT;

    public static final String NAME_FD = "TASK_ASSIGNMENTS.FD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FD,NAME_FD);}

    /**
     * Column START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_START_DT = 14;

    public static final int TYPE_START_DT = Types.TIMESTAMP;

    // This custom DB function returns the date portion of a timestamp field
    // It was written for use in both DB2 and Oracle
    public static final String NAME_START_DT = "TASK_ASSIGNMENTS.START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_START_DT = "ASSIGNMENTSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_START_DT, SqlUtil
            .convertTimestampToDate(NAME_START_DT));
    }

    /**
     * Column FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FINISH_DT = 15;

    public static final int TYPE_FINISH_DT = Types.TIMESTAMP;

    // This custom DB function returns the date portion of a timestamp field
    // It was written for use in both DB2 and Oracle
    public static final String NAME_FINISH_DT = "TASK_ASSIGNMENTS.FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_FINISH_DT = "ASSIGNMENTFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FINISH_DT, SqlUtil
            .convertTimestampToDate(NAME_FINISH_DT));
    }

    /**
     * Column DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DURATION = 16;

    public static final int TYPE_DURATION = Types.INTEGER;

    public static final String NAME_DURATION = "TASK_ASSIGNMENTS.DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DURATION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DURATION,NAME_DURATION);}

    /**
     * Column REGULAR_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_REGULAR_COST = 17;

    public static final int TYPE_REGULAR_COST = Types.DECIMAL;

    public static final String NAME_REGULAR_COST = "TASK_ASSIGNMENTS.REGULAR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REGULAR_COST = "COSTPERHOUR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REGULAR_COST, NAME_REGULAR_COST);
    }

    /**
     * Column SPECIAL_COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_COST = 18;

    public static final int TYPE_SPECIAL_COST = Types.DECIMAL;

    public static final String NAME_SPECIAL_COST = "TASK_ASSIGNMENTS.SPECIAL_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_COST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_COST,NAME_SPECIAL_COST);}

    /**
     * Column REGULAR_RATE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_REGULAR_RATE = 19;

    public static final int TYPE_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_REGULAR_RATE = "TASK_ASSIGNMENTS.REGULAR_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_REGULAR_RATE = "SELLINGRATEPERHOUR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REGULAR_RATE, NAME_REGULAR_RATE);
    }

    /**
     * Column SPECIAL_RATE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_RATE = 20;

    public static final int TYPE_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_SPECIAL_RATE = "TASK_ASSIGNMENTS.SPECIAL_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_SPECIAL_RATE = "SPECIALSELLINGRATEPERHOUR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_RATE, NAME_SPECIAL_RATE);
    }

    /**
     * Column BASELINE1_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_START = 21;

    public static final int TYPE_BASELINE1_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_START = "TASK_ASSIGNMENTS.BASELINE1_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_BASELINE1_START = "BASELINESTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_START, NAME_BASELINE1_START);
    }

    /**
     * Column BASELINE1_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_FINISH = 22;

    public static final int TYPE_BASELINE1_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_FINISH = "TASK_ASSIGNMENTS.BASELINE1_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_BASELINE1_FINISH = "BASELINEFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_FINISH, NAME_BASELINE1_FINISH);
    }

    /**
     * Column BASELINE1_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_DURATION = 23;

    public static final int TYPE_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE1_DURATION = "TASK_ASSIGNMENTS.BASELINE1_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_DURATION,NAME_BASELINE1_DURATION);}

    /**
     * Column BASELINE1_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_WORK = 24;

    public static final int TYPE_BASELINE1_WORK = Types.INTEGER;

    public static final String NAME_BASELINE1_WORK = "TASK_ASSIGNMENTS.BASELINE1_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_WORK = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_WORK,NAME_BASELINE1_WORK);}

    /**
     * Column BASELINE1_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE1_LABOR_COST = 25;

    public static final int TYPE_BASELINE1_LABOR_COST = Types.DECIMAL;

    public static final String NAME_BASELINE1_LABOR_COST = "TASK_ASSIGNMENTS.BASELINE1_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_LABOR_COST =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_LABOR_COST,NAME_BASELINE1_LABOR_COST);}

    /**
     * Column BASELINE1_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE1_LABOR_REVENUE = 26;

    public static final int TYPE_BASELINE1_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_BASELINE1_LABOR_REVENUE = "TASK_ASSIGNMENTS.BASELINE1_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_LABOR_REVENUE,NAME_BASELINE1_LABOR_REVENUE);}

    /**
     * Column BASELINE2_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_START = 27;

    public static final int TYPE_BASELINE2_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_START = "TASK_ASSIGNMENTS.BASELINE2_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_START,NAME_BASELINE2_START);}

    /**
     * Column BASELINE2_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_FINISH = 28;

    public static final int TYPE_BASELINE2_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_FINISH = "TASK_ASSIGNMENTS.BASELINE2_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_FINISH,NAME_BASELINE2_FINISH);}

    /**
     * Column BASELINE2_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_DURATION = 29;

    public static final int TYPE_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE2_DURATION = "TASK_ASSIGNMENTS.BASELINE2_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_DURATION,NAME_BASELINE2_DURATION);}

    /**
     * Column BASELINE2_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_WORK = 30;

    public static final int TYPE_BASELINE2_WORK = Types.INTEGER;

    public static final String NAME_BASELINE2_WORK = "TASK_ASSIGNMENTS.BASELINE2_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_WORK = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_WORK,NAME_BASELINE2_WORK);}

    /**
     * Column BASELINE2_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE2_LABOR_COST = 31;

    public static final int TYPE_BASELINE2_LABOR_COST = Types.DECIMAL;

    public static final String NAME_BASELINE2_LABOR_COST = "TASK_ASSIGNMENTS.BASELINE2_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_LABOR_COST =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_LABOR_COST,NAME_BASELINE2_LABOR_COST);}

    /**
     * Column BASELINE2_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE2_LABOR_REVENUE = 32;

    public static final int TYPE_BASELINE2_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_BASELINE2_LABOR_REVENUE = "TASK_ASSIGNMENTS.BASELINE2_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_LABOR_REVENUE,NAME_BASELINE2_LABOR_REVENUE);}

    /**
     * Column BASELINE3_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_START = 33;

    public static final int TYPE_BASELINE3_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_START = "TASK_ASSIGNMENTS.BASELINE3_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_START,NAME_BASELINE3_START);}

    /**
     * Column BASELINE3_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_FINISH = 34;

    public static final int TYPE_BASELINE3_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_FINISH = "TASK_ASSIGNMENTS.BASELINE3_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_FINISH,NAME_BASELINE3_FINISH);}

    /**
     * Column BASELINE3_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_DURATION = 35;

    public static final int TYPE_BASELINE3_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE3_DURATION = "TASK_ASSIGNMENTS.BASELINE3_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_DURATION,NAME_BASELINE3_DURATION);}

    /**
     * Column BASELINE3_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_WORK = 36;

    public static final int TYPE_BASELINE3_WORK = Types.INTEGER;

    public static final String NAME_BASELINE3_WORK = "TASK_ASSIGNMENTS.BASELINE3_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_WORK = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_WORK,NAME_BASELINE3_WORK);}

    /**
     * Column BASELINE3_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE3_LABOR_COST = 37;

    public static final int TYPE_BASELINE3_LABOR_COST = Types.DECIMAL;

    public static final String NAME_BASELINE3_LABOR_COST = "TASK_ASSIGNMENTS.BASELINE3_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_LABOR_COST =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_LABOR_COST,NAME_BASELINE3_LABOR_COST);}

    /**
     * Column BASELINE3_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE3_LABOR_REVENUE = 38;

    public static final int TYPE_BASELINE3_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_BASELINE3_LABOR_REVENUE = "TASK_ASSIGNMENTS.BASELINE3_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_LABOR_REVENUE,NAME_BASELINE3_LABOR_REVENUE);}

    /**
     * Column B_REGULAR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_B_REGULAR_COST = 39;

    public static final int TYPE_B_REGULAR_COST = Types.DECIMAL;

    public static final String NAME_B_REGULAR_COST = "TASK_ASSIGNMENTS.B_REGULAR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_B_REGULAR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_B_REGULAR_COST,NAME_B_REGULAR_COST);}

    /**
     * Column B_SPECIAL_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_B_SPECIAL_COST = 40;

    public static final int TYPE_B_SPECIAL_COST = Types.DECIMAL;

    public static final String NAME_B_SPECIAL_COST = "TASK_ASSIGNMENTS.B_SPECIAL_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_B_SPECIAL_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_B_SPECIAL_COST,NAME_B_SPECIAL_COST);}

    /**
     * Column B_REGULAR_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_B_REGULAR_RATE = 41;

    public static final int TYPE_B_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_B_REGULAR_RATE = "TASK_ASSIGNMENTS.B_REGULAR_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_B_REGULAR_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_B_REGULAR_RATE,NAME_B_REGULAR_RATE);}

    /**
     * Column B_SPECIAL_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_B_SPECIAL_RATE = 42;

    public static final int TYPE_B_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_B_SPECIAL_RATE = "TASK_ASSIGNMENTS.B_SPECIAL_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_B_SPECIAL_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_B_SPECIAL_RATE,NAME_B_SPECIAL_RATE);}

    /**
     * Column PLANNED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_START = 43;

    public static final int TYPE_PLANNED_START = Types.TIMESTAMP;

    public static final String NAME_PLANNED_START = "TASK_ASSIGNMENTS.PLANNED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PLANNED_START = "PROPOSEDSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PLANNED_START, NAME_PLANNED_START);
    }

    /**
     * Column PLANNED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_FINISH = 44;

    public static final int TYPE_PLANNED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PLANNED_FINISH = "TASK_ASSIGNMENTS.PLANNED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PLANNED_FINISH = "PROPOSEDFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PLANNED_FINISH, NAME_PLANNED_FINISH);
    }

    /**
     * Column PLANNED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_DURATION = 45;

    public static final int TYPE_PLANNED_DURATION = Types.INTEGER;

    public static final String NAME_PLANNED_DURATION = "TASK_ASSIGNMENTS.PLANNED_DURATION";

    public static final String PROPERTY_PLANNED_DURATION = "PROPOSEDDURATIONHOURS";
    static
    {
        // load and convert minutes to hours
        FIELDPROPERTYMAP.put(PROPERTY_PLANNED_DURATION, NAME_PLANNED_DURATION
                + " / 60");
    }

    /**
     * Column PLANNED_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_WORK = 46;

    public static final int TYPE_PLANNED_WORK = Types.INTEGER;

    public static final String NAME_PLANNED_WORK = "TASK_ASSIGNMENTS.PLANNED_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_WORK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PLANNED_WORK,NAME_PLANNED_WORK);}

    /**
     * Column PLANNED_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PLANNED_LABOR_COST = 47;

    public static final int TYPE_PLANNED_LABOR_COST = Types.DECIMAL;

    public static final String NAME_PLANNED_LABOR_COST = "TASK_ASSIGNMENTS.PLANNED_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_LABOR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_LABOR_COST,NAME_PLANNED_LABOR_COST);}

    /**
     * Column PLANNED_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PLANNED_LABOR_REVENUE = 48;

    public static final int TYPE_PLANNED_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_PLANNED_LABOR_REVENUE = "TASK_ASSIGNMENTS.PLANNED_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_LABOR_REVENUE,NAME_PLANNED_LABOR_REVENUE);}

    /**
     * Column PROPOSED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_START = 49;

    public static final int TYPE_PROPOSED_START = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_START = "TASK_ASSIGNMENTS.PROPOSED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROPOSED_START = "EXPECTEDSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_START, NAME_PROPOSED_START);
    }

    /**
     * Column PROPOSED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_FINISH = 50;

    public static final int TYPE_PROPOSED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_FINISH = "TASK_ASSIGNMENTS.PROPOSED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROPOSED_FINISH = "EXPECTEDFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_FINISH, NAME_PROPOSED_FINISH);
    }

    /**
     * Column PROPOSED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_DURATION = 51;

    public static final int TYPE_PROPOSED_DURATION = Types.INTEGER;

    public static final String NAME_PROPOSED_DURATION = "TASK_ASSIGNMENTS.PROPOSED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_DURATION,NAME_PROPOSED_DURATION);}

    /**
     * Column PROPOSED_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_WORK = 52;

    public static final int TYPE_PROPOSED_WORK = Types.INTEGER;

    public static final String NAME_PROPOSED_WORK = "TASK_ASSIGNMENTS.PROPOSED_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_WORK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_WORK,NAME_PROPOSED_WORK);}

    /**
     * Column PROPOSED_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROPOSED_LABOR_COST = 53;

    public static final int TYPE_PROPOSED_LABOR_COST = Types.DECIMAL;

    public static final String NAME_PROPOSED_LABOR_COST = "TASK_ASSIGNMENTS.PROPOSED_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_LABOR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_LABOR_COST,NAME_PROPOSED_LABOR_COST);}

    /**
     * Column PROPOSED_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROPOSED_LABOR_REVENUE = 54;

    public static final int TYPE_PROPOSED_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_PROPOSED_LABOR_REVENUE = "TASK_ASSIGNMENTS.PROPOSED_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_LABOR_REVENUE,NAME_PROPOSED_LABOR_REVENUE);}

    /**
     * Column SCHED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_START = 55;

    public static final int TYPE_SCHED_START = Types.TIMESTAMP;

    public static final String NAME_SCHED_START = "TASK_ASSIGNMENTS.SCHED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_START,NAME_SCHED_START);}

    /**
     * Column SCHED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_FINISH = 56;

    public static final int TYPE_SCHED_FINISH = Types.TIMESTAMP;

    public static final String NAME_SCHED_FINISH = "TASK_ASSIGNMENTS.SCHED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_FINISH,NAME_SCHED_FINISH);}

    /**
     * Column SCHED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_DURATION = 57;

    public static final int TYPE_SCHED_DURATION = Types.INTEGER;

    public static final String NAME_SCHED_DURATION = "TASK_ASSIGNMENTS.SCHED_DURATION";

    public static final String PROPERTY_SCHED_DURATION = "PLANNEDDURATIONHOURS";
    static
    {
        // load and convert minutes to hours
        FIELDPROPERTYMAP.put(PROPERTY_SCHED_DURATION, NAME_SCHED_DURATION
                + " / 60");
    }

    /**
     * Column SCHED_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_WORK = 58;

    public static final int TYPE_SCHED_WORK = Types.INTEGER;

    public static final String NAME_SCHED_WORK = "TASK_ASSIGNMENTS.SCHED_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_WORK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_WORK,NAME_SCHED_WORK);}

    /**
     * Column SCHED_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SCHED_LABOR_COST = 59;

    public static final int TYPE_SCHED_LABOR_COST = Types.DECIMAL;

    public static final String NAME_SCHED_LABOR_COST = "TASK_ASSIGNMENTS.SCHED_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_LABOR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_LABOR_COST,NAME_SCHED_LABOR_COST);}

    /**
     * Column SCHED_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SCHED_LABOR_REVENUE = 60;

    public static final int TYPE_SCHED_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_SCHED_LABOR_REVENUE = "TASK_ASSIGNMENTS.SCHED_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_LABOR_REVENUE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_LABOR_REVENUE,NAME_SCHED_LABOR_REVENUE);}

    /**
     * Column FORECAST_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_START = 61;

    public static final int TYPE_FORECAST_START = Types.TIMESTAMP;

    public static final String NAME_FORECAST_START = "TASK_ASSIGNMENTS.FORECAST_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_FORECAST_START = "FORECASTSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FORECAST_START, NAME_FORECAST_START);
    }

    /**
     * Column FORECAST_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_FINISH = 62;

    public static final int TYPE_FORECAST_FINISH = Types.TIMESTAMP;

    public static final String NAME_FORECAST_FINISH = "TASK_ASSIGNMENTS.FORECAST_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_FORECAST_FINISH = "FORECASTFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FORECAST_FINISH, NAME_FORECAST_FINISH);
    }

    /**
     * Column FORECAST_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_DURATION = 63;

    public static final int TYPE_FORECAST_DURATION = Types.INTEGER;

    public static final String NAME_FORECAST_DURATION = "TASK_ASSIGNMENTS.FORECAST_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_DURATION,NAME_FORECAST_DURATION);}

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 64;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "TASK_ASSIGNMENTS.ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ETC = "EETCHOURS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ETC, NAME_ETC);
    }

    /**
     * Column FORECAST_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_FORECAST_LABOR_COST = 65;

    public static final int TYPE_FORECAST_LABOR_COST = Types.DECIMAL;

    public static final String NAME_FORECAST_LABOR_COST = "TASK_ASSIGNMENTS.FORECAST_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_LABOR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_LABOR_COST,NAME_FORECAST_LABOR_COST);}

    /**
     * Column FORECAST_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_FORECAST_LABOR_REVENUE = 66;

    public static final int TYPE_FORECAST_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_FORECAST_LABOR_REVENUE = "TASK_ASSIGNMENTS.FORECAST_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_LABOR_REVENUE,NAME_FORECAST_LABOR_REVENUE);}

    /**
     * Column ACTUAL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_START = 67;

    public static final int TYPE_ACTUAL_START = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_START = "TASK_ASSIGNMENTS.ACTUAL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ACTUAL_START = "ACTUALSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_START, NAME_ACTUAL_START);
    }

    /**
     * Column ACTUAL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_FINISH = 68;

    public static final int TYPE_ACTUAL_FINISH = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_FINISH = "TASK_ASSIGNMENTS.ACTUAL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ACTUAL_FINISH = "ACTUALFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_FINISH, NAME_ACTUAL_FINISH);
    }

    /**
     * Column ACTUAL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_DURATION = 69;

    public static final int TYPE_ACTUAL_DURATION = Types.INTEGER;

    public static final String NAME_ACTUAL_DURATION = "TASK_ASSIGNMENTS.ACTUAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_DURATION,NAME_ACTUAL_DURATION);}

    /**
     * Column WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WORK_TODATE = 70;

    public static final int TYPE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_WORK_TODATE = "TASK_ASSIGNMENTS.WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_WORK_TODATE = "ACTUALWORK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE, NAME_WORK_TODATE);
    }

    /**
     * Column REGULAR_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REGULAR_TODATE = 71;

    public static final int TYPE_REGULAR_TODATE = Types.INTEGER;

    public static final String NAME_REGULAR_TODATE = "TASK_ASSIGNMENTS.REGULAR_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REGULAR_TODATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REGULAR_TODATE,NAME_REGULAR_TODATE);}

    /**
     * Column SPECIAL_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SPECIAL_TODATE = 72;

    public static final int TYPE_SPECIAL_TODATE = Types.INTEGER;

    public static final String NAME_SPECIAL_TODATE = "TASK_ASSIGNMENTS.SPECIAL_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_TODATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_TODATE,NAME_SPECIAL_TODATE);}

    /**
     * Column BILLABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BILLABLE = 73;

    public static final int TYPE_BILLABLE = Types.SMALLINT;

    public static final String NAME_BILLABLE = "TASK_ASSIGNMENTS.BILLABLE";

    public static final String PROPERTY_BILLABLE = "BILLABLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BILLABLE, SqlUtil.getBooleanValue(
            NAME_BILLABLE, 1));
    }

    /**
     * Column BILLABLE_WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BILLABLE_WORK_TODATE = 74;

    public static final int TYPE_BILLABLE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_BILLABLE_WORK_TODATE = "TASK_ASSIGNMENTS.BILLABLE_WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLABLE_WORK_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BILLABLE_WORK_TODATE,NAME_BILLABLE_WORK_TODATE);}

    /**
     * Column BILLABLE_REGULAR_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BILLABLE_REGULAR_TODATE = 75;

    public static final int TYPE_BILLABLE_REGULAR_TODATE = Types.INTEGER;

    public static final String NAME_BILLABLE_REGULAR_TODATE = "TASK_ASSIGNMENTS.BILLABLE_REGULAR_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLABLE_REGULAR_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BILLABLE_REGULAR_TODATE,NAME_BILLABLE_REGULAR_TODATE);}

    /**
     * Column BILLABLE_SPECIAL_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BILLABLE_SPECIAL_TODATE = 76;

    public static final int TYPE_BILLABLE_SPECIAL_TODATE = Types.INTEGER;

    public static final String NAME_BILLABLE_SPECIAL_TODATE = "TASK_ASSIGNMENTS.BILLABLE_SPECIAL_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLABLE_SPECIAL_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BILLABLE_SPECIAL_TODATE,NAME_BILLABLE_SPECIAL_TODATE);}

    /**
     * Column ACTUAL_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_LABOR_COST = 77;

    public static final int TYPE_ACTUAL_LABOR_COST = Types.DECIMAL;

    public static final String NAME_ACTUAL_LABOR_COST = "TASK_ASSIGNMENTS.ACTUAL_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_LABOR_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_LABOR_COST,NAME_ACTUAL_LABOR_COST);}

    /**
     * Column ACTUAL_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_LABOR_REVENUE = 78;

    public static final int TYPE_ACTUAL_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_ACTUAL_LABOR_REVENUE = "TASK_ASSIGNMENTS.ACTUAL_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_LABOR_REVENUE,NAME_ACTUAL_LABOR_REVENUE);}

    /**
     * Column MILESTONE_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MILESTONE_PERCENT = 79;

    public static final int TYPE_MILESTONE_PERCENT = Types.SMALLINT;

    public static final String NAME_MILESTONE_PERCENT = "TASK_ASSIGNMENTS.MILESTONE_PERCENT";

    public static final String PROPERTY_MILESTONE_PERCENT = "PERCENTAGECOMPLETE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MILESTONE_PERCENT, NAME_MILESTONE_PERCENT
                + " / 100");
    }
    static
    {
        FIELDPROPERTYMAP.put("COMPLETE", SqlUtil.getBooleanValue(
            NAME_MILESTONE_PERCENT, 10000));
    }

    /**
     * Column LAST_SUBMITTED of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LAST_SUBMITTED = 80;

    public static final int TYPE_LAST_SUBMITTED = Types.DATE;

    public static final String NAME_LAST_SUBMITTED = "TASK_ASSIGNMENTS.LAST_SUBMITTED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_SUBMITTED = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_SUBMITTED,NAME_LAST_SUBMITTED);}

    /**
     * Column BASELINE_WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE_WORK_TODATE = 81;

    public static final int TYPE_BASELINE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_BASELINE_WORK_TODATE = "TASK_ASSIGNMENTS.BASELINE_WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE_WORK_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_WORK_TODATE,NAME_BASELINE_WORK_TODATE);}

    /**
     * Column ACTIVITY_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ACTIVITY_COUNT = 82;

    public static final int TYPE_ACTIVITY_COUNT = Types.SMALLINT;

    public static final String NAME_ACTIVITY_COUNT = "TASK_ASSIGNMENTS.ACTIVITY_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTIVITY_COUNT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTIVITY_COUNT,NAME_ACTIVITY_COUNT);}

    /**
     * Column T_UNIT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_T_UNIT = 83;

    public static final int TYPE_T_UNIT = Types.SMALLINT;

    public static final String NAME_T_UNIT = "TASK_ASSIGNMENTS.T_UNIT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_T_UNIT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_T_UNIT,NAME_T_UNIT);}

    /**
     * Column C_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_C_EFFORT = 84;

    public static final int TYPE_C_EFFORT = Types.INTEGER;

    public static final String NAME_C_EFFORT = "TASK_ASSIGNMENTS.C_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_C_EFFORT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_C_EFFORT,NAME_C_EFFORT);}

    /**
     * Column C_UNIT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_C_UNIT = 85;

    public static final int TYPE_C_UNIT = Types.SMALLINT;

    public static final String NAME_C_UNIT = "TASK_ASSIGNMENTS.C_UNIT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_C_UNIT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_C_UNIT,NAME_C_UNIT);}

    /**
     * Column C_OCCURRENCE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_C_OCCURRENCE = 86;

    public static final int TYPE_C_OCCURRENCE = Types.SMALLINT;

    public static final String NAME_C_OCCURRENCE = "TASK_ASSIGNMENTS.C_OCCURRENCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_C_OCCURRENCE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_C_OCCURRENCE,NAME_C_OCCURRENCE);}

    /**
     * Column C_CONTOUR of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_C_CONTOUR = 87;

    public static final int TYPE_C_CONTOUR = Types.INTEGER;

    public static final String NAME_C_CONTOUR = "TASK_ASSIGNMENTS.C_CONTOUR";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_C_CONTOUR = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_C_CONTOUR,NAME_C_CONTOUR);}

    /**
     * Column CONTOUR_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONTOUR_TYPE = 88;

    public static final int TYPE_CONTOUR_TYPE = Types.INTEGER;

    public static final String NAME_CONTOUR_TYPE = "TASK_ASSIGNMENTS.CONTOUR_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONTOUR_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTOUR_TYPE,NAME_CONTOUR_TYPE);}

    /**
     * Column UNITS_MSP of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_UNITS_MSP = 89;

    public static final int TYPE_UNITS_MSP = Types.INTEGER;

    public static final String NAME_UNITS_MSP = "TASK_ASSIGNMENTS.UNITS_MSP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNITS_MSP = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_UNITS_MSP,NAME_UNITS_MSP);}

    /**
     * Column BAK_START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BAK_START_DT = 90;

    public static final int TYPE_BAK_START_DT = Types.TIMESTAMP;

    public static final String NAME_BAK_START_DT = "TASK_ASSIGNMENTS.BAK_START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BAK_START_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BAK_START_DT,NAME_BAK_START_DT);}

    /**
     * Column BAK_FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BAK_FINISH_DT = 91;

    public static final int TYPE_BAK_FINISH_DT = Types.TIMESTAMP;

    public static final String NAME_BAK_FINISH_DT = "TASK_ASSIGNMENTS.BAK_FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BAK_FINISH_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BAK_FINISH_DT,NAME_BAK_FINISH_DT);}

    /**
     * Column BAK_WORK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BAK_WORK = 92;

    public static final int TYPE_BAK_WORK = Types.INTEGER;

    public static final String NAME_BAK_WORK = "TASK_ASSIGNMENTS.BAK_WORK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_BAK_WORK = "LASTSCHEDULEWORK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BAK_WORK, NAME_BAK_WORK);
    }

    /**
     * Column BAK_ASSIGN_STRING of type Types.VARCHAR mapped to String.
     */
    public static final int ID_BAK_ASSIGN_STRING = 93;

    public static final int TYPE_BAK_ASSIGN_STRING = Types.VARCHAR;

    public static final String NAME_BAK_ASSIGN_STRING = "TASK_ASSIGNMENTS.BAK_ASSIGN_STRING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BAK_ASSIGN_STRING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BAK_ASSIGN_STRING,NAME_BAK_ASSIGN_STRING);}

    /**
     * Column TIMECODE2_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE2_ID = 94;

    public static final int TYPE_TIMECODE2_ID = Types.CHAR;

    public static final String NAME_TIMECODE2_ID = "TASK_ASSIGNMENTS.TIMECODE2_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE2_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE2_ID,NAME_TIMECODE2_ID);}

    /**
     * Column PROJECT_ASSIGNMENT_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROJECT_ASSIGNMENT_ID = 95;

    public static final int TYPE_PROJECT_ASSIGNMENT_ID = Types.VARCHAR;

    public static final String NAME_PROJECT_ASSIGNMENT_ID = "TASK_ASSIGNMENTS.PROJECT_ASSIGNMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ASSIGNMENT_ID =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ASSIGNMENT_ID,NAME_PROJECT_ASSIGNMENT_ID);}

    /**
     * Column TABLE_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TABLE_TYPE = 96;

    public static final int TYPE_TABLE_TYPE = Types.SMALLINT;

    public static final String NAME_TABLE_TYPE = "TASK_ASSIGNMENTS.TABLE_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TABLE_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TABLE_TYPE,NAME_TABLE_TYPE);}

    /**
     * Column ROLLUP_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ROLLUP_BITS = 97;

    public static final int TYPE_ROLLUP_BITS = Types.INTEGER;

    public static final String NAME_ROLLUP_BITS = "TASK_ASSIGNMENTS.ROLLUP_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLLUP_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_BITS,NAME_ROLLUP_BITS);}

    /**
     * Column UNRESOLVED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_UNRESOLVED_START = 98;

    public static final int TYPE_UNRESOLVED_START = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_START = "TASK_ASSIGNMENTS.UNRESOLVED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_START,NAME_UNRESOLVED_START);}

    /**
     * Column UNRESOLVED_FINISH of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_UNRESOLVED_FINISH = 99;

    public static final int TYPE_UNRESOLVED_FINISH = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_FINISH = "TASK_ASSIGNMENTS.UNRESOLVED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_FINISH,NAME_UNRESOLVED_FINISH);}

    /**
     * Column UNRESOLVED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_UNRESOLVED_DURATION = 100;

    public static final int TYPE_UNRESOLVED_DURATION = Types.INTEGER;

    public static final String NAME_UNRESOLVED_DURATION = "TASK_ASSIGNMENTS.UNRESOLVED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_DURATION,NAME_UNRESOLVED_DURATION);}

    /**
     * Column ESTIMATION_LABOR_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_LABOR_COST = 101;

    public static final int TYPE_ESTIMATION_LABOR_COST = Types.DECIMAL;

    public static final String NAME_ESTIMATION_LABOR_COST = "TASK_ASSIGNMENTS.ESTIMATION_LABOR_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_LABOR_COST =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_LABOR_COST,NAME_ESTIMATION_LABOR_COST);}

    /**
     * Column ESTIMATION_LABOR_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_LABOR_REVENUE = 102;

    public static final int TYPE_ESTIMATION_LABOR_REVENUE = Types.DECIMAL;

    public static final String NAME_ESTIMATION_LABOR_REVENUE = "TASK_ASSIGNMENTS.ESTIMATION_LABOR_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_LABOR_REVENUE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_LABOR_REVENUE,NAME_ESTIMATION_LABOR_REVENUE);}

    /**
     * Column ESTIMATION_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_LABOR_SAVING = 103;

    public static final int TYPE_ESTIMATION_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_ESTIMATION_LABOR_SAVING = "TASK_ASSIGNMENTS.ESTIMATION_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_LABOR_SAVING =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_LABOR_SAVING,NAME_ESTIMATION_LABOR_SAVING);}

    /**
     * Column BASELINE1_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE1_LABOR_SAVING = 104;

    public static final int TYPE_BASELINE1_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_BASELINE1_LABOR_SAVING = "TASK_ASSIGNMENTS.BASELINE1_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_LABOR_SAVING =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_LABOR_SAVING,NAME_BASELINE1_LABOR_SAVING);}

    /**
     * Column ACTUAL_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_LABOR_SAVING = 105;

    public static final int TYPE_ACTUAL_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_ACTUAL_LABOR_SAVING = "TASK_ASSIGNMENTS.ACTUAL_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_LABOR_SAVING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_LABOR_SAVING,NAME_ACTUAL_LABOR_SAVING);}

    /**
     * Column SCHED_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SCHED_LABOR_SAVING = 106;

    public static final int TYPE_SCHED_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_SCHED_LABOR_SAVING = "TASK_ASSIGNMENTS.SCHED_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_LABOR_SAVING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_LABOR_SAVING,NAME_SCHED_LABOR_SAVING);}

    /**
     * Column FORECAST_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_FORECAST_LABOR_SAVING = 107;

    public static final int TYPE_FORECAST_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_FORECAST_LABOR_SAVING = "TASK_ASSIGNMENTS.FORECAST_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_LABOR_SAVING =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_LABOR_SAVING,NAME_FORECAST_LABOR_SAVING);}

    /**
     * Column PLANNED_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PLANNED_LABOR_SAVING = 108;

    public static final int TYPE_PLANNED_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_PLANNED_LABOR_SAVING = "TASK_ASSIGNMENTS.PLANNED_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_LABOR_SAVING =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_LABOR_SAVING,NAME_PLANNED_LABOR_SAVING);}

    /**
     * Column PROPOSED_LABOR_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROPOSED_LABOR_SAVING = 109;

    public static final int TYPE_PROPOSED_LABOR_SAVING = Types.DECIMAL;

    public static final String NAME_PROPOSED_LABOR_SAVING = "TASK_ASSIGNMENTS.PROPOSED_LABOR_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_LABOR_SAVING =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_LABOR_SAVING,NAME_PROPOSED_LABOR_SAVING);}

    /**
     * Column LEVEL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LEVEL_START = 110;

    public static final int TYPE_LEVEL_START = Types.TIMESTAMP;

    public static final String NAME_LEVEL_START = "TASK_ASSIGNMENTS.LEVEL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LEVEL_START = "CALCULATEDLEVELEDSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LEVEL_START, NAME_LEVEL_START);
    }

    /**
     * Column LEVEL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LEVEL_FINISH = 111;

    public static final int TYPE_LEVEL_FINISH = Types.TIMESTAMP;

    public static final String NAME_LEVEL_FINISH = "TASK_ASSIGNMENTS.LEVEL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_LEVEL_FINISH = "CALCULATEDLEVELEDFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_LEVEL_FINISH, NAME_LEVEL_FINISH);
    }

    /**
     * Column LEVEL_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_LEVEL_EFFORT = 112;

    public static final int TYPE_LEVEL_EFFORT = Types.BIGINT;

    public static final String NAME_LEVEL_EFFORT = "TASK_ASSIGNMENTS.LEVEL_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_EFFORT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_EFFORT,NAME_LEVEL_EFFORT);}

    /**
     * Column CONSIDER of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CONSIDER = 113;

    public static final int TYPE_CONSIDER = Types.SMALLINT;

    public static final String NAME_CONSIDER = "TASK_ASSIGNMENTS.CONSIDER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONSIDER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONSIDER,NAME_CONSIDER);}

    /**
     * Column LEVEL_ASSIGN_STRING of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LEVEL_ASSIGN_STRING = 114;

    public static final int TYPE_LEVEL_ASSIGN_STRING = Types.VARCHAR;

    public static final String NAME_LEVEL_ASSIGN_STRING = "TASK_ASSIGNMENTS.LEVEL_ASSIGN_STRING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ASSIGN_STRING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ASSIGN_STRING,NAME_LEVEL_ASSIGN_STRING);}

    /**
     * Column CAPITAL_EXPENSE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CAPITAL_EXPENSE = 115;

    public static final int TYPE_CAPITAL_EXPENSE = Types.SMALLINT;

    public static final String NAME_CAPITAL_EXPENSE = "TASK_ASSIGNMENTS.CAPITAL_EXPENSE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CAPITAL_EXPENSE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_EXPENSE,NAME_CAPITAL_EXPENSE);}

    /**
     * Column BASELINE3_SCH_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BASELINE3_SCH_FLAG = 116;

    public static final int TYPE_BASELINE3_SCH_FLAG = Types.SMALLINT;

    public static final String NAME_BASELINE3_SCH_FLAG = "TASK_ASSIGNMENTS.BASELINE3_SCH_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_SCH_FLAG = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_SCH_FLAG,NAME_BASELINE3_SCH_FLAG);}

    /**
     * Column RATES_EDITED of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_RATES_EDITED = 117;

    public static final int TYPE_RATES_EDITED = Types.SMALLINT;

    public static final String NAME_RATES_EDITED = "TASK_ASSIGNMENTS.RATES_EDITED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RATES_EDITED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RATES_EDITED,NAME_RATES_EDITED);}

    /**
     * Column TIMECODE_1_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_ID = 118;

    public static final int TYPE_TIMECODE_1_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_1_ID = "TASK_ASSIGNMENTS.TIMECODE_1_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE_1_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_ID,NAME_TIMECODE_1_ID);}

    /**
     * Column TIMECODE_2_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_ID = 119;

    public static final int TYPE_TIMECODE_2_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_2_ID = "TASK_ASSIGNMENTS.TIMECODE_2_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE_2_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_ID,NAME_TIMECODE_2_ID);}

    /**
     * Column ASSIGNMENT_DELAY of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ASSIGNMENT_DELAY = 120;

    public static final int TYPE_ASSIGNMENT_DELAY = Types.BIGINT;

    public static final String NAME_ASSIGNMENT_DELAY = "TASK_ASSIGNMENTS.ASSIGNMENT_DELAY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ASSIGNMENT_DELAY = "DAYSOFDELAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ASSIGNMENT_DELAY, NAME_ASSIGNMENT_DELAY);
    }

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 121;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TASK_ASSIGNMENTS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column ASSIGN_UNITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ASSIGN_UNITS = 122;

    public static final int TYPE_ASSIGN_UNITS = Types.INTEGER;

    public static final String NAME_ASSIGN_UNITS = "TASK_ASSIGNMENTS.ASSIGN_UNITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_UNITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_UNITS,NAME_ASSIGN_UNITS);}

    /**
     * Column LAST_SCHED_MIN of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LAST_SCHED_MIN = 123;

    public static final int TYPE_LAST_SCHED_MIN = Types.INTEGER;

    public static final String NAME_LAST_SCHED_MIN = "TASK_ASSIGNMENTS.LAST_SCHED_MIN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_SCHED_MIN = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_SCHED_MIN,NAME_LAST_SCHED_MIN);}

    /**
     * Column RATE_ADJUSTMENT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_RATE_ADJUSTMENT = 124;

    public static final int TYPE_RATE_ADJUSTMENT = Types.INTEGER;

    public static final String NAME_RATE_ADJUSTMENT = "TASK_ASSIGNMENTS.RATE_ADJUSTMENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_RATE_ADJUSTMENT = "PERCENTAGERATEADJUSTMENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RATE_ADJUSTMENT, NAME_RATE_ADJUSTMENT);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 125;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TASK_ASSIGNMENTS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 126;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TASK_ASSIGNMENTS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column LABOR_COST_CC_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_LABOR_COST_CC_ID = 127;

    public static final int TYPE_LABOR_COST_CC_ID = Types.CHAR;

    public static final String NAME_LABOR_COST_CC_ID = "TASK_ASSIGNMENTS.LABOR_COST_CC_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LABOR_COST_CC_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LABOR_COST_CC_ID,NAME_LABOR_COST_CC_ID);}

    /**
     * Column LABOR_REVENUE_CC_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_LABOR_REVENUE_CC_ID = 128;

    public static final int TYPE_LABOR_REVENUE_CC_ID = Types.CHAR;

    public static final String NAME_LABOR_REVENUE_CC_ID = "TASK_ASSIGNMENTS.LABOR_REVENUE_CC_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LABOR_REVENUE_CC_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LABOR_REVENUE_CC_ID,NAME_LABOR_REVENUE_CC_ID);}

    /**
     * Column DEFAULT_LABOR_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_LABOR_FLAG = 129;

    public static final int TYPE_DEFAULT_LABOR_FLAG = Types.CHAR;

    public static final String NAME_DEFAULT_LABOR_FLAG = "TASK_ASSIGNMENTS.DEFAULT_LABOR_FLAG";

    public static final String PROPERTY_DEFAULT_LABOR_FLAG = "DEFAULTLABOR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_LABOR_FLAG, SqlUtil
            .getBooleanValue(NAME_DEFAULT_LABOR_FLAG, "Y"));
    }

    /**
     * Column COST_CENTER_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_COST_CENTER_BITS = 130;

    public static final int TYPE_COST_CENTER_BITS = Types.INTEGER;

    public static final String NAME_COST_CENTER_BITS = "TASK_ASSIGNMENTS.COST_CENTER_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_CENTER_BITS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_BITS,NAME_COST_CENTER_BITS);}

    /**
     * Column CREDIT_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CREDIT_COUNT = 131;

    public static final int TYPE_CREDIT_COUNT = Types.SMALLINT;

    public static final String NAME_CREDIT_COUNT = "TASK_ASSIGNMENTS.CREDIT_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREDIT_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREDIT_COUNT,NAME_CREDIT_COUNT);}

    /**
     * Column BAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BAC = 132;

    public static final int TYPE_BAC = Types.DECIMAL;

    public static final String NAME_BAC = "TASK_ASSIGNMENTS.BAC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BAC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BAC,NAME_BAC);}

    /**
     * Column BCWS of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWS = 133;

    public static final int TYPE_BCWS = Types.DECIMAL;

    public static final String NAME_BCWS = "TASK_ASSIGNMENTS.BCWS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BCWS,NAME_BCWS);}

    /**
     * Column BCWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWP = 134;

    public static final int TYPE_BCWP = Types.DECIMAL;

    public static final String NAME_BCWP = "TASK_ASSIGNMENTS.BCWP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWP = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BCWP,NAME_BCWP);}

    /**
     * Column ACWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ACWP = 135;

    public static final int TYPE_ACWP = Types.DECIMAL;

    public static final String NAME_ACWP = "TASK_ASSIGNMENTS.ACWP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACWP = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACWP,NAME_ACWP);}

    /**
     * Column EV_ETC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EV_ETC = 136;

    public static final int TYPE_EV_ETC = Types.DECIMAL;

    public static final String NAME_EV_ETC = "TASK_ASSIGNMENTS.EV_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EV_ETC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EV_ETC,NAME_EV_ETC);}

    /**
     * Column EAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EAC = 137;

    public static final int TYPE_EAC = Types.DECIMAL;

    public static final String NAME_EAC = "TASK_ASSIGNMENTS.EAC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EAC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EAC,NAME_EAC);}

    /**
     * Column CONTRACT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CONTRACT_ID = 138;

    public static final int TYPE_CONTRACT_ID = Types.CHAR;

    public static final String NAME_CONTRACT_ID = "TASK_ASSIGNMENTS.CONTRACT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONTRACT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTRACT_ID,NAME_CONTRACT_ID);}

    /**
     * Column CONTRACT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CONFIG_FLAG = 139;

    public static final int TYPE_CONFIG_FLAG = Types.SMALLINT;

    public static final String NAME_CONFIG_FLAG;

    static
    {
        // temporary patch to make the 7.1.0.0 code base work
        // against the 7.0.4.4 Database in read-only mode.
        if (TempReqProOverride.DISABLE_REQPRO)
        {
            NAME_CONFIG_FLAG = "TASK_ASSIGNMENTS.BILLABLE";
        }
        else
        {
            NAME_CONFIG_FLAG = "TASK_ASSIGNMENTS.CONFIG_FLAG";
        }
    }

    public static final String PROPERTY_CONFIG_FLAG = "SHOWNINTIMESHEET";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, SqlUtil.getBooleanValue(
            NAME_CONFIG_FLAG, 1));
    }

    public static final String TABLE_NAME = "TASK_ASSIGNMENTS";

    /**
     * Create an array of type string containing all the fields of the
     * TASK_ASSIGNMENTS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_TSK_ASSIGNMENT_ID, NAME_RES_ASSIGNMENT_ID, NAME_PROJECT_ID,
            NAME_TASK_ID, NAME_RESOURCE_ID, NAME_RESOURCE_TYPE,
            NAME_ACCEPTED_DATE, NAME_ESTIMATE, NAME_BEST, NAME_BIT_FLAG,
            NAME_ASSIGN_STRING, NAME_SD, NAME_FD, NAME_START_DT,
            NAME_FINISH_DT, NAME_DURATION, NAME_REGULAR_COST,
            NAME_SPECIAL_COST, NAME_REGULAR_RATE, NAME_SPECIAL_RATE,
            NAME_BASELINE1_START, NAME_BASELINE1_FINISH,
            NAME_BASELINE1_DURATION, NAME_BASELINE1_WORK,
            NAME_BASELINE1_LABOR_COST, NAME_BASELINE1_LABOR_REVENUE,
            NAME_BASELINE2_START, NAME_BASELINE2_FINISH,
            NAME_BASELINE2_DURATION, NAME_BASELINE2_WORK,
            NAME_BASELINE2_LABOR_COST, NAME_BASELINE2_LABOR_REVENUE,
            NAME_BASELINE3_START, NAME_BASELINE3_FINISH,
            NAME_BASELINE3_DURATION, NAME_BASELINE3_WORK,
            NAME_BASELINE3_LABOR_COST, NAME_BASELINE3_LABOR_REVENUE,
            NAME_B_REGULAR_COST, NAME_B_SPECIAL_COST, NAME_B_REGULAR_RATE,
            NAME_B_SPECIAL_RATE, NAME_PLANNED_START, NAME_PLANNED_FINISH,
            NAME_PLANNED_DURATION, NAME_PLANNED_WORK, NAME_PLANNED_LABOR_COST,
            NAME_PLANNED_LABOR_REVENUE, NAME_PROPOSED_START,
            NAME_PROPOSED_FINISH, NAME_PROPOSED_DURATION, NAME_PROPOSED_WORK,
            NAME_PROPOSED_LABOR_COST, NAME_PROPOSED_LABOR_REVENUE,
            NAME_SCHED_START, NAME_SCHED_FINISH, NAME_SCHED_DURATION,
            NAME_SCHED_WORK, NAME_SCHED_LABOR_COST, NAME_SCHED_LABOR_REVENUE,
            NAME_FORECAST_START, NAME_FORECAST_FINISH, NAME_FORECAST_DURATION,
            NAME_ETC, NAME_FORECAST_LABOR_COST, NAME_FORECAST_LABOR_REVENUE,
            NAME_ACTUAL_START, NAME_ACTUAL_FINISH, NAME_ACTUAL_DURATION,
            NAME_WORK_TODATE, NAME_REGULAR_TODATE, NAME_SPECIAL_TODATE,
            NAME_BILLABLE, NAME_BILLABLE_WORK_TODATE,
            NAME_BILLABLE_REGULAR_TODATE, NAME_BILLABLE_SPECIAL_TODATE,
            NAME_ACTUAL_LABOR_COST, NAME_ACTUAL_LABOR_REVENUE,
            NAME_MILESTONE_PERCENT, NAME_LAST_SUBMITTED,
            NAME_BASELINE_WORK_TODATE, NAME_ACTIVITY_COUNT, NAME_T_UNIT,
            NAME_C_EFFORT, NAME_C_UNIT, NAME_C_OCCURRENCE, NAME_C_CONTOUR,
            NAME_CONTOUR_TYPE, NAME_UNITS_MSP, NAME_BAK_START_DT,
            NAME_BAK_FINISH_DT, NAME_BAK_WORK, NAME_BAK_ASSIGN_STRING,
            NAME_TIMECODE2_ID, NAME_PROJECT_ASSIGNMENT_ID, NAME_TABLE_TYPE,
            NAME_ROLLUP_BITS, NAME_UNRESOLVED_START, NAME_UNRESOLVED_FINISH,
            NAME_UNRESOLVED_DURATION, NAME_ESTIMATION_LABOR_COST,
            NAME_ESTIMATION_LABOR_REVENUE, NAME_ESTIMATION_LABOR_SAVING,
            NAME_BASELINE1_LABOR_SAVING, NAME_ACTUAL_LABOR_SAVING,
            NAME_SCHED_LABOR_SAVING, NAME_FORECAST_LABOR_SAVING,
            NAME_PLANNED_LABOR_SAVING, NAME_PROPOSED_LABOR_SAVING,
            NAME_LEVEL_START, NAME_LEVEL_FINISH, NAME_LEVEL_EFFORT,
            NAME_CONSIDER, NAME_LEVEL_ASSIGN_STRING, NAME_CAPITAL_EXPENSE,
            NAME_BASELINE3_SCH_FLAG, NAME_RATES_EDITED, NAME_TIMECODE_1_ID,
            NAME_TIMECODE_2_ID, NAME_ASSIGNMENT_DELAY, NAME_REC_DATETIME,
            NAME_ASSIGN_UNITS, NAME_LAST_SCHED_MIN, NAME_RATE_ADJUSTMENT,
            NAME_REC_STATUS, NAME_REC_USER, NAME_LABOR_COST_CC_ID,
            NAME_LABOR_REVENUE_CC_ID, NAME_DEFAULT_LABOR_FLAG,
            NAME_COST_CENTER_BITS, NAME_CREDIT_COUNT, NAME_BAC, NAME_BCWS,
            NAME_BCWP, NAME_ACWP, NAME_EV_ETC, NAME_EAC, NAME_CONTRACT_ID,
            NAME_CONFIG_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new TaskAssignmentManager instance.
     * 
     * @return TaskAssignmentManager
     */
    public TaskAssignmentManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_TSK_ASSIGNMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new TaskAssignment();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        TaskAssignment taskAssignment = (TaskAssignment) rpmObject;
        taskAssignment.setID(rs.getString(ID_TSK_ASSIGNMENT_ID));

        //Calculated values used for set and delta methods
        double actualWork = DateUtil.convertMinutesToHours(
            rs.getInt(ID_WORK_TODATE)).doubleValue();
        double lastScheduleWork = DateUtil.convertMinutesToHours(
            rs.getInt(ID_BAK_WORK)).doubleValue() / 8;
        double expectedWork = DateUtil.convertMinutesToHours(
            rs.getInt(ID_PROPOSED_WORK)).doubleValue();
        double remainingHours = expectedWork - actualWork;
        double proposedDuration = DateUtil.convertMinutesToHours(
            rs.getInt(ID_PLANNED_DURATION)).doubleValue();
        double plannedDuration = DateUtil.convertMinutesToHours(
            rs.getInt(ID_SCHED_DURATION)).doubleValue();
        double baselineWork = DateUtil.convertMinutesToHours(
            rs.getInt(ID_BASELINE1_WORK)).doubleValue();
        double workSchedule = DateUtil.convertMinutesToHours(
            rs.getInt(ID_SCHED_WORK)).doubleValue();

        double percentageWork = 0;
        if (expectedWork > 0)
        {
            // we don't want to return a percentage > 100
            percentageWork = ((actualWork / expectedWork * 100) >= 100 ? 100
                    : (actualWork / expectedWork * 100));
        }
        Double percentageCompleted = new Double(((double) rs
            .getInt(ID_MILESTONE_PERCENT)) / 100);
        Double unitsPercentage = new Double(
            ((double) rs.getInt(ID_UNITS_MSP)) / 100);

        Boolean isComplete = new Boolean(
            percentageCompleted.doubleValue() == 100);

        boolean billable = true;
        if (rs.getInt(ID_BILLABLE) == 0)
            billable = false;

        boolean defaultLabor = decodeDefaultLabor(rs
            .getString(ID_DEFAULT_LABOR_FLAG));

        FinancialCategory financialCategory = FinancialCategoryUtil
            .toFinancialCategory(rs.getInt(ID_CAPITAL_EXPENSE));

        short configFlag = rs.getShort(ID_CONFIG_FLAG);
        Boolean shownInTimesheet = ManagerUtil.toBoolean(configFlag);

        if (isDelta)
        {
            taskAssignment.deltaActualFinishDate(Manager.getCalendar(rs,
                ID_ACTUAL_FINISH));
            taskAssignment.deltaActualStartDate(Manager.getCalendar(rs,
                ID_ACTUAL_START));
            taskAssignment.deltaAssignmentFinishDate(Manager.getCalendar(rs,
                ID_UNRESOLVED_FINISH));
            taskAssignment.deltaAssignmentStartDate(Manager.getTimestamp(rs,
                ID_START_DT));
            taskAssignment.deltaExpectedFinishDate(Manager.getCalendar(rs,
                ID_PROPOSED_FINISH));
            taskAssignment.deltaExpectedStartDate(Manager.getCalendar(rs,
                ID_PROPOSED_START));
            taskAssignment.deltaExpectedStartDate(Manager.getCalendar(rs,
                ID_PROPOSED_START));
            taskAssignment.deltaForecastFinishDate(Manager.getCalendar(rs,
                ID_FORECAST_FINISH));
            taskAssignment.deltaForecastStartDate(Manager.getCalendar(rs,
                ID_FORECAST_START));
            taskAssignment.deltaBaselineFinishDate(Manager.getCalendar(rs,
                ID_BASELINE1_FINISH));
            taskAssignment.deltaBaselineStartDate(Manager.getCalendar(rs,
                ID_BASELINE1_START));
            taskAssignment.deltaCalculatedLeveledFinishDate(Manager
                .getCalendar(rs, ID_LEVEL_FINISH));
            taskAssignment.deltaCalculatedLeveledStartDate(Manager.getCalendar(
                rs, ID_LEVEL_START));

            taskAssignment.deltaActualWork(new Double(actualWork));
            taskAssignment.deltaDaysOfDelay(new Integer(rs
                .getInt(ID_ASSIGNMENT_DELAY)));
            taskAssignment.deltaEETCHours(DateUtil.convertMinutesToHours(rs
                .getLong(ID_ETC)));
            taskAssignment.deltaExpectedWork(new Double(expectedWork));
            taskAssignment.deltaProposedDurationHours(new Double(
                proposedDuration));
            taskAssignment
                .deltaPlannedDurationHours(new Double(plannedDuration));
            taskAssignment.deltaPercentageComplete(percentageCompleted);
            taskAssignment.deltaComplete(isComplete);
            taskAssignment.deltaPercentageWork(new Double(percentageWork));
            taskAssignment.deltaRemainingHours(new Double(remainingHours));
            taskAssignment.deltaUnitsPercentage(unitsPercentage);
            taskAssignment.deltaLastScheduleWork(new Double(lastScheduleWork));
            taskAssignment.deltaBillable(new Boolean(billable));
            taskAssignment.deltaCostPerHour(new Double(rs
                .getDouble(ID_REGULAR_COST)));
            taskAssignment.deltaDefaultLabor(new Boolean(defaultLabor));
            taskAssignment.deltaSellingRatePerHour(new Double(rs
                .getDouble(ID_REGULAR_RATE)));
            taskAssignment.deltaSpecialCostPerHour(new Double(rs
                .getDouble(ID_SPECIAL_COST)));
            taskAssignment.deltaSpecialSellingRatePerHour(new Double(rs
                .getDouble(ID_SPECIAL_RATE)));
            taskAssignment.deltaPercentageRateAdjustment(new Integer(rs
                .getInt(ID_RATE_ADJUSTMENT)));
            taskAssignment.deltaBaselineWork(new Double(baselineWork));
            taskAssignment.deltaWorkSchedule(new Double(workSchedule));
            taskAssignment.deltaCostType(financialCategory);
            taskAssignment.deltaShownInTimesheet(shownInTimesheet);
        }
        else
        {
            taskAssignment.setActualFinishDate(Manager.getCalendar(rs,
                ID_ACTUAL_FINISH));
            taskAssignment.setActualStartDate(Manager.getCalendar(rs,
                ID_ACTUAL_START));
            taskAssignment.setAssignmentFinishDate(Manager.getCalendar(rs,
                ID_UNRESOLVED_FINISH));
            taskAssignment.setAssignmentStartDate(Manager.getTimestamp(rs,
                ID_START_DT));
            taskAssignment.setExpectedFinishDate(Manager.getCalendar(rs,
                ID_PROPOSED_FINISH));
            taskAssignment.setExpectedStartDate(Manager.getCalendar(rs,
                ID_PROPOSED_START));
            taskAssignment.setExpectedStartDate(Manager.getCalendar(rs,
                ID_PROPOSED_START));
            taskAssignment.setForecastFinishDate(Manager.getCalendar(rs,
                ID_FORECAST_FINISH));
            taskAssignment.setForecastStartDate(Manager.getCalendar(rs,
                ID_FORECAST_START));
            taskAssignment.setBaselineFinishDate(Manager.getCalendar(rs,
                ID_BASELINE1_FINISH));
            taskAssignment.setBaselineStartDate(Manager.getCalendar(rs,
                ID_BASELINE1_START));
            taskAssignment.setCalculatedLeveledFinishDate(Manager.getCalendar(
                rs, ID_LEVEL_FINISH));
            taskAssignment.setCalculatedLeveledStartDate(Manager.getCalendar(
                rs, ID_LEVEL_START));

            taskAssignment.setActualWork(new Double(actualWork));
            taskAssignment.setDaysOfDelay(new Integer(rs
                .getInt(ID_ASSIGNMENT_DELAY)));
            taskAssignment.setEETCHours(DateUtil.convertMinutesToHours(rs
                .getInt(ID_ETC)));
            taskAssignment.setExpectedWork(new Double(expectedWork));
            taskAssignment
                .setProposedDurationHours(new Double(proposedDuration));
            taskAssignment.setPlannedDurationHours(new Double(plannedDuration));
            taskAssignment.setPercentageComplete(percentageCompleted);
            taskAssignment.setComplete(isComplete);
            taskAssignment.setPercentageWork(new Double(percentageWork));
            taskAssignment.setRemainingHours(new Double(remainingHours));
            taskAssignment.setUnitsPercentage(unitsPercentage);
            taskAssignment.setLastScheduleWork(new Double(lastScheduleWork));

            taskAssignment.setBillable(new Boolean(billable));
            taskAssignment.setBaselineWork(new Double(baselineWork));
            taskAssignment.setWorkSchedule(new Double(workSchedule));
            taskAssignment.setCostPerHour(new Double(rs
                .getDouble(ID_REGULAR_COST)));
            taskAssignment.setDefaultLabor(new Boolean(defaultLabor));
            taskAssignment.setSellingRatePerHour(new Double(rs
                .getDouble(ID_REGULAR_RATE)));
            taskAssignment.setSpecialCostPerHour(new Double(rs
                .getDouble(ID_SPECIAL_COST)));
            taskAssignment.setSpecialSellingRatePerHour(new Double(rs
                .getDouble(ID_SPECIAL_RATE)));
            taskAssignment.setPercentageRateAdjustment(new Integer(rs
                .getInt(ID_RATE_ADJUSTMENT)));
            taskAssignment.setCostType(financialCategory);
            taskAssignment.setShownInTimesheet(shownInTimesheet);
        }
        return rpmObject;
    }

    private boolean decodeDefaultLabor(String defaultLaborString)
    {
        if (defaultLaborString != null
                && defaultLaborString.equalsIgnoreCase("Y"))
        {
            return true;
        }
        return false;
    }

    private String encodeDefaultLabor(Boolean defaultLabor)
    {
        if (defaultLabor != null && defaultLabor.booleanValue())
        {
            return "Y";
        }
        return "N";
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        //validate when is timecode used
        //		String timeCode1Id = resultSet.getString(ID_TIMECODE_1_ID);
        //		relatedFields.put(index, ID_TIMECODE_1_ID, timeCode1Id);
        //		
        //		String timeCode2Id = resultSet.getString(ID_TIMECODE_2_ID);
        //		relatedFields.put(index, ID_TIMECODE_2_ID, timeCode2Id);

        String parentId = resultSet.getString(ID_RES_ASSIGNMENT_ID);
        relatedFields.put(index, ID_RES_ASSIGNMENT_ID, parentId);

        String contractId = resultSet.getString(ID_CONTRACT_ID);
        relatedFields.put(index, ID_CONTRACT_ID, contractId);

        String laborCostId = resultSet.getString(ID_LABOR_COST_CC_ID);
        relatedFields.put(index, ID_LABOR_COST_CC_ID, laborCostId);

        String laborRevenueId = resultSet.getString(ID_LABOR_REVENUE_CC_ID);
        relatedFields.put(index, ID_LABOR_REVENUE_CC_ID, laborRevenueId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        TaskAssignment assignment = (TaskAssignment) rpmObject;

        if (rpmScope != null)
        {
            if (rpmScope.getParent() != null)
            {
                loadParentResourceTaskAssignment(assignment, rpmScope,
                    messageContext, relatedFields, index, isDelta);
            }

            if (rpmScope instanceof TaskAssignmentScope)
            {

                TaskAssignmentScope scope = (TaskAssignmentScope) rpmScope;

                if (scope.getContract() != null)
                {
                    loadContract(assignment, scope, messageContext,
                        relatedFields, index, isDelta);
                }

                if (scope.getCostLaborCode() != null)
                {
                    loadCostLaborCode(assignment, scope, messageContext,
                        relatedFields, index, isDelta);
                }

                if (scope.getRevenueLaborCode() != null)
                {
                    loadRevenueLabordCode(assignment, scope, messageContext,
                        relatedFields, index, isDelta);
                }
            }
        }

        return assignment;
    }

    private void loadParentResourceTaskAssignment(TaskAssignment assignment,
            RPMObjectScope rpmscope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldParentId = (String) relatedFields.get(index,
            ID_RES_ASSIGNMENT_ID);
        RPMObject newParent = null;

        ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) getManagerInstance(ResourceTaskAssignment.class
            .getName());

        if (isDelta)
        {
            newParent = assignment.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new ResourceTaskAssignment();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = manager.loadByPrimaryKey(newParent, rpmscope
                        .getParent(), messageContext, true);
                }
                else
                {
                    // In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = manager.loadByPrimaryKey(oldParent, rpmscope
                        .getParent(), messageContext, false);
                }
            }
            assignment.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                newParent = new ResourceTaskAssignment();
                newParent.setID(oldParentId);
                newParent = manager.loadByPrimaryKey(newParent, rpmscope
                    .getParent(), messageContext, false);
                assignment.setParent(newParent);
            }
        }
    }

    private void loadContract(TaskAssignment assignment,
            TaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        String contractId = (String) relatedFields.get(index, ID_CONTRACT_ID);
        if (contractId != null)
        {
            Contract contract = new Contract();
            contract.setID(contractId);
            ContractManager manager = (ContractManager) getManagerInstance(Contract.class
                .getName());
            contract = (Contract) manager.loadByPrimaryKey(contract, scope
                .getContract(), messageContext, isDelta);
            if (isDelta)
            {
                assignment.deltaContract(contract);
            }
            else
            {
                assignment.setContract(contract);
            }
        }
    }

    private void loadCostLaborCode(TaskAssignment assignment,
            TaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String costLaborCodeId = (String) relatedFields.get(index,
            ID_LABOR_COST_CC_ID);
        if (costLaborCodeId != null)
        {
            ChargeCode costLaborCode = loadChargeCode(costLaborCodeId, scope
                .getCostLaborCode(), messageContext, isDelta);

            if (isDelta)
                assignment.deltaCostLaborCode(costLaborCode);
            else
                assignment.setCostLaborCode(costLaborCode);
        }
    }

    private void loadRevenueLabordCode(TaskAssignment assignment,
            TaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String revenueLaborCodeId = (String) relatedFields.get(index,
            ID_LABOR_REVENUE_CC_ID);
        if (revenueLaborCodeId != null)
        {
            ChargeCode revenueLaborCode = loadChargeCode(revenueLaborCodeId,
                scope.getRevenueLaborCode(), messageContext, isDelta);

            if (isDelta)
                assignment.deltaRevenueLaborCode(revenueLaborCode);
            else
                assignment.setRevenueLaborCode(revenueLaborCode);
        }
    }

    private ChargeCode loadChargeCode(String chargeCodeId,
            ChargeCodeScope chargeCodeScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ChargeCode chargeCode = new ChargeCode();
        chargeCode.setID(chargeCodeId);
        ChargeCodeManager manager = (ChargeCodeManager) getManagerInstance(ChargeCode.class
            .getName());
        chargeCode = (ChargeCode) manager.loadByPrimaryKey(chargeCode,
            chargeCodeScope, messageContext, isDelta);
        return chargeCode;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TaskAssignment is ALWAYS deleted from ResourceTaskAssignment
        ExceptionUtil.handleUnsupportedAPICall(this, "TaskAssignment");
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceTaskAssignmentManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_RES_ASSIGNMENT_ID);
            where.append(" = ?");

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof TaskAssignmentManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_RES_ASSIGNMENT_ID);
            where.append(" = ? ");

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_RESOURCE_ID);
            where.append(" = ? ");

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public String getFilter()
    {

        //Excludes PersonalTaskAssignments and AdministrativeTaskAssignments

        SqlBuffer whereWbs = new SqlBuffer();
        whereWbs.append(WbsManager.NAME_TYPE_ID);
        whereWbs.append(" NOT ");
        whereWbs.appendIn();
        whereWbs.append(PersonalTask.TYPE_ID);
        whereWbs.appendComma();
        whereWbs.append(DefaultAdministrativeTask.TYPE_ID);
        whereWbs.appendCloseParenthesis();

        SqlBuffer sql = new SqlBuffer();
        sql.appendOpenParenthesis();
        sql.appendSubSelect(NAME_TASK_ID, WbsManager.NAME_ELEMENT_ID,
            WbsManager.TABLE_NAME, whereWbs.toString());
        sql.appendCloseParenthesis();
        sql.appendOr();
        sql.append(NAME_TASK_ID + " IN (SELECT " + ScopeManager.NAME_ELEMENT_ID
                + " FROM " + ScopeManager.TABLE_NAME);
        sql.appendWhere(ScopeManager.NAME_ELEMENT_ID + " IS NOT NULL");
        sql.appendCloseParenthesis();
        sql.appendCloseParenthesis();

        return sql.toString();
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        if (manager == null)
        {
            // example:
            // /Proposal[ID='123']/ResourceTaskAssignment/TaskAssignment
            joinCondition = new JoinCondition(TABLE_NAME, getFilter());

        }
        else if (manager instanceof ContractManager)
        {
            // example: /TaskAssignment/Contract
            joinCondition = JoinCondition.createSubSelect(
                ContractManager.NAME_ELEMENT_ID, NAME_CONTRACT_ID, TABLE_NAME,
                null);
        }
        else if (manager instanceof ChargeCodeManager)
        {
            // For XPath queries like:
            // /TaskAssignment/ChargeCode
            if ("costLaborCode".equalsIgnoreCase(container.getFieldName()))
            {
                joinCondition = JoinCondition.createSubSelect(
                    ChargeCodeManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_LABOR_COST_CC_ID, TABLE_NAME,
                    getFilter());
            }
            else
            {
                joinCondition = JoinCondition.createSubSelect(
                    ChargeCodeManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_LABOR_REVENUE_CC_ID, TABLE_NAME,
                    getFilter());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof ResourceTaskAssignmentManager)
        {
            // example: /ResourceTaskAssignment[TaskAssignment[ID='123']]
            joinCondition = JoinCondition.createSubSelect(
                ResourceTaskAssignmentManager.NAME_RES_ASSIGNMENT_ID,
                NAME_RES_ASSIGNMENT_ID, TABLE_NAME, condition);
        }
        else if (manager instanceof SummaryTimesheetManager)
        {
            // example: /SummaryTimesheet[TaskAssignment]
            joinCondition = JoinCondition.createSubSelect(
                SummaryTimesheetManager.NAME_TSK_ASSIGNMENT_ID,
                NAME_TSK_ASSIGNMENT_ID, TABLE_NAME, condition);
        }
        else if (manager instanceof StepManager)
        {
            // example: /Activity[TaskAssignment]
            joinCondition = JoinCondition.createSubSelect(
                StepManager.NAME_TSK_ASSIGNMENT_ID, NAME_TSK_ASSIGNMENT_ID,
                TABLE_NAME, condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        // TaskAssignment is ALWAYS created from ResourceTaskAssignment

        // Code below is a hack to get validate->create->update sequence working
        // Nobody should ever call this method directly

        // Retrieving ID that was already created by SP_SEARCH_2 call in
        // ResourceTaskAssignmentManager
        TaskAssignment taskAssignment = (TaskAssignment) rpmObject;
        // Assuming (isValid):
        // taskAssignment.getParent() != null
        // taskAssignment.getParent().getID() != null

        //Reload the same object to get the ID as the SP doesn't return one
        ArrayList reloadedObjects = loadByForeignKey(null, null,
            messageContext, this, new Object[]
            {
                taskAssignment.getParent().getID()
            }, null, null, false);

        if (reloadedObjects == null)
        {
            // TODO try to delete ResourceTaskAssignment also
            // So there is no garbage left in the database 
            throw new RPMException(
                "TaskAssignmentManager.createObject : TaskAssignment "
                        + "object was not created by the stored procedure.");
        }
        else if (reloadedObjects.size() == 1)
        {
            TaskAssignment object = (TaskAssignment) reloadedObjects.get(0);
            taskAssignment.setID(object.getID());
            taskAssignment.setCostType(object.getCostType());
        }
        else
        {
            throw new RPMException(
                "TaskAssignmentManager.createObject : One object TaskAssignment "
                        + "expected, instead got " + reloadedObjects.size());
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        TaskAssignment assignment = (TaskAssignment) rpmObject;
        TaskAssignmentScope scope = (TaskAssignmentScope) rpmScope;

        if (assignment.testPercentageWorkModified()
                || assignment.testUnitsPercentageModified()
                || assignment.testActualWorkModified()
                || assignment.testAssignmentStartDateModified()
                || assignment.testAssignmentFinishDateModified()
                || assignment.testDaysOfDelayModified()
                || assignment.testPercentageCompleteModified())
        {
            SP_U_PARTICIPANT(assignment, messageContext);
        }

        if (assignment.testEETCHoursModified())
        {
            SP_U_ASSIGN_ETC(assignment, messageContext);
        }

        if (assignment.testDefaultLaborModified())
        {
            SP_U_TA_DEFAULT_LABOR(assignment, messageContext);
        }

        if (assignment.testBillableModified()
                || assignment.testCostTypeModified())
        {
            SP_U_TA_BILLABLE(assignment, messageContext);
        }

        if (assignment.testPercentageRateAdjustmentModified()
                || assignment.testCostPerHourModified()
                || assignment.testSpecialCostPerHourModified()
                || assignment.testSellingRatePerHourModified()
                || assignment.testSpecialSellingRatePerHourModified())
        {
            SP_U_RATES(assignment, messageContext);
        }

        if (scope != null)
        {
            if (scope.getContract() != null && assignment.getContract() != null)
            {
                SP_A_CONTRACT(assignment, messageContext);
            }

            if ((scope.getCostLaborCode() != null)
                    || (scope.getRevenueLaborCode() != null))
            {
                SP_U_TA_LABOR_CC(assignment, messageContext);
            }

        }

        if (assignment.testShownInTimesheetModified())
        {
            Integer configFlag = ManagerUtil.toInteger(assignment
                .getShownInTimesheet());
            SP_U_TA_FLAG(assignment.getID(), configFlag, messageContext);
        }

        return rpmObject;
    }

    public TaskAssignment acceptRejectTaskAssignmentChanges(
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean accept, boolean EETC, boolean forecastDates,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ReloadType reloadType = messageContext.getReloadType();
        doAcceptRejectTaskAssignmentChanges(taskAssignment, scope, accept,
            EETC, forecastDates, messageContext);
        TaskAssignment outputAssignment = null;
        // Reload graph (default behavior) if the save was successful
        if (taskAssignment != null
                && getPrimaryKeyString(taskAssignment) != null)
        {
            if (reloadType == null || reloadType == ReloadType.ReloadResult)
            {
                outputAssignment = (TaskAssignment) load(taskAssignment, scope,
                    messageContext);
            }
        }

        messageContext.getCache().resetLoadedContainers();

        return outputAssignment;
    }

    private void doAcceptRejectTaskAssignmentChanges(
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean accept, boolean EETC, boolean forecastDates,
            MessageContext messageContext) throws RPMException
    {
        WbsMethodCheckpoint checkpoint = WbsMethodCheckpoint.getInstance();

        if (!checkpoint.validateAcceptRejectArguments(taskAssignment, true,
            EETC, forecastDates, messageContext))
        {
            return;
        }

        // Figure out the bit-flag needed by the stored procedure
        int flag = 0;
        if (EETC && forecastDates)
            flag = 3;
        else if (EETC)
            flag = 1;
        else if (forecastDates)
            flag = 2;

        // Figure out if the action is permitted in the first place
        if (EETC
                && !checkpoint.canApproveRejectEETC(taskAssignment, true,
                    messageContext))
        {
            return;
        }

        if (forecastDates
                && !checkpoint.canApproveRejectForecastDate(taskAssignment,
                    true, messageContext))
        {
            return;
        }

        try
        {
            if (checkpoint.canAcceptRejectTaskAssignmentChanges(taskAssignment,
                scope, accept, EETC, forecastDates, messageContext))
            {
                if (accept)
                    SP_ACPT_ETC_ASSIGN(taskAssignment, flag, messageContext);
                else
                    SP_RJCT_ETC_ASSIGN(taskAssignment, flag, messageContext);
            }
        }
        catch (SQLException e)
        {
            ExceptionUtil.handleException(this, e);
        }
    }

    private void SP_U_ASSIGN_ETC(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ASSIGN_ETC";
        Object args[] = new Object[4];

        args[0] = assignment.getID();
        args[1] = DateUtil.convertHoursToMinutes(assignment.getEETCHours());

        //TODO implement security later
        //ACC_REJ_BITFLAG : 1 has rights to accept/reject etc, 0 does not have
        // rights
        args[2] = new Integer(1);
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_PARTICIPANT(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, ParseException,
            RPMException
    {
        //ResourceTaskAssignment resourceTaskAssignment =
        // (ResourceTaskAssignment) assignment
        //        .getParent();
        //[{CALL
        // SP_U_PARTICIPANT('97E50B0FBB1248EDB5087FC507716D85','SCTASKRESPONSIBLE','2005-10-28','2005-11-07
        // 12:00:00.0',0,10000,0,0,'PMO_SUPERVISOR')}]

        // SP_U_PARTICIPANT assign_id='CDB141D9FC4D49138E1EA420A8A74D0E'
        // security_id='SCTASKPARTICIPANT' planned_start=null
        // planned_finish=null delay=0 units=0 work_percent=0 actual=0
        // res_id='PMO_SUPERVISOR'

        ResourceTaskAssignment parent = (ResourceTaskAssignment) assignment
            .getParent();

        if (parent == null)
        {
            // if the parent is null, we need to load it in order to use
            // the securityRole field
            TaskAssignmentScope taScope = new TaskAssignmentScope();
            taScope.setParent(new ResourceTaskAssignmentScope());

            TaskAssignment ta = (TaskAssignment) load(assignment, taScope,
                messageContext);
            parent = (ResourceTaskAssignment) ta.getParent();
        }

        int actualWorkMinutes = 0;
        if (assignment.getActualWork() != null
                && assignment.getActualWork().doubleValue() != 0)
        {
            actualWorkMinutes = DateUtil.convertHoursToMinutes(
                assignment.getActualWork()).intValue();
        }

        int percentageComplete = 0;

        // only one of these three fields can be modified at one time b/c 
        // they affect each other; this is enforced in TaskAssignmentCheckpoint
        if (assignment.testActualWorkModified())
        {
            if (assignment.getActualWork() != null)
            {
                actualWorkMinutes = DateUtil.convertHoursToMinutes(
                    assignment.getActualWork()).intValue();
            }
        }
        else if (assignment.testPercentageWorkModified())
        {
            if (assignment.getExpectedWork() != null
                    && assignment.getPercentageWork() != null)
            {
                actualWorkMinutes = DateUtil.convertHoursToMinutes(
                    new Double(assignment.getExpectedWork().doubleValue()
                            * assignment.getPercentageWork().doubleValue()
                            / 100)).intValue();
            }
        }
        else if (assignment.testPercentageCompleteModified())
        {
            if (assignment.getPercentageComplete() != null)
            {
                percentageComplete = (int) (assignment.getPercentageComplete()
                    .doubleValue() * 100);
            }
        }

        int unitPercentage = 0;
        if (assignment.getUnitsPercentage() != null)
        {
            unitPercentage = (int) (assignment.getUnitsPercentage()
                .doubleValue() * 100);
        }

        String name = "SP_U_PARTICIPANT";
        Object args[] = new Object[9];

        if (assignment.getID() == null)
        {
            throw new RPMException("Cannot update taskAssignment with ID=null");
        }
        args[0] = assignment.getID();
        if (parent != null && parent.getSecurityRole() != null)
        {
            args[1] = parent.getSecurityRole().getID();
        }
        //Start date
        args[2] = DateUtil.convertToStringWithTime(assignment
            .getAssignmentStartDate()); //Planned

        // finish date is a readonly field and is calculated when modifications are made
        //to the start date or the resource effort.
        // However, it is required when plannedDurationHours exists
        if (assignment.getPlannedDurationHours() != null)
        {
            args[3] = DateUtil.convertToStringWithTime(assignment
                .getAssignmentFinishDate());
        }
        args[4] = assignment.getDaysOfDelay(); //Delay
        args[5] = new Integer(unitPercentage);
        args[6] = new Integer(percentageComplete);
        args[7] = new Double(actualWorkMinutes);
        args[8] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_A_CONTRACT(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        //TODO this should be included in the validation
        RPMObject grandParent = assignment.getParent().getParent();
        if (grandParent instanceof ScopeElement)
            return;

        String name = "SP_A_CONTRACT";

        Object args[] = new Object[3];

        args[0] = assignment.getID();
        args[1] = assignment.getContract().getID();
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_TA_DEFAULT_LABOR(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_TA_DEFAULT_LABOR";

        Object args[] = new Object[3];

        args[0] = assignment.getID();
        args[1] = encodeDefaultLabor(assignment.getDefaultLabor());
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_TA_BILLABLE(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        int billable = 0;
        if (assignment.getBillable().booleanValue())
            billable = 1;

        final String procedureName = "SP_U_TA_BILLABLE";
        Object args[] = new Object[4];

        args[0] = assignment.getID(); // task_assignment_id
        args[1] = new Long(billable); // billable
        args[2] = FinancialCategoryUtil.toDBCode(assignment.getCostType()); // capital_expense
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_RATES(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) assignment
            .getParent();

        final String procedureName = "SP_U_RATES";
        Object args[] = new Object[7];

        args[0] = assignment.getID(); // id
        args[1] = assignment.getCostPerHour(); // reg_cost
        args[2] = assignment.getSpecialCostPerHour(); // spec_cost
        args[3] = assignment.getSellingRatePerHour(); // reg_rate
        args[4] = assignment.getSpecialSellingRatePerHour(); // spec_rate
        args[5] = assignment.getPercentageRateAdjustment(); // rate_adjustment

        // if the parent's resource and profile are both null, load them
        if (resourceTaskAssignment.getResource() == null
                && resourceTaskAssignment.getProfile() == null)
        {
            ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
            rtaScope.setProfile(new ProfileScope());
            rtaScope.setResource(new ResourceScope());

            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) getManagerInstance(ResourceTaskAssignment.class
                .getName());
            resourceTaskAssignment = (ResourceTaskAssignment) manager.load(
                resourceTaskAssignment, rtaScope, messageContext);
        }

        args[6] = resourceTaskAssignment.getResource() != null
                ? resourceTaskAssignment.getResource().getID()
                : resourceTaskAssignment.getProfile().getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_TA_LABOR_CC(TaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_TA_LABOR_CC";
        Object args[] = new Object[4];

        args[0] = assignment.getID(); // task_assignment_id
        args[1] = (assignment.getCostLaborCode() != null) ? assignment
            .getCostLaborCode().getID() : null; // labor_cost_cc_id
        args[2] = (assignment.getRevenueLaborCode() != null) ? assignment
            .getRevenueLaborCode().getID() : null;// labor_revenue_cc_id
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_ACPT_ETC_ASSIGN(RPMObject object, int flag,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_ACPT_ETC_ASSIGN";
        Object args[] = new Object[3];

        args[0] = object.getID(); // id
        args[1] = new Long(flag); // bitflag
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_RJCT_ETC_ASSIGN(RPMObject object, int flag,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_RJCT_ETC_ASSIGN";
        Object args[] = new Object[3];

        args[0] = object.getID(); // id
        args[1] = new Long(flag);
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_TA_FLAG(String id, Integer configFlag,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = id;
        args[1] = configFlag;
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_TA_FLAG", args);
    }
}