package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.ScheduleDateUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;

public class WbsScheduleDateManager extends AbstractRPMObjectManager
{
    //actualDate
    private static final HashMap ACTUALPROPERTYMAP = new HashMap();

    // baseline
    private static final HashMap BASELINE1PROPERTYMAP = new HashMap();

    //previousBaseline
    private static final HashMap BASELINE2PROPERTYMAP = new HashMap();

    //initialBaseline
    private static final HashMap BASELINE3PROPERTYMAP = new HashMap();

    //forecastDate
    private static final HashMap FORECASTPROPERTYMAP = new HashMap();

    //proposedDate
    private static final HashMap PLANNEDPROPERTYMAP = new HashMap();

    //expectedDate
    private static final HashMap PROPOSEDPROPERTYMAP = new HashMap();

    //plannedDate
    private static final HashMap SCHEDULEDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(WbsScheduleDate.class.getName());

        String[] objs = new String[CONTAINERS.size()];

        CONTAINERS.toArray(objs);

        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                WbsScheduleDateManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WBS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        ACTUALPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        SCHEDULEDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        FORECASTPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        PLANNEDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        PROPOSEDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        BASELINE1PROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        BASELINE2PROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
        BASELINE3PROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column BASELINE1_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_START = 2;

    public static final int TYPE_BASELINE1_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_START = "TMT_WBS.BASELINE1_START";

    public static final String PROPERTY_BASELINE1_START = "STARTDATE";
    static
    {
        BASELINE1PROPERTYMAP
            .put(PROPERTY_BASELINE1_START, NAME_BASELINE1_START);
    }

    /**
     * Column BASELINE1_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_FINISH = 3;

    public static final int TYPE_BASELINE1_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_FINISH = "TMT_WBS.BASELINE1_FINISH";

    public static final String PROPERTY_BASELINE1_FINISH = "FINISHDATE";
    static
    {
        BASELINE1PROPERTYMAP.put(PROPERTY_BASELINE1_FINISH,
            NAME_BASELINE1_FINISH);
    }

    /**
     * Column BASELINE2_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_START = 4;

    public static final int TYPE_BASELINE2_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_START = "TMT_WBS.BASELINE2_START";

    public static final String PROPERTY_BASELINE2_START = "STARTDATE";
    static
    {
        BASELINE2PROPERTYMAP
            .put(PROPERTY_BASELINE2_START, NAME_BASELINE2_START);
    }

    /**
     * Column BASELINE2_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_FINISH = 5;

    public static final int TYPE_BASELINE2_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_FINISH = "TMT_WBS.BASELINE2_FINISH";

    public static final String PROPERTY_BASELINE2_FINISH = "FINISHDATE";
    static
    {
        BASELINE2PROPERTYMAP.put(PROPERTY_BASELINE2_FINISH,
            NAME_BASELINE2_FINISH);
    }

    /**
     * Column BASELINE3_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_START = 6;

    public static final int TYPE_BASELINE3_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_START = "TMT_WBS.BASELINE3_START";

    public static final String PROPERTY_BASELINE3_START = "STARTDATE";
    static
    {
        BASELINE3PROPERTYMAP
            .put(PROPERTY_BASELINE3_START, NAME_BASELINE3_START);
    }

    /**
     * Column BASELINE3_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_FINISH = 7;

    public static final int TYPE_BASELINE3_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_FINISH = "TMT_WBS.BASELINE3_FINISH";

    public static final String PROPERTY_BASELINE3_FINISH = "FINISHDATE";
    static
    {
        BASELINE2PROPERTYMAP.put(PROPERTY_BASELINE3_FINISH,
            NAME_BASELINE3_FINISH);
    }

    /**
     * Column PLANNED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_START = 8;

    public static final int TYPE_PLANNED_START = Types.TIMESTAMP;

    public static final String NAME_PLANNED_START = "TMT_WBS.PLANNED_START";

    public static final String PROPERTY_PLANNED_START = "STARTDATE";
    static
    {
        PLANNEDPROPERTYMAP.put(PROPERTY_PLANNED_START, NAME_PLANNED_START);
    }

    /**
     * Column PLANNED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_FINISH = 9;

    public static final int TYPE_PLANNED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PLANNED_FINISH = "TMT_WBS.PLANNED_FINISH";

    public static final String PROPERTY_PLANNED_FINISH = "FINISHDATE";
    static
    {
        PLANNEDPROPERTYMAP.put(PROPERTY_PLANNED_FINISH, NAME_PLANNED_FINISH);
    }

    /**
     * Column PROPOSED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_START = 10;

    public static final int TYPE_PROPOSED_START = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_START = "TMT_WBS.PROPOSED_START";

    public static final String PROPERTY_PROPOSED_START = "STARTDATE";
    static
    {
        PROPOSEDPROPERTYMAP.put(PROPERTY_PROPOSED_START, NAME_PROPOSED_START);
    }

    /**
     * Column PROPOSED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_FINISH = 11;

    public static final int TYPE_PROPOSED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_FINISH = "TMT_WBS.PROPOSED_FINISH";

    public static final String PROPERTY_PROPOSED_FINISH = "FINISHDATE";
    static
    {
        PROPOSEDPROPERTYMAP.put(PROPERTY_PROPOSED_FINISH, NAME_PROPOSED_FINISH);
    }

    /**
     * Column SCHED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_START = 12;

    public static final int TYPE_SCHED_START = Types.TIMESTAMP;

    public static final String NAME_SCHED_START = "TMT_WBS.SCHED_START";

    public static final String PROPERTY_SCHED_START = "STARTDATE";
    static
    {
        SCHEDULEDPROPERTYMAP.put(PROPERTY_SCHED_START, NAME_SCHED_START);
    }

    /**
     * Column SCHED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_FINISH = 13;

    public static final int TYPE_SCHED_FINISH = Types.TIMESTAMP;

    public static final String NAME_SCHED_FINISH = "TMT_WBS.SCHED_FINISH";

    public static final String PROPERTY_SCHED_FINISH = "FINISHDATE";
    static
    {
        SCHEDULEDPROPERTYMAP.put(PROPERTY_SCHED_FINISH, NAME_SCHED_FINISH);
    }

    /**
     * Column FORECAST_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_START = 14;

    public static final int TYPE_FORECAST_START = Types.TIMESTAMP;

    public static final String NAME_FORECAST_START = "TMT_WBS.FORECAST_START";

    public static final String PROPERTY_FORECAST_START = "STARTDATE";
    static
    {
        FORECASTPROPERTYMAP.put(PROPERTY_FORECAST_START, NAME_FORECAST_START);
    }

    /**
     * Column FORECAST_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_FINISH = 15;

    public static final int TYPE_FORECAST_FINISH = Types.TIMESTAMP;

    public static final String NAME_FORECAST_FINISH = "TMT_WBS.FORECAST_FINISH";

    public static final String PROPERTY_FORECAST_FINISH = "FINISHDATE";
    static
    {
        FORECASTPROPERTYMAP.put(PROPERTY_FORECAST_FINISH, NAME_FORECAST_FINISH);
    }

    /**
     * Column ACTUAL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_START = 16;

    public static final int TYPE_ACTUAL_START = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_START = "TMT_WBS.ACTUAL_START";

    public static final String PROPERTY_ACTUAL_START = "STARTDATE";
    static
    {
        ACTUALPROPERTYMAP.put(PROPERTY_ACTUAL_START, NAME_ACTUAL_START);
    }

    /**
     * Column ACTUAL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_FINISH = 17;

    public static final int TYPE_ACTUAL_FINISH = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_FINISH = "TMT_WBS.ACTUAL_FINISH";

    public static final String PROPERTY_ACTUAL_FINISH = "PROPERTYNAME";
    static
    {
        ACTUALPROPERTYMAP.put(PROPERTY_ACTUAL_FINISH, NAME_ACTUAL_FINISH);
    }

    /**
     * Column PLANNED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_DURATION = 18;

    public static final int TYPE_PLANNED_DURATION = Types.INTEGER;

    public static final String NAME_PLANNED_DURATION = "TMT_WBS.PLANNED_DURATION";

    public static final String PROPERTY_PLANNED_DURATION = "PROPERTYNAME";
    static
    {
        PLANNEDPROPERTYMAP
            .put(PROPERTY_PLANNED_DURATION, NAME_PLANNED_DURATION);
    }

    /**
     * Column PROPOSED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_DURATION = 19;

    public static final int TYPE_PROPOSED_DURATION = Types.INTEGER;

    public static final String NAME_PROPOSED_DURATION = "TMT_WBS.PROPOSED_DURATION";

    public static final String PROPERTY_PROPOSED_DURATION = "DURATIONINMINUTES";

    static
    {
        PROPOSEDPROPERTYMAP.put(PROPERTY_PROPOSED_DURATION,
            NAME_PROPOSED_DURATION);
    }

    /**
     * Column SCHED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_DURATION = 20;

    public static final int TYPE_SCHED_DURATION = Types.INTEGER;

    public static final String NAME_SCHED_DURATION = "TMT_WBS.SCHED_DURATION";

    public static final String PROPERTY_SCHED_DURATION = "DURATIONINMINUTES";
    static
    {
        SCHEDULEDPROPERTYMAP.put(PROPERTY_SCHED_DURATION, NAME_SCHED_DURATION);
    }

    /**
     * Column FORECAST_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_DURATION = 21;

    public static final int TYPE_FORECAST_DURATION = Types.INTEGER;

    public static final String NAME_FORECAST_DURATION = "TMT_WBS.FORECAST_DURATION";

    public static final String PROPERTY_FORECAST_DURATION = "DURATIONINMINUTES";

    static
    {
        FORECASTPROPERTYMAP.put(PROPERTY_FORECAST_DURATION,
            NAME_FORECAST_DURATION);
    }

    /**
     * Column ACTUAL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_DURATION = 22;

    public static final int TYPE_ACTUAL_DURATION = Types.INTEGER;

    public static final String NAME_ACTUAL_DURATION = "TMT_WBS.ACTUAL_DURATION";

    public static final String PROPERTY_ACTUAL_DURATION = "DURATIONINMINUTES";
    static
    {
        ACTUALPROPERTYMAP.put(PROPERTY_ACTUAL_DURATION, NAME_ACTUAL_DURATION);
    }

    /**
     * Column BASELINE1_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_DURATION = 23;

    public static final int TYPE_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE1_DURATION = "TMT_WBS.BASELINE1_DURATION";

    public static final String PROPERTY_BASELINE1_DURATION = "DURATIONINMINUTES";

    static
    {
        BASELINE1PROPERTYMAP.put(PROPERTY_BASELINE1_DURATION,
            NAME_BASELINE1_DURATION);
    }

    /**
     * Column BASELINE2_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_DURATION = 24;

    public static final int TYPE_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE2_DURATION = "TMT_WBS.BASELINE2_DURATION";

    public static final String PROPERTY_BASELINE2_DURATION = "DURATIONINMINUTES";

    static
    {
        BASELINE2PROPERTYMAP.put(PROPERTY_BASELINE2_DURATION,
            NAME_BASELINE2_DURATION);
    }

    /**
     * Column BASELINE3_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_DURATION = 25;

    public static final int TYPE_BASELINE3_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE3_DURATION = "TMT_WBS.BASELINE3_DURATION";

    public static final String PROPERTY_BASELINE3_DURATION = "DURATIONINMINUTES";

    static
    {
        BASELINE3PROPERTYMAP.put(PROPERTY_BASELINE3_DURATION,
            NAME_BASELINE3_DURATION);
    }

    /**
     * Column MINUTE_PER_DAY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINUTE_PER_DAY = 26;

    public static final int TYPE_MINUTE_PER_DAY = Types.SMALLINT;

    public static final String NAME_MINUTE_PER_DAY = "TMT_WBS.MINUTE_PER_DAY";

    /**
     * Column PLANNED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PLANNED_EFFORT = 27;

    public static final int TYPE_PLANNED_EFFORT = Types.BIGINT;

    public static final String NAME_PLANNED_EFFORT = "TMT_WBS.PLANNED_EFFORT";

    public static final String PROPERTY_PLANNED_EFFORT = "WORKHOURS";
    static
    {
        PLANNEDPROPERTYMAP.put(PROPERTY_PLANNED_EFFORT, NAME_PLANNED_EFFORT);
    }

    /**
     * Column PROPOSED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PROPOSED_EFFORT = 28;

    public static final int TYPE_PROPOSED_EFFORT = Types.BIGINT;

    public static final String NAME_PROPOSED_EFFORT = "TMT_WBS.PROPOSED_EFFORT";

    public static final String PROPERTY_PROPOSED_EFFORT = "WORKHOURS";
    static
    {
        PROPOSEDPROPERTYMAP.put(PROPERTY_PROPOSED_EFFORT, NAME_PROPOSED_EFFORT);
    }

    /**
     * Column SCHED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHED_EFFORT = 29;

    public static final int TYPE_SCHED_EFFORT = Types.BIGINT;

    public static final String NAME_SCHED_EFFORT = "TMT_WBS.SCHED_EFFORT";

    public static final String PROPERTY_SCHED_EFFORT = "WORKHOURS";
    static
    {
        SCHEDULEDPROPERTYMAP.put(PROPERTY_SCHED_EFFORT, NAME_SCHED_EFFORT);
    }

    /**
     * Column FORECAST_EFFORT of type Types.BIGINT mapped to Long
     */
    public static final int ID_FORECAST_EFFORT = 30;

    public static final int TYPE_FORECAST_EFFORT = Types.BIGINT;

    public static final String NAME_FORECAST_EFFORT = "TMT_WBS.FORECAST_EFFORT";

    public static final String PROPERTY_FORECAST_EFFORT = "WORKHOURS";
    static
    {
        FORECASTPROPERTYMAP.put(PROPERTY_FORECAST_EFFORT, NAME_FORECAST_EFFORT);
    }

    /**
     * Column WORK_TODATE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_WORK_TODATE = 31;

    public static final int TYPE_WORK_TODATE = Types.BIGINT;

    public static final String NAME_WORK_TODATE = "TMT_WBS.WORK_TODATE";

    public static final String PROPERTY_WORK_TODATE = "WORKHOURS";
    static
    {
        ACTUALPROPERTYMAP.put(PROPERTY_WORK_TODATE, NAME_WORK_TODATE);
    }

    /**
     * Column BASELINE1_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE1_EFFORT = 32;

    public static final int TYPE_BASELINE1_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE1_EFFORT = "TMT_WBS.BASELINE1_EFFORT";

    public static final String PROPERTY_BASELINE1_EFFORT = "WORKHOURS";
    static
    {
        BASELINE1PROPERTYMAP.put(PROPERTY_BASELINE1_EFFORT,
            NAME_BASELINE1_EFFORT);
    }

    /**
     * Column BASELINE2_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE2_EFFORT = 33;

    public static final int TYPE_BASELINE2_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE2_EFFORT = "TMT_WBS.BASELINE2_EFFORT";

    public static final String PROPERTY_BASELINE2_EFFORT = "WORKHOURS";
    static
    {
        BASELINE2PROPERTYMAP.put(PROPERTY_BASELINE2_EFFORT,
            NAME_BASELINE2_EFFORT);
    }

    /**
     * Column BASELINE3_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE3_EFFORT = 34;

    public static final int TYPE_BASELINE3_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE3_EFFORT = "TMT_WBS.BASELINE3_EFFORT";

    public static final String PROPERTY_BASELINE3_EFFORT = "WORKHOURS";
    static
    {
        BASELINE3PROPERTYMAP.put(PROPERTY_BASELINE3_EFFORT,
            NAME_BASELINE3_EFFORT);
    }

    public static final String TABLE_NAME = "TMT_WBS";

    /**
     * Create an array of type string containing all the fields of the TMT_WBS
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_BASELINE1_START, NAME_BASELINE1_FINISH,
            NAME_BASELINE2_START, NAME_BASELINE2_FINISH, NAME_BASELINE3_START,
            NAME_BASELINE3_FINISH, NAME_PLANNED_START, NAME_PLANNED_FINISH,
            NAME_PROPOSED_START, NAME_PROPOSED_FINISH, NAME_SCHED_START,
            NAME_SCHED_FINISH, NAME_FORECAST_START, NAME_FORECAST_FINISH,
            NAME_ACTUAL_START, NAME_ACTUAL_FINISH, NAME_PLANNED_DURATION,
            NAME_PROPOSED_DURATION, NAME_SCHED_DURATION,
            NAME_FORECAST_DURATION, NAME_ACTUAL_DURATION,
            NAME_BASELINE1_DURATION, NAME_BASELINE2_DURATION,
            NAME_BASELINE3_DURATION, NAME_MINUTE_PER_DAY, NAME_PLANNED_EFFORT,
            NAME_PROPOSED_EFFORT, NAME_SCHED_EFFORT, NAME_FORECAST_EFFORT,
            NAME_WORK_TODATE, NAME_BASELINE1_EFFORT, NAME_BASELINE2_EFFORT,
            NAME_BASELINE3_EFFORT

    };

    private static final String ALL_FIELDS;

    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);

        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }

        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new WBSManager instance.
     * 
     * @return WBSManager
     */
    public WbsScheduleDateManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap(String fieldName, 
            String containerName) throws RPMException
    {
        HashMap fieldPropertyMap = null;

        if (fieldName.equalsIgnoreCase(WbsScheduleDate.ACTUAL))
        {
            fieldPropertyMap = ACTUALPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.EXPECTED))
        {
            fieldPropertyMap = PROPOSEDPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.FORECAST))
        {
            fieldPropertyMap = FORECASTPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.PLAN))
        {
            fieldPropertyMap = SCHEDULEDPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.PROPOSED))
        {
            fieldPropertyMap = PLANNEDPROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.BASELINE))
        {
            fieldPropertyMap = BASELINE1PROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.PREVIOUSBASELINE))
        {
            fieldPropertyMap = BASELINE2PROPERTYMAP;
        }
        else if (fieldName.equalsIgnoreCase(WbsScheduleDate.INITIALBASELINE))
        {
            fieldPropertyMap = BASELINE3PROPERTYMAP;
        }

        else
        {
            ExceptionUtil.handleUnsupportedContext(this, fieldName,
                ExceptionUtil.GET_PROPERTIES);
        }

        return fieldPropertyMap;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new WbsScheduleDate();
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {

        //Nothing to do
    }

    public boolean containerExistInCache(Object[] primaryKey,
            MessageContext messageContext)
    {
        //The cache cannot be used for this type of container because the key
        // is not unique.
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        WbsScheduleDate object = (WbsScheduleDate) rpmObject;
        object.setID(rs.getString(ID_ELEMENT_ID));

        //Use the contextName field to keep the name of the field we are
        // loading
        String contextName = rpmObject.getContextName();

        int startDateIndex = 0;
        int finishDateIndex = 0;
        int durationIndex = 0;
        int effortIndex = 0;

        if (contextName.equalsIgnoreCase(WbsScheduleDate.ACTUAL))
        {
            startDateIndex = ID_ACTUAL_START;
            finishDateIndex = ID_ACTUAL_FINISH;
            durationIndex = ID_ACTUAL_DURATION;
            effortIndex = ID_WORK_TODATE;

        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.EXPECTED))
        {
            startDateIndex = ID_PROPOSED_START;
            finishDateIndex = ID_PROPOSED_FINISH;
            durationIndex = ID_PROPOSED_DURATION;
            effortIndex = ID_PROPOSED_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.FORECAST))
        {
            startDateIndex = ID_FORECAST_START;
            finishDateIndex = ID_FORECAST_FINISH;
            durationIndex = ID_FORECAST_DURATION;
            effortIndex = ID_FORECAST_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.PLAN))
        {
            startDateIndex = ID_SCHED_START;
            finishDateIndex = ID_SCHED_FINISH;
            durationIndex = ID_SCHED_DURATION;
            effortIndex = ID_SCHED_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.PROPOSED))
        {
            startDateIndex = ID_PLANNED_START;
            finishDateIndex = ID_PLANNED_FINISH;
            durationIndex = ID_PLANNED_DURATION;
            effortIndex = ID_PLANNED_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.BASELINE))
        {
            startDateIndex = ID_BASELINE1_START;
            finishDateIndex = ID_BASELINE1_FINISH;
            durationIndex = ID_BASELINE1_DURATION;
            effortIndex = ID_BASELINE1_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.PREVIOUSBASELINE))
        {
            startDateIndex = ID_BASELINE2_START;
            finishDateIndex = ID_BASELINE2_FINISH;
            durationIndex = ID_BASELINE2_DURATION;
            effortIndex = ID_BASELINE2_EFFORT;
        }
        else if (contextName.equalsIgnoreCase(WbsScheduleDate.INITIALBASELINE))
        {
            startDateIndex = ID_BASELINE3_START;
            finishDateIndex = ID_BASELINE3_FINISH;
            durationIndex = ID_BASELINE3_DURATION;
            effortIndex = ID_BASELINE3_EFFORT;
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, contextName,
                ExceptionUtil.GET_PROPERTIES);
        }

        Calendar startDate = null;
        Calendar finishDate = null;
        startDate = Manager.getTimestamp(rs, startDateIndex);
        finishDate = Manager.getTimestamp(rs, finishDateIndex);

        int mins = rs.getInt(ID_MINUTE_PER_DAY);
        int duration = rs.getInt(durationIndex);
        int effort = rs.getInt(effortIndex);

        double days = (double)duration / mins;
        int hours = (duration % mins) / 60;
        double workEffort = (double) effort / 60.0;

        if (isDelta)
        {
            object.deltaStartDate(startDate);
            object.deltaFinishDate(finishDate);
            //This is done to avoid setting the modified flag to true when the
            // data was not
            // modified we assume that a null value is equal to 0 in the
            // database
            if (!(object.getDurationInMinutes() == null && duration == 0))
            {
                object.deltaDurationInMinutes(new Integer(duration));
            }
            if (!(object.getDays() == null && days == 0))
            {
                object.deltaDays(new Double(days));
            }
            if (!(object.getHours() == null && hours == 0))
            {
                object.deltaHours(new Double(hours));
            }
            if (!(object.getWorkHours() == null && workEffort == 0))
            {
                object.deltaWorkHours(new Double(workEffort));
            }
        }
        else
        {
            object.setStartDate(startDate);
            object.setFinishDate(finishDate);
            object.setDurationInMinutes(new Integer(duration));
            object.setDays(new Double(days));
            object.setHours(new Double(hours));
            object.setWorkHours(new Double(workEffort));
        }

        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return null;

    }

    /**
     *  
     */

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            //TODO validate
            if (condition != null)
            {
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.FOREIGN_KEY);

        return null;
    }

    private boolean testScheduleDateModified(WbsScheduleDate wbsScheduleDate)
    {
        if (wbsScheduleDate != null)
        {
            return wbsScheduleDate.testDaysModified()
                    || wbsScheduleDate.testFinishDateModified()
                    || wbsScheduleDate.testHoursModified()
                    || wbsScheduleDate.testStartDateModified()
                    || wbsScheduleDate.testWorkHoursModified()
                    || wbsScheduleDate.testDurationInMinutesModified();
        }
        return false;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        WbsScheduleDate object = (WbsScheduleDate) rpmObject;

        if (object != null && testScheduleDateModified(object))
        {

            String contextName = rpmObject.getContextName();
            
            object.setDurationInMinutes(new Integer(ScheduleDateUtil
                .roundDurationUpToQuarterHour(object.getDurationInMinutes())));

            //Schedule Dates.

            //ScheduleDates are read-only un the folowing contexts:
            //Expected, Baseline, PreviousBaeline, InitialBaseline

            //SP_U_PLANNED
            if (contextName.equalsIgnoreCase(WbsScheduleDate.PROPOSED))
            {
                SP_U_PLANNED(object, messageContext);
            }

            //SP_U_SCHED
            if (contextName.equalsIgnoreCase(WbsScheduleDate.PLAN))
            {
                SP_U_SCHED(object, messageContext);
            }

            //SP_U_ACTUAL
            if (contextName.equalsIgnoreCase(WbsScheduleDate.ACTUAL))
            {
                SP_U_ACTUAL(object, messageContext);
            }

            //SP_U_FORECAST
            if (contextName.equalsIgnoreCase(WbsScheduleDate.FORECAST))
            {
                SP_U_FORECAST(object, messageContext);
            }

        }

        return rpmObject;
    }

    private void SP_U_PLANNED(WbsScheduleDate object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_PLANNED";

        Object args[] = new Object[6];

        args[0] = object.getParent().getID();
        args[1] = object.getDurationInMinutes();
        //start date
        args[2] = DateUtil.convertToStringWithTime(object.getStartDate());
        //finish date is read-only
        args[3] = null;
        //p_planned_effort - planned effort is always set to 0
        args[4] = new Integer(0);

        args[5] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_SCHED(WbsScheduleDate object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_SCHED";

        Object args[] = new Object[5];

        args[0] = object.getParent().getID();
        args[1] = object.getDurationInMinutes();
        args[2] = DateUtil.convertToStringWithTime(object.getStartDate());
        //finish date is read-only
        args[3] = null;
        args[4] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_ACTUAL(WbsScheduleDate object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ACTUAL";

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(NAME_ELEMENT_ID);
        
        Object[] arguments = new Object[]
        {
            object.getParent().getID()
        };
        
        Integer duration = (Integer) ManagerUtil.selectColumnValue(
            Integer.class, object, "durationInMinutes", NAME_SCHED_DURATION,
            getTableName(), condition, arguments, messageContext);
        int percent = 0;
        if (duration != null && duration.intValue() > 0
                && object.getDurationInMinutes() != null)
        {
            percent = (int) Math.round(object.getDurationInMinutes()
                .doubleValue()
                    / duration.doubleValue() * 10000.0);
        }

        Object args[] = new Object[7];

        args[0] = object.getParent().getID();
        //bit-flag
        args[1] = new Integer(0);
        //start-date
        args[2] = DateUtil.convertToStringWithTime(object.getStartDate());
        //finish-date is read-only
        args[3] = null;
        //percent of plan duration
        args[4] = new Integer(percent);
        // p_close_task_right unused parameter, always 0!!
        args[5] = new Integer(0);

        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_FORECAST(WbsScheduleDate object,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String name = "SP_U_FORECAST";

        Object args[] = new Object[6];

        args[0] = object.getParent().getID();
        args[1] = object.getDurationInMinutes();
        //start-date is read-only
        args[2] = null;
        //finish date is read-only
        args[3] = null;
        //acc_rej_bitflag
        //if 1 then has rights to accept or reject forecast automatically, if 0
        // no rights.
        //SP is always called with acc_rej_bitflag=1 from the UI. 
        // -- see version 7 of this file, that's where the value of the SP changed.
        args[4] = new Integer(1);
        args[5] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
    
    public RPMObject getContainerInCacheWithScope(
            RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext)
    {
        // disable the cache since the primary key is only the ID
        // but the context is also important.
        return null;
    }
}
