/*
 * Created on Sept 26, 2006
 */
package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.workflow.checkpoints.WorkflowMethodCheckpoint;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;
import com.ibm.rpm.workflow.util.WorkflowUtil;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * @author mmatwiy
 */
public class RunningWorkflowProcessManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        RunningWorkflowProcessManager.class);

    static
    {
        containerMap.add(RunningWorkflowProcess.class);

        // TYPE_ID 70 from table TMT_WORKFLOWS maps to RunningWorkflowProcess
        containerMap.addExtraTypeIds(RunningWorkflowProcess.class, new int[]
        {
            WorkflowProcess.TYPE_ID
        });
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_WORKFLOWS.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_WORKFLOWS.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_WORKFLOWS.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_WORKFLOWS.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_WORKFLOWS.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column WKF_DEF_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_DEF_ID = 6;

    public static final int TYPE_WKF_DEF_ID = Types.CHAR;

    public static final String NAME_WKF_DEF_ID = "TMT_WORKFLOWS.WKF_DEF_ID";

    public static final String PROPERTY_WKF_DEF_ID = "DEFINITIONID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WKF_DEF_ID, NAME_WKF_DEF_ID);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 7;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WORKFLOWS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 8;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_WORKFLOWS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "WORKFLOWPROCESSNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 9;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_WORKFLOWS.PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column PARENT_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PARENT_TYPE = 10;

    public static final int TYPE_PARENT_TYPE = Types.SMALLINT;

    public static final String NAME_PARENT_TYPE = "TMT_WORKFLOWS.PARENT_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_TYPE,NAME_PARENT_TYPE);}

    /**
     * Column REQUEST_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REQUEST_ID = 11;

    public static final int TYPE_REQUEST_ID = Types.INTEGER;

    public static final String NAME_REQUEST_ID = "TMT_WORKFLOWS.REQUEST_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REQUEST_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REQUEST_ID,NAME_REQUEST_ID);}

    /**
     * Column REVISION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION = 12;

    public static final int TYPE_REVISION = Types.VARCHAR;

    public static final String NAME_REVISION = "TMT_WORKFLOWS.REVISION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REVISION = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION,NAME_REVISION);}

    /**
     * Column WKF_PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WKF_PRIORITY = 13;

    public static final int TYPE_WKF_PRIORITY = Types.SMALLINT;

    public static final String NAME_WKF_PRIORITY = "TMT_WORKFLOWS.WKF_PRIORITY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_PRIORITY = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_PRIORITY,NAME_WKF_PRIORITY);}

    /**
     * Column CREATED_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATE = 14;

    public static final int TYPE_CREATED_DATE = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATE = "TMT_WORKFLOWS.CREATED_DATE";

    public static final String PROPERTY_CREATED_DATE = "STARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATE, SqlUtil
            .convertTimestampToDate(NAME_CREATED_DATE));
    }

    /**
     * Column AUTHOR_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_AUTHOR_ID = 15;

    public static final int TYPE_AUTHOR_ID = Types.CHAR;

    public static final String NAME_AUTHOR_ID = "TMT_WORKFLOWS.AUTHOR_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_AUTHOR_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_AUTHOR_ID,NAME_AUTHOR_ID);}

    /**
     * Column EMAIL_ADDRESS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EMAIL_ADDRESS = 16;

    public static final int TYPE_EMAIL_ADDRESS = Types.VARCHAR;

    public static final String NAME_EMAIL_ADDRESS = "TMT_WORKFLOWS.EMAIL_ADDRESS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EMAIL_ADDRESS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_EMAIL_ADDRESS,NAME_EMAIL_ADDRESS);}

    /**
     * Column CLOSED_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CLOSED_DATE = 17;

    public static final int TYPE_CLOSED_DATE = Types.TIMESTAMP;

    public static final String NAME_CLOSED_DATE = "TMT_WORKFLOWS.CLOSED_DATE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CLOSED_DATE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CLOSED_DATE,NAME_CLOSED_DATE);}

    /**
     * Column OUTCOME_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_OUTCOME_ID = 18;

    public static final int TYPE_OUTCOME_ID = Types.INTEGER;

    public static final String NAME_OUTCOME_ID = "TMT_WORKFLOWS.OUTCOME_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OUTCOME_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OUTCOME_ID,NAME_OUTCOME_ID);}

    /**
     * Column WKF_STATUS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WKF_STATUS = 19;

    public static final int TYPE_WKF_STATUS = Types.INTEGER;

    public static final String NAME_WKF_STATUS = "TMT_WORKFLOWS.WKF_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_STATUS,NAME_WKF_STATUS);}

    /**
     * Column END_CYCLE of type Types.CHAR mapped to String.
     */
    public static final int ID_END_CYCLE = 20;

    public static final int TYPE_END_CYCLE = Types.CHAR;

    public static final String NAME_END_CYCLE = "TMT_WORKFLOWS.END_CYCLE";

    public static final String PROPERTY_END_CYCLE = "STATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_END_CYCLE, NAME_END_CYCLE);
    }

    /**
     * Column EXPIRES_DATE of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_EXPIRES_DATE = 21;

    public static final int TYPE_EXPIRES_DATE = Types.TIMESTAMP;

    public static final String NAME_EXPIRES_DATE = "TMT_WORKFLOWS.EXPIRES_DATE";

    // public static final String PROPERTY_END_CYCLE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_END_CYCLE,NAME_END_CYCLE);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 22;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_WORKFLOWS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 23;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_WORKFLOWS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 24;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_WORKFLOWS.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 25;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_WORKFLOWS.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column CHECKLIST of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CHECKLIST = 26;

    public static final int TYPE_CHECKLIST = Types.VARCHAR;

    public static final String NAME_CHECKLIST = "TMT_WORKFLOWS.CHECKLIST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHECKLIST = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST,NAME_CHECKLIST);}

    /**
     * Column SUBJECT of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SUBJECT = 27;

    public static final int TYPE_SUBJECT = Types.VARCHAR;

    public static final String NAME_SUBJECT = "TMT_WORKFLOWS.SUBJECT";

    public static final String PROPERTY_SUBJECT = "CURRENTSTEP";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUBJECT, NAME_SUBJECT);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 28;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_WORKFLOWS.DESCRIPTION";

    // public static final String PROPERTY_DESCRIPTION = "COMMENTS";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    // }

    /**
     * Column PROJECT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROJECT_NAME = 29;

    public static final int TYPE_PROJECT_NAME = Types.VARCHAR;

    public static final String NAME_PROJECT_NAME = "TMT_WORKFLOWS.PROJECT_NAME";

    // public static final String PROPERTY_PROJECT_NAME = "PROJECTNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_PROJECT_NAME, NAME_PROJECT_NAME);
    // }

    /**
     * Column ATTACHED_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ATTACHED_NAME = 30;

    public static final int TYPE_ATTACHED_NAME = Types.VARCHAR;

    public static final String NAME_ATTACHED_NAME = "TMT_WORKFLOWS.ATTACHED_NAME";

    // public static final String PROPERTY_ATTACHED_NAME = "ATTACHMENTNAME";
    // static
    // {
    // FIELDPROPERTYMAP.put(PROPERTY_ATTACHED_NAME, NAME_ATTACHED_NAME);
    // }

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 31;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "TMT_WORKFLOWS.STAGE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID);}

    /**
     * Column OUTCOME_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OUTCOME_NAME = 32;

    public static final int TYPE_OUTCOME_NAME = Types.VARCHAR;

    public static final String NAME_OUTCOME_NAME = "TMT_WORKFLOWS.OUTCOME_NAME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OUTCOME_NAME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OUTCOME_NAME,NAME_OUTCOME_NAME);}

    /**
     * Column STATE_CHANGE_WKF of type Types.VARCHAR mapped to String.
     */
    public static final int ID_STATE_CHANGE_WKF = 33;

    public static final int TYPE_STATE_CHANGE_WKF = Types.VARCHAR;

    public static final String NAME_STATE_CHANGE_WKF = "TMT_WORKFLOWS.STATE_CHANGE_WKF";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_STATE_CHANGE_WKF = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_STATE_CHANGE_WKF,NAME_STATE_CHANGE_WKF);}

    /**
     * Column WKF_GROUP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WKF_GROUP_ID = 34;

    public static final int TYPE_WKF_GROUP_ID = Types.INTEGER;

    public static final String NAME_WKF_GROUP_ID = "TMT_WORKFLOWS.WKF_GROUP_ID";

    public static final String PROPERTY_WKF_GROUP_ID = "GROUPID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WKF_GROUP_ID, NAME_WKF_GROUP_ID);
    }

    /**
     * Column PARENT_GROUP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PARENT_GROUP_ID = 35;

    public static final int TYPE_PARENT_GROUP_ID = Types.INTEGER;

    public static final String NAME_PARENT_GROUP_ID = "TMT_WORKFLOWS.PARENT_GROUP_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_GROUP_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_PARENT_GROUP_ID,NAME_PARENT_GROUP_ID);}

    public static final String TABLE_NAME = "TMT_WORKFLOWS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_WORKFLOWS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_WKF_DEF_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_PARENT_TYPE,
            NAME_REQUEST_ID, NAME_REVISION, NAME_WKF_PRIORITY,
            NAME_CREATED_DATE, NAME_AUTHOR_ID, NAME_EMAIL_ADDRESS,
            NAME_CLOSED_DATE, NAME_OUTCOME_ID, NAME_WKF_STATUS, NAME_END_CYCLE,
            NAME_EXPIRES_DATE, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_RANK, NAME_CHECKLIST, NAME_SUBJECT,
            NAME_DESCRIPTION, NAME_PROJECT_NAME, NAME_ATTACHED_NAME,
            NAME_STAGE_ID, NAME_OUTCOME_NAME, NAME_STATE_CHANGE_WKF,
            NAME_WKF_GROUP_ID, NAME_PARENT_GROUP_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new RunningWorkflowProcessManager instance.
     * 
     * @return RunningWorkflowProcessManager
     */
    public RunningWorkflowProcessManager()
    {
        super();
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }
    
    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D'");
        sql.appendAnd();
        sql.append(NAME_ELEMENT_ID);
        sql.append(" IS NOT NULL ");
        sql.appendEqual(NAME_TYPE_ID, WorkflowProcess.TYPE_ID);
        sql.appendAnd();

        // get the processes that are open, hanged, or suspended
        sql.appendOpenParenthesis();
        sql.append(NAME_END_CYCLE);
        sql.appendEqual();
        sql.append("'"
                + EnumeratedTypeUtil.getInstance().getValue(null,
                    WorkflowState.class, WorkflowState._Open) + "'");

        sql.appendOr();
        sql.append(NAME_END_CYCLE);
        sql.appendEqual();
        sql.append("'"
                + EnumeratedTypeUtil.getInstance().getValue(null,
                    WorkflowState.class, WorkflowState._Hanged) + "'");

        sql.appendOr();
        sql.append(NAME_END_CYCLE);
        sql.appendEqual();
        sql.append("'"
                + EnumeratedTypeUtil.getInstance().getValue(null,
                    WorkflowState.class, WorkflowState._Suspended) + "'");
        sql.appendCloseParenthesis();

        return sql.toString();
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        RunningWorkflowProcess runningWorkflowProcess = (RunningWorkflowProcess) rpmObject;

        // all fields are read-only
        runningWorkflowProcess.setID(rs.getString(ID_ELEMENT_ID));
        runningWorkflowProcess.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
        
        runningWorkflowProcess.setCurrentStep(rs.getString(ID_SUBJECT));
        runningWorkflowProcess.setStartDate(Manager.getCalendar(rs,
            ID_CREATED_DATE));
        runningWorkflowProcess.setState((WorkflowState) EnumeratedTypeUtil
            .getInstance().getType(null, WorkflowState.class,
                new Integer(rs.getString(ID_END_CYCLE))));
        runningWorkflowProcess.setWorkflowProcessName(rs
            .getString(ID_ELEMENT_NAME));

        return runningWorkflowProcess;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String workflowProcessId = resultSet.getString(ID_WKF_DEF_ID);
        relatedFields.put(index, ID_WKF_DEF_ID, workflowProcessId);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        // TODO: this whole method can be removed once this sub-table
        // join is defined in the metadata - Myriam
        if (manager instanceof CommunicationManager)
        {
            // for xpath like
            // /CommunicationManager[RunningWorkflowProcess[<condition>}]
            joinCondition = JoinCondition.createSubSelect(
                CommunicationManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFilter());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof RunningWorkflowProcessManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID,
                WbsManager.NAME_ELEMENT_ID, WbsManager.TABLE_NAME,
                WbsManager.NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID,
                ScopeManager.NAME_ELEMENT_ID, ScopeManager.TABLE_NAME,
                ScopeManager.NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof DocumentManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID,
                DocumentManager.NAME_ELEMENT_ID, manager
                    .getTableName(contextName), DocumentManager.NAME_ELEMENT_ID
                        + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        RunningWorkflowProcessScope scope = (RunningWorkflowProcessScope) rpmScope;

        if (scope != null && scope.getChildren() != null)
        {
            String definitionID = (String) relatedFields.get(index,
                ID_WKF_DEF_ID);

            loadWorkflowProcess((RunningWorkflowProcess) rpmObject,
                definitionID, scope.getChildren(), messageContext, isDelta);
        }

        return rpmObject;
    }

    private void loadWorkflowProcess(RunningWorkflowProcess runningProcess,
            String definitionID, WorkflowScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkflowManager manager = (WorkflowManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(WorkflowProcess.class.getName());

        RPMObject[] oldWorkflowProcess = new RPMObject[]
        {
            runningProcess.getWorkflowProcess()
        };

        Object[] args = new Object[]
        {
            definitionID
        };

        ArrayList temp = manager.loadByForeignKey(oldWorkflowProcess, scope,
            messageContext, this, args, null, null, isDelta);

        if (temp != null && !temp.isEmpty())
        {
            WorkflowProcess workflowProcess = (WorkflowProcess) temp.get(0);
            runningProcess.setWorkflowProcess(workflowProcess);
        }

    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    public RunningWorkflowProcess suspendProcess(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // validate the arguments
        WorkflowMethodCheckpoint.getInstance().validateSuspendWorkflowArgs(
            runningProcess, messageContext);

        RunningWorkflowProcess reloadedProcess = null;

        if (messageContext.isSuccessful()
                && WorkflowMethodCheckpoint.getInstance().canSuspendWorkflow(
                    runningProcess, scope, messageContext))
        {
            Integer groupId = WorkflowUtil.loadGroupIdFromRunningProcessId(
                runningProcess, messageContext);

            SP_SUSPEND_WKF_PROCESS(groupId, messageContext);

            reloadedProcess = (RunningWorkflowProcess) load(runningProcess,
                scope, messageContext);
        }

        return reloadedProcess;
    }

    public RunningWorkflowProcess resumeProcess(
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // validate the arguments
        WorkflowMethodCheckpoint.getInstance().validateResumeWorkflowArgs(
            runningProcess, messageContext);

        RunningWorkflowProcess reloadedProcess = null;

        if (messageContext.isSuccessful()
                && WorkflowMethodCheckpoint.getInstance().canResumeWorkflow(
                    runningProcess, scope, messageContext))
        {
            Integer groupId = WorkflowUtil.loadGroupIdFromRunningProcessId(
                runningProcess, messageContext);

            SP_RESUME_WKF_PROCESS(groupId, messageContext);

            reloadedProcess = (RunningWorkflowProcess) load(runningProcess,
                scope, messageContext);
        }

        return reloadedProcess;
    }

    public void cancelProcess(RunningWorkflowProcess runningProcess,
            MessageContext messageContext) throws RPMException, SQLException
    {
        // validate the arguments
        WorkflowMethodCheckpoint.getInstance().validateCancelWorkflowArgs(
            runningProcess, messageContext);

        if (messageContext.isSuccessful()
                && WorkflowMethodCheckpoint.getInstance().canCancelWorkflow(
                    runningProcess, messageContext))
        {
            Integer groupId = WorkflowUtil.loadGroupIdFromRunningProcessId(
                runningProcess, messageContext);
            SP_CLOSE_WKF_PROCESS(groupId, messageContext);
        }
    }

    private void SP_SUSPEND_WKF_PROCESS(Integer groupId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[2];
        args[0] = groupId;
        args[1] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_SUSPEND_WKF_PROCESS", args);
    }

    private void SP_RESUME_WKF_PROCESS(Integer groupId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[2];
        args[0] = groupId;
        args[1] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_RESUME_WKF_PROCESS", args);
    }

    private void SP_CLOSE_WKF_PROCESS(Integer groupId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[2];
        args[0] = groupId;
        args[1] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_CLOSE_WKF_PROCESS", args);
    }
}