package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.checkpoints.WorkflowMethodCheckpoint;
import com.ibm.rpm.workflow.constants.ErrorCodes;
import com.ibm.rpm.workflow.containers.GenericWorkflow;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowCategory;
import com.ibm.rpm.workflow.containers.WorkflowGroup;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;
import com.ibm.rpm.workflow.util.WorkflowTypeUtil;
import com.ibm.rpm.workflow.util.WorkflowUtil;
import com.ibm.rpm.xpathparser.XPathContainer;

public class WorkflowManager extends AbstractRPMObjectManager
{
    private static ContainerMap containerMap = new ContainerMap(
        WorkflowManager.class);
    static
    {
        containerMap.add(GenericWorkflow.class);
        containerMap.add(WorkflowProcess.class);
        containerMap.add(WorkflowCategory.class);
        containerMap.add(WorkflowGroup.class);

        // these type_ids map to a WorkflowGroup
        containerMap.addExtraTypeIds(WorkflowGroup.class, new int[]
        {
                Project.TYPE_ID, Deliverable.TYPE_ID, Task.TYPE_ID,
                ChangeRequest.TYPE_ID, Risk.TYPE_ID, Issue.TYPE_ID,
                Document.TYPE_ID, NoteMinutes.TYPE_ID, Requirement.TYPE_ID,
                Defect.TYPE_ID, ServiceRequest.TYPE_ID, Action.TYPE_ID
        });
    }

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WORKFLOW_CYCLES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_WORKFLOW_CYCLES.PARENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 3;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_WORKFLOW_CYCLES.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_WORKFLOW_CYCLES.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 5;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_WORKFLOW_CYCLES.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column START_WITH of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_START_WITH = 6;

    public static final int TYPE_START_WITH = Types.INTEGER;

    public static final String NAME_START_WITH = "TMT_WORKFLOW_CYCLES.START_WITH";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_START_WITH = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_START_WITH,NAME_START_WITH);}

    /**
     * Column REQUEST_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REQUEST_ID = 7;

    public static final int TYPE_REQUEST_ID = Types.INTEGER;

    public static final String NAME_REQUEST_ID = "TMT_WORKFLOW_CYCLES.REQUEST_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REQUEST_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REQUEST_ID,NAME_REQUEST_ID);}

    /**
     * Column OUTCOME_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_OUTCOME_ID = 8;

    public static final int TYPE_OUTCOME_ID = Types.INTEGER;

    public static final String NAME_OUTCOME_ID = "TMT_WORKFLOW_CYCLES.OUTCOME_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OUTCOME_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OUTCOME_ID,NAME_OUTCOME_ID);}

    /**
     * Column NEXT_STATE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_NEXT_STATE_ID = 9;

    public static final int TYPE_NEXT_STATE_ID = Types.INTEGER;

    public static final String NAME_NEXT_STATE_ID = "TMT_WORKFLOW_CYCLES.NEXT_STATE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NEXT_STATE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_NEXT_STATE_ID,NAME_NEXT_STATE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 10;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_WORKFLOW_CYCLES.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 11;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_WORKFLOW_CYCLES.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 12;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_WORKFLOW_CYCLES.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 13;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_WORKFLOW_CYCLES.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 14;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_WORKFLOW_CYCLES.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 15;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_WORKFLOW_CYCLES.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 16;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_WORKFLOW_CYCLES.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column PARTICIPANTS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PARTICIPANTS = 17;

    public static final int TYPE_PARTICIPANTS = Types.VARCHAR;

    public static final String NAME_PARTICIPANTS = "TMT_WORKFLOW_CYCLES.PARTICIPANTS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARTICIPANTS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARTICIPANTS,NAME_PARTICIPANTS);}

    /**
     * Column CHECKLIST of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CHECKLIST = 18;

    public static final int TYPE_CHECKLIST = Types.VARCHAR;

    public static final String NAME_CHECKLIST = "TMT_WORKFLOW_CYCLES.CHECKLIST";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHECKLIST = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST,NAME_CHECKLIST);}

    /**
     * Column NEXT_PROCESS_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_NEXT_PROCESS_ID = 19;

    public static final int TYPE_NEXT_PROCESS_ID = Types.CHAR;

    public static final String NAME_NEXT_PROCESS_ID = "TMT_WORKFLOW_CYCLES.NEXT_PROCESS_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NEXT_PROCESS_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_NEXT_PROCESS_ID,NAME_NEXT_PROCESS_ID);}

    /**
     * Column NEXT_STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_NEXT_STAGE_ID = 20;

    public static final int TYPE_NEXT_STAGE_ID = Types.INTEGER;

    public static final String NAME_NEXT_STAGE_ID = "TMT_WORKFLOW_CYCLES.NEXT_STAGE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_NEXT_STAGE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_NEXT_STAGE_ID,NAME_NEXT_STAGE_ID);}

    /**
     * Column MUST_OBEY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MUST_OBEY = 21;

    public static final int TYPE_MUST_OBEY = Types.SMALLINT;

    public static final String NAME_MUST_OBEY = "TMT_WORKFLOW_CYCLES.MUST_OBEY";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MUST_OBEY = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MUST_OBEY,NAME_MUST_OBEY);}

    /**
     * Column BLOB_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_BLOB_ID = 22;

    public static final int TYPE_BLOB_ID = Types.CHAR;

    public static final String NAME_BLOB_ID = "TMT_WORKFLOW_CYCLES.BLOB_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_BLOB_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_BLOB_ID,NAME_BLOB_ID);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 23;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_WORKFLOW_CYCLES.DESCRIPTION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DESCRIPTION = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 24;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "TMT_WORKFLOW_CYCLES.CONFIG_FLAG";

    public static final String PROPERTY_CONFIG_FLAG = "ACTIVE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, SqlUtil.getBooleanValue(
            NAME_CONFIG_FLAG, 3));
    }

    /**
     * Column LAYOUT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_LAYOUT_ID = 25;

    public static final int TYPE_LAYOUT_ID = Types.CHAR;

    public static final String NAME_LAYOUT_ID = "TMT_WORKFLOW_CYCLES.LAYOUT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LAYOUT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LAYOUT_ID,NAME_LAYOUT_ID);}

    /**
     * Column SECURITY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SECURITY_ID = 26;

    public static final int TYPE_SECURITY_ID = Types.CHAR;

    public static final String NAME_SECURITY_ID = "TMT_WORKFLOW_CYCLES.SECURITY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SECURITY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SECURITY_ID,NAME_SECURITY_ID);}

    /**
     * Column CHECKLIST_DESC of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CHECKLIST_DESC = 27;

    public static final int TYPE_CHECKLIST_DESC = Types.VARCHAR;

    public static final String NAME_CHECKLIST_DESC = "TMT_WORKFLOW_CYCLES.CHECKLIST_DESC";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHECKLIST_DESC = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST_DESC,NAME_CHECKLIST_DESC);}

    /**
     * Column RESPONSE_DAYS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_RESPONSE_DAYS = 28;

    public static final int TYPE_RESPONSE_DAYS = Types.INTEGER;

    public static final String NAME_RESPONSE_DAYS = "TMT_WORKFLOW_CYCLES.RESPONSE_DAYS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RESPONSE_DAYS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RESPONSE_DAYS,NAME_RESPONSE_DAYS);}

    /**
     * Column RESPONSE_MINUTES of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_RESPONSE_MINUTES = 29;

    public static final int TYPE_RESPONSE_MINUTES = Types.INTEGER;

    public static final String NAME_RESPONSE_MINUTES = "TMT_WORKFLOW_CYCLES.RESPONSE_MINUTES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RESPONSE_MINUTES = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_RESPONSE_MINUTES,NAME_RESPONSE_MINUTES);}

    /**
     * Column CHECKLIST_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHECKLIST_TYPE = 30;

    public static final int TYPE_CHECKLIST_TYPE = Types.INTEGER;

    public static final String NAME_CHECKLIST_TYPE = "TMT_WORKFLOW_CYCLES.CHECKLIST_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHECKLIST_TYPE = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST_TYPE,NAME_CHECKLIST_TYPE);}

    /**
     * Column ACTION_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ACTION_NAMES = 31;

    public static final int TYPE_ACTION_NAMES = Types.VARCHAR;

    public static final String NAME_ACTION_NAMES = "TMT_WORKFLOW_CYCLES.ACTION_NAMES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ACTION_NAMES = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ACTION_NAMES,NAME_ACTION_NAMES);}

    /**
     * Column EXTERNAL_ACTION_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EXTERNAL_ACTION_NAMES = 32;

    public static final int TYPE_EXTERNAL_ACTION_NAMES = Types.VARCHAR;

    public static final String NAME_EXTERNAL_ACTION_NAMES = "TMT_WORKFLOW_CYCLES.EXTERNAL_ACTION_NAMES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_EXTERNAL_ACTION_NAMES =
    // "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_ACTION_NAMES,NAME_EXTERNAL_ACTION_NAMES);}

    /**
     * Column SINGLE_MULTI of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SINGLE_MULTI = 33;

    public static final int TYPE_SINGLE_MULTI = Types.SMALLINT;

    public static final String NAME_SINGLE_MULTI = "TMT_WORKFLOW_CYCLES.SINGLE_MULTI";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SINGLE_MULTI = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SINGLE_MULTI,NAME_SINGLE_MULTI);}

    /**
     * Column OWNER_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_OWNER_ID = 34;

    public static final int TYPE_OWNER_ID = Types.VARCHAR;

    public static final String NAME_OWNER_ID = "TMT_WORKFLOW_CYCLES.OWNER_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_OWNER_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_OWNER_ID,NAME_OWNER_ID);}

    /**
     * Column PROCESS_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROCESS_NAMES = 35;

    public static final int TYPE_PROCESS_NAMES = Types.VARCHAR;

    public static final String NAME_PROCESS_NAMES = "TMT_WORKFLOW_CYCLES.PROCESS_NAMES";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PROCESS_NAMES = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PROCESS_NAMES,NAME_PROCESS_NAMES);}

    /**
     * Column WKF_QUERY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_QUERY_ID = 36;

    public static final int TYPE_WKF_QUERY_ID = Types.CHAR;

    public static final String NAME_WKF_QUERY_ID = "TMT_WORKFLOW_CYCLES.WKF_QUERY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_QUERY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_QUERY_ID,NAME_WKF_QUERY_ID);}

    public static final String TABLE_NAME = "TMT_WORKFLOW_CYCLES";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_WORKFLOW_CYCLES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_PARENT_ID, NAME_ELEMENT_NAME, NAME_TYPE_ID,
            NAME_LEVEL_ID, NAME_START_WITH, NAME_REQUEST_ID, NAME_OUTCOME_ID,
            NAME_NEXT_STATE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_RANK, NAME_PARTICIPANTS, NAME_CHECKLIST,
            NAME_NEXT_PROCESS_ID, NAME_NEXT_STAGE_ID, NAME_MUST_OBEY,
            NAME_BLOB_ID, NAME_DESCRIPTION, NAME_CONFIG_FLAG, NAME_LAYOUT_ID,
            NAME_SECURITY_ID, NAME_CHECKLIST_DESC, NAME_RESPONSE_DAYS,
            NAME_RESPONSE_MINUTES, NAME_CHECKLIST_TYPE, NAME_ACTION_NAMES,
            NAME_EXTERNAL_ACTION_NAMES, NAME_SINGLE_MULTI, NAME_OWNER_ID,
            NAME_PROCESS_NAMES, NAME_WKF_QUERY_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new WorkflowManager instance.
     * 
     * @return WorkflowManager
     */
    public WorkflowManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericWorkflow workflow = (GenericWorkflow) rpmObject;

        if (isDelta)
        {
            workflow.deltaName(rs.getString(ID_ELEMENT_NAME));
        }
        else
        {
            workflow.setName(rs.getString(ID_ELEMENT_NAME));
        }

        if (workflow instanceof WorkflowProcess)
        {
            WorkflowProcess process = (WorkflowProcess) rpmObject;
            Boolean active = new Boolean(rs.getInt(ID_CONFIG_FLAG) == 3);
            if (isDelta)
            {
                process.deltaActive(active);
            }
            else
            {
                process.setActive(active);
            }
        }
        return workflow;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        GenericWorkflow workflow = (GenericWorkflow) rpmObject;
        WorkflowScope scope = (WorkflowScope) rpmScope;

        if (scope != null)
        {
            if (scope.getParent() != null)
            {
                loadParent(workflow, scope, messageContext, relatedFields,
                    index, isDelta);
            }

            if (rpmObject instanceof WorkflowCategory
                    || rpmObject instanceof WorkflowGroup)
            {
                GenericWorkflow[] children = null;
                if (rpmObject instanceof WorkflowCategory)
                {
                    children = ((WorkflowCategory) rpmObject).getChildren();
                }
                else if (rpmObject instanceof WorkflowGroup)
                {
                    children = ((WorkflowGroup) rpmObject).getChildren();
                }

                if (scope.getChildren() != null)
                {
                    loadChildren(workflow, children, scope, messageContext,
                        isDelta);
                }
            }
        }

        return rpmObject;
    }

    private void loadParent(GenericWorkflow workflow, WorkflowScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);
        RPMObject newParent = null;

        if (isDelta)
        {
            newParent = workflow.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new WorkflowCategory();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = loadByPrimaryKey(newParent, scope.getParent(),
                        messageContext, true);
                }
                else
                {
                    // In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = loadByPrimaryKey(oldParent, scope.getParent(),
                        messageContext, false);
                }
            }
            workflow.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {

                newParent = new WorkflowCategory();
                newParent.setID(oldParentId);
                newParent = loadByPrimaryKey(newParent, scope.getParent(),
                    messageContext, false);

                workflow.setParent(newParent);
            }
        }
    }

    private void loadChildren(GenericWorkflow category,
            GenericWorkflow[] oldChildren, WorkflowScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
            this, new Object[]
            {
                category.getID()
            }, null, null, isDelta);

        if (temp != null && temp.size() > 0)
        {
            GenericWorkflow[] children = new GenericWorkflow[temp.size()];
            temp.toArray(children);
            if (category instanceof WorkflowCategory)
            {
                ((WorkflowCategory) category).setChildren(children);
            }
            else if (category instanceof WorkflowGroup)
            {
                ((WorkflowGroup) category).setChildren(children);
            }
        }
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        switch (typeId)
        {
        case WorkflowProcess.TYPE_ID:
        case WorkflowCategory.TYPE_ID:
            return containerMap.createContainer(typeId);
        case 1: // project
        case 2: // deliverable
        case 3: // task
        case 4: // change request
        case 5: // risk
        case 6: // issue
        case 11: // pool
        case 13: // resource
        case 14: // document
        case 242: // note minutes
        case 213: // Requirements
        case 226: // defects
        case 227: // service request
        case 222: // score cards
        case 257: // action item
            return new WorkflowGroup();
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return null;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WorkflowManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" =?");
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof RunningWorkflowProcessManager)
        {
            // for loading RunningWorkflowProcess.workflowProcess
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID, NAME_PARENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof AvailableWorkflowProcessManager)
        {
            // for loading AvailableWorkflowProcess.workflowProcess
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqualQuestionMark(NAME_ELEMENT_ID);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        // Dominique: once this join is in the metadata, 
        // this whole method can be deleted - Myriam
        if (manager instanceof CommunicationManager)
        {
            // for xpath like
            // /CommunicationManager[WorkflowProcess[<condition>}]
            joinCondition = JoinCondition.createSubSelect(
                CommunicationManager.NAME_WKF_DEF_ID, NAME_NEXT_PROCESS_ID,
                TABLE_NAME, getFilter());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    public String getFilter()
    {
        return NAME_REC_STATUS + "!='D'";
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    // ===============
    // PROCESS CONTROL
    // ===============

    public RunningWorkflowProcess startProcess(WorkflowProcess process,
            RPMObject target, RunningWorkflowProcessScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        String rank = WorkflowUtil.loadWorkflowRank(process, messageContext);

        // validate the arguments
        WorkflowMethodCheckpoint.getInstance().validateStartWorkflowArgs(
            process, rank, target, messageContext);

        RunningWorkflowProcess runningProcess = null;

        if (messageContext.isSuccessful()
                && WorkflowMethodCheckpoint.getInstance().canStartWorkflow(
                    process, scope, target, messageContext))
        {
            Integer tableType = WorkflowTypeUtil.rankToTableType(rank);
            String runningProcessId = null;
            
            try 
            {
                runningProcessId = SP_A_WORKFLOW(tableType, target.getID(),
                    process.getID(), messageContext);
            }
            catch (RPMException e)
            {
                if (ErrorCodes.HANGED_PROCESS == e.getErrorID())
                {
                    // if an exception was thrown because the process is hanged, we want 
                    // to return the error but we also want to load the RunningWorkflowProcess
                    messageContext.addException(e);
                    
                    RunningWorkflowProcess[] runningProcesses = null;

                    // The proc does not return a runningProcessId for hanged
                    // processes so we need to load the running process the cumbersome way.
                    // Opened defect #56842 for the database team to see if this can be rectified.
                    if (target instanceof WorkElement)
                    {
                        WorkElementScope workElementScope = new WorkElementScope();
                        workElementScope.setWorkflowProcessesRunning(scope);

                        WbsManager manager = (WbsManager) getManagerInstance(WorkElement.class
                            .getName());

                        WorkElement workElement = (WorkElement) manager.load(
                            target, workElementScope, messageContext);
                        runningProcesses = workElement == null ? null
                                : workElement.getWorkflowProcessesRunning();
                    }
                    else if (target instanceof GenericDocument)
                    {
                        DocumentScope documentScope = new DocumentScope();
                        documentScope.setWorkflowProcessesRunning(scope);

                        DocumentManager manager = (DocumentManager) getManagerInstance(Document.class
                            .getName());

                        GenericDocument document = (GenericDocument) manager
                            .load(target, documentScope, messageContext);
                        runningProcesses = document == null ? null : document
                            .getWorkflowProcessesRunning();
                    }
                    else if (target instanceof AbstractScope)
                    {
                        ScopeElementScope scopeElementScope = new ScopeElementScope();
                        scopeElementScope.setWorkflowProcessesRunning(scope);

                        ScopeManager manager = (ScopeManager) getManagerInstance(AbstractScope.class
                            .getName());

                        AbstractScope abstractScope = (AbstractScope) manager
                            .load(target, scopeElementScope, messageContext);
                        runningProcesses = abstractScope == null ? null
                                : abstractScope.getWorkflowProcessesRunning();
                    }

                    runningProcess = WorkflowUtil.findRunningProcessWithId(
                        runningProcesses, process.getID());
                }
                else
                {
                    throw e;
                }
            }
            
            // load the new RunningWorkflowProcess
            if (runningProcess == null && runningProcessId != null)
            {
                runningProcess = new RunningWorkflowProcess();
                runningProcess.setID(runningProcessId);
                
                RPMObjectManager manager = getManagerInstance(RunningWorkflowProcess.class);
                runningProcess = (RunningWorkflowProcess) manager.load(
                    runningProcess, scope, messageContext);
            }
            

        }
        return runningProcess;
    }

    public void associateProcess(WorkflowProcess process,
            GenericProject genericProject, boolean associate,
            boolean isDefault, MessageContext messageContext)
            throws RPMException, SQLException
    {
        // validate the arguments
        WorkflowMethodCheckpoint.getInstance().validateAssociateWorkflowArgs(
            process, genericProject, associate, isDefault, messageContext);

        if (messageContext.isSuccessful()
                && WorkflowMethodCheckpoint.getInstance().canAssociateWorkflow(
                    process, genericProject, associate, isDefault,
                    messageContext))
        {
            // 0: both flags are set to false, can't set default without
            // associate
            Integer bitflag = new Integer(0);

            if (associate)
            {
                if (isDefault)
                {
                    bitflag = new Integer(3);
                }
                else
                {
                    bitflag = new Integer(1);
                }
            }

            SP_U_PRJ_PROCESS(process.getID(), genericProject.getID(), bitflag,
                messageContext);
        }
    }

    private String SP_A_WORKFLOW(Integer tableType, String targetId,
            String processId, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = tableType;
        args[1] = targetId;
        args[2] = processId;
        args[3] = getUser(messageContext).getID();
        String id = executeProcedure(messageContext, "SP_A_WORKFLOW", args);

        return id;
    }

    private void SP_U_PRJ_PROCESS(String processId, String genericProjectId,
            Integer bitflag, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = genericProjectId;
        args[1] = processId;
        args[2] = bitflag;
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_PRJ_PROCESS", args);
    }
}