/*
 * Created on 11-Oct-2006
 * @author mmatwiy
 * Manager for container WorkflowRole. To be extended once 
 * Workflow Design is implemented.
 */
package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.xpathparser.XPathContainer;

public class WorkflowRoleManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        WorkflowRoleManager.class);

    static
    {
        containerMap.add(WorkflowRole.class);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "WKF_ROLES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 2;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "WKF_ROLES.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 3;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "WKF_ROLES.DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DESCRIPTION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 4;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "WKF_ROLES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 5;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "WKF_ROLES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    public static final String TABLE_NAME = "WKF_ROLES";

    /**
     * Create an array of type string containing all the fields of the WKF_ROLES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_DESCRIPTION,
            NAME_REC_USER, NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new WorkflowRoleManager instance.
     * @return WorkflowRoleManager
     */
    public WorkflowRoleManager()
    {
        super();
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }
    
    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObject container = null;
        try
        {
            container = new WorkflowRole();
        }
        catch (Exception x)
        {
            ExceptionUtil.handleException(this, x);
        }

        return container;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WorkflowRole role = (WorkflowRole) rpmObject;

        if (isDelta)
        {
            role.deltaName(rs.getString(ID_ELEMENT_NAME));
        }
        else
        {
            role.setName(rs.getString(ID_ELEMENT_NAME));
        }

        return role;

    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        return super.decodeRelatedFields(resultSet, relatedFields, index);
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        return super.loadRelatedObjects(rpmObject, rpmScope, messageContext,
            relatedFields, index, isDelta);
    }
    
    // override this method because this container has a type_id but
    // the database table doesn't have a type_id column so we can't send it
    protected String getFinalCondition(XPathContainer container,
            String contextName)
    {
        // Add the basic filtering condition
        SqlBuffer finalCondition = new SqlBuffer(getFilter(contextName));

        return finalCondition.toString();
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WorkflowRoleMappingManager)
        {
            // for loading WorkflowRoleMapping.workflowRole
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID,
                WorkflowRoleMappingManager.NAME_WKF_ROLE_ID,
                WorkflowRoleMappingManager.TABLE_NAME,
                WorkflowRoleMappingManager.NAME_WKF_ROLE_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

}