/*
 * Created on 6-Oct-2006
 * @author mmatwiy
 * 
 * Manager for container WorkflowRoleMapping. This allows users to map 
 * WorkflowRole attributes for GenericProjects.
 * 
 * Note that a WorkflowRoleMapping with no ID is a mapping that doesn't contain
 * a Resource, ContactGroup, or SecurityRole. 
 * 
 * Once one or more of these three are added and the WorkflowRoleMapping is saved,
 * the container will have an ID (the ID of the associated WorkflowRole). At this point
 * a row is added to table PROJECT_WKF_ROLES.
 */
package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.managers.ContactGroupsManager;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;

public class WorkflowRoleMappingManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        WorkflowRoleMappingManager.class);

    static
    {
        containerMap.add(WorkflowRoleMapping.class);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROJECT_WKF_ROLES.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_WKF_ROLE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);}

    /**
     * Column WKF_ROLE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_ROLE_ID = 2;

    public static final int TYPE_WKF_ROLE_ID = Types.CHAR;

    public static final String NAME_WKF_ROLE_ID = "PROJECT_WKF_ROLES.WKF_ROLE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_WKF_ROLE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_ROLE_ID,NAME_WKF_ROLE_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 3;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "PROJECT_WKF_ROLES.RESOURCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column SECURITY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SECURITY_ID = 4;

    public static final int TYPE_SECURITY_ID = Types.CHAR;

    public static final String NAME_SECURITY_ID = "PROJECT_WKF_ROLES.SECURITY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_SECURITY_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SECURITY_ID,NAME_SECURITY_ID);}

    /**
     * Column GROUP_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_GROUP_ID = 5;

    public static final int TYPE_GROUP_ID = Types.CHAR;

    public static final String NAME_GROUP_ID = "PROJECT_WKF_ROLES.GROUP_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_GROUP_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_GROUP_ID,NAME_GROUP_ID);}

    public static final String TABLE_NAME = "PROJECT_WKF_ROLES";

    /**
     * Create an array of type string containing all the fields of the PROJECT_WKF_ROLES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_WKF_ROLE_ID, NAME_RESOURCE_ID,
            NAME_SECURITY_ID, NAME_GROUP_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new WorkflowRoleMappingManager instance.
     * @return WorkflowRoleMappingManager
     */
    public WorkflowRoleMappingManager()
    {
        super();
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return null;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        return new String[]
        {
                NAME_PROJECT_ID, NAME_WKF_ROLE_ID
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;

        // parent is required for inserts and updates and is always loaded
        String projectId = mapping.getParent().getID();
        
        String roleId = mapping.getWorkflowRole() != null ? mapping
            .getWorkflowRole().getID() : mapping.getID();

        return new Object[]
        {
                projectId, roleId
        };
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        
        // for updates, the preload calls this method prior to the checkpoint
        // so we need to return an error from here if the parent is missing
        if (mapping.getParent() == null || mapping.getParent().getID() == null)
        {
            ExceptionUtil.createAndHandleRPMException(this.getClass(),
                ErrorCodes.NO_ID, new String[]
                {
                        StringUtil.getShortClassName(mapping.getClass()) + " " +
                        ValidationConstants.PARENT_FIELD
                });
        }
        
        // parent is required for inserts and updates
        String projectID = mapping.getParent().getID();

        if (mapping.getID() == null)   // the WorkflowRole ID
        {
            if ((projectID == null)
                    || (mapping.getWorkflowRole() == null || (mapping
                        .getWorkflowRole().getID() == null)))
            {
                invalidPrimaryKeys.append("ID");
                valid = false;
            }
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                invalidPrimaryKeys.toString());
        }
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject) throws RPMException
    {
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        mapping.setID((String) primaryKey[0]);

        return mapping;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObject container = null;
        try
        {
            container = new WorkflowRoleMapping();
        }
        catch (Exception x)
        {
            ExceptionUtil.handleException(this, x);
        }

        return container;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        mapping.setID(rs.getString(ID_WKF_ROLE_ID));

        return mapping;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);
        
        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkflowRoleMappingScope scope = (WorkflowRoleMappingScope) rpmScope;
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;

        // the WorkflowRole must be loaded first to set up the primary key
        loadWorkflowRole(mapping, messageContext, isDelta);
        
        // always load the parent; its ID is also part of the primary key
        loadParent(mapping, scope, messageContext, relatedFields, index, isDelta);

        if (scope != null)
        {
            if (scope.getContactGroup() != null)
            {
                loadContactGroup(mapping, scope.getContactGroup(),
                    messageContext, isDelta);
            }
            if (scope.getResource() != null)
            {
                loadResource(mapping, scope.getResource(), messageContext,
                    isDelta);
            }
            if (scope.getContactGroup() != null)
            {
                loadSecurityRole(mapping, scope.getSecurityRole(),
                    messageContext, isDelta);
            }
        }

        return mapping;
    }
    
    private void loadParent(WorkflowRoleMapping mapping,
            RPMObjectScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_PROJECT_ID);
        RPMObject newParent = null;
        
        WbsManager manager = (WbsManager) RPMManagerFactory.getInstance()
            .getRPMObjectManager(GenericProject.class.getName());
        
        RPMObjectScope parentScope = scope == null ? null : scope.getParent();

        if (isDelta)
        {
            newParent = mapping.getParent();
            RPMObject oldParent = null;

            if (oldParentId != null)
            {
                oldParent = new GenericProject();
                oldParent.setID(oldParentId);

                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = manager.loadByPrimaryKey(newParent, parentScope,
                        messageContext, true);
                }
                else
                {
                    // In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = manager.loadByPrimaryKey(oldParent, null,
                        messageContext, false);
                }
            }
            mapping.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                newParent = new WorkElement();
                newParent.setID(oldParentId);
                newParent = manager.loadByPrimaryKey(newParent, parentScope,
                    messageContext, false);
                mapping.setParent(newParent);
            }
        }
    }


    private void loadResource(WorkflowRoleMapping mapping,
            ResourceScope resourceScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceManager manager = (ResourceManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Resource.class.getName());

        RPMObject[] oldResource = new RPMObject[]
        {
            mapping.getResource()
        };

        ArrayList temp = manager.loadByForeignKey(oldResource, resourceScope,
            messageContext, this, getPrimaryKeyValues(mapping), null, null,
            isDelta);

        if (temp != null && !temp.isEmpty())
        {
            Resource resource = (Resource) temp.get(0);
            mapping.setResource(resource);
        }
    }

    private void loadContactGroup(WorkflowRoleMapping mapping,
            ContactGroupScope contactScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ContactGroupsManager manager = (ContactGroupsManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(ContactGroup.class.getName());

        RPMObject[] oldContactGroup = new RPMObject[]
        {
            mapping.getContactGroup()
        };

        ArrayList temp = manager.loadByForeignKey(oldContactGroup,
            contactScope, messageContext, this, getPrimaryKeyValues(mapping),
            null, null, isDelta);

        if (temp != null && !temp.isEmpty())
        {
            ContactGroup contactGroup = (ContactGroup) temp.get(0);
            mapping.setContactGroup(contactGroup);
        }
    }

    private void loadSecurityRole(WorkflowRoleMapping mapping,
            SecurityRoleScope securityRoleScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        SecurityRoleManager manager = (SecurityRoleManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(SecurityRole.class.getName());

        RPMObject[] oldSecurityRole = new RPMObject[]
        {
            mapping.getSecurityRole()
        };

        ArrayList temp = manager.loadByForeignKey(oldSecurityRole,
            securityRoleScope, messageContext, this,
            getPrimaryKeyValues(mapping), null, null, isDelta);

        if (temp != null && !temp.isEmpty())
        {
            SecurityRole securityRole = (SecurityRole) temp.get(0);
            mapping.setSecurityRole(securityRole);
        }
    }

    private void loadWorkflowRole(WorkflowRoleMapping mapping,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkflowRoleManager manager = (WorkflowRoleManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(WorkflowRole.class.getName());

        RPMObject[] oldWorkflowRole = new RPMObject[]
        {
            mapping.getWorkflowRole()
        };

        Object[] args = new Object[]
        {
            mapping.getID()
        };

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(mapping,
            oldWorkflowRole, null, messageContext, this, args, null, null,
            isDelta);

        if (temp != null && !temp.isEmpty())
        {
            WorkflowRole role = (WorkflowRole) temp.get(0);
            mapping.setWorkflowRole(role);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {
            // for loading GenericProject.workflowRoleMappings
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PROJECT_ID,
                WbsManager.NAME_ELEMENT_ID, WbsManager.TABLE_NAME,
                WbsManager.NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        //create handled by updateObject
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        mapping.setID(mapping.getWorkflowRole().getID());
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        WorkflowRoleMappingScope scope = (WorkflowRoleMappingScope) rpmScope;

        if (scope != null)
        {
            if ((mapping.testContactGroupModified() && scope.getContactGroup() != null)
                    || (mapping.testSecurityRoleModified() && scope
                        .getSecurityRole() != null)
                    || (mapping.testResourceModified())
                    && scope.getResource() != null)
            {
                SP_U_PRJ_WKF_ROLE(mapping, scope, messageContext);
            }
        }

        return mapping;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // a mapping is deleted by doing an update with a null
        // ContactGroup, Resource, and SecurityRole
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }
    
    private void SP_U_PRJ_WKF_ROLE(WorkflowRoleMapping mapping,
            WorkflowRoleMappingScope scope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        final String procedureName = "SP_U_PRJ_WKF_ROLE";
        Object args[] = new Object[7];

        // parent is required for both inserts and updates
        args[0] = mapping.getParent().getID();       //cannot be null
        args[1] = mapping.getWorkflowRole().getID(); //cannot be null

        args[2] = (mapping.getSecurityRole() == null || scope.getSecurityRole() == null)
                ? null : mapping.getSecurityRole().getID();

        args[3] = (mapping.getContactGroup() == null || scope
            .getContactGroup() == null) ? null : mapping.getContactGroup()
            .getID();

        args[4] = (mapping.getResource() == null || scope.getResource() == null)
                ? null : mapping.getResource().getID();

        args[5] = new Integer(0); //not used
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }
}