package com.ibm.rpm.xpathparser;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Stack;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.TransformerException;

import org.apache.xalan.Version;
import org.apache.xpath.Expression;
import org.apache.xpath.compiler.Compiler;
import org.apache.xpath.compiler.FunctionTable;
import org.apache.xpath.compiler.XPathParser;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.function.FuncIsNotNull;
import com.ibm.rpm.xpathparser.function.FuncIsNull;
import com.ibm.rpm.xpathparser.steps.StartXPath;

/**
 * @author gmeloche
 *  
 */
public class XPathToStack
{
    public static FunctionTable funcTable = new FunctionTable();
    
    //define external functions
    public static int FUNC_ISNULL = installFunction("isnull", new FuncIsNull());

    public static int FUNC_ISNOTNULL = installFunction("isnotnull",
        new FuncIsNotNull()); 

    public static Stack parse(String xpath) throws Exception
    {

        if (FUNC_ISNULL == 0 || FUNC_ISNOTNULL == 0)
        {
            throw new Exception("Xalan version " + Version.getMajorVersionNum()
                    + "." + Version.getReleaseVersionNum() + "."
                    + Version.getMaintenanceVersionNum()
                    + " is not compatible with the RPM Web Services API.");
        }
        
        Stack stack = null;

        try
        {
            XPathParser parser = new XPathParser(null, null);
            Compiler compiler = getCompiler();

            parser.initXPath(compiler, xpath, null);
            StartXPath step = new StartXPath(compiler.getOpMap(), compiler
                .getTokenQueue());
            step.build(0);
            stack = step.getStack();
        }
        catch (TransformerException e)
        {
            String[] params = new String[]
            {
                e.getMessage()
            };
            throw new RPMException(ErrorCodes.XPATH_INVALID_STRING, params, e);
        }
        catch (RPMException e){
            throw e;
        }
        catch (Exception e)
        {
            String[] params = new String[]
            {
                e.getMessage()
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params, e);
        }

        return stack;
    }
    
    private static Compiler getCompiler() throws Exception
    {

        Compiler compiler = null;
        Constructor constructor = null;

        //Xalan 2.7 compatible
        try
        {
            constructor = Compiler.class.getConstructor(new Class[]
            {
                    ErrorListener.class, SourceLocator.class,
                    FunctionTable.class
            });
            compiler = (Compiler) constructor.newInstance(new Object[]
            {
                    null, null, funcTable
            });
        }
        catch (Exception e)
        {
            // do nothing
        }

        //Xalan 2.6 compatible
        if (compiler == null)
        {
            compiler = new Compiler();
        }

        return compiler;
    }
    
    private static int installFunction(String name, Object object)
    {

        int funcIndex = 0;
        Method method = null;

        //Xalan 2.6 compatible
        try
        {
            method = FunctionTable.class.getMethod("installFunction",
                new Class[]
                {
                        String.class, Expression.class
                });
            Integer index = (Integer) method.invoke(null, new Object[]
            {
                    name, object
            });

            funcIndex = index.intValue();
        }
        catch (Exception e)
        {
            // check for another signature
        }

        if (method == null)
        {
            //Xalan 2.7 compatible
            try
            {

                method = FunctionTable.class.getMethod("installFunction",
                    new Class[]
                    {
                            String.class, Class.class
                    });

                Integer index = (Integer) method.invoke(funcTable, new Object[]
                {
                        name, object.getClass()
                });

                funcIndex = index.intValue();
            }
            catch (Exception e1)
            {
                //do nothing
            }
        }

        return funcIndex;

    } 
}
