package com.ibm.rpm.xpathparser.steps;

import java.util.ArrayList;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.xpathparser.FieldCondition;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectFunctionTable;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathToStack;

/**
 * @author kuschel
 * 
 * [OP_FUNCTION] [length] [FUNC_name] {OP_ARGUMENT} [ENDOP]
 * 
 * returns: XNodeSet XNumber XString XBoolean XRTree XObject
 * @xsl.usage advanced
 *  
 */
public class Function extends XPathStep
{
    private int function = 0;

    private FieldCondition condition = null;

    private ArrayList arguments = new ArrayList();

    /**
     * @param op_xpath
     * @param opMap
     * @param tokenQueue
     * @param parent
     */
    public Function(OpMapVector opMap, ObjectVector tokenQueue, XPathStep parent)
    {
        super(ReflectOpCodes.OP_FUNCTION, opMap, tokenQueue, parent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPostBuild(com.ibm.rpm.xpathparser.steps.XPathStep)
     */
    protected void notifyPostBuild(XPathStep predicate) throws XPathException,
            RPMException
    {
        
        
        String value;
        if (function == ReflectFunctionTable.FUNC_STARTS_WITH)
        {
            if(arguments.size() != 2){
                String[] params = new String[]
                {
                    "starts-with", "2"
                };
                throw new RPMException(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS,
                    params);                
            }            
            condition.setProperty((String) arguments.get(0));
            value = (String) arguments.get(1);            
            if (!(value.endsWith("'") && value.endsWith("'")))
            {
                String[] params = new String[]
                {
                    value
                };
                throw new RPMException(ErrorCodes.XPATH_INVALID_ARGUMENTS,
                    params);
            }
            
            // escape special characters and add the end wildcard
            value = SqlUtil.escapeSpecialCharactersAddWildcard(value);
            condition.setValue(value);
        }
        else if (function == ReflectFunctionTable.FUNC_CONTAINS)
        {
            if(arguments.size() != 2){
                String[] params = new String[]
                {
                    "contains", "2"
                };
                throw new RPMException(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS,
                    params);                
            }  
            condition.setProperty((String) arguments.get(0));
            value = (String) arguments.get(1);
            if (!(value.endsWith("'") && value.endsWith("'")))
            {
                String[] params = new String[]
                {
                    value
                };
                throw new RPMException(ErrorCodes.XPATH_INVALID_ARGUMENTS,
                    params);
            }
            
            // remove the start '
            value = value.substring(1, value.length());
            
            // escape special characters and add the end wildcard
            value = SqlUtil.escapeSpecialCharactersAddWildcard(value);
            
            // re-add the ' and add the first wildcard
            value = "'%" + value;
            condition.setValue(value);
        }
        else if (function == XPathToStack.FUNC_ISNULL
                || function == XPathToStack.FUNC_ISNOTNULL)
        {           
            if(arguments.size() != 1){
                String[] params = new String[]
                {
                        (function == XPathToStack.FUNC_ISNULL ? "isnull"
                                : "isnotnull"), "2"
                };
                throw new RPMException(ErrorCodes.XPATH_INVALID_NUMBER_OF_ARGUMENTS,
                    params);                
            }  
            condition.setProperty((String) arguments.get(0));            
            value = "NULL";
            condition.setValue(value);
        }
        else
        {
            String name = "operation code = " + function;
            String[] params = new String[]{
                   name
            };
            throw new RPMException(ErrorCodes.XPATH_UNSUPPORTED_OPERATION_CODE, params);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.XPathStep)
     */
    protected void notifyPreBuild(XPathStep predicate) throws XPathException,
            RPMException
    {
        XPathStep parent = getParent();
        int operation = parent.getOperation();
        Predicate pred = null;
        if (operation == ReflectOpCodes.OP_PREDICATE)
        {
            Table ptable = ((From) parent.getParent()).getTable();
            pred = new Predicate(Predicate.CONDITION);
            ptable.setPredicate(pred);
        }
        else if (operation == ReflectOpCodes.OP_AND
                || operation == ReflectOpCodes.OP_OR
                || operation == ReflectOpCodes.OP_FUNCTION)
        {
            pred = findPredicate(parent);
        }
        else
        {
            String[] params = new String[]
            {
                "Function " + function + " not supported"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
        
        condition = new FieldCondition(getFunction());
        if (pred != null)
        {
            pred.add(condition);
        }
        else
        {
            String[] params = new String[]
            {
                "predicate not defined "
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
    }

    public int build(int position) throws XPathException, RPMException
    {
        setLength(opMap.elementAt(++position) - 2);
        setFunction(opMap.elementAt(++position));
        notifyPreBuild(this);
        position = super.build(++position);
        notifyPostBuild(this);
        return position;
    }

    /**
     * @return Returns the function.
     */
    public int getFunction()
    {
        return function;
    }

    /**
     * @param function
     *            The function to set.
     */
    protected void setFunction(int function)
    {
        this.function = function;
    }

    /**
     * @return Returns the condition.
     *  
     */
    public FieldCondition getCondition()
    {
        return condition;
    }

    /**
     * @param condition
     *            The condition to set.
     */
    protected void setCondition(FieldCondition condition)
    {
        this.condition = condition;
    }

    /**
     * @param token
     */
    protected void addArgument(String token)
    {
        arguments.add(token);

    }

    /**
     * @return Returns the arguments.
     */
    public String[] getArguments()
    {
        return (String[]) arguments.toArray(new String[0]);
    }

}
