package com.ibm.rpm.xpathparser.steps;

import java.util.Stack;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.ContainerCondition;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;

/**
 * @author kuschel
 * 
 * [NODENAME] [index to ns token or EMPTY] [index to name token]
 * 
 * returns: XBoolean
 * @xsl.usage advanced
 */
public class Name extends TokenedXPathStep
{
    /**
     * @param op_xpath
     * @param opMap
     * @param tokenQueue
     * @param step
     */
    protected Name(OpMapVector opMap, ObjectVector tokenQueue, XPathStep step)
    {
        super(ReflectOpCodes.NODENAME, opMap, tokenQueue, step);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPostBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPreBuild(XPathStep predicate) throws XPathException, RPMException
    {
        XPathStep farparent = getParent().getParent().getParent();
        Table tparent = null;
        int operationCode = farparent.getOperation();
        Predicate parentPred = null;
        boolean isGroup = false;

        if (operationCode == ReflectOpCodes.OP_FUNCTION)
        {
            farparent = farparent.getParent();
            operationCode = farparent.getOperation();
        }
        
        if (operationCode == ReflectOpCodes.OP_AND
                || operationCode == ReflectOpCodes.OP_OR)
        {
            parentPred = ((Group) farparent).getPredicate();
            farparent = findGroupParent(farparent);
            operationCode = farparent.getOperation();
            isGroup = true;
        }

        if (operationCode == ReflectOpCodes.OP_PREDICATE)
        {
            Stack tableStack = ((XPathPredicate) farparent).getStack();
                       
            tparent = (Table)tableStack.peek();
            
            //assign the right parent if we are in a group
            if (isGroup && tparent.getPredicate() != null
                    && tparent.getPredicate().getType() == 0)
            {
                tparent = tparent.getParent();
            }
            
            From from = (From) getParent();
            Table table = null;
            if (from.getClass() == FromAttribute.class)
            {
                table = new Table("@" + getToken(), tparent);
            }
            else
            {
                table = new Table(getToken(), tparent);
            }

            ContainerCondition condition = new ContainerCondition();
            condition.getSubTables().add(table);
            
            if(from.isSubPredicate()){            
	            if (isGroup)
	            {
	                if (parentPred == null)
                    {
	                    String[] params = new String[]
                        {
                            predicate + " should be defined: " + tparent
                        };
                        throw new RPMException(ErrorCodes.INTERNAL_ERROR,
                            params);
                    }
	            }
	            else
	            {
	                parentPred = tparent.getPredicate();
	                if (parentPred == null)
	                {
	                    parentPred = new Predicate(Predicate.CONDITION);
	                    tparent.setPredicate(parentPred);
	                }
	                else
	                {
	                    /*
	                     * Note: at this point the predicate should always be null
	                     * but this is needed to support old syntax (/parent[id
	                     * ='0'][child])
	                     */
	                    parentPred.setType(ReflectOpCodes.OP_AND);
	                }
	            }
	            Predicate subpred = new Predicate(Predicate.CONDITION);
	            subpred.add(condition);
	            parentPred.add(subpred);
	            
                XPathPredicate pred = (XPathPredicate) farparent;
                pred.setStack(condition.getSubTables());
            }
            else{
                //will put next tables in the subTable stack
                tableStack.add(table);
            }

            from.setTable(table);
        }
        else if (operationCode == ReflectOpCodes.OP_XPATH)
        {
            XPathPredicate pred = (XPathPredicate) farparent;
            From from = (From) getParent();
            Table table = null;
            if (from.getClass() == FromAttribute.class)
            {
                table = new Table("@" + getToken(), tparent);
            }
            else
            {
                table = new Table(getToken(), tparent);
            }
            pred.getStack().add(table);
            from.setTable(table);
        }

        else if (operationCode == ReflectOpCodes.OP_ARGUMENT)
        {
            Function func = (Function) farparent.getParent();
            func.addArgument(getToken());
        }
        else if (farparent instanceof Comparator)
        {
            Comparator comp = (Comparator) farparent;
            comp.getCondition().setProperty(getToken());
        }
        else
        {
            String[] params = new String[]
            {
                "Operation " + operationCode + " not supported"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPostBuild(XPathStep predicate)
    {

    }

    public int build(int position) throws XPathException, RPMException
    {
        int nsindex = opMap.elementAt(++position);
        if (nsindex != ReflectOpCodes.EMPTY)
            setNamespace((String) tokenQueue.elementAt(nsindex));
        setToken((String) tokenQueue.elementAt(opMap.elementAt(++position)));
        notifyPreBuild(this);
        notifyPostBuild(this);
        return ++position;
    }

}
