package com.ibm.rpm.xpathparser.steps;

import java.util.ArrayList;
import java.util.Stack;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.xpathparser.Condition;
import com.ibm.rpm.xpathparser.ContainerCondition;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;

public class StartXPath extends XPathPredicate
{

    /**
     * @param opMap
     * @param tokenQueue
     */
    public StartXPath(OpMapVector opMap, ObjectVector tokenQueue)
    {
        super(ReflectOpCodes.OP_XPATH, opMap, tokenQueue, null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPostBuild(XPathStep predicate)
    {

        int operationCode = getOperation();
        if (operationCode == ReflectOpCodes.OP_XPATH)
        {
            prepareStack(getStack());
        }

    }

    /**
     *  
     */
    private void prepareStack(Stack stack)
    {
        Table table = null;
        int i = stack.size() - 1;
        while (i > -1)
        {
            table = (Table) stack.get(i);
            Predicate pred = table.getPredicate();
            if (pred != null)
            {
                ArrayList conditions = buildConditions(pred, null);
                table.setConditions(conditions);

            }
            i--;
        }
    }

    private ArrayList buildConditions(Predicate predicate, String operator)
    {

        ArrayList conditions = new ArrayList();

        Condition leftCond = predicate.getLeftCondition();
        if (leftCond != null)
        {
            leftCond.setOperator(operator);
            if (leftCond instanceof ContainerCondition)
            {
                prepareStack(((ContainerCondition) leftCond).getSubTables());
            }
            conditions.add(leftCond);
        }

        Predicate leftPred = predicate.getLeftPredicate();
        if (leftPred != null)
        {
            conditions.addAll(buildConditions(leftPred, operator));
        }

        Predicate rightPred = predicate.getRightPredicate();
        if (rightPred != null)
        {
            conditions.addAll(buildConditions(rightPred, predicate.getStringType()));
        }

        Condition rightCond = predicate.getRightCondition();
        if (rightCond != null)
        {
            rightCond.setOperator(predicate.getStringType());
            if (rightCond instanceof ContainerCondition)
            {
                prepareStack(((ContainerCondition) rightCond).getSubTables());
            }
            conditions.add(rightCond);
        }

        return conditions;
    }
}
