package com.ibm.rpm.xpathparser.steps;

import java.util.Stack;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.ReflectOpCodes;

/**
 * @author kuschel
 * 
 * [OP_PREDICATE] [length] {expression} [ENDOP] (For safety)
 * 
 * returns: XBoolean or XNumber
 * @xsl.usage advanced
 */
public class XPathPredicate extends XPathStep
{

    private Stack stack = null;
    
    /**
     * @param opMap
     * @param tokenQueue
     * @param step
     */
    protected XPathPredicate(OpMapVector opMap, ObjectVector tokenQueue,
            XPathStep step)
    {
        this(ReflectOpCodes.OP_PREDICATE, opMap, tokenQueue, step);
    }

    /**
     * @param op_xpath
     * @param opMap
     * @param tokenQueue
     * @param object
     */
    protected XPathPredicate(int op_xpath, OpMapVector opMap,
            ObjectVector tokenQueue, XPathStep step)
    {
        super(op_xpath, opMap, tokenQueue, step);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPostBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPreBuild(XPathStep predicate) throws XPathException,
            RPMException
    {
        int operationCode = getOperation();
        if (operationCode == ReflectOpCodes.OP_XPATH)
        {
            stack = new Stack();
        }
        else if (operationCode == ReflectOpCodes.OP_PREDICATE)
        {
            XPathStep farparent = getParent().getParent().getParent();

            if(farparent.getOperation() == ReflectOpCodes.OP_FUNCTION){
                farparent = farparent.getParent();
            }
            
            if (farparent.getOperation() == ReflectOpCodes.OP_AND
                    || farparent.getOperation() == ReflectOpCodes.OP_OR)
            {
                farparent = findGroupParent(farparent);
            }

            stack = ((XPathPredicate) farparent).getStack();
        }
        else
        {
            String[] params = new String[]
            {
                "Operation " + operationCode + " not supported"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPostBuild(XPathStep predicate)
    {

    }

    public int build(int position) throws XPathException, RPMException
    {
        setLength(opMap.elementAt(++position) - 2);
        notifyPreBuild(this);
        position = super.build(++position);
        notifyPostBuild(this);
        return position;
    }

    /**
     * @return Returns the stack.
     */
    public Stack getStack()
    {
        return stack;
    }

    /**
     * @param stack The stack to set.
     */
    public void setStack(Stack stack)
    {
        this.stack = stack;
    }
}
