package com.ibm.rpm.scopemanagement.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.managers.FinancialManager;
import com.ibm.rpm.financial.managers.WbsFinancialManager;
import com.ibm.rpm.financial.managers.WorksheetFinancialManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.ContainerToCheckpointMap;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.ScheduleDateUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.checkpoints.AbstractRequestCheckpoint;
import com.ibm.rpm.scopemanagement.checkpoints.AbstractScopeCheckpoint;
import com.ibm.rpm.scopemanagement.checkpoints.ScopeMethodCheckpoint;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractRequest;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.types.PrecisionType;
import com.ibm.rpm.scopemanagement.util.ReqProUtil;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.types.ScopeElementSecurityType;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.managers.AvailableWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;
import com.ibm.rpm.workflow.util.WorkflowUtil;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * Manager for the TMT_CRI table and related stored procedures.
 * 
 * <h3>Elements Shared Between WBS And Scope Management</h3>
 */
public class ScopeManager extends AbstractRPMObjectManager
{
    //private static Log logger = LogFactory.getLog(ScopeManager.class);

    //private static int OPEN_STATE_BIT_FLAG = 1;

    // =============
    // CONTAINER MAP
    // =============

    private static ContainerMap containerMap = new ContainerMap(
        ScopeManager.class);

    public static List abstractScopeTaskIds = new ArrayList();

    public static final int TYPE_ID_CHANGE_REQUEST_TASK = 65;

    public static final int TYPE_ID_RISK_TASK = 66;

    public static final int TYPE_ID_ISSUE_TASK = 67;

    public static final int TYPE_ID_DEFECT_TASK = 233;

    public static final int TYPE_ID_SERVICE_REQUEST_TASK = 234;

    public static final int TYPE_ID_REQUIREMENT_TASK = 235;

    public static final int TYPE_ID_ACTION_TASK = 258;

    static
    {
        // Design note: Duplicate is NOT a ScopeElement.
        containerMap.add(Duplicate.class);

        containerMap.add(ScopeElement.class);
        containerMap.add(ScopeFolder.class);
        containerMap.add(Requirement.class);
        containerMap.add(ChangeRequest.class);
        containerMap.add(ServiceRequest.class);
        containerMap.add(Action.class);
        containerMap.add(Defect.class);
        containerMap.add(Issue.class);
        containerMap.add(Risk.class);
        containerMap.add(ScopeDeliverable.class);
        containerMap.add(ScopeSummaryTask.class);
        containerMap.add(ScopeMilestone.class);
        containerMap.add(ScopeTask.class);
        containerMap.add(AbstractScopeTask.class);
        containerMap.add(AggregateScope.class);
        containerMap.add(AbstractAggregateScope.class);
        containerMap.add(AbstractScope.class);
        containerMap.add(AbstractRequest.class);
        containerMap.add(ScopeWorkProduct.class);

        //ReqPro containers
        containerMap.add(ReqProIntegrationFolder.class);
        containerMap.add(ReqProScopeFolder.class);
        containerMap.add(ReqProRequirement.class);

        // Extra type ids for task containers, map to their AbstractScope equivalents.
        containerMap.addExtraTypeIds(ChangeRequest.class, new int[]
        {
            TYPE_ID_CHANGE_REQUEST_TASK
        });
        containerMap.addExtraTypeIds(Risk.class, new int[]
        {
            TYPE_ID_RISK_TASK
        });
        containerMap.addExtraTypeIds(Issue.class, new int[]
        {
            TYPE_ID_ISSUE_TASK
        });
        containerMap.addExtraTypeIds(Defect.class, new int[]
        {
            TYPE_ID_DEFECT_TASK
        });
        containerMap.addExtraTypeIds(ServiceRequest.class, new int[]
        {
            TYPE_ID_SERVICE_REQUEST_TASK
        });
        containerMap.addExtraTypeIds(Requirement.class, new int[]
        {
            TYPE_ID_REQUIREMENT_TASK
        });
        containerMap.addExtraTypeIds(Action.class, new int[]
        {
            TYPE_ID_ACTION_TASK
        });

        abstractScopeTaskIds.add(new Integer(TYPE_ID_CHANGE_REQUEST_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_RISK_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_ISSUE_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_DEFECT_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_SERVICE_REQUEST_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_REQUIREMENT_TASK));
        abstractScopeTaskIds.add(new Integer(TYPE_ID_ACTION_TASK));
    }

    //TODO clean up later
    //TODO workaround for the subTypeID to instantiate the right classes (ReqProIntegrationFolder, ...)
    //    public static final int SUB_TYPE_ID_REQPRO_INTEGRATION_FOLDER = 1;
    //    public static final int SUB_TYPE_ID_REQPRO_SCOPE_FOLDER = 2;
    //    public static final int SUB_TYPE_ID_REQPRO_REQUIREMENT = 3;
    //    static
    //  {
    //        containerMap.addExtraTypeIds(ReqProIntegrationFolder.class,
    //              new int[] { SUB_TYPE_ID_REQPRO_INTEGRATION_FOLDER });
    //        containerMap.addExtraTypeIds(ReqProScopeFolder.class,
    //              new int[] { SUB_TYPE_ID_REQPRO_SCOPE_FOLDER });
    //        containerMap.addExtraTypeIds(ReqProRequirement.class,
    //              new int[] { SUB_TYPE_ID_REQPRO_REQUIREMENT });
    //  }

    // =========
    // FIELD MAP
    // =========

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_CRI.RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_RANK = "WBSSORTINGRANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RANK, NAME_RANK);
    }

    /**
     * Matches TMT_WBS.RANK. This column prepepded by "WBS#"
     * We won't support XPath on this field since the link is only a rank
     */
    public static final int ID_PARENT_RANK = 2;

    public static final int TYPE_PARENT_RANK = Types.VARCHAR;

    public static final String NAME_PARENT_RANK = "TMT_CRI.PARENT_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PARENT_RANK = "PARENTSORTINGRANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_RANK, NAME_PARENT_RANK);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 3;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_CRI.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TASK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, " CASE WHEN " + NAME_TYPE_ID
                + " IN (" + TYPE_ID_CHANGE_REQUEST_TASK + ", "
                + TYPE_ID_RISK_TASK + ", " + TYPE_ID_ISSUE_TASK + ", "
                + TYPE_ID_DEFECT_TASK + ", " + TYPE_ID_SERVICE_REQUEST_TASK
                + ", " + TYPE_ID_REQUIREMENT_TASK + ", " + TYPE_ID_ACTION_TASK
                + ") THEN 'true' ELSE 'false' END");
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 4;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_CRI.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 5;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "TMT_CRI.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PROJECT_ID = "PROJECTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 6;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_CRI.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column PARENT_TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PARENT_TYPE_ID = 7;

    public static final int TYPE_PARENT_TYPE_ID = Types.INTEGER;

    public static final String NAME_PARENT_TYPE_ID = "TMT_CRI.PARENT_TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_TYPE_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PARENT_TYPE_ID,NAME_PARENT_TYPE_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 8;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CRI.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column MAX_RANK of type Types.BIGINT mapped to Long.
     */
    public static final int ID_MAX_RANK = 9;

    public static final int TYPE_MAX_RANK = Types.BIGINT;

    public static final String NAME_MAX_RANK = "TMT_CRI.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 10;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_CRI.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 11;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_CRI.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 12;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_CRI.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column TRANSFERRED of type Types.CHAR mapped to String.
     */
    public static final int ID_TRANSFERRED = 13;

    public static final int TYPE_TRANSFERRED = Types.CHAR;

    public static final String NAME_TRANSFERRED = "TMT_CRI.TRANSFERRED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRANSFERRED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TRANSFERRED,NAME_TRANSFERRED);}

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 14;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "TMT_CRI.STAGE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 15;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_CRI.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column CREATED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATETIME = 16;

    public static final int TYPE_CREATED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_CREATED_DATETIME = "TMT_CRI.CREATED_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATETIME,NAME_CREATED_DATETIME);}

    /**
     * Column REFERENCE_NUMBER of type Types.BIGINT mapped to Long.
     */
    public static final int ID_REFERENCE_NUMBER = 17;

    public static final int TYPE_REFERENCE_NUMBER = Types.BIGINT;

    public static final String NAME_REFERENCE_NUMBER = "TMT_CRI.REFERENCE_NUMBER";

    public static final String PROPERTY_REFERENCE_NUMBER = "REFERENCENUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REFERENCE_NUMBER, SqlUtil.castAsChar(
            NAME_REFERENCE_NUMBER, 32));
    }

    /**
     * Column EXTERNAL_REFERENCE of type Types.CHAR mapped to String.
     */
    public static final int ID_EXTERNAL_REFERENCE = 18;

    public static final int TYPE_EXTERNAL_REFERENCE = Types.CHAR;

    public static final String NAME_EXTERNAL_REFERENCE = "TMT_CRI.EXTERNAL_REFERENCE";

    public static final String PROPERTY_EXTERNAL_REFERENCE = "EXTERNALREFERENCENUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_REFERENCE,
            NAME_EXTERNAL_REFERENCE);
    }

    /**
     * Column PRIORITY_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRIORITY_ID = 19;

    public static final int TYPE_PRIORITY_ID = Types.SMALLINT;

    public static final String NAME_PRIORITY_ID = "TMT_CRI.PRIORITY_ID";

    public static final String PROPERTY_PRIORITY_ID = "RANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIORITY_ID, NAME_PRIORITY_ID);
    }

    /**
     * Column PROPOSED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_PROPOSED_BY = 20;

    public static final int TYPE_PROPOSED_BY = Types.CHAR;

    public static final String NAME_PROPOSED_BY = "TMT_CRI.PROPOSED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_BY,NAME_PROPOSED_BY);}

    /**
     * Column PROPOSED_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_PROPOSED_DATETIME = 21;

    public static final int TYPE_PROPOSED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_DATETIME = "TMT_CRI.PROPOSED_DATETIME";

    public static final String PROPERTY_PROPOSED_DATETIME = "PROPOSEDBYDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_DATETIME, SqlUtil
            .convertTimestampToDate(NAME_PROPOSED_DATETIME));
    }

    /**
     * Column IMPACT_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_IMPACT_LEVEL_ID = 22;

    public static final int TYPE_IMPACT_LEVEL_ID = Types.INTEGER;

    public static final String NAME_IMPACT_LEVEL_ID = "TMT_CRI.IMPACT_LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_IMPACT_LEVEL_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_IMPACT_LEVEL_ID,NAME_IMPACT_LEVEL_ID);}

    /**
     * Column SD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SD = 23;

    public static final int TYPE_SD = Types.SMALLINT;

    public static final String NAME_SD = "TMT_CRI.SD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SD,NAME_SD);}

    /**
     * Column FD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FD = 24;

    public static final int TYPE_FD = Types.SMALLINT;

    public static final String NAME_FD = "TMT_CRI.FD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FD,NAME_FD);}

    /**
     * Column START_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_START_DATETIME = 25;

    public static final int TYPE_START_DATETIME = Types.TIMESTAMP;

    public static final String NAME_START_DATETIME = "TMT_CRI.START_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_START_DATETIME,NAME_START_DATETIME);}

    /**
     * Column FINISH_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FINISH_DATETIME = 26;

    public static final int TYPE_FINISH_DATETIME = Types.TIMESTAMP;

    public static final String NAME_FINISH_DATETIME = "TMT_CRI.FINISH_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FINISH_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FINISH_DATETIME,NAME_FINISH_DATETIME);}

    /**
     * Column PLANNED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_START = 27;

    public static final int TYPE_PLANNED_START = Types.TIMESTAMP;

    public static final String NAME_PLANNED_START = "TMT_CRI.PLANNED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PLANNED_START,NAME_PLANNED_START);}

    /**
     * Column PLANNED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_FINISH = 28;

    public static final int TYPE_PLANNED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PLANNED_FINISH = "TMT_CRI.PLANNED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_FINISH,NAME_PLANNED_FINISH);}

    /**
     * Column PLANNED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PLANNED_EFFORT = 29;

    public static final int TYPE_PLANNED_EFFORT = Types.BIGINT;

    public static final String NAME_PLANNED_EFFORT = "TMT_CRI.PLANNED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_EFFORT,NAME_PLANNED_EFFORT);}

    /**
     * Column PLANNED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_DURATION = 30;

    public static final int TYPE_PLANNED_DURATION = Types.INTEGER;

    public static final String NAME_PLANNED_DURATION = "TMT_CRI.PLANNED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_DURATION,NAME_PLANNED_DURATION);}

    /**
     * Column ESTIMATED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ESTIMATED_START = 31;

    public static final int TYPE_ESTIMATED_START = Types.TIMESTAMP;

    public static final String NAME_ESTIMATED_START = "TMT_CRI.ESTIMATED_START";

    public static final String PROPERTY_ESTIMATED_START = "ESTIMATEDSTARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_START, SqlUtil
            .convertTimestampToDate(NAME_ESTIMATED_START));
    }

    /**
     * Column ESTIMATED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ESTIMATED_FINISH = 32;

    public static final int TYPE_ESTIMATED_FINISH = Types.TIMESTAMP;

    public static final String NAME_ESTIMATED_FINISH = "TMT_CRI.ESTIMATED_FINISH";

    public static final String PROPERTY_ESTIMATED_FINISH = "ESTIMATEDFINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_FINISH, SqlUtil
            .convertTimestampToDate(NAME_ESTIMATED_FINISH));
    }

    /**
     * Column MITIGATION_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MITIGATION_COST = 33;

    public static final int TYPE_MITIGATION_COST = Types.DECIMAL;

    public static final String NAME_MITIGATION_COST = "TMT_CRI.MITIGATION_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MITIGATION_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MITIGATION_COST,NAME_MITIGATION_COST);}

    /**
     * Column SCOPE_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SCOPE_RANK_ID = 34;

    public static final int TYPE_SCOPE_RANK_ID = Types.SMALLINT;

    public static final String NAME_SCOPE_RANK_ID = "TMT_CRI.SCOPE_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCOPE_RANK_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCOPE_RANK_ID,NAME_SCOPE_RANK_ID);}

    /**
     * Column SCHEDULE_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SCHEDULE_RANK_ID = 35;

    public static final int TYPE_SCHEDULE_RANK_ID = Types.SMALLINT;

    public static final String NAME_SCHEDULE_RANK_ID = "TMT_CRI.SCHEDULE_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHEDULE_RANK_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHEDULE_RANK_ID,NAME_SCHEDULE_RANK_ID);}

    /**
     * Column COST_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_COST_RANK_ID = 36;

    public static final int TYPE_COST_RANK_ID = Types.SMALLINT;

    public static final String NAME_COST_RANK_ID = "TMT_CRI.COST_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_RANK_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_COST_RANK_ID,NAME_COST_RANK_ID);}

    /**
     * Column QUALITY_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_QUALITY_RANK_ID = 37;

    public static final int TYPE_QUALITY_RANK_ID = Types.SMALLINT;

    public static final String NAME_QUALITY_RANK_ID = "TMT_CRI.QUALITY_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_QUALITY_RANK_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_QUALITY_RANK_ID,NAME_QUALITY_RANK_ID);}

    /**
     * Column PROBABILITY_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PROBABILITY_RANK_ID = 38;

    public static final int TYPE_PROBABILITY_RANK_ID = Types.SMALLINT;

    public static final String NAME_PROBABILITY_RANK_ID = "TMT_CRI.PROBABILITY_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROBABILITY_RANK_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY_RANK_ID,NAME_PROBABILITY_RANK_ID);}

    /**
     * Column PRECISION_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRECISION_RANK_ID = 39;

    public static final int TYPE_PRECISION_RANK_ID = Types.SMALLINT;

    public static final String NAME_PRECISION_RANK_ID = "TMT_CRI.PRECISION_RANK_ID";

    public static final String PROPERTY_PRECISION_RANK_ID = "PRECISION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRECISION_RANK_ID, SqlUtil.castAsChar(
            NAME_PRECISION_RANK_ID, 1));
    }

    /**
     * Column EXPOSURE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPOSURE_ID = 40;

    public static final int TYPE_EXPOSURE_ID = Types.INTEGER;

    public static final String NAME_EXPOSURE_ID = "TMT_CRI.EXPOSURE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPOSURE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPOSURE_ID,NAME_EXPOSURE_ID);}

    /**
     * Column CONSEQUENSE_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CONSEQUENSE_COST = 41;

    public static final int TYPE_CONSEQUENSE_COST = Types.DECIMAL;

    public static final String NAME_CONSEQUENSE_COST = "TMT_CRI.CONSEQUENSE_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONSEQUENSE_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONSEQUENSE_COST,NAME_CONSEQUENSE_COST);}

    /**
     * Column PROBABILITY_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROBABILITY_COST = 42;

    public static final int TYPE_PROBABILITY_COST = Types.DECIMAL;

    public static final String NAME_PROBABILITY_COST = "TMT_CRI.PROBABILITY_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROBABILITY_COST = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY_COST,NAME_PROBABILITY_COST);}

    /**
     * Column APPROVED_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_APPROVED_DATETIME = 43;

    public static final int TYPE_APPROVED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_APPROVED_DATETIME = "TMT_CRI.APPROVED_DATETIME";

    public static final String PROPERTY_APPROVED_DATETIME = "APPROVEDBYDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_APPROVED_DATETIME, SqlUtil
            .convertTimestampToDate(NAME_APPROVED_DATETIME));
    }

    /**
     * Column APPROVED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_APPROVED_START = 44;

    public static final int TYPE_APPROVED_START = Types.TIMESTAMP;

    public static final String NAME_APPROVED_START = "TMT_CRI.APPROVED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_START,NAME_APPROVED_START);}

    /**
     * Column APPROVED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_APPROVED_FINISH = 45;

    public static final int TYPE_APPROVED_FINISH = Types.TIMESTAMP;

    public static final String NAME_APPROVED_FINISH = "TMT_CRI.APPROVED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_FINISH,NAME_APPROVED_FINISH);}

    /**
     * Column APPROVED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_APPROVED_BUDGET = 46;

    public static final int TYPE_APPROVED_BUDGET = Types.DECIMAL;

    public static final String NAME_APPROVED_BUDGET = "TMT_CRI.APPROVED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_BUDGET,NAME_APPROVED_BUDGET);}

    /**
     * Column CLOSED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CLOSED_BY = 47;

    public static final int TYPE_CLOSED_BY = Types.CHAR;

    public static final String NAME_CLOSED_BY = "TMT_CRI.CLOSED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLOSED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CLOSED_BY,NAME_CLOSED_BY);}

    /**
     * Column CLOSED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CLOSED_DATETIME = 48;

    public static final int TYPE_CLOSED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_CLOSED_DATETIME = "TMT_CRI.CLOSED_DATETIME";

    public static final String PROPERTY_CLOSED_DATETIME = "CLOSEDBYDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLOSED_DATETIME, SqlUtil
            .convertTimestampToDate(NAME_CLOSED_DATETIME));
    }

    /**
     * Column PLANNING_UNIT of type Types.CHAR mapped to String.
     */
    public static final int ID_PLANNING_UNIT = 49;

    public static final int TYPE_PLANNING_UNIT = Types.CHAR;

    public static final String NAME_PLANNING_UNIT = "TMT_CRI.PLANNING_UNIT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNING_UNIT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PLANNING_UNIT,NAME_PLANNING_UNIT);}

    /**
     * Column PLANNING_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNING_TYPE = 50;

    public static final int TYPE_PLANNING_TYPE = Types.INTEGER;

    public static final String NAME_PLANNING_TYPE = "TMT_CRI.PLANNING_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNING_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PLANNING_TYPE,NAME_PLANNING_TYPE);}

    /**
     * Column ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_BUDGET = 51;

    public static final int TYPE_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATION_BUDGET = "TMT_CRI.ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_BUDGET,NAME_ESTIMATION_BUDGET);}

    /**
     * Column CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CURRENT_BUDGET = 52;

    public static final int TYPE_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_CURRENT_BUDGET = "TMT_CRI.CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CURRENT_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CURRENT_BUDGET,NAME_CURRENT_BUDGET);}

    /**
     * Column FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_FORECAST_BUDGET = 53;

    public static final int TYPE_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_FORECAST_BUDGET = "TMT_CRI.FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_BUDGET,NAME_FORECAST_BUDGET);}

    /**
     * Column ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_BUDGET = 54;

    public static final int TYPE_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_ACTUAL_BUDGET = "TMT_CRI.ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_BUDGET = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_BUDGET,NAME_ACTUAL_BUDGET);}

    /**
     * Column BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE1_BUDGET = 55;

    public static final int TYPE_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE1_BUDGET = "TMT_CRI.BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_BUDGET,NAME_BASELINE1_BUDGET);}

    /**
     * Column BASELINE1_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_BASELINE1_DATETIME = 56;

    public static final int TYPE_BASELINE1_DATETIME = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_DATETIME = "TMT_CRI.BASELINE1_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_DATETIME,NAME_BASELINE1_DATETIME);}

    /**
     * Column BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE2_BUDGET = 57;

    public static final int TYPE_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE2_BUDGET = "TMT_CRI.BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_BUDGET,NAME_BASELINE2_BUDGET);}

    /**
     * Column BASELINE2_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_BASELINE2_DATETIME = 58;

    public static final int TYPE_BASELINE2_DATETIME = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_DATETIME = "TMT_CRI.BASELINE2_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_DATETIME,NAME_BASELINE2_DATETIME);}

    /**
     * Column BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE3_BUDGET = 59;

    public static final int TYPE_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE3_BUDGET = "TMT_CRI.BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_BUDGET,NAME_BASELINE3_BUDGET);}

    /**
     * Column BASELINE3_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_BASELINE3_DATETIME = 60;

    public static final int TYPE_BASELINE3_DATETIME = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_DATETIME = "TMT_CRI.BASELINE3_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_DATETIME,NAME_BASELINE3_DATETIME);}

    /**
     * Column S_ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATION_BUDGET = 61;

    public static final int TYPE_S_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATION_BUDGET = "TMT_CRI.S_ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATION_BUDGET,NAME_S_ESTIMATION_BUDGET);}

    /**
     * Column S_CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_CURRENT_BUDGET = 62;

    public static final int TYPE_S_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_S_CURRENT_BUDGET = "TMT_CRI.S_CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_CURRENT_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_CURRENT_BUDGET,NAME_S_CURRENT_BUDGET);}

    /**
     * Column S_FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_FORECAST_BUDGET = 63;

    public static final int TYPE_S_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_S_FORECAST_BUDGET = "TMT_CRI.S_FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_FORECAST_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_FORECAST_BUDGET,NAME_S_FORECAST_BUDGET);}

    /**
     * Column S_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE1_BUDGET = 64;

    public static final int TYPE_S_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE1_BUDGET = "TMT_CRI.S_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE1_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE1_BUDGET,NAME_S_BASELINE1_BUDGET);}

    /**
     * Column S_BASELINE1_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_S_BASELINE1_DATETIME = 65;

    public static final int TYPE_S_BASELINE1_DATETIME = Types.TIMESTAMP;

    public static final String NAME_S_BASELINE1_DATETIME = "TMT_CRI.S_BASELINE1_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE1_DATETIME =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE1_DATETIME,NAME_S_BASELINE1_DATETIME);}

    /**
     * Column S_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE2_BUDGET = 66;

    public static final int TYPE_S_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE2_BUDGET = "TMT_CRI.S_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE2_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE2_BUDGET,NAME_S_BASELINE2_BUDGET);}

    /**
     * Column S_BASELINE2_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_S_BASELINE2_DATETIME = 67;

    public static final int TYPE_S_BASELINE2_DATETIME = Types.TIMESTAMP;

    public static final String NAME_S_BASELINE2_DATETIME = "TMT_CRI.S_BASELINE2_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE2_DATETIME =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE2_DATETIME,NAME_S_BASELINE2_DATETIME);}

    /**
     * Column S_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE3_BUDGET = 68;

    public static final int TYPE_S_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE3_BUDGET = "TMT_CRI.S_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE3_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE3_BUDGET,NAME_S_BASELINE3_BUDGET);}

    /**
     * Column S_BASELINE3_DATETIME of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_S_BASELINE3_DATETIME = 69;

    public static final int TYPE_S_BASELINE3_DATETIME = Types.TIMESTAMP;

    public static final String NAME_S_BASELINE3_DATETIME = "TMT_CRI.S_BASELINE3_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE3_DATETIME =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE3_DATETIME,NAME_S_BASELINE3_DATETIME);}

    /**
     * Column R_ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATION_BUDGET = 70;

    public static final int TYPE_R_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATION_BUDGET = "TMT_CRI.R_ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATION_BUDGET,NAME_R_ESTIMATION_BUDGET);}

    /**
     * Column R_CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_CURRENT_BUDGET = 71;

    public static final int TYPE_R_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_R_CURRENT_BUDGET = "TMT_CRI.R_CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_CURRENT_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_CURRENT_BUDGET,NAME_R_CURRENT_BUDGET);}

    /**
     * Column R_FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_FORECAST_BUDGET = 72;

    public static final int TYPE_R_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_R_FORECAST_BUDGET = "TMT_CRI.R_FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_FORECAST_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_FORECAST_BUDGET,NAME_R_FORECAST_BUDGET);}

    /**
     * Column R_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE1_BUDGET = 73;

    public static final int TYPE_R_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE1_BUDGET = "TMT_CRI.R_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE1_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE1_BUDGET,NAME_R_BASELINE1_BUDGET);}

    /**
     * Column R_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE2_BUDGET = 74;

    public static final int TYPE_R_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE2_BUDGET = "TMT_CRI.R_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE2_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE2_BUDGET,NAME_R_BASELINE2_BUDGET);}

    /**
     * Column R_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE3_BUDGET = 75;

    public static final int TYPE_R_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE3_BUDGET = "TMT_CRI.R_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE3_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE3_BUDGET,NAME_R_BASELINE3_BUDGET);}

    /**
     * Column LAST_WF_TYPE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_WF_TYPE = 76;

    public static final int TYPE_LAST_WF_TYPE = Types.VARCHAR;

    public static final String NAME_LAST_WF_TYPE = "TMT_CRI.LAST_WF_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_TYPE,NAME_LAST_WF_TYPE);}

    /**
     * Column LAST_WF_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_LAST_WF_BY = 77;

    public static final int TYPE_LAST_WF_BY = Types.CHAR;

    public static final String NAME_LAST_WF_BY = "TMT_CRI.LAST_WF_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_BY,NAME_LAST_WF_BY);}

    /**
     * Column LAST_WF_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_WF_DATETIME = 78;

    public static final int TYPE_LAST_WF_DATETIME = Types.TIMESTAMP;

    public static final String NAME_LAST_WF_DATETIME = "TMT_CRI.LAST_WF_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_DATETIME,NAME_LAST_WF_DATETIME);}

    /**
     * Column MAX_REV_REACHED of type Types.CHAR mapped to String.
     */
    public static final int ID_MAX_REV_REACHED = 79;

    public static final int TYPE_MAX_REV_REACHED = Types.CHAR;

    public static final String NAME_MAX_REV_REACHED = "TMT_CRI.MAX_REV_REACHED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_REV_REACHED = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAX_REV_REACHED,NAME_MAX_REV_REACHED);}

    /**
     * Column MAX_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAX_REVISION = 80;

    public static final int TYPE_MAX_REVISION = Types.SMALLINT;

    public static final String NAME_MAX_REVISION = "TMT_CRI.MAX_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_REVISION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_REVISION,NAME_MAX_REVISION);}

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAJOR_REVISION = 81;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "TMT_CRI.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINOR_REVISION = 82;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "TMT_CRI.MINOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION);}

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_IMPACT = 83;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "TMT_CRI.REVISION_IMPACT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_TYPE = 84;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "TMT_CRI.REVISION_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION_HISTORY = 85;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "TMT_CRI.REVISION_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

    /**
     * Column TOOLTIP_DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TOOLTIP_DESCRIPTION = 86;

    public static final int TYPE_TOOLTIP_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_TOOLTIP_DESCRIPTION = "TMT_CRI.TOOLTIP_DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOOLTIP_DESCRIPTION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TOOLTIP_DESCRIPTION,NAME_TOOLTIP_DESCRIPTION);}

    /**
     * Column STATUS_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STATUS_ID = 87;

    public static final int TYPE_STATUS_ID = Types.INTEGER;

    public static final String NAME_STATUS_ID = "TMT_CRI.STATUS_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STATUS_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STATUS_ID,NAME_STATUS_ID);}

    /**
     * Column PUBLISHED of type Types.CHAR mapped to String.
     */
    public static final int ID_PUBLISHED = 88;

    public static final int TYPE_PUBLISHED = Types.CHAR;

    public static final String NAME_PUBLISHED = "TMT_CRI.PUBLISHED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PUBLISHED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED,NAME_PUBLISHED);}

    /**
     * Column SHARED of type Types.CHAR mapped to String.
     */
    public static final int ID_SHARED = 89;

    public static final int TYPE_SHARED = Types.CHAR;

    public static final String NAME_SHARED = "TMT_CRI.SHARED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SHARED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SHARED,NAME_SHARED);}

    /**
     * Column IMPACT_RANK_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_IMPACT_RANK_ID = 90;

    public static final int TYPE_IMPACT_RANK_ID = Types.SMALLINT;

    public static final String NAME_IMPACT_RANK_ID = "TMT_CRI.IMPACT_RANK_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_IMPACT_RANK_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_IMPACT_RANK_ID,NAME_IMPACT_RANK_ID);}

    /**
     * Column ASSIGN_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSIGN_NAMES = 91;

    public static final int TYPE_ASSIGN_NAMES = Types.VARCHAR;

    public static final String NAME_ASSIGN_NAMES = "TMT_CRI.ASSIGN_NAMES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_NAMES = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_NAMES,NAME_ASSIGN_NAMES);}

    /**
     * Column ASSIGN_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ASSIGN_COUNT = 92;

    public static final int TYPE_ASSIGN_COUNT = Types.SMALLINT;

    public static final String NAME_ASSIGN_COUNT = "TMT_CRI.ASSIGN_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_COUNT,NAME_ASSIGN_COUNT);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 93;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_CRI.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column RTF_BITS of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_RTF_BITS = 94;

    public static final int TYPE_RTF_BITS = Types.SMALLINT;

    public static final String NAME_RTF_BITS = "TMT_CRI.RTF_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RTF_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RTF_BITS,NAME_RTF_BITS);}

    /**
     * Column LAST_CHECKIN of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_CHECKIN = 95;

    public static final int TYPE_LAST_CHECKIN = Types.TIMESTAMP;

    public static final String NAME_LAST_CHECKIN = "TMT_CRI.LAST_CHECKIN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_CHECKIN = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_CHECKIN,NAME_LAST_CHECKIN);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 96;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_CRI.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 97;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_CRI.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column LAST_TRANSFER_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_TRANSFER_DT = 98;

    public static final int TYPE_LAST_TRANSFER_DT = Types.TIMESTAMP;

    public static final String NAME_LAST_TRANSFER_DT = "TMT_CRI.LAST_TRANSFER_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_TRANSFER_DT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_TRANSFER_DT,NAME_LAST_TRANSFER_DT);}

    /**
     * Column WBS_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_WBS_ID = 99;

    public static final int TYPE_WBS_ID = Types.VARCHAR;

    public static final String NAME_WBS_ID = "TMT_CRI.WBS_ID";

    public static final String PROPERTY_WBS_ID = "TRANSFEREDUNDERWBSROOT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WBS_ID, "CASE WHEN " + NAME_WBS_ID
                + " IS NULL THEN 'false' ELSE 'true' END");
    }

    /**
     * Column STAGE_LAST_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_STAGE_LAST_DT = 100;

    public static final int TYPE_STAGE_LAST_DT = Types.TIMESTAMP;

    public static final String NAME_STAGE_LAST_DT = "TMT_CRI.STAGE_LAST_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STAGE_LAST_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_LAST_DT,NAME_STAGE_LAST_DT);}

    /**
     * Column SCORE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SCORE = 101;

    public static final int TYPE_SCORE = Types.SMALLINT;

    public static final String NAME_SCORE = "TMT_CRI.SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCORE,NAME_SCORE);}

    /**
     * Column WEIGHT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WEIGHT = 102;

    public static final int TYPE_WEIGHT = Types.SMALLINT;

    public static final String NAME_WEIGHT = "TMT_CRI.WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WEIGHT,NAME_WEIGHT);}

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_SCORE = 103;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "TMT_CRI.CONVERTED_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE);}

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 104;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "TMT_CRI.CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT);}

    /**
     * Column SCORECARD_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_SCORECARD_ID = 105;

    public static final int TYPE_SCORECARD_ID = Types.CHAR;

    public static final String NAME_SCORECARD_ID = "TMT_CRI.SCORECARD_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_ID,NAME_SCORECARD_ID);}

    /**
     * Column PUBLISHED_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PUBLISHED_DT = 106;

    public static final int TYPE_PUBLISHED_DT = Types.TIMESTAMP;

    public static final String NAME_PUBLISHED_DT = "TMT_CRI.PUBLISHED_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PUBLISHED_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED_DT,NAME_PUBLISHED_DT);}

    /**
     * Column PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRIORITY = 107;

    public static final int TYPE_PRIORITY = Types.SMALLINT;

    public static final String NAME_PRIORITY = "TMT_CRI.PRIORITY";

    public static final String PROPERTY_PRIORITY = "PRIORITY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIORITY, NAME_PRIORITY);
    }

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 108;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "TMT_CRI.ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ETC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ETC,NAME_ETC);}

    /**
     * Column MILESTONE_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MILESTONE_PERCENT = 109;

    public static final int TYPE_MILESTONE_PERCENT = Types.SMALLINT;

    public static final String NAME_MILESTONE_PERCENT = "TMT_CRI.MILESTONE_PERCENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MILESTONE_PERCENT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_MILESTONE_PERCENT,NAME_MILESTONE_PERCENT);}

    /**
     * Column PERCENT_COMPLETE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PERCENT_COMPLETE = 110;

    public static final int TYPE_PERCENT_COMPLETE = Types.SMALLINT;

    public static final String NAME_PERCENT_COMPLETE = "TMT_CRI.PERCENT_COMPLETE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PERCENT_COMPLETE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PERCENT_COMPLETE,NAME_PERCENT_COMPLETE);}

    /**
     * Column REC_FLAGS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REC_FLAGS = 111;

    public static final int TYPE_REC_FLAGS = Types.INTEGER;

    public static final String NAME_REC_FLAGS = "TMT_CRI.REC_FLAGS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_FLAGS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_FLAGS,NAME_REC_FLAGS);}

    /**
     * Column ROLLUP_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ROLLUP_BITS = 112;

    public static final int TYPE_ROLLUP_BITS = Types.INTEGER;

    public static final String NAME_ROLLUP_BITS = "TMT_CRI.ROLLUP_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLLUP_BITS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_BITS,NAME_ROLLUP_BITS);}

    /**
     * Column WBS_ASSIGNED of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WBS_ASSIGNED = 113;

    public static final int TYPE_WBS_ASSIGNED = Types.INTEGER;

    public static final String NAME_WBS_ASSIGNED = "TMT_CRI.WBS_ASSIGNED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WBS_ASSIGNED = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WBS_ASSIGNED,NAME_WBS_ASSIGNED);}

    /**
     * Column DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DURATION = 114;

    public static final int TYPE_DURATION = Types.INTEGER;

    public static final String NAME_DURATION = "TMT_CRI.DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DURATION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DURATION,NAME_DURATION);}

    /**
     * Column ACTUAL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_START = 115;

    public static final int TYPE_ACTUAL_START = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_START = "TMT_CRI.ACTUAL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_START,NAME_ACTUAL_START);}

    /**
     * Column ACTUAL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_FINISH = 116;

    public static final int TYPE_ACTUAL_FINISH = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_FINISH = "TMT_CRI.ACTUAL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_FINISH,NAME_ACTUAL_FINISH);}

    /**
     * Column ACTUAL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_DURATION = 117;

    public static final int TYPE_ACTUAL_DURATION = Types.INTEGER;

    public static final String NAME_ACTUAL_DURATION = "TMT_CRI.ACTUAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_DURATION,NAME_ACTUAL_DURATION);}

    /**
     * Column SCHED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_START = 118;

    public static final int TYPE_SCHED_START = Types.TIMESTAMP;

    public static final String NAME_SCHED_START = "TMT_CRI.SCHED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_START = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_START,NAME_SCHED_START);}

    /**
     * Column SCHED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_FINISH = 119;

    public static final int TYPE_SCHED_FINISH = Types.TIMESTAMP;

    public static final String NAME_SCHED_FINISH = "TMT_CRI.SCHED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_FINISH = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_FINISH,NAME_SCHED_FINISH);}

    /**
     * Column SCHED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_DURATION = 120;

    public static final int TYPE_SCHED_DURATION = Types.INTEGER;

    public static final String NAME_SCHED_DURATION = "TMT_CRI.SCHED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_DURATION,NAME_SCHED_DURATION);}

    /**
     * Column PROPOSED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_START = 121;

    public static final int TYPE_PROPOSED_START = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_START = "TMT_CRI.PROPOSED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_START,NAME_PROPOSED_START);}

    /**
     * Column PROPOSED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_FINISH = 122;

    public static final int TYPE_PROPOSED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_FINISH = "TMT_CRI.PROPOSED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_FINISH,NAME_PROPOSED_FINISH);}

    /**
     * Column PROPOSED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_DURATION = 123;

    public static final int TYPE_PROPOSED_DURATION = Types.INTEGER;

    public static final String NAME_PROPOSED_DURATION = "TMT_CRI.PROPOSED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_DURATION,NAME_PROPOSED_DURATION);}

    /**
     * Column FORECAST_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_START = 124;

    public static final int TYPE_FORECAST_START = Types.TIMESTAMP;

    public static final String NAME_FORECAST_START = "TMT_CRI.FORECAST_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_START,NAME_FORECAST_START);}

    /**
     * Column FORECAST_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_FINISH = 125;

    public static final int TYPE_FORECAST_FINISH = Types.TIMESTAMP;

    public static final String NAME_FORECAST_FINISH = "TMT_CRI.FORECAST_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_FINISH,NAME_FORECAST_FINISH);}

    /**
     * Column FORECAST_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_DURATION = 126;

    public static final int TYPE_FORECAST_DURATION = Types.INTEGER;

    public static final String NAME_FORECAST_DURATION = "TMT_CRI.FORECAST_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_DURATION,NAME_FORECAST_DURATION);}

    /**
     * Column BASELINE1_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_START = 127;

    public static final int TYPE_BASELINE1_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_START = "TMT_CRI.BASELINE1_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_START,NAME_BASELINE1_START);}

    /**
     * Column BASELINE1_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_FINISH = 128;

    public static final int TYPE_BASELINE1_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_FINISH = "TMT_CRI.BASELINE1_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_FINISH,NAME_BASELINE1_FINISH);}

    /**
     * Column BASELINE1_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_DURATION = 129;

    public static final int TYPE_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE1_DURATION = "TMT_CRI.BASELINE1_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_DURATION,NAME_BASELINE1_DURATION);}

    /**
     * Column BASELINE1_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_EFFORT = 130;

    public static final int TYPE_BASELINE1_EFFORT = Types.INTEGER;

    public static final String NAME_BASELINE1_EFFORT = "TMT_CRI.BASELINE1_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_EFFORT,NAME_BASELINE1_EFFORT);}

    /**
     * Column UNRESOLVED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_UNRESOLVED_START = 131;

    public static final int TYPE_UNRESOLVED_START = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_START = "TMT_CRI.UNRESOLVED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_START,NAME_UNRESOLVED_START);}

    /**
     * Column UNRESOLVED_FINISH of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_UNRESOLVED_FINISH = 132;

    public static final int TYPE_UNRESOLVED_FINISH = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_FINISH = "TMT_CRI.UNRESOLVED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_FINISH,NAME_UNRESOLVED_FINISH);}

    /**
     * Column UNRESOLVED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_UNRESOLVED_DURATION = 133;

    public static final int TYPE_UNRESOLVED_DURATION = Types.INTEGER;

    public static final String NAME_UNRESOLVED_DURATION = "TMT_CRI.UNRESOLVED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_DURATION,NAME_UNRESOLVED_DURATION);}

    /**
     * Column CONSTRAINT_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CONSTRAINT_DT = 134;

    public static final int TYPE_CONSTRAINT_DT = Types.TIMESTAMP;

    public static final String NAME_CONSTRAINT_DT = "TMT_CRI.CONSTRAINT_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONSTRAINT_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONSTRAINT_DT,NAME_CONSTRAINT_DT);}

    /**
     * Column CONSTRAINT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONSTRAINT_TYPE = 135;

    public static final int TYPE_CONSTRAINT_TYPE = Types.INTEGER;

    public static final String NAME_CONSTRAINT_TYPE = "TMT_CRI.CONSTRAINT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONSTRAINT_TYPE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CONSTRAINT_TYPE,NAME_CONSTRAINT_TYPE);}

    /**
     * Column S_ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ACTUAL_BUDGET = 136;

    public static final int TYPE_S_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ACTUAL_BUDGET = "TMT_CRI.S_ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ACTUAL_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ACTUAL_BUDGET,NAME_S_ACTUAL_BUDGET);}

    /**
     * Column R_ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ACTUAL_BUDGET = 137;

    public static final int TYPE_R_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ACTUAL_BUDGET = "TMT_CRI.R_ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ACTUAL_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ACTUAL_BUDGET,NAME_R_ACTUAL_BUDGET);}

    /**
     * Column ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_TOTAL = 138;

    public static final int TYPE_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATION_TOTAL = "TMT_CRI.ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_TOTAL,NAME_ESTIMATION_TOTAL);}

    /**
     * Column S_ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATION_TOTAL = 139;

    public static final int TYPE_S_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_S_ESTIMATION_TOTAL = "TMT_CRI.S_ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATION_TOTAL,NAME_S_ESTIMATION_TOTAL);}

    /**
     * Column R_ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATION_TOTAL = 140;

    public static final int TYPE_R_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_R_ESTIMATION_TOTAL = "TMT_CRI.R_ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATION_TOTAL,NAME_R_ESTIMATION_TOTAL);}

    /**
     * Column WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WORK_TODATE = 141;

    public static final int TYPE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_WORK_TODATE = "TMT_CRI.WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_TODATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE,NAME_WORK_TODATE);}

    /**
     * Column BASELINE_WORK_TODATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE_WORK_TODATE = 142;

    public static final int TYPE_BASELINE_WORK_TODATE = Types.INTEGER;

    public static final String NAME_BASELINE_WORK_TODATE = "TMT_CRI.BASELINE_WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE_WORK_TODATE =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_WORK_TODATE,NAME_BASELINE_WORK_TODATE);}

    /**
     * Column SCHED_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_EFFORT = 143;

    public static final int TYPE_SCHED_EFFORT = Types.INTEGER;

    public static final String NAME_SCHED_EFFORT = "TMT_CRI.SCHED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_EFFORT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SCHED_EFFORT,NAME_SCHED_EFFORT);}

    /**
     * Column FORECAST_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_EFFORT = 144;

    public static final int TYPE_FORECAST_EFFORT = Types.INTEGER;

    public static final String NAME_FORECAST_EFFORT = "TMT_CRI.FORECAST_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_EFFORT,NAME_FORECAST_EFFORT);}

    /**
     * Column PROPOSED_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_EFFORT = 145;

    public static final int TYPE_PROPOSED_EFFORT = Types.INTEGER;

    public static final String NAME_PROPOSED_EFFORT = "TMT_CRI.PROPOSED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_EFFORT,NAME_PROPOSED_EFFORT);}

    /**
     * Column VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_VARIANCE = 146;

    public static final int TYPE_VARIANCE = Types.INTEGER;

    public static final String NAME_VARIANCE = "TMT_CRI.VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VARIANCE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_VARIANCE,NAME_VARIANCE);}

    /**
     * Column ACTUAL_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_VARIANCE = 147;

    public static final int TYPE_ACTUAL_VARIANCE = Types.INTEGER;

    public static final String NAME_ACTUAL_VARIANCE = "TMT_CRI.ACTUAL_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_VARIANCE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_VARIANCE,NAME_ACTUAL_VARIANCE);}

    /**
     * Column FORECAST_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_VARIANCE = 148;

    public static final int TYPE_FORECAST_VARIANCE = Types.INTEGER;

    public static final String NAME_FORECAST_VARIANCE = "TMT_CRI.FORECAST_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_VARIANCE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_VARIANCE,NAME_FORECAST_VARIANCE);}

    /**
     * Column BASELINE1_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_VARIANCE = 149;

    public static final int TYPE_BASELINE1_VARIANCE = Types.INTEGER;

    public static final String NAME_BASELINE1_VARIANCE = "TMT_CRI.BASELINE1_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_VARIANCE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_VARIANCE,NAME_BASELINE1_VARIANCE);}

    /**
     * Column CLIENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CLIENT_ID = 150;

    public static final int TYPE_CLIENT_ID = Types.CHAR;

    public static final String NAME_CLIENT_ID = "TMT_CRI.CLIENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLIENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CLIENT_ID,NAME_CLIENT_ID);}

    /**
     * Column CLIENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CLIENT_NAME = 151;

    public static final int TYPE_CLIENT_NAME = Types.VARCHAR;

    public static final String NAME_CLIENT_NAME = "TMT_CRI.CLIENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLIENT_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CLIENT_NAME,NAME_CLIENT_NAME);}

    /**
     * Column REASSIGN_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_REASSIGN_COUNT = 152;

    public static final int TYPE_REASSIGN_COUNT = Types.SMALLINT;

    public static final String NAME_REASSIGN_COUNT = "TMT_CRI.REASSIGN_COUNT";

    public static final String PROPERTY_REASSIGN_COUNT = "NUMBERTIMESREASSIGNED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REASSIGN_COUNT, NAME_REASSIGN_COUNT);
    }

    /**
     * Column DUPLICATE_REF_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DUPLICATE_REF_ID = 153;

    public static final int TYPE_DUPLICATE_REF_ID = Types.VARCHAR;

    public static final String NAME_DUPLICATE_REF_ID = "TMT_CRI.DUPLICATE_REF_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DUPLICATE_REF_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_DUPLICATE_REF_ID,NAME_DUPLICATE_REF_ID);}

    /**
     * Column OCCURED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_OCCURED_DATETIME = 154;

    public static final int TYPE_OCCURED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_OCCURED_DATETIME = "TMT_CRI.OCCURED_DATETIME";

    public static final String PROPERTY_OCCURED_DATETIME = "OCCUREDDATETIME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_OCCURED_DATETIME, SqlUtil
            .convertTimestampToDate(NAME_OCCURED_DATETIME));
    }

    /**
     * Column DOC_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DOC_COUNT = 155;

    public static final int TYPE_DOC_COUNT = Types.INTEGER;

    public static final String NAME_DOC_COUNT = "TMT_CRI.DOC_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DOC_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DOC_COUNT,NAME_DOC_COUNT);}

    /**
     * Column PROPOSED_BY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PROPOSED_BY_NAME = 156;

    public static final int TYPE_PROPOSED_BY_NAME = Types.VARCHAR;

    public static final String NAME_PROPOSED_BY_NAME = "TMT_CRI.PROPOSED_BY_NAME";

    public static final String PROPERTY_PROPOSED_BY_NAME = "PROPOSEDBYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_BY_NAME, NAME_PROPOSED_BY_NAME);
    }

    /**
     * Column CLOSED_BY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CLOSED_BY_NAME = 157;

    public static final int TYPE_CLOSED_BY_NAME = Types.VARCHAR;

    public static final String NAME_CLOSED_BY_NAME = "TMT_CRI.CLOSED_BY_NAME";

    public static final String PROPERTY_CLOSED_BY_NAME = "CLOSEDBYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLOSED_BY_NAME, NAME_CLOSED_BY_NAME);
    }

    /**
     * Column ESTIMATED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ESTIMATED_DURATION = 158;

    public static final int TYPE_ESTIMATED_DURATION = Types.INTEGER;

    public static final String NAME_ESTIMATED_DURATION = "TMT_CRI.ESTIMATED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_DURATION,NAME_ESTIMATED_DURATION);}

    /**
     * Column ESTIMATED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ESTIMATED_EFFORT = 159;

    public static final int TYPE_ESTIMATED_EFFORT = Types.BIGINT;

    public static final String NAME_ESTIMATED_EFFORT = "TMT_CRI.ESTIMATED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_EFFORT,NAME_ESTIMATED_EFFORT);}

    /**
     * Column HIGH_ESTIMATED_START of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_HIGH_ESTIMATED_START = 160;

    public static final int TYPE_HIGH_ESTIMATED_START = Types.TIMESTAMP;

    public static final String NAME_HIGH_ESTIMATED_START = "TMT_CRI.HIGH_ESTIMATED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_HIGH_ESTIMATED_START =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_HIGH_ESTIMATED_START,NAME_HIGH_ESTIMATED_START);}

    /**
     * Column HIGH_ESTIMATED_FINISH of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_HIGH_ESTIMATED_FINISH = 161;

    public static final int TYPE_HIGH_ESTIMATED_FINISH = Types.TIMESTAMP;

    public static final String NAME_HIGH_ESTIMATED_FINISH = "TMT_CRI.HIGH_ESTIMATED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_HIGH_ESTIMATED_FINISH =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_HIGH_ESTIMATED_FINISH,NAME_HIGH_ESTIMATED_FINISH);}

    /**
     * Column HIGH_ESTIMATED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_HIGH_ESTIMATED_DURATION = 162;

    public static final int TYPE_HIGH_ESTIMATED_DURATION = Types.INTEGER;

    public static final String NAME_HIGH_ESTIMATED_DURATION = "TMT_CRI.HIGH_ESTIMATED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_HIGH_ESTIMATED_DURATION =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_HIGH_ESTIMATED_DURATION,NAME_HIGH_ESTIMATED_DURATION);}

    /**
     * Column HIGH_ESTIMATED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_HIGH_ESTIMATED_EFFORT = 163;

    public static final int TYPE_HIGH_ESTIMATED_EFFORT = Types.BIGINT;

    public static final String NAME_HIGH_ESTIMATED_EFFORT = "TMT_CRI.HIGH_ESTIMATED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_HIGH_ESTIMATED_EFFORT =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_HIGH_ESTIMATED_EFFORT,NAME_HIGH_ESTIMATED_EFFORT);}

    /**
     * Column LOW_ESTIMATED_START of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_LOW_ESTIMATED_START = 164;

    public static final int TYPE_LOW_ESTIMATED_START = Types.TIMESTAMP;

    public static final String NAME_LOW_ESTIMATED_START = "TMT_CRI.LOW_ESTIMATED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOW_ESTIMATED_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LOW_ESTIMATED_START,NAME_LOW_ESTIMATED_START);}

    /**
     * Column LOW_ESTIMATED_FINISH of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_LOW_ESTIMATED_FINISH = 165;

    public static final int TYPE_LOW_ESTIMATED_FINISH = Types.TIMESTAMP;

    public static final String NAME_LOW_ESTIMATED_FINISH = "TMT_CRI.LOW_ESTIMATED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOW_ESTIMATED_FINISH =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LOW_ESTIMATED_FINISH,NAME_LOW_ESTIMATED_FINISH);}

    /**
     * Column LOW_ESTIMATED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LOW_ESTIMATED_DURATION = 166;

    public static final int TYPE_LOW_ESTIMATED_DURATION = Types.INTEGER;

    public static final String NAME_LOW_ESTIMATED_DURATION = "TMT_CRI.LOW_ESTIMATED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOW_ESTIMATED_DURATION =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LOW_ESTIMATED_DURATION,NAME_LOW_ESTIMATED_DURATION);}

    /**
     * Column LOW_ESTIMATED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_LOW_ESTIMATED_EFFORT = 167;

    public static final int TYPE_LOW_ESTIMATED_EFFORT = Types.BIGINT;

    public static final String NAME_LOW_ESTIMATED_EFFORT = "TMT_CRI.LOW_ESTIMATED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOW_ESTIMATED_EFFORT =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LOW_ESTIMATED_EFFORT,NAME_LOW_ESTIMATED_EFFORT);}

    /**
     * Column APPROVED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_APPROVED_EFFORT = 168;

    public static final int TYPE_APPROVED_EFFORT = Types.BIGINT;

    public static final String NAME_APPROVED_EFFORT = "TMT_CRI.APPROVED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_EFFORT,NAME_APPROVED_EFFORT);}

    /**
     * Column APPROVED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_APPROVED_DURATION = 169;

    public static final int TYPE_APPROVED_DURATION = Types.INTEGER;

    public static final String NAME_APPROVED_DURATION = "TMT_CRI.APPROVED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_DURATION,NAME_APPROVED_DURATION);}

    /**
     * Column ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BUDGET = 170;

    public static final int TYPE_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BUDGET = "TMT_CRI.ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BUDGET,NAME_ESTIMATED_BUDGET);}

    /**
     * Column R_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_BUDGET = 171;

    public static final int TYPE_R_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_BUDGET = "TMT_CRI.R_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_BUDGET,NAME_R_ESTIMATED_BUDGET);}

    /**
     * Column S_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_BUDGET = 172;

    public static final int TYPE_S_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_BUDGET = "TMT_CRI.S_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_BUDGET,NAME_S_ESTIMATED_BUDGET);}

    /**
     * Column HIGH_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_HIGH_ESTIMATED_BUDGET = 173;

    public static final int TYPE_HIGH_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_HIGH_ESTIMATED_BUDGET = "TMT_CRI.HIGH_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_HIGH_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_HIGH_ESTIMATED_BUDGET,NAME_HIGH_ESTIMATED_BUDGET);}

    /**
     * Column R_HIGH_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_HIGH_ESTIMATED_BUDGET = 174;

    public static final int TYPE_R_HIGH_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_HIGH_ESTIMATED_BUDGET = "TMT_CRI.R_HIGH_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_HIGH_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_HIGH_ESTIMATED_BUDGET,NAME_R_HIGH_ESTIMATED_BUDGET);}

    /**
     * Column S_HIGH_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_HIGH_ESTIMATED_BUDGET = 175;

    public static final int TYPE_S_HIGH_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_HIGH_ESTIMATED_BUDGET = "TMT_CRI.S_HIGH_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_HIGH_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_HIGH_ESTIMATED_BUDGET,NAME_S_HIGH_ESTIMATED_BUDGET);}

    /**
     * Column LOW_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LOW_ESTIMATED_BUDGET = 176;

    public static final int TYPE_LOW_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_LOW_ESTIMATED_BUDGET = "TMT_CRI.LOW_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOW_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LOW_ESTIMATED_BUDGET,NAME_LOW_ESTIMATED_BUDGET);}

    /**
     * Column R_LOW_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_LOW_ESTIMATED_BUDGET = 177;

    public static final int TYPE_R_LOW_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_LOW_ESTIMATED_BUDGET = "TMT_CRI.R_LOW_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_LOW_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_LOW_ESTIMATED_BUDGET,NAME_R_LOW_ESTIMATED_BUDGET);}

    /**
     * Column S_LOW_ESTIMATED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_LOW_ESTIMATED_BUDGET = 178;

    public static final int TYPE_S_LOW_ESTIMATED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_LOW_ESTIMATED_BUDGET = "TMT_CRI.S_LOW_ESTIMATED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_LOW_ESTIMATED_BUDGET =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_LOW_ESTIMATED_BUDGET,NAME_S_LOW_ESTIMATED_BUDGET);}

    /**
     * Column APPROVED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_APPROVED_BY = 179;

    public static final int TYPE_APPROVED_BY = Types.CHAR;

    public static final String NAME_APPROVED_BY = "TMT_CRI.APPROVED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_APPROVED_BY,NAME_APPROVED_BY);}

    /**
     * Column APPROVED_BY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_APPROVED_BY_NAME = 180;

    public static final int TYPE_APPROVED_BY_NAME = Types.VARCHAR;

    public static final String NAME_APPROVED_BY_NAME = "TMT_CRI.APPROVED_BY_NAME";

    public static final String PROPERTY_APPROVED_BY_NAME = "APPROVEDBYNAME";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_APPROVED_BY_NAME, NAME_APPROVED_BY_NAME);
    }

    /**
     * Column R_APPROVED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_APPROVED_BUDGET = 181;

    public static final int TYPE_R_APPROVED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_APPROVED_BUDGET = "TMT_CRI.R_APPROVED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_APPROVED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_APPROVED_BUDGET,NAME_R_APPROVED_BUDGET);}

    /**
     * Column EXPECTED_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPECTED_REVENUE = 182;

    public static final int TYPE_EXPECTED_REVENUE = Types.DECIMAL;

    public static final String NAME_EXPECTED_REVENUE = "TMT_CRI.EXPECTED_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPECTED_REVENUE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_REVENUE,NAME_EXPECTED_REVENUE);}

    /**
     * Column EXPECTED_MARGIN of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPECTED_MARGIN = 183;

    public static final int TYPE_EXPECTED_MARGIN = Types.DECIMAL;

    public static final String NAME_EXPECTED_MARGIN = "TMT_CRI.EXPECTED_MARGIN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPECTED_MARGIN = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_MARGIN,NAME_EXPECTED_MARGIN);}

    /**
     * Column PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PLANNED_BUDGET = 184;

    public static final int TYPE_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_PLANNED_BUDGET = "TMT_CRI.PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_BUDGET,NAME_PLANNED_BUDGET);}

    /**
     * Column R_PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_PLANNED_BUDGET = 185;

    public static final int TYPE_R_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_PLANNED_BUDGET = "TMT_CRI.R_PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_PLANNED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_PLANNED_BUDGET,NAME_R_PLANNED_BUDGET);}

    /**
     * Column S_PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_PLANNED_BUDGET = 186;

    public static final int TYPE_S_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_PLANNED_BUDGET = "TMT_CRI.S_PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_PLANNED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_PLANNED_BUDGET,NAME_S_PLANNED_BUDGET);}

    /**
     * Column PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROPOSED_BUDGET = 187;

    public static final int TYPE_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_PROPOSED_BUDGET = "TMT_CRI.PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_BUDGET,NAME_PROPOSED_BUDGET);}

    /**
     * Column R_PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_PROPOSED_BUDGET = 188;

    public static final int TYPE_R_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_PROPOSED_BUDGET = "TMT_CRI.R_PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_PROPOSED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_PROPOSED_BUDGET,NAME_R_PROPOSED_BUDGET);}

    /**
     * Column S_PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_PROPOSED_BUDGET = 189;

    public static final int TYPE_S_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_PROPOSED_BUDGET = "TMT_CRI.S_PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_PROPOSED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_PROPOSED_BUDGET,NAME_S_PROPOSED_BUDGET);}

    /**
     * Column EXPECTED_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXPECTED_COST = 190;

    public static final int TYPE_EXPECTED_COST = Types.DECIMAL;

    public static final String NAME_EXPECTED_COST = "TMT_CRI.EXPECTED_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPECTED_COST = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_COST,NAME_EXPECTED_COST);}

    /**
     * Column S_APPROVED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_APPROVED_BUDGET = 191;

    public static final int TYPE_S_APPROVED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_APPROVED_BUDGET = "TMT_CRI.S_APPROVED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_APPROVED_BUDGET = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_APPROVED_BUDGET,NAME_S_APPROVED_BUDGET);}

    /**
     * Column SCORECARD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCORECARD_COUNT = 192;

    public static final int TYPE_SCORECARD_COUNT = Types.INTEGER;

    public static final String NAME_SCORECARD_COUNT = "TMT_CRI.SCORECARD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_COUNT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_COUNT,NAME_SCORECARD_COUNT);}

    /**
     * Column BASELINE2_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_START = 193;

    public static final int TYPE_BASELINE2_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_START = "TMT_CRI.BASELINE2_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_START = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_START,NAME_BASELINE2_START);}

    /**
     * Column BASELINE2_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_FINISH = 194;

    public static final int TYPE_BASELINE2_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_FINISH = "TMT_CRI.BASELINE2_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_FINISH = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_FINISH,NAME_BASELINE2_FINISH);}

    /**
     * Column BASELINE2_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_DURATION = 195;

    public static final int TYPE_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE2_DURATION = "TMT_CRI.BASELINE2_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_DURATION,NAME_BASELINE2_DURATION);}

    /**
     * Column BASELINE2_EFFORT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_EFFORT = 196;

    public static final int TYPE_BASELINE2_EFFORT = Types.INTEGER;

    public static final String NAME_BASELINE2_EFFORT = "TMT_CRI.BASELINE2_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_EFFORT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_EFFORT,NAME_BASELINE2_EFFORT);}

    /**
     * Column CHANGE_TO_TASK of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CHANGE_TO_TASK = 197;

    public static final int TYPE_CHANGE_TO_TASK = Types.SMALLINT;

    public static final String NAME_CHANGE_TO_TASK = "TMT_CRI.CHANGE_TO_TASK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHANGE_TO_TASK = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CHANGE_TO_TASK,NAME_CHANGE_TO_TASK);}

    /**
     * Column APPROVED_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_APPROVED_REVENUE = 198;

    public static final int TYPE_APPROVED_REVENUE = Types.DECIMAL;

    public static final String NAME_APPROVED_REVENUE = "TMT_CRI.APPROVED_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_REVENUE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_REVENUE,NAME_APPROVED_REVENUE);}

    /**
     * Column APPROVED_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_APPROVED_SAVING = 199;

    public static final int TYPE_APPROVED_SAVING = Types.DECIMAL;

    public static final String NAME_APPROVED_SAVING = "TMT_CRI.APPROVED_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_SAVING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_SAVING,NAME_APPROVED_SAVING);}

    /**
     * Column TARGET_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_TARGET_DATE = 200;

    public static final int TYPE_TARGET_DATE = Types.DATE;

    public static final String NAME_TARGET_DATE = "TMT_CRI.TARGET_DATE";

    public static final String PROPERTY_TARGET_DATE = "TARGETDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TARGET_DATE, NAME_TARGET_DATE);
    }

    /**
     * Column ESTIMATED_REVENUE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_REVENUE = 201;

    public static final int TYPE_ESTIMATED_REVENUE = Types.DECIMAL;

    public static final String NAME_ESTIMATED_REVENUE = "TMT_CRI.ESTIMATED_REVENUE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_REVENUE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_REVENUE,NAME_ESTIMATED_REVENUE);}

    /**
     * Column ESTIMATED_SAVING of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_SAVING = 202;

    public static final int TYPE_ESTIMATED_SAVING = Types.DECIMAL;

    public static final String NAME_ESTIMATED_SAVING = "TMT_CRI.ESTIMATED_SAVING";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_SAVING = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_SAVING,NAME_ESTIMATED_SAVING);}

    /**
     * Column TOTAL_DURATION of type Types.BIGINT mapped to Long.
     */
    public static final int ID_TOTAL_DURATION = 203;

    public static final int TYPE_TOTAL_DURATION = Types.BIGINT;

    public static final String NAME_TOTAL_DURATION = "TMT_CRI.TOTAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOTAL_DURATION = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TOTAL_DURATION,NAME_TOTAL_DURATION);}

    /**
     * Column DUPLICATE_KEY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DUPLICATE_KEY = 204;

    public static final int TYPE_DUPLICATE_KEY = Types.INTEGER;

    public static final String NAME_DUPLICATE_KEY = "TMT_CRI.DUPLICATE_KEY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DUPLICATE_KEY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DUPLICATE_KEY,NAME_DUPLICATE_KEY);}

    /**
     * Column PREVIOUS_WKF_STEP of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PREVIOUS_WKF_STEP = 205;

    public static final int TYPE_PREVIOUS_WKF_STEP = Types.VARCHAR;

    public static final String NAME_PREVIOUS_WKF_STEP = "TMT_CRI.PREVIOUS_WKF_STEP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PREVIOUS_WKF_STEP = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PREVIOUS_WKF_STEP,NAME_PREVIOUS_WKF_STEP);}

    /**
     * Column PRIVATE of type Types.CHAR mapped to String.
     */
    public static final int ID_PRIVATE = 206;

    public static final int TYPE_PRIVATE = Types.CHAR;

    public static final String NAME_PRIVATE = "TMT_CRI.PRIVATE";

    public static final String PROPERTY_PRIVATE = "ELEMENTSECURITYTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIVATE, NAME_PRIVATE);
    }

    public static final String PROPERTY_PRIVATE2 = "PRIVATEFOLDER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIVATE2, SqlUtil.getBooleanValue(
            NAME_PRIVATE, "Y"));
    }

    /**
     * Column VISIBLE_AT_PARENT of type Types.CHAR mapped to String.
     */
    public static final int ID_VISIBLE_AT_PARENT = 207;

    public static final int TYPE_VISIBLE_AT_PARENT = Types.CHAR;

    public static final String NAME_VISIBLE_AT_PARENT = "TMT_CRI.VISIBLE_AT_PARENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VISIBLE_AT_PARENT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_VISIBLE_AT_PARENT,NAME_VISIBLE_AT_PARENT);}

    /**
     * Column TIMEPHASED_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TIMEPHASED_BITS = 208;

    public static final int TYPE_TIMEPHASED_BITS = Types.INTEGER;

    public static final String NAME_TIMEPHASED_BITS = "TMT_CRI.TIMEPHASED_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMEPHASED_BITS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_TIMEPHASED_BITS,NAME_TIMEPHASED_BITS);}

    /**
     * Column APPROVED_IMPACT_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_APPROVED_IMPACT_LEVEL_ID = 209;

    public static final int TYPE_APPROVED_IMPACT_LEVEL_ID = Types.INTEGER;

    public static final String NAME_APPROVED_IMPACT_LEVEL_ID = "TMT_CRI.APPROVED_IMPACT_LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_APPROVED_IMPACT_LEVEL_ID =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_APPROVED_IMPACT_LEVEL_ID,NAME_APPROVED_IMPACT_LEVEL_ID);}

    /**
     * Column ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL = 210;

    public static final int TYPE_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL = "TMT_CRI.ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL,NAME_ESTIMATED_ACTUAL);}

    /**
     * Column ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ETC = 211;

    public static final int TYPE_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ETC = "TMT_CRI.ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ETC = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ETC,NAME_ESTIMATED_ETC);}

    /**
     * Column R_ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ACTUAL = 212;

    public static final int TYPE_R_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ACTUAL = "TMT_CRI.R_ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ACTUAL,NAME_R_ESTIMATED_ACTUAL);}

    /**
     * Column R_ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ETC = 213;

    public static final int TYPE_R_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ETC = "TMT_CRI.R_ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ETC,NAME_R_ESTIMATED_ETC);}

    /**
     * Column S_ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_ACTUAL = 214;

    public static final int TYPE_S_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_ACTUAL = "TMT_CRI.S_ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_ACTUAL,NAME_S_ESTIMATED_ACTUAL);}

    /**
     * Column S_ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_ETC = 215;

    public static final int TYPE_S_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_ETC = "TMT_CRI.S_ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_ETC,NAME_S_ESTIMATED_ETC);}

    /**
     * Column ESTIMATED_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS = 216;

    public static final int TYPE_ESTIMATED_HOURS = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS = "TMT_CRI.ESTIMATED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS,NAME_ESTIMATED_HOURS);}

    /**
     * Column ESTIMATED_ACTUAL_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL_HOURS = 217;

    public static final int TYPE_ESTIMATED_ACTUAL_HOURS = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL_HOURS = "TMT_CRI.ESTIMATED_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL_HOURS,NAME_ESTIMATED_ACTUAL_HOURS);}

    /**
     * Column ESTIMATED_HOURS_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS_ETC = 218;

    public static final int TYPE_ESTIMATED_HOURS_ETC = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS_ETC = "TMT_CRI.ESTIMATED_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS_ETC = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS_ETC,NAME_ESTIMATED_HOURS_ETC);}

    /**
     * Column R_ESTIMATED_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS = 219;

    public static final int TYPE_R_ESTIMATED_HOURS = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS = "TMT_CRI.R_ESTIMATED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS,NAME_R_ESTIMATED_HOURS);}

    /**
     * Column R_ESTIMATED_ACTUAL_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ACTUAL_HOURS = 220;

    public static final int TYPE_R_ESTIMATED_ACTUAL_HOURS = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ACTUAL_HOURS = "TMT_CRI.R_ESTIMATED_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ACTUAL_HOURS,NAME_R_ESTIMATED_ACTUAL_HOURS);}

    /**
     * Column R_ESTIMATED_HOURS_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS_ETC = 221;

    public static final int TYPE_R_ESTIMATED_HOURS_ETC = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS_ETC = "TMT_CRI.R_ESTIMATED_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS_ETC =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS_ETC,NAME_R_ESTIMATED_HOURS_ETC);}

    /**
     * Column EXPECTED_PAYBACK_PERIOD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPECTED_PAYBACK_PERIOD = 222;

    public static final int TYPE_EXPECTED_PAYBACK_PERIOD = Types.INTEGER;

    public static final String NAME_EXPECTED_PAYBACK_PERIOD = "TMT_CRI.EXPECTED_PAYBACK_PERIOD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPECTED_PAYBACK_PERIOD =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_PAYBACK_PERIOD,NAME_EXPECTED_PAYBACK_PERIOD);}

    /**
     * Column EXPECTED_PAYBACK_MONTH of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPECTED_PAYBACK_MONTH = 223;

    public static final int TYPE_EXPECTED_PAYBACK_MONTH = Types.INTEGER;

    public static final String NAME_EXPECTED_PAYBACK_MONTH = "TMT_CRI.EXPECTED_PAYBACK_MONTH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPECTED_PAYBACK_MONTH =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EXPECTED_PAYBACK_MONTH,NAME_EXPECTED_PAYBACK_MONTH);}

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 224;

    public static final int TYPE_ASSET_ID = Types.CHAR;

    public static final String NAME_ASSET_ID = "TMT_CRI.ASSET_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID);}

    /**
     * Column ASSET_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSET_NAME = 225;

    public static final int TYPE_ASSET_NAME = Types.VARCHAR;

    public static final String NAME_ASSET_NAME = "TMT_CRI.ASSET_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_NAME,NAME_ASSET_NAME);}

    /**
     * Column FINANCIAL_CATEGORY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FINANCIAL_CATEGORY = 226;

    public static final int TYPE_FINANCIAL_CATEGORY = Types.INTEGER;

    public static final String NAME_FINANCIAL_CATEGORY = "TMT_CRI.FINANCIAL_CATEGORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FINANCIAL_CATEGORY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_FINANCIAL_CATEGORY,NAME_FINANCIAL_CATEGORY);}

    /**
     * Column ASSET_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ASSET_FLAG = 227;

    public static final int TYPE_ASSET_FLAG = Types.INTEGER;

    public static final String NAME_ASSET_FLAG = "TMT_CRI.ASSET_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_FLAG,NAME_ASSET_FLAG);}

    /**
     * Column DEFAULT_COL of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_COL = 228;

    public static final int TYPE_DEFAULT_COL = Types.CHAR;

    public static final String NAME_DEFAULT_COL = "TMT_CRI.DEFAULT_COL";

    public static final String PROPERTY_DEFAULT_COL = "DEFAULTSCOPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_COL, SqlUtil.getBooleanValue(
            NAME_DEFAULT_COL, "Y"));
    }

    /**
     * Column TEMPLATED of type Types.CHAR mapped to String.
     */
    public static final int ID_TEMPLATED = 229;

    public static final int TYPE_TEMPLATED = Types.CHAR;

    public static final String NAME_TEMPLATED = "TMT_CRI.TEMPLATED";

    public static final String PROPERTY_TEMPLATED = "PUBLISHED";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TEMPLATED, NAME_TEMPLATED);
    }

    /**
     * Column T_PARENT_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_T_PARENT_ID = 230;

    public static final int TYPE_T_PARENT_ID = Types.VARCHAR;

    public static final String NAME_T_PARENT_ID = "TMT_CRI.T_PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_T_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_T_PARENT_ID,NAME_T_PARENT_ID);}

    /**
     * Column T_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_T_RANK = 231;

    public static final int TYPE_T_RANK = Types.VARCHAR;

    public static final String NAME_T_RANK = "TMT_CRI.T_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_T_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_T_RANK,NAME_T_RANK);}

    /**
     * Column ELEMENT_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ELEMENT_FLAG = 232;

    public static final int TYPE_ELEMENT_FLAG = Types.SMALLINT;

    public static final String NAME_ELEMENT_FLAG = "TMT_CRI.ELEMENT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_FLAG,NAME_ELEMENT_FLAG);}

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 233;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "TMT_CRI.CONFIG_FLAG";

    public static final String PROPERTY_CONFIG_FLAG = "MANDATORY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, SqlUtil.getBooleanValue(
            NAME_CONFIG_FLAG, 1));
    }

    /**
     * Column SUB_TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUB_TYPE_ID = 234;

    public static final int TYPE_SUB_TYPE_ID = Types.INTEGER;

    public static final String NAME_SUB_TYPE_ID;

    static
    {
        if (TempReqProOverride.DISABLE_REQPRO)
        {
            NAME_SUB_TYPE_ID = "TMT_CRI.CONFIG_FLAG";
        }
        else
        {
            NAME_SUB_TYPE_ID = "TMT_CRI.SUB_TYPE_ID";
        }
    }

    public static final String TABLE_NAME = "TMT_CRI";

    /**
     * Create an array of type string containing all the fields of the TMT_CRI
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_PARENT_RANK, NAME_TYPE_ID, NAME_LEVEL_ID,
            NAME_PROJECT_ID, NAME_PARENT_ID, NAME_PARENT_TYPE_ID,
            NAME_ELEMENT_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_NAME, NAME_TRANSFERRED,
            NAME_STAGE_ID, NAME_CREATED_BY, NAME_CREATED_DATETIME,
            NAME_REFERENCE_NUMBER, NAME_EXTERNAL_REFERENCE, NAME_PRIORITY_ID,
            NAME_PROPOSED_BY, NAME_PROPOSED_DATETIME, NAME_IMPACT_LEVEL_ID,
            NAME_SD, NAME_FD, NAME_START_DATETIME, NAME_FINISH_DATETIME,
            NAME_PLANNED_START, NAME_PLANNED_FINISH, NAME_PLANNED_EFFORT,
            NAME_PLANNED_DURATION, NAME_ESTIMATED_START, NAME_ESTIMATED_FINISH,
            NAME_MITIGATION_COST, NAME_SCOPE_RANK_ID, NAME_SCHEDULE_RANK_ID,
            NAME_COST_RANK_ID, NAME_QUALITY_RANK_ID, NAME_PROBABILITY_RANK_ID,
            NAME_PRECISION_RANK_ID, NAME_EXPOSURE_ID, NAME_CONSEQUENSE_COST,
            NAME_PROBABILITY_COST, NAME_APPROVED_DATETIME, NAME_APPROVED_START,
            NAME_APPROVED_FINISH, NAME_APPROVED_BUDGET, NAME_CLOSED_BY,
            NAME_CLOSED_DATETIME, NAME_PLANNING_UNIT, NAME_PLANNING_TYPE,
            NAME_ESTIMATION_BUDGET, NAME_CURRENT_BUDGET, NAME_FORECAST_BUDGET,
            NAME_ACTUAL_BUDGET, NAME_BASELINE1_BUDGET, NAME_BASELINE1_DATETIME,
            NAME_BASELINE2_BUDGET, NAME_BASELINE2_DATETIME,
            NAME_BASELINE3_BUDGET, NAME_BASELINE3_DATETIME,
            NAME_S_ESTIMATION_BUDGET, NAME_S_CURRENT_BUDGET,
            NAME_S_FORECAST_BUDGET, NAME_S_BASELINE1_BUDGET,
            NAME_S_BASELINE1_DATETIME, NAME_S_BASELINE2_BUDGET,
            NAME_S_BASELINE2_DATETIME, NAME_S_BASELINE3_BUDGET,
            NAME_S_BASELINE3_DATETIME, NAME_R_ESTIMATION_BUDGET,
            NAME_R_CURRENT_BUDGET, NAME_R_FORECAST_BUDGET,
            NAME_R_BASELINE1_BUDGET, NAME_R_BASELINE2_BUDGET,
            NAME_R_BASELINE3_BUDGET, NAME_LAST_WF_TYPE, NAME_LAST_WF_BY,
            NAME_LAST_WF_DATETIME, NAME_MAX_REV_REACHED, NAME_MAX_REVISION,
            NAME_MAJOR_REVISION, NAME_MINOR_REVISION, NAME_REVISION_IMPACT,
            NAME_REVISION_TYPE, NAME_REVISION_HISTORY,
            NAME_TOOLTIP_DESCRIPTION, NAME_STATUS_ID, NAME_PUBLISHED,
            NAME_SHARED, NAME_IMPACT_RANK_ID, NAME_ASSIGN_NAMES,
            NAME_ASSIGN_COUNT, NAME_REC_USER, NAME_RTF_BITS, NAME_LAST_CHECKIN,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_LAST_TRANSFER_DT,
            NAME_WBS_ID, NAME_STAGE_LAST_DT, NAME_SCORE, NAME_WEIGHT,
            NAME_CONVERTED_SCORE, NAME_CONVERTED_WEIGHT, NAME_SCORECARD_ID,
            NAME_PUBLISHED_DT, NAME_PRIORITY, NAME_ETC, NAME_MILESTONE_PERCENT,
            NAME_PERCENT_COMPLETE, NAME_REC_FLAGS, NAME_ROLLUP_BITS,
            NAME_WBS_ASSIGNED, NAME_DURATION, NAME_ACTUAL_START,
            NAME_ACTUAL_FINISH, NAME_ACTUAL_DURATION, NAME_SCHED_START,
            NAME_SCHED_FINISH, NAME_SCHED_DURATION, NAME_PROPOSED_START,
            NAME_PROPOSED_FINISH, NAME_PROPOSED_DURATION, NAME_FORECAST_START,
            NAME_FORECAST_FINISH, NAME_FORECAST_DURATION, NAME_BASELINE1_START,
            NAME_BASELINE1_FINISH, NAME_BASELINE1_DURATION,
            NAME_BASELINE1_EFFORT, NAME_UNRESOLVED_START,
            NAME_UNRESOLVED_FINISH, NAME_UNRESOLVED_DURATION,
            NAME_CONSTRAINT_DT, NAME_CONSTRAINT_TYPE, NAME_S_ACTUAL_BUDGET,
            NAME_R_ACTUAL_BUDGET, NAME_ESTIMATION_TOTAL,
            NAME_S_ESTIMATION_TOTAL, NAME_R_ESTIMATION_TOTAL, NAME_WORK_TODATE,
            NAME_BASELINE_WORK_TODATE, NAME_SCHED_EFFORT, NAME_FORECAST_EFFORT,
            NAME_PROPOSED_EFFORT, NAME_VARIANCE, NAME_ACTUAL_VARIANCE,
            NAME_FORECAST_VARIANCE, NAME_BASELINE1_VARIANCE, NAME_CLIENT_ID,
            NAME_CLIENT_NAME, NAME_REASSIGN_COUNT, NAME_DUPLICATE_REF_ID,
            NAME_OCCURED_DATETIME, NAME_DOC_COUNT, NAME_PROPOSED_BY_NAME,
            NAME_CLOSED_BY_NAME, NAME_ESTIMATED_DURATION,
            NAME_ESTIMATED_EFFORT, NAME_HIGH_ESTIMATED_START,
            NAME_HIGH_ESTIMATED_FINISH, NAME_HIGH_ESTIMATED_DURATION,
            NAME_HIGH_ESTIMATED_EFFORT, NAME_LOW_ESTIMATED_START,
            NAME_LOW_ESTIMATED_FINISH, NAME_LOW_ESTIMATED_DURATION,
            NAME_LOW_ESTIMATED_EFFORT, NAME_APPROVED_EFFORT,
            NAME_APPROVED_DURATION, NAME_ESTIMATED_BUDGET,
            NAME_R_ESTIMATED_BUDGET, NAME_S_ESTIMATED_BUDGET,
            NAME_HIGH_ESTIMATED_BUDGET, NAME_R_HIGH_ESTIMATED_BUDGET,
            NAME_S_HIGH_ESTIMATED_BUDGET, NAME_LOW_ESTIMATED_BUDGET,
            NAME_R_LOW_ESTIMATED_BUDGET, NAME_S_LOW_ESTIMATED_BUDGET,
            NAME_APPROVED_BY, NAME_APPROVED_BY_NAME, NAME_R_APPROVED_BUDGET,
            NAME_EXPECTED_REVENUE, NAME_EXPECTED_MARGIN, NAME_PLANNED_BUDGET,
            NAME_R_PLANNED_BUDGET, NAME_S_PLANNED_BUDGET, NAME_PROPOSED_BUDGET,
            NAME_R_PROPOSED_BUDGET, NAME_S_PROPOSED_BUDGET, NAME_EXPECTED_COST,
            NAME_S_APPROVED_BUDGET, NAME_SCORECARD_COUNT, NAME_BASELINE2_START,
            NAME_BASELINE2_FINISH, NAME_BASELINE2_DURATION,
            NAME_BASELINE2_EFFORT, NAME_CHANGE_TO_TASK, NAME_APPROVED_REVENUE,
            NAME_APPROVED_SAVING, NAME_TARGET_DATE, NAME_ESTIMATED_REVENUE,
            NAME_ESTIMATED_SAVING, NAME_TOTAL_DURATION, NAME_DUPLICATE_KEY,
            NAME_PREVIOUS_WKF_STEP, NAME_PRIVATE, NAME_VISIBLE_AT_PARENT,
            NAME_TIMEPHASED_BITS, NAME_APPROVED_IMPACT_LEVEL_ID,
            NAME_ESTIMATED_ACTUAL, NAME_ESTIMATED_ETC, NAME_R_ESTIMATED_ACTUAL,
            NAME_R_ESTIMATED_ETC, NAME_S_ESTIMATED_ACTUAL,
            NAME_S_ESTIMATED_ETC, NAME_ESTIMATED_HOURS,
            NAME_ESTIMATED_ACTUAL_HOURS, NAME_ESTIMATED_HOURS_ETC,
            NAME_R_ESTIMATED_HOURS, NAME_R_ESTIMATED_ACTUAL_HOURS,
            NAME_R_ESTIMATED_HOURS_ETC, NAME_EXPECTED_PAYBACK_PERIOD,
            NAME_EXPECTED_PAYBACK_MONTH, NAME_ASSET_ID, NAME_ASSET_NAME,
            NAME_FINANCIAL_CATEGORY, NAME_ASSET_FLAG, NAME_DEFAULT_COL,
            NAME_TEMPLATED, NAME_T_PARENT_ID, NAME_T_RANK, NAME_ELEMENT_FLAG,
            NAME_CONFIG_FLAG, NAME_SUB_TYPE_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new TmtCriManager instance.
     * 
     * @return TmtCriManager
     */
    public ScopeManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        //TODO validate the primary key field name.
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /**
     * Returns TRUE if the int's value is 1, and false otherwise.
     * 
     * @param folder
     * @return
     */
    private Boolean getBooleanFrom01(int booleanInt)
    {
        Boolean booleanValue = Boolean.FALSE;
        if (booleanInt == 1)
        {
            booleanValue = Boolean.TRUE;
        }
        return booleanValue;
    }

    /**
     * Returns TRUE if the string's value is "Y", and false otherwise.
     * 
     * @param folder
     * @return
     */
    private Boolean getBooleanFromYN(String booleanString)
    {
        Boolean booleanValue = Boolean.FALSE;
        if (booleanString != null && booleanString.equalsIgnoreCase("Y"))
        {
            booleanValue = Boolean.TRUE;
        }
        return booleanValue;
    }

    private PrecisionType getPrecisionType(Integer type)
    {
        if (type == null)
            return null;

        switch (type.intValue())
        {
        case 0:
            return PrecisionType.Low;
        case 1:
            return PrecisionType.Medium;
        case 2:
            return PrecisionType.High;
        }

        return null;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        int subTypeID = resultSet.getInt(ID_SUB_TYPE_ID);

        if (TempReqProOverride.DISABLE_REQPRO || subTypeID == 0)
        {
            return containerMap.createContainer(typeId);
        }
        else if (subTypeID == 8 || subTypeID == 9) //CQ supported subTypeIDs
        {
            return containerMap.createContainer(typeId);
        }
        else if (ReqProUtil.isReqProObject(new Integer(subTypeID)))
        {
            Integer iSubTypeID = new Integer(subTypeID);
            ScopeElement reqProObject = null;
            //Here if the subTypeID != 0, we use it to 
            //create the ReqPro containers instead of the typeId
            if (ReqProUtil.isReqProIntegrationFolder(iSubTypeID))
            {
                reqProObject = new ReqProIntegrationFolder();
            }
            else if (ReqProUtil.isReqProScopeFolder(iSubTypeID))
            {
                reqProObject = new ReqProScopeFolder();
            }
            else if (ReqProUtil.isReqProRequirement(iSubTypeID))
            {
                reqProObject = new ReqProRequirement();
            }

            if (reqProObject != null)
            {
                reqProObject.assignTypeID(new Integer(typeId));
                return reqProObject;
            }
        }
        ExceptionUtil.handleUnsupportedSubTypeId(this, subTypeID);
        return null;

    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ScopeElement temp = (ScopeElement) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);
        String priority = rs.getString(ID_PRIORITY);
        String rank = rs.getString(ID_PRIORITY_ID);

        // ScopeElement fields
        if (name != null)
        {
            name = name.trim();
        }
        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            temp.deltaElementSecurityType(decodeScopeElementSecurityType(rs
                .getString(ID_PRIVATE)));
            temp.deltaWbsSortingRank(rs.getString(ID_RANK));
            temp.deltaParentSortingRank(rs.getString(ID_PARENT_RANK));
        }
        else
        {
            temp.setName(name);
            temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            temp.setElementSecurityType(decodeScopeElementSecurityType(rs
                .getString(ID_PRIVATE)));
            temp.setWbsSortingRank(rs.getString(ID_RANK));
            temp.setParentSortingRank(rs.getString(ID_PARENT_RANK));
        }

        // AbstractAggregateScope fields
        if (rpmObject instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope abstractAggregateScope = (AbstractAggregateScope) rpmObject;

            // Reset read-only fields to original values - never do an isDelta
            // referenceNumber
            abstractAggregateScope.setReferenceNumber(rs
                .getString(ID_REFERENCE_NUMBER));
            // transferedUnderWBSRoot
            Boolean transferedUnderWbsRoot = (rs.getString(ID_WBS_ID) == null)
                    ? Boolean.FALSE : Boolean.TRUE;
            abstractAggregateScope
                .setTransferedUnderWbsRoot(transferedUnderWbsRoot);

            if (isDelta)
            {
                // closedByDate
                Date closedByDate = rs.getDate(ID_CLOSED_DATETIME);
                abstractAggregateScope.deltaClosedByDate(DateUtil
                    .convertToCalendar(closedByDate));
                // closedByName
                abstractAggregateScope.deltaClosedByName(rs
                    .getString(ID_CLOSED_BY_NAME));
                // estimatedFinishDate
                Date estimatedFinishDate = rs.getDate(ID_ESTIMATED_FINISH);
                abstractAggregateScope.deltaEstimatedFinishDate(DateUtil
                    .convertToCalendar(estimatedFinishDate));
                // estimatedStartDate
                Date estimatedStartDate = rs.getDate(ID_ESTIMATED_START);
                abstractAggregateScope.deltaEstimatedStartDate(DateUtil
                    .convertToCalendar(estimatedStartDate));
                // priority
                if (priority != null)
                {
                    abstractAggregateScope.deltaPriority(Integer
                        .valueOf(priority));
                }
                // proposedByDate
                Date proposedByDate = rs.getDate(ID_PROPOSED_DATETIME);
                abstractAggregateScope.deltaProposedByDate(DateUtil
                    .convertToCalendar(proposedByDate));

                // proposedByName
                abstractAggregateScope.deltaProposedByName(rs
                    .getString(ID_PROPOSED_BY_NAME));

            }
            else
            {
                // closedByDate
                Date closedByDate = rs.getDate(ID_CLOSED_DATETIME);
                abstractAggregateScope.setClosedByDate(DateUtil
                    .convertToCalendar(closedByDate));
                // closedByName
                abstractAggregateScope.setClosedByName(rs
                    .getString(ID_CLOSED_BY_NAME));
                // estimatedFinishDate
                Date estimatedFinishDate = rs.getDate(ID_ESTIMATED_FINISH);
                abstractAggregateScope.setEstimatedFinishDate(DateUtil
                    .convertToCalendar(estimatedFinishDate));
                // estimatedStartDate
                Date estimatedStartDate = rs.getDate(ID_ESTIMATED_START);
                abstractAggregateScope.setEstimatedStartDate(DateUtil
                    .convertToCalendar(estimatedStartDate));
                // priority
                if (priority != null)
                {
                    abstractAggregateScope.setPriority(Integer
                        .valueOf(priority));
                }
                // proposedByDate
                Date proposedByDate = rs.getDate(ID_PROPOSED_DATETIME);
                abstractAggregateScope.setProposedByDate(DateUtil
                    .convertToCalendar(proposedByDate));
                //proposedByName
                abstractAggregateScope.setProposedByName(rs
                    .getString(ID_PROPOSED_BY_NAME));
            }
        }

        // AbstractScope fields
        if (rpmObject instanceof AbstractScope)
        {
            AbstractScope abstractScope = (AbstractScope) rpmObject;

            // Reset read-only fields to original values - never do an isDelta
            // duplicateNumber
            abstractScope.setDuplicateNumber(rs.getString(ID_DUPLICATE_KEY));

            if (isDelta)
            {
                // approvedByDate
                Date approvedByDate = rs.getDate(ID_APPROVED_DATETIME);
                abstractScope.deltaApprovedByDate(DateUtil
                    .convertToCalendar(approvedByDate));
                // approvedByName
                abstractScope.deltaApprovedByName(rs
                    .getString(ID_APPROVED_BY_NAME));
                // defaultScope
                abstractScope.deltaDefaultScope(getBooleanFromYN(rs
                    .getString(ID_DEFAULT_COL)));
                // externalReferenceNumber
                abstractScope.deltaExternalReferenceNumber(rs
                    .getString(ID_EXTERNAL_REFERENCE));
                // publishedType
                PublishedType pType = (PublishedType) EnumeratedTypeUtil
                    .getInstance().getType(ScopeElement.class,
                        PublishedType.class, rs.getString(ID_TEMPLATED));
                abstractScope.deltaPublished(pType);
                // rank
                if (rank != null)
                {
                    abstractScope.deltaRank(Integer.valueOf(rank));
                }
                // targetDate
                Date targetDate = rs.getDate(ID_TARGET_DATE);
                abstractScope.deltaTargetDate(DateUtil
                    .convertToCalendar(targetDate));

                // task
                String typeId = rs.getString(ID_TYPE_ID);
                if (abstractScopeTaskIds.contains(Integer.valueOf(typeId)))
                {
                    // this is an [AbstractScope]Task
                    abstractScope.deltaTask(new Boolean(true));
                }
                else
                {
                    abstractScope.deltaTask(new Boolean(false));
                }
            }
            else
            {
                // approvedByDate
                Date approvedByDate = rs.getDate(ID_APPROVED_DATETIME);
                abstractScope.setApprovedByDate(DateUtil
                    .convertToCalendar(approvedByDate));
                // approvedByName
                abstractScope.setApprovedByName(rs
                    .getString(ID_APPROVED_BY_NAME));
                // defaultScope
                abstractScope.setDefaultScope(getBooleanFromYN(rs
                    .getString(ID_DEFAULT_COL)));
                // externalReferenceNumber
                abstractScope.setExternalReferenceNumber(rs
                    .getString(ID_EXTERNAL_REFERENCE));
                // publishedType
                PublishedType pType = (PublishedType) EnumeratedTypeUtil
                    .getInstance().getType(ScopeElement.class,
                        PublishedType.class, rs.getString(ID_TEMPLATED));
                abstractScope.setPublished(pType);
                // rank
                if (rank != null)
                {
                    abstractScope.setRank(Integer.valueOf(rank));
                }
                // targetDate
                Date targetDate = rs.getDate(ID_TARGET_DATE);
                abstractScope.setTargetDate(DateUtil
                    .convertToCalendar(targetDate));

                // task
                String typeId = rs.getString(ID_TYPE_ID);
                if (abstractScopeTaskIds.contains(Integer.valueOf(typeId)))
                {
                    // this is an [AbstractScope]Task
                    abstractScope.setTask(new Boolean(true));
                }
                else
                {
                    abstractScope.setTask(new Boolean(false));
                }
            }
        }

        // AbstractScopeTask fields
        if (rpmObject instanceof AbstractScopeTask)
        {
            AbstractScopeTask abstractScopeTask = (AbstractScopeTask) rpmObject;

            // Reset read-only fields to original values - never do an isDelta
            // referenceNumber
            abstractScopeTask.setReferenceNumber(rs
                .getString(ID_REFERENCE_NUMBER));
            // transferedUnderWBSRoot
            Boolean transferedUnderWbsRoot = (rs.getString(ID_WBS_ID) == null)
                    ? Boolean.FALSE : Boolean.TRUE;
            abstractScopeTask.setTransferedUnderWbsRoot(transferedUnderWbsRoot);

            if (isDelta)
            {
                // closedByDate
                Date closedByDate = rs.getDate(ID_CLOSED_DATETIME);
                abstractScopeTask.deltaClosedByDate(DateUtil
                    .convertToCalendar(closedByDate));
                // closedByName
                abstractScopeTask.deltaClosedByName(rs
                    .getString(ID_CLOSED_BY_NAME));
                // mandatory
                abstractScopeTask.deltaMandatory(getBooleanFrom01(rs
                    .getInt(ID_CONFIG_FLAG)));
                // priority
                if (priority != null)
                {
                    abstractScopeTask.deltaPriority(Integer.valueOf(priority));
                }
                // proposedByDate
                Date proposedByDate = rs.getDate(ID_PROPOSED_DATETIME);
                abstractScopeTask.deltaProposedByDate(DateUtil
                    .convertToCalendar(proposedByDate));
                // proposedByName
                abstractScopeTask.deltaProposedByName(rs
                    .getString(ID_PROPOSED_BY_NAME));
            }
            else
            {
                // closedByDate
                Date closedByDate = rs.getDate(ID_CLOSED_DATETIME);
                abstractScopeTask.setClosedByDate(DateUtil
                    .convertToCalendar(closedByDate));
                // closedByName
                abstractScopeTask.setClosedByName(rs
                    .getString(ID_CLOSED_BY_NAME));
                // mandatory
                abstractScopeTask.setMandatory(getBooleanFrom01(rs
                    .getInt(ID_CONFIG_FLAG)));
                // priority
                if (priority != null)
                {
                    abstractScopeTask.setPriority(Integer.valueOf(priority));
                }
                // proposedByDate
                Date proposedByDate = rs.getDate(ID_PROPOSED_DATETIME);
                abstractScopeTask.setProposedByDate(DateUtil
                    .convertToCalendar(proposedByDate));
                // proposedByName
                abstractScopeTask.setProposedByName(rs
                    .getString(ID_PROPOSED_BY_NAME));
            }
        }

        // AbstractRequest fields
        if (rpmObject instanceof AbstractRequest)
        {
            AbstractRequest abstractRequest = (AbstractRequest) rpmObject;

            if (isDelta)
            {
                // numberTimesReassigned
                abstractRequest.deltaNumberTimesReassigned(new Integer(rs
                    .getInt(ID_REASSIGN_COUNT)));
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                abstractRequest.deltaOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
            else
            {
                // numberTimesReassigned
                abstractRequest.setNumberTimesReassigned(new Integer(rs
                    .getInt(ID_REASSIGN_COUNT)));
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                abstractRequest.setOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
        }

        // ScopeFolder fields
        if (isScopeFolder(rpmObject))
        {
            ScopeFolder scopeFolder = (ScopeFolder) rpmObject;

            if (isDelta)
            {
                // privateFolder
                scopeFolder.deltaPrivateFolder(getBooleanFromYN(rs
                    .getString(ID_PRIVATE)));
            }
            else
            {
                // privateFolder
                scopeFolder.setPrivateFolder(getBooleanFromYN(rs
                    .getString(ID_PRIVATE)));
            }
        }

        // Issue fields
        else if (isIssue(rpmObject))
        {
            Issue issue = (Issue) rpmObject;

            if (isDelta)
            {
                // numberTimesReassigned
                issue.deltaNumberTimesReassigned(new Integer(rs
                    .getInt(ID_REASSIGN_COUNT)));
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                issue.deltaOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
            else
            {
                // numberTimesReassigned
                issue.setNumberTimesReassigned(new Integer(rs
                    .getInt(ID_REASSIGN_COUNT)));
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                issue.setOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
        }

        // Risk fields
        else if (isRisk(rpmObject))
        {
            Risk risk = (Risk) rpmObject;

            if (isDelta)
            {
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                risk.deltaOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
            else
            {
                // occuredDateTime
                Date occuredDateTime = rs.getDate(ID_OCCURED_DATETIME);
                risk.setOccuredDateTime(DateUtil
                    .convertToCalendar(occuredDateTime));
            }
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String status = resultSet.getString(ID_STAGE_ID);
        relatedFields.put(index, ID_STAGE_ID, status);

        String assetID = resultSet.getString(ID_ASSET_ID);
        relatedFields.put(index, ID_ASSET_ID, assetID);

        String assetCodeID = resultSet.getString(ID_FINANCIAL_CATEGORY);
        relatedFields.put(index, ID_FINANCIAL_CATEGORY, assetCodeID);

        String cccID = resultSet.getString(ID_CLIENT_ID);
        relatedFields.put(index, ID_CLIENT_ID, cccID);

        String proposedBy = resultSet.getString(ID_PROPOSED_BY);
        relatedFields.put(index, ID_PROPOSED_BY, proposedBy);

        relatedFields.put(index, ID_PROJECT_ID, resultSet
            .getString(ID_PROJECT_ID));

        // AbstractAggregateScope - foreign key to Resources.resource_id
        relatedFields.put(index, ID_CLOSED_BY, resultSet
            .getString(ID_CLOSED_BY));

        // AbstractScope - foreign key to Resources.resource_id
        relatedFields.put(index, ID_APPROVED_BY, resultSet
            .getString(ID_APPROVED_BY));

        // ApprovedCostBenefits

        Integer aCBScheduleImpactInDays = new Integer(resultSet
            .getInt(ID_APPROVED_IMPACT_LEVEL_ID));
        relatedFields.put(index, ID_APPROVED_IMPACT_LEVEL_ID,
            aCBScheduleImpactInDays);

        Double aCBCost = new Double(resultSet.getDouble(ID_APPROVED_BUDGET));
        relatedFields.put(index, ID_APPROVED_BUDGET, aCBCost);

        Double aCBBillableRevenue = new Double(resultSet
            .getDouble(ID_APPROVED_REVENUE));
        relatedFields.put(index, ID_APPROVED_REVENUE, aCBBillableRevenue);

        Double aCBNonBillableBenefits = new Double(resultSet
            .getDouble(ID_APPROVED_SAVING));
        relatedFields.put(index, ID_APPROVED_SAVING, aCBNonBillableBenefits);

        Integer aCBWorkInHours = new Integer(resultSet
            .getInt(ID_APPROVED_EFFORT));
        relatedFields.put(index, ID_APPROVED_EFFORT, aCBWorkInHours);

        // Impact

        Integer impactScheduledImpactInDays = new Integer(resultSet
            .getInt(ID_IMPACT_LEVEL_ID));
        relatedFields.put(index, ID_IMPACT_LEVEL_ID,
            impactScheduledImpactInDays);

        Integer impactOrderOfMagnitudeWorkInHours = new Integer(resultSet
            .getInt(ID_ESTIMATED_EFFORT));
        relatedFields.put(index, ID_ESTIMATED_EFFORT,
            impactOrderOfMagnitudeWorkInHours);

        Integer impactOrderOfMagnitudeLowWorkInHours = new Integer(resultSet
            .getInt(ID_LOW_ESTIMATED_EFFORT));
        relatedFields.put(index, ID_LOW_ESTIMATED_EFFORT,
            impactOrderOfMagnitudeLowWorkInHours);

        Integer impactOrderOfMagnitudeHighWorkInHours = new Integer(resultSet
            .getInt(ID_HIGH_ESTIMATED_EFFORT));
        relatedFields.put(index, ID_HIGH_ESTIMATED_EFFORT,
            impactOrderOfMagnitudeHighWorkInHours);

        // OrderOfMagnitudeCostBenefits

        Double orderOfMagnitudeCostBenefitsCost = new Double(resultSet
            .getDouble(ID_ESTIMATED_BUDGET));
        relatedFields.put(index, ID_ESTIMATED_BUDGET,
            orderOfMagnitudeCostBenefitsCost);

        Double orderOfMagnitudeCostBenefitsLow = new Double(resultSet
            .getDouble(ID_LOW_ESTIMATED_BUDGET));
        relatedFields.put(index, ID_LOW_ESTIMATED_BUDGET,
            orderOfMagnitudeCostBenefitsLow);

        Double orderOfMagnitudeCostBenefitsHigh = new Double(resultSet
            .getDouble(ID_HIGH_ESTIMATED_BUDGET));
        relatedFields.put(index, ID_HIGH_ESTIMATED_BUDGET,
            orderOfMagnitudeCostBenefitsHigh);

        Double orderOfMagnitudeCostBenefitsBillableRevenue = new Double(
            resultSet.getDouble(ID_ESTIMATED_REVENUE));
        relatedFields.put(index, ID_ESTIMATED_REVENUE,
            orderOfMagnitudeCostBenefitsBillableRevenue);

        Double orderOfMagnitudeCostBenefitsNonBillableBenefits = new Double(
            resultSet.getDouble(ID_ESTIMATED_SAVING));
        relatedFields.put(index, ID_ESTIMATED_SAVING,
            orderOfMagnitudeCostBenefitsNonBillableBenefits);

        // RiskMatrix

        Double riskMatrixConsequenceCost = new Double(resultSet
            .getDouble(ID_CONSEQUENSE_COST));
        relatedFields
            .put(index, ID_CONSEQUENSE_COST, riskMatrixConsequenceCost);

        Integer riskMatrixImpact = new Integer(resultSet
            .getInt(ID_IMPACT_RANK_ID));
        relatedFields.put(index, ID_IMPACT_RANK_ID, riskMatrixImpact);

        Integer riskMatrixProbability = new Integer(resultSet
            .getInt(ID_PROBABILITY_RANK_ID));
        relatedFields.put(index, ID_PROBABILITY_RANK_ID, riskMatrixProbability);

        Integer riskMatrixPrecision = new Integer(resultSet
            .getInt(ID_PRECISION_RANK_ID));
        relatedFields.put(index, ID_PRECISION_RANK_ID, riskMatrixPrecision);

        Double riskMatrixProbabilityCost = new Double(resultSet
            .getDouble(ID_PROBABILITY_COST));
        relatedFields
            .put(index, ID_PROBABILITY_COST, riskMatrixProbabilityCost);

        Integer riskMatrixExposurePercent = new Integer(resultSet
            .getInt(ID_EXPOSURE_ID));
        relatedFields.put(index, ID_EXPOSURE_ID, riskMatrixExposurePercent);

        Double riskMatrixMitigationCost = new Double(resultSet
            .getDouble(ID_MITIGATION_COST));
        relatedFields.put(index, ID_MITIGATION_COST, riskMatrixMitigationCost);

        // WorkElement

        String workElementID = resultSet.getString(ID_WBS_ID);
        relatedFields.put(index, ID_WBS_ID, workElementID);

        Integer parentTypeID = new Integer(resultSet.getInt(ID_PARENT_TYPE_ID));
        relatedFields.put(index, ID_PARENT_TYPE_ID, parentTypeID);

        String projectID = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectID);

        // Duplicate

        String parentRank = resultSet.getString(ID_PARENT_RANK);
        relatedFields.put(index, ID_PARENT_RANK, parentRank);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (rpmObject != null)
        {
            SP_D_CRI(rpmObject, ManagerUtil.getParentId(this, rpmObject,
                messageContext, TABLE_NAME), messageContext);
        }
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_ELEMENT_ID);
        filter.append(" IS NOT NULL ");

        filter.append(" AND ");

        filter.append(NAME_TYPE_ID);
        filter.append(" <> 145 ");

        filter.append(" AND ");

        filter.append("((SELECT COUNT(*) FROM ");
        filter.append(WbsManager.TABLE_NAME);
        filter.append(" WHERE ");
        filter.append(WbsManager.NAME_ELEMENT_ID);
        filter.append("=");
        filter.append(NAME_PROJECT_ID);
        filter.append(" AND ");
        filter.append(WbsManager.NAME_REC_STATUS);
        filter.append("!='D') > 0) ");

        return filter.toString();
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        SqlBuffer baseCondition = new SqlBuffer();
        baseCondition.appendCondition(getFilter());
        baseCondition.appendIn(NAME_TYPE_ID, containerMap.getTypeIds(container
            .getType()));
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, baseCondition
                .toString());
        }
        else if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            where.append(" WHERE ");
            where.append(baseCondition);

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof PublishedScorecardManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(PublishedScorecardManager.NAME_TYPE_ID);
            where.append(" = ").append(Scorecard.TYPE_ID);
            where.append(" AND ");
            where.append(PublishedScorecardManager.NAME_RANK);
            where.append(" IN (SELECT ");
            where.append(NAME_RANK);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(baseCondition);

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof FinancialManager)
        {
            if (ContextUtil.SCOPE_CONTEXT.equalsIgnoreCase(contextName))
            {
                // For XPath like: /ChangeRequest[<condition>]/WorksheetFinancial
                joinCondition = JoinCondition.createSubSelect(
                    FinancialManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, baseCondition.toString());
            }
        }
        else if (manager instanceof ResourceTaskAssignmentManager)
        {
            if (ContextUtil.SCOPE_CONTEXT.equalsIgnoreCase(contextName))
            {
                // /ScopeElement/ResourceTaskAssignment
                joinCondition = JoinCondition.createSubSelect(
                    ResourceTaskAssignmentManager.NAME_ELEMENT_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, baseCondition.toString());
            }
        }
        else if (manager instanceof ReqProManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ReqProManager.NAME_CRI_ID, NAME_ELEMENT_ID, TABLE_NAME, null);
        }
        else if (manager instanceof DocumentManager)
        {
            if (ContextUtil.getInstance().isWbsContext(contextName))
            {
                joinCondition = new JoinCondition();
                joinCondition.setCondition(DocumentManager.NAME_LEVEL_ID
                        + "=2 AND "
                        + JoinCondition.createSubSelect(
                            DocumentManager.NAME_PROJECT_ID, NAME_PROJECT_ID,
                            TABLE_NAME, null).toString());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(getTableName());
            where.append(" WHERE ");
            where.append(getFilter());

            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof ScopeManagementModuleManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof WbsManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_PROJECT_ID);
            condition.append(" = ?");
            condition.append(" AND " + NAME_TYPE_ID + " = "
                    + ScopeFolder.TYPE_ID);
            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(joinCondition.getCondition());
                condition.append(extraWhere);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(condition.toString());
        }
        else if (manager instanceof PublishedScorecardManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append("'CRI#'||").append(ScopeManager.NAME_RANK);
            condition.append(" IN ( SELECT ");
            condition.append(PublishedScorecardManager.NAME_PARENT_RANK);
            condition.append(" FROM TMT_PUBLISHED_SCORECARDS");
            condition.append(" WHERE ");
            condition.append(PublishedScorecardManager.NAME_ELEMENT_ID);
            condition.append(" =?");
            condition.append(" AND ");
            condition.append(PublishedScorecardManager.NAME_ELEMENT_ID);
            condition.append(" IS NOT NULL ");
            condition.append(" AND ");
            condition.append(PublishedScorecardManager.NAME_REC_STATUS);
            condition.append(" != 'D')");
            condition.append(" AND ");
            condition.append(NAME_REC_STATUS);
            condition.append(" != 'D'");
            joinCondition = new JoinCondition();
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName(contextName));
        }

        return joinCondition;
    }

    private String getContextName(RPMObject rpmObject) throws RPMException
    {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {

            //Parent
            if (rpmScope.getParent() != null)
            {
                String oldParentId = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                Integer oldParentTypeId = (Integer) relatedFields.get(index,
                    ID_PARENT_TYPE_ID);
                RPMObject newParent = null;
                if (isDelta)
                {
                    newParent = rpmObject.getParent();
                    RPMObject oldParent = null;
                    if (oldParentId != null
                            && oldParentTypeId.intValue() != 154
                            && oldParentTypeId.intValue() != 288
                            && oldParentTypeId.intValue() != 1)
                    {
                        oldParent = new ScopeElement();
                        oldParent.setID(oldParentId);
                        if (newParent != null && newParent.getID() != null
                                && newParent.getID().equals(oldParentId))
                        {
                            oldParent = loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, true);
                        }
                        else
                        {
                            //In this case, old parent is loaded without the
                            // scope to avoid problem with the cache
                            oldParent = loadByPrimaryKey(oldParent, null,
                                messageContext, false);
                        }
                    }
                    rpmObject.deltaParent(oldParent);
                }
                else
                {
                    if (oldParentId != null
                            && oldParentTypeId.intValue() != 154
                            && oldParentTypeId.intValue() != 288
                            && oldParentTypeId.intValue() != 1)
                    {
                        if (newParent == null)
                        {//newParent not in cache
                            newParent = new ScopeElement();
                            newParent.setID(oldParentId);
                            newParent = loadByPrimaryKey(newParent, rpmScope
                                .getParent(), messageContext, false);
                        }
                        rpmObject.setParent(newParent);
                    }
                }
            }

            if (rpmScope instanceof ScopeElementScope)
            {

                ScopeElementScope scope = (ScopeElementScope) rpmScope;

                ScopeElement scopeIntfElement = (ScopeElement) rpmObject;

                if (scope.getChildren() != null)
                {
                    if (scopeIntfElement instanceof AggregateScope)
                    {
                        AggregateScope aggObj = (AggregateScope) scopeIntfElement;
                        loadScopeElements(aggObj, scope.getChildren(),
                            messageContext, isDelta);
                    }
                }

                loadScopeScheduleDates(scopeIntfElement, scope, messageContext,
                    isDelta);
                loadApprovedCostBenefits(scopeIntfElement, scope,
                    messageContext, isDelta, relatedFields, index);
                loadAttributeAssignment(scopeIntfElement, scope,
                    messageContext, isDelta);
                loadAsset(scopeIntfElement, scope, messageContext, isDelta,
                    relatedFields, index);
                loadAssetCode(scopeIntfElement, scope, messageContext, isDelta,
                    relatedFields, index);
                loadClientCostCenter(scopeIntfElement, scope, messageContext,
                    isDelta, relatedFields, index);
                loadDuplicates(scopeIntfElement, scope, messageContext,
                    isDelta, relatedFields, index);
                loadFinancialAssignments(scopeIntfElement, scope,
                    messageContext, isDelta);
                loadImpact(scopeIntfElement, scope, messageContext, isDelta,
                    relatedFields, index);
                loadMitigationFactors(scopeIntfElement, scope, messageContext,
                    isDelta);
                loadOrderOfMagnitudeCostBenefits(scopeIntfElement, scope,
                    messageContext, isDelta, relatedFields, index);
                loadResourceTaskAssignments(scopeIntfElement, scope,
                    messageContext, isDelta);
                loadRiskMatrix(scopeIntfElement, scope, messageContext,
                    isDelta, relatedFields, index);
                loadRtfAssignments(scopeIntfElement, scope, messageContext,
                    isDelta);
                loadStatusUpdate(scopeIntfElement, scope, messageContext,
                    isDelta);
                loadScorecardAssignment(scopeIntfElement, scope,
                    messageContext, isDelta);
                loadState(rpmObject, scope, messageContext, relatedFields,
                    index, isDelta);
                loadWBSFinancials(scopeIntfElement, scope, messageContext,
                    isDelta);
                loadWorkElement(scopeIntfElement, scope, messageContext,
                    relatedFields, index, isDelta);
                loadWorkSheetFinancials(scopeIntfElement, scope,
                    messageContext, isDelta);
                loadProject(scopeIntfElement, scope, messageContext,
                    relatedFields, index, isDelta);
                loadOwningWorkElement(scopeIntfElement, scope, messageContext,
                    relatedFields, index, isDelta);

                if (rpmObject instanceof AbstractScope)
                {
                    AbstractScope cri = (AbstractScope) rpmObject;

                    //DocumentFolder
                    loadDocumentFolder(cri, scope, messageContext, isDelta);

                    // WorkflowProcess
                    loadAvailableWorkflowProcesses(cri, scope, messageContext,
                        isDelta);
                    loadRunningWorkflowProcesses(cri, scope, messageContext,
                        isDelta);
                }
                if (scope.isCustomFieldAssignments())
                {
                    scopeIntfElement
                        .setCustomFieldAssignments(CustomFieldAssignmentUtil
                            .loadAssignedCustomFields(this, scopeIntfElement
                                .getCustomFieldAssignments(), scopeIntfElement,
                                messageContext, isDelta));
                    propagateContextName(scopeIntfElement, scopeIntfElement
                        .getCustomFieldAssignments());
                }
                //REQ PRO Objects
                if (scope != null && scope.getReqProData() != null)
                {
                    if (ReqProUtil.isReqProObject(scopeIntfElement))
                    {
                        loadReqProRelatedObjects(scopeIntfElement, scope,
                            messageContext, isDelta);
                    }
                }
            }
        }

        return rpmObject;
    }

    private void loadReqProRelatedObjects(ScopeElement scopeElement,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scopeElement instanceof ReqProIntegrationFolder)
        {
            loadReqProConnection((ReqProIntegrationFolder) scopeElement, scope,
                messageContext, isDelta);
        }
        else if (scopeElement instanceof ReqProScopeFolder)
        {
            loadReqProImportStatusForReqProScopeFolder(
                (ReqProScopeFolder) scopeElement, scope, messageContext,
                isDelta);
        }
        else if (scopeElement instanceof ReqProRequirement)
        {
            loadReqProImportStatusForReqProRequirement(
                (ReqProRequirement) scopeElement, scope, messageContext,
                isDelta);
        }
    }

    private void loadReqProConnection(
            ReqProIntegrationFolder reqProIntegrationFolder,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        ReqProConnection reqProConnection = reqProIntegrationFolder
            .getReqProConnection();
        if (reqProConnection == null)
        {
            reqProConnection = new ReqProConnection();
            reqProConnection.setID(reqProIntegrationFolder.getID());
        }

        ReqProManager reqProManager = (ReqProManager) RPMManagerFactory
            .getInstance()
            .getRPMObjectManager(ReqProConnection.class.getName());

        reqProConnection = (ReqProConnection) reqProManager.loadByPrimaryKey(
            reqProConnection, scope.getReqProData(), messageContext, isDelta);

        reqProIntegrationFolder.setReqProConnection(reqProConnection);
    }

    private void loadReqProImportStatusForReqProScopeFolder(
            ReqProScopeFolder reqProScopeFolder, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ReqProImportStatus reqProImportStatus = reqProScopeFolder
            .getReqProImportStatus();
        if (reqProImportStatus == null)
        {
            reqProImportStatus = new ReqProImportStatus();
            reqProImportStatus.setID(reqProScopeFolder.getID());
        }

        ReqProManager reqProManager = (ReqProManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                ReqProImportStatus.class.getName());

        reqProImportStatus = (ReqProImportStatus) reqProManager
            .loadByPrimaryKey(reqProImportStatus, scope.getReqProData(),
                messageContext, isDelta);

        reqProScopeFolder.setReqProImportStatus(reqProImportStatus);
    }

    private void loadReqProImportStatusForReqProRequirement(
            ReqProRequirement reqProRequirement, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        ReqProImportStatus reqProImportStatus = reqProRequirement
            .getReqProImportStatus();
        if (reqProImportStatus == null)
        {
            reqProImportStatus = new ReqProImportStatus();
            reqProImportStatus.setID(reqProRequirement.getID());
        }

        ReqProManager reqProManager = (ReqProManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                ReqProImportStatus.class.getName());

        reqProImportStatus = (ReqProImportStatus) reqProManager
            .loadByPrimaryKey(reqProImportStatus, scope.getReqProData(),
                messageContext, isDelta);

        reqProRequirement.setReqProImportStatus(reqProImportStatus);
    }

    private void loadProject(ScopeElement scopeElement,
            ScopeElementScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if (scope != null && scope.getProject() != null)
        {
            String oldProjectId = (String) relatedFields.get(index,
                ID_PROJECT_ID);
            GenericProject newProject = null;

            WbsManager wbsManager = (WbsManager) getManagerInstance(GenericProject.class);

            if (isDelta)
            {
                newProject = scopeElement.getProject();
                GenericProject oldProject = null;
                if (oldProjectId != null)
                {
                    oldProject = new GenericProject();
                    oldProject.setID(oldProjectId);
                    if (newProject != null && newProject.getID() != null
                            && newProject.getID().equals(oldProjectId))
                    {
                        oldProject = (GenericProject) wbsManager
                            .loadByPrimaryKey(newProject, scope.getProject(),
                                messageContext, true);
                    }
                    else
                    {
                        //In this case, old parent is loaded without the
                        // scope to avoid problem with the cache
                        oldProject = (GenericProject) wbsManager
                            .loadByPrimaryKey(oldProject, scope.getProject(),
                                messageContext, false);
                    }
                }
                scopeElement.deltaProject(oldProject);
            }
            else
            {
                if (oldProjectId != null)
                {
                    if (newProject == null)
                    {//newParent not in cache
                        newProject = new Project();
                        newProject.setID(oldProjectId);
                        newProject = (GenericProject) wbsManager
                            .loadByPrimaryKey(newProject, scope.getProject(),
                                messageContext, false);
                    }
                    scopeElement.setProject(newProject);
                }
            }
        }
    }

    private void loadScopeScheduleDates(ScopeElement object,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        if (scope.isScheduleDates())
        {

            //Note: If all the ScheduleDate fields are null,
            //the object should not be set in the ScopeElement

            //Cannot use the isDelta method on ScheduleDate objects

            RPMObjectManager manager = getManagerInstance(ScopeScheduleDate.class
                .getName());

            if (object instanceof AbstractAggregateScope)
            {

                AbstractAggregateScope aggregateScope = (AbstractAggregateScope) object;

                //proposedDate
                ScopeScheduleDate proposedDate = aggregateScope
                    .getProposedDate();
                if (proposedDate == null)
                {
                    proposedDate = new ScopeScheduleDate();
                }
                proposedDate.setID(object.getID());
                proposedDate.setContextName(ScopeScheduleDate.PROPOSED);
                proposedDate = (ScopeScheduleDate) manager.loadByPrimaryKey(
                    proposedDate, null, messageContext, isDelta);

                if (ScheduleDateUtil.isEmpty(proposedDate))
                {
                    proposedDate = null;
                }

                aggregateScope.setProposedDate(proposedDate);
            }

            if (object instanceof AbstractScope)
            {

                AbstractScope abstractScope = (AbstractScope) object;

                //approvedDate
                ScopeScheduleDate approvedDate = abstractScope
                    .getApprovedDate();
                if (approvedDate == null)
                {
                    approvedDate = new ScopeScheduleDate();
                }
                approvedDate.setID(object.getID());
                approvedDate.setContextName(ScopeScheduleDate.APPROVED);
                approvedDate = (ScopeScheduleDate) manager.loadByPrimaryKey(
                    approvedDate, null, messageContext, isDelta);

                if (ScheduleDateUtil.isEmpty(approvedDate))
                {
                    approvedDate = null;
                }

                abstractScope.setApprovedDate(approvedDate);

            }
        }
    }

    private void loadApprovedCostBenefits(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta, FieldValueMap relatedFields, int index)
            throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof AbstractRequest || element instanceof Action || element instanceof Requirement)
                || !scope.isApprovedCostBenefits())
            return;

        ApprovedCostBenefits approvedCostBenefits = new ApprovedCostBenefits();
        approvedCostBenefits.setScheduleImpactInDays((Integer) relatedFields
            .get(index, ID_APPROVED_IMPACT_LEVEL_ID));
        approvedCostBenefits.setCost((Double) relatedFields.get(index,
            ID_APPROVED_BUDGET));
        approvedCostBenefits.setBillableRevenue((Double) relatedFields.get(
            index, ID_APPROVED_REVENUE));
        approvedCostBenefits.setNonBillableBenefits((Double) relatedFields.get(
            index, ID_APPROVED_SAVING));
        Integer workInHours = (Integer) relatedFields.get(index,
            ID_APPROVED_EFFORT);
        workInHours = new Integer(workInHours.intValue() / 60);
        approvedCostBenefits.setWorkInHours(workInHours);

        if (isDelta)
        {
            if (element instanceof AbstractRequest)
            {
                ((AbstractRequest) element)
                    .deltaApprovedCostBenefits(approvedCostBenefits);
            }
            else if (element instanceof Action)
            {
                ((Action) element)
                    .deltaApprovedCostBenefits(approvedCostBenefits);
            }
            else if (element instanceof Requirement)
            {
                ((Requirement) element)
                    .deltaApprovedCostBenefits(approvedCostBenefits);
            }
        }
        else
        {
            if (element instanceof AbstractRequest)
            {
                ((AbstractRequest) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
            else if (element instanceof Action)
            {
                ((Action) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
            else if (element instanceof Requirement)
            {
                ((Requirement) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
        }
    }

    private void loadAsset(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        if (!(element instanceof AbstractScope) || !scope.isAsset())
            return;

        AbstractScope absScope = (AbstractScope) element;
        Asset asset = null;
        String assetId = null;

        assetId = (String) relatedFields.get(index, ID_ASSET_ID);

        if (assetId != null && !assetId.equals("0"))
        {
            asset = new Asset();
            asset.setID(assetId);
            AssetManager assetManager = (AssetManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Asset.class.getName());

            asset = (Asset) assetManager.loadByPrimaryKey(asset, null,
                messageContext, isDelta);
        }
        if (isDelta)
        {
            absScope.deltaAsset(asset);
        }
        else
        {
            absScope.setAsset(asset);
        }
    }

    private void loadAssetCode(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        if (!(element instanceof AbstractScope) || !scope.isAssetCode())
            return;

        AbstractScope absScope = (AbstractScope) element;
        AssetFinancialCategory afc = null;
        String afcID = null;

        afcID = (String) relatedFields.get(index, ID_FINANCIAL_CATEGORY);

        if (afcID != null && !afcID.equals("0"))
        {
            afc = new AssetFinancialCategory();
            afc.setID(afcID);

            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AssetFinancialCategory.class.getName());

            afc = (AssetFinancialCategory) manager.loadByPrimaryKey(afc, null,
                messageContext, isDelta);
        }

        if (isDelta)
        {
            absScope.deltaAssetCode(afc);
        }
        else
        {
            absScope.setAssetCode(afc);
        }
    }

    private void loadClientCostCenter(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta, FieldValueMap relatedFields, int index)
            throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof AbstractScope) || !scope.isClientCostCenter())
            return;

        AbstractScope absScope = (AbstractScope) element;

        String clientID = null;
        ClientCostCenter ccc = null;

        clientID = (String) relatedFields.get(index, ID_CLIENT_ID);

        if (clientID != null)
        {
            ccc = new ClientCostCenter();
            ccc.setID(clientID);

            ClientManager manager = (ClientManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ClientCostCenter.class.getName());

            ccc = (ClientCostCenter) manager.loadByPrimaryKey(ccc, null,
                messageContext, isDelta);
        }
        if (isDelta)
        {
            absScope.deltaClientCostCenter(ccc);
        }
        else
        {
            absScope.setClientCostCenter(ccc);
        }
    }

    private void loadDuplicates(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        if (!(element instanceof AbstractScope) || !scope.isDuplicates())
            return;

        AbstractScope abstractScope = (AbstractScope) element;

        // The scope element has duplicates
        if (abstractScope.getDuplicateNumber() != null)
        {
            Object[] args = new Object[]
            {
                    (String) relatedFields.get(index, ID_PARENT_RANK),
                    (String) relatedFields.get(index, ID_PROJECT_ID),
                    abstractScope.getID(), abstractScope.getDuplicateNumber()
            };

            // Here we replace the values because the parent_rank 
            // has a ? in it which causes SQL problems
            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_RANK + " LIKE '" + args[0] + "%'");
            where.append(" AND ");
            where.append(NAME_PROJECT_ID + " = '" + args[1] + "'");
            where.append(" AND ");
            where.append(NAME_ELEMENT_ID + " <> '" + args[2] + "'");
            where.append(" AND ");
            where.append(NAME_DUPLICATE_KEY + " = " + args[3]);

            ArrayList temp = loadByWhere(null, null, null, messageContext,
                where.toString(), null, getTableName(), getAllFields(),
                getContextName(element), isDelta);

            Duplicate[] duplicates = null;
            if (temp != null && !temp.isEmpty())
            {
                duplicates = new Duplicate[temp.size()];
                for (int i = 0; i < temp.size(); ++i)
                {
                    duplicates[i] = new Duplicate();
                    duplicates[i].setParent(abstractScope);
                    duplicates[i].setScopeElement((ScopeElement) temp.get(i));
                    // messageContext.removeContainerFromCache((ScopeElement)temp.get(i));
                }
                abstractScope.setDuplicates(duplicates);
            }
        }
    }

    private void loadFinancialAssignments(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

    }

    private void loadImpact(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        if (!(element instanceof AbstractScope) || !scope.isImpact())
            return;

        AbstractScope abstractScope = (AbstractScope) element;

        Impact impact = new Impact();
        impact.setScheduledImpactInDays((Integer) relatedFields.get(index,
            ID_IMPACT_LEVEL_ID));
        impact.setOrderOfMagnitudeWorkInHours((Integer) relatedFields.get(
            index, ID_ESTIMATED_EFFORT));
        impact.setOrderOfMagnitudeLowWorkInHours((Integer) relatedFields.get(
            index, ID_LOW_ESTIMATED_EFFORT));
        impact.setOrderOfMagnitudeHighWorkInHours((Integer) relatedFields.get(
            index, ID_HIGH_ESTIMATED_EFFORT));

        if (isDelta)
            abstractScope.deltaImpact(impact);
        else
            abstractScope.setImpact(impact);
    }

    private void loadMitigationFactors(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof Risk)
                || (scope.getMitigationFactors() == null))
            return;
        Risk rsk = (Risk) element;
        MitigationFactorManager manager = (MitigationFactorManager) RPMManagerFactory
            .getInstance()
            .getRPMObjectManager(MitigationFactor.class.getName());
        MitigationFactorScope mitScope = scope.getMitigationFactors();
        MitigationFactor[] oldMitigationFactor = rsk.getMitigationFactors();

        ArrayList temp = manager.loadByForeignKey(oldMitigationFactor,
            mitScope, messageContext, this, new Object[]
            {
                rsk.getID()
            }, "TYPE_ID = " + Risk.TYPE_ID, null, isDelta);
        if (temp != null && (temp.size() > 0))
        {
            MitigationFactor[] children = new MitigationFactor[temp.size()];
            temp.toArray(children);
            rsk.setMitigationFactors(children);
        }
    }

    private void loadOrderOfMagnitudeCostBenefits(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta, FieldValueMap relatedFields, int index)
            throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof AbstractScope)
                || !scope.isOrderOfMagnitudeCostBenefits())
            return;

        AbstractScope abstractScope = (AbstractScope) element;

        OrderOfMagnitudeCostBenefits orderOfMagnitudeCostBenefits = new OrderOfMagnitudeCostBenefits();
        orderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeCost((Double) relatedFields.get(index,
                ID_ESTIMATED_BUDGET));
        orderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeLow((Double) relatedFields.get(index,
                ID_LOW_ESTIMATED_BUDGET));
        orderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeHigh((Double) relatedFields.get(index,
                ID_HIGH_ESTIMATED_BUDGET));
        orderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeBillableRevenue((Double) relatedFields.get(
                index, ID_ESTIMATED_REVENUE));
        orderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeNonBillableBenefits((Double) relatedFields.get(
                index, ID_ESTIMATED_SAVING));

        if (isDelta)
            abstractScope
                .deltaOrderOfMagnitudeCostBenefits(orderOfMagnitudeCostBenefits);
        else
            abstractScope
                .setOrderOfMagnitudeCostBenefits(orderOfMagnitudeCostBenefits);
    }

    private void loadRiskMatrix(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        if (!(element instanceof Risk) || !scope.isRiskMatrix())
            return;

        Risk risk = (Risk) element;

        RiskMatrix riskMatrix = new RiskMatrix();
        riskMatrix.setConsequenceCost((Double) relatedFields.get(index,
            ID_CONSEQUENSE_COST));
        // Impact: 1 = 20, 2 = 40, 3 = 60, 4 = 80, 5 = 100
        riskMatrix.setImpact(new Integer(((Integer) relatedFields.get(index,
            ID_IMPACT_RANK_ID)).intValue() * 20));
        // Probability: 0 = 20, 1 = 40, 2 = 60, 3 = 80, 4 = 100
        riskMatrix.setProbability(new Integer((((Integer) relatedFields.get(
            index, ID_PROBABILITY_RANK_ID)).intValue() * 20) + 20));
        riskMatrix.setPrecision(getPrecisionType((Integer) relatedFields.get(
            index, ID_PRECISION_RANK_ID)));

        // Calculate the color
        int riskRanking = (riskMatrix.getImpact().intValue() + riskMatrix
            .getProbability().intValue());
        if (riskRanking >= 160)
        {
            riskMatrix.setRiskMatrixColor("Red");
        }
        // Exception 1
        else if (riskMatrix.getProbability().intValue() == 60
                && riskMatrix.getImpact().intValue() == 40)
        {
            riskMatrix.setRiskMatrixColor("Yellow");
        }
        else if ((riskRanking <= 100)
        // Exception 2
                || (riskRanking == 120 && riskMatrix.getImpact().intValue() == 20))
        {
            riskMatrix.setRiskMatrixColor("Green");
        }
        else
        {
            riskMatrix.setRiskMatrixColor("Yellow");
        }

        if (isDelta)
            risk.deltaRiskMatrix(riskMatrix);
        else
            risk.setRiskMatrix(riskMatrix);

        // Reset read-only fields to original values - never do an isDelta
        risk.getRiskMatrix().setExposurePercent(
            (Integer) relatedFields.get(index, ID_EXPOSURE_ID));
        risk.getRiskMatrix().setMitigationCost(
            (Double) relatedFields.get(index, ID_MITIGATION_COST));
        risk.getRiskMatrix().setProbabilityCost(
            (Double) relatedFields.get(index, ID_PROBABILITY_COST));
    }

    private void loadRtfAssignments(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (scope.isRtfAssignments())
        {
            if (element instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope absAggScope = (AbstractAggregateScope) element;
                RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
                    .getName());

                Object[] args = new Object[]
                {
                    absAggScope.getID()
                };
                ArrayList temp = rtfAssignmentManager
                    .loadByForeignKeyWithSpecifiedParent(element, absAggScope
                        .getRtfAssignments(), null, messageContext, this, args,
                        null, getContextName(element), isDelta);

                RtfAssignment[] rtfAssignments = null;
                if (temp != null && !temp.isEmpty())
                {
                    setParent(temp, absAggScope);
                    rtfAssignments = new RtfAssignment[temp.size()];
                    temp.toArray(rtfAssignments);
                    absAggScope.setRtfAssignments(rtfAssignments);
                }
            }
            else if (element instanceof AbstractScopeTask)
            {
                AbstractScopeTask absScopeTask = (AbstractScopeTask) element;
                RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
                    .getName());

                Object[] args = new Object[]
                {
                    absScopeTask.getID()
                };
                ArrayList temp = rtfAssignmentManager
                    .loadByForeignKeyWithSpecifiedParent(element, absScopeTask
                        .getRtfAssignments(), null, messageContext, this, args,
                        null, getContextName(element), isDelta);

                RtfAssignment[] rtfAssignments = null;
                if (temp != null && !temp.isEmpty())
                {
                    setParent(temp, absScopeTask);
                    rtfAssignments = new RtfAssignment[temp.size()];
                    temp.toArray(rtfAssignments);
                    absScopeTask.setRtfAssignments(rtfAssignments);
                }
            }
        }
    }

    private void loadStatusUpdate(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

    }

    private void loadWorkElement(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if (scope.getWorkElement() == null)
            return;

        WbsManager manager = (WbsManager) RPMManagerFactory.getInstance()
            .getRPMObjectManager(WorkElement.class.getName());

        // Get the correct WorkElement id
        String workElementID;
        if (isWorkElement(element, relatedFields, index))
        {
            String projectID = (String) relatedFields.get(index, ID_PROJECT_ID);
            workElementID = projectID;
        }
        else
        // If the item has a WorkElement, it's because it was explicitly
        // transfered under WBS root
        {
            String WbsID = (String) relatedFields.get(index, ID_WBS_ID);
            workElementID = WbsID;
        }

        WorkElement workElement;
        if (element.getWorkElement() == null)
        {
            workElement = new WorkElement();
            workElement.setID(workElementID);
        }
        else
        {
            workElement = element.getWorkElement();
        }

        if (workElement.getID() != null)
        {
            workElement = (WorkElement) manager.loadByPrimaryKey(workElement,
                scope.getWorkElement(), messageContext, isDelta);

            if (isDelta)
                element.deltaWorkElement(workElement);
            else
                element.setWorkElement(workElement);
        }
    }

    private void loadWorkSheetFinancials(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof AbstractAggregateScope))
            return;
        AbstractAggregateScope absScope = (AbstractAggregateScope) element;

        if (scope != null && scope.getWorksheetFinancials() != null)
        {
            WorksheetFinancialManager manager = (WorksheetFinancialManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    WorksheetFinancial.class.getName());

            ArrayList temp = manager.loadByForeignKey(absScope
                .getWorksheetFinancials(), scope.getWorksheetFinancials(),
                messageContext, this, new Object[]
                {
                    absScope.getID()
                }, null, null, isDelta);

            if (temp != null && temp.size() > 0)
            {
                WorksheetFinancial[] wf = new WorksheetFinancial[temp.size()];
                temp.toArray(wf);
                absScope.setWorksheetFinancials(wf);
            }
            propagateContextName(absScope, absScope.getWorksheetFinancials());
        }
    }

    private void loadWBSFinancials(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (scope == null || scope.getWbsFinancials() == null)
            return;

        if (element instanceof AbstractScope)
        {
            AbstractScope absScope = (AbstractScope) element;

            WbsFinancialManager manager = (WbsFinancialManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(WbsFinancial.class.getName());

            ArrayList temp = manager.loadByForeignKey(absScope
                .getWbsFinancials(), scope.getWbsFinancials(), messageContext,
                this, new Object[]
                {
                    absScope.getID()
                }, null, null, isDelta);

            if (temp != null && temp.size() > 0)
            {
                WbsFinancial[] wf = new WbsFinancial[temp.size()];
                temp.toArray(wf);
                absScope.setWbsFinancials(wf);
            }

            propagateContextName(absScope, absScope.getWbsFinancials());
        }
        else if (element instanceof AbstractScopeTask)
        {
            AbstractScopeTask absScope = (AbstractScopeTask) element;

            WbsFinancialManager manager = (WbsFinancialManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(WbsFinancial.class.getName());

            ArrayList temp = manager.loadByForeignKey(absScope
                .getWbsFinancials(), scope.getWbsFinancials(), messageContext,
                this, new Object[]
                {
                    absScope.getID()
                }, null, null, isDelta);

            if (temp != null && temp.size() > 0)
            {
                WbsFinancial[] wf = new WbsFinancial[temp.size()];
                temp.toArray(wf);
                absScope.setWbsFinancials(wf);
            }

            propagateContextName(absScope, absScope.getWbsFinancials());
        }
    }

    private void loadScorecardAssignment(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (!(element instanceof AbstractScope))
            return;
        AbstractScope abstractScope = (AbstractScope) element;

        if (scope != null && scope.getAssignedScorecard() != null)
        {
            PublishedScorecardManager manager = (PublishedScorecardManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AssignedScorecard.class.getName());

            String extraWhere = "AND SUBSTR(RANK, 1, 36) <> ('qqqq' || ELEMENT_ID)";
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                abstractScope, new RPMObject[]
                {
                    abstractScope.getAssignedScorecard()
                }, scope.getAssignedScorecard(), messageContext, this,
                new Object[]
                {
                    abstractScope.getID()
                }, extraWhere, abstractScope.getContextName(), isDelta);

            // There can be only 1 assignedScorecard
            if (temp != null && temp.size() > 0)
            {
                AssignedScorecard scorecardAssignment = (AssignedScorecard) temp
                    .get(0);
                if (isDelta)
                    abstractScope.deltaAssignedScorecard(scorecardAssignment);
                else
                    abstractScope.setAssignedScorecard(scorecardAssignment);
                scorecardAssignment.setParent(abstractScope);
            }
        }
    }

    private void loadScopeElements(AggregateScope aggObj,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ScopeElement[] oldChildren = aggObj.getChildren();
        StringBuffer whereClause = new StringBuffer();
        whereClause.append("TYPE_ID IN ");
        whereClause.append(" ( ");
        whereClause.append(ScopeFolder.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Requirement.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ChangeRequest.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ServiceRequest.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Action.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Defect.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Issue.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Risk.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ScopeDeliverable.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(Risk.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ScopeSummaryTask.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ScopeMilestone.TYPE_ID);
        whereClause.append(" , ");
        whereClause.append(ScopeTask.TYPE_ID);
        whereClause.append(" ) ");

        ArrayList temp = loadByForeignKeyWithSpecifiedParent(aggObj,
            oldChildren, scope, messageContext, this, new Object[]
            {
                aggObj.getID()
            }, whereClause.toString(), null, isDelta);

        if (temp != null)
        {
            ScopeElement[] children = new ScopeElement[temp.size()];
            temp.toArray(children);
            aggObj.setChildren(children);
        }
    }

    private void loadResourceTaskAssignments(ScopeElement scopeElement,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scope != null && scope.getResourceTaskAssignments() != null)
        {

            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceTaskAssignment.class.getName());

            String extraWhere = null;
            // String extraWhere = createExtraWhereForResourceJoin(scopeElement);

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                scopeElement, scopeElement.getResourceTaskAssignments(), scope
                    .getResourceTaskAssignments(), messageContext, this,
                new Object[]
                {
                    scopeElement.getID()
                }, extraWhere, null, isDelta);

            if (temp != null)
            {
                ResourceTaskAssignment[] resAss = new ResourceTaskAssignment[temp
                    .size()];
                temp.toArray(resAss);
                scopeElement.setResourceTaskAssignments(resAss);
                propagateContextName(scopeElement, scopeElement
                    .getResourceTaskAssignments());
            }
        }
    }

    private void loadAttributeAssignment(ScopeElement scopeElement,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scopeElement instanceof ScopeFolder)
            return;

        if (scope.isAttributeAssignments())
        {
            AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AttributeAssignment.class.getName());

            if (scopeElement instanceof AbstractScopeTask)
            {
                AbstractScopeTask absScopeTask = (AbstractScopeTask) scopeElement;
                ArrayList temp = attributeAssignmentManager.loadByForeignKey(
                    absScopeTask.getAttributeAssignments(), null,
                    messageContext, this, new Object[]
                    {
                        absScopeTask.getID()
                    }, null, getContextName(absScopeTask), isDelta);
                if (temp != null)
                {
                    setParent(temp, absScopeTask);
                    AttributeAssignment[] aa = new AttributeAssignment[temp
                        .size()];
                    temp.toArray(aa);
                    absScopeTask.setAttributeAssignments(aa);
                }
            }
            else if (scopeElement instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope absAggScope = (AbstractAggregateScope) scopeElement;
                ArrayList temp = attributeAssignmentManager.loadByForeignKey(
                    absAggScope.getAttributeAssignments(), null,
                    messageContext, this, new Object[]
                    {
                        absAggScope.getID()
                    }, null, getContextName(absAggScope), isDelta);
                if (temp != null)
                {
                    setParent(temp, absAggScope);
                    AttributeAssignment[] aa = new AttributeAssignment[temp
                        .size()];
                    temp.toArray(aa);
                    absAggScope.setAttributeAssignments(aa);
                }
            }
        }
    }

    private void loadState(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if (!(rpmObject instanceof AbstractScope))
            return;
        AbstractScope scopeElement = (AbstractScope) rpmObject;

        ScopeElementScope scope = (ScopeElementScope) rpmScope;
        if (rpmObject instanceof AbstractScope)
        {
            if (scope != null && scope.isState())
            {
                DatafieldState state = createState(rpmObject);

                state.setID((String) relatedFields.get(index, ID_STAGE_ID));

                if (state.getID() != null)
                {
                    DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                            state.getClass().getName());

                    state = (DatafieldState) manager.loadByPrimaryKey(state,
                        null, messageContext, isDelta);

                    if (isDelta)
                    {
                        scopeElement.deltaState(state);
                    }
                    else
                    {
                        scopeElement.setState(state);
                    }
                }
            }
        }
    }

    private void loadRunningWorkflowProcesses(AbstractScope abstractScope,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (scope.getWorkflowProcessesRunning() != null)
        {
            RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    RunningWorkflowProcess.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                abstractScope, abstractScope.getWorkflowProcessesRunning(),
                scope.getWorkflowProcessesRunning(), messageContext, this,
                new Object[]
                {
                    abstractScope.getID()
                }, null, null, isDelta);

            RunningWorkflowProcess[] workflowProcesses = new RunningWorkflowProcess[0];
            if (temp != null)
            {
                workflowProcesses = new RunningWorkflowProcess[temp.size()];
                temp.toArray(workflowProcesses);
                abstractScope.setWorkflowProcessesRunning(workflowProcesses);
            }
        }
    }

    private void loadAvailableWorkflowProcesses(AbstractScope abstractScope,
            ScopeElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (scope.getWorkflowProcessesAvailable() != null)
        {
            AvailableWorkflowProcessManager manager = (AvailableWorkflowProcessManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AvailableWorkflowProcess.class.getName());

            Integer groupTypeId = SP_S_WKF_PROCESS_TYPE(abstractScope
                .retrieveTypeID(), messageContext);

            String rank = WorkflowUtil.loadWorkflowProcessRankThroughParent(
                abstractScope, groupTypeId, messageContext);

            if (rank != null) // rank should never be null
            {
                SqlBuffer extraWhere = new SqlBuffer();
                extraWhere.appendSubSelect(
                    AvailableWorkflowProcessManager.NAME_PROCESS_ID,
                    WorkflowManager.NAME_ELEMENT_ID,
                    WorkflowManager.TABLE_NAME, WorkflowManager.NAME_RANK
                            + " like '" + rank + "%')");

                String projectId = WorkflowUtil.loadAssociatedProjectID(
                    abstractScope, messageContext);

                ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                    abstractScope, abstractScope
                        .getWorkflowProcessesAvailable(), scope
                        .getWorkflowProcessesAvailable(), messageContext, this,
                    new Object[]
                    {
                        projectId
                    }, extraWhere.toString(), null, isDelta);

                AvailableWorkflowProcess[] workflowProcesses = new AvailableWorkflowProcess[0];
                if (temp != null)
                {
                    workflowProcesses = new AvailableWorkflowProcess[temp
                        .size()];
                    temp.toArray(workflowProcesses);
                    abstractScope
                        .setWorkflowProcessesAvailable(workflowProcesses);
                }
            }
        }
    }

    /**
     * @param rpmObject
     * @return
     */
    private DatafieldState createState(RPMObject rpmObject)
    {
        DatafieldState state = null;
        if (isAction(rpmObject))
            state = new ActionState();
        else if (isChangeRequest(rpmObject))
            state = new ChangeRequestState();
        else if (isDefect(rpmObject))
            state = new DefectState();
        else if (isDeliverable(rpmObject))
            state = new DeliverableState();
        else if (isIssue(rpmObject))
            state = new IssueState();
        else if (isRequirement(rpmObject))
            state = new RequirementState();
        else if (isRisk(rpmObject))
            state = new RiskState();
        else if (isServiceRequest(rpmObject))
            state = new ServiceRequestState();

        return state;
    }

    /**
     * @param pObject
     * @throws SQLException
     * @throws RPMException
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        rpmObject.setID(SP_I_CRI((ScopeElement) rpmObject, messageContext));

        ((ScopeElement) rpmObject)
            .setElementSecurityType(ScopeElementSecurityType.ByInvitation);
    }

    public RPMObject createObjectFromTemplate(RPMObject inputObject,
            RPMObjectScope scope, MessageContext msgCtx) throws RPMException,
            SQLException, ParseException
    {
        RPMObject outputProject = null;

        //InputObject and Template cannot be null
        if (ScopeMethodCheckpoint.getInstance().isTemplateAndInputAvailable(
            inputObject, msgCtx))
        {
            //Validating input and template
            ScopeMethodCheckpoint.getInstance().validateTemplateAndInput(
                inputObject, msgCtx);

            //only if no errors are found...and input and template objects are valid
            if (msgCtx.isSuccessful())
            {
                if (ScopeMethodCheckpoint.getInstance().canCreateFromTemplate(
                    inputObject, msgCtx.getTemplate(), scope, msgCtx))
                {
                    //---AbstractRequest Case
                    if (inputObject instanceof AbstractRequest)
                    {
                        AbstractRequest inputARObject = (AbstractRequest) inputObject;
                        inputARObject.setModified();
                        AbstractRequestCheckpoint checkpoint = (AbstractRequestCheckpoint) ContainerToCheckpointMap
                            .getCheckpoint(inputObject);
                        checkpoint.validateAbstractRequestCreation(
                            inputARObject, msgCtx);

                        inputARObject.setID(SP_C_CRI(inputARObject, msgCtx));

                        outputProject = load(inputARObject, scope, msgCtx);
                    }
                    else
                    //---AbstractScope Case
                    {
                        AbstractScope inputASObject = (AbstractScope) inputObject;
                        inputASObject.setModified();
                        AbstractScopeCheckpoint checkpoint = (AbstractScopeCheckpoint) ContainerToCheckpointMap
                            .getCheckpoint(inputObject);
                        checkpoint.validateAbstractScopeCreation(inputASObject,
                            msgCtx);

                        inputASObject.setID(SP_C_CRI(inputASObject, msgCtx));

                        outputProject = load(inputASObject, scope, msgCtx);
                    }
                }
            }
        }

        return outputProject;
    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_CKIN_CRI(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            String workflowId = messageContext.getWorkflowId();
            if (workflowId == null)
                SP_CKOUT_CRI(rpmObject, messageContext);
            else
                SP_CHECK_OUT_WKF(rpmObject, workflowId, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        if (rpmObject.getID() == null)
            return;

        try
        {
            SP_UCKOUT_CRI(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }

    public void moveBeforeSibling(String sessionID, ScopeElement objectToMove,
            ScopeElement nextSibling, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ScopeMethodCheckpoint checkpoint = ScopeMethodCheckpoint.getInstance();
        checkpoint.validateScopeObjectMove(objectToMove, nextSibling,
            messageContext);

        if (messageContext.isSuccessful())
        {
            SP_M_CRI(objectToMove, nextSibling, false, messageContext);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ScopeElementScope scope = (ScopeElementScope) rpmScope;
        ScopeElement scopeElement = (ScopeElement) rpmObject;

        if (scopeElement.testNameModified())
        {
            SP_U_CRI_NAME(rpmObject, messageContext);
        }

        if (scopeElement.testElementSecurityTypeModified())
        {
            SP_U_CRI_PUBLISHED(rpmObject, messageContext);
        }

        if (rpmObject instanceof AbstractScope)
        {
            AbstractScope abstractScope = (AbstractScope) rpmObject;

            if (abstractScope.testDefaultScopeModified())
            {
                SP_U_DEFAULT_CRI(abstractScope, messageContext);
            }

            if (abstractScope.testTargetDateModified())
            {
                SP_U_CRI_TARGET_DATE(abstractScope, messageContext);
            }

            if (abstractScope.testPriorityModified()
                    || abstractScope.testProposedByNameModified()
                    || abstractScope.testProposedByDateModified()
                    || abstractScope.testExternalReferenceNumberModified()
                    || abstractScope.testRankModified())
            {
                SP_U_CRI_IDEN(abstractScope, messageContext);
            }

            if (abstractScope.testEstimatedStartDateModified()
                    || abstractScope.testEstimatedFinishDateModified()
                    || (rpmScope != null
                            && ((ScopeElementScope) rpmScope).isImpact() && abstractScope
                        .testImpactModified()))
            {
                SP_U_CRI_ESTIMATED(abstractScope, (ScopeElementScope) rpmScope,
                    messageContext);
            }

            if (rpmScope != null && ((ScopeElementScope) rpmScope).isImpact()
                    && abstractScope.testImpactModified())
            {
                SP_U_CRI_ESTIMATED_HIGH(abstractScope,
                    (ScopeElementScope) rpmScope, messageContext);
                SP_U_CRI_ESTIMATED_LOW(abstractScope,
                    (ScopeElementScope) rpmScope, messageContext);
            }

            if ((rpmScope != null && ((ScopeElementScope) rpmScope).isImpact() && abstractScope
                .testImpactModified())
                    || (rpmScope != null
                            && ((ScopeElementScope) rpmScope)
                                .isOrderOfMagnitudeCostBenefits() && abstractScope
                        .testOrderOfMagnitudeCostBenefitsModified()))
            {
                SP_U_CRI_QUAN(abstractScope, (ScopeElementScope) rpmScope,
                    messageContext);
            }

            if (!(abstractScope instanceof AbstractRequest))
            {
                if (abstractScope.testApprovedByDateModified()
                        || abstractScope.testApprovedByNameModified()
                        || abstractScope.testClosedByDateModified()
                        || abstractScope.testClosedByNameModified())
                {
                    SP_U_CRI_RESP(abstractScope, (ScopeElementScope) rpmScope,
                        messageContext);
                }

                if (isIssue(abstractScope))
                {
                    if (((Issue) abstractScope)
                        .testNumberTimesReassignedModified()
                            || ((Issue) abstractScope)
                                .testOccuredDateTimeModified())
                    {
                        SP_U_CRI_PROP((Issue) abstractScope, messageContext);
                    }
                }
                else if (isAction(abstractScope))
                {
                    Action action = (Action) rpmObject;
                    if (action.testApprovedCostBenefitsModified()
                            && ((ScopeElementScope) rpmScope)
                                .isApprovedCostBenefits())
                    {
                        SP_U_CRI_RESP(action, (ScopeElementScope) rpmScope,
                            messageContext);
                    }

                    if (action.testApprovedCostBenefitsModified()
                            && ((ScopeElementScope) rpmScope)
                                .isApprovedCostBenefits()
                            && action.getApprovedCostBenefits()
                                .getWorkInHours() != null)
                    {
                        SP_U_CRI_APPROVED(action, (ScopeElementScope) rpmScope,
                            messageContext);
                    }
                }
                else if (isRequirement(abstractScope))
                {
                    Requirement requirement = (Requirement) rpmObject;
                    if (requirement.testApprovedCostBenefitsModified()
                            && ((ScopeElementScope) rpmScope)
                                .isApprovedCostBenefits())
                    {
                        SP_U_CRI_RESP(requirement,
                            (ScopeElementScope) rpmScope, messageContext);
                    }

                    if (requirement.testApprovedCostBenefitsModified()
                            && ((ScopeElementScope) rpmScope)
                                .isApprovedCostBenefits()
                            && requirement.getApprovedCostBenefits()
                                .getWorkInHours() != null)
                    {
                        SP_U_CRI_APPROVED(requirement,
                            (ScopeElementScope) rpmScope, messageContext);
                    }
                }
                else if (isRisk(abstractScope))
                {
                    if (((Risk) abstractScope).testOccuredDateTimeModified())
                    {
                        SP_U_CRI_PROP((Risk) abstractScope, messageContext);
                    }
                }
            }
            else
            {
                AbstractRequest abstractRequest = (AbstractRequest) rpmObject;
                if ((abstractRequest.testApprovedCostBenefitsModified()
                        && rpmScope != null && ((ScopeElementScope) rpmScope)
                    .isApprovedCostBenefits())
                        || abstractRequest.testApprovedByDateModified()
                        || abstractRequest.testApprovedByNameModified()
                        || abstractRequest.testClosedByDateModified()
                        || abstractRequest.testClosedByNameModified())
                {
                    SP_U_CRI_RESP(abstractRequest,
                        (ScopeElementScope) rpmScope, messageContext);
                }

                if (abstractRequest.testApprovedDateModified()
                        || (abstractRequest.testApprovedCostBenefitsModified() && abstractRequest
                            .getApprovedCostBenefits().getWorkInHours() != null))
                {
                    SP_U_CRI_APPROVED(abstractRequest,
                        (ScopeElementScope) rpmScope, messageContext);
                }

                if (abstractRequest.testNumberTimesReassignedModified()
                        || abstractRequest.testOccuredDateTimeModified())
                {
                    SP_U_CRI_PROP(abstractRequest, messageContext);
                }
            }
        }
        else if (rpmObject instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope abstractAggregateScope = (AbstractAggregateScope) rpmObject;

            if (abstractAggregateScope.testPriorityModified()
                    || abstractAggregateScope.testProposedByNameModified()
                    || abstractAggregateScope.testProposedByDateModified())
            {
                SP_U_CRI_IDEN(abstractAggregateScope, messageContext);
            }

            if (abstractAggregateScope.testEstimatedStartDateModified()
                    || abstractAggregateScope.testEstimatedFinishDateModified())
            {
                SP_U_CRI_ESTIMATED(abstractAggregateScope,
                    (ScopeElementScope) rpmScope, messageContext);
            }

            if (abstractAggregateScope.testClosedByDateModified()
                    || abstractAggregateScope.testClosedByNameModified())
            {
                SP_U_CRI_RESP(abstractAggregateScope,
                    (ScopeElementScope) rpmScope, messageContext);
            }
        }
        else if (rpmObject instanceof AbstractScopeTask)
        {
            AbstractScopeTask abstractScopeTask = (AbstractScopeTask) rpmObject;

            if (abstractScopeTask.testPriorityModified()
                    || abstractScopeTask.testProposedByNameModified()
                    || abstractScopeTask.testProposedByDateModified())
            {
                SP_U_CRI_IDEN(abstractScopeTask, messageContext);
            }

            if (abstractScopeTask.testMandatoryModified())
            {
                SP_U_ELEMENT_CONFIG(abstractScopeTask, messageContext);
            }

            if (abstractScopeTask.testClosedByDateModified()
                    || abstractScopeTask.testClosedByNameModified())
            {
                SP_U_CRI_RESP(abstractScopeTask, (ScopeElementScope) rpmScope,
                    messageContext);
            }
        }

        if (isScopeFolder(rpmObject))
        {
            ScopeFolder scopeFolder = (ScopeFolder) rpmObject;

            if (scopeFolder.testPrivateFolderModified())
            {
                SP_U_CRI_PUBLISHED(scopeFolder, messageContext);
            }
        }

        if (scope != null)
        {
            //Schedule Dates.
            updateScopeScheduleDates(scopeElement, scope, messageContext);

            saveAttributeAssignments(scopeElement, scope, messageContext);
            saveClientCostCenter(scopeElement, scope, messageContext);
            saveDuplicates(scopeElement, scope, messageContext);
            saveMitigationFactors(scopeElement, scope, messageContext);
            savePublished(scopeElement, scope, messageContext);
            saveResourceTaskAssignments(scopeElement, scope, messageContext);
            saveRiskMatrix(scopeElement, scope, messageContext);
            saveRtfAssignments(scopeElement, scope, messageContext);
            saveStatusUpdate(scopeElement, scope, messageContext);
            saveScorecardAssignment(scopeElement, scope, messageContext);
            saveState(scopeElement, scope, messageContext);
            saveWorkSheetFinancials(scopeElement, scope, messageContext);
            saveWBSFinancials(scopeElement, scope, messageContext);

            saveDocumentFolder(scopeElement, scope, messageContext);

            propagateContextName(scopeElement, scopeElement
                .getCustomFieldAssignments());
            scopeElement
                .setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
                    scopeElement, scopeElement.getCustomFieldAssignments(),
                    scope.isCustomFieldAssignments(), messageContext, null));
        }

        // this conversion must happen after all other updates
        if (scopeElement instanceof AbstractScope
                && ((AbstractScope) scopeElement).getTask() != null)
        {
            AbstractScope abstractScope = (AbstractScope) scopeElement;

            Integer convertToTypeId = null;
            boolean isTask = ScopeManagementUtil.isAbstractScopeTask(
                abstractScope, messageContext);

            if (abstractScope.getTask().booleanValue() && !isTask)
            {
                // the current object is not a Task, get the Task type ID
                convertToTypeId = ScopeManagementUtil
                    .getAbstractScopeTaskTypeID(abstractScope);
            }
            else if (!(abstractScope.getTask().booleanValue()) && isTask)
            {
                // the current object is a task; have to get the equivalent
                // AbstractScope type ID
                convertToTypeId = ScopeManagementUtil
                    .getAbstractScopeTypeIDFromTask(abstractScope);
            }

            if (convertToTypeId != null)
            {
                // convert the AbstractScope to / from its Task equivalent
                SP_CHANGE_OBJ_TYPE(abstractScope.getID(), null, abstractScope
                    .retrieveTypeID(), convertToTypeId, messageContext);
            }
        }

        //REQ PRO Objects
        if (scope != null && scope.getReqProData() != null)
        {
            if (ReqProUtil.isReqProObject(scopeElement))
            {
                saveReqProObjects(scopeElement, scope, messageContext);
            }
        }

        if (scope != null)
        {
            //Object move
            if (scope.getParent() != null && scopeElement.testParentModified())
            {
                // Move within the current project
                SP_M_CRI(scopeElement, scopeElement.getParent(), true,
                    messageContext);
            }

            rpmObject = updateChildren(rpmObject, scope, messageContext);

            // Save asset must occur after updateChildren
            saveAssetAndAssetCode(scopeElement, scope, messageContext);
        }

        return rpmObject;
    }

    private ScopeElement saveReqProObjects(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (element instanceof ReqProIntegrationFolder)
        {
            ReqProIntegrationFolder reqProIntegrationFolder = (ReqProIntegrationFolder) element;
            if (reqProIntegrationFolder.getReqProConnection() != null)
            {
                ReqProManager manager = (ReqProManager) getManagerInstance(ReqProConnection.class
                    .getName());

                ReqProConnection reqProConnection = reqProIntegrationFolder
                    .getReqProConnection();
                reqProConnection.setParent(reqProIntegrationFolder);
                manager.internalSave(reqProConnection, scope.getReqProData(),
                    messageContext);
                reqProConnection.setContextName(ContextUtil.SCOPE_CONTEXT);
                reqProIntegrationFolder.setReqProConnection(reqProConnection);
            }
        }
        else if (element instanceof ReqProScopeFolder)
        {
            ReqProScopeFolder reqProScopeFolder = (ReqProScopeFolder) element;
            if (reqProScopeFolder.getReqProImportStatus() != null)
            {
                ReqProManager manager = (ReqProManager) getManagerInstance(ReqProImportStatus.class
                    .getName());

                ReqProImportStatus reqProImportStatus = reqProScopeFolder
                    .getReqProImportStatus();
                reqProImportStatus.setParent(reqProScopeFolder);
                manager.internalSave(reqProImportStatus, scope.getReqProData(),
                    messageContext);
                reqProImportStatus.setContextName(ContextUtil.SCOPE_CONTEXT);
                reqProScopeFolder.setReqProImportStatus(reqProImportStatus);
            }
        }
        else if (element instanceof ReqProRequirement)
        {
            ReqProRequirement reqProRequirement = (ReqProRequirement) element;
            if (reqProRequirement.getReqProImportStatus() != null)
            {
                ReqProManager manager = (ReqProManager) getManagerInstance(ReqProImportStatus.class
                    .getName());

                ReqProImportStatus reqProImportStatus = reqProRequirement
                    .getReqProImportStatus();
                reqProImportStatus.setParent(reqProRequirement);
                manager.internalSave(reqProImportStatus, scope.getReqProData(),
                    messageContext);
                reqProImportStatus.setContextName(ContextUtil.SCOPE_CONTEXT);
                reqProRequirement.setReqProImportStatus(reqProImportStatus);
            }
        }
        return element;
    }

    private void updateScopeScheduleDates(ScopeElement object,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.isScheduleDates())
        {

            RPMObjectManager manager = getManagerInstance(ScopeScheduleDate.class
                .getName());

            if (object instanceof AbstractAggregateScope)
            {

                AbstractAggregateScope aggregateScope = (AbstractAggregateScope) object;

                //proposedDate
                ScopeScheduleDate proposedDate = aggregateScope
                    .getProposedDate();
                if (proposedDate != null)
                {
                    proposedDate.setContextName(ScopeScheduleDate.PROPOSED);
                    proposedDate.setParent(object);
                    proposedDate = (ScopeScheduleDate) manager.internalSave(
                        proposedDate, null, messageContext);
                }

            }

            if (object instanceof AbstractScope)
            {

                AbstractScope abstractScope = (AbstractScope) object;

                //proposedDate
                ScopeScheduleDate approvedDate = abstractScope
                    .getApprovedDate();
                if (approvedDate != null)
                {
                    approvedDate.setContextName(ScopeScheduleDate.APPROVED);
                    approvedDate.setParent(object);
                    approvedDate = (ScopeScheduleDate) manager.internalSave(
                        approvedDate, null, messageContext);
                }

            }
        }
    }

    private void saveAssetAndAssetCode(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (!(pObject instanceof AbstractScope))
            return;
        if (!scope.isAsset() && !scope.isAssetCode())
            return;

        AbstractScope element = (AbstractScope) pObject;
        if (!element.testAssetModified() && !element.testAssetCodeModified())
            return;

        SP_U_CRI_ASSET(element, messageContext);
    }

    private void saveClientCostCenter(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (!(pObject instanceof AbstractScope) || !scope.isClientCostCenter())
            return;

        AbstractScope element = (AbstractScope) pObject;
        if (!element.testClientCostCenterModified())
            return;
        SP_U_CRI_CLIENT(element, messageContext);
    }

    private void saveDuplicates(ScopeElement pObject, ScopeElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (!(pObject instanceof AbstractScope) || !scope.isDuplicates())
            return;

        AbstractScope element = (AbstractScope) pObject;
        if (element.getDuplicates() == null)
            return;
        Duplicate[] duplicates = element.getDuplicates();
        for (int i = 0; i < duplicates.length; ++i)
        {
            if (duplicates[i] != null)
            {
                // Save ScopeElement if it has been modified?
                if ((element.getDuplicateNumber() == null && duplicates[i]
                    .getDeleted() == Boolean.FALSE)
                        || (!(element.getDuplicateNumber()
                            .equals(((AbstractScope) duplicates[i]
                                .getScopeElement()).getDuplicateNumber())))
                        || duplicates[i].getDeleted() == Boolean.TRUE)
                {
                    // Note: If the ScopeElement is already a duplicate, then
                    // this stored procedure will delete it
                    ((AbstractScope) duplicates[i].getScopeElement())
                        .setDuplicateNumber(SP_A_DUPLICATE_CRI(element,
                            (AbstractScope) duplicates[i].getScopeElement(),
                            messageContext));
                }
            }
        }
        element.setDuplicates(duplicates);
    }

    private void saveMitigationFactors(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        if (!(pObject instanceof Risk)
                || (scope.getMitigationFactors() == null))
            return;
        Risk element = (Risk) pObject;
        element.setMitigationFactors((MitigationFactor[]) (updateChildren(
            element, element.getMitigationFactors(), scope
                .getMitigationFactors(), messageContext, null)));
    }

    private void savePublished(ScopeElement pObject, ScopeElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (!(pObject instanceof AbstractScope))
            return;

        AbstractScope element = (AbstractScope) pObject;
        if (!element.testPublishedModified())
            return;
        //TODO Must implement template folder functionality first
        //SP_U_TEMPLATE(element, messageContext);
    }

    private void saveRiskMatrix(ScopeElement pObject, ScopeElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (!(pObject instanceof Risk) || !scope.isRiskMatrix())
            return;

        Risk element = (Risk) pObject;
        if (!element.testRiskMatrixModified())
            return;
        SP_U_RSK_QUAN(element, messageContext);
    }

    private void saveRtfAssignments(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (!scope.isRtfAssignments())
            return;

        if (pObject instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope absAggScope = (AbstractAggregateScope) pObject;
            absAggScope.setRtfAssignments((RtfAssignment[]) updateChildren(
                absAggScope, absAggScope.getRtfAssignments(), scope
                    .isRtfAssignments(), messageContext, null));
        }
        else if (pObject instanceof AbstractScopeTask)
        {
            AbstractScopeTask absScopeTask = (AbstractScopeTask) pObject;
            absScopeTask.setRtfAssignments((RtfAssignment[]) updateChildren(
                absScopeTask, absScopeTask.getRtfAssignments(), scope
                    .isRtfAssignments(), messageContext, null));
        }
    }

    private void saveStatusUpdate(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

    }

    private void saveScorecardAssignment(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (!(pObject instanceof AbstractScope)
                || scope.getAssignedScorecard() == null)
            return;

        AbstractScope element = (AbstractScope) pObject;
        element.setAssignedScorecard((AssignedScorecard) updateChild(element,
            element.getAssignedScorecard(), scope.getAssignedScorecard(),
            messageContext));
    }

    private void saveState(ScopeElement pObject, ScopeElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (!(pObject instanceof AbstractScope) || !scope.isState())
            return;

        AbstractScope element = (AbstractScope) pObject;
        if (!element.testStateModified() || element.getState() == null)
            return;
        if (element.getState().getID() == null)
            return;
        SP_U_STAGE_ID(element, messageContext);
    }

    private void saveWorkSheetFinancials(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getWorksheetFinancials() != null)
        {
            if (pObject instanceof AbstractScope)
            {
                AbstractScope absScope = (AbstractScope) pObject;
                absScope
                    .setWorksheetFinancials((WorksheetFinancial[]) updateChildren(
                        absScope, absScope.getWorksheetFinancials(), scope
                            .getWorksheetFinancials(), messageContext, null));
            }
        }
    }

    private void saveWBSFinancials(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getWbsFinancials() != null)
        {
            if (pObject instanceof AbstractScope)
            {
                AbstractScope absScope = (AbstractScope) pObject;
                absScope.setWbsFinancials((WbsFinancial[]) updateChildren(
                    absScope, absScope.getWbsFinancials(), scope
                        .getWbsFinancials(), messageContext, null));
            }
            else if (pObject instanceof AbstractScopeTask)
            {
                AbstractScopeTask absScope = (AbstractScopeTask) pObject;
                absScope.setWbsFinancials((WbsFinancial[]) updateChildren(
                    absScope, absScope.getWbsFinancials(), scope
                        .getWbsFinancials(), messageContext, null));
            }
        }
    }

    private void saveResourceTaskAssignments(ScopeElement scopeElement,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getResourceTaskAssignments() != null)
        {
            scopeElement
                .setResourceTaskAssignments((ResourceTaskAssignment[]) updateChildren(
                    scopeElement, scopeElement.getResourceTaskAssignments(),
                    scope.getResourceTaskAssignments(), messageContext,
                    ResourceTaskAssignment.class));
        }
    }

    private void saveAttributeAssignments(ScopeElement pObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.isAttributeAssignments())
        {
            if (pObject instanceof AbstractScopeTask)
            {
                AbstractScopeTask absScopeTask = (AbstractScopeTask) pObject;
                absScopeTask
                    .setAttributeAssignments((AttributeAssignment[]) updateChildren(
                        absScopeTask, absScopeTask.getAttributeAssignments(),
                        scope.isAttributeAssignments(), messageContext, null));
            }
            else if (pObject instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope absAggScope = (AbstractAggregateScope) pObject;
                absAggScope
                    .setAttributeAssignments((AttributeAssignment[]) updateChildren(
                        absAggScope, absAggScope.getAttributeAssignments(),
                        scope.isAttributeAssignments(), messageContext, null));
            }
        }
    }

    //    private void saveScorecard(AbstractScope cri, ScopeElementScope scope,
    //            MessageContext messageContext) throws RPMException, SQLException,
    //            ParseException
    //    {
    //        cri.setAssignedScorecard((AssignedScorecard)
    //                updateChild(cri, cri.getAssignedScorecard(),
    //                        scope.getAssignedScorecard(), messageContext));
    //    }

    private void loadDocumentFolder(AbstractScope cri, ScopeElementScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if (scope.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                cri.getDocumentFolder()
            };
            Object[] args = new Object[]
            {
                cri.getID()
            };
            SqlBuffer where = new SqlBuffer();
            where.appendEqual(DocumentManager.NAME_TYPE_ID,
                DocumentFolder.TYPE_ID);
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(cri,
                oldDocumentFolder, scope.getDocumentFolder(), messageContext,
                this, args, where.toString(),
                ContextUtil.SCOPE_DOCUMENT_CONTEXT, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder
                    .setContextName(ContextUtil.SCOPE_DOCUMENT_CONTEXT);
                cri.setDocumentFolder(documentFolder);
            }
        }
    }

    private ScopeElement saveDocumentFolder(ScopeElement element,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (element instanceof AbstractScope)
        {
            AbstractScope cri = (AbstractScope) element;
            if (scope.getDocumentFolder() != null
                    && cri.getDocumentFolder() != null)
            {
                DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

                DocumentFolder documentFolder = cri.getDocumentFolder();
                documentFolder.setParent(cri);
                manager.internalSave(documentFolder, scope.getDocumentFolder(),
                    messageContext);
                documentFolder
                    .setContextName(ContextUtil.SCOPE_DOCUMENT_CONTEXT);
                cri.setDocumentFolder(documentFolder);
            }
        }
        return element;
    }

    private boolean isWorkElement(ScopeElement element,
            FieldValueMap relatedFields, int index) throws RPMException
    {
        int parentTypeID = ((Integer) relatedFields.get(index,
            ID_PARENT_TYPE_ID)).intValue();

        return ((element instanceof ScopeFolder) && ((parentTypeID == Proposal.TYPE_ID)
                || (parentTypeID == Project.TYPE_ID) || (parentTypeID == WorkOrganization.TYPE_ID)));
    }

    private boolean isDefaultScopeElement(AbstractScope abstractScope)
    {
        return abstractScope.getDefaultScope() != null
                && abstractScope.getDefaultScope().booleanValue();
    }

    private boolean isScopeFolder(RPMObject rpmObject)
    {
        return rpmObject instanceof ScopeFolder;
    }

    private boolean isRequirement(RPMObject rpmObject)
    {
        return rpmObject instanceof Requirement;
    }

    private boolean isChangeRequest(RPMObject rpmObject)
    {
        return rpmObject instanceof ChangeRequest;
    }

    private boolean isServiceRequest(RPMObject rpmObject)
    {
        return rpmObject instanceof ServiceRequest;
    }

    private boolean isAction(RPMObject rpmObject)
    {
        return rpmObject instanceof Action;
    }

    private boolean isDefect(RPMObject rpmObject)
    {
        return rpmObject instanceof Defect;
    }

    private boolean isIssue(RPMObject rpmObject)
    {
        return rpmObject instanceof Issue;
    }

    private boolean isRisk(RPMObject rpmObject)
    {
        return rpmObject instanceof Risk;
    }

    private boolean isDeliverable(RPMObject rpmObject)
    {
        return rpmObject instanceof ScopeDeliverable;
    }

    public boolean isSummaryTask(RPMObject rpmObject)
    {
        return rpmObject instanceof ScopeSummaryTask;
    }

    public boolean isMilestone(RPMObject rpmObject)
    {
        return rpmObject instanceof ScopeMilestone;
    }

    public boolean isTask(RPMObject rpmObject)
    {
        return rpmObject instanceof ScopeTask;
    }

    protected RPMObject updateChildren(RPMObject rpmObject,
            ScopeElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject instanceof AggregateScope)
        {
            AggregateScope aggObj = (AggregateScope) rpmObject;
            propagateContextName(aggObj, aggObj.getChildren());
            aggObj.setChildren((ScopeElement[]) updateChildren(aggObj, aggObj
                .getChildren(), scope.getChildren(), messageContext,
                ScopeElement.class));
        }
        return rpmObject;
    }

    public void transferUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ScopeMethodCheckpoint checkpoint = ScopeMethodCheckpoint.getInstance();
        checkpoint.validateTransferAbstractScopeID(abstractScope,
            messageContext);

        if (messageContext.isSuccessful())
        {
            if (workElement == null || workElement.getID() == null)
            {
                if (checkpoint.canTransferUnderWBS(workElement, abstractScope,
                    messageContext))
                {
                    // transfer under WBS root, bitflag 0
                    SP_T_CRI(workElement, abstractScope, 0, messageContext);
                }
            }
            else
            {
                checkpoint.validateTransferAndMoveIDs(workElement,
                    abstractScope, messageContext);

                if (messageContext.isSuccessful())
                {
                    // cross transfer not valid
                    checkpoint.validateTransferUnderWbs(workElement,
                        abstractScope, messageContext);

                    if (messageContext.isSuccessful()
                            && checkpoint.canTransferUnderWBS(workElement,
                                abstractScope, messageContext))
                    {
                        // transfer within, bitflag 8
                        SP_T_CRI(workElement, abstractScope, 8, messageContext);
                    }
                }
            }
        }
    }

    public void moveUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ScopeMethodCheckpoint checkpoint = ScopeMethodCheckpoint.getInstance();
        checkpoint.validateTransferAbstractScopeID(abstractScope,
            messageContext);

        if (messageContext.isSuccessful())
        {
            checkpoint.validateTransferAndMoveIDs(workElement, abstractScope,
                messageContext);

            if (messageContext.isSuccessful()
                    && checkpoint.canMoveUnderWBS(workElement, abstractScope,
                        messageContext))
            {
                String projectId = WbsUtil.getWorkElementProjectID(workElement,
                    messageContext);

                if (abstractScope.getProject().getID().equals(projectId))
                {
                    // move within, bitflag 4
                    SP_T_CRI(workElement, abstractScope, 4, messageContext);
                }
                else
                {
                    // cross move, bitflag 6
                    SP_T_CRI(workElement, abstractScope, 6, messageContext);
                }
            }
        }
    }

    public void moveAndTransferUnderWBS(WorkElement workElement,
            AbstractScope abstractScope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ScopeMethodCheckpoint checkpoint = ScopeMethodCheckpoint.getInstance();
        checkpoint.validateTransferAbstractScopeID(abstractScope,
            messageContext);

        if (messageContext.isSuccessful())
        {
            checkpoint.validateTransferAndMoveIDs(workElement, abstractScope,
                messageContext);

            if (messageContext.isSuccessful()
                    && checkpoint.canMoveAndTransferUnderWBS(workElement,
                        abstractScope, messageContext))
            {
                String projectId = WbsUtil.getWorkElementProjectID(workElement,
                    messageContext);

                if (abstractScope.getProject().getID().equals(projectId))
                {
                    // move and transfer within, bitflag 12
                    SP_T_CRI(workElement, abstractScope, 12, messageContext);
                }
                else
                {
                    // cross move and transfer, bitflag 14
                    SP_T_CRI(workElement, abstractScope, 14, messageContext);
                }
            }
        }
    }

    public AbstractScope convertScopeElement(AbstractScope inputScopeElement,
            RPMObject outputScopeElement, RPMObjectScope rpmScope,
            MessageContext msgCtx) throws RPMException, SQLException,
            ParseException
    {
        // the inputScopeElement must be previously saved and 
        // the outputScopeElement must be of a valid type
        ScopeMethodCheckpoint checkpoint = ScopeMethodCheckpoint.getInstance();
        checkpoint.validateConvertScopeElement(inputScopeElement,
            outputScopeElement, rpmScope, msgCtx);

        AbstractScope outputElement = null;

        if (msgCtx.isSuccessful())
        {
            if (checkpoint.canConvertScopeElement(inputScopeElement,
                outputScopeElement, rpmScope, msgCtx))
            {
                if (inputScopeElement.getTask() != null
                        && inputScopeElement.getTask().booleanValue())
                {
                    // convert an [AbstractScope]Task to an [AbstractScope]Task
                    SP_CHANGE_OBJ_TYPE(
                        inputScopeElement.getID(),
                        outputScopeElement.getID(),
                        ScopeManagementUtil
                            .getAbstractScopeTaskTypeID(inputScopeElement),
                        ScopeManagementUtil
                            .getAbstractScopeTaskTypeID((AbstractScope) outputScopeElement),
                        msgCtx);
                }
                else
                {
                    // convert an AbstractScope to an AbstractScope
                    SP_CHANGE_OBJ_TYPE(inputScopeElement,
                        (ScopeElement) outputScopeElement, msgCtx);
                }

                //To be safe, remove inputScopeElement from messageContext cache -if it was there-
                //since now it has been converted which means the instance that 
                //-might- exist in the cache is not valid anymore
                msgCtx.getCache().removeContainerFromCache(inputScopeElement);

                outputElement = (AbstractScope) loadByPrimaryKey(
                    inputScopeElement, rpmScope, msgCtx, false);
            }
        }

        return outputElement;
    }

    private Object[] getProposedByValues(ScopeElement element,
            MessageContext messageContext) throws RPMException, ParseException, SQLException
    {
    	Object proposedByValues[] = new Object[2];
    	String proposedByName = null;
    	String proposedByDate = null;
    	boolean proposedByNameModified = false;
    	boolean proposedByDateModified = false;
    	if (element instanceof AbstractScopeTask) {
    		proposedByName = ((AbstractScopeTask) element).getProposedByName();
    		proposedByNameModified = ((AbstractScopeTask) element)
    		.testProposedByNameModified();
    		proposedByDate = DateUtil
    		.convertToStringWithTime(((AbstractScopeTask) element)
    				.getProposedByDate());
    		proposedByDateModified = ((AbstractScopeTask) element)
    		.testProposedByDateModified();
    	} else if (element instanceof AbstractAggregateScope) {
    		proposedByName = ((AbstractAggregateScope) element)
    		.getProposedByName();
    		proposedByNameModified = ((AbstractAggregateScope) element)
    		.testProposedByNameModified();
    		proposedByDate = DateUtil
    		.convertToStringWithTime(((AbstractAggregateScope) element)
    				.getProposedByDate());
    		proposedByDateModified = ((AbstractAggregateScope) element)
    		.testProposedByDateModified();
    	}
    	if (proposedByNameModified || proposedByDateModified) {
    		proposedByValues = getByNameByDateValues(proposedByName,
    				proposedByDate, proposedByNameModified,
    				proposedByDateModified, messageContext);
    	} else {
    		ScopeElement loadedElement = (ScopeElement) load(element, null,
    				messageContext);
    		if (element instanceof AbstractScopeTask) {
    			proposedByValues[0] = ((AbstractScopeTask) loadedElement)
    			.getProposedByName();
    			proposedByValues[1] = DateUtil
    			.convertToStringWithTime(((AbstractScopeTask) loadedElement)
    					.getProposedByDate());
    		} else if (element instanceof AbstractAggregateScope) {
    			proposedByValues[0] = ((AbstractAggregateScope) loadedElement)
    			.getProposedByName();
    			proposedByValues[1] = DateUtil
    			.convertToStringWithTime(((AbstractAggregateScope) loadedElement)
    					.getProposedByDate());
    		}
    	}
    	return proposedByValues; 
    }

    private Object[] getApprovedByValues(ScopeElement element,
            MessageContext messageContext) throws RPMException
    {
        Object approvedByValues[] = new Object[2];
        String approvedByDate = null;
        String approvedByName = null;
        boolean approvedByDateModified = false;
        boolean approvedByNameModified = false;
        if (element instanceof AbstractScope)
        {
            approvedByDate = DateUtil
                .convertToStringWithTime(((AbstractScope) element)
                    .getApprovedByDate());
            approvedByDateModified = ((AbstractScope) element)
                .testApprovedByDateModified();
            approvedByName = ((AbstractScope) element).getApprovedByName();
            approvedByNameModified = ((AbstractScope) element)
                .testApprovedByNameModified();
        }
        return getByNameByDateValues(approvedByName, approvedByDate,
            approvedByNameModified, approvedByDateModified, messageContext);
    }

    private Object[] getClosedByValues(ScopeElement element,
            MessageContext messageContext) throws RPMException
    {
        Object closedByValues[] = new Object[2];
        String closedByName = null;
        String closedByDate = null;
        boolean closedByNameModified = false;
        boolean closedByDateModified = false;
        if (element instanceof AbstractScopeTask)
        {
            closedByName = ((AbstractScopeTask) element).getClosedByName();
            closedByNameModified = ((AbstractScopeTask) element)
                .testClosedByNameModified();
            closedByDate = DateUtil
                .convertToStringWithTime(((AbstractScopeTask) element)
                    .getClosedByDate());
            closedByDateModified = ((AbstractScopeTask) element)
                .testClosedByDateModified();
        }
        else if (element instanceof AbstractAggregateScope)
        {
            closedByName = ((AbstractAggregateScope) element).getClosedByName();
            closedByNameModified = ((AbstractAggregateScope) element)
                .testClosedByNameModified();
            closedByDate = DateUtil
                .convertToStringWithTime(((AbstractAggregateScope) element)
                    .getClosedByDate());
            closedByDateModified = ((AbstractAggregateScope) element)
                .testClosedByDateModified();
        }
        return getByNameByDateValues(closedByName, closedByDate,
            closedByNameModified, closedByDateModified, messageContext);
    }

    private Object[] getByNameByDateValues(String byName, String byDate,
            boolean byNameModified, boolean byDateModified,
            MessageContext messageContext) throws RPMException
    {
    	Object byValues[] = new Object[2];
    	if (byName != null || byDate != null) {
    		if (byName == null && !byNameModified) {
    			byName = ((Resource) getUser(messageContext)).getFullName();
    		} else if (byDate == null && !byDateModified) {
    			Calendar currentDatabaseTimeStamp = messageContext.getFactory()
    			.getManagerCaller()
    			.getDatabaseTimestamp(messageContext);
    			byDate = DateUtil
    			.convertToStringWithTime(currentDatabaseTimeStamp);
    		}
    	}
    	byValues[0] = byName;
    	byValues[1] = byDate;

    	return byValues;
    }

    private void SP_CHANGE_OBJ_TYPE(AbstractScope inputScopeElement,
            ScopeElement outputScopeElement, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_CHANGE_OBJ_TYPE";
        Object args[] = new Object[6];

        args[0] = new Integer(0); // bitflag
        args[1] = inputScopeElement.getID();
        args[2] = inputScopeElement.retrieveTypeID();
        args[3] = outputScopeElement.getID();
        args[4] = outputScopeElement.retrieveTypeID();

        if (getUser(messageContext) != null)
        {
            args[5] = getUser(messageContext).getID();
        }

        executeProcedure(messageContext, name, args);
    }

    private void SP_CHANGE_OBJ_TYPE(String inputScopeElementID,
            String outputScopeElementID, Integer inputTypeId,
            Integer outputTypeId, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_CHANGE_OBJ_TYPE";
        Object args[] = new Object[6];

        args[0] = new Integer(0); // bitflag
        args[1] = inputScopeElementID;
        args[2] = inputTypeId;
        args[3] = outputScopeElementID;
        args[4] = outputTypeId;

        if (getUser(messageContext) != null)
        {
            args[5] = getUser(messageContext).getID();
        }

        executeProcedure(messageContext, name, args);
    }

    private void SP_CHANGE_OBJ_TYPE(ScopeElement inputScopeElement,
            ScopeElement outputScopeElement, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_CHANGE_OBJ_TYPE";
        Object args[] = new Object[6];

        args[0] = new Integer(0); // bitflag
        args[1] = inputScopeElement.getID();
        args[2] = inputScopeElement.retrieveTypeID();
        args[3] = outputScopeElement.getID();
        args[4] = outputScopeElement.retrieveTypeID();

        if (getUser(messageContext) != null)
        {
            args[5] = getUser(messageContext).getID();
        }

        executeProcedure(messageContext, name, args);
    }

    /*
     * private RPMObject[] instantiateChildrenArray(RPMObject child, int length) {
     * if (child instanceof ScopeElement) { return new ScopeElement[length]; }
     * return null; }
     */
    /////////////////Stored Procedures/////////////////////////////
    private void SP_D_CRI(RPMObject rpmObject, String parentId,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_CRI";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = parentId;
        //Must put "Q" else it returns an error message with question "are you sure..."
        args[2] = "Q";
        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }
        //The needed parameters for the SP are:
        //ELEMENT_ID, PARENT_ID, REC_STATUS, REC_USER
        executeProcedure(messageContext, name, args);
    }

    private String SP_C_CRI(RPMObject object, MessageContext messageContext)
            throws SQLException, RPMException
    {
        final String procedureName = "SP_C_CRI";

        Object args[] = new Object[6];

        AbstractScope scopeElement = (AbstractScope) object;

        args[0] = new Long(3); // Level ID
        args[1] = scopeElement.getParent().getID(); //target: ElementID

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(NAME_ELEMENT_ID);
        Object[] arguments = new Object[]{scopeElement.getParent().getID()};
        String scopeFolderID = (String) ManagerUtil.selectColumnValue(
                Object.class, null, null, NAME_PROJECT_ID, TABLE_NAME,
                condition, arguments, messageContext);

        args[2] = scopeFolderID; //foreign_id : getID(): ProjectID

        args[3] = messageContext.getTemplate().getID(); //Source
        args[4] = new Long(2); //bitFlag
        args[5] = getUser(messageContext).getID(); //user: 'PMO_SUPERVISOR'

        return executeProcedure(messageContext, procedureName, args);
    }

    private String SP_I_CRI(ScopeElement rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_CRI";
        Object args[] = new Object[6];
        args[0] = new Integer(3);
        args[1] = rpmObject.getName();
        if (rpmObject.getParent() != null)
        {
            args[2] = rpmObject.getParent().getID();
        }
        args[3] = null;
        args[4] = rpmObject.retrieveTypeID();
        args[5] = getUser(messageContext).getID();
        //The needed parameters for the SP are:
        //TABLE, ELEMENT_NAME, PARENT_ID, FOREIGN_ID, TYPE_ID, REC_USER
        return executeProcedure(messageContext, name, args);
    }

    private void SP_M_CRI(RPMObject rpmObject, RPMObject target,
            boolean toParent, MessageContext messageContext)
            throws SQLException, RPMException
    {
        final String procedureName = "SP_M_CRI";
        Object args[] = new Object[4];
        // target_id
        args[0] = target.getID();
        // source_id_list
        args[1] = rpmObject.getID();

        //bitflag
        if (toParent)
        {
            args[2] = new Long(4);
        }
        else
        {
            args[2] = new Long(5);
        }

        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_CKIN_CRI(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKIN_CRI";
        Object args[] = new Object[3];
        args[0] = rpmObject.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private String encodeElementSecurityType(ScopeElement element)
    {
        return (String) EnumeratedTypeUtil.getInstance().getValue(
            ScopeElement.class, ScopeElementSecurityType.class,
            element.getElementSecurityType().getValue());
    }

    private void SP_U_CRI_PUBLISHED(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CRI_PUBLISHED";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = encodeElementSecurityType((ScopeElement) rpmObject);
        args[2] = "Y";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_OUT_WKF(RPMObject rpmObject, String workflowID,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_CHECK_OUT_WKF";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = workflowID;
        args[2] = new Integer(3); // table type for scope element.
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CKOUT_CRI(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CKOUT_CRI";
        Object args[] = new Object[3];
        args[0] = rpmObject.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_UCKOUT_CRI(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_UCKOUT_CRI";
        Object args[] = new Object[3];
        args[0] = rpmObject.getID();
        args[1] = new Integer(0);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    // SP_A_DUPLICATE_CRI('DCD87074247F402BA82C0C5DEE2EE2D8','EB34A0881C424D249DAFC99CA0BBC70C','PMO_SUPERVISOR')
    private String SP_A_DUPLICATE_CRI(AbstractScope sourceObject,
            AbstractScope targetObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_A_DUPLICATE_CRI";
        Object args[] = new Object[3];
        args[0] = sourceObject.getID();
        args[1] = targetObject.getID();
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private String SP_U_DEFAULT_CRI(AbstractScope scopeObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_DEFAULT_CRI";
        Object args[] = new Object[3];
        args[0] = scopeObject.getID();
        args[1] = (isDefaultScopeElement(scopeObject)) ? "1" : "2";
        args[2] = getUser(messageContext).getID();
        //p_id, p_flag, p_rec_user
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    // SP_U_ELEMENT_CONFIG('4B72E49709D545AEA47CEE2AD2508663',3,0,'PMO_SUPERVISOR')
    private void SP_U_ELEMENT_CONFIG(AbstractScopeTask element,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ELEMENT_CONFIG";
        Object args[] = new Object[4];
        args[0] = element.getID();
        args[1] = new Integer(3); // Table type constant
        args[2] = (element.getMandatory().equals(Boolean.TRUE))
                ? new Integer(1) : new Integer(0);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_RSK_QUAN('73D78864E51E445BB08E490A8C51EEFB',16,0,0,0,0,4,2,3,80,65,65,'
    // PMO_SUPERVISOR')
    private void SP_U_RSK_QUAN(Risk element, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_RSK_QUAN";
        Object args[] = new Object[13];
        args[0] = element.getID();

        // Default values because this stored procedure does not accept any null
        // values
        args[1] = new Integer(0);
        args[2] = new Integer(0);
        args[3] = new Integer(0);
        args[4] = new Integer(0);
        args[5] = new Integer(0);
        args[6] = new Integer(0);
        args[8] = new Integer(0);
        args[9] = new Integer(0);
        args[10] = new Integer(0);
        args[11] = new Integer(0);

        if (element.getRiskMatrix().getMitigationCost() != null)
        {
            args[1] = element.getRiskMatrix().getMitigationCost();
        }

        if (element.getRiskMatrix().getProbability() != null)
        {
            args[6] = new Integer((element.getRiskMatrix().getProbability()
                .intValue() - 20) / 20);
        }

        if (element.getRiskMatrix().getPrecision() == PrecisionType.High)
        {
            args[7] = new Integer(2);
        }
        else if (element.getRiskMatrix().getPrecision() == PrecisionType.Medium)
        {
            args[7] = new Integer(1);
        }
        else
        {
            args[7] = new Integer(0);
        }

        if (element.getRiskMatrix().getImpact() != null)
        {
            args[8] = new Integer(element.getRiskMatrix().getImpact()
                .intValue() / 20);
        }

        if (element.getRiskMatrix().getExposurePercent() != null)
        {
            args[9] = element.getRiskMatrix().getExposurePercent();
        }

        if (element.getRiskMatrix().getConsequenceCost() != null)
        {
            args[10] = element.getRiskMatrix().getConsequenceCost();
        }

        if (element.getRiskMatrix().getProbabilityCost() != null)
        {
            args[11] = element.getRiskMatrix().getProbabilityCost();
        }

        args[12] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_STAGE_ID('FE88DDD9E7994202B7CFB6577ADE719E',3,5385,'PMO_SUPERVISOR')
    private String SP_U_STAGE_ID(AbstractScope scopeObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_STAGE_ID";
        Object args[] = new Object[4];
        args[0] = scopeObject.getID();
        args[1] = new Integer(3);
        args[2] = Integer.valueOf(scopeObject.getState().getID());
        args[3] = getUser(messageContext).getID();
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    // SP_U_TEMPLATE(3,'73D78864E51E445BB08E490A8C51EEFB','1','3BB720773013442E9FC5CA2205BD9AED','PMO_SUPERVISOR')
    public void SP_U_TEMPLATE(AbstractScope element,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_TEMPLATE";
        Object args[] = new Object[4];
        args[0] = new Integer(3); // Table constant
        args[1] = element.getID();
        args[2] = (element.getPublished() == PublishedType.ToTemplate) ? "1"
                : "0";
        args[3] = null; // T_PARENT_ID
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    // SP_U_CRI_APPROVED('66733D8B9CCD42FDB37A31C9086D7E6D',0,null,null,0,300,'PMO_SUPERVISOR')
    private void SP_U_CRI_APPROVED(AbstractScope element,
            ScopeElementScope scope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        String name = "SP_U_CRI_APPROVED";
        Object args[] = new Object[7];

        //reload missing information: ScopeScheduleDate
        if (!scope.isScheduleDates() || element.getApprovedDate() == null)
        {
            ScopeElement temp = new ScopeElement();
            temp.setID(element.getID());

            ScopeElementScope tempScope = new ScopeElementScope();
            tempScope.setScheduleDates(true);

            //TODO use direct sql instead
            temp = (ScopeElement) loadByPrimaryKey(temp, tempScope,
                messageContext, false);

            element.setApprovedDate(((AbstractScope) temp).getApprovedDate());

        }

        args[0] = element.getID();
        args[1] = new Integer(0);

        if (element.getApprovedDate() != null)
        {
            args[2] = DateUtil.convertToStringWithTime(element
                .getApprovedDate().getStartDate());

            //finishDate as a read-only field
            //when the parameter is null, the finishDate is calculated from the startDate and the duration. 
            args[3] = null;

            //fields days and hours are read-only
            if (element.getApprovedDate() != null
                    && element.getApprovedDate().getDurationInMinutes() != null)
            {
                args[4] = element.getApprovedDate().getDurationInMinutes();
            }

        }

        if (scope != null && scope.isApprovedCostBenefits())
        {
            if ((element instanceof AbstractRequest)
                    && (((AbstractRequest) element).getApprovedCostBenefits()
                        .getWorkInHours() != null))
            {
                args[5] = new Integer(((AbstractRequest) element)
                    .getApprovedCostBenefits().getWorkInHours().intValue() * 60);
            }
            else if ((element instanceof Requirement)
                    && (((Requirement) element).getApprovedCostBenefits()
                        .getWorkInHours() != null))
            {
                args[5] = new Integer(((Requirement) element)
                    .getApprovedCostBenefits().getWorkInHours().intValue() * 60);
            }
            else if ((element instanceof Action)
                    && (((Action) element).getApprovedCostBenefits()
                        .getWorkInHours() != null))
            {
                args[5] = new Integer(((Action) element)
                    .getApprovedCostBenefits().getWorkInHours().intValue() * 60);
            }
        }

        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_CRI_ASSET('A5F06D1BE5A34AD1B4BA9892800A670E',null,null,6503,'PMO_SUPERVISOR')
    //SP_U_CRI_ASSET('F40CFF6CBAB9416CAB24C5F6CE0A27BC','348685B66F424476A136B09FE0FCAA56','Asset1',6504,'PMO_SUPERVISOR')
    private void SP_U_CRI_ASSET(AbstractScope element,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        String name = "SP_U_CRI_ASSET";
        Object[] args = new Object[5];
        args[0] = element.getID();

        if (element.getAsset() != null)
        {
            AssetManager manager = (AssetManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Asset.class.getName());

            Asset reloaded = (Asset) manager.load(element.getAsset(), null,
                messageContext);

            args[1] = reloaded.getID();
            args[2] = reloaded.getName();
        }

        if (element.getAssetCode() != null)
            args[3] = new Integer(element.getAssetCode().getID());
        else
            args[3] = new Integer(6501);

        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_CRI_CLIENT('F643285BE2CE479B9E93FAED4377F4B5','PMOFFICE_DEFAULT_COST_CENTER____','RPM','PMO_SUPERVISOR')
    private void SP_U_CRI_CLIENT(AbstractScope element,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        String name = "SP_U_CRI_CLIENT";
        Object args[] = new Object[4];
        args[0] = element.getID();

        if (element.getClientCostCenter() != null)
        {
            ClientManager manager = (ClientManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ClientCostCenter.class.getName());

            ClientCostCenter reloaded = (ClientCostCenter) manager.load(element
                .getClientCostCenter(), null, messageContext);

            args[1] = reloaded.getID();
            args[2] = reloaded.getName();
        }
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_ESTIMATED(AbstractAggregateScope element,
            ScopeElementScope scope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_CRI_ESTIMATED";
        Object args[] = new Object[7];
        args[0] = element.getID();
        args[1] = new Integer(0);
        args[2] = DateUtil.convertToStringWithTime(element
            .getEstimatedStartDate());

        //estimatedFinishDate is read-only
        //when the parameter is null, the finishDate is calculated from the startDate and the duration. 
        args[3] = DateUtil.convertToStringWithTime(element
            .getEstimatedFinishDate());

        //this field is not visible in the UI
        if (element.getEstimatedDurationInMinutes() != null)
        {
            args[4] = element.getEstimatedDurationInMinutes();
        }
        else
        {
            args[4] = new Integer(0);
        }

        if (scope != null && scope.isImpact()
                && element instanceof AbstractScope
                && ((AbstractScope) element).getImpact() != null)
        {
            args[5] = ((AbstractScope) element).getImpact()
                .getOrderOfMagnitudeWorkInHours();

        }
        else
        {
            args[5] = new Integer(0);
        }

        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_ESTIMATED_HIGH(AbstractAggregateScope element,
            ScopeElementScope scope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_CRI_ESTIMATED_HIGH";
        Object args[] = new Object[7];
        args[0] = element.getID();
        args[1] = new Integer(0);
        args[2] = null; // TODO
        args[3] = null; // TODO
        args[4] = new Integer(0); // TODO

        if (scope != null && scope.isImpact()
                && element instanceof AbstractScope
                && ((AbstractScope) element).getImpact() != null)
        {
            args[5] = ((AbstractScope) element).getImpact()
                .getOrderOfMagnitudeHighWorkInHours();
        }

        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_ESTIMATED_LOW(AbstractAggregateScope element,
            ScopeElementScope scope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_CRI_ESTIMATED_LOW";
        Object args[] = new Object[7];
        args[0] = element.getID();
        args[1] = new Integer(0);
        args[2] = null; // TODO
        args[3] = null; // TODO
        args[4] = new Integer(0); // TODO

        if (scope != null && scope.isImpact()
                && element instanceof AbstractScope
                && ((AbstractScope) element).getImpact() != null)
        {
            args[5] = ((AbstractScope) element).getImpact()
                .getOrderOfMagnitudeLowWorkInHours();
        }

        args[6] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_CRI_IDEN('A15C400301074A548576CE5ECBF26EBD','34741',5,'3939',547,500,0,null,0,null,null,'ADMINISTRATOR','2005-12-09
    // 16:49:59.453','PMO_SUPERVISOR')
    private void SP_U_CRI_IDEN(AbstractAggregateScope element,
            MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
        String name = "SP_U_CRI_IDEN";
        Object args[] = new Object[14];
        Object proposedByValues[] = getProposedByValues(element, messageContext);
        args[0] = element.getID();
        args[1] = element.getReferenceNumber(); // Read Only
        args[2] = element.retrieveTypeID();
        args[3] = null; // AbstractAggregateScope does not have
        // externalReferenceNumber
        args[4] = new Integer(500); // AbstractAggregateScope does not have rank
        args[5] = element.getPriority();
        args[6] = new Integer(0);
        args[7] = null;
        args[8] = new Integer(0);
        args[9] = null;
        args[10] = null;
        args[11] = proposedByValues[0];
        args[12] = proposedByValues[1];
        args[13] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_IDEN(AbstractScope element,
            MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
        String name = "SP_U_CRI_IDEN";
        Object args[] = new Object[14];
        Object proposedByValues[] = getProposedByValues(element, messageContext);
        args[0] = element.getID();
        args[1] = element.getReferenceNumber(); // Read Only
        args[2] = element.retrieveTypeID();
        args[3] = element.getExternalReferenceNumber();
        args[4] = element.getRank();
        args[5] = element.getPriority();
        args[6] = new Integer(0);
        args[7] = null;
        args[8] = new Integer(0);
        args[9] = null;
        args[10] = null;
        args[11] = proposedByValues[0];
        args[12] = proposedByValues[1];
        args[13] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_IDEN(AbstractScopeTask element,
            MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
        String name = "SP_U_CRI_IDEN";
        Object args[] = new Object[14];
        Object proposedByValues[] = getProposedByValues(element, messageContext);
        args[0] = element.getID();
        args[1] = element.getReferenceNumber(); // Read Only
        args[2] = element.retrieveTypeID();
        args[3] = null; // AbstractScopeTask does not have
        // externalReferenceNumber
        args[4] = new Integer(500); // AbstractScopeTask does not have rank
        args[5] = element.getPriority();
        args[6] = new Integer(0);
        args[7] = null;
        args[8] = new Integer(0);
        args[9] = null;
        args[10] = null;
        args[11] = proposedByValues[0];
        args[12] = proposedByValues[1];
        args[13] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private String SP_U_CRI_NAME(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;

        String name = "SP_U_CRI_NAME";
        Object args[] = new Object[3];
        ScopeElement temp = (ScopeElement) rpmObject;
        args[0] = rpmObject.getID();
        args[1] = temp.getName();
        args[2] = getUser(messageContext).getID();
        //p_id, p_name, p_rec_user
        elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    private void SP_U_CRI_PROP(AbstractRequest element,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CRI_PROP";
        Object args[] = new Object[5];
        args[0] = element.getID();
        args[1] = element.getNumberTimesReassigned();
        args[2] = null;
        args[3] = DateUtil
            .convertToStringWithTime(element.getOccuredDateTime());
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_PROP(Issue element, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_CRI_PROP";
        Object args[] = new Object[5];
        args[0] = element.getID();
        args[1] = element.getNumberTimesReassigned();
        args[2] = null;
        args[3] = DateUtil
            .convertToStringWithTime(element.getOccuredDateTime());
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_PROP(Risk element, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_U_CRI_PROP";
        Object args[] = new Object[5];
        args[0] = element.getID();
        args[1] = new Integer(0);
        args[2] = null;
        args[3] = DateUtil
            .convertToStringWithTime(element.getOccuredDateTime());
        args[4] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private String SP_U_CRI_PUBLISHED(ScopeFolder scopeFolder,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String elementId = null;
        String name = "SP_U_CRI_PUBLISHED";
        Object args[] = new Object[4];
        args[0] = scopeFolder.getID();

        if (scopeFolder.getPrivateFolder() != null
                && scopeFolder.getPrivateFolder().booleanValue() == true)
        {
            args[1] = "Y";
        }
        else
        {
            args[1] = "N";
        }

        args[2] = "Y";
        args[3] = getUser(messageContext).getID();
        //p_id, p_published, p_visible_at_parent, p_rec_user
        elementId = executeProcedure(messageContext, name, args);
        return elementId;
    }

    //SP_U_CRI_QUAN('83B1E9458D5443ADB871D3BA04482BFA',307,0,0,0,0,0,'PMO_SUPERVISOR')
    private void SP_U_CRI_QUAN(AbstractScope element, ScopeElementScope scope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CRI_QUAN";
        Object args[] = new Object[8];
        args[0] = element.getID();

        if (scope != null && scope.isImpact() && element.getImpact() != null)
        {
            args[1] = element.getImpact().getScheduledImpactInDays();
        }
        else
        {
            args[1] = new Integer(0);
        }

        args[2] = new Integer(0);
        args[3] = new Integer(0);
        args[4] = new Integer(0);
        args[5] = new Integer(0);
        args[6] = new Integer(0);

        if (scope != null && scope.isOrderOfMagnitudeCostBenefits()
                && element.getOrderOfMagnitudeCostBenefits() != null)
        {
            if (element.getOrderOfMagnitudeCostBenefits()
                .getOrderOfMagnitudeHigh() != null)
            {
                args[2] = element.getOrderOfMagnitudeCostBenefits()
                    .getOrderOfMagnitudeHigh();
            }
            if (element.getOrderOfMagnitudeCostBenefits()
                .getOrderOfMagnitudeCost() != null)
            {
                args[3] = element.getOrderOfMagnitudeCostBenefits()
                    .getOrderOfMagnitudeCost();
            }
            if (element.getOrderOfMagnitudeCostBenefits()
                .getOrderOfMagnitudeLow() != null)
            {
                args[4] = element.getOrderOfMagnitudeCostBenefits()
                    .getOrderOfMagnitudeLow();
            }
            if (element.getOrderOfMagnitudeCostBenefits()
                .getOrderOfMagnitudeBillableRevenue() != null)
            {
                args[5] = element.getOrderOfMagnitudeCostBenefits()
                    .getOrderOfMagnitudeBillableRevenue();
            }
            if (element.getOrderOfMagnitudeCostBenefits()
                .getOrderOfMagnitudeNonBillableBenefits() != null)
            {
                args[6] = element.getOrderOfMagnitudeCostBenefits()
                    .getOrderOfMagnitudeNonBillableBenefits();
            }
        }

        args[7] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CRI_RESP(ScopeElement element, ScopeElementScope scope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        ApprovedCostBenefits approvedCostBenefits = null;

        if (scope != null && scope.isApprovedCostBenefits())
        {
            if (element instanceof AbstractRequest)
            {
                approvedCostBenefits = ((AbstractRequest) element)
                    .getApprovedCostBenefits();
            }
            else if (element instanceof Requirement)
            {
                approvedCostBenefits = ((Requirement) element)
                    .getApprovedCostBenefits();
            }
            else if (element instanceof Action)
            {
                approvedCostBenefits = ((Action) element)
                    .getApprovedCostBenefits();
            }
        }

        String name = "SP_U_CRI_RESP";
        Object args[] = new Object[10];
        Object approvedByValues[] = getApprovedByValues(element, messageContext);
        Object closedByValues[] = getClosedByValues(element, messageContext);
        args[0] = element.getID();

        if (element instanceof AbstractScope)
        {
            args[1] = approvedByValues[1];
            args[3] = approvedByValues[0];
        }

        if (approvedCostBenefits != null)
        {
            args[2] = approvedCostBenefits.getScheduleImpactInDays();
            args[4] = approvedCostBenefits.getCost();
            args[5] = approvedCostBenefits.getBillableRevenue();
            args[6] = approvedCostBenefits.getNonBillableBenefits();
        }
        else
        {
            args[2] = new Integer(0);
            args[4] = new Integer(0);
            args[5] = new Integer(0);
            args[6] = new Integer(0);
        }

        args[7] = closedByValues[0];
        args[8] = closedByValues[1];
        args[9] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //SP_U_CRI_REVISION('F643285BE2CE479B9E93FAED4377F4B5',0,137,4726,null,'PMO_SUPERVISOR')
    public void SP_U_CRI_REVISION(ScopeFolder scopeFolder,
            MessageContext messageContext) throws SQLException, RPMException
    {
        ExceptionUtil.handleMethodException(this, ErrorCodes.NOT_IMPLEMENTED,
            ExceptionUtil.SP_U_POOL_REVISION);
    }

    // SP_U_CRI_TARGET_DATE('66733D8B9CCD42FDB37A31C9086D7E6D','2005-12-23','PMO_SUPERVISOR')
    private void SP_U_CRI_TARGET_DATE(AbstractScope element,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_CRI_TARGET_DATE";
        Object args[] = new Object[3];
        args[0] = element.getID();
        args[1] = DateUtil.convertToString(element.getTargetDate());
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //    SP_T_CRI(null,'8C4A8F92A4B748B7B571012A44434CD9',0,'PMO_SUPERVISOR')
    private void SP_T_CRI(WorkElement workElement, ScopeElement scopeElement,
            int bitflag, MessageContext messageContext) throws SQLException,
            RPMException
    {
        String name = "SP_T_CRI";
        Object args[] = new Object[4];
        if (workElement != null)
            args[0] = workElement.getID();
        if (scopeElement != null)
            args[1] = scopeElement.getID();
        args[2] = new Integer(bitflag);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    public void SP_T_CRI(ScopeElement scopeElement, ScopeElement parent,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_T_CRI";
        Object args[] = new Object[4];
        args[0] = parent.getID();
        args[1] = scopeElement.getID();
        args[2] = new Integer(6);
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private Integer SP_S_WKF_PROCESS_TYPE(Integer typeId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_S_WKF_PROCESS_TYPE";
        Integer value = null;
        Object args[] = new Object[2];

        args[0] = typeId;
        args[1] = getUser(messageContext).getID();

        // we cannot call framework method executeProcedure directly 
        // because SP_S_* procedures do not return an error code
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);
            if (rs.next())
            {
                value = new Integer(rs.getInt(1));
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return value;
    }

    private ScopeElementSecurityType decodeScopeElementSecurityType(
            String dbValue) throws RPMException
    {
        return (ScopeElementSecurityType) EnumeratedTypeUtil.getInstance()
            .getType(ScopeElement.class, ScopeElementSecurityType.class,
                dbValue);
    }

    protected void decodeSecurityRoleIds(MessageContext msgContext,
            ResultSet rs, String contextName) throws RPMException, SQLException
    {
        String objectId = rs.getString(ID_ELEMENT_ID);

        String securityRoleIds = null;

        // CRI security
        securityRoleIds = (String) rs.getObject("CRI_SECURITY_ID");
        ManagerUtil.setSecurityRoleIds(msgContext, ScopeElement.class,
            objectId, securityRoleIds);

        // WBS security
        securityRoleIds = (String) rs.getObject("WBS_SECURITY_ID");
        ManagerUtil.setSecurityRoleIds(msgContext, ScopeElement.class,
            objectId, securityRoleIds);
    }

    protected Object loadWithSecurity(MessageContext messageContext,
            Connection c, String selectStmtStr,
            Object[] argValues, String contextName) throws SQLException,
            RPMException
    {
        String name = "SP_S_CRI_PAGES";

        Object args[] = new Object[15];

        String[] output = ManagerUtil.processSelectStatementBeforeSecurityCall(
            selectStmtStr, argValues);
        String columnList = output[0];
        String whereClause = output[2];

        //Get loadParams from the messageContext
        LoadParams loadParams = messageContext.getLoadParams();
        Integer startIndex = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getStartIndex() != null) ? loadParams.getStartIndex()
                : new Integer(1);
        Integer endIndex = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getEndIndex() != null) ? loadParams.getEndIndex() : new Integer(0);//Default '0' = load all
        String orderBy = (loadParams != null && !loadParams.isAlreadySet())
                ? loadParams.getOrderBy() : null;

        //for internal security mechanism, the loadParams.isForceBuildSecurity is used
//        Integer buildSecurity = (loadParams != null && ((!loadParams
//            .isAlreadySet() && loadParams.isBuildSecurity()) || (loadParams
//            .isForceBuildSecurity()))) ? new Integer(1) : new Integer(0);
        //To improve performance, we will always build the security
                
        // idsType Default '1' = All Project (values are mutually exclusive even if separate arguments exist for projectIDs/portfolioIDs and orgFilterIds)
        //      1=return all projects that are visible by resource 
        //      2=ids is a list of project IDs 
        //      3=ids is a list of portfolio IDs 
        //      4=ids is a organization filter id
        Integer idsType = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getIdsType() != null) ? loadParams.getIdsType() : new Integer(1);
        String idsList = (loadParams != null && !loadParams.isAlreadySet())
                ? loadParams.getIdsList() : null;
        String attributesIdsList = (loadParams != null && !loadParams
            .isAlreadySet()) ? loadParams.getAttributesIdsList() : null;
        // viewType Default to All (0=all, 1=my )
        Integer viewType = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getViewType() != null) ? loadParams.getViewType() : new Integer(0);
        // inclusions Default to All (0=Exclude Closed and Completed;32=Include Closed;64=Include Completed;96=Include Closed and Completed)        
        Integer inclusions = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getInclusions() != null) ? loadParams.getInclusions()
                : new Integer(96);

        whereClause = (loadParams != null && !loadParams.isAlreadySet() && loadParams
            .getExtraWhere() != null) ? (whereClause + " "
                + loadParams.getExtraWhere() + " ") : whereClause;

        boolean isProjectIdListUseful = (idsType.intValue()==2)||(idsType.intValue()==3);
        
        if (!isProjectIdListUseful)
        {
            idsList = null;
        }
        
        if (idsList == null || idsList.length() == 0)
        {
            String tableName = getTableName();
            ManagerUtil.IDStorage storage = ManagerUtil.getRestrictedProjectIdList(messageContext,
                name, whereClause, idsType, idsList, tableName);
            idsList = storage.idsList;
            idsType = storage.idsType;
        }            
            
        args[0] = idsType; // p_bitflag
        args[1] = idsList; // p_ids
        args[2] = (idsType.intValue() == 4) ? idsList : null; // p_org_filter_id
        args[3] = getUser(messageContext).getID(); // p_resource_id
        args[4] = viewType;//p_view_type
        args[5] = new Integer(0);//p_stage_id  0 : all stages(filtering will be in the where clause)
        args[6] = new Integer(0);//p_type_id   0 : all types (filtering will be in the where clause)
        args[7] = attributesIdsList; //filter_string
        args[8] = new Integer(1);//Always build_security
        args[9] = inclusions; // p_active_record
        args[10] = whereClause; // p_where_clause
        args[11] = orderBy;//p_order_by
        args[12] = columnList;
        args[13] = startIndex;//p_start_from
        args[14] = endIndex;//p_end_to
        
        return Manager.executeStoredProcedure(c, name, args, spParamsTypes, true, messageContext, true, useDBStatementsForLOBS);
    }

    private static final int[] spParamsTypes = new int[]{
        java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,
        java.sql.Types.INTEGER,java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER,
        java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER
    }; 
    
    /**
     * There are two ways to load data from other managers : by primary key and
     * by foreign keys. We need to load a WbsElement using the rank of the
     * current object. This field is not the primary key of the WbsManager, so
     * we can't load by primary key. Then again, there is already a join
     * condition between the two tables in this.getJoinCondition(), and we can't
     * create a second link to WbsManager in that method, given the way things
     * are designed. Therefore, we can't load by foreign key either.
     */
    private void loadOwningWorkElement(ScopeElement scElement,
            ScopeElementScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException
    {
        if (scope != null && scope.isWbsParent())
        {
            String rank = (String) relatedFields.get(index, ID_PARENT_RANK);
            final String marker = "WBS#";
            if (rank.startsWith(marker))
            {
                rank = rank.substring(marker.length());

                String selectStmt = "SELECT ELEMENT_ID FROM TMT_WBS WHERE RANK = '"
                        + rank + "'";
                Connection c = null;
                ResultSet rs = null;
                try
                {
                    c = Manager.getConnection(messageContext);
                    selectStmt = Manager.makeStatementUncommitedRead(
                        selectStmt, c);

                    rs = ManagerUtil.executeSqlQuery(messageContext, c,
                        selectStmt, null);

                    if (rs.next())
                    {
                        String wbsElementId = rs.getString(1);

                        WbsManager wbsMgr = (WbsManager) RPMManagerFactory
                            .getInstance().getRPMObjectManager(
                                WorkElement.class.getName());
                        WorkElement blank = new WorkElement();
                        blank.setID(wbsElementId);

                        blank = (WorkElement) wbsMgr.loadByPrimaryKey(blank,
                            null, messageContext, isDelta);

                        if (isDelta)
                        {
                            scElement.deltaWbsParent(blank);
                        }
                        else
                        {
                            scElement.setWbsParent(blank);
                        }
                    }
                }
                catch (ParseException e)
                {
                    ExceptionUtil.handleException(this, e);
                }
                finally
                {
                    Manager.close(messageContext, rs);
                    Manager.releaseConnection(messageContext, c);
                }
            }
        }
    }
}
